package AcmeDiagramLegend;

import java.util.MissingResourceException;
import java.util.ResourceBundle;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import AcmeDiagramLegend.prefs.LegendPreferences;

/**
 * The main plugin class to be used in the desktop.
 */
public class AcmeDiagramLegendPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static AcmeDiagramLegendPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
    
    /** ID's for plugin changes **/
    public final static int ADD_LEGEND = 3000;
    public final static int REMOVE_LEGEND = 3001;
    public final static int REORDER_LEGEND = 3002;
    public final static int ADD_LEGEND_ITEM = 3003;
    public final static int REMOVE_LEGEND_ITEM = 3004;
    
    /** The default size of the legend **/
    public final static Dimension DEFAULT_LEGEND_SIZE = new Dimension (410, 150);
    public static final int LAYOUT_LEGEND = 4;
    public static final int LAYOUT_TABLE = 3;
    public static final int LAYOUT_FREEFORM = 2;
    public static final int LAYOUT_FLOW_HORIZONTAL = 1;
    public static final int LAYOUT_FLOW_VERTICAL = 0;
	public static final String ABOVE = "ABOVE";
	public static final String RIGHT = "RIGHT";
	public static final String BELOW = "BELOW";
	public static final String LEFT = "LEFT";
	
	/**
	 * The constructor.
	 */
	public AcmeDiagramLegendPlugin() {
		super();
		plugin = this;
		try {
			resourceBundle= ResourceBundle.getBundle("AcmeDiagramLegend.AcmeDiagramLegendPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * Returns the shared instance.
	 */
	public static AcmeDiagramLegendPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= AcmeDiagramLegendPlugin.getDefault().getResourceBundle();
		try {
			return (bundle!=null ? bundle.getString(key) : key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#initializeDefaultPreferences(org.eclipse.jface.preference.IPreferenceStore)
	 */
	protected void initializeDefaultPreferences(IPreferenceStore store) {
		LegendPreferences.setDefaults(store);
		store.setDefault("LegendItem.Layout", LAYOUT_LEGEND);
	}

}
