/*
 * Created on Nov 3, 2003
 *
 */
package AcmeDiagramLegend;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.SelectionEditPolicy;

import AcmeDiagramLegend.actions.DeleteLegendCommand;
import AcmeDiagramLegend.model.AcmeLegend;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import AcmeDiagramLegend.model.AcmeLegendItemIcon;
import AcmeDiagramLegend.model.ColumnSeparator;
import AcmeDiagramLegend.parts.ColumnSeparatorEditPart;
import AcmeDiagramLegend.parts.LegendEditPart;
import AcmeDiagramLegend.parts.LegendItemDescriptionEditPart;
import AcmeDiagramLegend.parts.LegendItemIconEditPart;
import AcmeStudio.ui.editors.diagram.IElementFactory;
import AcmeStudio.ui.editors.diagram.policies.ElementSelectionEditPolicy;
import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl
 * Created Nov 3, 2003
 * 
 * Implements the methods for IElementFactory that allow legends to be added to AcmeStudio
 */
public class LegendFactory implements IElementFactory {

	/**
	 * 
	 */
	public LegendFactory() {
		super();
	}

	/**
     * Gets the legend associated with this system, and returns it
	 */
	public List getModels(AcmeSystem parent) {
		AcmeLegend legend = (AcmeLegend )parent.getUserData (AcmeDiagramLegendPlugin.class);
		if (legend == null) return new ArrayList ();
		else {
			List models = new ArrayList ();
			models.add (legend);
			return models;
		}
	}

	/**
     * Creates a new edit part for the legend, to be used by AcmeStudio
	 */
	public EditPart getEditPart(Object child) {
		if (child instanceof AcmeLegend) {
			return new LegendEditPart ();
		}
        else if (child instanceof AcmeLegendItemDescription) {
            return new LegendItemDescriptionEditPart ();
        }
        else if (child instanceof AcmeLegendItemIcon) {
        	return new LegendItemIconEditPart ();
        }
        else if (child instanceof ColumnSeparator) {
        	return new ColumnSeparatorEditPart ();
        }
		return null;
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.IElementFactory#getSelectionPolicy(java.lang.Object)
	 */
	public SelectionEditPolicy getSelectionPolicy(Object target) {
		if (target instanceof LegendEditPart) {
			return new ElementSelectionEditPolicy ();
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.IElementFactory#getDeleteCommand(java.lang.Object)
	 */
	public Command getDeleteCommand(Object model) {
		if (model instanceof AcmeLegend) {
			return new DeleteLegendCommand ((AcmeLegend )model);
		}
		return null;
	}

}
