/*
 * Created on Nov 5, 2003
 *  
 */
package AcmeDiagramLegend;

import java.io.*;
import java.util.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import AcmeDiagramLegend.model.AcmeLegend;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.core.resource.datapersistence.IDataPersistence;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.util.Helper;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.extensions.IElementExtension;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeSystem;
import acmeLib.tools.GraphWalker;
import acmeLib.tools.WalkerElementOperation;
/**
 * @author Bradley Schmerl
 *  
 */
public class LegendPersistor implements IDataPersistence {

	
private static final String	ASSOCIATEDSYSTEM_ATTR	= "associatedSystem";
	private static final String	DESCRIPTION_ATTR		= "description";
	private static final String	TYPE_ATTR				= "type";
	private static final String	ELEMENT_ATTR			= "element";
	private static final String	ITEM_TAG				= "item";
	private static final String	HEIGHT_ATTR				= "height";
	private static final String	WIDTH_ATTR				= "width";
	private static final String	ZORDER_ATTR				= "zOrder";
	private static final String	RELATIVELOCATION_ATTR	= "relativeLocation";
	private static final String	NAME_ATTR				= "name";
	private static final String	Y_ATTR					= "y";
	private static final String	X_ATTR					= "x";
	private static final String	ROLESWIDTH_ATTR			= "rolesWidth";
	private static final String	PORTSWIDTH_ATTR			= "portsWidth";
	private static final String	CONNECTORSWIDTH_ATTR	= "connectorsWidth";
	private static final String	COMPONENTSWIDTH_ATTR	= "componentsWidth";

	/**
	 * @author Bradley Schmerl Created Jan 26, 2004
	 */
	public class LegendTracker implements IElementExtension {

		/**
		 *  
		 */
		public LegendTracker() {
			super ();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see acmeLib.core.extensions.IElementExtension#isPersistent()
		 */
		public boolean isPersistent() {
			return false;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see acmeLib.core.extensions.IElementExtension#injectIntoElement(acmeLib.elements.AcmeElement)
		 */
		public void injectIntoElement(AcmeElement element) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see acmeLib.core.extensions.IElementExtension#extractFromElement(acmeLib.elements.AcmeElement)
		 */
		public void extractFromElement(AcmeElement element) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see acmeLib.core.extensions.IElementExtension#isInjected(acmeLib.elements.AcmeElement)
		 */
		public boolean isInjected(AcmeElement element) {
			return false;
		}
	}

	/**
	 * Loads the legend, if there is one. If the file <system>.acme has just
	 * been loaded, then this will try to load the legend stored in
	 * <system>.legend.xml
	 * 
	 * @param design
	 *                    The Acme design to load the legend into
	 * @param resource
	 *                    The Acme file that was just loaded
	 * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#load(AcmeStudio.core.resource.ASDesign,
	 *           File)
	 */
	public void load(ASDesign asDesign, File resource) {
		String acmeFilename = resource.getAbsolutePath ();
		AcmeDesign design = asDesign.getDesign ();
		String legendFilename = null;
		if (acmeFilename.endsWith (".acme")) {
			legendFilename = createLegendFilename (asDesign, acmeFilename);
			File legendFile = new File (legendFilename);
			boolean oldSoDelete = false;
			// Only try to load if there is a legend
			if (!legendFile.exists ()) {
				// Try old path, for backwards compatibility
				String oldLegendFilename;
				oldLegendFilename = acmeFilename.substring (0, acmeFilename.length () - 5) + ".legend.xml";
				if (!oldLegendFilename.equals (legendFilename)) {
					oldSoDelete = true;
				}
				legendFile = new File (oldLegendFilename);
			}
			if (!legendFile.exists ()) return;
			try {
				DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance ().newDocumentBuilder ();
				Document doc = docBuilder.parse (legendFile);
				AcmeSystem sys = Helper.getAcmeSystem (design);
				Map legends = importLegends (design, doc.getDocumentElement ());
				for (Iterator iter = legends.entrySet().iterator (); iter.hasNext ();) {
					Map.Entry element = (Map.Entry )iter.next ();
					AcmeSystem system = (AcmeSystem )design.lookupQualifiedName((String )element.getKey ());
					AcmeLegend legend = (AcmeLegend )element.getValue();
					legend.setParent (system);
					legend.setParentDesign (system.getParentDesign ());
					system.setUserData (AcmeDiagramLegendPlugin.class, legend);
					design.setUserData (AcmeDiagramLegendPlugin.class, new LegendTracker ());
				}
				
				if (oldSoDelete) {
					legendFile.delete ();
				}
			}
			catch (ParserConfigurationException e) {
				e.printStackTrace ();
			}
			catch (FactoryConfigurationError e) {
				e.printStackTrace ();
			}
			catch (SAXException e) {
				e.printStackTrace ();
			}
			catch (IOException e) {
				e.printStackTrace ();
			}
		}
	}

	/**
	 * @param design
	 * @param documentElement
	 * @return
	 */
	private Map importLegends(AcmeDesign design, Element element) {
		Map legendMap = new HashMap ();
		if (element.getNodeName ().equals ("legend")) { // only one legend in the legend file
			AcmeLegend legend = importLegend (design, element);
			String assocSys = element.getAttribute(ASSOCIATEDSYSTEM_ATTR);
			if (assocSys != null && !"".equals(assocSys)) {
				legendMap.put (assocSys, legend);
			}
			else {
				AcmeSystem rootSystem = (AcmeSystem )design.getSystems().getNextItem();
				legendMap.put (rootSystem.getQualifiedName(), legend);
			}
		}
		else {
			NodeList legends = element.getElementsByTagName("legend");
			for (int i = 0; i < legends.getLength(); i++) {
				Element legendNode = (Element )legends.item(i);
				AcmeLegend legend = importLegend (design, legendNode);
				String assocSys = legendNode.getAttribute(ASSOCIATEDSYSTEM_ATTR);
				legendMap.put (assocSys, legend);
			}
		}
		
		return legendMap;
	}

	protected boolean attributeOk(Element element, String attribute) {
		return element.getAttribute (attribute) != null && element.getAttribute (attribute).length () != 0;
	}

	/**
	 * Takes a legend element and turns it into an AcmeLegend, to be added to
	 * the design
	 * 
	 * @param system
	 *                    The system that the legend will be added to (used to resolve
	 *                    type names)
	 * @param element
	 *                    The XML element containing the legend
	 * @return The parsed and imported AcmeLegend
	 */
	private AcmeLegend importLegend(AcmeDesign design, Element element) {
		String sysName = element.getAttribute (ASSOCIATEDSYSTEM_ATTR);
		AcmeSystem system = null;
		if (sysName != null || !"".equals(sysName)) {
			system = (AcmeSystem )design.lookupQualifiedName(sysName);
		}
		else {
			system = (AcmeSystem )design.getSystems ().getNextItem();
		}
		AcmeLegend legend = new AcmeLegend (element.getAttribute (NAME_ATTR));
		try {
			if (attributeOk (element, X_ATTR)) {
				int x = Integer.parseInt (element.getAttribute (X_ATTR));
				int y = Integer.parseInt (element.getAttribute (Y_ATTR));
				SystemView.setLocation (legend, new Point (x, y));
			}
			else {
				String location = element.getAttribute (RELATIVELOCATION_ATTR);
				legend.setLocation (location);
			}
			if (attributeOk (element, ZORDER_ATTR)) {
				int zOrder = Integer.parseInt (element.getAttribute (ZORDER_ATTR));
				legend.setZOrder (zOrder);
			}
			int w = Integer.parseInt (element.getAttribute (WIDTH_ATTR));
			int h = Integer.parseInt (element.getAttribute (HEIGHT_ATTR));
			Helper.setSize (legend, new Dimension (w, h));
			String compW = element.getAttribute (COMPONENTSWIDTH_ATTR);
			String connW = element.getAttribute (CONNECTORSWIDTH_ATTR);
			String portW = element.getAttribute (PORTSWIDTH_ATTR);
			String roleW = element.getAttribute (ROLESWIDTH_ATTR);
			if (attributeOk (element, COMPONENTSWIDTH_ATTR)) {
				legend.setColumnWidth (AcmeLegend.COMPONENT_COLUMN, Integer.parseInt (compW));
			}
			if (attributeOk (element, CONNECTORSWIDTH_ATTR)) {
				legend.setColumnWidth (AcmeLegend.CONNECTOR_COLUMN, Integer.parseInt (connW));
			}
			if (attributeOk (element, PORTSWIDTH_ATTR)) {
				legend.setColumnWidth (AcmeLegend.PORT_COLUMN, Integer.parseInt (portW));
			}
			if (attributeOk (element, ROLESWIDTH_ATTR)) {
				legend.setColumnWidth (AcmeLegend.ROLE_COLUMN, Integer.parseInt (roleW));
			}
			NodeList itemNodes = element.getElementsByTagName (ITEM_TAG);
			for (int i = 0; i < itemNodes.getLength (); i++) {
				Element itemXML = (Element )itemNodes.item (i);
				String elementName = itemXML.getAttribute (ELEMENT_ATTR);
				if (elementName != null && elementName.length () != 0) {
					AcmeElement elementAcme = (AcmeElement )design.lookupQualifiedName (elementName);
					if (element != null) {
						String description = itemXML.getAttribute (DESCRIPTION_ATTR);
						AcmeLegendItemDescription item = new AcmeLegendItemDescription (elementAcme, description);
						legend.addLegendItem (item);
					}
				}
				else {
					String typeName = itemXML.getAttribute (TYPE_ATTR);
					AcmeElementType type = null;
					if (typeName.equals ("component"))
						type = ConstantTypes.COMPONENT_BASE_TYPE;
					else if (typeName.equals ("connector"))
						type = ConstantTypes.CONNECTOR_BASE_TYPE;
					else if (typeName.equals ("role")) {
						type = ConstantTypes.ROLE_BASE_TYPE;
					}
					else if (typeName.equals ("port"))
						type = ConstantTypes.PORT_BASE_TYPE;
					else {
						
						
						type = (AcmeElementType )system.lookupName (typeName, true);
						
					}
					if (type != null) {
						String description = itemXML.getAttribute (DESCRIPTION_ATTR);
						AcmeLegendItemDescription item = new AcmeLegendItemDescription (type, description);
						legend.addLegendItem (item);
					}
				}
			}
		}
		catch (Throwable e) {
			e.printStackTrace ();
		}
		return legend;
	}

	Collection extractSystemsWithLegends(AcmeDesign design) {
		final Set systems = new HashSet ();
		WalkerElementOperation extractLegendedSystemsOp = new WalkerElementOperation () {

			public void doElementOp(AcmeElement elt) {
				if (elt instanceof AcmeSystem) {
					AcmeLegend legend = (AcmeLegend )elt.getUserData (AcmeDiagramLegendPlugin.class);
					if (legend != null) {
						systems.add (elt);
					}
				}
			}
		};
		GraphWalker.WalkOverAllElements(design, extractLegendedSystemsOp, GraphWalker.WALK_BOTTOM_UP);
		return systems;
	}

	/**
	 * Saves the legend embedded in the systems of the design
	 * 
	 * @param design
	 *                    the design that was saved
	 * @param resource
	 *                    the file that the design was saved to
	 * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#save(AcmeStudio.core.resource.ASDesign,
	 *           File)
	 */
	public void save(ASDesign asDesign, File resource) {
		AcmeDesign design = asDesign.getDesign ();
		Collection systems = extractSystemsWithLegends (design);
		String acmeFilename = resource.getAbsolutePath ();
		String legendFilename = null;
		// Delete legend file if there are no longer any legends
		if (systems.size () == 0) {
			LegendTracker tracker = (LegendTracker )design.getUserData (AcmeDiagramLegendPlugin.class);
			if (tracker == null) return;
			legendFilename = createLegendFilename (asDesign, acmeFilename);
			File legendFile = new File (legendFilename);
			if (legendFile.exists ()) {
				legendFile.delete ();
				design.setUserData (AcmeDiagramLegendPlugin.class, null);
			}
			return;
		}
		// save the legend
		if (acmeFilename.endsWith (".acme")) {
			legendFilename = createLegendFilename (asDesign, acmeFilename);
			File file = new File (legendFilename);
			try {
				FileOutputStream fos = new FileOutputStream (file);
				Iterator systemIterator = systems.iterator ();
				Element topLevel = null;
				StringWriter sw = new StringWriter ();
				DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance ().newDocumentBuilder ();
				Document doc = docBuilder.newDocument ();
				while (systemIterator.hasNext ()) {
					AcmeSystem system = (AcmeSystem )systemIterator.next ();
					AcmeLegend legend = (AcmeLegend )system.getUserData (AcmeDiagramLegendPlugin.class);
					Element elem = exportLegend (doc, legend);
					elem.setAttribute(ASSOCIATEDSYSTEM_ATTR, system.getQualifiedName ());
					if (systems.size () == 1) topLevel = elem;
					else {
						if (topLevel == null) topLevel = doc.createElement("legends");
						topLevel.appendChild(elem);
					}
				}
				TransformerFactory tFactory = TransformerFactory.newInstance ();
				Transformer transformer = tFactory.newTransformer ();
				DOMSource source = new DOMSource (topLevel);
				StreamResult result = new StreamResult (sw);
				transformer.transform (source, result);
				fos.write (sw.toString ().getBytes ());
				fos.close ();
			}
			catch (FileNotFoundException e) {
				e.printStackTrace ();
			}
			catch (ParserConfigurationException e) {
				e.printStackTrace ();
			}
			catch (FactoryConfigurationError e) {
				e.printStackTrace ();
			}
			catch (IOException e) {
				e.printStackTrace ();
			}
			catch (TransformerConfigurationException e) {
				e.printStackTrace ();
			}
			catch (TransformerException e) {
				e.printStackTrace ();
			}
			
		}
	}

	/**
	 * Given an acme filename (ending in .acme), creates the filename for the
	 * legend
	 * 
	 * @param acmeFilename
	 *                    the acme filename
	 * @return the legend filename
	 */
	private String createLegendFilename(ASDesign design, String acmeFilename) {
		IPath dataPath = null;
		try {
			dataPath = design.getDataDirectory ();
		}
		catch (IOException e) {
			String legendFilename;
			legendFilename = acmeFilename.substring (0, acmeFilename.length () - 5) + ".legend.xml";
			return legendFilename;
		}
		Path acmeFilepath = new Path (acmeFilename);
		String filename = acmeFilepath.lastSegment ();
		dataPath = dataPath.append (filename.substring (0, filename.length () - 5) + ".legend.xml");
		return dataPath.toString ();
	}

	/**
	 * Creates an XML element corresponding to the legend
	 * 
	 * @param doc
	 *                    the Document that the element will be attached to
	 * @param legend
	 *                    The legend
	 * @return the new XML element representing the legend
	 */
	private Element exportLegend(Document doc, AcmeLegend legend) {
		Element legendXML = doc.createElement ("legend");
		legendXML.setAttribute (NAME_ATTR, legend.getName ());
		// TODO: Make sure that layout is defined
		legendXML.setAttribute ("layout", "Unknown");
		Point location = SystemView.getLocation (legend, false);
		if (location != null) {
			legendXML.setAttribute (X_ATTR, new Integer (location.x).toString ());
			legendXML.setAttribute (Y_ATTR, new Integer (location.y).toString ());
		}
		legendXML.setAttribute (WIDTH_ATTR, new Integer (Helper.getSize (legend).width).toString ());
		legendXML.setAttribute (HEIGHT_ATTR, new Integer (Helper.getSize (legend).height).toString ());
		legendXML.setAttribute (ZORDER_ATTR, new Integer (Helper.getZOrder (legend)).toString ());
		legendXML.setAttribute (RELATIVELOCATION_ATTR, legend.getLocation ());
		int componentsWidth = legend.getColumnWidth (AcmeLegend.COMPONENT_COLUMN);
		int connectorsWidth = legend.getColumnWidth (AcmeLegend.CONNECTOR_COLUMN);
		int roleWidth = legend.getColumnWidth (AcmeLegend.ROLE_COLUMN);
		int portWidth = legend.getColumnWidth (AcmeLegend.PORT_COLUMN);
		if (componentsWidth > 0) {
			legendXML.setAttribute (COMPONENTSWIDTH_ATTR, new Integer (componentsWidth).toString ());
		}
		if (connectorsWidth > 0) {
			legendXML.setAttribute (CONNECTORSWIDTH_ATTR, new Integer (connectorsWidth).toString ());
		}
		if (portWidth > 0) {
			legendXML.setAttribute (PORTSWIDTH_ATTR, new Integer (portWidth).toString ());
		}
		if (roleWidth > 0) {
			legendXML.setAttribute (ROLESWIDTH_ATTR, new Integer (roleWidth).toString ());
		}
		for (Iterator iter = legend.getItems ().iterator (); iter.hasNext ();) {
			Object o = iter.next ();
			if (o instanceof AcmeLegendItemDescription) {
				AcmeLegendItemDescription item = (AcmeLegendItemDescription )o;
				if (item.getDescription () == "") continue;
				Element itemXML = doc.createElement (ITEM_TAG);
				if (item.getType () != null)
					itemXML.setAttribute (TYPE_ATTR, item.getType ().getTypeName ());
				else
					itemXML.setAttribute (ELEMENT_ATTR, item.getElement ().getQualifiedName ());
				itemXML.setAttribute (DESCRIPTION_ATTR, item.getDescription ());
				legendXML.appendChild (itemXML);
			}
		}
		return legendXML;
	}

	/**
	 * @param design
	 * @return
	 * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#extractData(AcmeStudio.core.resource.ASDesign,
	 *           java.io.File)
	 */
	public Object extractData(ASDesign design) {
		Map legendMap = new HashMap ();
		AcmeEnumeration systems = design.getDesign ().getSystems ();
		while (systems.moreItems ()) {
			AcmeSystem system = (AcmeSystem )systems.getNextItem ();
			AcmeLegend legend = (AcmeLegend )system.getUserData (AcmeDiagramLegendPlugin.class);
			if (legend != null) {
				legendMap.put (system.getName (), legend);
			}
		}
		return legendMap;
	}

	/**
	 * @param design
	 * @param data
	 * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#injectData(AcmeStudio.core.resource.ASDesign,
	 *           java.io.File)
	 */
	public void injectData(ASDesign design, Object data) {
		Map legendMap = (Map )data;
		for (Iterator iter = legendMap.entrySet ().iterator (); iter.hasNext ();) {
			Map.Entry entry = (Map.Entry )iter.next ();
			AcmeSystem system = design.getDesign ().getSystem ((String )entry.getKey ());
			system.setUserData (AcmeDiagramLegendPlugin.class, (AcmeLegend )entry.getValue ());
		}
	}
}
