/*
 * Created on Nov 6, 2003
 * 
 */
package AcmeDiagramLegend.actions;

import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import AcmeDiagramLegend.AcmeDiagramLegendPlugin;
import AcmeDiagramLegend.dialogs.FormatLegendDialog;
import AcmeDiagramLegend.figures.LegendFigure;
import AcmeDiagramLegend.model.AcmeLegend;
import AcmeDiagramLegend.parts.LegendEditPart;
import AcmeDiagramLegend.parts.LegendFlowEditPolicy;
import AcmeDiagramLegend.parts.LegendLayout;
import AcmeDiagramLegend.parts.LegendLayoutEditPolicy;
import AcmeDiagramLegend.parts.LegendXYLayoutEditPolicy;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;

/**
 * @author Bradley Schmerl
 * Implements an action (that will be attached to the diagram editors context menu) to change the format of the legend
 *
 */
public class FormatLegendAction
    implements IEditorActionDelegate, IObjectActionDelegate {

    /** The corresponding edit part in the diagram for the legend **/
    private LegendEditPart _targetEditPart;

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorActionDelegate#setActiveEditor(org.eclipse.jface.action.IAction, org.eclipse.ui.IEditorPart)
     */
    public void setActiveEditor(IAction action, IEditorPart targetEditor) {
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
    }

    /**
     * Raises the format dialog and then saves the options (either changes the layout or appearance of the legend)
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {
        FormatLegendDialog dialog = new FormatLegendDialog (Display.getCurrent().getActiveShell());
        if (dialog.open () == IDialogConstants.OK_ID) {
            LegendFigure figure = (LegendFigure )_targetEditPart.getFigure();
            int layout = AcmeDiagramLegendPlugin.getDefault().getPreferenceStore().getInt("LegendItem.Layout");
            switch (layout) {
                case AcmeDiagramLegendPlugin.LAYOUT_FREEFORM:
                    figure.getContentsPane().setLayoutManager(new XYLayout ());
                    _targetEditPart.changeLayoutPolicy(new LegendXYLayoutEditPolicy ());
                    break;
                case AcmeDiagramLegendPlugin.LAYOUT_FLOW_HORIZONTAL:
                    figure.getContentsPane().setLayoutManager(new FlowLayout (true));
                    _targetEditPart.changeLayoutPolicy (new LegendFlowEditPolicy ());
                    break;
                    case AcmeDiagramLegendPlugin.LAYOUT_FLOW_VERTICAL:
                    	figure.getContentsPane ().setLayoutManager(new FlowLayout (false));
                    	_targetEditPart.changeLayoutPolicy(new LegendFlowEditPolicy ());
                    	break;
                    case AcmeDiagramLegendPlugin.LAYOUT_LEGEND:
                    	figure.getContentsPane ().setLayoutManager(new LegendLayout ((AcmeLegend )_targetEditPart.getModel()));
                    	_targetEditPart.changeLayoutPolicy(new LegendLayoutEditPolicy ());
                    	break;
                    }
            _targetEditPart.refresh();
        
        
        }     
    }

    /**
     * This action is only enabled if the diagram editor is active and a legend is selected
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
    	if (!(selection instanceof IStructuredSelection)) {
    		action.setEnabled (false);
    		return;
    	}
        AcmeElement [] elements = Helper.getAcmeElementsFromSelection(selection);
        
        if (elements.length == 1 && elements [0] instanceof AcmeLegend) {
            _targetEditPart = (LegendEditPart )((IStructuredSelection )selection).getFirstElement();
            action.setEnabled(true);
        }
        else {
            action.setEnabled (false);
        }
    }

}
