/*
 * Created on Nov 4, 2003
 * 
 */
package AcmeDiagramLegend.actions;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import AcmeDiagramLegend.AcmeDiagramLegendPlugin;
import AcmeDiagramLegend.model.AcmeLegend;
import AcmeStudio.core.ASContext;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl
 *
 * Performs the action to generate a legend, based on an Acme system
 */
public class GenerateLegendAction implements IWorkbenchWindowActionDelegate {

    /** The diagram edit part that represents the system **/
    private SystemEditPart _part = null;
    
    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
     */
    public void dispose() {
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window) {
    }

    /**
     * Runs the action.
     */
    public void run(IAction action) {
        // Get the diagram editor
        DiagramEditor editor = ASContext.current ().getActiveEditor ().getDiagramEditor();
      
        if (editor != null) {
            _part = editor.getRootEditPart ();
            AcmeSystem system = (AcmeSystem )_part.getModel();
            
            // Generate the legend
            AcmeLegend legend = AcmeLegend.generateFromSystem(system);
            legend.setParentDesign(system.getParentDesign ());
            int iNumRows=0;
            
            if(legend._num_components>iNumRows) iNumRows=legend._num_components;
            if(legend._num_connectors>iNumRows) iNumRows=legend._num_connectors;
            if(legend._num_ports>iNumRows) iNumRows=legend._num_ports;
            if(legend._num_roles>iNumRows) iNumRows=legend._num_roles;
            Dimension legend_size = new Dimension (AcmeDiagramLegendPlugin.getDefault().getPreferenceStore().getInt ("AcmeLegend.maxWidth"),iNumRows*30+80);
            //Helper.setSize(legend, AcmeDiagramLegendPlugin.DEFAULT_LEGEND_SIZE);
            Helper.setSize(legend, legend_size);
            setInitialLocation (legend, system);
            
            // Announce the changes so that the diagram can be updated
            AcmeStudioChangeManager.globalChangeManager.onPluginChange(AcmeDiagramLegendPlugin.ADD_LEGEND, system, legend, this, system.getParentDesign());
        }
    }

    /**
	 * @param legend
	 */
	private void setInitialLocation(AcmeLegend legend, AcmeSystem system) {
		String initialPosition = AcmeDiagramLegendPlugin.getDefault().getPreferenceStore().getString ("AcmeLegend.legendPosition");
		legend.setLocation (initialPosition);
		
	}

	/**
     * Changes the enablement of the action, based on what is selected
     */
    public void selectionChanged(IAction action, ISelection selection) {
        action.setEnabled (true);
        
        // Disable the action if there is no diagram editor
        if (ASContext.current () == null ||
                ASContext.current ().getActiveEditor () == null ||
                ASContext.current ().getActiveEditor ().getDiagramEditor () == null) {
            action.setEnabled(false);
            
        }
        else {
            DiagramEditor editor = ASContext.current ().getActiveEditor ().getDiagramEditor();
            // Disable the action if there is already a legend generated for this system
            if (editor.getAcmeSystem().getUserData (AcmeDiagramLegendPlugin.class) != null) {
                action.setEnabled (false);
            }
        }
        
    }

}
