/*
 * Created on Nov 6, 2003
 * 
 */
package AcmeDiagramLegend.actions;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateRequest;

import AcmeDiagramLegend.model.AcmeLegend;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.elements.*;

/**
 * @author Bradley Schmerl
 *
 * Utility class for creating legend items (used by LegendFlowEditPolicy and LegendXYEditPolicy)
 */
public class ItemCommandFactory {
    /**
     * Creates a command for creating a legend item
     * @param request The originating request
     * @param part The edit part of the legend
     * @param location The location of the new item
     * @return The command to be used to create the item
     */
    public static Command getCreateCommand (CreateRequest request, EditPart part, Point location) {
       
        AcmeElementType et = null;
    
        // The object type contains the type of object dragged onto the diagram
        // It is either an AcmeElementType or a Class (AcmeComponent, AcmeConnector, ...)
        Object newObjectType = request.getNewObjectType ();
        if (newObjectType instanceof AcmeElementType) {
            et = (AcmeElementType) newObjectType;
        }
        else if (newObjectType instanceof Class) {
            Class clazz = (Class) newObjectType;
            if (clazz.equals(AcmeComponent.class)) et = ConstantTypes.COMPONENT_BASE_TYPE;
            else if (clazz.equals (AcmeConnector.class)) et = ConstantTypes.CONNECTOR_BASE_TYPE;
            else if (clazz.equals (AcmeRoleType.class)) et = ConstantTypes.ROLE_BASE_TYPE;
            else if (clazz.equals (AcmePortType.class)) et = ConstantTypes.PORT_BASE_TYPE;
        }
        
        AcmeLegendItemDescription child = new AcmeLegendItemDescription (et);
        AcmeLegend legend = (AcmeLegend )part.getModel ();
        
        // Generate and initialize the command
        ItemLifecycleCommand create = new ItemLifecycleCommand (part, ItemLifecycleCommand.ADD_ITEM);
        create.setItemDescription (child);
        create.setLegend (legend);
        create.setLocation (location);
        create.setLabel ("Create");
        return create;
    }

	/**
	 * @param child
	 * @param host
	 * @param location
	 * @return
	 */
	public static Command getAddCommand(EditPart child, EditPart host, Point location) {
		if (!(child.getModel () instanceof AcmeElement)) return null;
		AcmeElement element = (AcmeElement )child.getModel ();
//		AcmeElementType et = (AcmeElementType )element.getDeclaredTypes().getNextItem();
		  AcmeLegendItemDescription legendChild = new AcmeLegendItemDescription (element);
	      AcmeLegend legend = (AcmeLegend )host.getModel ();
	        
		ItemLifecycleCommand add = new ItemLifecycleCommand (host, ItemLifecycleCommand.ADD_ITEM);
		add.setItemDescription(legendChild);
		add.setLegend (legend);
		add.setLocation(location);
		add.setLabel ("Add");
		return add;
		
	}
}
