/*
 * Created on Nov 6, 2003
 * 
 */
package AcmeDiagramLegend.actions;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;

import AcmeDiagramLegend.AcmeDiagramLegendPlugin;
import AcmeDiagramLegend.model.AcmeLegend;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import AcmeDiagramLegend.model.AcmeLegendItemIcon;
import AcmeStudio.util.AcmeStudioChangeManager;

/**
 * @author Bradley Schmerl
 *
 * A command for creating or deleting legend items
 */
public class ItemLifecycleCommand extends Command {

    /** The id to use when adding and item **/
    public static final String ADD_ITEM = "$ADD_ITEM";
    /** The id to use when deleteing an item **/
    public static final String REMOVE_ITEM = "$DEL_ITEM";
    
    /** The location of the item (if it is being created) **/
    private Point _location;
    /** The legend item that was created or removed **/
    private AcmeLegendItemDescription _itemDesc;
    /** The legend item that was created or removed **/
    private AcmeLegendItemIcon _itemIcon;
    /** The legend that the item belongs to **/
    private AcmeLegend _legend;
    /** The edit part of the legend **/
    private EditPart _part;
    /** The id of this command (ADD_ITEM or DELETE_ITEM) **/
    private String _id;

    /**
     * Creates the command
     * @param part The edit part of the legend
     * @param id The id for the command
     */
    public ItemLifecycleCommand(EditPart part, String id) {
        _part = part;
        _id = id;
    }

    /**
     * Sets the location of the new item
     * @param location
     */
    public void setLocation(Point location) {
        _location = location;
    }

    /**
     * Sets the target item of the command
     * @param child
     */
    public void setItemDescription(AcmeLegendItemDescription child) {
        _itemDesc = child;
    }

    public void setItemIcon(AcmeLegendItemIcon icon) {
    	_itemIcon = icon;
    }

    /**
     * Sets the target legend of the command
     * @param legend
     */
    public void setLegend(AcmeLegend legend) {
        _legend = legend;
    }

    
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        if (_id.equals (ADD_ITEM)) {
            add();
        }
        else if (_id.equals (REMOVE_ITEM)) {
            remove ();
        }
    }

    /**
     *  Adds the target item to the legend
     */
    private void add() {
    	_legend.addLegendItem(_itemDesc);
        _itemDesc.setLocation (_location);
        AcmeStudioChangeManager.globalChangeManager.onPluginChange(AcmeDiagramLegendPlugin.ADD_LEGEND_ITEM, _legend, _itemDesc, this, _legend.getParentDesign());
        _part.refresh();
        _part.activate();
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute ();
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        if (_id.equals (ADD_ITEM)) {
            remove();
        }
        else if (_id.equals (REMOVE_ITEM)) {
            add ();
        }
    }

    /**
     * Removes the target item from the legend
     */
    private void remove() {
    	_legend.removeItem (_itemDesc);
    	_legend.removeItem (_itemIcon);
    	AcmeStudioChangeManager.globalChangeManager.onPluginChange(AcmeDiagramLegendPlugin.REMOVE_LEGEND_ITEM, _legend, _itemDesc, this, _legend.getParentDesign ());
    	_part.refresh();
    }

}
