/*
 * Created on Nov 6, 2003
 * 
 */
package AcmeDiagramLegend.actions;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.commands.Command;
import AcmeDiagramLegend.figures.LegendFigure;
import AcmeDiagramLegend.figures.LegendItemLabel;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import AcmeDiagramLegend.parts.LegendItemDescriptionEditPart;

/**
 * @author Bradley Schmerl
 *
 * The command for moving and resizing an item in the legend
 */
public class SetItemConstraintCommand extends Command {
    
    private Dimension _newSize;
    private Dimension _oldSize;
    private Point _newPos;
    private Point _oldPos;
    private AcmeLegendItemDescription _item;
    private LegendItemDescriptionEditPart _part;
	/**
     * @param part
     */
    public SetItemConstraintCommand(LegendItemDescriptionEditPart part) {
        _part = part;
    }
    /**
     * @param lf
     */
    public void setLegendView(LegendFigure lf) {
    }
    /**
     * @param item
     */
    public void setItem(AcmeLegendItemDescription item) {
        _item = item;
    }
    /**
     * @param rectangle
     */
    public void setLocation(Rectangle rectangle) {
    	//Rectangle tempRect=new Rectangle(10,10,100,100);
        _newPos = rectangle.getLocation();
        _newSize = rectangle.getSize();
    }
    
    
    

    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        setLabel ("Resize/Move Legend Item for " + _item.getType().getTypeName());
        if (_oldPos == null || !_oldPos.equals (_newPos)) {
            move (_newPos);
        }
        if (_oldSize == null || !_oldSize.equals (_newSize)) {
        	resize (_newSize, _oldSize);
        }
        
        _part.refresh ();
    }

    /**
     * @param pos
     */
    private void move(Point pos) {
        _item.setLocation (pos);
        
        _part.refresh();
    }
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
       execute ();
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        move (_oldPos);
        resize (_oldSize, _newSize);
    }
	/**
	 * @param size
	 */
	private void resize(Dimension newSize, Dimension origSize) {
		int delta = _item.getWidth () - newSize.width;
		_item.setWidth (_item.getWidth () - delta);
//		_itemFigure.setSize (size);
//		String [] newLines = _itemFigure.getText ().split("\n");
//		_itemFigure.setText("");
//		for (int i = 0; i < newLines.length; i++) {
//			_itemFigure.setText (_itemFigure.getText() + " " + newLines [i]);
//		}
//		
//		
//		String [] tokens = _itemFigure.getText ().split (" ");
//		Font font = _itemFigure.getFont ();
//		String newText = "";
//		String line = "";
//		int lineWidth = 0;
//		final int spaceWidth = FigureUtilities.getTextWidth (" ", font);
//		for (int i = 0; i < tokens.length; i++) {
//			String token = tokens [i];
//			if (token.equals ("")) continue;
//			int width = FigureUtilities.getTextWidth  (token, font);
//			
//			if (lineWidth + width < size.width) {
//				lineWidth += width;
//				line += token;
//				if (i + 1 == tokens.length && lineWidth + spaceWidth < size.width) {
//					newText += " " + line;
//					continue;
//				}
//				else if (i + 1 == tokens.length) {
////					line += "\n" + token + " ";
//					newText += line;
//					continue;
//				}
//				if (lineWidth + spaceWidth < size.width) {
//					lineWidth += spaceWidth;
//					line += " ";
//				}
//				else {
//					lineWidth = 0;
//					newText += line + "\n";
//					line = "";
//				}
//			}
//			else {
//				lineWidth = width;
//				newText += line + "\n" + token + " ";
//				line = "";
//			}
//			
//		}
//		_item.setDescription(newText);
//		_itemFigure.setText(newText);
	}
	/**
	 * @param label
	 */
	public void setItemFigure(LegendItemLabel label) {
		
	}

}
