/*
 * Created on Feb 4, 2004
 *
 */
package AcmeDiagramLegend.actions;

import java.util.Iterator;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.commands.CompoundCommand;

import AcmeDiagramLegend.model.AcmeLegend;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import AcmeDiagramLegend.model.ColumnSeparator;
import AcmeDiagramLegend.parts.LegendEditPart;
import AcmeDiagramLegend.parts.LegendItemDescriptionEditPart;
import AcmeDiagramLegend.parts.ColumnSeparatorEditPart;



/**
 * @author Bradley Schmerl
 * Created Feb 4, 2004
 */
public class SetSeparatorConstraintCommand extends CompoundCommand {

	private ColumnSeparatorEditPart _separatorPart;
	private LegendEditPart _legendPart;
	private int _locationX;
	private ColumnSeparator _separator;

	/**
	 * 
	 */
	public SetSeparatorConstraintCommand(ColumnSeparatorEditPart sepPart, LegendEditPart part, int xLoc) {
		super ();
		_separatorPart = sepPart;
		_legendPart = part;
		_locationX = xLoc;
		_separator = (ColumnSeparator )sepPart.getModel ();
		for (Iterator iter = _separator.getMembers(); iter.hasNext (); ) {
			AcmeLegendItemDescription item = (AcmeLegendItemDescription) iter.next ();
			LegendItemDescriptionEditPart itemEp = findEditPart (item);
			SetItemConstraintCommand command = new SetItemConstraintCommand (itemEp);
			command.setItem(item);
			Rectangle bounds = itemEp.getFigure().getBounds().getCopy ();
			bounds.width = bounds.width - (bounds.x + bounds.width - xLoc);
			command.setLocation(bounds);
			this.add(command);
		}
		
	}

	/**
	 * @param item
	 * @return
	 */
	private LegendItemDescriptionEditPart findEditPart(AcmeLegendItemDescription item) {
		LegendItemDescriptionEditPart found = null;
		for (Iterator iter = _legendPart.getChildren ().iterator () ; iter.hasNext () && (found == null); ) {
			Object p = iter.next ();
			if (p instanceof LegendItemDescriptionEditPart) {
				LegendItemDescriptionEditPart candEP = (LegendItemDescriptionEditPart) p;
				if (candEP.getModel().equals (item)) {
					found = candEP;
				}
			}
		}
		return found;
	}

	/**
	 * @param label
	 */
	public SetSeparatorConstraintCommand(String label) {
		super (label);
	}

	/**
	 * @param i
	 */
	public void setX(int i) {
		_locationX = i;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		AcmeLegend legend = (AcmeLegend )_legendPart.getModel ();
		int column = ((ColumnSeparator )_separatorPart.getModel()).getColumn ();
		int oldX = _separatorPart.getFigure ().getBounds ().x;
		int delta = oldX - _locationX;
		legend.setColumnWidth(column, legend.getColumnWidth(column) - delta);
		super.execute ();
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		super.redo ();
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		super.undo ();
	}
}
