/*
 * Created on Nov 4, 2003
 * 
 */
package AcmeDiagramLegend.figures;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.handles.HandleBounds;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import AcmeDiagramLegend.AcmeDiagramLegendPlugin;
import AcmeDiagramLegend.model.AcmeLegend;
import AcmeDiagramLegend.parts.LegendLayout;
import AcmeDiagramLegend.parts.LegendTableLayout;
import AcmeStudio.ui.editors.diagram.visualInfo.VisualInformation;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Debug;
import AcmeStudio.util.Helper;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.elements.AcmeElement;
import acmeLib.properties.AcmeFloat;
import acmeLib.properties.AcmeProperty;

/**
 * @author Bradley Schmerl
 * 
 * The figure for the legend
 */
public class LegendFigure extends LabeledContainer implements HandleBounds, AcmeModelChangeListener {

    private IFigure _pane;
    private AcmeElement _model;
	private int _legendWidth;
	private int _legendHeight;
    
    public LegendFigure () {
    	setBorder (new CompoundBorder (new LegendBorder (), new GroupBoxBorder ()));
//        ScrollPane scrollpane = new ScrollPane ();
        _pane = new FreeformLayer ();
        LayoutManager manager = null;
        int layout = AcmeDiagramLegendPlugin.getDefault().getPreferenceStore().getInt("LegendItem.Layout");
        switch (layout) {
            case AcmeDiagramLegendPlugin.LAYOUT_FREEFORM:
                manager = new XYLayout ();
                break;
               case AcmeDiagramLegendPlugin.LAYOUT_FLOW_HORIZONTAL:
                   manager = new FlowLayout (true);
                   break;
                   case AcmeDiagramLegendPlugin.LAYOUT_FLOW_VERTICAL:
                   	manager = new FlowLayout (false);
                   	break;
                   case AcmeDiagramLegendPlugin.LAYOUT_TABLE:
                   	manager = new LegendTableLayout();
                   	break;
                   case AcmeDiagramLegendPlugin.LAYOUT_LEGEND:
                   	//Label fred = new Label("Fred");
                   	//add(fred);
                   	//Label fred2 = new Label("Fred2");
                   	//_pane.setConstraint(fred2,new Rectangle(10,10,-1,-1));
                   	
                   	
                   	manager = new LegendLayout((AcmeLegend )_model);
                   	//legend._manager=(LegendLayout)manager;
                   	//((LegendLayout)manager)._num_components=legend._num_components;
                   	//((LegendLayout)manager)._num_connectors=legend._num_connectors;
                   	//((LegendLayout)manager)._num_roles=legend._num_roles;
                   	//((LegendLayout)manager)._num_ports=legend._num_ports;
                   	break;
                   default:
                         manager = new FlowLayout (false);
        }
        _pane.setLayoutManager(manager);
        setLayoutManager(new StackLayout ());
        FreeformViewport ffv = new FreeformViewport ();
        add (ffv);
//        add (scrollpane);
//        scrollpane.setViewport(new FreeformViewport ());
//        scrollpane.setContents (_pane);
        ffv.setContents(_pane);
        setBackgroundColor (ColorConstants.listBackground);
        setOpaque(true);
        //setSize (200, 100);
//        this.setLabel("Legend:");
        calculateSize ();
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.handles.HandleBounds#getHandleBounds()
     */
    public Rectangle getHandleBounds() {
        return getBounds ().getCropped (new Insets (2, 0, 2, 0));
    }

    public IFigure getContentsPane () {
        return _pane;
    }
    
    
    /* (non-Javadoc)
     * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
     */
    public Dimension getPreferredSize(int wHint, int hHint) {
        Dimension prefSize = super.getPreferredSize(wHint, hHint);
        Dimension defaultSize = new Dimension (200, 100);
        prefSize.union (defaultSize);
        return prefSize;
    }
    
    public void calculateSize () {

    	_legendWidth=((LegendLayout)_pane.getLayoutManager()).getLegendWidth ();
    	_legendHeight=((LegendLayout)_pane.getLayoutManager()).getLegendHeight ();
       	Dimension size = new Dimension (_legendWidth, _legendHeight + 50);
    	this.setSize (size);
		if (_model != null) 
			Helper.setSize(_model, size);
		else 
			Debug.errorln ("_model is null in LegendFigure");
    }

    protected void generateHeader (Graphics graphics) {
    	AcmeLegend legend = (AcmeLegend )_model;
    	legend.setColumnWidth(AcmeLegend.COMPONENT_COLUMN, -1);
    	legend.setColumnWidth(AcmeLegend.CONNECTOR_COLUMN, -1);
    	legend.setColumnWidth(AcmeLegend.PORT_COLUMN, -1);
    	legend.setColumnWidth(AcmeLegend.ROLE_COLUMN, -1);
    	graphics.pushState();
    	Font classFont = new Font(null, "TimesNewRoman", 10, SWT.BOLD);
    	int iNumColumns=0;
    	LegendLayout layout = (LegendLayout)_pane.getLayoutManager();
		int iNumComponents=(layout).getNumComponents ();
    	int iNumConnectors=(layout).getNumConnectors ();
    	int iNumRoles=(layout).getNumRoles ();
    	int iNumPorts=(layout).getNumPorts ();
    	


    	int iComponentX=(layout).getComponentX ();
    	int iComponentWidth = layout.getComponentWidth ();
    	int iConnectorX=(layout).getConnectorX ();
    	int iConnectorWidth = layout.getConnectorWidth ();
    	int iPortX=(layout).getPortX ();
    	int iPortWidth = layout.getPortWidth ();
    	int iRoleX=(layout).getRoleX ();
    	int iRoleWidth = layout.getRoleWidth ();
    	
    	graphics.setFont(classFont);

    	//Rectangle rect = getBounds ().getCopy ();
    	Rectangle rect = getBounds ().getCopy ();
    	rect.crop (new Insets (2, 0, 2, 0));
    	graphics.fillRectangle (rect);
    	int prevX = 0;
    	
    	if(iNumComponents>0)
    	{	
    		graphics.drawText("Components",rect.x+iComponentX+10,rect.y+25);
    		iNumColumns++;
    		prevX = iConnectorX;
    		legend.setColumnWidth (AcmeLegend.COMPONENT_COLUMN, iComponentWidth);
//    		Debug.diagnoseln ("LegendFigure.generateHeader: " + "comp width = " + legend.getColumnWidth(AcmeLegend.COMPONENT_COLUMN));
    		//graphics.drawLine(rect.x+(iNumColumns*iColumnWidth)+30,rect.y+15,rect.x+(iNumColumns*iColumnWidth)+30,rect.y+rect.height-20);
    	}
    	if(iNumConnectors>0)
    	{	
    		graphics.drawText("Connectors",rect.x+iConnectorX+10,rect.y+25);
    		legend.setColumnWidth (AcmeLegend.CONNECTOR_COLUMN, iConnectorWidth);
    		prevX += iPortX;
//    		if (iNumColumns > 0) graphics.drawLine(rect.x+iConnectorX+5,rect.y+15,rect.x+iConnectorX+5,rect.y+rect.height-20);
//    		Debug.diagnoseln ("LegendFigure.generateHeader: " + "conn width = " + legend.getColumnWidth(AcmeLegend.CONNECTOR_COLUMN));
    		iNumColumns++;
    	}
    	if(iNumPorts>0)
    	{	
    		graphics.drawText("Ports",rect.x+iPortX+10,rect.y+25);
    		legend.setColumnWidth (AcmeLegend.PORT_COLUMN, iPortWidth);
    		prevX += iRoleX;
//    		if (iNumColumns > 0) graphics.drawLine(rect.x+iPortX+5,rect.y+15,rect.x+iPortX+5,rect.y+rect.height-20);
//    		Debug.diagnoseln ("LegendFigure.generateHeader: " + "port width = " + legend.getColumnWidth(AcmeLegend.PORT_COLUMN));
    		iNumColumns++;
    	}
    	if(iNumRoles>0)
    	{	
    		graphics.drawText("Roles",rect.x+iRoleX+10,rect.y+25);
    		legend.setColumnWidth (AcmeLegend.ROLE_COLUMN, iRoleWidth);
//    		if (iNumColumns > 0) graphics.drawLine(rect.x+iRoleX+5,rect.y+15,rect.x+iRoleX+5,rect.y+rect.height-20);
//    		Debug.diagnoseln ("LegendFigure.generateHeader: " + "role width = " + legend.getColumnWidth(AcmeLegend.ROLE_COLUMN));
    		iNumColumns++;
    	}
    	classFont.dispose();
    	graphics.popState ();
    }
    
    protected void paintFigure (Graphics graphics) {
        if (AcmeDiagramLegendPlugin.getDefault().getPreferenceStore().getInt("LegendItem.Layout")==AcmeDiagramLegendPlugin.LAYOUT_LEGEND)
        {	
        	generateHeader(graphics);
        }
    }

    /**
     * @param legend
     */
    public void setModel(AcmeElement model) {
        this._model = model;
        if (_pane.getLayoutManager() instanceof LegendLayout) {
			LegendLayout layout = (LegendLayout) _pane.getLayoutManager();
			layout.setLegend ((AcmeLegend )model);
		}
    }
    
    public AcmeElement getModel () {
        return _model;
    }

    /* (non-Javadoc)
     * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
     */
    protected boolean useLocalCoordinates() {
        return true;
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
	 */
	public void changeModel(AcmeStudioChangeEvent evt) {
		if (Helper.eventRelatedToDesign(evt, _model.getParentDesign())) {
			switch (evt.getEventType()) {
				case AcmeStudioChangeManager.SET_PROPERTY:
					if (evt.getTarget().equals (_model)) {
						Point location = getLocation();
						AcmeProperty visProp = (AcmeProperty) evt.getSource ();
						if (visProp.getName ().equals ("vis-x")) {
							location.x = (int) ((AcmeFloat )visProp.getValue ()).getValue ();
							setLocation (location);
							this.invalidate();
						}
						else if (visProp.getName ().equals ("vis-y")) {
							location.y = (int) ((AcmeFloat )visProp.getValue ()).getValue ();
							setLocation (location);
							this.invalidate ();
						}
					}
				case AcmeStudioChangeManager.UPDATE_VIS_INFO:
					if (evt.getTarget ().equals (_model)) {
						Point location = getLocation ();
						VisualInformation vis = (VisualInformation )evt.getSource ();
						location.x = vis.getX();
						location.y = vis.getY ();
						setLocation (location);
						this.invalidate ();
					}
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#addNotify()
	 */
	public void addNotify() {
		super.addNotify();
		AcmeStudioChangeManager.registerListener(this);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#removeNotify()
	 */
	public void removeNotify() {
		super.removeNotify();
		AcmeStudioChangeManager.removeListener(this);
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.util.AcmeModelChangeListener#interestedInEvent(int)
	 */
	public boolean interestedInEvent(int eventType) {
		return eventType == AcmeStudioChangeManager.SET_PROPERTY || eventType == AcmeStudioChangeManager.UPDATE_VIS_INFO;
	}
}
