/*
 * Created on Nov 3, 2003
 *
 */
package AcmeDiagramLegend.model;

import java.util.*;

import AcmeDiagramLegend.AcmeDiagramLegendPlugin;
import AcmeStudio.ui.editors.diagram.IDiagramModelExtension;
import AcmeStudio.util.Helper;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.extensions.IElementExtension;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.elements.*;
import acmeLib.tools.GraphWalker;
import acmeLib.tools.WalkerElementOperation;

/**
 * @author Bradley Schmerl
 * Created Nov 3, 2003
 */
public class AcmeLegend extends AcmeElement implements IElementExtension, IDiagramModelExtension {

	protected Object _inFrontOf;
	
    protected List _items = new ArrayList ();
	private int _layout;
	public int _num_legend_items,_num_components,_num_connectors;
	public int _num_ports,_num_roles,_num_others,_legend_height,_legend_width;
	//public LegendLayout _manager;
	private String _location;
	
	public final static int COMPONENT_COLUMN = 0;
	public final static int CONNECTOR_COLUMN = 1;
	public final static int PORT_COLUMN = 2;
	public final static int ROLE_COLUMN = 3;
	
	private int [] colWidths = {200, 200, 150, 150};
	private ColumnSeparator [] separators = new ColumnSeparator [4];
	
	/**
	 * 
	 */
	public AcmeLegend() {
		super("Legend");
		
		_num_legend_items=0;
		_num_components=0;
		_num_connectors=0;
		_num_ports=0;
		_num_roles=0;
		_num_others=0;
		
		initializeSeparators ();
	}

	private void initializeSeparators() {
		addRowSeparator (COMPONENT_COLUMN);
		addRowSeparator (ROLE_COLUMN);
		addRowSeparator (CONNECTOR_COLUMN);
		addRowSeparator (PORT_COLUMN);
	}

	/**
	 * @param name
	 */
	public AcmeLegend(String name) {
		super(name);
	
		_num_legend_items=0;
		_num_components=0;
		_num_connectors=0;
		_num_ports=0;
		_num_roles=0;
		_num_others=0;
		
		initializeSeparators ();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#copy()
	 */
	public AcmeElement copy() {
		return null;
	}

    /**
     * Generates a legend from the types used in the AcmeSystem and adds it as UserData to that system
     * @param system The system to construct the legend for
     */
    public static AcmeLegend generateFromSystem (AcmeSystem system) {
    	AcmeLegend legend = new AcmeLegend ();
    	final Map types = new HashMap ();
        WalkerElementOperation op = new WalkerElementOperation () {

            public void doElementOp(AcmeElement elt) {
                if (elt instanceof AcmeSystem) return;
                AcmeEnumeration dts = elt.getDeclaredTypes();
                if (dts.getSize () > 0) {
                	AcmeObject obj = dts.getNextItem ();
                	if (obj instanceof AcmeElement) {
                		AcmeElement et = (AcmeElement )obj;
	                	if (!types.containsKey(et.getQualifiedName())) {
	                		types.put (et.getQualifiedName (), et);
	                	}
                	}
                }
                else {
                    if (elt instanceof AcmeComponent) {
                        types.put (ConstantTypes.COMPONENT_BASE_TYPE.getQualifiedName(), ConstantTypes.COMPONENT_BASE_TYPE);
                    }
                    else if (elt instanceof AcmeConnector) {
                        types.put (ConstantTypes.CONNECTOR_BASE_TYPE.getQualifiedName(), ConstantTypes.CONNECTOR_BASE_TYPE);
                        
                    }
                    else if (elt instanceof AcmeRole) {
                        types.put (ConstantTypes.ROLE_BASE_TYPE.getQualifiedName (), ConstantTypes.ROLE_BASE_TYPE);
                    }
                    else if (elt instanceof AcmePort) {
                        types.put (ConstantTypes.PORT_BASE_TYPE.getQualifiedName (), ConstantTypes.PORT_BASE_TYPE);
                    }
                    
                }
                
            }
            
        };
        
        GraphWalker.WalkOverAllElements(system, op, GraphWalker.WALK_TOP_DOWN);
        

        
        for (Iterator iter = types.keySet ().iterator(); iter.hasNext();) {
            AcmeElementType type = (AcmeElementType )types.get (iter.next());
            legend.addLegendItem (new AcmeLegendItemDescription (type));
            //System.out.println("AL:Type=="+type.);
            if (type instanceof AcmeComponent) {
            	legend._num_components++;
            }
            if (type instanceof AcmeConnector) {
            	legend._num_connectors++;
            }
            if (type instanceof AcmeRole) {
            	legend._num_roles++;
            }
            if (type instanceof AcmePort) {
            	legend._num_ports++;
            }
            legend._num_legend_items++;
        }
        

        
        
        legend.setParent(system);
        legend.setParentDesign(system.getParentDesign ());
        
        system.setUserData(AcmeDiagramLegendPlugin.class, legend);
        
        if (AcmeDiagramLegendPlugin.getDefault ().getPreferenceStore ().getBoolean ("LegendItem.UseLargeIcons")) {
        	legend.colWidths [COMPONENT_COLUMN] = 250;
        }
        
        return legend;
        
    }

    /**
	 * @param component_column2
	 */
	public void addRowSeparator(int column) {
		ColumnSeparator separator = new ColumnSeparator (this, column);
		_items.add (separator);
		separators [column] = separator;
	}

	/**
     * @param item
     */
    public void addLegendItem(AcmeLegendItemDescription item) {
    	AcmeElementType type = item._type;
    	AcmeElement element = item.getElement();
    	AcmeLegendItemIcon item2 = null;
    	if (type != null) item2 = new AcmeLegendItemIcon (item._type);
    	else item2 = new AcmeLegendItemIcon (element);
    	item2.setItemDescription(item);
    	item.setItemIcon(item2);
    	_items.add(item2);
    	_items.add(item);
    	
    	if (type instanceof AcmeComponentType || element instanceof AcmeComponent) {
			getRowSeparator (COMPONENT_COLUMN).addMember (item);
		}
    	else if (type instanceof AcmeConnectorType || element instanceof AcmeConnector) {
    		getRowSeparator (CONNECTOR_COLUMN).addMember (item);
    	}
    	else if (type instanceof AcmeRoleType || element instanceof AcmeRole) {
    		getRowSeparator (ROLE_COLUMN).addMember (item);
    	}
    	else if (type instanceof AcmePortType || element instanceof AcmePort) {
    		getRowSeparator (PORT_COLUMN).addMember (item);
    	}
    	
    	
    }
    
    /**
	 * @param component_column2
	 * @return
	 */
	private ColumnSeparator getRowSeparator(int col) {
		return separators [col];
	}

	public List getItems () {
        return _items;
    }

    /* (non-Javadoc)
     * @see acmeLib.core.extensions.IElementExtension#isPersistent()
     */
    public boolean isPersistent() {
        return true;
    }

    /* (non-Javadoc)
     * @see acmeLib.core.extensions.IElementExtension#injectIntoElement(acmeLib.elements.AcmeElement)
     */
    public void injectIntoElement(AcmeElement element) {
    }

    /* (non-Javadoc)
     * @see acmeLib.core.extensions.IElementExtension#extractFromElement(acmeLib.elements.AcmeElement)
     */
    public void extractFromElement(AcmeElement element) {
    }

    /* (non-Javadoc)
     * @see acmeLib.core.extensions.IElementExtension#isInjected(acmeLib.elements.AcmeElement)
     */
    public boolean isInjected(AcmeElement element) {
        return false;
    }

	/**
	 * @param layout
	 */
	public void setLayout(int layout) {
		_layout = layout;
	}
    
  
	/**
	 * @return Returns the layout.
	 */
	public int getLayout() {
		return _layout;
	}

	/**
	 * @param initialPosition
	 */
	public void setLocation(String initialPosition) {
		_location = initialPosition;
	}
	/**
	 * @return Returns the location.
	 */
	public String getLocation() {
		return _location;
	}
	
	public int getColumnWidth (int column) {
		return colWidths [column];
	}
	
	public void setColumnWidth (int column, int width) {
		colWidths [column] = width;
	}

	/**
	 * @param i
	 * @return
	 */
	public int getColumnStart(int col) {
		int start = 0;
		int s = 0;
		while (start == 0) {
			if (colWidths [s] > 0) start = colWidths [s];
			else s++;
		}
	
		for (int i = s+1; i <= col; i++) {
			start += colWidths [i];
		}
		return start;
	}
	
	public void removeItem (Object item) {
		_items.remove (item);
		if (item instanceof AcmeLegendItemDescription) {
			AcmeLegendItemDescription desc = (AcmeLegendItemDescription )item;
			AcmeElementType type = desc._type;
			
			if (type instanceof AcmeComponentType) {
				getRowSeparator (COMPONENT_COLUMN).removeMember (desc);
				
			}
			else if (type instanceof AcmeConnectorType) {
				getRowSeparator (CONNECTOR_COLUMN).removeMember (desc);
			}
			else if (type instanceof AcmeRoleType) {
				getRowSeparator (ROLE_COLUMN).removeMember (desc);
			}
			else if (type instanceof AcmePortType) {
				getRowSeparator (PORT_COLUMN).removeMember (desc);
			}
			
		}
	}

	/**
	 * @param type
	 * @return
	 */
	public boolean containsType(AcmeElementType type) {
		boolean found = false;
		for (Iterator iter = _items.iterator (); iter.hasNext () && !found; ) {
			Object o = iter.next ();
			if (o instanceof AcmeLegendItemDescription) {
				AcmeLegendItemDescription desc = (AcmeLegendItemDescription) o;
				if (desc.getType().getTypeName().equals (type.getTypeName ())) {
					found = true;
				}
			}
			
		}
		return found;
	}

	/**
	 * @return
	 * @see AcmeStudio.ui.editors.diagram.IDiagramModelExtension#getInFrontOf()
	 **/
	public Object getInFrontOf() {
		return _inFrontOf;
	}

	/**
	 * 
	 * @see AcmeStudio.ui.editors.diagram.IDiagramModelExtension#setInFrontOf()
	 **/
	public void setInFrontOf(Object inFrontOf) {
		_inFrontOf = inFrontOf;
	}
	
	
	/**
	 * @return
	 * @see AcmeStudio.ui.editors.diagram.IDiagramModelExtension#getZOrder()
	 **/
	public int getZOrder() {
		return Helper.getZOrder(this);
	}
	
	public void setZOrder (int order) {
		Helper.setZOrder(this, order);
	}
}
