/*
 * Created on Nov 5, 2003
 * 
 */
package AcmeDiagramLegend.parts;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.tools.CellEditorLocator;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Text;

import AcmeDiagramLegend.figures.LegendItemLabel;

/**
 * @author Bradley Schmerl
 * 
 * Locates a text editor next to the cell editor
 *
 */
public final class LabelCellEditorLocator implements CellEditorLocator {
	private static int WIN_X_OFFSET = -4;
	private static int WIN_W_OFFSET = 5;
	private static int GTK_X_OFFSET = 0;
	private static int GTK_W_OFFSET = 0;
	private static int MAC_X_OFFSET = -3;
	private static int MAC_W_OFFSET = 9;
	private static int MAC_Y_OFFSET = -3;
	private static int MAC_H_OFFSET = 6;
    private LegendItemLabel _label;
    
    public LabelCellEditorLocator (LegendItemLabel label) {
        setLabel (label);
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.tools.CellEditorLocator#relocate(org.eclipse.jface.viewers.CellEditor)
     */
    public void relocate(CellEditor celleditor) {
    	if((_label.getText() == "") /*&& 
    			(_label.getType() instanceof AcmeComponent ||
    			_label.getType () instanceof AcmeConnector ||
    			_label.getType () instanceof AcmePort ||
    			_label.getType () instanceof AcmeRole)*/)
    			
    	{
    		Text text = (Text )celleditor.getControl ();
    		Rectangle rect = _label.getClientArea().getCopy ();
    		_label.translateToAbsolute(rect);
    		text.setBounds(rect.x , rect.y, 0 , 0);
    		
    	}
    	else
    	{	
    		Text text = (Text )celleditor.getControl ();
        	
        	Rectangle rect = _label.getClientArea ().getCopy();
        	_label.translateToAbsolute(rect);
        	int xOffset = 0;
    		int wOffset = 0;
    		int yOffset = 0;
    		int hOffset = 0;
        	if (SWT.getPlatform().equalsIgnoreCase("gtk")) {
        		xOffset = GTK_X_OFFSET;
        		wOffset = GTK_W_OFFSET;
        	}
        	else if (SWT.getPlatform().equalsIgnoreCase("carbon")) {
    			xOffset = MAC_X_OFFSET; yOffset = MAC_Y_OFFSET; wOffset = MAC_W_OFFSET; hOffset = MAC_H_OFFSET;
    		}
        	else {
        		xOffset = WIN_X_OFFSET;
        		wOffset = WIN_W_OFFSET;
        	}
        	text.setBounds(rect.x + xOffset, rect.y+yOffset, rect.width + wOffset, 100 + hOffset);

    		}
    	}

    /**
     * @return Returns the label.
     */
    public LegendItemLabel getLabel() {
        return _label;
    }

    /**
     * @param label The label to set.
     */
    public void setLabel(LegendItemLabel label) {
        _label = label;
    }

}
