/*
 * Created on Nov 4, 2003
 * 
 */
package AcmeDiagramLegend.parts;

import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.FigureListener;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.AutoexposeHelper;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.ExposeHelper;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.ViewportAutoexposeHelper;
import org.eclipse.gef.editparts.ViewportExposeHelper;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;

import AcmeDiagramLegend.AcmeDiagramLegendPlugin;
import AcmeDiagramLegend.figures.LegendFigure;
import AcmeDiagramLegend.figures.LegendItemIcon;
import AcmeDiagramLegend.figures.LegendItemLabel;
import AcmeDiagramLegend.model.AcmeLegend;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import AcmeDiagramLegend.model.AcmeLegendItemIcon;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.ElementEditPart;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl
 * An edit part that represents a legend in the diagram. An edit part represents the controller in the MVC pattern.
 */
public class LegendEditPart
    extends ElementEditPart implements FigureListener, AcmeModelChangeListener {
  

    /**
     * Creates the view of this controller
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    protected IFigure createFigure() {
        LegendFigure f = new LegendFigure ();
        f.setLabel ("Legend:"); 
        f.addFigureListener(this);
        return f;
    }



    /* (non-Javadoc)
     * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class key) {
        if (key == AutoexposeHelper.class) {
            return new ViewportAutoexposeHelper (this);
        }
        else if (key == ExposeHelper.class) {
            return new ViewportExposeHelper (this);
        }
        return super.getAdapter(key);
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.GraphicalEditPart#getContentPane()
     */
    public IFigure getContentPane() {
       return  ((LegendFigure )getFigure ()).getContentsPane ();
    }

    /**
     * Refreshes the views associated with this legend.
     * Also, creates a layout constraint for this part, giving its location in the diagram
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    protected void refreshVisuals() {
        if (getModel () == null) return;
        
        LegendFigure f = (LegendFigure )getFigure ();
        AcmeLegend legend = (AcmeLegend )getModel ();
        f.setModel (legend);
        Point location = getLocation (legend);
       
        Dimension size = Helper.getSize (legend);
        if (size == null) size = new Dimension (500, 500);
        Rectangle r = new Rectangle (location, size);
        f.setBounds (r);
//            f.setSize (size);
//        
//        f.setLocation (location);
 
        ((GraphicalEditPart ) getParent ()).setLayoutConstraint (this, getFigure (), r);
        
    }

    /**
	 * @param legend
	 * @return
	 */
	private Point getLocation(AcmeLegend legend) {
		Point location = SystemView.getLocation (legend, false);
		Dimension size = Helper.getSize (legend);
		if (location == null || (location.x == 0 && location.y ==0)) { // Need to calculate based on system bounds
			AcmeSystem system = (AcmeSystem )legend.getParent ();
			system.setUserData (AcmeDiagramLegendPlugin.class, null);
			Rectangle systemBounds = SystemView.getSystemBounds(system);
			system.setUserData(AcmeDiagramLegendPlugin.class, legend);
			String position = legend.getLocation ();
			if (position == null || position == "") {
				position = AcmeDiagramLegendPlugin.getDefault().getPreferenceStore().getString ("AcmeLegend.legendPosition");
				legend.setLocation (position);
			}
			location = new Point (0,0);
			if (position.equals (AcmeDiagramLegendPlugin.ABOVE)) {
				location.y = systemBounds.y - size.height - 20;
				location.x = systemBounds.x + (systemBounds.width - size.width)/2;
			}
			else if (position.equals (AcmeDiagramLegendPlugin.BELOW)) {
				location.y = systemBounds.y + systemBounds.height + 20;
				location.x = systemBounds.x + (systemBounds.width - size.width)/2;
			}
			else if (position.equals (AcmeDiagramLegendPlugin.RIGHT)) {
				location.x = systemBounds.x + size.width  + systemBounds.width + 20;
				location.y = systemBounds.y + (systemBounds.height - size.height)/2;
			}
			else if (position.equals (AcmeDiagramLegendPlugin.LEFT)) {
				location.x = systemBounds.x - size.width - 20;
				location.y = systemBounds.y - (systemBounds.height - size.height)/2;
			}
		}
		else {
			location.x = location.x - size.width/2;
			location.y = location.y -size.height/2;
		}
		return location;
	}



	/* (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
     */
    protected List getModelChildren() {
        AcmeLegend l = (AcmeLegend )getModel ();
        return l.getItems();
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    protected void createEditPolicies() {
        super.createEditPolicies();
//        installEditPolicy (EditPolicy.SELECTION_FEEDBACK_ROLE, new LegendHighlightEditPolicy ());
        
        // Sets the layout depending on preferences
        int layout = AcmeDiagramLegendPlugin.getDefault().getPreferenceStore().getInt("LegendItem.Layout");
        EditPolicy layoutPolicy = null;
        switch (layout) {
        	case AcmeDiagramLegendPlugin.LAYOUT_FREEFORM:
        		layoutPolicy = new LegendXYLayoutEditPolicy ();
        		break;
        	case AcmeDiagramLegendPlugin.LAYOUT_TABLE:
        		layoutPolicy = new LegendTableLayoutEditPolicy ();
        		break;
        	case AcmeDiagramLegendPlugin.LAYOUT_LEGEND:
        		layoutPolicy = new LegendLayoutEditPolicy ();
        		break;
            case AcmeDiagramLegendPlugin.LAYOUT_FLOW_HORIZONTAL:
            case AcmeDiagramLegendPlugin.LAYOUT_FLOW_VERTICAL:
            default:
                layoutPolicy = new LegendFlowEditPolicy ();
        }       
        changeLayoutPolicy (layoutPolicy);
 
    }

    /**
     * Changes the layout policy and adjusts child items as necessary
     * @param policy
     */
    public void changeLayoutPolicy(EditPolicy policy) {
        installEditPolicy (EditPolicy.LAYOUT_ROLE, policy);
        
        if (!(policy instanceof XYLayoutEditPolicy)) {
            List lChildren = getChildren();
            for (Iterator iter = lChildren.iterator(); iter.hasNext();) {
                LegendItemDescriptionEditPart cep = (LegendItemDescriptionEditPart) iter.next();
                AcmeLegendItemDescription item = (AcmeLegendItemDescription )cep.getModel();
                item.setLocation(null);
            }
        }
    }



    /* (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshChildren()
     */
    protected void refreshChildren() {
        List lChildren = getChildren ();
        for (Iterator iter = lChildren.iterator(); iter.hasNext();) {
        	Object p = iter.next ();
        	if (p instanceof LegendItemDescriptionEditPart) {
				LegendItemDescriptionEditPart part = (LegendItemDescriptionEditPart) p;
				AcmeLegendItemDescription desc = (AcmeLegendItemDescription )part.getModel ();
				LegendItemLabel lFigure = (LegendItemLabel )part.getFigure ();
				if (desc.getLocation () != null) {
					lFigure.setLocation (desc.getLocation ());
				}
				part.refreshVisuals();
			}
        	else if (p instanceof LegendItemIconEditPart) {
        		LegendItemIconEditPart part = (LegendItemIconEditPart) p;
				AcmeLegendItemIcon icon = (AcmeLegendItemIcon )part.getModel ();
				LegendItemIcon lFigure = (LegendItemIcon )part.getFigure ();
				if (icon.getLocation () != null) {
					lFigure.setLocation (icon.getLocation ());
				}
				part.refreshVisuals();
			}
        	else if (p instanceof ColumnSeparatorEditPart){
        		ColumnSeparatorEditPart part =  (ColumnSeparatorEditPart )p;
        		part.refreshVisuals();
        	}
        }
        super.refreshChildren();
        
    }

	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#setModel(java.lang.Object)
	 */
	public void setModel(Object model) {
		super.setModel(model);
		AcmeLegend legend = (AcmeLegend )model;
		int layout = legend.getLayout ();
		switch (layout) {
			case AcmeDiagramLegendPlugin.LAYOUT_FLOW_HORIZONTAL:
				changeLayoutPolicy (new LegendFlowEditPolicy ());
				break;
			case AcmeDiagramLegendPlugin.LAYOUT_FLOW_VERTICAL:
				changeLayoutPolicy (new LegendFlowEditPolicy ());
				break;
			case AcmeDiagramLegendPlugin.LAYOUT_FREEFORM:
				changeLayoutPolicy(new LegendXYLayoutEditPolicy ());
				break;
			case AcmeDiagramLegendPlugin.LAYOUT_TABLE:
				changeLayoutPolicy(new LegendTableLayoutEditPolicy ());
				break;
			case AcmeDiagramLegendPlugin.LAYOUT_LEGEND:
				changeLayoutPolicy(new LegendFlowEditPolicy ());
				break;
			}
	}


	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.FigureListener#figureMoved(org.eclipse.draw2d.IFigure)
	 */
	public void figureMoved(IFigure source) {
		if (source instanceof LegendFigure) {
			this.refresh ();
		}
	}



	/* (non-Javadoc)
	 * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
	 */
	public void changeModel(AcmeStudioChangeEvent evt) {
		if (Helper.eventRelatedToDesign(evt, ((AcmeLegend )getModel ()).getParentDesign())) {
			switch (evt.getEventType ()) {
				case AcmeStudioChangeManager.ADD_COMPONENT:
				case AcmeStudioChangeManager.ADD_PORT:
				case AcmeStudioChangeManager.ADD_ROLE:
				case AcmeStudioChangeManager.ADD_CONNECTOR:
					AcmeElement source = (AcmeElement )evt.getSource ();
					if (source.getDeclaredTypes ().getSize () > 0) {
						AcmeElementType type = (AcmeElementType )source.getDeclaredTypes ().getNextItem ();
						AcmeLegend legend = (AcmeLegend )getModel ();
						if (!legend.containsType (type)) {
							legend.addLegendItem(new AcmeLegendItemDescription (type));
						}
					}
			}
			this.refresh ();
		}
	}
	
	

	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#addNotify()
	 */
	public void addNotify() {
		super.addNotify ();
		AcmeStudioChangeManager.registerListener(this);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#removeNotify()
	 */
	public void removeNotify() {
		super.removeNotify ();
		AcmeStudioChangeManager.removeListener(this);
	}



	/* (non-Javadoc)
	 * @see AcmeStudio.util.AcmeModelChangeListener#interestedInEvent(int)
	 */
	public boolean interestedInEvent(int eventType) {
		switch (eventType) {
			case AcmeStudioChangeManager.ADD_COMPONENT:
			case AcmeStudioChangeManager.ADD_PORT:
			case AcmeStudioChangeManager.ADD_ROLE:
			case AcmeStudioChangeManager.ADD_CONNECTOR:
				return true;
		}
		return false;
	}
}
