/*
 * Created on Nov 5, 2003
 * 
 */
package AcmeDiagramLegend.parts;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.FlowLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;

import AcmeDiagramLegend.actions.ItemCommandFactory;
import AcmeDiagramLegend.model.AcmeLegend;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import AcmeDiagramLegend.model.ReorderPartCommand;

/**
 * @author Bradley Schmerl
 * 
 * Defines the policies for sub-component manipulations for adding legend items to the legend.
 * Specifically, this defines policies for the flow layout of the legend.
 *
 */
public class LegendFlowEditPolicy extends FlowLayoutEditPolicy {

    /* (non-Javadoc)
     * @see org.eclipse.gef.editpolicies.OrderedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, org.eclipse.gef.EditPart)
     */
    protected Command createAddCommand(EditPart child, EditPart after) {
        return null;
    }

    /**
     * Creates the command for moving a legend item
     * @param child the item to move
     * @param after the item that this child is moved after
     * @return the command to reorder the children
     */
    protected Command createMoveChildCommand(EditPart child, EditPart after) {
        if (child instanceof LegendItemDescriptionEditPart && after instanceof LegendItemDescriptionEditPart) {
            LegendItemDescriptionEditPart cEp = (LegendItemDescriptionEditPart )child;
            AcmeLegendItemDescription childModel = (AcmeLegendItemDescription )cEp.getModel ();
            AcmeLegend legend = (AcmeLegend )getHost ().getModel();
            int oldIndex = getHost ().getChildren ().indexOf(child);
            int newIndex = getHost ().getChildren ().indexOf (after);
            if (newIndex > oldIndex) newIndex --;
            ReorderPartCommand command = new ReorderPartCommand (childModel, legend, oldIndex, newIndex);
            return command;
        }
        return null;
    }

    /**
     * Returns the command to create a new legend item
     * @param request The create request that defines the command
     * @return the create command
     * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
     */
    protected Command getCreateCommand(CreateRequest request) {
        return ItemCommandFactory.getCreateCommand(request, getHost (), null);
        
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
     */
    protected Command getDeleteDependantCommand(Request request) {
        return null;
    }

}
