/*
 * Created on Nov 5, 2003
 * 
 */
package AcmeDiagramLegend.parts;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.AccessibleEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.swt.accessibility.AccessibleControlEvent;
import org.eclipse.swt.accessibility.AccessibleEvent;
import AcmeDiagramLegend.figures.LegendItemLabel;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import AcmeStudio.ui.editors.diagram.parts.ElementEditPart;

/**
 * @author Bradley Schmerl
 *
 */
public class LegendItemDescriptionEditPart extends ElementEditPart {

    protected AccessibleEditPart createAccessible () {
        return new AccessibleGraphicalEditPart () {
          public void getValue (AccessibleControlEvent e) {
              e.result = ((AcmeLegendItemDescription )getModel ()).getDescription();
          }
          
          public void getName (AccessibleEvent e) {
              e.result = "Legend item";
          }
        };
    }
    
    private DirectEditManager _manager;
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    protected IFigure createFigure() {
        LegendItemLabel label = new LegendItemLabel ();
        if (getModel () != null) {
        	label.setText(((AcmeLegendItemDescription )getModel ()).getDescription());
        }
        return label;
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    protected void createEditPolicies() {
        installEditPolicy(EditPolicy.COMPONENT_ROLE, new LegendElementEditPolicy ());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LegendItemDirectEditPolicy ());
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    protected void refreshVisuals() {
        AcmeLegendItemDescription item = (AcmeLegendItemDescription )getModel ();
        
        LegendItemLabel label = (LegendItemLabel )getFigure ();
        label.setText (item.getDescription ());
        if (item.getLocation() != null) label.setLocation (item.getLocation ());
        label.setSize (label.getPreferredSize(item.getWidth(), -1));
        super.refreshVisuals();
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
     */
    public void performRequest(Request req) {
        if (req.getType () == RequestConstants.REQ_DIRECT_EDIT) {
            if (_manager == null) {
                _manager = new LegendLabelEditManager (this, LegendLabelCellEditor.class, new LabelCellEditorLocator ((LegendItemLabel )getFigure ()));
            }
            _manager.show ();
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.EditPart#setModel(java.lang.Object)
     */
    public void setModel(Object model) {
        super.setModel(model);
        AcmeLegendItemDescription item = (AcmeLegendItemDescription )model;
        LegendItemLabel label = (LegendItemLabel )getFigure ();
        label.setModel ((AcmeLegendItemDescription )model);
//        label.setType(item.getType ());
   

    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.EditPart#deactivate()
     */
    public void deactivate() {
        super.deactivate();
//        if (_vis != null && _visualizationChangedListener != null) {
//            _vis.removePropertyChangeListener(null, _visualizationChangedListener);
//            _visualizationChangedListener = null;
//        }
    }

}
