/*
 * Created on Nov 5, 2003
 * 
 */
package AcmeDiagramLegend.parts;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.DirectEditPolicy;
import org.eclipse.gef.requests.DirectEditRequest;

import AcmeDiagramLegend.model.AcmeLegendItemDescription;
import AcmeStudio.model.ModelCommand;

/**
 * @author Bradley Schmerl
 * Specifies the policy for editing the labels associated with each legend item. 
 *
 */
public class LegendItemDirectEditPolicy extends DirectEditPolicy {

    /**
     * A command to change the description of the legend item (allows it to be undone, etc)
     *
     */
    protected static class ChangeDescriptionCommand extends ModelCommand {

        private String _newDescription;
        private AcmeLegendItemDescription _item;
        private LegendItemDescriptionEditPart _part;
        private String _oldDescription;
      
        /**
         * @param label
         */
        public ChangeDescriptionCommand(LegendItemDescriptionEditPart item, String text) {
        	super ("Change description for legend item");
            _item = (AcmeLegendItemDescription )item.getModel ();
	        String label = "Change description for ";
	        if (_item.getType () != null) 
	        	label += _item.getType ().getTypeName ();
	        else 
	        	label += "legend item";
           setLabel (label);
        
            _part = item;
            if (text != null) {
                _newDescription = text.replaceAll ("\r","");
            }
            else {
                _newDescription = "";
            }
        }
        
        /* (non-Javadoc)
         * @see org.eclipse.gef.commands.Command#execute()
         */
        public void execute() {
            _oldDescription = _item.getDescription();
            _item.setDescription (_newDescription);
            _part.refresh();
        }

        /* (non-Javadoc)
         * @see org.eclipse.gef.commands.Command#redo()
         */
        public void redo() {
            execute ();
        }

        /* (non-Javadoc)
         * @see org.eclipse.gef.commands.Command#undo()
         */
        public void undo() {
            _item.setDescription (_oldDescription);
            _part.refresh ();
        }

    }
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.editpolicies.DirectEditPolicy#getDirectEditCommand(org.eclipse.gef.requests.DirectEditRequest)
     */
    protected Command getDirectEditCommand(DirectEditRequest request) {
        String labelText = (String )request.getCellEditor().getValue ();
        LegendItemDescriptionEditPart item = (LegendItemDescriptionEditPart )getHost ();
        
        return new ChangeDescriptionCommand (item, labelText);
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.editpolicies.DirectEditPolicy#showCurrentEditValue(org.eclipse.gef.requests.DirectEditRequest)
     */
    protected void showCurrentEditValue(DirectEditRequest request) {
        getHostFigure().getUpdateManager().performUpdate();
    }

}
