/*
 * Created on Nov 5, 2003
 * 
 */
package AcmeDiagramLegend.parts;

import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.tools.CellEditorLocator;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Text;
import AcmeDiagramLegend.figures.LegendItemLabel;

/**
 * @author Bradley Schmerl
 *
 */
public class LegendLabelEditManager extends DirectEditManager {
    
    /**
     * @param source
     * @param editorType
     * @param locator
     */
    public LegendLabelEditManager(GraphicalEditPart source, Class editorType, CellEditorLocator locator) {
        super(source, editorType, locator);
    }


    Font _scaledFont;
    
    private class LocalVerifyListener implements VerifyListener {
    	private Dimension _cSize;
    	
    	public void verifyText(VerifyEvent event) {
    		Text text = (Text )getCellEditor ().getControl ();
    		if (_cSize == null) _cSize = new Dimension (text.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    		String oldText = text.getText ();
    		String leftText = oldText.substring (0, event.start);
    		String rightText = oldText.substring (event.end, oldText.length ());
    		
    		String s = leftText + event.text + rightText;
    		Dimension tSize = FigureUtilities.getTextExtents (s, text.getFont ());
    		
    		Dimension size = null;
    		if (tSize.width > _cSize.width) {
    			size = tSize;
    		}
    		else {
    			size = _cSize;
    		}
    		Point location = text.getLocation ();
//    		Debug.diagnoseln ("LocalVerifyListener.verifyText: " + size.width);
//    		getCellEditor ().getControl ().setBounds (location.x, location.y, size.width, 300);
    	}
    	
    	public void resetSize () {
    		_cSize = null;
    	}
    	
    }
    
    private VerifyListener _verifyListener;

    /* (non-Javadoc)
     * @see org.eclipse.gef.tools.DirectEditManager#initCellEditor()
     */
    protected void initCellEditor() {
    	getEditPart().getFigure().setVisible(false);
        Text text = (Text )getCellEditor ().getControl ();
        
        _verifyListener = new LocalVerifyListener ();
        text.addVerifyListener(_verifyListener);
        
        LegendItemLabel label = (LegendItemLabel )getEditPart ().getFigure ();
        String initialLabelText = label.getText ();
        getCellEditor().setValue(initialLabelText);
        IFigure figure = getEditPart ().getFigure ();
        _scaledFont = figure.getFont ();
        FontData data = _scaledFont.getFontData() [0];
        Dimension fontSize = new Dimension (0, data.getHeight ());
        label.translateToAbsolute (fontSize);
        data.setHeight (fontSize.height);
        _scaledFont = new Font (null, data);
        
        text.setFont (_scaledFont);
        text.selectAll ();
    }

    protected void bringDown () {
        Font disposeFont = _scaledFont;
        _scaledFont = null;
        super.bringDown ();
        if (disposeFont != null) disposeFont.dispose();
        getEditPart ().getFigure().setVisible (true);
    }
    
    protected void unhookListeners () {
        super.unhookListeners();
        Text text = (Text )getCellEditor ().getControl ();
        text.removeVerifyListener(_verifyListener);
        _verifyListener = null;
    }


}
