/*
 * Created on Nov 6, 2003
 * 
 */
package AcmeDiagramLegend.parts;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;

import AcmeDiagramLegend.actions.ItemCommandFactory;
import AcmeDiagramLegend.actions.SetItemConstraintCommand;
import AcmeDiagramLegend.figures.LegendFigure;
import AcmeDiagramLegend.model.AcmeLegendItemDescription;

/**
 * @author Bradley Schmerl
 * Defines the policy for freeform layout of the legend item in a legend
 *
 */
public class LegendTableLayoutEditPolicy extends XYLayoutEditPolicy {

    /* (non-Javadoc)
     * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, java.lang.Object)
     */
    protected Command createAddCommand(EditPart child, Object constraint) {
        return null;
        
    }

    /**
     * Creates a command that moves a legend item within the legend
     * @param child the legend item to be moved
     * @param constraint the new position & size of the item
     * @return The command to change the location/size of the legend item
     * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
     */
    protected Command createChangeConstraintCommand(
        EditPart child,
        Object constraint) {
        if (child instanceof LegendItemDescriptionEditPart) {
            SetItemConstraintCommand locationCommand = new SetItemConstraintCommand ((LegendItemDescriptionEditPart )child);
            LegendEditPart lep = (LegendEditPart )getHost ();
            LegendFigure lf = (LegendFigure )lep.getFigure ();
            locationCommand.setLegendView (lf);
            locationCommand.setItem ((AcmeLegendItemDescription )child.getModel ());
            locationCommand.setLocation ((Rectangle )constraint);
            return locationCommand;
        }
        
        return null;
    }

    /**
      * Returns the command to create a new legend item
      * @param request The create request that defines the command
      * @return the create command
      * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
      */
     protected Command getCreateCommand(CreateRequest request) {
         // Find the new location
        Rectangle constraint = (Rectangle )getConstraintFor(request);
        Point location = constraint.getLocation ();
      
        return ItemCommandFactory.getCreateCommand(request, getHost (), location);   
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
     */
    protected Command getDeleteDependantCommand(Request request) {
        return null;
    }

    /*
    protected Object getConstraintFor(Point point) {
    	return null;
    }
    
    protected Object getConstraintFor(Rectangle rectangle) {
    	return null;
    }
    */
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy#getCurrentConstraintFor(org.eclipse.gef.GraphicalEditPart)
     */
    protected Rectangle getCurrentConstraintFor(GraphicalEditPart child) {
        Rectangle rect = null;//super.getCurrentConstraintFor(child);
        if (rect == null) {
            rect = new Rectangle (child.getFigure ().getBounds());
        }
        return rect;
    }

}
