/*
 * Created on Feb 5, 2004
 *
 */
package AcmeDiagramLegend.parts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.handles.AbstractHandle;
import org.eclipse.gef.handles.MoveHandleLocator;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.tools.DragEditPartsTracker;



/**
 * @author Bradley Schmerl
 * Created Feb 5, 2004
 */
public class SeparatorSelectionEditPolicy extends NonResizableEditPolicy  {

	
	private class SeparatorMoveHandle extends AbstractHandle {
		private boolean _moving;

		public SeparatorMoveHandle (GraphicalEditPart owner) {
			setOwner (owner);
			setLocator (new MoveHandleLocator (owner.getFigure ()));
			setCursor (Cursors.getDirectionalCursor (PositionConstants.EAST));
		}
		
		public void paintFigure (Graphics g) {
			Rectangle r = getBounds ();
			g.setBackgroundColor (ColorConstants.white);
			g.setForegroundColor (ColorConstants.lightBlue);
			g.setXORMode(isMoving ());
			g.drawLine (r.x + r.width/2, r.y, r.x + r.width/2, r.y + r.height);
			PointList leftTriangle = new PointList ();
			leftTriangle.addPoint (r.x + r.width-1, r.y + r.height/2 - 10);
			leftTriangle.addPoint (r.x + r.width-1, r.y + r.height/2);
			leftTriangle.addPoint (r.x, r.y + r.height/2 - 5);
			
			PointList rightTriangle = new PointList ();
			rightTriangle.addPoint (r.x, r.y +r.height/2);
			rightTriangle.addPoint (r.x, r.y + r.height/2 + 10);
			rightTriangle.addPoint (r.x + r.width-1, r.y + r.height/2 + 5);
			
			g.fillPolygon(leftTriangle);
			g.fillPolygon(rightTriangle);
			g.drawPolygon(leftTriangle);
			g.drawPolygon(rightTriangle);
		}

		/**
		 * @return
		 */
		private boolean isMoving() {
			return _moving;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.gef.handles.AbstractHandle#createDragTracker()
		 */
		protected DragTracker createDragTracker() {
			return new DragEditPartsTracker (getOwner ()) {
				// Makes sure that the request is restricted in the y direction
				protected void updateTargetRequest () {
					super.updateTargetRequest ();
					ChangeBoundsRequest request = (ChangeBoundsRequest )getTargetRequest();
					request.setMoveDelta(new Point (request.getMoveDelta().x, 0));
				}
			};
		}
		/**
		 * @param moving The moving to set.
		 */
		public void setMoving(boolean moving) {
			_moving = moving;
		}
	}
	/**
	 * 
	 */
	public SeparatorSelectionEditPolicy() {
		super ();
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.SelectionHandlesEditPolicy#createSelectionHandles()
	 */
	protected List createSelectionHandles() {
		List ret = new ArrayList ();
		SeparatorMoveHandle handle = new SeparatorMoveHandle ((GraphicalEditPart )getHost ());
		handle.setDragTracker (handle.createDragTracker());
		ret.add (handle);
		return ret;
		
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.NonResizableEditPolicy#createDragSourceFeedbackFigure()
	 */
	protected IFigure createDragSourceFeedbackFigure() {
//		SeparatorMoveHandle handle = new SeparatorMoveHandle ((GraphicalEditPart )getHost ());
//		handle.setBounds (((GraphicalEditPart)getHost()).getFigure().getBounds());
//		handle.setMoving(true);
//		addFeedback (handle);
//		return handle;
		return super.createDragSourceFeedbackFigure();
	}
}
