/*
 * Created on Feb 5, 2004
 *
 */
package AcmeDiagramLegend.parts;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.handles.AbstractHandle;
import org.eclipse.gef.handles.RelativeHandleLocator;
import org.eclipse.gef.tools.ResizeTracker;
import org.eclipse.swt.graphics.Color;


/**
 * @author schmerl
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class TriangleHandle extends AbstractHandle {

	protected static final int DEFAULT_HANDLE_SIZE = 10;
	
	{
		init ();
	}
	
	
	private int cursorDirection = 0;
	private PointList _points;
	
	public TriangleHandle () {}
	
	public TriangleHandle (GraphicalEditPart owner, Locator loc) {
		super (owner, loc);
	}
	
	public TriangleHandle (GraphicalEditPart owner, int direction) {
		setOwner (owner);
		setLocator (new RelativeHandleLocator (owner.getFigure (), direction));
		setCursor (Cursors.getDirectionalCursor(direction));
		cursorDirection = direction;
	}
	protected Color getBorderColor () {
		return (isPrimary ())?ColorConstants.lightBlue : ColorConstants.blue;
	}
	
	protected Color getFillColor () {
		return (isPrimary ())?ColorConstants.white : ColorConstants.lightBlue;
	}
	
	protected void init () {
		setPreferredSize (new Dimension (DEFAULT_HANDLE_SIZE, DEFAULT_HANDLE_SIZE));
	}
	
	protected boolean isPrimary () {
		return getOwner ().getSelected () == EditPart.SELECTED_PRIMARY;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gef.handles.AbstractHandle#createDragTracker()
	 */
	
	public void paintFigure (Graphics g) {
		Rectangle r = getBounds ();
		r.shrink (1,1);
		try {
			g.setBackgroundColor (getFillColor ());
			g.fillPolygon(getTrianglePoints ());
			g.setForegroundColor(getBorderColor());
			g.drawPolygon(getTrianglePoints ());
		}
		finally {
			r.expand (1,1);
		}
	}
	
	/**
	 * @return
	 */
	private PointList getTrianglePoints() {
		if (_points == null) {
			_points = new PointList ();
			Rectangle r = getBounds ().getCopy ();
			switch (cursorDirection) {
				case PositionConstants.EAST:
					_points.addPoint(r.x + r.width, r.y);
					_points.addPoint (r.x + r.width, r.y + r.height);
					_points.addPoint (r.x, r.y + r.height/2);
					break;
				case PositionConstants.WEST:
					_points.addPoint (r.x, r.y);
					_points.addPoint (r.x, r.y+r.height);
					_points.addPoint (r.x + r.width, r.y + r.height/2);
					break;
				case PositionConstants.NORTH:
					_points.addPoint(r.x, r.y + r.height);
					_points.addPoint (r.x + r.width, r.y + r.height);
					_points.addPoint (r.x + r.width/2, r.y);
					break;
				case PositionConstants.SOUTH:
					_points.addPoint (r.x, r.y);
					_points.addPoint (r.x + r.width, r.y);
					_points.addPoint (r.x + r.width/2, r.y + r.height);
					break;
							
			}
		}
		return _points;
	}

	protected DragTracker createDragTracker() {
		return new ResizeTracker (getOwner (), cursorDirection);
	}

}