package AcmeStudio;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.MenuListener;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IPerspectiveListener;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.views.navigator.IResourceNavigator;

import AcmeStudio.core.ASContext;
import AcmeStudio.core.resource.ASPath;
import AcmeStudio.core.resource.ASPathIterator;
import AcmeStudio.core.resource.IFamilyContributor;
import AcmeStudio.prefs.ConnectionPatternPreferences;
import AcmeStudio.prefs.DebugPreferences;
import AcmeStudio.prefs.DiagramEditorPreferences;
import AcmeStudio.prefs.DiagramToolbarPreferences;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.prefs.TextEditorSynchronizationPreferences;
import AcmeStudio.prefs.VisPreferences;
import AcmeStudio.ui.views.ElementView;
import AcmeStudio.ui.views.parts.MaintainedTypesList;
import AcmeStudio.util.Debug;
import AcmeStudio.util.DesignRuleHolder;

/**
 * The main _plugin class to be used in the desktop.
 */
public class AcmeStudioPlugin extends AbstractUIPlugin
implements IPerspectiveListener, MenuListener {
    //private static final String CURDIR_KWD = "@{CUR}";
    private static final String FAMILY_SUBDIR_KWD = "@{FAMILY_SUBDIR}";
    private static final String PROJECT_KWD = "@{PROJECT}";
    private static final String PLUGIN_STATE_KWD = "@{PLUGIN_STATE}";
    private static final String PLUGIN_KWD = "@{PLUGIN}";
    private static final String AS_INSTALLED_KWD = "@{AS_INSTALLED}";

	/** The shared instance */
	private static AcmeStudioPlugin _plugin = null;
    /** Search paths */
    private static List _searchPaths = null;
    /** Default font name, based on OS */
    private static String _defaultFontName = null;
    static {
        if (System.getProperty("os.name").indexOf("Linux") > -1) {
            // we're in Linux, use "Sans" font 
            _defaultFontName = "Sans";
        } else {
            _defaultFontName = "Arial";
        }
    }

    /** Resource bundle */
    private ResourceBundle _resourceBundle = null;
    /** Message text resource bundle */
    private ResourceBundle _msgBundle = null;

	/**
	 * The constructor.
	 */
	public AcmeStudioPlugin () {
		super(); 
		_plugin = this;
        getActiveWindow().addPerspectiveListener(this);
//        IMenuManager manager = getActiveWindow().getShell().getMenuBar().
//        addMenuListener(this);
        ASContext.initialize();
	}

    /**
     * Finds and returns the Element View.
     * @return  the current Element View instance
     */
    public static ElementView findElementView () {
        ElementView view = (ElementView )findView("AcmeStudio.ui.views.ElementView");
        return view;
    }

    /**
     * Finds and returns the Resource Navigator.
     * <p>
     * Part of Bug Fix #371
     * @return  the current Resource Navigator instance
     */
    public static IResourceNavigator findNavigator () {
        IResourceNavigator view = (IResourceNavigator )findView("org.eclipse.ui.views.ResourceNavigator");
        return view;
    }

    /**
     * Finds and returns an IViewPart in the workbench that matches the
     * given part id, which should uniquely identify a view per workbench page
     * <p>
     * This is refactored to more views to be searched
     * @param viewName  qualified name of the view part
     * @return  the view part
     */
    public static IViewPart findView(String viewName) {
        IViewPart part = null;

        IWorkbench workbench= PlatformUI.getWorkbench();
        IWorkbenchWindow[] windows= workbench.getWorkbenchWindows();
        for (int i = 0; i < windows.length; i++) {
            IWorkbenchWindow iWorkbenchWindow = windows[i];
            IWorkbenchPage[] pages = iWorkbenchWindow.getPages();
            for (int j = 0; j < pages.length; j++) {
                IWorkbenchPage iWorkbenchPage = pages[j];
                part = iWorkbenchPage.findView(viewName);
            }
        }

        return part;
    }

    /**
     * Searches through the AcmeStudio family search path for the named file.
     * TODO: Need to handle subdirectories
     * @param filename  the file name to search for
     * @return Object  the file found in the path, either File or IFile
     */
    public static Object findFileInFamPaths (String filename) {
        Object retFile = null;
        IPath filepath = new Path(filename);
        Iterator iter = getSearchPaths().iterator();
        // BugFix: break should break out of the outer loop
        OUTER: while (iter.hasNext()) {
            ASPath asPath = (ASPath) iter.next();
            ASPathIterator pathIter = null;
            try {
                pathIter = asPath.getPathIterator();
            } catch (CoreException e) {
                e.printStackTrace();
                continue;
            }
            while (pathIter.hasNext()) {
                IPath path = pathIter.nextPath();
                IContainer container = getRoot().getContainerForLocation(path);
                if (container != null) {  // can treat as IResource
                    IFile file = container.getFile(filepath);
                    if (file.exists()) {
                        retFile = file;
                        break OUTER;  // stop after finding the first one
                    }
                } else {  // juse Java file I/O
                    File file = path.append(filepath).toFile();
                    if (file.exists()) {
                        retFile = file;
                        break OUTER;
                    }
                }
            }
        }
        return retFile;
    }

	/**
	 * Returns the shared instance.
     * @return AcmeStudioPlugin  the shared instance
	 */
	public static AcmeStudioPlugin getDefault () {
		return _plugin;
	}

    /**
     * Returns the current active workbench window.  A shorthand for
     * getDefault().getWorkbench().getActiveWorkbenchWindow().
     * @return IWorkbenchWindow the active window
     */
    public static IWorkbenchWindow getActiveWindow () {
        return getDefault().getWorkbench().getActiveWorkbenchWindow();
    }

	/**
	 * Returns the workspace instance.
     * @return IWorkspace the workspace instance
	 */
	public static IWorkspace getWorkspace () {
		return ResourcesPlugin.getWorkspace();
	}

    /**
     * Returns the workspace root instance.
     * @return IWorkspaceRoot the workspace root instance
     */
    public static IWorkspaceRoot getRoot () {
        return getWorkspace().getRoot();
    }

	/**
	 * Returns the string from the _plugin's resource bundle, or 'key' if not found.
     * @param key  the key to find resource string
     * @return String  the resource string found, or 'key' if not found
	 */
	public static String getResourceString (String key) {
		ResourceBundle bundle = AcmeStudioPlugin.getDefault().getResourceBundle();
        if (bundle == null) {
            return key;
        }
		try {
			return evalKeyword(bundle.getString(key));
		} catch (MissingResourceException e) {
			return key;
		}
	}

    /**
     * Returns the string from the _plugin's message resource bundle, or
     * 'key' if not found.
     * @param key  the key to find message
     * @return STring  the message found, or 'key' if not found
     */
    public static String getMessage (String key) {
        ResourceBundle bundle = AcmeStudioPlugin.getDefault().getMessageBundle();
        if (bundle == null) {
            return key;
        }
        try {
            return bundle.getString(key);
        } catch (MissingResourceException e) {
            return key;
        }
    }

    /**
     * Loads search paths from property resource file, storing them as ASPath.
     * ISSUE (01-29-2003):  How to know when to update this list of paths?<br>
     * (08-01-2003):  This list of paths shouldn't need to be updated, as
     *   each component ASPath listens to context change for project changes
     *   that might signal the need to refresh.  If refresh doesn't occur as
     *   expected, such as when project switches, check the context change
     *   behavior to make sure it's correct.<br>
     * (05-03-2004):  Family contributors' search paths are collected here.
     *   This design needs to change if dynamic update of plug-in becomes
     *   possible. 
     * @return List  list of ASPaths corresponding to search paths
     */
    public static List getSearchPaths () {
        if (_searchPaths != null) {
            return _searchPaths;
        }
        // otherwise, lazy initialize the search path
        _searchPaths = new ArrayList();

        String pathStr = "";
        String SEP = getResourceString("AcmeStudio.path.separator");

        // get the search path property and store the individual path,
        // substituting placeholders for actual values
        String paths = getResourceString("AS_SEARCH_PATH");
        
        List contributors = getFamilyContributors ();
        for (Iterator iter = contributors.iterator (); iter.hasNext ();) {
        	IFamilyContributor contributor = (IFamilyContributor )iter.next ();
        	String [] contributions = contributor.getPathContributions();
        	for (int i = 0; i < contributions.length; i++) {
        		
        		String contribution = contributions [i];
				paths += SEP + contribution;
        	}
        }
        
        StringTokenizer pathTokens = new StringTokenizer(paths, getResourceString("AcmeStudio.path.separator"));
        while (pathTokens.hasMoreTokens()) {
            String path = pathTokens.nextToken();
            ASPath asPath = new ASPath(path);
            _searchPaths.add(asPath);
            pathStr += path;
            if (pathTokens.hasMoreTokens()) {
                pathStr += SEP;
            }
        }
        
        //System.setProperty("AS_SEARCH_PATH", paths);
        updatePathVars();
        return _searchPaths;
    }

    /**
     * Returns a path separated concatenation of the search paths from
     * getSearchPath().
     * @return String  concatenated string of paths.
     */
    public static String getSearchPathString () {
        String pathStr = "";
        String SEP = getResourceString("AcmeStudio.path.separator");

        getSearchPaths();  // initialize search path array if necessary
        ListIterator i = _searchPaths.listIterator();
        if (i.hasNext()) {
        	pathStr = ((ASPath )i.next()).getPathStr();
        }
        while (i.hasNext()) {
        	pathStr += SEP + ((ASPath )i.next()).getPathStr();
        }
        
        return pathStr;
    }

    private static List getFamilyContributors() {
		List contributors = new ArrayList ();
        IExtensionRegistry registry = Platform.getExtensionRegistry();
        IExtensionPoint point = registry.getExtensionPoint ("AcmeStudio.familyDirectory");
        IExtension [] extensions = point.getExtensions ();
        for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement [] elements = extensions [i].getConfigurationElements();
			for (int j = 0; j < elements.length; j++) {
				if ("contributor".equals (elements [j].getName ())) {
					try {
						IFamilyContributor contributor = (IFamilyContributor )elements [j].createExecutableExtension("class");
						contributors.add (contributor);
					}
					catch (CoreException e) {
						Debug.errorln (e);
					}
				}
			}
		}
        return contributors;
	}

	/**
     * Updates the environment path variables that might be supplied to the
     * ArmaniParser for family import.  Calling this method ensures that the
     * variables are substituted with explicit values relevant to the current
     * stste, e.g., project.
     */
    public static void updatePathVars () {
        String[] pathVars =
            {ASPath.SEARCH_PATH, ASPath.PROJECT_PATH, ASPath.GLOBAL_PATH};
        String AS_PSEP = getResourceString("AcmeStudio.path.separator");
        String AS_FSEP = getResourceString("AcmeStudio.file.separator");
        String J_PSEP = System.getProperty("path.separator");
        String J_FSEP = System.getProperty("file.separator");
        if (J_FSEP.equals("\\")) {
            J_FSEP = "\\\\";  // to escape the backslash inside of regex Pattern
        }
        for (int i=0; i < pathVars.length; i++) {
            // replace AcmeStudio file/path separators with system separator;
            // may turn out to be the same.
        	String pathVal = "";
        	if (pathVars [i].equals (ASPath.SEARCH_PATH)) {
        		pathVal = getSearchPathString ();
        	}
        	else {
        		pathVal = getResourceString(pathVars[i]);
        	}
            pathVal = pathVal.replaceAll(AS_PSEP, J_PSEP);
            pathVal = pathVal.replaceAll(AS_FSEP, J_FSEP);
            System.setProperty(pathVars[i], pathVal);
        }
    }

    /**
     * Given a string, finds and substitutes all keywords, i.e., @{&lt;KEYWORD&gt;},
     * with its corresponding value.
     * @param str  the string for which to substitute keywords
     * @return String  the new, keyword-substituted string
     */
    private static String evalKeyword (String str) {
        if (str == null) {
            return null;
        }

        StringBuffer strBuf = new StringBuffer(str);
        String path = null;
        int bIdx = 0, eIdx = 0;
        // this loop always starts from the beginning, and doesn't end until
        // no more @{...} is found; so the substitution behavior equates to
        // recursive substitution.
        while (bIdx > -1 && eIdx > -1) {  // get each substitution token
            bIdx = strBuf.indexOf("@{");
            if (bIdx >= 0) {
                eIdx = strBuf.indexOf("}", eIdx);
                if (eIdx > bIdx) {
                    eIdx++;
                    // entire token found, substitute, if possible
                    String var = strBuf.substring(bIdx, eIdx);
                    if (var.equals(FAMILY_SUBDIR_KWD)) {
                        // replace with family subdirectory name
                        path = getResourceString("AcmeStudio.family.subdir");
                    } else if (var.equals(PROJECT_KWD)) {
                        // get absolute path to project directory
                        IProject proj = ASContext.current().getActiveProject();
                        if (proj == null) {
                            Debug.warnln("AcmeStudioPlugin.evalKeyword():  No project is active yet!");
                            continue;
                        } else {
                            path = proj.getLocation().toOSString();
                        }
                    } else if (var.equals(PLUGIN_STATE_KWD)) {
                        // get absolute path to _plugin metadata directory
                        Plugin plugin = AcmeStudioPlugin.getDefault();
                        path = Platform.getPluginStateLocation(plugin).toOSString();
                    } else if (var.equals(PLUGIN_KWD)) {
                        // get absolute path to _plugin folder
                        URL url = Platform.find (getDefault ().getBundle (), new Path(""));
                        if (url == null) {  // non-existent _plugin??? skip it
                            Debug.warnln("AcmeStudioPlugin.evalKeyword():  Unable to find plug-in install path!");
                            continue;
                        }
                        try {
                            url = Platform.asLocalURL(url);
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                        path = url.getPath();
                        if (path.matches("^/[A-z]:/.*")) {
                        	path = path.substring(1);
                        }
                    }
//                    else if (var.equals (AS_INSTALLED_KWD)) {
//                    	IPath p = Platform.getPluginStateLocation (AcmeStudioPlugin.getDefault ());
//                    	p.removeLastSegments(2);
//                    	path = p.toOSString();
//                    }
                    strBuf.replace(bIdx, eIdx, path);
                }
            }
        }
        
        return strBuf.toString();
    }

    public static String getDefaultFontName () {
        return _defaultFontName;
    }

    /**
     * @see org.eclipse.ui.IPerspectiveListener#perspectiveActivated(IWorkbenchPage, IPerspectiveDescriptor)
     */
    public void perspectiveActivated(IWorkbenchPage page, IPerspectiveDescriptor perspective) {
    }

    /**
     * @see org.eclipse.ui.IPerspectiveListener#perspectiveChanged(IWorkbenchPage, IPerspectiveDescriptor, String)
     */
    public void perspectiveChanged(
        IWorkbenchPage page,
        IPerspectiveDescriptor perspective,
        String changeId) {
    }

    /**
     * @see org.eclipse.swt.events.MenuListener#menuHidden(MenuEvent)
     */
    public void menuHidden(MenuEvent e) {
    }

    /**
     * @see org.eclipse.swt.events.MenuListener#menuShown(MenuEvent)
     */
    public void menuShown(MenuEvent e) {
        Debug.errorln(getEclipseHome());
    }

    /**
     * Returns the path to the eclipse home directory.
     * @return IPath the path corresponding to eclipse home
     */
    public IPath getEclipseHome () {
        URL installUrl = getDefault ().getBundle().getEntry("/"); 
        IPath path = new Path(installUrl.getPath());
        
        return path;
    }

    /**
     * Shorthand to the Unique Identifier of the _plugin.
     * @return String  the Unique ID of this _plugin
     */
    public String getID () {
        return getBundle ().getSymbolicName ();
    }

	/**
	 * Returns the _plugin's resource bundle.
     * @return ResourceBundle  the resource bundle of this _plugin
	 */
	public ResourceBundle getResourceBundle () {
        if (_resourceBundle == null) {
            try {
                _resourceBundle = ResourceBundle.getBundle("AcmeStudio.acmestudioplugin");
            } catch (MissingResourceException x) {
                _resourceBundle = null;
            }
        }
		return _resourceBundle;
	}

    /**
     * Returns the _plugin's message resource bundle.
     * @return ResourceBundle  the message resource bundle of this _plugin
     */
    public ResourceBundle getMessageBundle () {
        if (_msgBundle == null) {
            try {
                _msgBundle = ResourceBundle.getBundle("AcmeStudio.messagetext");
            } catch (MissingResourceException x) {
                _msgBundle = null;
            }
        }
        return _msgBundle;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.ui.plugin.AbstractUIPlugin#initializeDefaultPreferences(org.eclipse.jface.preference.IPreferenceStore)
     */
    protected void initializeDefaultPreferences(IPreferenceStore store) {
        store.setDefault ("ConstraintEvaluation", DesignRuleHolder.BACKGROUNDEVALUATION);
        store.setDefault ("AutoRefresh", false);
        TextEditorSynchronizationPreferences.setDefaults (store);
        DiagramEditorPreferences.setDefaults(store);
        DiagramToolbarPreferences.setDefaults(store);
        VisPreferences.setDefaults(store);
        ConnectionPatternPreferences.setDefaults(store);
        GlobalVisPreferences.setDefaults(store);
//        TestCompletionActions.setDefaults (store);
        DebugPreferences.setDefaults(store);
        store.setDefault (MaintainedTypesList.DISTINGUISH_DECLARED_INSTANTIATED_TYPES, false);
    }

}
