/*
 * Created on Jun 4, 2003
 *
 */
package AcmeStudio;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.*;

import AcmeStudio.model.ModelCommand;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElement;
import acmeLib.properties.AcmeProperty;
import acmeLib.properties.FloatType;

/**
 * @author schmerl
 */
public class TestPopupAction implements IEditorActionDelegate, IObjectActionDelegate {

    /**
     * This is the class containing the command to be executed by the
     * action. ModelCommands are executed in the context of an AcmeDesign
     * and can be undone or redone.
     *
     */
    class ExampleCommand extends ModelCommand {
        /** What the command is applied to **/
        protected AcmeComponent _target;
        
        // The label is the name of the command that will appear in the undo
        // and redo menus
        public ExampleCommand (String label, AcmeComponent target) {           
            super (label);
            _target = target;
        }

        /**
         * Adds a property called "someProp" to the element.
         * Processes any exceptions and put them as the command's _errorReason.
         */
        protected void doWork () {
            try {
                _target.addProperty ("someProp", new FloatType ("float"), null);
                AcmeStudioChangeManager.globalChangeManager.onAddProperty (_target, _target.getProperty ("someProp"), this);
            }
            catch (AcmeIllegalNameException e) {
                // Set the _errorReason, to be checked later on by the action
                this._errorReason = e;
            }
        }
        
        /**
         * Remove "someProp" from the component
         *
         */
        protected void undoWork () {
            AcmeProperty prop = _target.getProperty ("someProp");
            _target.removeProperty ("someProp");
            AcmeStudioChangeManager.globalChangeManager.onRemoveProperty(_target, prop, this);
        }

        /**
         * Execute the command
         */
        public void execute() {
            doWork ();
        }

        /* (non-Javadoc)
         * @see org.eclipse.gef.commands.Command#redo()
         */
        public void redo() {
            doWork ();
        }

        /* (non-Javadoc)
         * @see org.eclipse.gef.commands.Command#undo()
         */
        public void undo() {
            undoWork ();
        }

    }

    /** The selected component to apply a command to **/
    protected AcmeComponent _targetComponent;
    
    
    public TestPopupAction() {
        super();
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorActionDelegate#setActiveEditor(org.eclipse.jface.action.IAction, org.eclipse.ui.IEditorPart)
     */
    public void setActiveEditor(IAction action, IEditorPart targetEditor) {
        
    }

    /**
     * Returns the command for this action
     * @return the command
     */
    protected ExampleCommand getCommand () {
        return new ExampleCommand ("something", _targetComponent);   
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {
        // Ignore action here
        
        // Get the command
        ModelCommand cmd = getCommand ();
        
        // Execute the command
        cmd.execute (_targetComponent.getParentDesign());
        
        // Check for any errors in the execution
        try {
            cmd.checkError ();
        }
        catch (Exception e) {
            // Process any error that comes from the command
            MessageDialog.openError(Display.getCurrent ().getActiveShell (), "Error", "Could not execute command");
        }
    }



    /**
     * Called when a selection in a selection provider changes.
     */
    public void selectionChanged(IAction action, ISelection selection) {
        // Unravel the selection to get an AcmeElement
        AcmeElement [] elements = Helper.getAcmeElementsFromSelection (selection);
        
        // Only activate this action if the selection is a component, and only one component is selected
        if (elements.length == 1 && elements [0] instanceof AcmeComponent) {
            _targetComponent = (AcmeComponent )elements [0];
            action.setEnabled (true);                
        }
        else {
            action.setEnabled (false);
        }

    }

    /**
     * Called when a workbench part is activated
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
    }

}
