package AcmeStudio.core;

import org.eclipse.core.resources.IProject;

import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.util.Debug;
import acmeLib.elements.AcmeDesign;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 *
 * An event representing the context change event.
 */
public class ASContextEvent {

    /** Flag indicating whether project previously changed */
    private boolean _hasProjectChanged = false;
    /** Flag indicating whether design previously changed */
    private boolean _hasDesignChanged = false;
    /** Flag indicating whether editor previously changed */
    private boolean _hasEditorChanged = false;

    /**
     * Constructor for ASContextEvent.
     * @deprecated  use instead #ASContextEvent(IProject, ASDesign, MultiPageAcmeEditor)
     */
    public ASContextEvent(IProject oldProj, AcmeDesign oldDesign, MultiPageAcmeEditor oldEditor) {
        this(ASContext.current(), oldProj, (ASDesign )ASModel.current().getASResource(oldDesign), oldEditor);
    }

    /**
     * Constructor for ASContextEvent.
     */
    public ASContextEvent(ASContext context, IProject oldProj, ASDesign oldDesign, MultiPageAcmeEditor oldEditor) {
        try {
            //Debug.traceln("ASContextEvent.ASContextEvent: Initialized members; current () == " + context);
            String trace = "old P|D|E == (" + oldProj + "|" + oldDesign + "|" + oldEditor + ")";
            if ((oldProj == null && context.getActiveProject() != null)
                    || (oldProj != null
                            && ! oldProj.equals(context.getActiveProject()))) {
                
                Debug.traceln("ASContextEvent: Project changed, " + trace + ", new P == " + context.getActiveProject());
                _hasProjectChanged = true;
            }
            if ((oldDesign == null && context.getActiveDesign() != null)
                    || (oldDesign != null
                            && ! oldDesign.equals(context.getActiveDesign()))) {
                
                Debug.traceln("ASContextEvent: Design changed, " + trace + ", new D == " + context.getActiveDesign());
                _hasDesignChanged = true;
            }
            if ((oldEditor == null && context.getActiveEditor() != null)
                    || (oldEditor != null
                            && ! oldEditor.equals(context.getActiveEditor()))) {
                
                Debug.traceln("ASContextEvent: Editor changed, " + trace + ", new E == " + context.getActiveEditor());
                _hasEditorChanged = true;
            }
        } catch (Exception e) {
            Debug.traceln("ASContextEvent: raised exception");
            Debug.errorln(e);
        }
    }

    /**
     * Returns whether the design has changed.
     * @return boolean  <code>true</code> if design changed, <code>false</code> otherwise.
     */
    public boolean hasDesignChanged() {
        return _hasDesignChanged;
    }

    /**
     * Returns whether the editor has changed.
     * @return boolean  <code>true</code> if editor changed, <code>false</code> otherwise.
     */
    public boolean hasEditorChanged() {
        return _hasEditorChanged;
    }

    /**
     * Returns whether the project has changed.
     * @return boolean  <code>true</code> if project changed, <code>false</code> otherwise.
     */
    public boolean hasProjectChanged() {
        return _hasProjectChanged;
    }

}
