package AcmeStudio.core;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import AcmeStudio.AcmeStudioPlugin;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 *
 * AcmeStudio implementation of Eclipse Platform Status object to facilitate
 * the throwing of CoreException.
 * 
 * History:
 *   SWC - Changed parent class to MultiStatus to utilize listing feature in Error Dialog
 */
public class ASStatus extends MultiStatus implements ASStatus_Constants {

    /**
     * Creates a new ASStatus object.
     * @param severity  the severity; one of <code>OK</code>, <code>ERROR</code>,
     *                  <code>INFO</code>, or <code>WARNING</code>
     * @param code  the plug-in-specific status code, or OK
     * @param message  a human-readable message, localized to the current locale
     * @param exception  a low-level exception, or null if not applicable
     */
    public ASStatus(int severity, int code, String message, Throwable exception) {
        super(AcmeStudioPlugin.getDefault().getID(), OK, message, exception);

        // add a new Status child object that takes the exception stack trace
        // and forms a message string
        ByteArrayOutputStream b = new ByteArrayOutputStream();
        exception.printStackTrace(new PrintStream(b));
        add(new Status(severity, AcmeStudioPlugin.getDefault().getID(), code, b.toString(), exception));
        while (exception.getCause() != null) {
            exception = exception.getCause();
            b = new ByteArrayOutputStream();
            exception.printStackTrace(new PrintStream(b));
            add(new Status(severity, AcmeStudioPlugin.getDefault().getID(), code, b.toString(), exception));
        }
    }
}
