package AcmeStudio.core.actions;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.actions.RetargetAction;

/**
 * @author schmerl
 *
 * Implements retargeting, which allows an action to be reset.
 */
public class AcmeStudioRetargetAction extends RetargetAction {

	boolean _checkStyle = false;

    /**
     * AcmeStudioRetargetAction allows actions to be redefined. Used in
     * toolbars to allow focussed editor to change the action, rather'
     * than having to remove existing and then addd
     * @param id The id of the action
     * @param lable The label to be used
     */
    public AcmeStudioRetargetAction(String id, String label) {
        super(id, label);
    }
    
    public AcmeStudioRetargetAction (String id, String label, boolean check) {
    	super (id, label);
    	_checkStyle = check;
    }

    /** 
     * Sets the reference action to the new action
     * @param action The action that is called when the retarget
     *                  action is done
     */
    public void setAction(IAction action) {
        setActionHandler(action);
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#getDescription()
     */
    public String getDescription() {
        if (getActionHandler() != null)
            return getActionHandler().getDescription();
        return super.getDescription();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#getHoverImageDescriptor()
     */
    public ImageDescriptor getHoverImageDescriptor() {
        if (getActionHandler() != null)
            return getActionHandler().getHoverImageDescriptor();
        return super.getHoverImageDescriptor();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#getImageDescriptor()
     */
    public ImageDescriptor getImageDescriptor() {
        if (getActionHandler() != null)
            return getActionHandler().getImageDescriptor();
        return super.getImageDescriptor();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#getText()
     */
    public String getText() {
        if (getActionHandler() != null)
            return getActionHandler().getText();
        return super.getText();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#getToolTipText()
     */
    public String getToolTipText() {
        if (getActionHandler() != null)
            return getActionHandler().getToolTipText();
        return super.getToolTipText();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#getDisabledImageDescriptor()
     */
    public ImageDescriptor getDisabledImageDescriptor() {
        if (getActionHandler() != null)
            return getActionHandler().getDisabledImageDescriptor();
        return super.getDisabledImageDescriptor();
    }

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		super.run();
		if (getActionHandler () != null) {
//			setChecked (getActionHandler ().isChecked());
//			setEnabled (getActionHandler ().isEnabled ());
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IAction#getStyle()
	 */
	public int getStyle() {
		if (_checkStyle) return IAction.AS_CHECK_BOX;
		return super.getStyle();
	}

}
