package AcmeStudio.core.actions;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.SWTGraphics;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;

/**
 * @author schmerl
 *
 */
public class ExportToImageAction implements IWorkbenchWindowActionDelegate {
    private SystemEditPart _systemEditPart = null;
    private SystemView _systemView = null;

    public class ExportToImageCommand extends org.eclipse.gef.commands.Command {

        public ExportToImageCommand() {
            super(ID);
        }

        public void execute() {
            // Creeat the dialog to save the image file
            doSaveAs();
        }
    }

    private void doSaveAs () {
    	DiagramEditor editor = ASContext.current ().getActiveEditor().getDiagramEditor();
    	if (editor != null) {
    		_systemEditPart = editor.getRootEditPart();
    		if (_systemEditPart != null) {
    			_systemView = (SystemView )_systemEditPart.getFigure ();
    		}
    	}
    	
    	Shell shell = AcmeStudioPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell ();
    	
    	if (_systemView == null) {
    		MessageDialog.openInformation (shell, AcmeStudioPlugin.getMessage ("ExportToImageAction.NoDiagramTitle"), AcmeStudioPlugin.getMessage ("ExportToImageAction.NoDiagramMsg")); //$NON-NLS-1$ //$NON-NLS-2$
    	}
    	else {
    		Device device = Display.getCurrent();
    		FileDialog dialog = new FileDialog (shell, SWT.SAVE);
    		dialog.setFilterExtensions(new String [] {"*.jpg", /*"*.png", "*.gif",*/ "*.bmp", "*.tif", "*.tiff"});
    		String filename = dialog.open ();
    		
    		if (filename != null) {
    			int format = SWT.NONE;
    			String lcfe = filename.toLowerCase();
    			if (lcfe.endsWith (".jpg")) {
    				format = SWT.IMAGE_JPEG;
    			}
    			else if (lcfe.endsWith (".png")) {
    				format = SWT.IMAGE_PNG;
    			}
    			else if (lcfe.endsWith(".gif")) {
    				format = SWT.IMAGE_GIF;
    			}
    			else if (lcfe.endsWith (".bmp")) {
    				format = SWT.IMAGE_BMP_RLE;
    			}
    			else if (lcfe.endsWith (".tif") || lcfe.endsWith (".tiff")) {
    				format = SWT.IMAGE_TIFF;
    			}
    			else {
    				filename += ".jpg";
    				format = SWT.IMAGE_JPEG;
    			}
    			
    			if (format != SWT.NONE) {
    				Rectangle r = _systemView.getBounds();
    				Image image = null;
    				GC gc = null;
    				Graphics g = null;
    				try {
    					image = new Image (device, r.width, r.height);
    					gc = new GC (image);
    					g = new SWTGraphics (gc);
    					g.translate (r.x * -1, r.y * -1);
    					_systemView.paint (g);
    					
    					ImageLoader imageLoader = new ImageLoader ();
    					imageLoader.data = new ImageData [] {image.getImageData()};
    					
    					imageLoader.save (filename, format);
    				}
    				catch (SWTError e) {
    					if (e.getMessage().equals ("Not implemented")) {
    						MessageDialog.openError(Display.getCurrent().getActiveShell(), AcmeStudioPlugin.getMessage("ExportToImageAction.ExportErrorTitle"), AcmeStudioPlugin.getMessage("ExportToImageAction.ExportErrorMsg")); //$NON-NLS-1$ //$NON-NLS-2$
    					}
    					else throw e;
    				}
    				finally {
    					if (g != null) g.dispose ();
    					if (gc != null) gc.dispose ();
    					if (image != null) image.dispose ();
    				}
    			}
    		}
    	}
    	
    }
    
    public static final String ID = "exptoimg";

    public ExportToImageAction() {
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
     */
    public void dispose() {
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window) {
       
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(IAction)
     */
    public void run(IAction action) {
        new ExportToImageCommand().execute();
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
        action.setEnabled(true);
        if (ASContext.current() == null
            || ASContext.current().getActiveEditor() == null
            || ASContext.current().getActiveEditor().getDiagramEditor() == null) {

            action.setEnabled(false);
        }

    }

    /**
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        new ExportToImageCommand().execute();
    }

}
