package AcmeStudio.core.actions;

import java.util.List;

import org.eclipse.gef.Request;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASModel;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.model.FamilyCommand;
import AcmeStudio.model.ModelCompoundCommand;
import AcmeStudio.ui.views.parts.NewFamilyDialog;
import AcmeStudio.ui.views.parts.RemoveElementDialog;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeFamily;

/**
 * @author schmerl
 */
public class FamilyAction
    extends Action
    implements IWorkbenchWindowActionDelegate {
        
        IWorkbenchWindow _window;
        ASDesign _design;
        AcmeFamily _family;
        
        private Request _request;

    public FamilyAction (String type) {
        super ();
        _request = new Request (type);
        if (type.equals (FamilyCommand.ADD_FAMILY)) {
            setText (AcmeStudioPlugin.getMessage ("FamilyAction.AddMsg")); //$NON-NLS-1$
        }
        else if (type.equals (FamilyCommand.REMOVE_FAMILY)) {
            setText (AcmeStudioPlugin.getMessage("FamilyAction.RemoveMsg")); //$NON-NLS-1$
        }
        setId (type);
    }

    /**
     * @deprecated use instead #setDesign(ASDesign)
     */
    public void setAcmeDesign (AcmeDesign design) {
        _design = (ASDesign )ASModel.current().getASResource(design);
    }

    /**
     * Sets a new ASDesign object
     * @param design  new ASDesign object
     */
    public void setDesign (ASDesign design) {
        _design = design;
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
     */
    public void dispose() {
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window) {
        _window = window;
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(IAction)
     */
    public void run(IAction action) {
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
//        Debug.diagnoseln("FamilyAction.selectionChanged");
    }

    public void run () {
        if (_design == null) return;
        if (_request.getType ().equals (FamilyCommand.ADD_FAMILY)) {
            List fams = Helper.getAllFamilies(getAcmeDesign());
    
            NewFamilyDialog dialog = new NewFamilyDialog (AcmeStudioPlugin.getDefault ().getWorkbench ().getActiveWorkbenchWindow ().getShell (),
                                                        fams, _family);
            if (dialog.open () == IDialogConstants.OK_ID) {
                if (dialog.getImportation () == null) {
                    _family = (AcmeFamily )dialog.getNewType ();
                    FamilyCommand command = new FamilyCommand ((String )_request.getType (), _family, getAcmeDesign(), null);
                    AcmeStudioChangeManager.globalChangeManager.getCommandStack(_design).execute(command);
                }
                else {
                    FamilyCommand command = new FamilyCommand ((String )_request.getType (), dialog.getImportation (), getAcmeDesign(), null);
                    AcmeStudioChangeManager.globalChangeManager.getCommandStack(_design).execute(command);
                }
            }
        }
        else {
            AcmeEnumeration families = getAcmeDesign().getFamilies ();
            RemoveElementDialog dialog = new RemoveElementDialog (AcmeStudioPlugin.getDefault ().getWorkbench ().getActiveWorkbenchWindow ().getShell (),
                                                                   families);
            if (dialog.open () == IDialogConstants.OK_ID) {
                AcmeElement [] selections = dialog.getSelections ();
                if (selections.length == 0) return;
                ModelCompoundCommand command = new ModelCompoundCommand (AcmeStudioPlugin.getMessage("FamilyAction.RemoveActionName")); //$NON-NLS-1$
                for (int i = 0; i < selections.length; i++) {
                    FamilyCommand c = new FamilyCommand ((String )_request.getType (), (AcmeFamily )selections [i], getAcmeDesign(), null);
                    command.add (c);
                }
                AcmeStudioChangeManager.globalChangeManager.getCommandStack(_design).execute (command);
                
            }
        }
    }        

    /**
     * Returns the encapsulated AcmeDesign object
     * @return AcmeDesign  the AcmeDesign object
     */
    private AcmeDesign getAcmeDesign () {
        return (_design == null) ? null : _design.getDesign();
    }

}
