/*
 * Created on Oct 20, 2003
 * 
 */
package AcmeStudio.core.actions;

import org.eclipse.gef.Request;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASModel;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.model.GroupCommand;
import AcmeStudio.model.ModelCompoundCommand;
import AcmeStudio.ui.views.parts.NewElementDialog;
import AcmeStudio.ui.views.parts.RemoveElementDialog;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.filters.GroupTypeEnumFilter;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeGroup;
import acmeLib.elements.AcmeSystem;

/**
 * @author schmerl
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class GroupAction
    extends Action
    implements IWorkbenchWindowActionDelegate {
    
    private Request _request;
    private AcmeSystem _system;
    private AcmeElement[] _selectedGroups;
    private AcmeGroup _group;

    public GroupAction (String type) {
        super ();
        _request = new Request (type);
        if (type.equals (GroupCommand.ADD_GROUP)) {
            setText (AcmeStudioPlugin.getMessage("GroupAction.AddGroup")); //$NON-NLS-1$
            setToolTipText(AcmeStudioPlugin.getMessage("GroupAction.AddGroupDescription")); //$NON-NLS-1$
            setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_NEW_GROUP));
        }
        else if (type.equals (GroupCommand.REMOVE_GROUP)) {
            setText (AcmeStudioPlugin.getMessage("GroupAction.RemoveGroup")); //$NON-NLS-1$
            setToolTipText(AcmeStudioPlugin.getMessage("GroupAction.RemoveGroupDescription")); //$NON-NLS-1$
            setImageDescriptor (ImageManager.getImageDescriptor(ImageManager.ID_DEL_GROUP));
        }
        setId (type);
    }

    
    
    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
     */
    public void dispose() {
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window) {
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
        _system = null;
        AcmeElement [] selections = Helper.getAcmeElementsFromSelection(selection);
        if (selections.length > 0) {
            _system = Helper.getAcmeSystem(selections [0]);
        }
    }
    
    public boolean calculateEnabled () {
        return _system != null;
    }
    
    public void run () {
        if (_system == null) return;
        if (_request.getType ().equals (GroupCommand.ADD_GROUP)) {
            AcmeEnumeration groupTypes = _system.getAvailableTypes().castToAcmeSetVector().enumerate (new GroupTypeEnumFilter ());
            NewElementDialog dialog = new NewElementDialog (Display.getDefault ().getActiveShell(), groupTypes, AcmeGroup.class, _system);
            if (dialog.open () == IDialogConstants.OK_ID) {
                _group = (AcmeGroup )dialog.getNewElement ();
                GroupCommand command = new GroupCommand (GroupCommand.ADD_GROUP, _group, _system, null);
                AcmeStudioChangeManager.globalChangeManager.getCommandStack (getASDesign()).execute (command);
            }
        }
        else {
            if (_selectedGroups == null) {
                AcmeEnumeration groups = _system.getGroups ();
                RemoveElementDialog dialog = new RemoveElementDialog (Display.getDefault().getActiveShell(), groups);
                if (dialog.open () == IDialogConstants.OK_ID) {
                    _selectedGroups = dialog.getSelections ();
                }
            }
            if (_selectedGroups.length == 0) return;
            ModelCompoundCommand command = new ModelCompoundCommand (AcmeStudioPlugin.getMessage("GroupAction.RemoveGroupTitle")); //$NON-NLS-1$
            for (int i = 0; i < _selectedGroups.length; i++) {
                GroupCommand c = new GroupCommand (GroupCommand.REMOVE_GROUP, (AcmeGroup )_selectedGroups [i], _system, null);
                command.add (c);
            }
            AcmeStudioChangeManager.globalChangeManager.getCommandStack (getASDesign()).execute (command);
        }
    }

    /**
     * @return Returns the selectedGroups.
     */
    public AcmeElement[] getSelectedGroups() {
        return _selectedGroups;
    }

    /**
     * @param selectedGroups The selectedGroups to set.
     */
    public void setSelectedGroups(AcmeElement[] selectedGroups) {
        _selectedGroups = selectedGroups;
    }

    /**
     * @return Returns the system.
     */
    public AcmeSystem getSystem() {
        return _system;
    }

    /**
     * @param system The system to set.
     */
    public void setSystem(AcmeSystem system) {
        _system = system;
    }

    private ASDesign getASDesign () {
        return (ASDesign )ASModel.current().getASResource(getSystem().getParentDesign());
    }
    
}
