/*
 * Created on Jan 6, 2004
 *
 */
package AcmeStudio.core.actions;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.dialogs.ListDialog;

import AcmeStudio.ui.views.parts.ElementTypeInspector;
import AcmeStudio.ui.views.parts.NewPropertyTypeDialog;
import AcmeStudio.ui.views.providers.AcmeTypeLabelProvider;
import AcmeStudio.ui.views.providers.ListContentProvider;
import AcmeStudio.util.Helper;
import acmeLib.AcmeType;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.properties.AcmePropertyType;



/**
 * @author Bradley Schmerl
 * Created Jan 6, 2004
 */
public class InspectTypeAction implements IWorkbenchWindowActionDelegate {

	private List _types;
	private AcmeElement _scope;

	/**
	 * 
	 */
	public InspectTypeAction() {
		super ();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
	 */
	public void dispose() {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		Shell shell = Display.getCurrent ().getActiveShell ();
		ListDialog list = new ListDialog (shell) ;
		list.setTitle("Select type to inspect");
		Collections.sort(_types, new Comparator () {

			public int compare(Object arg0, Object arg1) {
				AcmeType e1 = (AcmeType) arg0;
				AcmeType e2 = (AcmeType) arg1;
				return String.CASE_INSENSITIVE_ORDER.compare(e1.getTypeName(), e2.getTypeName());
			}
		
		});
		list.setContentProvider (new ListContentProvider ());
		list.setLabelProvider(new AcmeTypeLabelProvider ());
		list.setInput (_types);
		if (list.open() == ListDialog.OK) {
			if (list.getResult ().length > 0) {
				AcmeType type = (AcmeType) list.getResult() [0];
				if (type instanceof AcmeElementType) {
					ElementTypeInspector inspector = new ElementTypeInspector (shell, (AcmeElementType )type);
					inspector.open ();
					
				}
				else if (type instanceof AcmePropertyType) {
					NewPropertyTypeDialog inspector = new NewPropertyTypeDialog (shell, _scope, (AcmePropertyType )type);
					inspector.setEditable (false);
					inspector.open();
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		action.setEnabled (false);
		AcmeElement [] elements = Helper.getAcmeElementsFromSelection(selection);
		if (elements != null && elements.length > 0) {
			AcmeEnumeration types = elements [0].getAvailableTypes().castToAcmeSetVector().enumerate();
			_types = Helper.convertAcmeEnumerationToList(types);
			_scope = elements [0];
			if (_types.size() > 0) {
				action.setEnabled (true);
			}
		}
		else {
			_types = null;
			_scope = null;
		}
	}
}
