package AcmeStudio.core.actions;

import java.io.File;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.util.Debug;
import acmeLib.armaniParser.ParseError;
import acmeLib.core.exceptions.AcmeException;

/**
 * Our sample action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class OpenDesignAction implements IWorkbenchWindowActionDelegate {
	private IWorkbenchWindow window;
	/**
	 * The constructor.
	 */
	public OpenDesignAction() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {

		// Open a file dialog
		FileDialog openFileDialog = new FileDialog(window.getShell(), SWT.OPEN);
		openFileDialog.setFilterExtensions(new String[] { "*.acme" });
		String filename = openFileDialog.open();

		// Get the filename selected
        ASDesign asDesign = new ASDesign(null, null);
        try {
            asDesign.loadFrom(new File(filename));
        } catch (ParseError e) {
            Debug.errorln(e);
            MessageDialog.openError(window.getShell(),
                AcmeStudioPlugin.getMessage("OpenDesignAction.openErrorTitle"),
                e.getMessage());
        } catch (AcmeException e) {
            Debug.errorln(e);
             MessageDialog.openError(window.getShell(),
                 AcmeStudioPlugin.getMessage("OpenDesignAction.openErrorTitle"),
                 e.getMessage());           
        
        } catch (CoreException e) {
            Debug.errorln(e);
            ErrorDialog.openError(window.getShell(),
                AcmeStudioPlugin.getMessage("OpenDesignAction.openErrorTitle"),
                e.getStatus().getMessage(), e.getStatus());
        }
	}

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
}