package AcmeStudio.core.actions;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASModel;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.model.ModelCompoundCommand;
import AcmeStudio.model.SystemCommand;
import AcmeStudio.ui.views.parts.NewElementDialog;
import AcmeStudio.ui.views.parts.RemoveElementDialog;
import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeSystem;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class SystemAction extends Action implements IWorkbenchWindowActionDelegate {

    private ASDesign _design;
    private AcmeSystem _system;

    private Request _request;

    public SystemAction (String type) {
        super ();
        _request = new Request (type);
        if (type.equals (SystemCommand.ADD_SYSTEM)) {
            setText ("Add System...");
        }
        else if (type.equals (SystemCommand.REMOVE_SYSTEM)) {
            setText ("Remove Systems...");
        }
        setId (type);
    }
    
    /**
     * @deprecated use instead #setDesign(ASDesign)
     */
    public void setAcmeDesign (AcmeDesign design) {
        _design = (ASDesign )ASModel.current().getASResource(design);
    }

    /**
     * Sets a new ASDesign object
     * @param design  new ASDesign object
     */
    public void setDesign (ASDesign design) {
        _design = design;
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
     */
    public void dispose() {
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window) {
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(IAction)
     */
    public void run(IAction action) { 
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
        _design = null;
        
        if (selection instanceof StructuredSelection) {
            StructuredSelection sel = (StructuredSelection )selection;
            if (sel.size () > 0) {
                Object o = sel.getFirstElement ();
                if (o instanceof EditPart) {
                    EditPart ep = (EditPart )o;
                    if (ep.getModel() instanceof AcmeElement) {
                        setAcmeDesign(((AcmeElement )ep.getModel ()).getParentDesign());
                    }
                }
            }   
        }
    }

    public boolean calculateEnabled () {
        return _design != null;
    }
    /**
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        if (_design == null) return;
        if (_request.getType ().equals (SystemCommand.ADD_SYSTEM)) {
            AcmeEnumeration families = getAcmeDesign().getFamilies ();
            NewElementDialog dialog = new NewElementDialog (AcmeStudioPlugin.getDefault ().getWorkbench ().getActiveWorkbenchWindow ().getShell (),
                                                        families, AcmeSystem.class, getAcmeDesign());
            if (dialog.open () == IDialogConstants.OK_ID) {
                _system = (AcmeSystem )dialog.getNewElement();
                SystemCommand command = new SystemCommand (SystemCommand.ADD_SYSTEM, _system, getAcmeDesign(), null);   
                AcmeStudioChangeManager.globalChangeManager.getCommandStack(_design).execute (command);
            }
        }
        else {
            AcmeEnumeration systems = getAcmeDesign().getSystems();
            RemoveElementDialog dialog = new RemoveElementDialog (AcmeStudioPlugin.getDefault ().getWorkbench ().getActiveWorkbenchWindow ().getShell (),
                                                        systems);
            if (dialog.open() == IDialogConstants.OK_ID) {
                AcmeElement [] selections = dialog.getSelections();
                if (selections.length == 0) return;
                 ModelCompoundCommand command = new ModelCompoundCommand ("Remove Systems");
                 for (int i = 0; i < selections.length; i++) {
                    SystemCommand c = new SystemCommand (SystemCommand.REMOVE_SYSTEM, (AcmeSystem )selections [i], getAcmeDesign(), null);
                    command.add (c);                        
                }
                AcmeStudioChangeManager.globalChangeManager.getCommandStack (_design).execute (command);
                   
            } 
        }
    }

    /**
     * Returns the encapsulated AcmeDesign object
     * @return AcmeDesign  the AcmeDesign object
     */
    private AcmeDesign getAcmeDesign () {
        return (_design == null) ? null : _design.getDesign();
    }

}
