/*
 * Created on Aug 25, 2003
 */
package AcmeStudio.core.resource.datapersistence;

import java.io.File;

import AcmeStudio.core.resource.ASDesign;

/**
 * Defines routines for plugins to extend to save and load data annotations to an Acme design
 */
public interface IDataPersistence {

	/**
	 * Called immediately after an AcmeDesign is loaded into AcmeStudio. Subclasses should override
	 * to load plugin-specific information about the design.
	 * @param design The design that was loaded
	 * @param resource The file that the design was loaded from
	 */
    public void load (ASDesign design, File resource);

    /**
     * Called immediately after an AcmeDesign is saved by AcmeStudio. Sublcasses should override
     * to save plugin-specific information about the design
     * @param design The Acme design that was saved
     * @param resource The file to which the design was saved
     */
    public void save (ASDesign design, File resource);
    
    /**
     * Returns the data to be persisted that is in the design. The object returned
     * will be passed back into injectData.
     * @param design The design containing the data
     * @return the data to be persisted
     */
    public Object extractData (ASDesign design);
    
   /**
    * Annotates the design with data. The data passed in will have been retrieved
    * by a call to extractData
    * @param design The design to annotae
    * @param data The data do annotate with
    */
    public void injectData (ASDesign design, Object data);

}
