package AcmeStudio.model;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeFamily;

/**
 * @author schmerl
 *
 * Command to add or delete a family from a design, depending on the kind
 * supplied to the constructor.
 */
public class FamilyCommand extends ModelCommand {
    private AcmeFamily _family;
    private AcmeDesign _design;
    private Object _announcer;
    private boolean _import;
    private String _filename;
    
    public static final String ADD_FAMILY = "$AddFam$";
    public static final String REMOVE_FAMILY = "$DelFam$";
    
    public FamilyCommand (String kind, AcmeFamily fam, AcmeDesign des, Object ann) {
        super (kind);
        setKind (kind);
        _family = fam;
        _design = des;
        _announcer = ann;
        _import = false;
    }
    
    public FamilyCommand (String kind, String file, AcmeDesign des,  Object ann) {
        super (kind);
        setKind (kind);
        _family = null;
        _design = des;
        _announcer = ann;
        _import = true;
        _filename = file;   
    }
    
    public boolean canExecute () {
        return (_family != null || _filename != null) && _design != null;    
    }
        
    
    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        if (getKind ().equals (ADD_FAMILY)) {
            addFamily ();
        }
        else if (getKind ().equals (REMOVE_FAMILY)) {
            if (MessageDialog.openConfirm (Display.getDefault ().getActiveShell (), "Confirm delete", "This command cannot be undone. Really remove family?")) {
                removeFamily ();
            }
        }
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        if (getKind ().equals (ADD_FAMILY)) {
            addFamily ();
        }
        else if (getKind ().equals (REMOVE_FAMILY)) {
            removeFamily ();
        }        
    }

    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        if (getKind ().equals (ADD_FAMILY)) {
            removeFamily ();
        }
//        else if (_request.getType ().equals (REMOVE_FAMILY)) {
//            addFamily ();
//        }        
    }

    /**
     * @see org.eclipse.gef.commands.Command#canUndo()
     */
    public boolean canUndo() {
        return getKind ().equals (ADD_FAMILY);
    }

    protected void addFamily () {
        if (!_import) {
            try {
                _design.addFamily (_family);
                _family.setParentDesign (_design);
                AcmeStudioChangeManager.globalChangeManager.onAddFamily (_design, _family, _announcer);
            }
            catch (AcmeIllegalNameException e) {
                _errorReason = e;
            }
        }    
        else {
            _design.addDesignToImport (_filename);
            try {
                _design.resolveImportedDesigns ();
                AcmeStudioChangeManager.globalChangeManager.onImportDesign (_design, _filename, _announcer);               
            }
            catch (acmeLib.armaniParser.ParseException e) {
                _errorReason = e;
                _design.removeImportedDesign (_filename);   
            }
            catch (acmeLib.acmeParser.ParseException e) {
            	_errorReason = e;
            	_design.removeImportedDesign (_filename);
            }

        }
    }
    
    protected void removeFamily () {
        MessageDialog.openInformation (Display.getDefault ().getActiveShell(), "Unimplemented command", "This command is currently unimplemented")    ;
    }

}
