/*
 * Created on Sep 29, 2003
 *
 */
package AcmeStudio.model;

import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.designRules.DesignAnalysis;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeFamily;

/**
 * @author Bradley Schmerl
 * Created Sep 29, 2003
 */
public class FunctionCommand extends ModelCommand {

	public static String ADD_FUNCTION = "$ADD_FUNCTION";
	public static String REMOVE_FUNCTION = "$REM_FUNCTION";
	private DesignAnalysis _function;
	private AcmeElement _namespace;
	private Object _announcer;

	/**
	 * @param string
	 * @param newFunction
	 * @param _family
	 * @param object
	 */
	public FunctionCommand(String command, DesignAnalysis newFunction, AcmeFamily family, Object announcer) {
		super (command);
		setKind (command);
		_function = newFunction;
		_namespace = family;
		_announcer = announcer;
	}

	
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		return (_function != null) && (_namespace != null);
	}
	
	protected void addFunction () {
		if (_namespace instanceof AcmeFamily) {
			AcmeFamily family = (AcmeFamily) _namespace;
			try {
				family.addDesignAnalysis(_function);
				AcmeStudioChangeManager.globalChangeManager.onAddDesignAnalysis (_function, family, _announcer);
				
			}
			catch (Exception e) {
				_errorReason = e;
			}
		}
		else if (_namespace instanceof AcmeDesign) {
			AcmeDesign design = (AcmeDesign) _namespace;
			try {
				design.addDesignAnalysis(_function);
				AcmeStudioChangeManager.globalChangeManager.onAddDesignAnalysis (_function, design, _announcer);
			}
			catch (Exception e) {
				_errorReason = e;
			}
		}
	}
	
	protected void removeFunction () {
		if (_namespace instanceof AcmeFamily) {
			AcmeFamily family = (AcmeFamily) _namespace;
			try {
				family.removeDesignAnalysis(_function);
				AcmeStudioChangeManager.globalChangeManager.onRemoveDesignAnalysis (_function, family, _announcer);
			}
			catch (Exception e) {
				_errorReason = e;
			}
		}
		else if (_namespace instanceof AcmeDesign) {
			AcmeDesign design = (AcmeDesign) _namespace;
			try {
				design.addDesignAnalysis(_function);
				AcmeStudioChangeManager.globalChangeManager.onRemoveDesignAnalysis (_function, design, _announcer);
			}
			catch (Exception e) {
				_errorReason = e;
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		if (getKind ().equals (ADD_FUNCTION)) {
			setLabel ("Add function " + _function.getName());
			addFunction();
		}
		else if (getKind ().equals (REMOVE_FUNCTION)) {
			setLabel ("Remove function " + _function.getName ());
			removeFunction();
		}
	}
	
	public void redo () {
		execute ();
	}
	
	public void undo () {
		if (getKind ().equals (ADD_FUNCTION)) {
			removeFunction();
		}
		else if (getKind ().equals (REMOVE_FUNCTION)) {
			addFunction ();
		}
	}

}
