/*
 * Created on Oct 20, 2003
 * 
 */
package AcmeStudio.model;

import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.AcmeGroup;
import acmeLib.elements.AcmeSystem;

/**
 * @author schmerl
 */
public class GroupCommand extends ModelCommand {

    public static final String ADD_GROUP = "$AddGroup$";
    public static final String REMOVE_GROUP = "$DelGroup$";
    private AcmeSystem _system;
    private Object _announcer;
    private AcmeGroup _group;
    
    
    /**
     * @param label
     */
    public GroupCommand(String kind, AcmeGroup group, AcmeSystem sys, Object ann) {
        super(kind);
        setKind(kind);
        _system = sys;
        _group = group;
        _announcer = ann;
    }

    
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute() {
        return _group != null && _system != null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        if (getKind ().equals (ADD_GROUP)) {
            setLabel ("Add group " + _group.getName ());
            addGroup ();
        }
        else if (getKind ().equals (REMOVE_GROUP)) {
            setLabel ("Delete group " + _group.getName ());
            removeGroup ();
        }
    }

    /**
     * 
     */
    private void removeGroup() {
        _system.removeGroup(_group);
        AcmeStudioChangeManager.globalChangeManager.onRemoveGroup(_system, _group, _announcer);
    }


    /**
     * 
     */
    private void addGroup() {
        try {
            _system.addGroup (_group);
            AcmeStudioChangeManager.globalChangeManager.onRemoveGroup (_system, _group, _announcer);
        } catch (AcmeIllegalNameException e) {
           _errorReason = e;
        }
    }


    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute ();
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        if (getKind ().equals (ADD_GROUP)) removeGroup ();
        else if (getKind ().equals (REMOVE_GROUP)) addGroup ();
    }

}
