package AcmeStudio.model;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;

import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.elements.AcmeDesign;

/**
 * @author schmerl
 *
 * Base class for commands to modify AcmeStudio's Acme models. 
 */
public abstract class ModelCommand extends Command {

    protected Exception _errorReason = null;

    /** Identifies the kind of command for commands that support multiple operations */
    private Request _request = null;

    public ModelCommand (String label) {
        super (label);   
    }

    public void execute (AcmeDesign context) {
        AcmeStudioChangeManager.globalChangeManager.getCommandStack(context).execute (this);
    }
    
    public void checkError () throws Exception {
        if (_errorReason != null) {
            throw _errorReason;
        }   
    }
    
    public boolean canUndo () {
        return true;
    }

    public String getKind () {
        String kind = "";
        if (_request != null) {
            kind = (String) _request.getType();
        }
        return kind;
    }

    public void setKind (String kind) {
        if (_request == null) {
            _request = new Request(kind);
        } else {
            _request.setType(kind);
        }
    }

}
