package AcmeStudio.model;

import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.AcmeElement;
import acmeLib.representations.AcmeRepresentation;

/**
 * @author schmerl
 *
 * Command, identified by getKind(), to add or delete a representation from an
 * AcmeElement.
 */
public class RepresentationCommand extends ModelCommand {


    private static String ADD_REP_REQUEST = "$AddRep$";
    private static String REMOVE_REP_REQUEST = "$DelRep$";
    
    public static String ADD_REP = ADD_REP_REQUEST;
    public static String REMOVE_REP = REMOVE_REP_REQUEST;

    private AcmeRepresentation _newRep;
    private AcmeElement _target;
    private Object _announcer;
    
    public RepresentationCommand (String kind, AcmeRepresentation rep, AcmeElement target, Object announcer) {
        super (kind);
        setKind (kind);
        _newRep = rep;
        _target = target;
        _announcer = announcer;
    }
    
    public boolean canExecute () {
        return (_newRep != null) && (_target != null);
    }
    
    public boolean canUndo () {
        return canExecute ();
    }
    
    protected void removeRep () {
        _target.removeRepresentation (_newRep);
        AcmeStudioChangeManager.globalChangeManager.onRemoveRepresentation (_target, _newRep, _announcer);        
    }
    
    protected void addRep () {
        try {
            _target.addRepresentation (_newRep);
            AcmeStudioChangeManager.globalChangeManager.onAddRepresentation (_target, _newRep, _announcer);
            
        }
        catch (AcmeIllegalNameException e) {
            _errorReason = e;
        }
    }
    
    public void execute () {
        if (getKind ().equals (ADD_REP)) {
            addRep ();   
            setLabel ("Add representation " + _newRep.getName());
        }
        else if (getKind ().equals (REMOVE_REP)) {
            setLabel ("Remove representation " + _newRep.getName ());
            removeRep ();
        }
    }
 
    public void redo () {
        execute ();
    }
    
    public void undo () {
        if (_errorReason != null) return;
        if (getKind ().equals (ADD_REP)) {
            removeRep ();
        }
        else if (getKind ().equals (REMOVE_REP)) {
            addRep ();
        }
    }
}
