/*
 * Created on Jul 7, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package AcmeStudio.model;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.*;

/**
 * @author schmerl
 *
 * Command to set the new name of an AcmeElement.
 */
public class SetElementNameCommand extends ModelCommand {

	private AcmeElement _target;
	String _newName;
	String _oldName;
	private boolean	_did = true;

	/**
	 * @return
	 */
	public String getNewName() {
		return _newName;
	}

	/**
	 * @param newName
	 */
	public void setNewName(String newName) {
		this._newName = newName;
	}

	/**
	 * @return
	 */
	public String getOldName() {
		return _oldName;
	}

	/**
	 * @param oldName
	 */
	public void setOldName(String oldName) {
		this._oldName = oldName;
	}

	/**
	 * @param label
	 */
	public SetElementNameCommand(String label, String newName, String oldName, AcmeElement target) {
		super(label);
		_oldName = oldName;
		_newName = newName;
		_target = target;
	}

	

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
	      if (_target instanceof AcmeElementType) {
        	// Should raise a warning
        	if (!MessageDialog.openConfirm(Display.getCurrent().getActiveShell(), "Changing name of type", "Changing the name a type will not propagate to instances of the type, and visualizations will be lost. Continue?")) {
        		_did = false;
        		return;
        	}
        }
	    redo ();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		if (!_did) return;
        AcmeElement parent = _target.getParent();
  
        
        
		try {
			_target.setName(_newName);
			AcmeStudioChangeManager.globalChangeManager.onSetName (_target, _newName, null);
		} catch (AcmeIllegalNameException e) {
            try {
                _target.setName (_oldName);
                // setName removes element from parent. If the name already exists, then
                // _target is suddenly parentless. Thus, we need to add it again.
                if (parent.lookupName(_oldName, false) == null) {
                    if (parent instanceof AcmeSystem) {
                        AcmeSystem system = (AcmeSystem )parent;
                        if (_target instanceof AcmeComponent) {
                            AcmeComponent comp = (AcmeComponent )_target;
                            system.addComponent(comp);
                        }
                        else if (_target instanceof AcmeConnector) {
                            AcmeConnector conn = (AcmeConnector )_target;
                            system.addConnector(conn);
                        }
                    }
                    else if (parent instanceof AcmeConnector) {
                        AcmeConnector conn = (AcmeConnector )parent;
                        if (_target instanceof AcmeRole) {
                            AcmeRole role = (AcmeRole )_target;
                            conn.addRole (role);
                        }
                    }
                    else if (parent instanceof AcmeComponent) {
                        AcmeComponent comp = (AcmeComponent )parent;
                        if (_target instanceof AcmePort) {
                            AcmePort port = (AcmePort )_target;
                            comp.addPort (port);
                        }
                    }
                }
            }
            catch (AcmeIllegalNameException e1) {
                e1.printStackTrace();
            }
			this._errorReason = e;		
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		try {
			_target.setName (_oldName);
		} catch (AcmeIllegalNameException e) {
			// Should never occur because we're setting to the old name
		}
		AcmeStudioChangeManager.globalChangeManager.onSetName (_target, _oldName, null);
	}


}
