package AcmeStudio.model;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeFamily;

/**
 * @author schmerl
 *
 * Command, identified by getKind(), to assign or unassign a super type to/from
 * a type.
 */
public class SupertypeAssignmentCommand extends ModelCommand {
    
    private AcmeElementType _type;
    private AcmeElementType _supertype;
    private Object _announcer;
    
    public SupertypeAssignmentCommand (String commandType, AcmeElementType target, AcmeElementType supertype, Object a) {
        super (commandType);
        setKind (commandType);
        _type = target;
        _supertype = supertype;
        _announcer = a;   
    }

    public boolean canExecute () {
        return (_type != null && _supertype != null);
    }
    
    public boolean canUndo () {
        return !getKind ().equals (TypeAssignmentCommand.UNASSIGN_FAMILY);
    }
    
    public void execute () {
        if (getKind ().equals (TypeAssignmentCommand.ASSIGN_TYPE) && !(_type instanceof AcmeFamily)) {
            setLabel (AcmeStudioPlugin.getMessage ("SupertypeAssignmentCommand.AssignTypeLabel") + _supertype.getTypeName ()); //$NON-NLS-1$
            assignSuperType ();
        }
        else if (getKind ().equals (TypeAssignmentCommand.UNASSIGN_TYPE) && !(_type instanceof AcmeFamily)) {
            unassignSuperType ();
            setLabel (AcmeStudioPlugin.getMessage ("SupertypeAssignmentCommand.UnassignTypeLabel") + _supertype.getTypeName()); //$NON-NLS-1$
        }
    }
    
    public void assignSuperType () {
        _type.addSuperType (_supertype);
        AcmeStudioChangeManager.globalChangeManager.onAssignType (_type, _supertype, _announcer);
    }
    
    public void unassignSuperType () {
        _type.removeSuperType (_supertype);
        AcmeStudioChangeManager.globalChangeManager.onUnAssignType( _type, _supertype, _announcer);
    }
    
    public void redo () {
        execute ();
    }
    
    public void undo () {
        if (getKind ().equals (TypeAssignmentCommand.ASSIGN_TYPE) && !(_type instanceof AcmeFamily)) {
            unassignSuperType ();
        }
        else if (getKind ().equals (TypeAssignmentCommand.UNASSIGN_TYPE) && !(_type instanceof AcmeFamily)) {
            assignSuperType ();
        }
    }        
    
}
