package AcmeStudio.model;

import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeSystem;

/**
 * @author schmerl
 *
 * Command, identified by getKind(), to add or delete a system from an AcmeDesign.
 */
public class SystemCommand extends ModelCommand {
    private AcmeSystem _system;
    private AcmeDesign _design;
    private Object _announcer;
        
    public static final String ADD_SYSTEM = "$AddSys$";
    public static final String REMOVE_SYSTEM = "$DelSys$";

    public SystemCommand (String kind, AcmeSystem sys, AcmeDesign des, Object ann) {
        super (kind);
        setKind (kind);
        _system = sys; 
        _design = des;   
        _announcer = ann;
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute() {
        return _system != null && _design != null;
    }

    protected void addSystem () {
        try {
            _design.addSystem (_system);
            AcmeStudioChangeManager.globalChangeManager.onAddSystem (_design, _system, _announcer);   
        }
        catch (AcmeIllegalNameException e) {
            _errorReason = e;
        }
    }

    protected void delSystem () {
        _design.removeSystem (_system);
        AcmeStudioChangeManager.globalChangeManager.onRemoveSystem (_design, _system, _announcer);
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        if (getKind ().equals (ADD_SYSTEM)) {
            setLabel ("Add system " + _system.getName ());
            addSystem ();   
        }
        else if (getKind ().equals (REMOVE_SYSTEM)) {
            setLabel ("Delete system " + _system.getName ());
            delSystem ();
        }
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute ();
    }

    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        if (getKind ().equals (ADD_SYSTEM)) {
            delSystem ();   
        }
        else if (getKind ().equals (REMOVE_SYSTEM)) {
            addSystem ();
        }

    }

}
