/*
 * Created on Jul 17, 2003
 *
 */
package AcmeStudio.model;

import java.util.*;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.*;
import acmeLib.representations.AcmeBinding;
import acmeLib.representations.AcmeBindingsMap;
import acmeLib.representations.AcmeRepresentation;

/**
 * @author Bradley Schmerl
 * Created Jul 17, 2003
 */
public class WrapRepresentationCommand extends ModelCommand {

	protected Set _wrappedElements = new HashSet();
	protected Set _wrappedAttachments = new HashSet ();
	protected AcmeRepresentation _rep;
	protected AcmeComponent _newComp;

	protected Object _announcer;

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		super.redo();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		AcmeSystem sys = (AcmeSystem )_newComp.getParent ();
		AcmeRepresentation rep = _newComp.getRepresentation("rep");
		_newComp.removeRepresentation(rep);
		AcmeStudioChangeManager.globalChangeManager.onRemoveRepresentation(_newComp, rep, _announcer);
		sys.removeComponent(_newComp);
		AcmeStudioChangeManager.globalChangeManager.onRemoveComponent(sys, _newComp, this._announcer);
		
		Iterator i = _wrappedElements.iterator ();
		while (i.hasNext ()) {
			AcmeElement element = (AcmeElement )i.next ();
			if (element instanceof AcmeComponent) {
				AcmeComponent comp = (AcmeComponent )element;
				try {
					sys.addComponent (comp);
				} catch (AcmeIllegalNameException e) {
					e.printStackTrace();
				}
				AcmeStudioChangeManager.globalChangeManager.onAddComponent (sys, comp, this._announcer);
				
			}
			else if (element instanceof AcmeConnector) {
				AcmeConnector conn = (AcmeConnector )element;
				try {
					sys.addConnector (conn);
				} catch (AcmeIllegalNameException e) {
					e.printStackTrace();
				}
				AcmeStudioChangeManager.globalChangeManager.onAddConnector (sys, conn, _announcer);
			}
		}	
		i = _wrappedAttachments.iterator ();
		while (i.hasNext ()) {
			Object o = i.next ();
			AcmeAttachment att = (AcmeAttachment )o;
			sys.addAttachment(att);
			AcmeStudioChangeManager.globalChangeManager.onAddAttachment(sys, att, _announcer);
		}	
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		wrapIntoRepresentation();
	}

	/**
	 * @param label
	 */
	public WrapRepresentationCommand(String label) {
		super(label);
	}

	
	
	protected void wrapIntoRepresentation() {
		// portBindPoints = all ports in selection not attached to things in the selection
		Map portBindPoints = new HashMap();
		// roleBindPoints = all roles in selection not attached to things in the selection
		Map roleBindPoints = new HashMap();
		// Copy elements, to place in the representation
		Set copiedElements = new HashSet();
		Map oldPortToNewPort = new HashMap();
		Map oldRoleToNewRole = new HashMap();
		Set origPortAttachments = new HashSet();
		AcmeSystem origSystem = null;
		Iterator i = _wrappedElements.iterator();
		while (i.hasNext()) {
			AcmeElement element = (AcmeElement) i.next();
			if (element == null) continue;
			origSystem = (AcmeSystem) element.getParent();
			AcmeElement elementCopy = element.copy();
			copiedElements.add(elementCopy);
			if (element instanceof AcmeComponent) {
				AcmeComponent origComp = (AcmeComponent) element;
				AcmeComponent copyComp = (AcmeComponent) elementCopy;
				AcmeEnumeration origPorts = origComp.getPorts();
				while (origPorts.moreItems()) {
					AcmePort origPort = (AcmePort) origPorts.getNextItem();
					AcmePort copyPort = copyComp.getPort(origPort.getName());
					oldPortToNewPort.put(origPort, copyPort);
					if (origSystem.getAttachments(origPort).getSize() != 0) {
						origPortAttachments.add(origPort);
						AcmeEnumeration attachments =
							origSystem.getAttachments(origPort);
						while (attachments.moreItems()) {
							AcmeAttachment attachment =
								(AcmeAttachment) attachments.getNextItem();
							if (!_wrappedElements
								.contains(attachment.getRole().getParent())) {
								portBindPoints.put(origPort, origPort.copy());
							}
							else {
								/* BRS: July 2004
									Fixed bug that stopped undo from working properly.
									Add the attachment as one that needs to be redone if the
									the command is undone*/
								_wrappedAttachments.add(attachment);
							}
						}
					}

				}
			} else if (element instanceof AcmeConnector) {
				AcmeConnector origConn = (AcmeConnector) element;
				AcmeConnector copyConn = (AcmeConnector) elementCopy;
				AcmeEnumeration origRoles = origConn.getRoles();
				while (origRoles.moreItems()) {
					AcmeRole origRole = (AcmeRole) origRoles.getNextItem();
					AcmeRole copyRole = copyConn.getRole(origRole.getName());
					oldRoleToNewRole.put(origRole, copyRole);
					AcmeEnumeration attachments =
						origSystem.getAttachments(origRole);
					while (attachments.moreItems()) {
						AcmeAttachment attachment =
							(AcmeAttachment) attachments.getNextItem();
						if (!_wrappedElements
							.contains(attachment.getPort().getParent())) {
							roleBindPoints.put(origRole, origRole.copy());
						}
						else {
							/* BRS: July 2004
								Fixed bug that stopped undo from working properly.
								Add the attachment as one that needs to be redone if the
								the command is undone*/
							_wrappedAttachments.add (attachment);
						}
					}
				}
			}

		}

		try {
			// !portBindPoints.empty && wrapping into connector rep -> error
			// !roleBindPoints.emtpy && wrapping into component rep -> error
			if (!roleBindPoints.isEmpty())
				return;
			// Copy all elements in the selection
			String name = Helper.getUniqueName(origSystem, "compRep");
			AcmeComponent newComponent = new AcmeComponent(name); 
			_newComp = newComponent;
			origSystem.addComponent(newComponent);
			newComponent.setParentDesign(origSystem.getParentDesign());
			SystemView.setLocation(newComponent, calculateCenter());
			AcmeStudioChangeManager.globalChangeManager.onAddComponent(
				origSystem,
				newComponent,
				this._announcer);
			i = portBindPoints.keySet().iterator();
			while (i.hasNext()) {
				AcmePort port = (AcmePort) i.next();
				AcmePort newPort = (AcmePort) portBindPoints.get(port);
				newPort.setParent (null);
				newPort.setName (Helper.getUniqueName(newComponent, newPort.getName()));
				newComponent.addPort(newPort);
				AcmeStudioChangeManager.globalChangeManager.onAddPort(
					newComponent,
					newPort,
					this._announcer);
				// Weave in attachments
				AcmeEnumeration attachments = origSystem.getAttachments(port);
				while (attachments.moreItems()) {
					AcmeAttachment attachment =
						(AcmeAttachment) attachments.getNextItem();
					AcmeRole attachedRole = attachment.getRole();
					// Always remove the attachment
					origSystem.removeAttachment(attachment);
					_wrappedAttachments.add (attachment);
					AcmeStudioChangeManager
						.globalChangeManager
						.onRemoveAttachment(
						origSystem,
						attachment,
						this._announcer);
					if (!_wrappedElements
						.contains(attachment.getRole().getParent())) {

						AcmeAttachment newAtt =
							new AcmeAttachment(newPort, attachedRole);
						origSystem.addAttachment(newAtt);
						AcmeStudioChangeManager
							.globalChangeManager
							.onAddAttachment(
							origSystem,
							newAtt,
							this._announcer);
					}
				}
			}
			// Create a new system and paste copies into the system
			AcmeSystem repSystem = new AcmeSystem("rep");
			AcmeEnumeration families = origSystem.getDeclaredTypes ();
			while (families.moreItems ()) {
				AcmeFamily family = (AcmeFamily )families.getNextItem ();
				repSystem.addDeclaredType(family);
			}
			families = origSystem.getInstantiatedTypes();
			while (families.moreItems ()) {
				AcmeFamily family = (AcmeFamily )families.getNextItem ();
				repSystem.addInstantiatedType(family);
			}
			repSystem.setParentDesign (origSystem.getParentDesign ()); 
			_rep =
				new AcmeRepresentation(
					repSystem,
					newComponent,
					new AcmeBindingsMap(),
					"rep");
			newComponent.addRepresentation(_rep);
			
			AcmeStudioChangeManager.globalChangeManager.onAddRepresentation(
				newComponent,
				_rep,
				this._announcer);

			// Paste the copies in
			i = copiedElements.iterator();
			while (i.hasNext()) {
				AcmeElement cpElement = (AcmeElement) i.next();
				if (cpElement instanceof AcmeConnector) {
					AcmeConnector cpConn = (AcmeConnector) cpElement;
					repSystem.addConnector(cpConn);
					AcmeStudioChangeManager.globalChangeManager.onAddConnector(
						repSystem,
						cpConn,
						this._announcer);
				} else if (cpElement instanceof AcmeComponent) {
					AcmeComponent cpComp = (AcmeComponent) cpElement;
					repSystem.addComponent(cpComp);
					AcmeStudioChangeManager.globalChangeManager.onAddComponent(
						repSystem,
						cpComp,
						this._announcer);
				}
			}

			// Duplicate original attachments
			i = origPortAttachments.iterator();
			while (i.hasNext()) {
				AcmePort origPort = (AcmePort) i.next();
				AcmePort newPort = (AcmePort) oldPortToNewPort.get(origPort);
				AcmeEnumeration attachments =
					origSystem.getAttachments(origPort);
				while (attachments.moreItems()) {
					AcmeAttachment att =
						(AcmeAttachment) attachments.getNextItem();
					if (_wrappedElements.contains(att.getRole().getParent())) {
						AcmeAttachment newAtt =
							new AcmeAttachment(
								newPort,
								(AcmeRole) oldRoleToNewRole.get(att.getRole()));
						repSystem.addAttachment(newAtt);
						AcmeStudioChangeManager
							.globalChangeManager
							.onAddAttachment(
							repSystem,
							newAtt,
							_announcer);
					}
				}
			}

			// Add the bindings
			i = portBindPoints.keySet().iterator();
			while (i.hasNext()) {
				AcmePort oldPort = (AcmePort) i.next();
				AcmePort innerPort = (AcmePort) oldPortToNewPort.get(oldPort);
				AcmePort outerPort = (AcmePort) portBindPoints.get(oldPort);
				AcmeBinding binding = new AcmeBinding(outerPort, innerPort);
				((AcmeBindingsMap) _rep.getAbstractionMap()).addBinding(
					binding);
				AcmeStudioChangeManager.globalChangeManager.onAddBinding(
					_rep,
					binding,
					this._announcer);
			}

			// Delete elements in the selection
			i = _wrappedElements.iterator();
			while (i.hasNext()) {
				Object o = i.next();
				if (o instanceof AcmeComponent) {

					AcmeComponent comp = (AcmeComponent) o;
					origSystem.removeComponent(comp);
					AcmeStudioChangeManager
						.globalChangeManager
						.onRemoveComponent(
						origSystem,
						comp,
						this._announcer);

				} else if (o instanceof AcmeConnector) {
					AcmeConnector conn = (AcmeConnector) o;
					origSystem.removeConnector(conn);
					AcmeStudioChangeManager
						.globalChangeManager
						.onRemoveConnector(
						origSystem,
						conn,
						this._announcer);
				}
			}
		} catch (AcmeIllegalNameException e) {
			MessageDialog.openError(Display.getCurrent().getActiveShell(), "Error", "Could not convert to representation: " + e.getMessage ());
		}
	}

	/**
	 * @return
	 */
	private Point calculateCenter() {
		return SystemView.getBounds(_wrappedElements).getCenter();
	}

	public void setTargetElements(List elements) {
		if (!elements.isEmpty()) {
			_wrappedElements.clear();
			_wrappedElements.addAll(elements);
			
			// Weed out null pointers
			Iterator i = _wrappedElements.iterator ();
			while (i.hasNext ()) {
				Object o = i.next ();
				if (o == null) i.remove ();
			}
		}

	}

}
