/*
 * Created on Aug 6, 2004
 * 
 */
package AcmeStudio.prefs;

import org.eclipse.jface.preference.*;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import AcmeStudio.AcmeStudioPlugin;
import com.swtdesigner.FieldLayoutPreferencePage;
import com.swtdesigner.SWTResourceManager;

/**
 * @author Bradley Schmerl
 *
 */
public class ConnectionPatternPreferences extends FieldLayoutPreferencePage
		implements IWorkbenchPreferencePage {

	private Composite _paletteComposite;
	public static final String ELEMENT_HIGHTLIGHT_COLOR = "Connection.highlightColor";
	public static final String ELEMENT_HIGHLIGHT_WIDTH = "Connection.highlightWidth";

	private static final Color DEFAULT_HIGHLIGHT_COLOR = SWTResourceManager.getColor (240, 0, 0);
	public static final String  SHOW_PALETTE_TREE			= "DiagramEditor.showPaletteTree";
	public static final String  RESTRICT_TO_RULES 		= "DiagramEditor.restrictToRules";
	public static final String PALETTE_DISPLAY = "Connection.ruleDisplayType";
	
	public static final String USE_PATTERN_IMAGE = "image";
	public static final String USE_PATTERN_TEXT = "text";
	private BooleanFieldEditor	_showPaletteField;
	private RadioGroupFieldEditor	_paletteDisplayPrefs;
	


	
	public ConnectionPatternPreferences () {
		super ("Connection Patterns");
	}
	/**
	 * 
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
//	protected Control ss(Composite parent) {
////		Composite groupParent = getFieldEditorParent();
//		Composite groupParent = new Composite (parent, SWT.NULL);
//		Composite composite;
//		groupParent.setLayout (new GridLayout ());
//		Group feedbackOptions = new Group (groupParent, SWT.NONE);
//		feedbackOptions.setText ("Connection Feedback Options");
//		GridData fogd = new GridData (GridData.FILL_BOTH);
//		fogd.horizontalSpan = GridData.FILL;
//		feedbackOptions.setLayoutData(fogd);
//		addField (new ColorFieldEditor (ELEMENT_HIGHTLIGHT_COLOR, "Color to use when highlighting connection endpoints", feedbackOptions));
//		addField (new IntegerFieldEditor (ELEMENT_HIGHLIGHT_WIDTH, "Width of line to use when highlighting connection endpoints", feedbackOptions));
//		Group behaviorOptions = new Group (groupParent, SWT.NONE);
//		behaviorOptions.setText("Connection options:");
//		GridData bogd = new GridData (GridData.FILL_BOTH);
//		bogd.horizontalSpan = GridData.FILL;
//		behaviorOptions.setLayoutData(bogd);
//		BooleanFieldEditor usePalettePref = new BooleanFieldEditor (SHOW_PALETTE_TREE, "Allow users to choose specific rules from the palette", behaviorOptions);
//		addField (usePalettePref);
//		BooleanFieldEditor useRoleMatchPref = new BooleanFieldEditor (RESTRICT_TO_RULES, "Use connection patterns to match role dragging", behaviorOptions);
//		addField (useRoleMatchPref);
//		
//		return groupParent;
//	}

	/**
	 * @param workbench
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	/**
	 * @return
	 * @see org.eclipse.jface.preference.PreferencePage#doGetPreferenceStore()
	 */
	protected IPreferenceStore doGetPreferenceStore() {
		return AcmeStudioPlugin.getDefault().getPreferenceStore();
	}
	
	public static void setDefaults (IPreferenceStore store) {
		store.setDefault(SHOW_PALETTE_TREE, false);
		store.setDefault (RESTRICT_TO_RULES, true);
		store.setDefault(ELEMENT_HIGHLIGHT_WIDTH, 4);
		PreferenceConverter.setDefault(store, ELEMENT_HIGHTLIGHT_COLOR, DEFAULT_HIGHLIGHT_COLOR.getRGB());
		store.setDefault (PALETTE_DISPLAY, USE_PATTERN_IMAGE);
	}
	/**
	 * @param parent
	 * @return
	 * @see com.swtdesigner.FieldLayoutPreferencePage#createPageContents(org.eclipse.swt.widgets.Composite)
	 **/
	protected Control createPageContents(Composite parent) {
		Composite composite;
		Composite container = new Composite (parent, SWT.NULL);
		GridLayout gridLayout = new GridLayout ();
		container.setLayout (gridLayout);

		Group group = new Group(container, SWT.NONE);
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_FILL));
		group.setText("Feedback Options:");
		group.setLayout(new GridLayout());

		composite = new Composite(group, SWT.NONE);

		addField(new ColorFieldEditor(ConnectionPatternPreferences.ELEMENT_HIGHTLIGHT_COLOR, "Color to use when highlighting connection endpoints:", composite));

		composite = new Composite(group, SWT.NONE);

		addField(new IntegerFieldEditor("", "Width of line to use when highlighting connection endpoints:", composite));

		group = new Group(container, SWT.NONE);
		group.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		group.setText("Interaction Options:");
		group.setLayout(new GridLayout());

		composite = new Composite(group, SWT.NONE);

		 _showPaletteField = new BooleanFieldEditor(ConnectionPatternPreferences.SHOW_PALETTE_TREE, "Allow specific rules to be chosen from the palette", composite);

		addField(_showPaletteField);
		

		_paletteComposite = new Composite(group, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalIndent = 20;
		_paletteComposite.setLayoutData(gridData);

		 _paletteDisplayPrefs = new RadioGroupFieldEditor(ConnectionPatternPreferences.PALETTE_DISPLAY, "Palette display", 1, new String[][] {new String[] {"Use text description of pattern", ConnectionPatternPreferences.USE_PATTERN_TEXT}, new String[] {"Use image depiction of pattern", ConnectionPatternPreferences.USE_PATTERN_IMAGE}}, _paletteComposite);
		addField(_paletteDisplayPrefs);

		composite = new Composite(group, SWT.NONE);

		addField(new BooleanFieldEditor(ConnectionPatternPreferences.RESTRICT_TO_RULES, "Use connection patterns when dragging roles", composite));
		
		
	
		return container;
	}
	
	
	/**
	 * @param event
	 * @see com.swtdesigner.FieldLayoutPreferencePage#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 **/
	public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange (event);
		_paletteComposite.setEnabled (_showPaletteField.getBooleanValue ());
		_paletteDisplayPrefs.setEnabled (_showPaletteField.getBooleanValue (), _paletteComposite);
	}
}
