/*
 * Created on Aug 5, 2003
 */
package AcmeStudio.prefs;

import org.eclipse.jface.preference.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.util.Debug;

/**
 * Allows user to set debug preferences, including the settings offered by
 * Log4J such as log level.
 * 
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 */
public class DebugPreferences
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	/**
	 * @param style
	 */
	public DebugPreferences() {
		super("Debugging", FieldEditorPreferencePage.GRID);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors() {
        RadioGroupFieldEditor levelRadio = new RadioGroupFieldEditor(
            Debug.PREFKEY_LEVEL,
            "AcmeStudio debugging level:",
            6,
            new String[][] {
                { "Off", Debug.LEVEL_NONE },
                { "Debug", Debug.LEVEL_DEBUG },
                { "Info", Debug.LEVEL_INFO },
                { "Warning", Debug.LEVEL_WARN },
                { "Error", Debug.LEVEL_ERROR },
                { "Fatal", Debug.LEVEL_FATAL}
            },
            getFieldEditorParent(),
            true
        );
        addField(levelRadio);

        Group fileGrp = new Group (getFieldEditorParent(), SWT.NONE);
        fileGrp.setText("Log file options:");
        fileGrp.setLayout(new FillLayout(SWT.VERTICAL));
        GridData ggd = new GridData(GridData.FILL_HORIZONTAL);
        fileGrp.setLayoutData(ggd);
        StringFieldEditor logPattern =
            new StringFieldEditor (Debug.PREFKEY_PATTERN, "Log entry pattern:", fileGrp);
        FieldEditor maxSize =
            new IntegerFieldEditor (Debug.PREFKEY_MAX_SIZE, "Maximum file size (KB) before rolling over:", fileGrp);
        IntegerFieldEditor maxBackups =
            new IntegerFieldEditor (Debug.PREFKEY_MAX_BACKUPS, "Number of lastest rollover files to keep:", fileGrp);
        StringFieldEditor logFile =
            new StringFieldEditor (Debug.PREFKEY_FILE, "Name of log file:", fileGrp);
        addField (logPattern);
        addField (maxSize);
        addField (maxBackups);
        addField (logFile);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

    /* (non-Javadoc)
     * @see org.eclipse.jface.preference.PreferencePage#doGetPreferenceStore()
     */
    protected IPreferenceStore doGetPreferenceStore () {
        return AcmeStudioPlugin.getDefault().getPreferenceStore();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
     */
    protected void performDefaults() {
        super.performDefaults();
        DebugPreferences.setDefaults(doGetPreferenceStore());
    }

    public static void setDefaults (IPreferenceStore store) {
        store.setDefault (Debug.PREFKEY_LEVEL, Debug.DEFAULT_LEVEL);
        store.setDefault (Debug.PREFKEY_PATTERN, Debug.DEFAULT_PATTERN);
        store.setDefault (Debug.PREFKEY_MAX_SIZE, Debug.DEFAULT_MAX_SIZE);
        store.setDefault (Debug.PREFKEY_MAX_BACKUPS, Debug.DEFAULT_MAX_BACKUPS);
        store.setDefault (Debug.PREFKEY_FILE, Debug.DEFAULT_FILE);
    }

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 */
	public boolean performOk() {
        boolean rv = super.performOk();
        // Bug fix #381: debug setting will now be effected properly.
        Debug.reset();
		return rv;
	}

}
