/*
 * Created on Aug 11, 2004
 *
 */
package AcmeStudio.prefs;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorActionBarContributor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.ui.editors.MultiPageActionBarContributor;


/**
 * @author Bradley Schmerl
 * Created Aug 11, 2004
 * Contains preferences for what goes in a diagram's toolbar
 */
public class DiagramToolbarPreferences extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

	public static final String	ALIGNMENT_TOOLBAR	= "DiagramEditor.AlignmentTools";
	public static final String ZOOM_TOOLBAR = "DiagramEditor.ZoomTools";
	public static final String ORDERING_TOOLBAR = "DiagramEditor.OrderingTools";


	public DiagramToolbarPreferences() {
		super ("Toolbar", FieldEditorPreferencePage.GRID);
	}


	public void init(IWorkbench workbench) {
		
	}

	/**
	 * 
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 **/
	protected void createFieldEditors() {
		BooleanFieldEditor alignmentTools = new BooleanFieldEditor (ALIGNMENT_TOOLBAR, "Alignment tools", getFieldEditorParent());
		BooleanFieldEditor zoomTools = new BooleanFieldEditor (ZOOM_TOOLBAR, "Zoom tools", getFieldEditorParent ());
		BooleanFieldEditor orderTools = new BooleanFieldEditor (ORDERING_TOOLBAR, "Ordering tools", getFieldEditorParent());
		addField (alignmentTools);
		addField (zoomTools);
		addField (orderTools);
	}
	
	
	/**
	 * @return
	 * @see org.eclipse.jface.preference.PreferencePage#doGetPreferenceStore()
	 **/
	protected IPreferenceStore doGetPreferenceStore() {
		return AcmeStudioPlugin.getDefault().getPreferenceStore();
	}
	
	public static void setDefaults (IPreferenceStore store) {
		store.setDefault (ALIGNMENT_TOOLBAR, true);
		store.setDefault (ZOOM_TOOLBAR, true);
		store.setDefault (ORDERING_TOOLBAR, true);
	}
	
	
	/**
	 * 
	 * @see org.eclipse.jface.preference.PreferencePage#performApply()
	 **/
	protected void performApply() {
		super.performApply ();
		updateToolbars ();
	}
	
	
	/**
	 * @return
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 **/
	public boolean performOk() {
		boolean ret = super.performOk ();
		updateToolbars ();
		return ret;
	}


	/**
	 * 
	 */
	private void updateToolbars() {
		MultiPageAcmeEditor mpe = ASContext.current().getActiveEditor();
		if (mpe != null) {
			 IEditorActionBarContributor abc = mpe.getEditorSite().getActionBarContributor();
			IPreferenceStore store = getPreferenceStore();
			if (abc instanceof MultiPageActionBarContributor) {
				((MultiPageActionBarContributor )abc).updateTools (store.getBoolean (ALIGNMENT_TOOLBAR), store.getBoolean(ORDERING_TOOLBAR), store.getBoolean(ZOOM_TOOLBAR));
			}
		}
	}
}
