/*
 * Created on Jun 3, 2004
 *
 */
package AcmeStudio.ui.editors;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.ListenerList;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.ui.*;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.part.MultiPageSelectionProvider;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import AcmeStudio.core.resource.provider.ASResourceProvider;
import AcmeStudio.model.TypeAssignmentCommand;
import AcmeStudio.ui.editors.actions.NewDesignAnalysisAction;
import AcmeStudio.ui.editors.actions.NewElementTypeAction;
import AcmeStudio.ui.editors.actions.NewPropertyTypeAction;
import AcmeStudio.ui.editors.actions.SuperTypeAssignmentMenuAction;
import AcmeStudio.ui.editors.diagram.actions.NavigateToImportedFamilyAction;
import AcmeStudio.ui.editors.style.parts.ElementTypeList;
import AcmeStudio.ui.editors.style.parts.StyleList;
import AcmeStudio.ui.editors.style.parts.TypesPage;
import AcmeStudio.ui.views.DesignBrowser;
import AcmeStudio.ui.views.ElementViewPage;
import AcmeStudio.ui.views.IElementViewPage;
import AcmeStudio.util.*;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.elements.*;


/**
 * @author Bradley Schmerl
 * Created Jun 3, 2004
 */
public class AcmeStyleEditor extends FormEditor implements ISelectionChangedListener, ISelectionProvider, AcmeModelChangeListener {

	private MultiPageAcmeEditor	_parentEditor;
	private ASResourceProvider	_resourceProv;
	private AcmeFamily	_acmeFamily;
	private AcmeElementType	_selectedType;
	
	private ListenerList _selectionChangedListeners = new ListenerList ();
	
	private Map _actions = new HashMap ();
	private java.util.List  _selectionActions = new ArrayList (2);
	private TypesPage	_stylePage;
	private DiagramEditor	_structureEd;
	private DesignAnalysisEditor _analysisEd;
	private DesignBrowser _designBrowser = null;
	private ElementViewPage _elementViewPage = null;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
	 */
	protected void setInput(IEditorInput input) {
		super.setInput (input);
		if (input instanceof IFileEditorInput) {
		   Debug.diagnoseln ("AcmeStyleEditor.setInput: " + "Unexpected IFileEditorInput received!");
		}
		else if (input instanceof ASEditorInput) {
			ASEditorInput model = (ASEditorInput )input;
			_resourceProv = model.getResourceProvider();
			AcmeSystem system = (AcmeSystem )model.getElement ();
			_acmeFamily = Helper.getAcmeFamily (system.getParentDesign());
			this.setContentDescription (getFamily().getName () + " Family");
			_selectedType = getFamily();
		}
		createActions ();
		setPartName (input.getName ());
	}
	/**
	 * 
	 */
	private void createActions() {
		_actions.put (TypeAssignmentCommand.ASSIGN_FAMILY, new SuperTypeAssignmentMenuAction (this, TypeAssignmentCommand.ASSIGN_FAMILY));
        _actions.put (TypeAssignmentCommand.UNASSIGN_FAMILY, new SuperTypeAssignmentMenuAction (this, TypeAssignmentCommand.UNASSIGN_FAMILY));
        _actions.put (TypeAssignmentCommand.ASSIGN_TYPE, new SuperTypeAssignmentMenuAction (this, TypeAssignmentCommand.ASSIGN_TYPE));
        _actions.put (TypeAssignmentCommand.UNASSIGN_TYPE, new SuperTypeAssignmentMenuAction (this, TypeAssignmentCommand.UNASSIGN_TYPE));
        
        _actions.put (NewElementTypeAction.NEW_COMPONENT_TYPE, new NewElementTypeAction (this, NewElementTypeAction.NEW_COMPONENT_TYPE, getFamily()));
        _actions.put (NewElementTypeAction.NEW_CONNECTOR_TYPE, new NewElementTypeAction (this, NewElementTypeAction.NEW_CONNECTOR_TYPE, getFamily()));
        _actions.put (NewElementTypeAction.NEW_PORT_TYPE, new NewElementTypeAction (this, NewElementTypeAction.NEW_PORT_TYPE, getFamily()));
        _actions.put (NewElementTypeAction.NEW_ROLE_TYPE, new NewElementTypeAction (this, NewElementTypeAction.NEW_ROLE_TYPE, getFamily()));
        _actions.put (NewPropertyTypeAction.ID, new NewPropertyTypeAction (this, getFamily()));
        _actions.put (NewDesignAnalysisAction.ID, new NewDesignAnalysisAction (this, getFamily()));
        _actions.put (NavigateToImportedFamilyAction.ID, new NavigateToImportedFamilyAction (this)) ;   
        
        _selectionActions.add (getAction (TypeAssignmentCommand.ASSIGN_FAMILY));
        _selectionActions.add (getAction (TypeAssignmentCommand.UNASSIGN_FAMILY));
        _selectionActions.add (getAction (TypeAssignmentCommand.ASSIGN_TYPE));
        _selectionActions.add (getAction (TypeAssignmentCommand.UNASSIGN_TYPE));
        _selectionActions.add (getAction (NavigateToImportedFamilyAction.ID));
         
	}
	/**
	 * @param assign_family
	 * @return
	 */
	public IAction  getAction(String actionID) {
		return (IAction )_actions.get (actionID);
	}
	/**
	 * 
	 */
	public AcmeStyleEditor(MultiPageAcmeEditor parentEditor) {
		super ();
		
		_parentEditor = parentEditor;
		AcmeStudioChangeManager.registerListener(this);
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.editor.FormEditor#addPages()
	 */
	protected void addPages() {
		AcmeSystem sys = null;
		if (getEditorInput () instanceof ASEditorInput) {
			sys = Helper.getAcmeSystem (((ASEditorInput )getEditorInput ()).getElement ());
		}
		_stylePage = new TypesPage (this);
		_structureEd = new DiagramEditor (_parentEditor, sys);
		_analysisEd = new DesignAnalysisEditor (this);
		try {
			addPage (_stylePage);
			addPage (_structureEd, getEditorInput ());
			addPage (_analysisEd, getEditorInput ());
			setPageText(0, "Types");
			setPageText(1, "Structure");
			setPageText(2, "Functions");
		}
		catch (PartInitException e) {
			Debug.diagnoseln(e);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#doSaveAs()
	 */
	public void doSaveAs() {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		return false;
	}

    public AcmeFamily getFamily () {
        return _acmeFamily;
    }

    /**
	 * @param object
	 */
	public void refocusStyleLists(ElementTypeList source) {
	}
	/**
	 * @param family
	 */
	public void dispatchNewSelection(AcmeElementType acmeType) {
		_selectedType = acmeType;
		fireSelectionChanged (new StructuredSelection (_selectedType));
	}
	/**
	 * @param selection
	 */
	private void fireSelectionChanged(StructuredSelection selection) {
		Iterator as = _selectionActions.iterator();
		while (as.hasNext ()) {
			SelectionAction a = (SelectionAction )as.next ();
			a.update ();
		}
		
		SelectionChangedEvent event = new SelectionChangedEvent (this, selection);
		Object [] listeners = _selectionChangedListeners.getListeners ();
		
		for (int i = 0; i < listeners.length; i++) {
			((ISelectionChangedListener )listeners [i]).selectionChanged (event);
		}
		// Report change to the site's selection provider so that it can enable global actions
		getSite ().getSelectionProvider().setSelection(selection);
	}
	/**
	 * @param list
	 */
	public void refreshStyleLists(StyleList source) {
		if (_stylePage.getComponentsSection () != null) {
			if (source != _stylePage.getComponentsSection().getStyleList()) {
				_stylePage.getComponentsSection().getStyleList().refresh(source);
			}
		}
		if (_stylePage.getConnectorsSection() != null) {
			if (source != _stylePage.getConnectorsSection ().getStyleList()) {
				_stylePage.getConnectorsSection ().getStyleList().refresh (source);
			}
		}
		if (_stylePage.getPortsSection() != null && source != _stylePage.getPortsSection().getStyleList()) {
			_stylePage.getPortsSection().getStyleList().refresh(source);
		}
		if (_stylePage.getRolesSection() != null && source != _stylePage.getRolesSection().getStyleList()) {
			_stylePage.getRolesSection().getStyleList().refresh(source);
		}
	}
	/**
	 * 
	 */
	public void registerKeyBindings() {
		IKeyBindingService kbs = getEditorSite ().getKeyBindingService();
		IAction action;
		
		action = this.getAction (ActionFactory.UNDO.getId ());
		if (action != null) {
			action.setActionDefinitionId(ITextEditorActionDefinitionIds.UNDO);
			kbs.registerAction(action);
		}
		action = this.getAction (ActionFactory.REDO.getId ());
		if (action != null) {
			action.setActionDefinitionId(ITextEditorActionDefinitionIds.REDO);
			kbs.registerAction (action);
		}
	}
	

	/**
	 * @return
	 */
	public ISelection getSelection() {
		return new StructuredSelection (_selectedType);
	}
	
	public Object getAdapter (Class adapter) {
		if (adapter == IContentOutlinePage.class) {
			return _designBrowser;
		}
		else if (adapter == IElementViewPage.class) {
			return _elementViewPage;
		}
		else if (adapter == ActionRegistry.class) {
			return this.getActiveEditor ().getAdapter (adapter);
		}
		if (adapter == CommandStack.class) {
			return AcmeStudioChangeManager.globalChangeManager.getCommandStack(_resourceProv.getASDesign());
		}
		Object ad = super.getAdapter (adapter);
		if (ad == null && adapter.equals (CommandStack.class)) {
			return _parentEditor.getAdapter (adapter);
		}
		return ad;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		if (!(event.getSource () instanceof MultiPageSelectionProvider)) {
			IStructuredSelection iss = (IStructuredSelection )event.getSelection ();
			if (iss.getFirstElement () instanceof AcmeElementType) {
				_stylePage.setSelection (iss);
			}
			else if (iss.getFirstElement () instanceof AcmeElement) {
				if (this.getActiveEditor () == _structureEd) {
					_structureEd.selectionChanged (event);
				}
			}
		}
	}
	
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.MultiPageEditorPart#pageChange(int)
	 */
	protected void pageChange(int newPageIndex) {
		super.pageChange (newPageIndex);
		IEditorPart activeEditor = getEditor (newPageIndex);
		
		// BUGFIX: To get around FormEditor bug
		if (getContainer() instanceof CTabFolder) {
			CTabFolder folder = (CTabFolder )getContainer();
			if (folder.getItem(newPageIndex).getData() == null) {
				folder.getItem(newPageIndex).setData(activeEditor);
			}
			
		}
		_parentEditor.setActiveSubEditor(activeEditor);
	}
	
	public void setEditable (boolean canEdit) {
	}
	
	public void setDirty (boolean d) {
		if (_resourceProv != null && _resourceProv.isDirty () != d) {
			_resourceProv.setDirty(d);
		}
		firePropertyChange (IEditorPart.PROP_DIRTY);
		firePropertyChange (IEditorPart.PROP_INPUT);
	}
	/**
	 * @return
	 */
	public DiagramEditor getDiagramEditor() {
		return _structureEd;
	}
	/**
	 * @return
	 */
	public ASResourceProvider getResourceProvider() {
		return _resourceProv;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#addSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void addSelectionChangedListener(ISelectionChangedListener listener) {
		_selectionChangedListeners.add(listener);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#removeSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void removeSelectionChangedListener(ISelectionChangedListener listener) {
		_selectionChangedListeners.remove (listener);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#setSelection(org.eclipse.jface.viewers.ISelection)
	 */
	public void setSelection(ISelection selection) {
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
	 */
	public void changeModel(AcmeStudioChangeEvent evt) {
		switch (evt.getEventType ()) {
			case AcmeStudioChangeManager.ADD_TYPE:
			case AcmeStudioChangeManager.REMOVE_TYPE:
			case AcmeStudioChangeManager.SET_NAME:
			case AcmeStudioChangeManager.SET_COMMENT: {
				_stylePage.getComponentsSection().refresh ();
				_stylePage.getConnectorsSection().refresh ();
				_stylePage.getPortsSection().refresh ();
				_stylePage.getRolesSection().refresh ();
				_stylePage.getPropertiesSection().refresh ();
				_stylePage.getManagedForm().reflow(true);
			}
		}
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.util.AcmeModelChangeListener#interestedInEvent(int)
	 */
	public boolean interestedInEvent(int eventType) {
		return eventType == AcmeStudioChangeManager.ADD_TYPE || eventType == AcmeStudioChangeManager.REMOVE_TYPE
				|| eventType == AcmeStudioChangeManager.SET_NAME || eventType == AcmeStudioChangeManager.SET_COMMENT;
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		super.dispose ();
		AcmeStudioChangeManager.removeListener(this);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setSite(org.eclipse.ui.IWorkbenchPartSite)
	 */
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.MultiPageEditorPart#getEditor(int)
	 */
	protected IEditorPart getEditor(int pageIndex) {
		IEditorPart editor =  super.getEditor (pageIndex);
		return editor;
	}

    private AcmeDesign getDesign () {
        return (getFamily() == null) ? null : getFamily().getParentDesign();
    }

}
