package AcmeStudio.ui.editors;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.projection.ProjectionSupport;
import org.eclipse.jface.text.source.projection.ProjectionViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IKeyBindingService;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.texteditor.AbstractDecoratedTextEditor;
import org.eclipse.ui.texteditor.DefaultRangeIndicator;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditorActionConstants;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.actions.OpenTypeHierarchyAction;
import AcmeStudio.core.resource.provider.IResourceProviderListener;
import AcmeStudio.core.resource.provider.ResourceProviderEvent;
import AcmeStudio.ui.editors.text.AcmeFoldingStructureProvider;
import AcmeStudio.ui.editors.text.AcmeSourceViewerConfiguration;
import AcmeStudio.ui.editors.text.actions.TextEditorTypeHierarchyAction;
import AcmeStudio.ui.editors.text.filters.FilterOutVisAction;
import acmeLib.AcmeObject;
import acmeLib.core.io.Region;

/**
 * @author schmerl
 * 
 * History:
 *   = BugFix# 359, entirely removed cursor position saving and reinstatement.
 *     Default AbstractDecoratedTextEditor's cursor behavior works beautifully well.
 */
public class AcmeTextEditor
	extends AbstractDecoratedTextEditor
	implements ISelectionProvider, IResourceProviderListener, ISelectionChangedListener {

	/**
	 * @param parent
	 * @param ruler
	 * @param styles
	 * @return
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#createSourceViewer(org.eclipse.swt.widgets.Composite, org.eclipse.jface.text.source.IVerticalRuler, int)
	 **/
	protected ISourceViewer createSourceViewer(Composite parent, IVerticalRuler ruler, int styles) {
		ISourceViewer viewer = new ProjectionViewer (parent, ruler, getOverviewRuler(), isOverviewRulerVisible(), styles);
		getSourceViewerDecorationSupport(viewer);
		return viewer;
	}
	private MultiPageAcmeEditor _parentEditor;
	private boolean _canEdit = true;
	private AcmeObject _selectedObject;
	/**
	 * Constructor for AcmeTextEditor.
	 */
	public AcmeTextEditor(MultiPageAcmeEditor parentEditor) {
		super();

		_parentEditor = parentEditor;
		setDocumentProvider(parentEditor.getResourceProvider());
		// This causes dirty bit to turn on!  Bug Fix #38
		//        parentEditor.getResourceProvider().setIsSynchedWithText(false);
		//        try {
		//            parentEditor.getResourceProvider().updateText();
		//        }
		//        catch (CoreException e) {
		//            Debug.errorln(e);
		//        }
	}

	/**
	 * @see AcmeStudio.core.resource.provider.IResourceProviderListener#providerChanged(ResourceProviderEvent)
	 */
	public void providerChanged(ResourceProviderEvent event) {
		IEditorInput input = event.getProvider().getTextEditorInput();
		setInput(input);
	}

	/**
	 * @see org.eclipse.ui.texteditor.ITextEditor#close(boolean)
	 */
	public void close(boolean save) {
		setSite(_parentEditor.getSite()); // SWC:  to prevent ClassCastException
		super.close(save);
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		_parentEditor = null;

		super.dispose();
	}

	/**
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#doSetInput(IEditorInput)
	 */
	protected void doSetInput(IEditorInput input) throws CoreException {
		super.doSetInput(input);
	}

	/**
	 * @see org.eclipse.ui.editors.text.TextEditor#initializeEditor()
	 */
	protected void initializeEditor() {
		super.initializeEditor();
		setSourceViewerConfiguration(new AcmeSourceViewerConfiguration());
		setRangeIndicator(new DefaultRangeIndicator());
		setEditorContextMenuId("#AcmeTextEditorContext");
		setRulerContextMenuId("#AcmeTextEditorRuleContext");
	}

	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(Class)
	 */
	public Object getAdapter(Class adapter) {
		if (adapter == IContentOutlinePage.class)
			return _parentEditor.getAdapter(adapter);
		if (_projectionSupport != null) {
			Object adptr = _projectionSupport.getAdapter(getSourceViewer (), adapter);
			if (adptr != null) return adptr;
		}
		return super.getAdapter(adapter);
	}

	/**
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#setDocumentProvider(IDocumentProvider)
	 */
	protected void setDocumentProvider(IDocumentProvider provider) {
		super.setDocumentProvider(provider);
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
        // BugFix# 359, removed cursor position saving and reinstatement.
		super.setFocus();
	}

	/**
	 * @see org.eclipse.ui.IEditorPart#doSave(IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		super.doSave(monitor);
	}

	/**
	 * @see org.eclipse.ui.texteditor.ITextEditor#isEditable()
	 */
	public boolean isEditable() {
		return _canEdit && super.isEditable();
	}

	public void setEditable(boolean b) {
		_canEdit = b;
		updateState(getEditorInput());
	}

	private boolean _generatingSelection = false;
	private ProjectionSupport	_projectionSupport;
	private AcmeFoldingStructureProvider	_projectionProvider;
	/**
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#handleCursorPositionChanged()
	 */
	protected void handleCursorPositionChanged() {
		super.handleCursorPositionChanged();
        // BugFix# 359, removed cursor position saving and reinstatement.
		// Only update selection if the region manager is valid
		if (_generatingSelection)
			return;
//		TextRegionManager rMgr = _parentEditor.getResourceProvider().getRegionManager();
//		if (_parentEditor.getResourceProvider().isRegionManagerValid()) {
//			AcmeObject o = rMgr.getAcmeObjectAtPos(getCursorOffset());
//			processSelection(o);
//		}
	}

	protected void processSelection(AcmeObject o) {
		if (o != null) {

			SelectionChangedEvent evt = new SelectionChangedEvent(this, new StructuredSelection(o));
			_selectedObject = o;
			_parentEditor.selectionChanged(evt);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		if (event.getSelectionProvider() == this)
			return;
		if (event.getSelection() instanceof IStructuredSelection) {
			IStructuredSelection iss = (IStructuredSelection) event.getSelection();
			Object selObj = iss.getFirstElement();
			if (selObj instanceof AcmeObject) {
				AcmeObject selAcmeObj = (AcmeObject) selObj;
				if (selAcmeObj == _selectedObject)
					return;
				Region region = _parentEditor.getResourceProvider().getRegion(selAcmeObj);
				if (region != null && region.isValid()) {
					int bIdx = region.getBeginIndex();
					int eIdx = region.getEndIndex();
					_generatingSelection = true;
					this.setHighlightRange(bIdx, eIdx - bIdx, true);
					_generatingSelection = false;
				}

			}
		}

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setSite(org.eclipse.ui.IWorkbenchPartSite)
	 */
	protected void setSite(IWorkbenchPartSite site) {

		super.setSite(site);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#addSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void addSelectionChangedListener(ISelectionChangedListener listener) {
//		Debug.diagnoseln("AcmeTextEditor.addSelectionChangedListener: " + "should never be called");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#getSelection()
	 */
	public ISelection getSelection() {
		if (_selectedObject != null)
			return new StructuredSelection(_selectedObject);
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#removeSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void removeSelectionChangedListener(ISelectionChangedListener listener) {
//		Debug.diagnoseln("AmceTextEditor.removeSelectionChangedListener: should never be called");

	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#setSelection(org.eclipse.jface.viewers.ISelection)
	 */
	public void setSelection(ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			if (((IStructuredSelection) selection).getFirstElement() instanceof AcmeObject) {
				_selectedObject = (AcmeObject) ((IStructuredSelection) selection).getFirstElement();
			}
		}
	}

	public void registerKeyBindings() {
		IKeyBindingService kbs = getEditorSite().getKeyBindingService();

		IAction action;

		action = this.getAction(ITextEditorActionConstants.UNDO);
		kbs.registerAction(action);

		action = this.getAction(ITextEditorActionConstants.REDO);
		kbs.registerAction(action);

		action = this.getAction(ITextEditorActionConstants.COPY);
		kbs.registerAction(action);

		action = this.getAction(ITextEditorActionConstants.CUT);
		kbs.registerAction(action);

		action = this.getAction(ITextEditorActionConstants.PASTE);
		kbs.registerAction(action);

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#createActions()
	 */
	protected void createActions() {

		super.createActions();

		Action filterAction = new FilterOutVisAction(this);
		setAction(FilterOutVisAction.ID, filterAction);

		TextEditorTypeHierarchyAction browseTypeAction =
			new TextEditorTypeHierarchyAction(AcmeStudioPlugin.getDefault().getResourceBundle(), "", this);
		setAction(OpenTypeHierarchyAction.ID, browseTypeAction);
		markAsSelectionDependentAction(OpenTypeHierarchyAction.ID, true);

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#editorContextMenuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 */
	protected void editorContextMenuAboutToShow(IMenuManager menu) {
		super.editorContextMenuAboutToShow(menu);
		if (isEditable()) {
			addAction(menu, ITextEditorActionConstants.GROUP_REST, OpenTypeHierarchyAction.ID);
		}
	}

	/**
	 *  Adds support for projection/folding
	 * @param parent
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 **/
	public void createPartControl(Composite parent) {
		super.createPartControl (parent);
		
		ProjectionViewer projectionViewer = (ProjectionViewer )getSourceViewer ();
		_projectionSupport = new ProjectionSupport (projectionViewer, getAnnotationAccess(), getSharedColors());
		_projectionSupport.addSummarizableAnnotationType("org.eclipse.ui.workbench.texteditor.error");
		_projectionSupport.addSummarizableAnnotationType("org.eclipse.ui.workbench.texteditor.warning");
//		_projectionSupport.setHoverControlCreator(new IInformationControlCreator () {
//
//			public IInformationControl createInformationControl(Shell parent) {
//				return new CustomSourceInformationControl (parent, IDocument.DEFAULT_CONTENT_TYPE);
//			}
//			
//		});
		_projectionSupport.install ();
		
		_projectionProvider = new AcmeFoldingStructureProvider  ();
		_projectionProvider.install (this, projectionViewer);
		
		if (isFoldingEnabled ()) {
			projectionViewer.doOperation (ProjectionViewer.TOGGLE);
		}
	}

	/**
	 * @return
	 */
	private boolean isFoldingEnabled() {
		return true;
	}
}
