package AcmeStudio.ui.editors;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.List;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import AcmeStudio.ui.editors.actions.NewDesignAnalysisAction;
import AcmeStudio.ui.editors.analysis.actions.ParseAnalysisAction;
import AcmeStudio.ui.views.parts.DesignRuleBrowser;
import AcmeStudio.ui.widgets.AcmeErrorTree;
import AcmeStudio.util.ASEditorInput;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.designRules.DesignAnalysis;
import acmeLib.elements.AcmeFamily;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class DesignAnalysisEditor extends EditorPart implements ISelectionChangedListener, AcmeModelChangeListener {

    private AcmeFamily _family;
    private DesignRuleBrowser _textEditor;
    private List _nameList;
    private AcmeErrorTree _errorTree;
    private ActionRegistry _actions;
	private AcmeStyleEditor _parentEditor;
   
//    private DesignForm _form;
    
    /**
     * Constructor for DesignAnalysisEditor.
     */
    public DesignAnalysisEditor(AcmeStyleEditor parentEditor) {
        super();
//        _form = new DesignForm (this);  
        _parentEditor = parentEditor;
        AcmeStudioChangeManager.registerListener(this);
    }

    /**
     * @see org.eclipse.ui.IEditorPart#doSave(IProgressMonitor)
     */
    public void doSave(IProgressMonitor monitor) {
    }

    /**
     * @see org.eclipse.ui.IEditorPart#doSaveAs()
     */
    public void doSaveAs() {
    }

    /**
     * @see org.eclipse.ui.IEditorPart#gotoMarker(IMarker)
     */
    public void gotoMarker(IMarker marker) {
    }

    /**
     * @see org.eclipse.ui.IEditorPart#init(IEditorSite, IEditorInput)
     */
    public void init(IEditorSite site, IEditorInput input)
    throws PartInitException {
        setSite (site);
        setInput (input);

    }

    /**
     * @see org.eclipse.ui.IEditorPart#isDirty()
     */
    public boolean isDirty() {
        return false;
    }

    /**
     * @see org.eclipse.ui.IEditorPart#isSaveAsAllowed()
     */
    public boolean isSaveAsAllowed() {
        return false;
    }


    /**
     * @see org.eclipse.ui.IWorkbenchPart#createPartControl(Composite)
     */
    public void createPartControl(Composite parent) {
        parent.setLayout (new GridLayout (2, false));
        Composite listGroup = new Composite (parent, SWT.NONE);
        listGroup.setLayout (new GridLayout (2, false));
        GridData lggd = new GridData ();
        lggd.horizontalAlignment = GridData.HORIZONTAL_ALIGN_FILL;
        lggd.verticalAlignment = GridData.VERTICAL_ALIGN_FILL;
        lggd.grabExcessVerticalSpace = true;
        listGroup.setLayoutData (lggd);
        
        _nameList = new List (listGroup, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL);
//        _nameList.setSorter (new NameSorter ());
        _nameList.addSelectionListener(new SelectionAdapter () {
            public void widgetSelected (SelectionEvent evt) {
                String [] selection = _nameList.getSelection();
                if (selection.length == 1) {
                    DesignAnalysis an = (DesignAnalysis )_nameList.getData (selection [0]);
                    _textEditor.navigateTo (an);
                }
            }
        });
        
       
		GridData nrd = new GridData ();
		nrd.widthHint = 200;
		nrd.heightHint = 200;
		nrd.grabExcessVerticalSpace = true;
		nrd.verticalAlignment = GridData.VERTICAL_ALIGN_FILL;
		nrd.horizontalSpan = GridData.FILL;
		_nameList.setLayoutData(nrd);
		
		Button addButton = new Button (listGroup, SWT.PUSH);
		addButton.setText("New...");
		addButton.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected (SelectionEvent evt) {
				_actions.getAction(NewDesignAnalysisAction.ID).run();
			}
		});
		
		Button removeButton = new Button (listGroup, SWT.PUSH);
		removeButton.setText ("Remove");
		removeButton.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected (SelectionEvent evt) {
				String [] isel = _nameList.getSelection();
				for (int i = 0; i < isel.length; i++) {
					String da = isel [i];
					_family.removeDesignAnalysis(da);
				}
				fillEditor(_family);
//				_nameList.refresh ();
			}
		});
		
		Composite eComposite = new Composite (parent, SWT.NONE);
		GridData egd = new GridData ();
		egd.grabExcessHorizontalSpace = true;
		egd.grabExcessVerticalSpace = true;
		egd.horizontalAlignment = GridData.FILL;
		egd.verticalAlignment = GridData.FILL;
		eComposite.setLayoutData(egd);
        eComposite.setLayout (new GridLayout (1, false));

        _textEditor = new DesignRuleBrowser (eComposite);
        GridData tgd = new GridData ();
        tgd.grabExcessHorizontalSpace = true;
        tgd.grabExcessVerticalSpace = true;
        tgd.horizontalAlignment = GridData.FILL;
        tgd.verticalAlignment = GridData.FILL;
        _textEditor.setLayoutData(tgd);   
//        eComposite.setLayoutData(tgd);
        fillEditor(_family);
        
        _textEditor.addSelectionChangedListener(this);
        
        _errorTree = new AcmeErrorTree (eComposite);
        GridData etgd = new GridData ();
        etgd.horizontalAlignment = GridData.FILL;
        _errorTree.getTree().setLayoutData(etgd);
        Button parseButton = new Button (_textEditor.getButtonArea (), SWT.PUSH);
        parseButton.setText ("Parse");
        parseButton.addSelectionListener(new SelectionAdapter () {
        	public void widgetSelected (SelectionEvent evt)  {
        		ParseAnalysisAction doParse = (ParseAnalysisAction )getAction (ParseAnalysisAction.PARSE);
        		if (doParse != null && doParse.isEnabled())  {
        			doParse.run();
        				
        				fillEditor (_family);
        				_nameList.setSelection(new String [] {doParse._designAnalysis.getName()});
//        			_nameList.getList ().refresh ();
//        				_nameList.setSelection (new StructuredSelection (doParse._designAnalysis));

        		}
        	}
        });
        
		initializeActionRegistry ();
    }

    /**
     * @see org.eclipse.ui.IWorkbenchPart#setFocus()
     */
    public void setFocus() {
    }

    /**
     * @see org.eclipse.ui.part.EditorPart#setInput(IEditorInput)
     */
    protected void setInput(IEditorInput input) {
        if (input instanceof ASEditorInput) {
            ASEditorInput model = (ASEditorInput )input;
            _family = (AcmeFamily )model.getElement ();
            this.setPartName ("Functions");
            this.setContentDescription ("Functions");
        }
       // createActions ();
    }

    protected void fillEditor (AcmeFamily family) {
        if (family == null) return;
     
        AcmeEnumeration analyses = family.getDesignAnalyses();
        java.util.List names = new ArrayList ();
        while (analyses.moreItems()) {
            DesignAnalysis an = (DesignAnalysis )analyses.getNextItem ();
            if (_nameList.indexOf(an.getName ()) != -1) {
            	_nameList.remove (an.getName ());
            }
            names.add(an);           
        }
        
        Collections.sort (names, new Comparator () {

			public int compare(Object an1, Object an2) {
				DesignAnalysis da1 = (DesignAnalysis) an1;
				DesignAnalysis da2 = (DesignAnalysis) an2;
				return String.CASE_INSENSITIVE_ORDER.compare(da1.getName (), da2.getName ());
			}
        
        });
        
        for (Iterator iter = names.iterator (); iter.hasNext (); ) {
			DesignAnalysis da = (DesignAnalysis) iter.next ();
			_nameList.add (da.getName ());
			_nameList.setData (da.getName (), da);
		}
           
    }

    public void setEditable (boolean canEdit) {
        
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
     */
    public void selectionChanged(SelectionChangedEvent event) {
        if (event.getSource() != this) {
            ISelection selection = event.getSelection();
            if (selection instanceof IStructuredSelection) {
                Object sel = ((IStructuredSelection )selection).getFirstElement();
                if (sel instanceof DesignAnalysis) {
                	_nameList.setSelection(new String [] {((DesignAnalysis )sel).getName ()});
                	_nameList.showSelection();
                }
            }
        }
    }
    
    protected void initializeActionRegistry () {
        _actions = new ActionRegistry ();
        createActions ();
    }
    
    protected void createActions () {
        ParseAnalysisAction action = new ParseAnalysisAction (_textEditor, _errorTree, _family);
        _actions.registerAction(action);
        
        NewDesignAnalysisAction nda = new NewDesignAnalysisAction (this, _family);
        _actions.registerAction(nda);
    }

    public IAction getAction (String id) {
        return _actions.getAction(id);
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
	 */
	public void changeModel(AcmeStudioChangeEvent evt) {
		switch (evt.getEventType ()) {
			case AcmeStudioChangeManager.ADD_DESIGN_ANALYSIS:
			case AcmeStudioChangeManager.REMOVE_DESIGN_ANALYSIS:
				fillEditor (_family);
				break;
		}
	}
	
	public boolean interestedInEvent (int et) {
		return et == AcmeStudioChangeManager.ADD_DESIGN_ANALYSIS ||
			et == AcmeStudioChangeManager.REMOVE_DESIGN_ANALYSIS;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		Object ad =  super.getAdapter(adapter);
		if (ad == null && adapter.equals (CommandStack.class)) {
			ad = _parentEditor.getAdapter (adapter);
		}
		return ad;
	}

}
