package AcmeStudio.ui.editors;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.LightweightSystem;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.parts.ScrollableThumbnail;
import org.eclipse.draw2d.parts.Thumbnail;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.internal.GEFMessages;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorActionBarContributor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.ide.IGotoMarker;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiPageEditorActionBarContributor;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.part.MultiPageSelectionProvider;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.core.resource.provider.ASResourceProvider;
import AcmeStudio.core.resource.provider.IResourceProviderListener;
import AcmeStudio.core.resource.provider.ResourceProviderEvent;
import AcmeStudio.ui.editors.diagram.actions.NavigateUpAction;
import AcmeStudio.ui.editors.diagram.parts.ElementEditPart;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.ui.views.DesignBrowser;
import AcmeStudio.ui.views.ElementView;
import AcmeStudio.ui.views.ElementViewPage;
import AcmeStudio.ui.views.IElementViewPage;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Constants;
import AcmeStudio.util.Debug;
import AcmeStudio.util.Helper;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.AcmeObject;
import acmeLib.armaniParser.ParseError;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeFamily;
import acmeLib.elements.AcmeSystem;
import acmeLib.representations.AcmeRepresentation;

/**
 * @author Bradley Schmerl (schmerl@cs.cmu.edu)
 * Provides a multipage editor for acme designs, with an editor for:
 * 1. The Acme source (can only edit with this if there was a parse error)
 * 2. Each family in the design
 * 3. Each system in the design
 * 
 * TODO: Should change overview to show diagram in family editor (if it is visible)
 *
 */
public class MultiPageAcmeEditor
    extends MultiPageEditorPart
    implements ISelectionChangedListener, AcmeModelChangeListener, IResourceProviderListener {

    /** Prefixes for tabs of different types of _sysEditorMap
     */
    public static final String FAMILY_PAGE = "Family - ",
        SYSTEM_PAGE = "System - ";

    /** The page index of the text editor */
    private static final int TEXT_EDITOR_IDX = 0;

    /** The design browser for this design **/
    private DesignBrowser _designBrowser = null;

    /** The element view for this design **/
    private ElementViewPage _elementViewPage = null;

    /** The diagram editor that is currently active **/
    private DiagramEditor _activeDiagramEditor = null;

    /** The acme source editor for the current design **/
    private AcmeTextEditor _acmeSourceEditor;
    
    /** The cursor position in AcmeTextEditor, removed with BugFix #359 */
//    private int _cursorPos = 0;

    /** A hashtable of system names to _sysEditorMap, to get the image
     *  of representations **/
    private Map _sysEditorMap = new Hashtable();

    /** The set of _sysEditorMap that are inactive (not viewed) because
     * they contain representations**/
    private Vector _inactiveDiagramEditors = new Vector();

    /** List of top-level editors in this multipage editor */
    private java.util.List _topEditors = new ArrayList();

    /** The AcmeStudio resource provider managing the ASResources */
    private ASResourceProvider _resourceProv = null;

    /** The AcmeDesign-encapsulating ASDesign that is managed by the editor **/
    private ASDesign _design = null;

    /**
     * The resource tracker of the editor.
     */
    private ResourceTracker _resourceListener = new ResourceTracker();
    /** Flag to track whether we're currently renewing the pages */
    private boolean _renewingPages = false;

    /** System overview thumbnail */
    private Thumbnail _thumbnail = null;
    /** For displaying system overview */
    private LightweightSystem _lws = null;


    /**
     * Constructor for MultiPageAcmeEditor.
     */
    public MultiPageAcmeEditor() {
        super();
    }


    private IStructuredSelection _lastSelection = null;
    /**
     * Make sure to convert edit parts as selected from diagram _sysEditorMap to 
     * acme elements, which are understoon by everyone else.
     * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(SelectionChangedEvent)
     */
    public void selectionChanged(SelectionChangedEvent event) {
        addSelectionListenerToDesignBrowser();

        if (event.getSelection() instanceof IStructuredSelection) {
            IStructuredSelection iss =
                (IStructuredSelection) event.getSelection();
                
            if (iss.equals (_lastSelection)) return;

            Vector convertedSel = new Vector();
            Iterator sel = iss.iterator();
            while (sel.hasNext()) {
                Object theSel = sel.next();
                if (theSel instanceof ElementEditPart) {
                    ElementEditPart partSelected = (ElementEditPart) theSel;
                    convertedSel.add(partSelected.getModel());
                } else if (theSel instanceof RoleEditPart) {
                    RoleEditPart partSelected = (RoleEditPart) theSel;
                    convertedSel.add(partSelected.getModel());
                } else if (theSel instanceof AcmeObject) {
                    convertedSel.add(theSel);
                }
            }
    
            SelectionChangedEvent copy =
                new SelectionChangedEvent(
                    (ISelectionProvider) event.getSource(),
                    new StructuredSelection(convertedSel));
            if (copy.getSelection ().equals (_lastSelection)) 
                return;
            _lastSelection = (IStructuredSelection )copy.getSelection ();
            if (_designBrowser != null) {
                _designBrowser.selectionChanged(copy);
                _elementViewPage.selectionChanged(copy);
            }
            ElementView elementView = AcmeStudioPlugin.findElementView();
            if (elementView != null) {
                elementView.selectionChanged(copy);
            }
            if (!getResourceProvider ().isInTextEditor ()) {
                // Make sure that selections can fire in all the editors
                // This fixes a bug of an element not being selected in 
                // the diagram editor of the style editor when it is selected
                // in the outline view.
            	Iterator i = _topEditors.iterator();
            	while (i.hasNext ()) {
            		ISelectionChangedListener l = (ISelectionChangedListener )i.next ();
            		if (l != _acmeSourceEditor) {
            			l.selectionChanged (copy);
            		}
            	}
            	
//                Iterator i = _sysEditorMap.keySet().iterator();
//                while (i.hasNext()) {
//                    ISelectionChangedListener l =
//                        (ISelectionChangedListener) _sysEditorMap.get(i.next());
//                    l.selectionChanged(copy);
//                }
            }
            else {
                 _acmeSourceEditor.selectionChanged(copy); 
            } 
        }

    }

    /**
     * Responds to changes in the model, focusing on those that change the _sysEditorMap
     * that need to be displayed by the multipage editor
     * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudioChangeEvent)
     */
    public void changeModel(AcmeStudioChangeManager.AcmeStudioChangeEvent evt) {

        // Only respond if the editor is interested
        if (Helper.eventRelatedToDesign(evt, getAcmeDesign())) {

            switch (evt.getEventType()) {
                case AcmeStudioChangeManager.ADD_SYSTEM :
                    // Add a system editor
                    if (evt.getTarget().equals(getAcmeDesign())) {
                        int index =
                            addDiagramEditor((AcmeSystem) evt.getSource());
                        setActivePage(index);
                    }
                    break;
                case AcmeStudioChangeManager.REMOVE_SYSTEM :
                    // Remove a system editor
                    if (evt.getTarget().equals(getAcmeDesign())) {
                        int numEditors = this.getPageCount();

                        int foundEditor = -1;
                        for (int index = 0; index < numEditors; index++) {
                            IEditorPart part = this.getEditor(index);
                            if (part instanceof DiagramEditor
                                && ((DiagramEditor) part).getAcmeSystem().equals(evt.getSource())) {
                                foundEditor = index;
                                index = numEditors;
                            }
                        }
                        if (foundEditor != 0) {
                            this.setActivePage(foundEditor - 1);
                        }

                        if (foundEditor >= 0) {
                            this.removePage(foundEditor);
                        }
                    }
                    break;
                case AcmeStudioChangeManager.ADD_REPRESENTATION :
                    {
                        // Add a represetation editor
                        AcmeElement elem = (AcmeElement) evt.getTarget();
                        if (elem.getParentDesign().equals(getAcmeDesign())) {
                            AcmeRepresentation rep = (AcmeRepresentation) evt.getSource();
                            try {
                                addRepresentationEditor(rep.getSystem());

                            }
                            catch (PartInitException e) {
                            }
                        }
                        break;
                    }
                case AcmeStudioChangeManager.REMOVE_REPRESENTATION:
                {
                	// Remove the representation editor -- make sure
                	// we're in the top editor first
                	AcmeRepresentation rep = (AcmeRepresentation )evt.getSource ();
                	if (getActiveEditor() instanceof DiagramEditor &&
                		((DiagramEditor )getActiveEditor ()).getAcmeSystem() == rep.getSystem()) {
                		AcmeElement elem = (AcmeElement )evt.getTarget ();
                		AcmeSystem sys = (AcmeSystem )elem.getParent ();
                		DiagramEditor ed = (DiagramEditor )getActiveEditor ();
                		NavigateUpAction action = (NavigateUpAction )ed.getActionRegistry().getAction(NavigateUpAction.ID);
                		action.setParentSystem(sys);
                		action.run ();
                		_sysEditorMap.remove (ed.getAcmeSystem ());
                		ed.dispose();
                		}
                	break;
                	
                }
                case AcmeStudioChangeManager.ADD_FAMILY :
                    // Add a family editor
                    {
                        AcmeFamily fam = (AcmeFamily) evt.getSource();
                        AcmeDesign des = (AcmeDesign) evt.getTarget();
                        if (des.equals(getAcmeDesign())) {
                            int index = addFamilyEditor(fam);
                            setActivePage(index);
                        }
                        break;
                    }
                case AcmeStudioChangeManager.REMOVE_FAMILY :
                    {
                        // remove a family editor that corresponds to the family
                        AcmeFamily fam = (AcmeFamily) evt.getSource();
                        AcmeDesign des = (AcmeDesign) evt.getTarget();
                        if (des.equals(getAcmeDesign())) {
                            int numEditors = getPageCount();
                            int foundEditor = -1;
                            for (int i = 0; i < numEditors; i++) {
                                IEditorPart part = this.getEditor(i);
                                if (part instanceof AcmeStyleEditor
                                    && ((AcmeStyleEditor) part).getFamily().equals(fam)) {
                                    foundEditor = i;
                                    i = numEditors;
                                }
                            }
                            if (foundEditor != 0) {
                                this.setActivePage(foundEditor - 1);
                            }
                            if (foundEditor >= 0) {
                                this.removePage(foundEditor);
                            }
                        }
                        break;
                    }
                case AcmeStudioChangeManager.IMPORT_DESIGN :
                    {
//                        // Add a family editor for each family in the imported design
//                        AcmeDesign des = (AcmeDesign) evt.getTarget();
//                        String importedDesF = (String) evt.getSource();
//                        AcmeDesign newDesign = des.getImportedDesign(importedDesF);
//                        if (newDesign != null) {
//                            AcmeEnumeration families = newDesign.getFamilies();
//                            while (families.moreItems()) {
//                                AcmeFamily family = (AcmeFamily) families.getNextItem();
//                                addFamilyEditor(family);
//                            }
//                        }
                    }
                    break;
                default :
            }
            // Something changed, so make this editor dirty
            getResourceProvider().setSynchedWithText(false);
            setDirty(true);
        }

    }
    
    public boolean interestedInEvent (int et) {
    	return true;
    }

    /**
     * @see AcmeStudio.core.IResourceProviderListener#providerChanged(ResourceProviderEvent)
     */
    public void providerChanged(ResourceProviderEvent event) {
        if (event.getKind() != ResourceProviderEvent.ABOUT_TO_REMOVE) {
            ASResourceProvider prov = event.getProvider();
            
            if ((event.getKind() & ResourceProviderEvent.AS_RESOURCE_NEED_RELOAD) > 0) {
                prov.reconnect();
            } else {
                // set new input
                setInput(prov.getTextEditorInput());
                // store text editor location
//                if (_acmeSourceEditor != null) {
//                    _cursorPos = _acmeSourceEditor.getCursorOffset();
//                }
            }
            if ((event.getKind() & ResourceProviderEvent.AS_RESOURCE_NEED_RELOAD) > 0 ||
                (event.getKind() & ResourceProviderEvent.AS_RESOURCE_RELOADED) > 0) {
                // save state of whether we're in the text editor
                boolean wasInTextEditor = prov.isInTextEditor();
                String systemOpen = null;
                if (_activeDiagramEditor != null)
                	systemOpen = _activeDiagramEditor.getAcmeSystem().getQualifiedName ();
                // causes the constituent editors to be refreshed
                renewPages();
                if (wasInTextEditor) {
                    setActivePage(TEXT_EDITOR_IDX);
                }
                if (systemOpen != null) {
                	AcmeSystem openSystem = (AcmeSystem )_design.getDesign ().lookupQualifiedName(systemOpen);
                	if (openSystem != null) {
                		swapDiagramEditor(openSystem, _activeDiagramEditor.getAcmeSystem ());
                	}
                }
                // refresh the design browser
                if (_designBrowser != null) {
                    _designBrowser.setDesign(getDesign());
                }
            }

            refreshThumbnail ();
        } else {
            if (isDirty()) {  // prompt to save or close
                _partListener.partActivated(this);
            } else {
                closeEditor(false);
            }
        }
    }

    /**
     * Adds a new representation editor. This editor is not displayed, 
     * rather it is used for drawing previews
     * @param system The system that is inside the representation
     */
    protected void addRepresentationEditor(AcmeSystem system)
    throws PartInitException {
        DiagramEditor inactiveDiagramEditor = new DiagramEditor(this, system);
        IEditorSite site = createSite(inactiveDiagramEditor);
        inactiveDiagramEditor.init(site, getResourceProvider().getSystemEditorInput(system));
        Composite dummyParent = new Composite(getContainer(), SWT.NONE);
        inactiveDiagramEditor.createPartControl(dummyParent);
//        inactiveDiagramEditor.setInput(new ASEditorInput (getResourceProvider().getSystem);
        _sysEditorMap.put(system, inactiveDiagramEditor);
        _inactiveDiagramEditors.add(inactiveDiagramEditor);
    }

    /**
     * Adds and displays a diagram editor for a particular system
     * @param system The system to create a diagram editor for
     * @return the index of the page that was added
     */
    protected int addDiagramEditor(AcmeSystem system) {
        int index = 0;
        try {
            // Create the editor and add it to the multipage
            DiagramEditor sysEd = new DiagramEditor(this, system);
            _topEditors.add(sysEd);
            index = addPage(sysEd, getResourceProvider().getSystemEditorInput(system));
            setPageText(index, SYSTEM_PAGE + system.getName());
            _sysEditorMap.put(system, sysEd);
            _activeDiagramEditor = sysEd;

            // Create an invisible editor for each representation of the system
            Vector v = Helper.getSubAcmeSystems(system);
            for (int i = 0; i < v.size(); i++) {
                AcmeSystem sys = (AcmeSystem) v.get(i);
                addRepresentationEditor(sys);
            }
            
            if (system.getParentRepresentation() != null) {
            	AcmeSystem pSys = (AcmeSystem )system.getParentRepresentation().getParent ().getParent ();
            	addRepresentationEditor(pSys);
            }
            
            this.getEditorSite().getActionBars().updateActionBars();
        } catch (PartInitException e) {
        }
        return index;
    }

    /**
     * @see org.eclipse.ui.part.MultiPageEditorPart#createPages()
     */
    protected void createPages() {
        renewPages();
    }

    /**
     * Creates all the _sysEditorMap that should be associated with the design
     */
    protected void renewPages(AcmeEnumeration systems) {
        _renewingPages = true;

        // First, remove all pages
        if (getPageCount() > 0) {
            super.setActivePage(TEXT_EDITOR_IDX);  // don't trigger setInput
            for (int i = getPageCount(); i > 1; i--) {
                removeEditorsForPage(i - 1);
                removePage(i - 1);
            }
        }
        try {
            int index = -1;
            if (getPageCount() == 0) {
                // Add source editor
                _acmeSourceEditor = new AcmeTextEditor(this);
                _topEditors.add(_acmeSourceEditor);
                index = addPage(_acmeSourceEditor, getResourceProvider().getTextEditorInput());
                setPageText(index, "Acme Source");
//                _acmeSourceEditor.setCursorPos(_cursorPos);
            }
            // Only create other _sysEditorMap if the design opened ok
            if (getResourceProvider().canOpenDesign()) {
                // allow editing of the source
                _acmeSourceEditor.setEditable(true);
                
                // Create style _sysEditorMap
                Iterator famIter = getResourceProvider().getAllFamilies().iterator();
                while (famIter.hasNext()) {
                    AcmeFamily family = (AcmeFamily) famIter.next();
                    if (family.getParentDesign() == getResourceProvider ().getAcmeDesign() ) {
                    	index = addFamilyEditor(family);
                    }
                }

                // Create system _sysEditorMap
                //AcmeEnumeration systems = _design.getSystems();
                while (systems.moreItems()) {
                    AcmeSystem system = (AcmeSystem) systems.getNextItem();
                    index = addDiagramEditor(system);

                }
                
                initContentPages();
                getSite().getWorkbenchWindow().getPartService().addPartListener(_partListener);

            } else {
                MessageDialog.openError(
                    getEditorSite().getShell(),
                    AcmeStudioPlugin.getMessage("MultiPageAcmeEditor.parseErrorTitle"),
                    AcmeStudioPlugin.getMessage("MultiPageAcmeEditor.openParseError"));
            }

            if (systems.getSize() == 0 && getPageCount() > TEXT_EDITOR_IDX + 1) {
                // family being edited
                // make sure that the primary family is brought into focus
                this.setActivePage(TEXT_EDITOR_IDX + 1);
            } else {
                if (getPageCount() <= index || index == -1) {
                    // index not in range of pages available
                    // bring text editor into focus
                    this.setActivePage(TEXT_EDITOR_IDX);
                } else {
                    // bring system into focus
                    this.setActivePage(index);
                }
            }

        } catch (PartInitException e) {
            Debug.errorln(e);
            closeEditor(true);
        }
        
        _renewingPages = false;
    }

	private void initContentPages() {
		if (_designBrowser != null) {
		    _designBrowser.setDesign(_design);
		} else {
            _designBrowser = new DesignBrowser(_design);
            // Add the element view as a selection listener
            this.getSite().getSelectionProvider().addSelectionChangedListener(_designBrowser);
		}
		if (_elementViewPage != null) {
		    _elementViewPage.setDesign(_design);
		} else {
            _elementViewPage = new ElementViewPage(_design);
            // Add the desgn broswer as a selection listener
            this.getSite ().getSelectionProvider ().addSelectionChangedListener (_elementViewPage);
		}
	}

    protected void renewPages () {
        if (getAcmeDesign() != null) {
            renewPages (getAcmeDesign().getSystems ());
        }
        else {
            renewPages (new AcmeEnumeration ());
        }
    }

    /**
     * Sets the active editor to be displayed based on the text
     * @param pageType the string that the title of the page should match
     */
    public void setActiveAcmePage(String pageType) {
        if (pageType != null) {
            for (int i = getPageCount() - 1; i >= 0; i--) {
                if (this.getPageText(i).startsWith(pageType)) {
                    this.setActivePage(i);
                    return;
                }
            }
        }
    }

    /**
     * Adds a new family editor to the multipage
     * @param family the family to create the editor for
     * @return the index of the editor in the multipage
     */
    protected int addFamilyEditor(AcmeFamily family) {
        int index;
        try {
            AcmeStyleEditor famEd = new AcmeStyleEditor(this);
            _topEditors.add(famEd);
            index = addPage(famEd, getResourceProvider().getFamilyEditorInput(family));
            setPageText(index, FAMILY_PAGE + family.getName());
            // Only allow editing if the family was not imported
            if (family.getParentDesign() != getAcmeDesign()) {
                famEd.setEditable (false);   
            }
            famEd.addSelectionChangedListener(this);
        } catch (PartInitException e) {
            index = -1;
        }
        return index;
    }

    /**
     * Change the active page and refresh the multipage editor.
     * @param activeAcmeSystem Current active acme system
     * @param oldSystem The system that we are changing from
     */
    public void swapDiagramEditor(AcmeSystem activeAcmeSystem, AcmeSystem oldSystem) {
        
        
        AcmeVector systems = new AcmeVector ();
        systems.addItem(activeAcmeSystem);
        renewPages (systems.enumerate ());
        
        refreshThumbnail();
        
        if (_activeDiagramEditor.requiresOverview()) {
        	_activeDiagramEditor.configureOverview();
        }
        
//        renewPages (new AcmeVector (activeAcmeSystem).enumerate ());
//        
//        _topEditors.remove (_activeDiagramEditor);
//        _activeDiagramEditor = new DiagramEditor(this);
//        try {
//            // find the diagram editor corresponding to the oldSystem and remove it
//            int count = this.getPageCount();
//            int index = 0;
//            boolean foundOld = false;
//            
//            while (index < count && !foundOld) {
//                IEditorPart part = this.getEditor(index);
//                if (part instanceof DiagramEditor && ((DiagramEditor) part).getAcmeSystem() == oldSystem) {
//                    foundOld = true;
//                } else {
//                    index++;
//                }
//
//            }
//            this.setActivePage(TEXT_EDITOR_IDX);  // go to the source editor for safety
//            removePage(index);
//            _topEditors.add (_activeDiagramEditor);
//            // add the new editor
//            index = addPage(_activeDiagramEditor, getResourceProvider().getSystemEditorInput(activeAcmeSystem));
//            setPageText(index, _activeDiagramEditor.getTitle());
//            setActivePage(index);
//            _designBrowser = new DesignBrowser(_activeDiagramEditor.getAcmeDesign());
//            _elementViewPage = AcmeStudioPlugin.findElementView();
//            int size = _inactiveDiagramEditors.size();
//            // Add representations
//            for (int i = 0; i < size; i++) {
//                DiagramEditor inactiveDiagramEditor = (DiagramEditor) _inactiveDiagramEditors.get(i);
//                if (inactiveDiagramEditor.getAcmeSystem().equals(activeAcmeSystem)) {
//                    inactiveDiagramEditor = new DiagramEditor(this);
//                    IEditorSite site = createSite(inactiveDiagramEditor);
//                    inactiveDiagramEditor.init(site, getResourceProvider().getSystemEditorInput(activeAcmeSystem));
//                    Composite dummyParent = new Composite(getContainer(), SWT.NONE);
//                    inactiveDiagramEditor.createPartControl(dummyParent);
//                    _inactiveDiagramEditors.set(i, inactiveDiagramEditor);
//                }
//            }
//            _activeAcmeSystem = activeAcmeSystem;
//        } catch (PartInitException e) {
//            Debug.errorln(e);
//        }
//        
//        refreshThumbnail ();
         
    }

    /**
     * Sets the site, and adds a partlistener
     * @param site the new site of the editor
     */
    protected void setSite(IWorkbenchPartSite site) {
        super.setSite(site);
    }

    /**
     * Get all the system _sysEditorMap in this editor
     * @return a vector of all the system _sysEditorMap
     */
    public Vector getSystemEditors() {
        final Vector v = new Vector();

        if (_sysEditorMap != null && !_sysEditorMap.isEmpty()) {
        	Iterator i = _sysEditorMap.values().iterator();
        	while (i.hasNext()) {
        		v.add(i.next());
        	}
        }
        return v;
    }

    /**
     * @see org.eclipse.ui.IEditorPart#doSave(IProgressMonitor)
     */
    public void doSave(IProgressMonitor monitor) {
        WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
            public void execute(final IProgressMonitor monitor)
            throws CoreException {
                if (getResourceProvider().saveAcme(monitor)) {
                    // Save the command stack
                    AcmeStudioChangeManager.globalChangeManager
                        .getCommandStack(_design).markSaveLocation();
                }
                getResourceProvider().getFile().refreshLocal(IResource.DEPTH_ONE, monitor);
            }
        };

        boolean success = false;
        try {
            if (getResourceProvider().getFile() == null) {
                performSaveAs();
            } else {
                injectOrderInformation();
                op.run(monitor);
            }
            success = true;
        } catch (InterruptedException e) {
            // ignore
        } catch (InvocationTargetException e) {
            Debug.errorln(e);
            MessageDialog.openError(getSite().getShell(),
                AcmeStudioPlugin.getMessage("MultiPageAcmeEditor.saveErrorTitle"),
                e.getTargetException().getMessage());
        }

        if (monitor != null) {
            monitor.setCanceled(!success);
            monitor.done();
        }
    }

    /**
     * @see org.eclipse.ui.IEditorPart#doSaveAs()
     */
    public void doSaveAs() {
        performSaveAs();
    }

    /**
     * @see org.eclipse.ui.IEditorPart#gotoMarker(IMarker)
     */
    public void gotoMarker(IMarker marker) {
        // bring up the source editor
        // set editor line to marker's line.
        AcmeTextEditor editor = (AcmeTextEditor) getEditor(TEXT_EDITOR_IDX);
        if (editor != null) {
        	IGotoMarker gotoMarker = (IGotoMarker )editor.getAdapter(IGotoMarker.class);
        	if (gotoMarker != null)
        		gotoMarker.gotoMarker(marker);
            setActivePage(TEXT_EDITOR_IDX);
        }

        // bonus:  bring up Diagram Editor and highlight the element with error
    }

    /**
     * @see org.eclipse.ui.IEditorPart#isSaveAsAllowed()
     */
    public boolean isSaveAsAllowed() {
        return true;
    }

    /**
     * @see org.eclipse.ui.IEditorPart#init(IEditorSite, IEditorInput)
     */
    public void init(IEditorSite site, IEditorInput input)
        throws PartInitException {
        super.init(site, input);
        try {
            setInput(input);
        } catch (Exception e) {
            Debug.traceln ("MultiPageAcmeEditor.init: got exception");
            Debug.traceln (e);
        }
    }

    /**
     * Perform SAVEAS action.
     * @return True if the document is successfully saved.
     */
    protected boolean performSaveAs() {
        // Open the save as dialog
        SaveAsDialog dialog = new SaveAsDialog(getSite().getWorkbenchWindow().getShell());
        dialog.setOriginalFile(((IFileEditorInput) getEditorInput()).getFile());
        dialog.open();
        final IPath path = dialog.getResult();
        if (path == null) {  // no save
            return false;
        }

        WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
            public void execute(final IProgressMonitor monitor)
            throws CoreException {
                // Try to do the save
                try {
                    getResourceProvider().saveAcmeAs(monitor, path);
                } finally {
                    monitor.setCanceled(true);
                    monitor.done();
                }
            }
        };

        // Try to do the save
        boolean success = false;
        try {
            new ProgressMonitorDialog(getSite().getShell()).run(false, true, op);
            success = true;
        } catch (InterruptedException e) {
            // ignore
        } catch (InvocationTargetException e) {
            Debug.errorln(e);
            MessageDialog.openError(getSite().getShell(),
                AcmeStudioPlugin.getMessage("MultiPageAcmeEditor.saveErrorTitle"),
                e.getTargetException().getMessage());
        }

        return success;
    }

    /**
	 * 
	 */
	private void injectOrderInformation() {
		Vector ses = getSystemEditors();
		for (int i = 0; i < ses.size(); i++) {
			DiagramEditor ed = (DiagramEditor )ses.get(i);
			ed.injectOrderInformation ();
		}
	}


	/**
     * The PartListener of the editor.
     */
    private IPartListener _partListener = new IPartListener() {
        public void partActivated(IWorkbenchPart part) {
            if (part == MultiPageAcmeEditor.this) {
                if (MultiPageAcmeEditor.this.getActiveEditor() instanceof DiagramEditor) {
                    DiagramEditor ed = (DiagramEditor )MultiPageAcmeEditor.this.getActiveEditor ();
                    ed.getGraphicalViewer().setSelection(ed.getGraphicalViewer ().getSelection());
                }
                MultiPageAcmeEditor.this.getSite().getSelectionProvider().addSelectionChangedListener(MultiPageAcmeEditor.this);
                MultiPageAcmeEditor.this.getSite().getSelectionProvider().addSelectionChangedListener(_designBrowser);
                if (_elementViewPage != null) MultiPageAcmeEditor.this.getSite().getSelectionProvider().addSelectionChangedListener(_elementViewPage);

                if (!((FileEditorInput) getEditorInput()).getFile().exists()) {
                    Shell shell = getSite().getShell();
                    String title = GEFMessages.GraphicalEditor_FILE_DELETED_TITLE_UI;
                    String message = GEFMessages.GraphicalEditor_FILE_DELETED_WITHOUT_SAVE_INFO;
                    String[] buttons =
                        {
                            GEFMessages.GraphicalEditor_SAVE_BUTTON_UI,
                            GEFMessages.GraphicalEditor_CLOSE_BUTTON_UI };
                    MessageDialog dialog =
                        new MessageDialog(shell, title, null, message, MessageDialog.QUESTION, buttons, 0);
                    if (dialog.open() == 0) {
                        if (!performSaveAs())
                            partActivated(part);
                    } else {
                        closeEditor(false);
                    }
                } else {
                    ASContext.current().setActiveDesign(_design);
                    ASContext.current().setActiveEditor(MultiPageAcmeEditor.this);
                    AcmeStudioChangeManager.registerListener(MultiPageAcmeEditor.this);
                }
            }
        }

        public void partBroughtToTop(IWorkbenchPart part) {
        }

        public void partClosed(IWorkbenchPart part) {
        }

        public void partDeactivated(IWorkbenchPart part) {
            AcmeStudioChangeManager.removeListener(MultiPageAcmeEditor.this);
        }
        public void partOpened(IWorkbenchPart part) {
        }
    };

    /**
     * @see org.eclipse.core.runtime.IAdaptable#getAdapter(Class)
     */
    public Object getAdapter(Class adapter) {
        if (adapter == IContentOutlinePage.class) {
            return _designBrowser;
            //new DesignBrowser (this.getSystemEditor().getAcmeDesign());
        } else if (adapter == IElementViewPage.class) {
            return _elementViewPage;
//        } else if (adapter == IContributedContentsView.class) {
//            return new IContributedContentsView() {
//                public IWorkbenchPart getContributingPart() {
//                    return _elementViewPage;
//                }
//            };
        } else if (adapter == ElementView.class) {
//            Debug.diagnoseln("Querying for element view");
        } else if (adapter == ActionRegistry.class && this.getActiveEditor() != null) {
            return new ActionRegistry() {
                public void dispose() {
                    if (MultiPageAcmeEditor.this.getActiveEditor() != null) {
                        ActionRegistry reg =
                            (ActionRegistry) MultiPageAcmeEditor.this.getActiveEditor().getAdapter(
                                ActionRegistry.class);
                        if (reg != null)
                            reg.dispose();
                    }
                }
                public IAction getAction(Object key) {
                    if (MultiPageAcmeEditor.this.getActiveEditor() != null) {
                        ActionRegistry reg =
                            (ActionRegistry) MultiPageAcmeEditor.this.getActiveEditor().getAdapter(
                                ActionRegistry.class);
                        if (reg != null)
                            return reg.getAction(key);
                    }
                    return null;
                }
                public Iterator getActions() {
                    if (MultiPageAcmeEditor.this.getActiveEditor() != null) {
                        ActionRegistry reg =
                            (ActionRegistry) MultiPageAcmeEditor.this.getActiveEditor().getAdapter(
                                ActionRegistry.class);
                        if (reg != null)
                            return reg.getActions();
                    }
                    return null;
                }
                public void registerAction(IAction action) {
                    if (MultiPageAcmeEditor.this.getActiveEditor() != null) {
                        ActionRegistry reg =
                            (ActionRegistry) MultiPageAcmeEditor.this.getActiveEditor().getAdapter(
                                ActionRegistry.class);
                        if (reg != null)
                            reg.registerAction(action);
                    }

                }
            };
        }
        Object a = super.getAdapter(adapter);
        if (a == null && this.getActiveEditor() != null) {
            a = this.getActiveEditor().getAdapter(adapter);
        }
        return a;
    }

    private boolean _selectionAdded = false;

	private AcmeStyleEditor	_activeFamilyEditor;

    public void addSelectionListenerToDesignBrowser() {
        if (_designBrowser != null
            && _designBrowser.getControl() != null
            && !_designBrowser.getControl().isDisposed()
            && !_selectionAdded) {
            _designBrowser.addSelectionChangedListener(this);
//            _designBrowser.addSelectionChangedListener (_elementViewPage);
            _selectionAdded = true;
        }
    }

    /**
     * @see org.eclipse.ui.part.MultiPageEditorPart#getActiveEditor()
     */
    public IEditorPart getActiveEditor() {
        return super.getActiveEditor();
    }

    public DesignBrowser getDesignBrowser() {
        return _designBrowser;
    }

    public DiagramEditor getDiagramEditor() {
    	if (_activeDiagramEditor != null) return _activeDiagramEditor;
    	else if (_activeFamilyEditor != null) return _activeFamilyEditor.getDiagramEditor ();
    	else return null;
    }

    public IEditorPart[] getTopEditors () {
        return (IEditorPart[]) _topEditors.toArray();
    }

    /**
     * Returns the ASResourceProvider managing the resource used by the editor
     * @return ASResourceProvider  the ASResourceProvider object
     */
    public ASResourceProvider getResourceProvider() {
        return _resourceProv;
    }

    /**
     * Sets the ASResourceProvider managing the resource used by the editor
     * @param resourceProv  the new ASResourceProvider object
     */
    protected void setResourceProvider(ASResourceProvider resourceProv) {
        _resourceProv = resourceProv;
    }

    /**
     * Close the editor.
     */
    protected void closeEditor(final boolean save) {
        getSite().getPage().closeEditor(MultiPageAcmeEditor.this, save);
    }

    /**
      * Disposes of the editor.
      */
    public void dispose() {
        // clear out data members
        getSite().getWorkbenchWindow().getPartService().removePartListener(_partListener);
        getSite().getSelectionProvider().removeSelectionChangedListener(this);
        _partListener = null;
        if (getEditorInput() instanceof FileEditorInput) {
            ((FileEditorInput) getEditorInput()).getFile().getWorkspace().removeResourceChangeListener(
                _resourceListener);
            _resourceListener = null;
        }
        if (_resourceProv != null) {
            // clean up resource provider ASAP
            _resourceProv.removeResourceProviderListener(this);
            _resourceProv.disconnect();
            _resourceProv = null;
        }
        if (_topEditors != null) {
            _topEditors.clear();
            _topEditors = null;
        }
        if (_sysEditorMap != null) {
            _sysEditorMap.clear();
            _sysEditorMap = null;
        }
        if (_inactiveDiagramEditors != null) {
            _inactiveDiagramEditors.clear();
            _inactiveDiagramEditors = null;
        }
        _design = null;
        _acmeSourceEditor = null;
        _activeDiagramEditor = null;
        _lws = null;
        if (_thumbnail != null) {
            _thumbnail.deactivate();
            _thumbnail.setSource(null);
            _thumbnail = null;
        }
        if (_designBrowser != null) {
            getSite().getSelectionProvider().removeSelectionChangedListener(_designBrowser);
            // design browser most likely already disposed by ContentOutline.
            _designBrowser = null;
        }
        if (_elementViewPage != null) {
            getSite().getSelectionProvider().removeSelectionChangedListener(_elementViewPage);
            _elementViewPage.dispose();
            _elementViewPage = null;
        }
        super.dispose();
    }

    protected void setInput(IEditorInput input) {
        // this is BAD... if AcmeDesign has been changed, _design is not updated
    	//if (input == getEditorInput()) return;

        super.setInput(input);

        if (input instanceof IFileEditorInput) {
            IFile file = ((IFileEditorInput) input).getFile();
            try {
                if (getResourceProvider() == null) {
                	setResourceProvider(new ASResourceProvider());
                	getResourceProvider().connect(this);
                	getResourceProvider().addResourceProviderListener(this);
                	
                } else {
                    if (!file.equals(getResourceProvider().getFile())) {
                        getResourceProvider().disconnect();
                        getResourceProvider().connect(this);
                        getResourceProvider().addResourceProviderListener(this);
                    }
                }

                Debug.traceln ("MultiPageAcmeEditor.setInput: getting resource provider == " + getResourceProvider());
                _design = getResourceProvider().getASDesign();
                Debug.traceln ("MultiPageAcmeEditor.setInput: input == " + input + "; resourceProvider.asDesign == " + _design);
                // [SWC 2004-06-15] setTitle deprecated.
                // Change code to use the two replacement methods. 
                setPartName(file.getName());
                setContentDescription(file.getFullPath().toOSString());
            } catch (CoreException e) {
                ErrorDialog.openError(getEditorSite().getShell(),
                    AcmeStudioPlugin.getMessage("MultiPageAcmeEditor.openErrorTitle"),
                    "Encountered unexpected error while opening resource!", e.getStatus());
                closeEditor(true);
            } catch (ParseError e) {
                // don't worry about this error at this point, as it will be
                // handled later when the editors are created.
                // but, make sure to add this as the resource provider listener
                getResourceProvider().addResourceProviderListener(this);
            }
            catch (Throwable t) {
            	MessageDialog.openError(getEditorSite().getShell(),
                        AcmeStudioPlugin.getMessage("MultiPageAcmeEditor.openErrorTitle "),
                        "An error occurred opening this file - will only open the source editor:\n" + t.getMessage());
            	 t.printStackTrace();
                    closeEditor(true);
            }
        }
        Debug.traceln ("MultipageAcmeEditor.setInput: returning normally");
    }

    /**
    * Inherit setInput.
    * @see org.eclipse.ui.part.EditorPart.setInput(IEditorInput).
    */
    protected void superSetInput(IEditorInput input) {
        // The workspace never changes for an editor.  So, removing and re-adding the 
        // resourceListener is not necessary.  But it is being done here for the sake
        // of proper implementation.  Plus, the resourceListener needs to be added 
        // to the workspace the first time around.
        if (getEditorInput() != null) {
            IFile file = ((FileEditorInput) getEditorInput()).getFile();
            file.getWorkspace().removeResourceChangeListener(_resourceListener);
        }

        super.setInput(input);

        if (getEditorInput() != null) {
            IFile file = ((FileEditorInput) getEditorInput()).getFile();
            file.getWorkspace().addResourceChangeListener(_resourceListener);
            // [SWC 2004-06-15] setTitle deprecated.
            // Change code to use the two replacement methods. 
            setPartName(file.getName());
            setContentDescription(file.getFullPath().toOSString());
        }
    }

    /** Listens to changes to the file system in the workspace, and 
    * makes changes accordingly.
    * 1) An open, saved file gets deleted -> close the editor
    * 2) An open file gets renamed or moved -> change the editor's input
    *    accordingly.
    */
    class ResourceTracker implements IResourceChangeListener, IResourceDeltaVisitor {
        /**
         * Handler of resource change event.
         * @param event Resource change event.
         */
        public void resourceChanged(IResourceChangeEvent event) {
            IResourceDelta delta = event.getDelta();
            try {
                if (delta != null)
                    delta.accept(this);
            } catch (CoreException e) {
                Debug.errorln(e);
            }
        }
        /**
         * @see org.eclipse.core.resources.IResourceDeltaVisitor#visit(IResourceDelta).
         */
        public boolean visit(IResourceDelta delta) {
            boolean result = false;
            if (delta == null
                || !delta.getResource().equals(((FileEditorInput) getEditorInput()).getFile())) {
                result = true;
            } else if (delta.getKind() == IResourceDelta.REMOVED) {
                if ((IResourceDelta.MOVED_TO & delta.getFlags()) == 0) {
                    // if the file was deleted
                    // NOTE: The case where an open, unsaved file is deleted is being handled by the 
                    // PartListener added to the Workbench in the initialize() method.
                    if (!isDirty())
                        closeEditor(false);
                } else { // else if it was moved or renamed
                    final IFile newFile =
                        ResourcesPlugin.getWorkspace().getRoot().getFile(delta.getMovedToPath());
                    Display display = getSite().getShell().getDisplay();
                    display.asyncExec(new Runnable() {
                        public void run() {
                            superSetInput(new FileEditorInput(newFile));
                        }
                    });
                }
            }
            return result;
        }
    }

    /**
     * @see org.eclipse.ui.part.MultiPageEditorPart#pageChange(int)
     */
    protected void pageChange(int newPageIndex) {
        if (_renewingPages
            || newPageIndex >= getPageCount()) {
            // do not enter, lest we cause unexpected exception
            return;
        }

        this.getSite ().getSelectionProvider().removeSelectionChangedListener(this);
        if (_designBrowser != null)
            this.getSite ().getSelectionProvider ().removeSelectionChangedListener(_designBrowser);
        if (_elementViewPage != null) 
            this.getSite ().getSelectionProvider ().removeSelectionChangedListener(_elementViewPage);

        if (getResourceProvider().isInTextEditor()
            && !(getActiveEditor() instanceof AcmeTextEditor)) {

			Debug.diagnoseln("MultiPageAcmeEditor.pageChange: " + "Changing from text editor page");
            getResourceProvider().setInTextEditor(false);
            try {
                if (!getResourceProvider().isSynchedWithText()) {
                	Debug.diagnoseln("MultiPageAcmeEditor.pageChange: " + "Replacing AcmeDesign...");
                    // this call will also take care of extractions
                    getResourceProvider().reloadFromText();
                    return;
                }
            } catch (CoreException e) {
                ErrorDialog.openError(getEditorSite().getShell(),
                    "Source Sync Error", e.getStatus().getMessage(), e.getStatus());
                setActivePage(TEXT_EDITOR_IDX);
                getResourceProvider().setInTextEditor(true);
                return;
            }
        } else if (!getResourceProvider().isInTextEditor()
                   && getActiveEditor() instanceof AcmeTextEditor) {
            Debug.diagnoseln("MultiPageAcmeEditor.pageChange: " + "Changing to text editor page");
            getResourceProvider().setInTextEditor(true);
            try {
                if (!getResourceProvider().isSynchedWithText()) {
                	Debug.diagnoseln("MultiPageAcmeEditor.pageChange: " + "Replacing Text Editor content...");
                    getResourceProvider().updateText();
                }
            } catch (CoreException e) {
                ErrorDialog.openError(getEditorSite().getShell(),
                    "Source Update Error", e.getStatus().getMessage(), e.getStatus());
            }
        }
        //super.pageChange(newPageIndex);
        // Copied from super
        Control control = getControl (newPageIndex);
        if (control != null) control.setVisible (true);
        
        setFocus ();
        IEditorPart activeEditor = getEditor (newPageIndex);
        IEditorActionBarContributor contributor = getEditorSite ().getActionBarContributor();
        if (contributor != null && contributor instanceof MultiPageEditorActionBarContributor) {
        	((MultiPageEditorActionBarContributor )contributor).setActiveEditor (activeEditor);
        }
        if (activeEditor != null) {
        	ISelectionProvider selectionProvider = activeEditor.getSite().getSelectionProvider();
        	ISelection selection = selectionProvider.getSelection ();
        	if (selection == null) selection = new StructuredSelection (Constants.EMPTY_ELEMENTS);
        	((MultiPageSelectionProvider )getSite ().getSelectionProvider()).fireSelectionChanged(new SelectionChangedEvent (selectionProvider, selection));
        }
        ///////////////////////////////////////////////////////////////////////
        // Register key bindings to KeyBindingService
        //IEditorPart newEditor = this.getEditor(newPageIndex);
        //registerKeyBindings(newEditor);
        
        this.getSite().getSelectionProvider().addSelectionChangedListener(this);
        if (_designBrowser != null) 
            this.getSite().getSelectionProvider().addSelectionChangedListener(_designBrowser);
        if (_elementViewPage != null)
            this.getSite().getSelectionProvider().addSelectionChangedListener(_elementViewPage);
            
		IEditorPart newEditor = this.getEditor(newPageIndex);
		ISelection selection = null;
		if (newEditor instanceof DiagramEditor ) {
			selection = ((DiagramEditor )newEditor).getSelection ();
		}
		else if (newEditor instanceof AcmeTextEditor) {
			selection = ((AcmeTextEditor )newEditor).getSelection ();
		}
		else if (newEditor instanceof AcmeStyleEditor) {
			selection = ((AcmeStyleEditor )newEditor).getSelection ();
		}
		if (selection != null) {
		
		SelectionChangedEvent evt = new SelectionChangedEvent (getSite ().getSelectionProvider(), selection);
		selectionChanged (evt);
		}
    }

    /**
     * Returns current AcmeDesign
     * @return AcmeDesign
     * @deprecated use instead #getDesign() which returns ASDesign
     */
    public AcmeDesign getAcmeDesign() {
        return (_design == null) ? null : _design.getDesign();
    }

    /**
     * Returns current ASDesign
     * @return ASDesign
     */
    public ASDesign getDesign() {
        return _design;
    }

    /**
     * @see org.eclipse.ui.IEditorPart#isDirty()
     */
    public boolean isDirty() {
    	if (getResourceProvider () != null)
    		return getResourceProvider().isDirty();
    	return true;
    }

    /**
     * Sets the new dirty status.
     * @param d  the new dirty status
     */
    public void setDirty (boolean d) {
        if (getResourceProvider().isDirty() != d) {
            // this prevents circular invocation
            getResourceProvider().setDirty(d);
        }
        firePropertyChange(IEditorPart.PROP_DIRTY);
        firePropertyChange(IEditorPart.PROP_INPUT);
    }

    /**
     * Removes all editors for the page indicated by index i.
     * @param i  the page index
     */
    protected void removeEditorsForPage(int i) {
        IEditorPart editor = getEditor(i);
        _topEditors.remove(editor);
        if (editor instanceof DiagramEditor) {
            DiagramEditor dEd = (DiagramEditor) editor;
			(dEd).getSystemView().setElementsLoaded(false);
            if (editor.equals(_activeDiagramEditor)) {
                _activeDiagramEditor = null;
            }
            AcmeSystem sys = (dEd).getAcmeSystem();
            AcmeStudioChangeManager.removeListener(dEd);
            _sysEditorMap.remove(sys);
            
            for (int j = 0; j < _inactiveDiagramEditors.size(); j++) {
                DiagramEditor repEditor = (DiagramEditor )_inactiveDiagramEditors.get(j);
                if (repEditor.getAcmeSystem ().getParentDesign ().equals (getAcmeDesign())) {
                    _inactiveDiagramEditors.remove(j);
                    j--;   
                }   
                AcmeStudioChangeManager.removeListener(repEditor);
            }
        } else if (editor instanceof AcmeTextEditor) {
            _acmeSourceEditor = null;
        }
    }
    /**
     * Sets a flag in ASResourceProvider indicating whether or not we're now in
     * the TextEditor.
     * @see org.eclipse.ui.part.MultiPageEditorPart#setActivePage(int)
     */
    protected void setActivePage(int pageIndex) {
        if (getEditor (pageIndex) instanceof DiagramEditor) {
            if (((DiagramEditor )getEditor (pageIndex)).getRefreshNeeded()) {
                renewPages ();      
            } 
        }
        super.setActivePage(pageIndex);
        if (getEditor (pageIndex) instanceof AcmeStyleEditor) {
        	_activeFamilyEditor = ((AcmeStyleEditor )getEditor (pageIndex));
        }
        
        if (pageIndex == TEXT_EDITOR_IDX) {
            getResourceProvider().setInTextEditor(true);
            _acmeSourceEditor.setFocus();  // to make sure cursor gets set
        } else {
            getResourceProvider().setInTextEditor(false);
        }


        // Register key bindings to KeyBindingService
        IEditorPart newEditor = this.getEditor(pageIndex);
        registerKeyBindings(newEditor);
    }

    private void registerKeyBindings(IEditorPart newEditor) {
        if (newEditor instanceof AcmeTextEditor) {
            ((AcmeTextEditor)newEditor).registerKeyBindings();
        } else if (newEditor instanceof DiagramEditor) {
            ((DiagramEditor)newEditor).registerKeyBindings();
        } else if (newEditor instanceof AcmeStyleEditor) {
            ((AcmeStyleEditor)newEditor).registerKeyBindings();
        }
    }

    public Canvas getOverview (Canvas overview) {
        //Canvas overview = new Canvas (parent, SWT.NONE);
        if (_activeDiagramEditor != null) {
            
        	
        	_lws = new LightweightSystem (overview);
        	if (_activeDiagramEditor != null) {
        		RootEditPart rep = _activeDiagramEditor.getGraphicalViewer().getRootEditPart();
        		if (rep instanceof ScalableFreeformRootEditPart) {
        			ScalableFreeformRootEditPart root = (ScalableFreeformRootEditPart )rep;
        			_thumbnail = new ScrollableThumbnail ((Viewport )root.getFigure ());
        			_thumbnail.setSource (root.getLayer (LayerConstants.PRINTABLE_LAYERS));
        			_lws.setContents(_thumbnail);
        		}
        	}
        }
        else {
            Label label = new Label (overview, SWT.NONE);
            label.setText ("No overview available");
        }
        return overview;
    }
    
    public void setThumbnailVisible (boolean vis) {
        if (_thumbnail != null) {
            _thumbnail.setVisible(vis);    
        }
    }
    
    public boolean canShowOverview () {
        return _activeDiagramEditor != null;
    }
    
    private void refreshThumbnail () {
        if (_activeDiagramEditor != null) {
            RootEditPart rep = _activeDiagramEditor.getGraphicalViewer().getRootEditPart();
            if (rep instanceof ScalableFreeformRootEditPart) {
                ScalableFreeformRootEditPart root = (ScalableFreeformRootEditPart) rep;
                _thumbnail = new ScrollableThumbnail((Viewport) root.getFigure());
                _thumbnail.setSource(root.getLayer(LayerConstants.PRINTABLE_LAYERS));
                if (_lws != null) {
                    _lws.setContents(_thumbnail);
                    //_lws.getUpdateManager ().performUpdate ();
                }
            }
        }    
        
    }

	/**
	 * @param activeEditor
	 */
	public void setActiveSubEditor(IEditorPart activeSubEditor) {
		IEditorPart activeEditor = this.getActiveEditor();
		if (activeEditor instanceof AcmeStyleEditor) {
			MultiPageActionBarContributor contributor = (MultiPageActionBarContributor )getEditorSite().getActionBarContributor();
			contributor.getStyleContributor ().setActivePage (activeSubEditor);
		}
	}

	/**
	 * @return
	 */
	public AcmeTextEditor getSourceEditor() {
		return _acmeSourceEditor;
	}
}
