package AcmeStudio.ui.editors;

import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.*;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.part.EditorActionBarContributor;
import org.eclipse.ui.part.MultiPageEditorActionBarContributor;

import AcmeStudio.core.actions.FamilyAction;
import AcmeStudio.core.actions.SystemAction;
import AcmeStudio.model.FamilyCommand;
import AcmeStudio.model.SystemCommand;
import AcmeStudio.ui.editors.diagram.DiagramActionBarContributor;
import AcmeStudio.ui.editors.diagram.actions.ChangeOrderAction;
import AcmeStudio.ui.editors.text.TextActionBarContributor;

/**
 * @author schmerl
 *
 * Manages the contributions of the Multipage Acme Editor - which commands are active, what tools are shown...
 */
public class MultiPageActionBarContributor
    extends MultiPageEditorActionBarContributor {

    protected DiagramActionBarContributor _editorActions = null;
    protected MultiPageStyleEditorActionBarContributor _styleActions = null;
    protected TextActionBarContributor _textActions = null;
    
    protected EditorActionBarContributor _currentContributor = null;
    protected SubToolBarManager _diagramTBM;
    protected SubToolBarManager _styleTBM;
    protected SubToolBarManager _textTBM;
    
    SystemAction _addSystemAction = new SystemAction (SystemCommand.ADD_SYSTEM);
    SystemAction _removeSystemAction = new SystemAction (SystemCommand.REMOVE_SYSTEM);
    FamilyAction _addFamilyAction = new FamilyAction (FamilyCommand.ADD_FAMILY);
    FamilyAction _removeFamilyAction = new FamilyAction (FamilyCommand.REMOVE_FAMILY);
//    ImportFamilyAction _importFamilyACtion = new ImportFamilyAction (ImportFamilyAction.ID);

    private void configureToolBars (boolean dVis, boolean sVis, boolean tVis) {
        if (_diagramTBM != null) {
        	_diagramTBM.setVisible(dVis);
         }
        if (_styleTBM != null) _styleTBM.setVisible (sVis);
        if (_textTBM != null) _textTBM.setVisible (tVis);
        if (_diagramTBM != null) _diagramTBM.getParent().update(true);
 
        if (_editorActions != null && _editorActions.getActionBars () != null) {
        	_editorActions.getActionBars().getToolBarManager().update(true);
        	_editorActions.getActionBars().updateActionBars();
        }
        if (_styleActions != null && _styleActions.getActionBars () != null) 
        	_styleActions.getActionBars().updateActionBars();
        if (_textActions != null && _textActions.getActionBars () != null)
        	_textActions.getActionBars ().updateActionBars();
    }
    /**
     * @see org.eclipse.ui.part.MultiPageEditorActionBarContributor#setActivePage(IEditorPart)
     */
    public void setActivePage(IEditorPart activeEditor) {
        if (activeEditor instanceof DiagramEditor) {
            if (_editorActions == null) {
                _editorActions = new DiagramActionBarContributor ();
                _editorActions.init (this.getActionBars(), this.getPage ());
            }
            configureToolBars (true, false, false);

            _editorActions.setActiveEditor (activeEditor);
//            _editorActions.setActiveEditor (activeEditor);
        }
        else if (activeEditor instanceof AcmeStyleEditor) {
            if (_styleActions == null) {
                _styleActions = new MultiPageStyleEditorActionBarContributor ();
                _styleActions.init (this.getActionBars (), this.getPage ());
            }
            configureToolBars (true, true, false);

            _styleActions.setActiveEditor (activeEditor);
//            _currentContributor = _styleActions;
        }
        else if (activeEditor instanceof AcmeTextEditor) {
            if (_textActions == null) {
                _textActions = new TextActionBarContributor ();
                _textActions.init (this.getActionBars (), this.getPage ());
            }
            configureToolBars(false, false, true);
            _textActions.setActiveEditor (activeEditor);
            ((AcmeTextEditor) activeEditor).registerKeyBindings();
//            _currentContributor = _textActions;
        }
        if (getActionBars () != null) {
            getActionBars().updateActionBars();
            getActionBars ().getToolBarManager().update (true);
        }
//        if (_editorActions != null) {
//            _editorActions.setActiveEditor (activeEditor);
//        }
//            
//        }
//        if (_styleActions != null) {
//            _styleActions.setActiveEditor (activeEditor);
//        }
//        if (_textActions != null) {
//            _textActions.setActiveEditor (activeEditor);
//        }
    }

    public MultiPageActionBarContributor () {
        super ();
        _editorActions = new DiagramActionBarContributor ();
        _styleActions = new MultiPageStyleEditorActionBarContributor ();
        _textActions = new TextActionBarContributor (); 
    }
    /**
     * @see org.eclipse.ui.IEditorActionBarContriutor#init(IActionBars, IWorkbenchPage)
     */
    public void init(IActionBars bars, IWorkbenchPage page) {
//        super.init(bars, page);
        IMenuManager menuManager = bars.getMenuManager ();
        IContributionItem viewMenu = menuManager.findUsingPath ("AcmeStudio.view");         
        if (viewMenu == null) {
            viewMenu =  new MenuManager ("View", "AcmeStudio.view");
        
            ((MenuManager )viewMenu).add(new GroupMarker ("zooming"));
            ((MenuManager )viewMenu).add (new Separator ());
	        ((MenuManager )viewMenu).add (new GroupMarker ("options"));
	        ((MenuManager )viewMenu).add (new Separator ());
	        ((MenuManager )viewMenu).add (new GroupMarker ("visualizations"));
	        ((MenuManager )viewMenu).add (new GroupMarker ("additions"));
	        menuManager.insertAfter (IWorkbenchActionConstants.M_EDIT, viewMenu);
        }

        IContributionItem familyMenu = menuManager.findUsingPath("AcmeStudio.family");
        if (familyMenu == null) {
            familyMenu = createFamilyMenu ();
            bars.getMenuManager ().prependToGroup("additions", familyMenu);  
        }
        
        IContributionItem designMenu = menuManager.findUsingPath ("AcmeStudio.design");
        if (designMenu == null) {      
            designMenu = createDesignMenu ();
            bars.getMenuManager().prependToGroup ("additions", designMenu);
        }
        
        
        _editorActions.init (bars, page);
        _styleActions.init(bars, page);
        _textActions.init(bars, page);
        super.init (bars, page); 
//        _editorActions = new DiagramActionBarContributor ();
//        _editorActions.init (bars, page);
//        _styleActions.init (bars, page);
//        _textActions.init (bars, page);     
 
    }

    protected IContributionItem createFamilyMenu () {
        MenuManager familyMenu = new MenuManager ("Family", "AcmeStudio.family");
        familyMenu.add (new GroupMarker ("family"));
        familyMenu.add (new GroupMarker ("assignment"));
        familyMenu.add (new GroupMarker ("types"));
        familyMenu.add (new Separator ());
        familyMenu.add (new GroupMarker ("analyses"));
        familyMenu.add (new Separator ());
        familyMenu.add (new GroupMarker ("additions"));
//        familyMenu.appendToGroup ("family", _addFamilyAction);
//        familyMenu.appendToGroup ("family", _importFamilyAction);
//        familyMenu.appendToGroup ("family", _removeFamilyAction);
        return familyMenu;
    }

    protected IContributionItem createDesignMenu () {
        MenuManager designMenu = new MenuManager ("Design", "AcmeStudio.design");
        designMenu.add (new GroupMarker ("system"));
        designMenu.add (new GroupMarker ("elements"));
        designMenu.add (new Separator ());
        designMenu.add (new GroupMarker ("types"));
        designMenu.add (new Separator ());
        designMenu.add (new GroupMarker ("representations"));
        designMenu.add (new Separator ());
        designMenu.add (new GroupMarker ("analyses"));
        designMenu.add (new GroupMarker ("additions"));
        
        MenuManager analysesMenu = new MenuManager ("Analyses", "AcmeStudio.analyses");
        analysesMenu.add(new GroupMarker ("additions"));
        designMenu.appendToGroup ("analyses", analysesMenu);
//        designMenu.appendToGroup ("system", _addSystemAction);
//        designMenu.appendToGroup ("system", _removeSystemAction);
        return designMenu;
    }
    /**
     * @see org.eclipse.ui.IEditorActionBarContributor#setActiveEditor(IEditorPart)
     */
    public void setActiveEditor(IEditorPart targetEditor) {
        super.setActiveEditor (targetEditor);
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToToolBar(org.eclipse.jface.action.IToolBarManager)
     */
    public void contributeToToolBar(IToolBarManager toolBarManager) {
        _diagramTBM = new SubToolBarManager (toolBarManager);
        _styleTBM = new SubToolBarManager (toolBarManager);
        _textTBM = new SubToolBarManager (toolBarManager);
        

        _editorActions.contributeToToolBar(_diagramTBM);
        _styleActions.contributeToToolBar(_styleTBM);
        _textActions.contributeToToolBar(_textTBM);

    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToMenu(org.eclipse.jface.action.IMenuManager)
     */
    public void contributeToMenu(IMenuManager menuManager) {

        super.contributeToMenu(menuManager);
    }
	/**
	 * @return
	 */
	public MultiPageStyleEditorActionBarContributor getStyleContributor() {
		return _styleActions;
	}
	
	public DiagramActionBarContributor getDiagramContributor () {
		return _editorActions;
	}

	public TextActionBarContributor getTextContributor () {
		return _textActions;
	}
	/**
	 * @param alignment
	 * @param ordering
	 * @param zoom
	 */
	public void updateTools(boolean alignment, boolean ordering, boolean zoom) {
		if (_editorActions != null) {
			IToolBarManager tbm = _editorActions.getActionBars().getToolBarManager();
			if (alignment &&  tbm.find(GEFActionConstants.ALIGN_CENTER) == null) {
				_editorActions.contributeAlignmentTools(tbm);
			}
			else if (!alignment && tbm.find (GEFActionConstants.ALIGN_CENTER) != null){
				_editorActions.removeAlignmentTools(tbm);
			}
			
			if (ordering && tbm.find(ChangeOrderAction.MOVE_BACK) == null) {
				_editorActions.contributeOrderTools(tbm);
			}
			else if (!ordering && tbm.find (ChangeOrderAction.MOVE_BACK) != null){
				_editorActions.removeOrderTools(tbm);
			}
			
			if ( zoom && tbm.find(GEFActionConstants.ZOOM_IN) == null) {
				_editorActions.contributeZoomTools(tbm);
			}
			else if (!zoom && tbm.find (GEFActionConstants.ZOOM_IN) != null){
				_editorActions.removeZoomTools(tbm);
			}
			tbm.update(true);
			_editorActions.getActionBars().updateActionBars();
		}
	}
	
}
