package AcmeStudio.ui.editors.actions;

/*
 * (c) Copyright IBM Corp. 2000, 2002.
 * All Rights Reserved.
 */

import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.swt.SWT;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.internal.WorkbenchImages;

import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.actions.CopyElementCommand;

/**
 * Copies a property to the clipboard.
 */
public class CopyAcmeElementAction extends SelectionAction {

    public static final String ID =
        ActionFactory.COPY.getId ();

    /**
     * Creates the action.
     */
    public CopyAcmeElementAction(DiagramEditor editor) {
        super(editor);
    }


    /** 
     * Updates enablement based on the current selection
     */
//    public void selectionChanged(IStructuredSelection sel) {
//        setEnabled(!sel.isEmpty());
//    }
    
    /**
     * Initializes this action's text and images.
     */
    protected void init() {
        super.init();
        setId(ID);
        setText("Copy");
        setToolTipText("Copy Acme Element to Clipboard");
        setImageDescriptor(
            WorkbenchImages.getImageDescriptor(
                ISharedImages.IMG_TOOL_COPY));
        // As of Eclipse 3.0, IMG_TOOL_COPY_HOVER == IMG_TOOL_COPY
        setHoverImageDescriptor(
            WorkbenchImages.getImageDescriptor(
                ISharedImages.IMG_TOOL_COPY));
        setDisabledImageDescriptor(
            WorkbenchImages.getImageDescriptor(
                ISharedImages.IMG_TOOL_COPY_DISABLED));
        setAccelerator(SWT.CTRL | 'c');

        setEnabled(false);
    }

    /**
     * Create a command to remove the selected objects.
     *
     * @param objects The objects to be deleted.
     *
     * @return The command to remove the selected objects.
     */
    public static Command createCopyCommand(List objects) {
        if (objects.isEmpty())
            return null;
//        Object obj = objects.get(0);
//        if (!(obj instanceof EditPart))
//            return null;
//
//        EditPart editPart = (EditPart)obj;
        
        CopyElementCommand command = new CopyElementCommand(objects);
        return command;
    }

    /**
     * Returns <code>true</code> if the selected objects can
     * be deleted.  Returns <code>false</code> if there are
     * no objects selected or the selected objects are not
     * {@link EditPart}s.
     */
    protected boolean calculateEnabled() {
        Command cmd = createCopyCommand(getSelectedObjects());
        if (cmd == null)
            return false;
        return cmd.canExecute();
    }

    /**
     * Performs the delete action on the selected objects.
     */
    public void run() {
        execute(createCopyCommand(getSelectedObjects()));
    }

}
