/*
 * Created on Jun 18, 2003
 *
 */
package AcmeStudio.ui.editors.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.requests.GroupRequest;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.internal.WorkbenchImages;

import AcmeStudio.ui.editors.diagram.parts.*;

/**
 * @author Bradley Schmerl
 * 
 * Custom delete action that makes sure editparts get deleted in the right order, which is
 * attachments, then roles and ports, then components and connectors
 */
public class DiagramDeleteAction extends SelectionAction {
    public static final String ID = ActionFactory.DELETE.getId ();

    public DiagramDeleteAction(IEditorPart editor) {
        this((IWorkbenchPart)editor);
    }

    public DiagramDeleteAction(IWorkbenchPart part) {
        super(part);
        setLazyEnablementCalculation(false);
    }

    public DiagramDeleteAction(IEditorPart editor, String label) {
        super(editor);
        setText(label);
    }

    protected void init() {
        super.init();
        setText("Delete elements");
        setToolTipText("Deletes the selected elements");
        setId(ActionFactory.DELETE.getId ());
        // As of Eclipse 3.0, IMG_TOOL_DELETE_HOVER == IMG_TOOL_DELETE
        setHoverImageDescriptor(WorkbenchImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
        setImageDescriptor(WorkbenchImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
        setDisabledImageDescriptor(WorkbenchImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE_DISABLED));
        setEnabled(false);

    }
    /* (non-Javadoc)
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
        Command cmd = createDeleteElementCommand(getSelectedObjects());
        if (cmd == null)
            return false;
        return cmd.canExecute();
    }

    protected Command createDeleteElementCommand(List objects) {
        if (objects.isEmpty())
            return null;
        if (!(objects.get(0) instanceof EditPart))
            return null;
        GroupRequest deleteReq = new GroupRequest(RequestConstants.REQ_DELETE);
       

        // Partition selections into attachments, roles, ports, components, connectors
        List attachments = new ArrayList();
        List roles = new ArrayList();
        List ports = new ArrayList();
        List components = new ArrayList();
        List connectors = new ArrayList();
        List others = new ArrayList();

        Iterator i = objects.iterator();
        while (i.hasNext()) {
            EditPart object = (EditPart)i.next();
            if (object instanceof AttachmentEditPart) {
                attachments.add(object);
            }
            else if (object instanceof RoleEditPart) {
                roles.add(object);
            }
            else if (object instanceof PortEditPart) {
                ports.add(object);
            }
            else if (object instanceof ComponentEditPart) {
                components.add(object);
            }
            else if (object instanceof ConnectorEditPart) {
                connectors.add(object);
            }
            else {
                others.add(object);
            }
        }
        List sortedSelection = new ArrayList();
        sortedSelection.addAll(attachments);
        sortedSelection.addAll(roles);
        // BugFix #168
        sortedSelection.addAll(ports);
        sortedSelection.addAll(components);
        sortedSelection.addAll(connectors);
        sortedSelection.addAll(others);

        deleteReq.setEditParts(sortedSelection);
        
		Command deleteCmd = null;
		if (sortedSelection.size () > 1) {
        	CompoundCommand compoundCmd = new CompoundCommand("Delete");
        	i = sortedSelection.iterator();
        	while (i.hasNext()) {
            	EditPart ep = (EditPart)i.next();
            	Command cmd = ep.getCommand(deleteReq);
            	if (cmd != null)
                	compoundCmd.add(cmd);
        	}
        	deleteCmd = compoundCmd;
		}
		else if (sortedSelection.size () == 1){
			EditPart ep = (EditPart )sortedSelection.get (0);
			deleteCmd = ep.getCommand (deleteReq);
		}

        return deleteCmd;

    }

    public void run() {
        execute(createDeleteElementCommand(getSelectedObjects()));
    }

}
