/*
 * Created on Jul 15, 2004
 *
 */
package AcmeStudio.ui.editors.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.ui.actions.SelectAllAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;
import AcmeStudio.ui.editors.diagram.parts.PortEditPart;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;


/**
 * @author Bradley Schmerl
 * Created Jul 15, 2004
*  Specialization of SelectAllAction that does not select roles or ports
 */
public class DiagramSelectAllAction extends SelectAllAction {

	private final IWorkbenchPart	_part;
	/**
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 **/
	public void run() {
		GraphicalViewer viewer = (GraphicalViewer )_part.getAdapter (GraphicalViewer.class);
		if (viewer != null) {
			List children = viewer.getContents ().getChildren ();
			
			// Filter out roles and ports, because they'll be deleted with components and connectors
			// This makes undo work properly
			List filteredChildren = new ArrayList ();
		    for (Iterator iter = children.iterator (); iter.hasNext ();) {
				Object element = iter.next ();
				if (!(element instanceof RoleEditPart) && !(element instanceof PortEditPart)) {
					filteredChildren.add (element);
				}
			}
		    viewer.setSelection (new StructuredSelection (filteredChildren));
		}
	}
	/**
	 * @param part
	 */
	public DiagramSelectAllAction(IWorkbenchPart part) {
		super (part);
		_part = part;
	}
}
