package AcmeStudio.ui.editors.actions;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.*;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.ActionFactory;

import AcmeStudio.core.actions.OpenTypeHierarchyAction;
import AcmeStudio.model.RepresentationCommand;
import AcmeStudio.model.TypeAssignmentCommand;
import AcmeStudio.ui.editors.diagram.actions.*;

/**
 * @author schmerl
 *
 */
public class EditorContextMenuProvider
    extends org.eclipse.gef.ContextMenuProvider {

    private ActionRegistry actionRegistry;
    private IEditorPart editorPart;
    

    /**
     * @return
     */
    public IEditorPart getEditorPart() {
        return editorPart;
    }

    /**
     * @param part
     */
    public void setEditorPart(IEditorPart part) {
        editorPart = part;
    }

    public EditorContextMenuProvider(
        EditPartViewer viewer,
        ActionRegistry registry) {
        super(viewer);
        setActionRegistry(registry);
        
    }

    protected void addContextContributions(IMenuManager menu) {
        IAction action =
            getActionRegistry().getAction(NewElementAction.NEW_PORT);
        if (action.isEnabled()) {
            menu.add(action);
        }

        action = getActionRegistry().getAction(NewElementAction.NEW_ROLE);
        if (action.isEnabled()) {
            menu.add(action);
        }

        action = getActionRegistry().getAction(NewBindingAction.NEW_BINDING);
        if (action.isEnabled()) {
            menu.add(action);
        }

        action =
            getActionRegistry().getAction(TypeAssignmentCommand.ASSIGN_TYPE);
        if (action.isEnabled()) {
            IMenuManager assSubMenu =
                ((TypeAssignmentMenuAction) action).buildMenu();
            menu.add(assSubMenu);
        }

        action =
            getActionRegistry().getAction(TypeAssignmentCommand.UNASSIGN_TYPE);
        if (action.isEnabled()) {
            IMenuManager unassSubMenu =
                ((TypeAssignmentMenuAction) action).buildMenu();
            menu.add(unassSubMenu);
        }

        action =
            getActionRegistry().getAction(TypeAssignmentCommand.ASSIGN_FAMILY);
        if (action.isEnabled()) {

            IMenuManager subMenu =
                ((TypeAssignmentMenuAction) action).buildMenu();
            menu.add(subMenu);
        }

        action =
            getActionRegistry().getAction(
                TypeAssignmentCommand.UNASSIGN_FAMILY);
        if (action.isEnabled()) {

            IMenuManager subMenu =
                ((TypeAssignmentMenuAction) action).buildMenu();
            menu.add(subMenu);
        }
		action = getActionRegistry ().getAction (NavigateToImportedFamilyAction.ID);
		if (action.isEnabled ())  {
			IMenuManager subMenu = ((NavigateToImportedFamilyAction )action).buildMenu();
			menu.add (subMenu);
		}
        buildRepresentationMenu(menu);

    }

    protected void addAlignmentContributions(IMenuManager menu) {
        IAction action =
            getActionRegistry().getAction(
                ResetPortAlignmentCommand.RESET_ALIGNMENT);
        if (action.isEnabled()) {
            IMenuManager alignmentSubMenu =
                ((ResetPortAlignmentAction) action).buildMenu();
            menu.add(alignmentSubMenu);
        }

        action =
            getActionRegistry().getAction(ResetSystemLayoutCommand.RESET_SYSTEMLAYOUT);
        menu.add(action);

        action =
            getActionRegistry().getAction(AdjustPortAlignmentCommand.ADJUST);
        if (action.isEnabled()) {
            IMenuManager alignmentSubMenu =
                ((AdjustPortAlignmentAction) action).buildMenu();
            menu.add(alignmentSubMenu);
        }
        
        action =
            getActionRegistry().getAction(
                ResetRoleRouteCommand.RESET_ROUTE);
        if (action.isEnabled()) {
            IMenuManager alignmentSubMenu =
                ((ResetRoleRouteAction) action).buildMenu();
            menu.add(alignmentSubMenu);
        }
        
    }

    protected void buildRepresentationMenu(IMenuManager menu) {
        IAction actionAR =
            getActionRegistry().getAction(RepresentationCommand.ADD_REP);
        RepresentationAction actionDR =
            (RepresentationAction) getActionRegistry().getAction(
                RepresentationCommand.REMOVE_REP);
        IAction actionNU = getActionRegistry().getAction(NavigateUpAction.ID);
        ExportUnboundToParentAction actionBindings = (ExportUnboundToParentAction )getActionRegistry().getAction (ExportUnboundToParentAction.ID);

        boolean addedSep = false;
        if (actionAR.isEnabled()) {
            menu.add(new Separator("Representations"));
            addedSep = true;
            menu.add(actionAR);
            IMenuManager deleteSubMenu = actionDR.buildMenu();
            menu.add(deleteSubMenu);
        }

        if (actionNU.isEnabled()) {
            if (!addedSep)
                menu.add(new Separator("Representations"));
            menu.add(actionNU);
            IMenuManager bindingsSubMenu = actionBindings.buildMenu ();
            
            menu.add(bindingsSubMenu);
        }
        
    }

    protected void addEditorPartContributions(IMenuManager menu) {
        //      menu.add (getGraphicalEditor ().getAction (IWorkbenchActionConstants.SAVE));
    }

    protected void addEditDomainContributions(IMenuManager menu) {
        menu.add(getActionRegistry().getAction(ActionFactory.UNDO.getId ()));
        menu.add(getActionRegistry().getAction(ActionFactory.REDO.getId ()));
    }

    protected void addEditPartViewerContributions(IMenuManager menu) {
        IAction action =
            getActionRegistry().getAction(ActionFactory.DELETE.getId ());
        if (action.isEnabled())
            menu.add(action);

        //        action = getGraphicalEditor ().getAction (IWorkbenchActionConstants.CUT);
        //        if (action.isEnabled())
        //            menu.add (action);

        action = getActionRegistry().getAction(ActionFactory.COPY.getId ());
        if (action.isEnabled())
            menu.add(action);

        action = getActionRegistry().getAction(ActionFactory.PASTE.getId ());
        if (action.isEnabled())
            menu.add(action);

        menu.add(new Separator());

        action = getActionRegistry().getAction(NewElementAction.NEW_COMPONENT);
        if (action.isEnabled())
            menu.add(action);

        action = getActionRegistry().getAction(NewElementAction.NEW_CONNECTOR);
        if (action.isEnabled()) {
            menu.add(action);
        }

        menu.add(new Separator());

        action = getActionRegistry().getAction(EditTypeVisualizationAction.ID);
        if (action.isEnabled()) {
            menu.add(action);
        }

        action = getActionRegistry ().getAction (OpenTypeHierarchyAction.ID);
        if (action.isEnabled ()) {
            menu.add (action);
        }
        
        action = getActionRegistry ().getAction (HideSelectionAction.ID);
        if (action.isEnabled ()) {
            menu.add (action);
        }

        action = getActionRegistry ().getAction (ShowFiguresAction.ID);
        if (action.isEnabled ()) {
            menu.add (action);
        }
        
        action = getActionRegistry ().getAction (UnhighlightAllAction.ID);
        if (action.isEnabled ()) {
            menu.add (action);
        }
    }

    /**
     * @see org.eclipse.gef.ui.parts.ContextMenuProvider#buildContextMenu(IMenuManager, EditPartViewer)
     */
    public void buildContextMenu(IMenuManager menu) {
        addContextContributions(menu);
        menu.add(new Separator());
        addEditDomainContributions(menu);
        menu.add(new Separator());
        addEditPartViewerContributions(menu);
        menu.add(new Separator());
        addEditorPartContributions(menu);
        menu.add(new Separator());
        addAlignmentContributions(menu);
        menu.add (new GroupMarker ("additions"));
    }

    private ActionRegistry getActionRegistry() {
        return actionRegistry;
    }

    private void setActionRegistry(ActionRegistry registry) {
        actionRegistry = registry;
    }

}
