package AcmeStudio.ui.editors.actions;

import java.util.List;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IEditorPart;

import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.actions.AddBindingCommand;
import AcmeStudio.ui.editors.diagram.parts.*;
import acmeLib.elements.AcmeElement;

/**
 * @author schmerl
 *
 */
public class NewBindingAction extends SelectionAction {

	private static final String NEW_BINDING_REQUEST = "New binding";
	public static final String NEW_BINDING = "New binding";
	
	Request request;	


	/**
	 * Constructor for NewBindingAction.
	 * @param editor
	 */
	public NewBindingAction(IEditorPart editor) {
		super(editor);
		setText (NEW_BINDING);
		setId (NEW_BINDING);
		request = new Request (NEW_BINDING_REQUEST);
	}

	/**
	 * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		if (getSelectedObjects ().size () != 2) {
			return false;
		}
		
		List parts = getSelectedObjects ();
		
		Object sel1 = parts.get (0);
		Object sel2 = parts.get (1);
		
		if ((sel1 instanceof PortEditPart && sel2 instanceof BindingPointEditPart) ||
			 (sel1 instanceof BindingPointEditPart && sel2 instanceof PortEditPart) ||
			 (sel1 instanceof RoleEditPart && sel2 instanceof BindingPointEditPart) ||
			 (sel1 instanceof BindingPointEditPart && sel2 instanceof RoleEditPart)) {
			 	return true;
			 }
		
		return false;		
			
	}

	private Command getCommand () {
		DiagramEditor editor = (DiagramEditor)this.getWorkbenchPart();
		SystemEditPart rootPart = editor.getRootEditPart ();
		
		List selections = getSelectedObjects ();
	
		
		AcmeElement inner, outer;
		BindingPointEditPart bpe = null;
        ElementEditPart pep = null;
		if (selections.get (0) instanceof BindingPointEditPart) {
			bpe = (BindingPointEditPart )selections.get (0);
            pep = (ElementEditPart )selections.get (1);
		}
		else {
            pep = (ElementEditPart )selections.get (0);
			bpe = (BindingPointEditPart )selections.get (1);
		}
		
        inner = (AcmeElement )pep.getModel ();
        outer = ((SystemEditPart.BindingPoint )bpe.getModel ())._outer;
        
		AddBindingCommand command = new AddBindingCommand (outer, inner, rootPart);
		  	
		return command;
	}

	/**
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		execute (getCommand ());
	}

}
