package AcmeStudio.ui.editors.actions;

import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.model.ModelCommand;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.actions.CreateElementCommand;
import AcmeStudio.ui.editors.diagram.figures.AcmeVisElement;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.ui.editors.diagram.parts.ConnectorEditPart;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.ui.views.parts.NewElementDialog;
import AcmeStudio.util.ImageManager;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeDuplicateNameException;
import acmeLib.core.filters.ComponentTypeEnumFilter;
import acmeLib.core.filters.ConnectorTypeEnumFilter;
import acmeLib.core.filters.PortTypeEnumFilter;
import acmeLib.core.filters.RoleTypeEnumFilter;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;

/**
 * @author schmerl
 *  
 */
public class NewElementAction extends SelectionAction implements
        IWorkbenchWindowActionDelegate
{

    private static final String NEW_COMPONENT_REQUEST = "New Component";
    private static final String NEW_CONNECTOR_REQUEST = "New Connector";
    private static final String NEW_PORT_REQUEST = "New Port";
    private static final String NEW_ROLE_REQUEST = "New Role";

    public static final String NEW_COMPONENT = "New Component";
    public static final String NEW_CONNECTOR = "New Connector";
    public static final String NEW_PORT = "New Port";
    public static final String NEW_ROLE = "New Role";

    Request request;

    /**
     * Constructor for NewElementAction.
     * 
     * @param editor
     * @param style
     */
    public NewElementAction(IEditorPart editor, String type)
    {
        super(editor);

        if (type.equals(NEW_COMPONENT))
        {
            request = new Request(NEW_COMPONENT_REQUEST);
            setImageDescriptor(ImageManager
                    .getSmallImageDescriptor(ImageManager.ID_COMP));
        }
        else if (type.equals(NEW_CONNECTOR))
        {
            request = new Request(NEW_CONNECTOR_REQUEST);
            setImageDescriptor(ImageManager
                    .getSmallImageDescriptor(ImageManager.ID_CONN));
        }
        else if (type.equals(NEW_PORT))
        {
            request = new Request(NEW_PORT_REQUEST);
            setImageDescriptor(ImageManager
                    .getSmallImageDescriptor(ImageManager.ID_PORT));
        }
        else if (type.equals(NEW_ROLE))
        {
            request = new Request(NEW_ROLE_REQUEST);
            setImageDescriptor(ImageManager
                    .getSmallImageDescriptor(ImageManager.ID_ROLE));
        }
        setText(type);
        setId(type);

    }

    protected boolean calculateEnabled()
    {
        return canPerformAction();
    }

    private boolean canPerformAction()
    {
        if (request.getType().equals(NEW_COMPONENT_REQUEST)
                || request.getType().equals(NEW_CONNECTOR_REQUEST))
        {
            return true;
        }

        if (getSelectedObjects().size() != 1)
        {
            return false;
        }

        List parts = getSelectedObjects();
        Object o = parts.get(0);
        if (!(o instanceof EditPart))
        {
            return false;
        }
        EditPart part = (EditPart) o;
        if (request.getType().equals(NEW_PORT_REQUEST))
        {
            if (!(part instanceof ComponentEditPart))
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        if (request.getType().equals(NEW_ROLE_REQUEST))
        {
            if (!(part instanceof ConnectorEditPart))
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        return false;

    }

    private Command getCommand()
    {
        DiagramEditor editor = (DiagramEditor) this.getWorkbenchPart();
        SystemEditPart rootPart = editor.getRootEditPart();
        CreateElementCommand create = new CreateElementCommand(rootPart);
        
        SystemView systemView = (SystemView) rootPart.getFigure();
        create.setSystemView(systemView);

        Shell shell = Display.getDefault().getActiveShell();
        AcmeSystem system = (AcmeSystem) rootPart.getModel();
        if (request.getType().equals(NEW_COMPONENT_REQUEST))
        {
            AcmeEnumeration availableTypes = (system).getAvailableTypes()
                    .castToAcmeVector()
                    .enumerate(new ComponentTypeEnumFilter());
            NewElementDialog dialog = new NewElementDialog(shell,
                    availableTypes, AcmeComponent.class, system);
            if (dialog.open() == IDialogConstants.OK_ID)
            {
                AcmeElement el = dialog.getNewElement();
                create.setLabel("Create Component " + el.getName());
                create.setParent(system);
                create.setSystem(system);
                create.setChild(el);
                create.setLocation(new Point(100, 100));
            }
            else
            {
                return null;
            }
        }
        else if (request.getType().equals(NEW_CONNECTOR_REQUEST))
        {
            AcmeEnumeration availableTypes = (system).getAvailableTypes()
                    .castToAcmeVector()
                    .enumerate(new ConnectorTypeEnumFilter());
            NewElementDialog dialog = new NewElementDialog(shell,
                    availableTypes, AcmeConnector.class, system);
            if (dialog.open() == IDialogConstants.OK_ID)
            {
                AcmeElement el = dialog.getNewElement();
                create.setLabel("Create Connector " + el.getName());
                create.setChild(el);
                create.setLocation(new Point(100, 100));
                create.setParent(system);
                create.setSystem(system);
            }
            else
            {
                return null;
            }
        }
        else if (request.getType().equals(NEW_PORT_REQUEST))
        {
            AcmeEnumeration availableTypes = (system).getAvailableTypes()
                    .castToAcmeVector().enumerate(new PortTypeEnumFilter());
            NewElementDialog dialog = new NewElementDialog(shell,
                    availableTypes, AcmePort.class, system);
            if (dialog.open() == IDialogConstants.OK_ID)
            {
                AcmeElement el = dialog.getNewElement();
                List parts = getSelectedObjects();
                ComponentEditPart compPart = (ComponentEditPart) parts.get(0);
                AcmeComponent parentComp = (AcmeComponent) compPart.getModel();
                create.setLabel("Create Port " + el.getName());
                create.setParent(parentComp);
                create.setChild(el);
                Point compLoc = systemView.getElementLocation(parentComp);
                AcmeVisElement visElement = systemView
                        .findFigureByLocation(compLoc);
                if (visElement != null)
                {
                    create.setParent(visElement.getModel());
                    create.setParentVis(visElement);

                }
                create.setLocation(compLoc);
            }
            else
            {
                return null;
            }
        }
        else if (request.getType().equals(NEW_ROLE_REQUEST))
        {
            AcmeEnumeration availableTypes = (system).getAvailableTypes()
                    .castToAcmeVector().enumerate(new RoleTypeEnumFilter());
            NewElementDialog dialog = new NewElementDialog(shell,
                    availableTypes, AcmeRole.class, system);
            if (dialog.open() == IDialogConstants.OK_ID)
            {
                AcmeElement el = dialog.getNewElement();
                List parts = getSelectedObjects();
                ConnectorEditPart connPart = (ConnectorEditPart) parts.get(0);
                AcmeConnector parentConn = (AcmeConnector) connPart.getModel();
                create.setLabel("Create Role " + el.getName());
                create.setChild(el);
                Point connLoc = SystemView.getLocation(parentConn, true);
                connLoc.x += 75;
                create.setLocation(new Point(0, 75));
                create.setParent(parentConn);
            }
            else
            {
                return null;
            }
        }

        return (Command)create;
    }

    public void run()
    {
        ModelCommand command = (ModelCommand) getCommand();
        execute(command);
        if (command != null)
        {
            try
            {
                command.checkError();
            }
            catch (AcmeDuplicateNameException e)
            {
                MessageDialog.openError(Display.getDefault().getActiveShell(),
                        "Duplicate name", e.getMessage());
            }
            catch (Exception e)
            {
                MessageDialog.openError(Display.getDefault().getActiveShell(),
                        "Error", e.getMessage());

            }
        }
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window)
    {
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(IAction)
     */
    public void run(IAction action)
    {
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection)
    {
    }

}