package AcmeStudio.ui.editors.actions;

import java.util.List;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.model.ModelCommand;
import AcmeStudio.model.TypeCommand;
import AcmeStudio.ui.views.parts.wizards.newTypes.NewElementTypeWizard;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import acmeLib.core.container.AcmeVector;
import acmeLib.core.exceptions.AcmeDuplicateNameException;
import acmeLib.core.filters.ComponentTypeEnumFilter;
import acmeLib.core.filters.ConnectorTypeEnumFilter;
import acmeLib.core.filters.PortTypeEnumFilter;
import acmeLib.core.filters.RoleTypeEnumFilter;
import acmeLib.elements.*;

/**
 * @author schmerl
 *
 */
public class NewElementTypeAction extends SelectionAction implements IWorkbenchWindowActionDelegate {

	private static final String NEW_COMPONENT_TYPE_REQUEST = "New Component Type";
	private static final String NEW_CONNECTOR_TYPE_REQUEST = "New Connector Type";
	private static final String NEW_PORT_TYPE_REQUEST = "New Port Type";
	private static final String NEW_ROLE_TYPE_REQUEST = "New Role Type";
	
	public static final String NEW_COMPONENT_TYPE = "New Component Type";
	public static final String NEW_CONNECTOR_TYPE = "New Connector Type";
	public static final String NEW_PORT_TYPE = "New Port Type";
	public static final String NEW_ROLE_TYPE = "New Role Type";
    AcmeFamily _family;
		
	Request request;

	/**
	 * Constructor for NewElementTypeAction.
	 * @param editor
	 * @param style
	 */
	public NewElementTypeAction(IEditorPart editor, String type, AcmeFamily context) {
		super(editor);
		_family = context;
		if (type.equals (NEW_COMPONENT_TYPE)) {		
			request = new Request (NEW_COMPONENT_TYPE_REQUEST);
			setImageDescriptor (ImageManager.getSmallImageDescriptor (ImageManager.ID_COMP));
		}
		else if (type.equals (NEW_CONNECTOR_TYPE)) {
			request = new Request (NEW_CONNECTOR_TYPE_REQUEST);
			setImageDescriptor (ImageManager.getSmallImageDescriptor (ImageManager.ID_CONN));
		}
		else if (type.equals (NEW_PORT_TYPE)) {
			request = new Request (NEW_PORT_TYPE_REQUEST);
			setImageDescriptor (ImageManager.getSmallImageDescriptor (ImageManager.ID_PORT));
		}
		else if (type.equals (NEW_ROLE_TYPE)) {
			request = new Request (NEW_ROLE_TYPE_REQUEST);
			setImageDescriptor (ImageManager.getSmallImageDescriptor (ImageManager.ID_ROLE));
		}
		setText (type);
		setId (type);
	}

	protected boolean calculateEnabled () {
		return _family != null;
	}
	
	public void run () {
        ModelCommand command = (ModelCommand )getCommand ();
        execute (command);
        if (command != null) {
            try {
                command.checkError();
            }
            catch (AcmeDuplicateNameException e) {
                MessageDialog.openError(
                    Display.getDefault().getActiveShell(),
                    "Duplicate name",
                    e.getMessage());
            }
            catch (Exception e) {
                MessageDialog.openError(
                    Display.getDefault().getActiveShell(),
                    "Error",
                    e.getMessage());

            }
        }
	}

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window) {
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(IAction)
     */
    public void run(IAction action) {
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
    }

    protected Command getCommand () {
        TypeCommand command = null;
        NewElementTypeWizard wizard = null; 
        Class category = null;
        List superTypes = null;
        AcmeVector availableTypes = _family.getAvailableTypes().castToAcmeSetVector();
        if (request.getType ().equals (NEW_COMPONENT_TYPE)) {
            category = AcmeComponentType.class;
            superTypes = Helper.convertAcmeEnumerationToList(availableTypes.enumerate (new ComponentTypeEnumFilter ()));
         }
        else if (request.getType ().equals (NEW_CONNECTOR_TYPE)) {
            category = AcmeConnectorType.class;
            superTypes = Helper.convertAcmeEnumerationToList(availableTypes.enumerate (new ConnectorTypeEnumFilter ()));
        }
        else if (request.getType ().equals (NEW_ROLE_TYPE)) {
            category = AcmeRoleType.class;
            superTypes = Helper.convertAcmeEnumerationToList(availableTypes.enumerate (new RoleTypeEnumFilter ()));
        }
        else if (request.getType ().equals (NEW_PORT_TYPE)) {
            category = AcmePortType.class;
            superTypes = Helper.convertAcmeEnumerationToList(availableTypes.enumerate (new PortTypeEnumFilter ()));
        }
        
        if (category != null) {
        	wizard = new NewElementTypeWizard (_family, category, superTypes);
        	WizardDialog dialog = new WizardDialog (Display.getDefault ().getActiveShell (), wizard);
                         
            if (dialog.open() == WizardDialog.OK) {
                AcmeElementType newType = wizard.getNewType();
                command = new TypeCommand (TypeCommand.ADD_TYPE, newType, _family, null);
            }
        }
        return command;

    }

}
