/*
 * Created on Jun 9, 2003
 *
 */
package AcmeStudio.ui.editors.actions;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.model.ModelCommand;
import AcmeStudio.model.TypeCommand;
import AcmeStudio.ui.views.parts.NewPropertyTypeDialog;
import acmeLib.core.exceptions.AcmeDuplicateNameException;
import acmeLib.elements.AcmeFamily;
import acmeLib.properties.AcmePropertyType;

/**
 * @author schmerl
 */
public class NewPropertyTypeAction extends SelectionAction implements IWorkbenchWindowActionDelegate {

    public static final String ID = "$NewPropType$";

    protected AcmeFamily _family;
    /**
     * @param editor
     */
    public NewPropertyTypeAction(IEditorPart editor, AcmeFamily context) {
        super(editor);
        _family = context;
        setText ("New Property Type");
        setId (ID);
    }


    /* (non-Javadoc)
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
        return _family != null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        ModelCommand command = (ModelCommand )getCommand ();
        execute (command);
        if (command != null) {
            try {
                command.checkError ();
            }
            catch (AcmeDuplicateNameException e) {
                MessageDialog.openError (Display.getDefault ().getActiveShell(), "Duplicate name", e.getMessage ());
            }
            catch (Exception e) {
                MessageDialog.openError (Display.getDefault ().getActiveShell (), "Error", e.getMessage ());   
            }
        }
    }
    
    public Command getCommand () {
        TypeCommand command = null;
        NewPropertyTypeDialog dialog = null;
        
        dialog = new NewPropertyTypeDialog (Display.getDefault().getActiveShell(), _family);
        // BUGFIX#250: Can now edit the type
        dialog.setEditable(true);
        if (dialog.open () == IDialogConstants.OK_ID) {
            AcmePropertyType newType = dialog.getNewType ();
            command = new TypeCommand (TypeCommand.ADD_TYPE, newType, _family, null);
           
        }
        
        return command;
    }
}
