package AcmeStudio.ui.editors.actions;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.dialogs.ListSelectionDialog;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.model.ModelCommand;
import AcmeStudio.model.ModelCompoundCommand;
import AcmeStudio.model.RepresentationCommand;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.ui.editors.diagram.actions.ZoomToFitAction;
import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.ui.editors.diagram.parts.ConnectorEditPart;
import AcmeStudio.ui.editors.diagram.parts.ElementEditPart;
import AcmeStudio.ui.views.parts.AddRepresentationDialog;
import AcmeStudio.util.IMenuBuilder;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeSystem;
import acmeLib.representations.AcmeRepresentation;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class RepresentationAction
	extends SelectionAction
	implements IMenuBuilder {

	private Request _request;
	private AcmeRepresentation _rep;
	private AcmeElement _target;

	public RepresentationAction(String type, IWorkbenchPart editor) {
		super(editor);

		if (type.equals(RepresentationCommand.ADD_REP)) {
			setText("Add Representation ...");
			_request = new Request(type);
		} else if (type.equals(RepresentationCommand.REMOVE_REP)) {
			setText("Remove Representation");
			_request = new Request(type);
		}
		setId(type);

	}

	protected RepresentationAction specialize(
		AcmeRepresentation rep,
		AcmeElement target) {
		RepresentationAction action =
			new RepresentationAction(
				RepresentationCommand.REMOVE_REP,
				this.getWorkbenchPart());
		action.setRep(rep);
		action.setTarget(target);
		action.setText(rep.getName());
		return action;
	}

	protected void setRep(AcmeRepresentation rep) {
		_rep = rep;
	}

	protected void setTarget(AcmeElement target) {
		_target = target;
	}

	public IMenuManager buildMenu() {
		MenuManager manager = new MenuManager(getText());
		if (getSelectedObjects().size() != 0) {
			if ((getSelectedObjects().get(0) instanceof ElementEditPart)) {
				ElementEditPart ep =
					(ElementEditPart) getSelectedObjects().get(0);
				if (ep.getModel() instanceof AcmeElement) {
					AcmeElement elem = (AcmeElement) ep.getModel();
					AcmeEnumeration reps = elem.getRepresentations();
					while (reps.moreItems()) {
						AcmeRepresentation rep =
							(AcmeRepresentation) reps.getNextItem();
						manager.add(specialize(rep, elem));
					}
				}
			}
		}
		return manager;
	}

	/**
	 * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		if (_rep != null && _target != null) {
			return true;
		}

		if (getSelectedObjects().size() != 1) {
			return false;
		}

		Object o = getSelectedObjects().get(0);

		boolean validSelection =
			(o instanceof ComponentEditPart)
				|| (o instanceof ConnectorEditPart);

		if (validSelection
			&& _request.getType().equals(RepresentationCommand.REMOVE_REP)) {
			ElementEditPart ep = (ElementEditPart) o;
			boolean canDelete =
				((AcmeElement) ep.getModel()).getRepresentations().getSize()
					!= 0;
			validSelection = validSelection && canDelete;
		}
		return validSelection;
	}

	public void run() {
		ModelCommand command = (ModelCommand) getCommand();
		execute(command);
		if (command != null) {
			try {
				command.checkError();
				if (_request.getType ().equals (RepresentationCommand.ADD_REP) && AcmeStudioPlugin
					.getDefault()
					.getPreferenceStore()
					.getBoolean("DiagramEditor.AutoOpenRepresentations")) {
					MultiPageAcmeEditor mp =
						ASContext.current().getActiveEditor();
					if (mp != null) {
						// synch the tool bar
						IContentOutlinePage cop =
							(IContentOutlinePage) mp.getAdapter(
								IContentOutlinePage.class);
						cop.setFocus();
						mp.swapDiagramEditor(
							_rep.getSystem(),
							(AcmeSystem) _rep.getParent().getParent());
						mp.getDiagramEditor().getActionRegistry().getAction(ZoomToFitAction.ID).run ();
					}
				}

			} catch (Exception e) {
				MessageDialog.openError(
					Display.getDefault().getActiveShell(),
					"Error",
					e.getMessage());
			}
		}
	}

	protected Command getCommand() {
		AcmeElement elem = null;
		if (getSelectedObjects().size() == 0) {
			elem = _target;
		} else {

			EditPart o = (EditPart) getSelectedObjects().get(0);
			elem = (AcmeElement) o.getModel();
		}
		Command command = null;

		if (_request.getType().equals(RepresentationCommand.ADD_REP)) {
			AddRepresentationDialog dialog =
				new AddRepresentationDialog(
					Display.getDefault().getActiveShell(),
					elem);
			if (dialog.open() == IDialogConstants.OK_ID) {
				_rep = dialog.getRepresentation ();
				command =
					new RepresentationCommand(
						RepresentationCommand.ADD_REP,
						dialog.getRepresentation(),
						elem,
						null);
			}
		} else if (
			_request.getType().equals(RepresentationCommand.REMOVE_REP)) {
			if (_rep == null) {
				ListSelectionDialog dialog =
					new ListSelectionDialog(
						Display.getDefault().getActiveShell(),
						elem,
						new IStructuredContentProvider() {
					private final Object[] mto = new Object[0];
					public void dispose() {
					}
					public void inputChanged(Viewer v, Object n, Object o) {
					}
					public Object[] getElements(Object o) {
						Object[] elements = mto;
						if (o instanceof AcmeElement) {
							AcmeElement elem = (AcmeElement) o;
							elements =
								new Object[elem.getRepresentations().getSize()];
							AcmeEnumeration reps = elem.getRepresentations();
							int index = 0;
							while (reps.moreItems()) {
								elements[index++] =
									((AcmeRepresentation) reps.getNextItem())
										.getName();
							}
						}
						return elements;
					}

				}, new LabelProvider() {
					public String getText(Object o) {
						return (String) o;
					}
				}, "Choose representations to remove");
				if (dialog.open() == IDialogConstants.OK_ID) {
					Object[] selections = dialog.getResult();
					if (selections.length == 1) {

						AcmeRepresentation rep =
							elem.getRepresentation((String) selections[0]);
						command =
							new RepresentationCommand(
								RepresentationCommand.REMOVE_REP,
								rep,
								elem,
								null);
					} else if (selections.length > 1) {
						ModelCompoundCommand comm =
							new ModelCompoundCommand("DeleteReps");
						for (int i = 0; i < selections.length; i++) {
							AcmeRepresentation rep =
								elem.getRepresentation((String) selections[i]);
							comm.add(
								new RepresentationCommand(
									RepresentationCommand.REMOVE_REP,
									rep,
									elem,
									null));
						}
						command = comm;
					}

				}
			} else {
				command =
					new RepresentationCommand(
						RepresentationCommand.REMOVE_REP,
						_rep,
						elem,
						null);
			}
		}
		return command;
	}
}
