package AcmeStudio.ui.editors.actions;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;

import AcmeStudio.model.SupertypeAssignmentCommand;
import AcmeStudio.model.TypeAssignmentCommand;
import AcmeStudio.util.IMenuBuilder;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.filters.*;
import acmeLib.elements.*;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class SuperTypeAssignmentMenuAction
    extends SelectionAction
    implements IMenuBuilder {

    private Map _assignActionCache = new HashMap ();
    private Map _unassignActionCache = new HashMap ();
    Request _request;
    private boolean isGroupAction = true;
    private AcmeElementType _superType;
    private AcmeElementType _targetType;

    public SuperTypeAssignmentMenuAction (IWorkbenchPart part, String type) {
        super (part);  
        if (type.equals (TypeAssignmentCommand.ASSIGN_TYPE)) {
            setText ("Add supertype");
        }
        else if (type.equals (TypeAssignmentCommand.ASSIGN_FAMILY)) {
            setText ("Add superfamily");
        }
        else if (type.equals (TypeAssignmentCommand.UNASSIGN_FAMILY)) {
            setText ("Remove superfamily");
        }
        else if (type.equals (TypeAssignmentCommand.UNASSIGN_TYPE)) {
            setText ("Remove supertype");
        }
        setId (type);
        _request = new Request (type);
    }

    /**
     * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
        return !isGroupAction ||
                (getSelectedObjects ().size () != 0 && 
                 getSelectedObjects ().get (0) instanceof AcmeElementType);
    }
    
   

    /**
     * @see AcmeStudio.util.IMenuBuilder#buildMenu()
     */
    public IMenuManager buildMenu() {
        MenuManager manager = new MenuManager(getText());
        if (getSelectedObjects().size() == 0)
            return manager;
        if (getSelectedObjects().get(0) instanceof AcmeElementType) {
            AcmeElementType eType =
                (AcmeElementType) getSelectedObjects().get(0);

            if (_request
                .getType()
                .equals(TypeAssignmentCommand.UNASSIGN_FAMILY)
                || _request.getType().equals(
                    TypeAssignmentCommand.UNASSIGN_TYPE)) {
                AcmeEnumeration types = eType.getSuperTypes().enumerate();
                while (types.moreItems()) {
                    AcmeElementType type =
                        (AcmeElementType) types.getNextItem();
                    if (type.getTypeName ().equals ("system") || type.getTypeName ().equals ("component") ||
                        type.getTypeName ().equals ("connector") ||type.getTypeName ().equals ("port") ||
                        type.getTypeName ().equals ("role") || type.getTypeName ().equals ("group"))
                        continue;
                    SuperTypeAssignmentMenuAction action = null;
                    if (_unassignActionCache.containsKey(type)) {
                        action =
                            (
                                SuperTypeAssignmentMenuAction) _unassignActionCache
                                    .get(
                                type);
                    }
                    else {

                        action =
                            new SuperTypeAssignmentMenuAction(
                                getWorkbenchPart(),
                                (String) _request.getType());
                        action.setIndividualAction();
                        action._superType = type;
                    }
                    action._targetType = eType;
                    action.setText(type.getTypeName());
                    manager.add(action);
                }
            }
            else if (
                _request.getType().equals(TypeAssignmentCommand.ASSIGN_FAMILY)
                    || _request.getType().equals(
                        TypeAssignmentCommand.ASSIGN_TYPE)) {
                AcmeEnumeration types = getAvailableTypes(eType);
                while (types.moreItems()) {
                    AcmeElementType type =
                        (AcmeElementType) types.getNextItem();
                    SuperTypeAssignmentMenuAction action = null;
                    if (_assignActionCache.containsKey(type)) {
                        action =
                            (
                                SuperTypeAssignmentMenuAction) _assignActionCache
                                    .get(
                                type);

                    }
                    else {
                        action =
                            new SuperTypeAssignmentMenuAction(
                                getWorkbenchPart(),
                                (String) _request.getType());
                        action.setIndividualAction();
                        action._superType = type;
                        _assignActionCache.put(type, action);
                    }
                    action._targetType = eType;
                    action.setText(type.getTypeName());
                    manager.add(action);
                }
                if (_request
                    .getType()
                    .equals(TypeAssignmentCommand.ASSIGN_FAMILY)) {
                    ImportFamilyAction action =
                        new ImportFamilyAction(getWorkbenchPart());
                    manager.add(action);
                }
            }

        }
        return manager;
    }
    
    private AcmeEnumeration getAvailableTypes (final AcmeElementType type) {
        AcmeEnumeration types = type.getAvailableTypes ();
        types = types.castToAcmeSetVector().enumerate(new AcmeObjectFilter () {
            public boolean test (AcmeObject o) {
                if (o instanceof AcmeElementType) {
                    AcmeElementType t = (AcmeElementType )o;
                    return !(o.equals (type) ||
                        t.getTypeName().equals ("system") ||
                        t.getTypeName ().equals ("component") ||
                        t.getTypeName ().equals ("connector") ||
                        t.getTypeName ().equals ("port") ||
                        t.getTypeName ().equals ("role") ||
                        t.getTypeName ().equals ("group"));
                }
                return !o.equals (type);
            }    
        });
        if (type instanceof AcmeComponentType) {
            types = types.castToAcmeVector().enumerate(new ComponentTypeEnumFilter ());
        }
        else if (type instanceof AcmeConnectorType) {
            types = types.castToAcmeVector().enumerate (new ConnectorTypeEnumFilter ());
        }
        else if (type instanceof AcmePortType) {
            types = types.castToAcmeVector().enumerate(new PortTypeEnumFilter ());
        }
        else if (type instanceof AcmeRoleType) {
            types = types.castToAcmeVector ().enumerate (new RoleTypeEnumFilter ());
        }
        else if (type instanceof AcmeFamily) {
            types = types.castToAcmeVector ().enumerate (new acmeLib.core.filters.AcmeObjectFilter () {
                public boolean test (AcmeObject o) {
                    return  o instanceof AcmeFamily;
                }
            });   
        }
        else {
            types = new AcmeEnumeration ();   
        }   
        return types;     
    }
    
    private void setIndividualAction () {
        isGroupAction = false;
    }
    
    private Command getCommand () {
        if (!isGroupAction) {
            if (_targetType != null) {
                Command command = new SupertypeAssignmentCommand ((String )_request.getType (), _targetType, _superType, null);
                return command;
            }           
        }
        return null;
    }
    /**
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        if (!isGroupAction) {
            if (_request.getType ().equals (TypeAssignmentCommand.UNASSIGN_FAMILY)) {
                if (MessageDialog.openConfirm(Display.getDefault ().getActiveShell (), "Command confirm", "This command cannot be undone")) {
                    execute (getCommand ());
                }
            }
            else {
                execute(getCommand ());
            }
        }                
    }

}
