/*
 * Created on Mar 18, 2003
 *
 */
package AcmeStudio.ui.editors.analysis.actions;

import java.io.ByteArrayInputStream;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.ui.views.parts.DesignRuleBrowser;
import AcmeStudio.ui.widgets.AcmeErrorTree;
import AcmeStudio.util.ImageManager;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.designRules.AcmeResolutionException;
import acmeLib.designRules.DesignAnalysis;
import acmeLib.elements.AcmeFamily;
import acmeLib.errorManagement.AcmeErrorManager;
import acmeLib.errorManagement.AcmeParseError;
import acmeLib.typeManagement.AcmeTypeManager;

/**
 * @author schmerl
 */
public class ParseAnalysisAction extends Action implements IWorkbenchWindowActionDelegate {

    public static final String PARSE = "$PARSE$";

    private DesignRuleBrowser _browser;
    private AcmeErrorTree _errors;
    private AcmeFamily _scope;
    public DesignAnalysis _designAnalysis;

    public ParseAnalysisAction (DesignRuleBrowser browser, AcmeErrorTree errors, AcmeFamily scope) {
        super ("Parse Design Analysis", ImageManager.getImageDescriptor(ImageManager.ID_PARSE));
        setId (PARSE);
        _browser = browser;
        _errors = errors;
        _scope = scope;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
     */
    public void dispose() {

    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window) {
 
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
    	_designAnalysis = null;
        String analysis = _browser.getText();
        ByteArrayInputStream bis = new ByteArrayInputStream (analysis.getBytes ());
        ArmaniParser.ReInit (bis);
        AcmeErrorManager.clearAllErrors ();
        DesignAnalysis oldDA = (DesignAnalysis )_browser.getCurrentlyDisplayed();
        _scope.removeDesignAnalysis(oldDA);
        _errors.setErrorSource(null);
        DesignAnalysis da = null;
        try  {
        	da = ArmaniParser.DesignAnalysisDeclaration(_scope);
        	if (AcmeErrorManager.getNumErrorsReported() > 0)  {
        		AcmeEnumeration errors = AcmeErrorManager.getErrors ();
        		_errors.setErrorSource(errors);
        		return;
        	}
        }
        catch (ParseException e)  {
        	_errors.setErrorSource (new AcmeParseError (e));
        }
        if (da != null)  {
        	try {
				_scope.addDesignAnalysis(da);
				da.resolveTypeReferences();
				da.resolveInstanceReferences();
			} catch (AcmeIllegalNameException e1) {
				e1.printStackTrace();
			} catch (AcmeResolutionException e) {
				AcmeVector errEnum = new AcmeVector ();
				errEnum.addItem (e.getError ());
				_errors.setErrorSource(errEnum.enumerate ());
				AcmeTypeManager.typecheckingErrorStack.disconnect();
				return;
			}
        	if (!da.typecheck ())  {
        		AcmeVector errors = AcmeTypeManager.typecheckingErrorStack.getCausalErrors();
        		_errors.setErrorSource (errors.enumerate ());
        		AcmeTypeManager.typecheckingErrorStack.disconnect ();
        	}
        }
        _browser.navigateTo (da);
        _designAnalysis = da;
        
		  
        
          
        
    }

}
