/*
 * Created on Jun 11, 2004
 *
 */
package AcmeStudio.ui.editors.diagram;

import org.eclipse.gef.EditDomain;
import org.eclipse.gef.dnd.TemplateTransferDragSourceListener;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.gef.ui.palette.PaletteViewerProvider;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.palette.extension.AcmeStudioPaletteEditPartFactory;


/**
 * @author Bradley Schmerl
 * Created Jun 11, 2004
 */
public class DiagramPaletteViewerProvider extends PaletteViewerProvider {

	private DiagramEditor	_editor;

	/**
	 * @param graphicalViewerDomain
	 */
	public DiagramPaletteViewerProvider(EditDomain graphicalViewerDomain, DiagramEditor editor) {
		super (graphicalViewerDomain);
		_editor = editor;
	}
	
	/**
	 * @param parent
	 * @return
	 * @see org.eclipse.gef.ui.palette.PaletteViewerProvider#createPaletteViewer(org.eclipse.swt.widgets.Composite)
	 **/
	public PaletteViewer createPaletteViewer(Composite parent) {
		final PaletteViewer pViewer = super.createPaletteViewer (parent);
		if (_editor.getAcmeSystem ().getParentRepresentation() != null) {
			_editor.createParentOverview(parent);
			
			// HACK: This listener listens for when the palette viewer is resized (because of a layout or resizing)
			// to reset the size of the paletteviewer to accommodate the parent overview, and also to position 
			// the overview properly.
			Listener resizeListener = new Listener () {

				public void handleEvent(Event event) {
					pViewer.getControl ().removeListener (SWT.Resize, this);
					Rectangle area = pViewer.getControl ().getBounds();
					Rectangle origOverviewArea = _editor.getOverviewParent ().getBounds();
					if (origOverviewArea.width != area.width || Math.abs (origOverviewArea.height - area.height/2) > 2) {
						int height = area.height / 3;
						area.height = 2 * height;
						pViewer.getControl().setBounds (area);
						Rectangle overviewArea = new Rectangle (area.x, area.y + area.height, area.width, height);
						_editor.getOverviewParent().setBounds (overviewArea);
					}
					pViewer.getControl ().addListener (SWT.Resize, this);
				}
				
			};
			pViewer.getControl ().addListener (SWT.Resize, resizeListener);
			
		}
		return pViewer;
	}
	private IMenuListener _menuListener;
	
	public void configurePaletteViewer (PaletteViewer viewer) {
		viewer.setContextMenu(new AcmePaletteContextMenuProvider (viewer, _editor.getPaletteManager()));
	    viewer.setCustomizer(null);
	    viewer.setEditPartFactory(new AcmeStudioPaletteEditPartFactory ());

	}
	
	public  void hookPaletteViewer (PaletteViewer viewer) {
		super.hookPaletteViewer (viewer);
/*		final CopyTemplateAction copy = (CopyTemplateAction )_editor.getActionRegistry ().getAction (GEFActionConstants.COPY);
		viewer.addSelectionChangedListener (copy);
		if (_menuListener == null) 
			_menuListener = new IMenuListener () {
				public void menuAboutToShow (IMenuManager manager) {
					manager.appendToGroup (GEFActionConstants.GROUP_COPY, copy);
				}
			};
		viewer.getContextMenu ().addMenuListener (_menuListener);
		((IEditorSite )_editor.getSite ()).getActionBars ().setGlobalActionHandler (GEFActionConstants.COPY, copy);*/
		initializePaletteViewer (viewer);
	}

	/**
	 * @param paletteViewer
	 */
	public void initializePaletteViewer(PaletteViewer viewer) {
		if (_editor.getPaletteManager () != null) {
//			_editor.getPaletteManager ().reset ();
			viewer.setActiveTool(null);
			viewer.setPaletteRoot (_editor.getPaletteManager ().getPaletteRoot());
		}
		viewer.enableVerticalScrollbar(true);
		viewer.addDragSourceListener(new TemplateTransferDragSourceListener (viewer));
	}

}
