/*
 * Created on Jul 20, 2003
 *
 */
package AcmeStudio.ui.editors.diagram;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.SelectionEditPolicy;

import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl
 * Interface that allows plugins to add items on the diagram
 *
 */
public interface IElementFactory {
	/**
	 * Returns the models that are extensions of this system. Elements in the
	 * list should extend AcmeElement.
	 * @param parent The system containing extensions
	 * @return the list of models that a plugin has extended the system with
	 */
    public List getModels(AcmeSystem parent);
    
    /**
     * Returns a new edit part associated with the object, which will be one
     * of the elements returned from getModels
     * @param child The element 
     * @return A new editpart to add to the diagram
     */
    public EditPart getEditPart(Object child);
    
    /**
     * Gets the selection policy for the extension
     * @param target the object
     * @return The selection edit policy
     */
    public SelectionEditPolicy getSelectionPolicy (Object target);
    
	/**
	 * Returns the command to delete the model from the diagram
	 * @param model The model to delete
	 * @return the command to use to delete the model
	 */
	public Command getDeleteCommand(Object model);
}
