package AcmeStudio.ui.editors.diagram.actions;

import org.eclipse.gef.commands.Command;

import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.util.AcmeStudioChangeManager;
import acmeLib.elements.AcmeElement;
import acmeLib.representations.AcmeBinding;
import acmeLib.representations.AcmeBindingsMap;
import acmeLib.representations.AcmeRepresentation;

/**
 * @author schmerl
 *
 */
public class AddBindingCommand extends Command {

	AcmeElement _inner = null;
	AcmeElement _outer = null;
	SystemEditPart _editPart = null;
	private AcmeBinding	_newBinding;
	
	/**
	 * Constructor for AddBindingCommand.
	 */
	public AddBindingCommand(AcmeElement outer, AcmeElement inner, SystemEditPart editPart) {
		super("Create Binding");
		_inner = inner;
		_outer = outer;
		_editPart = editPart;
	}

	

	/**
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		return (_inner != null && _outer != null && super.canExecute());
	}

	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
        setLabel ("Bind " + _outer.getName () + " to " + _inner.getName ());
		doExecute ();         
	}

	/**
	 * 
	 */
	private void doExecute() {
		_newBinding = new AcmeBinding (_outer, _inner);
		AcmeRepresentation rep = _inner.getParentRepresentation();
		AcmeBindingsMap bm = (AcmeBindingsMap )rep.getAbstractionMap();
        if (bm == null) {
            rep.setAbstractionMap(new AcmeBindingsMap ());
            bm = (AcmeBindingsMap )rep.getAbstractionMap();
        }
		bm.addBinding (_newBinding);
        bm.setParentRep (rep);
        AcmeStudioChangeManager.globalChangeManager.onAddBinding (rep, _newBinding, this);
		_editPart.setModel (rep.getSystem());
        _editPart.refresh();
	}



	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		doExecute ();
	}

	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		AcmeRepresentation rep = _inner.getParentRepresentation();
		AcmeBindingsMap bm = (AcmeBindingsMap )rep.getAbstractionMap();
		bm.removeBinding(_newBinding);
		AcmeStudioChangeManager.globalChangeManager.onRemoveBinding(_inner.getParentRepresentation(), _newBinding, this);
		
		_editPart.refresh ();
	}



	/**
	 * @param element
	 */
	public void setInner(AcmeElement element) {
		_inner = element;
	}



	/**
	 * @param element
	 */
	public void setOuter(AcmeElement element) {
		_outer = element;
	}

}
