package AcmeStudio.ui.editors.diagram.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;

import AcmeStudio.ui.editors.diagram.parts.PortEditPart;
import AcmeStudio.ui.editors.diagram.styles.*;
import AcmeStudio.ui.editors.diagram.styles.constants.*;
import AcmeStudio.util.Helper;
import AcmeStudio.util.IMenuBuilder;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmePort;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 */
public class AdjustPortAlignmentAction
    extends SelectionAction
    implements IMenuBuilder {

    private Request _request;

    private boolean isGroupAction = true;

    private AcmeComponent _acmeComponent;
    private String _portName;
    private String _portToExchange = null;

    public AdjustPortAlignmentAction(
        IWorkbenchPart editor,
        AcmeComponent acmeComponent,
        String portName,
        String type) {
        super(editor);
        if (type.equals(AdjustPortAlignmentCommand.ADJUST)) {
            setText("Adjust Port Alignment");
        }
        setId(type);
        _request = new Request(type);
        _acmeComponent = acmeComponent;
        _portName = portName;
    }

    public AdjustPortAlignmentAction(
        IWorkbenchPart editor,
        AcmeComponent acmeComponent,
        String portName,
        String portToExchange,
        String type) {
        this(editor, acmeComponent, portName, type);
        _portToExchange = portToExchange;
    }

    public AdjustPortAlignmentAction(IEditorPart editor, String type) {
        super(editor);
        if (type.equals(AdjustPortAlignmentCommand.ADJUST)) {
            setText("Adjust Port Alignment");
        }
        setId(type);
        _request = new Request(type);
    }

    /**
     * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
        if (isGroupAction) {
            return validateSelection();
        }

        return calculateElementEnabled();
    }

    protected boolean calculateElementEnabled() {
        if (_acmeComponent == null)
            return false;
        Vector v = Helper.getChildrenByOrder(_acmeComponent);
        if (v.size() < 2)
            return false;

        ArrayList portRules = new ArrayList();
        ArrayList ports = new ArrayList();
        int index = -1;
        int i = 0;
        for (i = 0; i < v.size(); i++) {
            AcmePort port = (AcmePort) v.get(i);
            if (port.getName().equals(_portName)) {
                index = i;
            }
            AcmeEnumeration aeTypes = port.getDeclaredTypes();
            ComponentVisualization cv =
                (ComponentVisualization) VisualStyleRegistry
                    .getVisualization(_acmeComponent)
                    .copy(_acmeComponent);
            PlacementRule rule = null;
            List rules = cv.getPortPlacementRules();
            boolean foundRule = false;
            while (!foundRule && aeTypes.moreItems()) {
                String portType =
                    ((AcmeElementType) aeTypes.getNextItem())
						    .getTypeName();
                Iterator iter = rules.iterator();
                while (iter.hasNext()) {
                    PlacementRule obj =
                        (PlacementRule) iter.next();
                    if (obj._type.equals("port")
                        || obj._type.equals(portType)) {
                        foundRule = true;
                        rule = obj;
                        break;
                    }
                }
            }
            portRules.add(rule);
            ports.add(port);
        }

        if (_request.getType().equals(AdjustPortAlignmentCommand.SWAP)) {
            PlacementRule rule1 =
                (PlacementRule) portRules.get(index);
            AcmePort port = (AcmePort) ports.get(index);
            if (Helper.getY(port) != null)
                return false;
            if (rule1 == null)
                return false;
            for (i = 0; i < ports.size(); i++) {
                port = (AcmePort) ports.get(i);
                if (Helper.getY(port) != null)
                    continue;
                if (!port.getName().equals(_portToExchange))
                    continue;
                PlacementRule rule2 =
                    (PlacementRule) portRules.get(i);
                if (rule2 == null)
                    continue;
                if ((rule1._alignment
                    == AlignmentOptions.LEFT
                    || rule1._side
                        == SideOptions.LEFT)
                    && (rule2._alignment
                        == AlignmentOptions.LEFT
                        || rule2._side
                            == SideOptions.LEFT)
                    || (rule1._alignment
                        == AlignmentOptions.RIGHT
                        || rule1._side
                            == SideOptions.RIGHT)
                    && (rule2._alignment
                        == AlignmentOptions.RIGHT
                        || rule2._side
                            == SideOptions.RIGHT)) {
                    return true;
                }
                if ((rule1._alignment
                    == AlignmentOptions.LEFT
                    || rule1._side
                        == SideOptions.LEFT)
                    && (rule2._alignment
                        == AlignmentOptions.LEFT
                        || rule2._side
                            == SideOptions.LEFT)
                    || (rule1._alignment
                        == AlignmentOptions.RIGHT
                        || rule1._side
                            == SideOptions.RIGHT)
                    && (rule2._alignment
                        == AlignmentOptions.RIGHT
                        || rule2._side
                            == SideOptions.RIGHT)) {
                    return true;
                }
                return false;
            }
        }

        if (_request.getType().equals(AdjustPortAlignmentCommand.MOVEUP)) {
            PlacementRule rule1 =
                (PlacementRule) portRules.get(index);
            AcmePort port = (AcmePort) ports.get(index);
            if (Helper.getY(port) != null)
                return false;
            if (rule1 == null)
                return false;
            for (i = 0; i < index; i++) {
                PlacementRule rule2 =
                    (PlacementRule) portRules.get(i);
                port = (AcmePort) ports.get(i);
                if (Helper.getY(port) != null)
                    continue;
                if (rule2 == null)
                    continue;
                if ((rule1._alignment
                    == AlignmentOptions.LEFT
                    || rule1._side
                        == SideOptions.LEFT)
                    && (rule2._alignment
                        == AlignmentOptions.LEFT
                        || rule2._side
                            == SideOptions.LEFT)
                    || (rule1._alignment
                        == AlignmentOptions.RIGHT
                        || rule1._side
                            == SideOptions.RIGHT)
                    && (rule2._alignment
                        == AlignmentOptions.RIGHT
                        || rule2._side
                            == SideOptions.RIGHT)) {
                    _portToExchange = ((AcmePort) ports.get(i)).getName();
                    return true;
                }
            }
            return false;
        }

        if (_request.getType().equals(AdjustPortAlignmentCommand.MOVEDOWN)) {
            AcmePort port = (AcmePort) ports.get(index);
            if (Helper.getY(port) != null)
                return false;

            PlacementRule rule1 =
                (PlacementRule) portRules.get(index);
            if (rule1 == null)
                return false;
            for (i = index + 1; i < ports.size(); i++) {
                PlacementRule rule2 =
                    (PlacementRule) portRules.get(i);
                port = (AcmePort) ports.get(i);
                if (Helper.getY(port) != null)
                    continue;
                if (rule2 == null)
                    continue;
                if ((rule1._alignment
                    == AlignmentOptions.LEFT
                    || rule1._side
                        == SideOptions.LEFT)
                    && (rule2._alignment
                        == AlignmentOptions.LEFT
                        || rule2._side
                            == SideOptions.LEFT)
                    || (rule1._alignment
                        == AlignmentOptions.RIGHT
                        || rule1._side
                            == SideOptions.RIGHT)
                    && (rule2._alignment
                        == AlignmentOptions.RIGHT
                        || rule2._side
                            == SideOptions.RIGHT)) {
                    _portToExchange = ((AcmePort) ports.get(i)).getName();
                    return true;
                }
            }
            return false;
        }
        if (_request.getType().equals(AdjustPortAlignmentCommand.MOVELEFT)) {
            AcmePort port = (AcmePort) ports.get(index);
            if (Helper.getX(port) != null)
                return false;

            PlacementRule rule1 =
                (PlacementRule) portRules.get(index);
            if (rule1 == null)
                return false;
            for (i = 0; i < index; i++) {
                port = (AcmePort) ports.get(i);
                if (Helper.getX(port) != null)
                    continue;

                PlacementRule rule2 =
                    (PlacementRule) portRules.get(i);
                if (rule2 == null)
                    continue;
                if (((rule1._alignment
                    == AlignmentOptions.TOP
                    || rule1._side
                        == SideOptions.TOP)
                    && (rule2._alignment
                        == AlignmentOptions.TOP
                        || rule2._side
                            == SideOptions.TOP)
                    || (rule1._alignment
                        == AlignmentOptions.BOTTOM
                        || rule1._side
                            == SideOptions.BOTTOM)
                    && (rule2._alignment
                        == AlignmentOptions.BOTTOM
                        || rule2._side
                            == SideOptions.BOTTOM))
                    && !((rule1._alignment
                        == AlignmentOptions.LEFT
                        || rule1._side
                            == SideOptions.LEFT)
                        && (rule2._alignment
                            == AlignmentOptions.LEFT
                            || rule2._side
                                == SideOptions.LEFT)
                        || (rule1._alignment
                            == AlignmentOptions.RIGHT
                            || rule1._side
                                == SideOptions.RIGHT)
                        && (rule2._alignment
                            == AlignmentOptions.RIGHT
                            || rule2._side
                                == SideOptions
                                    .RIGHT))) {
                    _portToExchange = ((AcmePort) ports.get(i)).getName();
                    return true;
                }
            }
            return false;
        }
        if (_request.getType().equals(AdjustPortAlignmentCommand.MOVERIGHT)) {
            AcmePort port = (AcmePort) ports.get(index);
            if (Helper.getX(port) != null)
                return false;

            PlacementRule rule1 =
                (PlacementRule) portRules.get(index);
            if (rule1 == null)
                return false;
            for (i = index + 1; i < ports.size(); i++) {
                port = (AcmePort) ports.get(i);
                if (Helper.getX(port) != null)
                    continue;

                PlacementRule rule2 =
                    (PlacementRule) portRules.get(i);
                if (rule2 == null)
                    continue;
                if (((rule1._alignment
                    == AlignmentOptions.TOP
                    || rule1._side
                        == SideOptions.TOP)
                    && (rule2._alignment
                        == AlignmentOptions.TOP
                        || rule2._side
                            == SideOptions.TOP)
                    || (rule1._alignment
                        == AlignmentOptions.BOTTOM
                        || rule1._side
                            == SideOptions.BOTTOM)
                    && (rule2._alignment
                        == AlignmentOptions.BOTTOM
                        || rule2._side
                            == SideOptions.BOTTOM))
                    && !((rule1._alignment
                        == AlignmentOptions.LEFT
                        || rule1._side
                            == SideOptions.LEFT)
                        && (rule2._alignment
                            == AlignmentOptions.LEFT
                            || rule2._side
                                == SideOptions.LEFT)
                        || (rule1._alignment
                            == AlignmentOptions.RIGHT
                            || rule1._side
                                == SideOptions.RIGHT)
                        && (rule2._alignment
                            == AlignmentOptions.RIGHT
                            || rule2._side
                                == SideOptions
                                    .RIGHT))) {
                    _portToExchange = ((AcmePort) ports.get(i)).getName();
                    return true;
                }
            }
            return false;
        }

        return false;
    }

    protected void setIndividualAction() {
        isGroupAction = false;
    }

    public IMenuManager buildMenu() {
        MenuManager manager = new MenuManager(getText());
        if (getSelectedObjects().size() == 1) {
            if (getSelectedObjects().get(0) instanceof PortEditPart) {
                PortEditPart ep = (PortEditPart) getSelectedObjects().get(0);
                if (ep.getModel() instanceof AcmePort) {
                    AcmePort element = (AcmePort) ep.getModel();
                    AcmeComponent comp = (AcmeComponent) element.getParent();
                    String portName = element.getName();
                    if (_request
                        .getType()
                        .equals(AdjustPortAlignmentCommand.ADJUST)) {
                        AdjustPortAlignmentAction action =
                            new AdjustPortAlignmentAction(
                                getWorkbenchPart(),
                                comp,
                                portName,
                                AdjustPortAlignmentCommand.MOVEUP);
                        action.setIndividualAction();
                        action.setText("Move Up");
                        manager.add(action);
                        action =
                            new AdjustPortAlignmentAction(
                                getWorkbenchPart(),
                                comp,
                                portName,
                                AdjustPortAlignmentCommand.MOVEDOWN);
                        action.setIndividualAction();
                        action.setText("Move Down");
                        manager.add(action);
                        action =
                            new AdjustPortAlignmentAction(
                                getWorkbenchPart(),
                                comp,
                                portName,
                                AdjustPortAlignmentCommand.MOVELEFT);
                        action.setIndividualAction();
                        action.setText("Move Left");
                        manager.add(action);
                        action =
                            new AdjustPortAlignmentAction(
                                getWorkbenchPart(),
                                comp,
                                portName,
                                AdjustPortAlignmentCommand.MOVERIGHT);
                        action.setIndividualAction();
                        action.setText("Move Right");
                        manager.add(action);
                    }
                }
            }
        } else if (getSelectedObjects().size() == 2) {
            if (getSelectedObjects().get(0) instanceof PortEditPart
                && getSelectedObjects().get(1) instanceof PortEditPart) {
                PortEditPart ep0 = (PortEditPart) getSelectedObjects().get(0);
                PortEditPart ep1 = (PortEditPart) getSelectedObjects().get(1);
                if (ep0.getModel() instanceof AcmePort
                    && ep1.getModel() instanceof AcmePort) {
                    AcmePort element0 = (AcmePort) ep0.getModel();
                    AcmePort element1 = (AcmePort) ep1.getModel();
                    AcmeComponent comp = (AcmeComponent) element0.getParent();
                    if (!comp.equals(element1.getParent()))
                        return manager;
                    String portName0 = element0.getName();
                    String portName1 = element1.getName();
                    if (_request
                        .getType()
                        .equals(AdjustPortAlignmentCommand.ADJUST)) {
                        AdjustPortAlignmentAction action =
                            new AdjustPortAlignmentAction(
                                getWorkbenchPart(),
                                comp,
                                portName0,
                                portName1,
                                AdjustPortAlignmentCommand.SWAP);
                        action.setIndividualAction();
                        action.setText("Swap");
                        manager.add(action);
                    }
                }
            }
        }
        return manager;
    }

    private boolean validateSelection() {
        if (getSelectedObjects().size() == 1) {
            if (getSelectedObjects().get(0) instanceof PortEditPart) {
                return true;
            }
        } else if (getSelectedObjects().size() == 2) {
            if (getSelectedObjects().get(0) instanceof PortEditPart
                && getSelectedObjects().get(1) instanceof PortEditPart) {
                return true;
            }
        }
        return false;
    }

    /**
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        if (!isGroupAction) {
            execute(getCommand());
        }
    }

    protected Command getCommand() {
        if (!isGroupAction) {
            Command command =
                new AdjustPortAlignmentCommand(
                    (String) _request.getType(),
                    _acmeComponent,
                    _portName,
                    _portToExchange,
                    null);
            return command;
        }
        return null;
    }

}
