package AcmeStudio.ui.editors.diagram.actions;

import AcmeStudio.model.ModelCommand;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeComponent;

/**
 * @author yh
 *
 */
public class AdjustPortAlignmentCommand extends ModelCommand {

    private static final String MOVEUP_REQUEST = "$MoveUpPort$";
    private static final String MOVEDOWN_REQUEST = "$MoveDownPort$";
    private static final String MOVELEFT_REQUEST = "$MoveLeftPort$";
    private static final String MOVERIGHT_REQUEST = "$MoveRightPort$";
    private static final String SWAP_REQUEST = "$SwapPorts$";
    private static final String ADJUST_REQUEST = "$AdjustPortAlignment$";

    public static final String MOVEUP = MOVEUP_REQUEST;
    public static final String MOVEDOWN = MOVEDOWN_REQUEST;
    public static final String MOVELEFT = MOVELEFT_REQUEST;
    public static final String MOVERIGHT = MOVERIGHT_REQUEST;
    public static final String SWAP = SWAP_REQUEST;
    
    public static final String ADJUST = ADJUST_REQUEST;

    private AcmeComponent _acmeComponent;
    private Object _announcer;
    private String _portName;
    private String _portToExchange;
    private String _commandType;

    public AdjustPortAlignmentCommand(
        String commandType,
        AcmeComponent acmeComponent,
        String portName,
        String portToExchange,
        Object announcer) {
        super(commandType);
        _announcer = announcer;
        _portName = portName;
        _portToExchange = portToExchange;
        _acmeComponent = acmeComponent;
        _commandType = commandType;
        if (_commandType.equals(MOVEUP)) {
            this.setLabel("Move Up "+portName);
        } else if (_commandType.equals(MOVEDOWN)) {
            this.setLabel("Move Down "+portName);
        } else if (_commandType.equals(MOVELEFT)) {
            this.setLabel("Move Left "+portName);
        } else if (_commandType.equals(MOVERIGHT)) {
            this.setLabel("Move Right "+portName);
        }

    }

    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute() {
        return (
            _acmeComponent.getPort(_portName) != null
                && _acmeComponent.getPort(_portToExchange) != null);
    }

    /**
     * @see org.eclipse.gef.commands.Command#canUndo()
     */
    public boolean canUndo() {
        return true;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        Helper.exchangePortOrder(_acmeComponent, _portName, _portToExchange);
        AcmeStudioChangeManager.globalChangeManager.onAlignPorts(
            _acmeComponent,
            _announcer);
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
    }

    /**
     * This doesn't really undo instantiations
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        Helper.exchangePortOrder(_acmeComponent, _portToExchange, _portName);
        AcmeStudioChangeManager.globalChangeManager.onAlignPorts(
            _acmeComponent,
            _announcer);
    }
}
