/*
 * Created on Aug 9, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.actions;

import java.util.Iterator;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.util.ImageManager;


/**
 * @author Bradley Schmerl
 * Created Aug 9, 2004
 * Changs the ordering of elements on the canvas
 */
public class ChangeOrderAction extends SelectionAction {

	public static final String MOVE_FORWARD = "$MoveForw$";
	public static final String MOVE_BACK = "$MoveBack$";
	public static final String MOVE_TO_FRONT = "$MoveFront$";
	public static final String MOVE_TO_BACK = "$Move2Back$";
	/**
	 * Creates the acion
	 * @param part
	 * @param id The command to create. One of MOVE_FORWARD, MOVE_BACK, MOVE_TO_FRONT, MOVE_TO_BACK.
	 */
	public ChangeOrderAction(IWorkbenchPart part, String id) {
		super (part);
		setId (id);
		if (MOVE_FORWARD.equals (id)) {
			setText ("Move forward");
			setToolTipText("Move selected items forward");
			setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_MOVE_FORWARD));
		}
		else if (MOVE_BACK.equals (id)) {
			setText("Move back");
			setToolTipText("Move selected items back");
			setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_MOVE_BACK));
		}
		else if (MOVE_TO_BACK.equals (id)) {
			setText ("Move to back");
			setToolTipText("Move selected items to the back");
			setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_MOVE_TO_BACK));
		}
		else if (MOVE_TO_FRONT.equals (id)) {
			setText ("Move to front");
			setToolTipText("Move selected items to the front");
			setImageDescriptor (ImageManager.getImageDescriptor(ImageManager.ID_MOVE_TO_FRONT));
		}
	}

	/**
	 * @return something other than the system is selected
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 **/
	protected boolean calculateEnabled() {
		return !this.getSelectedObjects().isEmpty () && !(this.getSelectedObjects().size() == 1 && this.getSelectedObjects().get(0) instanceof SystemEditPart);
	}
	
	
	/**
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 **/
	public void run() {
		DiagramEditor editor = (DiagramEditor )getWorkbenchPart();
		SystemEditPart sEP = (SystemEditPart )editor.getRootEditPart ();
		Iterator i = getSelectedObjects().iterator ();
		String id = getId();
		for (Iterator iter = getSelectedObjects ().iterator (); iter.hasNext ();) {
			EditPart part = (EditPart )iter.next();
			
			if (MOVE_FORWARD.equals (id)) {
				sEP.moveForward (part);
			}
			else if (MOVE_BACK.equals (id)) {
				sEP.moveBack (part);
			}
			else if (MOVE_TO_BACK.equals (id)) {
				sEP.moveToBack (part);
			}
			else if (MOVE_TO_FRONT.equals (id)) {
				sEP.moveToFront (part);
			}
		}
		sEP.refresh ();
	}
}
