/*
 * Created on Jul 6, 2004
 *  
 */
package AcmeStudio.ui.editors.diagram.actions;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.geometry.Rectangle;

import AcmeStudio.model.ModelCommand;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.ruler.commands.ChangeGuideCommand;
import AcmeStudio.ui.editors.diagram.ruler.model.DiagramGuide;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;
/**
 * @author Bradley Schmerl Created Jul 6, 2004
 */
public class CloneCommand extends ModelCommand {

	
	private List					_elements;
	private AcmeElement				_parent;
	private List					_newElements;
	private Map						_bounds;
	private ChangeGuideCommand		_vGuideCommand, _hGuideCommand;
	private DiagramGuide			_hGuide, _vGuide;
	private int						_hAlignment, _vAlignment;

	/**
	 */
	public CloneCommand() {
		super ("Clone elements");
		_newElements = new LinkedList ();
		_elements = new LinkedList ();
	}

	/**
	 * @param parent
	 */
	public void setParent(AcmeElement parent) {
		_parent = parent;
	}

	/**
	 * @param element
	 * @param newBounds
	 */
	public void addPart(AcmeElement element, Rectangle newBounds) {
		_elements.add (element);
		if (_bounds == null) _bounds = new HashMap ();
		_bounds.put (element, newBounds);
	}

	/**
	 * @param guide
	 * @param alignment
	 * @param isHorizontal
	 */
	public void setGuide(DiagramGuide guide, int alignment, boolean isHorizontal) {
		if (isHorizontal) {
			_hGuide = guide;
			_hAlignment = alignment;
		}
		else {
			_vGuide = guide;
			_vAlignment = alignment;
		}
	}

	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		Map connectionPartMap = new HashMap ();
		List newConnections = new LinkedList ();
		Iterator i = _elements.iterator ();
		AcmeElement part = null;
		while (i.hasNext ()) {
			part = (AcmeElement )i.next ();
			if (_bounds != null && _bounds.containsKey (part)) {
				clonePart (part, part.getParent(), (Rectangle )_bounds.get (part));
			}
		}
		if (_elements.isEmpty ()) return;
	}

	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		execute ();

	}

	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		if (_hGuideCommand != null) {
			_hGuideCommand.undo ();
		}
		if (_vGuideCommand != null) 
			_vGuideCommand.undo ();

		Iterator i = _newElements.iterator();
		while (i.hasNext ()) {
			AcmeElement element = (AcmeElement )i.next ();
			removePart (element);
		}
	}

	/**
	 * @param element
	 */
	private void removePart(AcmeElement element) {
		AcmeElement parent = element.getParent ();
		if (element instanceof AcmeComponent) {
			AcmeSystem system = (AcmeSystem )parent;
			AcmeComponent component = (AcmeComponent )element;
			system.removeComponent (component);
			AcmeStudioChangeManager.globalChangeManager.onRemoveComponent (system, component, null);
		}
		else if (element instanceof AcmeConnector) {
			AcmeConnector connector = (AcmeConnector )element;
			AcmeSystem system = (AcmeSystem )parent;
			system.removeConnector (connector);
			AcmeStudioChangeManager.globalChangeManager.onRemoveConnector (system, connector, null);
		}
		else if (element instanceof AcmePort) {
			AcmePort port = (AcmePort )element;
			AcmeComponent comp = (AcmeComponent )parent;
			comp.removePort (port);
			AcmeStudioChangeManager.globalChangeManager.onRemovePort (comp, port, null);
		}
		else if (element instanceof AcmeRole) {
			AcmeRole role = (AcmeRole )element;
			AcmeConnector conn = (AcmeConnector )parent;
			AcmeStudioChangeManager.globalChangeManager.onRemoveRole (conn, role, null);
		}
	}

	protected void clonePart(AcmeElement oldPart, AcmeElement newParent, Rectangle newBounds) {
		AcmeElement newElement = null;
		AcmeElement oldElement = (AcmeElement )oldPart;
		newElement = oldElement.copy ();
		newElement.setParent (null);
		SystemView.removeVisualization (newElement);
		try {
			newElement.setName (Helper.getUniqueName (newParent, newElement.getName ()));
		}
		catch (AcmeIllegalNameException e) {
			// Should never happen
		}
	
		if (newBounds != null) {
			Helper.setSize (newElement, newBounds.getSize ());
			SystemView.setLocation (newElement,  newBounds.getCenter ());
		}
		try {
			if (newElement instanceof AcmeComponent) {
				AcmeComponent component = (AcmeComponent )newElement;
				((AcmeSystem )newParent).addComponent (component);
				AcmeStudioChangeManager.globalChangeManager.onAddComponent ((AcmeSystem )newParent, component, null);
				_newElements.add (newElement);
			}
			else if (newElement instanceof AcmeConnector) {
				AcmeConnector connector = (AcmeConnector )newElement;
				AcmeSystem parent = (AcmeSystem )newParent;
				parent.addConnector (connector);
				AcmeStudioChangeManager.globalChangeManager.onAddConnector (parent, connector, null);
				_newElements.add (newElement);
			}
			else if (newElement instanceof AcmePort) {
				AcmePort port = (AcmePort )newElement;
				AcmeComponent component = (AcmeComponent )newParent;
				component.addPort (port);
				AcmeStudioChangeManager.globalChangeManager.onAddPort (component, port, null);
			}
			else if (newElement instanceof AcmeRole) {
				AcmeRole role = (AcmeRole )newElement;
				AcmeConnector connector = (AcmeConnector )newParent;
				connector.addRole (role);
				AcmeStudioChangeManager.globalChangeManager.onAddRole (connector, role, null);
			}
		}
		catch (AcmeIllegalNameException e1) {
			this._errorReason = e1;
		}
	}

	/**
	 * @return
	 */
	public List getParts() {
		return _elements;
	}
}