package AcmeStudio.ui.editors.diagram.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import AcmeStudio.core.ASContext;
import AcmeStudio.model.ModelHelper;
import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.ui.editors.diagram.parts.ConnectorEditPart;
import AcmeStudio.ui.editors.diagram.parts.PortEditPart;
import AcmeStudio.ui.editors.diagram.styles.AlertNotificationDialog;
import AcmeStudio.ui.editors.diagram.styles.SystemVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.util.Helper;
import AcmeStudio.wizards.ConnectionPatternTypePage;
import AcmeStudio.wizards.ConnectionPatternWizard;
import AcmeStudio.wizards.connection.PortConnectionRule;
import AcmeStudio.wizards.connection.PortConnectionRuleKey;
import AcmeStudio.wizards.connection.PortRoleAssociation;
import acmeLib.AcmeType;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.elements.*;

public class ConnectElementsCommand extends AcmeStudio.model.ConnectElementsCommand
{
    /**
     * This static member holds any rule that has been deemed sticky
     */
    public static PortConnectionRule stickyRule;
    private EditPart editPart = null;
    private int numberMatches;
    List matchRule = null;
    private boolean fStickyRuleNoMatch = false;
    private ConnectionPatternWizard NRwizard;
	private EditPart source;
	private EditPart target;
	
    public ConnectElementsCommand(EditPart editPart)
    {
        super();
        this.editPart = editPart;
        _announcer = editPart.getRoot().getViewer();
    }

    public synchronized void execute()
    {
        AlertNotificationDialog alertDialog;
        clearExtraneousObjects();

        //If ports were selected, get the parent component
        //This data is needed to determine which rules the selection
        //matches
        getParentComponentData();

        //This should only be called from ElementNodeEditPolicy,
        //where the connection would have been generated with a palette
        //based connection tool (it might be the null rule, indictating a need
        //for the wizard though

        if (_rule == null)
        {
            //this method mutates _rule
            checkForMatchingRule();
        }

        if (_rule != null) //If there is a rule to be used
        {
            if (ConnectElementsCommand.stickyRule == null)
            {
                useRuleToCreateConnection();    
            }
            else
            {
                //ensure the stickyRule is valid
                //If the source is a connector, make sure the target part of the rule is not N/A
                if (getSource() instanceof ConnectorEditPart && !_rule.getAssociations()[1].getRoleType().equals("N/A"))
                    useRuleToCreateConnection();
                //If the target is a connector, make sure the source part of the rule is not N/A
                else if (getTarget() instanceof ConnectorEditPart && !_rule.getAssociations()[0].getRoleType().equals("N/A"))
                    useRuleToCreateConnection();
                //If the connection does not include a connector, make sure there is no N/A in the rule
                else if (!(getSource() instanceof ConnectorEditPart) && !(getTarget() instanceof ConnectorEditPart) &&
                        	!_rule.getAssociations()[0].getRoleType().equals("N/A") && !_rule.getAssociations()[1].getRoleType().equals("N/A"))
                    useRuleToCreateConnection();
                else
                {
                    alertDialog = new AlertNotificationDialog(Display.getDefault().getActiveShell(), 
            		"The sticky rule you are using is invalid for the attempted connection.");
                    int retCode = alertDialog.open();
                }
            }
        }
        else
        {
            if (numberMatches > 1)
            {
                disambiguateAndCreateConnection();
                return;
            }
            else
            {
                //No rules matched the selection -- Inform the user
                
                if (!fStickyRuleNoMatch)
                {
                    StringBuffer msg = new StringBuffer("There are no rules which match the objects you selected.  Please try your connection again.");
                    if (VisualStyleRegistry.getSysVis(getSystem()).getAllowNewRule())
                        msg.append("\nIf you would like to create a new rule, please use the menu choice at the top of the screen.");
	                alertDialog = new AlertNotificationDialog(Display.getDefault().getActiveShell(), msg.toString() );
	                
                }
                else
                {
                    alertDialog = new AlertNotificationDialog(Display.getDefault().getActiveShell(), 
            		"There are no rules which match the rule that you have marked to use as sticky.");
                    fStickyRuleNoMatch = false;
                }
                int retCode = alertDialog.open();
            }
        }

    }

    /**
     * @param matchRule
     */
    private void disambiguateAndCreateConnection()
    {
        NRwizard = new ConnectionPatternWizard(getSystem(), matchRule, getPortTypeArray(), getComponentTypeArray(), false);
        
        WizardDialog dialog = new WizardDialog(Display.getDefault()
                .getActiveShell(), NRwizard) {
        	
			/**
			 * @param parent
			 * @see org.eclipse.jface.wizard.WizardDialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
			 **/
			protected void createButtonsForButtonBar(Composite parent) {
				super.createButtonsForButtonBar (parent);
				getButton (IDialogConstants.FINISH_ID).setText("Connect");
			}
        };
        dialog.create();
        dialog.open();
        if (!(IDialogConstants.CANCEL_ID == dialog.getReturnCode()))
        {
            // if the user didn't cancel

            //Build the rule out of the different parts on the dialog
            PortConnectionRule userChosen = new PortConnectionRule();
            userChosen.setConnectorType(NRwizard.getTypePage().selectionConn);
            userChosen.addAssociation(new PortRoleAssociation(
                    NRwizard.getTypePage().selectionPorts[0], NRwizard.getTypePage().selectionRoles[0], "r"));
            userChosen.addAssociation(new PortRoleAssociation(
                    NRwizard.getTypePage().selectionPorts[1], NRwizard.getTypePage().selectionRoles[1], "r"));

            _rule = userChosen;

            if (NRwizard.getTypePage().selectionRad == ConnectionPatternTypePage.USE_RULE_STICKY)
            {
                //Set the sticky rule static var in this class so it is reused
                // as sticky
                ConnectElementsCommand.stickyRule = _rule;
            }

            if (_ports.get(0) != null && _ports.get(1) != null)
            {
                setPortTypeFromPort(0);
                setPortTypeFromPort(1);

                connConnection = false;
                connSource = false;

                reverse = false;

                undoPortsExisting[0] = false;
                undoPortsExisting[1] = false;
                
                super.execute();
                editPart.refresh();
                editPart.activate();
            }
            else
            {
                AcmeSystem context = Helper.getAcmeSystem(ASContext.current()
                        .getActiveDesign().getDesign());
                Object portTypeObj = context.lookupName(
                        NRwizard.getTypePage().selectionPorts[0], true);
                Object portTypeObj2 = context.lookupName(
                        NRwizard.getTypePage().selectionPorts[1], true);

                if (portTypeObj instanceof AcmePortType
                        && portTypeObj2 instanceof AcmePortType || _conn != null)
                {
                    AcmePortType pt1 = (AcmePortType) portTypeObj;
                    AcmePortType pt2 = (AcmePortType) portTypeObj2;

                    _portTypes = new ArrayList();
                    _portTypes.add(pt1);
                    _portTypes.add(pt2);

                    if (_conn != null)
                    {
                        connConnection = true;
                        connSource = (_ports.get(0) != null || _components.get(0) != null) ? false : true;
                    }
                    else
                    {
                        connConnection = false;
                        connSource = false;
                    }
                    
                    reverse = false;

                    createNecessaryPorts();

                    super.execute();
                    editPart.refresh();
                    editPart.activate();
                }
            }
        }
    }

    /**
     *  
     */
    private void useRuleToCreateConnection()
    {
        AcmeSystem context;
        Object portTypeObj;
        Object portTypeObj2;
        
        if (_ports.get(0) != null && _ports.get(1) != null)
        {
            //No ports need to be created. Simply make sure all necessary data
            //is set and continue

            setPortTypeFromPort(0);
            setPortTypeFromPort(1);

            connConnection = false;
            connSource = false;

            reverse = false;

            undoPortsExisting[0] = false;
            undoPortsExisting[1] = false;
            
            super.execute();
            editPart.refresh();
            editPart.activate();
        }
       /* else if (_conn != null)
        {
            
        }*/
        else
        {         
            //Get the port types to create
        	// TODO: It is possible that we are connecting in the default structure of a family, in which case the system/context will be null!
            context = _system;/*Helper.getAcmeSystem(ASContext.current()
                    .getActiveDesign());*/
            portTypeObj = context.lookupName(
                    _rule.getPortTypes()[0].getTypeName(), true);
            portTypeObj2 = context.lookupName(
                    _rule.getPortTypes()[1].getTypeName(), true);

            if ((portTypeObj == null && portTypeObj2 == null) || ((portTypeObj instanceof AcmePortType
                    && portTypeObj2 instanceof AcmePortType) || _conn != null))
            {
                //if both portTypeObj are null, it means we are connecting via the generic rule
	            prepareAndExecute(portTypeObj, portTypeObj2);
            }
            else
            {
                Vector oneRuleHolder = new Vector();
                oneRuleHolder.add(_rule);
                
                //The rule does not specify port data for both associations...
                //and the user has not clicked two ports.
                //This means the user must disambiguate the port type for at
                //least one side of the connection
                NRwizard = new ConnectionPatternWizard(getSystem(), matchRule, getPortTypeArray(), getComponentTypeArray(), false);
                
                WizardDialog dialog = new WizardDialog(Display.getDefault()
                        .getActiveShell(), NRwizard);
                dialog.create();
                dialog.open();
                if (!(IDialogConstants.CANCEL_ID == dialog.getReturnCode()))
                {
                    //Build the rule out of the different parts on the dialog
                    PortConnectionRule userChosen = new PortConnectionRule();
                    userChosen.setConnectorType(NRwizard.getTypePage().selectionConn);
                    userChosen.addAssociation(new PortRoleAssociation(
                            NRwizard.getTypePage().selectionPorts[0], NRwizard.getTypePage().selectionRoles[0], "r"));
                    userChosen.addAssociation(new PortRoleAssociation(
                            NRwizard.getTypePage().selectionPorts[1], NRwizard.getTypePage().selectionRoles[1], "r"));

                    _rule = userChosen;
                    
                    //Now that we have a fully data populated rule, get the data
                    //and run prepare and execute
                    portTypeObj = context.lookupName(
                            _rule.getPortTypes()[0].getTypeName(), true);
                    portTypeObj2 = context.lookupName(
                            _rule.getPortTypes()[1].getTypeName(), true);
                    
                    prepareAndExecute(portTypeObj, portTypeObj2);
                }
            }
        }
    }

    /**
     * @param portTypeObj
     * @param portTypeObj2
     */
    private void prepareAndExecute(Object portTypeObj, Object portTypeObj2)
    {
        _portTypes = new ArrayList();
        _portTypes.add(portTypeObj);
        _portTypes.add(portTypeObj2);

        if (_conn != null)
        {
            connConnection = true;
            connSource = (_ports.get(0) != null || _components.get(0) != null) ? false : true;
        }
        else
        {
            connConnection = false;
            connSource = false;
        }

        reverse = false;

        createNecessaryPorts();

        super.execute();
        editPart.refresh();
        editPart.activate();
    }

    /**
     * @return
     */
    private void checkForMatchingRule()
    {
        int numberExactMatches = 0;
        
        if (ConnectElementsCommand.stickyRule != null)
        {
            //figure out if the stickyRule satisfies the items clicked on for
            // this connection
            //if so, set it
            //There's lots of casting in this call - it simply checks to see if
            //getTarget and getSource are the right type (if the rule specifies types
            //for the ports)
            if ((ConnectElementsCommand.stickyRule.getAssociations()[0]
                    .getPortType() != null && (!(((AcmeElement) this
                            .getSource().getModel()) instanceof AcmePort) || (((AcmeElementType) ((AcmeElement) this
                    .getSource().getModel()).getDeclaredTypes().getNextItem())
                    .getTypeName().equals(ConnectElementsCommand.stickyRule
                    .getAssociations()[0].getPortType()))))
                    && (ConnectElementsCommand.stickyRule.getAssociations()[1]
                            .getPortType() != null && (!(((AcmeElement) this
                                    .getTarget().getModel()) instanceof AcmePort) || (((AcmeElementType) ((AcmeElement) this
                            .getTarget().getModel()).getDeclaredTypes()
                            .getNextItem()).getTypeName()
                            .equals(ConnectElementsCommand.stickyRule
                                    .getAssociations()[1].getPortType())))))
                _rule = ConnectElementsCommand.stickyRule;
            else
            {
                fStickyRuleNoMatch = true;
                ConnectElementsCommand.stickyRule = null;
            }
        }
        else
        {
            //check to see if there is one one rule that matches what the user
            // clicked
            //If so, bypass the connection wizard and just use the rule
            AcmePortType [] portTypeArray = getPortTypeArray();
			AcmeComponentType [] componentTypeArray = getComponentTypeArray();
			AcmeConnectorType connectorType = getSafeConnectorType();
			matchRule = SystemVisualization.checkExistingRules(portTypeArray,
                    componentTypeArray, connectorType, getSystem());

            if (matchRule != null)
                numberMatches = matchRule.size();
            else
                numberMatches = 0;

            if (numberMatches == 1) {
            	PortConnectionRuleKey k1 = DEFAULT_CONNECTION_RULE.generateKey();
            	PortConnectionRuleKey k2 = ((PortConnectionRule )matchRule.get (0)).generateKey ();
            	if (k1.equals (k2)) {
	            	getPotentialSuperRules (portTypeArray, componentTypeArray, connectorType);
            		
            		
            	}
            	
            }
            
            
            for (Iterator iter = matchRule.iterator(); iter.hasNext(); )
            {
                PortConnectionRule r = (PortConnectionRule)iter.next();
                if (r.generateKey().equals(new PortConnectionRuleKey(
                        		portTypeArray, componentTypeArray, connectorType)))
                {
                    numberExactMatches++;
                    _rule = r;
                }
                	
                if (numberExactMatches > 1)
                    _rule = null; //It is still ambiguous, so leave it as such
            }
            
            if (numberMatches == 1)
                _rule = (PortConnectionRule) matchRule.get(0);
            else if (numberMatches > 1)
            {
                //Check to see if one of them is marked as a priority
                //If so, use that
                for (Iterator iter = matchRule.iterator(); iter.hasNext()
                        && _rule == null;)
                {
                    PortConnectionRule next = (PortConnectionRule) iter.next();
                    if (next.isPriorityRule())
                        _rule = next;
                }
            }
        }
    }

    /**
	 * @param portTypeArray
	 * @param componentTypeArray
	 * @param connectorType
	 */
	private void getPotentialSuperRules(AcmePortType [] portTypeArray, AcmeComponentType [] componentTypeArray, AcmeConnectorType connectorType) {
		// Let's check to see if there are any matches on the supertypes
		AcmeElementType key1 = (AcmeElementType ) (portTypeArray[0] != null ? (AcmeType) portTypeArray[0]
												                                    : (AcmeType) componentTypeArray [0]);
		AcmeElementType key2 = (AcmeElementType )(portTypeArray[1] != null ? (AcmeType) portTypeArray[1]
												                                    : (AcmeType) componentTypeArray[1]);
		if (key1 == null) key1 = connectorType;
		if (key2 == null) key2 = connectorType;
		AcmeElementType origKey1 = key1;
		AcmeElementType origKey2 = key2;
		AcmeFamily family = (AcmeFamily )ModelHelper.getMostCommonFamily(key1, key2);
		AcmeElementType superType = null;
		AcmeFamily superFam = family;
		if (family != ModelHelper.getDeclaredIn(key1) || (family == ModelHelper.getDeclaredIn (key2) && family == ModelHelper.getDeclaredIn (key1))) {
			if (key1.getSuperTypes().getSize () >= 1) {
				 superType = (AcmeElementType )key1.getSuperTypes().getItemAt (0);
				 superFam = ModelHelper.getDeclaredIn (superType);
				while (superFam == family) {
					if (superType.getSuperTypes().getSize () >= 1) {
						superType = (AcmeElementType )superType.getSuperTypes().getItemAt (0);
						superFam = ModelHelper.getDeclaredIn (superType);
					}
					else break;		
				}
				key1 = superType;
				superType = null;
			}
		}
		if (family != ModelHelper.getDeclaredIn (key2) || (family == ModelHelper.getDeclaredIn (key2) && family == ModelHelper.getDeclaredIn (origKey1))) {
			if (key2.getSuperTypes ().getSize () >= 1) {
				superType = (AcmeElementType )key2.getSuperTypes().getItemAt (0);
				while (superFam != ModelHelper.getDeclaredIn (superType)) {
					if (superType.getSuperTypes ().getSize () >= 1) {
						superType = (AcmeElementType )superType.getSuperTypes().getItemAt (0);
					}
					else break;
				}
				key2 = superType;
			}
		}
			if (superType != null && superFam == ModelHelper.getDeclaredIn (superType)) {
				AcmeComponentType [] newComponentTypeArray = new AcmeComponentType [2];
				AcmePortType [] newPortTypeArray = new AcmePortType [2];
				AcmeConnectorType newConnectorType = connectorType;
				if (key1 instanceof AcmeConnectorType) newConnectorType = (AcmeConnectorType )key1;
				else if (portTypeArray [0] != null) newPortTypeArray [0] = (AcmePortType )key1;
				else newComponentTypeArray [0] = (AcmeComponentType )key1;
				if (key2 instanceof AcmeConnectorType) newConnectorType = (AcmeConnectorType )key2;
				if (portTypeArray [1] != null) newPortTypeArray [1] = (AcmePortType )key2;
				else newComponentTypeArray[1] = (AcmeComponentType )key2;
				
				matchRule = SystemVisualization.checkExistingRules(newPortTypeArray, newComponentTypeArray, newConnectorType, getSystem ());
				if (matchRule.size() == 1) {
					PortConnectionRuleKey k1 = DEFAULT_CONNECTION_RULE.generateKey();
	            	PortConnectionRuleKey k2 = ((PortConnectionRule )matchRule.get (0)).generateKey ();
	            	if (k1.equals (k2)) {
	            		getPotentialSuperRules(newPortTypeArray, newComponentTypeArray, connectorType);
	            	}
				}
			
		}
			
	}

	/**
     * @return
     */
    private AcmeConnectorType getSafeConnectorType()
    {
        if (getConnector() != null)
            return (AcmeConnectorType)getConnector().getDeclaredTypes().getNextItem();
        return null;
    }

    private void setPortTypeFromPort(int i)
    {
        if (_portTypes.isEmpty() || _portTypes.size() == 1)
            _portTypes.add((AcmePortType) ((AcmePort) _ports.get(i))
                    .getDeclaredTypes().getNextItem());
        else
            _portTypes.set(i, (AcmePortType) ((AcmePort) _ports.get(i))
                    .getDeclaredTypes().getNextItem());
    }

    private void getParentComponentData()
    {
        for (int i = 0; i < 2; i++)
        {
            AcmePort temp = (AcmePort) _ports.get(i);

            if (temp != null)
            {
                //get the parent info
                _components.set(i, temp.getParent());
            }
        }
    }

    /*
     * This method analyzes what the source and target editparts are and creates
     * the ports that will be necessary to create the connection
     */
    public void createNecessaryPorts()
    {
        EditPart epSource = getSource();
        EditPart epTarget = getTarget();
        if (epSource instanceof ComponentEditPart)
        {
            //it is necessary to create the source port on the component
            AcmePort p = generatePort(epSource, 0);
            _ports.set(0, p);
            undoPortsExisting[0] = true;
        }
        else
            undoPortsExisting[0] = false;
        if (epTarget instanceof ComponentEditPart)
        {
            //it is necessary to create the target port on the component
            AcmePort p = generatePort(epTarget, 1);
            _ports.set(1, p);
            undoPortsExisting[1] = true;
        }
        else
            undoPortsExisting[1] = false;        
    }

    private AcmePort generatePort(EditPart epSource, int i)
    {
        AcmePortType t = (AcmePortType) _portTypes.get(i);
        if (i == 0)
            return createPort(Helper.getUniqueName((AcmeComponent) _components
                    .get(0), _rule.getAssociations()[0].getPortName() != null ? _rule.getAssociations()[0].getPortName() : "p"), (AcmeComponent) epSource.getModel(),
                    t, getComponentLocation1(), 0);
        else
            return createPort(Helper.getUniqueName((AcmeComponent) _components
                    .get(1), _rule.getAssociations()[1].getPortName() != null ? _rule.getAssociations()[1].getPortName() : "p"), (AcmeComponent) epSource.getModel(),
                    t, getComponentLocation2(), 1);
    }

    public void clearExtraneousObjects()
    {
        EditPart epSource = getSource();
        EditPart epTarget = getTarget();
        if (epSource instanceof ComponentEditPart)
        {
            this.setPort1(null);
            this.setLocation1(null);

            if (!(epTarget instanceof ConnectorEditPart))
            {
                this.setConnector(null);
                this.setLocationConn(null);
            }
        }
        else if (epSource instanceof ConnectorEditPart)
        {
            this.setPort1(null);
            this.setLocation1(null);
            this.setComponent1(null);
            this.setComponentLocation1(null);
        }
        else if (epSource instanceof PortEditPart)
        {
            this.setComponent1(null);
            this.setComponentLocation1(null);
            if (!(epTarget instanceof ConnectorEditPart))
            {
                this.setConnector(null);
                this.setLocationConn(null);
            }
        }
        if (epTarget instanceof ComponentEditPart)
        {
            this.setPort2(null);
            this.setLocation2(null);
            if (!(epSource instanceof ConnectorEditPart))
            {
                this.setConnector(null);
                this.setLocationConn(null);
            }
        }
        else if (epTarget instanceof ConnectorEditPart)
        {
            this.setPort2(null);
            this.setLocation2(null);
            this.setComponent2(null);
            this.setComponentLocation2(null);
        }
        else if (epTarget instanceof PortEditPart)
        {
            this.setComponent2(null);
            this.setComponentLocation2(null);
            if (!(epSource instanceof ConnectorEditPart))
            {
                this.setConnector(null);
                this.setLocationConn(null);
            }
        }
    }

    public void redo()
    {
        createNecessaryPorts();
        super.redo();
        editPart.refresh();
    }

    public void undo()
    {
        super.undo();
        editPart.refresh();
    }

    /**
     * @see org.eclipse.gef.commands.Command#canUndo()
     */
    public boolean canUndo()
    {
        return true;
    }

    //	private void loadRule()
    //	{
    //		SystemVisualization vis = VisualStyleRegistry.getSysVis(Helper
    //				.getMostCommonFamily(this.getPort1Type(), getPort2Type()));
    //		if (vis != null)
    //			_rule = vis.getPortConnectionRule(new PortConnectionRuleKey(
    //					getPortTypeArray()));
    //		else
    //			//_rule = DEFAULT_CONNECTION_RULE;
    //			_rule = null;
    //	}

    /**
     * @return
     */
    private AcmePortType[] getPortTypeArray()
    {
        AcmePortType[] pts = new AcmePortType[_ports.size()];
        for (int i = 0; i < pts.length; i++)
        {
            if (_ports.get(i) != null)
            {
                AcmePort port = (AcmePort) _ports.get(i);
                if (port.getDeclaredTypes().getSize() == 0)
                {
                    pts[i] = ConstantTypes.PORT_BASE_TYPE;
                }
                else
                {
                    pts[i] = (AcmePortType) port.getDeclaredTypes()
                            .getNextItem();
                }
            }
        }
        return pts;
    }

    /**
     * @return
     */
    private AcmePort[] getPortArray()
    {
        AcmePort[] pts = new AcmePort[_ports.size()];
        for (int i = 0; i < pts.length; i++)
        {
            AcmePort port = (AcmePort) _ports.get(i);
            pts[i] = port;
        }
        return pts;
    }

    /**
     * @return
     */
    private AcmeComponent[] getComponentArray()
    {
        AcmeComponent[] cps = new AcmeComponent[_components.size()];
        for (int i = 0; i < cps.length; i++)
        {
            AcmeComponent comp = (AcmeComponent) _components.get(i);
            cps[i] = comp;
        }
        return cps;
    }

    private AcmeComponentType[] getComponentTypeArray()
    {
        AcmeComponentType[] cps = new AcmeComponentType[_components.size()];
        for (int i = 0; i < cps.length; i++)
        {
            if (_components.get(i) != null)
            {
                AcmeComponent comp = (AcmeComponent) _components.get(i);
                if (comp.getDeclaredTypes().getSize() == 0)
                {
                    cps[i] = ConstantTypes.COMPONENT_BASE_TYPE;
                }
                else
                {
                    cps[i] = (AcmeComponentType) comp.getDeclaredTypes()
                            .getNextItem();
                }
            }
        }
        return cps;
    }

    public String toString()
    {
        StringBuffer ret = new StringBuffer();

        ret.append("Comps: " + printAcmeElement(_components) + "\n");
        ret.append("Ports: " + printAcmeElement(_ports) + "\n");

        return ret.toString();
    }

    private String printAcmeElement(List a)
    {
        StringBuffer ret = new StringBuffer();
        for (Iterator iter = a.iterator(); iter.hasNext();)
            ret.append(((AcmeElementType) ((AcmeElement) iter.next())
                    .getDeclaredTypes().getNextItem()).getTypeName()
                    + ", ");
        return ret.toString();
    }
    

	/**
	 * @return Returns the source.
	 */
	public EditPart getSource()
	{
		return source;
	}

	/**
	 * @param source
	 *                 The source to set.
	 */
	public void setSource(EditPart source)
	{
		this.source = source;
	}

	/**
	 * @return Returns the target.
	 */
	public EditPart getTarget()
	{
		return target;
	}

	/**
	 * @param target
	 *                 The target to set.
	 */
	public void setTarget(EditPart target)
	{
		this.target = target;
	}

  
}