package AcmeStudio.ui.editors.diagram.actions;

import java.io.StringWriter;
import java.util.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Display;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.ui.editors.diagram.visualInfo.VisualizationSaveWalkerOp;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.elements.*;
import acmeLib.tools.GraphWalker;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * An implementation of AbstractCommand that controls Role movement
 * 
 */
public class CopyElementCommand extends Command {

    /**
     * The acme element to be copied
     */
    private List _acmeElements = null;

    /**
     * System clipboard
     */
    private Clipboard _clipboard = null;

    private Object _oldClipboardData = null;

    /**
     * Constructor
     */
    public CopyElementCommand(List editParts) {
//        _editPart = editPart;
        Iterator i = editParts.iterator();
        _acmeElements = new ArrayList (2);
        // Two passes through the list
        // 1) Add all components and connectors
        // 2) Add all ports and roles that whose parents aren't selected
        // 3) Add all the connections
        _clipboard = new Clipboard (Display.getCurrent ());
        setLabel ("Copy");
        boolean hasOthers = false;
        while (i.hasNext ()) {
            Object sel = i.next ();
            Object obj = null;
            if (sel instanceof EditPart) {
                obj = ((EditPart )sel).getModel ();
            }
            else {
                obj = sel;
            }
            if (obj instanceof AcmeComponent || obj instanceof AcmeConnector) {
                _acmeElements.add (obj);   
            }
            else {
                hasOthers = true;   
            }
        }
        
        if (hasOthers) {
            i = editParts.iterator ();
            while (i.hasNext ()) {
                Object sel = i.next ();
                Object obj = null;
                if (sel instanceof EditPart) {
                    obj = ((EditPart )sel).getModel ();
                }
                else {
                    obj = sel;
                }
                if (obj instanceof AcmePort || obj instanceof AcmeRole) {
                    AcmeElement el = (AcmeElement )obj;
                    if (!_acmeElements.contains (el.getParent())) {
                        _acmeElements.add (obj);
                    }
                }
            }
        }
        
        Set attachments = new HashSet ();
        i = _acmeElements.iterator();
        while (i.hasNext ()) {
        	AcmeElement el = (AcmeElement )i.next ();
        	if (el instanceof AcmeComponent) {
        		AcmeComponent comp = (AcmeComponent )el;
        		AcmeEnumeration ports = comp.getPorts ();
        		while (ports.moreItems()) {
        			AcmePort port = (AcmePort )ports.getNextItem ();
        			if (comp.getParent () != null) {
	        		    AcmeEnumeration atts = ((AcmeSystem )comp.getParent ()).getAttachments(port);
	        			while (atts.moreItems()) {
	        				AcmeAttachment att = (AcmeAttachment )atts.getNextItem ();
	        				AcmeRole r = att.getRole ();
	        				if (_acmeElements.contains (r.getParent())) {
	        					attachments.add(att);
	        				}
	        			}
        			}
        		}
        	}
        	else if (el instanceof AcmeConnector) {
        		AcmeConnector conn = (AcmeConnector )el;
        		AcmeEnumeration roles = conn.getRoles ();
        		while (roles.moreItems ()) {
        			AcmeRole role = (AcmeRole )roles.getNextItem ();
        			if (conn.getParent () != null) {
	        			AcmeEnumeration atts = ((AcmeSystem )conn.getParent ()).getAttachments (role);
	        			while (atts.moreItems ()) {
	        				AcmeAttachment att = (AcmeAttachment )atts.getNextItem ();
	        				AcmePort p = att.getPort ();
	        				if (_acmeElements.contains (p.getParent ())) {
	        					attachments.add (att);
	        				}
	        			}
	        		}
        		}
        	}
        }
        
        _acmeElements.addAll(attachments);
    }

    /**
     * @see org.eclipse.gef.commands.AbstraceCommand#canExecute
     */
    public boolean canExecute() {
        return _acmeElements != null && !_acmeElements.isEmpty();
    }

    /**
     * Copy the acme element to the clipboard
     */
    private void copyToClipboard() {
        // Place the acme element on the clipboard
        AcmeOutputStream aos = new AcmeOutputStream();
        String script = "";
        try {
            AcmeDesign newAcmeDesign = new AcmeDesign();
            final AcmeSystem newAcmeSystem = new AcmeSystem("ClippedSystem");

//            AcmeEnumeration families = acmeSystem.getDeclaredTypes();
//            while (families.moreItems ()) {
//                AcmeFamily family = (AcmeFamily )families.getNextItem ();
//                newAcmeDesign.addFamily (family);
//                newAcmeSystem.addDeclaredType (family);
//            }
                
            Map visMap = new HashMap ();
            
  
            
            Iterator i = _acmeElements.iterator();
            while (i.hasNext ()) {
                   
                AcmeElement acmeElement = (AcmeElement )i.next ();  
                        
                if (acmeElement instanceof AcmeComponent) {
                    
                	AcmeComponent	newComp	= (AcmeComponent) acmeElement.copy();
					newAcmeSystem.addComponent(newComp);
				
	               } else if (acmeElement instanceof AcmeConnector) {
                    
                	AcmeConnector	newConn	= (AcmeConnector) acmeElement.copy();
					newAcmeSystem.addConnector(newConn);

                } else if (acmeElement instanceof AcmePort) {
                    
                	AcmePort	newPort	= (AcmePort )acmeElement;
					newAcmeSystem.addPort (newPort); //Component((AcmeComponent) acmeElement.getParent().copy());

                } else if (acmeElement instanceof AcmeRole) {
                    
                	AcmeRole	newRole	= (AcmeRole )acmeElement;
					newAcmeSystem.addRole (newRole); //Connector((AcmeConnector) acmeElement.getParent().copy());

                } 
                else if (acmeElement instanceof AcmeAttachment) {
                	newAcmeSystem.addAttachment( (AcmeAttachment )acmeElement);
                }
            }
            newAcmeDesign.addSystem(newAcmeSystem);
            newAcmeDesign.exportArmani(aos);
       		script = aos.toString ();
            
	        if (AcmeStudioPlugin.getDefault() .getPreferenceStore().getBoolean (GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
					DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance ().newDocumentBuilder ();
					Document doc = docBuilder.newDocument ();
					Element elem = doc.createElement ("visualInformation");
					VisualizationSaveWalkerOp walker = new VisualizationSaveWalkerOp (elem, doc);
					GraphWalker.WalkOverAllElements(newAcmeSystem, walker, GraphWalker.WALK_TOP_DOWN);
					
					StringWriter sw = new StringWriter ();
					TransformerFactory tFactory = TransformerFactory.newInstance();
					Transformer transformer = tFactory.newTransformer();
					DOMSource source = new DOMSource (elem);
				    StreamResult result = new StreamResult (sw);
				    transformer.transform (source, result);
					
					script += "\n";
					script += sw.toString();
			}
        } catch (Exception e) {
        }
       




        // This should be commented out when TextTransfer is changed to
        // AcmeTransfer
        String text = "AcmeElement\n";

        text += script;

        Object[] data = new Object[] { text };
        Transfer transferType = TextTransfer.getInstance();
        Transfer[] transferTypes = new Transfer[] { transferType };
        _clipboard.setContents(data, transferTypes);
        
//        Debug.diagnoseln(script);
    }

    /**
     * Save data on the clipboard
     */
    private void saveClipboard() {
        Transfer transferType = TextTransfer.getInstance();
        _oldClipboardData = _clipboard.getContents(transferType);
    }

    /**
     * Restore data to the clipboard
     */
    private void restoreClipboard() {
        // Place old data on the clipboard
        Object[] data = new Object[] { _oldClipboardData };
        Transfer transferType = TextTransfer.getInstance();
        Transfer[] transferTypes = new Transfer[] { transferType };
        _clipboard.setContents(data, transferTypes);
    }

    /**
     * @see org.eclipse.gef.commands.AbstraceCommand#execute
     */
    public void execute() {
        saveClipboard();
        copyToClipboard();
    }

    /**
     * @see org.eclipse.gef.commands.AbstractCommand#undo()
     */
    public void undo() {
        restoreClipboard();
    }

    /**
     * @see org.eclipse.gef.commands.AbstractCommand#redo()
     */
    public void redo() {
        execute();
    }
}
