package AcmeStudio.ui.editors.diagram.actions;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;

import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeRole;

public class CreateBendpointCommand extends BendpointCommand {
    private RoleEditPart _editPart = null;
    private Point _roleStart = null;
    private Point _roleEnd = null;
    private SystemView _sv = null;
	private boolean	_redoing;

    public void setEditPart(RoleEditPart editPart) {
        this._editPart = editPart;
        this.setLabel("Create Bendpoint");
    }
    public void setRoleStart(Point roleStart) {
        this._roleStart = roleStart;
    }
    public void setRoleEnd(Point roleEnd) {
        this._roleEnd = roleEnd;
    }
    public void execute() {
        //AbsoluteBendpoint abp = new AbsoluteBendpoint(this.getLocation());
        RoleView rv = (RoleView)_editPart.getFigure();
        _sv = (SystemView) rv.getParent();
        rv._isBendpointBeingCreated = true;
        rv._isBendpointMoving = false;
        AcmeRole role = getRole();
        PointList bendpoints = null;
        if (_redoing) {
        	bendpoints = Helper.getBendpoints (getRole ());
        	if (bendpoints == null) bendpoints = new PointList ();
        	Helper.addBendpoint(getRole(), getIndex(), this.getLocation(), bendpoints);
        }
        else {
        	bendpoints = _sv.getBendpoints(role, _roleStart, _roleEnd, null);
        }
        bendpoints = _sv.layoutBendpoints(
            role,
            bendpoints,
            _roleStart,
            _roleEnd);
        SystemView.cleanBendpoints(bendpoints, _roleStart, _roleEnd);            
        Helper.setBendpoints(role, bendpoints);
        _editPart.refresh();
        super.execute();
        rv._isBendpointBeingCreated = false;
    }

    public void redo () {
    	_redoing = true;
    	execute ();
    	_redoing = false;
    }
    
    public void undo() {
        super.undo();
        AcmeRole role = getRole();
        Helper.removeBendpoint(role, getIndex(), _roleStart, _roleEnd);
        PointList bendpoints = _sv.getBendpoints(role, _roleStart, _roleEnd, null);
        bendpoints = _sv.layoutBendpoints(
            role,
            bendpoints,
            _roleStart,
            _roleEnd);
        SystemView.cleanBendpoints(bendpoints, _roleStart, _roleEnd);
        Helper.setBendpoints(role, bendpoints);
        _editPart.refresh();
    }

}
