package AcmeStudio.ui.editors.diagram.actions;

import AcmeStudio.ui.editors.diagram.visualInfo.ConnectionPatternInformation;
import AcmeStudio.wizards.connection.PortConnectionRule;
import acmeLib.elements.AcmeSystem;

/**
 * This class represents the command object for creating a connection
 * pattern
 * 
 * @author LFW
 *
 */
public class CreateRuleCommand
	extends AcmeStudio.model.ModelCommand {

    //The rule to add
	private PortConnectionRule _rule = null;
	
	//The relevant system
	private AcmeSystem _sys;
	
	public CreateRuleCommand(PortConnectionRule rule, AcmeSystem sys) {
		super("CreateRuleCommand");
		this._rule = rule;
		_sys = sys;
	}

/**
 * Execute method which adds the rule
 */
	public synchronized void execute() {
		ConnectionPatternInformation info = getConnectionPatternInfo();
		info.addConnectionPattern(_rule);
	}


	/**
	 * @return
	 */
	private ConnectionPatternInformation getConnectionPatternInfo() {
		ConnectionPatternInformation info = (ConnectionPatternInformation) _sys.getUserData (ConnectionPatternInformation.CONNECTION_PATTERNS_KEY);
		if (info == null) {
			info = new ConnectionPatternInformation ();
			_sys.setUserData(ConnectionPatternInformation.CONNECTION_PATTERNS_KEY, info);
		}
		return info;
	}


	public void redo() {
        execute ();
	}


	public void undo() {
        ConnectionPatternInformation info = getConnectionPatternInfo();
        info.getConnectionPatterns().remove(_rule);
	}

	/**
	 * @see org.eclipse.gef.commands.Command#canUndo()
	 */
	public boolean canUndo() {
		return true;
	}

	/**
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
	    	//For now, return true
	    return true;
	}
}
