package AcmeStudio.ui.editors.diagram.actions;

import java.util.Vector;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorActionBarContributor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import AcmeStudio.core.ASContext;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.Utils;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeSystem;
import acmeLib.representations.AcmeRepresentation;

/**
 * @author Bradley Schmerl (schmerl@cs.cmu.edu)
 *
 * Provides an action for navigating up a representation
 * 
 */
public class DrillDownAction extends SelectionAction {

    public static final String ID = "$DrillDownRep$";

    protected AcmeSystem _innerSystem;
    private EditPart _editPart;
    
    private static class RepresentationLabelProvider extends LabelProvider {

    	
    		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
		 */
		public Image getImage(Object element) {
			if (element instanceof AcmeRepresentation) {
				AcmeRepresentation rep = (AcmeRepresentation) element;
				MultiPageAcmeEditor mp = ASContext.current ().getActiveEditor();
				if (mp != null) {
					Vector editors = mp.getSystemEditors ();
					SystemView systemView = null;
					for (int i = 0; i < editors.size (); i++) {
						DiagramEditor editor = (DiagramEditor )editors.get (i);
						systemView = (SystemView )editor.getFigure (rep.getSystem());
						if (systemView != null) break;
					}
					if (systemView != null) {
						systemView.paintOverview ();
						Image img = systemView.getOverview ();
						if (img != null) {
							ImageData imgData = img.getImageData ();
							double aspectRatio = (double )imgData.width/100;
							int scaledHeight = Utils.zeroGuard ((int )(imgData.height/aspectRatio));
							imgData = imgData.scaledTo (100, scaledHeight);
							String key = systemView.getModel ().getName () + "_overview_RepView";
							ImageManager.disposeImage(key);
							return ImageManager.createImage(key, imgData, false);
				
						}
					}
				}
			}
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
		 */
		public String getText(Object element) {
			if (element instanceof AcmeRepresentation) {
				AcmeRepresentation rep = (AcmeRepresentation) element;
				return rep.getName();
			}
			return "";
		}

}

    public DrillDownAction(IEditorPart editor) {
        super(editor);
        setId(ID);
        setText("Drill to representation");
        setToolTipText("Drill to representation");
    }

    public void setEditPart(EditPart editPart) {
        this._editPart = editPart;
    }

    /**
     * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
        if (_editPart != null) {
            Object obj = _editPart.getModel();
            if (obj instanceof AcmeElement) {
                AcmeElement elem = (AcmeElement) obj;
                AcmeEnumeration ae = elem.getRepresentations();
                return ae != null && ae.getSize() > 0;
            }
        }
        return false;
    }

    /**
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        openRep();
//        IContentOutlinePage cop =
//            (IContentOutlinePage) getWorkbenchPart().getAdapter(
//                IContentOutlinePage.class);
//        cop.setFocus();
//        IElementViewPage ev = (IElementViewPage )getWorkbenchPart().getAdapter (IElementViewPage.class);
//        if (ev.getControl ().isVisible()) {
//        	ev.setFocus();
//        }
//        else {
//        	
//        }
//        ((MultiPageAcmeEditor) getWorkbenchPart()).getActiveEditor().setFocus();
       
        MultiPageAcmeEditor mpe = ASContext.current ().getActiveEditor ();
        if (mpe != null) {
        	IEditorActionBarContributor abc = mpe.getEditorSite().getActionBarContributor();
        	abc.setActiveEditor (null);
        	abc.setActiveEditor (mpe);
        }
        
    }

    private void openRep() {
        if (_editPart != null) {
            Object obj = _editPart.getModel();
            if (obj instanceof AcmeElement) {
                AcmeElement elem = (AcmeElement) obj;
                AcmeEnumeration ae = elem.getRepresentations();
                if (ae != null && ae.getSize() > 0) {
                    AcmeSystem sys = null;
                    if (ae.getSize () == 1) {
                        sys = ((AcmeRepresentation) ae.getNextItem()).getSystem();
                    }
                    else {
                    	ElementListSelectionDialog reps = new ElementListSelectionDialog (Display.getCurrent().getActiveShell(), new RepresentationLabelProvider ());
                    	reps.setElements(Helper.convertAcmeEnumerationToList(ae).toArray());
                    	reps.setMessage("Please choose a representation to open");
                    	if (reps.open () == IDialogConstants.OK_ID) {
                    		Object [] rep = reps.getResult ();
                    		if (rep != null & rep.length > 0) {
                    			sys = ((AcmeRepresentation )rep [0]).getSystem ();
                    		}		
                    	}	
                    }
                    if (sys != null) {
	                    MultiPageAcmeEditor mp =
	                        ASContext.current().getActiveEditor();
	                    if (mp != null) {
	                        mp.swapDiagramEditor(
	                            sys,
	                            (AcmeSystem) sys
	                                .getParentRepresentation()
	                                .getParent()
	                                .getParent());
	                    }
                    }
                }

            }
        }
    }
}