package AcmeStudio.ui.editors.diagram.actions;

import java.beans.PropertyChangeEvent;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.ui.editors.diagram.figures.*;
import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.ui.editors.diagram.parts.ConnectorEditPart;
import AcmeStudio.ui.editors.diagram.parts.PortEditPart;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.ui.editors.diagram.styles.EditVisualizationDialog;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import acmeLib.elements.AcmeElementType;

/**
 * @author schmerl
 *
 */
public class EditTypeVisualizationAction extends org.eclipse.gef.ui.actions.SelectionAction implements IWorkbenchWindowActionDelegate {
	private EditPart _currentSelection;
	public class EditTypeVisCommand extends org.eclipse.gef.commands.Command {
		private boolean	_allowOverridingType;

		public EditTypeVisCommand () {
			super (ID);
		}
		
        public void execute() {
            Shell shell =
                AcmeStudioPlugin
                    .getDefault()
                    .getWorkbench()
                    .getActiveWorkbenchWindow()
                    .getShell();
            ElementVisualization vis = null;
            AcmeElementType type = null;
            IAcmeElementFigure view = getView(_currentSelection);
            if (view != null) {
                if (view.getModel().getDeclaredTypes().getSize () > 0)
                    type = (AcmeElementType )view.getModel().getDeclaredTypes().getNextItem();
                vis = view.getVisualization();
            }

            if (vis != null) {
                EditVisualizationDialog dialog = null;
                if (type != null) {
                    dialog = new EditVisualizationDialog(
                        shell,
                        view.getModel(), type);
                }
                else {
                    dialog = new EditVisualizationDialog (shell, vis, view.getModel ());
                }
                dialog.setCanOverrideTypeVis(_allowOverridingType);
                if (dialog.open() == IDialogConstants.OK_ID) {
                	boolean hasSpecialVis = VisualStyleRegistry.hasSpecificVisualization(view.getModel ());
                	if (hasSpecialVis && !dialog.isOverrideTypeVis()) {
                		VisualStyleRegistry.setVisualizationForElement(view.getModel (), null);
                	}
                	
                	ElementVisualization	newVis	= dialog.getVisualization ();
					if (dialog.isOverrideTypeVis()) {
                		VisualStyleRegistry.setVisualizationForElement (view.getModel (), newVis);
                		newVis.addSuperVisualization(vis);
                		((AcmeVisElement )view).propertyChange (new PropertyChangeEvent (this, ElementVisualization.PROP_VIS_RESET,  vis, newVis));
                	}
                	else if (!hasSpecialVis) {
                		vis.copyInto (newVis);
                	}
                	else if (hasSpecialVis) {
                		((AcmeVisElement )view).propertyChange (new PropertyChangeEvent (this, ElementVisualization.PROP_VIS_RESET, vis, newVis));
                	}
                	_currentSelection.getRoot().refresh();
                		
                }
                
            }
        }

		/**
		 * @param b
		 */
		public void setOverrideType(boolean allowOverridingType) {
			_allowOverridingType = allowOverridingType;
			
		}
        
	}
	
	public static final String
		ID = "edvisln",
		ID_REQUEST = "edvisln";
	/**
     * Constructor for EditTypeVisualizationAction.
     */
    public EditTypeVisualizationAction(IEditorPart editor) {
        super (editor);
        setText ("Edit Visualization");
        setId (ID);
        setToolTipText ("Edits the visualization  of the selected element");
        //setImageDescriptor (ImageManager.getImageDescriptor (ImageManager.ID_EDIT_TYPE_VISUALIZATION));
//        ImageDescriptor desc = ImageDescriptor.createFromFile (AcmeStudioPlugin.class, "icons/edit_type_vis.gif");
//        setImageDescriptor(desc);
    }
    
    public EditTypeVisualizationAction () {
    	super ((IWorkbenchPart )null);
    	setText ("Edit Visualization for Type");
        setId (ID);
        setToolTipText ("Edits the visualization for the type of the selected element");
 //       setImageDescriptor (ImageManager.getImageDescriptor (ImageManager.ID_EDIT_TYPE_VISUALIZATION));	
//        ImageDescriptor desc = ImageDescriptor.createFromFile (AcmeStudioPlugin.class, "icons/edit_type_vis.gif");
//        setImageDescriptor(desc);
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
     */
    public void dispose() {
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
     */
    public void init(IWorkbenchWindow window) {

    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(IAction)
     */
    public void run(IAction action) {
 		new EditTypeVisCommand ().execute();
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
    	_currentSelection = null;
    	if (selection instanceof StructuredSelection) {
    		StructuredSelection sel = (StructuredSelection )selection;
    		if (sel.size () == 1) {
    			Object o = sel.getFirstElement (); 
    			if (o instanceof EditPart) {
    				_currentSelection = ((EditPart )sel.getFirstElement());	
    			}
    			
    		}
    	}
    }
    
   /**
     * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
    	if (this.getWorkbenchPart()  != null) {
    		if (getSelectedObjects ().size () != 1) return false;
    		List parts = getSelectedObjects ();
    		Object o = parts.get (0);
    		if (!(o instanceof EditPart)) return false;
    		_currentSelection = (EditPart )o;
    	}

    	boolean result =  (_currentSelection != null) &&
    			((_currentSelection instanceof ComponentEditPart) ||
    			(_currentSelection instanceof ConnectorEditPart) ||
    			(_currentSelection instanceof PortEditPart) ||
    			(_currentSelection instanceof RoleEditPart));
    			
    	if (!result) return false;
    	
    	return true;
    	// BugFix #24 Disable if it is a default visualization
//    	ElementVisualization vis = getView (_currentSelection).getVisualization();
//    	return vis != VisualStyleRegistry.DEFAULT_COMPONENT_VISUALIZATION &&
//    		   vis != VisualStyleRegistry.DEFAULT_CONNECTOR_VISUALIZATION &&
//    		   vis != VisualStyleRegistry.DEFAULT_PORT_VISUALIZATION &&
//    		   vis != VisualStyleRegistry.DEFAULT_ROLE_VISUALIZATION;
    }

    /**
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
    	ElementVisualization vis = getView (_currentSelection).getVisualization();
    	boolean isDefaultVis = (vis == VisualStyleRegistry.DEFAULT_COMPONENT_VISUALIZATION ||
    		   vis == VisualStyleRegistry.DEFAULT_CONNECTOR_VISUALIZATION ||
    		   vis == VisualStyleRegistry.DEFAULT_PORT_VISUALIZATION ||
    		   vis == VisualStyleRegistry.DEFAULT_ROLE_VISUALIZATION);
        EditTypeVisCommand editTypeVisCommand = new EditTypeVisCommand ();
        editTypeVisCommand.setOverrideType (!isDefaultVis);
		editTypeVisCommand.execute ();
    }
	/**
	 * Returns a figure corresponding to an editor part
	 * @param part the Part to find the view of
	 * @return the view
	 */
	public static IAcmeElementFigure getView (EditPart part) {
		if (part instanceof ComponentEditPart) {
						return
							(ComponentView) ((ComponentEditPart) part)
								.getFigure();
		}
	    else if (part instanceof ConnectorEditPart) {
	    	return (ConnectorView )((ConnectorEditPart )part).getFigure();
	    }
	    else if (part instanceof RoleEditPart) {
	    	return (RoleView )((RoleEditPart )part).getFigure();
	    }
	    else if (part instanceof PortEditPart) {
	    	return (PortView )((PortEditPart )part).getFigure ();
	    }
	    return null;
	}

}
