/*
 * Created on Jul 27, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;

import AcmeStudio.core.ASModel;
import AcmeStudio.model.BindingCommand;
import AcmeStudio.model.ModelHelper;
import AcmeStudio.ui.editors.diagram.parts.PortEditPart;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.ui.views.parts.MatchUnboundDialog;
import AcmeStudio.util.Helper;
import AcmeStudio.util.IMenuBuilder;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;
import acmeLib.representations.AcmeBinding;
import acmeLib.representations.AcmeBindingsMap;
import acmeLib.representations.AcmeRepresentation;


/**
 * @author Bradley Schmerl
 * Created Jul 27, 2004
 */
public class ExportUnboundToParentAction extends SelectionAction implements IMenuBuilder {

	public static final String ID = "$ExportUnbound$";
	private AcmeRepresentation	_parentRep;
	/**
	 * @param part
	 */
	public ExportUnboundToParentAction(IWorkbenchPart part) {
		super (part);
		setId (ID);
		setText ("Bind ports to parent");
		setToolTipText("Binds unattached and unbound ports/roles to ports/roles in the parent representation");
	}

	/**
	 * @return
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 **/
	protected boolean calculateEnabled() {
		List selObj = this.getSelectedObjects();
		if (selObj.isEmpty())
			return false;
		Object obj = selObj.get(0);
		if (obj instanceof EditPart) {
			Object o = ((EditPart) obj).getModel();
			if (o instanceof AcmeElement) {
				AcmeElement elem = (AcmeElement) ((EditPart) obj).getModel();
				AcmeSystem sys = Helper.getAcmeSystem(elem);
				if (sys == null) return false;
				_parentRep = sys.getParentRepresentation();
				return sys != null && sys.getParentRepresentation() != null;
			}
		}
		return false;
	}
	/**
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 **/
	public void run(List unboundInner) {
		ASModel.current ().getRuleManager ().suspendEvaluation();
		boolean isConRep = ModelHelper.isConnectorRep(_parentRep.getSystem ());
		BindingCommand command = new BindingCommand (_parentRep, null);
		command.setPointsToBind(unboundInner);
		// If there are now outer ports or roles, then obviously everything must be created
		if ((isConRep && ((AcmeConnector )_parentRep.getParent ()).getRoles ().getSize () == 0) || ((!isConRep && ((AcmeComponent )_parentRep.getParent ()).getPorts ().getSize () == 0))) {
			command.setCreateAll (true);
		}
		else {
			AcmeEnumeration outers = null;
			if (isConRep) outers = ((AcmeConnector )_parentRep.getParent ()).getRoles ();
			else outers = ((AcmeComponent )_parentRep.getParent ()).getPorts ();
			// Try to do disambiguation
			command.setCreateAll (false);
	
			if (!isConRep) {
				List unboundOuter = Helper.convertAcmeEnumerationToList(outers);
				List currentlyBound = new ArrayList ();
				// Separate outer elements into bound and unbound
				AcmeEnumeration existingBindings = ((AcmeBindingsMap )_parentRep.getAbstractionMap()).getBindings ();
				while (existingBindings.moreItems ()) {
					AcmeBinding binding = (AcmeBinding )existingBindings.getNextItem ();
					if (unboundOuter.contains (binding.getOuter())) {
						unboundOuter.remove (binding.getOuter ());
						currentlyBound.add (binding.getOuter ());
					}
				}
				if (checkOneToOneCorrespondence (unboundInner, unboundOuter)) {
					command.setCreateAll (true);
				}
				else {
					MatchUnboundDialog dialog = new MatchUnboundDialog (Display.getCurrent().getActiveShell(), unboundInner, unboundOuter, currentlyBound, _parentRep); 
					if (dialog.open () == IDialogConstants.OK_ID) {
						command.setDisambiguation (dialog.getInnerToOuterMap ());
					}
				}
			}
		}
		if (command.canExecute ()) execute (command);
		ASModel.current ().getRuleManager ().resumeEvaluation();
	}

	/**
	 * @param unboundInner
	 * @param unboundOuter
	 * @return
	 */
	private boolean checkOneToOneCorrespondence(List unboundInner, List unboundOuter) {
		return false;
//		List check = new ArrayList (unboundOuter);
//		for (Iterator iter = unboundInner.iterator (); iter.hasNext ();) {
//			AcmeElement element = (AcmeElement )iter.next ();
//			if (unboundOuter.)
//		}
	}

	/**
	 * @return
	 * @see AcmeStudio.util.IMenuBuilder#buildMenu()
	 **/
	public IMenuManager buildMenu() {
		MenuManager manager = new MenuManager (getText ());
		WorkbenchPartAction allUnboundPorts = new WorkbenchPartAction (this.getWorkbenchPart()) {
			
			protected boolean calculateEnabled() {
				return ExportUnboundToParentAction.this.calculateEnabled();
			}

			public void run() {
				 ExportUnboundToParentAction.this.run (ModelHelper.collectUnboundBindingPointsInRepresentation(_parentRep));
			}
			
		};
		allUnboundPorts.setText("All unbound ports");
		manager.add (allUnboundPorts);
		
		WorkbenchPartAction selectedPorts = new WorkbenchPartAction (this.getWorkbenchPart()) {

			protected boolean calculateEnabled() {
				List selection = ExportUnboundToParentAction.this.getSelectedObjects();
				if (selection.isEmpty()) return false;
				boolean allPorts = true;
				for (Iterator iter = selection.iterator (); iter.hasNext () && allPorts;) {
					Object sel = (Object )iter.next ();
					if ((_parentRep.getParent() instanceof AcmeComponent  && !(sel instanceof AcmePort || sel instanceof PortEditPart)) ||
							(_parentRep.getParent () instanceof AcmeConnector && !(sel instanceof AcmeRole ||sel instanceof RoleEditPart))) {
						allPorts = false;
					}
				}
				return allPorts;
			}
	
			public void run() {
				List thePorts = new LinkedList ();
				for (Iterator iter = ExportUnboundToParentAction.this.getSelectedObjects().iterator (); iter.hasNext ();) {
					Object port = (Object )iter.next ();
					if (port instanceof AcmeElement) thePorts.add (port);
					else if (port instanceof EditPart) thePorts.add (((EditPart )port).getModel ());
				}
				ExportUnboundToParentAction.this.run (thePorts);
			}
		};
		selectedPorts.setText("Selected ports");
		manager.add (selectedPorts);
		return manager;
	}
	

}
