/*
 * Created on Oct 17, 2003
 * 
 */
package AcmeStudio.ui.editors.diagram.actions;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorPart;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.figures.AcmeVisElement;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.util.Helper;
import acmeLib.elements.*;
import acmeLib.tools.GraphWalker;
import acmeLib.tools.WalkerElementOperation;

/**
 * @author Bradley Schmerl
 * 
 * Action to highlight elements in the diagram, by dimming all others
 */
public class HighlightObjectsInDiagramAction extends SelectionAction {
    /** The ID of the action **/
    public static String ID = "$Highlight$";
    /** The editor that will display highlighted elements **/
    protected DiagramEditor _editor;
    
    /**
     * Constructs the action 
     * @param editor
     */
    public HighlightObjectsInDiagramAction(IEditorPart editor) {
        super(editor);
        _editor = (DiagramEditor )editor;
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
        return !getSelectedObjects().isEmpty ();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        
        // Do some list manipulation
       final List selection_ = Arrays.asList (Helper.getAcmeElementsFromSelection(getSelection()));
       final List selection = new ArrayList ();
       selection.addAll (selection_);  
       
       // This will contain the list of elements to dim
       final List toDim = new ArrayList ();
       
       // The operation to walk through the system, collecting elements to dim (the elements that are not included in the selection)
       WalkerElementOperation op = new WalkerElementOperation () {
           /* (non-Javadoc)
            * @see acmeLib.tools.WalkerElementOperation#doElementOp(acmeLib.elements.AcmeElement)
            */
           public void doElementOp(AcmeElement elt) {
               if (elt instanceof AcmeSystem || elt instanceof AcmeGroup) return;
           
               if (!selection.contains (elt)) {
                   toDim.add (elt);
               }
           
           }   
       };
       
       // Walk over the system
       GraphWalker.WalkOverAllElements(_editor.getAcmeSystem(), op, GraphWalker.WALK_TOP_DOWN);

       // Dim the unselected elements
       for (Iterator iter = toDim.iterator(); iter.hasNext();) {
           AcmeElement element = (AcmeElement) iter.next();
           GraphicalEditPart ep = (GraphicalEditPart )_editor.getEditPart(element);
           if (ep != null) {
               IFigure figure = ep.getFigure();
               dimFigure (figure);
           }
       }
       _editor.setHighlightMode (true);
    }

    /**
     * @param figure
     */
    private void dimFigure(IFigure figure) {
        int transparency = AcmeStudioPlugin.getDefault().getPreferenceStore().getInt("DiagramEditor.highlightTransparency");
        if (figure instanceof AcmeVisElement) {
            AcmeVisElement ave = (AcmeVisElement) figure;
            ave.setTransparent (transparency);
            
        }
        else if (figure instanceof RoleView) {
            RoleView rv = (RoleView) figure;
            rv.setTransparent (transparency);
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.ui.actions.SelectionAction#setSelection(org.eclipse.jface.viewers.ISelection)
     */
    public void setSelection(ISelection selection) {
        super.setSelection(selection);
    }

}
