package AcmeStudio.ui.editors.diagram.actions;
/*
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;

import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeRole;

public class MoveBendpointCommand extends BendpointCommand {

    private PointList _oldBendpoints;
    private RoleEditPart _editPart = null;
    private SystemView _sv = null;

    public void setEditPart(RoleEditPart editPart) {
        this._editPart = editPart;
        this.setLabel("Move Bendpoint");
    }

    public void execute() {
        //AbsoluteBendpoint abp = new AbsoluteBendpoint(this.getLocation());
        AcmeRole role = getRole();
        _oldBendpoints = Helper.getBendpoints(role);
        RoleView rv = (RoleView) _editPart.getFigure();
        _sv = (SystemView) rv.getParent();
        rv._isBendpointMoving = false;
        Point roleStart = rv.getStart();
        Point roleEnd = rv.getEnd();
        PointList bendpoints =
            _sv.getBendpoints(role, roleStart, roleEnd, null);
        bendpoints =
            _sv.layoutBendpoints(role, bendpoints, roleStart, roleEnd);
        Helper.setBendpoint(role, getIndex(), getLocation(), bendpoints);
        bendpoints =
            _sv.getBendpoints(role, roleStart, roleEnd, null);
        bendpoints =
            _sv.layoutBendpoints(role, bendpoints, roleStart, roleEnd);
        SystemView.cleanBendpoints(bendpoints, roleStart, roleEnd);
        Helper.setBendpoints(role, bendpoints);
        super.execute();
        _editPart.refresh();
    }

    //    public boolean canExecute() {
    //        ConnectorVisualization.RoleRouteRule rule = Helper.getRoleRouteRule(getRole());
    //        if (rule != null
    //            && rule._roleRoute
    //                != ConnectorVisualization.RoleRouteOptions.STRAIGHT)
    //            return false;
    //        return true;
    //    }

    public void undo() {
        super.undo();
        if (_oldBendpoints != null) {
            AcmeRole role = getRole();
            Helper.setBendpoints(role, _oldBendpoints);
            RoleView rv = (RoleView) _editPart.getFigure();
            Point roleStart = rv.getStart();
            Point roleEnd = rv.getEnd();
            PointList bendpoints =
                _sv.getBendpoints(role, roleStart, roleEnd, null);
            bendpoints =
                _sv.layoutBendpoints(
                    getRole(),
                    bendpoints,
                    roleStart,
                    roleEnd);
            SystemView.cleanBendpoints(bendpoints, roleStart, roleEnd);                    
            Helper.setBendpoints(role, bendpoints);
            super.execute();
            _editPart.refresh();
        }
    }

}
