package AcmeStudio.ui.editors.diagram.actions;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.ui.editors.diagram.figures.PortView;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.PortEditPart;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.ui.editors.diagram.visualInfo.VisualInformation;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeAttachment;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;
import acmeLib.properties.AcmeFloat;
import acmeLib.properties.AcmeProperty;
/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * An implementation of AbstractCommand that controls Role movement
 * 
 */
public class MoveRoleCommand extends Command {
    /**
     * The attachment before move
     */
    private AcmeAttachment _oldAttachment = null;
    /**
     * The attachment after move
     */
    private AcmeAttachment _newAttachment = null;
    /**
     * The acme role this command is applied to
     */
    private AcmeRole _role = null;
    /**
     * The acme port to which the role is being attached to
     */
    private AcmePort _port = null;
    /**
     * The acme role editpart this command is applied to
     */
    private RoleEditPart _roleEditPart = null;
    /**
     * The acme system in which this command is executed
     */
    private AcmeSystem _acmeSystem = null;
    /**
     * The edit part to which the role is approaching
     */
    private EditPart _targetEditPart;
    /**
     * The location where this role is moved to
     */
    private Point _location = null;
    /**
     * The location where this role is moved from
     */
    private Point _locationBefore = null;
    /**
     * True if the role is attached to a port
     */
    private boolean _isAttachedToPort = false;
    /**
     * True if the role is attached to an available port
     */
    private boolean _isAttachedToAvailablePort = false;
    private PointList _oldBendpoints;
    private SystemView _sv = null;
	private Point	_oldConnectorLocation;
	private AcmeConnector	_connector;
    
    /**
     * Constructor
     */
    public MoveRoleCommand(
        EditPart targetEditPart,
        boolean isAttachedToPort,
        boolean isAttachedToAvailablePort) {
        super("Role Move");
        this._targetEditPart = targetEditPart;
        this._isAttachedToPort = isAttachedToPort;
        this._isAttachedToAvailablePort = isAttachedToAvailablePort;
    }
    /**
     * Retrieve the old attachment and store it in _oldAttachment
     */
    private void getOldAttachment() {
        if (_acmeSystem == null) {
            _acmeSystem = Helper.getRootAcmeSystem(_role);
        }
        if (_acmeSystem == null)
            return;
        AcmeEnumeration attachments = _acmeSystem.getAttachments(_role);
        if (attachments.getSize () > 0) _oldAttachment = (AcmeAttachment )attachments.getNextItem();
//        for (int i = 0; i < attachments.size(); i++) {
//            AcmeAttachment attachment =
//                (AcmeAttachment) attachments.elementAt(i);
//            if (attachment.getRole() == _role) {
//                _oldAttachment = attachment;
//                return;
//            }
//        }
    }
    /**
     * Detach the role from the old attachment
     */
    private void removeOldAttachment() {
        if (_acmeSystem == null) {
            _acmeSystem = Helper.getRootAcmeSystem(_role);
        }
        if (this._oldAttachment != null) {
            AcmePort acmePort = _oldAttachment.getPort();
            AcmeComponent acmeComponent = (AcmeComponent) acmePort.getParent();
            _acmeSystem.removeAttachment(_oldAttachment);
            AcmeStudioChangeManager.globalChangeManager.onRemoveAttachment(_acmeSystem, _oldAttachment, this);
            EditPart componentEditPart =
                (EditPart) _targetEditPart
                    .getRoot()
                    .getViewer()
                    .getEditPartRegistry()
                    .get(
                    acmeComponent);
                        PortEditPart portEditPart =
                            (PortEditPart) _targetEditPart
                                .getRoot()
                                .getViewer()
                                .getEditPartRegistry()
                                .get(
                                acmePort);
                        portEditPart.refresh ();
            //            PortView portView = (PortView) portEditPart.getFigure();
            ////            Visualization visStyle = portView.getVisualization();
            ////            visStyle.setFillColor(ElementColorConstants.portFillColor);
            ////            visStyle.setOutlineColor(ElementColorConstants.portOutlineColor);
            componentEditPart.refresh();
        }
    }
    /**
     * Detach the role from the new attachment
     */
    private void removeNewAttachment() {
        if (_acmeSystem == null) {
            _acmeSystem = Helper.getRootAcmeSystem(_role);
        }
        if (this._newAttachment != null) {
            AcmePort acmePort = _newAttachment.getPort();
            AcmeComponent acmeComponent = (AcmeComponent) acmePort.getParent();
            _acmeSystem.removeAttachment(_newAttachment);
            AcmeStudioChangeManager.globalChangeManager.onRemoveAttachment(_acmeSystem, _newAttachment, this);
            EditPart componentEditPart =
                (EditPart) _targetEditPart
                    .getRoot()
                    .getViewer()
                    .getEditPartRegistry()
                    .get(
                    acmeComponent);
            //            PortEditPart portEditPart =
            //                (PortEditPart) _targetEditPart
            //                    .getRoot()
            //                    .getViewer()
            //                    .getEditPartRegistry()
            //                    .get(
            //                    acmePort);
            //            PortView portView = (PortView) portEditPart.getFigure();
            //            Visualization visStyle = portView.getVisualization();
            //            visStyle.setFillColor(ElementColorConstants.portFillColor);
            //            visStyle.setOutlineColor(ElementColorConstants.portOutlineColor);
            componentEditPart.refresh();
            //            AcmeStudioChangeManager.globalChangeManager.onRemoveAttachment(
            //                _acmeSystem,
            //                _newAttachment,
            //                this);
        }
    }
    /**
     * Attach the role from the old attachment
     */
    private void restoreOldAttachment() {
        if (_acmeSystem == null) {
            _acmeSystem = Helper.getRootAcmeSystem(_role);
        }
        if (this._oldAttachment != null) {
            AcmePort acmePort = _oldAttachment.getPort();
            AcmeComponent acmeComponent = (AcmeComponent) acmePort.getParent();
            _acmeSystem.addAttachment(this._oldAttachment);
            AcmeStudioChangeManager.globalChangeManager.onAddAttachment(_acmeSystem, _oldAttachment, this);
            EditPart componentEditPart =
                (EditPart) _targetEditPart
                    .getRoot()
                    .getViewer()
                    .getEditPartRegistry()
                    .get(
                    acmeComponent);
            //            PortEditPart portEditPart =
            //                (PortEditPart) _targetEditPart
            //                    .getRoot()
            //                    .getViewer()
            //                    .getEditPartRegistry()
            //                    .get(
            //                    acmePort);
            //            PortView portView = (PortView) portEditPart.getFigure();
            //            Visualization visStyle = portView.getVisualization();
            //            visStyle.setFillColor(ElementColorConstants.portFillColor);
            //            visStyle.setOutlineColor(ElementColorConstants.portOutlineColor);
            componentEditPart.refresh();
            //            AcmeStudioChangeManager.globalChangeManager.onAddAttachment(
            //                _acmeSystem,
            //                _oldAttachment,
            //                this);
        }
    }
    /**
     * Attach the role from the new attachment
     */
    private void addNewAttachment() {
        if (_acmeSystem == null) {
            _acmeSystem = Helper.getRootAcmeSystem(_role);
        }
        if (this._newAttachment != null) {
            _acmeSystem.addAttachment(this._newAttachment);
            AcmeStudioChangeManager.globalChangeManager.onAddAttachment(_acmeSystem, _newAttachment, this);
        }
    }
    /**
     * Return true if the role is attached to a port
     */
    public boolean isAttachedToPort() {
        return _isAttachedToPort;
    }
    /**     * Return true if the role is attached to a port without attachment
     */
    public boolean isAttachedToAvailablePort() {
        return _isAttachedToAvailablePort;
    }
    /**
     * @see org.eclipse.gef.commands.AbstraceCommand#canExecute
     */
    public boolean canExecute() {
        return true;
    }
    /**
     * @see org.eclipse.gef.commands.AbstraceCommand#execute
     */
    public void execute() {
        boolean hadLocation = false;
        if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
        	hadLocation = _role.getProperty("vis-x") != null;
        }
        else {
        	hadLocation = _role.getUserData("vis-information") != null;
        }
        _oldBendpoints = Helper.getBendpoints(_role);
        RoleView roleView = (RoleView) (_roleEditPart).getFigure();
        _locationBefore = roleView._endBeforeMoving;
        getOldAttachment();
        if (_targetEditPart instanceof PortEditPart) {
            PortEditPart portEditPart = (PortEditPart) _targetEditPart;
            _port = (AcmePort) portEditPart.getModel();
            if (_acmeSystem == null) {
                _acmeSystem = Helper.getRootAcmeSystem(_port);
            }
            PortView portView =
                (PortView) ((PortEditPart) _targetEditPart).getFigure();
            //            Visualization visStyle = portView.getVisualization();
            //            visStyle.setFillColor(ElementColorConstants.attachmentFillColor);
            //            visStyle.setOutlineColor(
            //                ElementColorConstants.attachmentOutlineColor);
            if (_acmeSystem.getAttachment(_port, _role) == null) {
                removeOldAttachment();
                _newAttachment = new AcmeAttachment(_port, _role);
                _acmeSystem.addAttachment(_newAttachment);
                AcmeStudioChangeManager.globalChangeManager.onAddAttachment(
                    _acmeSystem,
                    _newAttachment,
                    this);
                portEditPart.getParent().refresh();
            }
            Rectangle portBounds = portView.getBounds();
            _location = portBounds.getCenter();
            roleView.setEnd(_location);
            SystemView.setLocation(_role, _location);
            portEditPart.refresh();
        } else {
            removeOldAttachment();
            roleView.setEnd(_location);
            SystemView.setLocation(_role, _location);
        }
        roleView.setSelectionDistance(-1.0);
        if (_acmeSystem == null) {
            _acmeSystem = Helper.getRootAcmeSystem(_role);
        }
//        SystemEditPart systemEditPart =
//            (SystemEditPart) _roleEditPart
//                .getRoot()
//                .getViewer()
//                .getEditPartRegistry()
//                .get(
//                _acmeSystem);
//        SelectionHandlesEditPolicy selPolicy =
//            systemEditPart.getSelectionHandlesEditPolicy();

        roleView._isMoving = false;
//        if (selPolicy instanceof RoleEndpointEditPolicy) {
//            //((RoleEndpointEditPolicy) selPolicy).removeSelectionHandles();
//            //((RoleEndpointEditPolicy) selPolicy).createSelectionHandles();
//        }

        _roleEditPart.refresh();
        
        moveConnectorIfNecessary (roleView);

    	if (!AcmeStudioPlugin.getDefault() .getPreferenceStore().getBoolean (GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
    		
	        AcmeProperty visX = _role.getProperty("vis-x");
	        AcmeProperty visY = _role.getProperty("vis-y");
	        AcmeStudioChangeManager cMgr =
	            AcmeStudioChangeManager.globalChangeManager;
	        // Use update vis info announcement
	        VisualInformation vis = new VisualInformation ();
	        vis.setX((int )((AcmeFloat )visX.getValue()).getValue());
	        vis.setY((int )((AcmeFloat )visX.getValue ()).getValue ());
	        cMgr.onUpdateVisInformation(_role, vis, this);
    	}
    	else {
    		VisualInformation vis = (VisualInformation )_role.getUserData ("vis-information");
    		AcmeStudioChangeManager.globalChangeManager.onUpdateVisInformation(_role, vis, this);
    	}
/*        if (!hadLocation) {
            cMgr.onAddProperty(_role, visX, this);
            cMgr.onAddProperty(_role, visY, this);
        } else {
            cMgr.onSetProperty(_role, visX, visX.getValue(), this);
            cMgr.onSetProperty(_role, visY, visY.getValue(), this);
        }*/
    }
    /**
	 * 
	 */
	private void moveConnectorIfNecessary(RoleView roleView) {
//		AcmeRole role = (AcmeRole )roleView.getModel ();
//		_connector = (AcmeConnector )role.getParent ();
//		if (_connector.getRoles().getSize() == 2) {
//			ElementVisualization vis0 = VisualStyleRegistry.getVisualization (_connector);
//			ConnectorVisualization vis = (ConnectorVisualization )VisualStyleRegistry.getVisualization(_connector);
//			if (vis.autoCenterBinaryConnectors ()) {
//				_oldConnectorLocation = SystemView.getLocation (_connector, false);
//				List roleList = Helper.convertAcmeEnumerationToList(_connector.getRoles ());
//				Point rl1 = SystemView.getLocation ((AcmeElement )roleList.get (0), false);
//				Point rl2 = SystemView.getLocation ((AcmeElement )roleList.get (1), false);
//				Point center = new Rectangle (rl1, rl2).getCenter();
//				moveConnector (_connector, center);
//			}
//		}
//		if (((RoleVisualization )roleView.getVisualization()).autoCenterConnectors ()) {
//			AcmeConnector connector = (AcmeConnector )_role.getParent ();
//			// Need to get the connector's edit part
//			
//			// Get all the roles for the connector
//			
//			
//		}
	}
	/**
	 * @param connector
	 * @param location
	 */
	private void moveConnector(AcmeConnector connector, Point location) {
		if (!AcmeStudioPlugin.getDefault ().getPreferenceStore ().getBoolean (GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visX = connector.getProperty("vis-x");
		    AcmeProperty visY = connector.getProperty("vis-y");
		    AcmeStudioChangeManager cMgr =
		        AcmeStudioChangeManager.globalChangeManager;
		    // Use update vis info announcement
		    VisualInformation visInfo = new VisualInformation ();
		    visInfo.setX((int )((AcmeFloat )visX.getValue()).getValue());
		    visInfo.setY((int )((AcmeFloat )visY.getValue ()).getValue ());
		    cMgr.onUpdateVisInformation(_role, visInfo, this);
		}
		else {
			VisualInformation visInfo = (VisualInformation )connector.getUserData ("vis-information");
			visInfo.setX(location.x);
			visInfo.setY(location.y);
			AcmeStudioChangeManager.globalChangeManager.onUpdateVisInformation(connector, visInfo, this);
		}
	}
	/**
     * Return the target edit part
     * @return the target edit part
     */
    public EditPart getTargetEditPart() {
        return _targetEditPart;
    }
    /**
     * Set the role edit part
     * @param roleEditPart the edit part to be assigned
     */
    public void setRoleEditPart(RoleEditPart roleEditPart) {
        this._roleEditPart = roleEditPart;
        this._role = (AcmeRole) roleEditPart.getModel();
    }
    /**
     * Set the location to which the role is moved
     * @param location The destination of movement
     */
    public void setLocation(Point location) {
        this._location = location;
    }
    /**
     * @see org.eclipse.gef.commands.AbstractCommand#undo()
     */
    public void undo() {
        removeNewAttachment();
        restoreOldAttachment();
        RoleView roleView = (RoleView) (_roleEditPart).getFigure();
        if (_locationBefore != null) {
            roleView.setEnd(_locationBefore);
            SystemView.setLocation(_role, _locationBefore);
            restoreBendpoints();
        }
        if (_oldConnectorLocation != null) {
        	moveConnector (_connector, _oldConnectorLocation);
        }
    }
    /**
     * @see org.eclipse.gef.commands.AbstractCommand#redo()
     */
    public void redo() {
        removeOldAttachment();
        addNewAttachment();
        RoleView roleView = (RoleView) (_roleEditPart).getFigure();
        roleView.setEnd(_location);
        SystemView.setLocation(_role, _location);
    }
    private void restoreBendpoints() {
        if (_oldBendpoints != null) {
            Helper.setBendpoints(_role, _oldBendpoints);
        } else {
            Helper.clearBendpoints(_role);
        }
        RoleView rv = (RoleView) _roleEditPart.getFigure();
        _sv = (SystemView) rv.getParent();
        Point roleStart = rv.getStart();
        Point roleEnd = rv.getEnd();
        PointList bendpoints =
            _sv.getBendpoints(_role, roleStart, roleEnd, null);
        bendpoints =
            _sv.layoutBendpoints(_role, bendpoints, roleStart, roleEnd);
        SystemView.cleanBendpoints(bendpoints, roleStart, roleEnd);
        Helper.setBendpoints(_role, bendpoints);
        _roleEditPart.refresh();
    }
}
