/*
 * Created on Jul 2, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.util.OpenStrategy;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.util.IMenuBuilder;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeFamily;
import acmeLib.elements.AcmeImportedDesignRecord;
import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl
 * Created Jul 2, 2003
 */
public class NavigateToImportedFamilyAction
	extends SelectionAction
	implements IMenuBuilder {

	private String _targetFile;

	private boolean _isGroupAction = true;

	public static final String ID = "$OPENFAM$";

	public NavigateToImportedFamilyAction (IWorkbenchPart part)  {
		super (part);
		setText ("Open Family Editor");
		setId (ID);
	}
	
	public NavigateToImportedFamilyAction (IWorkbenchPart part, String familyName, String filename)  {
		super (part);
		setText (familyName);
		setId (ID);
		_targetFile = filename;
		setIndividualAction();
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		if (_isGroupAction)  {
			if (getSelectedObjects ().size () > 0)  {
				if (getSelectedObjects().get(0) instanceof SystemEditPart)  {
					return true;
				}
			}
			return false;
		}
			return true;
	}
	
	protected void setIndividualAction ()  {
		_isGroupAction = false;
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.util.IMenuBuilder#buildMenu()
	 */
	public IMenuManager buildMenu() {
		MenuManager manager = new MenuManager (getText ());
		if (getSelectedObjects ().size () > 0 && getSelectedObjects().get(0) instanceof SystemEditPart)  {
			SystemEditPart ep = (SystemEditPart )getSelectedObjects().get(0);
			AcmeSystem system = (AcmeSystem )ep.getModel();
			AcmeEnumeration families = system.getDeclaredTypes();
			while (families.moreItems()) {
				AcmeFamily family = (AcmeFamily )families.getNextItem ();
				if (family.getParentDesign() != system.getParentDesign ())  {
					// it is imported, find the design to open
					AcmeEnumeration ids = system.getParentDesign().getImportedDesigns().enumerate();
					while (ids.moreItems ())  {
						AcmeImportedDesignRecord rec = (AcmeImportedDesignRecord )ids.getNextItem();
						if (rec.getResolvedDesign().getFamily(family.getName ()) != null)  {
							NavigateToImportedFamilyAction action = new NavigateToImportedFamilyAction (getWorkbenchPart(), family.getName (), rec.getCanonicalFilename());
							manager.add (action);
						}
					}
					
				}
			}
		}
		return manager;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		if (!_isGroupAction)  {
			
			try {
				// TODO: Get the IFile from the canonical name
				IPath path = new Path (_targetFile);
				IFile file = AcmeStudioPlugin.getRoot().getFileForLocation(path);
                if (file != null) {
                    getWorkbenchPart().getSite().getPage ().openEditor(new FileEditorInput (file), "AcmeStudio.ui.editors.MultiPageAcmeEditor", OpenStrategy.activateOnOpen()) ;
                }
			} catch (PartInitException e) {
				e.printStackTrace();
			}
		}
	}

}
