package AcmeStudio.ui.editors.diagram.actions;

import java.util.List;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IEditorActionBarContributor;
import org.eclipse.ui.IEditorPart;
import AcmeStudio.core.ASContext;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeSystem;
import acmeLib.representations.AcmeRepresentation;

/**
 * @author Bradley Schmerl (schmerl@cs.cmu.edu)
 *
 * Provides an action for navigating up a representation
 * 
 */
public class NavigateUpAction extends SelectionAction {

    public static final String ID = "$NavUpRep$";

    protected AcmeElement _outerSystem;

	/**
	 * @param system
	 */
	public void setParentSystem(AcmeSystem system) {
		_outerSystem = system;
	}

    public NavigateUpAction(IEditorPart editor) {
        super(editor);
        setId(ID);
        setText("Navigate up representation");
        setToolTipText("Navigate to parent of representation");
        setImageDescriptor(
            ImageManager.getImageDescriptor(ImageManager.ID_NAV_UP));
//        setDisabledImageDescriptor(
//            ImageManager.getImageDescriptor(ImageManager.ID_NAV_UP_DISABLED));
    }

    /**
     * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {

        List selObj = this.getSelectedObjects();
        if (selObj.isEmpty())
            return false;
        Object obj = selObj.get(0);
        if (obj instanceof EditPart) {
            Object o = ((EditPart) obj).getModel();
            if (o instanceof AcmeElement) {
                AcmeElement elem = (AcmeElement) ((EditPart) obj).getModel();
                AcmeSystem sys = Helper.getAcmeSystem(elem);
                return sys != null && sys.getParentRepresentation() != null;
            }

        }
        return false;
        //        if (getEditorPart() instanceof DiagramEditor) {
        //            DiagramEditor editor = (DiagramEditor )getEditorPart ();
        //            EditPart ep = editor.getRootEditPart ();
        //            return editor.getAcmeSystem () != null && editor.getAcmeSystem().getParentRepresentation() != null;
        //
        //        }
        //        return false;
    }

    /**
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        DiagramEditor editor = (DiagramEditor) getWorkbenchPart();
        EditPart ep = editor.getRootEditPart();
        AcmeElement system = (AcmeElement) ep.getModel();
        AcmeRepresentation rep = system.getParentRepresentation();
        if (rep != null) {
            MultiPageAcmeEditor mp = ASContext.current().getActiveEditor();
            AcmeElement parentSystem = rep.getParent();
            if (parentSystem == null) parentSystem = _outerSystem;
     		if (!(parentSystem instanceof AcmeSystem) ) {
     			parentSystem = parentSystem.getParent ();
     		}
            if (mp != null) {
       
                mp.swapDiagramEditor(
                    (AcmeSystem) parentSystem,
                    (AcmeSystem) system);

            }
            // synch the tool bar
//            IContentOutlinePage cop =
//                (IContentOutlinePage) mp.getAdapter(IContentOutlinePage.class);
//            cop.setFocus();
//            IElementViewPage ev = (IElementViewPage )mp.getAdapter (IElementViewPage.class);
//            ev.setFocus();
//            mp.getActiveEditor().setFocus();
            MultiPageAcmeEditor mpe = ASContext.current ().getActiveEditor ();
            if (mpe != null) {
            	IEditorActionBarContributor abc = mpe.getEditorSite().getActionBarContributor();
            	abc.setActiveEditor (null);
            	abc.setActiveEditor (mpe);
            }

            //            try {
            //                editor.init (editor.getEditorSite(), new AcmeModelEditorInput (parentSystem));
            //            }
            //            catch (PartInitException e) {
            //            }
        }
    }

}
