package AcmeStudio.ui.editors.diagram.actions;

/*
 * (c) Copyright IBM Corp. 2000, 2002.
 * All Rights Reserved.
 */

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.internal.WorkbenchImages;

import AcmeStudio.ui.editors.DiagramEditor;

/**
 * Copies a property to the clipboard.
 */
public class PasteAcmeElementAction extends SelectionAction {

    public static final String ID =
        ActionFactory.PASTE.getId ();

    /**
     * Creates the action.
     */
    public PasteAcmeElementAction(DiagramEditor editor) {
        super(editor);
    }


    /** 
     * Updates enablement based on the current selection
     */
//    public void selectionChanged(IStructuredSelection sel) {
//        setEnabled(!sel.isEmpty());
//    }
    
    /**
     * Initializes this action's text and images.
     */
    protected void init() {
        super.init();
        setId(ID);
        setText("Paste");
        setToolTipText("Paste Acme Element from Clipboard");
        setImageDescriptor(
            WorkbenchImages.getImageDescriptor(
                ISharedImages.IMG_TOOL_PASTE));
        // As of Eclipse 3.0, IMG_TOOL_PASTE_HOVER == IMG_TOOL_PASTE
        setHoverImageDescriptor(
            WorkbenchImages.getImageDescriptor(
                ISharedImages.IMG_TOOL_PASTE));
        setDisabledImageDescriptor(
            WorkbenchImages.getImageDescriptor(
        ISharedImages.IMG_TOOL_PASTE_DISABLED));
        setAccelerator(SWT.CTRL | 'v');

        setEnabled(false);
    }

    /**
     * Create a command to remove the selected objects.
     *
     * @param objects The objects to be deleted.
     *
     * @return The command to remove the selected objects.
     */
    public static Command createPasteCommand(List objects) {
        if (objects.isEmpty())
            return null;
        Object obj = objects.get(0);
        if (!(obj instanceof EditPart))
            return null;

        EditPart editPart = (EditPart)obj;
        
        PasteElementCommand command = new PasteElementCommand(editPart);
        return command;
    }

    /**
     * Returns <code>true</code> if the selected objects can
     * be deleted.  Returns <code>false</code> if there are
     * no objects selected or the selected objects are not
     * {@link EditPart}s.
     */
    protected boolean calculateEnabled() {
        Transfer transferType = TextTransfer.getInstance();
        Clipboard clipboard = new Clipboard(Display.getCurrent());

        

        String text = (String) clipboard.getContents(transferType);
        return text != null && text.length () > 0 && getSelectedObjects ().size () > 0;
//        Command cmd = createPasteCommand(getSelectedObjects());
//        if (cmd == null)
//            return false;
//        return cmd.canExecute();
    }

    /**
     * Performs the delete action on the selected objects.
     */
    public void run() {
        execute(createPasteCommand(getSelectedObjects()));
    }

}
