package AcmeStudio.ui.editors.diagram.actions;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.viewers.*;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Display;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.core.ASModel;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.ui.editors.diagram.visualInfo.VisualInfoPersistor;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.*;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * An implementation of AbstractCommand that controls Role movement
 * 
 */
public class PasteElementCommand extends CreateElementCommand {

    /**
     * The acme element to be copied
     */
    private AcmeElement _acmeHost = null;

    private String _childName = null;

    private String _script = null;

    /**
     * The acme element to be copied
     */
    private AcmeSystem _acmeSystem = null;

    /**
     * System clipboard
     */
    private Clipboard _clipboard = null;

    /**
     * The elements that are to be (or have been) pasted
     */
    private List _pastedElements;

	private String	_visScript;

    /**
     * Constructor
     */
    public PasteElementCommand(EditPart editPart) {
        super(editPart);
        if (editPart != null) {
            Object obj = editPart.getModel();
            if (obj instanceof AcmeElement) {
                _acmeHost = (AcmeElement) obj;
                _clipboard = new Clipboard(Display.getCurrent());
                readClipboard();
                String toPaste = "";
                if (_childName != null) {
                    toPaste = " " + _childName;
                }
                setLabel("Paste" + toPaste);
                // BugFix #58 - discriminate system edit part if the system is selected
                SystemEditPart systemEditPart = null;
                if (obj instanceof AcmeSystem) {
                    systemEditPart = (SystemEditPart )editPart;
                }
                else {
                    systemEditPart =
                        (SystemEditPart) editPart.getParent();
                }
                SystemView systemView = (SystemView) systemEditPart.getFigure();
                setSystemView(systemView);
            }

        }
    }

    /**
     * @see org.eclipse.gef.commands.AbstraceCommand#canExecute
     */
    public boolean canExecute() {
        return _clipboard != null;
    }

    /**
     * Read data from the clipboard
     */
    private void readClipboard() {
        Transfer transferType = TextTransfer.getInstance();
        String text = (String) _clipboard.getContents(transferType);

        StringTokenizer st = new StringTokenizer(text, "\n");
        String tag = null;
        // This should be taken out when we have the AcmeTransfer
        if (st.hasMoreTokens()) {
            tag = st.nextToken();
            if (!tag.equals("AcmeElement")) {
                return;
            }
        }
        //        if (st.hasMoreTokens()) {
        //            _childType = st.nextToken();
        //        }
        //        if (st.hasMoreTokens()) {
        //            childName = st.nextToken();
        //        }
        //        if (st.hasMoreTokens()) {
        //            _parentName = st.nextToken();
        //        }
        //        _script = "";
        //        while (st.hasMoreTokens()) {
        //            _script += st.nextToken();
        //        }

        _script = text.substring("AcmeElement\n".length());

        _acmeSystem = Helper.getRootAcmeSystem(_acmeHost);

        //        Object obj = _acmeSystem.lookupName(childName, false);
        //        if (_childType.equals("Component")
        //            && _acmeHost instanceof AcmeSystem
        //            || _childType.equals("Connector")
        //            && _acmeHost instanceof AcmeSystem
        //            || _childType.equals("Port")
        //            && _acmeHost instanceof AcmeComponent
        //            || _childType.equals("Role")
        //            && _acmeHost instanceof AcmeConnector) {
        //            _childName = childName;
        //        }

    }

    /**
     * @see org.eclipse.gef.commands.AbstraceCommand#execute
     */
    public void execute() {
    	carveOffVisualInformation ();
        prepareToPaste(_script);
        doPaste();
        selectedPastedComponents ();
        //        super.execute();
    }

    /**
	 * 
	 */
	private void selectedPastedComponents() {
		final StructuredSelection sel = new StructuredSelection (_pastedElements);
		SelectionChangedEvent sce = new SelectionChangedEvent (new ISelectionProvider () {

			public void addSelectionChangedListener(ISelectionChangedListener listener) {
			}

			public ISelection getSelection() {
				return sel;
			}

			public void removeSelectionChangedListener(ISelectionChangedListener listener) {
			}

			public void setSelection(ISelection selection) {
			}
			
		}, sel);
		ASContext.current().getActiveEditor().getDiagramEditor().selectionChanged(sce);
	}

	/**
	 * 
	 */
	private void carveOffVisualInformation() {
		int visIndex = _script.indexOf("<?xml");
		if (visIndex != -1) {
			_visScript = _script.substring (visIndex);
			_script = _script.substring (0, visIndex - 1);
		}
	}

	/**
     * @see org.eclipse.gef.commands.AbstractCommand#undo()
     */
    public void undo() {
    	ASModel.current ().getRuleManager ().suspendEvaluation();
        Iterator i = _pastedElements.iterator();
        while (i.hasNext()) {
            AcmeElement el = (AcmeElement) i.next();
            if (el instanceof AcmeConnector) {
                ((AcmeSystem) _acmeHost).removeConnector((AcmeConnector) el);
                AcmeStudioChangeManager.globalChangeManager.onRemoveConnector(
                    (AcmeSystem) _acmeHost,
                    (AcmeConnector) el,
                    null);

            } else if (el instanceof AcmeComponent) {
                ((AcmeSystem) _acmeHost).removeComponent((AcmeComponent) el);
                AcmeStudioChangeManager.globalChangeManager.onRemoveComponent(
                    (AcmeSystem) _acmeHost,
                    (AcmeComponent) el,
                    null);
            } else if (el instanceof AcmePort) {
                ((AcmeComponent) _acmeHost).removePort((AcmePort) el);
                AcmeStudioChangeManager.globalChangeManager.onRemovePort(
                    (AcmeComponent) _acmeHost,
                    (AcmePort) el,
                    null);
            } else if (el instanceof AcmeRole) {
                ((AcmeConnector) _acmeHost).removeRole((AcmeRole) el);
                AcmeStudioChangeManager.globalChangeManager.onRemoveRole(
                    (AcmeConnector) _acmeHost,
                    (AcmeRole) el,
                    null);
            } else if (el instanceof AcmeAttachment) {
                ((AcmeSystem) _acmeHost).removeAttachment((AcmeAttachment) el);
                AcmeStudioChangeManager.globalChangeManager.onRemoveAttachment(
                    (AcmeSystem) _acmeHost,
                    (AcmeAttachment) el,
                    null);
            }
        }
        ASModel.current ().getRuleManager().resumeEvaluation();
    }

    /**
     * @see org.eclipse.gef.commands.AbstractCommand#redo()
     */
    public void redo() {
        doPaste();
    }

    private void prepareToPaste(String script) {
        setParent(_acmeHost);
        setSystem(_acmeSystem);

        AcmeSystem clippedSystem = parseAcmeSystem(_script);
        clippedSystem.resolveInstanceReferences();

        if (AcmeStudioPlugin.getDefault() .getPreferenceStore().getBoolean (GlobalVisPreferences.KEEP_VIS_SEPARATE) && _visScript != null) {
        	try {
				DocumentBuilder db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
				Document doc = db.parse(new ByteArrayInputStream (_visScript.getBytes()));
				AcmeDesign dummyDesign = new AcmeDesign ();
				dummyDesign.addSystem(clippedSystem);
				VisualInfoPersistor.importVisualInfo(dummyDesign, doc.getDocumentElement());
			}
			catch (AcmeIllegalNameException e) {
				e.printStackTrace();
			}
			catch (ParserConfigurationException e) {
				e.printStackTrace();
			}
			catch (FactoryConfigurationError e) {
				e.printStackTrace();
			}
			catch (SAXException e) {
				e.printStackTrace();
			}
			catch (IOException e) {
				e.printStackTrace();
			}
        }
        
        // Pull out all the elements.
        // If host is a component, only pull out ports
        // If host is a connector, only pull out roles
        // If host is a system, pull out connectors and components
        // if host is a component or connector, but there are only components and connectors
        //   to be pasted, make the host the parent of the host

        // Check types of each element. If the types don't exist in
        // the namespace, then remove the type from the clipboard
        // (This could depend on preferences, might want to import
        //  families to the system if the type doesn't exist)

        if (!(_acmeHost instanceof AcmeSystem)
            && ((clippedSystem.getComponents().getSize() != 0
                || clippedSystem.getConnectors().getSize() != 0))) {
            _acmeHost = _acmeHost.getParent();
        }

        if (_acmeHost instanceof AcmeSystem) {
            _pastedElements =
                Helper.convertAcmeEnumerationToList(
                    clippedSystem.getComponents());
            _pastedElements.addAll(
                Helper.convertAcmeEnumerationToList(
                    clippedSystem.getConnectors()));
            _pastedElements.addAll(
                Helper.convertAcmeEnumerationToList(
                    clippedSystem.getAttachments()));
        } else if (_acmeHost instanceof AcmeConnector) {
            _pastedElements =
                Helper.convertAcmeEnumerationToList(clippedSystem.getRoles());
        } else if (_acmeHost instanceof AcmeComponent) {
            _pastedElements =
                Helper.convertAcmeEnumerationToList(clippedSystem.getPorts());
        }

        //        stitchInTypes (_pastedElements);
        stitchInNames();
        stitchInPositions();

    }

    private void stitchInPositions() {
        Iterator i = _pastedElements.iterator();
        while (i.hasNext()) {
            AcmeElement el = (AcmeElement) i.next();
            Point location = SystemView.getLocation(el, true);
            location.translate(25, 25);
            SystemView.setLocation(el, location);
        }
    }

    private void stitchInNames() {
        Map nameMap = new HashMap();
        Iterator i = _pastedElements.iterator();

        while (i.hasNext()) {
            AcmeElement el = (AcmeElement) i.next();
            if (el instanceof AcmeAttachment) {
                //            	AcmeAttachment att = (AcmeAttachment )el;
                //            	AcmePort port =
                // Might not have to do anything 
            } else {

                String name = Helper.getUniqueName(_acmeHost, nameMap, el.getName());
                nameMap.put(el.getName(), name);
                try {
                    el.setName(name);

                } catch (AcmeIllegalNameException e) {
                    // Should never happen
                }
            }
        }
    }

    private void doPaste() {
        // Add each element and announce addition
        Iterator i = _pastedElements.iterator();
//        Debug.diagnoseln ("PasteElementCommand.doPaste: " + "start");
        ASModel.current ().getRuleManager ().suspendEvaluation ();
        while (i.hasNext()) {
            AcmeElement el = (AcmeElement) i.next();
            try {
                if (el instanceof AcmeConnector) {
                    ((AcmeSystem) _acmeHost).addConnector((AcmeConnector) el);
                    el.resolveTypeReferences();
                    el.instantiateElements();
                    el.resolveInstanceReferences();
                    AcmeStudioChangeManager.globalChangeManager.onAddConnector(
                        (AcmeSystem) _acmeHost,
                        (AcmeConnector) el,
                        null);

                } else if (el instanceof AcmeComponent) {
                    ((AcmeSystem) _acmeHost).addComponent((AcmeComponent) el);
                    el.resolveTypeReferences();
                    el.instantiateElements();
                    el.resolveInstanceReferences();
                    AcmeStudioChangeManager.globalChangeManager.onAddComponent(
                        (AcmeSystem) _acmeHost,
                        (AcmeComponent) el,
                        null);
                } else if (el instanceof AcmePort) {
                    ((AcmeComponent) _acmeHost).addPort((AcmePort) el);
                    
                    Point location = SystemView.getPortLocation_static((AcmePort) el);
//                        location.translate(25, 25);
                        SystemView.setLocation(el, location);

                        SystemView.normalizePortPosition(
                            (AcmePort) el,
                            (AcmeComponent) el.getParent(),
                            location);
                    
                    el.resolveTypeReferences();
                    el.resolveInstanceReferences();
                    AcmeStudioChangeManager.globalChangeManager.onAddPort(
                        (AcmeComponent) _acmeHost,
                        (AcmePort) el,
                        null);
                } else if (el instanceof AcmeRole) {
                    ((AcmeConnector) _acmeHost).addRole((AcmeRole) el);
                    el.resolveTypeReferences();
                    el.resolveInstanceReferences();
                    AcmeStudioChangeManager.globalChangeManager.onAddRole(
                        (AcmeConnector) _acmeHost,
                        (AcmeRole) el,
                        null);
                } else if (el instanceof AcmeAttachment) {
                    el.getParent().resolveInstanceReferences();
                    ((AcmeSystem) _acmeHost).addAttachment((AcmeAttachment) el);
                    AcmeStudioChangeManager
                        .globalChangeManager
                        .onAddAttachment(
                        (AcmeSystem) _acmeHost,
                        (AcmeAttachment) el,
                        null);
                }
            } catch (AcmeIllegalNameException e) {
                // Should never happen because the name has been set based
                // on existing name
                e.printStackTrace();
            }
        }
             {
//            	Debug.diagnoseln ("PasteElementCommand.doPaste: " + "end");
             	ASModel.current ().getRuleManager ().resumeEvaluation ();
            }

        // Make sure selection is the pasted stuff   
        //        Can't be done because the new things aren't added to the diagram yet...?
        //        _oldSelections = _editPart.getRoot().getViewer().getSelection();
        //        StructuredSelection newSel = new StructuredSelection ();
        //        Iterator i = _pastedElements.iterator
    }

    private AcmeSystem parseAcmeSystem(String script) {
        AcmeSystem result = null;

        InputStream inStream = new ByteArrayInputStream(script.getBytes());

        if (!ArmaniParser.parserConstructed()) {
            new ArmaniParser(inStream);
        }
        ArmaniParser.initDesign();
        ArmaniParser.ReInit(inStream);
        try {
            AcmeDesign acmeDesign = ArmaniParser.parse_AcmeDesign();
            //            acmeDesign.doPostParsingProcessing();

            result = acmeDesign.getSystem("ClippedSystem");
        } catch (ParseException e) {
        }

        return result;
    }

}
