package AcmeStudio.ui.editors.diagram.actions;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.ui.IWorkbenchPart;

import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.util.IMenuBuilder;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmePort;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 */
public class ResetPortAlignmentAction
    extends SelectionAction
    implements IMenuBuilder {

    private Request _request;

    private boolean isGroupAction = true;
    private boolean _isEmptyGroup = true;

    private AcmeComponent _acmeComponent;
    private String _portType;
    private LinkedList _ports;

    public ResetPortAlignmentAction(
        IWorkbenchPart editor,
        AcmeComponent acmeComponent,
        String portType,
        String type) {
        super(editor);
        if (type.equals(ResetPortAlignmentCommand.RESET_ALIGNMENT)) {
            setText("Reset Port Alignment");
        }
        setId(type);
        _request = new Request(type);
        _acmeComponent = acmeComponent;
        _portType = portType;
    }

    public ResetPortAlignmentAction(IWorkbenchPart editor, String type) {
        super(editor);
        if (type.equals(ResetPortAlignmentCommand.RESET_ALIGNMENT)) {
            setText("Reset Port Alignment");
        }
        setId(type);
        _request = new Request(type);
    }

    /**
     * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
        if (isGroupAction) {
            if (!_isEmptyGroup) {
                _isEmptyGroup = true;
                return true;
            }
        }

        return calculateElementEnabled();
    }

    protected boolean calculateElementEnabled() {
        if (_acmeComponent == null)
            return false;
        _ports = new LinkedList();
        AcmeEnumeration ae = _acmeComponent.getPorts();
        while (ae.moreItems()) {
            AcmePort port = (AcmePort) ae.getNextItem();
            AcmeEnumeration aeTypes = port.getDeclaredTypes();
            boolean portAdded = false;
            while (!portAdded && aeTypes.moreItems()) {
      			// Port types can be element types, too, so make this resilient to that
                AcmeElementType acmeElementType = (AcmeElementType) aeTypes.getNextItem();
                if (acmeElementType == null) continue;
				String portType =
                    (acmeElementType).getTypeName();
                if (portType != null && (portType.equals(_portType) || _portType.equals("All"))) {
                    _ports.add(port);
                    portAdded = true;
                }
            }
        }
        return _ports.size() > 0;

    }

    protected void setIndividualAction() {
        isGroupAction = false;
    }

    protected Set getAvailableTypes(AcmeElement element) {
        Set ret = new HashSet();
        if (element instanceof AcmeComponent) {
            AcmeEnumeration ae = ((AcmeComponent) element).getPorts();
            while (ae.moreItems()) {
                AcmePort port = (AcmePort) ae.getNextItem();
                AcmeEnumeration aeTypes = port.getDeclaredTypes();
                while (aeTypes.moreItems()) {
                    AcmeElementType type =
                        (AcmeElementType) aeTypes.getNextItem();
                    ret.add(type.getTypeName());
                }

            }
        }
        ret.add("All");
        return ret;
    }

    public IMenuManager buildMenu() {
        MenuManager manager = new MenuManager(getText());
        if (getSelectedObjects().size() != 1)
            return manager;
        if (getSelectedObjects().get(0) instanceof ComponentEditPart) {
            ComponentEditPart ep =
                (ComponentEditPart) getSelectedObjects().get(0);
            if (ep.getModel() instanceof AcmeComponent) {
                AcmeComponent element = (AcmeComponent) ep.getModel();

                if (_request
                    .getType()
                    .equals(ResetPortAlignmentCommand.RESET_ALIGNMENT)) {
                    Set types = getAvailableTypes(element);
                    if (types.size() > 1) {
                        _isEmptyGroup = false;
                    } else {
                        return manager;
                    }
                    Iterator i = types.iterator();
                    while (i.hasNext()) {
                        String typeName = (String) i.next();
                        ResetPortAlignmentAction action = null;
                        action =
                            new ResetPortAlignmentAction(
                                getWorkbenchPart(),
                                element,
                                typeName,
                                (String) _request.getType());
                        action.setIndividualAction();
                        action.setText(typeName);

                        manager.add(action);
                    }
                }
            }
        }
        return manager;
    }

    /**
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        if (!isGroupAction) {
            execute(getCommand());
        }
    }

    protected Command getCommand() {
        if (!isGroupAction) {
            Command command =
                new ResetPortAlignmentCommand(
                    (String) _request.getType(),
                    _acmeComponent,
                    _ports,
                    _portType,
                    null);
            return command;
        }
        return null;
    }

}
