package AcmeStudio.ui.editors.diagram.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.geometry.Point;

import AcmeStudio.model.ModelCommand;
import AcmeStudio.ui.editors.diagram.styles.*;
import AcmeStudio.ui.editors.diagram.styles.constants.*;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmePort;

/**
 * @author yh
 *
 */
public class ResetPortAlignmentCommand extends ModelCommand {

    private static final String RESET_ALIGNMENT_REQUEST =
        "$ResetPortAlignment$";

    public static final String RESET_ALIGNMENT = RESET_ALIGNMENT_REQUEST;

    private List _oldXList, _oldYList, _portList;
    private AcmeComponent _acmeComponent;
    private Object _announcer;
    public ResetPortAlignmentCommand(
        String commandType,
        AcmeComponent acmeComponent,
        List ports,
        String portType,
        Object announcer) {
        super(commandType);
        _announcer = announcer;
        _portList = ports;
        _acmeComponent = acmeComponent;
    }

    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute() {
        return (_portList != null) && (_portList.size() > 0);
    }

    /**
     * @see org.eclipse.gef.commands.Command#canUndo()
     */
    public boolean canUndo() {
        return true;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        setLabel("Reset Port Alignment of " + _acmeComponent.getName());
        Iterator iter = _portList.iterator();
        _oldXList = new ArrayList();
        _oldYList = new ArrayList();
        ComponentVisualization cv = null;
        Object vis = VisualStyleRegistry.getVisualization(_acmeComponent).copy(_acmeComponent);
        if (vis instanceof ComponentVisualization)
            cv = (ComponentVisualization) vis;

        if (cv != null) {
            while (iter.hasNext()) {
                AcmePort port = (AcmePort) iter.next();
                String portType =
                    ((AcmeElementType) port.getDeclaredTypes().getNextItem())
                        .getTypeName();
                PlacementRule rule = null;
                Iterator iterRules = cv.getPortPlacementRules().iterator();
                while (iterRules.hasNext()) {
                    PlacementRule aRule =
                        (PlacementRule) iterRules
                            .next();
                    if (aRule._type.equals(portType)) {
                        rule = aRule;
                        break;
                    }
                }

                Point x = null;
                Point y = null;
                if (rule != null) {
                    if (rule._alignment
                        == AlignmentOptions.LEFT
                        || rule._alignment
                            == AlignmentOptions.RIGHT
                        || rule._side
                            == SideOptions.LEFT
                        || rule._side
                            == SideOptions.RIGHT) {
                        x = Helper.getX(port);
                        if (x != null) {
                            Helper.clearX(port);
                        }
                    }
                    if (rule._alignment
                        == AlignmentOptions.TOP
                        || rule._alignment
                            == AlignmentOptions.BOTTOM
                        || rule._side
                            == SideOptions.TOP
                        || rule._side
                            == SideOptions.BOTTOM) {
                        y = Helper.getY(port);
                        if (y != null) {
                            Helper.clearY(port);
                        }
                    }
                    if (rule._alignment
                        == AlignmentOptions.CENTER) {
                        x = Helper.getX(port);
                        if (x != null) {
                            Helper.clearX(port);
                        }
                        y = Helper.getY(port);
                        if (y != null) {
                            Helper.clearY(port);
                        }
                    }
                    _oldXList.add(x);
                    _oldYList.add(y);
                }
            }
            AcmeStudioChangeManager.globalChangeManager.onAlignPorts(
                _acmeComponent,
                _announcer);
        }
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
    }

    /**
     * This doesn't really undo instantiations
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        Iterator iter = _portList.iterator();
        int i = 0;
        while (iter.hasNext()) {
            AcmePort port = (AcmePort) iter.next();
            Point x = (Point) _oldXList.get(i);
            Point y = (Point) _oldYList.get(i);
            if (x != null) {
                Helper.setX(port, x.x);
            }
            if (y != null) {
                Helper.setY(port, y.y);
            }
            i++;
        }
        AcmeStudioChangeManager.globalChangeManager.onAlignPorts(
            _acmeComponent,
            _announcer);
    }
}
