package AcmeStudio.ui.editors.diagram.actions;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.ui.IWorkbenchPart;

import AcmeStudio.ui.editors.diagram.parts.ConnectorEditPart;
import AcmeStudio.util.IMenuBuilder;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeRole;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 */
public class ResetRoleRouteAction
    extends SelectionAction
    implements IMenuBuilder {

    private Request _request;

    private boolean isGroupAction = true;
    private boolean _isEmptyGroup = true;

    private AcmeConnector _acmeConnector;
    private String _roleType;
    private LinkedList _roles;

    public ResetRoleRouteAction(
        IWorkbenchPart editor,
        AcmeConnector acmeConnector,
        String roleType,
        String type) {
        super(editor);
        if (type.equals(ResetPortAlignmentCommand.RESET_ALIGNMENT)) {
            setText("Reset Port Alignment");
        }
        setId(type);
        _request = new Request(type);
        _acmeConnector = acmeConnector;
        _roleType = roleType;
    }

    public ResetRoleRouteAction(IWorkbenchPart editor, String type) {
        super(editor);
        if (type.equals(ResetRoleRouteCommand.RESET_ROUTE)) {
            setText("Reset Role Route");
        }
        setId(type);
        _request = new Request(type);
    }

    /**
     * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
        if (isGroupAction) {
            buildMenu();
            if (!_isEmptyGroup) {
                _isEmptyGroup = true;
                return true;
            }
        }

        return calculateElementEnabled();
    }

    protected boolean calculateElementEnabled() {
        if (_acmeConnector == null)
            return false;
        _roles = new LinkedList();
        AcmeEnumeration ae = _acmeConnector.getRoles();
        while (ae.moreItems()) {
            AcmeRole role = (AcmeRole) ae.getNextItem();
            if (_roleType.equals("All")) {
                _roles.add(role);
                continue;
            }
            AcmeEnumeration aeTypes = role.getDeclaredTypes();
            boolean roleAdded = false;
            while (!roleAdded && aeTypes.moreItems()) {
                String roleType =
                    ((AcmeElementType) aeTypes.getNextItem()).getTypeName();
                if (roleType.equals(_roleType)) {
                    _roles.add(role);
                    roleAdded = true;
                }
            }
        }
        return _roles.size() > 0;

    }

    protected void setIndividualAction() {
        isGroupAction = false;
    }

    protected Set getAvailableTypes(AcmeElement element) {
        Set ret = new HashSet();
        if (element instanceof AcmeConnector) {
            AcmeEnumeration ae = ((AcmeConnector) element).getRoles();
            while (ae.moreItems()) {
                AcmeRole role = (AcmeRole) ae.getNextItem();
                AcmeEnumeration aeTypes = role.getDeclaredTypes();
                while (aeTypes.moreItems()) {
                    AcmeElementType type =
                        (AcmeElementType) aeTypes.getNextItem();
                    ret.add(type.getTypeName());
                }

            }
        }
        ret.add("All");
        return ret;
    }

    public IMenuManager buildMenu() {
        MenuManager manager = new MenuManager(getText());
        if (getSelectedObjects().size() != 1)
            return manager;
        if (getSelectedObjects().get(0) instanceof ConnectorEditPart) {
            ConnectorEditPart ep =
                (ConnectorEditPart) getSelectedObjects().get(0);
            if (ep.getModel() instanceof AcmeConnector) {
                AcmeConnector element = (AcmeConnector) ep.getModel();

                if (_request
                    .getType()
                    .equals(ResetRoleRouteCommand.RESET_ROUTE)) {
                    Set types = getAvailableTypes(element);
                    if (types.size() > 0) {
                        _isEmptyGroup = false;
                    } else {
                        return manager;
                    }
                    Iterator i = types.iterator();
                    while (i.hasNext()) {
                        String typeName = (String) i.next();
                        ResetRoleRouteAction action = null;
                        action =
                            new ResetRoleRouteAction(
                                getWorkbenchPart(),
                                element,
                                typeName,
                                (String) _request.getType());
                        action.setIndividualAction();
                        action.setText(typeName);

                        manager.add(action);
                    }
                }
            }
        }
        return manager;
    }

    /**
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        if (!isGroupAction) {
            execute(getCommand());
        }
    }

    protected Command getCommand() {
        if (!isGroupAction) {
            Command command =
                new ResetRoleRouteCommand(
                    (String) _request.getType(),
                    _acmeConnector,
                    _roles,
                    _roleType,
                    null);
            return command;
        }
        return null;
    }

}
