package AcmeStudio.ui.editors.diagram.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.geometry.PointList;

import AcmeStudio.model.ModelCommand;
import AcmeStudio.ui.editors.diagram.styles.ConnectorVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeRole;

/**
 * @author yh
 *
 */
public class ResetRoleRouteCommand extends ModelCommand {

    private static final String RESET_ROUTE_REQUEST = "$ResetRoleRoute$";

    public static final String RESET_ROUTE = RESET_ROUTE_REQUEST;

    private List _oldBendpointsList, _roleList;
    private AcmeConnector _acmeConnector;
    private Object _announcer;
    public ResetRoleRouteCommand(
        String commandType,
        AcmeConnector acmeConnector,
        List roles,
        String roleType,
        Object announcer) {
        super(commandType);
        _announcer = announcer;
        _roleList = roles;
        _acmeConnector = acmeConnector;
    }

    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute() {
        return (_roleList != null) && (_roleList.size() > 0);
    }

    /**
     * @see org.eclipse.gef.commands.Command#canUndo()
     */
    public boolean canUndo() {
        return true;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        setLabel("Reset Role Route of " + _acmeConnector.getName());
        Iterator iter = _roleList.iterator();
        _oldBendpointsList = new ArrayList();
        ConnectorVisualization cv = null;
        Object vis =
            VisualStyleRegistry.getVisualization(_acmeConnector).copy(
                _acmeConnector);
        if (vis instanceof ConnectorVisualization)
            cv = (ConnectorVisualization) vis;

        if (cv != null) {
            while (iter.hasNext()) {
                AcmeRole role = (AcmeRole) iter.next();
                AcmeElementType aet = (AcmeElementType) role.getDeclaredTypes().getNextItem(); 
                String roleType = aet != null ? aet.getTypeName() : "";
                ConnectorVisualization.RoleRouteRule rule = null;
                Iterator iterRules = cv.getRoleRouteRules().iterator();
                while (iterRules.hasNext()) {
                    ConnectorVisualization.RoleRouteRule aRule =
                        (ConnectorVisualization.RoleRouteRule) iterRules.next();
                    if (aRule._roleType.equals(roleType)
                        || aRule._roleType.equals("role")) {
                        rule = aRule;
                        break;
                    }
                }

                PointList bendpoints = null;
                if (rule != null) {
                    bendpoints = Helper.getBendpoints(role);
                    Helper.clearBendpoints(role);
                }
                _oldBendpointsList.add(bendpoints);
            }
            AcmeStudioChangeManager.globalChangeManager.onRouteRoles(
                _acmeConnector,
                _announcer);
        }
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
    }

    /**
     * This doesn't really undo instantiations
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        Iterator iter = _roleList.iterator();
        int i = 0;
        while (iter.hasNext()) {
            AcmeRole role = (AcmeRole) iter.next();
            PointList bendpoints = (PointList) _oldBendpointsList.get(i);
            if (bendpoints != null) {
                Helper.setBendpoints(role, bendpoints);
            }
            i++;
        }
        AcmeStudioChangeManager.globalChangeManager.onRouteRoles(
            _acmeConnector,
            _announcer);
    }
}
