package AcmeStudio.ui.editors.diagram.actions;

import java.util.HashMap;

import AcmeStudio.model.ModelCommand;
import AcmeStudio.ui.editors.diagram.figures.SystemView;

/**
 * @author yh
 *
 */
public class ResetSystemLayoutCommand extends ModelCommand {

    private static final String RESET_SYSTEMLAYOUT_REQUEST =
        "$ResetSystemLayout$";

    public static final String RESET_SYSTEMLAYOUT = RESET_SYSTEMLAYOUT_REQUEST;

    private SystemView _systemView;
    private HashMap _undoMap;
    
    public ResetSystemLayoutCommand(
        String commandType,
        SystemView systemView) {
        super(commandType);
        _systemView = systemView;
    }

    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute() {
        return _systemView != null;
    }

    /**
     * @see org.eclipse.gef.commands.Command#canUndo()
     */
    public boolean canUndo() {
        return _systemView != null;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        setLabel("Reset System Layout");
        _undoMap = _systemView.resetLayout();
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
    }

    /**
     * This doesn't really undo instantiations
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        _systemView.undoResetLayout(_undoMap);
    }
}
