/*
 * Created on Aug 4, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package AcmeStudio.ui.editors.diagram.actions;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Display;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.model.ModelCommand;
import AcmeStudio.prefs.ConnectionPatternPreferences;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.ui.editors.diagram.parts.PortEditPart;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Geometry;
import AcmeStudio.util.Helper;
import AcmeStudio.wizards.ConnectionPatternWizard;
import AcmeStudio.wizards.connection.PortConnectionRule;
import AcmeStudio.wizards.connection.PortRoleAssociation;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.AcmeAttachment;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeComponentType;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmePortType;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;

/**
 * @author Larry Waldman
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class RoleConnectCommand extends ModelCommand
{
    //The component the role is being connected to
    private AcmeComponent _component;
    
    //the role being connected
    private AcmeRole _role;
    
    private PortConnectionRule _rule;
    private AcmeConnector _conn;
    private List _rules = null;
    private List _portTypes = null; 
    private AcmePort undoPort = null;
    private boolean connConnection;
    private boolean connSource;
    private Object _announcer;
    private AcmeSystem _system;
    
    /**
     * The location where this role is moved to
     */
    private Point _location = null;
    /**
     * The location where this role is moved from
     */
    private Point _locationBefore = null;
    
    /**
     * The attachment before move
     */
    private AcmeAttachment _oldAttachment = null;
    /**
     * The attachment after move
     */
    private AcmeAttachment _newAttachment = null;

	private DiagramEditor _diagramEditor;
    
    /**
     * C'Tor
     * 
     * @author LFW
     * 
     * @param the component target of the connection
     * @param the role being connected to the target
     */
    public RoleConnectCommand(AcmeComponent target, AcmeRole role)
    {
        super("RoleConnectCommand");
        _component = target;
        _role = role;
        _conn = (AcmeConnector)role.getParent();
        _rules = null;
        
        _diagramEditor = (DiagramEditor) ASContext.current()
        .getActiveEditor().getDiagramEditor();
        
        EditPart ep = (RoleEditPart) _diagramEditor.getEditPart(role);

        _system = Helper.getRootAcmeSystem(_role);
    }
    
    /**
     * C'Tor
     * 
     * @author LFW
     * 
     * @param the component target of the connection
     * @param the role being connected to the target
     * @param A list of rules that the role, connector, and target match
     */
    public RoleConnectCommand(AcmeComponent target, AcmeRole role, List rules)
    {
        super("RoleConnectCommand");
        _component = target;
        _role = role;
        _conn = (AcmeConnector)role.getParent();
        _rules = rules;
        
        _diagramEditor = (DiagramEditor) ASContext.current()
        .getActiveEditor().getDiagramEditor();
        
        EditPart ep = (RoleEditPart) _diagramEditor.getEditPart(role);
        
        _system = Helper.getRootAcmeSystem(_role);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#canUndo()
     */
    public boolean canUndo()
    {
        return true;
    }
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return true;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        _portTypes = new ArrayList();
		RoleView roleView = (RoleView) ((RoleEditPart)_diagramEditor.getEditPart(_role)).getFigure();
        _locationBefore = roleView._endBeforeMoving;
        
        ConnectionPatternWizard NRwizard;
        
        //initialize the items to make up the general rule
        AcmePortType[] pt = new AcmePortType[2];
        AcmeComponentType[] ct = new AcmeComponentType[2];
        
        //get the old attachment and store the info
        getOldAttachment();
        
        ct[1] = (AcmeComponentType)_component.getDeclaredTypes().getNextItem();
        
        //Add a rule that will give us the functionality we want
        PortConnectionRule newRule = new PortConnectionRule();
        newRule.setConnectorType(getAEType(_conn));
        PortRoleAssociation assoc = new PortRoleAssociation("", getAEType(_role), "", "", getAEType(_component));
        newRule.addAssociation(assoc);
        newRule.addAssociation(assoc);
        
        //If restrict to rules is set, then _rules already has values, filter them
        //if not, give it the defining rule that was constructed above
        if (!AcmeStudioPlugin.getDefault ().getPreferenceStore().getBoolean
                (ConnectionPatternPreferences.RESTRICT_TO_RULES))
            _rules.add(newRule);
        
        if (_rules.size() != 0)
        {
            if (!AcmeStudioPlugin.getDefault ().getPreferenceStore().getBoolean
                (ConnectionPatternPreferences.RESTRICT_TO_RULES) || _rules.size() > 1)
            {
	            //There is no specific rule to use, so use the wizard to let the user choose
	            NRwizard = new ConnectionPatternWizard(_system, _rules, pt, ct, false);
	            
	            WizardDialog dialog = new WizardDialog(Display.getDefault()
	                    .getActiveShell(), NRwizard);
	            dialog.create();
	            dialog.open();
	            if (!(IDialogConstants.CANCEL_ID == dialog.getReturnCode()))
	            {
	                // if the user didn't cancel
	
	                //Build the rule out of the different parts on the dialog
	                PortConnectionRule userChosen = new PortConnectionRule();
	                userChosen.setConnectorType(NRwizard.getTypePage().selectionConn);
	                userChosen.addAssociation(new PortRoleAssociation(
	                        NRwizard.getTypePage().selectionPorts[0], NRwizard.getTypePage().selectionRoles[0], "r"));
	                userChosen.addAssociation(new PortRoleAssociation(
	                        NRwizard.getTypePage().selectionPorts[1], NRwizard.getTypePage().selectionRoles[1], "r"));
	
	                _rule = userChosen;
	            }
            }
            else
                _rule = (PortConnectionRule)_rules.get(0);
            
            if (_rule != null)
            {    
	            AcmeSystem context = Helper.getAcmeSystem(ASContext.current()
	                    .getActiveDesign().getDesign());
	            Object portTypeObj = context.lookupName(
	                    _rule.getAssociations()[0].getPortType(), true);
	            Object portTypeObj2 = context.lookupName(
	                    _rule.getAssociations()[1].getPortType(), true);
	
	            if (portTypeObj instanceof AcmePortType
	                    && portTypeObj2 instanceof AcmePortType || _conn != null)
	            {
	                //Set the vars necessary for actually creating the connection
	                AcmePortType pt1 = (AcmePortType) portTypeObj;
	                AcmePortType pt2 = (AcmePortType) portTypeObj2;
	                
	                _portTypes = new ArrayList();
	                _portTypes.add(pt1);
	                _portTypes.add(pt2);
	
	                if (_conn != null)
	                {
	                    connConnection = true;
	                    connSource = true;
	                }
	                else
	                {
	                    connConnection = false;
	                    connSource = false;
	                }
	                
	                if (_oldAttachment != null) 
	                {
	                    removeOldAttachment();
	                }
	                
	                AcmePort newP = createNecessaryPort();
	                
	                //set the undoPort in case this port needs to be removed
	                undoPort = newP;
	                
	                _newAttachment = new AcmeAttachment(newP, _role);
	                
	                _system.addAttachment(_newAttachment);
					AcmeStudioChangeManager.globalChangeManager
					.onAddAttachment(_system, _newAttachment, _announcer);
					
			        _diagramEditor.getEditPart(_system).refresh();
			        _diagramEditor.getEditPart(_system).activate();
					((PortEditPart )_diagramEditor.getEditPart(_newAttachment.getPort())).refreshAttachments();
	            }
            }
        }
    }

    /**
     * @param newRule
     * @return
     */
    private List SystemVisualization(PortConnectionRule newRule)
    {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * This method uses pre-set variables and creates the necessary port
     * to connect the role to
     * 
     * @author LFW
     * 
     * @return AcmePort the port that was created
     */
    private AcmePort createNecessaryPort()
    {
        //it is necessary to create the target port on the component
        
        //First, get the edit part representing the component
       
        
        ComponentEditPart ep = (ComponentEditPart) _diagramEditor.getEditPart(_component);
        
        AcmePortType t = (AcmePortType) _portTypes.get(1);
        
        return createPort(Helper.getUniqueName((AcmeComponent) _component, 
                _rule.getAssociations()[1].getPortName() != null ? _rule.getAssociations()[1].getPortName() : "p"),
                (AcmeComponent) ep.getModel(),
                t, ep.getFigure().getBounds().getCenter());  
    }
    
    /**
     * This method does the low level creation of the port, as specified by
     * createNecessaryPort.
     * 
     * @author LFW
     * 
     * @param String the name of the port
     * @param AcmeComponent the component to attach the port to
     * @param AcmePortType the type of the port to be created
     */
	protected AcmePort createPort(String name, AcmeComponent component, 
	    	AcmePortType type, Point location)
	{
		name = Helper.getUniqueName(component, name);
		AcmePort port = null;
		// BRS: Changed create port to handle types correctly.
		port = new AcmePort (name);
		if (type != null) {
			port.addDeclaredType(type);
			port.addInstantiatedType(type);
			port.instantiateElements();
		}
		port.setParentDesign (component.getParentDesign ());
		
		Point compLocation = null;
		Point compLocation2 = null;
		
		//Set the location of the connector and component
		compLocation = SystemView.getLocation(_conn, false);
		compLocation2 = SystemView.getLocation(component, false);
		    
		int side;
		
		//determine the location to create the port
		if (compLocation.x > compLocation2.x)
		    side = Geometry.getSideOfPoint (compLocation2, compLocation);
		else
		    side = Geometry.getOppositeSide(Geometry.getSideOfPoint(compLocation2, compLocation));
		    
		Point finalP = new Point();

		if (component != null)
		{
		        finalP = compLocation2;
		    
			//First do it for the first part of the connection
			if (side == PositionConstants.NORTH)
			    finalP.y = compLocation2.y - (Helper.getSize(component).height / 2);
			else if (side == PositionConstants.EAST)
			    finalP.x = compLocation2.x + (Helper.getSize(component).width / 2);
			else if (side == PositionConstants.SOUTH)
			    finalP.y = compLocation2.y + (Helper.getSize(component).height / 2);
			else if (side == PositionConstants.WEST)
			    finalP.x = compLocation2.x - (Helper.getSize(component).width / 2);
		}
		
		//finalP = finalP.translate(location);
		Point temp = new Point(finalP.x - location.x, finalP.y - location.y);		
		SystemView.setLocation(port, temp);
		_location = temp;

		try
		{
			component.addPort(port);
			if (type != null)
			{
				port.addDeclaredType(type);
				port.addInstantiatedType(type);
				port.instantiateElements();
			}
			AcmeStudioChangeManager.globalChangeManager.onAddPort(component, 
			port, 
			_announcer);
		}
		catch (AcmeIllegalNameException e)
		{
			return null;
		}
		return port;
	}
    
	/**
	 * This method takes in an AcmeElement and returns back a string containing
	 * its type.  It is safe to call, for if e has no declared type, "" is returned
	 * 
	 * @author LFW
	 * 
	 * @param AcmeElement the element to get the type of
	 * 
	 * @return String A string containing the type information
	 */
    private String getAEType(AcmeElement e)
    {
    	if (e.getDeclaredTypes().getSize () > 0) {
    		return ((AcmeElementType)e.getDeclaredTypes().getNextItem()).getTypeName();
    	}
    	else {
    		return "";
    	}
    }
    
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        //removeOldAttachment();
        AcmePort newP = createNecessaryPort();
        
        //set the undoPort in case this port needs to be removed
        undoPort = newP;
        
        _newAttachment = new AcmeAttachment(newP, _role);
        
        addNewAttachment();
      
        RoleView roleView = (RoleView) ((RoleEditPart)_diagramEditor.getEditPart(_role)).getFigure();
        roleView.setEnd(_location);
        SystemView.setLocation(_role, _location);
        ((PortEditPart )_diagramEditor.getEditPart(newP)).refreshAttachments();
    }
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        removeNewAttachment();

        RoleView roleView = (RoleView) ((RoleEditPart)_diagramEditor.getEditPart(_role)).getFigure();
        if (_locationBefore != null) {
            roleView.setEnd(_locationBefore);
            SystemView.setLocation(_role, _locationBefore);
        }
        
        removePort();
    }
    
	private void removePort()
	{
        AcmePort port = undoPort;
        
        if (_system.getAttachments(port).getSize() == 0)
            _component.removePort(port);
        
        AcmeStudioChangeManager.globalChangeManager.onRemovePort(_component, port, _announcer);
	}
	
	
	
	   /**
     * Retrieve the old attachment and store it in _oldAttachment
     */
    private void getOldAttachment() {
        if (_system == null) {
            _system = Helper.getRootAcmeSystem(_role);
        }
        if (_system == null)
            return;
        AcmeEnumeration attachments = _system.getAttachments(_role);
        if (attachments.getSize () > 0) _oldAttachment = (AcmeAttachment )attachments.getNextItem();
    }
    /**
     * Detach the role from the old attachment
     */
    private void removeOldAttachment() {
        if (_system == null) {
            _system = Helper.getRootAcmeSystem(_role);
        }
        if (this._oldAttachment != null) {
            AcmePort acmePort = _oldAttachment.getPort();
            AcmeComponent acmeComponent = (AcmeComponent) acmePort.getParent();
            _system.removeAttachment(_oldAttachment);
            AcmeStudioChangeManager.globalChangeManager.onRemoveAttachment(_system, _oldAttachment, this);
            
           
            EditPart componentEditPart =
                (EditPart) _diagramEditor.getEditPart(acmeComponent);
                        EditPart portEditPart =
                            _diagramEditor.getEditPart(acmePort);
                        portEditPart.refresh ();
            componentEditPart.refresh();
        }
    }
    /**
     * Detach the role from the new attachment
     */
    private void removeNewAttachment() {
        if (_system == null) {
            _system = Helper.getRootAcmeSystem(_role);
        }
        if (this._newAttachment != null) {
            AcmePort acmePort = _newAttachment.getPort();
            AcmeComponent acmeComponent = (AcmeComponent) acmePort.getParent();
            _system.removeAttachment(_newAttachment);
            AcmeStudioChangeManager.globalChangeManager.onRemoveAttachment(_system, _newAttachment, this);
           
            EditPart componentEditPart =
                _diagramEditor.getEditPart(acmeComponent);

            componentEditPart.refresh();
        }
    }

    /**
     * Attach the role from the new attachment
     */
    private void addNewAttachment() {
        if (_system == null) {
            _system = Helper.getRootAcmeSystem(_role);
        }
        if (this._newAttachment != null) {
            _system.addAttachment(this._newAttachment);
            AcmeStudioChangeManager.globalChangeManager.onAddAttachment(_system, _newAttachment, this);
            EditPart componentEditPart =
                _diagramEditor.getEditPart(_newAttachment.getPort().getParent ());
            
            ((PortEditPart )_diagramEditor.getEditPart(_newAttachment.getPort())).refreshAttachments();
	        _diagramEditor.getEditPart(_system).refresh();
	        _diagramEditor.getEditPart(_system).activate();
        }
    }
	
    
}
