package AcmeStudio.ui.editors.diagram.actions;
/*
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2001, 2002 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.RootEditPart;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.ruler.model.DiagramGuide;
import AcmeStudio.ui.editors.diagram.visualInfo.VisualInformation;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;
import acmeLib.properties.AcmeFloat;
import acmeLib.properties.AcmeProperty;

public class SetElementConstraintCommand
    extends org.eclipse.gef.commands.Command {

    private Dimension newSize;
    private Dimension oldSize;

    private Point newPos;
    private Point oldPos;
    private AcmeElement _element;
    private SystemView _systemView;
	private RootEditPart _root;
	private DiagramGuide _newHGuide;
	private int _newHAlignment;
	private DiagramGuide _newVGuide;
	private int _newVAlignment;
	private DiagramGuide _oldVGuide;
	private DiagramGuide _oldHGuide;
	private int _oldVAlignment;
	private int _oldHAlignment;

    public SetElementConstraintCommand(GraphicalEditPart editPart) {
        super();
        /* BugFix#309: Don't save the editpart, because it will be different on redo, so look it up each time we need it */
        _element = (AcmeElement )editPart.getModel ();
        _root = editPart.getRoot ();
    }

    protected void resize(Dimension size) {
        Helper.setSize(_element, size);
        VisualInformation vis = null;
        if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
	        AcmeProperty visWidth = _element.getProperty("vis-width");
	        AcmeProperty visHeight = _element.getProperty("vis-height");
	        
	        vis = new VisualInformation ();
	        vis.setWidth((int )((AcmeFloat )visWidth.getValue ()).getValue ());
	        vis.setHeight((int )((AcmeFloat )visHeight.getValue ()).getValue ());
        }
        else {
        	vis = Helper.getVisualInformation(_element);
        	
        }
        AcmeStudioChangeManager cMgr =
        	AcmeStudioChangeManager.globalChangeManager;
        cMgr.onUpdateVisInformation(_element, vis, this);

    }

    protected void move(Point location, boolean calcCenter) {

        if (_element instanceof AcmePort) {
            Point moveTo = null;
          	Dimension portSize = Helper.getSize (_element);
            if (!calcCenter) {
                moveTo = location.getCopy ().translate(portSize.width/2, portSize.height/2);
            } else {
                AcmeComponent parent = (AcmeComponent) _element.getParent();
                Point compLocation = _systemView.getElementLocation(parent);
                moveTo = new Point();
                moveTo.x = location.x - compLocation.x;// - compSize.width / 2;
                moveTo.y = location.y - compLocation.y;// - compSize.height / 2;
                moveTo.x += portSize.width/2;
                moveTo.y += portSize.height/2;
                _systemView.setElementLocation(_element, moveTo, true);
                SystemView.normalizePortPosition(
                    (AcmePort) _element,
                    parent,
                    moveTo);
            }
            _systemView.setElementLocation(_element, moveTo, true);
            calcCenter = false;
        }

        boolean hadLocation = true;
        if (_element.getProperty("vis-x") == null) {
            hadLocation = false;
        }

        if (calcCenter) {
            Rectangle bounds = new Rectangle(location, newSize);
            Point center = bounds.getCenter();
            SystemView.setLocation(_element, center);
        } else {
            if (!(_element instanceof AcmePort))
            SystemView.setLocation(_element, location);
        }

        if (!AcmeStudioPlugin.getDefault() .getPreferenceStore().getBoolean (GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
        	
        AcmeProperty visX = _element.getProperty("vis-x");
        AcmeProperty visY = _element.getProperty("vis-y");

        VisualInformation vis = new VisualInformation ();
        vis.setX ((int )((AcmeFloat )visX.getValue()).getValue ());
        vis.setY ((int )((AcmeFloat )visY.getValue ()).getValue ());
        
        
        AcmeStudioChangeManager cMgr =
            AcmeStudioChangeManager.globalChangeManager;
        cMgr.onUpdateVisInformation(_element, vis, this);
        }
        else {
        	VisualInformation vis = (VisualInformation )_element.getUserData ("vis-information");
        	AcmeStudioChangeManager.globalChangeManager.onUpdateVisInformation(_element, vis, this);
        }
//        if (!hadLocation) {
//            cMgr.onAddProperty(element, visX, this);
//            cMgr.onAddProperty(element, visY, this);
//        } else {
//            cMgr.onSetProperty(element, visX, visX.getValue(), this);
//            cMgr.onSetProperty(element, visY, visY.getValue(), this);
//        }

    }

    public void execute() {
        setLabel("Resize/Move " + _element.getName());
        if (oldSize != null && !oldSize.equals(newSize)) {
            resize(newSize);
        }
        if (oldPos == null || !oldPos.equals(new Rectangle(newPos, newSize).getCenter())) {
        	_oldVGuide = AcmeStudio.ui.editors.diagram.ruler.Helper.getGuide(_element, false);
        	_oldHGuide = AcmeStudio.ui.editors.diagram.ruler.Helper.getGuide(_element, true);
        	if (_oldVGuide != null) {
        		_oldVAlignment = ((Integer )_oldVGuide.getMap ().get (_element)).intValue ();
        	}
        	if (_oldHGuide != null) {
        		_oldHAlignment = ((Integer )_oldHGuide.getMap ().get (_element)).intValue ();
        	}
            move(newPos, true);
            changeGuide (_oldHGuide, _newHGuide, _newHAlignment);
            changeGuide (_oldVGuide, _newVGuide, _newVAlignment);
        }
        ((EditPart )_root.getViewer().getEditPartRegistry().get (_element)).refresh ();
    }

    /**
	 * @param guide
	 * @param guide2
	 * @param alignment
	 */
	private void changeGuide(DiagramGuide oldGuide, DiagramGuide newGuide, int alignment) {
		
		if (oldGuide != null && oldGuide != newGuide) {
			oldGuide.detachElement(_element);
		}
		if (newGuide != null) {
			newGuide.attachElement(_element, alignment);
		}
	}

	public String getDescription() {
        String name = _element.getQualifiedName();
        return "SetConstraint " + name;
    }

    public void redo() {
        execute();
    }

    public void setLocation(Rectangle r) {
        newPos = r.getLocation();
        newSize = r.getSize();
        if ((newSize.width ==0)|| (newSize.height ==0)) {
         	newSize = oldSize;
        }
    }

    public void setLocation(Point p) {
        newPos = p;
    }

    public void setElement(AcmeElement element) {
        this._element = element;
        boolean shouldSetLocation = true;
        if (element instanceof AcmePort) {
            shouldSetLocation = false;
            if (_systemView != null)
            	oldPos = _systemView.getPortLocation((AcmePort)element);
            else 
            	oldPos = SystemView.getPortLocation_static ((AcmePort )element);
        } else {
            oldPos = SystemView.getLocation(element, shouldSetLocation);
        }
        oldSize = Helper.getSize(element);
    }

    public void undo() {
        resize(oldSize);
        move(oldPos, false);
        changeGuide (_newVGuide, _oldVGuide, _oldVAlignment);
        changeGuide (_newHGuide, _oldHGuide, _oldHAlignment);
        
        
        GraphicalEditPart editPart = (GraphicalEditPart )_root.getViewer().getEditPartRegistry().get (_element);
        editPart.refresh();
    }

    /**
     * @return SystemView
     */
    public SystemView getSystemView() {
        return _systemView;
    }

    /**
     * Sets the _systemView.
     * @param _systemView The _systemView to set
     */
    public void setSystemView(SystemView _systemView) {
        this._systemView = _systemView;
    }

	/**
	 * @param object
	 * @param alignment
	 */
	public void setHorizontalGuide(DiagramGuide guide, int alignment) {
		_newHGuide = guide;
		_newHAlignment = alignment;
	}

	/**
	 * @param object
	 * @param alignment
	 */
	public void setVerticalGuide(DiagramGuide guide, int alignment) {
		_newVGuide = guide;
		_newVAlignment = alignment;
	}

}
