/*
 * Created on Oct 15, 2003
 * 
 * To change the template for this generated file go to Window - Preferences -
 * Java - Code Generation - Code and Comments
 */
package AcmeStudio.ui.editors.diagram.actions;

import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.dialogs.ListSelectionDialog;

import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.figures.AcmeVisElement;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeGroup;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;

/**
 * @author schmerl
 */
public class ShowFiguresAction extends Action {

    /**
	 * @author schmerl
	 * 
	 * To change the template for this generated type comment go to Window -
	 * Preferences - Java - Code Generation - Code and Comments
	 */
    public class GroupLabelProvider extends LabelProvider {

        /*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
		 */
        public String getText(Object element) {
            String label = "";
            if (element instanceof AcmeGroup) {
                AcmeGroup group = (AcmeGroup) element;
                if (!group.getName().equals("anonymous")) {
                    label = group.getName() + " : ";
                }
                AcmeEnumeration members = group.getMembers();
                while (members.moreItems()) {
                    AcmeElement el = (AcmeElement) members.getNextItem();
                    label += el.getName();
                    if (members.moreItems())
                        label += ", ";
                }
            }
            return label;
        }

    }
    public class GroupContentProvider implements IStructuredContentProvider {

        /*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
        public Object[] getElements(Object inputElement) {
            if (inputElement instanceof List) {
                return ((List) inputElement).toArray();
            }
            return null;
        }

        /*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
        public void dispose() {
            // TODO Auto-generated method stub

        }

        /*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *           java.lang.Object, java.lang.Object)
		 */
        public void inputChanged(
            Viewer viewer,
            Object oldInput,
            Object newInput) {
            // TODO Auto-generated method stub

        }

    }
    private DiagramEditor _editor;
    public final static String ID = "$ShowFigures$";

    public ShowFiguresAction(DiagramEditor editor) {
        _editor = editor;
        setId(ID);
        setText("Show hidden elements...");
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IAction#isEnabled()
	 */
    public boolean isEnabled() {
        
        if (_editor.getHiddenFigureGroups ().size() == 1) {
            setText ("Show hidden elements");
        }
        else {
            setText ("Show hidden elements...");
        }
        
        return !_editor.getHiddenFigureGroups().isEmpty();
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
    public void run() {
        if (_editor.getHiddenFigureGroups().size() == 1) {
            _editor.getGraphicalViewer().deselectAll();
            showFigures((AcmeGroup) _editor.getHiddenFigureGroups().get(0));
            _editor.getHiddenFigureGroups().clear();
        } else {
            ListSelectionDialog dialog =
                new ListSelectionDialog(
                    Display.getCurrent().getActiveShell(),
                    _editor.getHiddenFigureGroups(),
                    new GroupContentProvider(),
                    new GroupLabelProvider(),
                    "Select the group to show:");
            if (dialog.open() == IDialogConstants.OK_ID) {
                _editor.getGraphicalViewer().deselectAll();
                Object[] results = dialog.getResult();
                for (int i = 0; i < results.length; i++) {
                    showFigures((AcmeGroup) results[i]);
                    _editor.getHiddenFigureGroups().remove(results[i]);
                }
            }
        }

    }

    /**
	 * @param list
	 */
    private void showFigures(AcmeGroup elementsToShow) {
        AcmeEnumeration elements = elementsToShow.getMembers();

        while (elements.moreItems()) {
            AcmeElement element = (AcmeElement) elements.getNextItem();
            IFigure figure = _editor.getFigure(element);
            showFigure(figure);
            EditPart ep = _editor.getEditPart(element);
            if (ep != null) {
                _editor.getGraphicalViewer().select(ep);
            }
            if (element instanceof AcmeComponent) {
                AcmeComponent comp = (AcmeComponent) element;
                AcmeEnumeration ports = comp.getPorts();
                while (ports.moreItems()) {
                    AcmePort port = (AcmePort) ports.getNextItem();
                    IFigure f = _editor.getFigure(port);
                    showFigure(f);
                }
            } else if (element instanceof AcmeConnector) {
                AcmeConnector conn = (AcmeConnector) element;
                AcmeEnumeration roles = conn.getRoles();
                while (roles.moreItems()) {
                    AcmeRole role = (AcmeRole) roles.getNextItem();
                    IFigure f = _editor.getFigure(role);
                    showFigure(f);
                }
            }

        }

    }

    private void showFigure(IFigure figure) {
        if (figure != null) {
            if (!figure.isVisible()) figure.setVisible(true);
            if (figure instanceof AcmeVisElement) {
                AcmeVisElement ve = (AcmeVisElement) figure;
                ve.setTransparent (100);
            }
            else if (figure instanceof RoleView) {
                RoleView rv = (RoleView) figure;
                rv.setTransparent(100);
            }
            else {
                figure.setVisible(true);
            }
        }
    }

}
