/*
 * Created on Jul 21, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.actions;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import AcmeStudio.core.ASModel;
import AcmeStudio.model.ModelCommand;
import AcmeStudio.model.WrapRepresentationCommand;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;
import acmeLib.representations.AcmeBinding;

/**
 * @author Bradley Schmerl
 * Created Jul 21, 2003
 */
public class WrapRepresentationAction
	extends SelectionAction
	implements IWorkbenchWindowActionDelegate {

	protected List _selection;

	public static final String WRAPREP_ID = "$WrapRep$";
	public static final String UNWRAPREP_ID = "$UnWrapRep$";

	private AcmeDesign _parentDesign;

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		ASModel.current ().getRuleManager ().suspendEvaluation();
		getCommand ().execute (_parentDesign);
		((DiagramEditor )getWorkbenchPart()).getRootEditPart().refresh ();
		ASModel.current ().getRuleManager ().resumeEvaluation();
	}
	
	public ModelCommand getCommand () {
		WrapRepresentationCommand command = new WrapRepresentationCommand (getText ());
		Iterator i = _selection.iterator();
		while (i.hasNext ()) {
			AcmeElement element = (AcmeElement )i.next ();
			_parentDesign = element.getParentDesign();
			if (element instanceof AcmeRole || element instanceof AcmePort || element instanceof AcmeBinding) {
				i.remove();
			}
		}
		command.setTargetElements(_selection);
		return command;
	}

	/**
	 * @param editor
	 */
	public WrapRepresentationAction(IEditorPart editor, String id) {
		super(editor);
		setId (id);
		if (id.equals (WRAPREP_ID)) {
			setText ("Encapsulate as component");
		}
		else if (id.equals (UNWRAPREP_ID)) {
			setText ("Pull representation up");
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		if (getSelectedObjects().isEmpty ()) return false;
		_selection = new ArrayList ();
		
		AcmeElement[] l = Helper.getAcmeElementsFromSelection(getSelection());
		_selection.addAll (Arrays.asList(l));
		return !_selection.isEmpty ();
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window) {

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		
	}

}
