package AcmeStudio.ui.editors.diagram.figures;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.internal.ide.dialogs.UpdateProjectCapabilityWizard;

import AcmeStudio.ui.editors.diagram.figures.SystemView.WrapData;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.RoleVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.ui.editors.diagram.styles.constants.Orientation;
import AcmeStudio.ui.editors.diagram.styles.constants.SideOptions;
import AcmeStudio.util.Geometry;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;

/**
 * @author schmerl
 *
 */
public class BindingPointView extends Shape /*implements ElementView*/ {

	protected Map bindingAnchors = new Hashtable();
	protected List bindings = new Vector();
	private SystemEditPart.BindingPoint model = null;
	ElementVisualization _vis;
	double _rotation = 0;

	private Rectangle _bounds = new Rectangle();

	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
	 */
	public void paintFigure(Graphics graphics) {
		// Make sure the binding point moves to be on the border of the representation
		adjustViewToRepresentation();
		super.paintFigure(graphics);
	}

	/**
	 * Constructor for BindingPointView.
	 */
	public BindingPointView() {
		super();
	}

	/**
	 * @see org.eclipse.draw2d.Shape#fillShape(Graphics)
	 */
	protected void fillShape(Graphics graphics) {
		_vis.fillShape(graphics, _bounds, null, _rotation);
	}

	/**
	 * @see org.eclipse.draw2d.Shape#outlineShape(Graphics)
	 */
	protected void outlineShape(Graphics graphics) {
		_vis.outlineShape(graphics, getBounds());
	}

	/**
	 * @see AcmeStudio.editors.diagram.view.ElementView#setModel(AcmeElement)
	 */
	public void setModel(SystemEditPart.BindingPoint model) {
		this.model = model;

		_vis = VisualStyleRegistry.getVisualization(this.model._outer).copy();

		Label tt = new Label();
		tt.setText(model._outer.getQualifiedName());
		this.setToolTip(tt);
		adjustViewToRepresentation();
	}
	
	private static Point adjustPortPosition(Point compLocation, Dimension compSize, Point portLocation, AcmePort port) {
		if (portLocation == null || compLocation == null) return portLocation;
		int side = Geometry.getSideOfPoint(compLocation, new Point (portLocation.x + compLocation.x, portLocation.y + compLocation.y));
		int yIncr = 0; 
		int xIncr = 0;
		if (side == PositionConstants.NORTH || side == PositionConstants.SOUTH) {
			xIncr = 10;
		}
		else if (side == PositionConstants.EAST || side == PositionConstants.WEST) {
			yIncr = 10;
		}
		portLocation.x += xIncr;
		portLocation.y += yIncr;
		
		if (portLocation.x > compSize.width / 2) {
			portLocation.x = - compSize.width /2;
			WrapData data = (WrapData )port.getUserData("WrapData");
			if (data == null) {
				data = new WrapData ();
				port.setUserData ("WrapData", data);
			}
			data._times++;
		}
		
		if (portLocation.y > compSize.height/2) {
			portLocation.y = -compSize.height/2;
			WrapData data = (WrapData )port.getUserData("WrapData");
			if (data == null) {
				data = new WrapData ();
				port.setUserData ("WrapData", data);
			}
			data._times++;
		}
		return portLocation;
	}

	private void adjustViewToRepresentation() {
		AcmeElement parentElement = this.model._outer.getParent ();
		AcmeElement parentRep = parentElement;

		Dimension parentBounds = Helper.getSize(parentRep);
		Point parentLocation = SystemView.getLocation(parentElement, false);
		
		Rectangle bounds = null;
		IFigure parent = getParent();
		if (model._outer instanceof AcmePort) {
			Point location = null;
	
			location = SystemView.getPortLocation_static ((AcmePort )this.model._outer);
			SystemView.updatePortPositionToBorder(location, parentBounds);
			bounds = SystemView.getSystemBounds(model._parentRep.getSystem());
			bounds = RepresentationBorderView.calculateRepresentationBoundary(parentBounds.getCopy(), bounds.getCopy());
			double scale = (double) bounds.width / (double) parentBounds.width;

			Dimension size = Helper.getSize(this.model._outer);
			// Following makes sure that the port has visual width and heigh
			// and that we're not reliant on the vis for the size, as
			// this the width of the vis will change
			Helper.setSize(this.model._outer, size);
			Helper.scaleSize(size, scale);

			if (location != null) { // The location is determined by the visualization
				Helper.scaleLocation(location, scale);
				Point center = bounds.getCenter ();
				SystemView.updatePortPositionToBorder(location, bounds.getSize ());
				location = location.translate(bounds.getCenter());
				
				location = location.translate(-size.width / 2, -size.height / 2);
				this.setLocation(location);
				this.model.location = location;
			}

			this.setSize(size);
			this.model.size = size;
		}
		else {
			Point location = SystemView.getLocation(this.model._outer, false);
			Point connLocation = SystemView.getLocation (parentRep, false);
			Dimension connSize = Helper.getSize (parentRep);
			Dimension size = Helper.getSize(this.model._outer);

			location = location.translate (connLocation.negate ());
			location = location.translate (-size.width/2, -size.height/2);
			
			bounds = SystemView.getSystemBounds(model._parentRep.getSystem());		
			bounds = RepresentationBorderView.calculateRepresentationBoundary(parentBounds.getCopy(), bounds.getCopy());
			double scale = (double) bounds.width / (double) parentBounds.width;
			
			Helper.setSize(this.model._outer, size);
			Helper.scaleSize(size, scale);

			if (location != null) {

				Helper.scaleLocation(location, scale);
				Point ptLocation = location.getCopy ();
				
				

				Dimension diff = location.getDifference (connLocation.translate (connSize.width, connSize.height));
				diff = diff.getExpanded(-size.width/2, -size.height/2);
				double angle = Math.atan2 (diff.height, diff.width);
				double angleDeg = Math.toDegrees(angle);
				bounds.expand((int )(2*scale), (int )(2*scale));
				
				Point intersectingPoint = new Point (0,0);
				int side = computeIntersection (intersectingPoint, ptLocation, bounds.getBottomRight ().x, bounds.getBottomRight ().y, bounds.getTopLeft ().x, bounds.getTopLeft ().y);
				intersectingPoint.translate (-size.width/2, -size.height/2);
				

				this.setLocation(intersectingPoint);
				this.model.location = ptLocation;

				this.setSize(size);
				this.model.size = size;
				
				if ((_vis instanceof RoleVisualization) &&  !((RoleVisualization )_vis).rotateRole()) {
					_rotation = 100;
				}
				else {
					if (_vis instanceof RoleVisualization) {
						if (((RoleVisualization )_vis).getOrientation() == Orientation.IN) {
							angle += Math.PI;
						}
						_rotation = angle;
					}
				}
				

			}

		}

		
		_bounds = this.getBounds();
	}

	/**
	 * @param intersectingPoint
	 * @param ptLocation
	 * @param i
	 * @param j
	 * @param k
	 * @param l
	 * @return
	 */
	private int computeIntersection(Point intersectingPoint, Point ptLocation, int x1, int y1, int x2, int y2) {
		if (x1 > x2) {
			int tmp = x1; x1 = x2; x2 = tmp;
		}
		if (y1 > y2) {
			int tmp = y1; y1 = y2; y2 = tmp;
		}
		
		double cx = x1 + (x2 - x1)/2;
		double cy = y1 + (y2-y1)/2;
		double w = x2  - x1;
		double h = y2 - y1;
		int y = ptLocation.y;
		int x = ptLocation.x;
		boolean upper_right = ((y-y1) - h + (h/w)*(x-x1)) >= 0;
		boolean upper_left = ((y-y1) - (h/w)*(x-x1))>=0;
		
		if (upper_left && upper_right) {
			double delta = (x-cx)/(y-cy);
			intersectingPoint.x = (int )(h*delta/2 + cx);
			intersectingPoint.y = (int )(h/2 + cy);
			return SideOptions.TOP;
		}
		else if (!upper_right && !upper_left) {
			double delta = (x-cx)/(y-cy);
			intersectingPoint.x = (int )(-h*delta/2 + cx);
			intersectingPoint.y = (int )(-h/2 + cy);
			return SideOptions.BOTTOM;
		}
		else if (!upper_right && upper_left) {
			double delta = (y-cy)/(x-cx);
			intersectingPoint.x = (int )(-w/2+cx);
			intersectingPoint.y = (int )(-w*delta/2 + cy);
			return SideOptions.LEFT;
		}
		else if (upper_right && !upper_left) {
			double delta = (y-cy)/(x-cx);
			intersectingPoint.x = (int )(w/2 + cx);
			intersectingPoint.y = (int )(w*delta/2+cy);
			return SideOptions.RIGHT;
		}
		return -1;
		
	}

	/**
	 * @see AcmeStudio.editors.diagram.view.ElementView#getModel()
	 */
	public SystemEditPart.BindingPoint getModel() {
		return this.model;
	}

}
