package AcmeStudio.ui.editors.diagram.figures;

import java.util.Hashtable;
import java.util.Map;

import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;

import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;
import acmeLib.representations.AcmeBinding;

/**
 * @author schmerl
 *
 */
public class BindingView extends PolylineConnection implements IAcmeElementFigure {

    protected Map connectionAnchors = new Hashtable();
    private SystemEditPart.Binding model = null;
    private SystemView _systemView = null;
    private FigureCanvas figureCanvas = null;
    private Point _innerLocation = null;
    private Point _outerLocation = null;

    public void setFigureCanvas(FigureCanvas fc) {
        this.figureCanvas = fc;
    }

    public FigureCanvas getFigureCanvas() {
        return this.figureCanvas;
    }

    /**
     * Constructor for BindingView.
     */
    public BindingView() {
        super();
    }

    /**
     * @see AcmeStudio.editors.diagram.view.ElementView#setModel(AcmeElement)
     */
    public void setModel(SystemEditPart.Binding model) {
        this.model = model;
        scale(1.0);
    }

    /**
     * @see AcmeStudio.editors.diagram.view.ElementView#setModel(AcmeElement)
     */
    public void setModel(AcmeElement model) {
    }

    /**
     * @see AcmeStudio.editors.diagram.view.ElementView#getModel()
     */
    public AcmeElement getModel() {
        return this.model._binding;
    }


    private double outerScale;
    public void setEndPoints(RepresentationBorderView parentRepView) {
        outerScale = parentRepView.getScale();
        this.setForegroundColor(ElementColorConstants.gray);
        this.setLineWidth(1);
        this.setLineStyle(Graphics.LINE_DASH);

        scale(outerScale);
    }

    private void scale(double outerScale) {
        SystemEditPart.BindingPoint bp = model._outer;
        Point outerLocation = new Point();
        if (bp == null || bp.location == null || bp.size == null)
            return;
        Dimension outerSize = bp.size;
        outerLocation.x = bp.location.x + outerSize.width / 2;
        outerLocation.y = bp.location.y + outerSize.height / 2;

        AcmeBinding acmeBinding = model._binding;
        if (acmeBinding.getInner () instanceof AcmePort) {
	        AcmePort innerPort = (AcmePort) acmeBinding.getInner();
	        Point innerPortLocation = _systemView.getPortLocation (innerPort);
	        AcmeComponent comp = (AcmeComponent) innerPort.getParent();
	        Point innerLocation = new Point();
	        Point compLocation = _systemView.getElementLocation(comp);
	        innerLocation.x = compLocation.x + innerPortLocation.x;
	        innerLocation.y = compLocation.y + innerPortLocation.y;
	        if (_innerLocation == null
	            || !_innerLocation.equals(innerLocation)
	            || _outerLocation == null
	            || !_outerLocation.equals(outerLocation)) {
	            _innerLocation = innerLocation;
	            _outerLocation = outerLocation;
	            this.setStart(outerLocation);
	            this.setEnd(innerLocation);
	        }
        }
        else {
        	AcmeRole innerRole = (AcmeRole )acmeBinding.getInner ();
        	Point innerLocation = SystemView.getLocation (innerRole, false);
        	if (_innerLocation == null || !_innerLocation.equals (innerLocation) ||
        		_outerLocation == null || !_outerLocation.equals (outerLocation)) {
        			_innerLocation = innerLocation;
        			_outerLocation = outerLocation;
        			this.setStart (outerLocation);
        			this.setEnd (innerLocation);
        		}
        }
    }

    /**
     * @see org.eclipse.draw2d.PolylineConnection#layout()
     */
    public void layout() {
    }

    /* (non-Javadoc)
     * @see org.eclipse.draw2d.IFigure#paint(org.eclipse.draw2d.Graphics)
     */
    public void paint(Graphics graphics) {
        scale(1.0);
        super.paint(graphics);
    }

    public ElementVisualization getVisualization() {
        return null;
    }

    /**
     * @return
     */
    public SystemView getSystemView() {
        return _systemView;
    }

    /**
     * @param view
     */
    public void setSystemView(SystemView view) {
        _systemView = view;
    }

}
