/*
 * Created on Jun 18, 2004
 *  
 */
package AcmeStudio.ui.editors.diagram.figures;

import java.util.*;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.swt.widgets.Display;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.util.Helper;
/**
 * @author Bradley Schmerl Created Jun 18, 2004
 */
public class IntersectionCalculationThread extends Thread {

	private SystemEditPart	_root;
	private boolean			_reset	= false;

	/**
	 *  
	 */
	public IntersectionCalculationThread(SystemEditPart system) {
		super ();
		_root = system;
	}

	/**
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		Rectangle rect1 = new Rectangle ();
		Rectangle rect2 = new Rectangle ();
		List intersections = new ArrayList ();
		List connections = collectConnections ();
		Set toRefresh = new HashSet ();
		for (int i = connections.size () - 1; i >= 0; i--) {
			if (checkReset ()) return;;
			final RoleEditPart rep1 = (RoleEditPart )connections.get (i);
			RoundedJumpingRoleConnection conn = (RoundedJumpingRoleConnection )rep1.getFigure ();
			conn.clearIntersections ();
			intersections.clear ();
			for (int j = i - 1; j >= 0; j--) { //<
															// connections.size
															// (); j++) {
				if (checkReset ()) return;
				RoleEditPart rep2 = (RoleEditPart )connections.get (j);
				RoundedJumpingRoleConnection conn2 = (RoundedJumpingRoleConnection )rep2.getFigure ();
				if (!conn.intersects (conn2.getBounds ())) continue;
				PointList plConn1 = conn.getPoints ();
				PointList plConn2 = conn2.getPoints ();
				for (int pl1 = 1; pl1 < plConn1.size (); pl1++) {
					if (checkReset ()) return;;
					final Point pl1p1 = plConn1.getPoint (pl1 - 1);
					Point pl1p2 = plConn1.getPoint (pl1);
					for (int pl2 = 1; pl2 < plConn2.size (); pl2++) {
						if (checkReset ()) return;
						Point pl2p1 = plConn2.getPoint (pl2 - 1);
						Point pl2p2 = plConn2.getPoint (pl2);
						IntersectingPoint ip = Helper.getIntersectingPoint (pl1p1, pl1p2, pl2p1, pl2p2);
						ip.setBetweenBendpoints (pl1p1, pl1p2);
						// Set up the rectangles to contain the points of the
						// ends
						// could used new rectangle, but would rather reuse
						// objects
						rect1.setSize (0, 0);
						rect2.setSize (0, 0);
						rect1.setLocation (pl1p1);
						rect1.union (pl1p2);
						rect2.setLocation (pl2p1);
						rect2.union (pl2p2);
						if (ip != null && rect1.contains (ip) && rect2.contains (ip)) {
							intersections.add (ip);
						}
					}
					Collections.sort (intersections, new Comparator () {

						public int compare(Object o1, Object o2) {
							Point p0 = (Point )o1;
							Point p1 = (Point )o2;
							if (pl1p1.getDistance (p0) < pl1p1.getDistance (p1)) return -1;
							return 1;
						}
					});
					conn.addIntersections (pl1p1, pl1p2, intersections);
					toRefresh.add (rep1);
					Display.getDefault().asyncExec(new Runnable () {
						public void run () {
							rep1.refreshVisuals();
						}
					});
				}
			}
		}
	}

	/**
	 * @return
	 */
	private boolean checkReset() {
		if (_reset) {
			_reset = false;
			run ();
			return true;
		}
		return false;
	}

	/**
	 * @return
	 */
	private List collectConnections() {
		ArrayList result = new ArrayList ();
		for (Iterator iter = _root.getChildren ().iterator (); iter.hasNext ();) {
			GraphicalEditPart child = (GraphicalEditPart )iter.next ();
			if (child instanceof RoleEditPart) {
				result.add (child);
			}
		}
		return result;
	}
}