package AcmeStudio.ui.editors.diagram.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import AcmeStudio.ui.editors.diagram.styles.PortVisualization;
import AcmeStudio.ui.editors.diagram.styles.constants.Orientation;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElement;

public class PortView extends AcmeVisElement {
    double _rotation = 100;
    private Point _location = null;
    private Dimension _size = null;

    public void scale() {
        AcmeElement e = _model.getParent();
        if (e instanceof AcmeComponent) {
            _size = Helper.getSize(getModel());
            if (_size == null) {
                _size = _vis.getSize();
            }
            this.setSize(_size);
            


            if (_vis instanceof PortVisualization && _location != null) {
                PortVisualization pv = (PortVisualization) _vis; 
                if (pv.rotatePort()) {
                    setRotation (Math.atan2(_location.y - _size.height, _location.x - _size.width));
                    if (pv.getOrientation() == Orientation.IN) {
                        setRotation (_rotation + Math.PI);
                    }
                } else {
                    setRotation (100);
                }
            }
        }
    }

    public PortView() {
    }

    /**
     * @see org.eclipse.draw2d.IFigure#addNotify()
     */
    public void addNotify() {
        super.addNotify();
//        if (getParent() != null && _model != null) {
//            scale();
//        }
    }

    public void setModel(AcmeElement model) {
        super.setModel(model);

        scale();
    }

    /**
     * @see org.eclipse.draw2d.Shape#fillShape(Graphics)
     */
    protected void fillShape(Graphics graphics) {
        if (_rotation != 100) {
            Dimension innerSize = new Dimension(_size);
            //recalculateSize();
            Point location = _location;
            if (isTransparent ()) location = ORIGIN;
            _vis.fillShape(
                graphics,
                new Rectangle(location, _size),
                innerSize,
                _rotation);
        } else {
            if (_location != null && _size != null) {
                Dimension innerSize = new Dimension(_size);
                //recalculateSize();
                Point location = _location;
                if (isTransparent ()) location = ORIGIN;
                _vis.fillShape(
                    graphics,
                    new Rectangle(location, _size),
                    innerSize,
                    0.0);
            }
            //super.fillShape(graphics);
        }

    }

    public void setRotation(double angle) {
        _rotation = angle;
//        this.repaint();
    }
    
    public double getRotation() {
        return _rotation;
    }

    public Dimension getPortSize() {
        return _size;
    }
    
    public void  setLocation(Point location) {
        //_location = location;
        super.setLocation(location);
    }

    public void  setRelativeLocation(Point location) {
        _location = location;
        scale();
    }
}