package AcmeStudio.ui.editors.diagram.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import com.swtdesigner.SWTResourceManager;

import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.ui.editors.diagram.styles.constants.LabelOptions;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeSystem;

/**
 * @author schmerl
 *
 */
public class RepresentationBorderView extends AcmeVisElement {


    public static final int BORDER_SPACE = 100;
    
	Rectangle _bounds;
	double _scaleSetting;
    AcmeSystem _enclosedSystem;

	private boolean	_noLabel;
    
    
//	/**
//	 * 
//	 */
//	public RepresentationBorderView() {
//	}
    
//    Button _upButton;
	/**
	 * Constructor for RepresentationBorderView.
	 */
	public RepresentationBorderView(AcmeElement model, Rectangle bounds, AcmeSystem enclosedSystem) {
		super();
		init (model, enclosedSystem);
        if (getModel ().getUserData(DiagramEditor.INLINEREPRESENTATION) == null) {
        	_noLabel = true;
        }
        else _noLabel = false;
        

	}


	/**
	 * @param model
	 * @param enclosedSystem
	 */
	public void init(AcmeElement model, AcmeSystem enclosedSystem) {
		setOpaque(false);
		setModel (model);
        _enclosedSystem = enclosedSystem;
        _bounds = getBorderBounds ();
//        _bounds = new Rectangle (
//            bounds.translate (new Point (
//                bounds.x < 0 ? -bounds.x : 0,
//                bounds.y < 0 ? -bounds.y : 0)));
        ElementVisualization vis = VisualStyleRegistry.getVisualization(_model);
        if (_vis.getLabelSource() != LabelOptions.NOLABEL) {
        	_vis = vis.copy ();
        	_vis.setLabelSource(LabelOptions.NOLABEL);
        }
        else {
        	_vis = vis;
        }
	}

	/**
	 * @see org.eclipse.draw2d.Shape#fillShape(Graphics)
	 */
	protected void fillShape(Graphics graphics) {
        _vis.fillHollowShape (graphics, /*getBounds ()*/_bounds);
	}
    
    

	
	protected Rectangle scale (Rectangle bounds) {
		Rectangle b = new Rectangle (bounds);
        b.expand (BORDER_SPACE, BORDER_SPACE);

		this.setBounds (b);
		_scaleSetting = ((double )b.width)/((double )Helper.getSize (_model).width);
		
		Point location = this.getLocation();
		Dimension size = this.getSize ();
		this.setLocation (location);
		this.setSize (size);
        _vis.setSize (size);
        
        return new Rectangle (location, size);
		
	}
	
	public Rectangle getBorderBounds () { 
        Dimension pSize = Helper.getSize(getModel ());

        Rectangle bounds = null;
        bounds = SystemView.getSystemBounds(_enclosedSystem);
        
		return calculateRepresentationBoundary(pSize, bounds);
    
    }


	public static Rectangle calculateRepresentationBoundary(
		Dimension pSize,
		Rectangle bounds) {
//		if (bounds.width < pSize.width * 4
//			&& bounds.height < pSize.height * 4) {
//			bounds.width = pSize.width * 4;
//			bounds.height = pSize.height * 4;
//
//		}
		
		// Fix the origin so that it doesn't seem to jump around as much
		bounds.width += bounds.x;	
		bounds.height += bounds.y;
		bounds.x = 0;
		bounds.y = 0;

		if (pSize.width < bounds.width || pSize.height < bounds.height) {
			if (pSize.width < bounds.width) {
				pSize.height = pSize.height * bounds.width / pSize.width;
				pSize.width = bounds.width;
			}
			if (pSize.height < bounds.height) {
				pSize.width = pSize.width * bounds.height / pSize.height;
				pSize.height = bounds.height;
			}
			//            while (pSize.width < bounds.width
			//                || pSize.height < bounds.height) {
			//                pSize.width += 20;
			//                pSize.height += 20
			//                    * (float) bounds.height
			//                    / (float) bounds.width;
			//                ;
			//            }
			bounds.setSize(pSize);
		}
		bounds.expand(BORDER_SPACE, BORDER_SPACE);
		//        scale (bounds);
		return bounds;
	}
		

	/**
	 * @see org.eclipse.draw2d.IFigure#containsPoint(int, int)
	 */
	public boolean containsPoint(int arg0, int arg1) {
		return false;
	}

	public double getScale () {
		return _scaleSetting;
	}

    /**
     * @see org.eclipse.draw2d.IFigure#addNotify()
     */
    public void addNotify() {
        super.addNotify();
//        scale ();
    }

    /**
     * @see org.eclipse.draw2d.Shape#outlineShape(Graphics)
     */
    protected void outlineShape(Graphics graphics) {
        _vis.outlineShape(graphics, _bounds/*getBounds ()*/);
        
        if (_noLabel) return;
        graphics.pushState();
        FontData fontData = _vis.getFont ().getFontData()[0];
        Font font = SWTResourceManager.getFont(fontData.getName (), 30, fontData.getStyle ());
//		Font font = new Font (null, fontData.getName(), 30, fontData.getStyle());
		graphics.setFont(font);
		graphics.setForegroundColor(ColorConstants.black);
		Rectangle textBounds = new Rectangle (_bounds.x, _bounds.y + _bounds.height, _bounds.width, 30);
		AcmeElement parentElement = _enclosedSystem.getParentRepresentation().getParent();
		if (parentElement == null) return;
		String label = parentElement.getName ();
		Dimension labelExtents = FigureUtilities.getTextExtents(label, font);
		graphics.drawText(label, textBounds.x + (textBounds.width - labelExtents.width)/2, textBounds.y );
		
		textBounds.y += labelExtents.height;
		label = "Representation - " + _enclosedSystem.getName ();
		labelExtents = FigureUtilities.getTextExtents (label, font);
		graphics.drawText(label, textBounds.x + (textBounds.width - labelExtents.width)/2, textBounds.y);
      /*  Font font = _vis.getFont().*/
//		font.dispose();
        graphics.popState ();
    }

    /* (non-Javadoc)
     * @see org.eclipse.draw2d.IFigure#paint(org.eclipse.draw2d.Graphics)
     */
    public void paint(Graphics graphics) {
        // Calculate the new size
        _bounds = getBorderBounds ();
        Rectangle bounds = _bounds.getCopy ();
        if (!_noLabel) {
        	bounds.height += 120;
        }
        	setBounds (bounds);
        super.paint(graphics);
    }

    public void setNoLabel (boolean noLabel) {
    	_noLabel = noLabel;
    }
}
