package AcmeStudio.ui.editors.diagram.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.RotatableDecoration;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import AcmeStudio.ui.editors.diagram.styles.RoleVisualization;
import AcmeStudio.ui.editors.diagram.styles.constants.Orientation;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeRole;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * Provides role decoration.
 * 
 */
public class RoleDecoration
    extends AcmeVisElement
    implements RotatableDecoration {

    private RoleView _roleView = null;
    Dimension _innerSize = null;
    Dimension _size = null;
    Point _location = null;
    double _rotation = 100;

    /**
     * Constructor
     * @param model The model represented by this decoration
     */
    public RoleDecoration(AcmeRole model, RoleView rv) {
        setModel(model);
        _roleView = rv;
    }

    /**
     * Set the model
     * @param model The acme role model
     */
    public void setModel(AcmeRole model) {
        super.setModel(model);
        _location = SystemView.getLocation(model, true);
        _size = Helper.getSize(model);
        if (_size == null) {
            _size = getVisualization().getSize();
        }
        _innerSize = new Dimension(_size);
        //        _innerSize.scale(0.9, 0.9);
        setLocation(_location);
    }

    /**
     * Set the location of the decoration
     * @param p The location
     */
    public void setLocation(Point p) {
        _location = p;
        //recalculateSize();
        recalculateLocation();
        super.setLocation(_location);
        super.setSize(_size);
    }

    /**
     * @see org.eclipse.draw2d.RotatableDecoration#setReferencePoint(Point)
     */
    public void setReferencePoint(Point ref) {
    }

    /**
     * @see org.eclipse.draw2d.Shape#fillShape(Graphics)
     */
    protected void fillShape(Graphics graphics) {
        double rotation = 0.0;
        if ((_vis instanceof RoleVisualization)
            && !((RoleVisualization) _vis).rotateRole()) {
            _rotation = 100;
        } else {
            if (_roleView != null && _vis instanceof RoleVisualization
                /*&& (!_roleView._isMoving || _rotation == 100)*/) {
                Dimension diff =
                    _roleView.getEnd().getDifference(
                        _roleView.getLastBendpoint());
                double angle =
                    Math.atan2(diff.height, diff.width);
                //            _vis.fillShape(
                //                graphics,
                //                new Rectangle(_location, _size),
                //                _innerSize, angle);
                if ((_vis instanceof RoleVisualization)
                    && ((RoleVisualization) _vis).getOrientation()
                        == Orientation.IN) {
                    angle += Math.PI;
                }
                _rotation = angle;
                //System.err.println("rotation:"+angle);
                rotation = angle;
            } else if (_rotation != 100) {
                rotation = _rotation;
            }
        }
        Point location = _location;
        if (isTransparent ()) location = ORIGIN;
        _vis.fillShape(
            graphics,
            new Rectangle(location, _size),
            _innerSize,
            rotation);
    }

    private void recalculateLocation() {
        if (_location != null) {
            _location.x -= _size.width / 2;
            _location.y -= _size.height / 2;
        }
    }
    
    public double getRotation() {
        return _rotation;
    }
    
}