package AcmeStudio.ui.editors.diagram.figures;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.AbstractRouter;
import org.eclipse.draw2d.Bendpoint;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.PrecisionPoint;

/**
 * A Router that routes a {@link Connection} that has manual 
 * {@link org.eclipse.draw2d.Bendpoint Bendpoints}.
 */
public class RoleRouter
    extends AbstractRouter
{

private Map constraints = new HashMap(11);

private static final PrecisionPoint A_POINT = new PrecisionPoint();

/**
 * Gets the constraint for the given {@link Connection}.  
 *
 * @param connection The connection whose constraint we are retrieving
 * @return The constraint
 */
public Object getConstraint(Connection connection) {
    return constraints.get(connection);
}

/**
 * Removes the given connection from the map of constraints.
 *
 * @param connection The connection to remove
 */
public void remove(Connection connection) {
    constraints.remove(connection);
}

/**
 * Routes the {@link Connection}.  Expects the constraint to be a {@link java.util.List}
 * of {@link Point Points} representing the bendpoints.
 *
 * @param conn The connection to route
 */
public void route(Connection conn) {
    PointList points = conn.getPoints();
    if (points.size() == 0) {
    	Point start = getStartPoint(conn);
    	Point end = getEndPoint (conn);
    	points.addPoint (start);
    	points.addPoint (end);
    }
    
    
    List bendpoints = (List)getConstraint(conn);
    if (bendpoints == null) {
     	// RESET if there was one bendpoint previously
    	if (points.size () > 2) {
    		Point ref1 = ((RoleView )conn).getStart ();
    		Point ref2 = ((RoleView )conn).getEnd();
    		
    		points.removeAllPoints ();
    		A_POINT.setLocation(ref1);
    		points.addPoint (A_POINT);
    		A_POINT.setLocation (ref2);
    		points.addPoint (A_POINT);
    		conn.setPoints (points);
    		
    	}
        bendpoints = Collections.EMPTY_LIST;
        return;
    }
    
    boolean sameBendpoints = points.size () == bendpoints.size ();
    int j = 0;
    while (sameBendpoints && j < points.size()) {
    	if (!points.getPoint(j).equals (bendpoints.get (j)))
    			sameBendpoints = false;
    	j++;
    }
    
    if (sameBendpoints) return;
    
    if (points.size() < 2) return;
    Point ref1 = null, ref2 = null;
    ref1 = ((RoleView)conn).getStart();
    ref2 = ((RoleView)conn).getEnd();

    points.removeAllPoints();


//    if (!bendpoints.isEmpty()) {
//        ref1 = new Point(((Bendpoint)bendpoints.get(0)).getLocation());
//        //conn.translateToAbsolute(ref1);
//        ref2 = new Point(((Bendpoint)bendpoints.get(bendpoints.size() - 1)).getLocation());
//        //conn.translateToAbsolute(ref2);
//    }

    A_POINT.setLocation(ref1);
    //conn.translateToRelative(A_POINT);
    points.addPoint(A_POINT);

    for (int i = 0; i < bendpoints.size(); i++) {
        Bendpoint bp = (Bendpoint)bendpoints.get(i);
        points.addPoint(bp.getLocation());
    }

    A_POINT.setLocation(ref2);
    //conn.translateToRelative(A_POINT);
    points.addPoint(A_POINT);
    conn.setPoints(points);
}

/**
 * Sets the constraint for the given {@link Connection}.
 *
 * @param connection The connection whose constraint we are setting
 * @param constraint The constraint
 */
public void setConstraint(Connection connection, Object constraint) {
    constraints.put(connection, constraint);
}

}


