/*
 * Created on Dec 17, 2003
 *  
 */
package AcmeStudio.ui.editors.diagram.figures;
import java.util.*;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.util.Geometry;
import AcmeStudio.util.Helper;

/**
 * @author Bradley Schmerl Created Dec 17, 2003
 */
public class RoundedJumpingRoleConnection extends PolylineConnection {

	List _points = new ArrayList ();
	
	/**
	 * 
	 */
	public RoundedJumpingRoleConnection() {
		super ();
		if (_points == null) _points = new ArrayList ();
	}
	
	
	
	/**
	 * @param pt
	 * @see org.eclipse.draw2d.Polyline#addPoint(org.eclipse.draw2d.geometry.Point)
	 **/
	public void addPoint(Point pt) {
		super.addPoint (pt);
		if (_points == null) _points = new ArrayList ();
		synchronized (_points) {
			_points.add (pt);
		}
	}
	/**
	 * @param pt
	 * @param index
	 * @see org.eclipse.draw2d.Polyline#insertPoint(org.eclipse.draw2d.geometry.Point, int)
	 **/
	public void insertPoint(Point pt, int index) {
		super.insertPoint (pt, index);
		synchronized (_points) {
			_points.add(index, pt);
		}
	}
	/**
	 * 
	 * @see org.eclipse.draw2d.Polyline#removeAllPoints()
	 **/
	public void removeAllPoints() {
		super.removeAllPoints ();
		synchronized (_points) {
			_points.clear();
		}
	}
	/**
	 * @param index
	 * @see org.eclipse.draw2d.Polyline#removePoint(int)
	 **/
	public void removePoint(int index) {
		super.removePoint (index);
		synchronized (_points) {
			_points.remove (index);
		}
	}
	/**
	 * @param end
	 * @see org.eclipse.draw2d.Polyline#setEnd(org.eclipse.draw2d.geometry.Point)
	 **/
	public void setEnd(Point end) {
		if (getPoints ().size() < 2) addPoint (end);
		else {
			erase ();
			getPoints ().setPoint (end, getPoints ().size () - 1);
			bounds = null;
			repaint ();
		}
		synchronized (_points) {
			if (_points.size() < 2) _points.add(end);
			else {
				_points.set(_points.size () - 1, end);
			}
		}
	}
	
	/**
	 * @param pt
	 * @param index
	 * @see org.eclipse.draw2d.Polyline#setPoint(org.eclipse.draw2d.geometry.Point, int)
	 **/
	public void setPoint(Point pt, int index) {
		super.setPoint (pt, index);
		synchronized (_points) {
			if (index >= _points.size ()) 
				_points.add (pt);
			else {
				int newIndex = 0;
				int iterIndex = 0;
				while (newIndex < index) {
					if (!(_points.get (iterIndex) instanceof IntersectingPoint)) 
						newIndex++;
					iterIndex ++;
				}
				if (iterIndex > _points.size()) _points.add (pt);
				else _points.set (iterIndex, pt);
			}
		}
	}
	/**
	 * @param points
	 * @see org.eclipse.draw2d.Polyline#setPoints(org.eclipse.draw2d.geometry.PointList)
	 **/
	public void setPoints(PointList pts) {
		synchronized (_points) {
		this._points.clear();
		for (int i = 0; i < pts.size(); i++) {
			_points.add (pts.getPoint (i));
		}
		}
		super.setPoints (pts);
	
	}
	/**
	 * @param start
	 * @see org.eclipse.draw2d.Polyline#setStart(org.eclipse.draw2d.geometry.Point)
	 **/
	public void setStart(Point start) {
		synchronized (_points) {
		if (_points.size () == 0) _points.add (start);
		else _points.set(0, start);
		}
		super.setStart (start);
	
	}
private static final int	INTERSECTION_DISTANCE_TOLERANCE	= 5;
	public Rectangle getBounds() {
		return super.getBounds ().getCopy ().expand (10, 10);
	}

	/**
	 * Outlines the shape for this complex connection. The connection can
	 * do the following:<br>
	 * <ol>
	 * <li>Round the bendpoints, using a bezier curve</li>
	 * <li>Place jumps over intersecting lines
	 * </ol>
	 */
	protected  void outlineShape(Graphics g) {
//		drawWithNewIntersection (g);
		
		
		drawWithOldIntersection (g);
	}

	

	/**
	 * @param g
	 */
	private void drawWithOldIntersection(Graphics g) {
		boolean roundCorners = AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean("DiagramEditor.RoundedCorners") ;
		boolean jumpIntersections = AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean("DiagramEditor.JumpIntersections") ;

		ArrayList connections = new ArrayList ();
		if (jumpIntersections) connections = collectConnections ();
		PointList ps = getPoints ();
		if (ps.size () == 0) return;
		
		// Iterate through bendpoints
		Point pp = ps.getPoint (0);
		for (int i = 1; i < ps.size (); i++) {
			Point p = ps.getPoint (i);
			Point ppo = pp/*ps.getPoint (i - 1)*/;
			Point cp1 = p.getCopy ();
			Point cp2 = p.getCopy ();
			
			// Don't worry about rounding
			if (i == ps.size () - 1 || !roundCorners) {
				drawLine (g, pp, p, connections, jumpIntersections);
				pp = p;
				continue;
			}
			
			// Get the next point on
			Point pn = ps.getPoint (i + 1).getCopy ();

			// Set up the control points
			cp1 = getCloserPoint (p, ppo, 5);
			Point fp = getFurtherPoint (p, ppo, 5);
			if (ppo.getDistance (fp) < ppo.getDistance (cp1)) {
				cp1 = fp;
			}
			cp2 = getCloserPoint (p, pn, 5);
			fp = getFurtherPoint(p, pn, 5);
			if (pn.getDistance (fp) < pn.getDistance (cp2)) {
				cp2 = fp;
			}
			
			// Draw straight line to control point	
			drawLine(g, ppo, cp1, connections, jumpIntersections);
			// Draw bezier around curve
			drawBezier (g, cp1, p, cp2);
		
			// Next bendpoint
			pp = cp2;
			
			

		}
	}



	/**
	 * @param g
	 */
	private void drawWithNewIntersection(Graphics g) {
		// Get the preferences to use
		if (_isMoving) clearIntersections ();
		boolean roundCorners = AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean("DiagramEditor.RoundedCorners") ;
		boolean jumpIntersections = AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean("DiagramEditor.JumpIntersections") ;
		
	
		List ps = null;
		synchronized (_points) {
			 ps = new ArrayList (_points);
		}
		if (ps.size () == 0) {
			return;
		}
		
		Point p0 = (Point )ps.get (0);
		for (int i = 1; i < ps.size (); i++) {
			Point p1 = (Point )ps.get (i);
			if (p1 instanceof IntersectingPoint) {
				if (jumpIntersections) {
					// what to do if the jump is too close to the bendpoints?
					if (!(p0 instanceof IntersectingPoint) && p0.getDistance (p1) < 5) continue;
					if (i+1 != ps.size () && !(ps.get (i+1) instanceof IntersectingPoint) && p1.getDistance ((Point )ps.get (i+1))< 5 ) continue;
					// Calculate the next point index to use, which will be > i if the next jump point is too close to this one, i otherwise
					int nextI = calculateNextPointIndex (p1, ps, i);
					// Calculate the point that the jump will start
					Point [] jumpEnds = calculateJumpEnds (p0, p1, ps, i, nextI); 
					// Draw a line to that point
					g.drawLine(p0, jumpEnds [0]);
					// Draw the jump
					 drawJump (g, jumpEnds [0], jumpEnds [1], jumpEnds [2]);
					// make sure  i is the next point to visit
					i = nextI;
					// make the next segment be where the jump ends
					p0 = jumpEnds [2];			
				}
				else {
					i++;
				}
			}
			// don't round corners
			else if (i == ps.size () - 1 || !roundCorners) {
				g.drawLine(p0, p1);
				p0 = p1;
			}
			// Round a corner
			else {
				Point pn = ((Point )ps.get(i+1)).getCopy ();
				// Set up control points
				Point cp1 = getCloserPoint (p1, p0, 5);
				Point fp = getFurtherPoint (p1, p0, 5);
				if (p0.getDistance (fp) < p0.getDistance (cp1)) 
					cp1 = fp;
				Point cp2 = getCloserPoint (p1, pn, 5);
				fp = getFurtherPoint (p1, pn, 5);
				if (pn.getDistance (fp) < p0.getDistance (cp2))
					cp2 = fp;
				g.drawLine (p0, cp1);
				drawBezier (g, cp1, p1, cp2);
				p0 = cp2;
			}
			
			
		}
	}



	/**
	 * Calculates the beginning of the jump arc
	 * @param startPoint Where the jump will be coming from
	 * @param jumpOverPoint The point around which to jump
	 * @param ps The list of points
	 * @param i the index of jumpOverPoint in points
	 * @param nextI the index of the next furthest away jump point
	 * @return
	 */
	private Point [] calculateJumpEnds(Point startPoint, Point jumpOverPoint, List ps, int i, int nextI) {
		int radius = 5*(nextI - i + 1);
		Point midpoint = jumpOverPoint;
		if (nextI != i) {
			Point nextPoint = (Point )ps.get (nextI);
			midpoint = new Point ((jumpOverPoint.x + nextPoint.x)/2, (jumpOverPoint.y + nextPoint.y)/2);
		}
		Point cp = getCloserPoint (midpoint, startPoint, radius);
		Point fp = getFurtherPoint (midpoint, startPoint, radius);
		if (cp.getDistance(startPoint) > fp.getDistance (startPoint)) {
			Point tmp = cp;
			cp = fp;
			fp = tmp;
		}
		return new Point [] {cp, midpoint, fp};
	}

	/**
	 * Calculates the index of the next intersecting point, taking into account the proximity of
	 * the next point
	 * @param intersectingPoint The intersecting point to get the next index of
	 * @param ps The list of points
	 * @param i the index in points of intesecting point
	 * @return the index of the next point (if there are intersecting points that are too close), or i otherwise
	 */
	private int calculateNextPointIndex(Point intersectingPoint, List ps, int i) {
		int nextI = i;
		while (nextI < _points.size () - 1 && _points.get (nextI+1) instanceof IntersectingPoint && ((Point )_points.get(nextI)).getDistance ((Point )_points.get (nextI+1)) < INTERSECTION_DISTANCE_TOLERANCE) {
			nextI++;
		}
		return nextI ;
	}

	/**
	 * @param g
	 * @param cp1
	 * @param p
	 * @param cp2
	 */
	private void drawBezier(Graphics g, Point cp0, Point cp1, Point cp2) {
		Point [] ptArray = new Point [3];
		
		Vector points = new Vector ();
		points.add (cp0);
		points.add (cp1);
		points.add (cp2);
		
		Point p0, p1;
		p0 = (Point )points.elementAt (0);
		for (int i = 1; i <= 5; i++) {
			double t = i/5.0;
			points.copyInto(ptArray);
			p1 = Geometry.evalBezier(ptArray, t);
			g.drawLine(p0.x, p0.y, p1.x, p1.y);
//			g.drawLine (p0.x, p0.y, p0.x, p0.y);
			p0 = p1;
		}
		
	}

	private void getIntersectionsBetweenPoints(ArrayList connections, List intersections, final Point startPoint, Point p1) {
		for (Iterator I = connections.iterator (); I.hasNext (); ) {
			RoundedJumpingRoleConnection conn = (RoundedJumpingRoleConnection) I.next ();
			if (conn == this) break;
			PointList cps = conn.getPoints ();
			for (int j = 1; j < cps.size (); j++) {
				Point cp0 = cps.getPoint (j - 1);
				Point cp1 = cps.getPoint (j);
				Point ip = Helper.getIntersectingPoint (startPoint, p1, cp0, cp1);
				if (ip != null && new Rectangle (startPoint, p1).contains (ip) && new Rectangle (cp0, cp1).contains (ip)
				&& ip.getDistance(p1) > 10 && ip.getDistance (startPoint) > 10 && ip.getDistance (cp0) > 10 && ip.getDistance (cp1) > 10) {
//					intersections.add (conn);
					intersections.add (ip);
//					intersections.add (p0);
				}
			}
			Collections.sort(intersections, new Comparator () {

				public int compare(Object arg0, Object arg1) {
					Point p0 = (Point) arg0;
					Point p1 = (Point) arg1;
					if (startPoint.getDistance(p0) < startPoint.getDistance (p1)) {
						return -1;
					}
					return 1;
					
				}
			
			}
					);
		}
	}

	/**
	 * Draws the jump arc
	 * @param g
	 * @param start the beginning of the jump
	 * @param middle the middle of the jump
	 * @param end the end of the jump
	 * @return
	 */
	private void drawJump(Graphics g, Point start, Point middle, Point end) {
		int startX = start.x;
		int endX = end.x;
		if (startX < endX) {
			Point t = start;
			start = end;
			end = start;
		}
		Rectangle r = new Rectangle (start, end);
		
		if (start.y == end.y) {
			r.setSize(r.width, 10);
			g.drawArc (r,  0, 180);
		}
		else {
			double angle = -Math.toDegrees (Math.atan (getSlope (start, end)));
			if (angle == 0) {
				g.drawArc (r, 1, 179);
			}
			else if (angle == 90) {
				g.drawArc (r, 90, 180);
			}
			else if (angle == -90) {
				g.drawArc (r, 90, 180);
			}
			else {
				g.drawArc (r, (int )angle, 180);
			}
		}
		
	}
	private void drawLine(Graphics g, Point ps, Point pe, ArrayList connections, boolean jumpIntersections)
	{
		Point startPoint = ps;
		if (jumpIntersections) {
			Vector intersections = new Vector ();
			getIntersectionsBetweenPoints (connections, intersections, ps, pe);
			
			for (int i = 0; i < intersections.size(); i++) {
//				Iterator iter = intersections.iterator (); iter.hasNext (); ) {
//			}
//				RoundedPolylineConnection conn = (RoundedPolylineConnection) iter.next ();
				int radius = 5;
				Point intersectionPoint = (Point) intersections.elementAt (i);
				Point nextIP = Point.SINGLETON;
				if (i < intersections.size () - 1) {
					nextIP = (Point )intersections.elementAt (i+1);
					if (nextIP.getDistance (intersectionPoint) < 10) {
						radius = 10;
						intersectionPoint = new Point ((intersectionPoint.x + nextIP.x)/2, (intersectionPoint.y + nextIP.y)/2);
						i++;
					}
				}
//				Point discard = (Point) iter.next ();
				Point ipCloserToStart = getCloserPoint (intersectionPoint, startPoint, radius);
				Point ipFurtherFromStart = getFurtherPoint (intersectionPoint, startPoint, radius);
				if (startPoint.getDistance (ipCloserToStart) > startPoint.getDistance (ipFurtherFromStart)) {
					Point tmp = ipCloserToStart;
					ipCloserToStart = ipFurtherFromStart;
					ipFurtherFromStart = tmp;
				}
				Point bp = ipCloserToStart;
				Point ep = ipFurtherFromStart;
				if (bp.x < ep.x) {
					bp = ipFurtherFromStart;
					ep = ipCloserToStart;
				}
				double angle = -Math.toDegrees (Math.atan (getSlope (bp, ep)));
				g.drawLine (startPoint, ipCloserToStart);
				Rectangle rectangle = new Rectangle (intersectionPoint.x - radius, intersectionPoint.y - radius, 2*radius, 2 * radius);
				
				//			g.drawRectangle (rectangle);
				if (angle == 0) {
					g.drawArc (rectangle, 1, 179);
				}
				else if (angle == 90) {
					g.drawArc (rectangle, 90, 180);
				}
				else if (angle == -90) {
					g.drawArc (rectangle, 90, 180);
				}
				else {
					g.drawArc (rectangle, (int) angle, 180);
				}
				startPoint = ipFurtherFromStart;
			}
		}	
		g.drawLine (startPoint, pe);
	}

	/**
	 * @param intersectionPoint
	 * @param startPoint
	 * @return
	 */
	private Point getFurtherPoint(Point intersectionPoint, Point startPoint, int radius) {
		double slope = getSlope (intersectionPoint, startPoint);
		int incr = radius;
		if (Double.isInfinite (slope)) {
			Point fp = intersectionPoint.getCopy ();
			fp.y += radius;
			return fp;
		}
		double xIncr = incr;
		double yIncr = slope * xIncr;
		if (Math.abs (xIncr) < Math.abs (yIncr)) {
			yIncr = incr;
			xIncr = yIncr * 1 / slope;
		}
		Point fp = intersectionPoint.getCopy ();
		fp.x += xIncr;
		fp.y += slope * xIncr;
		return fp;
	}

	/**
	 * @param intersectionPoint
	 * @param startPoint
	 * @return
	 */
	private double getSlope(Point p0, Point p1) {
		double slope = ((double) (p1.y - p0.y)) / ((double) (p1.x - p0.x));
		return slope;
	}

	/**
	 * @param intersectionPoint
	 * @param startPoint
	 * @return
	 */
	private Point getCloserPoint(Point intersectionPoint, Point startPoint, int radius) {
		double slope = getSlope (intersectionPoint, startPoint);
		if (Double.isInfinite (slope)) {
			Point fp = intersectionPoint.getCopy ();
			fp.y -= radius;
			return fp;
		}
		int incr = -radius;
		double xIncr = incr;
		double yIncr = slope * xIncr;
		if (Math.abs (xIncr) < Math.abs (yIncr)) {
			yIncr = incr;
			xIncr = yIncr * 1 / slope;
		}
		Point fp = intersectionPoint.getCopy ();
		fp.x += xIncr;
		fp.y += yIncr;
		return fp;
	}

	private ArrayList collectConnections() {
		ArrayList result = new ArrayList ();
		collectConnections (this.getParent (), result);
		return result;
	}

	private void collectConnections(IFigure figure, ArrayList list) {
		for (Iterator I = figure.getChildren ().iterator (); I.hasNext (); ) {
			IFigure child = (IFigure) I.next ();

			//if (child == this)
			//continue;

//			collectConnections (child, list);
			if (!(child instanceof RoundedJumpingRoleConnection)) 
				continue;
			if (child.getBounds().intersects(this.getBounds()))
					list.add (child);
		}
	}

	private IFigure getRoot() {
		IFigure figure = this;
		while (figure.getParent () != null)
			figure = figure.getParent ();
		return figure;
	}

	/**
	 * @param intersections
	 */
	public void addIntersections(Point bp1, Point bp2, List intersections) {
		synchronized (_points) {
			int i1 = -1; int i2 = -1;
			for (int i = 0; i < _points.size () && (i1 == -1 || i2 == -1); i++) {
				if (_points.get(i).equals (bp1)) i1 = i;
				if (_points.get (i).equals (bp2)) i2 = i;
			}
			
			if (i1 != -1 && i2 != -1) {
				if (i2 < i1) {
					int t = i2;
					i2 = i1;
					i1 = t;
				}
				for (Iterator iter = intersections.iterator (); iter.hasNext ();) {
					IntersectingPoint point = (IntersectingPoint )iter.next ();
					if (!_points.get(i2-1).equals (point))
						_points.add(i2++, point);
				}
			}
		}
	}

	/**
	 * 
	 */
	public  void clearIntersections() {
		synchronized (_points) {
		for (Iterator iter = _points.iterator (); iter.hasNext ();) {
			Point p = (Point )iter.next ();
			if (p instanceof IntersectingPoint) {
				_points.remove(p);
			}
		}
		}
	}
	public boolean	_isMoving	= false;
}