package AcmeStudio.ui.editors.diagram.figures;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.*;
import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.RootEditPart;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.core.ASModel;
import AcmeStudio.model.ModelHelper;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.ui.editors.diagram.IElementFactory;
import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.ui.editors.diagram.styles.*;
import AcmeStudio.ui.editors.diagram.styles.ConnectorVisualization.RoleRouteOptions;
import AcmeStudio.ui.editors.diagram.styles.constants.SideOptions;
import AcmeStudio.ui.editors.diagram.visualInfo.VisualInformation;
import AcmeStudio.util.DesignEvaluationCache;
import AcmeStudio.util.Geometry;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.extensions.IElementExtension;
import acmeLib.core.filters.AcmeObjectFilter;
import acmeLib.elements.*;
import acmeLib.properties.AcmeFloat;
import acmeLib.properties.AcmeProperty;
import acmeLib.properties.FloatType;
import com.swtdesigner.SWTResourceManager;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * Provides the figure for AcmeSystem.
 * 
 */
public class SystemView
	extends FreeformLayer
	implements IAcmeElementFigure, PropertyChangeListener {

	/**
	 * The AcmeSystem represented by this figure
	 */
	private AcmeSystem _model = null;
	private SystemEditPart _part = null;

	private SystemVisualization _vis = null;

	/**
	 * The representation of the model
	 */
	private AcmeElement _representationOf;

	/**
	 * The representation border
	 */
	private RepresentationBorderView _representationBorder;

	private Rectangle _bounds = null;

	/**
	 * The representation dimension
	 */
	//    private Dimension _visSize;

	private boolean _shouldComputeChildLocations = false;

	public boolean _showRepresentation = true;

	/**
	 * The overview image of the figure
	 */
	private Image _overview = null;

	/**
	 * Whether there is an error in the system
	 */
	private boolean _hasError = false;
	private ImageFigure _imgErr = null;

	private Hashtable _childLocations = new Hashtable();
	private Hashtable _childPlacementRules = new Hashtable();
	private HashMap _nameMap = new HashMap();

	private boolean _elementsLoaded = false;
	private boolean	_shouldSortPaintingOrder = false;
	
    /**
     * @return
     */
    public boolean isElementsLoaded() {
        return _elementsLoaded;
    }

    /**
     * @param loaded
     */
    public void setElementsLoaded(boolean loaded) {
        _elementsLoaded = loaded;
    }

	/**
	 * Constructor
	 */
	public SystemView() {
		setLayoutManager(new FreeformLayout());
		setBorder(new MarginBorder(0));
		setBackgroundColor(ElementColorConstants.backgroundColor);
		setOpaque(false);

	}

	/**
	 * Set the model represented by this figure
	 * @param model The model to be represented
	 */
	public void setModel(AcmeElement model) {
		this._model = (AcmeSystem) model;

		if (_model.getParentRepresentation() != null) {
			_representationOf = model.getParentRepresentation().getParent();

			if ((_representationOf instanceof AcmeComponent || _representationOf instanceof AcmeConnector)) {
				_representationBorder =
					new RepresentationBorderView(
						_representationOf,
						bounds,
						_model);
				_representationBorder.setOpaque(false);
				this.add(_representationBorder);
			}
		}
		ASModel
			.current()
			.getRuleManager()
			.addPropertyChangeListener(this);

		
		resetSysVis();
	}

	/**
	 * Resets the visualization, by relooking it up and changing the
	 * registration of change to the new visualization.
	 */
	public void resetSysVis() {
		SystemVisualization newVis = VisualStyleRegistry.getSysVis(_model);
		if (newVis != _vis) {
			if (_vis != null)
				_vis.removePropertyChangeListener(
					_model.getQualifiedName(),
					this);
			_vis = newVis;
			_vis.addPropertyChangeListener(_model.getQualifiedName(), this);
			//if (_editPart != null) _editPart.refresh();
			_shouldComputeChildLocations = true;
			//this.repaint();
		}
	}

	/**
	 * Get the model this figure is representing
	 * @return The AcmeSystem model
	 */
	public AcmeElement getModel() {
		return this._model;
	}

	/**
	 * Get the overview of this figure
	 * @return The overview
	 */
	public Image getOverview() {
		paintOverview ();
		return this._overview;
	}

	/**
	 * Override org.eclipse.draw2d.figure#paint(Graphics) to
	 * control overlapping by adjusting painting order of 
	 * the children
	 *
	 * @see org.eclipse.draw2d.figure#paint(Graphics)
	 */
	public void paint(Graphics graphics) {
		AcmeSystem system = (AcmeSystem )getModel ();
//		if (_representationBorder != null && system.getUserData (DiagramEditor.INLINEREPRESENTATION) != null) {
//			_representationBorder.setVisible(false);
//		}
//		else if (_representationBorder != null) {
//			_representationBorder.setVisible (true);
//		}
		if (_showRepresentation) {
			
		}
		//if (_isVisible) {
		paintGraphics(graphics);
//		paintOverview();

		      if (_hasError) {
		          
		           
		//            _imgErr.setLocation (new Point (this.getClientArea().x, this.getClientArea().y));
		          Rectangle clip = Rectangle.SINGLETON;
		          if (_imgErr.intersects(graphics.getClip(clip))) {
		          
		          _imgErr.setLocation (new Point (graphics.getClip (clip).x, graphics.getClip (clip).y));
		          }
		      
		}

	}

	public void paintOverview() {
		Rectangle r = getSystemBounds(this._model);
		if (r == null || r.width < 1 || r.height < 1)
			r = getBounds();
		if (r.width < 1 || r.height < 1)
			return;
		String key = _model.getName() + "_overview";
		if (_overview != null) {
			ImageManager.imageDisposed(
				"SystemView.paintOverview." + key,
				_overview);
			ImageManager.disposeImage(key);
			_overview.dispose();
		}
		Image srcImg = new Image(Display.getDefault(), this.getBounds().width
		 - r.x
		, this.getBounds().height
		 - r.y
		);
		GC gc = new GC(srcImg);
		Graphics g = new SWTGraphics(gc);
		g.translate(-r.x, -r.y);
		paintGraphics(g);
		_overview =
			ImageManager.createImage(
				key,
				getBounds().width,
				getBounds().height, false);
		ImageManager.imageCreated("SystemView.paintOverview", srcImg);
		gc.copyArea(_overview, 0, 0);
		gc.dispose();
		ImageManager.imageDisposed("SystemView.paintOverview", srcImg);
		srcImg.dispose();
	}

	public void paintGraphics(Graphics graphics) {
		// Move border outline to the back
		if (_representationBorder != null) {
			List children = getChildren();
			int index = getChildren().indexOf(_representationBorder);
			if (index != 0) {
				for (int i = index; i > 0; i--) {
					children.set(i, children.get(i - 1));
				}
				children.set(0, _representationBorder);
			}
			if (getModel ().getUserData(DiagramEditor.INLINEREPRESENTATION) != null) {
				_representationBorder.setNoLabel (true);
			}
			else {
				_representationBorder.setNoLabel (false);
			}
		}

       if (_shouldSortPaintingOrder) {
		// Push BindingView to the end of the list
		// so that it will be shown on top of other figures
		//_shouldSortPaintingOrder = false;
		List children = getChildren();
		int n = children.size();
		int j = n - 1;
		for (int i = 0; i < j; i++) {
			Object obj = children.get(i);
			if (obj instanceof BindingView) {
				children.set(i, children.get(j));
				children.set(j, obj);
				j--;
			}
		}

		// Make sure ports are before their host components
		Hashtable portViews = new Hashtable();
		Hashtable portViewIndex = new Hashtable();
		for (int i = 0; i < n; i++) {
			Object obj = children.get(i);
			if (obj instanceof PortView) {
				PortView pv = (PortView) obj;
				portViews.put(pv.getModel(), pv);
				portViewIndex.put(pv.getModel(), new Integer(i));
			}
			if (obj instanceof ComponentView) {
				ComponentView cv = (ComponentView) obj;
				AcmeComponent acmeComponent = (AcmeComponent) cv.getModel();
				Vector acmePorts = Helper.getChildren(acmeComponent);
				if (acmePorts != null) {
					int index = n;
					for (int k = 0; k < acmePorts.size(); k++) {
						AcmePort acmePort = (AcmePort) acmePorts.get(k);
						PortView pv = (PortView) portViews.get(acmePort);
						if (pv != null) {
							int newIndex =
								((Integer) portViewIndex.get(acmePort))
									.intValue();
							if (newIndex < index) {
								index = newIndex;
							}
						}
					}
					if (index < n) {
						for (j = i; j > index; j--) {
							children.set(j, children.get(j - 1));
						}
						children.set(index, cv);
					}
				}
			}
		}

		// Make sure roles are behind their attached ports
		portViews = new Hashtable();
		portViewIndex = new Hashtable();
		for (int i = 0; i < n; i++) {
			Object obj = children.get(i);
			if (obj instanceof PortView) {
				PortView pv = (PortView) obj;
				portViews.put(pv.getModel(), pv);
				portViewIndex.put(pv.getModel(), new Integer(i));
			}
			if (obj instanceof RoleView) {
				RoleView rv = (RoleView) obj;
				AcmeRole acmeRole = (AcmeRole) rv.getModel();
				AcmePort acmePort = ModelHelper.findAttachedPort(acmeRole);
				if (acmePort != null) {
					PortView pv = (PortView) portViews.get(acmePort);
					if (pv != null) {
						int index =
							((Integer) portViewIndex.get(acmePort)).intValue();
						for (j = i; j > index; j--) {
							children.set(j, children.get(j - 1));
						}
						children.set(index, rv);
					}
				}
			}
		}

		// Make sure roles are behind their host connectors
		Hashtable roleViews = new Hashtable();
		Hashtable roleViewIndex = new Hashtable();
		for (int i = n - 1; i >= 0; i--) {
			Object obj = children.get(i);
			if (obj instanceof RoleView) {
				RoleView rv = (RoleView) obj;
				roleViews.put(rv.getModel(), rv);
				roleViewIndex.put(rv.getModel(), new Integer(i));
			}
			if (obj instanceof ConnectorView) {
				ConnectorView cv = (ConnectorView) obj;
				AcmeConnector acmeConnector = (AcmeConnector) cv.getModel();
				Vector acmeRoles = Helper.getChildren(acmeConnector);
				if (acmeRoles != null) {
					int index = -1;
					for (int k = 0; k < acmeRoles.size(); k++) {
						AcmeRole acmeRole = (AcmeRole) acmeRoles.get(k);
						RoleView rv = (RoleView) roleViews.get(acmeRole);
						if (rv != null) {
							int newIndex =
								((Integer) roleViewIndex.get(acmeRole))
									.intValue();
							if (newIndex > index) {
								index = newIndex;
							}
						}
					}
					if (index > -1) {
						for (j = i; j < index; j++) {
							children.set(j, children.get(j + 1));
						}
						children.set(index, cv);
					}
				}
			}
		}
       }
       
		super.paint(graphics);
		//        }
	}

	private void paintGrid(Graphics graphics, List children) {
		SystemVisualization sysVis = getSysVis();
		if (sysVis == null)
			return;

		if (!_elementsLoaded) {
			for (int i = 0; i < children.size(); i++) {
				_elementsLoaded = true;
				_shouldComputeChildLocations = true;
				IFigure child = (IFigure) children.get(i);
				if (child instanceof AcmeVisElement) {
					AcmeVisElement ave = (AcmeVisElement) child;
					AcmeElement e = ave.getModel();
					if (e instanceof AcmeComponent
						|| e instanceof AcmeConnector) {
						updateElement(e, child.getBounds().getCopy());
					}
				}
			}
		}

		Rectangle r = getBounds();
		if (_bounds == null) {
			_bounds = r.getCopy();
		} else {
			if (r.height != _bounds.height
				|| r.width != _bounds.width
				|| _shouldComputeChildLocations) {
				_bounds = r.getCopy();
				computeChildLocations();
				_shouldComputeChildLocations = false;
				_part.refresh();
			}
		}

		MultiPageAcmeEditor mp = ASContext.current().getActiveEditor();
		if (mp != null) {
			DiagramEditor editor = mp.getDiagramEditor();
			if (editor != null && editor.isShowGrid()) {
				graphics.pushState();
				graphics.setLineWidth(2);
				graphics.setForegroundColor(SWTResourceManager.getColor ( 0xEE, 0xEE, 0xFF));
				graphics.drawRectangle(r);
				graphics.setLineStyle(Graphics.LINE_DASHDOTDOT);
				int rows = sysVis.getRows();
				int cols = sysVis.getCols();
				rows = rows == 0 ? 1 : rows;
				cols = cols == 0 ? 1 : cols;

				int w = r.width / cols;
				int h = r.height / rows;
				for (int i = 0; i < rows; i++) {
					int y = r.y + i * h;
					graphics.drawLine(r.x, y, r.right(), y);
				}
				for (int i = 0; i < cols; i++) {
					int x = r.x + i * w;
					graphics.drawLine(x, r.y, x, r.bottom());
				}
				graphics.restoreState();
				graphics.popState();
			}
		}
	}

	protected void paintChildren(Graphics graphics) {
		List children = getChildren();
		IFigure child;

//		paintGrid(graphics, children);

		Rectangle clip = Rectangle.SINGLETON;
		for (int i = 0; i < children.size(); i++) {
			child = (IFigure) children.get(i);
			if (child.isVisible()
				&& child.intersects(graphics.getClip(clip))) {
				graphics.pushState();
				graphics.clipRect(child.getBounds().getCopy().expand(50, 50));
				

				if (child instanceof AcmeVisElement
					|| child instanceof RoleView) {
					AcmeVisElement ave = null;
					ElementVisualization vis = null;
					if (child instanceof RoleView) {
						RoleView	roleView	= (RoleView) child;
						ave = (roleView).getDecoration();
						vis = (roleView).getVisualization();
					} else {
						ave = (AcmeVisElement) child;
						vis = ave.getVisualization();
					}
					if (vis.getReplicateShape()) {
						double rotation = 0;
						if (ave instanceof PortView) {
							rotation = ((PortView) ave).getRotation();
							if (rotation == 100)
								rotation = 0;
						} else if (ave instanceof RoleDecoration) {
							rotation = ((RoleDecoration) ave).getRotation();
							if (rotation == 100)
								rotation = 0;
						}
						Rectangle r =
							new Rectangle(
								ave.getLocation().translate(6, 6),
								ave.getSize());
						vis.fillShape(graphics, r, null, rotation);
						vis.outlineShape(graphics, r);
						r =
							new Rectangle(
								ave.getLocation().translate(3, 3),
								ave.getSize());
						vis.fillShape(graphics, r, null, rotation);
						vis.outlineShape(graphics, r);
					}
				}
				child.paint(graphics);

				IFigure labelFigure = child;
				if (child instanceof RoleView) {
					RoleView	roleView	= (RoleView) child;
					labelFigure = (roleView).getDecoration();
					/*if (labelFigure.isVisible() || roleView._isMoving) */labelFigure.paint(graphics);
				}
				graphics.restoreState();
				graphics.popState();

				if (labelFigure instanceof AcmeVisElement) {
					AcmeVisElement ave = (AcmeVisElement) labelFigure;
					Rectangle constraint = (Rectangle )getLayoutManager().getConstraint(ave);
					if (constraint == null) {
						constraint = ave.getBounds ();
					}
					ElementVisualization vis = ave.getVisualization();
                    if (ave.isTransparent()) {
                        vis.drawLabel (graphics, ave.getModel (), constraint.getLocation(), constraint.getSize (), ave.getTransparency ());
                    }
                    else {
					//vis.fillShape(graphics, ave.getModel(), ave.getLocation().translate(10, 10), ave.getSize());
					vis.drawLabel(
						graphics,
						ave.getModel(),
						constraint.getLocation(),
						constraint.getSize());
                    }
				}
			}
		}
	}

	public AcmeVisElement findFigureByLocation(Point location) {
		List children = getChildren();
		int n = children.size();
		boolean found = false;
		AcmeVisElement figure = null;
		for (int i = 0; !found && i < n; i++) {
			Object obj = children.get(i);
			/*
			 * Filter out the representation. This fixes Bug #50
			 */
			if (obj instanceof AcmeVisElement
				&& !(obj instanceof RepresentationBorderView)) {
				figure = (AcmeVisElement) obj;
				if (figure.getBounds().contains(location)) {
					found = true;
				}
			}
		}
		if (!found) {
			figure = null;
		}
		return figure;
	}

	public RepresentationBorderView getComponentRepresentationBorder() {
		return (RepresentationBorderView) _representationBorder;
	}

	public void setBindingPoints(List bp) {
	}

	public ElementVisualization getVisualization() {
		return null;
	}

	public SystemVisualization getSysVis() {
		SystemVisualization sysVis = null;
		if (_model != null)
			sysVis = VisualStyleRegistry.getSysVis(_model);
		return sysVis;
	}

	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String propertyName = evt.getPropertyName();
		if (propertyName.equals(DesignEvaluationCache.PROP_ELEMENT_TYPECHECK)
			&& evt.getNewValue() == _model
			&& AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(
				"DiagramEditor.DisplayFloatingError")) {
			_hasError = evt.getOldValue() != null;
			if (_hasError) {
				if (_imgErr == null) {
					createErrorFigure ();
				}
				_imgErr.setVisible (true);
			}
			// BugFix 116
			else if (
				_imgErr != null
					&& _imgErr.getParent() == this
					&& this.getChildren().contains(_imgErr)) {
				_imgErr.setVisible(false);
			}
			repaint();
		} else if (propertyName.equals(Visualization.PROP_VIS_RESET)) {
			resetSysVis();
		}
		//repaint();
	}

	private void createErrorFigure() {
		Image img = ImageManager.getIcon(ImageManager.ID_BIG_ERROR);
		_imgErr = new ImageFigure(img);
		_imgErr.setSize(
			img.getBounds().width,
			img.getBounds().height);
		this.add(_imgErr);
	}

	public void updateElement(AcmeElement newElement, Rectangle r) {
		SystemVisualization vis = getSysVis();
		if (vis == null)
			return;
		PlacementRule rule = vis.getPlacementRule(newElement);
		if (rule != null) {
			String name = newElement.getName();
			_nameMap.put(name, newElement);
			_childPlacementRules.put(name, rule);
			_childLocations.put(name, new Location(r));
			_shouldComputeChildLocations = true;
		}
	}

	public Location getCalculatedLocation(AcmeElement e) {
		return (Location) _childLocations.get(e.getName());
	}

	public HashMap resetLayout() {
		Iterator iter = _childLocations.keySet().iterator();
		HashMap locations = new HashMap();
		while (iter.hasNext()) {
			String key = (String) iter.next();
			Location l = (Location) _childLocations.get(key);
			if (l.getStatus() == Location.DISABLED) {
				l.setStatus(Location.ENABLED);
				AcmeElement e = (AcmeElement) _nameMap.get(key);
				locations.put(key, getLocation(e, false));
				Helper.clearX(e);
				Helper.clearY(e);
				_shouldComputeChildLocations = true;
			}
		}
		repaint();
		return locations;
	}

	public void undoResetLayout(HashMap locations) {
		Iterator iter = _childLocations.keySet().iterator();
		while (iter.hasNext()) {
			String key = (String) iter.next();
			if (locations.containsKey(key)) {
				Location l = (Location) _childLocations.get(key);
				l.setStatus(Location.DISABLED);
				AcmeElement e = (AcmeElement) _nameMap.get(key);
				setLocation(e, (Point) locations.get(key));
				this._shouldComputeChildLocations = true;
			}
		}
		repaint();
	}

	public void computeChildLocations() {
		SystemVisualization vis = getSysVis();
		if (vis == null)
			return;
		int cols = vis.getCols();
		int rows = vis.getRows();
		rows = rows == 0 ? 1 : rows;
		cols = cols == 0 ? 1 : cols;

		int gridCount = cols * rows;
		if (gridCount == 1) return;
		
		Vector names[] = new Vector[gridCount + 1];
		for (int i = 1; i <= gridCount; i++) {
			names[i] = new Vector();
		}
		Iterator iter = _childPlacementRules.keySet().iterator();
		while (iter.hasNext()) {
			String key = (String) iter.next();
			PlacementRule rule = (PlacementRule) _childPlacementRules.get(key);
			names[rule.getRegion()].add(key);
		}
		for (int i = 1; i <= gridCount; i++) {
			Vector nameList = names[i];
			Collections.sort(nameList);
			Vector ruleList = new Vector();
			for (int j = 0; j < nameList.size(); j++) {
				ruleList.add(_childPlacementRules.get(nameList.get(j)));
			}
			if (ruleList.size() > 0) {
				computeChildLocations(
					nameList,
					ruleList,
					computeRegion(cols, rows, i, getBounds()));
			}
		}
	}

	private Rectangle computeRegion(
		int cols,
		int rows,
		int region,
		Rectangle r) {
		final Dimension myBorder = new Dimension(1, 1);
		int w = r.width / cols;
		int h = r.height / rows;
		int xIndex = (region - 1) % cols;
		int yIndex = (region - 1) / cols;
		int x = r.x + w * xIndex;
		int y = r.y + h * yIndex;
		//        int x = w * xIndex;
		//        int y = h * yIndex;
		return (new Rectangle(x, y, w, h)).shrink(myBorder.width, myBorder.height);
	}

	private void computeChildLocations(
		Vector nameList,
		Vector ruleList,
		Rectangle r) {

		final int MARGIN = 10;
		LinkedList leftElems = new LinkedList();
		LinkedList rightElems = new LinkedList();
		LinkedList topElems = new LinkedList();
		LinkedList bottomElems = new LinkedList();
		LinkedList hcenterElems = new LinkedList();
		LinkedList vcenterElems = new LinkedList();

		LinkedList leftAlignElems = new LinkedList();
		LinkedList rightAlignElems = new LinkedList();
		LinkedList topAlignElems = new LinkedList();
		LinkedList bottomAlignElems = new LinkedList();
		LinkedList centerAlignElems = new LinkedList();
		for (int i = 0; i < nameList.size(); i++) {
			PlacementRule rule = (PlacementRule) ruleList.get(i);
			String name = (String) nameList.get(i);

			Location oldLocation = (Location) _childLocations.get(name);
			if (oldLocation.getStatus() == Location.DISABLED)
				continue;

			if (rule._side == SideOptions.LEFT) {
				leftElems.add(name);
			} else if (rule._side == SideOptions.RIGHT) {
				rightElems.add(name);
			} else if (rule._side == SideOptions.HCENTER) {
				hcenterElems.add(name);
			} else if (rule._side == SideOptions.TOP) {
				topElems.add(name);
			} else if (rule._side == SideOptions.BOTTOM) {
				bottomElems.add(name);
			} else if (rule._side == SideOptions.VCENTER) {
				vcenterElems.add(name);
			}

			if (rule._alignment == AlignmentOptions.LEFT) {
				leftAlignElems.add(name);
			} else if (rule._alignment == AlignmentOptions.RIGHT) {
				rightAlignElems.add(name);
			}
			if (rule._alignment == AlignmentOptions.TOP) {
				topAlignElems.add(name);
			} else if (rule._alignment == AlignmentOptions.BOTTOM) {
				bottomAlignElems.add(name);
			}
			if (rule._alignment == AlignmentOptions.CENTER) {
				centerAlignElems.add(name);
			}
		}

		int currentLeftY = MARGIN,
			currentRightY = MARGIN,
			currentHCenterY = MARGIN,
			currentTopX = MARGIN,
			currentBottomX = MARGIN,
			currentVCenterX = MARGIN;
		int currentLeftMaxY = MARGIN,
			currentRightMaxY = MARGIN,
			currentHCenterMaxY = MARGIN,
			currentTopMaxX = MARGIN,
			currentBottomMaxX = MARGIN,
			currentVCenterMaxX = MARGIN;
		int currentLeftMidCount = 0,
			currentRightMidCount = 0,
			currentTopMidCount = 0,
			currentBottomMidCount = 0,
			currentVCenterMidCount = 0,
			currentHCenterMidCount = 0;
		for (int i = 0; i < nameList.size(); i++) {
			String name = (String) nameList.get(i);
			Location oldLocation = (Location) _childLocations.get(name);
			if (oldLocation.getStatus() == Location.DISABLED)
				continue;
			Rectangle oldRect = oldLocation.getRectangle();
			int spacing = MARGIN;
			Point newPos = new Point();
			if (leftElems.contains(name)) {
				newPos.x = r.x + oldRect.width / 2 + spacing;
				if (topAlignElems.contains(name)) {
					newPos.y = r.y + oldRect.height / 2 + currentLeftY;
					currentLeftY += spacing + oldRect.height;
				} else if (bottomAlignElems.contains(name)) {
					newPos.y =
						r.y + r.height - oldRect.height / 2 - currentLeftMaxY;
					currentLeftMaxY += spacing + oldRect.height;
				} else if (centerAlignElems.contains(name)) {
					newPos = null;
					currentLeftMidCount++;
				}
			} else if (rightElems.contains(name)) {
				newPos.x = r.x + r.width - oldRect.width / 2 - spacing;
				if (topAlignElems.contains(name)) {
					newPos.y = r.y + oldRect.height / 2 + currentRightY;
					currentRightY += spacing + oldRect.height;
				} else if (bottomAlignElems.contains(name)) {
					newPos.y =
						r.y + r.height - oldRect.height / 2 - currentRightMaxY;
					currentRightMaxY += spacing + oldRect.height;
				} else if (centerAlignElems.contains(name)) {
					newPos = null;
					currentRightMidCount++;
				}
			} else if (hcenterElems.contains(name)) {
				newPos.x = r.x + r.width / 2;
				if (topAlignElems.contains(name)) {
					newPos.y = r.y + oldRect.height / 2 + currentHCenterY;
					currentHCenterY += spacing + oldRect.height;
				} else if (bottomAlignElems.contains(name)) {
					newPos.y =
						r.y
							+ r.height
							- oldRect.height / 2
							- currentHCenterMaxY;
					currentHCenterMaxY += spacing + oldRect.height;
				} else if (centerAlignElems.contains(name)) {
					newPos = null;
					currentHCenterMidCount++;
				}
			} else if (topElems.contains(name)) {
				newPos.y = r.y + oldRect.height / 2 + spacing;
				if (leftAlignElems.contains(name)) {
					newPos.x = r.x + oldRect.width / 2 + currentTopX;
					currentTopX += spacing + oldRect.width;
				} else if (rightAlignElems.contains(name)) {
					newPos.x =
						r.x + r.width - oldRect.width / 2 - currentTopMaxX;
					currentTopMaxX += spacing + oldRect.width;
				} else if (centerAlignElems.contains(name)) {
					newPos = null;
					currentTopMidCount++;
				}
			} else if (bottomElems.contains(name)) {
				newPos.y = r.y + r.height - oldRect.height / 2 - spacing;
				if (leftAlignElems.contains(name)) {
					newPos.x = r.x + currentBottomX;
					currentBottomX += spacing + oldRect.width;
				} else if (rightAlignElems.contains(name)) {
					newPos.x =
						r.x + r.width - oldRect.width / 2 - currentBottomMaxX;
					currentBottomMaxX += spacing + oldRect.width;
				} else if (centerAlignElems.contains(name)) {
					newPos = null;
					currentBottomMidCount++;
				}
			} else if (vcenterElems.contains(name)) {
				newPos.y = r.y + r.height / 2;
				if (leftAlignElems.contains(name)) {
					newPos.x = r.x + oldRect.width / 2 + currentVCenterX;
					currentVCenterX += spacing + oldRect.width;
				} else if (rightAlignElems.contains(name)) {
					newPos.x =
						r.x + r.width - oldRect.width / 2 - currentVCenterMaxX;
					currentVCenterMaxX += spacing + oldRect.width;
				} else if (centerAlignElems.contains(name)) {
					newPos = null;
					currentVCenterMidCount++;
				}
			}

			if (newPos != null) {
				_childLocations.put(
					name,
					new Location(
						new Rectangle(
							newPos.x,
							newPos.y,
							oldRect.width,
							oldRect.height)));
			} else {
				_childLocations.put(
					name,
					new Location(
						new Rectangle(-1, -1, oldRect.width, oldRect.height)));
			}
		}

		// Handle "center" cases
		int currentLeftMidY = 0,
			currentRightMidY = 0,
			currentTopMidX = 0,
			currentBottomMidX = 0,
			currentVCenterMidX = 0,
			currentHCenterMidY = 0;
		for (int i = 0; i < nameList.size(); i++) {
			String name = (String) nameList.get(i);
			Location oldLocation = (Location) _childLocations.get(name);
			if (oldLocation.getStatus() == Location.DISABLED)
				continue;

			Rectangle oldRect = oldLocation.getRectangle();
			if (oldRect.x == -1 && oldRect.y == -1) {
				int spacing = MARGIN;
				Point newPos = new Point();
				if (topElems.contains(name)) {
					int half = oldRect.width / 2;
					newPos.y = r.y + oldRect.height / 2 + spacing;
					int xStart =
						r.x
							+ r.width / 2
							- (currentTopMidCount * half
								+ (currentTopMidCount - 1) * spacing / 2);
					newPos.x =
						xStart + currentTopMidX * (spacing + half * 2) + half;
					currentTopMidX++;
				} else if (bottomElems.contains(name)) {
					int half = oldRect.width / 2;
					newPos.y = r.y + r.height - oldRect.height / 2 - spacing;
					int xStart =
						r.x
							+ r.width / 2
							- (currentBottomMidCount * half
								+ (currentBottomMidCount - 1) * spacing / 2);
					newPos.x =
						xStart
							+ currentBottomMidX * (spacing + half * 2)
							+ half;
					currentBottomMidX++;
				} else if (vcenterElems.contains(name)) {
					int half = oldRect.width / 2;
					newPos.y = r.y + r.height / 2;
					int xStart =
						r.x
							+ r.width / 2
							- (currentVCenterMidCount * half
								+ (currentVCenterMidCount - 1) * spacing / 2);
					newPos.x =
						xStart
							+ currentVCenterMidX * (spacing + half * 2)
							+ half;
					currentVCenterMidX++;
				} else if (leftElems.contains(name)) {
					int half = oldRect.height / 2;
					newPos.x = r.x + oldRect.width / 2 + spacing;
					int yStart =
						r.y
							+ r.height / 2
							- (currentLeftMidCount * half
								+ (currentLeftMidCount - 1) * spacing / 2);
					newPos.y =
						yStart + currentLeftMidY * (spacing + half * 2) + half;
					currentLeftMidY++;
				} else if (rightElems.contains(name)) {
					int half = oldRect.height / 2;
					newPos.x = r.x + r.width - oldRect.width / 2 - spacing;
					int yStart =
						r.y
							+ r.height / 2
							- (currentRightMidCount * half
								+ (currentRightMidCount - 1) * spacing / 2);
					newPos.y =
						yStart + currentRightMidY * (spacing + half * 2) + half;
					currentRightMidY++;
				} else if (hcenterElems.contains(name)) {
					int half = oldRect.height / 2;
					newPos.x = r.x + r.width / 2;
					int yStart =
						r.y
							+ r.height / 2
							- (currentHCenterMidCount * half
								+ (currentHCenterMidCount - 1) * spacing / 2);
					newPos.y =
						yStart
							+ currentHCenterMidY * (spacing + half * 2)
							+ half;
					currentHCenterMidY++;
				}

				_childLocations.put(
					name,
					new Location(
						new Rectangle(
							newPos.x,
							newPos.y,
							oldRect.width,
							oldRect.height)));
			}
		}
	}

	public static boolean isRectMaximized(Rectangle rect) {
		return (rect != null && rect.x != Integer.MAX_VALUE);
	}

	public void adjustErrorFigure(Viewport vp) {
		if (_hasError) {
			_imgErr.setLocation(vp.getClientArea().getLocation());
		}
	}
	
	/**
	 * Resets all visualization information associated with the element
	 * @param element the element to remove visual info from
	 */
	public static void removeVisualization(AcmeElement element) {
		if (!AcmeStudioPlugin.getDefault ().getPreferenceStore ().getBoolean (GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeEnumeration visProperties = element.getProperties().castToAcmeSetVector().enumerate(new AcmeObjectFilter () {

				public boolean test(AcmeObject obj) {
					if (obj instanceof AcmeProperty && ((AcmeProperty )obj).getName ().startsWith("vis-"))
						return true;
					return false;
				}
				
			});
			
			while (visProperties.moreItems ()) {
				AcmeProperty prop = (AcmeProperty )visProperties.getNextItem ();
				element.removeProperty(prop);
			}
		}
		else {
			element.setUserData("vis-information", null);
		}
	}

	public static void setLocation(AcmeElement element, Point location) {
		if (!AcmeStudioPlugin.getDefault() .getPreferenceStore().getBoolean (GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visX = element.getProperty("vis-x");
			AcmeProperty visY = element.getProperty("vis-y");
			if (visX == null) {
				visX = new AcmeProperty(new FloatType("float"));
				try {
					visX.setName("vis-x");
					element.addProperty(visX);
				} catch (AcmeIllegalNameException e) {
					return;
				}
			}
			if (visY == null) {
				visY = new AcmeProperty(new FloatType("float"));
				try {
					visY.setName("vis-y");
					element.addProperty(visY);
				} catch (AcmeIllegalNameException e) {
					return;
				}
			}
			visX.setValue(new AcmeFloat(location.x));
			visY.setValue(new AcmeFloat(location.y));
		}
		else {
			VisualInformation vis = (VisualInformation )element.getUserData ("vis-information");
			if (vis == null) {
				vis = new VisualInformation ();
				element.setUserData ("vis-information", vis);
			}
			vis.setX(location.x);
			vis.setY(location.y);
		}
	}

	public void setElementLocation(
		AcmeElement element,
		Point location,
		boolean force) {
		Location l = getCalculatedLocation(element);
		if (force) {
			setLocation(element, location);
			if (l != null) {
				l.setStatus(Location.DISABLED);
			}
		} else if (l == null) {
			setLocation(element, location);
		}
	}

	public Point getElementLocation(AcmeElement element) {
		Point location = getLocation(element, false);
		if (location != null) {
			Location l = getCalculatedLocation(element);
			if (l != null) {
				l.setStatus(Location.DISABLED);
				l.setRectangle(
					new Rectangle(location, Helper.getSize(element)));
			}
			return location;
		}

		Location childBounds = getCalculatedLocation(element);
		if (childBounds == null) {
			location = getLocation(element, false);
		} else {
			location = childBounds.getRectangle().getLocation();
		}

		// randomly set the location
		if (location == null) {
			location = new Point(Math.random()*100, Math.random()*100);
		}

		return location;
	}

	public Point getPortLocation (AcmePort port) {
		AcmeComponent comp = (AcmeComponent )port.getParent ();
		GraphicalViewer gv = (GraphicalViewer )this.getPart().getAdapter (GraphicalViewer.class);
		if (gv == null) return getPortLocation_static (port);
		ComponentEditPart cep = (ComponentEditPart )gv.getEditPartRegistry().get (comp);
		if (cep == null) return getPortLocation_static (port);
		Point location = cep.calculateChildPortLocation(port);
		return location;
	}
	
	
	public static Point getPortLocation_static(AcmePort port) {
		AcmeComponent comp = (AcmeComponent) port.getParent();
		Vector ports = Helper.getChildrenByOrder(comp);
		Map portPositions = normalizePortPositions(ports, comp);
		return (Point )portPositions.get (port);
	}

	public static Point getLocation(
		AcmeElement acmeElement,
		boolean shouldSetValue) {
		boolean isType = false;
		if (acmeElement instanceof AcmeElementType) {
//			Debug.traceln("Getting location of " + acmeElement.getName());
			isType = true;
		}
		Point point = new Point();
		if (!AcmeStudioPlugin.getDefault() .getPreferenceStore().getBoolean (GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			
			AcmeProperty visX = acmeElement.getProperty("vis-x");
			AcmeProperty visY = acmeElement.getProperty("vis-y");
			if (!shouldSetValue
				&& (visX == null || visY == null)
				&& !(acmeElement instanceof AcmePort)) {
				return null;
			}
			boolean hasVis = true;
			try {
				if (acmeElement instanceof AcmePort) {
					return null;
				}
				if (visX == null) {
					visX = new AcmeProperty(new FloatType("float"));
					visX.setName("vis-x");
					visX.setValue(new AcmeFloat((float) Math.random() * 1000));
					if (!isType)
						acmeElement.addProperty(visX);
					hasVis = false;
				}
				if (visY == null) {
					visY = new AcmeProperty(new FloatType("float"));
					visY.setName("vis-y");
					visY.setValue(new AcmeFloat((float) Math.random() * 1000));
					if (!isType)
						acmeElement.addProperty(visY);
					hasVis = false;
				}
				if (acmeElement instanceof AcmeRole && !hasVis) {
					Point pL = getLocation(acmeElement.getParent(), shouldSetValue);
					Dimension pS = Helper.getSize(acmeElement.getParent());
					float xValue = (float) Math.random() * 200 - 100;
					xValue += pL.x;
					if (Math.abs(xValue - pL.x) < pS.width) {
						xValue = xValue + (pS.width + 10);
					}
					visX.setValue(new AcmeFloat(xValue));
					float yValue = (float) Math.random() * 200 - 100;
					yValue += pL.y;
					if (Math.abs(yValue - pL.y) < pS.height) {
						yValue = yValue + (pS.height + 10);
					}
					visY.setValue(new AcmeFloat(yValue));
				}
			} catch (AcmeIllegalNameException e) {
				return point;
			}
			point = new Point();
			Object vX = visX.getValue();
			Object vY = visY.getValue();
			if (vX != null && vY != null) {
				int x = (int) ((AcmeFloat) visX.getValue()).getValue();
				int y = (int) ((AcmeFloat) visY.getValue()).getValue();
				point.setLocation(x, y);
			}
			return point;
		}
		else {
			VisualInformation vis = (VisualInformation) acmeElement.getUserData ("vis-information");
			boolean hadVis = true;
			if (vis == null) {
				if (acmeElement.getProperty ("vis-x") == null) {
					hadVis = false;
					vis = new VisualInformation ();
				}
				else {
					vis = new VisualInformation ();
					vis.extractFromElement(acmeElement);
					hadVis = true;
				}
				if (shouldSetValue && !isType) {
					acmeElement.setUserData ("vis-information", vis);
				}
				else return null;
				if (acmeElement instanceof AcmePort) return null;
				
			}
			if (!hadVis) {
				vis.setX((int )(Math.random ()*1000));
				vis.setY((int )(Math.random () * 1000));
			}
			
			if (acmeElement instanceof AcmeRole && !hadVis) {
				Point pL = getLocation (acmeElement.getParent (), shouldSetValue);
				Dimension pS = Helper.getSize (acmeElement.getParent ());
				int xValue = (int )(Math.random ()*200 - 100);
				xValue += pL.x;
				if (Math.abs (xValue -pL.x) < pS.width) {
					xValue = xValue + (pS.width + 10);
				}
				vis.setX (xValue);
				int yValue = (int )(Math.random () * 200 - 100);
				yValue += pL.y;
				if (Math.abs (yValue - pL.y) < pS.height) {
					yValue = yValue + (pS.height + 10);
				}
				vis.setY (yValue);
			}
			return vis.getLocation ();
		}
	}

	public void savePortPosition(
		AcmePort port,
		AcmeComponent component,
		Point newPos,
		PlacementRule rule) {
		Point compLocation = getElementLocation(component).getCopy();
		Dimension compSize = Helper.getSize(component);
		//        compLocation.x += compSize.width / 2;
		//        compLocation.y += compSize.height / 2;
        
		newPos.x -= compLocation.x;
		newPos.y -= compLocation.y;
		
		if (rule == null
			|| rule._side == SideOptions.DEFAULT
			&& rule._alignment == AlignmentOptions.DEFAULT) {
		    updatePortPositionToBorder (newPos, compSize);
			Helper.setX(port, newPos.x);
			Helper.setY(port, newPos.y);
		} else {
			if (rule._side == SideOptions.DEFAULT
				|| rule._alignment == AlignmentOptions.DEFAULT) {
				if (rule._side == SideOptions.LEFT
					|| rule._side == SideOptions.RIGHT
					|| rule._alignment == AlignmentOptions.LEFT
					|| rule._alignment == AlignmentOptions.RIGHT) {
					Helper.setY(port, newPos.y);
				} else {
					Helper.setX(port, newPos.x);
				}
			}
		}
	}

	public static void updatePortPositionToBorder(Point newPos, Dimension compSize) {
        
        boolean inVerticalBounds = (newPos.y > -compSize.height/2) && (newPos.y < compSize.height/2);
        boolean inHorizontalBounds = (newPos.x > -compSize.width/2) && (newPos.x < compSize.height/2);
        int distanceFromTop = Math.abs(compSize.height / 2 + newPos.y);
        int distanceFromBottom = Math.abs(compSize.height / 2 - newPos.y);
        int distanceFromLeft = Math.abs(compSize.width / 2 + newPos.x);
        int distanceFromRight = Math.abs(compSize.width / 2 - newPos.x);
        int verticalCloseness = Math.min(distanceFromTop, distanceFromBottom);
        int horizontalCloseness = Math.min(distanceFromLeft, distanceFromRight);
        if ((inVerticalBounds && inHorizontalBounds) || (!inVerticalBounds && !inHorizontalBounds)) {
            if (horizontalCloseness <= verticalCloseness) {
            	// Closer to the top or bottom
            	if (distanceFromLeft < distanceFromRight) {
            		// CLoseer to the left
            		newPos.x = -compSize.width / 2;
            	} else {
            		newPos.x = compSize.width / 2;
            	}
            } else {
            	if (distanceFromTop <= distanceFromBottom) {
            		newPos.y = -compSize.height / 2;
            	} else {
            		newPos.y = compSize.height / 2;
            	}
            }
        }
        else if (inVerticalBounds && !inHorizontalBounds) {
            if (distanceFromLeft < distanceFromRight) 
                newPos.x = -compSize.width /2;
            else
                newPos.x = compSize.width/2;
        }
        else if (!inVerticalBounds && inHorizontalBounds) {
            if (distanceFromTop <= distanceFromBottom)
                newPos.y = -compSize.height/2;
            else 
                newPos.y = compSize.height/2;
        }
        if (newPos.x > compSize.width / 2) {
        	newPos.x = compSize.width / 2;
        } else if (newPos.x < -compSize.width / 2) {
        	newPos.x = -compSize.width / 2;
        }
        if (newPos.y > compSize.height / 2) {
        	newPos.y = compSize.height / 2;
        } else if (newPos.y < -compSize.height / 2) {
        	newPos.y = -compSize.height / 2;
        }
    }

	/**
	 * Sets the port positions for the ports of a component, and returns
	   the positions in a map from port->position
    **/
    public static Map normalizePortPositions(
		Vector ports,
		AcmeComponent component) {
		final int MARGIN = 5;
		Map portPositions = new HashMap();
		Object vis = VisualStyleRegistry.getVisualization(component);
		ComponentVisualization cv = null;
		if (vis instanceof ComponentVisualization)
			cv = (ComponentVisualization) vis;
		LinkedList placementRules = null;
		if (cv != null)
			placementRules = cv.getPortPlacementRules();
		Dimension compSize = Helper.getSize(component);
		LinkedList leftPorts = new LinkedList();
		LinkedList rightPorts = new LinkedList();
		LinkedList topPorts = new LinkedList();
		LinkedList bottomPorts = new LinkedList();

		LinkedList leftAlignPorts = new LinkedList();
		LinkedList rightAlignPorts = new LinkedList();
		LinkedList topAlignPorts = new LinkedList();
		LinkedList bottomAlignPorts = new LinkedList();
		LinkedList centerAlignPorts = new LinkedList();
		
		List nonalignedPorts = new LinkedList ();
		
		// Sort ports into their appropriate alignment lists
		for (int i = 0; placementRules != null && i < ports.size(); i++) {
			AcmePort port = (AcmePort) ports.get(i);
			if (port == null) continue;
			PlacementRule rule = getPlacementRuleForPort (placementRules, port);
			if (rule == null
				|| Helper.getX(port) != null
				&& Helper.getY(port) != null) {
				nonalignedPorts.add (port);
				continue;
			}
			if (rule._side == SideOptions.LEFT) {
				leftPorts.add(port);
			} else if (rule._side == SideOptions.RIGHT) {
				rightPorts.add(port);
			}
			if (rule._side == SideOptions.TOP) {
				topPorts.add(port);
			} else if (rule._side == SideOptions.BOTTOM) {
				bottomPorts.add(port);
			}

			if (rule._alignment == AlignmentOptions.LEFT) {
				leftAlignPorts.add(port);
			} else if (rule._alignment == AlignmentOptions.RIGHT) {
				rightAlignPorts.add(port);
			}
			if (rule._alignment == AlignmentOptions.TOP) {
				topAlignPorts.add(port);
			} else if (rule._alignment == AlignmentOptions.BOTTOM) {
				bottomAlignPorts.add(port);
			}
			if (rule._alignment == AlignmentOptions.CENTER) {
				centerAlignPorts.add(port);
			}
		}
		//Point compLocation = getElementLocation(component);
	
		//Point compCenter = compLocation;
		int currentLeftY = MARGIN,
			currentRightY = MARGIN,
			currentTopX = MARGIN,
			currentBottomX = MARGIN;
		int currentLeftMaxY = MARGIN,
			currentRightMaxY = MARGIN,
			currentTopMaxX = MARGIN,
			currentBottomMaxX = MARGIN;
		int currentLeftMidCount = 0,
			currentRightMidCount = 0,
			currentTopMidCount = 0,
			currentBottomMidCount = 0;
		
		for (int i = 0; i < ports.size(); i++) {
			AcmePort port = (AcmePort) ports.get(i);
			if (port == null) {
				continue;
			}
			int spacing = MARGIN;
			Dimension portSize = getRealSize (port);
			if (portSize != null)
				spacing += portSize.width > portSize.height
					? portSize.width
					: portSize.height;
			Point newPos = getAdjustedPortPosition (compSize, port);
			if (leftPorts.contains(port)) {
				newPos.x = -compSize.width / 2;
				if (topAlignPorts.contains(port)) {
					newPos.y = -compSize.height / 2 + currentLeftY;
					currentLeftY += spacing;
				} else if (bottomAlignPorts.contains(port)) {
					newPos.y = compSize.height / 2 - currentLeftMaxY;
					currentLeftMaxY += spacing;
				} else if (centerAlignPorts.contains(port)) {
					newPos = null;
					currentLeftMidCount++;
				}
			} else if (rightPorts.contains(port)) {
				newPos.x = compSize.width / 2;
				if (topAlignPorts.contains(port)) {
					newPos.y = -compSize.height / 2 + currentRightY;
					currentRightY += spacing;
				} else if (bottomAlignPorts.contains(port)) {
					newPos.y = compSize.height / 2 - currentRightMaxY;
					currentRightMaxY += spacing;
				} else if (centerAlignPorts.contains(port)) {
					newPos = null;
					currentRightMidCount++;
				}
			} else if (topPorts.contains(port)) {
				newPos.y = -compSize.height / 2;
				if (leftAlignPorts.contains(port)) {
					newPos.x = -compSize.width / 2 + currentTopX;
					currentTopX += spacing;
					if (currentLeftY == 0) {
						currentLeftY += spacing;
					}
				} else if (rightAlignPorts.contains(port)) {
					newPos.x = compSize.width / 2 - currentTopMaxX;
					currentTopMaxX += spacing;
					if (currentRightY == 0) {
						currentRightY += spacing;
					}
				} else if (centerAlignPorts.contains(port)) {
					newPos = null;
					currentTopMidCount++;
				}
			} else if (bottomPorts.contains(port)) {
				newPos.y = compSize.height / 2;
				if (leftAlignPorts.contains(port)) {
					newPos.x = -compSize.width / 2 + currentBottomX;
					currentBottomX += spacing;
					if (currentLeftMaxY == 0) {
						currentLeftMaxY += spacing;
					}
				} else if (rightAlignPorts.contains(port)) {
					newPos.x = compSize.width / 2 - currentBottomMaxX;
					currentBottomMaxX += spacing;
					if (currentRightMaxY == 0) {
						currentRightMaxY += spacing;
					}
				} else if (centerAlignPorts.contains(port)) {
					newPos = null;
					currentBottomMidCount++;
				}
			}
			portPositions.put(port, newPos);
		}
		int currentLeftMidY = 0,
			currentRightMidY = 0,
			currentTopMidX = 0,
			currentBottomMidX = 0;
		// Handle "default" and "center" cases
		for (int i = 0; i < ports.size(); i++) {
			AcmePort port = (AcmePort) ports.get(i);
			if (port == null) continue;
			Point newPos = (Point) portPositions.get(port);
			if (newPos == null) {
				int spacing = MARGIN;
				int half = 4;
				int offset = 2;
				Dimension portSize = Helper.getSize(port);
				if (portSize == null) {
					portSize =
						VisualStyleRegistry.getVisualization(port).getSize();
				}
				newPos = new Point();
				if (topPorts.contains(port) || topAlignPorts.contains(port)) {
					if (portSize != null) {
						half = portSize.width / 2;
					}
					newPos.y = -compSize.height / 2;
					if (centerAlignPorts.contains(port)) {
						int xStart =
							- (
								currentTopMidCount * half
									+ (currentTopMidCount - 1) * spacing / 2);
						newPos.x =
							xStart
								+ currentTopMidX * (spacing + half * 2)
								+ half;
						currentTopMidX++;
					}
				} else if (
					bottomPorts.contains(port)
						|| bottomAlignPorts.contains(port)) {
					if (portSize != null) {
						half = portSize.width / 2;
					}
					newPos.y = compSize.height / 2;
					if (centerAlignPorts.contains(port)) {
						int xStart =
							- (
								currentBottomMidCount * half
									+ (currentBottomMidCount - 1) * spacing / 2);
						newPos.x =
							xStart
								+ currentBottomMidX * (spacing + half * 2)
								+ half;
						currentBottomMidX++;
					}
				}
				if (leftPorts.contains(port)
					|| leftAlignPorts.contains(port)) {
					if (portSize != null) {
						half = portSize.height / 2;
					}
					newPos.x = -compSize.width / 2;
					if (centerAlignPorts.contains(port)) {
						int yStart =
							- (
								currentLeftMidCount * half
									+ (currentLeftMidCount - 1) * spacing / 2)
								+ offset;
						newPos.y =
							yStart
								+ currentLeftMidY * (spacing + half * 2)
								+ half / 2;
						currentLeftMidY++;
					}
				} else if (
					rightPorts.contains(port)
						|| rightAlignPorts.contains(port)) {
					if (portSize != null) {
						half = portSize.height / 2;
					}
					newPos.x = compSize.width / 2;
					if (centerAlignPorts.contains(port)) {
						int yStart =
							- (
								currentRightMidCount * half
									+ (currentRightMidCount - 1) * spacing / 2)
								+ offset;
						newPos.y =
							yStart
								+ currentRightMidY * (spacing + half * 2)
								+ half / 2;
						currentRightMidY++;
					}
				}
				if (newPos == null || (newPos.x == 0 && newPos.y == 0)) {
					newPos = Helper.calculateRandomPortPoint(compSize, port);
				}
				portPositions.put(port, newPos);
			}

		}
		
		// Place unplaced ports so that they don't overlap with other ports
		Iterator iter = nonalignedPorts.iterator ();
		while (iter.hasNext ()) {
			AcmePort port = (AcmePort )iter.next ();
			Point pos = (Point )portPositions.get (port);
			if (pos == null || pos.x == 0 && pos.y ==0) {
				pos = Helper.calculateRandomPortPoint(compSize, port);
				SystemView.updatePortPositionToBorder(pos, compSize);
				SystemView.setLocation(port, pos);
				portPositions.put (port, pos);
			}
			int times = 0;
			while (overlapsWithOthers (port, pos, portPositions) && times < 50) {
				pos = adjustPortPosition (SystemView.getLocation(component, false), compSize, pos,  port);
				SystemView.setLocation (port, pos);
				WrapData data = (WrapData )port.getUserData("WrapData");
				if (data != null && data._times > 1) {
					pos = Helper.calculateRandomPortPoint(compSize, port);
					SystemView.updatePortPositionToBorder(pos, compSize);
					SystemView.setLocation (port, pos);
					port.setUserData ("WrapData", null);
					portPositions.put (port, pos);
					break;
				}
				else portPositions.put (port, pos);
				times++;
			}
		}
			
		
		return portPositions;
	}

	    static  class WrapData implements IElementExtension {
	    	int _times = 0;
	    	
			/**
			 * @return
			 * @see acmeLib.core.extensions.IElementExtension#isPersistent()
			 **/
			public boolean isPersistent() {
				return false;
			}

			/**
			 * @param element
			 * @see acmeLib.core.extensions.IElementExtension#injectIntoElement(acmeLib.elements.AcmeElement)
			 **/
			public void injectIntoElement(AcmeElement element) {
			}

			/**
			 * @param element
			 * @see acmeLib.core.extensions.IElementExtension#extractFromElement(acmeLib.elements.AcmeElement)
			 **/
			public void extractFromElement(AcmeElement element) {
			}

			/**
			 * @param element
			 * @return
			 * @see acmeLib.core.extensions.IElementExtension#isInjected(acmeLib.elements.AcmeElement)
			 **/
			public boolean isInjected(AcmeElement element) {
				return false;
			}
	    	
	    };
	/**
	 * @param location
	 * @param pos
	 * @return
	 */
	private static Point adjustPortPosition(Point compLocation, Dimension compSize, Point portLocation, AcmePort port) {
		if (portLocation == null || compLocation == null) return portLocation;
		int side = Geometry.getSideOfPoint(compLocation, new Point (portLocation.x + compLocation.x, portLocation.y + compLocation.y));
		int yIncr = 0; 
		int xIncr = 0;
		if (side == PositionConstants.NORTH || side == PositionConstants.SOUTH) {
			xIncr = 10;
		}
		else if (side == PositionConstants.EAST || side == PositionConstants.WEST) {
			yIncr = 10;
		}
		portLocation.x += xIncr;
		portLocation.y += yIncr;
		
		if (portLocation.x > compSize.width / 2) {
			portLocation.x = - compSize.width /2;
			WrapData data = (WrapData )port.getUserData("WrapData");
			if (data == null) {
				data = new WrapData ();
				port.setUserData ("WrapData", data);
			}
			data._times++;
		}
		
		if (portLocation.y > compSize.height/2) {
			portLocation.y = -compSize.height/2;
			WrapData data = (WrapData )port.getUserData("WrapData");
			if (data == null) {
				data = new WrapData ();
				port.setUserData ("WrapData", data);
			}
			data._times++;
		}
		return portLocation;
	}

	/**
	 * @param element
	 * @return
	 */
	private static Dimension getRealSize(AcmeElement element) {
		Dimension size = Helper.getSize(element);
		if (size == null) {
			size = VisualStyleRegistry.getVisualization(element).getSize();
		}
		return size;
	}

	/**
	 * @param compSize
	 * @param port
	 * @return
	 */
	private static Point getAdjustedPortPosition(Dimension compSize, AcmePort port) {
		Point newPos = new Point();
		Point posX = Helper.getX(port);
		Point posY = Helper.getY(port);
		if (posX != null || posY != null) {
			if (posY != null && posX != null) {
				newPos.x = posX.x;
				newPos.y = posY.y;
				updatePortPositionToBorder(newPos, compSize);
			} else {
				if (posX != null) {
					newPos.x = posX.x;
				}
				if (posY != null) {
					newPos.y = posY.y;
				}
			}
		}
		return newPos;
	}

	/**
	 * @param placementRules
	 * @param port
	 * @return
	 */
	private static PlacementRule getPlacementRuleForPort(LinkedList placementRules, AcmePort port) {
		AcmeEnumeration ae = port.getDeclaredTypes();
		PlacementRule rule = null;
		boolean foundRule = false;
		while (!foundRule && ae.moreItems()) {
			String portType =
				((AcmeElementType) ae.getNextItem()).getTypeName();
			Iterator iter = placementRules.iterator();
			while (iter.hasNext()) {
				PlacementRule obj = (PlacementRule) iter.next();
				if (obj._type.equals("port")
					|| obj._type.equals(portType)) {
					foundRule = true;
					rule = obj;
					break;
				}
			}
		}
		return rule;
	}

	/**
	 * @param pos
	 * @param portPositions
	 * @return
	 */
	private static boolean overlapsWithOthers(AcmePort port, Point pos, Map portPositions) {
		boolean overlaps = false;
		Iterator i = portPositions.entrySet ().iterator ();
		Dimension size = Helper.getSize(port);
		Rectangle portRect = new Rectangle (pos.x - size.width/2, pos.y - size.height/2, size.width, size.height);
		while (i.hasNext () && !overlaps) {
			Map.Entry entry = (Map.Entry )i.next ();
			if (entry.getKey () == port) continue;
			AcmePort p  = (AcmePort )entry.getKey ();
			Dimension pSize = Helper.getSize (p);
			Point pPos = (Point )entry.getValue();
			Rectangle pRect = new Rectangle (pPos.x - pSize.width/2, pPos.y - pSize.height/2, pSize.width, pSize.height);
			overlaps = pRect.intersects(portRect);
		}
		return overlaps;
	}

	public static boolean normalizePortPosition(
		AcmePort port,
		AcmeComponent component,
		Point newPos) {
		boolean shouldSetLocation = false;
		Dimension compSize = Helper.getSize(component);

		if (getPortLocation_static(port) != null) {
			shouldSetLocation = true;
            updatePortPositionToBorder(newPos, compSize);
		}
		return shouldSetLocation;
	}

	public void relocatePortAfterResize(
		AcmePort port,
		AcmeComponent component,
		Dimension oldSize,
		Point portLocation) {
		if (portLocation == null) {
			return;
		}
		Dimension componentSize = Helper.getSize(component);
		double xRatio = (double) componentSize.width / (double) oldSize.width;
		double yRatio = (double) componentSize.height / (double) oldSize.height;
		portLocation.x = (int) (portLocation.x * xRatio);
		portLocation.y = (int) (portLocation.y * yRatio);
		normalizePortPosition(port, component, portLocation);
		//        Helper.setLocation (port, portLocation);
	}
	/**
	 * @return SystemEditPart
	 */
	public SystemEditPart getPart() {
		return _part;
	}

	/**
	 * Sets the _part.
	 * @param _part The _part to set
	 */
	public void setPart(SystemEditPart _part) {
		this._part = _part;
	}

	public static Rectangle getSystemBounds(AcmeSystem system) {
		AcmeEnumeration components = system.getComponents();
		AcmeEnumeration connectors = system.getConnectors();
		if (components.getSize() == 0 && connectors.getSize() == 0) {
			return new Rectangle(0, 0, 0, 0);
		}

		Set elements = new HashSet();
		elements.addAll(Helper.convertAcmeEnumerationToList(components));
		elements.addAll(Helper.convertAcmeEnumerationToList(connectors));
        
        Iterator i = SystemEditPart.factories.iterator();
        while (i.hasNext()) {
            IElementFactory factory = (IElementFactory) i.next();
            elements.addAll(factory.getModels(system));
        }
        
        
		if (elements.size() == 0) {
            return new Rectangle(0, 0, 0, 0);   
        }
        
		return getBounds(elements);
	}

	public static Rectangle getBounds(Set elements) {
		if (elements == null || elements.isEmpty())
			return new Rectangle(0, 0, 0, 0);
		Iterator i = elements.iterator();
		Point topLeft = new Point(Integer.MAX_VALUE, Integer.MAX_VALUE);
		Point bottomRight = new Point(Integer.MIN_VALUE, Integer.MIN_VALUE);

		while (i.hasNext()) {
			
			Object obj = i.next();
			if (!(obj instanceof AcmeElement)) continue;
			AcmeElement element = (AcmeElement) obj;
			Point location = SystemView.getLocation(element, false);
			if (location == null) {
				MultiPageAcmeEditor activeEditor = ASContext.current().getActiveEditor();
				if (activeEditor == null) continue;
				
				DiagramEditor diagramEditor = activeEditor.getDiagramEditor();
				if (diagramEditor == null) continue;
				GraphicalEditPart ed = (GraphicalEditPart )diagramEditor.getEditPart(element);
				if (ed != null) {
					IFigure f = ed.getFigure();
					location = f.getBounds().getCenter ();
				}
				else continue;
			}
				
			Dimension size = Helper.getSize(element);
			Point cTL =
				new Point(
					location.x - size.width / 2,
					location.y - size.height / 2);
			Point cBR =
				new Point(
					location.x + size.width / 2,
					location.y + size.height / 2);
			if (topLeft.x > cTL.x)
				topLeft.x = cTL.x;
			if (topLeft.y > cTL.y)
				topLeft.y = cTL.y;
			if (bottomRight.x < cBR.x)
				bottomRight.x = cBR.x;
			if (bottomRight.y < cBR.y)
				bottomRight.y = cBR.y;
			if (element instanceof AcmeConnector) {
				AcmeConnector conn = (AcmeConnector) element;

				/*
				 * Need to process roles, because they are outside connectors. This
				 * fixes Bug #47
				 */
				AcmeEnumeration roles = conn.getRoles();
				while (roles.moreItems()) {
					AcmeRole role = (AcmeRole) roles.getNextItem();
					Point rLocation = SystemView.getLocation(role, false);
					if (rLocation == null)
						continue;
					Dimension rSize = Helper.getSize(role);
					Point rCTL =
						new Point(
							rLocation.x - rSize.width / 2,
							rLocation.y - rSize.height / 2);
					Point rCBR =
						new Point(
							rLocation.x + rSize.width / 2,
							rLocation.y + rSize.height / 2);
					if (topLeft.x > rCTL.x)
						topLeft.x = rCTL.x;
					if (topLeft.y > rCTL.y)
						topLeft.y = rCTL.y;
					if (bottomRight.x < rCBR.x)
						bottomRight.x = rCBR.x;
					if (bottomRight.y < rCBR.y)
						bottomRight.y = rCBR.y;
				}
			}
		}
		if (topLeft.x != Integer.MAX_VALUE
			&& topLeft.y != Integer.MAX_VALUE
			&& bottomRight.x != Integer.MIN_VALUE
			&& bottomRight.y != Integer.MIN_VALUE) {
			return new Rectangle(
				topLeft.x,
				topLeft.y,
				bottomRight.x - topLeft.x,
				bottomRight.y - topLeft.y);
		} else {
			return new Rectangle(0, 0, 0, 0);
		}
	}

	public Rectangle getSystemBounds() {
		if (_model == null)
			return new Rectangle(0, 0, 0, 0);
		AcmeSystem system = _model;
		AcmeEnumeration components = system.getComponents();
		AcmeEnumeration connectors = system.getConnectors();
		if (components.getSize() == 0 && connectors.getSize() == 0) {
			return new Rectangle(0, 0, 0, 0);
		}
		Point topLeft = new Point(Integer.MAX_VALUE, Integer.MAX_VALUE);
		Point bottomRight = new Point(Integer.MIN_VALUE, Integer.MIN_VALUE);
		if (_representationBorder != null && _representationBorder.isVisible ()) {
			topLeft = _representationBorder.getBounds().getTopLeft();
			bottomRight = _representationBorder.getBounds().getBottomRight();
		}
		while (components.moreItems()) {
			AcmeComponent comp = (AcmeComponent) components.getNextItem();
			Point location = getElementLocation(comp);
			Dimension size = Helper.getSize(comp);
			Point cTL =
				new Point(
					location.x - size.width / 2,
					location.y - size.height / 2);
			Point cBR =
				new Point(
					location.x + size.width / 2,
					location.y + size.height / 2);
			if (topLeft.x > cTL.x)
				topLeft.x = cTL.x;
			if (topLeft.y > cTL.y)
				topLeft.y = cTL.y;
			if (bottomRight.x < cBR.x)
				bottomRight.x = cBR.x;
			if (bottomRight.y < cBR.y)
				bottomRight.y = cBR.y;
		}
		while (connectors.moreItems()) {
			AcmeConnector conn = (AcmeConnector) connectors.getNextItem();
			Point location = getElementLocation(conn);
			Dimension size = Helper.getSize(conn);
			Point cTL =
				new Point(
					location.x - size.width / 2,
					location.y - size.height / 2);
			Point cBR =
				new Point(
					location.x + size.width / 2,
					location.y + size.height / 2);
			if (topLeft.x > cTL.x)
				topLeft.x = cTL.x;
			if (topLeft.y > cTL.y)
				topLeft.y = cTL.y;
			if (bottomRight.x < cBR.x)
				bottomRight.x = cBR.x;
			if (bottomRight.y < cBR.y)
				bottomRight.y = cBR.y;
			/*
			 * Need to process roles, because they are outside connectors. This
			 * fixes Bug #47
			 */
			AcmeEnumeration roles = conn.getRoles();
			while (roles.moreItems()) {
				AcmeRole role = (AcmeRole) roles.getNextItem();
				Point rLocation = getElementLocation(role);
				Dimension rSize = Helper.getSize(role);
				Point rCTL =
					new Point(
						rLocation.x - rSize.width / 2,
						rLocation.y - rSize.height / 2);
				Point rCBR =
					new Point(
						rLocation.x + rSize.width / 2,
						rLocation.y + rSize.height / 2);
				if (topLeft.x > rCTL.x)
					topLeft.x = rCTL.x;
				if (topLeft.y > rCTL.y)
					topLeft.y = rCTL.y;
				if (bottomRight.x < rCBR.x)
					bottomRight.x = rCBR.x;
				if (bottomRight.y < rCBR.y)
					bottomRight.y = rCBR.y;
			}
		}
		if (topLeft.x != Integer.MAX_VALUE
			&& topLeft.y != Integer.MAX_VALUE
			&& bottomRight.x != Integer.MIN_VALUE
			&& bottomRight.y != Integer.MIN_VALUE) {
			return new Rectangle(
				topLeft.x,
				topLeft.y,
				bottomRight.x - topLeft.x,
				bottomRight.y - topLeft.y);
		} else {
			return new Rectangle(0, 0, 0, 0);
		}
	}

	public synchronized PointList layoutBendpoints(
		AcmeRole acmeRole,
		PointList bendpoints,
		Point roleStart,
		Point roleEnd) {
		ConnectorVisualization.RoleRouteRule rule =
			Helper.getRoleRouteRule(acmeRole);
		if (rule == null
			|| rule._roleRoute == RoleRouteOptions.STRAIGHT
			|| bendpoints == null
			|| bendpoints.size() == 0
			|| roleStart == null
			|| roleEnd == null) {
			return bendpoints;
		}
		int size = bendpoints.size();
		if (size <= 1) {
			Helper.removeBendpoint(acmeRole, 0, roleStart, roleEnd);
			bendpoints = getBendpoints(acmeRole, roleStart, roleEnd, null);
		} else {
			bendpoints = calculateRoute(bendpoints, roleStart, roleEnd);
		}
		if (bendpoints != null
			&& bendpoints.size() > 0
			&& rule._roleRoute == RoleRouteOptions.ELBOW) {
			AcmePort port = ModelHelper.findAttachedPort(acmeRole);
			if (port != null) {
				AcmeElement comp = port.getParent();
				if (comp instanceof AcmeComponent) {
					Point portLocation = getPortLocation(port);
					Dimension portSize = Helper.getSize(port);
					Dimension compSize = Helper.getSize(port.getParent());
					bendpoints =
						injectBendpoint(
							bendpoints,
							roleStart,
							roleEnd,
							portLocation,
							portSize,
							compSize);
				}
			}
		}
		Helper.setBendpoints(acmeRole, bendpoints);
		return bendpoints;
	}

	private static PointList injectBendpoint(
		PointList bendpoints,
		Point roleStart,
		Point roleEnd,
		Point portLocation,
		Dimension portSize,
		Dimension compSize) {
		final int scale = 5;
		Point newBendpoint = null;
		int index = 0;
		Point lastBendpoint = bendpoints.getLastPoint();
		if (portLocation != null && compSize != null) {
			if (portLocation.x >= compSize.width / 2) {
				if (roleEnd.x == lastBendpoint.x) {
					int newX = roleEnd.x + portSize.width * scale;
					newBendpoint = new Point(newX, roleEnd.y);
					index = bendpoints.size();
				} else if (roleEnd.x > lastBendpoint.x) {
					int newX = roleEnd.x + portSize.width * scale;
					int newY =
						roleStart.y > roleEnd.y
							? roleEnd.y + compSize.height + portSize.height
							: roleEnd.y - compSize.height - portSize.height;
					newBendpoint = new Point(newX, newY);
					index = bendpoints.size() - 1;
				}
			} else if (portLocation.x <= -compSize.width / 2) {
				if (roleEnd.x == lastBendpoint.x) {
					int newX = roleEnd.x - portSize.width * scale;
					newBendpoint = new Point(newX, roleEnd.y);
					index = bendpoints.size();
				} else if (roleEnd.x < lastBendpoint.x) {
					int newX = roleEnd.x - portSize.width * scale;
					int newY =
						roleStart.y > roleEnd.y
							? roleEnd.y + compSize.height + portSize.height
							: roleEnd.y - compSize.height - portSize.height;
					newBendpoint = new Point(newX, newY);
					index = bendpoints.size() - 1;
				}
			} else if (portLocation.y >= compSize.height / 2) {
				if (roleEnd.y == lastBendpoint.y) {
					int newY = roleEnd.y + portSize.height * scale;
					newBendpoint = new Point(roleEnd.x, newY);
					index = bendpoints.size();
				} else if (roleEnd.y > lastBendpoint.y) {
					int newY = roleEnd.y + portSize.height * scale;
					int newX =
						roleStart.x > roleEnd.x
							? roleEnd.x + compSize.width + portSize.width
							: roleEnd.x - compSize.width - portSize.width;
					newBendpoint = new Point(newX, newY);
					index = bendpoints.size() - 1;
				}
			} else if (portLocation.y <= -compSize.height / 2) {
				if (roleEnd.y == lastBendpoint.y) {
					int newY = roleEnd.y - portSize.height * scale;
					newBendpoint = new Point(roleEnd.x, newY);
					index = bendpoints.size();
				} else if (roleEnd.y < lastBendpoint.y) {
					int newY = roleEnd.y - portSize.height * scale;
					int newX =
						roleStart.x > roleEnd.x
							? roleEnd.x + compSize.width + portSize.width
							: roleEnd.x - compSize.width - portSize.width;
					newBendpoint = new Point(newX, newY);
					index = bendpoints.size() - 1;
				}
			}
			if (newBendpoint != null) {
				bendpoints.insertPoint(newBendpoint, index);
				bendpoints = calculateRoute(bendpoints, roleStart, roleEnd);
			}
		}
		return bendpoints;
	}

	private static PointList calculateRoute(
		PointList bendpoints,
		Point roleStart,
		Point roleEnd) {
		Point currentPoint = null,
			previousPoint = null,
			nextPoint = null,
			refPoint = null;
		boolean refAhead = false;
		int size = bendpoints.size(), i = 0;
		for (i = size - 1; i >= 0; i--) {
			previousPoint =
				i == size - 1 ? roleEnd : bendpoints.getPoint(i + 1);
			currentPoint = bendpoints.getPoint(i);
			nextPoint = i == 0 ? roleStart : bendpoints.getPoint(i - 1);
			if (currentPoint.x != nextPoint.x
				&& currentPoint.y != nextPoint.y
				&& currentPoint.x != previousPoint.x
				&& currentPoint.y != previousPoint.y) {
				if (refPoint == null) {
					refAhead = false;
					refPoint = i == 1 ? roleStart : bendpoints.getPoint(i - 2);
				}
				break;
			} else {
				refAhead = true;
				refPoint = previousPoint;
			}
		}
		if (i == -1) {
			refPoint = null;
			for (i = size - 1; i >= 0; i--) {
				nextPoint = i == 0 ? roleStart : bendpoints.getPoint(i - 1);
				currentPoint = bendpoints.getPoint(i);
				previousPoint =
					i == size - 1 ? roleEnd : bendpoints.getPoint(i + 1);
				if (currentPoint.x != nextPoint.x
					&& currentPoint.y != nextPoint.y
					|| currentPoint.x != previousPoint.x
					&& currentPoint.y != previousPoint.y) {
					if (refPoint == null) {
						refAhead = false;
						refPoint =
							i == 1 ? roleStart : bendpoints.getPoint(i - 2);
					}
					break;
				} else {
					refAhead = true;
					refPoint = previousPoint;
				}
			}
		}
		if (i == -1) {
			return bendpoints;
		}
		if (refAhead) {
			if (previousPoint.x == refPoint.x) {
				currentPoint.x = nextPoint.x;
				previousPoint.y = currentPoint.y;
			} else if (previousPoint.y == refPoint.y) {
				currentPoint.y = nextPoint.y;
				previousPoint.x = currentPoint.x;
			}
			if (i < size - 1) {
				bendpoints.setPoint(currentPoint, i);
				bendpoints.setPoint(previousPoint, i + 1);
			}
		} else {
			if (nextPoint.x == refPoint.x) {
				currentPoint.x = previousPoint.x;
				nextPoint.y = currentPoint.y;
			} else if (nextPoint.y == refPoint.y) {
				currentPoint.y = previousPoint.y;
				nextPoint.x = currentPoint.x;
			}
			if (i > 0) {
				bendpoints.setPoint(currentPoint, i);
				bendpoints.setPoint(nextPoint, i - 1);
			}
		}
		return bendpoints;
	}

	public synchronized PointList arrangeBendpoints(
		AcmeRole acmeRole,
		PointList bendpoints,
		Point roleStart,
		Point roleEnd) {
		ConnectorVisualization.RoleRouteRule rule =
			Helper.getRoleRouteRule(acmeRole);
		if (rule == null
			|| rule._roleRoute == ConnectorVisualization.RoleRouteOptions.STRAIGHT
			|| bendpoints == null
			|| bendpoints.size() == 0
			|| roleStart == null
			|| roleEnd == null) {
			return bendpoints;
		}
		int size = bendpoints.size();
		if (size <= 1) {
			Helper.removeBendpoint(acmeRole, 0, roleStart, roleEnd);
			bendpoints = getBendpoints(acmeRole, roleStart, roleEnd, null);
		} else {
			bendpoints =
				SystemView.calculateRoute(bendpoints, roleStart, roleEnd);
		}
		return bendpoints;
	}

	public static void cleanBendpoints(
		PointList bendpoints,
		Point roleStart,
		Point roleEnd) {
		if (bendpoints == null
			|| bendpoints.size() == 0
			|| roleStart == null
			|| roleEnd == null) {
			return;
		}
		Point currentPoint = null, previousPoint = null, nextPoint = null;
		int size = bendpoints.size(), i = 0;
		for (i = 0; i < size; i++) {
			previousPoint = i == 0 ? roleStart : bendpoints.getPoint(i - 1);
			currentPoint = bendpoints.getPoint(i);
			nextPoint = i == size - 1 ? roleEnd : bendpoints.getPoint(i + 1);
			if (previousPoint.x == currentPoint.x
				&& currentPoint.x == nextPoint.x
				|| previousPoint.y == currentPoint.y
				&& currentPoint.y == nextPoint.y) {
				bendpoints.removePoint(i--);
				size--;
			}
		}
	}

	/**
	 * @param role
	 * @param rule
	 * @param roleStart
	 * @param roleEnd
	 */
	public PointList getBendpoints(
		AcmeRole role,
		Point roleStart,
		Point roleEnd,
		Dimension diff) {
		if (role == null)
			return null;
		PointList manuallyCreatedBendPoints = Helper.getBendpoints(role);
		if (manuallyCreatedBendPoints != null) {
			if (diff != null && (diff.width != 0 || diff.height != 0)) {
				for (int i = 0; i < manuallyCreatedBendPoints.size(); i++) {
					Point p = manuallyCreatedBendPoints.getPoint(i);
					p.translate(diff);
					manuallyCreatedBendPoints.setPoint(p, i);
				}
				Helper.setBendpoints(role, manuallyCreatedBendPoints);
			}
			return manuallyCreatedBendPoints;
		}
		ConnectorVisualization.RoleRouteRule rule =
			Helper.getRoleRouteRule(role);
		if (rule == null)
			return null;
		PointList result = null;
		Point bendpoint = new Point();
		if (rule._roleRoute == RoleRouteOptions.VBUS) {
			bendpoint.setLocation(roleStart.x, roleEnd.y);
		} else if (rule._roleRoute == RoleRouteOptions.HBUS) {
			bendpoint.setLocation(roleEnd.x, roleStart.y);
		} else if (rule._roleRoute == RoleRouteOptions.ELBOW) {
			AcmePort port = ModelHelper.findAttachedPort(role);
			boolean bpSet = false;
			if (port != null) {
				AcmeElement comp = port.getParent();
				if (comp instanceof AcmeComponent) {
					Point portLocation = getPortLocation(port);
					Dimension compSize = Helper.getSize(port.getParent());
					Point location = SystemView.getLocation (port.getParent (), false);
				
					if (portLocation != null && compSize != null) {
						if (Math.abs(portLocation.x) >= compSize.width / 2) {
							bendpoint.setLocation(roleStart.x, roleEnd.y);
							bpSet = true;
						} else {
							bendpoint.setLocation(roleEnd.x, roleStart.y);
							bpSet = true;
						}
					}
				}
			}
			if (!bpSet) {
				bendpoint.setLocation(roleEnd.x, roleStart.y);
			}
		}
		result = new PointList();
		result.addPoint(bendpoint);
		return result;
	}

	
	/**
	 * @return boolean
	 */
	public boolean shouldComputeChildLocations() {
		return _shouldComputeChildLocations;
	}

	/**
	 * Sets the _shouldComputeChildLocations.
	 * @param _shouldComputeChildLocations The _shouldComputeChildLocations to set
	 */
	public void setShouldComputeChildLocations(boolean _shouldComputeChildLocations) {
		this._shouldComputeChildLocations = _shouldComputeChildLocations;
	}

	/**
	 * @return Hashtable
	 */
	public Hashtable getChildLocations() {
		return _childLocations;
	}

	/**
	 * Sets the _childLocations.
	 * @param _childLocations The _childLocations to set
	 */
	public void setChildLocations(Hashtable _childLocations) {
		this._childLocations = _childLocations;
	}

	Map _modelToFigure = new HashMap ();
	
	
	/* (non-Javadoc)
	 * TODO: Make sure that children are added in the right order
	 */
	public void add(IFigure child, Object constraint, int index) {
		// computer actual index
		int realIndex = index;
		if (child instanceof RepresentationBorderView) {
			realIndex = 0;
		}
		else if (child instanceof BindingView) {
			realIndex = 1;
		}
		else if (child instanceof PortView) {
			
			AcmeElement	model	= ((PortView )child).getModel ();
			ComponentView parent = (ComponentView )_modelToFigure.get (model.getParent ());
			if (parent != null) {
				int parentIndex = getChildren ().indexOf (parent);
				if (realIndex != -1 && realIndex <= parentIndex) {
					realIndex = parentIndex+1;
				}
			}
			_modelToFigure.put (model, child);
		}
		else if (child instanceof ComponentView) {
			AcmeElement	model	= ((ComponentView )child).getModel ();
			_modelToFigure.put (model, child);
		}
		else if (child instanceof ConnectorView) {
			AcmeElement model = ((ConnectorView )child).getModel ();
			_modelToFigure.put (model, child);
		}
		else if (child instanceof RoleView) {
			AcmeElement model = ((RoleView )child).getModel ();
			ConnectorView parent = (ConnectorView )_modelToFigure.get (model.getParent ());
			if (parent != null) {
				int parentIndex = getChildren ().indexOf (parent);
				if (parentIndex == 0) {
					realIndex = 0;
				}
				else if (realIndex != -1 && realIndex >= parentIndex) {
					realIndex = parentIndex - 1;
				}
			}
		}
		super.add (child, constraint, realIndex);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#remove(org.eclipse.draw2d.IFigure)
	 */
	public void remove(IFigure child) {
		if (child instanceof AcmeVisElement) {
			_modelToFigure.remove(((AcmeVisElement )child).getModel());
		}
		if (child.getParent () == this) super.remove (child);
	}

	/**
	 * Makes sure that roles are behind ports in attachments
	 * @param portView The port that is involved in an attachment
	 * @param roleView The role that is involved in the attachment
	 */
	public void moveBehindAttachments(PortView portView, RoleView roleView) {
		int portIndex = getChildren ().indexOf (portView);
		int roleIndex = getChildren ().indexOf (roleView);
		
		if (roleIndex > portIndex) {
			getChildren ().remove (roleView);
			getChildren ().add (portIndex - 1, roleView);
		}
	}


}