/*
 * Created on Jan 9, 2005
 *
 */
package AcmeStudio.ui.editors.diagram.palette;

import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.tools.CreationTool;
import AcmeStudio.ui.editors.diagram.figures.VisualizationFeedbackFigure;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import acmeLib.elements.AcmeElement;


/**
 * @author Bradley Schmerl
 * Created Jan 9, 2005
 */
public class AcmeElementCreationTool extends CreationTool {
	private VisualizationFeedbackFigure feedbackFigure;
	
	
	/**
	 * 
	 */
	public AcmeElementCreationTool(CreationFactory factory) {
		super (factory);
	}
	
	
	/**
	 * @return
	 * @see org.eclipse.gef.tools.CreationTool#handleDragInProgress()
	 **/
	protected boolean handleDragInProgress() {
		boolean ret = super.handleDragInProgress ();
		if (isInState(STATE_DRAG_IN_PROGRESS)) {
			showCreationFeedback ();
		}
		return ret;
	}
	
	private void showCreationFeedback () {
		CreateRequest req = (CreateRequest )getTargetRequest ();
		Rectangle rect = new Rectangle (req.getLocation (), req.getSize ());
		getFeedbackFigure ().translateToRelative (rect);
		getFeedbackFigure ().setBounds (rect);
	}
	
	private IFigure getFeedbackFigure () {
		if (feedbackFigure == null) {
			AcmeElement element = (AcmeElement )((CreateRequest )getTargetRequest()).getNewObject ();
			ElementVisualization vis = VisualStyleRegistry.getVisualization(element);
			feedbackFigure = new VisualizationFeedbackFigure (vis);
			FigureUtilities.makeGhostShape (feedbackFigure);
			addFeedback (feedbackFigure);
		}
		return feedbackFigure;
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.tools.CreationTool#deactivate()
	 **/
	public void deactivate() {
		if (isInState (STATE_DRAG_IN_PROGRESS)) {
			eraseCreationFeedback ();
			eraseTargetFeedback ();
		}
	}
	
	private void eraseCreationFeedback () {
		if (feedbackFigure != null) {
			removeFeedback (feedbackFigure);
			feedbackFigure = null;
		}
	}
	
	
	/**
	 * @param button
	 * @return
	 * @see org.eclipse.gef.tools.CreationTool#handleButtonUp(int)
	 **/
	protected boolean handleButtonUp(int button) {
		if (stateTransition(STATE_DRAG | STATE_DRAG_IN_PROGRESS, STATE_TERMINAL)) {
			eraseTargetFeedback ();
			eraseCreationFeedback();
			performCreation (button);
		}
		handleFinished ();
		return true;
	}
}
