package AcmeStudio.ui.editors.diagram.palette;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import org.eclipse.gef.Tool;
import org.eclipse.gef.palette.CombinedTemplateCreationEntry;
import org.eclipse.gef.palette.PaletteContainer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.jface.resource.ImageDescriptor;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.ui.editors.diagram.templates.SystemTemplateFactory;
import AcmeStudio.util.IManagedImageListener;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.ManagedImage;
import acmeLib.elements.AcmeElementType;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AcmeTypeTemplateEntry extends CombinedTemplateCreationEntry implements PropertyChangeListener, IManagedImageListener {

	public static final String PROP_TEMPL_CHANGE = "$TemplChange$";

	ElementVisualization _vis;
	AcmeElementType _type;
	PaletteImageDescriptor _image;

	private ElementVisualization	_visCopy;

	public AcmeTypeTemplateEntry(AcmeElementType template, String label) {
		super(label, "", template, new SystemTemplateFactory(template), null, null);

		_type = template;
		ImageDescriptor image = setVisualization(template, false);
		setSmallIcon(image);
		setLargeIcon(image);
		_vis.addPropertyChangeListener(_type.getTypeName(), this);

	}

	public AcmeTypeTemplateEntry(Class template, String label) {
		super(label, "", template, new SystemTemplateFactory(template), null, null);
	}

	protected ImageDescriptor setVisualization(AcmeElementType type, boolean force) {
		_vis = VisualStyleRegistry.getVisualization(type);

		String key = type.getTypeName() + "_preview_palette_" + type.getClass().getName();

		if (_image == null || _image.isDisposed()) {

			_image = PaletteImageDescriptor.createDescriptor(key, ImageManager.createImage(key, 20, 12, false));

		}
		if (_visCopy != null) {
			_visCopy.getSmallImage(type).dispose(this);
			_visCopy.dispose ();
		}
		_visCopy = _vis.copy();
		ManagedImage mim = _visCopy.getSmallImage(type);
		mim.reference (this);
		_image.setImage(mim);
		return _image;
	}

	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(ElementVisualization.PROP_VIS_CHANGED)) {
			_vis.removePropertyChangeListener (_type.getTypeName(), this);
			ImageDescriptor img = setVisualization((AcmeElementType) getTemplate(), true);
			//            PaletteImageDescriptor oldImg = (PaletteImageDescriptor )getSmallIcon ();
			setSmallIcon(img);
			setLargeIcon(img);
			//            oldImg.dispose ();
			_vis.addPropertyChangeListener(_type.getTypeName (), this);
			fireImageChanged();
		}
		else if (evt.getPropertyName().equals(ElementVisualization.PROP_VIS_RESET)) {
			_vis.removePropertyChangeListener(_type.getTypeName(), this);
//			this._image.dispose();
			setSmallIcon(null);
			setLargeIcon(null);
			            ImageDescriptor img = setVisualization ((AcmeElementType )getTemplate(), true);
			            setSmallIcon(img);
			            setLargeIcon(img);
//			            _vis.addPropertyChangeListener(_type.getTypeName (),this);
			_vis.addPropertyChangeListener(_type.getTypeName (), this);
			fireImageChanged();
			
		}
		else if (evt.getPropertyName().equals(PaletteContainer.PROPERTY_CHILDREN)) {
//			Debug.diagnoseln("AcmeTypeTemplateEntry.propertyChange: " + "removing children");
		}
		else if (evt.getPropertyName().equals(PROPERTY_SMALL_ICON)) {
			if (evt.getSource() == this) {
//				Debug.diagnoseln(
//					"AcmeTypeTemplateEntry.propertyChange: " + this._type.getTypeName() + "'s small image is changing");
			}
		}
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		super.addPropertyChangeListener (listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener l) {
		super.removePropertyChangeListener(l);
	}

	protected void fireImageChanged() {
		if (listeners != null) {
			listeners.firePropertyChange(PaletteEntry.PROPERTY_SMALL_ICON, null, null);
			listeners.firePropertyChange(AcmeTypeTemplateEntry.PROP_TEMPL_CHANGE, null, null);
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object obj) {
		if (this.getClass().isInstance(obj)) {
			AcmeTypeTemplateEntry o = (AcmeTypeTemplateEntry) obj;
			return _type.getTypeName().compareTo(o._type.getTypeName());

		}
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.palette.PaletteEntry#setLargeIcon(org.eclipse.jface.resource.ImageDescriptor)
	 */
	public void setLargeIcon(ImageDescriptor icon) {
		if (icon != null && icon != this._image) {
			_image.dispose();
		}
		super.setLargeIcon(icon);

	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.palette.PaletteEntry#setSmallIcon(org.eclipse.jface.resource.ImageDescriptor)
	 */
	public void setSmallIcon(ImageDescriptor icon) {
		if (icon != null && icon != this._image) {
			_image.dispose();
		}
		super.setSmallIcon(icon);
	}

	public void dispose() {
		
		if (listeners != null) {
			PropertyChangeListener[] l = listeners.getPropertyChangeListeners();
			for (int i = 0; i < l.length; i++) {
				listeners.removePropertyChangeListener(l[i]);
			}
//			listeners = null;
		}
		_visCopy.dispose ();
	}

	
	/**
	 * @return
	 * @see org.eclipse.gef.palette.CreationToolEntry#createTool()
	 **/
	public Tool createTool() {
		return new AcmeElementCreationTool (factory);
	}

	/**
	 * @param image
	 * @see AcmeStudio.util.IManagedImageListener#imageDisposed(AcmeStudio.util.ManagedImage)
	 **/
	public void imageDisposed(ManagedImage image) {
		fireImageChanged ();
	}
}
