package AcmeStudio.ui.editors.diagram.palette;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import org.eclipse.gef.Tool;
import org.eclipse.gef.palette.CombinedTemplateCreationEntry;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.ui.editors.diagram.styles.constants.LabelOptions;
import AcmeStudio.ui.editors.diagram.templates.SystemTemplateFactory;
import AcmeStudio.util.IManagedImageListener;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.ManagedImage;
import AcmeStudio.util.Utils;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.elements.*;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class GlobalTemplatePaletteEntry
    extends CombinedTemplateCreationEntry
    implements PropertyChangeListener, IManagedImageListener {

    ElementVisualization _vis;
    ElementVisualization _visWithoutLabel;
    int _defaultType;
    PaletteImageDescriptor _image;

    PropertyChangeSupport _listeners = new PropertyChangeSupport(this);
	private ElementVisualization	_visCopy;

    public GlobalTemplatePaletteEntry(Class clazz, String label) {
        super(label, "", clazz, new SystemTemplateFactory (clazz), null, null);
        if (clazz.equals(AcmeComponent.class)) {
            _defaultType = ElementVisualization.DEFAULT_COMPONENT;
        } else if (clazz.equals(AcmeConnector.class)) {
            _defaultType = ElementVisualization.DEFAULT_CONNECTOR;
        } else if (clazz.equals(AcmeRole.class)) {
            _defaultType = ElementVisualization.DEFAULT_ROLE;
        } else if (clazz.equals(AcmePort.class)) {
            _defaultType = ElementVisualization.DEFAULT_PORT;
        }
        _image = setVisualization(_defaultType);
        setSmallIcon(_image);
        setLargeIcon(_image);
        _vis.addPropertyChangeListener("", this);
    }

    protected PaletteImageDescriptor setVisualization(int type) {
    	if (_vis != null) _vis.removePropertyChangeListener("", this);
    	AcmeElementType eltType = null;
        if (type == ElementVisualization.DEFAULT_COMPONENT) {
            _vis =
                VisualStyleRegistry.getVisualization(
                    new AcmeComponent("__test__"));
            eltType = ConstantTypes.COMPONENT_BASE_TYPE;
        } else if (type == ElementVisualization.DEFAULT_CONNECTOR) {
            _vis =
                VisualStyleRegistry.getVisualization(
                    new AcmeConnector("__test__"));
            eltType = ConstantTypes.CONNECTOR_BASE_TYPE;
        } else if (type == ElementVisualization.DEFAULT_PORT) {
            _vis =
                VisualStyleRegistry.getVisualization(new AcmePort("__test__"));
            eltType = ConstantTypes.PORT_BASE_TYPE;
        } else if (type == ElementVisualization.DEFAULT_ROLE) {
            _vis =
                VisualStyleRegistry.getVisualization(new AcmeRole("__test__"));
            eltType = ConstantTypes.ROLE_BASE_TYPE;
        }
        _vis.addPropertyChangeListener("", this);
        if (_visWithoutLabel != null) _visWithoutLabel.dispose ();
        _visWithoutLabel = _vis.copy();
        _visWithoutLabel.setLabelSource(LabelOptions.NOLABEL);
        ManagedImage preview = _visWithoutLabel.getPreviewImage(null);
        preview.reference (this);
        ImageData previewImageData = preview.getImageData();
        Image smallImage = null;
        String key = "preview_palette_" + _defaultType;
        ImageManager.disposeImage(key);

        int imageHeight = preview.getBounds().height;
        int imageWidth = preview.getBounds().height;
        float aspectRatio = (float)imageHeight / (float)imageWidth;

        ImageData sid = previewImageData;
        if (imageHeight > 12) {
            int scaledWidth = Utils.zeroGuard((int) (12 * 1 / aspectRatio));
            sid = previewImageData.scaledTo(scaledWidth, 12);
            smallImage = ImageManager.createImage(key, sid, true);
//            smallImage = new Image(null, sid);
//            ImageManager.imageCreated ("GlobalTemplatePaletteEntry.setVisualization." + key, smallImage);
        } else if (imageWidth > 20) {
            int scaledHeight = Utils.zeroGuard((int) (20 * aspectRatio));
            sid = previewImageData.scaledTo(20, scaledHeight);
            smallImage = ImageManager.createImage(key, sid, true);
//            smallImage = new Image(null, sid);
//            ImageManager.imageCreated ("GlobalTemplatePaletteEntry.setVisualization." + key,smallImage);
        } else {
            smallImage = ImageManager.createImage(key, preview.getImage ().getImageData(), true);
//            smallImage = new Image(null, preview.getImage (), 0);
//            ImageManager.imageCreated ("GlobalTemplatePaletteEntry.setVisualization." + key,smallImage);
        }

        if (_image != null) {
            _image.dispose();
        }
//        smallImage.dispose ();
        _image = PaletteImageDescriptor.createDescriptor(key, smallImage);

        _visWithoutLabel.dispose ();
        if (_visCopy != null) {
        	_visCopy.getSmallImage(eltType).dispose (this);
        	_visCopy.dispose ();
        }
        _visCopy = _vis.copy ();
        ManagedImage mim = _visCopy.getSmallImage (eltType);
        mim.reference (this);
        _image.setImage (mim);
        return _image;
    }

    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals(ElementVisualization.PROP_VIS_CHANGED)) {
            ImageDescriptor img = setVisualization(_defaultType);
            setSmallIcon(img);
            setLargeIcon(img);
            fireImageChanged();
        }
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        _listeners.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener l) {
        _listeners.removePropertyChangeListener(l);
    }

    protected void fireImageChanged() {
        _listeners.firePropertyChange(
            AcmeTypeTemplateEntry.PROP_TEMPL_CHANGE,
            null,
            null);
    }
    
    public void dispose () {
        _image.dispose ();   
    }
    
    
	/**
	 * @return
	 * @see org.eclipse.gef.palette.CreationToolEntry#createTool()
	 **/
	public Tool createTool() {
		return new AcmeElementCreationTool (factory);
	}

	/**
	 * @param image
	 * @see AcmeStudio.util.IManagedImageListener#imageDisposed(AcmeStudio.util.ManagedImage)
	 **/
	public void imageDisposed(ManagedImage image) {
		fireImageChanged ();
	}
}
