package AcmeStudio.ui.editors.diagram.palette;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.*;
import org.eclipse.core.runtime.*;
import org.eclipse.gef.palette.*;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.prefs.ConnectionPatternPreferences;
import AcmeStudio.prefs.DiagramEditorPreferences;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.palette.connection.RuleBasedConnectionCreationToolEntry;
import AcmeStudio.ui.editors.diagram.palette.extension.AcmeStudioPaletteSeparator;
import AcmeStudio.ui.editors.diagram.palette.extension.IPaletteExtender;
import AcmeStudio.ui.editors.diagram.palette.extension.PaletteTab;
import AcmeStudio.ui.editors.diagram.palette.extension.PaletteTree;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.SystemVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.util.Debug;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.ManagedImage;
import AcmeStudio.wizards.connection.PortConnectionRule;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.*;

/**
 * @author schmerl
 * 
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates. To enable and disable the creation of type
 * comments go to Window>Preferences>Java>Code Generation.
 */
public class PaletteManager implements PropertyChangeListener
{

    protected PaletteRoot _root;
    protected AcmeSystem _acmeSystem;
    protected AcmeDesign _acmeDesign;
    protected DiagramEditor _diagramEditor;

    protected List _topLevelEntries = new ArrayList(2);
    private static final PaletteEntry DUMMY_PALETTE_ENTRY = new GlobalTemplatePaletteEntry(
            AcmeComponent.class, "_dummy");

    private static final int COMPONENT_PROC = 0;
    private static final int PORT_PROC = 1;
	private PaletteGroup	_architectureGroup;

    public PaletteManager(AcmeSystem system, DiagramEditor parent)
    {
        _acmeSystem = system;
        if (_acmeSystem != null)
        {
            _acmeDesign = system.getParentDesign();
        }
        _diagramEditor = parent;
        _root = createPalette();

    }

    public PaletteRoot getPaletteRoot()
    {
        return _root;
    }

    /**
     * Create the palette for the diagram editor.
     * 
     * @return The root of the palette.
     */
    protected PaletteRoot createPalette() {
		PaletteRoot acmePalette = new PaletteRoot ();
		acmePalette.setUserModificationPermission (PaletteRoot.PERMISSION_NO_MODIFICATION);
		PaletteTab tab = new PaletteTab ();
		acmePalette.add (tab);
		_architectureGroup = new PaletteGroup ("Architecture");
		List entries = new ArrayList ();
		entries.add (createControlGroup (acmePalette));
		_architectureGroup.addAll (entries);
		updateTypeEntries (_architectureGroup);
		tab.add (_architectureGroup);
		tab.setActiveEntry (_architectureGroup);
		IExtensionRegistry registry = Platform.getExtensionRegistry ();
		IExtensionPoint point = registry.getExtensionPoint ("AcmeStudio.paletteTabs");
		IExtension [] extensions = point.getExtensions ();
		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement [] elements = extensions [i].getConfigurationElements ();
			for (int j = 0; j < elements.length; j++) {
				if ("tabEntry".equals (elements [j].getName ())) {
					try {
						IPaletteExtender extender = (IPaletteExtender )elements [j].createExecutableExtension ("extenderClass");
						if (extender.isApplicable (_acmeSystem)) {
							PaletteGroup newGroup = new PaletteGroup (elements [j].getAttribute ("name"));
							extender.createPaletteChildren (newGroup,_acmeSystem);
							tab.add (newGroup);
						}
					}
					catch (CoreException e) {
						Debug.errorln (e);
					}
				}
			}
		}
		return acmePalette;
	}

	protected void updateTypeEntries(PaletteContainer acmePalette)
    {
        List entries = new ArrayList();
        _topLevelEntries = createCategories();
        entries.addAll(_topLevelEntries);
        acmePalette.addAll(entries);

    }

    /**
     * Create the categories for the palette.
     */
    protected List createCategories()
    {
        List categories = new ArrayList();
        //        categories.add(createControlGroup());

        if (AcmeStudioPlugin.getDefault().getPluginPreferences().getString(DiagramEditorPreferences.ORGANIZE_PALETTE).equals(DiagramEditorPreferences.ORGANIZE_PALETTE_FAMILY)) {
        
	        createCategoriesByFamily (categories);
        }
        else if (AcmeStudioPlugin.getDefault().getPluginPreferences().getString(DiagramEditorPreferences.ORGANIZE_PALETTE).equals(DiagramEditorPreferences.ORGANIZE_PALETTE_ELEMENT)) {
        	createCategoriesByElement (categories);
        }
        return categories;
    }

    /**
	 * @param categories
	 */
	private void createCategoriesByElement(List categories) {
		AcmeEnumeration availableTypes = null;
		List connectionEntries = new ArrayList ();
		if (_acmeSystem instanceof AcmeFamily) {
			AcmeFamily systemFam = (AcmeFamily )_acmeSystem;
			 availableTypes = systemFam.getAvailableTypes();
			 processConnectionRules (connectionEntries, systemFam, new HashSet ());
		}
		else if (_acmeSystem != null) {
			availableTypes = new AcmeEnumeration ();
			AcmeEnumeration families = _acmeSystem.getDeclaredTypes();
			while (families.moreItems ()) {
				AcmeFamily acmeFamily = (AcmeFamily )families.getNextItem();
				availableTypes.concatenate((acmeFamily).getAvailableTypes());
				processConnectionRules (connectionEntries, acmeFamily, new HashSet ());
			}
		}
		availableTypes = availableTypes.castToAcmeSetVector().enumerate();
		
		List componentsList = new ArrayList ();
		List connectorsList = new ArrayList ();
		List portsList = new ArrayList ();
		List rolesList = new ArrayList ();
		
		processTypesIntoLists(componentsList, connectorsList, portsList, rolesList, availableTypes);
        if (AcmeStudioPlugin.getDefault ().getPreferenceStore().getBoolean
                (ConnectionPatternPreferences.SHOW_PALETTE_TREE))
        {
        	PaletteDrawer patternCat = new PaletteDrawer ("Connection Patterns");
        	PaletteTree root = new PaletteTree ("All families", "Connection rules for all families", null );
        	for (Iterator iter = connectionEntries.iterator (); iter.hasNext ();) {
				PaletteTree tree = (PaletteTree )iter.next ();
				root.add (tree);
			}
        	root.checkActiveEntry();
//        	root.addAll(connectionEntries);
//        	categories.add (root);
        	patternCat.add (root);
        	categories.add (patternCat);
        }
		PaletteDrawer typesCat = new PaletteDrawer ("All Types");
		categories.add (typesCat);
        AcmeStudioPaletteSeparator compSeparator = new AcmeStudioPaletteSeparator ("components", "Components");
        typesCat.add(compSeparator);
        List tools = new ArrayList ();
        tools.add(new GlobalTemplatePaletteEntry( AcmeComponent.class, "Component"));
		tools.addAll (componentsList);
		typesCat.addAll (tools);
		compSeparator.addAllManaged(tools);
		
		tools.clear ();
        AcmeStudioPaletteSeparator connectorCat = new AcmeStudioPaletteSeparator ("connectors", "Connectors");
		tools.add (new GlobalTemplatePaletteEntry (AcmeConnector.class, "Connector"));
		tools.addAll (connectorsList);
		typesCat.add (connectorCat);
		typesCat.addAll (tools);
		connectorCat.addAllManaged(tools);
		
		tools.clear ();
		AcmeStudioPaletteSeparator portCat = new AcmeStudioPaletteSeparator ("ports", "Ports");
		tools.add (new GlobalTemplatePaletteEntry (AcmePort.class, "Port"));
		tools.addAll (portsList);
		typesCat.add (portCat);
		typesCat.addAll (tools);
		portCat.addAllManaged (tools);
		
		tools.clear ();
		AcmeStudioPaletteSeparator roleCat = new AcmeStudioPaletteSeparator ("roles", "Roles");
		tools.add (new GlobalTemplatePaletteEntry (AcmeRole.class, "Role"));
		tools.addAll (rolesList);
		typesCat.add (roleCat);
		typesCat.addAll (tools);
		roleCat.addAllManaged(tools);
		
		if (AcmeStudioPlugin.getDefault ().getPreferenceStore().getBoolean
                (ConnectionPatternPreferences.SHOW_PALETTE_TREE))
        {
			roleCat.setOpen (false);
			portCat.setOpen (false);
			connectorCat.setOpen (false);
        }
	}

	/**
	 * @param connectionEntries
	 * @param family
	 * @param visited
	 */
	private void processConnectionRules(List connectionEntries, AcmeFamily family, HashSet visited) {
		if (!visited.contains (family.getTypeName())) {
			visited.add(family.getTypeName());
			PaletteTree connectionTreeRoot = new PaletteTree(family.getTypeName() + "  Connections",
					"Connections for " + family.getTypeName(), null);
			addConnectRulesToStack(connectionTreeRoot, family);
			connectionEntries.add(connectionTreeRoot);        
			AcmeEnumeration supers = family.getSuperTypes().enumerate ();
			while (supers.moreItems ()) {
				AcmeFamily fam = (AcmeFamily )supers.getNextItem ();
				processConnectionRules(connectionEntries, fam, visited);
			}
		}
	}
	

	/**
	 * @param categories
	 */
	private void createCategoriesByFamily(List categories) {
		PaletteDrawer elementsCat = createElementsCategory();
		categories.add(elementsCat);

		if (_acmeSystem instanceof AcmeFamily)
		{
		    AcmeFamily systemFam = (AcmeFamily) _acmeSystem;
		    Set familySet = new HashSet();
		    familySet.add("system");
		    familySet.add(systemFam.getTypeName());
		    elementsCat.setInitialState(PaletteDrawer.INITIAL_STATE_CLOSED);
		    PaletteDrawer systemCat = createFamilyCategory(systemFam);
		    categories.add(systemCat);
		    systemCat.setInitialState(PaletteDrawer.INITIAL_STATE_OPEN);

		    processSuperFamilies(systemFam, categories, familySet);

		}
		else if (_acmeSystem != null)
		{
		    Set familySet = new HashSet();
		    familySet.add("system");
		    AcmeEnumeration families = _acmeSystem.getDeclaredTypes();
		    if (families.moreItems())
		    {
		        elementsCat.setInitialState(PaletteDrawer.INITIAL_STATE_CLOSED);
		    }
		    while (families.moreItems())
		    {
		    
		        AcmeObject o = families.getNextItem();
		        if (!(o instanceof AcmeFamily)) continue;
				AcmeFamily fam = (AcmeFamily) o;
		        if (!familySet.contains(fam.getTypeName()))
		        {
		            familySet.add(fam.getTypeName());
		            PaletteDrawer familyCat = createFamilyCategory(fam);
		            categories.add(familyCat);
		            // Include a palette for all superfamilies (BugFix 124)
		            processSuperFamilies(fam, categories, familySet);
		            familyCat.setInitialState(PaletteDrawer.INITIAL_STATE_OPEN);
		        }
		    }
		}
	}

	/**
     * Processes a family's supertypes, so that a category is included for each
     * family recursing up to the root.
     * 
     * @param fam
     *                 The family whose super types we should add
     * @param categories
     *                 The list to add the supertypes' family to
     * @param familySet
     *                 The set of visited families (so that we don't add a family
     *                 twice)
     */
    private void processSuperFamilies(AcmeFamily fam, List categories,
            Set familySet)
    {
        AcmeEnumeration families = fam.getSuperTypes().enumerate();

        while (families.moreItems())
        {
            AcmeFamily f = (AcmeFamily) families.getNextItem();
            if (!familySet.contains(f.getTypeName()))
            {
                familySet.add(f.getTypeName());
                PaletteDrawer familyCat = createFamilyCategory(f);
                familyCat.setInitialState(PaletteDrawer.INITIAL_STATE_CLOSED);
                categories.add(familyCat);
                processSuperFamilies(f, categories, familySet);
            }
        }
    }

    protected PaletteDrawer createFamilyCategory(AcmeFamily fam)
    {
        PaletteDrawer famCategory = new PaletteDrawer(fam.getTypeName());
        List famList = new ArrayList();

        //add the rule tree for this family
        boolean showPalette = AcmeStudioPlugin.getDefault ().getPreferenceStore().getBoolean
                (ConnectionPatternPreferences.SHOW_PALETTE_TREE);
		if (showPalette)
        {
	        PaletteTree connectionTreeRoot = new PaletteTree("Connections",
	                "Connections", null);
	
	        if (addConnectRulesToStack(connectionTreeRoot, fam)) {
	
	        	famList.add(connectionTreeRoot);        
	        }
        }
        
        Comparator compTo = new Comparator()
        {
            public int compare(Object o1, Object o2)
            {
                if (o1 instanceof AcmeTypeTemplateEntry
                        && o2 instanceof AcmeTypeTemplateEntry)
                {
                    AcmeTypeTemplateEntry t1 = (AcmeTypeTemplateEntry) o1;
                    AcmeTypeTemplateEntry t2 = (AcmeTypeTemplateEntry) o2;
                    return t1.compareTo(t2);
                }
                return 0;
            }
        };

        List compList = new ArrayList();
        List connList = new ArrayList();
        List roleList = new ArrayList();
        List portList = new ArrayList();

        AcmeEnumeration types = _acmeSystem.getAvailableTypes();
        types = fam.getTypes();
        processTypesIntoLists(compList, connList, portList, roleList, types);

        Collections.sort(compList, compTo);
        Collections.sort(connList, compTo);
        Collections.sort(roleList, compTo);
        Collections.sort(portList, compTo);

        if (!compList.isEmpty())
        {
            AcmeStudioPaletteSeparator compSeparator = new AcmeStudioPaletteSeparator(
                    fam.getName() + ".components", "Components");
            famList.add(compSeparator);
            famList.addAll(compList);
            compSeparator.addAllManaged(compList);
        }
        if (!connList.isEmpty())
        {
            AcmeStudioPaletteSeparator connSeparator = new AcmeStudioPaletteSeparator(
                    fam.getName() + ".connectors", "Connectors");
            famList.add(connSeparator);
            famList.addAll(connList);
            connSeparator.addAllManaged (connList);
            if (showPalette) connSeparator.setOpen (false);
        }
        if (!portList.isEmpty())
        {
            AcmeStudioPaletteSeparator portSeparator = new AcmeStudioPaletteSeparator(
                    fam.getName() + ".ports", "Ports");
            famList.add(portSeparator);
            famList.addAll(portList);
            portSeparator.addAllManaged(portList);
            if (showPalette) portSeparator.setOpen (false);
        }
        if (!roleList.isEmpty())
        {
            AcmeStudioPaletteSeparator roleSeparator = new AcmeStudioPaletteSeparator(
                    fam.getName() + ".roles", "Roles");
            famList.add(roleSeparator);
            famList.addAll(roleList);
            roleSeparator.addAllManaged(roleList);
            if (showPalette) roleSeparator.setOpen (false);
        }
        famCategory.addAll(famList);
       
        return famCategory;
    }

    /**
     * Create the element category for the palette.
     * 
     * @return Root of the element category.
     */
    protected PaletteDrawer createElementsCategory()
    {

        PaletteDrawer globalCategory = new PaletteDrawer("Global Typespace");
        List globalList = new ArrayList();

        List connectorsList = new ArrayList();

        List portsList = new ArrayList();

        List rolesList = new ArrayList();
        List componentsList = new ArrayList();
        CombinedTemplateCreationEntry template = new GlobalTemplatePaletteEntry(
                AcmeComponent.class, "Component");

        globalList.add(template);

        template = new GlobalTemplatePaletteEntry(AcmeConnector.class,
                "Connector");

        globalList.add(template);

        template = new GlobalTemplatePaletteEntry(AcmePort.class, "Port");

        globalList.add(template);

        template = new GlobalTemplatePaletteEntry(AcmeRole.class, "Role");

        globalList.add(template);

        if (_acmeDesign != null)
        {
            AcmeEnumeration types = _acmeDesign.getAvailableTypes();
            processTypesIntoLists(componentsList, connectorsList, portsList,
                    rolesList, types);
        }

        globalCategory.addAll(globalList);

        return globalCategory;

    }

    private void processTypesIntoLists(List componentsList,
            List connectorsList, List portsList, List rolesList,
            AcmeEnumeration types)
    {
        CombinedTemplateCreationEntry template;
        while (types.moreItems())
        {

            Object o = types.getNextItem();
            if (o instanceof AcmeElementType && !(o instanceof AcmeFamily))
            {
                AcmeElementType type = (AcmeElementType) o;
                ElementVisualization vis = VisualStyleRegistry
                        .getVisualization(type);
                boolean showInPalette = (vis == null || (vis != null && !vis
                        .isHiddenFromPalette()));

                List theList = null;
                if (type instanceof AcmeComponentType)
                {
                    theList = componentsList;
                }
                else if (type instanceof AcmeConnectorType)
                {
                    theList = connectorsList;
                }
                else if (type instanceof AcmeRoleType)
                {
                    theList = rolesList;
                }
                else if (type instanceof AcmePortType)
                {
                    theList = portsList;
                }

                // Only add to palette if it is a valid type (i.e., filter out
                // Element types)
                // BugFix #142
                if (theList != null && showInPalette)
                {
                    template = new AcmeTypeTemplateEntry(type, type
                            .getTypeName());
                    template.addPropertyChangeListener(this);

                    theList.add(template);
                }

            }
        }
    }

    /**
     * Create the control category for the palette.
     * 
     * @return Root of the control category.
     */
    private PaletteContainer createControlGroup(PaletteRoot root)
    {
        RuleBasedConnectionCreationToolEntry entry;

        PaletteGroup controlGroup = new PaletteGroup("Control");
        List entries = new ArrayList();
        ToolEntry tool = new PanningSelectionToolEntry("Select");
        root.setDefaultEntry(tool);
        entries.add(tool);
        
        //The first item to be added will be the general connect tool, which 
        //spawns the wizard.  Add the item if there is at least 1 connection pattern,
        //or if we are in a binding command
        if (_acmeSystem.getParentRepresentation() != null || (VisualStyleRegistry.getSysVis(_acmeSystem).getPortConnectionRules() != null && VisualStyleRegistry.getSysVis(_acmeSystem).getAllPortConnectionRules(null).size() >= 1))
        {
            entry = new RuleBasedConnectionCreationToolEntry(
                    "Connect", "Connect", null, ImageManager
                    .getImageDescriptor(ImageManager.ID_CONN
                            + ImageManager.SMALL), ImageManager
                    .getImageDescriptor(ImageManager.ID_CONN), _diagramEditor.getEditDomain(), null);

//        	entry.setEditDomain ((AcmeStudioEditDomain )_diagramEditor.getEditDomain());
            entries.add(entry);        
        }
        controlGroup.addAll(entries);
        return controlGroup;
    }

    /*
     * LFW: Helper method created to fill the palette stack with all existing
     * rules
     */
    private boolean addConnectRulesToStack(PaletteTree root, AcmeFamily fam)
    {         
    	boolean addToTree = false;
//        AcmeEnumeration families = _acmeSystem.getDeclaredTypes();
//        
//        // 	First, iterate through the families        
//        
//        while (families.moreItems())
//        {
//            AcmeFamily thisFam = (AcmeFamily) families.getNextItem();
//            
//            if (thisFam.equals(fam))
//            {
	            //Create a new subtree for the family level of the palette tree
	            //PaletteTree subTree = new PaletteTree(fam.getName(), fam.getName(),
	             //       null);
	            //PaletteTree newTree = null;
	
	            //root.add(subTree);
	
	            SystemVisualization vis = VisualStyleRegistry.getSysVis(fam);
	            
	            if (vis.getPortConnectionRules() != null) //there are rules
	            {
		            //Pass in null to get all rules back
		            List ret = vis.getAllPortConnectionRules(null);
		        
		            addToTree = ! (ret.size () == 1 && ret.get (0) == VisualStyleRegistry.DEFAULT_PORT_CONNECTION_RULE);
		            if (addToTree) {
		            	//Create the component hierarchy
			            createHierarchy(root, ret, PaletteManager.COMPONENT_PROC);
			
			            //Now add the port based branch of the palette tree
			            PaletteTree newTree = new PaletteTree("By Ports", "By Ports", null);
			            root.add(newTree);
			            createHierarchy(newTree, ret, PaletteManager.PORT_PROC);
		            }
	            }
//            }
//        }
	            return addToTree;
    }

    /**
     * @param subTree
     * @param ret
     */
    private void createHierarchy(PaletteTree subTree, List ret, int type)
    {
        RuleBasedConnectionCreationToolEntry entry;
        PaletteTree newTree;
        HashMap storedInfo = new HashMap();

        //iterate over each rule and get the next tree level info
        for (Iterator iter = ret.iterator(); iter.hasNext();)
        {

            PortConnectionRule rule = (PortConnectionRule) iter.next();
            String dispString;

            if (type == PaletteManager.COMPONENT_PROC)
                dispString = rule.getCompLevelString();
            else
                dispString = rule.getPortLevelString();
            boolean useRuleImage = AcmeStudioPlugin.getDefault().getPreferenceStore().getString(ConnectionPatternPreferences.PALETTE_DISPLAY).equals (ConnectionPatternPreferences.USE_PATTERN_IMAGE);
            String label = useRuleImage?"":rule.toString ();
            ImageDescriptor image = useRuleImage?getRuleImageDescriptor (rule):ImageManager.getImageDescriptor (ImageManager.ID_CONN + ImageManager.SMALL);
            
            if (storedInfo.get(dispString) != null)
            {
                //This type of comp entry has already been added to the subtree
                //so simply add it to the existing subtree for it
                PaletteTree existing = (PaletteTree) storedInfo.get(dispString);
                
                entry = new RuleBasedConnectionCreationToolEntry(label,  rule.toString(), null, image, image, _diagramEditor.getEditDomain (), rule);
//                entry.setEditDomain((AcmeStudioEditDomain )_diagramEditor.getEditDomain());
                existing.add(entry);
            }
            else
            {
                //create the new subtree to contain rules falling under this
                //comp type
                newTree = new PaletteTree(dispString, dispString, null);
                subTree.add(newTree);

                //Store this subtree in the hashmap under the compstring key,
                //so it can be accessed later to fill up with the necessary data
                storedInfo.put(dispString, newTree);

                //Now add the rule to this subtree
                entry = new RuleBasedConnectionCreationToolEntry(label, rule.toString (), null, image, image,  _diagramEditor.getEditDomain(), rule);
//                entry.setEditDomain((AcmeStudioEditDomain )_diagramEditor.getEditDomain());
                newTree.add(entry);
            }
        }
    }

    /**
	 * @param rule
	 * @return
	 */
	private ImageDescriptor getRuleImageDescriptor(PortConnectionRule rule) {
		Image image = ImageManager.getExistingImage(rule.toString ());
		if (image == null) {
			image = Helper.generateConnectionRuleImage(rule, _acmeSystem);
		}
		ManagedImage mim = ManagedImage.get(rule.toString ());
		if (mim == null) {
			mim = new ManagedImage (image, this);
			ManagedImage.put(rule.toString (), mim);
		}
		return new PaletteImageDescriptor (mim);
	}

	/**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt)
    {
        if (evt.getPropertyName().equals(
                AcmeTypeTemplateEntry.PROP_TEMPL_CHANGE))
        {

            if (_diagramEditor != null) _diagramEditor.refreshPalette(true);
        }
    }

    private List digOutTemplateEntries(PaletteContainer c)
    {
        List entries = new ArrayList();
        List children = c.getChildren();
        Iterator i = children.iterator();
        while (i.hasNext())
        {
            Object child = i.next();
            if (child instanceof PaletteContainer)
            {
                entries.addAll(digOutTemplateEntries((PaletteContainer) child));
            }
            else if (child instanceof AcmeTypeTemplateEntry
                    || child instanceof GlobalTemplatePaletteEntry)
            {
                //                Debug.diagnoseln("found " + child.toString ());
                entries.add(child);
            }

        }
        return entries;
    }

    public void updateFamily(AcmeFamily family)
    {
        PaletteDrawer familyCategory = null;
        Iterator iter = _topLevelEntries.iterator();
        while (iter.hasNext() && familyCategory == null)
        {
            PaletteDrawer drawer = (PaletteDrawer) iter.next();
            if (drawer.getLabel().equals(family.getName()))
                familyCategory = drawer;
        }
        if (familyCategory != null)
        {

            PaletteDrawer newCat = createFamilyCategory(family);
            int oldIndex = 0;
            int newIndex = 0;
            if (newCat.getChildren().size() > familyCategory.getChildren()
                    .size())
            {
                int indexNE = 0;
                int i;
                for (i = 0; i < familyCategory.getChildren().size(); i++, indexNE++)
                {
                    PaletteEntry oE = (PaletteEntry) familyCategory
                            .getChildren().get(i);
                    PaletteEntry nE = (PaletteEntry) newCat.getChildren().get(
                            indexNE);
                    if (!equivalentEntry(oE, nE))
                    {
                        familyCategory.add(i, nE);
                    }
                }
                for (indexNE = i; indexNE < newCat.getChildren().size(); indexNE++)
                {
                    familyCategory.add((PaletteEntry) newCat.getChildren().get(
                            indexNE));
                }
            }
            else
            {
                int indexNE = 0;
                int i;
                for (i = 0; i < newCat.getChildren().size(); i++, indexNE++)
                {
                    PaletteEntry oE = (PaletteEntry) familyCategory
                            .getChildren().get(i);
                    PaletteEntry nE = (PaletteEntry) newCat.getChildren().get(
                            indexNE);
                    if (!equivalentEntry(oE, nE))
                    {
                        familyCategory.remove(oE);
                        i--;
                        indexNE--;
                    }
                }
                for (indexNE = i; i < familyCategory.getChildren().size(); indexNE++)
                {
                    familyCategory.remove((PaletteEntry) familyCategory
                            .getChildren().get(i));
                }
            }

            /*
             * int index = _topLevelEntries.indexOf(familyCategory);
             * _topLevelEntries.remove(familyCategory); familyCategory =
             * createFamilyCategory(family); _topLevelEntries.add(index,
             * familyCategory);
             */
            // force refreshing
            familyCategory.add(DUMMY_PALETTE_ENTRY);
            familyCategory.remove(DUMMY_PALETTE_ENTRY);
        }
    }

    /**
     * @param oe
     * @param ne
     * @return
     */
    private boolean equivalentEntry(PaletteEntry oe, PaletteEntry ne)
    {
        if (oe.getClass() != ne.getClass())
            return false;
        else if (oe.getLabel().equals(ne.getLabel()))
            return true;
        return false;
    }

    public void reset()
    {
        List acmeTemplates = new ArrayList();
        Iterator i = _topLevelEntries.iterator();
        while (i.hasNext())
        {
        	Object obj = i.next ();
        	if (obj instanceof PaletteContainer) {
        		PaletteContainer c = (PaletteContainer )obj;
        		acmeTemplates.addAll(digOutTemplateEntries(c));
        		_architectureGroup.remove(c);
        	}
        	else if (obj instanceof PaletteEntry) {
				PaletteEntry e = (PaletteEntry )obj;
				_architectureGroup.remove (e);
			} 
        		
        }

        i = acmeTemplates.iterator();
        while (i.hasNext())
        {
            Object o = i.next();
            if (o instanceof AcmeTypeTemplateEntry)
            {
                AcmeTypeTemplateEntry entry = (AcmeTypeTemplateEntry) o;
                entry.setSmallIcon(null);
                entry.setLargeIcon(null);
            }
        }

  
        updateTypeEntries(_architectureGroup);
        _diagramEditor.getEditDomain().setPaletteViewer(_diagramEditor.getEditDomain().getPaletteViewer());
    }

    /**
     * @param root
     */
    private void removeAcmeTypeEntries(PaletteRoot root)
    {
        Object[] children = root.getChildren().toArray();
        //		for (int i = 0; i < children.length; i++) {
        //			root.remove((PaletteEntry )children [i]);
        //		}
    }

    public void dispose()
    {
        List acmeTemplates = new ArrayList();
        if (_topLevelEntries != null)
        {
            Iterator i = _topLevelEntries.iterator();
            while (i.hasNext())
            {
                PaletteContainer c = (PaletteContainer) i.next();
                acmeTemplates.addAll(digOutTemplateEntries(c));
                if (_root != null)
                    _root.remove(c);
            }
            _topLevelEntries.clear();
            _topLevelEntries = null;
        }

        Iterator i = acmeTemplates.iterator();
        while (i.hasNext())
        {
            Object e = i.next();
            if (e instanceof AcmeTypeTemplateEntry)
            {
                AcmeTypeTemplateEntry entry = (AcmeTypeTemplateEntry) e;
                entry.dispose();
            }
            else if (e instanceof GlobalTemplatePaletteEntry)
            {
                GlobalTemplatePaletteEntry entry = (GlobalTemplatePaletteEntry) e;
                entry.dispose();
            }
        }
        acmeTemplates.clear();
        _diagramEditor = null;
        _root = null;
    }

	/**
	 * @return Returns the acmeSystem.
	 */
	public AcmeSystem getAcmeSystem() {
		return _acmeSystem;
	}
}