/*
 * Created on Jun 15, 2004
 *  
 */
package AcmeStudio.ui.editors.diagram.palette.connection;

import org.eclipse.draw2d.SWTGraphics;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.tools.ConnectionCreationTool;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.prefs.ConnectionPatternPreferences;
import AcmeStudio.ui.editors.diagram.actions.AddBindingCommand;
import AcmeStudio.ui.editors.diagram.actions.ConnectElementsCommand;
import AcmeStudio.ui.editors.diagram.palette.PaletteImageDescriptor;
import AcmeStudio.ui.editors.diagram.parts.BindingPointEditPart;
import AcmeStudio.util.Constants;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.ManagedImage;
import AcmeStudio.wizards.connection.PortConnectionRule;

/**
 * @author Bradley Schmerl Created Jun 15, 2004
 */
public class RuleBasedConnectionCreationTool extends ConnectionCreationTool
{

    private Object _rule;
    private Object _origRule;
	private ToolEntry _myToolEntry;
	private Image	_stickyImage;

    /**
     * @param rule
     */
    public RuleBasedConnectionCreationTool(Object rule)
    {
        _rule = rule;
        _origRule = rule;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.tools.AbstractConnectionCreationTool#handleCreateConnection()
     */
    protected boolean handleCreateConnection()
    {
        eraseSourceFeedback();
        Command endCommand = getCommand();
        Request holder = getTargetRequest();

        if (endCommand instanceof ConnectElementsCommand)
        {
            ConnectElementsCommand cpc = (ConnectElementsCommand) endCommand;
            //			cpc.setRule (_rule);
        }
        setCurrentCommand(endCommand);
        if (getTargetAnchor(holder) != null)
            getTargetAnchor(holder).eraseTargetFeedback(holder);
        executeCurrentCommand();
        return true;
    }
    
    protected void executeCurrentCommand() {
    	Command comm = getCurrentCommand();
    	
    	if (comm instanceof ConnectElementsCommand)
        {
            ConnectElementsCommand currentCommand = (ConnectElementsCommand) comm;
            super.executeCurrentCommand();
    	
	    	// Change the name to be the sticky rule, if there is one
	    	_myToolEntry = getDomain().getPaletteViewer().getActiveTool();
    	    PortConnectionRule sticky = ConnectElementsCommand.stickyRule;
	    	if (sticky != null) {
	    		if (AcmeStudioPlugin.getDefault().getPreferenceStore().getString(ConnectionPatternPreferences.PALETTE_DISPLAY).equals(ConnectionPatternPreferences.USE_PATTERN_TEXT)) {
		    		_myToolEntry.setLabel(sticky.toString());
		    		_myToolEntry.setDescription(sticky.toString ());
		    		_myToolEntry.setSmallIcon(ImageManager.getImageDescriptor(ImageManager.ID_LOCKED));
	    		}
	    		else {
	    			Image lockImage = ImageManager.getIcon(ImageManager.ID_LOCKED);
	    			Image ruleImage = Helper.generateConnectionRuleImage(sticky, currentCommand.getSystem() );
	    			if (_stickyImage != null) _stickyImage.dispose ();
	    			int height = Math.max (ruleImage.getImageData ().height, lockImage.getImageData ().height);
					int width = ruleImage.getImageData().width + lockImage.getImageData ().width + 2;
					_stickyImage = new Image (null, width, height);
					GC gc = new GC (_stickyImage);
					SWTGraphics graphics = new SWTGraphics (gc);
					graphics.pushState ();
					graphics.setForegroundColor(Constants.TRANSPARENT_COLOR);
					graphics.setBackgroundColor(Constants.TRANSPARENT_COLOR);
					graphics.fillRectangle (0, 0, width, height);
					graphics.popState ();
					graphics.drawImage (lockImage, 0, (height - lockImage.getImageData().height)/2);
					graphics.drawImage (ruleImage, lockImage.getImageData ().width + 2, (height - ruleImage.getImageData().height)/2);
					gc.dispose ();
					graphics.dispose();
//					ruleImage.dispose ();
					
					ImageData data  = _stickyImage.getImageData();
					data.transparentPixel = data.palette.getPixel(Constants.TRANSPARENT_COLOR.getRGB ());
					_stickyImage.dispose ();
					_stickyImage = new Image (null, data);
					
					_myToolEntry.setLabel("");
					_myToolEntry.setSmallIcon(new PaletteImageDescriptor (new ManagedImage (_stickyImage, this)));
					_myToolEntry.setDescription(sticky.toString ());
					
	    		}
	    		if (_origRule == null) //this is the general connector
	    		    _rule = sticky;
	    	}
	    	else
	    	{
	    	    //ensure that _rule is null if it should be
	    	    _rule = _origRule;
	    	}
        }
    	else if (comm instanceof AddBindingCommand)
    	{
    	    super.executeCurrentCommand();
    	}
    	
    }

    protected Command getCommand()
    {
        if (getTargetEditPart() == null
                && getTargetAnchor(getTargetRequest()) == null)
            return null;

        if (getTargetAnchor(getTargetRequest()) != null)
            return getTargetAnchor(getTargetRequest()).getCommand(
                    getTargetRequest());

        return getTargetEditPart().getCommand(getTargetRequest());
    }

    private EditPart getTargetAnchor(Request r)
    {
        /* BRS: Added to cater for connection of Bindings in representations */
        if (r instanceof CreateConnectionRequest) {
        	
        
            GraphicalEditPart graphicalEditPart = (GraphicalEditPart) r.getExtendedData().get("snap");
            EditPart tep = ((CreateConnectionRequest )r).getTargetEditPart();
            if (graphicalEditPart == null && tep instanceof BindingPointEditPart)
            	return tep;
			return graphicalEditPart;
        }
        return null;
    }

    /**
     * Asks the target editpart to show target feedback and sets the target
     * feedback flag.
     */
    protected void showTargetFeedback()
    {
        if (!isShowingTargetFeedback())
         
	        if (getTargetAnchor(getTargetRequest()) == null)
	            super.showTargetFeedback();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.tools.TargetingTool#createTargetRequest()
     */
    protected Request createTargetRequest()
    {
        Request req = super.createTargetRequest();
//      add the rule to extended data for access later on
        req.getExtendedData().put("Rule", _rule); 
        return req;
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.Tool#activate()
     */
    public void activate()
    {
        super.activate();
        
    }
	/**
	 * On deactivation, check to see if the active palette entry is different
	 * If it is, then reset the connection tool to general and unstick the rule
	 * if it was sticky
	 * @see org.eclipse.gef.Tool#deactivate()
	 */
	public void deactivate() {
		super.deactivate();
		ToolEntry newEntry = getDomain().getPaletteViewer().getActiveTool();
		if (newEntry != _myToolEntry && _origRule == null) {
			if (_myToolEntry != null) {
				_myToolEntry.setLabel("Connect");
				_myToolEntry.setSmallIcon(ImageManager
                    .getImageDescriptor(ImageManager.ID_CONN
                            + ImageManager.SMALL));
				if (_stickyImage != null) _stickyImage.dispose ();
			}
	        if (_origRule == null)
	            ConnectElementsCommand.stickyRule = null;
		}
	}
}

