/*
 * Created on Jun 15, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.palette.connection;

import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Tool;
import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import AcmeStudio.ui.editors.diagram.palette.AcmeStudioEditDomain;
import AcmeStudio.ui.editors.diagram.palette.IPaletteViewerListener;


/**
 * @author Bradley Schmerl
 * Created Jun 15, 2004
 */
public class RuleBasedConnectionCreationToolEntry extends ConnectionCreationToolEntry implements IPaletteViewerListener{

	private Object	_rule;
	private EditDomain _editDomain;
	private String _fullLabel;
	private PaletteViewer _currentPaletteViewer;
	/** Is called when the PaletteViewer is resized **/
	private Listener handler = new Listener () {

		public void handleEvent(Event event) {
			setLabel (_fullLabel);
		}
		
	};

	/**
	 * @param label
	 * @param shortDesc
	 * @param factory
	 * @param iconSmall
	 * @param iconLarge
	 */
	public RuleBasedConnectionCreationToolEntry(String label, String shortDesc, CreationFactory factory, ImageDescriptor iconSmall,
			ImageDescriptor iconLarge, EditDomain domain, Object rule) {
		super (label, shortDesc, factory, iconSmall, iconLarge);
		_rule = rule;
		_editDomain = domain;
		setLabel (label);
		((AcmeStudioEditDomain )_editDomain).addPaletteViewerListener(this);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gef.palette.ToolEntry#createTool()
	 */
	public Tool createTool() {
		return new RuleBasedConnectionCreationTool (_rule);
	}
	/**
	 * @param editDomain
	 */
	public void setEditDomain(AcmeStudioEditDomain editDomain) {
		_editDomain = editDomain;
		editDomain.addPaletteViewerListener(this);
		
	}
	/**
	 * The label is truncated so that it fits with the width of the palette
	 * @param label
	 * @see org.eclipse.gef.palette.PaletteEntry#setLabel(java.lang.String)
	 */
	public void setLabel(String label) {
		_fullLabel = label;
		try {
			GraphicalEditPart part = (GraphicalEditPart) _editDomain.getPaletteViewer().getEditPartRegistry().get (this);
			if (part == null) part = (GraphicalEditPart )_currentPaletteViewer.getEditPartRegistry().get (this);
			IFigure figure = part.getFigure();
			IFigure flowpage = (IFigure )((IFigure )figure.getChildren ().get(0)).getChildren().get (1);
			int maxWidth = flowpage.getClientArea().width;
			Font font = flowpage.getFont();
			final String ELLIPSIS = "...";
			String ruleText = label;
			if (FigureUtilities.getTextWidth(ruleText, font) > maxWidth) {
				maxWidth -= FigureUtilities.getTextWidth (ELLIPSIS, font);
				int index = 0;
				String ruleSubText = "";
				do {
					ruleSubText += ruleText.charAt(index++);
				}
				while (FigureUtilities.getTextWidth(ruleSubText, font) < maxWidth);
				ruleText = ruleSubText.substring(0, ruleSubText.length () - 2) + ELLIPSIS;
			}
			super.setLabel (ruleText);
		}
		catch (Exception e) {
			//Debug.errorln("RuleBasedConnectionCreationToolEntry.setLabel(String):  BRS - SOmething bad happened");
			super.setLabel (label);
		}
		
	}
	
	public String getLabel () {
		return super.getLabel();
	}
	/**
	 * @param paletteViewer
	 * @see AcmeStudio.ui.editors.diagram.palette.IPaletteViewerListener#setPaletteViewer(org.eclipse.gef.ui.palette.PaletteViewer)
	 */
	public void setPaletteViewer(PaletteViewer paletteViewer) {
		if (_currentPaletteViewer != paletteViewer) {
			if (_currentPaletteViewer != null) _currentPaletteViewer.getControl().removeListener(SWT.Resize, handler);
			_currentPaletteViewer = paletteViewer;
			if (paletteViewer != null) {
				setLabel (_fullLabel);
			}
			if (_currentPaletteViewer != null) _currentPaletteViewer.getControl ().addListener(SWT.Resize, handler);
		}
	}
	
	
	/**
	 * @param obj
	 * @return
	 * @see java.lang.Object#equals(java.lang.Object)
	 **/
	public boolean equals(Object obj) {
		if (obj instanceof RuleBasedConnectionCreationToolEntry) {
			return this._rule.equals (((RuleBasedConnectionCreationToolEntry )obj)._rule);
		}
		return false;
	}

}
