/*
 * Created on Jan 4, 2004
 */
package AcmeStudio.ui.editors.diagram.palette.extension;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.ChangeEvent;
import org.eclipse.draw2d.ChangeListener;
import org.eclipse.draw2d.Clickable;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.internal.ui.palette.editparts.PaletteEditPart;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import com.swtdesigner.SWTResourceManager;

import AcmeStudio.AcmeStudioPlugin;

/**
 * @author Bradley Schmerl
 * The edit part for the AcmeStudio specific separator
 */
public class AcmeStudioPaletteSeparatorEditPart extends PaletteEditPart implements PropertyChangeListener {

	private boolean	_expanded = true;

	/**
	 * The figure to use for the separator - the label with a line either side of it
	 */
	private static class SeparatorFigure extends Clickable {
		/** The font used for the label **/
		private static  Font FONT;
		static {
			int size = 7;
		   if (SWT.getPlatform().equalsIgnoreCase("carbon")) {
		   	size = 9;
		   }
		   FONT = SWTResourceManager.getFont ( AcmeStudioPlugin.getDefaultFontName(), size, 0);
		}
		/** The label of the separator **/
		String label = null;
		boolean _expanded = true;
		private int _neededHeight;

		/**
		 * @param entry The Separator model that contains the label
		 */
		public SeparatorFigure(PaletteEntry entry) {
			label = entry.getLabel();
			_neededHeight = FigureUtilities.getTextExtents(label, FONT).height;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
		 */
		public Dimension getPreferredSize(int wHint, int hHint) {
			return new Dimension (wHint, _neededHeight + 2);
		}

		private static final Insets CROP = new Insets (1, 3, 1, 4);

		private boolean	_mouseOver = false;
		/* (non-Javadoc)
		 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
		 */
		protected void paintFigure(Graphics g) {
			Rectangle r = getBounds ().getCropped (CROP);
			g.pushState ();
			if (label != null) {
				if (!_mouseOver)
					g.setForegroundColor (ColorConstants.buttonDarker);
				else {
					g.setForegroundColor(ColorConstants.buttonDarkest);
				}
				g.setFont(FONT);
				Dimension textSize = FigureUtilities.getStringExtents (label, FONT);
				int centerY = r.getCenter().y;
				if (_expanded) {
					PointList points = new PointList ();
					points.addPoint (r.getLeft ().x, centerY - 2);
					points.addPoint (r.getLeft ().x + 5, centerY - 2);
					points.addPoint (r.getLeft().x + 3, centerY + 3);
					g.drawPolygon(points);
				}
				else {
					PointList points = new PointList ();
					points.addPoint (r.getLeft().x, centerY - 2);
					points.addPoint (r.getLeft ().x, centerY + 3);
					points.addPoint (r.getLeft ().x + 5, centerY);
					g.drawPolygon (points);
				}
//				g.drawLine (r.getLeft ().x, centerY, r.getLeft ().x + 3, centerY);
				g.drawLine (r.getLeft ().x + 10 + textSize.width, centerY, r.getRight ().x, centerY);
				g.drawText (label, r.getLeft().x + 11, r.getTop().y);
			}
			else {
				g.setForegroundColor (ColorConstants.buttonDarker);
				g.drawLine (r.getLeft ().x, r.getCenter ().y, r.getRight ().x, r.getCenter ().y);
			}
			g.popState ();
		}

		/**
		 * @param b
		 * 
		 */
		public void setMouseOver(boolean b) {
			_mouseOver =  b;
			repaint ();
		}

	}
	
	public AcmeStudioPaletteSeparatorEditPart (PaletteEntry model) {
		super (model);
		setModel (model);
		setExpanded (((AcmeStudioPaletteSeparator )model).isOpen ());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		final SeparatorFigure separatorFigure = new SeparatorFigure ((PaletteEntry )getModel ());
		separatorFigure.addChangeListener(new ChangeListener () {

			public void handleStateChanged(ChangeEvent event) {
				
				if (separatorFigure.getModel ().isPressed()) {
					
					setExpanded(!isExpanded ());
					separatorFigure._expanded = isExpanded ();
					expand ();
				}
				else if (separatorFigure.getModel ().isMouseOver()) {
					separatorFigure.setMouseOver (true);
				}
				else if (!separatorFigure.getModel ().isMouseOver ()) {
					separatorFigure.setMouseOver (false);
				}
			}
			
		});
		expand ();
		return separatorFigure;
	}

	boolean isExpanded () {
		return _expanded;
	}
	
	void setExpanded (boolean expanded) {
		_expanded = expanded;
		((SeparatorFigure )getFigure ())._expanded = expanded;
	}
	
	
	void expand () {
		AcmeStudioPaletteSeparator model = (AcmeStudioPaletteSeparator )getModel ();
			List managed = model.getManaged();
			for (Iterator iter = managed.iterator (); iter.hasNext ();) {
				PaletteEntry entry = (PaletteEntry )iter.next ();
				entry.setVisible(isExpanded ());
			}
	
	}
	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String property = evt.getPropertyName();
		if (property.equals (PaletteEntry.PROPERTY_LABEL)) {
			refreshVisuals ();
		}
		else if (property.equals (AcmeStudioPaletteSeparator.PROPERTY_OPEN)) {
			boolean expanded = ((Boolean )evt.getNewValue()).booleanValue();
			setExpanded (expanded);
			
			expand ();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate() {
		super.activate();
		PaletteEntry model = (PaletteEntry )getModel ();
		model.addPropertyChangeListener (this);
	}



	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate() {
		PaletteEntry model = (PaletteEntry )getModel ();
		model.removePropertyChangeListener(this);
		super.deactivate();
	}

	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 **/
	protected void refreshVisuals() {
		super.refreshVisuals ();
	}
}
