/*
 * Created on Jan 3, 2005
 *
 */
package AcmeStudio.ui.editors.diagram.palette.extension;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.requests.CreationFactory;
import acmeLib.elements.AcmeSystem;


/**
 * @author Bradley Schmerl
 * Created Jan 3, 2005
 */
public interface IPaletteExtender {
	/**
	 * Adds the tools and templates specific to a particular extension. These tools must be added to the parentGroup
	 * @param parentGroup The group to add the new palette tools to. This group is added as a section in the palette by AcmeStudio
	 */
	public void createPaletteChildren (PaletteGroup parentGroup, AcmeSystem targetSystem);
	
	/**
	 * Returns true if the extension is applicable for this particular system. If true is returned, AcmeStudio
	 * will call createPaletteChildren to add a new section to the palette
	 * @param system The Acme system being tested as a candidate for this extension
	 * @return Whether the system should ignore this extension for this system
	 */
	public boolean isApplicable (AcmeSystem system);
	
	/**
	 * Returns true if the tool from the palette can be dropped over the current edit part
	 * @param toDrop The object to drop
	 * @param over What the object will be dropped over if the tool is released
	 * @return
	 */
	public boolean canDrop (Object toDrop, EditPart over);
	
	/**
	 * Returns the creation factory that the palette uses to create the object
	 * @param toDrop The object to create the factory for
	 * @return
	 */
	public CreationFactory getCreationFactory (Object toDrop);
}
