/*
 * Created on Dec 15, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.palette.extension;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.internal.ui.palette.editparts.PaletteEditPart;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.Display;
import com.swtdesigner.SWTResourceManager;
import AcmeStudio.util.Constants;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.Utils;


/**
 * @author Bradley Schmerl
 * Created Dec 15, 2004
 */
public class PaletteTabEditPart extends PaletteEditPart {
	static org.eclipse.swt.graphics.Font plainFont ;
	static org.eclipse.swt.graphics.Font boldFont ;
	{
		org.eclipse.swt.graphics.Font f = Display.getCurrent().getSystemFont();
		int size = 6;
		   if (SWT.getPlatform().equalsIgnoreCase("carbon")) {
		   	size = 9;
		   }
		   if (SWT.getPlatform ().equalsIgnoreCase("gtk")) {
		   	size = 8;
		   }
		plainFont = SWTResourceManager.getFont ( f.getFontData() [0].getName(), size, 0);
		boldFont = SWTResourceManager.getBoldFont(plainFont);
		
		
	}
	private static final Dimension EMPTY_DIMENSION = new Dimension (0,0);
	
	/**
	 * Called when the active tab is changed to synch up the display
	 */
	private PropertyChangeListener tabChangedListener = new PropertyChangeListener () {
		public void propertyChange (PropertyChangeEvent event) {
			if (event.getPropertyName ().equals (PaletteTab.PROPERTY_ACTIVE_TAB))
				activeTabChanged (event.getOldValue (), event.getNewValue ());
		}
	};

	/**
	 * A class that draws a label with the text rotated 90 degrees. 
	 * Uses Draw2D to draw the rotated text
	 */
	public class TabFigure extends Label {
//		final Graphics2DRenderer renderer = new Graphics2DRenderer ();
		
		final static int MINHEIGHT = 20;
		
		Image boldImage = null;
		Image plainImage = null;
		
		
		

		public TabFigure (String label) {
//			super ("");
//			createImage (label);
			super (" " + label + " ");
		}
		
		
		/**
		 * Because the text is rotated, the width and height are to be exchanged
		 * @param wHint
		 * @param hHint
		 * @return
		 * @see org.eclipse.draw2d.Label#getPreferredSize(int, int)
		 **/
		public Dimension getPreferredSize(int wHint, int hHint) {
			if (prefSize == null) {
				prefSize = calculateLabelSize (getTextSize ());
				Insets insets = getInsets ();
				prefSize.expand (insets.getWidth (), insets.getHeight ());
				if (getLayoutManager() != null) prefSize.union (getLayoutManager().getPreferredSize(this, wHint, hHint));
				int w = prefSize.width;
				prefSize.width = prefSize.height;
				prefSize.height = w;
			}
			if (wHint >= 0 && wHint < prefSize.width) {
				Dimension minSize = getMinimumSize (wHint, hHint);
				Dimension result = prefSize.getCopy ();
				result.width = Math.min (result.width, wHint);
				result.width = Math.max (minSize.width, result.width);
				return result;
			}
			return prefSize;
		}
		
		
		/**
		 * Because the text is rotated, the width and height are to be exchanged
		 * @param w
		 * @param h
		 * @return
		 * @see org.eclipse.draw2d.Label#getMinimumSize(int, int)
		 **/
		public Dimension getMinimumSize(int w, int h) {
			if (minSize != null) return minSize;
			minSize = new Dimension ();
			Dimension labelSize = calculateLabelSize(FigureUtilities.getTextExtents ("...", getFont ()).intersect (FigureUtilities.getTextExtents (getText (), getFont ())));
			labelSize.width += 30;
			Insets insets = getInsets ();
			labelSize.expand (insets.getWidth (), insets.getHeight ());
			minSize.union (labelSize);
			minSize.height = Math.max (minSize.height, MINHEIGHT);
			int width = minSize.width;
			minSize.width = minSize.height;
			minSize.height = width;
			return minSize;
		}
		
		
		/**
		 * @return the new bounds
		 * @see org.eclipse.draw2d.Figure#getBounds()
		 **/
		public Rectangle getBounds() {
			Rectangle origBounds = super.getBounds ();
			Clickable parent = (Clickable )getParent ();
			if (parent != null && parent.isSelected ()) {
				return new Rectangle (origBounds.x, origBounds.y, origBounds.width, origBounds.height + 26 - (origBounds.height - 26)/3);
			}
			return origBounds;
		}
		
		
		/**
		 * @param graphics
		 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
		 **/
		protected void paintFigure(Graphics graphics) {
			Color t = Constants.DEFAULT_ERROR_BACKGROUND;
			Clickable parent = (Clickable )getParent();
			paintSelected (graphics, parent.isSelected ());
		}
		
		
		/**
		 * Paints the curved tab border around the figure
		 * @param graphics
		 * @see org.eclipse.draw2d.Figure#paintBorder(org.eclipse.draw2d.Graphics)
		 **/
		protected void paintBorder(Graphics graphics) {
			Clickable parent = (Clickable )getParent();
			graphics.pushState ();
			graphics.setForegroundColor(ColorConstants.buttonDarker);
			if (parent.isSelected ()) {
				int d = getSize ().height;
				int curveWidth = 26 + d;
				int curveIndent = curveWidth / 3;
				int [] right = new int [] {0,0,1,0,1,2,2,3,2,5,3,6,3,7,5,9,5,10,6,11,6+d,11+d,7+d,12+d,7+d,13+d,9+d,15+d,9+d,16+d,10+d,17+d,10+d,19+d,11+d,20+d,11+d,22+d,12+d, 23+d};
				int [] left = new int [] {6,0,5,1,4,1,1,4,1,5,0,6};
				
				int [] shape = new int [4 + left.length + right.length + 4];
				int index = 0;
				int x = getLocation ().x;
				int y = getLocation ().y;
				shape[index++] = x+1;
				shape[index++] = y ;
				shape[index++] = x+1;
				shape[index++] = y;
				for (int i = 0; i < left.length/2; i++) {
					shape [index++] = x + getSize ().width - 1 - left [2*i];
					shape [index++] = y + left[2*i+1];
				}
				for (int i = 0; i < right.length/2; i++) {
					shape[index++] = x + getSize ().width - 1 - right [2*i];
					shape[index++] = y + getSize ().height -30+ /*curveIndent +*/ right [2*i+1];
				}
				shape[index++] = x ;
				shape[index++] = y+ getSize ().height + curveWidth - curveIndent;
				shape[index++] = x ;
				shape[index++] = y + getSize ().height + curveWidth - curveIndent;
				graphics.drawPolyline(shape);
			}
			else {
//				graphics.drawRectangle(getBounds ().x, getBounds ().y, getBounds ().width - 1, getBounds ().height - 1);;
				int [] left = new int [] {6,0,5,1,4,1,1,4,1,5,0,6};
				int shape [] = new int [4 + left.length * 2 + 4];
				int index = 0;
				int x = getLocation ().x;
				int y = getLocation ().y;
				shape[index++] = x + 1;
				shape[index++] = y;
				shape[index++] = x + 1;
				shape[index++] = y;
				for (int i = 0; i < left.length/2; i++) {
					shape [index++] = x + getSize ().width - 1 - left [2*i];
					shape[index++] = y + left [2*i+1];
				}
				for (int i = left.length/2-1; i >=0; i--) {
					shape [index++] = x + getSize ().width - 1 - left [2*i];
					shape[index++] = y + getSize ().height - 1 - left[2*i+1];
				}
				shape [index++] = x + 1;
				shape [index++] = y + getSize ().height - 1;
				shape [index++] = x + 1;
				shape [index++] = y + getSize ().height - 1;
				graphics.drawPolyline (shape);
			}
			graphics.popState ();
		}
		
		/**
		 * Paint the text of the label so that it is rotated 90 degrees
		 * @param graphics
		 * @param selected
		 */
		void paintSelected (Graphics graphics, boolean selected) {
			graphics.pushState ();
			if (selected) {
				graphics.setFont(boldFont);
				graphics.setForegroundColor(ColorConstants.buttonDarkest);
			}
			else {
				graphics.setFont(plainFont);
				graphics.setForegroundColor(ColorConstants.buttonDarker);
			}
			Image imageToPaint = null;
			
			String key = getText () + "_pt_";
			
			if (selected) {
				key += "bold";
				if (boldImage == null) {
					boldImage = ImageManager.getExistingImage(key);
					if (boldImage == null) {
						Image i = newRotatedImage (graphics, key);
						boldImage = i;
					}
				}
				imageToPaint = boldImage;
			}
			else {
				if (plainImage == null) {
					plainImage = ImageManager.getExistingImage (key);
					if (plainImage == null) {
						plainImage = newRotatedImage (graphics, key);
					}
				}
				imageToPaint = plainImage;
			}
			
			graphics.drawImage(imageToPaint, getLocation ().x, getLocation ().y);
			graphics.popState ();
		}


		/**
		 * @param graphics
		 * @param key
		 * @return
		 */
		private Image newRotatedImage(Graphics graphics, String key) {
			Dimension d = FigureUtilities.getTextExtents(getText (), boldFont);
			Image unrotatedImage = new Image (null, d.width, d.height);
			GC gc = new GC (unrotatedImage);
			gc.setFont(graphics.getFont());
			gc.setForeground(graphics.getForegroundColor());
			gc.setBackground(ColorConstants.menuBackground);
			gc.fillRectangle (unrotatedImage.getBounds ());
			gc.drawString(getText (), 0, 0);
			gc.dispose ();
			ImageData rotatedData = Utils.rotateImage (unrotatedImage.getImageData(), SWT.RIGHT);
			rotatedData.transparentPixel = rotatedData.palette.getPixel (ColorConstants.menuBackground.getRGB());
			Image i = ImageManager.createImage(key, rotatedData, false);
			unrotatedImage.dispose ();
			return i;
		}
	}
	
	private Figure	contentsFigure;

	private Figure	buttonBar;

	private IFigure	activeFigure;

	private Object	activeEntry;

	private FigureListener	sizeChangeListener;
	
	
	/**
	 * Creates the edit part
	 */
	public PaletteTabEditPart(PaletteEntry model) {
		super (model);
	}
	
	
	/**
	 * When activated, make sure that the active tab is displayed and that
	 * the part responds to changes in the active figure.
	 * 
	 * @see org.eclipse.gef.internal.ui.palette.editparts.PaletteEditPart#activate()
	 **/
	public void activate() {
		checkActiveTabSync ();
		PaletteTab model = (PaletteTab )getModel ();
		model.addPropertyChangeListener(tabChangedListener);
		super.activate ();
	}
	
	/**
	 * Check that the activeTab's figure is displayed
	 *
	 */
	private void checkActiveTabSync () {
		PaletteTab tab = (PaletteTab )getModel ();
		Clickable activeButton = null;
		if (activeEntry == null &&!tab.getChildren ().isEmpty ()) {
			activeEntry = tab.getChildren ().get (0);
			activeTabChanged (null, activeEntry);
			activeButton = (Clickable )buttonBar.getChildren ().get (0);
			activeButton.setSelected(true);
		}
		else if (activeEntry != null) {
			activeTabChanged (null, activeEntry);
			 activeButton = (Clickable )buttonBar.getChildren ().get (
					tab.getChildren ().indexOf (activeEntry));
			if (activeButton != null) {
				activeButton.setSelected (true);
			}
		}
		for (Iterator iter = buttonBar.getChildren ().iterator (); iter.hasNext ();) {
			Clickable button = (Clickable )iter.next ();
			if (button != activeButton) button.setSelected (false);
			buttonBar.getLayoutManager().invalidate();
			buttonBar.getLayoutManager().layout(button);
		}
	}
	
	/**
	 * Makes the active entry's tab figure visible
	 * @param oldValue
	 * @param newValue
	 */
	private void activeTabChanged (Object oldValue, Object newValue) {
		GraphicalEditPart part = null;
		IFigure tabContentFigure = null;
		if (newValue != null) {
			part = (GraphicalEditPart )getViewer ().getEditPartRegistry().get (newValue);
			tabContentFigure = part.getFigure ();
			tabContentFigure.setVisible(true);
			activeFigure = tabContentFigure;
			activeEntry = newValue;
		}
		
		if (oldValue != null) {
			part = (GraphicalEditPart )getViewer ().getEditPartRegistry().get (oldValue);
			if (part != null) {
				part.getFigure ().setVisible (false);
			}
		}
	}
	/**
	 * @return a new figure corrsponding to the Tab. 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 **/
	protected IFigure createFigure() {
		Figure topFigure = new Figure () {
			public Dimension getPreferredSize(int wHint, int hHint) {
				if (PaletteTabEditPart.this.getChildren().isEmpty ()) return EMPTY_DIMENSION;
				return super.getPreferredSize (wHint, hHint);
			}
			
			
			/**
			 * 
			 * @see org.eclipse.draw2d.Figure#invalidate()
			 **/
			public void invalidate() {
				super.invalidate ();
			}
		};
		topFigure.setLayoutManager (new BorderLayout () {
			protected Dimension calculatePreferredSize(IFigure container, int wHint, int hHint) {
				Dimension sPref = super.calculatePreferredSize (container, wHint, hHint);
				if (container.getParent ().getSize().contains(sPref)) {
					return sPref;
				}
				else {
					Viewport vp = getViewport (container);
					Rectangle ca = vp.getClientArea (); // The original area is bigger here, so it is used...
					Dimension d = ca.getSize ().getCopy();
					d.width += 2* buttonBar.getSize().width;
					return d;
				}
//				return sPref;
			}
			
			
		});
		contentsFigure = new Figure () {
			
			/**
			 * 
			 * @see org.eclipse.draw2d.Figure#layout()
			 **/
			protected void layout() {
				for (Iterator iter = getChildren ().iterator (); iter.hasNext ();) {
					Figure fig = (Figure )iter.next ();
					fig.setBounds(this.getClientArea());
				}
				super.layout ();
			}
		};
		StackLayout stackLayout = new StackLayout () {
			protected Dimension calculatePreferredSize(IFigure figure, int wHint, int hHint) {
				return super.calculatePreferredSize (figure, wHint, hHint);
			}
			
		};
		stackLayout.setObserveVisibility (true);
		contentsFigure.setLayoutManager (stackLayout);
		topFigure.add (contentsFigure, BorderLayout.CENTER);
		buttonBar = new Figure ();
		buttonBar.setBorder(new AbstractBorder () {

			public Insets getInsets(IFigure figure) {
				return new Insets (0);
			}

			public void paint(IFigure figure, Graphics graphics, Insets insets) {
				Rectangle rect = getPaintRectangle(figure, insets);
				graphics.drawLine(rect.x, rect.y, rect.x, rect.y + rect.height);
			}
			
		});
		buttonBar.setLayoutManager(new FlowLayout (FlowLayout.VERTICAL));
		
		PaletteTab tab = (PaletteTab )getModel ();
		for (Iterator iter = tab.getChildren ().iterator (); iter.hasNext ();) {
			final PaletteEntry entry = (PaletteEntry )iter.next ();
			TabFigure tabLabel = new TabFigure (entry.getLabel ());
//			tabLabel.setBorder (null);
//			Label tabLabel = new Label (entry.getLabel());
			Clickable tabButton = new Clickable (tabLabel, Clickable.STYLE_TOGGLE) {
				
				/**
				 * @param graphics
				 * @see org.eclipse.draw2d.Clickable#paintBorder(org.eclipse.draw2d.Graphics)
				 **/
				protected void paintBorder(Graphics graphics) {
				}
				
				/**
				 * @return
				 * @see org.eclipse.draw2d.Figure#getBounds()
				 **/
				public Rectangle getBounds() {
					return ((IFigure )getChildren ().get (0)).getBounds();
				}
			};
//			tabButton.setBorder(null);
			tabButton.addActionListener(new ActionListener () {
				public void actionPerformed(ActionEvent event) {
					if (activeEntry != entry) 
						activeTabChanged (activeEntry, entry);
					for (Iterator i = buttonBar.getChildren ().iterator (); i.hasNext ();) {
						Clickable c = (Clickable )i.next ();
						if (c != event.getSource()) c.setSelected(false);
						else c.setSelected (true);
						buttonBar.getLayoutManager().invalidate();
						buttonBar.getLayoutManager().layout(c);
					}
						
				}			
			});
			buttonBar.add (tabButton);
		}
		
		topFigure.add (buttonBar, BorderLayout.RIGHT);
		
		
		
		return topFigure;
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#addNotify()
	 **/
	public void addNotify() {
		super.addNotify ();
		Viewport vp = getViewport (figure);
		if (vp != null) {
			 
			sizeChangeListener = new FigureListener () {

				public void figureMoved(IFigure source) {
					getFigure ().invalidate ();
				}
				
			};
			vp.addFigureListener (sizeChangeListener);

		}
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#removeNotify()
	 **/
	public void removeNotify() {
		super.removeNotify ();
		Viewport vp = getViewport (getFigure ());
		if (vp != null) {
			vp.removeFigureListener(sizeChangeListener);
		}
	}
	
	
	Viewport getViewport (IFigure container) {
		if (container instanceof Viewport) return (Viewport )container;
		else return (getViewport (container.getParent ()));
	}
	
	/**
	 * 
	 * @see org.eclipse.gef.internal.ui.palette.editparts.PaletteEditPart#deactivate()
	 **/
	public void deactivate() {
		((PaletteEntry )getModel ()).removePropertyChangeListener(tabChangedListener);
		super.deactivate ();
	}
	
	
	/**
	 * @return
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getContentPane()
	 **/
	public IFigure getContentPane() {
		return contentsFigure;
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshChildren()
	 **/
	protected void refreshChildren() {
		super.refreshChildren ();
		for (Iterator iter = getChildren ().iterator (); iter.hasNext ();) {
			GraphicalEditPart editPart = (GraphicalEditPart )iter.next ();
			if (!editPart.getFigure ().equals (activeFigure))
				editPart.getFigure ().setVisible (false);
		}
	}
	
}
