/*
 * Created on Jun 15, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.palette.extension;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;

import org.eclipse.draw2d.ActionEvent;
import org.eclipse.draw2d.ActionListener;
import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.ButtonBorder;
import org.eclipse.draw2d.ButtonModel;
import org.eclipse.draw2d.ChangeEvent;
import org.eclipse.draw2d.ChangeListener;
import org.eclipse.draw2d.Clickable;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteListener;
import org.eclipse.gef.palette.PaletteStack;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.gef.ui.palette.PaletteViewerPreferences;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.MenuListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;


/**
 * The EditPart for a PaletteTree
 * 
 * @author Bradley Schmerl
 * Created Jun 15, 2004
 */
public class PaletteTreeEditPart extends org.eclipse.gef.internal.ui.palette.editparts.PaletteEditPart {

	class TreeMenuListener implements MenuListener {

		private Menu menu;
		private Display d;
		
		/**
		 * Creates a new listener to listen to the menu that is used to select the active tool
		 * on a stack. Disposes the stack with an asyncExec after hidden is called
		 * @param menu
		 * @param d
		 */
		TreeMenuListener (Menu menu, Display d) {
			this.menu = menu;
			this.d = d;
		}
		/**
		 * @param e
		 * @see org.eclipse.swt.events.MenuListener#menuHidden(org.eclipse.swt.events.MenuEvent)
		 **/
		public void menuHidden(MenuEvent e) {
			d.asyncExec(new Runnable () {

				public void run() {
					if (menu != null) {
						if (!menu.isDisposed()) {
							menu.dispose ();
						}
						menu = null;
					}
				}
				
			});
		}

		/**
		 * @param e
		 * @see org.eclipse.swt.events.MenuListener#menuShown(org.eclipse.swt.events.MenuEvent)
		 **/
		public void menuShown(MenuEvent e) {
		}
		
	};
	
	class RolloverArrow extends Clickable {
		/**
		 * Creates a new Clickable with a TriangleFigure as its child.
		 */
		RolloverArrow () {
			super (new TriangleFigure ());
			setRolloverEnabled (true);
			setBorder (new ButtonBorder (ButtonBorder.SCHEMES.TOOLBAR));
			setOpaque (false);
			setStyle (Clickable.STYLE_BUTTON);
		}
		/**
		 * Draws a checkered pattern to emulate a toggle button that is in the selected state
		 * @param graphics The graphics object used to paint
		 */
		protected void fillCheckeredRectangle (Graphics graphics) {
			// Method taken from ToggleButton - because this figure *isn't* a toggle button,
			// but should match the checkered look of the tool's toggle button
			graphics.setBackgroundColor(ColorConstants.button);
			graphics.setForegroundColor(ColorConstants.buttonLightest);
			Rectangle rect = getClientArea (Rectangle.SINGLETON).crop (new Insets (1, 1, 0, 0));
			graphics.fillRectangle (rect);
			graphics.clipRect (rect);
			graphics.translate (rect.x, rect.y);
			int n = rect.width + rect.height;
			for (int i = 1; i < n; i += 2) {
				graphics.drawLine (0, i, i, 0);
			}
			graphics.restoreState ();
		}
		
		/**
		 * Returns fals so that the focus rectangle is not drawn
		 * @see org.eclipse.draw2d.IFigure#hasFocus
		 */
		public boolean hasFocus () {
			return false;
		}
		
		/**
		 * 
		 * @param graphics
		 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
		 *
		 */
		protected void paintFigure (Graphics graphics) {
			if (isSelected () && isOpaque ()) {
				fillCheckeredRectangle(graphics);
			}
			else 
				super.paintFigure (graphics);
		}
		
	};
	
	/**
	 * Represents the right triangle for choosing the from a menu of options
	 */
	class TriangleFigure extends Figure {
		
		/**
		 * Creates a new TriangleFigure with preferred size -1, 7
		 */
		TriangleFigure () {
			super ();
			setPreferredSize (7, -1);
		}
		
		public void paint (Graphics graphics) {
			Rectangle rect = getBounds ().getCopy ();
			graphics.translate (getLocation ());
			graphics.setBackgroundColor (ColorConstants.listForeground);
			
			// fill the arrow
			int [] points = new int [8];
			points [0] = rect.width /2;
			points [1] = rect.height/2 - 3;
			points [2] = rect.width - 1;
			points [3] = rect.height/2;
			points [4] =  rect.width/2;
			points [5] = rect.height/2 + 3;
			points [6] = rect.width /2;
			points [7] = rect.height /2 - 3;
			
			graphics.fillPolygon (points);
			graphics.translate (getLocation ().getNegated());
		}
	};
	
	private static final Dimension EMPTY_DIMENSION = new Dimension(0, 0);

//	 listen to changes in palette layout.
	private PropertyChangeListener paletteLayoutListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent event) {
			if (event.getPropertyName().equals(PaletteViewerPreferences.PREFERENCE_LAYOUT)) {
				int newLayoutMode = ((Integer)event.getNewValue()).intValue();
				if (newLayoutMode == PaletteViewerPreferences.LAYOUT_LIST
						|| newLayoutMode == PaletteViewerPreferences.LAYOUT_DETAILS) {
					if (!getFigure().getChildren().contains(arrowFigure))
						getFigure().add(arrowFigure, BorderLayout.RIGHT);
				} else {
					if (getFigure().getChildren().contains(arrowFigure))
						getFigure().remove(arrowFigure);
				}
			}
		}
	};

//	 listen to changes in stack
	private PropertyChangeListener treeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent event) {
			if (event.getPropertyName().equals(PaletteStack.PROPERTY_ACTIVE_ENTRY))
				activeEntryChanged(event.getOldValue(), event.getNewValue());
		}
	};

//	 listen to changes of clickable tool figure
	private ChangeListener clickableListener = new ChangeListener() {
		public void handleStateChanged(ChangeEvent event) {
			if (event.getPropertyName().equals(ButtonModel.MOUSEOVER_PROPERTY))
				arrowFigure.getModel().setMouseOver(activeFigure.getModel().isMouseOver());
			else if (event.getPropertyName().equals(ButtonModel.ARMED_PROPERTY))
				arrowFigure.getModel().setArmed(activeFigure.getModel().isArmed());
		}
	};

//	 listen to changes of arrow figure
	private ChangeListener clickableArrowListener = new ChangeListener() {
		public void handleStateChanged(ChangeEvent event) {
			if (event.getPropertyName().equals(ButtonModel.MOUSEOVER_PROPERTY))
				activeFigure.getModel().setMouseOver(arrowFigure.getModel().isMouseOver());
			if (event.getPropertyName().equals(ButtonModel.ARMED_PROPERTY))
				activeFigure.getModel().setArmed(arrowFigure.getModel().isArmed());
		}
	};

//	 listen to see if arrow is pressed
	private ActionListener actionListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			openMenu();		
		}
	};

//	 listen to see if active tool is changed in palette
	private PaletteListener paletteListener = new PaletteListener() {
		public void activeToolChanged(PaletteViewer palette, ToolEntry tool) {
			if (getTree().treeContainsEntry(tool)) {
				if (!arrowFigure.getModel().isSelected())
					arrowFigure.getModel().setSelected(true);
				if (!getTree().getActiveEntry().equals(tool)) {
					// Find the root of the tree, and setActiveEntry on it (which cascades down)
					PaletteTreeEditPart root = PaletteTreeEditPart.this;
					while (root.getParent () instanceof PaletteTreeEditPart) 
						root = (PaletteTreeEditPart )root.getParent ();
					root.getTree().setActiveEntry(tool);
				}
			} else
				arrowFigure.getModel().setSelected(false);
		}	
	};

	private Clickable activeFigure;
	private RolloverArrow arrowFigure;
	private Figure contentsFigure;
	private Menu menu;

	public PaletteViewer getPaletteViewer () {
		return (PaletteViewer )getViewer ();
	}
	
	/**
	 * Creates a new PaletteTreeEditPart with the given PaletteTree as its model
	 * @param model The PaletteTree to associate with this EditPart
	 */
	public PaletteTreeEditPart(PaletteEntry model) {
		super (model);
		model.addPropertyChangeListener(treeListener);
	}

	/**
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate() {
		// in case the model is out of sync
		checkActiveEntrySync();
		getPaletteViewer().addPaletteListener(paletteListener);
		getPaletteViewer().getPaletteViewerPreferences().addPropertyChangeListener(paletteLayoutListener);
		super.activate();
	}

	/**
	 * Called when the active entry has changed.
	 * 
	 * @param oldValue the old model value (can be null)
	 * @param newValue the new model value (can be null)
	 */
	private void activeEntryChanged(Object oldValue, Object newValue) {
		GraphicalEditPart part = null;
		Clickable clickable = null;

		if (newValue != null) {
			part = (GraphicalEditPart)getViewer().getEditPartRegistry().get(newValue);
			clickable = (Clickable)part.getFigure();
			clickable.setVisible(true);
			clickable.addChangeListener(clickableListener);
			activeFigure = clickable;
		}

		if (oldValue != null) {
			part = (GraphicalEditPart)getViewer().getEditPartRegistry().get(oldValue);
			// if part is null, its no longer a child.
			if (part != null) {
				clickable = (Clickable)part.getFigure();
				clickable.setVisible(false);
				clickable.removeChangeListener(clickableListener);
			}
		}
	}

	private void checkActiveEntrySync() {
		if (activeFigure == null)
			activeEntryChanged(null, getTree().getActiveEntry());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	public IFigure createFigure() {
		Figure figure = new Figure() {
			public Dimension getPreferredSize(int wHint, int hHint) {
				if (PaletteTreeEditPart.this.getChildren().isEmpty())
					return EMPTY_DIMENSION;
				return super.getPreferredSize(wHint, hHint);
			}
			
			public void paintBorder(Graphics graphics) {
				int layoutMode = getPreferenceSource().getLayoutSetting();
				if (layoutMode == PaletteViewerPreferences.LAYOUT_LIST
				  || layoutMode == PaletteViewerPreferences.LAYOUT_DETAILS)
					return;
				
				Rectangle rect = getBounds().getCopy();

				graphics.translate(getLocation());
				graphics.setBackgroundColor(ColorConstants.listForeground);
				
				// fill the corner arrow
				int[] points = new int[6];
				
				points[0] = rect.width ;
				points[1] = rect.height - 5;
				points[2] = rect.width ;
				points[3] = rect.height;
				points[4] = rect.width - 5;
				points[5] = rect.height;
			
				graphics.fillPolygon(points);
				
				graphics.translate(getLocation().getNegated());
			}
			
			IFigure buildTooltip() {
				return createToolTip();
			}
		};
		figure.setLayoutManager(new BorderLayout());
		
		contentsFigure = new Figure();
		
		StackLayout stackLayout = new StackLayout();
		// make it so the stack layout does not allow the invisible figures to contribute
		// to its bounds
		stackLayout.setObserveVisibility(true);
		contentsFigure.setLayoutManager(stackLayout);
		
		figure.add(contentsFigure, BorderLayout.CENTER);
		
		arrowFigure = new RolloverArrow();
		
		arrowFigure.addChangeListener(clickableArrowListener);
		
		arrowFigure.addActionListener(actionListener);

//		int layoutMode = getPreferenceSource().getLayoutSetting();
//		if (layoutMode == PaletteViewerPreferences.LAYOUT_LIST
//		  || layoutMode == PaletteViewerPreferences.LAYOUT_DETAILS)
			figure.add(arrowFigure, BorderLayout.RIGHT);
		
		return figure;
	}

	/**
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate() {
		if (activeFigure != null) 
			activeFigure.removeChangeListener(clickableListener);
		arrowFigure.removeActionListener(actionListener);
		arrowFigure.removeChangeListener(clickableArrowListener);
		getTree().removePropertyChangeListener(treeListener);
		getPaletteViewer().removePaletteListener(paletteListener);
		getPaletteViewer().getPaletteViewerPreferences().removePropertyChangeListener(paletteLayoutListener);
		
		super.deactivate();
	}

	/**
	 * @see org.eclipse.gef.EditPart#eraseTargetFeedback(org.eclipse.gef.Request)
	 */
	public void eraseTargetFeedback(Request request) {	
		Iterator children = getChildren().iterator();
		
		while (children.hasNext()) {
			GraphicalEditPart part = (GraphicalEditPart )children.next();
			part.eraseTargetFeedback(request);
		}
		super.eraseTargetFeedback(request);
	}

	/**
	 * @see org.eclipse.gef.GraphicalEditPart#getContentPane()
	 */
	public IFigure getContentPane() {
		return contentsFigure;
	}

	private PaletteTree getTree () {
		return (PaletteTree )getModel ();
	}
	
	/**
	 * Opens the menu to display the choices for the active entry
	 *
	 */
	public void openMenu () {
		MenuManager menuManager = getTree ().getSubMenu (getPaletteViewer ());
		
		menu = menuManager.createContextMenu(getPaletteViewer ().getControl ());
		Rectangle figureBounds = getFigure ().getBounds ().getCopy ();
		getFigure ().translateToAbsolute(figureBounds);
		
		Point menuLocation = getPaletteViewer ().getControl ().toDisplay (figureBounds.getBottomLeft ().x, figureBounds.getBottomLeft ().y);
		
		// Remove feedback from the arrow Figure and children figures
		arrowFigure.getModel ().setMouseOver (false);
		eraseTargetFeedback(new Request (RequestConstants.REQ_SELECTION));
		menu.setLocation (menuLocation);
		menu.addMenuListener (new TreeMenuListener (menu, getViewer ().getControl ().getDisplay ()));
		menu.setVisible (true);
	}


	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshChildren()
	 */
	protected void refreshChildren() {
		super.refreshChildren();

		Iterator children = getChildren().iterator();
		
		while (children.hasNext()) {
			Object ep = children.next ();
			
			GraphicalEditPart editPart = (GraphicalEditPart )ep;
			if (!editPart.getFigure().equals(activeFigure))
				editPart.getFigure().setVisible(false);
		}

	}

	
	/**
	 * @see org.eclipse.gef.EditPart#showTargetFeedback(org.eclipse.gef.Request)
	 */
	public void showTargetFeedback(Request request) {
		// if menu is showing, don't show feedback. this is a fix
		// for the occasion when show is called after forced erase
		if (menu != null && !menu.isDisposed() && menu.isVisible())
			return;
			
		Iterator children = getChildren().iterator();
		while (children.hasNext()) {
			GraphicalEditPart part = (GraphicalEditPart)children.next();
			part.showTargetFeedback(request);
		}
		
		super.showTargetFeedback(request);
	}

	
	/**
	 *  The tree structure is stored in the PaletteTree model. We need to add the actual edit parts to
	 * the root edit part.
	 * @param child
	 * @param index
	 * @see org.eclipse.gef.editparts.AbstractEditPart#addChild(org.eclipse.gef.EditPart, int)
	 **/
	protected void addChild(EditPart child, int index) {
		if (getParent() instanceof PaletteTreeEditPart) {
			((PaletteTreeEditPart )getParent ()).addChild (child, -1);
		}
		else {
			super.addChild (child, index);
		}
	}
	
}
