package AcmeStudio.ui.editors.diagram.parts;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.AccessibleEditPart;
import org.eclipse.swt.accessibility.AccessibleEvent;

import AcmeStudio.ui.editors.diagram.figures.PortView;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.PortVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.ui.editors.diagram.styles.constants.Orientation;
import acmeLib.elements.AcmeAttachment;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;

/**
 * Holds a component
 */
public class AttachmentEditPart extends ElementEditPart {

    protected AccessibleEditPart createAccessible() {
        return new AccessibleGraphicalEditPart() {
            public void getName(AccessibleEvent e) {
                e.result = ((AcmeAttachment) getModel()).getQualifiedName();
            }
        };
    }

    public void activate() {
        if (getModel() == null)
            return;
    }
    /**
     * Creates a new Port Figure and returns it.
     *
     * @return  Figure representing the port.
     */
    protected IFigure createFigure() {
        return new Figure();
    }

    protected void refreshVisuals() {
        if (getModel() == null)
            return;
        AcmeAttachment attachment = (AcmeAttachment) getModel();
        AcmePort port = attachment.getPort();
        AcmeRole role = attachment.getRole();

        RoleEditPart roleEditPart =
            (RoleEditPart) getRoot().getViewer().getEditPartRegistry().get(
                role);
        if (roleEditPart == null)
            return;
        RoleView roleView = (RoleView) roleEditPart.getFigure();
        if (roleView == null)
            return;

        PortEditPart portEditPart =
            (PortEditPart) getRoot().getViewer().getEditPartRegistry().get(
                port);
        if (portEditPart == null)
            return;
        PortView portView = (PortView) portEditPart.getFigure();
        if (portView == null)
            return;
        //        Visualization visStyle = portView.getVisualization();
        //        visStyle.setFillColor(ElementColorConstants.attachmentFillColor);
        //        visStyle.setOutlineColor(ElementColorConstants.attachmentOutlineColor);

        Rectangle portBounds = portView.getBounds();
        Point endPoint = portBounds.getCenter();
        roleView._isMoving = true;
        roleView.setEnd(endPoint);
        SystemView.setLocation(role, endPoint);
        roleEditPart.refreshVisuals();
        //portEditPart.refreshVisuals();
        ElementVisualization vis = VisualStyleRegistry.getVisualization(port);
//        int direction = vis.getDirection();
//        if (direction == PositionConstants.SOUTH || direction == PositionConstants.WEST) {
//            angle += Math.PI;
//        }

        
        if (vis instanceof PortVisualization) {
            boolean alignPortWithAttachedRole = ((PortVisualization)vis).alignPort();
            int orientation = ((PortVisualization)vis).getOrientation();
            if (alignPortWithAttachedRole) {
                Dimension diff = roleView.getEnd().getDifference(roleView.getLastBendpoint());
                double angle = Math.atan2(diff.height, diff.width);
                if (orientation == Orientation.OUT)
                    angle += Math.PI;
                portView.setRotation(angle);       
            }
        }
    }

    /**
     * Returns the children of this through the model.
     *
     * @return  Children of this as a List.
     */
    protected List getModelChildren() {
        Vector children = new Vector();
        return children;
    }

}
