package AcmeStudio.ui.editors.diagram.parts;

import java.util.Iterator;
import java.util.Map;
import java.util.Vector;
import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.FrameBorder;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.draw2d.parts.Thumbnail;
import org.eclipse.gef.*;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.prefs.DiagramEditorPreferences;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.ui.editors.diagram.actions.DrillDownAction;
import AcmeStudio.ui.editors.diagram.figures.*;
import AcmeStudio.ui.editors.diagram.policies.InlineRepresentationLayout;
import AcmeStudio.ui.editors.diagram.policies.ItemOutlineEditPolicy;
import AcmeStudio.ui.editors.diagram.styles.ComponentVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeSystem;
import acmeLib.properties.AcmeProperty;
import acmeLib.representations.AcmeRepresentation;
/**
 * @author Hong Yan (yh@cs.cmu.edu)
 * 
 * The diagram controller of AcmeComponent.
 *  
 */
public class ComponentEditPart extends MovableEditPart {

	/*
	 * LFW: Created to add 'red square highlight' behavior to ports
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies ();
		installEditPolicy (EditPolicy.SELECTION_FEEDBACK_ROLE, new ItemOutlineEditPolicy ());
	}

	/**
	 * Creates a new Component Figure and returns it.
	 * 
	 * @return Figure representing the component.
	 */
	protected IFigure createFigure() {
		if (getModel () == null) return null;
		AcmeVisElement f = (AcmeVisElement )ElementFigureFactory.createComponentView ();
		AcmeElement acmeElement = (AcmeElement )getModel ();
		f.setModel (acmeElement);
		f.setEditPart (this);
		f.setLayoutManager (new BorderLayout ());
		//        	FreeformViewport port = new FreeformViewport ();
		//        	ScalableFreeformLayeredPane pane = new ScalableFreeformLayeredPane
		// ();
		//        	pane.setScale (.25);
		//        	port.setContents (pane);
		//        	f.setLayoutManager(new BorderLayout ());
		//        	f.add (port);
		return f;
	}
	/**
	 * Returns the children of this through the model.
	 * 
	 * @return Children of this as a List.
	 */
	//    protected List getModelChildren() {
	//    	if (((AcmeComponent )getModel ()).getRepresentations().getSize() == 1) {
	//    		Vector children = new Vector();
	//    		AcmeRepresentation rep = (AcmeRepresentation )((AcmeComponent )getModel
	// ()).getRepresentations ().getNextItem();
	//			children.add(rep);
	//    		// children.addAll (Helper.getChildren ((AcmeComponent )getModel ()));
	//    		return children;
	//    	}
	//    	else {
	//    		return super.getModelChildren ();
	//    	}
	//    }
	boolean				_needPortsRefreshed	= true;
	private Map			_portLocationMap;
	private Thumbnail	_repThumbnail;

	public Point calculateChildPortLocation(AcmePort port) {
		updatePortPositions ();
		return (Point )_portLocationMap.get (port);
	}

	private void updatePortPositions() {
		if (_needPortsRefreshed) {
			AcmeComponent acmeComponent = (AcmeComponent )getModel ();
			SystemEditPart se = (SystemEditPart )getRoot ().getViewer ().getEditPartRegistry ().get ((acmeComponent).getParent ());
			ComponentView componentView = (ComponentView )getFigure ();
			//    		SystemView systemView = (SystemView )se.getFigure();
			Vector ports = Helper.getChildrenByOrder (acmeComponent);
			_portLocationMap = SystemView.normalizePortPositions (ports, acmeComponent);
			_needPortsRefreshed = false;
			refreshComponentSize ();
		}
	}

	/**
	 * Updates the visual aspect of this.
	 */
protected void refreshVisuals() {
//  	Debug.diagnoseln ("ComponentEditPart.refreshVisuals: " + "");
    	if (getModel() == null)
    		return;
    	
    	AcmeComponent	component	= (AcmeComponent )getModel ();
 
    	setModel(getModel());
    	
        if (component.getRepresentations().getSize () == 1 && AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(DiagramEditorPreferences.INLINE_REPRESENTATIONS)) {
        	ComponentView f = (ComponentView )getFigure ();
        	f.setLayoutManager(new InlineRepresentationLayout ());
      
        	if (!f.getChildren().contains(_repThumbnail)) {
	        	AcmeSystem sys = ((AcmeRepresentation )component.getRepresentations().getNextItem()).getSystem();
	        	DiagramEditor sysEd = Helper.getDiagramEditorForSystem (sys);
	        	if (sysEd != null) {
	        		sysEd.hideRepresentationBorders ();
	        	RootEditPart rep = sysEd.getGraphicalViewer().getRootEditPart();
	        	if (rep != null) {
//	        		FigureCanvas fc = new FigureCanvas ();
	        		ScalableFreeformRootEditPart root = ((ScalableFreeformRootEditPart )rep);
					 _repThumbnail = new Thumbnail (root.getFigure ());
	        		_repThumbnail.setSource (root.getLayer (LayerConstants.PRINTABLE_LAYERS));
	        		_repThumbnail.setVisible(true);
//	        		_repThumbnail.setBorder (new FrameBorder ());
	        		f.add (_repThumbnail, BorderLayout.CENTER);
	        	}
	        	}
        	}
        }
//  	if (_needPortsRefreshed) {
    	
    	
    	
    	
    	refreshPorts(component);
    	super.refreshVisuals();
//    	}
//  	Rectangle bounds = refreshComponentSize ();
    	Point location = SystemView.getLocation(component, false);
    	if (location != null) {
    		Dimension size = Helper.getSize (component);
    		if (size == null) {
    			size = ((ComponentView )getFigure ()).getVisualization().getSize();
    		}
    		Rectangle bounds = new Rectangle (location, size);
    		bounds.translate(-bounds.width/2, -bounds.height/2);
    		if (getParent () != null)
	    		((GraphicalEditPart) getParent()).setLayoutConstraint(
	    				this,
						getFigure(),
						bounds);
    	}
//  	if (_needPortsRefreshed) {
//  	_needPortsRefreshed = false;
//  	se.refreshAttachments();
//       	SystemEditPart se =
//    		(SystemEditPart) getRoot().getViewer().getEditPartRegistry().get(
//    				(component).getParent());
//    	se.getFigure().repaint();
//  	}
        
    }
	private void refreshPorts(AcmeComponent component) {
		//  	calculateAllPortPositions (bounds);
		boolean fullRefresh = false;
		if (_needPortsRefreshed) fullRefresh = true;
		Vector ports = Helper.getChildrenByOrder (component);
		for (Iterator iter = ports.iterator (); iter.hasNext ();) {
			AcmePort port = (AcmePort )iter.next ();
			PortEditPart pep = (PortEditPart )getRoot ().getViewer ().getEditPartRegistry ().get (port);
			if (pep == null) continue;
			if (fullRefresh)
				pep.refreshVisuals ();
			else
				pep.refreshAttachments ();
		}
	}

	private void calculateAllPortPositions(Rectangle bounds) {
		AcmeComponent acmeComponent = (AcmeComponent )getModel ();
		Vector ports = Helper.getChildrenByOrder (acmeComponent);
		for (int i = 0; i < ports.size (); i++) {
			Object obj = ports.elementAt (i);
			if (obj instanceof AcmePort) {
				AcmePort port = (AcmePort )obj;
				PortEditPart portEditPart = (PortEditPart )getRoot ().getViewer ().getEditPartRegistry ().get (port);
				if (portEditPart == null) continue;
				PortView portView = (PortView )portEditPart.getFigure ();
				// Parent view has been renewed -- need to chang the children's
				// view
				if (portView.getParent () != getFigure ().getParent ()) {
					portView.setParent (getFigure ().getParent ());
				}
				updatePortPositions ();
				Point portLocation = (Point )_portLocationMap.get (port);
				portView.setModel (port);
				Dimension portSize = Helper.getSize (port);
				if (portSize == null) {
					portSize = VisualStyleRegistry.getVisualization (port).getSize ();
				}
				portView.setRelativeLocation (portLocation);
				portLocation.x = portLocation.x + bounds.x + bounds.width / 2 - portSize.width / 2;
				portLocation.y = portLocation.y + bounds.y + bounds.height / 2 - portSize.height / 2;
				portView.setLocation (portLocation);
				Rectangle r = portView.getBounds ();
				((GraphicalEditPart )getParent ()).setLayoutConstraint (portEditPart, portView, r);
			}
		}
	}

	private Rectangle refreshComponentSize() {
		Vector ports = null;
		ComponentVisualization cv = null;
		Object vis = this.getVis ();
		if (vis instanceof ComponentVisualization) {
			cv = (ComponentVisualization )vis;
		}
		AcmeComponent acmeComponent = (AcmeComponent )getModel ();
		Rectangle bounds = getBounds ();
		if (cv != null && cv.resizeToFitPorts ()) {
			boolean growWider = false, growHigher = false;
			do {
				growWider = false;
				growHigher = false;
				if (_portLocationMap == null) continue;
				for (Iterator iter = _portLocationMap.entrySet ().iterator (); iter.hasNext () && !(growWider && growHigher);) {
					Point pos = (Point )((Map.Entry )iter.next ()).getValue ();
					if (!growWider && Math.abs (pos.x * 2) > bounds.width) growWider = true;
					if (!growHigher && Math.abs (pos.y * 2) > bounds.height) growHigher = true;
				}
				if (growWider || growHigher) {
					if (growWider) bounds.width += 10;
					if (growHigher) bounds.height += 10;
					Helper.setSize (acmeComponent, bounds.getSize ());
					if (ports == null) ports = Helper.getChildrenByOrder (acmeComponent);
					_portLocationMap = SystemView.normalizePortPositions (ports, acmeComponent);
				}
			}
			while (growWider || growHigher);
		}
		bounds.x = bounds.x - bounds.width / 2;
		bounds.y = bounds.y - bounds.height / 2;
		Rectangle r = new Rectangle (bounds.getLocation (), bounds.getSize ());
		return r;
	}

	/*
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	public void performRequest(Request req) {
		super.performRequest (req);
		Object reqType = req.getType ();
		if (RequestConstants.REQ_OPEN.equals (reqType)) {
			MultiPageAcmeEditor mp = ASContext.current ().getActiveEditor ();
			DrillDownAction action = new DrillDownAction (mp);
			action.setEditPart (this);
			action.run ();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
	 */
	public void changeModel(AcmeStudioChangeEvent evt) {
		if (evt.getTarget ().equals (getModel ())
				|| ((evt.getTarget () instanceof AcmePort) && (((AcmeComponent )getModel ()).getPort (((AcmePort )evt.getTarget ())
						.getName ()) == evt.getTarget ()))) {
			//    		if (Helper.eventRelatedToDesign(evt, ((AcmeElement )getModel
			// ()).getParentDesign())) {
			switch (evt.getEventType ()) {
				case AcmeStudioChangeManager.ADD_PORT :
				case AcmeStudioChangeManager.REMOVE_PORT :
				case AcmeStudioChangeManager.UPDATE_VIS_INFO :
					_needPortsRefreshed = true;
					break;
				case AcmeStudioChangeManager.SET_PROPERTY : {
					AcmeProperty prop = (AcmeProperty )evt.getSource ();
					if (prop.getName ().startsWith ("vis-")) {
						_needPortsRefreshed = true;
					}
				}
					break;
			}
			this.refresh ();
		}
		else {
			super.changeModel (evt);
		}
		//    	}
	}

	public boolean interestedInEvent(int et) {
		return et == AcmeStudioChangeManager.ADD_PORT || et == AcmeStudioChangeManager.REMOVE_PORT
				|| et == AcmeStudioChangeManager.SET_PROPERTY || et == AcmeStudioChangeManager.UPDATE_VIS_INFO;
	}

	/**
	 * @return
	 */
	public Rectangle getBounds() {
		ComponentView componentView = (ComponentView )getFigure ();
		SystemView systemView = (SystemView )componentView.getParent ();
		Point location = null;
		if (systemView != null)
			location = systemView.getElementLocation ((AcmeComponent )getModel ());
		else
			location = SystemView.getLocation ((AcmeComponent )getModel (), false);
		if (location == null) return null;
		AcmeComponent acmeComponent = (AcmeComponent )getModel ();
		Dimension size = Helper.getSize (acmeComponent);
		if (size == null) {
			size = componentView.getVisualization ().getSize ();
		}
		return new Rectangle (location, size);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see AcmeStudio.ui.editors.diagram.parts.ElementEditPart#resizeIfNecessary()
	 */
	protected boolean resizeIfNecessary() {
		if (super.resizeIfNecessary ()) {
			_needPortsRefreshed = true;
			refreshPorts ((AcmeComponent )getModel ());
			return true;
		}
		else {
			SystemView systemView = (SystemView )((AcmeVisElement )getFigure ()).getParent ();
			Point location = null;
			if (systemView != null)
				location = systemView.getElementLocation ((AcmeElement )getModel ());
			else
				location = SystemView.getLocation ((AcmeElement )getModel (), false);
			Dimension size = Helper.getSize ((AcmeElement )getModel ());
			if (getParent () != null)
				((GraphicalEditPart )getParent ()).setLayoutConstraint (this, getFigure (), new Rectangle (location, size));
		}
		return false;
	}
}
