package AcmeStudio.ui.editors.diagram.parts;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import AcmeStudio.core.ASContext;
import AcmeStudio.model.ModelHelper;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.ui.editors.diagram.actions.DrillDownAction;
import AcmeStudio.ui.editors.diagram.figures.*;
import AcmeStudio.ui.editors.diagram.policies.ItemOutlineEditPolicy;
import AcmeStudio.ui.editors.diagram.styles.ConnectorVisualization;
import AcmeStudio.util.Helper;
import acmeLib.elements.*;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * The control of AcmeConnector.
 * 
 */
public class ConnectorEditPart extends ElementEditPart {

    private AcmeConnector _acmeConnector = null;
    private Dimension _size = null;
    private Point _location = null;
    private SystemView _sv = null;

    /*
     * LFW: Created to add 'red square highlight' behavior to ports
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    protected void createEditPolicies()
    {
    	super.createEditPolicies();
    	installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, new ItemOutlineEditPolicy());
    }
    
    /**
     * Creates a new Component Figure and returns it.
     *
     * @return  Figure representing the component.
     */
    protected IFigure createFigure() {
        if (getModel() == null)
            return null;
        AcmeVisElement f =
            (AcmeVisElement) ElementFigureFactory.createConnectorView();
        f.setModel((AcmeElement) getModel());
        f.setEditPart(this);
        return f;
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#refreshVisuals()
     */
    protected void refreshVisuals() {
        if (getModel() != null) {
            _location = SystemView.getLocation((AcmeConnector) getModel(), true);
            ConnectorView connectorView = (ConnectorView) getFigure();
            _size = Helper.getSize((AcmeConnector) getModel());
            ConnectorVisualization connVis = null;
            if (connectorView.getVisualization () instanceof ConnectorVisualization) {
				connVis = (ConnectorVisualization )connectorView.getVisualization ();
				
			}
            else {
            	connVis = new ConnectorVisualization ();
            	connVis.copyInto (connectorView.getVisualization ());
            }
            if (_size == null) {
				_size = connVis.getSize();
            }

            SystemEditPart se =
                (SystemEditPart) getRoot()
                    .getViewer()
                    .getEditPartRegistry()
                    .get(
                    ((AcmeConnector) getModel()).getParent());

            if (_size != null && _location != null) {
                _location.translate(-_size.width / 2, -_size.height / 2);

                Rectangle r = new Rectangle(_location, _size);
                _acmeConnector = (AcmeConnector) getModel();
                refreshRoles();
                refreshAttachments();
                if (connVis.autoCenterBinaryConnectors()) r = recenterConnector (connVis, r);
                ((GraphicalEditPart) getParent()).setLayoutConstraint(
                    this,
                    getFigure(),
                    r);

            }

            super.refreshVisuals();
            if (se != null) se.getFigure().repaint();
        }
    }

    
	/**
	 * 
	 */
	public void recenterConnectorIfRequired() {
		ConnectorVisualization cv = (ConnectorVisualization )((ConnectorView )getFigure ()).getVisualization();
		if (!cv.autoCenterBinaryConnectors()) return;
		Rectangle orig = new Rectangle (_location, _size);
		orig = recenterConnector (cv, orig);
		((GraphicalEditPart) getParent()).setLayoutConstraint(
                this,
                getFigure(),
                orig);
	}
    /**
	 * @param connVis
	 * @param r
	 * @return
	 */
	private Rectangle recenterConnector(ConnectorVisualization connVis, Rectangle r) {
		AcmeConnector conn = (AcmeConnector )getModel ();
		if (conn.getRoles ().getSize () != 2) return r;
		List roles = Helper.convertAcmeEnumerationToList(conn.getRoles ());
		AcmeRole r1 = (AcmeRole )roles.get (0);
		AcmeRole r2 = (AcmeRole )roles.get (1);
		Point r1l = SystemView.getLocation (r1, false);
		Point r2l = SystemView.getLocation (r2, false);
		Point center = new Rectangle (r1l, r2l).getCenter ();
		Point connCenter = center.getCopy ();
		
		SystemView.setLocation (conn, center);
		for (Iterator iter = roles.iterator (); iter.hasNext ();) {
			AcmeRole role = (AcmeRole )iter.next ();
			RoleEditPart rep = (RoleEditPart )getRoot ().getViewer ().getEditPartRegistry ().get (role);
			if (rep != null) {
				RoleView rv = (RoleView )rep.getFigure ();
				SystemView sv = (SystemView )rv.getParent ();
				if (sv == null) return r;
				Point roleStart = center;
				Point roleEnd = rv.getEnd ();
				rv.setStart (roleStart);
			}
		}
		Rectangle newR = r.getCopy ();
		connCenter.translate (-r.width/2, -r.height/2);
		newR.setLocation (connCenter);
		return newR;
	}

	//TODO: This should really be in RoleEditPart
    public void refreshRole (AcmeRole role) {
    	if (role.getParent() != getModel ()) return;
    	RoleEditPart roleEditPart =
    		(RoleEditPart) getRoot().getViewer().getEditPartRegistry().get(
    				role);
    	refreshRole (roleEditPart);
    		
    }
    
    protected void refreshRoles() {
        if (_acmeConnector == null)
            _acmeConnector = (AcmeConnector) getModel();
        Vector roles = Helper.getChildren(_acmeConnector);
        for (int i = 0; i < roles.size(); i++) {
            AcmeRole role = (AcmeRole) roles.elementAt(i);
            refreshRole (role);
        }
    }

    private void refreshAttachments() {
        Vector roles = Helper.getChildren(_acmeConnector);
        for (int i = 0; i < roles.size(); i++) {
            AcmeRole role = (AcmeRole) roles.elementAt(i);
            AcmeAttachment attachment = ModelHelper.findAttachment(role);
            if (attachment != null) {
            	PortEditPart portEditPart = (PortEditPart )getRoot ().getViewer ().getEditPartRegistry().get (attachment.getPort ());
            	if (portEditPart != null) {
            		portEditPart.refreshVisuals();
            	}
//                AttachmentEditPart attachmentEditPart =
//                    (AttachmentEditPart) getRoot()
//                        .getViewer()
//                        .getEditPartRegistry()
//                        .get(
//                        attachment);
//                if (attachmentEditPart != null)
//                    attachmentEditPart.refreshVisuals();
            }
        }
    }

    /**
     * Returns the children of this through the model.
     *
     * @return  Children of this as a List.
     */
    protected List getModelChildren() {
        Vector children = new Vector();
        return children;
    }

	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	public void performRequest(Request req) {
		super.performRequest (req);
		Object reqType = req.getType ();
		if (reqType.equals (RequestConstants.REQ_OPEN)) {
			MultiPageAcmeEditor mp = ASContext.current ().getActiveEditor ();
			DrillDownAction action = new DrillDownAction (mp);
			action.setEditPart(this);
			action.run ();
		}
	}

	/**
	 * @param part
	 */
	public void refreshRole(RoleEditPart roleEditPart) {
		if (roleEditPart != null) {
			AcmeRole role = (AcmeRole )roleEditPart.getModel ();
    		RoleView roleView = (RoleView) roleEditPart.getFigure();
    		_sv = (SystemView) roleView.getParent();
    		if (_sv == null) return;
    		Dimension diff =
    			roleView.getEnd().getDifference(roleView.getStart());
    		Point roleEndBeforeMoving = roleView.getEnd().getCopy();
    		Point roleStart =
    			new Point(
    					_location.x + _size.width / 2,
						_location.y + _size.height / 2);
    		Point roleEnd = roleEndBeforeMoving;
    		if (((AcmeSystem )_sv.getModel()).getAttachments(role).getSize () == 0)
    			roleEnd = roleStart.getCopy().translate(diff);
    		roleView.setStart(roleStart);
    		PointList bendPoints =
    			_sv.getBendpoints(
    					role,
						roleStart,
						roleEnd,
						roleEnd.getDifference(roleEndBeforeMoving));
    		if (roleView._isBendpointMoving || roleView._isMoving) {
    			bendPoints = _sv.arrangeBendpoints(role, bendPoints, roleStart, roleEnd);
    		} else {
    			bendPoints = _sv.layoutBendpoints(role, bendPoints, roleStart, roleEnd);
    		}
    		
    		SystemView.cleanBendpoints(bendPoints, roleStart, roleEnd);
    		roleView.setBendpoints(bendPoints);
    		roleEditPart.refreshBendpoints();
    		roleView.setEnd(roleEnd);
    	}
		
	}


}
