package AcmeStudio.ui.editors.diagram.parts;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.ui.IActionFilter;

import AcmeStudio.ui.editors.diagram.figures.AcmeVisElement;
import AcmeStudio.ui.editors.diagram.figures.IAcmeElementFigure;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.policies.ElementDirectEditPolicy;
import AcmeStudio.ui.editors.diagram.policies.ElementEditPolicy;
import AcmeStudio.ui.editors.diagram.policies.ElementNameCellEditorLocator;
import AcmeStudio.ui.editors.diagram.policies.ElementNameEditManager;
import AcmeStudio.ui.editors.diagram.policies.ElementNodeEditPolicy;
import AcmeStudio.ui.editors.diagram.ruler.model.DiagramGuide;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeFamily;
import acmeLib.elements.AcmeSystem;

/**
 * Base class of all element controls
 */
abstract public class ElementEditPart
    extends org.eclipse.gef.editparts.AbstractGraphicalEditPart
    implements NodeEditPart, IActionFilter {

    private ElementNameEditManager _manager;
	private DiagramGuide	_horizontalGuide;
	private DiagramGuide	_verticalGuide;
	/**
     * @see org.eclipse.gef.EditPart#createEditPolicies()
     */
    protected void createEditPolicies() {
        installEditPolicy(EditPolicy.COMPONENT_ROLE, new ElementEditPolicy());
        installEditPolicy(
            EditPolicy.GRAPHICAL_NODE_ROLE,
            new ElementNodeEditPolicy());
        installEditPolicy (EditPolicy.DIRECT_EDIT_ROLE, new ElementDirectEditPolicy ());
    }

    
	
    /**
     * @see org.eclipse.gef.EditPart#activate()
     */
    public void activate() {
        if (isActive())
            return;
        super.activate();
    }

    /**
     * Makes the EditPart insensible to changes in the model
     * by removing itself from the model's list of listeners.
     */
    public void deactivate() {
        if (!isActive())
            return;
        
        super.deactivate();
    }

    /**
     Returns the ConnectionAnchor that the passed ConnectionEditPart should
     use with its Connection figure.
     */
    public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
        return null;
    }

    /**
     Returns the ConnectionAnchor that the passed ConnectionEditPart should
     use with its Connection figure.
     */
    public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
        return null;
    }

    /**
     Returns the ConnectionAnchor that is closest to the point <x,y>.
     This is used during mouse interactions.
     */
    public ConnectionAnchor getSourceConnectionAnchor(Request request) {
        return null;
    }

    /**
     Returns the ConnectionAnchor that is closest to the point <x,y>.
     This is used during mouse interactions.
     */
    public ConnectionAnchor getTargetConnectionAnchor(Request request) {
        return null;
    }

    protected void configureToolTip() {

        Label tt = new Label();
        tt.setText(((AcmeElement) getModel()).getQualifiedName());
        if (getModel() instanceof SystemEditPart.BindingPoint) {
            tt.setText(
                ((SystemEditPart.BindingPoint) getModel())
                    ._outer
                    .getQualifiedName());
        }
        getFigure().setToolTip(tt);

    }

    /**
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    protected void refreshVisuals() {
        if (!(getModel() instanceof AcmeElement))
            return;
        AcmeElement elem = (AcmeElement) getModel();

        if (!(elem instanceof AcmeSystem)) {
            configureToolTip();
          resizeIfNecessary ();
        }
        
        super.refreshVisuals();
    }

    /**
	 * 
	 */
	protected boolean resizeIfNecessary() {
		// Don't forget to change PortEditPart.resizeIfNecessary every time this method is changed
	        SystemView systemView = (SystemView) ((AcmeVisElement )getFigure ()).getParent();
		ElementVisualization vis = getVis ();
		if (vis != null && vis.shouldResizeToFitLabel()) {
			Rectangle figureBounds = getFigure ().getBounds ();
			Rectangle bounds = vis.getLabelBounds((AcmeElement )getModel (), figureBounds);
	        Point location = null;
	        if (systemView != null) 
	            location =systemView.getElementLocation((AcmeElement) getModel());
	        else
	        	location = SystemView.getLocation((AcmeElement )getModel (), false);
	        Dimension size = Helper.getSize ((AcmeElement )getModel ());
	        if (size == null) {
	        	size = ((AcmeVisElement )getFigure ()).getVisualization().getSize ();
	        }
	        Rectangle newBounds = new Rectangle (location, bounds.getSize ());
			newBounds = newBounds.expand(2*vis.getLabelMargin(), 2*vis.getLabelMargin());
			
			this.getFigure().setMinimumSize(newBounds.getSize ());
			
			boolean doResize = false;
			if (bounds.width > figureBounds.width) {
				newBounds.width = bounds.width;
				doResize = true;
			}
			else {
				newBounds.width = figureBounds.width;
			}
			if (bounds.height > figureBounds.height) {
				newBounds.height = bounds.height;
				doResize = true;
			}
			else {
				newBounds.height = figureBounds.height;
			}
			if (doResize) { 
				newBounds.x = newBounds.x - newBounds.width / 2;
				newBounds.y = newBounds.y - newBounds.height/2;
				((GraphicalEditPart )getParent ()).setLayoutConstraint (this, getFigure (), newBounds);
				Helper.setSize((AcmeElement )this.getModel (), newBounds.getSize());
				
			}
			return doResize;
		}

		return false;
	}

	public ElementVisualization getVis() {
        if (this.getFigure() instanceof AcmeVisElement) {
            return ((AcmeVisElement)this.getFigure()).getVisualization(); 
        }
        return null;
    }

    public Point getCanvasOffset() {
        int offsetY = 0, offsetX = 0;
        FigureCanvas fc = (FigureCanvas) getViewer().getControl();
        Rectangle r = fc.getViewport().getClientArea();
        offsetY = r.y;
        offsetX = r.x;
        return new Point(offsetX, offsetY);
    }

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		return null;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	public void performRequest(Request req) {
		if (req.getType () == RequestConstants.REQ_DIRECT_EDIT) {
			if (_manager == null && getFigure () instanceof IAcmeElementFigure) {
				_manager = new ElementNameEditManager (this, TextCellEditor.class, new ElementNameCellEditorLocator ((IAcmeElementFigure )getFigure ()));
			}
			if (_manager != null) _manager.show ();
		}
		super.performRequest (req);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionFilter#testAttribute(java.lang.Object, java.lang.String, java.lang.String)
	 */
	public boolean testAttribute(Object target, String name, String value) {
		if (target instanceof ElementEditPart) {
			ElementEditPart ep = (ElementEditPart) target;
			if (ep.getModel () instanceof AcmeElement) {
				AcmeElement system = Helper.getAcmeSystem((AcmeElement )ep.getModel ());
				if (system == null) return false;
				if (name.equalsIgnoreCase("family")) {
					Object f = system.lookupName(value, true);
					if (f instanceof AcmeFamily && system.declaresType((AcmeFamily )f)) {
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * @return
	 */
	public DiagramGuide  getHorizontalGuide() {
		return _horizontalGuide;
	}

	/**
	 * @return
	 */
	public DiagramGuide getVerticalGuide() {
		return _verticalGuide;
	}

	/**
	 * @param horizontalGuide The horizontalGuide to set.
	 */
	public void setHorizontalGuide(DiagramGuide horizontalGuide) {
		_horizontalGuide = horizontalGuide;
	}
	/**
	 * @param verticalGuide The verticalGuide to set.
	 */
	public void setVerticalGuide(DiagramGuide verticalGuide) {
		_verticalGuide = verticalGuide;
	}

}
