/*
 * Created on Feb 17, 2004
 *  
 */
package AcmeStudio.ui.editors.diagram.parts;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.tools.DragEditPartsTracker;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmePort;
/**
 * @author Bradley Schmerl Created Feb 17, 2004
 */
public class PortDragTracker extends DragEditPartsTracker {
    private ComponentEditPart _parent;

    /**
     * @param sourceEditPart
     */
    public PortDragTracker(EditPart sourceEditPart, ComponentEditPart parent) {
        super (sourceEditPart);
        _parent = parent;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.tools.TargetingTool#updateTargetRequest()
     */
    protected void updateTargetRequest() {
        repairStartLocation ();
        ChangeBoundsRequest request = (ChangeBoundsRequest )getTargetRequest ();
        request.setEditParts (getOperationSet ());
        Dimension delta = getDragMoveDelta ();
        Rectangle componentRectangle = _parent.getFigure ().getBounds ().getCopy ();
        _parent.getFigure ().translateToAbsolute (componentRectangle);
        Rectangle innerRectangle = componentRectangle.getCopy ().expand (-4, -4);
        Point location = getStartLocation ().getCopy ();
        Point proposedLocation = location.getCopy ();
        proposedLocation.x += delta.width;
        proposedLocation.y += delta.height;
        Dimension portSize = ((PortEditPart )getSourceEditPart ()).getFigure ().getSize ().getCopy ();
        _parent.getFigure ().translateToAbsolute (portSize);
//        proposedLocation.x += portSize.width/2;
        proposedLocation.y += portSize.height/2;
         calculatePortPosition2 (delta, componentRectangle, proposedLocation);
        request.setMoveDelta (new Point (delta.width, delta.height));
        request.getExtendedData ().clear ();
        request.setLocation (getLocation ());
        request.setType (getCommandName ());
    }

    private void calculatePortPostion1(Dimension delta, Rectangle componentRectangle, Point proposedLocation) {
        int distanceFromTop = Math.abs (componentRectangle.y - proposedLocation.y);
        int distanceFromBottom = Math.abs (componentRectangle.y + componentRectangle.height - proposedLocation.y);
        int distanceFromLeft = Math.abs (componentRectangle.x - proposedLocation.x);
        int distanceFromRight = Math.abs (componentRectangle.x + componentRectangle.width - proposedLocation.x);
        int verticalCloseness = Math.min (distanceFromTop, distanceFromBottom);
        int horizontalCloseness = Math.min (distanceFromLeft, distanceFromRight);
        if (horizontalCloseness <= verticalCloseness) {
            if (distanceFromLeft < distanceFromRight) {
                delta.width = getStartLocation ().x - componentRectangle.x;
            }
            else {
                delta.width = getStartLocation ().x - componentRectangle.width;
            }
        }
        else {
            if (distanceFromTop <= distanceFromBottom) {
                delta.height = getStartLocation ().y - componentRectangle.y;
            }
            else {
                delta.height = getStartLocation ().y - componentRectangle.height;
            }
        }
        //		if (proposedLocation.x + delta.width < componentRectangle.x -
        // portSize.width / 2) {
        //			delta.width = componentRectangle.x - portSize.width/2 -
        // proposedLocation.x;
        //		}
        //		else if (proposedLocation.x + delta.width > componentRectangle.x +
        // componentRectangle.width + portSize.width/2) {
        //			delta.width = componentRectangle.x + componentRectangle.width +
        // portSize.width / 2 - proposedLocation.x;
        //		}
        //// else if (proposedLocation.x + delta.width > innerRectangle.x -
        // portSize.width/2) {
        //// delta.width = innerRectangle.x - portSize.width/2 -
        // proposedLocation.x;
        //// }
        //// else if (proposedLocation.x + delta.width < innerRectangle.x +
        // innerRectangle.width + portSize.width/2) {
        //// delta.width = innerRectangle.x + innerRectangle.width -
        // portSize.width/2 - proposedLocation.x;
        //// }
        //		
        //		if (proposedLocation.y + delta.height < componentRectangle.y -
        // portSize.height /2) {
        //			delta.height = componentRectangle.y - portSize.height / 2 -
        // proposedLocation.y;
        //			
        //		}
        //		else if (proposedLocation.y + delta.height > componentRectangle.y +
        // componentRectangle.height + portSize.width/2) {
        //			delta.height = componentRectangle.y + componentRectangle.height +
        // portSize.width/2 - proposedLocation.y;
        //		}
        //// else if (proposedLocation.y + delta.height > innerRectangle.y -
        // portSize.height/2) {
        //// delta.height = innerRectangle.y - portSize.height/2 -
        // proposedLocation.y;
        //// }
        //// else if (proposedLocation.y + delta.height < innerRectangle.y +
        // innerRectangle.width - portSize.height/2) {
        //// delta.height = innerRectangle.y + innerRectangle.width -
        // portSize.height/2 - proposedLocation.y;
        //// }
    }

    private void calculatePortPosition2(Dimension delta, Rectangle componentRectangle, Point proposedLocation) {
        Point componentCenter = componentRectangle.getCenter ();
        Point newPos = new Point (proposedLocation.x - componentCenter.x, proposedLocation.y - componentCenter.y);
        SystemView.normalizePortPosition ((AcmePort )getSourceEditPart ().getModel (), (AcmeComponent )_parent
                        .getModel (), newPos);
        delta.width = (componentCenter.x + newPos.x) - getStartLocation ().x;
        delta.height = (componentCenter.y + newPos.y) - getStartLocation ().y;
    }
}
