package AcmeStudio.ui.editors.diagram.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;

import AcmeStudio.ui.editors.diagram.figures.ElementFigureFactory;
import AcmeStudio.ui.editors.diagram.figures.PortView;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.policies.ItemOutlineEditPolicy;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.PortVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.ui.editors.diagram.styles.constants.Orientation;
import AcmeStudio.util.Helper;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeAttachment;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;

/**
 * Holds a component
 */
public class PortEditPart
    extends MovableEditPart
    implements PropertyChangeListener {

    AcmeComponent _parentComponent;
    ComponentEditPart _parentEditPart;

    /*
     * LFW: Created to add 'red square highlight' behavior to ports
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    protected void createEditPolicies()
    {
    	super.createEditPolicies();
    	installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, new ItemOutlineEditPolicy());
    }
    
    /**
     * Creates a new Port Figure and returns it.
     *
     * @return  Figure representing the port.
     */
    protected IFigure createFigure() {
        if (getModel() == null)
            return null;
        PortView f = (PortView) ElementFigureFactory.createPortView();
        f.setEditPart(this);
        AcmePort model = (AcmePort) getModel();
        f.setModel(model);
        ElementVisualization vis = VisualStyleRegistry.getVisualization(model);
        vis.addPropertyChangeListener(model.getQualifiedName() + "r", this);

        AcmeElement e = ((AcmePort) getModel()).getParent();
//        this.refreshVisuals ();
        if (e instanceof AcmeComponent) {
            _parentComponent = (AcmeComponent) e;
            _parentEditPart =
                (ComponentEditPart) getRoot()
                    .getViewer()
                    .getEditPartRegistry()
                    .get(
                    _parentComponent);
     
        }
        return f;
    }

    protected void refreshVisuals() {
        if (_parentEditPart != null) {
        	refreshPosition ();
        }
        refreshAttachments ();
        super.refreshVisuals();
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.parts.ElementEditPart#resizeIfNecessary()
	 */
	protected boolean resizeIfNecessary() {
		ElementVisualization vis = getVis ();
		if (vis.shouldResizeToFitLabel()) {
			AcmePort port = (AcmePort )getModel ();
			Rectangle bounds = vis.getLabelBounds((AcmeElement )getModel (), getFigure ().getBounds ());
			
			Point location = getFigure ().getBounds ().getLocation();
			Dimension size = Helper.getSize (port);
			if (size == null) size = vis.getSize();
			Rectangle newBounds = new Rectangle (location, bounds.getSize ());
			newBounds.expand (vis.getLabelMargin (),  vis.getLabelMargin()); 
			boolean doResize = false;
			if (bounds.width > getFigure ().getBounds ().width) {
				newBounds.width = bounds.width;
				doResize = true; 
			}
			if (bounds.height > getFigure ().getBounds ().height) {
				newBounds.height = bounds.height;
				doResize = true;
			}
			if (doResize) {
				newBounds.x = newBounds.x - newBounds.width /2;
				newBounds.y = newBounds.y - newBounds.height/2;
				Rectangle r = (Rectangle )((GraphicalEditPart )getParent ()).getContentPane().getLayoutManager().getConstraint(getFigure ());
				r.setSize (newBounds.getSize ());
				((GraphicalEditPart )getParent ()).setLayoutConstraint (this, getFigure (), r);
				Helper.setSize (port, newBounds.getSize ());
			}
			return doResize;
		}
		return false;
	}
	
    private void refreshPosition() {
		// Calculate the port positions based on parent
		AcmePort	port	= (AcmePort )getModel ();
		
		PortView	view	= (PortView )getFigure ();
		(view).setModel(port);
		Point location = _parentEditPart.calculateChildPortLocation(port);
		if (location == null) {
			// If there is no location, then we better force  refresh in the component
			_parentEditPart._needPortsRefreshed = true;
			location = _parentEditPart.calculateChildPortLocation(port);
			if (location == null) location = new Point (0,0);
		}
		Rectangle parentBounds = _parentEditPart.getBounds ();
		Dimension size = Helper.getSize (port);
		if (size == null) size = VisualStyleRegistry.getVisualization(port).getSize ();
		location = location.getCopy ();
		view.setRelativeLocation(location);
		location.x = location.x + parentBounds.x /*+ parentBounds.width /2*/  - size.width/2;
		location.y = location.y + parentBounds.y /*+ parentBounds.height /2*/ - size.height/2;
		view.setLocation (location);
		
		ElementVisualization	visualization	= view.getVisualization();
		if ((visualization instanceof PortVisualization) && ((PortVisualization )visualization).rotatePort()) {
			Dimension diff = parentBounds.getLocation ().getDifference (location);
			double angle = Math.atan2 (diff.height, diff.width);
			if (((PortVisualization )visualization).getOrientation() == Orientation.OUT) angle += Math.PI;
			view.setRotation(angle);
		}
		
		Rectangle r = new Rectangle (location, size);
		((GraphicalEditPart )getParent ()).setLayoutConstraint (this, view, r);
	}

    /**
     * Refresh the port visualization if it is involved in an attachment.
    **/
    public void refreshAttachments () {
    	
    	AcmePort	port	= (AcmePort )getModel ();
    	AcmeEnumeration attachments = Helper.getAcmeSystem(port).getAttachments(port);
    	
    	PortView portView = (PortView )getFigure ();
    	
    	
    	
    	while (attachments.moreItems ()) {
    		AcmeAttachment attachment = (AcmeAttachment )attachments.getNextItem ();
    		AcmeRole role = attachment.getRole ();
    		
    		RoleEditPart roleEditPart = (RoleEditPart )getRoot ().getViewer ().getEditPartRegistry().get (role);
    		if (roleEditPart == null) continue;
    		RoleView roleView = (RoleView )roleEditPart.getFigure ();
    		
    		Rectangle portBounds = getFigure ().getBounds ();
    		Point endPoint = portBounds.getCenter ();
    		roleView._isMoving = true;
    		roleView.setEnd (endPoint);
    		SystemView.setLocation (role, endPoint);
    		roleEditPart.refreshAttachment (this);
    		
//  		PortView	portView	= (PortView )getFigure ();
    		((SystemView )getFigure ().getParent()).moveBehindAttachments (portView, roleView);
    		ElementVisualization vis = portView.getVisualization();
    		if (vis instanceof PortVisualization) {
    			PortVisualization portVis = (PortVisualization )vis;
    			boolean alignPortWithAttachedRole = portVis.alignPort();
    			if (alignPortWithAttachedRole) {
    				int orientation = portVis.getOrientation();
    				Dimension diff = roleView.getEnd ().getDifference (roleView.getLastBendpoint());
    				double angle = Math.atan2 (diff.height, diff.width);
    				if (orientation == Orientation.OUT) angle += Math.PI;
    				portView.setRotation (angle);
    			}
    		}
    		
    	}
		
    }
	/**
     * Returns the children of this through the model.
     *
     * @return  Children of this as a List.
     */
    protected List getModelChildren() {
        Vector children = new Vector();
        return children;
    }

    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals(ElementVisualization.PROP_VIS_RESET)) {
            AcmePort model = (AcmePort) getModel();
            ElementVisualization vis = VisualStyleRegistry.getVisualization(model);
            vis.addPropertyChangeListener(model.getQualifiedName() + "r", this);

            refreshVisuals();
        }
    }


	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#getDragTracker(org.eclipse.gef.Request)
	 */
//	public DragTracker getDragTracker(Request request) {
//		return new PortDragTracker (this, _parentEditPart);
//	}

}
