/*
 * Created on Feb 17, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.parts;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.tools.ResizeTracker;


/**
 * @author Bradley Schmerl
 * Created Feb 17, 2004
 */
public class ResizePortTracker extends ResizeTracker {

	private GraphicalEditPart owner;
	/**
	 * @param owner
	 * @param direction
	 */
	public ResizePortTracker(GraphicalEditPart owner, int direction) {
		super (owner, direction);
		this.owner = owner;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.tools.SimpleDragTracker#updateSourceRequest()
	 */
	protected void updateSourceRequest() {
		ChangeBoundsRequest request = (ChangeBoundsRequest) getSourceRequest();
		Dimension d = getDragMoveDelta();

		Point location = new Point(getLocation());
		Point corner = new Point(0, 0);
		Dimension resize = new Dimension(0, 0);	

		if (getCurrentInput().isShiftKeyDown() && owner != null) {
			int origHeight = owner.getFigure().getBounds().height;
			int origWidth = owner.getFigure().getBounds().width;
			float ratio = 1;
			
			if (origWidth != 0 && origHeight != 0) {
				ratio = ((float)origHeight / (float)origWidth);
			}
			
			if (getResizeDirection() == PositionConstants.SOUTH_EAST) {
				if (d.height > (d.width * ratio)) {
					d.width = (int)(d.height / ratio);
				} else {
					d.height = (int)(d.width * ratio);
				}
			} else if (getResizeDirection() == PositionConstants.NORTH_WEST) {
				if (d.height < (d.width * ratio)) {
					d.width = (int)(d.height / ratio);
				} else {
					d.height = (int)(d.width * ratio);
				}
			} else if (getResizeDirection() == PositionConstants.NORTH_EAST) {
				if (-(d.height) > (d.width * ratio)) {
					d.width = -(int)(d.height / ratio);
				} else {
					d.height = -(int)(d.width * ratio);
				}
			} else if (getResizeDirection() == PositionConstants.SOUTH_WEST) {
				if (-(d.height) < (d.width * ratio)) {
					d.width = -(int)(d.height / ratio);
				} else {
					d.height = -(int)(d.width * ratio);
				}
			}
		}
		
		
		if ((getResizeDirection() & PositionConstants.NORTH) != 0) {
			if (getCurrentInput().isControlKeyDown()) {
				resize.height -= d.height;
			}
			corner.y += d.height;
			resize.height -= d.height;
		}
		if ((getResizeDirection() & PositionConstants.SOUTH) != 0) {
			if (getCurrentInput().isControlKeyDown()) {
				corner.y -= d.height;
				resize.height += d.height;
			}
			resize.height += d.height;
		}
		if ((getResizeDirection() & PositionConstants.WEST) != 0) {
			if (getCurrentInput().isControlKeyDown()) {
				resize.width -= d.width;
			}
			corner.x += d.width;
			resize.width -= d.width;
		}
		if ((getResizeDirection() & PositionConstants.EAST) != 0) {
			if (getCurrentInput().isControlKeyDown()) {
				corner.x -= d.width;
				resize.width += d.width;
			}
			resize.width += d.width;
		}

		request.setMoveDelta(corner);
		request.setSizeDelta(resize);
		request.setLocation(location);
		request.setEditParts(getOperationSet());

		request.getExtendedData().clear();
	}
}
