package AcmeStudio.ui.editors.diagram.parts;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import org.eclipse.draw2d.AbsoluteBendpoint;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.ManhattanConnectionRouter;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.gef.*;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.tools.ConnectionEndpointTracker;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.ui.IActionFilter;
import AcmeStudio.ui.editors.diagram.figures.*;
import AcmeStudio.ui.editors.diagram.policies.*;
import AcmeStudio.ui.editors.diagram.styles.ConnectorVisualization;
import AcmeStudio.util.Helper;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.*;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * The control of AcmeRole.
 * 
 */
public class RoleEditPart extends AbstractConnectionEditPart implements IActionFilter
/*implements PropertyChangeListener*/ {

	/**
	 * Roles now understand change bounds requests
	 * @param req
	 * @return
	 * @see org.eclipse.gef.EditPart#understandsRequest(org.eclipse.gef.Request)
	 */
	public boolean understandsRequest(Request req) {
		if (req instanceof ChangeBoundsRequest) 
			return true;
		return super.understandsRequest(req);
	}
	
	
	
	
	private ConnectorEditPart _connectorEditPart = null;
	private ElementNameEditManager _manager;

	/**
	 * Creates the edit policies.
	 */
	protected void createEditPolicies() {
		updateEditPolicies();
	}

	/**
	 * Creates the edit policies.
	 */
	public void updateEditPolicies() {
		EditPart parent = getParent();
		if (parent instanceof SystemEditPart) {
			SystemEditPart system = (SystemEditPart) parent;
			SystemView view = (SystemView) system.getFigure();
			if (view != null) { // && view.getIsVisible()) {
				installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE, new RoleEndpointEditPolicy());
				installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new ElementNodeEditPolicy());
				installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new ElementDirectEditPolicy ());
				installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, new ElementSelectionEditPolicy ());
			}
			else {
				if (getEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE) != null) {
					removeEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE);
				}
				if (getEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE) != null) {
					removeEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE);
				}
				if (getEditPolicy (EditPolicy.DIRECT_EDIT_ROLE) != null) {
					removeEditPolicy (EditPolicy.DIRECT_EDIT_ROLE);
				}
			}
		}
		//Note that the Connection is already added to the diagram and knows its Router.
		refreshBendpointEditPolicy();
	}

	/**
	 * Creates a new Role Figure and returns it.
	 *
	 * @return  Figure representing the role.
	 */
	protected IFigure createFigure() {
		AcmeElement model = (AcmeElement) getModel();
		if (model == null)
			return null;

		RoleView f = (RoleView) ElementFigureFactory.createRoleView();
		f.setModel(model);
		f.setFigureCanvas((FigureCanvas) this.getViewer().getControl());

		//        Visualization vis = VisualStyleRegistry.getVisualization(model);
		//        vis.addPropertyChangeListener(model.getQualifiedName() + "r", this);

		return f;
	}

	/**
	 * Returns the children of this through the model.
	 *
	 * @return  Children of this as a List.
	 */
	protected List getModelChildren() {
		Vector children = new Vector();
		return children;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart.deactivateFigure()
	 */
	protected void deactivateFigure() {
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart.activateFigure()
	 */
	protected void activateFigure() {
		if (getModel() != null) {
			AcmeElement e = ((AcmeRole) getModel()).getParent();
			if (e instanceof AcmeConnector) {
				_connectorEditPart = (ConnectorEditPart) getRoot().getViewer().getEditPartRegistry().get(e);
				if (_connectorEditPart != null)
					_connectorEditPart.refreshVisuals();
//				else
//					Debug.diagnoseln ("RoleEditPart.activateFigure: " + "connectorEP is null");
			}
		}
	}

	public void refreshBendpoints() {
		if (getConnectionFigure().getConnectionRouter() instanceof ManhattanConnectionRouter)
			return;
		
		RoleRouter rr = new RoleRouter();
		RoleView rv = (RoleView) getFigure();
		rv.setConnectionRouter(rr);

		PointList modelConstraint = ((RoleView) getFigure()).getBendpoints();
		if (modelConstraint != null) {
			List figureConstraint = new ArrayList();
			for (int i = 0; i < modelConstraint.size(); i++) {
				AbsoluteBendpoint abp = new AbsoluteBendpoint(modelConstraint.getPoint(i));
				figureConstraint.add(abp);
			}
			getConnectionFigure().setRoutingConstraint(figureConstraint);
		}
		else {
			getConnectionFigure ().setRoutingConstraint (new ArrayList ());
		}

		rv.setEnd((rv.getEnd()));
	}

	private void refreshBendpointEditPolicy() {
		if (getConnectionFigure().getConnectionRouter() instanceof ManhattanConnectionRouter)
			installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, null);
		else
			installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new RoleBendpointEditPolicy());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	public void refreshVisuals() {
		refreshAttachments ();
		if (_connectorEditPart != null) {
			_connectorEditPart.refreshRole(this);
			ConnectorView connView = (ConnectorView )_connectorEditPart.getFigure ();
			if (connView.getVisualization () instanceof ConnectorVisualization) {
				ConnectorVisualization cv = (ConnectorVisualization )connView.getVisualization ();
				if (cv.autoCenterBinaryConnectors()) _connectorEditPart.refreshVisuals();
			}
		}
		/*((SystemView )getFigure ().getParent ()).moveBehindAttachments()*/
//		refreshBendpoints();
		super.refreshVisuals();
	}



	/**
	 * 
	 */
	private void refreshAttachments() {
		
		
		AcmeRole	role	= (AcmeRole )getModel ();
		
		AcmeEnumeration attachments = Helper.getAcmeSystem (role).getAttachments (role);
		while (attachments.moreItems ()) {
			AcmeAttachment attachment = (AcmeAttachment )attachments.getNextItem ();
			AcmePort port = attachment.getPort ();
			PortEditPart portEditPart = (PortEditPart )getRoot ().getViewer ().getEditPartRegistry().get (port);
			if (portEditPart == null) continue;
			refreshAttachment (portEditPart);
		}
	}
	
	public void refreshAttachment (PortEditPart toPort) {
		if (getFigure ().getParent () != null) {
			PortView portView = (PortView )toPort.getFigure ();
			((SystemView )getFigure ().getParent ()).moveBehindAttachments(portView, (RoleView )getFigure ());
			if (_connectorEditPart != null)  {
				_connectorEditPart.refreshRole((AcmeRole )getModel ());
				_connectorEditPart.recenterConnectorIfRequired ();
			}
//			refreshBendpoints ();
		}
	}

	/**
	 * Overridden to return a default <code>DragTracker</code> for RoleEditPart.
	 * @see org.eclipse.gef.EditPart#getDragTracker(Request)
	 */
	public DragTracker getDragTracker(Request request) {
//		RoleTracker tracker = new RoleTracker(this);
		RoleEndpointTracker tracker = new RoleEndpointTracker (this);
		tracker.setViewer(this.getViewer());
		tracker.setCommandName(ConnectionEndpointTracker.REQ_RECONNECT_TARGET);
		return tracker;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	public void performRequest(Request req) {
		if (req.getType() == RequestConstants.REQ_DIRECT_EDIT) {
			if (_manager == null) {
				_manager = new ElementNameEditManager (this, TextCellEditor.class, new ElementNameCellEditorLocator ((IAcmeElementFigure )getFigure ()));
			}
			_manager.show ();
		}
		super.performRequest (req);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionFilter#testAttribute(java.lang.Object, java.lang.String, java.lang.String)
	 */
	public boolean testAttribute(Object target, String name, String value) {
		if (target instanceof RoleEditPart) {
			RoleEditPart ep = (RoleEditPart) target;
			
			AcmeElement system = Helper.getAcmeSystem((AcmeElement )ep.getModel ());
			if (system == null) return false;
			if (name.equalsIgnoreCase("family")) {
				Object f = system.lookupName(value, true);
				if (f instanceof AcmeFamily && system.declaresType((AcmeFamily )f)) {
					return true;
				}
			}
		}
		return false;
	}
}
