package AcmeStudio.ui.editors.diagram.parts;

import java.util.Collection;
import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.SharedCursors;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.gef.tools.ConnectionEndpointTracker;
import org.eclipse.swt.graphics.Cursor;

import AcmeStudio.ui.editors.diagram.actions.MoveRoleCommand;
import AcmeStudio.ui.editors.diagram.figures.RoleView;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 * 
 * The tracker of role end point.
 *  
 */

public class RoleEndpointTracker extends ConnectionEndpointTracker
{

    private boolean _selected = false;

    /**
     * @param button
     * @return @see org.eclipse.gef.tools.AbstractTool#handleButtonDown(int)
     */
    protected boolean handleButtonDown(int button)
    {
        EditPartViewer viewer = getCurrentViewer();
        List selectedObjects = viewer.getSelectedEditParts();
        if (getCurrentInput().isControlKeyDown())
        {
            if (selectedObjects.contains(getConnectionEditPart()))
                viewer.deselect(getConnectionEditPart());
            else
            {
                viewer.appendSelection(getConnectionEditPart());
                _selected = true;
            }
        }
        else if (getCurrentInput().isShiftKeyDown())
        {
            _selected = true;
            viewer.appendSelection(getConnectionEditPart());
        }
        else
        {
            viewer.select(getConnectionEditPart());
            _selected = true;
        }
        return true;
    }

    /**
     * Constructor
     */
    public RoleEndpointTracker(ConnectionEditPart cep)
    {
        super(cep);
    }

    /**
     * @see org.eclipse.gef.tools.ConnectionEndpointTracker#calculateCursor()
     */
    protected Cursor calculateCursor()
    {
        Command command = getCurrentCommand();
        Cursor cursor = getDefaultCursor();
        if (command instanceof MoveRoleCommand)
        {
            MoveRoleCommand moveRoleCommand = (MoveRoleCommand) command;

            // Can a port be attached to more than one roles?
            if (moveRoleCommand.isAttachedToPort())
            {
                //                if (!moveRoleCommand.isAttachedToAvailablePort()) {
                //                    cursor = SharedCursors.NO;
                //                } else {
                cursor = SharedCursors.CURSOR_PLUG;
                //                }
            }
        }
        return cursor;
    }

    /**
     * @return @see org.eclipse.gef.tools.AbstractTool#handleDragStarted()
     */
    protected boolean handleDragStarted()
    {
        if (_selected)
        {
            EditPartViewer viewer = getCurrentViewer();
            viewer.deselect(getConnectionEditPart());
        }
        return super.handleDragStarted();
    }

    /**
     * @param button
     * @return @see org.eclipse.gef.tools.ConnectionEndpointTracker#handleButtonUp(int)
     */
    protected boolean handleButtonUp(int button)
    {
        if (_selected)
        {
            EditPartViewer viewer = getCurrentViewer();
            viewer.select(getConnectionEditPart());
        }
        return super.handleButtonUp(button);
    }

    /*
     * This method calculates the target under the mouse. If the item being
     * moved is a role, then the target under the mouse is calculated at the end
     * of the role.
     * 
     * When an item is obtained, it is stored in the request extended data so
     * that the policies working with the data know what item is involved
     * 
     * @return boolean whether the item under the mouse was updated by this
     * method
     */
    protected boolean updateTargetUnderMouse()
    {
        // If we have a role, the target is not under the mouse, but under the
        // role end
        if (getTargetRequest() instanceof ReconnectRequest)
        {
            ReconnectRequest request = (ReconnectRequest) getTargetRequest();
            boolean changed = false;
            if (!isTargetLocked())
            {
                Collection exclude = getExclusionSet();
                Point end = ((RoleView) ((RoleEditPart) request
                        .getConnectionEditPart()).getFigure()).getEnd();
                ((GraphicalEditPart) request.getConnectionEditPart()
                        .getParent()).getFigure().translateToAbsolute(end);
                if (getCurrentViewer () == null) setViewer (getConnectionEditPart().getViewer ());
                EditPart editPart = getCurrentViewer().findObjectAtExcluding(
                        end, exclude, getTargetingConditional());
                if (editPart != null)
                {
                    editPart = editPart.getTargetEditPart(getTargetRequest());
                }
                changed = getTargetEditPart() != editPart;
                setTargetEditPart(editPart);
            }
            getTargetRequest().getExtendedData().put("over",
                    getTargetEditPart());
            return changed;
        }
        else
        {
            return super.updateTargetUnderMouse();
        }
    }
}