package AcmeStudio.ui.editors.diagram.parts;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.SharedCursors;
import org.eclipse.gef.handles.ConnectionHandle;
import org.eclipse.gef.tools.ConnectionEndpointTracker;

import AcmeStudio.ui.editors.diagram.figures.ElementColorConstants;

/**
 * A handle used at the end of the {@link Connection}.  This is
 * treated differently than the start of the Connection.
 */
final public class RoleMidHandle extends ConnectionHandle {

    private boolean visible = true;
//    private double _scale = 1.0;
    /**
     * Creates a new ConnectionEndHandle, sets its owner to <code>owner</code>,
     * and sets its locator to a {@link ConnectionLocator}.
     */
    public RoleMidHandle(
        ConnectionEditPart owner,
        boolean visible,
        int distance) {
        setCursor(SharedCursors.HAND);
        setOwner(owner);
        setLocator(new RoleLocator(getConnection(), distance));
        this.visible = visible;
    }

    /**
     * Creates a new ConnectionEndHandle.
     */
    public RoleMidHandle() {
        setCursor(SharedCursors.HAND);
    }

    /**
     * Creates and returns a new {@link ConnectionEndpointTracker}.
     */
    protected DragTracker createDragTracker() {
        if (isFixed())
            return null;
        ConnectionEndpointTracker tracker;
        tracker = new RoleEndpointTracker((ConnectionEditPart) getOwner());
        tracker.setCommandName(ConnectionEndpointTracker.REQ_RECONNECT_TARGET);
        tracker.setDefaultCursor(getCursor());
        return tracker;
    }

    /**
     * Draws the handle with fill color and outline color dependent 
     * on the primary selection status of the owner editpart.
     *
     * @param g The graphics used to paint the figure.
     */
    public void paintFigure(Graphics g) {
        if (visible) {
            Point center = getBounds().getCenter();
            Rectangle r = new Rectangle(center, new Dimension(1, 1));
            g.setBackgroundColor(ElementColorConstants.roleFillColor);
            g.fillRectangle(r);
        }
    }

}
