/*
 * Created on Jul 29, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.parts;

import java.util.Vector;

import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.SharedCursors;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.LocationRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.gef.tools.DragEditPartsTracker;
import org.eclipse.swt.graphics.Cursor;

import AcmeStudio.model.ModelHelper;
import AcmeStudio.ui.editors.diagram.actions.MoveRoleCommand;
import AcmeStudio.ui.editors.diagram.figures.PortView;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;

/**
 * @author HongY
 *
 */
public class RoleTracker extends DragEditPartsTracker {

    /* (non-Javadoc)
     * @see org.eclipse.gef.tools.AbstractTool#handleButtonDown(int)
     */
    protected boolean handleButtonDown(int button) {
        boolean ret = super.handleButtonDown(button);
        this.getSourceEditPart().setSelected(EditPart.SELECTED_PRIMARY);
        return ret;
    }

    protected void showSourceFeedback() {
        super.showSourceFeedback();
        Request request = createReconnectRequest();
        if (request != null) {
            this.getSourceEditPart().showSourceFeedback(request);
        }
    }

    /**
     * @param sourceEditPart
     */
    public RoleTracker(EditPart sourceEditPart) {
        super(sourceEditPart);
    }

    protected Command getCommand() {
        ReconnectRequest request = createReconnectRequest();
        if (request == null)
            return null;
        RoleEditPart roleEditPart = (RoleEditPart) getSourceEditPart();
        AcmeRole role = (AcmeRole) roleEditPart.getModel();
        AcmeSystem acmeSystem = Helper.getRootAcmeSystem(role);
        return getMoveRoleCommand(roleEditPart, request, acmeSystem);
    }

    private ReconnectRequest createReconnectRequest() {
        RoleEditPart roleEditPart = (RoleEditPart) getSourceEditPart();
        //        RootEditPart root = roleEditPart.getRoot();
        //        EditPartViewer viewer = root.getViewer();
        RoleView roleView = (RoleView) roleEditPart.getFigure();
        //        AcmeRole role = (AcmeRole) roleEditPart.getModel();
        //        AcmeSystem acmeSystem = Helper.getRootAcmeSystem(role);
        //        SystemEditPart systemEditPart =
        //            (SystemEditPart) viewer.getEditPartRegistry().get(acmeSystem);

        ChangeBoundsRequest targetRequest =
            (ChangeBoundsRequest) getTargetRequest();

        Point location = targetRequest.getLocation();
//        location = new Point (50, 50);
        if (!roleView._isMoving
            && getDistance(roleEditPart, location)
                > getSelectionThreshold(roleView))
            return null;
        ReconnectRequest request = new ReconnectRequest();
        request.setLocation(location);
        request.setConnectionEditPart(roleEditPart);
        request.setType(REQ_RECONNECT_TARGET);
        //request.setTargetEditPart(systemEditPart.findPortEditPart(location));
        return request;
    }

    /**
     * @see org.eclipse.gef.tools.ConnectionEndpointTracker#calculateCursor()
     */
    protected Cursor calculateCursor() {
        Command command = getCurrentCommand();
        Cursor cursor = getDefaultCursor();
        if (command instanceof MoveRoleCommand) {
            MoveRoleCommand moveRoleCommand = (MoveRoleCommand) command;
            //EditPart part = moveRoleCommand.getTargetEditPart();
            // Can a port be attached to more than one roles?
            if (moveRoleCommand.isAttachedToPort()) {
                //                if (!moveRoleCommand.isAttachedToAvailablePort()) {
                //                    cursor = SharedCursors.NO;
                //                } else {
                return SharedCursors.CURSOR_PLUG;
                //                }
            } else
                return SharedCursors.HAND;
        }
        return cursor;
    }

    public static Point getRoleEnd(RoleEditPart roleEditPart) {
        RootEditPart root = roleEditPart.getRoot();
        RoleView roleView = (RoleView) roleEditPart.getFigure();
        Point end = roleView.getEnd();
        Point zoomedEnd = end.getCopy();
        if (root instanceof ScalableFreeformRootEditPart) {
            ScalableFreeformRootEditPart scalableRoot =
                (ScalableFreeformRootEditPart) root;
            ScalableFigure scalablePane = null;
            ZoomManager zm = scalableRoot.getZoomManager();
            if (zm != null && zm.getZoom() != 1.0) {
                scalablePane = zm.getScalableFigure();
                scalablePane.translateToParent(zoomedEnd);
            }
        }
        return zoomedEnd;
    }

    public static void moveRole(
        RoleEditPart roleEditPart,
        LocationRequest request,
        AcmeSystem acmeSystem) {
        RoleView roleView = (RoleView) roleEditPart.getFigure();
        if (!roleView._isMoving) {
            roleView._isMoving = true;
            roleView._endBeforeMoving = roleView.getEnd();
        }

        RootEditPart root = roleEditPart.getRoot();
        EditPartViewer viewer = root.getViewer();
        if (viewer != null) {
            FigureCanvas figureCanvas = (FigureCanvas) viewer.getControl();
            roleView.setFigureCanvas(figureCanvas);
        }

        Point zoomedOffset = roleView.getOffset();
        Point start = roleView.getLastBendpoint();
        Point end = roleView.getEnd();
        Point offset = zoomedOffset.getCopy();
        Point zoomedStart = start.getCopy();
        Point zoomedEnd = end.getCopy();
        Point zoomedLocation = request.getLocation();
        Point location = zoomedLocation.getCopy();

        double scale = 1.0;
        if (root instanceof ScalableFreeformRootEditPart) {
            ScalableFreeformRootEditPart scalableRoot =
                (ScalableFreeformRootEditPart) root;
            ScalableFigure scalablePane = null;
            ZoomManager zm = scalableRoot.getZoomManager();
            if (zm != null && (scale = zm.getZoom()) != 1.0) {
                scalablePane = zm.getScalableFigure();
                scalablePane.translateToParent(zoomedStart);
                scalablePane.translateToParent(zoomedEnd);
                scalablePane.translateFromParent(offset);
                scalablePane.translateFromParent(location);
            }
        }

        double d =
            RoleTracker.calculateSelectionDistance(
                zoomedStart,
                zoomedEnd,
                zoomedLocation,
                roleView.getSelectionDistance(),
                zoomedOffset,
                scale);

        if (roleView.getSelectionDistance() == -1)
            roleView.setSelectionDistance(d);
        Point endPoint =
            RoleTracker.calculateRoleEndPointFromSelection(
                start,
                end,
                location,
                roleView.getSelectionDistance(),
                offset);
        roleView.setEnd(endPoint);

        if (acmeSystem == null) {
            AcmeRole role = (AcmeRole) roleEditPart.getModel();
            acmeSystem = Helper.getRootAcmeSystem(role);
        }
        SystemEditPart systemEditPart =
            (SystemEditPart) viewer.getEditPartRegistry().get(acmeSystem);
        roleEditPart.refreshVisuals();
        systemEditPart.getFigure().repaint();
    }

    public static int getSelectionThreshold(RoleView roleView) {
        ElementVisualization visStyle = roleView.getVisualization();
        Point start = roleView.getLastBendpoint();
        Point end = roleView.getEnd();
        Dimension size = visStyle.getSize();

        int r = (size.width > size.height) ? size.width : size.height;
        r /= 3;
        int d = (int) start.getDistance(end) / 2;
        d = d > 15 * r ? 15 * r : d;
        int rr = (size.width > size.height) ? size.width : size.height;
        if (d > rr * 2)
            d = rr * 2;
        return d;
    }

    public static int getDistance(
        RoleEditPart roleEditPart,
        Point zoomedLocation) {
        RootEditPart root = roleEditPart.getRoot();
        RoleView roleView = (RoleView) roleEditPart.getFigure();

        EditPartViewer viewer = root.getViewer();
        if (viewer != null) {
            FigureCanvas figureCanvas = (FigureCanvas) viewer.getControl();
            roleView.setFigureCanvas(figureCanvas);
        }
        Point zoomedOffset = roleView.getOffset();

        Point end = roleView.getEnd();
        Point zoomedEnd = end.getCopy();
        double zoom = 1.0;
        if (root instanceof ScalableFreeformRootEditPart) {
            ScalableFreeformRootEditPart scalableRoot =
                (ScalableFreeformRootEditPart) root;
            ScalableFigure scalablePane = null;
            ZoomManager zm = scalableRoot.getZoomManager();
            if (zm != null && (zoom = zm.getZoom()) != 1.0) {
                scalablePane = zm.getScalableFigure();
                scalablePane.translateToParent(zoomedEnd);
            }
        }
        Dimension diff = new Dimension();
        diff.height = zoomedOffset.y;
        diff.width = zoomedOffset.x;
        Point location = zoomedLocation.getCopy();
        location.translate(diff);

        return (int) (zoomedEnd.getDistance(location) / zoom);
    }

    public static Command getMoveRoleCommand(
        RoleEditPart roleEditPart,
        ReconnectRequest request,
        AcmeSystem acmeSystem) {
        RoleView roleView = (RoleView) roleEditPart.getFigure();
        if (acmeSystem == null) {
            AcmeRole role = (AcmeRole) roleEditPart.getModel();
            acmeSystem = Helper.getRootAcmeSystem(role);
        }
        if (acmeSystem == null)
            return null;
        RootEditPart root = roleEditPart.getRoot();
        boolean isInPort = false;
        boolean isInAvailablePort = false;

        Point zoomedOffset = roleView.getOffset();
        Point start = roleView.getLastBendpoint();
        Point end = roleView.getEnd();
        Point offset = zoomedOffset.getCopy();
        Point zoomedStart = start.getCopy();
        Point zoomedEnd = end.getCopy();
        Point zoomedLocation = request.getLocation();
        Point location = zoomedLocation.getCopy();
        double scale = 1.0;

        if (root instanceof ScalableFreeformRootEditPart) {
            ScalableFreeformRootEditPart scalableRoot =
                (ScalableFreeformRootEditPart) root;
            ZoomManager zm = scalableRoot.getZoomManager();
            ScalableFigure scalablePane = zm.getScalableFigure();
            if ((scale = zm.getZoom()) != 1.0) {
                scalablePane.translateToParent(zoomedStart);
                scalablePane.translateToParent(zoomedEnd);
                scalablePane.translateFromParent(offset);
                scalablePane.translateFromParent(location);
            }
        }

        if (roleView.getSelectionDistance() == -1.0) {
            double distance =
                RoleTracker.calculateSelectionDistance(
                    zoomedStart,
                    zoomedEnd,
                    zoomedLocation,
                    roleView.getSelectionDistance(),
                    zoomedOffset,
                    scale);
            roleView.setSelectionDistance(distance);
        }
        location =
            RoleTracker.calculateRoleEndPointFromSelection(
                start,
                end,
                location,
                roleView.getSelectionDistance(),
                offset);

        Vector ports = Helper.getPorts(acmeSystem);
        AcmePort port = null;
        PortEditPart portEditPart = null;
        for (int i = 0; i < ports.size(); i++) {
            port = (AcmePort) ports.get(i);
            portEditPart =
                (PortEditPart) root.getViewer().getEditPartRegistry().get(port);
            if (portEditPart == null)
                continue;
            PortView portView = (PortView) portEditPart.getFigure();
            if (portView == null)
                continue;
            Rectangle rec = portView.getBounds();
            if (rec.contains(location)) {
                isInPort = true;
                isInAvailablePort = ModelHelper.isAvailablePort(port);
                break;
            }
        }

        EditPart editPart =
            (EditPart) root.getViewer().getEditPartRegistry().get(acmeSystem);
        if (isInPort) {
            editPart = portEditPart;
        }
        MoveRoleCommand cmd =
            new MoveRoleCommand(editPart, isInPort, isInAvailablePort);
        cmd.setLocation(location);
        
        cmd.setRoleEditPart(roleEditPart);
        
        //if the target is a port, reset the location?
        if (cmd.getTargetEditPart() instanceof PortEditPart)
        {
        	
//            cmd.setLocation(((PortEditPart)cmd.getTargetEditPart()).getFigure().getBounds().getCenter());
//            request.setLocation(((PortEditPart)cmd.getTargetEditPart()).getFigure().getBounds().getCenter()); 
        }
        
        return cmd;
    }

    public static double calculateSelectionDistance(
        Point start,
        Point end,
        Point selection,
        double oldSelectionDistance,
        Point offset,
        double scale) {
        if (oldSelectionDistance > 0)
            return oldSelectionDistance;
        double selectionDistance = oldSelectionDistance;
        double offsetX = offset.x;
        double offsetY = offset.y;
        //            double x0 = (double) start.x;
        //            double y0 = (double) start.y;
        double x2 = end.x;
        double y2 = end.y;
        double x1 = selection.x + offsetX;
        double y1 = selection.y + offsetY;
        //            double deltaX1 = x1 - x0;
        //            double deltaY1 = y1 - y0;
        double deltaX2 = x2 - x1;
        double deltaY2 = y2 - y1;
        //            double d1 = Math.sqrt(deltaX1*deltaX1+deltaY1*deltaY1);
        //            double d2 = Math.sqrt(deltaX2*deltaX2+deltaY2*deltaY2);
        //            double d = Math.sqrt((x2-x0)*(x2-x0)+(y2-y0)*(y2-y0));
        //            if ((d1+d2-d)/d < 0.008) {
        selectionDistance = Math.sqrt(deltaX2 * deltaX2 + deltaY2 * deltaY2);
        //            }
        return selectionDistance / scale;
    }

    public static Point calculateRoleEndPointFromSelection(
        Point start,
        Point end,
        Point selection,
        double selectionDistance,
        Point offset) {
        double x0 = start.x;
        double y0 = start.y;
        double offsetX = offset.x;
        double offsetY = offset.y;
        double x1 = selection.x + offsetX;
        double y1 = selection.y + offsetY;
        double deltaX1 = x1 - x0;
        double deltaY1 = y1 - y0;
        double distance = selectionDistance;
        double alpha = Math.atan(deltaY1 / deltaX1);
        if (deltaX1 < 0) {
            alpha = Math.PI + Math.atan(deltaY1 / deltaX1);
        }
        double deltaX = distance * Math.cos(alpha);
        double deltaY = distance * Math.sin(alpha);
        int x = (int) (x1 + deltaX);
        int y = (int) (y1 + deltaY);
        return new Point(x, y);
    }

}
