package AcmeStudio.ui.editors.diagram.parts;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.FreeformViewport;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.ScalableFreeformLayeredPane;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.CompoundSnapToHelper;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Request;
import org.eclipse.gef.SnapToGeometry;
import org.eclipse.gef.SnapToGrid;
import org.eclipse.gef.SnapToGuides;
import org.eclipse.gef.SnapToHelper;
import org.eclipse.gef.editpolicies.SnapFeedbackPolicy;
import org.eclipse.gef.rulers.RulerProvider;
import org.eclipse.gef.tools.MarqueeDragTracker;
import org.eclipse.ui.internal.WorkbenchPlugin;

import AcmeStudio.ui.editors.diagram.IDiagramModelExtension;
import AcmeStudio.ui.editors.diagram.IElementFactory;
import AcmeStudio.ui.editors.diagram.figures.ElementFigureFactory;
import AcmeStudio.ui.editors.diagram.figures.PortView;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.policies.ElementSelectionEditPolicy;
import AcmeStudio.ui.editors.diagram.policies.SystemXYLayoutEditPolicy;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.elements.AcmeAttachment;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeSystem;
import acmeLib.representations.AcmeBinding;
import acmeLib.representations.AcmeRepresentation;
/**
 * @author Hong Yan (yh@cs.cmu.edu)
 * 
 * The control of AcmeSystem.
 *  
 */
public class SystemEditPart extends ElementEditPart implements LayerConstants, AcmeModelChangeListener {

	public static List	factories	= null;
	
	/** 
	 * A list of lists of ordered children. Each element of the list is a list with a component 
	 * or a connector as its first element, followed by all the children of that child in order 
	**/
	protected LinkedList _orderedChildren;

	protected static void createElementFactories() {
		factories = new LinkedList ();
		try {
			IExtensionRegistry registry = Platform.getExtensionRegistry();
			IExtensionPoint point = registry.getExtensionPoint ("AcmeStudio.elementFactory");
			IExtension [] extensions = point.getExtensions ();
			for (int i = 0; i < extensions.length; i++) {
				IConfigurationElement [] configs = extensions [i].getConfigurationElements ();
				if (configs != null && configs.length > 0) {
					String id = configs [0].getAttribute ("id"); //$NON-NLS-1$
					if (id != null) {
						factories.add (WorkbenchPlugin.createExtension (configs [0], "class"));
					}
				}
			}
		} catch (CoreException e) {
			e.printStackTrace ();
		}
	}

	/**
	 * The selection handle edit policy applied to this system edit part
	 */
	//    private SelectionHandlesEditPolicy _selectionEditPolicy = null;
	//    private SelectionHandlesEditPolicy _bendpointSelectionEditPolicy = null;
	public static class BindingPoint /* extends AcmeElement */{

		public AcmeElement			_outer;
		public List					_associatedBindings;
		public AcmeRepresentation	_parentRep;
		public Point				location	= null;
		public Dimension			size		= null;

		public BindingPoint() {
		}

		public boolean equals(Object o) {
			return (o instanceof BindingPoint) && ((BindingPoint )o)._outer == _outer;
		}
	}

	public static class Binding /* extends AcmeElement */{

		public AcmeBinding	_binding;
		public BindingPoint	_outer;

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Object#hashCode()
		 */
		public int hashCode() {
			//			return _binding.hashCode ();
			return super.hashCode ();
		}

		public boolean equals(Object o) {
			return (o instanceof Binding) && ((Binding )o)._binding == _binding;
		}
	}
	/**
	 * The binding points
	 */
	private List	_bindingPoints	= new ArrayList ();

	private static final Object	START_SENTINEL	= new Integer (-1);

	/**
	 * Installs EditPolicies specific to this.
	 */
	protected void createEditPolicies() {
		super.createEditPolicies ();
		updateEditPolicies ();
		createElementFactories ();
	}

	/**
	 * Installs EditPolicies specific to this.
	 */
	public void updateEditPolicies() {
		IFigure view = getFigure ();
		if (view != null) { //.getIsVisible()) {
			installEditPolicy (EditPolicy.LAYOUT_ROLE, new SystemXYLayoutEditPolicy ());
			installEditPolicy (EditPolicy.SELECTION_FEEDBACK_ROLE, new ElementSelectionEditPolicy ());
			installEditPolicy ("Snap Feedback", new SnapFeedbackPolicy ());
			//super.createEditPolicies();
		} else {
			if (getEditPolicy (EditPolicy.LAYOUT_ROLE) != null) {
				removeEditPolicy (EditPolicy.LAYOUT_ROLE);
			}
			if (getEditPolicy (EditPolicy.SELECTION_FEEDBACK_ROLE) != null) {
				removeEditPolicy (EditPolicy.SELECTION_FEEDBACK_ROLE);
			}
			if (getEditPolicy ("Snap Feedback") != null) {
				removeEditPolicy ("Snap Feedback");
			}
		}
	}

	/**
	 * Returns a Figure to represent this.
	 * 
	 * @return Figure.
	 */
	protected IFigure createFigure() {
		SystemView view = (SystemView )ElementFigureFactory.createSystemView ();
		
		view.setLayoutManager (new FreeformLayout ());
		view.setBorder (new MarginBorder (5));
		view.setBindingPoints (_bindingPoints);
		AcmeSystem acmeSystem = (AcmeSystem )getModel ();
		if (acmeSystem != null) {
			view.setModel (acmeSystem);
		}
		view.setPart (this);
		return view;
	}

	/**
	 * Returns the children of this through the model.
	 * 
	 * @return Children of this as a List.
	 */
	protected List getModelChildren() {
		fillBindingPoints ();
		AcmeSystem sys = (AcmeSystem )getModel ();

		List lChildren = null;
		if (_orderedChildren == null) {
			_orderedChildren = Helper.getOrderedChildren(sys);
		}
		lChildren = new LinkedList ();
//		if (sys.getParentRepresentation () != null) {
//			lChildren.add (new AcmeRepresentationModel (sys));
//		}
		
		for (Iterator iter = _orderedChildren.iterator (); iter.hasNext ();) {
			List ch = (List )iter.next ();
			lChildren.addAll (ch);
		}
		lChildren.addAll (_bindingPoints);
		lChildren.addAll (getBindings ());
		Iterator iter = factories.iterator ();
		while (iter.hasNext ()) {
			IElementFactory factory = (IElementFactory )iter.next ();
			List models = factory.getModels (sys);
			if (models != null && models.size () > 0)
				lChildren.addAll (models);

			
		}
		
		// now go about reordering them
		iter = factories.iterator();
		while (iter.hasNext ()) {
			IElementFactory factory = (IElementFactory )iter.next ();
			List models = factory.getModels (sys);
			for (Iterator iterator = models.iterator (); iterator.hasNext ();) {
				Object o = iterator.next ();
				if (o instanceof IDiagramModelExtension) {
					IDiagramModelExtension dm = (IDiagramModelExtension )o;
					int oldIndex = lChildren.indexOf(dm);
					if (oldIndex == dm.getZOrder ()) continue;
					
					if (dm.getZOrder () != IDiagramModelExtension.UNDEFINED_ZORDER) {
						int newIndex = dm.getZOrder ();
						if (dm.getZOrder () > oldIndex) {
							newIndex --;
						}
						if (newIndex >= lChildren.size ()) newIndex = lChildren.size () - 1;
						lChildren.remove (dm);
						lChildren.add (newIndex, dm);
					}
					else {
						Object ifo = dm.getInFrontOf();
						if (ifo == null) continue;
						if (ifo == START_SENTINEL) {
							lChildren.remove (dm);
							lChildren.add (0, dm);
							dm.setZOrder(0);
						}
						else {
							int newIndex = lChildren.indexOf (ifo) + 1;
							dm.setZOrder (newIndex);
							if (oldIndex > newIndex) newIndex --;
							if (newIndex >= lChildren.size ()) newIndex = lChildren.size () - 1;
							if (newIndex != -1) {
								lChildren.remove(dm);
								lChildren.add(newIndex, dm);
							}
						}
					}
				}
			}
		}
		return lChildren;
	}

	protected List getBindings() {
		List existingBindings = Helper.getBindings ((AcmeSystem )getModel ());
		List bindings = new ArrayList ();
		Iterator i = _bindingPoints.iterator ();
		while (i.hasNext ()) {
			BindingPoint bp = (BindingPoint )i.next ();
			Iterator j = bp._associatedBindings.iterator ();
			while (j.hasNext ()) {
				AcmeBinding b = (AcmeBinding )j.next ();
				if (existingBindings.contains (b)) {
					//                    Binding binding = (Binding )_cachedBindings.get (b);
					//                    if (binding == null) {
					Binding binding = new Binding ();
					binding._binding = b;
					binding._outer = bp;
					//                    	_cachedBindings.put (b, binding);
					//                    }
					if (b.getInner () != null) {
						bindings.add (binding);
					}
				}
			}
		}
		return bindings;
	}

	protected void fillBindingPoints() {
		_bindingPoints.clear ();
		List bindings = Helper.getBindings ((AcmeSystem )getModel ());
		List emptyBindings = Helper.getAdditionalBindingPoints ((AcmeSystem )getModel (), bindings);
		bindings.addAll (emptyBindings);
		Iterator i = bindings.iterator ();
		while (i.hasNext ()) {
			Object obj = i.next ();
			if (obj instanceof AcmeBinding) {
				AcmeBinding binding = (AcmeBinding )obj;
				BindingPoint bp = new BindingPoint ();
				bp._parentRep = ((AcmeSystem )getModel ()).getParentRepresentation ();
				bp._outer = binding.getOuter ();
				int index = _bindingPoints.indexOf (bp);
				if (index != -1) {
					bp = (BindingPoint )_bindingPoints.get (index);
					if (!bp._associatedBindings.contains (binding)) {
						bp._associatedBindings.add (binding);
					}
				} else {
					bp._associatedBindings = new ArrayList ();
					bp._associatedBindings.add (binding);
					_bindingPoints.add (bp);
				}
			}
		}
		//        _bindingPoints = Helper.getBindings((AcmeSystem) getModel());
		//        
		//        List additionalEmptyPoints =
		//            Helper.getAdditionalBindingPoints(
		//                (AcmeSystem) getModel(),
		//                _bindingPoints);
		//        _bindingPoints.addAll(additionalEmptyPoints);
	}

	/**
	 * Set the selection handle edit policy
	 * 
	 * @param policy
	 *            The policy
	 */
	//    public void setSelectionHandlesEditPolicy(SelectionHandlesEditPolicy
	// policy) {
	//        this._selectionEditPolicy = policy;
	//    }
	//
	//    public void
	// setBendpointSelectionHandlesEditPolicy(SelectionHandlesEditPolicy
	// policy) {
	//        this._bendpointSelectionEditPolicy = policy;
	//    }
	//
	//    /**
	//     * Get the selection handle edit policy
	//     * @return policy The policy
	//     */
	//    public SelectionHandlesEditPolicy getSelectionHandlesEditPolicy() {
	//        return _selectionEditPolicy;
	//    }
	//
	//    public SelectionHandlesEditPolicy
	// getBendpointSelectionHandlesEditPolicy() {
	//        return _bendpointSelectionEditPolicy;
	//    }
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getDragTracker(Request)
	 */
	public DragTracker getDragTracker(Request req) {
		return new MarqueeDragTracker ();
	}

	/**
	 * Get the target edit part
	 * 
	 * @param request
	 *            The request
	 * @return this
	 */
	public EditPart getTargetEditPart(Request request) {
		return this;
	}

	/**
	 * @see org.eclipse.gef.EditPart#setModel(Object)
	 */
	public void setModel(Object model) {
		super.setModel (model);
		_orderedChildren = null;
		fillBindingPoints ();
	}

	/**
	 * @see org.eclipse.gef.EditPart#setParent(EditPart)
	 */
	public void setParent(EditPart parent) {
		super.setParent (parent);
		//        if (_bindingPoints == null || _bindingPoints.isEmpty())
		//            return;
		//        Iterator bIt = _bindingPoints.iterator();
		//        while (bIt.hasNext()) {
		//            AcmeBinding binding = (AcmeBinding) bIt.next();
		//
		//            BindingPointEditPart bindingPoint =
		//                (BindingPointEditPart) getRoot()
		//                    .getViewer()
		//                    .getEditPartRegistry()
		//                    .get(
		//                    binding);
		//            if (bindingPoint != null) {
		//                BindingPointView bpv =
		//                    (BindingPointView) bindingPoint.getFigure();
		//                bpv.adjustViewToRepresentation(
		//                    ((SystemView) this.getFigure())
		//                        .getComponentRepresentationBorder());
		//                this.getFigure().add(bpv);
		//            }
		//        }
	}

	public void refreshAttachments() {
		AcmeSystem acmeSystem = (AcmeSystem )getModel ();
		Vector attachments = Helper.getAttachments (acmeSystem);
		for (int i = 0; i < attachments.size (); i++) {
			AcmeAttachment attachment = (AcmeAttachment )attachments.elementAt (i);
			AttachmentEditPart attachmentEditPart = (AttachmentEditPart )getRoot ().getViewer ().getEditPartRegistry ().get (
					attachment);
			if (attachmentEditPart == null)
				continue;
			attachmentEditPart.refreshVisuals ();
		}
	}

	public void refresh() {
		super.refresh ();
		SystemView sv = null;
		if (getFigure () instanceof FreeformViewport)
			sv = (SystemView )((ScalableFreeformLayeredPane )getFigure ().getChildren().get (getFigure ().getChildren ().size() - 1)).getChildren().get (0);
		else 
			sv = (SystemView )getFigure ();
		if (sv != null) {
			Hashtable childLocations = sv.getChildLocations ();
			if (childLocations != null && childLocations.size () > 0) {
				List children = getChildren ();
				for (int i = 0; i < children.size (); i++) {
					EditPart editPart = (EditPart )children.get (i);
					if (editPart instanceof ComponentEditPart) {
						ComponentEditPart cep = (ComponentEditPart )editPart;
						AcmeComponent ac = (AcmeComponent )cep.getModel ();
						if (childLocations.containsKey (ac.getName ())) {
							editPart.refresh ();
						}
					} else if (editPart instanceof ConnectorEditPart) {
						ConnectorEditPart cep = (ConnectorEditPart )editPart;
						AcmeConnector ac = (AcmeConnector )cep.getModel ();
						if (childLocations.containsKey (ac.getName ())) {
							editPart.refresh ();
						}
					}
				}
			}
		}
//		refreshAttachments ();
	}

	public PortEditPart findPortEditPart(Point location) {
		SystemView sv = (SystemView )getFigure ();
		if (sv != null) {
			List children = getChildren ();
			for (int i = 0; i < children.size (); i++) {
				EditPart editPart = (EditPart )children.get (i);
				if (editPart instanceof PortEditPart) {
					PortEditPart pep = (PortEditPart )editPart;
					PortView pv = (PortView )pep.getFigure ();
					if (pv.getBounds ().contains (location)) {
						return pep;
					}
				}
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		
		if (adapter == SnapToHelper.class) {
			int numStrategy = 0;
			Object rulerVisible = getViewer ().getProperty (RulerProvider.PROPERTY_RULER_VISIBILITY);
			if (rulerVisible != null && Boolean.TRUE.equals (rulerVisible)) {
				numStrategy++;
			}
			Object snapEnabled = getViewer ().getProperty (SnapToGeometry.PROPERTY_SNAP_ENABLED);
			if (Boolean.TRUE.equals (snapEnabled)) {
				numStrategy++;
			}
			Object gridEnabled = getViewer ().getProperty (SnapToGrid.PROPERTY_GRID_ENABLED);
			if (Boolean.TRUE.equals (gridEnabled)) {
				numStrategy++;
			}
			if (numStrategy > 0) {
				SnapToHelper ss[] = new SnapToHelper [numStrategy];
				int index = 0;
				if (Boolean.TRUE.equals (rulerVisible))
					ss [index++] = new SnapToGuides (this);
				if (Boolean.TRUE.equals (snapEnabled))
					ss [index++] = new AcmeStudio.ui.editors.diagram.policies.SnapToGeometry (this);
				if (Boolean.TRUE.equals (gridEnabled))
					ss [index++] = new SnapToGrid (this);
				return new CompoundSnapToHelper (ss);
			}
		}
		return super.getAdapter (adapter);
	}

	/**
	 * Makes sure that the visual information of all the elements is right, based on their order in _orderedChildren
	 * This needs to be called just before a system is saved.
	 */
	public void injectOrderInformation() {
		int index = 0;
		for (Iterator iter = getModelChildren ().iterator (); iter.hasNext ();) {
			Object obj = iter.next ();
			if (obj instanceof AcmeElement) {
				AcmeElement element = (AcmeElement )obj;
				Helper.setZOrder (element, index++);
			}
			else if (obj instanceof IDiagramModelExtension) {
				IDiagramModelExtension dme = (IDiagramModelExtension )obj;
				dme.setZOrder (index++);
			}
			else index++;
		}
//		
//		int index = 0;
//		for (Iterator iter = _orderedChildren.iterator (); iter.hasNext ();) {
//			List elementList = (List )iter.next ();
//			for (Iterator iter2 = elementList.iterator (); iter2.hasNext ();) {
//				AcmeElement element = (AcmeElement )iter2.next ();
//				Helper.setZOrder (element, index++);
//			}
//		}
	}

	/**
	 * When elements are added or removed, keep the orderedChildren updated
	 * @param evt
	 * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
	 **/
	public void changeModel(AcmeStudioChangeEvent evt) {
		if (Helper.eventRelatedToDesign(evt, ((AcmeSystem )getModel ()).getParentDesign ())) {
			if (!(evt.getTarget () instanceof AcmeElement)) return;
			AcmeElement target = (AcmeElement )evt.getTarget();
			if (Helper.getAcmeSystem(target) != getModel ()) return;
			switch (evt.getEventType()) {
				case AcmeStudioChangeManager.ADD_PORT:
				case AcmeStudioChangeManager.ADD_ROLE:
				{
					AcmeElement elementToLookFor = ((AcmeElement )evt.getSource ()).getParent ();
					int elementIndex = -1;
					
					List elementList = null;
					for (Iterator iter = _orderedChildren.iterator () ; iter.hasNext () && elementIndex == -1;) {
						List list = (List )iter.next ();
						if (list.get (0).equals (elementToLookFor)) {
							elementIndex = _orderedChildren.indexOf (list);
							elementList = list;
						}
					}
					if (elementList != null) {
						elementList.add (1, evt.getSource ());
					}
				}
				break;
				case AcmeStudioChangeManager.REMOVE_ROLE:
				case AcmeStudioChangeManager.REMOVE_PORT:
				{
					/* BugFix#309: Don't look for parent, look for target (because parent == null) */
					AcmeElement elementToLookFor = (AcmeElement )evt.getTarget ();
					int elementIndex = -1;
					
					List elementList = null;
					for (Iterator iter = _orderedChildren.iterator () ; iter.hasNext () && elementIndex == -1;) {
						List list = (List )iter.next ();
						if (list.get (0).equals (elementToLookFor)) {
							elementIndex = _orderedChildren.indexOf (list);
							elementList = list;
						}
					}
					if (elementList != null) {
						elementList.remove (evt.getSource ());
					}
				}
				break;
				case AcmeStudioChangeManager.ADD_COMPONENT: {
					List componentList = new LinkedList ();
					componentList.add(evt.getSource ());
					componentList.addAll (Helper.getChildren((AcmeComponent )evt.getSource ()));
					_orderedChildren.addFirst(componentList);
					break;
				}
				case AcmeStudioChangeManager.REMOVE_COMPONENT: 
				case AcmeStudioChangeManager.REMOVE_CONNECTOR:{
					Object elementToLookFor = evt.getSource ();
					int elementIndex = -1;
					
					List elementList = null;
					for (Iterator iter = _orderedChildren.iterator () ; iter.hasNext () && elementIndex == -1;) {
						List list = (List )iter.next ();
						if (list.get (0).equals (elementToLookFor)) {
							elementIndex = _orderedChildren.indexOf (list);
							elementList = list;
						}
					}
					_orderedChildren.remove (elementList);
					getFigure ().repaint ();
					break;
				}
				case AcmeStudioChangeManager.ADD_CONNECTOR: {
					List connectorList = new LinkedList ();
					connectorList.add(evt.getSource ());
					connectorList.addAll (Helper.getChildren((AcmeConnector )evt.getSource ()));
					_orderedChildren.addFirst(connectorList);
					break;
				}
				
			}
			refresh ();
//			// Output ordered chilren
//			Debug.diagnoseln ("SystemEditPart.changeModel: " + AcmeStudioChangeManager.eventTypeAsString(evt.getEventType()) + " for system " + ((AcmeSystem )getModel ()).getQualifiedName());
//			StringBuffer bf = new StringBuffer ();
//			Iterator it = _orderedChildren.iterator ();
//			while (it.hasNext ()) {
//				List l = (List )it.next();
//				AcmeElement el = (AcmeElement )l.get (0);
//				bf.append(el.getName());
//				bf.append(" ");
//			}
//			Debug.diagnoseln ("SystemEditPart.changeModel: " + bf.toString());
		}
	}

	/**
	 * @param eventType
	 * @return
	 * @see AcmeStudio.util.AcmeModelChangeListener#interestedInEvent(int)
	 **/
	public boolean interestedInEvent(int eventType) {
		switch (eventType) {
			case AcmeStudioChangeManager.ADD_COMPONENT:
			case AcmeStudioChangeManager.ADD_CONNECTOR:
			case AcmeStudioChangeManager.ADD_PORT:
			case AcmeStudioChangeManager.ADD_ROLE:
			case AcmeStudioChangeManager.REMOVE_COMPONENT:
			case AcmeStudioChangeManager.REMOVE_CONNECTOR:
			case AcmeStudioChangeManager.REMOVE_PORT:
			case AcmeStudioChangeManager.REMOVE_ROLE:
				return true;
			default:
				return false;
		}
	}
	
	
	/**
	 * 
	 * @see AcmeStudio.ui.editors.diagram.parts.ElementEditPart#activate()
	 **/
	public void activate() {
		super.activate ();
		AcmeStudioChangeManager.registerPriorityListener(this);
	}
	
	
	/**
	 * 
	 * @see AcmeStudio.ui.editors.diagram.parts.ElementEditPart#deactivate()
	 **/
	public void deactivate() {
		super.deactivate ();
		AcmeStudioChangeManager.removeListener(this);
	}

	/**
	 * Moves a part forward in the diagram
	 * @param part the part to move
	 */
	public void moveForward(EditPart part) {
		int increment = 1;
		move (part, increment);
	}

	/**
	 * Move a part back in the diagram
	 * @param part the Part to move
	 */
	public void moveBack(EditPart part) {
		int increment = -1;
		move (part, increment);

		
	}

	/**
	 * Moves the order of children by an increment
	 * @param part The part to reorder
	 * @param increment Whether to reorder forward or backward
	 */
	private void move(EditPart part, int increment) {
		if (part.getModel () instanceof IDiagramModelExtension) {
			// This stuff works, but can't work out how to persist the order
			List children = getModelChildren ();
			int index = children.indexOf (part.getModel ());
			int newIndex = index + increment - 1;
			if (newIndex < children.size ()) {
				Object inFrontOf = children.get (newIndex);
				((IDiagramModelExtension )part.getModel ()).setInFrontOf (inFrontOf);
				((IDiagramModelExtension )part.getModel ()).setZOrder (IDiagramModelExtension.UNDEFINED_ZORDER);
			}
			return;
		}
		Object elementToLookFor = null;
		boolean movingChild = false;
		if (part instanceof RoleEditPart || part instanceof PortEditPart) {
			elementToLookFor = ((AcmeElement )part.getModel ()).getParent ();
			movingChild = true;
		}
		else {
			elementToLookFor = ((AcmeElement )part.getModel ());
		}
			
		int elementIndex = -1;
		
		List elementList = null;
		for (Iterator iter = _orderedChildren.iterator () ; iter.hasNext () && elementIndex == -1;) {
			List list = (List )iter.next ();
			if (list.get (0).equals (elementToLookFor)) {
				elementIndex = _orderedChildren.indexOf (list);
				elementList = list;
			}
		}
		
		if (movingChild) {
			if (elementIndex != -1) {
				int index = elementList.indexOf (part.getModel ());
				if (index > 1 && index < elementList.size () - 1)  {
					elementList.remove (part.getModel ());
					index += increment;
					elementList.add (index, part.getModel ());
					return;
				}
			}
		}
		
		if ((elementIndex == 0 && increment < 0)|| (elementIndex == _orderedChildren.size () - 1 && increment > 0)) return;
		_orderedChildren.remove (elementIndex);
		_orderedChildren.add (elementIndex + increment, elementList);
	}

	/**
	 * Moves part to the back of the diagram
	 * @param part the part to move
	 */
	public void moveToBack(EditPart part) {
		if (part.getModel () instanceof IDiagramModelExtension) {
			IDiagramModelExtension dme = (IDiagramModelExtension )part.getModel ();
			dme.setInFrontOf(START_SENTINEL);
			dme.setZOrder(IDiagramModelExtension.UNDEFINED_ZORDER);
		}
		else {
			AcmeElement elementToLookFor = null;
			boolean movingChild = false;
			if (part instanceof RoleEditPart || part instanceof PortEditPart) {
				elementToLookFor = ((AcmeElement )part.getModel ()).getParent ();
				movingChild = true;
			}
			else {
				elementToLookFor = ((AcmeElement )part.getModel ());
				int elementIndex = -1;
				List elementList = null;
				for (Iterator iter = _orderedChildren.iterator (); iter.hasNext () && elementIndex == -1;) {
					List list = (List )iter.next ();
					if (list.get (0).equals (elementToLookFor)) {
						elementIndex = _orderedChildren.indexOf (list);
						elementList = list;
					}
				}
				if (movingChild) {
					elementList.remove (part.getModel ());
					elementList.add (0, part.getModel ());
				}
				else {
					_orderedChildren.remove (elementList);
					_orderedChildren.addFirst (elementList);
				}
			}
		}
	}

	/**
	 * Move a part to the front of the diagam
	 * 
	 * @param part
	 *                    the part to move
	 */
	public void moveToFront(EditPart part) {
		if (part.getModel () instanceof IDiagramModelExtension) {
			IDiagramModelExtension model = (IDiagramModelExtension )part.getModel ();
			List children = getModelChildren ();
			Object first = children.get(children.size() - 1);
			if (first != part)  {
				model.setInFrontOf(first);
				model.setZOrder(IDiagramModelExtension.UNDEFINED_ZORDER);
			}
			return;
		}
		AcmeElement elementToLookFor = null;
		boolean movingChild = false;
		if (part instanceof RoleEditPart || part instanceof PortEditPart) {
			elementToLookFor = ((AcmeElement )part.getModel ()).getParent ();
			movingChild = true;
		}
		else {
			elementToLookFor = ((AcmeElement )part.getModel ());
		}
			
		int elementIndex = -1;
		
		List elementList = null;
		for (Iterator iter = _orderedChildren.iterator () ; iter.hasNext () && elementIndex == -1;) {
			List list = (List )iter.next ();
			if (list.get (0).equals (elementToLookFor)) {
				elementIndex = _orderedChildren.indexOf (list);
				elementList = list;
			}
		}
		
		if (movingChild) {
			elementList.remove (part.getModel ());
			elementList.add (part.getModel ());
		}
		else {
			_orderedChildren.remove(elementList);
			_orderedChildren.addLast(elementList);
		}
	}
	
	
	/**
	 * @param request
	 * @see org.eclipse.gef.EditPart#showTargetFeedback(org.eclipse.gef.Request)
	 **/
	public void showTargetFeedback(Request request) {
		super.showTargetFeedback(request);
	}
}