package AcmeStudio.ui.editors.diagram.policies;

import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Cursors;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Locator;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.PrecisionPoint;
import org.eclipse.draw2d.geometry.PrecisionRectangle;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Handle;
import org.eclipse.gef.handles.AbstractHandle;
import org.eclipse.gef.handles.RelativeHandleLocator;
import org.eclipse.gef.handles.ResizableHandleKit;
import org.eclipse.gef.tools.ResizeTracker;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Cursor;

import AcmeStudio.ui.editors.diagram.figures.RoleDecoration;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.parts.PortEditPart;
import AcmeStudio.ui.editors.diagram.parts.ResizePortTracker;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import acmeLib.elements.AcmeElement;
/**
 * @author schmerl
 * 
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates. To enable and disable the creation of
 * type comments go to Window>Preferences>Java>Code Generation.
 */
public class ConstrainedResizeHandleKit {

	/**
	 * 
	 * Relocates a handle based on the rotation of the role
	 *
	 */
	public static class RoleHandleLocator extends RelativeHandleLocator {

		private double relativeY;
		private double relativeX;

		/**
		 * @param reference
		 * @param location
		 */
		public RoleHandleLocator(RoleDecoration role, int location) {
			super(role, location);
			switch (location & PositionConstants.NORTH_SOUTH) {
			case PositionConstants.NORTH:
				relativeY = 0; break;
			case PositionConstants.SOUTH:
				relativeY = 1.0; break;
			default:
				relativeY = 0.5;
			}
			
			switch (location & PositionConstants.EAST_WEST) {
			case PositionConstants.WEST:
				relativeX = 0; break;
			case PositionConstants.EAST:
				relativeX = 1.0; break;
			default:
				relativeX = 0.5;
			}
	
		}

		/**
		 * @param target
		 * @see org.eclipse.draw2d.Locator#relocate(org.eclipse.draw2d.IFigure)
		 */
		public void relocate(IFigure target) {
			RoleDecoration reference = (RoleDecoration )getReferenceFigure();
			Rectangle targetBounds = new PrecisionRectangle(getReferenceBox().getResized(-1, -1));
			reference.translateToAbsolute(targetBounds);
			target.translateToRelative(targetBounds);
			
			
			targetBounds.resize(1, 1);
			PrecisionPoint center = new PrecisionPoint (targetBounds.getCenter());
			Dimension targetSize = target.getPreferredSize();

			targetBounds.x
				+= (int) (targetBounds.width * relativeX);
			targetBounds.y
				+= (int) (targetBounds.height * relativeY);
			targetBounds.setSize(targetSize);

			// rotate target bounds location according to the role's rotation
			double rotation = reference.getRotation();
			double theta = Math.atan2 (targetBounds.y - center.y, targetBounds.x - center.x) + rotation;
			double r = targetBounds.getLocation().getDistance(center);
			targetBounds.x = (int )(center.x + r * Math.cos (theta) -((targetSize.width + 1)/2));
			targetBounds.y = (int )(center.y + r * Math.sin (theta) -((targetSize.height + 1)/2));
			target.setBounds(targetBounds);		
			
		}
	}
	public static class RoundResizeHandle extends AbstractHandle {
		private int cursorDirection = 0;
		protected static final int DEFAULT_HANDLE_SIZE = 7;
		
		{
			init ();
		}
		
		public RoundResizeHandle () {}
		
		public RoundResizeHandle (GraphicalEditPart owner, Locator loc) {
			super (owner, loc);
		}
		
		public RoundResizeHandle (GraphicalEditPart owner, Locator loc, Cursor c) {
			super (owner, loc, c);
		}
		
		/**
		 * Creates a new RoundResizeHandle for the given GraphicalEditPart.
		 * <code>direction</code> is the relative direction from the 
		 * center of the owner figure.  For example, <code>SOUTH_EAST</code>
		 * would place the handle in the lower-right corner of its
		 * owner figure.  These direction constants can be found in
		 * {@link org.eclipse.draw2d.PositionConstants}.
		 * @param owner owner of the ResizeHandle
		 * @param direction relative direction from the center of the owner figure
		 */
		public RoundResizeHandle(GraphicalEditPart owner, int direction) {
			setOwner(owner);
			setLocator(new RelativeHandleLocator(owner.getFigure(), direction));
			setCursor(Cursors.getDirectionalCursor(direction));
			cursorDirection = direction;
		}		
		
		protected Color getBorderColor () {
			return (isPrimary ()?ColorConstants.white:ColorConstants.black);
		}
		
		protected Color getFillColor () {
			return (isPrimary ()?ColorConstants.black:ColorConstants.white);
		}
		
		protected void init () {
			setPreferredSize(new Dimension(DEFAULT_HANDLE_SIZE, DEFAULT_HANDLE_SIZE));	
		}
		/* (non-Javadoc)
		 * @see org.eclipse.gef.handles.AbstractHandle#createDragTracker()
		 */
		protected DragTracker createDragTracker() {
			return new ResizeTracker (getOwner (), cursorDirection);
		}
		/**
		 * Returns <code>true</code> if the handle's owner is the primary selection.
		 * @return <code>true</code> if the handles owner has primary selection.
		 */
		protected boolean isPrimary() {
			return getOwner().getSelected() == EditPart.SELECTED_PRIMARY;
		}	
		
		/**
		 * Draws the handle with fill color and outline color dependent 
		 * on the primary selection status of the owner editpart.
		 *
		 * @param g The graphics used to paint the figure.
		 */
		public void paintFigure(Graphics g) {
			Rectangle r = getBounds();
			r.shrink(1, 1);
			try {
				g.setBackgroundColor(getFillColor());
				g.fillOval(r.x, r.y, r.width, r.height);
				g.setForegroundColor(getBorderColor()); 
				g.drawOval(r.x, r.y, r.width, r.height);
			} finally {
				//We don't really own rect 'r', so fix it.
				r.expand(1, 1);
			}
		}
	}
	
	
	public static void addHandles(GraphicalEditPart part, List handles) {
		if (part instanceof RoleEditPart) {
//			RoleView roleView = (RoleView) part.getFigure();
//			Figure roleEnd = roleView.getDecoration();
//			MoveHandle moveHandle = new MoveHandle (part, new MoveHandleLocator (roleEnd));
//			handles.add (moveHandle);
			createResizeHandles(part, handles);
		}
		else if (part.getModel () instanceof AcmeElement) {
			ResizableHandleKit.addMoveHandle (part, handles);
			createResizeHandles(part, handles);
		}
		else {
			ResizableHandleKit.addHandles (part, handles);
		}
	}

	/**
	 * @param part
	 * @param handles
	 */
	private static void createResizeHandles(GraphicalEditPart part, List handles) {
		ElementVisualization vis = VisualStyleRegistry.getVisualization ((AcmeElement )part.getModel ());
		if (vis != null) {
			if (vis.canResizeHeight ()) {
				handles.add (createHandle (part, PositionConstants.NORTH));
				handles.add (createHandle (part, PositionConstants.SOUTH));
			}
			if (vis.canResizeWidth ()) {
				handles.add (createHandle (part, PositionConstants.EAST));
				handles.add (createHandle (part, PositionConstants.WEST));
			}
			if (vis.canResizeHeight () && vis.canResizeWidth ()) {
				handles.add (createHandle (part, PositionConstants.SOUTH_EAST));
				handles.add (createHandle (part, PositionConstants.SOUTH_WEST));
				handles.add (createHandle (part, PositionConstants.NORTH_WEST));
				handles.add (createHandle (part, PositionConstants.NORTH_EAST));
			}
		}
	}

	static Handle createHandle(GraphicalEditPart owner, int direction) {
		RoundResizeHandle handle = null;
		if (owner instanceof RoleEditPart) {
			RoleView roleView = (RoleView )owner.getFigure ();
			RoleDecoration decoration = roleView.getDecoration();
			handle = new RoundResizeHandle (owner, new RoleHandleLocator (decoration, direction));
			
			
		}
		else {
			handle = new RoundResizeHandle (owner, direction);
		}
			
		ResizeTracker tracker;
		if (owner instanceof PortEditPart) {
			tracker = new ResizePortTracker (owner, direction);
		}
		else if (owner instanceof RoleEditPart) {
			tracker = new ResizeRoleTracker ((RoleEditPart )owner, direction);
		}
		else 
			tracker = new ResizeTracker (owner, direction);
		handle.setDragTracker (tracker);
		return handle;
		
	}
}
