/*
 * Created on Jan 19, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.policies;

import org.eclipse.gef.tools.CellEditorLocator;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Text;

import AcmeStudio.ui.editors.diagram.figures.IAcmeElementFigure;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.constants.LabelOptions;



/**
 * @author Bradley Schmerl
 * Created Jan 19, 2004
 */
public class ElementNameCellEditorLocator implements CellEditorLocator {
	private static int WIN_X_OFFSET = -4;
	private static int WIN_W_OFFSET = 5;
	private static int GTK_X_OFFSET = 0;
	private static int GTK_W_OFFSET = 0;
	private static int MAC_X_OFFSET = -3;
	private static int MAC_W_OFFSET = 9;
	private static int MAC_Y_OFFSET = -3;
	private static int MAC_H_OFFSET = 6;
	
	private IAcmeElementFigure _figure;
	private Point	_size = new Point (-1,-1);


	/**
	 * 
	 */
	public ElementNameCellEditorLocator(IAcmeElementFigure figure) {
		super ();
		_figure = figure;

	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.tools.CellEditorLocator#relocate(org.eclipse.jface.viewers.CellEditor)
	 */
	public void relocate(CellEditor celleditor) {
		Text text = (Text )celleditor.getControl ();
		if (text == null) return;
		refreshSize (text);

		Point pref = text.computeSize (-1, -1);
//		Debug.diagnoseln ("ElementNameCellEditorLocator.relocate: " + "size = " + _size.toString ());
		if (pref.x < _size.x) pref.x = _size.x;
//		Debug.diagnoseln ("ElementNameCellEditorLocator.relocate: " + "after size = " + _size.toString ());
		ElementVisualization vis = _figure.getVisualization();
		
		org.eclipse.draw2d.geometry.Point pos;
		if (vis != null && vis.getLabelSource() == LabelOptions.ACMEID) {
		  pos = vis.getLabelBounds (_figure.getModel (), _figure.getBounds ()).getCopy ().getTopLeft();
		}
		else {
			org.eclipse.draw2d.geometry.Point center = _figure.getBounds ().getCenter ();
//			center.translate(-pref.x/2, -pref.y/2);
			pos = center.translate (-pref.x/4, -pref.y/4);
			
		}
		_figure.translateToAbsolute(pos);
		int xOffset = 0;
		int wOffset = 0;
		int yOffset = 0;
		int hOffset = 0;
		
		if (SWT.getPlatform().equalsIgnoreCase("gtk")) {
			xOffset = GTK_X_OFFSET; wOffset = GTK_W_OFFSET;
		}
		else if (SWT.getPlatform().equalsIgnoreCase("carbon")) {
			xOffset = MAC_X_OFFSET; yOffset = MAC_Y_OFFSET; wOffset = MAC_W_OFFSET; hOffset = MAC_H_OFFSET;
		}
		else {
			xOffset = WIN_X_OFFSET; wOffset = WIN_W_OFFSET;
		}
		
		
		text.setBounds (pos.x + xOffset, pos.y + yOffset, pref.x + wOffset, pref.y + hOffset);
		
	}

	/**
	 * 
	 */
	private void refreshSize(Text text) {
		if (_size.x != -1) return;
		String origLabel = _figure.getModel().getName();
		if (origLabel == null) return;
		GC gc = new GC (text);
		_size = gc.textExtent(origLabel);
		gc.dispose ();
	}

	/**
	 * @param size
	 */
	public void setSize(Point size) {
		_size = size;
	}
}
