/*
 * Created on Jan 19, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.policies;

import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import AcmeStudio.ui.editors.diagram.figures.IAcmeElementFigure;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.Listeners;
import acmeLib.core.globals.LanguageConfig;



/**
 * @author Bradley Schmerl
 * Created Jan 19, 2004
 */
public class ElementNameEditManager extends DirectEditManager {

	private ElementNameCellEditorLocator	_locator;
	
	private Shell _errorPopup;
	/**
	 * @param source
	 * @param editorType
	 * @param locator
	 */
	public ElementNameEditManager(GraphicalEditPart source, Class editorType, ElementNameCellEditorLocator locator) {
		super (source, editorType, locator);
		_locator = locator;
		createErrorPopup ();
		
	}

	/**
	 * 
	 */
	private void createErrorPopup() {
		
		Display	display	= Display.getCurrent();
		_errorPopup = new Shell (display, SWT.ON_TOP);
		GridLayout gl = new GridLayout (2, false);
		gl.marginHeight = 2;
		gl.marginWidth = 2;
		_errorPopup.setLayout (gl);
		_errorPopup.setBackground(display.getSystemColor(SWT.COLOR_INFO_BACKGROUND));
		CLabel tipLabel = new CLabel (_errorPopup, SWT.NONE);
		tipLabel.setBackground(display.getSystemColor (SWT.COLOR_INFO_BACKGROUND));
		tipLabel.setImage(ImageManager.getIcon(ImageManager.ID_ERROR));
		tipLabel.setText ("The name cannot be empty or an Acme keyword.");
	}

	boolean _nameError = false;
	Font _scaledFont;
	private VerifyListener _verifyListener;
	/* (non-Javadoc)
	 * @see org.eclipse.gef.tools.DirectEditManager#initCellEditor()
	 */
	protected void initCellEditor() {
		_verifyListener = new VerifyListener () {
			public void verifyText (VerifyEvent event) {
				Text text  = (Text )getCellEditor ().getControl ();
				String oldText = text.getText ();
				String leftText = oldText.substring (0, event.start);
				String rightText = oldText.substring (event.end, oldText.length ());
				GC gc = new GC (text);
				String s = leftText + event.text + rightText;
				Point size = gc.textExtent (s);
				gc.dispose ();
				if (size.x != 0) {
					size = text.computeSize (size.x, SWT.DEFAULT);
				}
				Point origSize = getCellEditor ().getControl ().getSize ();
				if (size.x > getCellEditor ().getControl ().getSize ().x && origSize.x != 0) {
					getCellEditor().getControl ().setSize (size.x, size.y);
					_locator.setSize (size);
				}
				
				// BugFix# 298: Warning pops up if the name is empty or an Acme keyword
				if (s.length () == 0 || LanguageConfig.isAcmeKeyword(s) || LanguageConfig.isArmaniKeyword (s)) {
					_nameError = true;
					
					FigureCanvas	parent	= (FigureCanvas )text.getParent ();
					Point pt = text.getLocation();
					pt.x += parent.getLocation().x;
					pt.y += parent.getLocation ().y;
					pt.y += text.getSize().y;
//					pt.y += text.getSize().y;
					Point tipPosition = parent.toDisplay (pt);
					_errorPopup.pack ();
					_errorPopup.setLocation(tipPosition);
					_errorPopup.setVisible (true);
				}
				else {
					_nameError = false;
					_errorPopup.setVisible (false);
				}
			}
		};
		Text text = (Text )getCellEditor ().getControl ();
		text.addVerifyListener(Listeners.VERIFY_ACMEID);
		text.addVerifyListener(_verifyListener);
		String initialLabelText = "";
		IAcmeElementFigure figure = (IAcmeElementFigure )getEditPart ().getFigure ();
		initialLabelText = figure.getModel ().getName ();
		if (initialLabelText == null) return;
		getCellEditor ().setValue (initialLabelText);
		_scaledFont = figure.getVisualization().getFont();
		FontData data = _scaledFont.getFontData() [0];
		Dimension fontSize = new Dimension (0, data.getHeight ());
		figure.translateToAbsolute(fontSize);
		data.setHeight (fontSize.height);
		_scaledFont = new Font (null, data);
		text.setFont(_scaledFont);
		text.setSelection(0);
		text.showSelection ();
	}
	
	protected void bringDown () {
		Font disposeFont = _scaledFont;
		_scaledFont = null;
		super.bringDown ();
		if (disposeFont != null) disposeFont.dispose();
		_errorPopup.setVisible (false);
//		_errorPopup.dispose();
	}
	
	protected void unhookListeners () {
		super.unhookListeners ();
		Text text = (Text )getCellEditor ().getControl ();
		text.removeVerifyListener(_verifyListener);
		text.removeVerifyListener(Listeners.VERIFY_ACMEID);
		_verifyListener = null;
	}
}
