package AcmeStudio.ui.editors.diagram.policies;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.*;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ZoomListener;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.swt.SWT;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.prefs.ConnectionPatternPreferences;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.actions.AddBindingCommand;
import AcmeStudio.ui.editors.diagram.actions.ConnectElementsCommand;
import AcmeStudio.ui.editors.diagram.actions.MoveRoleCommand;
import AcmeStudio.ui.editors.diagram.actions.RoleConnectCommand;
import AcmeStudio.ui.editors.diagram.parts.*;
import AcmeStudio.ui.editors.diagram.styles.SystemVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.util.Helper;
import AcmeStudio.wizards.connection.PortConnectionRule;
import AcmeStudio.wizards.connection.PortConnectionRuleKey;
import AcmeStudio.wizards.connection.PortRoleAssociation;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.elements.*;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 * 
 * The policy for elements' graphical behavior.
 *  
 */
public class ElementNodeEditPolicy
        extends
        /* SnapToGuideFeedbackEditPolicy */org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy
{

    /**
     * The acme system in which this policy applies.
     */
    private AcmeSystem _acmeSystem = null;
    private static XYAnchor _sourceAnchor;
    private static Point _trueLocation;
    private static double _zoom = 1.0;

    private List figures = new ArrayList();
    private ZoomManager zoomManager;

    /**
     * Returns a connection to be used as feeback during creates.
     * 
     * @param req
     *                 the operation being performed
     * @return a connection to use as feedback
     */
    protected Connection createDummyConnection(Request req)
    {
        PolylineConnection conn = new PolylineConnection();
        conn.setLineStyle(SWT.LINE_DOT);
        return conn;
    }

    /**
     * Returns the Command that will create the connection. This is second part
     * of creation. {@link CreateConnectionRequest#getStartCommand()}is used
     * here to obtain the contribution from the EditPart from which the User
     * started the <i>creation </i>.
     * 
     * @param request
     *                 the CreateConnectionRequest
     * @return the complete command to create a connection
     */
    protected Command getConnectionCompleteCommand(
            CreateConnectionRequest request)
    {
        Command command = null;
        //Get back the host edit part, which may be based on a snap
        EditPart ep = getHostWithSnap(request);

        PortConnectionRule _rule;

        //Grab the rule out of the tool used
        _rule = (PortConnectionRule) request.getExtendedData().get("Rule");

        //grab the snapped item out of the request
        GraphicalEditPart savedSnap = (GraphicalEditPart) request
                .getExtendedData().get("snap");

        //check to see if the editpart is an instance of a connector representation
        boolean isConnectorRep = isConnectorRep(ep);
        request.setTargetEditPart(ep);

        GraphicalEditPart eraseSnap = (GraphicalEditPart) request
                .getExtendedData().get("eraseSnap");

        //If we must erase the snap object feedback
        if (eraseSnap != null && eraseSnap != ep)
        {
            eraseSnap.eraseTargetFeedback(request);
            request.getExtendedData().remove("eraseSnap");
        }

        if (ep instanceof PortEditPart
                && (savedSnap == null || savedSnap instanceof PortEditPart))
        {
            //we are over a port
            if (request.getStartCommand() instanceof ConnectElementsCommand)
            {
                ConnectElementsCommand cpcomm = (ConnectElementsCommand) request
                        .getStartCommand();

                //Construct the rule that the user is trying to use
                PortConnectionRuleKey chosen = ElementNodeEditPolicy
                        .constructChosenRule(request, ep);

                //Ensure that we are working with a rule that coincides with
                // the chosen tool on the pallette
                if ((_rule != null && chosen.matches(_rule.generateKey())) || 
                        (_rule == null && getAllMatchingRules(ep,chosen).size() > 0))
                {
                    cpcomm.setPort2((AcmePort) ep.getModel());
                    cpcomm.setLocation2(((PortEditPart) ep).getFigure()
                            .getBounds().getCenter());

                    cpcomm.setTarget(ep);

                    if (_rule != null)
                        cpcomm.setRule(_rule);

                    command = cpcomm;
                    unhookCanvasChangeListener();
                }
            }
            else if (request.getStartCommand() instanceof AddBindingCommand
                    && !isConnectorRep)
            {
                AddBindingCommand bcomm = (AddBindingCommand) request
                        .getStartCommand();
                bcomm.setInner((AcmeElement) ep.getModel());
                command = bcomm;
                unhookCanvasChangeListener();
            }
        }
        else if (ep instanceof ComponentEditPart
                && (savedSnap == null || savedSnap instanceof ComponentEditPart))
        {
            //We are over a component
            request.setTargetEditPart(ep);
            if (request.getStartCommand() instanceof ConnectElementsCommand)
            {
                ConnectElementsCommand cpcomm = (ConnectElementsCommand) request
                        .getStartCommand();

                //Construct the chosen rule that the user is trying to use
                PortConnectionRuleKey chosen = ElementNodeEditPolicy
                        .constructChosenRule(request, ep);

                //Ensure that we are working with a rule that coincides with
                // the chosen
                //tool on the palette
                if ((_rule != null && chosen.matches(_rule.generateKey())) || 
                        (_rule == null && getAllMatchingRules(ep,chosen).size() > 0))
                {
                    cpcomm.setComponent2((AcmeComponent) ep.getModel());
                    cpcomm.setComponentLocation2(((ComponentEditPart) ep)
                            .getFigure().getBounds().getCenter());

                    if (_rule != null)
                        cpcomm.setRule(_rule);

                    cpcomm.setTarget(ep);
                    command = cpcomm;
                    unhookCanvasChangeListener();
                }
            }
        }
        else if (ep instanceof ConnectorEditPart
                && (savedSnap == null || savedSnap instanceof ConnectorEditPart))
        {
            //we are over a connector
            if (!(request.getSourceEditPart() instanceof ConnectorEditPart))
            {
                //Ensure we are not connecting a connector -> connector
                if (_rule == null || connectorMatches(ep, _rule))
                {
                    request.setTargetEditPart(ep);
                    if (request.getStartCommand() instanceof ConnectElementsCommand)
                    {
                        ConnectElementsCommand cpcomm = (ConnectElementsCommand) request
                                .getStartCommand();

                        cpcomm.setConnector((AcmeConnector) ep.getModel());
                        cpcomm.setLocationConn(((ConnectorEditPart) ep)
                                .getFigure().getBounds().getCenter());

                        if (_rule != null)
                            cpcomm.setRule(_rule);

                        cpcomm.setTarget(ep);
                        command = cpcomm;
                    }
                    unhookCanvasChangeListener();
                }
            }
        }
        else if (ep instanceof BindingPointEditPart
                && !(request.getStartCommand() instanceof AddBindingCommand))
        {
            BindingPointEditPart bep = (BindingPointEditPart) ep;
            request.setTargetEditPart(ep.getParent());
            ConnectElementsCommand cpcomm = (ConnectElementsCommand) request
                    .getStartCommand();
            command = new AddBindingCommand(
                    (AcmePort) ((SystemEditPart.BindingPoint) bep.getModel())._outer,
                    cpcomm.getPort1(), (SystemEditPart) ep.getParent());
            unhookCanvasChangeListener();
        }
        else if (ep instanceof BindingPointEditPart
                && request.getStartCommand() instanceof AddBindingCommand)
        {
            BindingPointEditPart bep = (BindingPointEditPart) ep;
            request.setTargetEditPart(ep.getParent());
            AddBindingCommand bcomm = (AddBindingCommand) request
                    .getStartCommand();
            bcomm
                    .setOuter(((SystemEditPart.BindingPoint) bep.getModel())._outer);
            command = bcomm;
            unhookCanvasChangeListener();
        }
        else if (ep instanceof RoleEditPart && isConnectorRep
                && request.getStartCommand() instanceof AddBindingCommand)
        {
            AddBindingCommand bcomm = (AddBindingCommand) request
                    .getStartCommand();
            bcomm.setInner((AcmeElement) ep.getModel());
            command = bcomm;
            unhookCanvasChangeListener();
        }
        if (command == null) {
        	request.setTargetEditPart(ep);
        	// Goto extension to see if there is another command
        	for (int i = 0; i < SystemXYLayoutEditPolicy._extensions.length && (command == null || command instanceof UnexecutableCommand); i++) {
        		try {
        			command = SystemXYLayoutEditPolicy._extensions [i].getCommand (request);
        		}
        		catch (Exception e) {}
        	}
        }
        return command;
    }

    /**
     * @param ep
     * @param chosen
     * @return
     */
    private List getAllMatchingRules(EditPart ep, PortConnectionRuleKey chosen)
    {
        List allMatchingRules;
        
        //get an instance of the appropriate systemvisualization
        SystemVisualization thisSysVis = VisualStyleRegistry.getSysVis((AcmeElement)ep.getModel()); 
        
        //get all portconnection rules, so we can see if any match the currently chosen rule
        List allRules = thisSysVis.getAllPortConnectionRules(null);
        
        allMatchingRules = SystemVisualization.getMatchingRulesInList(chosen, allRules);
        return allMatchingRules;
    }

    /**
     * This method checks to see if the cursor is currently snapped -- if so,
     * the snapped object is returned. Otherwise, the host is retrieved like
     * normal
     * 
     * @author LFW
     */
    private EditPart getHostWithSnap(CreateConnectionRequest request)
    {
        GraphicalEditPart savedSnap = (GraphicalEditPart) request
                .getExtendedData().get("snap");

        if (savedSnap != null)
            return savedSnap;
        return getHost();
    }

    /**
     * Returns the Command that represents the first half of creating a
     * connection. This Command will be passed to the target node EditPart. The
     * target node may do anything necessary to create a Command that represents
     * the entire creation.
     * 
     * @param request
     *                 the CreateConnectionRequest
     * @see #getConnectionCompleteCommand(CreateConnectionRequest)
     * @return a Command representing half of a connection creation
     */
    protected Command getConnectionCreateCommand(CreateConnectionRequest request)
    {
        EditPart ep = getHost();
        Command comm = null;

        PortConnectionRule _rule;

        //Grab the rule out of the tool used
        _rule = (PortConnectionRule) request.getExtendedData().get("Rule");

        //grab the snapped item out of the request
        GraphicalEditPart savedSnap = (GraphicalEditPart) request
                .getExtendedData().get("snap");

        if (ep instanceof PortEditPart
                && (savedSnap == null || savedSnap instanceof PortEditPart))
        {
            //ensure that the choice the user has made conforms to the rule
            // tool used
            if (_rule == null || portMatches(ep, _rule))
            {
                ConnectElementsCommand command = new ConnectElementsCommand(getHost()
                        .getParent());
                command.setPort1((AcmePort) ep.getModel());
                command.setLocation1(((PortEditPart) ep).getFigure()
                        .getBounds().getCenter());

                command.setSource(ep);
                request.setStartCommand(command);
                request.setSourceEditPart(ep);
                ep.showTargetFeedback(request);
                hookCanvasChangeListener();
                comm = command;
            }
        }
        else if (ep instanceof ComponentEditPart
                && (savedSnap == null || savedSnap instanceof ComponentEditPart))
        {
            //ensure that the choice the user has made conforms to the rule
            // tool used
            if (_rule == null || componentMatches(ep, _rule))
            {
                ConnectElementsCommand command = new ConnectElementsCommand(getHost()
                        .getParent());
                command.setComponent1((AcmeComponent) ep.getModel());
                command.setComponentLocation1(((ComponentEditPart) ep)
                        .getFigure().getBounds().getCenter());

                command.setSource(ep);
                request.setStartCommand(command);
                request.setSourceEditPart(ep);
                ep.showTargetFeedback(request);
                hookCanvasChangeListener();
                comm = command;
            }
        }
        else if (ep instanceof ConnectorEditPart
                && (savedSnap == null || savedSnap instanceof ConnectorEditPart))
        {
            //ensure that the choice the user has made conforms to the rule
            // tool used
            if (_rule == null || connectorMatches(ep, _rule))
            {
                ConnectElementsCommand command = new ConnectElementsCommand(getHost()
                        .getParent());
                command.setConnector((AcmeConnector) ep.getModel());
                command.setLocationConn(((ConnectorEditPart) ep).getFigure()
                        .getBounds().getCenter());

                command.setSource(ep);
                request.setStartCommand(command);
                request.setSourceEditPart(ep);
                ep.showTargetFeedback(request);
                hookCanvasChangeListener();
                comm = command;
            }
        }
        else if (ep instanceof BindingPointEditPart)
        {
            BindingPointEditPart bep = (BindingPointEditPart) ep;
            AddBindingCommand command = new AddBindingCommand(
                    ((SystemEditPart.BindingPoint) ep.getModel())._outer, null,
                    (SystemEditPart) bep.getParent());
            hookCanvasChangeListener();
            request.setStartCommand(command);
            request.setSourceEditPart(ep);
            comm = command;
        }
        else if (ep instanceof RoleEditPart && isConnectorRep(ep))
        {
            AddBindingCommand command = new AddBindingCommand(null,
                    (AcmeElement) ep.getModel(), (SystemEditPart) ep
                            .getParent());
            hookCanvasChangeListener();
            request.setStartCommand(command);
            request.setSourceEditPart(ep);
            comm = command;
        }
        else
        {
            request.setStartCommand(null);
        }

        return comm;
    }

    /**
     * Determine whether or not the editpart passed in is a representation
     * of a connector
     * 
     * @param ep the editpart to check
     * @return boolean
     */
    private boolean isConnectorRep(EditPart ep)
    {
        boolean isConnectorRep = false;

        SystemEditPart sep = null;
        if (ep instanceof SystemEditPart)
            sep = (SystemEditPart) ep;
        else
            sep = (SystemEditPart) ep.getParent();

        AcmeSystem system = (AcmeSystem) sep.getModel();
        if (system.getParentRepresentation() != null)
        {
            if (system.getParentRepresentation().getParent() instanceof AcmeConnector)
            {
                isConnectorRep = true;
            }
        }
        return isConnectorRep;
    }

    private PropertyChangeListener canvasChangeListener = new PropertyChangeListener()
    {
        public void propertyChange(PropertyChangeEvent event)
        {
            if (_trueLocation != null)
            {
                Point p = adjust();
                _sourceAnchor.setLocation(p);
            }
        }
    };

    private ZoomListener canvasZoomListener = new ZoomListener()
    {
        public void zoomChanged(double zoom)
        {
            Point p = zoom(_trueLocation, zoom);
            _sourceAnchor.setLocation(p);
        }
    };

    private ZoomManager getZoomManager()
    {
        ScalableFreeformRootEditPart scalableRoot = (ScalableFreeformRootEditPart) getHost()
                .getRoot();
        ZoomManager zm = scalableRoot.getZoomManager();
        return zm;
    }

    private Point zoom(Point p, double zoom)
    {
        _zoom = zoom;
        return p.getCopy().scale(zoom);
    }

    private Point adjust()
    {
        try
        {
            return zoom(_trueLocation.getCopy().translate(
                    getOffset().getNegated().scale(1.0 / _zoom)), _zoom);
        }
        catch (Exception e)
        {
            return _trueLocation;
        }
    }

    private Point getOffset()
    {
        Point offset = getViewport().getClientArea().getLocation().getCopy();
        return offset;
    }

    private void hookCanvasChangeListener()
    {
        getViewport().getHorizontalRangeModel().addPropertyChangeListener(
                canvasChangeListener);
        getViewport().getVerticalRangeModel().addPropertyChangeListener(
                canvasChangeListener);
        getZoomManager().addZoomListener(canvasZoomListener);
    }

    private Viewport getViewport()
    {
        FigureCanvas fc = (FigureCanvas) getHost().getRoot().getViewer()
                .getControl();
        return fc.getViewport();
    }

    private void unhookCanvasChangeListener()
    {
        getViewport().getHorizontalRangeModel().removePropertyChangeListener(
                canvasChangeListener);
        getViewport().getVerticalRangeModel().removePropertyChangeListener(
                canvasChangeListener);
        getZoomManager().removeZoomListener(canvasZoomListener);
    }

    /**
     * Returns the <code>Command</code> to reconnect a connection's <i>target
     * </i> end to the host.  This is used in dragging objects such as roles around
     * 
     * @param request
     *                 the ReconnectRequest
     * @return a Command
     */
    protected Command getReconnectTargetCommand(ReconnectRequest request)
    {
        RoleConnectCommand rccCmd = null;
        boolean ruleMatch = false;
        List matchRule = null;

        ConnectionEditPart ep = request.getConnectionEditPart();
        if (!(ep instanceof RoleEditPart))
            return null;
        RoleEditPart roleEditPart = (RoleEditPart) request
                .getConnectionEditPart();

        if (_acmeSystem == null)
        {
            AcmeRole role = (AcmeRole) roleEditPart.getModel();
            _acmeSystem = Helper.getRootAcmeSystem(role);
        }
        if (_acmeSystem == null)
            return null;

        //generate the appropriate moveRoleCommand up to this point
        MoveRoleCommand cmd = (MoveRoleCommand) RoleTracker.getMoveRoleCommand(
                roleEditPart, request, _acmeSystem);

        //Check to see whether we should be connecting the role to a component instead
        EditPart savedep = (GraphicalEditPart) request.getExtendedData().get(
                "savedHighlightReconnect");
        EditPart target = cmd.getTargetEditPart();

        EditPart onComponent = (EditPart) request.getExtendedData().get("over");
        if (!(onComponent instanceof ComponentEditPart))
            onComponent = null;

        if (!(target instanceof PortEditPart) && onComponent != null)
            target = onComponent;

        //at this point if the target is a componentEditPart, we are connecting to that

        if (AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(
                ConnectionPatternPreferences.RESTRICT_TO_RULES))
        {
            if (target instanceof PortEditPart || target instanceof ComponentEditPart)
            {
	            //since the target is an instance of componentEP, 
	            //and restrict to rules is on, make sure that the component type
	            //and all match a rule
	            AcmePortType[] pt = new AcmePortType[2];
	            AcmeComponentType[] ct = new AcmeComponentType[2];
	
	            AcmeConnectorType conntype = (AcmeConnectorType) ((AcmeRole) roleEditPart
	                    .getModel()).getParent()
	                    .getDeclaredTypes().getNextItem();
	            
	            if (target instanceof ComponentEditPart)
	            {
	                ct[1] = (AcmeComponentType) ((AcmeComponent) target.getModel())
	                    .getDeclaredTypes().getNextItem();
	
		            //handle the case of generic objects
		            if (ct[1] == null)
		                ct[1] = ConstantTypes.COMPONENT_BASE_TYPE;
	            }
	            else if (target instanceof PortEditPart)
	            {
	                pt[1] = (AcmePortType) ((AcmePort) target.getModel())
	                .getDeclaredTypes().getNextItem();
	
	            //handle the case of generic objects
	            if (pt[1] == null)
	                pt[1] = ConstantTypes.PORT_BASE_TYPE;                
	            }
	            
	            if (conntype == null)
	                conntype = ConstantTypes.CONNECTOR_BASE_TYPE;
	            
	            matchRule = SystemVisualization
	                    .checkExistingRules(pt, ct,
	                            conntype,
	                            _acmeSystem);
	
	            matchRule = filterRoleTypes(matchRule, (AcmeRole) roleEditPart.getModel());
	            
	            if (matchRule.size() > 0)
	                ruleMatch = true;
            }
            else
                ruleMatch = true;
        }
        else
            ruleMatch = true;

        if (ruleMatch
                && (target instanceof PortEditPart || target instanceof ComponentEditPart))
        {
            if (savedep != target)
            {
                if (savedep != null)
                    savedep.eraseTargetFeedback(request);
                request.getExtendedData()
                        .put("savedHighlightReconnect", target);
                target.showTargetFeedback(request);
            }
            if (target instanceof ComponentEditPart)
            {

                if (matchRule == null)
                    matchRule = new ArrayList();

                rccCmd = new RoleConnectCommand((AcmeComponent) target
                        .getModel(), (AcmeRole) roleEditPart.getModel(),
                        matchRule);
            }
        }
        else if (savedep != null)
        {
            request.getExtendedData().remove("savedHighlightReconnect");
            savedep.eraseTargetFeedback(request);

        }

        //if rccCmd is not null, it should be returned

        if (rccCmd == null && ruleMatch == true)
            return cmd;
        else
            return rccCmd;
    }

    /**
     * Returns the <code>Command</code> to reconnect a connection's <i>source
     * </i> end to the host.
     * 
     * @param request
     *                 the ReconnectRequest
     * @return a Command
     */
    protected Command getReconnectSourceCommand(ReconnectRequest request)
    {
        return null;
    }

    /**
     * @see org.eclipse.gef.EditPolicy#showTargetFeedback(Request)
     */
    public void showTargetFeedback(Request request)
    {
        super.showTargetFeedback(request);
        if (REQ_SELECTION_HOVER.equals(request.getType()))
        {
            if (!(getHost() instanceof ElementEditPart))
            {
                return;
            }

            if (_acmeSystem == null)
            {
                ElementEditPart editPart = (ElementEditPart) getHost();
                if (editPart.getModel() instanceof AcmeElement)
                {
                    AcmeElement element = (AcmeElement) editPart.getModel();
                    _acmeSystem = Helper.getRootAcmeSystem(element);
                }
                else if (editPart.getModel() instanceof SystemEditPart.BindingPoint)
                {
                    SystemEditPart.BindingPoint bp = (SystemEditPart.BindingPoint) editPart
                            .getModel();
                    _acmeSystem = Helper.getRootAcmeSystem(bp._outer);
                }
            }
        }
    }

    /**
     * This method takes care of adding snapping functionality to the different
     * connection tools.
     * 
     * @date Created on 6/30/04 
     * @author LFW
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getTargetConnectionAnchor(org.eclipse.gef.requests.CreateConnectionRequest)
     */
    protected ConnectionAnchor getTargetConnectionAnchor(
            CreateConnectionRequest request)
    {

        XYAnchor anchor = null;
        //Store the cursor location in two formats for use with snapping
        Point origCursorLoc = request.getLocation().getCopy();

        Point scaledCursorLoc = origCursorLoc.getCopy();

        getViewport().translateToAbsolute(scaledCursorLoc);
        //Gets the nearest editPart to where the cursor is -- this is the
        //candidate for the snap
        GraphicalEditPart temp = getNearestEditPart(scaledCursorLoc);

        //It is possible that there has already been a snap (and the user issimply
        //moving the cursor in that vicinity -- store the existing snapped-toobj
        //if possible
        GraphicalEditPart savedSnap = (GraphicalEditPart) request
                .getExtendedData().get("snap");
        /* BRS: Added to cater for connection of Bindings in representations */

        if (request.getStartCommand() instanceof ConnectElementsCommand)
        {
            //Construct the rule that would be currently chose should the user click
            PortConnectionRuleKey chosen = ElementNodeEditPolicy
                    .constructChosenRule(request, temp);

            PortConnectionRule savedRule = (PortConnectionRule)request.getExtendedData().get("Rule");
            
            if ((savedRule != null && (chosen.matches(savedRule.generateKey()))) ||
                    (savedRule == null && getAllMatchingRules(request.getSourceEditPart(), chosen).size() > 0))
            {
                //If the user is using the general connect tool or if the object
                // they are over makes a valid rule
                if (temp != null)
                {
                    if (savedSnap != null)
                    {
                        if (!temp.getFigure().getBounds().getCenter().equals(
                                savedSnap.getFigure().getBounds().getCenter()))
                        {
                            savedSnap.eraseTargetFeedback(request);

                            savedSnap = temp;
                        }
                    }
                    else
                    {
                        //If snapping to something, erase all feedbacks that exist
                        ItemOutlineEditPolicy.removeAllHighlights(request);
                        savedSnap = temp;
                    }
                }
                else
                //There was no valid object to snap to
                {
                    request.getExtendedData().put("eraseSnap", savedSnap);
                    request.getExtendedData().remove("snap");
                    savedSnap = null;

                }

                if (savedSnap != null)
                {
                    request.getExtendedData().put("snap", savedSnap);
                    //request.setTargetEditPart(savedSnap);
                    savedSnap.showTargetFeedback(request);
                    //Return the point for the snap based at the center of theobj
                    Point savedSnapCenter = savedSnap.getFigure().getBounds()
                            .getCenter();
                    savedSnap.getFigure().translateToAbsolute(savedSnapCenter);
                    getViewport().translateToRelative(savedSnapCenter);
                    anchor = new XYAnchor(savedSnapCenter);
                }
                else
                    anchor = new XYAnchor(origCursorLoc);
            }
            else if (savedRule == null && getAllMatchingRules(request.getSourceEditPart(), chosen).size() == 0)
            {
                request.getExtendedData().put("eraseSnap", savedSnap);
                request.getExtendedData().remove("snap");
                savedSnap = null;
                
                anchor = new XYAnchor(origCursorLoc);
            }
            else
            {
                //Return back the original cursor location so there is no
                // visible snap
                anchor = new XYAnchor(origCursorLoc);
            }
        }
        else
        {
            anchor = new XYAnchor(origCursorLoc);
        }
        return anchor;
    }

    /**
     * This method looks at all the editparts in the diagram and calculates
     * the closest one to fall within the specified 'boundry' for closeness of a
     * snap
     * 
     * @author LFW
     * 
     * @param Point the point to start looking from
     * @param String the type we are searching
     */
    private GraphicalEditPart getNearestEditPart(Point p)
    {
        DiagramEditor mpe = (DiagramEditor) ASContext.current()
                .getActiveEditor().getDiagramEditor();

        GraphicalEditPart closest = null;
        double min = 15 * getZoomManager().getZoom();
        double componentmin = 20 * getZoomManager().getZoom();

        Map epMap = mpe.getGraphicalViewer().getEditPartRegistry();
        if (epMap != null)
        {
            Iterator iter = epMap.keySet().iterator();
            while (iter.hasNext())
            {
                Object obj = iter.next();
                if (obj instanceof AcmeElement)
                {
	                AcmeElement elem = (AcmeElement) obj;
	
	                IFigure figure = mpe.getFigure(elem);
	                Point loc = figure.getBounds().getCenter().getCopy();
	
	                figure.translateToAbsolute(loc);
	
	                //Now that the coordinates are correct, check the distance
	                if (elem instanceof AcmePort
	                        || elem instanceof AcmeConnector
	                        || elem instanceof AcmeComponent)
	                {
	                    double d = p.getDistance(loc);
	
	                    if (d < ((elem instanceof AcmeComponent) ? componentmin
	                            : min))
	                    {
	                        min = d;
	                        closest = (GraphicalEditPart) mpe.getEditPart(elem);
	                    }
	                }
                }
            }
        }
        return closest;
    }

    protected ConnectionAnchor getSourceConnectionAnchor(
            CreateConnectionRequest request)
    {

        _sourceAnchor = new XYAnchor(request.getLocation());
        Point offset = getOffset();
        _zoom = getZoomManager().getZoom();
        _trueLocation = _sourceAnchor.getReferencePoint().getCopy();
        _trueLocation.translate(offset);
        _trueLocation.scale(1 / _zoom);
        return _sourceAnchor;

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.EditPolicy#eraseSourceFeedback(org.eclipse.gef.Request)
     */
    public void eraseSourceFeedback(Request request)
    {
        super.eraseSourceFeedback(request);
        for (Iterator iter = figures.iterator(); iter.hasNext();)
        {
            IFigure fig = (IFigure) iter.next();
            if (getLayer(LayerConstants.FEEDBACK_LAYER).getChildren().contains(
                    fig))
            {
                fig.getParent().remove(fig);
            }
        }
        figures.clear();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.EditPolicy#setHost(org.eclipse.gef.EditPart)
     */
    public void setHost(EditPart host)
    {
        super.setHost(host);
        zoomManager = (ZoomManager) getHost().getViewer().getProperty(
                ZoomManager.class.toString());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.EditPolicy#showSourceFeedback(org.eclipse.gef.Request)
     */
    public void showSourceFeedback(Request request)
    {
        super.showSourceFeedback(request);

        if (request.getType().equals(REQ_MOVE)
                || request.getType().equals(REQ_RESIZE))
        {
            eraseSourceFeedback(request);
            ChangeBoundsRequest req = (ChangeBoundsRequest) request;
            highlightGuide((Integer) req.getExtendedData().get(
                    SnapToGuides.KEY_HORIZONTAL_GUIDE), false);
            highlightGuide((Integer) req.getExtendedData().get(
                    SnapToGuides.KEY_VERTICAL_GUIDE), true);
        }
    }

    /**
     * @param integer
     * @param b
     */
    private void highlightGuide(Integer pos, boolean horizontal)
    {
        if (pos == null)
        {
            return;
        }
        int position = pos.intValue();
        if (zoomManager != null)
        {
            position = (int) Math.round(position
                    * (zoomManager.getZoom() / zoomManager.getUIMultiplier()));
        }
        Rectangle diagramBounds = getLayer(LayerConstants.FEEDBACK_LAYER)
                .getBounds();
        Rectangle figBounds = Rectangle.SINGLETON;
        figBounds.setBounds(diagramBounds);
        if (horizontal)
        {
            figBounds.height = 1;
            figBounds.y = position;
        }
        else
        {
            figBounds.x = position;
            figBounds.width = 1;
        }
        IFigure fig = new Figure();
        fig.setOpaque(true);
        fig.setBackgroundColor(ColorConstants.red);
        addFeedback(fig);
        fig.setBounds(figBounds);
        figures.add(fig);
    }

    /*
     * @author LFW
     * 
     * This method checks to see if the editpart passed in matches the port
     * information contained in the rule passed in.
     */
    private boolean portMatches(EditPart ep, PortConnectionRule _rule)
    {
        boolean retVal = false;

        String PT1 = ((PortRoleAssociation[]) _rule.getAssociations())[0]
                .getPortType();

        String CT1 = ((PortRoleAssociation[]) _rule.getAssociations())[0]
                .getComponentType();

        if (PT1 == null || PT1.equals(""))
            retVal = true; //For the generic case everything should match
        else
        {
            AcmePort epPort = (AcmePort) ep.getModel();

            retVal = (PortConnectionRuleKey.isSubtype(((AcmeElementType) epPort
                    .getDeclaredTypes().getNextItem()).getTypeName(), PT1));

            if (retVal == true && CT1 != null && !CT1.equals("")) 
            //match is still possible, check to see if the parent matches
            {
                String ct = ((AcmeComponentType) ((AcmePort) ep.getModel())
                        .getParent().getDeclaredTypes().getNextItem())
                        .getTypeName();
                retVal = (PortConnectionRuleKey.isSubtype(ct, CT1));
            }
        }
        return retVal;
    }

    /*
     * @author LFW
     * 
     * This method checks to see if the editpart passed in matches the component
     * information contained in the rule passed in.
     */
    private boolean componentMatches(EditPart ep, PortConnectionRule _rule)
    {
        boolean retVal = false;

        String CT1 = ((PortRoleAssociation[]) _rule.getAssociations())[0]
                .getComponentType();

        AcmeComponent epComp = (AcmeComponent) ep.getModel();

        if (CT1 != null && CT1.equals(""))
            return true;

        return (PortConnectionRuleKey.isSubtype(((AcmeElementType) epComp
                .getDeclaredTypes().getNextItem()).getTypeName(), CT1));
    }

    /*
     * @author LFW
     * 
     * This method checks to see if the editpart passed in matches the connector
     * information contained in the rule passed in.
     */
    private boolean connectorMatches(EditPart ep, PortConnectionRule _rule)
    {
        AcmeConnector epConn = (AcmeConnector) ep.getModel();

        if (_rule.getConnectorType() == null
                || _rule.getConnectorType().equals(""))
            return true; //This is for the generic rule case
        else
            return PortConnectionRuleKey.isSubtype(((AcmeElementType) epConn
                    .getDeclaredTypes().getNextItem()).getTypeName(), _rule
                    .getConnectorType());
    }

    /*
     * @author LFW
     * 
     * This static method retrieves relevant information from the command passed
     * in, and using the editpart passed in (which presumably represents the
     * current editpart that is being considered for addition to the command),
     * constructs the rule that would be made up if the editpart was added.
     * 
     * This is typically used for checking whether or not the currently
     * moused-over item is a valid item for the currently selected rule
     */
    public static PortConnectionRuleKey constructChosenRule(
            CreateConnectionRequest req, EditPart ep)
    {
        AcmePortType[] pt = new AcmePortType[2];
        AcmeComponentType[] ct = new AcmeComponentType[2];
        AcmeConnectorType conntype = null;
        
        if (req.getStartCommand() instanceof ConnectElementsCommand)
        {
        ConnectElementsCommand cmd = (ConnectElementsCommand) req.getStartCommand();

        if (REQ_CONNECTION_START.equals(req.getType()))
        {
            if (ep instanceof PortEditPart)
            {
                pt[0] = (AcmePortType) ((AcmePort) ep.getModel())
                        .getDeclaredTypes().getNextItem();

                //Also set the component type based on the port's parent
                //for further matching accuracy
                ct[0] = (AcmeComponentType) ((AcmePort) ep.getModel())
                        .getParent().getDeclaredTypes().getNextItem();
            }
            else if (ep instanceof ComponentEditPart)
            {
                ct[0] = (AcmeComponentType) ((AcmeComponent) ep.getModel())
                        .getDeclaredTypes().getNextItem();
            }
            else if (ep instanceof ConnectorEditPart)
            {
                conntype = (AcmeConnectorType) ((AcmeConnector) ep.getModel())
                        .getDeclaredTypes().getNextItem();
            }
        }
        else
        {
            if (cmd.getPort1() != null)
            {
                pt[0] = (AcmePortType) cmd.getPort1().getDeclaredTypes()
                        .getNextItem();
                //              Also set the component type based on the port's parent
                //for further matching accuracy
                //ct[0] =
                // (AcmeComponentType)((AcmePort)ep.getModel()).getParent().getDeclaredTypes().getNextItem();
            }

            if (cmd.getComponent1() != null)
                ct[0] = (AcmeComponentType) cmd.getComponent1()
                        .getDeclaredTypes().getNextItem();

            if (cmd.getConnector() != null)
                conntype = (AcmeConnectorType) cmd.getConnector()
                        .getDeclaredTypes().getNextItem();
        }

        if (REQ_CONNECTION_END.equals(req.getType()))
        {
            if (ep instanceof PortEditPart)
            {
                pt[1] = (AcmePortType) ((AcmePort) ep.getModel())
                        .getDeclaredTypes().getNextItem();

                //Also set the component type based on the port's parent
                //for further matching accuracy
                ct[1] = (AcmeComponentType) ((AcmePort) ep.getModel())
                        .getParent().getDeclaredTypes().getNextItem();
            }
            else if (ep instanceof ComponentEditPart)
            {
                ct[1] = (AcmeComponentType) ((AcmeComponent) ep.getModel())
                        .getDeclaredTypes().getNextItem();
            }
            else if (ep instanceof ConnectorEditPart)
            {
                conntype = (AcmeConnectorType) ((AcmeConnector) ep.getModel())
                        .getDeclaredTypes().getNextItem();
            }
        }
        }
        return (new PortConnectionRuleKey(pt, ct, conntype));

    }

    /**
     * This method simply takes in an AcmeElement and returns back
     * the string representing the type.  It will return "" if there is no
     * declared type
     * 
     * @param e
     * @return
     */
    private String getAEType(AcmeElement e)
    {
        if (e.getDeclaredTypes().getSize() > 0)
        {
            return ((AcmeElementType) e.getDeclaredTypes().getNextItem())
                    .getTypeName();
        }
        else
        {
            return "";
        }
    }
    
    /**
     * This method goes through a list of rules and checks them
     * against the role type passed in, filtering as it goes
     * 
     * @author LFW
     * 
     * @param List the list of rules to filter on
     * @param AcmeRole the role to filter the list against
     */
    private List filterRoleTypes(List matchRule, AcmeRole role)
    {
        List ret = new ArrayList();
        // TODO Auto-generated method stub
        for (Iterator iter = matchRule.iterator(); iter.hasNext(); )
        {
            PortConnectionRule r = (PortConnectionRule)iter.next();
//            String ruleRoleType0 = r.getAssociations () [0].getRoleType ();
            String ruleRoleType1 = r.getAssociations () [1].getRoleType ();
            String toMatchType = getAEType (role);
            if (/*ruleRoleType0.equals (toMatchType) || */ruleRoleType1.equals (toMatchType))
                ret.add(r);
        }
        return ret;
    }
}

