package AcmeStudio.ui.editors.diagram.policies;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.gef.handles.ConnectionEndHandle;
import org.eclipse.gef.handles.ConnectionHandle;
import org.eclipse.gef.handles.ConnectionStartHandle;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.figures.VisualizationFeedbackFigure;
import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.ui.editors.diagram.parts.ElementEditPart;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import acmeLib.elements.*;
import acmeLib.representations.AcmeBinding;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * The policy for selecting elements.
 * 
 */
public class ElementSelectionEditPolicy extends ResizableEditPolicy {
	
	/**
	 * @author Bradley Schmerl
	 * Makes sure that selection feedback is rotated with the role
	 */
	public class RotatedFeedbackFigure extends Polygon implements IFigure {

		PointList rotatedPoints = new PointList ();
		private final double _rotation;
		/**
		 * @param rotation
		 */
		public RotatedFeedbackFigure(double rotation) {
			_rotation = rotation;
			
		}
		
		
		/**
		 * @param rect
		 * @see org.eclipse.draw2d.IFigure#setBounds(org.eclipse.draw2d.geometry.Rectangle)
		 */
		public void setBounds(Rectangle rect) {
			Point center = rect.getCenter ();
			
			rotatedPoints.removeAllPoints();
			PointList origPoints = new PointList ();
			origPoints.addPoint (rect.getTopLeft());
			origPoints.addPoint (rect.getTopRight());
			origPoints.addPoint (rect.getBottomRight());
			origPoints.addPoint (rect.getBottomLeft());
			
			for (int i = 0; i < origPoints.size (); i++) {
				Point p = origPoints.getPoint (i);
				double theta = Math.atan2 (p.y - center.y, p.x - center.x) + _rotation;
				double r = center.getDistance(p);
				rotatedPoints.addPoint((int )(center.x + r * Math.cos (theta)), (int )(center.y + r * Math.sin(theta)));
			}
			super.setBounds(rotatedPoints.getBounds());
			setPoints(rotatedPoints);
		}
		

	}
    private IFigure _feedback;

	/**
     * Create selection handles
     */
    protected List createSelectionHandles() {
        List ret = new ArrayList();
        Object obj = getHost().getModel();
        if ((obj instanceof AcmeElement)) {
            AcmeElement model = (AcmeElement) obj;
            if (!(model instanceof AcmeBinding) && (!(model instanceof AcmeElementType)) && (!(model instanceof AcmeSystem))) {
                ConstrainedResizeHandleKit.addHandles(
                    (GraphicalEditPart) getHost(),
                    ret);
            }
        } else if ((obj instanceof SystemEditPart.BindingPoint)) {
            ConstrainedResizeHandleKit.addHandles(
                (GraphicalEditPart) getHost(),
                ret);
        } else if ((obj instanceof SystemEditPart.Binding)) {
//            SystemEditPart.Binding binding = (SystemEditPart.Binding) obj;
            ConnectionHandle startHandle =
                new ConnectionStartHandle((ConnectionEditPart) getHost());
            ConnectionHandle endHandle =
                new ConnectionEndHandle((ConnectionEditPart) getHost());
            ret.add(startHandle);
            ret.add(endHandle);
        }
        else if (obj instanceof AcmeSystem) { // this is a representation
            ConstrainedResizeHandleKit.addHandles ((GraphicalEditPart )getHost (), ret);
        }

        return ret;
    }

    /**
     * Erase feedback indicating that the receiver object is 
     * being dragged.  This method is called when a drag is
     * completed or cancelled on the receiver object.
     * @param dragTracker org.eclipse.gef.tools.DragTracker The drag tracker of the tool performing the drag.
     */
    public void eraseSourceFeedback(Request request) {
        super.eraseSourceFeedback(request);
    }

    /**
     * Erase feedback indicating that the receiver object is 
     * being dragged.  This method is called when a drag is
     * completed or cancelled on the receiver object.
     * @param dragTracker org.eclipse.gef.tools.DragTracker The drag tracker of the tool performing the drag.
     */
    public void showTargetFeedback(Request request) {
    }

    /**
     * @see org.eclipse.gef.editpolicies.SelectionEditPolicy#setSelectedState(int)
     */
    protected void setSelectedState(int type) {
        super.setSelectedState(type);
    }

    /**
     * @see org.eclipse.gef.editpolicies.ResizableEditPolicy#getResizeCommand(ChangeBoundsRequest)
     */
    protected Command getResizeCommand(ChangeBoundsRequest request) {
        return super.getResizeCommand(request);
    }

    protected Command getMoveCommand(ChangeBoundsRequest request) {
        List selectedEditParts =
            getHost().getRoot().getViewer().getSelectedEditParts();
        Iterator iter = selectedEditParts.iterator();
        boolean compSelected = false;
        while (iter.hasNext()) {
            Object obj = iter.next();
            if (obj instanceof ComponentEditPart) {
                compSelected = true;
                break;
            }
        }
        Object obj = getHost().getModel();
        if ((obj instanceof AcmePort) && compSelected) {
            return null;
        } else {
            return super.getMoveCommand(request);
        }
    }

    /**
	 * @return
	 * @see org.eclipse.gef.editpolicies.NonResizableEditPolicy#getInitialFeedbackBounds()
	 */
	protected Rectangle getInitialFeedbackBounds() {
		if (getHost () instanceof RoleEditPart) {
			RoleView roleView = (RoleView )((GraphicalEditPart )getHost ()).getFigure ();
			return roleView.getDecoration().getBounds();
		}
		return super.getInitialFeedbackBounds();
	}
	
	/**
	 * @return
	 * @see org.eclipse.gef.editpolicies.NonResizableEditPolicy#createDragSourceFeedbackFigure()
	 */
	protected IFigure createDragSourceFeedbackFigure() {
		if (getHost () instanceof ElementEditPart) {
			ElementEditPart part = (ElementEditPart )getHost ();
			Object model = part.getModel ();
			AcmeElement element = null;
			if (model instanceof AcmeElement) {
				element = (AcmeElement )model;
			}
			else if (model instanceof SystemEditPart.BindingPoint) {
				element = (AcmeElement )((SystemEditPart.BindingPoint )model)._outer;
			}
			else 
				return super .createDragSourceFeedbackFigure();
			ElementVisualization vis = VisualStyleRegistry.getVisualization(element);
			if (vis == null) {
				return super.createDragSourceFeedbackFigure();
			}
			else {
				VisualizationFeedbackFigure feedback = new VisualizationFeedbackFigure (vis);
				FigureUtilities.makeGhostShape (feedback);
				addFeedback (feedback);
				_feedback = feedback;
			}
		}
		else if (getHost () instanceof RoleEditPart) {
			RoleEditPart rep = (RoleEditPart )getHost ();
			AcmeRole role = (AcmeRole )rep.getModel ();
			double rotation = ((RoleView )rep.getFigure ()).getDecoration().getRotation();
			ElementVisualization vis = VisualStyleRegistry.getVisualization (role);
			VisualizationFeedbackFigure feedback = new VisualizationFeedbackFigure (vis, rotation);
			FigureUtilities.makeGhostShape(feedback);
			addFeedback (feedback);
			_feedback = feedback;
		}
		else {
			super.createDragSourceFeedbackFigure();
		}
		return _feedback;
	}
	
}