/*
 * Created on Jun 29, 2004 @author LFW
 *  
 */
package AcmeStudio.ui.editors.diagram.policies;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.*;
import org.eclipse.gef.editpolicies.GraphicalEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.Color;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.prefs.ConnectionPatternPreferences;
import AcmeStudio.ui.editors.diagram.actions.AddBindingCommand;
import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.ui.editors.diagram.parts.PortEditPart;
import AcmeStudio.ui.editors.diagram.styles.SystemVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.wizards.connection.PortConnectionRule;
import AcmeStudio.wizards.connection.PortConnectionRuleKey;
import acmeLib.elements.AcmeElement;
import com.swtdesigner.SWTResourceManager;

/*
 * This edit policy creates the outline around objects that are moused over with
 * the connection tool.
 * 
 * @author Larry Waldman
 */
public class ItemOutlineEditPolicy extends GraphicalEditPolicy
{
    //Class level constants which control the type of border (color/size)
    public static Color highlightColor = SWTResourceManager.getColor (240, 0, 0);
    private int borderWidth = 4;

    private Figure highlight;

    private static Vector allHighlightedFigs = new Vector();

    /**
     * This method erases the highlight for the associated item
     */
    public void eraseTargetFeedback(Request request)
    {
        if (highlight != null)
        {
            ItemOutlineEditPolicy.allHighlightedFigs.remove(getHost());

            highlight.erase();
            IFigure feedbackLayer = getLayer(LayerConstants.SCALED_FEEDBACK_LAYER);
            if (feedbackLayer.getChildren().contains(highlight)) {
            	feedbackLayer.remove(highlight);
            }
            highlight = null;
        }
    }

    /**
     * Iterate through all highlighted figures and remove the highlights
     * 
     * @param request
     */
    public static void removeAllHighlights(Request request)
    {
        Vector copy = new Vector(allHighlightedFigs);
        for (Iterator iter = copy.iterator(); iter.hasNext();)
        {
            GraphicalEditPart fig = (GraphicalEditPart) iter.next();

            fig.eraseTargetFeedback(request);
        }
    }

    private IFigure getContainerFigure()
    {
        return ((GraphicalEditPart) getHost()).getFigure();
    }

    public EditPart getTargetEditPart(Request request)
    {
        return request.getType().equals(RequestConstants.REQ_SELECTION_HOVER) ? getHost()
                : null;
    }

    /**
     * Called from showHighlight, this method sets the size and border,
     * and adds the highlight to the feedback layer
     * 
     * @param c
     */
    private void setContainerHighlight(Color c)
    {
        Rectangle bound = getContainerFigure().getBounds();
        borderWidth = AcmeStudioPlugin.getDefault().getPreferenceStore().getInt(ConnectionPatternPreferences.ELEMENT_HIGHLIGHT_WIDTH);
        //Copy the rectangle so the original figure isn't changed,
        //and expand it so the border on it will show
        highlight.setBounds(new Rectangle(bound).expand(borderWidth + 1,
                borderWidth + 1));

        highlight.setBorder(new HighlightBorder(bound, borderWidth, c));
//        Debug.reportStack ("ItemOutlineEditPolicy.setContainerHighlight: " + "adding " + highlight);
        getLayer(LayerConstants.SCALED_FEEDBACK_LAYER).add(highlight);
    }

    /**
     * If the figure in question is not yet highlighted, showHighlight sets the
     * highlight color based on what is set in preferences, and calls setContainerHighlight.
     *
     */
    protected void showHighlight()
    {
        if (highlight == null)
        {
            highlight = scanForExistingHighlight (); //new Figure();
//            if (highlightColor != null) highlightColor.dispose();
            highlightColor = SWTResourceManager.getColor (PreferenceConverter.getColor(AcmeStudioPlugin.getDefault().getPreferenceStore(), ConnectionPatternPreferences.ELEMENT_HIGHTLIGHT_COLOR));
            setContainerHighlight(highlightColor);
        }
    }

    /**
	 * @return
	 */
	private Figure scanForExistingHighlight() {
		IFigure feedbackLayer = getLayer (LayerConstants.SCALED_FEEDBACK_LAYER);
		Figure hl = null;
		Iterator iter = feedbackLayer.getChildren().iterator ();
		while (iter.hasNext() && hl == null) {
			IFigure figure = (IFigure) iter.next();
			if (figure.getBorder() instanceof HighlightBorder) {
				hl = (Figure )figure;
			}
		}
		
		if (hl == null) hl = new Figure ();
		return hl;
	}

	/*
     * The main entry method, check to see if this item should be highlighted
     * (whether it is valid based on the rule chosen on the palette) and if so,
     * highlight it
     */
    public void showTargetFeedback(Request request)
    {
        Request req = null;
        PortConnectionRuleKey chosen = null;
        GraphicalEditPart holder = null;
        
        if (request instanceof CreateConnectionRequest)
        {
            req = (CreateConnectionRequest) request;

            holder = (GraphicalEditPart) getHost();
 			/* BRS: Added to cater for connection of Bindings in representations */
            chosen = ElementNodeEditPolicy.constructChosenRule(
                    (CreateConnectionRequest) req, holder);
        }
        else if (request instanceof ReconnectRequest)
        {
            req = (ReconnectRequest) request;
        }

        //find out whether or not the request passed in is of the correct type
        boolean reqCorrectType = (request.getType().equals(RequestConstants.REQ_MOVE)
                								|| request.getType().equals(RequestConstants.REQ_ADD)
                								|| request.getType().equals(RequestConstants.REQ_CLONE)
                								|| request.getType().equals(RequestConstants.REQ_CONNECTION_START)
                								|| request.getType().equals(RequestConstants.REQ_CONNECTION_END) 
                								|| request.getType().equals(RequestConstants.REQ_CREATE));
       
        if (reqCorrectType) //if the request is of the correct type
        {
            if (req == null) //if req is null, just show the highlight
                conditionallyShowHighlight(req);
            else
            {
                if (req instanceof CreateConnectionRequest)
                {
                    if (((CreateConnectionRequest)req).getStartCommand() != null)
                    {
                        PortConnectionRule passedRule = ((PortConnectionRule) req.getExtendedData().get("Rule"));
                        if (passedRule != null && chosen.matches(passedRule.generateKey()))
                            conditionallyShowHighlight(req);
                        else if (passedRule == null)
                        {
                            List allMatchingRules;
                            
                            //get an instance of the appropriate systemvisualization
                            SystemVisualization thisSysVis = VisualStyleRegistry.getSysVis((AcmeElement)getHost().getModel()); 
                            
                            //get all portconnection rules, so we can see if any match the currently chosen rule
                            List allRules = thisSysVis.getAllPortConnectionRules(null);
                            
                            allMatchingRules = SystemVisualization.getMatchingRulesInList(chosen, allRules);
                            
                            if (allMatchingRules.size() > 0)
                                conditionallyShowHighlight(req);
                        }
                    }
                }
            }
        }
        else if (req instanceof ReconnectRequest && req.getType().equals(RequestConstants.REQ_RECONNECT_TARGET))
        {
            Object savedep = ((ReconnectRequest) req).getExtendedData().get("savedHighlightReconnect");
            if (savedep instanceof PortEditPart && getHost() instanceof PortEditPart)
            	conditionallyShowHighlight(req);
            else if ((savedep instanceof ComponentEditPart && getHost() instanceof ComponentEditPart))
                conditionallyShowHighlight(req);
        }
    }
    
    /**
     * This method is called in the final stages of showing the highlight, when
     * the highlight should be shown, and all that needs to be checked is
     * whether or not the item is already highlighted, and whether this is
     * within a representation
     */
    private void conditionallyShowHighlight(Request req)
    {
        if (req instanceof CreateConnectionRequest) {
        	CreateConnectionRequest ccr = (CreateConnectionRequest )req;
        	if (ccr.getStartCommand() instanceof AddBindingCommand && (getHost () instanceof PortEditPart)) {
        		if (!ItemOutlineEditPolicy.allHighlightedFigs.contains(getHost()))
                    ItemOutlineEditPolicy.allHighlightedFigs.add(getHost());
        		showHighlight ();
        	}
        	else {
                if (!ItemOutlineEditPolicy.allHighlightedFigs.contains(getHost()))
                    ItemOutlineEditPolicy.allHighlightedFigs.add(getHost());
                showHighlight();
        	}
        } else {
        	//add the figure to the list of items that have highlighting shown
            if (!ItemOutlineEditPolicy.allHighlightedFigs.contains(getHost()))
                ItemOutlineEditPolicy.allHighlightedFigs.add(getHost());
            showHighlight();
        }
    }
}