package AcmeStudio.ui.editors.diagram.policies;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.handles.BendpointCreationHandle;
import org.eclipse.gef.handles.BendpointHandle;
import org.eclipse.gef.handles.BendpointMoveHandle;
import org.eclipse.gef.requests.BendpointRequest;

import AcmeStudio.ui.editors.diagram.actions.CreateBendpointCommand;
import AcmeStudio.ui.editors.diagram.actions.DeleteBendpointCommand;
import AcmeStudio.ui.editors.diagram.actions.MoveBendpointCommand;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;

public class RoleBendpointEditPolicy
    extends org.eclipse.gef.editpolicies.BendpointEditPolicy {

    private AcmeSystem _acmeSystem = null;
    
    protected Command getCreateBendpointCommand(BendpointRequest request) {
        CreateBendpointCommand com = new CreateBendpointCommand();
        Point p = request.getLocation();
        Connection conn = getConnection();
        RoleEditPart ep = (RoleEditPart) getHost();
        com.setEditPart(ep);
        conn.translateToRelative(p);

        com.setLocation(p);
        Point ref1 = ((RoleView) getConnection()).getStart();
        Point ref2 = ((RoleView) getConnection()).getEnd();

        conn.translateToRelative(ref1);
        conn.translateToRelative(ref2);

        com.setRelativeDimensions(p.getDifference(ref1), p.getDifference(ref2));
        com.setRole((AcmeRole) request.getSource().getModel());
        com.setIndex(request.getIndex());
        com.setRoleStart(ref1);
        com.setRoleEnd(ref2);
        return com;
    }

    protected Command getMoveBendpointCommand(BendpointRequest request) {
        AcmeRole role = (AcmeRole) request.getSource().getModel();
        //        if (Helper.getBendpoints(role) == null)
        //            return getCreateBendpointCommand(request);

        MoveBendpointCommand com = new MoveBendpointCommand();
        Point p = request.getLocation();
        Connection conn = getConnection();

        conn.translateToRelative(p);

        com.setLocation(p);
        com.setEditPart((RoleEditPart) getHost());

        Point ref1 = ((RoleView) getConnection()).getStart();
        Point ref2 = ((RoleView) getConnection()).getEnd();

        conn.translateToRelative(ref1);
        conn.translateToRelative(ref2);

        com.setRelativeDimensions(p.getDifference(ref1), p.getDifference(ref2));
        com.setRole(role);
        com.setIndex(request.getIndex());
        return com;
    }

    protected Command getDeleteBendpointCommand(BendpointRequest request) {
        DeleteBendpointCommand com = new DeleteBendpointCommand();
        Point p = request.getLocation();
        com.setLocation(p);
       com.setEditPart((RoleEditPart )request.getSource ());
        com.setRole((AcmeRole) request.getSource().getModel());
        com.setRoleStart (((RoleView )getConnection ()).getStart ());
        com.setRoleEnd (((RoleView )getConnection ()).getEnd ());
        com.setIndex(request.getIndex());
        return com;
    }

    public void eraseTargetFeedback(Request request) {
        if (REQ_SELECTION.equals(request.getType())) {
            //removeSelectionHandles();
        }
    }

    protected void addSelectionHandles() {
        super.addSelectionHandles();
        RoleEditPart roleEditPart = (RoleEditPart) getHost();
        if (_acmeSystem == null) {

            AcmeRole role = (AcmeRole) roleEditPart.getModel();
            _acmeSystem = Helper.getRootAcmeSystem(role);
        }
//        SystemEditPart systemEditPart =
//            (SystemEditPart) getHost()
//                .getRoot()
//                .getViewer()
//                .getEditPartRegistry()
//                .get(
//                _acmeSystem);
//        if (systemEditPart != null) {
//            SelectionHandlesEditPolicy selPolicy =
//                systemEditPart.getBendpointSelectionHandlesEditPolicy();
//            //            if (selPolicy instanceof RoleBendpointEditPolicy) {
//            //                ((RoleBendpointEditPolicy) selPolicy).removeSelectionHandles();
//            //            }
//            systemEditPart.setBendpointSelectionHandlesEditPolicy(this);
//        }
    }

    protected synchronized void showCreateBendpointFeedback(BendpointRequest request) {
        RoleView rv = (RoleView) getConnection();
        SystemView sv = (SystemView) rv.getParent();
        if (rv._isBendpointBeingCreated)
            return;
        if (getHost().getRoot().getViewer() != null) {
            FigureCanvas figureCanvas =
                (FigureCanvas) getHost().getRoot().getViewer().getControl();
            rv.setFigureCanvas(figureCanvas);
        }
        Point zoomedOffset = rv.getOffset();
        Point start = rv.getStart();
        Point end = rv.getEnd();
        Point offset = zoomedOffset.getCopy();
        Point zoomedStart = start.getCopy();
        Point zoomedEnd = end.getCopy();
        Point zoomedLocation = request.getLocation();
        Point location = zoomedLocation.getCopy();

        RootEditPart root = getHost().getRoot();
        if (root instanceof ScalableFreeformRootEditPart) {
            ScalableFreeformRootEditPart scalableRoot =
                (ScalableFreeformRootEditPart) root;
            ScalableFigure scalablePane = null;
            ZoomManager zm = scalableRoot.getZoomManager();
            if (zm != null && zm.getZoom() != 1.0) {
                scalablePane = zm.getScalableFigure();
                scalablePane.translateToParent(zoomedStart);
                scalablePane.translateToParent(zoomedEnd);
                scalablePane.translateFromParent(offset);
                scalablePane.translateFromParent(location);
            }
        }

        //super.showCreateBendpointFeedback(request);
        Point ref1 = start;
        Point ref2 = end;
        AcmeRole role = (AcmeRole) request.getSource().getModel();
        int i = request.getIndex();
        if (rv._isBendpointMoving) {
            Helper.removeBendpoint(role, i, ref1, ref2);
        } else {
            rv._isBendpointMoving = true;
        }
        PointList bendpoints = sv.getBendpoints(role, ref1, ref2, null);
        bendpoints = sv.arrangeBendpoints(role, bendpoints, ref1, ref2);
        Dimension diff = new Dimension();
        diff.height = offset.y;
        diff.width = offset.x;
        location.translate(diff);
        bendpoints =
            Helper.addBendpoint(role, i, location, bendpoints);
        sv.arrangeBendpoints(role, bendpoints, ref1, ref2);
        Helper.setBendpoints(role, bendpoints);
        ((RoleEditPart) getHost()).refresh();
    }

    protected synchronized void showMoveBendpointFeedback(BendpointRequest request) {
        RoleView rv = (RoleView) getConnection();
        SystemView sv = (SystemView) rv.getParent();
        rv._isBendpointMoving = true;
        if (getHost().getRoot().getViewer() != null) {
            FigureCanvas figureCanvas =
                (FigureCanvas) getHost().getRoot().getViewer().getControl();
            rv.setFigureCanvas(figureCanvas);
        }
        Point zoomedOffset = rv.getOffset();
        Point start = rv.getStart();
        Point end = rv.getEnd();
        Point offset = zoomedOffset.getCopy();
        Point zoomedStart = start.getCopy();
        Point zoomedEnd = end.getCopy();
        Point zoomedLocation = request.getLocation();
        Point location = zoomedLocation.getCopy();

        RootEditPart root = getHost().getRoot();
        if (root instanceof ScalableFreeformRootEditPart) {
            ScalableFreeformRootEditPart scalableRoot =
                (ScalableFreeformRootEditPart) root;
            ScalableFigure scalablePane = null;
            ZoomManager zm = scalableRoot.getZoomManager();
            if (zm != null && zm.getZoom() != 1.0) {
                scalablePane = zm.getScalableFigure();
                scalablePane.translateToParent(zoomedStart);
                scalablePane.translateToParent(zoomedEnd);
                scalablePane.translateFromParent(offset);
                scalablePane.translateFromParent(location);
            }
        }
        
        //if (rv._isBendpointBeingCreated) return;
        //super.showMoveBendpointFeedback(request);
        Point ref1 = start;
        Point ref2 = end;
        AcmeRole role = (AcmeRole) request.getSource().getModel();
        int i = request.getIndex();
        PointList bendpoints = sv.getBendpoints(role, ref1, ref2, null);
        bendpoints = sv.arrangeBendpoints(role, bendpoints, ref1, ref2);
//        if (bendpoints == null || bendpoints.size () <= i)
        	super.showMoveBendpointFeedback(request);
//        if (bendpoints == null || bendpoints.size() <= i) {
//            super.showMoveBendpointFeedback(request);
//            return;
//        }
        //        if (i == bendpoints.size() - 1) {
        //            super.showMoveBendpointFeedback(request);
        //            return;
        //        } 
        Dimension diff = new Dimension();
        diff.height = offset.y;
        diff.width = offset.x;
        location.translate(diff);
        bendpoints = Helper.setBendpoint(role, i, location, bendpoints);
        //Helper.getBendpoints(role, ref1, ref2, null);
        bendpoints = sv.arrangeBendpoints(role, bendpoints, ref1, ref2);
        Helper.setBendpoints(role, bendpoints);
        ((RoleEditPart) getHost()).refresh();
    }

    public void removeSelectionHandles() {
        super.removeSelectionHandles();
    }

    public void propertyChange(PropertyChangeEvent evt) {
        if (getHost().getSelected() != EditPart.SELECTED_NONE) {
            if (handles == null) {
                addSelectionHandles();
            } else {
                int count = handles.size();
                int points = getConnection().getPoints().size();
                if (count != points * 2 - 3)
                    addSelectionHandles();
            }
        }
    }

    protected List createSelectionHandles() {
        List list = new ArrayList();
        ConnectionEditPart connEP = (ConnectionEditPart) getHost();
        PointList points = getConnection().getPoints();
        for (int i = 0; i < points.size() - 2; i++) {
            BendpointHandle bh = new BendpointCreationHandle(connEP, i);
            Dimension size = bh.getPreferredSize();
            bh.setPreferredSize((int) (size.width*1.5), (int) (size.height*1.5));
            list.add(bh);
            bh = new BendpointMoveHandle(connEP, i);
            size = bh.getPreferredSize();  
            bh.setPreferredSize((int) (size.width*1.5), (int) (size.height*1.5));
            list.add(bh);
        }
        if (points.size() == 2) {
            BendpointHandle bh = new BendpointCreationHandle(connEP, 0);
            Dimension size = bh.getPreferredSize();  
            bh.setPreferredSize((int) (size.width*1.5), (int) (size.height*1.5));
            list.add(bh);
        }

        return list;
    }

}