package AcmeStudio.ui.editors.diagram.policies;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseListener;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.gef.*;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.handles.ConnectionHandle;
import org.eclipse.gef.requests.LocationRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import AcmeStudio.ui.editors.diagram.actions.DeleteElementCommand;
import AcmeStudio.ui.editors.diagram.actions.MoveRoleCommand;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.ui.editors.diagram.parts.RoleMidHandle;
import AcmeStudio.ui.editors.diagram.parts.RoleTracker;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * The edit policy regarding role endpoint
 * 
 */
public class RoleEndpointEditPolicy
    extends org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy
    implements MouseListener {

    /**
     * The acme system in which this policy applies.
     */
    private AcmeSystem _acmeSystem = null;
    final static int _HOVER_DELAY = 8;

    /**
     * @see org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy#addSelectonHandles()
     */
    protected void addSelectionHandles() {
        super.addSelectionHandles();
        RoleEditPart roleEditPart = (RoleEditPart) getHost();
        RoleView rv = (RoleView) getConnectionFigure();
        rv.setLineWidth(rv.getVisualization().getOutlineWidth()*2);
        
        if (_acmeSystem == null) {

            AcmeRole role = (AcmeRole) roleEditPart.getModel();
            _acmeSystem = Helper.getRootAcmeSystem(role);
        }
//        SystemEditPart systemEditPart =
//            (SystemEditPart) getHost()
//                .getRoot()
//                .getViewer()
//                .getEditPartRegistry()
//                .get(
//                _acmeSystem);
//        if (systemEditPart != null) {
//            SelectionHandlesEditPolicy selPolicy =
//                systemEditPart.getSelectionHandlesEditPolicy();
//            if (selPolicy instanceof RoleEndpointEditPolicy) {
////                ((RoleEndpointEditPolicy) selPolicy).removeSelectionHandles();
//            } else if (selPolicy instanceof ElementSelectionEditPolicy) {
//                //                ((ElementSelectionEditPolicy) selPolicy)
//                //                    .removeSelectionHandles();
//            }
//            systemEditPart.setSelectionHandlesEditPolicy(this);
//        }
    }

    /**
     * Get the connection figure this policy applies to
     * @return The connection figure
     */
    protected PolylineConnection getConnectionFigure() {
        return (PolylineConnection) ((GraphicalEditPart) getHost()).getFigure();
    }

    /**
     * @see org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy#removeSelectonHandles()
     */
    public void removeSelectionHandles() {
        super.removeSelectionHandles();
        RoleView rv = (RoleView) getConnectionFigure();
        rv.setLineWidth(rv.getVisualization().getOutlineWidth());
        //_timer = 0;
    }

    /**
     * @see org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy#showConnectionMoveFeedback(ReconnectRequest)
     */
    protected void showConnectionMoveFeedback(ReconnectRequest request) {
        super.showConnectionMoveFeedback(request);
        if (_acmeSystem == null) {
            RoleEditPart roleEditPart = (RoleEditPart) getHost();
            AcmeRole role = (AcmeRole) roleEditPart.getModel();
            _acmeSystem = Helper.getRootAcmeSystem(role);
        }
        RoleTracker.moveRole((RoleEditPart) getHost(), request, _acmeSystem);
    }

    /**
     * @see org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy#createSelectionHandles()
     */
    public List createSelectionHandles() {
        RoleView roleView = (RoleView) getConnectionFigure();
        //        AcmeElement acmeElement = roleView.getModel();
        //        dispatchNewSelection(acmeElement);
        int d = RoleTracker.getSelectionThreshold(roleView);

        List list = new ArrayList();
        for (int i = 0; i <= d; i++) {
            ConnectionHandle roleMidHandle = createSelectionHandle(i, false);
            roleMidHandle.addMouseListener(this);
            list.add(roleMidHandle);
        }
        ConstrainedResizeHandleKit.addHandles ((GraphicalEditPart )getHost(), list);
        return list;
    }

    /**
     * Create a single selection handle
     * @param distance The distance to the end of the connection
     * @param visible If the selection handle is visible
     */
    protected ConnectionHandle createSelectionHandle(
        int distance,
        boolean visible) {
        return new RoleMidHandle(
            (ConnectionEditPart) getHost(),
            visible,
            distance);
    }

    /**
     * @see org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy#getCommand()
     */
    public Command getCommand(Request request) {
        if (request == null) return null;
        if (REQ_DELETE.equals(request.getType())) {
            DeleteElementCommand deleteCmd =
                new DeleteElementCommand(getHost());
            deleteCmd.setChild((AcmeElement) getHost().getModel());
            return deleteCmd;
        } else if (request instanceof ReconnectRequest) {
            ReconnectRequest rr = (ReconnectRequest) request;
            MoveRoleCommand cmd =
                new MoveRoleCommand(rr.getTarget(), false, false);
            cmd.setLocation(((LocationRequest) request).getLocation());
            cmd.setRoleEditPart((RoleEditPart) getHost());
            return cmd;
            
        }
        return super.getCommand(request);
    }

//    private void selectRole() {
//        Point location = _currentRequestLocation;
//        RoleEditPart roleEditPart = (RoleEditPart) getHost();
//        if (_acmeSystem == null) {
//            AcmeRole role = (AcmeRole) roleEditPart.getModel();
//            _acmeSystem = Helper.getRootAcmeSystem(role);
//        }
//        RootEditPart root = roleEditPart.getRoot();
//        GraphicalViewer viewer = (GraphicalViewer) root.getViewer();
//        //        int selectionSize = viewer.getSelectedEditParts().size();
//        //        if (selectionSize == 0 || (selectionSize > 0 && viewer.getSelectedEditParts().get(0) != roleEditPart)) {
//        //            viewer.select(roleEditPart);
//        //        }
//
////        SystemEditPart systemEditPart =
////            (SystemEditPart) root.getViewer().getEditPartRegistry().get(
////                _acmeSystem);
////        SelectionHandlesEditPolicy selPolicy =
////            systemEditPart.getSelectionHandlesEditPolicy();
////        if (selPolicy instanceof RoleEndpointEditPolicy) {
//////            ((RoleEndpointEditPolicy) selPolicy).removeSelectionHandles();
////        }
//        RoleView roleView = (RoleView) getConnectionFigure();
//        Point offset = roleView.getOffset().getCopy();
//        Point start = roleView.getLastBendpoint().getCopy();
//        Point end = roleView.getEnd().getCopy();
//
//        if (root instanceof ScalableFreeformRootEditPart) {
//            ScalableFreeformRootEditPart scalableRoot =
//                (ScalableFreeformRootEditPart) root;
//            ScalableFreeformLayeredPane scalablePane = null;
//            ZoomManager zm = scalableRoot.getZoomManager();
//            if (zm != null) {
//                scalablePane = zm.getPane();
//                scalablePane.translateToParent(start);
//                scalablePane.translateToParent(end);
//            }
//        }
//        double selectionDistance =
//            Helper.calculateSelectionDistance(
//                start,
//                end,
//                location,
//                -1.0,
//                offset);
//        Visualization visStyle = roleView.getVisualization();
//        int d = getSelectionThreshold(start, end, visStyle.getSize());
//        if (handles == null
//            && selectionDistance <= d
//            && selectionDistance >= 0) {
//  
//            selPolicy = systemEditPart.getBendpointSelectionHandlesEditPolicy();
//            if (selPolicy instanceof RoleBendpointEditPolicy) {
//                ((RoleBendpointEditPolicy) selPolicy).removeSelectionHandles();
//            }
//
//            addSelectionHandles();
//            MultiPageAcmeEditor editor = ASContext.current().getActiveEditor();
//        }
//    }
    /**
     * @see org.eclipse.gef.editpolicies.AbstraceEditPolicy#showTargetFeedback(Request)
     */
    public void showTargetFeedback(Request request) {
        super.showTargetFeedback(request);
//        if (!_locked && REQ_SELECTION.equals(request.getType())) {
//            _locked = true;
//            if (_timer++ > _HOVER_DELAY) {
//                _timer = 0;
//
//                _currentRequestLocation =
//                    ((LocationRequest) request).getLocation();
//                selectRole();
//            }
//            _locked = false;
//        }
    }


    /**
     * @see org.eclipse.gef.editpolicies.AbstraceEditPolicy#eraseTargetFeedback(Request)
     */
    public void eraseTargetFeedback(Request request) {
        super.eraseTargetFeedback(request);
//        if (REQ_SELECTION_HOVER.equals(request.getType())) {
//            //removeSelectionHandles();
//        }
    }

    /**
     * Called when a pressed mouse button has been released.
     * @param me The MouseEvent object
     */
    public void mouseReleased(MouseEvent me) {
    }

    public void mousePressed(MouseEvent me) {
        GraphicalViewer viewer =
            (GraphicalViewer) getHost().getRoot().getViewer();
        int selectionSize = viewer.getSelectedEditParts().size();
        if (selectionSize == 0
            || (selectionSize > 0
                && viewer.getSelectedEditParts().get(0) != getHost())) {
            viewer.select(getHost());
        }
    }

    public void mouseDoubleClicked(MouseEvent me) {
    }

	/* (non-Javadoc)
	 * @see org.eclipse.gef.EditPolicy#understandsRequest(org.eclipse.gef.Request)
	 */
	public boolean understandsRequest(Request req) {
		if (RequestConstants.REQ_MOVE.equals (req.getType())) return true;
		return super.understandsRequest (req);
	}
}