package AcmeStudio.ui.editors.diagram.policies;

/*
 * Licensed Material - Property of IBM (C) Copyright IBM Corp. 2001, 2002 - All
 * Rights Reserved. US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
import java.util.Iterator;
import java.util.List;
import org.eclipse.core.runtime.*;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.*;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editpolicies.SelectionEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.rulers.RulerProvider;
import AcmeStudio.model.AddElementCommand;
import AcmeStudio.model.TypeAssignmentCommand;
import AcmeStudio.ui.editors.diagram.IElementFactory;
import AcmeStudio.ui.editors.diagram.actions.CloneCommand;
import AcmeStudio.ui.editors.diagram.actions.CreateElementCommand;
import AcmeStudio.ui.editors.diagram.actions.SetElementConstraintCommand;
import AcmeStudio.ui.editors.diagram.figures.AcmeVisElement;
import AcmeStudio.ui.editors.diagram.figures.RoleView;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.*;
import AcmeStudio.ui.editors.diagram.ruler.commands.ChangeGuideCommand;
import AcmeStudio.ui.editors.diagram.ruler.model.DiagramGuide;
import AcmeStudio.ui.editors.diagram.styles.Location;
import AcmeStudio.ui.editors.diagram.templates.AssignTypeTemplateFactory;
import AcmeStudio.util.Debug;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeSystem;
public class SystemXYLayoutEditPolicy extends org.eclipse.gef.editpolicies.XYLayoutEditPolicy {

	public static XYLayoutEditPolicy []	_extensions	= null;

	/**
	 *  
	 */
	public SystemXYLayoutEditPolicy() {
		super ();
		if (_extensions == null) {
			IExtensionRegistry registry = org.eclipse.core.runtime.Platform.getExtensionRegistry ();
			IExtensionPoint point = registry.getExtensionPoint ("AcmeStudio.XYLayoutPolicy");
			if (point == null) return;
			IExtension [] extensions = point.getExtensions ();
			_extensions = new XYLayoutEditPolicy [extensions.length];
			for (int i = 0; i < extensions.length; i++) {
				IConfigurationElement [] elements = extensions [i].getConfigurationElements ();
				for (int j = 0; j < elements.length; j++) {
					if ("XYLayoutPolicy".equals (elements [j].getName ())) {
						try {
							XYLayoutEditPolicy policy = (XYLayoutEditPolicy )elements [j].createExecutableExtension ("class");
							_extensions [i] = policy;
						}
						catch (CoreException e) {
							Debug.errorln (e);
						}
					}
				}
			}
		}
	}

	/**
	 * @param request
	 * @return @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCloneCommand(org.eclipse.gef.requests.ChangeBoundsRequest)
	 */
	protected Command getCloneCommand(ChangeBoundsRequest request) {
		CloneCommand clone = new CloneCommand ();
		Iterator i = request.getEditParts ().iterator ();
		GraphicalEditPart currPart = null;
		while (i.hasNext ()) {
			currPart = (GraphicalEditPart )i.next ();
			AcmeElement element = (AcmeElement )currPart.getModel ();
			if (!(element instanceof AcmeSystem)) clone.addPart (element, (Rectangle )getConstraintForClone (currPart, request));
		}
		Integer guidePos = (Integer )request.getExtendedData ().get (SnapToGuides.KEY_HORIZONTAL_GUIDE);
		if (guidePos != null) {
			int hAlignment = ((Integer )request.getExtendedData ().get (SnapToGuides.KEY_HORIZONTAL_ANCHOR)).intValue ();
			clone.setGuide (findGuideAt (guidePos.intValue (), true), hAlignment, true);
		}
		// Attach to vertical guide, if one is given
		guidePos = (Integer )request.getExtendedData ().get (SnapToGuides.KEY_VERTICAL_GUIDE);
		if (guidePos != null) {
			int vAlignment = ((Integer )request.getExtendedData ().get (SnapToGuides.KEY_VERTICAL_ANCHOR)).intValue ();
			clone.setGuide (findGuideAt (guidePos.intValue (), false), vAlignment, false);
		}
		if (clone.getParts ().isEmpty ()) return null;
		return clone;
	}

	protected Command createAddCommand(EditPart childEditPart, Object constraint) {
		if (!(childEditPart.getModel () instanceof AcmeElement)) return null;
		AcmeElement element = (AcmeElement )childEditPart.getModel ();
		Rectangle rect = (Rectangle )constraint;
		AddElementCommand add = new AddElementCommand ();
		SystemEditPart systemEditPart = (SystemEditPart )getHost ();
		add.setParent ((AcmeSystem )systemEditPart.getModel ());
		SystemView systemView = (SystemView )systemEditPart.getFigure ();
		add.setChild (element);
		add.setLabel ("element added");
		add.setDebugLabel ("add element");
		SetElementConstraintCommand setConstraint = new SetElementConstraintCommand ((ElementEditPart )childEditPart);
		setConstraint.setSystemView (systemView);
		setConstraint.setLocation (rect);
		setConstraint.setElement (element);
		setConstraint.setLabel ("constraint set");
		setConstraint.setDebugLabel ("set constraint");
		return add.chain (setConstraint);
	}

	protected Command createChangeConstraintCommand(EditPart child, Object constraint) {
		if (!(child instanceof BindingEditPart) && child.getModel () instanceof AcmeElement) {
			SetElementConstraintCommand locationCommand = new SetElementConstraintCommand ((GraphicalEditPart )child);
			SystemEditPart systemEditPart = (SystemEditPart )getHost ();
			SystemView systemView = (SystemView )systemEditPart.getFigure ();
			locationCommand.setSystemView (systemView);
			locationCommand.setElement ((AcmeElement )child.getModel ());
			locationCommand.setLocation ((Rectangle )constraint);
			return locationCommand;
		}
		return null;
	}

	protected EditPolicy createChildEditPolicy(EditPart child) {
		if (child instanceof ComponentEditPart || child instanceof ConnectorEditPart || child instanceof PortEditPart
				|| child instanceof AttachmentEditPart || child instanceof RepresentationBorderEditPart) {
			return new ElementSelectionEditPolicy ();
		}
		else if (child instanceof RoleEditPart) {
			return new RoleEndpointEditPolicy ();
		}
		Iterator i = SystemEditPart.factories.iterator ();
		while (i.hasNext ()) {
			IElementFactory factory = (IElementFactory )i.next ();
			SelectionEditPolicy policy = null;
			if ((policy = factory.getSelectionPolicy (child)) != null) {
				return policy;
			}
		}
		return new ElementSelectionEditPolicy ();
	}

	protected Command getCreateCommand(CreateRequest request) {
		Object object = request.getNewObject ();
		if (object instanceof AssignTypeTemplateFactory.AssignTypeTarget) {
			AssignTypeTemplateFactory.AssignTypeTarget target = (AssignTypeTemplateFactory.AssignTypeTarget )object;
			TypeAssignmentCommand assign = new TypeAssignmentCommand (TypeAssignmentCommand.ASSIGN_TYPE, target.getTarget (),
					(AcmeElementType )request.getNewObjectType (), true, getHost ().getViewer ());
			return assign;
		}
		else if (object instanceof AcmeElement) {
			AcmeElement child = (AcmeElement )object;
			CreateElementCommand create = new CreateElementCommand (getHost ());
			SystemEditPart systemEditPart = (SystemEditPart )getHost ();
			SystemView systemView = (SystemView )systemEditPart.getFigure ();
			AcmeSystem system = (AcmeSystem )systemEditPart.getModel ();
			create.setSystem (system);
			create.setSystemView (systemView);
			create.setChild (child);
			Rectangle constraint = (Rectangle )getConstraintFor (request);
			Point location = constraint.getLocation ();
			AcmeVisElement visElement = systemView.findFigureByLocation (location);
			if (visElement != null) {
				create.setParent (visElement.getModel ());
				create.setParentVis (visElement);
				//            create.setSize(visElement.getSize());
			}
			Dimension size = request.getSize();
			if (size == null) {
				size = Helper.getSize (child);
			}
			else {
				create.setSize (size);
			}
			Rectangle r = new Rectangle (location, size);
			systemView.updateElement (child, r);
			systemView.computeChildLocations ();
			Location newL = null;
			if ((newL = systemView.getCalculatedLocation (child)) != null) {
				location = newL.getRectangle ().getLocation ();
			}
			create.setLocation (location);
			create.setLabel ("Create");
			
			return create;
		}
		return null;
	}

	protected Command getDeleteDependantCommand(Request request) {
		return null;
	}

	protected Command getOrphanChildrenCommand(Request request) {
		return null;
	}

	//// protected void showDragTargetFeedback(Request request) {
	//// Helper.debug("entered");
	//		// Don't show any target feedback on the GraphViewer
	//		// background.
	//	}
	//protected void showSizeOnDropFeedback(Tool tool) {
	//	if (((CreationTool)tool).getNewObject() instanceof Circuit) {
	//		super.showSizeOnDropFeedback(tool);
	//	}
	//}
	public void showTargetFeedback(Request request) {
		//Helper.debug("entered feedback");
	}

	public void showSourceFeedback(Request request) {
		//Helper.debug("entered feedback");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy#getCurrentConstraintFor(org.eclipse.gef.GraphicalEditPart)
	 */
	protected Rectangle getCurrentConstraintFor(GraphicalEditPart child) {
		Rectangle r = super.getCurrentConstraintFor (child);
		if (r == null) {
			return new Rectangle (0, 0, 0, 0);
		}
		return r;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.requests.ChangeBoundsRequest,
	 *            org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(ChangeBoundsRequest request, EditPart child, Object constraint) {
//		if (child.getModel () instanceof AcmeElement) {
			GraphicalEditPart part = (GraphicalEditPart )child;
			SetElementConstraintCommand cmd = (SetElementConstraintCommand )createChangeConstraintCommand (child, constraint);
			if (cmd == null) return null;
			Command result = cmd;
			if (child instanceof ElementEditPart) {
				ElementEditPart eepart = (ElementEditPart )child;
				if ((request.getResizeDirection () & PositionConstants.NORTH_SOUTH) != 0) {
					result = chainHorizontalGuide (request, eepart, result, true);
				}
				if ((request.getResizeDirection () & PositionConstants.EAST_WEST) != 0) {
					result = chainVerticalGuide (request, eepart, result, true);
				}
				if (request.getType ().equals (REQ_MOVE_CHILDREN) || request.getType ().equals (REQ_ALIGN_CHILDREN)) {
					result = chainHorizontalGuide (request, eepart, result, false);
					result = chainVerticalGuide (request, eepart, result, false);
				}
			}
			return cmd;
//		}
//		return null;
	}

	/**
	 * @param request
	 * @param part
	 * @param result
	 */
	private Command chainVerticalGuide(ChangeBoundsRequest request, ElementEditPart part, Command result, boolean checkPart) {
		Integer guidePos = (Integer )request.getExtendedData ().get (SnapToGuides.KEY_VERTICAL_GUIDE);
		if (guidePos != null) {
			int vAlignment = ((Integer )request.getExtendedData ().get (SnapToGuides.KEY_VERTICAL_ANCHOR)).intValue ();
			ChangeGuideCommand cgm = new ChangeGuideCommand (part, false);
			cgm.setNewGuide (findGuideAt (guidePos.intValue (), false), vAlignment);
			result = result.chain (cgm);
		}
		else if (checkPart && part.getVerticalGuide () != null) {
			int alignment = part.getVerticalGuide ().getAlignment (part);
			int edgeBeingResized = 0;
			if ((request.getResizeDirection () & PositionConstants.WEST) != 0)
				edgeBeingResized = -1;
			else
				edgeBeingResized = 1;
			if (alignment == edgeBeingResized) {
				ChangeGuideCommand cgm = new ChangeGuideCommand (part, false);
				result = result.chain (cgm);
			}
		}
		return result;
	}

	/**
	 * @param request
	 * @param part
	 * @param result
	 * @return
	 */
	private Command chainHorizontalGuide(ChangeBoundsRequest request, ElementEditPart part, Command result, boolean checkPart) {
		Integer guidePos = (Integer )request.getExtendedData ().get (SnapToGuides.KEY_HORIZONTAL_GUIDE);
		if (guidePos != null) {
			int hAlignment = ((Integer )request.getExtendedData ().get (SnapToGuides.KEY_HORIZONTAL_ANCHOR)).intValue ();
			ChangeGuideCommand cgm = new ChangeGuideCommand (part, true);
			cgm.setNewGuide (findGuideAt (guidePos.intValue (), true), hAlignment);
			result = result.chain (cgm);
		}
		else if (checkPart && part.getHorizontalGuide () != null) {
			int alignment = part.getHorizontalGuide ().getAlignment (part);
			int edgeBeingResized = 0;
			if ((request.getResizeDirection () & PositionConstants.NORTH) != 0) {
				edgeBeingResized = -1;
			}
			else {
				edgeBeingResized = 1;
			}
			if (alignment == edgeBeingResized) {
				ChangeGuideCommand cgm = new ChangeGuideCommand (part, true);
				result = result.chain (cgm);
			}
		}
		return result;
	}

	/**
	 * @param i
	 * @param b
	 * @return
	 */
	private DiagramGuide findGuideAt(int pos, boolean horizontal) {
		List guides = ((RulerProvider )getHost ().getViewer ().getProperty (
				horizontal ? RulerProvider.PROPERTY_VERTICAL_RULER : RulerProvider.PROPERTY_HORIZONTAL_RULER)).getGuides ();
		for (Iterator i = guides.iterator (); i.hasNext ();) {
			DiagramGuide guide = (DiagramGuide )i.next ();
			if (pos == guide.getPosition ()) {
				return guide;
			}
		}
		throw new RuntimeException ("SystemXYLayoutPolicy: Guide not found at position " + pos);
	}

	/**
	 * @param request
	 * @param child
	 * @return @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getConstraintFor(org.eclipse.gef.requests.ChangeBoundsRequest,
	 *                org.eclipse.gef.GraphicalEditPart)
	 */
	protected Object getConstraintFor(ChangeBoundsRequest request, GraphicalEditPart child) {
		if (child instanceof RoleEditPart) {
			RoleView roleView = (RoleView )child.getFigure ();
			Rectangle rect = roleView.getDecoration ().getBounds ();
			rect = request.getTransformedRectangle (rect);
			rect.translate (getLayoutOrigin ().getNegated ());
			return getConstraintFor (rect);
		}
		else
			return super.getConstraintFor (request, child);
	}

	/**
	 * @param request
	 * @return @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getCommand(org.eclipse.gef.Request)
	 */
	public Command getCommand(Request request) {
		Command command = null;
		for (int i = 0; i < _extensions.length && (command == null || command instanceof UnexecutableCommand); i++) {
			_extensions [i].setHost(getHost ());
			_extensions [i].setXyLayout(getXYLayout());
			try {
				command = _extensions [i].getCommand (request);
			}
			catch (Exception e) {
				// continue trying other extensions
			}
		}
		if (command == null || command instanceof UnexecutableCommand) {
			command = super.getCommand (request);
		}
		return command;
	}
}