/*
 * Created on Dec 8, 2003
 *  
 */
package AcmeStudio.ui.editors.diagram.ruler;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.rulers.RulerChangeListener;
import org.eclipse.gef.rulers.RulerProvider;

import AcmeStudio.ui.editors.diagram.ruler.commands.CreateGuideCommand;
import AcmeStudio.ui.editors.diagram.ruler.commands.DeleteGuideCommand;
import AcmeStudio.ui.editors.diagram.ruler.commands.MoveGuideCommand;
import AcmeStudio.ui.editors.diagram.ruler.model.DiagramGuide;
import AcmeStudio.ui.editors.diagram.ruler.model.DiagramRuler;

/**
 * @author Bradley Schmerl Created Dec 8, 2003
 */
public class DiagramRulerProvider extends RulerProvider {

	private DiagramRuler _ruler;
	private List _listeners = new ArrayList();

	private PropertyChangeListener _rulerListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
			if (evt.getPropertyName().equals(DiagramRuler.PROPERTY_CHILDREN)) {
				DiagramGuide guide = (DiagramGuide) evt.getNewValue();
				if (getGuides().contains(guide)) {
					guide.addPropertyChangeListener(_guideListener);
				}
				else {
					guide.removePropertyChangeListener(_guideListener);
				}
				for (int i = 0; i < _listeners.size(); i++) {
					((RulerChangeListener) _listeners.get(i)).notifyGuideReparented(guide);
				}
			}
			else {
				for (int i = 0; i < _listeners.size(); i++) {
					((RulerChangeListener) _listeners.get(i)).notifyUnitsChanged(_ruler.getUnit());
				}
			}
		}
	};
	
	private PropertyChangeListener _guideListener = new PropertyChangeListener () {
		public void propertyChange(PropertyChangeEvent evt) {
			if (evt.getPropertyName().equals(DiagramGuide.PROPERTY_CHILDREN)) {
				for (int i = 0; i < _listeners.size(); i++) {
					((RulerChangeListener)_listeners.get(i))
					.notifyPartAttachmentChanged(evt.getNewValue(), evt.getSource());
				}
			} else {
				for (int i = 0; i < _listeners.size(); i++) {
					((RulerChangeListener)_listeners.get(i))
					.notifyGuideMoved(evt.getSource());
				}
			}
		}	
	};
	/**
	 *  
	 */
	public DiagramRulerProvider(DiagramRuler ruler) {
		_ruler = ruler;
		_ruler.addPropertyChangeListener(_rulerListener);
		List guides = getGuides ();
		for (Iterator iter = guides.iterator(); iter.hasNext(); ) {
			DiagramGuide guide = (DiagramGuide) iter.next();
			guide.addPropertyChangeListener(_guideListener);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#getCreateGuideCommand(int)
	 */
	public Command getCreateGuideCommand(int position) {
		return new CreateGuideCommand (_ruler, position);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#getDeleteGuideCommand(java.lang.Object)
	 */
	public Command getDeleteGuideCommand(Object guide) {
		return new DeleteGuideCommand ((DiagramGuide )guide, _ruler);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#getMoveGuideCommand(java.lang.Object,
	 *      int)
	 */
	public Command getMoveGuideCommand(Object guide, int positionDelta) {
		return new MoveGuideCommand ((DiagramGuide )guide, positionDelta);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#getGuides()
	 */
	public List getGuides() {
		return _ruler.getGuides();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#getGuidePositions()
	 */
	public int[] getGuidePositions() {
		List guides = getGuides ();
		int [] result = new int [guides.size ()];
		for (int i = 0; i < result.length; i++) {
			result [i] = ((DiagramGuide )guides.get (i)).getPosition ();
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#getGuidePosition(java.lang.Object)
	 */
	public int getGuidePosition(Object guide) {
		return ((DiagramGuide )guide).getPosition ();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#getRuler()
	 */
	public Object getRuler() {
		return _ruler;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#getUnit()
	 */
	public int getUnit() {
		return _ruler.getUnit();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#setUnit(int)
	 */
	public void setUnit(int newUnit) {
		_ruler.setUnit (newUnit);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#addRulerChangeListener(org.eclipse.gef.ui.parts.RulerChangeListener)
	 */
	public void addRulerChangeListener(RulerChangeListener listener) {
		if (!_listeners.contains (listener)) {
			_listeners.add (listener);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.parts.RulerProvider#removeRulerChangeListener(org.eclipse.gef.ui.parts.RulerChangeListener)
	 */
	public void removeRulerChangeListener(RulerChangeListener listener) {
		_listeners.remove (listener);
	}

}