/*
 * Created on Jun 3, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.ruler.commands;

import org.eclipse.gef.commands.Command;

import AcmeStudio.ui.editors.diagram.parts.ElementEditPart;
import AcmeStudio.ui.editors.diagram.ruler.model.DiagramGuide;
import acmeLib.elements.AcmeElement;


/**
 * @author Bradley Schmerl
 * Created Jun 3, 2004
 */
public class ChangeGuideCommand extends Command {

	private ElementEditPart _part;
	private DiagramGuide _oldGuide, _newGuide;
	private int _oldAlign, _newAlign;
	private boolean _horizontal;
	/**
	 * 
	 */
	public ChangeGuideCommand(ElementEditPart part, boolean horizontalGuide) {
		super ();
		_part = part;
		_horizontal = horizontalGuide;
	}

	protected void changeGuide (DiagramGuide oldGuide, DiagramGuide newGuide, int newAlignment) {
		if (oldGuide != null && oldGuide != newGuide) {
			oldGuide.detachElement((AcmeElement )_part.getModel());
		}
		
		// Re-attach the part even if the old and the new guide are the same
		// because the alignment could have changed
		if (newGuide != null) {
			newGuide.attachElement((AcmeElement )_part.getModel (), newAlignment);
		}
	}
	
	public void execute () {
		// Cache the old values
		_oldGuide = _horizontal ? _part.getHorizontalGuide () : _part.getVerticalGuide ();
		if (_oldGuide != null) {
			_oldAlign = _oldGuide.getAlignment (_part);
		}
		redo ();
	}

	public void redo () {
		changeGuide (_oldGuide, _newGuide, _newAlign);
	}
	
	public void setNewGuide (DiagramGuide guide, int alignment) {
		_newGuide = guide;
		_newAlign = alignment;
	}
	
	public void undo () {
		changeGuide (_newGuide, _oldGuide, _oldAlign);
	}
}
