/*
 * Created on Dec 8, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.ruler.commands;

import java.util.Iterator;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.ruler.model.DiagramGuide;
import AcmeStudio.ui.editors.diagram.visualInfo.VisualInformation;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;
import acmeLib.properties.AcmeProperty;

/**
 * @author Bradley Schmerl
 * Created Dec 8, 2003
 */
public class MoveGuideCommand extends Command {

	private DiagramGuide _guide;
	private int _pDelta;

	/**
	 * 
	 */
	public MoveGuideCommand(DiagramGuide guide, int positionDelta) {
		super ("Move guide");
		_guide = guide;
		_pDelta = positionDelta;
	}

	public void execute () {
		_guide.setPosition (_guide.getPosition () + _pDelta);
		Iterator i = _guide.getElements().iterator ();
		while (i.hasNext()) {
			AcmeElement element = (AcmeElement) i.next();
			Point location = SystemView.getLocation(element, false).getCopy ();
			if (!_guide.isHorizontal()) {
				location.x += _pDelta;
			}
			else {
				location.y += _pDelta;
			}
			SystemView.setLocation(element, location);
				AcmeStudioChangeManager cMgr = AcmeStudioChangeManager.globalChangeManager;
			if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
				AcmeProperty visX = element.getProperty ("vis-x");
				AcmeProperty visY = element.getProperty ("vis-y");
				if (visX != null) {
					cMgr.onSetProperty (element, visX, visX.getValue (), this);
					cMgr.onSetProperty (element, visY, visY.getValue (), this);
				}
			}
			else {
				VisualInformation vis = Helper.getVisualInformation(element);
				cMgr.onUpdateVisInformation(element, vis, this);
			}
		}
	}

	public void undo () {
		_guide.setPosition (_guide.getPosition () - _pDelta);
		Iterator i = _guide.getElements ().iterator();
		while (i.hasNext()) {
			AcmeElement element = (AcmeElement) i.next();
			Point location = SystemView.getLocation(element, false).getCopy();
			if (!_guide.isHorizontal()) {
				location.x -= _pDelta;
			}
			else {
				location.y -= _pDelta;
			}
			SystemView.setLocation(element, location);
		}
	}
}
