/*
 * Created on Dec 8, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.ruler.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import AcmeStudio.ui.editors.diagram.parts.ElementEditPart;
import AcmeStudio.ui.editors.diagram.ruler.Helper;
import acmeLib.core.extensions.IElementExtension;
import acmeLib.elements.AcmeElement;

/**
 * @author Bradley Schmerl
 * Created Dec 8, 2003
 */
public class DiagramGuide implements IElementExtension{

	public static final String PROPERTY_CHILDREN = "subparts changed";
	public static final String PROPERTY_POSITION = "position changed";
	
	protected PropertyChangeSupport _listeners = new PropertyChangeSupport (this);
	private Map _map;
	private int _position;
	private boolean _horizontal;
	/**
	 * 
	 */
	public DiagramGuide() {
	}
	
	public DiagramGuide (boolean isHorizontal) {
		setHorizontal (isHorizontal);
	}

	public void attachElement (AcmeElement element, int alignment) {
		if (getMap ().containsKey (element)) {
			int key = ((Integer )getMap ().get (element)).intValue ();
			if (key == alignment) {
				return;
			}
		}
		getMap ().put (element, new Integer (alignment));
		DiagramGuide parent = Helper.getGuide (element, isHorizontal ());
		if (parent != null && parent != this) {
			parent.detachElement (element);
		}
		if (isHorizontal ()) {
			Helper.setElementHorizontalGuide (element, this);
		}
		else {
			Helper.setElementVerticalGuide (element, this);
		}
		_listeners.firePropertyChange(PROPERTY_CHILDREN, null, element);
	}
	
	public void addPropertyChangeListener (PropertyChangeListener l) {
		_listeners.addPropertyChangeListener(l);
	}
	/**
	 * @return
	 */
	public Map getMap() {
		if (_map == null) {
			_map = new Hashtable ();
		}
		return _map;
	}

	public Set getElements () {
		return getMap ().keySet ();
	}
	
	public int getPosition () {
		return _position;
	}
	
	public boolean isHorizontal () {
		return _horizontal;
	}
	
	public void detachElement (AcmeElement element) {
		if (getMap ().containsKey (element)) {
			getMap ().remove (element);
			if (isHorizontal ()) {
				Helper.setElementHorizontalGuide (element, null);
			}
			else {
				Helper.setElementVerticalGuide (element, null);
			}
			_listeners.firePropertyChange (PROPERTY_CHILDREN, null, element);
		}
	}
	
	public void removePropertyChangeListener (PropertyChangeListener l) {
		_listeners.removePropertyChangeListener(l);
	}
	
	/**
	 * @param isHorizontal
	 */
	public void setHorizontal(boolean isHorizontal) {
		_horizontal =isHorizontal;
	}
	
	public void setPosition (int offset) {
		if (_position != offset) {
			int oldValue = _position;
			_position = offset;
			_listeners.firePropertyChange (PROPERTY_POSITION, new Integer (oldValue), new Integer (_position));
		}
	}

	/* (non-Javadoc)
	 * @see acmeLib.core.extensions.IElementExtension#isPersistent()
	 */
	public boolean isPersistent() {
		return true;
	}

	/* (non-Javadoc)
	 * @see acmeLib.core.extensions.IElementExtension#injectIntoElement(acmeLib.elements.AcmeElement)
	 */
	public void injectIntoElement(AcmeElement element) {
	}

	/* (non-Javadoc)
	 * @see acmeLib.core.extensions.IElementExtension#extractFromElement(acmeLib.elements.AcmeElement)
	 */
	public void extractFromElement(AcmeElement element) {
	}

	/* (non-Javadoc)
	 * @see acmeLib.core.extensions.IElementExtension#isInjected(acmeLib.elements.AcmeElement)
	 */
	public boolean isInjected(AcmeElement element) {
		return false;
	}

	/**
	 * @param part
	 * @return
	 */
	public int getAlignment(ElementEditPart part) {
		if (getMap ().get(part) != null) {
			return ((Integer )getMap ().get (part)).intValue ();
		}
		return -2;
	}

}
