/*
 * Created on Dec 8, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.ruler.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.rulers.RulerProvider;

/**
 * @author Bradley Schmerl
 * Created Dec 8, 2003
 */
public class DiagramRuler {

	public static final String PROPERTY_CHILDREN = "children changed";
	public static final String PROPERTY_UNIT = "units changed";
	
	protected PropertyChangeSupport _listeners = new PropertyChangeSupport (this);
	private int _unit;
	private boolean _horizontal;
	private List _guides = new ArrayList ();
	/**
	 * 
	 */
	public DiagramRuler(boolean isHorizontal, int unit) {
		_horizontal = isHorizontal;
		setUnit (unit);
	}
	
	public DiagramRuler (boolean isHorizontal) {
		this (isHorizontal, RulerProvider.UNIT_INCHES);
	}

	public void addGuide (DiagramGuide guide) {
		if (!_guides.contains (guide)) {
			guide.setHorizontal (!isHorizontal ());
			_guides.add (guide);
			_listeners.firePropertyChange(PROPERTY_CHILDREN, null, guide);
		}
	}
	
	/**
	 * @return
	 */
	public boolean isHorizontal() {
		return _horizontal;
	}

	public void addPropertyChangeListener (PropertyChangeListener listener) {
		_listeners.addPropertyChangeListener(listener);
	}
	
	public List getGuides () {
		return _guides;
	}
	
	public int getUnit () {
		return _unit;
	}
	
	public boolean isHidden () {
		return false;
	}

	public void removeGuide (DiagramGuide guide) {
		if (_guides.contains (guide)) {
			_guides.remove (guide);
			_listeners.firePropertyChange(PROPERTY_CHILDREN, null, guide);
		}
	}
	
	public void removePropertyChangeListener (PropertyChangeListener l) {
		_listeners.removePropertyChangeListener(l);
	}
	
	public void setHidden (boolean isHidden) {
	}
	
	public void setUnit (int newUnit) {
		if (_unit != newUnit) {
			int oldUnit = _unit;
			_unit = newUnit;
			_listeners.firePropertyChange(PROPERTY_UNIT, oldUnit, newUnit);
		}
	}
	
}
