/*
 * Created on Feb 23, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.styles;

import java.util.Iterator;
import java.util.LinkedList;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import AcmeStudio.ui.editors.diagram.styles.constants.*;

/**
 * @author schmerl
 */
public class ComponentVisualization extends ElementVisualization{

	protected static long FLAG_DEFAULT_RESTRICTPORTTYPES = LAST_DEFAULT_FLAG << 2;
	protected static long FLAG_DEFAULT_ALLOWEDPORTS = FLAG_DEFAULT_RESTRICTPORTTYPES << 2;
	protected static long FLAG_DEFAULT_FORCEPORTPLACEMENT = FLAG_DEFAULT_ALLOWEDPORTS << 2;
	protected static long FLAG_DEFAULT_PLACEMENTRULES = FLAG_DEFAULT_FORCEPORTPLACEMENT << 2;
	protected static long FLAG_DEFAULT_RESIZETOFITPORTS = FLAG_DEFAULT_PLACEMENTRULES << 2;
	
    protected boolean _restrictPortTypes = false;
    protected LinkedList _allowedPortTypes = new LinkedList();
    protected boolean _forcePortPlacement = true;
    protected LinkedList _portPlacementRules = new LinkedList();
    protected boolean _resizeToFitPorts = false;

    public ComponentVisualization() {
        super(ElementVisualization.DEFAULT_COMPONENT);
        setAllDefaults ();
    }

    public ComponentVisualization(int type) {
        super(type);
    }

    /**
     * @return boolean
     */
    public boolean restrictPortTypes() {
        return _restrictPortTypes;
    }

    public boolean resizeToFitPorts() {
        return _resizeToFitPorts;
    }

    public LinkedList getAllowedPortTypes() {
        return _allowedPortTypes;
    }

    /**
     * @return boolean
     */
    public boolean forcePortPlacement() {
        return _forcePortPlacement;
    }

    public LinkedList getPortPlacementRules() {
        return _portPlacementRules;
    }

    public void setAllowedPortTypes(LinkedList allowedPortTypes) {
    	if (!allowedPortTypes.equals (_allowedPortTypes)) {
	        this._allowedPortTypes = allowedPortTypes;
	        _defaultFlags &= ~FLAG_DEFAULT_ALLOWEDPORTS;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    public void copyAllowedPortTypes(LinkedList allowedPortTypes) {
        if (allowedPortTypes != null) {
            Iterator iter = allowedPortTypes.iterator();
            this._allowedPortTypes = new LinkedList();
            while (iter.hasNext()) {
                this._allowedPortTypes.add(iter.next());
            }
            _defaultFlags &= ~FLAG_DEFAULT_ALLOWEDPORTS;
        }
    }

    public void setPortPlacementRules(LinkedList portPlacementRules) {
    	if (!_portPlacementRules.equals (portPlacementRules)) {
	        this._portPlacementRules = portPlacementRules;
	        _defaultFlags &= ~FLAG_DEFAULT_PLACEMENTRULES;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    public void copyPortPlacementRules(LinkedList portPlacementRules) {
        if (portPlacementRules != null) {
            Iterator iter = portPlacementRules.iterator();
            this._portPlacementRules = new LinkedList();
            while (iter.hasNext()) {
                this._portPlacementRules.add(iter.next());
            }
            
        }
    }

    /**
     * Sets the rotateRoleAroundConnectorCenter.
     * @param rotateRoleAroundConnectorCenter The rotateRoleAroundConnectorCenter to set
     */
    public void setRestrictPortTypes(boolean restrictPortTypes) {
    	if (restrictPortTypes != _restrictPortTypes) {
	        this._restrictPortTypes = restrictPortTypes;
	        _defaultFlags &= ~FLAG_DEFAULT_RESTRICTPORTTYPES;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    public void setResizeToFitPorts(boolean resizeToFitPorts) {
    	if (resizeToFitPorts != _resizeToFitPorts) {
	        this._resizeToFitPorts = resizeToFitPorts;
	        _defaultFlags &= ~FLAG_DEFAULT_RESIZETOFITPORTS;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    public void setForcePortPlacement(boolean forcePortPlacement) {
    	if (forcePortPlacement != _forcePortPlacement) {
	        this._forcePortPlacement = forcePortPlacement;
	        _defaultFlags &= ~FLAG_DEFAULT_FORCEPORTPLACEMENT;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    /* (non-Javadoc)
     * @see AcmeStudio.ui.editors.diagram.styles.Visualization#copyInto(AcmeStudio.ui.editors.diagram.styles.Visualization)
     */
    public void copyInto(ElementVisualization from) {
        _copying = true;
        if (from instanceof ComponentVisualization) {
            setRestrictPortTypes(
                ((ComponentVisualization) from).restrictPortTypes());
            setForcePortPlacement(
                ((ComponentVisualization) from).forcePortPlacement());
            copyAllowedPortTypes(
                ((ComponentVisualization) from).getAllowedPortTypes());
            copyPortPlacementRules(
                ((ComponentVisualization) from).getPortPlacementRules());
            setResizeToFitPorts(
                ((ComponentVisualization) from).resizeToFitPorts());
        }
        super.copyInto(from);
    }
    
    public ElementVisualization copy () {
    	ComponentVisualization vis = new ComponentVisualization ();
    	vis.copyInto (this);
    	return vis;
    }

    public void importTypeSpecificVisualization(Element visElement) {
        Element restrictNode =
            (Element) visElement.getElementsByTagName("restrictPortType").item(
                0);
        setRestrictPortTypes(importOn(restrictNode));
        setAllowedPortTypes(importAllowedTypes(restrictNode));

        Element ruleNode =
            (Element) visElement.getElementsByTagName(
                "forcePortPlacement").item(
                0);
        setForcePortPlacement(importOn(ruleNode));
        setPortPlacementRules(importPortPlacementRules(ruleNode));

        Element resizeNode =
            (Element) visElement.getElementsByTagName("resizeToFitPorts").item(
                0);
        setResizeToFitPorts(importOn(resizeNode));
    }
    
    

    public void exportTypeSpecificVisualization(Document doc, Element root) {
        Element restrictNode = doc.createElement("restrictPortType");
        restrictNode.setAttribute("on", restrictPortTypes() ? "true" : "false");
        exportAllowedTypes(restrictNode, getAllowedPortTypes());

        Element ruleNode = doc.createElement("forcePortPlacement");
        ruleNode.setAttribute("on", forcePortPlacement() ? "true" : "false");
        exportPortPlacementRules(doc, ruleNode, getPortPlacementRules());

        Element resizeNode = doc.createElement("resizeToFitPorts");
        resizeNode.setAttribute("on", resizeToFitPorts() ? "true" : "false");

        root.appendChild(restrictNode);
        root.appendChild(ruleNode);
        root.appendChild(resizeNode);
        
        root.setAttribute ("type", "COMPONENT");
    }

    static LinkedList importPortPlacementRules(Element el) {
        LinkedList result = null;
        if (el != null) {
            NodeList elements = el.getElementsByTagName("portPlacementRule");
            result = new LinkedList();
            for (int i = 0; i < elements.getLength(); i++) {
                Element e = (Element) elements.item(i);
                PlacementRule rule = new PlacementRule();
                rule._type = e.getAttribute("portType");
                rule._side =
                    ((Integer) SideOptions
                        .SIDES
                        .get(e.getAttribute("side")))
                        .intValue();
                rule._alignment =
                    ((Integer) AlignmentOptions
                        .ALIGNMENTS
                        .get(e.getAttribute("alignment")))
                        .intValue();
                result.add(rule);
            }
        }
        return result;
    }

    static void exportPortPlacementRules(
        Document doc,
        Element el,
        LinkedList rules) {
        if (rules != null) {
            Iterator iter = rules.iterator();
            while (iter.hasNext()) {
                PlacementRule rule = (PlacementRule) iter.next();
                Element e = doc.createElement("portPlacementRule");
                e.setAttribute("portType", rule._type);
                e.setAttribute(
                    "side",
                    (String) SideOptions.SIDENAMES.get(rule._side));
                e.setAttribute(
                    "alignment",
                    (String) AlignmentOptions.ALIGNMENTNAMES.get(
                        rule._alignment));
                el.appendChild(e);
            }
        }
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#unify(AcmeStudio.ui.editors.diagram.styles.ElementVisualization)
	 */
	public ElementVisualization unify(ElementVisualization from) {
		ComponentVisualization to = new ComponentVisualization ();
		unify (from, to);
		return to;
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#setAllDefaults()
	 */
	protected void setAllDefaults() {
		super.setAllDefaults ();
		_defaultFlags |= FLAG_DEFAULT_ALLOWEDPORTS | FLAG_DEFAULT_FORCEPORTPLACEMENT | FLAG_DEFAULT_PLACEMENTRULES | FLAG_DEFAULT_RESIZETOFITPORTS | FLAG_DEFAULT_RESTRICTPORTTYPES;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#blendVisualization(AcmeStudio.ui.editors.diagram.styles.ElementVisualization, boolean)
	 */
	public void blendVisualization(ElementVisualization vis, boolean treatVisAsSuper) {
		super.blendVisualization (vis, treatVisAsSuper);
		_copying = true;
		if (vis instanceof ComponentVisualization) {
			ComponentVisualization cVis = (ComponentVisualization )vis;
			
			if (shouldBlendPart(cVis, FLAG_DEFAULT_ALLOWEDPORTS)) {
				setAllowedPortTypes(new LinkedList (cVis.getAllowedPortTypes()));
				forceDefault(FLAG_DEFAULT_ALLOWEDPORTS);
			}
			if (shouldBlendPart (cVis, FLAG_DEFAULT_FORCEPORTPLACEMENT)) {
				setForcePortPlacement(cVis.forcePortPlacement());
				forceDefault (FLAG_DEFAULT_FORCEPORTPLACEMENT);
			}
			if (shouldBlendPart (cVis, FLAG_DEFAULT_PLACEMENTRULES)) {
				setPortPlacementRules(new LinkedList (cVis.getPortPlacementRules()));
				forceDefault (FLAG_DEFAULT_PLACEMENTRULES);
			}
			if (shouldBlendPart (cVis, FLAG_DEFAULT_RESIZETOFITPORTS)) {
				setResizeToFitPorts(cVis.resizeToFitPorts());
				forceDefault (FLAG_DEFAULT_RESIZETOFITPORTS);
			}
			if (shouldBlendPart (cVis, FLAG_DEFAULT_RESTRICTPORTTYPES)) {
				setRestrictPortTypes(cVis.restrictPortTypes());
				forceDefault (FLAG_DEFAULT_RESTRICTPORTTYPES);
			}
			
		}
		_copying = false;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#importDefaults(AcmeStudio.ui.editors.diagram.styles.ElementVisualization, org.w3c.dom.Element)
	 */
	protected void importDefaults(ElementVisualization vis, Element visElement) {
		super.importDefaults (vis, visElement);
    	NodeList dfs = visElement.getElementsByTagName("defaults");
    	if (dfs.getLength() > 0) {
    		Element defaults = (Element )dfs.item (0);
    		setImportedDefault(vis, defaults, "allowedPorts", FLAG_DEFAULT_ALLOWEDPORTS);
    		setImportedDefault(vis, defaults, "forcePortPlacement", FLAG_DEFAULT_FORCEPORTPLACEMENT);
    		setImportedDefault(vis, defaults, "placementRules", FLAG_DEFAULT_PLACEMENTRULES);
    		setImportedDefault(vis, defaults, "resizeToFitPorts", FLAG_DEFAULT_RESIZETOFITPORTS);
    		setImportedDefault(vis, defaults, "restrictPortTypes", FLAG_DEFAULT_RESTRICTPORTTYPES);
    	}
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#exportDefaults(org.w3c.dom.Document)
	 */
	public Element exportDefaults(Document doc) {
		Element defaults = super.exportDefaults (doc);
  		exportDefaultAttribute(defaults, "allowedPorts", FLAG_DEFAULT_ALLOWEDPORTS);
  		exportDefaultAttribute(defaults, "forcePortPlacement", FLAG_DEFAULT_FORCEPORTPLACEMENT);
  		exportDefaultAttribute(defaults, "placementRules", FLAG_DEFAULT_PLACEMENTRULES);
  		exportDefaultAttribute(defaults, "resizeToFitPorts", FLAG_DEFAULT_RESIZETOFITPORTS);
  		exportDefaultAttribute(defaults, "restrictPortTypes", FLAG_DEFAULT_RESTRICTPORTTYPES);
		return defaults;
	}
}
