/*
 * Created on Feb 23, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.styles;

import java.util.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author yh
 */
public class ConnectorVisualization extends ElementVisualization {

    /**
     * Enumeration of route options
     */
    public static class RoleRouteOptions {
        public static final int STRAIGHT = 0, HBUS = 1, VBUS = 2, ELBOW = 3;
        public static final String[] LABELS =
            new String[] { "Straight", "Horizontal Bus", "Vertical Bus", "Elbow" };

        public static final List ROUTENAMES = new ArrayList();
        public static final Hashtable ROUTES = new Hashtable();
        static {
            ROUTENAMES.add(STRAIGHT, "Straight");
            ROUTENAMES.add(HBUS, "Horizontal Bus");
            ROUTENAMES.add(VBUS, "Vertical Bus");
            ROUTENAMES.add(ELBOW, "Elbow");

            ROUTES.put("straight", new Integer(STRAIGHT));
            ROUTES.put("horizontal bus", new Integer(HBUS));
            ROUTES.put("vertical bus", new Integer(VBUS));
            ROUTES.put("elbow", new Integer(ELBOW));
        }
    }

    public static class RoleRouteRule {
        public String _roleType = null;
        public int _roleRoute = RoleRouteOptions.STRAIGHT;
        public boolean equals(Object obj) {
            if (obj instanceof RoleRouteRule) {
                if (((RoleRouteRule) obj)._roleType.equals(this._roleType))
                    return true;
            }
            return false;
        }
    }

    protected static final long FLAG_DEFAULT_RESTRICT_ROLETYPES = LAST_DEFAULT_FLAG << 2;
    protected static final long FLAG_DEFAULT_ALLOWEDROLETYPES = FLAG_DEFAULT_RESTRICT_ROLETYPES << 2;
    protected static final long FLAG_DEFAULT_ROLEROUTES = FLAG_DEFAULT_ALLOWEDROLETYPES << 2;
    protected static final long FLAG_DEFAULT_ROLECOUNT = FLAG_DEFAULT_ROLEROUTES << 2;
    protected static final long FLAG_DEFAULT_AUTOCENTER_BINARY_CONNECTORS = FLAG_DEFAULT_ROLECOUNT << 2;
    
    protected boolean _restrictRoleTypes = false;
    protected LinkedList _allowedRoleTypes = new LinkedList();
    protected LinkedList _roleRouteRules = new LinkedList();
    protected int _defaultRoleCount = 2;
	private boolean	_autoCenter;
    
    public ConnectorVisualization() {
        super(ElementVisualization.DEFAULT_CONNECTOR);
    }

    public ConnectorVisualization(int type) {
        super(type);
    }

    public LinkedList getRoleRouteRules() {
        return _roleRouteRules;
    }

    public int getDefaultRoleCount() {
        return _defaultRoleCount;
    }
    
    public void setDefaultRoleCount(int count) {
    	if (count != _defaultRoleCount) {
	        this._defaultRoleCount = count;
	        _defaultFlags &= ~FLAG_DEFAULT_ROLECOUNT;
    	}
    }

    public void setAllowedRoleTypes(LinkedList allowedRoleTypes) {
    	if (!allowedRoleTypes.equals (_allowedRoleTypes)) {
	        this._allowedRoleTypes = allowedRoleTypes;
	        _defaultFlags &= ~FLAG_DEFAULT_ALLOWEDROLETYPES;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    /**
     * Sets the rotateRoleAroundConnectorCenter.
     * @param rotateRoleAroundConnectorCenter The rotateRoleAroundConnectorCenter to set
     */
    public void setRestrictRoleTypes(boolean restrictRoleTypes) {
    	if (restrictRoleTypes != _restrictRoleTypes) {
	        this._restrictRoleTypes = restrictRoleTypes;
	        _defaultFlags &= ~FLAG_DEFAULT_RESTRICT_ROLETYPES;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }
    

    public void copyAllowedRoleTypes(LinkedList allowedPortTypes) {
        if (allowedPortTypes != null) {
            Iterator iter = allowedPortTypes.iterator();
            this._allowedRoleTypes = new LinkedList();
            while (iter.hasNext()) {
                this._allowedRoleTypes.add(iter.next());
            }
        }
    }
    
    public void setRoleRouteRules(LinkedList roleRouteRules) {
    	if (!_roleRouteRules.equals (roleRouteRules)) {
	        this._roleRouteRules = roleRouteRules;
	        _defaultFlags &= ~FLAG_DEFAULT_ROLEROUTES;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    public void copyRoleRouteRules(LinkedList rolePlacementRules) {
        if (rolePlacementRules != null) {
            Iterator iter = rolePlacementRules.iterator();
            this._roleRouteRules = new LinkedList();
            while (iter.hasNext()) {
                this._roleRouteRules.add(iter.next());
            }
        }
    }

    /* (non-Javadoc)
     * @see AcmeStudio.ui.editors.diagram.styles.Visualization#copyInto(AcmeStudio.ui.editors.diagram.styles.Visualization)
     */
    public void copyInto(ElementVisualization from) {
        _copying = true;
        if (from instanceof ConnectorVisualization) {
            copyRoleRouteRules(
                ((ConnectorVisualization) from).getRoleRouteRules());
            setRestrictRoleTypes(
                ((ConnectorVisualization) from).restrictRoleTypes());
            copyAllowedRoleTypes(
                ((ConnectorVisualization) from).getAllowedRoleTypes());
            _defaultRoleCount = 
                ((ConnectorVisualization) from).getDefaultRoleCount();
            _autoCenter = ((ConnectorVisualization )from).autoCenterBinaryConnectors();

        }
        super.copyInto(from);
    }
    
    public ElementVisualization copy () {
    	ConnectorVisualization vis = new ConnectorVisualization ();
    	vis.copyInto (this);
    	return vis;
    }

    public void importTypeSpecificVisualization(Element visElement) {
        Element restrictNode =
            (Element) visElement.getElementsByTagName("restrictPortType").item(
                0);
        setRestrictRoleTypes(importOn(restrictNode));
        setAllowedRoleTypes(importAllowedTypes(restrictNode));

        Element ruleNode =
            (Element) visElement.getElementsByTagName("forceRoleRoute").item(0);
        setRoleRouteRules(importRoleRouteRules(ruleNode));

        Element numberNode =
            (Element) visElement.getElementsByTagName("defaultRoleCount").item(0);
        int count = importDefaultRoleCount(numberNode);
        if (count >= 0)
            setDefaultRoleCount(count);
        NodeList autoCenterNode = visElement.getElementsByTagName ("autoCenterBinaryConnector");
        if (autoCenterNode != null) {
        	setAutoCenterBinaryConnector (importOn ((Element )autoCenterNode.item (0)));
        }
    }

    /**
	 * @param b
	 */
	private void setAutoCenterBinaryConnector(boolean autoCenter) {
    	if (autoCenter != _restrictRoleTypes) {
	        this._restrictRoleTypes = autoCenter;
	        _defaultFlags &= ~FLAG_DEFAULT_AUTOCENTER_BINARY_CONNECTORS;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
	}

	static int importDefaultRoleCount(Element el) {
        String value = null;
        int count = -1;
        if (el != null && (value = el.getAttribute("value")) != null) {
            try {
                count = Integer.parseInt(value); 
            } catch (Exception e) {
            }
        }
        return count;
    }

    public void exportTypeSpecificVisualization(Document doc, Element root) {
        Element restrictNode = doc.createElement("restrictPortType");
        restrictNode.setAttribute("on", restrictRoleTypes() ? "true" : "false");
        exportAllowedTypes(restrictNode, getAllowedRoleTypes());

        Element ruleNode = doc.createElement("forceRoleRoute");
        exportRoleRouteRules(doc, ruleNode, getRoleRouteRules());
        
        Element roleCountNode = doc.createElement("defaultRoleCount");
        roleCountNode.setAttribute("value", Integer.toString(_defaultRoleCount));

        Element autoCenterNode = doc.createElement ("autoCenterBinaryConnector");
        autoCenterNode.setAttribute ("on", autoCenterBinaryConnectors()?"true":"false");
        
        root.appendChild(restrictNode);
        root.appendChild(ruleNode);
        root.appendChild(roleCountNode);
        root.appendChild(autoCenterNode);
        
        root.setAttribute ("type", "CONNECTOR");
    }

    public LinkedList getAllowedRoleTypes() {
        return _allowedRoleTypes;
    }

    /**
     * @return boolean
     */
    public boolean restrictRoleTypes() {
        return _restrictRoleTypes;
    }

    static LinkedList importRoleRouteRules(Element el) {
        LinkedList result = null;
        if (el != null) {
            NodeList elements = el.getElementsByTagName("roleRouteRule");
            result = new LinkedList();
            for (int i = 0; i < elements.getLength(); i++) {
                Element e = (Element) elements.item(i);
                RoleRouteRule rule = new RoleRouteRule();
                rule._roleType = e.getAttribute("roleType");
                Integer route =
                    (Integer) RoleRouteOptions.ROUTES.get(
                        e.getAttribute("route").toLowerCase ());
                if (route != null) {
                    rule._roleRoute = route.intValue();
                    result.add(rule);
                }
            }
        }
        return result;
    }

    static void exportRoleRouteRules(
        Document doc,
        Element el,
        LinkedList rules) {
        if (rules != null) {
            Iterator iter = rules.iterator();
            while (iter.hasNext()) {
                RoleRouteRule rule = (RoleRouteRule) iter.next();
                Element e = doc.createElement("roleRouteRule");
                e.setAttribute("roleType", rule._roleType);
                e.setAttribute(
                    "route",
                    (String) RoleRouteOptions.ROUTENAMES.get(rule._roleRoute));
                el.appendChild(e);
            }
        }
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#unify(AcmeStudio.ui.editors.diagram.styles.ElementVisualization)
	 */
	public ElementVisualization unify(ElementVisualization from) {
		ConnectorVisualization to = new ConnectorVisualization ();
		unify (from, to);
		return to;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#blendVisualization(AcmeStudio.ui.editors.diagram.styles.ElementVisualization, boolean)
	 */
	public void blendVisualization(ElementVisualization vis, boolean treatVisAsSuper) {
		super.blendVisualization (vis, treatVisAsSuper);
		if (vis instanceof ConnectorVisualization) {
			_copying = true;
			ConnectorVisualization cVis = (ConnectorVisualization )vis;
			if (shouldBlendPart(cVis, FLAG_DEFAULT_ALLOWEDROLETYPES)) {
				setAllowedRoleTypes (new LinkedList (cVis.getAllowedRoleTypes()));
				forceDefault (FLAG_DEFAULT_ALLOWEDROLETYPES);
			}
			if (shouldBlendPart(cVis, FLAG_DEFAULT_RESTRICT_ROLETYPES)) {
				setRestrictRoleTypes(cVis.restrictRoleTypes());
				forceDefault (FLAG_DEFAULT_RESTRICT_ROLETYPES);
			}
			if (shouldBlendPart (cVis, FLAG_DEFAULT_ROLECOUNT)) {
				setDefaultRoleCount(cVis.getDefaultRoleCount());
				forceDefault (FLAG_DEFAULT_ROLECOUNT);
			}
			if (shouldBlendPart (cVis, FLAG_DEFAULT_ROLEROUTES)) {
				setRoleRouteRules(new LinkedList (cVis.getRoleRouteRules()));
				forceDefault (FLAG_DEFAULT_ROLEROUTES);
			}
			_copying = false;
		}
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#setAllDefaults()
	 */
	protected void setAllDefaults() {
		super.setAllDefaults ();
		_defaultFlags |= FLAG_DEFAULT_ALLOWEDROLETYPES | FLAG_DEFAULT_RESTRICT_ROLETYPES | FLAG_DEFAULT_ROLECOUNT | FLAG_DEFAULT_ROLEROUTES;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#exportDefaults(org.w3c.dom.Document)
	 */
	public Element exportDefaults(Document doc) {
		Element defaults =  super.exportDefaults (doc);
		exportDefaultAttribute (defaults, "allowedRoles", FLAG_DEFAULT_ALLOWEDROLETYPES);
		exportDefaultAttribute(defaults, "restrictRoleTypes", FLAG_DEFAULT_RESTRICT_ROLETYPES);
		exportDefaultAttribute(defaults, "roleCount", FLAG_DEFAULT_ROLECOUNT);
		exportDefaultAttribute(defaults, "roleRoutes", FLAG_DEFAULT_ROLEROUTES);
		return defaults;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#importDefaults(AcmeStudio.ui.editors.diagram.styles.ElementVisualization, org.w3c.dom.Element)
	 */
	protected void importDefaults(ElementVisualization vis, Element visElement) {
		super.importDefaults (vis, visElement);
		NodeList dfs = visElement.getElementsByTagName("defaults");
    	if (dfs.getLength() > 0) {
    		Element defaults = (Element )dfs.item (0);
    		setImportedDefault(vis, defaults, "allowedRoles", FLAG_DEFAULT_ALLOWEDROLETYPES);
    		setImportedDefault(vis, defaults, "restrictRoleTypes", FLAG_DEFAULT_RESTRICT_ROLETYPES);
    		setImportedDefault(vis, defaults, "roleCount", FLAG_DEFAULT_ROLECOUNT);
    		setImportedDefault(vis, defaults, "roleRoutes", FLAG_DEFAULT_ROLEROUTES);
    	}
	}

	/**
	 * @return
	 */
	public boolean autoCenterBinaryConnectors() {
		return _autoCenter;
	}

	/**
	 * @param autoCenter
	 */
	public void setAutoCenterBinaryConnectors(boolean autoCenter) {
		_autoCenter = autoCenter;
	}
}
