package AcmeStudio.ui.editors.diagram.styles;

import java.util.Collections;
import java.util.Iterator;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.GroupBoxBorder;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ColumnLayoutData;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import com.swtdesigner.SWTResourceManager;

import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import AcmeStudio.wizards.ConnectionPatternWizard;
import AcmeStudio.wizards.connection.PortConnectionRule;
import AcmeStudio.wizards.connection.PortRoleAssociation;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeFamily;

/**
 * @author Hong Implements the dialog for editing or creating a new visual style
 *             for depicting the system in a diagram.
 *  
 */
public class EditSysVisDialog extends Dialog
{
    /**
     * The location of the dialog box
     */
    protected Composite _parent;
    /**
     * The style being edited. Note, this is a copy of an original style, which
     * should faciliate undo
     */
    protected SystemVisualization _sysVis;
    /**
     * The type information for the style (this is the type that the style will
     * be associated with)
     */
    protected AcmeFamily _family;

    /**
     * A combo box that holds all the types that a new style can be created for
     */
    protected Combo _typeCombo;
    protected Combo _rowsCombo;
    protected Combo _colsCombo;
    protected Combo _layerStyleCombo;
    protected Combo _layersCombo;
    protected FigureCanvas _layoutPreview;
    protected List _typeList;
    protected ElementAlignmentList _alignElementTypeTable;
    protected TabFolder _layoutFolder;
    protected Button _allowRuleDefine;

    /**
     * Indicates whether we are creating a new visual style or editing an
     * existing style.
     */
    protected boolean _newVisualization;
    /**
     * The folder containing all the pages for the different options.
     */
    protected TabFolder _folder;

    /**
     * The composite containing all the information to do with setting type
     * specific styles.
     */
    protected Composite _layoutPage;

    /**
     * The choice of options for layout types.
     */
    private String[] _basicLayouts = SystemVisualization.Layout.LABELS;

    /**
     * The combo that contains all the valid stock shapes (entries from
     * _stockShapes)
     */
    protected Combo _layoutCombo;
    private Composite _connectionPage;
    private TableViewer _ruleViewer;

    /**
     * Constructor for EditVisualizationDialog. This takes a copy of the visual
     * style so that it can be changed directly and then not copied over if the
     * action is cancelled
     * 
     * @param parentShell
     *                 The shell where the dialog will be raised
     * @param vs
     *                 The visual style to edit or create
     */
    public EditSysVisDialog(Shell parentShell, Visualization vs)
    {
        super(parentShell);
        _sysVis = ((SystemVisualization) vs).copy();
    }

    /**
     * Constructor for EditVisualizationDialog, which creates a new
     * visualization Finds the best fitting style from the registry, and then
     * copies the style
     * 
     * @param parentShell
     *                 The Shell where the dialog will be raised param type The type
     *                 that will be associated with the new style
     */
    public EditSysVisDialog(Shell parentShell, AcmeFamily type,
            boolean isCreating)
    {
        super(parentShell);
        _family = type;
        _sysVis = VisualStyleRegistry.getSysVis(type).copy();
        _newVisualization = isCreating;
    }

    /**
     * Creates the dialog area, which has a preview, the type the visualization
     * is associated with, and a tabbed folder containing pages that allow
     * various options of the style to be edited.
     * 
     * @param parent
     *                 The place where all the controls go
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        _parent = (Composite) super.createDialogArea(parent);
        GridLayout gl = (GridLayout) _parent.getLayout();
        gl.verticalSpacing = 10;
        // The number of tabs in the folder
        int tabCount = 0;
        // Create the shape page and add it to the tab
        _folder = new TabFolder(_parent, SWT.NONE);
        createConnectionRulePage(tabCount++);
        createLayoutPage(tabCount);
        // Set all the controls to contain information based
        // on the visual style
        //importStyle();
        // Make sure the preview has an image to start with

        _folder.pack();
        //        _parent.pack();
        return _parent;
    }

    private Composite createGridLayoutPropertyPage(Composite parent)
    {
        Composite page = new Composite(parent, SWT.NONE);
        GridData gd = new GridData(GridData.FILL_BOTH);
        page.setLayoutData(gd);
        page.setLayout(new GridLayout(2, false));
        Label l = new Label(page, SWT.BORDER);
        l.setText("This is work in progress, and does not work well");
        GridData lgd = new GridData();
        lgd.horizontalSpan = GridData.FILL_HORIZONTAL;
        lgd.horizontalAlignment = GridData.HORIZONTAL_ALIGN_CENTER;
        l.setLayoutData(lgd);
        Label label = new Label(page, SWT.NONE);
        label.setText("Number of Rows:");
        _rowsCombo = new Combo(page, SWT.DROP_DOWN);
        // Valid region number range is [1, MAX_ITEM_NUMBER]
        final int MAX_ITEM_NUMBER = 10;
        String[] regionNumbers = new String[MAX_ITEM_NUMBER];
        for (int i = 1; i <= MAX_ITEM_NUMBER; i++)
        {
            regionNumbers[i - 1] = Integer.toString(i);
        }
        _rowsCombo.setItems(regionNumbers);
        _rowsCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        int rows = _sysVis.getRows();
        _rowsCombo.select(rows - 1);
        _rowsCombo.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                _sysVis.setRows(_rowsCombo.getSelectionIndex() + 1);
                _alignElementTypeTable.setRegions(_sysVis.getRows()
                        * _sysVis.getCols());
                updateLayoutPreview();
            }
        });
        _rowsCombo.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {
                int rows = -1;
                try
                {
                    rows = Integer.parseInt(_rowsCombo.getText());
                }
                catch (Exception ex)
                {
                    _rowsCombo.setText("");
                    return;
                }
                if (rows > 0 && rows <= MAX_ITEM_NUMBER)
                {
                    _sysVis.setRows(rows);
                    _alignElementTypeTable.setRegions(_sysVis.getRows()
                            * _sysVis.getCols());
                    updateLayoutPreview();
                }
            }

            public void keyReleased(KeyEvent e)
            {
            }
        });
        label = new Label(page, SWT.NONE);
        label.setText("Number of Columns:");
        _colsCombo = new Combo(page, SWT.DROP_DOWN);
        _colsCombo.setItems(regionNumbers);
        _colsCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        int cols = _sysVis.getCols();
        _colsCombo.select(cols - 1);
        _colsCombo.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                _sysVis.setCols(_colsCombo.getSelectionIndex() + 1);
                _alignElementTypeTable.setRegions(_sysVis.getRows()
                        * _sysVis.getCols());
                updateLayoutPreview();
            }
        });
        _colsCombo.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {
                int cols = -1;
                try
                {
                    cols = Integer.parseInt(_colsCombo.getText());
                }
                catch (Exception ex)
                {
                    _colsCombo.setText("");
                    return;
                }
                if (cols > 0 && cols <= MAX_ITEM_NUMBER)
                {
                    _sysVis.setCols(cols);
                    _alignElementTypeTable.setRegions(_sysVis.getRows()
                            * _sysVis.getCols());
                    updateLayoutPreview();
                }
            }

            public void keyReleased(KeyEvent e)
            {
            }
        });
        return page;
    }

    private Composite createLayeredLayoutPropertyPage(Composite parent)
    {
        Composite page = new Composite(parent, SWT.NONE);
        GridData gd = new GridData(GridData.FILL_BOTH);
        page.setLayoutData(gd);
        page.setLayout(new GridLayout(2, false));
        Label label = new Label(page, SWT.NONE);
        label.setText("Style:");
        _layerStyleCombo = new Combo(page, SWT.DROP_DOWN);
        String[] layerStyles = new String[2];
        layerStyles[0] = "Vertical";
        layerStyles[1] = "Horizontal";
        _layerStyleCombo.setItems(layerStyles);
        _layerStyleCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        if (_sysVis.getRows() == 0)
            _layerStyleCombo.select(1);
        if (_sysVis.getCols() == 0)
            _layerStyleCombo.select(0);
        _layerStyleCombo.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                if (_layerStyleCombo.getSelectionIndex() == 0)
                {
                    _sysVis.setRows(_sysVis.getCols());
                    _sysVis.setCols(0);
                    _alignElementTypeTable.setRegions(_sysVis.getRows());
                }
                else
                {
                    _sysVis.setCols(_sysVis.getRows());
                    _sysVis.setRows(0);
                    _alignElementTypeTable.setRegions(_sysVis.getCols());
                }
                updateLayoutPreview();
            }
        });
        _layerStyleCombo.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {
                _layerStyleCombo.setText("");
            }

            public void keyReleased(KeyEvent e)
            {
            }
        });
        label = new Label(page, SWT.NONE);
        label.setText("Number of Layers:");
        _layersCombo = new Combo(page, SWT.DROP_DOWN);
        // Valid layer number range is [1, MAX_LAYER_NUMBER]
        final int MAX_LAYER_NUMBER = 10;
        String[] layerNumbers = new String[MAX_LAYER_NUMBER];
        for (int i = 1; i <= MAX_LAYER_NUMBER; i++)
        {
            layerNumbers[i - 1] = Integer.toString(i);
        }
        _layersCombo.setItems(layerNumbers);
        _layersCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        int cols = _sysVis.getCols();
        int rows = _sysVis.getRows();
        if (rows == 0)
            _layersCombo.select(cols - 1);
        if (cols == 0)
            _layersCombo.select(rows - 1);
        _layersCombo.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                if (_sysVis.getRows() == 0)
                {
                    _sysVis.setCols(_layersCombo.getSelectionIndex() + 1);
                    _alignElementTypeTable.setRegions(_sysVis.getCols());
                }
                else
                {
                    _sysVis.setRows(_layersCombo.getSelectionIndex() + 1);
                    _alignElementTypeTable.setRegions(_sysVis.getRows());
                }
                updateLayoutPreview();
            }
        });
        _layersCombo.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {
                if (_sysVis.getRows() == 0)
                {
                    int cols = -1;
                    try
                    {
                        cols = Integer.parseInt(_layersCombo.getText());
                    }
                    catch (Exception ex)
                    {
                        _layersCombo.setText("");
                        return;
                    }
                    if (cols > 0 && cols <= MAX_LAYER_NUMBER)
                    {
                        _sysVis.setCols(cols);
                        _alignElementTypeTable.setRegions(cols);
                        updateLayoutPreview();
                    }
                }
                else
                {
                    int rows = -1;
                    try
                    {
                        rows = Integer.parseInt(_layersCombo.getText());
                    }
                    catch (Exception ex)
                    {
                        _layersCombo.setText("");
                        return;
                    }
                    if (rows > 0 && rows <= MAX_LAYER_NUMBER)
                    {
                        _sysVis.setRows(rows);
                        _alignElementTypeTable.setRegions(rows);
                        updateLayoutPreview();
                    }
                }
            }

            public void keyReleased(KeyEvent e)
            {
            }
        });
        return page;
    }

    private void createLayoutProperties(TabFolder parent, int layout)
    {
        TabItem tabItem = null;
        if (parent.getItemCount() == 0)
        {
            tabItem = new TabItem(parent, SWT.NONE, 0);
        }
        else
        {
            tabItem = parent.getItem(0);
        }
        tabItem.setText("Layout Properties");
        Composite page = null;
        switch (layout)
        {
            case SystemVisualization.Layout.LAYERED:
                page = createLayeredLayoutPropertyPage(parent);
                break;
            default:
                page = createGridLayoutPropertyPage(parent);
        }
        tabItem.setControl(page);
    }

    /**
     * Creates the group to allow choosing of shapes. Includes components to: a)
     * choose a stock shape b) choose an image to display c) indicate whether
     * the shape should be replicated
     * 
     * @param parent
     *                 the element to put these components into
     */
    private void createLayoutSelectionGroup(Composite parent)
    {
        Group layoutSelectionGroup = new Group(parent, SWT.NONE);
        GridLayout gl = new GridLayout(2, false);
        layoutSelectionGroup.setLayout(gl);
        layoutSelectionGroup.setText("System Layout");
        Composite left = new Composite(layoutSelectionGroup, SWT.NONE);
        left.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        Composite right = new Composite(layoutSelectionGroup, SWT.NONE);
        left.setLayout(new GridLayout(2, false));
        Label label = new Label(left, SWT.NONE);
        label.setText("Layout Name:");
        _layoutCombo = new Combo(left, SWT.DROP_DOWN);
        layoutSelectionGroup.setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        _layoutCombo.setItems(_basicLayouts);
        _layoutCombo.select(_sysVis.getLayout());
        _layoutCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        _layoutFolder = new TabFolder(left, SWT.NONE);
        _layoutCombo.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                int index = _layoutCombo.getSelectionIndex();
                _sysVis.setLayout(index);
                switch (_layoutCombo.getSelectionIndex())
                {
                }
                createLayoutProperties(_layoutFolder, index);
            }
        });
        GridData gd = new GridData(GridData.FILL_BOTH);
        gd.horizontalSpan = 2;
        _layoutFolder.setLayoutData(gd);
        createLayoutProperties(_layoutFolder, _sysVis.getLayout());
        gd = new GridData(GridData.HORIZONTAL_ALIGN_END);
        right.setLayoutData(gd);
        right.setLayout(new GridLayout(1, false));
        Group layoutPreviewGroup = new Group(right, SWT.NONE);
        gd = new GridData(GridData.FILL_BOTH);
        layoutPreviewGroup.setLayoutData(gd);
        layoutPreviewGroup.setLayout(new GridLayout(1, false));
        _layoutPreview = new FigureCanvas(layoutPreviewGroup);
        gd = new GridData(GridData.FILL_BOTH);
        _layoutPreview.setLayoutData(gd);
        updateLayoutPreview();
    }

    private void updateLayoutPreview()
    {
        int MAX_WIDTH = 150;
        int MAX_HEIGHT = 120;
        int MIN_WIDTH = 20;
        int MIN_HEIGHT = 20;
        int rows = _sysVis.getRows();
        int cols = _sysVis.getCols();
        rows = rows == 0 ? 1 : rows;
        cols = cols == 0 ? 1 : cols;
        Figure panel = new Panel();
        //panel.setBackgroundColor(ColorConstants.white);
        panel.setSize(new Dimension(MAX_WIDTH, MAX_HEIGHT));
        Rectangle r = panel.getClientArea();
        int w = r.width / cols;
        int h = r.height / rows;
        for (int i = 1; i <= rows; i++)
        {
            for (int j = 1; j <= cols; j++)
            {
                org.eclipse.draw2d.Label label = new org.eclipse.draw2d.Label(
                        "");
                String id = Integer.toString((i - 1) * cols + j);
                if (w < MIN_WIDTH || h < MIN_HEIGHT)
                {
                    label.setToolTip(new org.eclipse.draw2d.Label(id));
                }
                else
                {
                    label.setText(id);
                }
                GroupBoxBorder border = new GroupBoxBorder();
                border.setLabel("");
                border.setFont(SWTResourceManager.getFont("", 1, 1));
                label.setBorder(border);
                label.setBounds(new Rectangle((j - 1) * w, (i - 1) * h, w, h));
                panel.add(label);
            }
        }
        _layoutPreview.setContents(panel);
        //panel.repaint();
    }

    /**
     * Creates the port page that contains controls to edit the port style.
     * 
     * @param parent
     *                 Where the page will go
     */
    private Composite createSystemLayoutPage(Composite parent)
    {
        Composite thePage = new Composite(parent, SWT.NONE);
        GridLayout pageLayout = new GridLayout(2, false);
        thePage.setLayout(pageLayout);
        // The left column
        Composite leftColumn = new Composite(thePage, SWT.NONE);
        GridData gd = new GridData();
        gd.grabExcessHorizontalSpace = true;
        gd.horizontalAlignment = GridData.FILL;
        leftColumn.setLayoutData(gd);
        leftColumn.setLayout(new GridLayout(1, false));
        createLayoutSelectionGroup(leftColumn);
        createElementAlignmentGroup(leftColumn);
        return thePage;
    }

    /**
     * Returns the _style.
     * 
     * @return Visualization
     */
    public SystemVisualization getSysVis()
    {
        return _sysVis;
    }

    private void createLayoutPage(int tabCount)
    {
        String title = null;
        title = "System Visualization";
        _layoutPage = createSystemLayoutPage(_folder);
        if (title != null)
        {
            TabItem specialTab = new TabItem(_folder, SWT.NONE, tabCount);
            specialTab.setText(title);
            specialTab.setControl(_layoutPage);
        }
    }

    /**
     * @param tabCount
     */
    private void createConnectionRulePage(int tabCount)
    {
        String title = null;
        title = "Connection Rules";
        _connectionPage = createConnectionRulePage(_folder);
        if (title != null)
        {
            TabItem tab = new TabItem(_folder, SWT.NONE, tabCount);
            tab.setText(title);
            tab.setControl(_connectionPage);
        }
    }

    /**
     * @param _folder2
     * @return
     */
    private Composite createConnectionRulePage(TabFolder parent)
    {
        Composite thePage = new Composite(parent, SWT.NONE);
        GridLayout pageLayout = new GridLayout(1, false);
        thePage.setLayout(pageLayout);
        Label l = new Label(thePage, SWT.NONE);
        l
                .setText("Existing rules - [Component | Port] | Role -> Connector -> Role | [Port | Component ]:");
        _ruleViewer = new TableViewer(thePage, SWT.FULL_SELECTION | SWT.BORDER
                | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);
        _ruleViewer.setContentProvider(new RuleContentProvider());
        _ruleViewer.setLabelProvider(new RuleLabelProvider());
        Table table = _ruleViewer.getTable();
        table.setLayoutData(new GridData(GridData.FILL_HORIZONTAL
                | GridData.FILL_VERTICAL));
        table.setHeaderVisible(true);
        table.setLinesVisible(true);
        TableLayout layout = new TableLayout();
//        TableColumn p1Col = new TableColumn(table, SWT.CENTER, 0);
//        p1Col.setText("Name:");
//        ColumnLayoutData p1Data = new ColumnWeightData(10, true);
//        layout.addColumnData(p1Data);
        
        TableColumn imgCol = new TableColumn (table, SWT.CENTER, 0);
        imgCol.setText ("");
        ColumnLayoutData imgData = new ColumnWeightData (20, false);
        layout.addColumnData(imgData);
        
        TableColumn p2Col = new TableColumn(table, SWT.CENTER, 1);
        p2Col.setText("Rule Description:");
        ColumnLayoutData p2Data = new ColumnWeightData(80, true);
        layout.addColumnData(p2Data);
        table.setLayout(layout);
        _ruleViewer.setInput(_sysVis);
        Composite buttonGroup = new Composite(thePage, SWT.NONE);
        buttonGroup.setLayout(new FillLayout(SWT.HORIZONTAL));
        GridData bgd = new GridData();
        bgd.horizontalAlignment = GridData.CENTER;
        Button addButton = new Button(buttonGroup, SWT.PUSH);
        addButton.setText("New...");
        addButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                //Show the wizard for creating a new connection pattern
                ConnectionPatternWizard wizard = new ConnectionPatternWizard(_family, true);
                
                WizardDialog dialog = new WizardDialog(Display.getDefault()
                        .getActiveShell(), wizard);
                dialog.create();
                dialog.open();
                if (!(IDialogConstants.CANCEL_ID == dialog.getReturnCode()))
                {
                    // Build the rule out of the different parts on the dialog
                    PortConnectionRule userChosen = new PortConnectionRule();
                    userChosen.setConnectorType(wizard.getTypePage().selectionConn);
                    userChosen.addAssociation(new PortRoleAssociation(
                            wizard.getTypePage().selectionPorts[0],
                            wizard.getTypePage().selectionRoles[0],
                            wizard.getNamePage().nameRoles[0],
                            wizard.getNamePage().namePorts[0],
                            wizard.getTypePage().selectionComps[0]));
                    userChosen.addAssociation(new PortRoleAssociation(
                            wizard.getTypePage().selectionPorts[1],
                            wizard.getTypePage().selectionRoles[1],
                            wizard.getNamePage().nameRoles[1],
                            wizard.getNamePage().namePorts[0],
                            wizard.getTypePage().selectionComps[1]));

                    userChosen.setSource(wizard.getNamePage().ruleBi ? null
                            : wizard.getTypePage().selectionPorts[0]);

                    PortConnectionRule rule = userChosen;
                    _sysVis.addPortConnectionRule(rule);

                    //anytime something is added, be sure to remove
                    //the default connection rule, in case it is still there
                    _sysVis.removePortConnectionRule(VisualStyleRegistry.DEFAULT_PORT_CONNECTION_RULE);
                    
                    //if the source of the rule is set to null, it means it
                    // should be bidirectional, and we need to create the reverse of
                    // it
                    if (rule.getSource() == null)
                        _sysVis.addPortConnectionRule(rule.getReverse());
                    _ruleViewer.refresh();
                }
            }
        });
        Button removeButton = new Button(buttonGroup, SWT.PUSH);
        removeButton.setText("Remove");
        removeButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                ISelection sel = _ruleViewer.getSelection();
                if (sel instanceof IStructuredSelection)
                {
                    IStructuredSelection isel = (IStructuredSelection) sel;
                    if (!isel.isEmpty())
                    {
                        for (Iterator i = isel.iterator(); i.hasNext();)
                        {
                            PortConnectionRule rule = (PortConnectionRule) i
                                    .next();

                            java.util.List mylist = _sysVis
                                    .getPortConnectionRules();

                            for (int j = 0; j < mylist.size(); j++)
                            {
                                if (((PortConnectionRule) mylist.get(j))
                                        .generateKey().equals(
                                                rule.generateKey()))
                                    mylist.remove(j);
                            }
                            
                            //if there is nothing left, be sure to add the generic rule
                            //back in
                            if (mylist.size() == 0)
                                _sysVis.addPortConnectionRule(VisualStyleRegistry.DEFAULT_PORT_CONNECTION_RULE);
                        }
                        
                        _ruleViewer.refresh();
                    }
                }
            }
        });
        Button modButton = new Button(buttonGroup, SWT.PUSH);
        modButton.setText("Modify");
        modButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                ISelection sel = _ruleViewer.getSelection();
                if (sel instanceof IStructuredSelection)
                {
                    IStructuredSelection isel = (IStructuredSelection) sel;
                    if (!isel.isEmpty())
                    {
                        for (Iterator i = isel.iterator(); i.hasNext();)
                        {
                            PortConnectionRule rule = (PortConnectionRule) i
                                    .next();

                            //Show the wizard for creating a new connection pattern
                            ConnectionPatternWizard wizard = new ConnectionPatternWizard(_family, true, rule);
                            
                            WizardDialog dialog = new WizardDialog(Display.getDefault()
                                    .getActiveShell(), wizard);
                            dialog.create();
                            dialog.open();
                            if (!(IDialogConstants.CANCEL_ID == dialog.getReturnCode()))
                            {
                            	ImageManager.disposeImage(rule.toString ());
                                rule.setConnectorType(wizard.getTypePage().selectionConn);
                                rule.getAssociations()[0].setPortType(wizard.getTypePage().selectionPorts[0]);
                                rule.getAssociations()[0].setRoleType(wizard.getTypePage().selectionRoles[0]);
                                rule.getAssociations()[0].setRoleName(wizard.getNamePage().nameRoles[0]);
                                rule.getAssociations()[0].setPortName(wizard.getNamePage().namePorts[0]);
                                rule.getAssociations()[0].setComponentType(wizard.getTypePage().selectionComps[0]);
                                rule.getAssociations()[1].setPortType(wizard.getTypePage().selectionPorts[1]);
                                rule.getAssociations()[1].setRoleType(wizard.getTypePage().selectionRoles[1]);
                                rule.getAssociations()[1].setRoleName(wizard.getNamePage().nameRoles[1]);
                                rule.getAssociations()[1].setPortName(wizard.getNamePage().namePorts[1]);
                                rule.getAssociations()[1].setComponentType(wizard.getTypePage().selectionComps[1]);

                                rule.setSource(wizard.getNamePage().ruleBi ? null
                                        : wizard.getTypePage().selectionPorts[0]);
                            }
                        }
                        _ruleViewer.refresh();
                    }
                }
            }
        });
        _allowRuleDefine = new Button(thePage, SWT.CHECK);
        _allowRuleDefine
                .setText("Allow users of this family to define new system-specific patterns");
        _allowRuleDefine.setSelection(_sysVis.getAllowNewRule());

        _allowRuleDefine.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                _sysVis.setAllowNewRule(_allowRuleDefine
                        .getSelection());
            }
        });
        return thePage;
    }

    private void createElementAlignmentGroup(Composite parent)
    {
        Group mainGroup = new Group(parent, SWT.NONE);
        GridData sgld = new GridData(GridData.FILL_HORIZONTAL);
        mainGroup.setLayoutData(sgld);
        mainGroup.setText("Element Alignment");
        GridLayout sggl = new GridLayout(3, false);
        mainGroup.setLayout(sggl);
        _alignElementTypeTable = new ElementAlignmentList(mainGroup, SWT.NONE);
        GridData pard = new GridData(GridData.FILL_BOTH);
        pard.grabExcessHorizontalSpace = true;
        _alignElementTypeTable.setLayoutData(pard);
        _alignElementTypeTable.setSize(300, 80);
        _alignElementTypeTable.setInput(_sysVis.getPlacementRules());
        _alignElementTypeTable
                .setRegions(_sysVis.getRows() * _sysVis.getCols());
        Composite buttons = new Composite(mainGroup, SWT.NONE);
        buttons.setLayout(new GridLayout(1, true));
        Button deleteButton = new Button(buttons, SWT.PUSH);
        deleteButton.setText(">>");
        deleteButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                java.util.List list = _alignElementTypeTable
                        .deleteSelectedRules();
                for (Iterator iter = list.iterator(); iter.hasNext();)
                {
                    String sel = (String) iter.next();
                    _typeList.add(sel);
                }
                EditVisualizationDialog.sortList(_typeList);
            }
        });
        Button addButton = new Button(buttons, SWT.PUSH);
        addButton.setText("<<");
        addButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent evt)
            {
                int[] selections = _typeList.getSelectionIndices();
                for (int i = 0; i < selections.length; i++)
                {
                    int index = selections[i];
                    PlacementRule rule = new PlacementRule();
                    rule._type = _typeList.getItem(index);
                    if (!_sysVis.getPlacementRules().contains(rule))
                        _sysVis.getPlacementRules().add(rule);
                }
                _typeList.remove(selections);
                _alignElementTypeTable.setInput(_sysVis.getPlacementRules());
            }
        });
        Composite ruleContainer = new Composite(mainGroup, SWT.NONE);
        ruleContainer.setLayout(new GridLayout(1, false));
        Label label = new Label(ruleContainer, SWT.NONE);
        pard = new GridData(GridData.FILL_HORIZONTAL);
        label.setLayoutData(pard);
        label.setText("Available Types:");
        Composite alignPortContainer = new Composite(ruleContainer, SWT.NONE);
        _typeList = new List(alignPortContainer, SWT.BORDER | SWT.H_SCROLL
                | SWT.V_SCROLL | SWT.MULTI);
        pard = new GridData(GridData.FILL_BOTH);
        _typeList.setLayoutData(pard);
        _typeList.setSize(150, 80);
        refreshTypesInAlignmentGroup();
    }

    private void refreshTypesInAlignmentGroup()
    {
        _typeList.removeAll();
        AcmeVector types = _family.getTypes().castToAcmeSetVector();
        AcmeEnumeration componentEnum = types
                .enumerate(new acmeLib.core.filters.ComponentTypeEnumFilter());
        AcmeEnumeration connectorEnum = types
                .enumerate(new acmeLib.core.filters.ConnectorTypeEnumFilter());
        java.util.List comps = new java.util.LinkedList();
        java.util.List conns = new java.util.LinkedList();
        while (componentEnum.moreItems())
        {
            comps.add(((AcmeElementType) componentEnum.getNextItem())
                    .getTypeName());
        }
        while (connectorEnum.moreItems())
        {
            conns.add(((AcmeElementType) connectorEnum.getNextItem())
                    .getTypeName());
        }
        Collections.sort(comps);
        Collections.sort(conns);
        Iterator i = comps.iterator();
        while (i.hasNext())
        {
            _typeList.add((String) i.next());
        }
        i = conns.iterator();
        while (i.hasNext())
        {
            _typeList.add((String) i.next());
        }
    }

    private class RuleContentProvider implements IStructuredContentProvider
    {

        /*
         * (non-Javadoc)
         * 
         * @see org.eclipse.jface.viewers.IContentProvider#dispose()
         */
        public void dispose()
        {
        }

        /*
         * (non-Javadoc)
         * 
         * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
         *         java.lang.Object, java.lang.Object)
         */
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
        {
        }

        /*
         * (non-Javadoc)
         * 
         * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
         */
        public Object[] getElements(Object inputElement)
        {
            if (inputElement instanceof SystemVisualization)
            {
                SystemVisualization sv = (SystemVisualization) inputElement;
                Object[] children = new Object[sv.getPortConnectionRules()
                        .size()];
                int index = 0;
                for (Iterator i = sv.getPortConnectionRules().iterator(); i
                        .hasNext();)
                {
                    PortConnectionRule rule = (PortConnectionRule) i.next();
                    children[index++] = rule;
                }
                return children;
            }
            return new Object[0];
        }
    }

    private class RuleLabelProvider extends LabelProvider implements
            ITableLabelProvider
    {

        /*
         * (non-Javadoc)
         * 
         * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
         *         int)
         */
        public Image getColumnImage(Object element, int columnIndex)
        {
        	if (columnIndex == 0) {
        		PortConnectionRule rule = (PortConnectionRule )element;
        		Image image = ImageManager.getExistingImage(rule.toString());
        		if (image == null) {
        			image = Helper.generateConnectionRuleImage(rule, _family);
        			
        		}
        		return image;
        	}
        	return null;
        }

        /*
         * (non-Javadoc)
         * 
         * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
         *         int)
         */
        public String getColumnText(Object element, int columnIndex)
        {
            if (element instanceof PortConnectionRule)
            {
                PortConnectionRule rule = (PortConnectionRule) element;
                if (!rule.getName().equals("generic"))
                {
	                switch (columnIndex)
	                {
//	                    case 0:
//	                        return rule.getName();
	                    case 1:
	                        return rule.toString();
//	                    case 3:
//	                        return generateRoleString(rule);
	                }
                }
            }
            return "";
        }

        /**
         * @param rule
         * @return
         */
        private String generateRoleString(PortConnectionRule rule)
        {
            PortRoleAssociation[] assocs = rule.getAssociations();
            String ret = "";
            for (int i = 0; i < assocs.length; i++)
            {
                if (assocs[i].getRoleName() != null)
                {
                    ret += assocs[i].getRoleName() + " : ";
                }
                ret += assocs[i].getRoleType();
                if (i < assocs.length - 1)
                    ret += ", ";
            }
            return ret;
        }

        /**
         * @param rule
         * @return
         */
        private String generatePortString(PortConnectionRule rule)
        {
            PortRoleAssociation[] assocs = rule.getAssociations();
            String ret = "";
            for (int i = 0; i < assocs.length; i++)
            {
                ret += assocs[i].getPortType();
                if (i < assocs.length - 1)
                    ret += ", ";
            }
            return ret;
        }
    }
}