/*
 * Created on Feb 16, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.styles;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.forms.widgets.FormToolkit;
import AcmeStudio.ui.views.parts.PropertyValueInspector;
import AcmeStudio.util.Helper;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.properties.AcmeProperty;
import acmeLib.properties.AcmePropertyValue;
import acmeLib.typeManagement.AcmeTypeManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @author schmerl
 * 
 * Allows editing or creating of visual variants
 */
public class EditVariantDialog extends Dialog implements ImagePreviewer {
   /* Should consider making this a subclass of EditVisDialog
    * This would save duplicating all the color stuff,
    * though it's not clear how things like varying port position
    * or other things would work.
    */

    /** The variant being edited **/
    protected VisVariant _variant = null;
    
    protected Text _txtVariantName;
    protected String _variantName;
    protected Combo _lstCondition;
    protected int _condition;
    protected Combo _lstProperty;
    protected String _property;
    protected Text _txtValue;
    protected AcmePropertyValue _value;
    protected Button _btn;
    
    
    protected Button _btnOutlineColor;
    protected Color _outlineColor;
    protected Button _btnNoLine;
    protected Button _btnSolidLine;
    protected Button _btnDotLine;
    protected Button _btnDashLine;
    protected Button _btnDashDotLine;
    protected Button _btnDashDotDotLine;
    protected Button _btnExclusive;
    
    ImageSelectionGroup _imageSelectionGroup;   
    protected Text _txtOutlineWidth;
    
    AcmeElement _type;
    ElementVisualization _vis;

 

    private Text _txtContainedIn;

    private Button _btnPropCondition;

    private Button _btnTypeCondition;

    /** 
     * Constructs a dialog that creates a new variant 
     *  @param shell Where to open the dialog 
     */
    public EditVariantDialog (Shell shell, ElementVisualization basedOn, AcmeElement type) {
        super (shell);
        //_variant = new Visualization.Variant ();
        _type = type;
        _vis = basedOn.copy ();
    }
    
    /**
     * Constructs a dialog that edits an existing variant
     * @param shell Where to open the dialog
     * @param var The variant to edit
     */
    public EditVariantDialog (Shell shell, VisVariant var, AcmeElement type) {
        super (shell);
        _variant = var;
        _type = type;
        _vis = _variant._style.copy ();
    }
    
    

    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite par =  (Composite )super.createDialogArea(parent);
        par.setBounds(0, 0, 0, 0);
        par.setLayout(new GridLayout (2, false));
        Group nameGroup = new Group (par, SWT.NONE);
        nameGroup.setLayout(new GridLayout (2, false));
        GridData ngd = new GridData (GridData.FILL_HORIZONTAL);
        ngd.horizontalSpan = GridData.HORIZONTAL_ALIGN_FILL;
        ngd.grabExcessHorizontalSpace = true;
        nameGroup.setLayoutData(ngd);
        
        Label nameLabel = new Label (nameGroup, SWT.NONE);
        nameLabel.setText ("Variant name:");
        _txtVariantName = new Text (nameGroup, SWT.BORDER | SWT.SINGLE);
        GridData vngd = new GridData (GridData.FILL_HORIZONTAL);
        vngd.horizontalAlignment = GridData.FILL;
        vngd.grabExcessHorizontalSpace = true;
        _txtVariantName.setLayoutData(vngd);
        Group conditionGroup = new Group (par, SWT.NONE);
        conditionGroup.setText ("Condition");
        GridData cgd = new GridData (GridData.FILL_HORIZONTAL);
        cgd.horizontalSpan = GridData.FILL;
        conditionGroup.setLayoutData(cgd);
        final GridLayout gridLayout = new GridLayout (3, false);
        gridLayout.numColumns = 3;
        conditionGroup.setLayout (gridLayout);
        
        _btnPropCondition = new Button (conditionGroup, SWT.RADIO);
        _btnPropCondition.setText("Condition is based on property");
        SelectionAdapter conditionSelectionAdapter = new SelectionAdapter () {
          public void widgetSelected (SelectionEvent evt) {
              if (evt.widget == _btnPropCondition) {
                  setConditionEnablement(true);
              }
              else {
                  setConditionEnablement (false);
              }
          }

        private void setConditionEnablement(boolean prop) {
            _lstProperty.setEnabled(prop);
              _lstCondition.setEnabled (prop);
              _txtValue.setEnabled (prop);
              _txtContainedIn.setEnabled (!prop);
        }
        };
        _btnPropCondition.addSelectionListener (conditionSelectionAdapter);
        GridData pcgd = new GridData ();
        pcgd.horizontalSpan = 3;
        _btnPropCondition.setLayoutData(pcgd);
        
        Label propLabel = new Label (conditionGroup, SWT.NONE);
        propLabel.setText ("Property:");
        Label conditionLabel = new Label (conditionGroup, SWT.NONE);
        conditionLabel.setText ("Condition:");
        Label valueLabel = new Label (conditionGroup, SWT.NONE);
        valueLabel.setText ("Value:");
        
        _lstProperty = new Combo (conditionGroup, SWT.DROP_DOWN);
        _lstProperty.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        _lstCondition = new Combo (conditionGroup, SWT.DROP_DOWN | SWT.READ_ONLY);
        _lstCondition.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        _txtValue = new Text (conditionGroup, SWT.BORDER);
        _txtValue.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        _btnTypeCondition = new Button (conditionGroup, SWT.RADIO);
        _btnTypeCondition.setText ("Condition is based on containment in type");
        _btnTypeCondition.addSelectionListener(conditionSelectionAdapter);
        GridData tcgd = new GridData ();
        tcgd.horizontalSpan = 3;
        _btnTypeCondition.setLayoutData (tcgd);
        
        Label ciL = new  Label(conditionGroup, SWT.NONE);
        ciL.setText ("Type name:");
        GridData cigd = new GridData ();
        cigd.horizontalSpan = GridData.FILL;
  
        ciL.setLayoutData(cigd);
        
        _txtContainedIn = new Text (conditionGroup, SWT.BORDER | SWT.SINGLE);
        cigd = new GridData (GridData.FILL_HORIZONTAL);
        cigd.horizontalSpan = GridData.FILL;
        cigd.horizontalAlignment = GridData.HORIZONTAL_ALIGN_FILL;
        _txtContainedIn.setLayoutData(cigd);
        
        Group variationGroup = new Group (par, SWT.NONE);
        variationGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        variationGroup.setText ("Variation:");
        variationGroup.setLayout (new GridLayout (2, false));
        
        final ColorDialog colorDialog = new ColorDialog (variationGroup.getShell());
        
        Label fillLabel = new Label (variationGroup, SWT.NONE);
        fillLabel.setText ("Fill Color:");
        _btn = new Button (variationGroup, SWT.PUSH);
        _btn.addSelectionListener(new SelectionAdapter () {
        	public void widgetSelected (SelectionEvent evt) {
        		RGB rgb = colorDialog.open ();
        		if (rgb == null) return;
        		Image image = _btn.getImage ();
        		if (image == null) {
        			image = new Image (Display.getDefault(), 30, 12);
        		}
        		drawButtonImage (image, rgb);
        		_btn.setImage (image);
//        		_btn.setText ("");
        		updatePreview ();
        		_vis.setFillColor(SWTResourceManager.getColor (rgb));
        	}
        });
        
        Label outlineLabel = new Label (variationGroup, SWT.NONE);
        outlineLabel.setText ("Outline Color:");
        _btnOutlineColor = new Button (variationGroup, SWT.PUSH);
        _btnOutlineColor.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected (SelectionEvent evt) {
				RGB rgb = colorDialog.open ();
				if (rgb == null) return;
				Image image = _btnOutlineColor.getImage ();
				if (image == null) {
					image = new Image (Display.getDefault (), 30, 12);
				}
				drawButtonImage (image, rgb);
				updatePreview ();
				_btnOutlineColor.setImage (image);
//				_btnOutlineColor.setText("");
				_vis.setOutlineColor(SWTResourceManager.getColor (rgb));
			}
		});
        
        Label outlineStyle = new Label (variationGroup, SWT.NONE);
        outlineStyle.setText ("Outline Style:");
        
		SelectionAdapter buttonSelected = new SelectionAdapter () {
		   public void widgetSelected (SelectionEvent evt) {
        		if (((Button )evt.widget).getSelection () && evt.widget.getData() instanceof Integer) {
        			Integer style = (Integer )evt.widget.getData ();
        			_vis.setOutlineStyle(style.intValue());
        			updatePreview(); 		
        		}
		   }
	   };
	   
	   	Group outlineGroup = new Group (variationGroup, SWT.NONE);
	   	outlineGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
       	outlineGroup.setLayout (new GridLayout (2, true));
       	_btnNoLine = new Button (outlineGroup, SWT.RADIO);
       	_btnNoLine.setText ("No line");
        GridData nlgd = new GridData (); nlgd.horizontalSpan = GridData.FILL;
        _btnNoLine.setLayoutData(nlgd);
        _btnNoLine.addSelectionListener(buttonSelected);

        FormToolkit toolkit = new FormToolkit (Display.getCurrent());
        _btnSolidLine = EditVisualizationDialog.createLineStyleButton(toolkit, outlineGroup, SWT.LINE_SOLID, buttonSelected);
        _btnDotLine = EditVisualizationDialog.createLineStyleButton(toolkit, outlineGroup, SWT.LINE_DOT, buttonSelected);
        _btnDashLine = EditVisualizationDialog.createLineStyleButton(toolkit, outlineGroup, SWT.LINE_DASH, buttonSelected);
        _btnDashDotLine = EditVisualizationDialog.createLineStyleButton(toolkit, outlineGroup, SWT.LINE_DASHDOT, buttonSelected);
        _btnDashDotDotLine = EditVisualizationDialog.createLineStyleButton(toolkit, outlineGroup, SWT.LINE_DASHDOTDOT, buttonSelected);
        
        
        Label imageLabel = new Label (variationGroup, SWT.NONE);
        imageLabel.setText ("Decorations: ");
        _imageSelectionGroup = new ImageSelectionGroup (variationGroup, _vis, this);

        _btnExclusive = new Button (variationGroup, SWT.CHECK);
        _btnExclusive.setText("Cannot be blended with other variants");
        GridData egd = new GridData ();
        egd.horizontalSpan = GridData.FILL;
        _btnExclusive.setLayoutData(egd);
                
        importVariant ();
        
        
        return par;
    }
    
    /**
	 * 
	 */
	public void updatePreview() {
	
	}

	/**
	 * @param image
	 * @param rgb
	 */
	protected void drawButtonImage(Image image, RGB rgb) {
		Color col = SWTResourceManager.getColor (rgb);
		GC gc = new GC (image);
		gc.setBackground(col);
		gc.setForeground(col);
		Rectangle bounds = image.getBounds ();
		gc.fillRectangle(0, 0, bounds.width, bounds.height);
		gc.drawRectangle(0, 0, bounds.width, bounds.height);
		gc.dispose ();
//		col.dispose();	
	}

	private static AcmeEnumeration collectAllProperties (AcmeElementType type) {
		AcmeEnumeration props = ((AcmeElement )type).getProperties();
		
		AcmeEnumeration supers = type.getSuperTypes().enumerate();
		while (supers.moreItems ()) {
			AcmeElementType t = (AcmeElementType) supers.getNextItem ();
			props.concatenate(collectAllProperties (t));
		}
		
		props = props.castToAcmeSetVector().enumerate();
		return props;
	}
	
	private void importVariant () {
		// Do _lstProperty
		AcmeEnumeration props = collectAllProperties ((AcmeElementType )_type);
		java.util.List pList = Helper.convertAcmeEnumerationToList(props);
		Collections.sort(pList, new Comparator () {

			public int compare(Object o1, Object o2) {
				AcmeProperty t1 = (AcmeProperty )o1;
				AcmeProperty t2 = (AcmeProperty )o2;
				return String.CASE_INSENSITIVE_ORDER.compare(t1.getName(), t2.getName ());
			}
		
		});
		
		props = Helper.convertCollectionToAcmeEnumeration(pList);
		
		while (props.moreItems()) {
			AcmeProperty prop = (AcmeProperty )props.getNextItem();
			_lstProperty.add (prop.getName());
			_lstProperty.setData(prop.getName (), prop);
		}
		
        if (_variant != null) {
            _txtVariantName.setText (_variant._name);
            _txtVariantName.setEnabled (false);
        }
        for (Iterator iter = VisVariant.OPNAMES.iterator(); iter.hasNext();) {
        	String element = (String) iter.next();
        	_lstCondition.add (element);
        }
        _btnPropCondition.setSelection (true);
		if (_variant != null && _variant._type == VisVariant.PROPERTY_TYPE) importPropertyContext();
        else if (_variant != null && _variant._type == VisVariant.CONTEXT_TYPE) importContext ();
		
		if (_variant != null) {
			final Image fillImage = new Image (Display.getDefault (), 30, 12);
			drawButtonImage(fillImage, _variant._style.getFillColor().getRGB());
			_btn.setImage (fillImage);
            //BugFix# 360, same fill and outline color gets loaded, now fixed 
            final Image outlineImage = new Image (Display.getDefault(), 30, 12);
			drawButtonImage (outlineImage, _variant._style.getOutlineColor().getRGB ());
			_btnOutlineColor.setImage (outlineImage);
			switch (_variant._style.getOutlineStyle()) {
				case SWT.NONE: _btnNoLine.setSelection (true); break;
				case SWT.LINE_SOLID: _btnSolidLine.setSelection (true); break;
				case SWT.LINE_DOT: _btnDotLine.setSelection (true); break;
				case SWT.LINE_DASH: _btnDashLine.setSelection (true); break;
				case SWT.LINE_DASHDOT: _btnDashDotLine.setSelection (true); break;
				case SWT.LINE_DASHDOTDOT: _btnDashDotDotLine.setSelection (true); break;
			}
			_imageSelectionGroup.importStyle ();
            _btnExclusive.setSelection(_variant._exclusive);
			
		}
		else {
			final Image fillImage = new Image (Display.getDefault (), 30, 12);
			drawButtonImage (fillImage, _vis.getFillColor().getRGB());
			_btn.setImage (fillImage);
			//_btn.setText ("Default");
			final Image outlineImage = new Image (Display.getDefault(), 30, 12);
			drawButtonImage (outlineImage, _vis.getOutlineColor().getRGB ());
			_btnOutlineColor.setImage (outlineImage);
			//_btnOutlineColor.setText ("Default");
			
		
		}
		
		
    }

	private void importPropertyContext() {
	    _btnPropCondition.setSelection (true);
        if (_variant != null) _lstProperty.setText(_variant._property);
		
		
		
		if (_variant != null) _lstCondition.setText ((String )VisVariant.OPNAMES.get (_variant._operator));
		
		if (_variant != null) _txtValue.setText(_variant._value.toString());
    }
    
    private void importContext () {
        _btnTypeCondition.setSelection(true);
        _txtContainedIn.setText (_variant._contextType);
    }

    private boolean exportVariant () {
		boolean ok = true;
			
		if (_variant == null) {
				
			_variant = new VisVariant ();
		}
		_variant._name = _txtVariantName.getText();
		if (_variant._name == null || _variant._name.equals ("")) {
			ok = false;
			MessageDialog.openError(Display.getCurrent().getActiveShell(), "Error", "The variant must be given an name");
		}
		if (_lstProperty.isEnabled()) {
		    
		    _variant._type = VisVariant.PROPERTY_TYPE;
		    _variant._property = _lstProperty.getText();
		    _variant._operator = VisVariant.OPNAMES.indexOf (_lstCondition.getText());
		    try {
		        _value = PropertyValueInspector.getPropertyValue(_txtValue.getText ());
		        AcmeProperty prop = (AcmeProperty )_lstProperty.getData(_lstProperty.getText());
		        if (prop != null) {
		            if (!AcmeTypeManager.typecheckProperty(prop.getType(), _value)) {
		                ok = false;
		                MessageDialog.openError (Display.getCurrent ().getActiveShell (), "Type error", "The value does not match the type of the property.\nPlease re-enter.");
		            }
		            _variant._value = _value;
		        }
		        
		    } catch (ParseException e) {
		        MessageDialog.openError(Display.getCurrent().getActiveShell(), "Parse Error", "Could not parse the property value.\nPlease re-enter.");
		        ok = false;
		    }
		}
        else {
            _variant._type = VisVariant.CONTEXT_TYPE;
            _variant._contextType = _txtContainedIn.getText ();
        }
		if (ok) {
			_variant._style = _vis.copy ();
            _variant._exclusive = _btnExclusive.getSelection();
		}
		
		return ok;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		if (exportVariant ()) {
			_imageSelectionGroup.dispose ();
		  	super.okPressed();
//		  	if (img1 != null) img1.dispose ();
//		  	if (img2 != null) img2.dispose ();
		}
	}
	
	public VisVariant getVariant () {
		return _variant;
	}

}
