/*
 * Created on Feb 23, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.styles;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import AcmeStudio.ui.editors.diagram.styles.constants.Orientation;

/**
 * @author schmerl
 */
public class PortVisualization extends ElementVisualization {

	protected static final long FLAG_DEFAULT_ROTATEPORT = LAST_DEFAULT_FLAG << 2;
	protected static final long FLAG_DEFAULT_ALIGN_WITH_ROLE = FLAG_DEFAULT_ROTATEPORT << 2;
	protected static final long FLAG_DEFAULT_ORIENTATION = FLAG_DEFAULT_ALIGN_WITH_ROLE << 2;
	
    private boolean _rotatePortAroundComponentCenter = false;
    private boolean _alignPortWithAttachedRole = false;
    private int _orientation = Orientation.OUT;

    public PortVisualization() {
        super(ElementVisualization.DEFAULT_PORT);
    }

    public PortVisualization(int type) {
        super(type);
    }

    /**
     * @return boolean
     */
    public boolean rotatePort() {
        return _rotatePortAroundComponentCenter;
    }

    /**
     * @return boolean
     */
    public boolean alignPort() {
        return _alignPortWithAttachedRole;
    }

    /**
     * @return orientation
     */
    public int getOrientation() {
        return _orientation;
    }

    /**
     * Sets the alignPortWithAttachedRole.
     * @param alignPortWithAttachedRole The alignPortWithAttachedRole to set
     */
    public void setAlignPort(boolean alignPortWithAttachedRole) {
    	if (alignPortWithAttachedRole != _alignPortWithAttachedRole) {
	        this._alignPortWithAttachedRole = alignPortWithAttachedRole;
	        _defaultFlags &= ~FLAG_DEFAULT_ALIGN_WITH_ROLE;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    /**
     * Sets the rotatePortAroundComponentCenter.
     * @param rotatePortAroundComponentCenter The rotatePortAroundComponentCenter to set
     */
    public void setRotatePort(boolean rotatePortAroundComponentCenter) {
    	if (rotatePortAroundComponentCenter != _rotatePortAroundComponentCenter) {
	        this._rotatePortAroundComponentCenter = rotatePortAroundComponentCenter;
	        _defaultFlags &= ~FLAG_DEFAULT_ROTATEPORT;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    /**
     * Sets the orientation
     * @param orientation The orientation to set
     */
    public void setOrientation(int orientation) {
    	if (orientation != _orientation) {
	        this._orientation = orientation;
	        _defaultFlags &= ~FLAG_DEFAULT_ORIENTATION;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    /* (non-Javadoc)
     * @see AcmeStudio.ui.editors.diagram.styles.Visualization#copyInto(AcmeStudio.ui.editors.diagram.styles.Visualization)
     */
    public void copyInto(ElementVisualization from) {
        _copying = true;
        if (from instanceof PortVisualization) {
            this._rotatePortAroundComponentCenter =
                ((PortVisualization) from)._rotatePortAroundComponentCenter;
            this._alignPortWithAttachedRole =
                ((PortVisualization) from)._alignPortWithAttachedRole;
            this._orientation = ((PortVisualization) from)._orientation;
        }
        super.copyInto(from);
    }
    
    public ElementVisualization copy () {
    	PortVisualization vis = new PortVisualization ();
    	vis.copyInto (this);
    	return vis;
    }

    public void importTypeSpecificVisualization(Element visElement) {
        Element rotateNode =
            (Element) visElement.getElementsByTagName(
                "rotateAroundComponentCenter").item(
                0);
        boolean rotate = importOn(rotateNode);
        setRotatePort(rotate);

        Element alignNode =
            (Element) visElement.getElementsByTagName(
                "alignWithAttachedRole").item(
                0);
        boolean align = importOn(alignNode);
        setAlignPort(align);

        Element orientationNode =
            (Element) visElement.getElementsByTagName("pointer").item(0);
        setOrientation(importOrientation(orientationNode));
    }

    public void exportTypeSpecificVisualization(Document doc, Element root) {
        Element rotateNode = doc.createElement("rotateAroundComponentCenter");
        rotateNode.setAttribute("on", rotatePort() ? "true" : "false");
        root.appendChild(rotateNode);

        Element alignNode = doc.createElement("alignWithAttachedRole");
        alignNode.setAttribute("on", alignPort() ? "true" : "false");
        root.appendChild(alignNode);

        if (_orientation >= 0) {
            Element orientationNode = doc.createElement("pointer");
            orientationNode.setAttribute(
                "orientation",
                (String) Orientation.ORIENTATIONNAMES.get(_orientation));
            root.appendChild(orientationNode);
        }
        root.setAttribute ("type", "PORT");
    }

    /* (non-Javadoc)
     * @see AcmeStudio.ui.editors.diagram.styles.Visualization#fillShape(org.eclipse.draw2d.Graphics, org.eclipse.draw2d.geometry.Rectangle, org.eclipse.draw2d.geometry.Dimension, double)
     */
    public void fillShape(
        Graphics graphics,
        Rectangle rect,
        Dimension size,
        double rotation) {
        super.fillShape(graphics, rect, size, rotation);
    }

    /* (non-Javadoc)
     * @see AcmeStudio.ui.editors.diagram.styles.Visualization#unify(AcmeStudio.ui.editors.diagram.styles.Visualization)
     */
    public ElementVisualization unify(ElementVisualization from) {
    	
    	PortVisualization to = new PortVisualization ();
    	unify (from, to);
    	return to;
    }
    
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#blendVisualization(AcmeStudio.ui.editors.diagram.styles.ElementVisualization, boolean)
	 */
	public void blendVisualization(ElementVisualization vis, boolean treatVisAsSuper) {
		super.blendVisualization (vis, treatVisAsSuper);
		if (vis instanceof PortVisualization) {
			PortVisualization pVis = (PortVisualization )vis;
			_copying = true;
			if (shouldBlendPart(vis, FLAG_DEFAULT_ALIGN_WITH_ROLE)) {
				setAlignPort(pVis.alignPort());
				forceDefault (FLAG_DEFAULT_ALIGN_WITH_ROLE);
			}
			if (shouldBlendPart(vis, FLAG_DEFAULT_ORIENTATION)) {
				setOrientation(pVis.getOrientation());
				forceDefault (FLAG_DEFAULT_ORIENTATION);
			}
			if (shouldBlendPart (vis, FLAG_DEFAULT_ROTATEPORT)) {
				setRotatePort(pVis.rotatePort());
				forceDefault (FLAG_DEFAULT_ROTATEPORT);
			}
			_copying = false;
		}
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#setAllDefaults()
	 */
	protected void setAllDefaults() {
		super.setAllDefaults ();
		_defaultFlags |= FLAG_DEFAULT_ALIGN_WITH_ROLE | FLAG_DEFAULT_ORIENTATION | FLAG_DEFAULT_ROTATEPORT;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#exportDefaults(org.w3c.dom.Document)
	 */
	public Element exportDefaults(Document doc) {
		Element defaults = super.exportDefaults (doc);
		exportDefaultAttribute(defaults, "alignWithRole", FLAG_DEFAULT_ALIGN_WITH_ROLE);
		exportDefaultAttribute(defaults, "orientation", FLAG_DEFAULT_ORIENTATION);
		exportDefaultAttribute(defaults, "rotatePort", FLAG_DEFAULT_ROTATEPORT);
		return defaults;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#importDefaults(AcmeStudio.ui.editors.diagram.styles.ElementVisualization, org.w3c.dom.Element)
	 */
	protected void importDefaults(ElementVisualization vis, Element visElement) {
		super.importDefaults (vis, visElement);
		NodeList dfs = visElement.getElementsByTagName("defaults");
    	if (dfs.getLength() > 0) {
    		Element defaults = (Element )dfs.item (0);
    		setImportedDefault(vis, defaults, "alignWithRole", FLAG_DEFAULT_ALIGN_WITH_ROLE);
    		setImportedDefault(vis,defaults, "orientation", FLAG_DEFAULT_ORIENTATION);
    		setImportedDefault(vis,defaults, "rotatePort", FLAG_DEFAULT_ROTATEPORT);
    	}
	}
}
