/*
 * Created on Feb 23, 2003
 *
 */
package AcmeStudio.ui.editors.diagram.styles;

import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.SWTGraphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Display;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import AcmeStudio.ui.editors.diagram.figures.ElementColorConstants;
import AcmeStudio.ui.editors.diagram.styles.constants.HorizontalTextAlignmentOptions;
import AcmeStudio.ui.editors.diagram.styles.constants.Orientation;
import AcmeStudio.ui.editors.diagram.styles.constants.VerticalTextAlignmentOptions;
import AcmeStudio.util.Constants;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.ManagedImage;
import AcmeStudio.util.Utils;
import acmeLib.elements.AcmeElementType;

/**
 * @author schmerl
 */
public class RoleVisualization extends ElementVisualization {

	protected static final long FLAG_DEFAULT_ROTATEROLE = LAST_DEFAULT_FLAG << 2;
	protected static final long FLAG_DEFAULT_ORIENTATION = FLAG_DEFAULT_ROTATEROLE << 2;
	final int	PREVIEW_LINE_LENGTH	= 10;
    private boolean _rotateRoleAroundConnectorCenter = true;
    private int _orientation = Orientation.OUT;
    
    public RoleVisualization() {
        super(ElementVisualization.DEFAULT_ROLE);        
    }

    public RoleVisualization(int type) {
        super(type);
    }

    /**
     * @return boolean
     */
    public boolean rotateRole() {
        return _rotateRoleAroundConnectorCenter;
    }

    /**
     * @return orientation
     */
    public int getOrientation() {
        return _orientation;
    }

    /**
     * Sets the rotateRoleAroundConnectorCenter.
     * @param rotateRoleAroundConnectorCenter The rotateRoleAroundConnectorCenter to set
     */
    public void setRotateRole(boolean rotateRoleAroundConnectorCenter) {
    	if (rotateRoleAroundConnectorCenter != _rotateRoleAroundConnectorCenter) {
	        this._rotateRoleAroundConnectorCenter = rotateRoleAroundConnectorCenter;
	        _defaultFlags &= ~FLAG_DEFAULT_ROTATEROLE;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    /**
     * Sets the orientation
     * @param orientation The orientation to set
     */
    public void setOrientation(int orientation) {
    	if (orientation != _orientation) {
	        this._orientation = orientation;
	        _defaultFlags &= ~FLAG_DEFAULT_ORIENTATION;
	        firePropertyChange(PROP_VIS_CHANGED, this);
    	}
    }

    /* (non-Javadoc)
     * @see AcmeStudio.ui.editors.diagram.styles.Visualization#copyInto(AcmeStudio.ui.editors.diagram.styles.Visualization)
     */
    public void copyInto(ElementVisualization from) {
        _copying = true;
        if (from instanceof RoleVisualization) {
            setRotateRole(
                ((RoleVisualization) from).rotateRole());
            this._orientation = ((RoleVisualization) from)._orientation;
        }
        super.copyInto(from);
    }
    
    public ElementVisualization copy () {
    	RoleVisualization vis = new RoleVisualization ();
    	vis.copyInto (this);
    	return vis;
    	
    }

    public void importTypeSpecificVisualization(Element visElement) {
        Element rotateNode =
            (Element) visElement.getElementsByTagName(
                "rotateAroundConnectorCenter").item(
                0);
        boolean rotate = importOn(rotateNode);
        setRotateRole(rotate);

        Element orientationNode =
            (Element) visElement.getElementsByTagName("pointer").item(0);
        setOrientation(importOrientation(orientationNode));
    }

    public void exportTypeSpecificVisualization(Document doc, Element root) {
        Element rotateNode = doc.createElement("rotateAroundConnectorCenter");
        rotateNode.setAttribute("on", rotateRole() ? "true" : "false");
        root.appendChild(rotateNode);

        if (_orientation >= 0) {
            Element orientationNode = doc.createElement("pointer");
            orientationNode.setAttribute(
                "orientation",
                (String) Orientation.ORIENTATIONNAMES.get(_orientation));
            root.appendChild(orientationNode);
        }
        
        root.setAttribute ("type", "ROLE");
    }
    
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#unify(AcmeStudio.ui.editors.diagram.styles.ElementVisualization)
	 */
	public ElementVisualization unify(ElementVisualization from) {
		RoleVisualization to = new RoleVisualization ();
		unify (from, to);
		return to;
	}
	
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#blendVisualization(AcmeStudio.ui.editors.diagram.styles.ElementVisualization, boolean)
	 */
	public void blendVisualization(ElementVisualization vis, boolean treatVisAsSuper) {
		super.blendVisualization (vis, treatVisAsSuper);
		if (vis instanceof RoleVisualization) {
			RoleVisualization rVis = (RoleVisualization )vis;
			_copying = true;
			
			if (shouldBlendPart(vis, FLAG_DEFAULT_ORIENTATION)) {
				setOrientation(rVis.getOrientation());
				forceDefault (FLAG_DEFAULT_ORIENTATION);
			}
			if (shouldBlendPart (vis, FLAG_DEFAULT_ROTATEROLE)) {
				setRotateRole(rVis.rotateRole());
				forceDefault (FLAG_DEFAULT_ROTATEROLE);
			}
			_copying = false;
		}
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#setAllDefaults()
	 */
	protected void setAllDefaults() {
		super.setAllDefaults ();
		_defaultFlags |= FLAG_DEFAULT_ORIENTATION | FLAG_DEFAULT_ROTATEROLE;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#exportDefaults(org.w3c.dom.Document)
	 */
	public Element exportDefaults(Document doc) {
		Element defaults = super.exportDefaults (doc);
		exportDefaultAttribute(defaults, "orientation", FLAG_DEFAULT_ORIENTATION);
		exportDefaultAttribute(defaults, "rotateRole", FLAG_DEFAULT_ROTATEROLE);
		return defaults;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#importDefaults(AcmeStudio.ui.editors.diagram.styles.ElementVisualization, org.w3c.dom.Element)
	 */
	protected void importDefaults(ElementVisualization vis, Element visElement) {
		super.importDefaults (vis, visElement);
		NodeList dfs = visElement.getElementsByTagName("defaults");
    	if (dfs.getLength() > 0) {
    		Element defaults = (Element )dfs.item (0);
    		setImportedDefault(vis, defaults, "orientation", FLAG_DEFAULT_ORIENTATION);
    		setImportedDefault(vis, defaults, "rotateRole", FLAG_DEFAULT_ROTATEROLE); 		
    	}
	}
	
	
	/**
	 * @return
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#generateKey()
	 **/
	protected String generateKey() {
		String key = super.generateKey ();
		key += "r";
		return key;
	}
	
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#resetPreview(acmeLib.elements.AcmeElementType)
	 */
	protected void resetPreview(AcmeElementType type) {
		cleanUpPreviewImages(type);
		if (_size.width <= 0 || _size.height <=0) return;
		String key = generateKey ();
		if (ManagedImage.get (key) != null) {
			ManagedImage mip = ManagedImage.get (key);
			_preview = mip;
			mip.reference (this);
			mip = ManagedImage.get (key + "nl");
			_previewNoLabel = mip;
			if (mip != null) mip.reference (this);
			return;
		}
		
		
		String theLabel = generatePreviewLabel (type); 
		
	       Dimension size = new Dimension(_size);
	        org.eclipse.draw2d.geometry.Point shapePoint =
	            new org.eclipse.draw2d.geometry.Point(0, 0);
	        if (theLabel != "") {
	            Font font = getFont();
	            Dimension textSize =
	                FigureUtilities.getStringExtents(theLabel, font);
	            int min = textSize.width + 20;
	            switch (_labelHorizontalAlignment) {
	                case HorizontalTextAlignmentOptions.LEFT_OUTSIDE :
	                    size.width = _size.width + min;
	                    shapePoint = shapePoint.translate(min, 0);
	                    break;
	                case HorizontalTextAlignmentOptions.RIGHT_OUTSIDE :
	                    size.width = _size.width + min;
	                    break;
	                default :
	                    size.width =
	                        _size.width + 20 > min ? _size.width + 20 : min;
	                    shapePoint =
	                        shapePoint.translate((size.width - _size.width) / 2, 0);
	            }
	            switch (_labelVerticalAlignment) {
	                case VerticalTextAlignmentOptions.ABOVE :
	                    size.height = _size.height + textSize.height + 20;
	                    shapePoint = shapePoint.translate(0, textSize.height + 20);
	                    break;
	                case VerticalTextAlignmentOptions.BELOW :
	                    size.height = _size.height + textSize.height + 20;
	                    break;
	                default :
	                    size.height =
	                        _size.height > textSize.height + 20
	                            ? _size.height
	                            : textSize.height + 20;
	                    shapePoint =
	                        shapePoint.translate(
	                            0,
	                            (size.height - _size.height) / 2);
	            }
	        }
	        if (getReplicateShape()) {
	            size = size.expand(8, 8);
	        }
	        

			size.expand (PREVIEW_LINE_LENGTH, 0);
	        shapePoint.translate(PREVIEW_LINE_LENGTH, 0);
	        Image pim_preview = new Image(Display.getDefault(), size.width, size.height);
	        Image pim_previewNoLabel = new Image (Display.getDefault (), size.width, size.height);
	        ImageManager.imageCreated(
	            (type == null ? "" : type.getTypeName())
	                + "Visualization.resetPreview" + this,
	            pim_preview);
	        ImageManager.imageCreated ((type==null?"":type.getTypeName ()) + "Visualization.previewNoLabel" + this,
	        			pim_previewNoLabel);
	        generatePreview(pim_preview, true, theLabel, shapePoint);
	        generatePreview(pim_previewNoLabel, false, theLabel, shapePoint);
	        _preview = new ManagedImage (pim_preview, this);
	        ManagedImage.put (key, _preview);
	        _previewNoLabel = new ManagedImage (pim_previewNoLabel, this);
	        ManagedImage.put (key+"nl", _previewNoLabel);
	}
	
	
	/**
	 * @param type
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#resetSmallImage(acmeLib.elements.AcmeElementType)
	 **/
	protected void resetSmallImage(AcmeElementType type) {
		String key = generateKey () + "sm";
		if (ManagedImage.containsKey (key)) {
			_smallImage = ManagedImage.get (key);
			_smallImage.reference (this);
			return;
		}
		int imageHeight = getSize ().height;
		int visWidth = getSize ().width;
		int imageWidth = 5 + visWidth;
		float aspectRatio = (float )imageHeight/(float )visWidth;
		if (imageHeight > 12) {
			imageHeight = 12;
			imageWidth = Utils.zeroGuard((int )(12/aspectRatio));
			if (imageWidth < 3) imageWidth = 20;
		}
		if (imageWidth > 20) {
			imageWidth = 20;
			imageHeight = Utils.zeroGuard ((int )(20*aspectRatio));
		}
		
		Image tempImage = new Image (null, imageWidth, imageHeight);
		GC gc = new GC (tempImage);
		SWTGraphics graphics = new SWTGraphics (gc);
		graphics.pushState ();
		graphics.setForegroundColor(Constants.TRANSPARENT_COLOR);
		graphics.setBackgroundColor(Constants.TRANSPARENT_COLOR);
		graphics.fillRectangle (0, 0, imageWidth, imageHeight);
		graphics.popState ();
		
		graphics.pushState();
		graphics.setForegroundColor(getOutlineColor());
		graphics.setLineStyle(getOutlineStyle());
		graphics.setLineWidth(getOutlineWidth());
		graphics.drawLine(0, imageHeight/2, 5 + visWidth/2, imageHeight/2);
		graphics.popState();
		
		Rectangle rectangle = new Rectangle (5, 0, visWidth-1, imageHeight-1);
		fillShape (graphics, rectangle, null, 0);
		outlineShape (graphics, rectangle);
		
		ImageData data = tempImage.getImageData();
		data.transparentPixel = data.palette.getPixel(Constants.TRANSPARENT_COLOR.getRGB ());
		Image smi = new Image (null, data);
		ImageManager.imageCreated(getSmallImageLabel(type), smi);
		_smallImage = new ManagedImage (smi, this);
		ManagedImage.put (key, _smallImage);
		tempImage.dispose ();
		gc.dispose ();
		graphics.dispose ();
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.editors.diagram.styles.ElementVisualization#generatePreview(org.eclipse.swt.graphics.Image, boolean, java.lang.String, org.eclipse.draw2d.geometry.Point)
	 */
	protected void generatePreview(Image preview, boolean doLabel, String theLabel, Point shapePoint) {
		GC previewGC = new GC (preview);
		SWTGraphics previewGraphics = new SWTGraphics (previewGC);
		previewGraphics.setBackgroundColor(ElementColorConstants.white);
		Rectangle labelRect = new Rectangle (shapePoint.x, shapePoint.y, _size.width, _size.height);
		previewGraphics.fillRectangle (labelRect);
		previewGraphics.pushState();
		previewGraphics.setForegroundColor(getOutlineColor());
		previewGraphics.setLineStyle(getOutlineStyle());
		previewGraphics.setLineWidth(getOutlineWidth());
		previewGraphics.drawLine(0, _size.height/2, shapePoint.x, _size.height/2);
		previewGraphics.popState();
		
		
		Rectangle r = new Rectangle (shapePoint, _size);
		if (getReplicateShape ()) {
			Point shadowPoint = shapePoint.getCopy().translate (6,6);
			Rectangle rectShadow = new Rectangle (shadowPoint, _size);
			fillShape (previewGraphics, rectShadow, null, 0);
			outlineShape (previewGraphics, rectShadow);
			shadowPoint = shadowPoint.translate (-3, -3);
			rectShadow = new Rectangle (shadowPoint, _size);
			fillShape (previewGraphics, rectShadow, null, 0);
			outlineShape (previewGraphics, rectShadow);
		}
		fillShape (previewGraphics, r, null, 0);
		outlineShape (previewGraphics, r);
		if (doLabel) drawLabel (previewGraphics, labelRect, theLabel, _textColor);
		previewGraphics.dispose();
		previewGC.dispose ();
	}
}
