/*
 * Created on Oct 21, 2003
 * 
 */
package AcmeStudio.ui.editors.diagram.styles;

import java.util.ArrayList;
import java.util.List;

import acmeLib.elements.*;
import acmeLib.properties.*;


public class VisVariant {
    public static final int EQUALS = 0,
        LT = 1,
        GT = 2,
        NEQ = 3,
        LTE = 4,
        GTE = 5;
    public static final List OPNAMES = new ArrayList();
    static {
        OPNAMES.add(EQUALS, "==");
        OPNAMES.add(LT, "<");
        OPNAMES.add(GT, ">");
        OPNAMES.add(NEQ, "!=");
        OPNAMES.add(LTE, "<=");
        OPNAMES.add(GTE, ">=");
    }

    public static final int PROPERTY_TYPE = 0;
    public static final int CONTEXT_TYPE = 1;

    public int _type = PROPERTY_TYPE;
    public int _operator;
    public String _contextType;
    public String _property;
    public ElementVisualization _style;
    public String _name;
    public AcmePropertyValue _value;
    public boolean _exclusive;
    public boolean equals(Object o) {
        return (o instanceof VisVariant)
            && (((VisVariant) o)._name.equals(_name));
    }
    
    
    public boolean appliesTo(AcmeElement elem) {
        if (_type == PROPERTY_TYPE) {
            AcmeProperty prop = elem.getProperty(_property);
            if (prop != null) {
                AcmePropertyValue val1 = prop.getValue();
                AcmePropertyValue val2 = _value;
                return appliesTo(val1, val2, _operator);
            }
            else if (_type == CONTEXT_TYPE) {
                Object o = elem.lookupName(_contextType, true);
                if (o instanceof AcmeElementType) {
                    AcmeElementType type = (AcmeElementType) o;
                    if (elem instanceof AcmePort) {
                        AcmePort port = (AcmePort )elem;
                        AcmeComponent parent = (AcmeComponent )port.getParent ();
                        return parent.declaresType(type);
                    }
                    else if (elem instanceof AcmeRole) {
                        AcmeRole role = (AcmeRole) elem;
                        AcmeElement parent = role.getParent ();
                        return parent.declaresType (type);
                    }
                }
            }
        }
        return false;
    }

    public static boolean appliesTo(AcmePropertyValue val1, AcmePropertyValue val2, int operator) {
        boolean applies = false;
        if (val1 instanceof AcmeInt && val2 instanceof AcmeInt) {
            int v1 = ((AcmeInt) val1).getValue();
            int v2 = ((AcmeInt) val2).getValue();
            switch (operator) {
                case EQUALS :
                    applies = v1 == v2;
                    break;
                   case LT :
                       applies = v1 < v2;
                       break;
                      case GT :
                          applies = v1 > v2;
                          break;
                         case NEQ :
                             applies = v1 != v2;
                             break;
                            case LTE :
                                applies = v1 <= v2;
                                break;
                               case GTE :
                                   applies = v1 >= v2;
            }
        } else if (
                    val1 instanceof AcmeFloat && val2 instanceof AcmeFloat) {
            double v1 = ((AcmeFloat) val1).getValue();
            double v2 = ((AcmeFloat) val2).getValue();
            switch (operator) {
                case EQUALS :
                    applies = v1 == v2;
                    break;
                   case LT :
                       applies = v1 < v2;
                       break;
                      case GT :
                          applies = v1 > v2;
                          break;
                         case NEQ :
                             applies = v1 != v2;
                             break;
                            case LTE :
                                applies = v1 <= v2;
                                break;
                               case GTE :
                                   applies = v1 >= v2;
                                   break;
            }
        } else if (
                    val1 instanceof AcmeBoolean
                    && val2 instanceof AcmeBoolean) {
            boolean v1 = ((AcmeBoolean) val1).getValue();
            boolean v2 = ((AcmeBoolean) val2).getValue();
            switch (operator) {
                case EQUALS :
                    applies = v1 == v2;
                    break;
                   case NEQ :
                       applies = v1 != v2;
                       break;
            }
        } else if (
                    val1 instanceof AcmeString
                    && val2 instanceof AcmeString) {
            String v1 = ((AcmeString) val1).getValue();
            String v2 = ((AcmeString) val2).getValue();
            switch (operator) {
                case EQUALS :
                    applies = v1.equals(v2);
                    break;
                   case NEQ :
                       applies = !v1.equals(v2);
                       break;
                      case LT :
                          applies = v1.compareTo(v2) < 0;
                          break;
                         case GT :
                             applies = v1.compareTo(v2) > 0;
                             break;
                            case LTE :
                                applies = v1.compareTo(v2) <= 0;
                                break;
                               case GTE :
                                   applies = v1.compareTo(v2) >= 0;
                                   break;
            }
        }
         
        return applies;
    }
}