package AcmeStudio.ui.editors.diagram.styles;

import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import AcmeStudio.core.ASModel;
import AcmeStudio.core.resource.ASVisualStyle;
import AcmeStudio.model.ModelHelper;
import AcmeStudio.prefs.VisPreferences;
import AcmeStudio.ui.editors.diagram.visualInfo.VisualInformation;
import AcmeStudio.util.Debug;
import AcmeStudio.util.Helper;
import AcmeStudio.wizards.connection.PortConnectionRule;
import AcmeStudio.wizards.connection.PortRoleAssociation;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.*;

/**
 * @author schmerl
 *
 */
public class VisualStyleRegistry {

    private static VisualStyle _defaultVisualStyle = new VisualStyle();
    static Map _cache = new HashMap ();
    
    static IVisualStyleListener _visListener = new IVisualStyleListener () {

		public void changeVisualStyle(VisualStyle style) {
			_cache.clear ();
		}
    	
    	
    };
    

    public static ComponentVisualization DEFAULT_COMPONENT_VISUALIZATION =
        (ComponentVisualization) VisPreferences.loadVisFromStore(
            ElementVisualization.DEFAULT_COMPONENT);
    public static ConnectorVisualization DEFAULT_CONNECTOR_VISUALIZATION =
        (ConnectorVisualization) VisPreferences.loadVisFromStore(
            ElementVisualization.DEFAULT_CONNECTOR);
    public static PortVisualization DEFAULT_PORT_VISUALIZATION =
        (PortVisualization) VisPreferences.loadVisFromStore(
            ElementVisualization.DEFAULT_PORT);
    public static RoleVisualization DEFAULT_ROLE_VISUALIZATION =
        (RoleVisualization) VisPreferences.loadVisFromStore(
            ElementVisualization.DEFAULT_ROLE);
    public static final SystemVisualization DEFAULT_SYSTEM_VISUALIZATION =
        new SystemVisualization();

    public static PortConnectionRule DEFAULT_PORT_CONNECTION_RULE;
    
    static {
        _defaultVisualStyle.add(
            AcmeComponentType.class.getName(),
            DEFAULT_COMPONENT_VISUALIZATION);
        _defaultVisualStyle.add(
            AcmeConnectorType.class.getName(),
            DEFAULT_CONNECTOR_VISUALIZATION);
        _defaultVisualStyle.add(
            AcmePortType.class.getName(),
            DEFAULT_PORT_VISUALIZATION);
        _defaultVisualStyle.add(
            AcmeRoleType.class.getName(),
            DEFAULT_ROLE_VISUALIZATION);
        _defaultVisualStyle.addSysVis(
            AcmeFamily.class.getName(),
            DEFAULT_SYSTEM_VISUALIZATION);
	    //No rule was defined, so define a generic one
	    PortConnectionRule rule = new PortConnectionRule();
	    PortRoleAssociation assoc1 = new PortRoleAssociation("", "",
	            "r", "p", "");
	    PortRoleAssociation assoc2 = new PortRoleAssociation("", "",
	            "r", "p", "");
		rule.addAssociation(assoc1);
		rule.addAssociation(assoc2);
		rule.setConnectorType("");		  
		rule.setName("generic");
		DEFAULT_PORT_CONNECTION_RULE = rule;
		SystemVisualization sysVis = new SystemVisualization ();
		sysVis.addPortConnectionRule(rule);
		_defaultVisualStyle.addSysVis("default", sysVis);
		DEFAULT_SYSTEM_VISUALIZATION.addPortConnectionRule(rule);
    }

    public static void addStyle (AcmeFamily fam, VisualStyle style) {
//        _styles.put (fam, style);
        ASVisualStyle asVStyle = ASModel.current().getASVisualStyle(fam);
        if (asVStyle != null) {
            asVStyle.setVisualStyle(style);
            style.addVisualStyleListener(_visListener);
            _cache.clear ();
        }
    }

    public static void removeStyle (AcmeFamily fam) {
//        _styles.remove (fam);
        ASVisualStyle asVStyle = ASModel.current().getASVisualStyle(fam);
        if (asVStyle != null) {
        	asVStyle.getVisualStyle().removeVisualStyleListener(_visListener);
            asVStyle.setVisualStyle(null);
        }
    }

    /**
     * Gets the visual style associated with a family. If the
     * family has no direct visual style, the family hierarchy is
     * traversed in a depth-first manner to find a visual style
     * @param fam The family to find the style for
     * @return The visual style associated with fam
     */
    public static VisualStyle getVisualStyle(AcmeFamily fam) {
        //		Object style = _styles.get (fam);
        //		if (style == null) {
        //			AcmeEnumeration fams = fam.getSuperTypes().enumerate();
        //			boolean foundStyle = false;
        //			while (fams.moreItems() && !foundStyle) {
        //				AcmeFamily family = (AcmeFamily )fams.getNextItem();
        //				style = getVisualStyle (family);
        //				if (style != _defaultVisualStyle) {
        //					foundStyle = true;
        //				}
        //			}
        //				
        //		}

        Object style = ASModel.current().getVisualStyle(fam);
        if (style == null) {
        	AcmeEnumeration fams = ModelHelper.gatherSuperTypes(fam);
        	boolean foundStyle = false;
        	while (fams.moreItems () && !foundStyle) {
        		AcmeFamily family = (AcmeFamily )fams.getNextItem ();
        		style = ASModel.current ().getVisualStyle (family);
        		foundStyle = style != _defaultVisualStyle;
        	}
        }

        if (style != null) {
            return (VisualStyle) style;
        } else {
            return _defaultVisualStyle;
        }
    }

    public static boolean hasVisualStyle(AcmeFamily fam) {
        Object style = ASModel.current().getVisualStyle(fam);
        return (style != null);
    }

    public static ElementVisualization getVisualization(AcmeElement el) {
    	if (el instanceof AcmeElementType) return getVisualization ((AcmeElementType )el);
        AcmeEnumeration types = el.getDeclaredTypes();
        ElementVisualization vis = null;

        // Check to see if the element has an overriding visualization
        VisualInformation visInfo = (VisualInformation )el.getUserData("vis-information");
        if (visInfo != null) {
        	if (visInfo.getVisualization() != null) {
        		return visInfo.getVisualization();
        	}
        }
        
//        while (types.moreItems()) {// && (vis == null || isDefaultVisualization (vis))) {
//            AcmeElementType type = (AcmeElementType) types.getNextItem();
//            ElementVisualization eVis = getVisualization (type);
//            if (eVis != null && isDefaultVisualization (vis)) 
//            	vis = vis.copy();
//            vis.blendVisualization (eVis);
//        }

        while (types.moreItems() && (vis == null || isDefaultVisualization (vis))) {
            AcmeElementType type = (AcmeElementType) types.getNextItem();
            vis = getVisualization (type);
            
        }

        if (vis == null) {
            if (el instanceof AcmeComponent) {
                vis =
                    _defaultVisualStyle.getVisualization(
                        AcmeComponentType.class.getName());
            } else if (el instanceof AcmeConnector) {
                vis =
                    _defaultVisualStyle.getVisualization(
                        AcmeConnectorType.class.getName());
            } else if (el instanceof AcmePort) {
                vis =
                    _defaultVisualStyle.getVisualization(
                        AcmePortType.class.getName());
            } else if (el instanceof AcmeRole) {
                vis =
                    _defaultVisualStyle.getVisualization(
                        AcmeRoleType.class.getName());
            }
        }
        return vis;
    }

    public static SystemVisualization getSysVis(AcmeElement el) {
    	if (el == null) return null;
        if (el instanceof AcmeElementType)
            return getSysVis((AcmeElementType) el);

        AcmeEnumeration types = el.getDeclaredTypes();
        SystemVisualization vis = null;

        while (types.moreItems() && vis == null) {
            AcmeElementType type = (AcmeElementType) types.getNextItem();
            vis = getSysVis(type);
        }

        if (vis == null) 
        	vis = _defaultVisualStyle.getSysVis("default");
        return vis;
    }

    public static ElementVisualization getVisualization(AcmeElementType type) {
        ElementVisualization vis = null;
        if (_cache != null) {
        	vis = (ElementVisualization )_cache.get (type);
        	if (vis != null) return vis;
        }
        if (type instanceof AcmeElement) {
            AcmeElement element = (AcmeElement) type;
            AcmeFamily fam = null;
            if (element instanceof AcmeFamily) {
                fam = (AcmeFamily) element;
            } else if (element.getParent() instanceof AcmeFamily) {
                fam = (AcmeFamily) element.getParent();
            } else {
                if (type instanceof AcmeComponentType) {
                    return DEFAULT_COMPONENT_VISUALIZATION;
                }
                else if (type instanceof AcmeConnectorType) {
                    return DEFAULT_CONNECTOR_VISUALIZATION;
                }
                else if (type instanceof AcmeRoleType) {
                    return DEFAULT_ROLE_VISUALIZATION;
                }
                else if (type instanceof AcmePortType) {
                    return DEFAULT_PORT_VISUALIZATION;
                }
            }
            if (fam != null) {
//                VisualStyle style = getVisualStyle(fam);
                vis = recurseGetVisualization(fam, type, new HashSet ());
            }
        }
        vis = VisualStyle.ensureVisIsRightClass(type, vis);
        _cache.put (type, vis);
        return vis;
    }

    private static SystemVisualization getSysVis(AcmeElementType type) {
        SystemVisualization vis = null;
        if (type instanceof AcmeElement) {
            AcmeElement element = (AcmeElement) type;
            AcmeFamily fam = null;
            if (element instanceof AcmeFamily) {
                fam = (AcmeFamily) element;
            } else {
                fam = (AcmeFamily) element.getParent();
            }
            VisualStyle style = getVisualStyle(fam);

            vis = style.getSysVis(fam);

            if (vis == null) {
                vis = _defaultVisualStyle.getSysVis(AcmeFamily.class.getName());
            }
        }
        return vis;
    }

    public static boolean hasVisualStyle(AcmeElementType type) {
        AcmeElement element = (AcmeElement) type;
        AcmeFamily fam = (AcmeFamily) element.getParent();
        return hasVisualStyle(fam);
    }

    private static ElementVisualization recurseGetVisualization(
        AcmeFamily family,
        AcmeElementType type,
		Set visited) {
    	VisualStyle style = getVisualStyle(family);
        ElementVisualization vis = style.getVisualization(type);
       
        if (vis == null) {
        	try {
            AcmeEnumeration superTypes = type.getSuperTypes().enumerate();
            boolean foundVis = false;
            while (superTypes.moreItems() && !foundVis) {
                AcmeElementType et = (AcmeElementType) superTypes.getNextItem();
                if (!visited.contains (et)) {
                	visited.add (et);
                	AcmeFamily parentFamily = (AcmeFamily )((AcmeElement )et).getParent ();
                	if (parentFamily != null)
                		vis = recurseGetVisualization(parentFamily, et, visited);
	    
	                if (vis != null)
	                    foundVis = true;
                }
            }
        	}
            catch (Exception e) {
            	Debug.diagnoseln (e);
            }
        }

        if (vis == null) {
            if (type instanceof AcmeComponentType) {
                vis =
                    _defaultVisualStyle.getVisualization(
                        AcmeComponentType.class.getName());
            } else if (type instanceof AcmeConnectorType) {
                vis =
                    _defaultVisualStyle.getVisualization(
                        AcmeConnectorType.class.getName());
            } else if (type instanceof AcmePortType) {
                vis =
                    _defaultVisualStyle.getVisualization(
                        AcmePortType.class.getName());
            } else if (type instanceof AcmeRoleType) {
                vis =
                    _defaultVisualStyle.getVisualization(
                        AcmeRoleType.class.getName());
            }
        }
        return vis;
    }

    public static void updateDefaultVisualizations() {
    	ElementVisualization oldVis =DEFAULT_COMPONENT_VISUALIZATION; 
        DEFAULT_COMPONENT_VISUALIZATION =
            (ComponentVisualization) VisPreferences.loadVisFromStore(
                ElementVisualization.DEFAULT_COMPONENT);
        oldVis.firePropertyChange(Visualization.PROP_VIS_CHANGED, DEFAULT_COMPONENT_VISUALIZATION);
        oldVis = DEFAULT_CONNECTOR_VISUALIZATION;
        DEFAULT_CONNECTOR_VISUALIZATION =
            (ConnectorVisualization) VisPreferences.loadVisFromStore(
                ElementVisualization.DEFAULT_CONNECTOR);
        oldVis.firePropertyChange(Visualization.PROP_VIS_CHANGED, DEFAULT_CONNECTOR_VISUALIZATION);
        oldVis = DEFAULT_PORT_VISUALIZATION;
        DEFAULT_PORT_VISUALIZATION =
            (PortVisualization) VisPreferences.loadVisFromStore(
                ElementVisualization.DEFAULT_PORT);
        oldVis.firePropertyChange(Visualization.PROP_VIS_CHANGED, DEFAULT_PORT_VISUALIZATION);
        oldVis = DEFAULT_ROLE_VISUALIZATION;
        DEFAULT_ROLE_VISUALIZATION =
            (RoleVisualization) VisPreferences.loadVisFromStore(
                ElementVisualization.DEFAULT_ROLE);
        oldVis.firePropertyChange(Visualization.PROP_VIS_CHANGED, DEFAULT_ROLE_VISUALIZATION);
        _defaultVisualStyle.add(
            AcmeComponentType.class.getName(),
            DEFAULT_COMPONENT_VISUALIZATION);
        _defaultVisualStyle.add(
            AcmeConnectorType.class.getName(),
            DEFAULT_CONNECTOR_VISUALIZATION);
        _defaultVisualStyle.add(
            AcmePortType.class.getName(),
            DEFAULT_PORT_VISUALIZATION);
        _defaultVisualStyle.add(
            AcmeRoleType.class.getName(),
            DEFAULT_ROLE_VISUALIZATION);
    }

	/**
	 * @param vis
	 * @return
	 */
	public static boolean isDefaultVisualization(ElementVisualization vis) {
		return vis == DEFAULT_COMPONENT_VISUALIZATION || 
			vis == DEFAULT_CONNECTOR_VISUALIZATION ||
			vis == DEFAULT_PORT_VISUALIZATION ||
			vis == DEFAULT_ROLE_VISUALIZATION;
	}

	/**
	 * @param element
	 * @return
	 */
	public static boolean hasSpecificVisualization(AcmeElement element) {
		VisualInformation visInfo = (VisualInformation )element.getUserData("vis-information");
		if (visInfo != null) {
			return (visInfo.getVisualization() != null);
		}
		return false;
	}

	/**
	 * @param element
	 * @param visualization
	 */
	public static void setVisualizationForElement(AcmeElement element, ElementVisualization visualization) {
		ElementVisualization existingVis = getVisualization(element);
		VisualInformation visInfo = Helper.getVisualInformation(element);
		if (visInfo.getVisualization() != null && visualization != null) {
			PropertyChangeListener [] oldListeners = visInfo.getVisualization().getListeners().getPropertyChangeListeners();
			for (int i = 0; i < oldListeners.length; i++) {
				visInfo.getVisualization().removePropertyChangeListener("", oldListeners [i]);
				visualization.addPropertyChangeListener("", oldListeners [i]);
			}
			
		}
		visInfo.setVisualization(visualization);
		if (visualization != null) visualization.addSuperVisualization(existingVis);
//		visualization.firePropertyChange(Visualization.PROP_VIS_CHANGED, visualization);
//		visInfo.getVisualization().firePropertyChange(Visualization.PROP_VIS_CHANGED, visInfo.getVisualization());
	}
	
	public static boolean isDefaultVisualStyle (VisualStyle style) {
		return style.equals (_defaultVisualStyle);
	}
}
