package AcmeStudio.ui.editors.diagram.styles;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

/**
 * @author Hong Yan (yh@cs.cmu.edu)
 *
 * The base class providing the visual styles for figures.
 */
public abstract class Visualization implements IPropertySource {

    public static final String PROP_VIS_CHANGED = "$VisChanged$";
    public static final String PROP_VIS_RESET = "$VisReset$";

    private PropertyChangeSupport _listeners =
        new PropertyChangeSupport(this);

    /** Whether the visualization properties are dirty (i.e., any change) */
    private boolean _isDirty = false;

    protected boolean _copying = false;

    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
    public IPropertyDescriptor[] getPropertyDescriptors() {
        return null;
    }

    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(Object)
     */
    public Object getPropertyValue(Object id) {
        return null;
    }

    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(Object)
     */
    public boolean isPropertySet(Object id) {
        return false;
    }

    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(Object)
     */
    public void resetPropertyValue(Object id) {
    }

    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(Object, Object)
     */
    public void setPropertyValue(Object id, Object value) {
    }

    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
     */
    public Object getEditableValue() {
        return this;
    }

    public void addPropertyChangeListener(
        String key,
        PropertyChangeListener l) {
//    	if (l != this)
        _listeners.addPropertyChangeListener(l);
    }

    public void removePropertyChangeListener(
        String key,
        PropertyChangeListener l) {
        _listeners.removePropertyChangeListener(l);
    }

    protected boolean _fireChanges = true;

    protected void firePropertyChange(String prop, Object newValue) {
        setDirty(true);
        if (!_fireChanges)
            return;
        if (!_copying) {
            _listeners.firePropertyChange(prop, null, newValue);
        }
    }

    protected void fireStructureChange(String prop, Object child) {
        setDirty(true);
        if (!_fireChanges)
            return;
        if (!_copying) {
            _listeners.firePropertyChange(prop, null, child);
        }
    }

    public void resetVisualization() {
        _listeners.firePropertyChange(PROP_VIS_RESET, null, this);
    }

    public void beginChangeBlock() {
        _fireChanges = false;
    }

    public void endChangeBlock() {
        _fireChanges = true;
        firePropertyChange(PROP_VIS_CHANGED, this);
    }

    /**
     * Returns whether this visualization has been changed
     * @return boolean  whether visualization has been changed
     */
    public boolean isDirty() {
        return _isDirty;
    }

    /**
     * Sets dirtiness
     * @param diry  dirtiness flag to set to
     */
    public void setDirty(boolean dirty) {
        _isDirty = dirty;
    }

	public PropertyChangeSupport getListeners() {
		return _listeners;
	}
}
