package AcmeStudio.ui.editors.diagram.templates;

import java.util.LinkedList;
import java.util.List;
import org.eclipse.core.runtime.*;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.dnd.TemplateTransfer;
import org.eclipse.gef.dnd.TemplateTransferDropTargetListener;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.IWorkbenchConstants;
import org.eclipse.ui.internal.WorkbenchPlugin;
import org.eclipse.ui.part.IDropActionDelegate;
import org.eclipse.ui.part.PluginDropAdapter;
import org.eclipse.ui.part.PluginTransferData;
import AcmeStudio.ui.editors.diagram.palette.extension.IPaletteExtender;
import AcmeStudio.ui.editors.diagram.parts.ComponentEditPart;
import AcmeStudio.ui.editors.diagram.parts.ConnectorEditPart;
import AcmeStudio.ui.editors.diagram.parts.ElementEditPart;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudio.ui.editors.diagram.styles.ComponentVisualization;
import AcmeStudio.ui.editors.diagram.styles.ConnectorVisualization;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.util.Debug;
import acmeLib.elements.*;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class SystemTemplateTransferDropTargetListener
	extends TemplateTransferDropTargetListener {

	public SystemTemplateTransferDropTargetListener(EditPartViewer viewer) {
		super(viewer);
	}

	protected boolean canDrop(Class clazz, Point p) {
		EditPart over = getViewer().findObjectAt(p);
		boolean droppable = false;
		boolean isPortOrRole = false;
		if (clazz.equals(AcmeRole.class)) {
			droppable = over instanceof ConnectorEditPart;
			isPortOrRole = true;
		} else if (clazz.equals(AcmePort.class)) {
			droppable = over instanceof ComponentEditPart;
			isPortOrRole = true;
		} else
			droppable = true;

		if (droppable && isPortOrRole) {
			droppable = false;
			ElementVisualization vis = ((ElementEditPart) over).getVis();
			droppable = isAllowedType(vis);
		}

		return droppable;

	}

	protected boolean canDrop(AcmeElementType type, Point p) {
		EditPart over = getViewer().findObjectAt(p);
		boolean droppable = false;
		boolean isPortOrRole = false;
		if (type instanceof AcmeRoleType && over instanceof ConnectorEditPart) {
			droppable = over instanceof ConnectorEditPart;
			isPortOrRole = true;
		} else if (type instanceof AcmePortType && over instanceof ComponentEditPart) {
			droppable = over instanceof ComponentEditPart;
			isPortOrRole = true;
		} else if ((type instanceof AcmeComponentType || type instanceof AcmeConnectorType) && over instanceof SystemEditPart) {
			droppable = over instanceof SystemEditPart;
		} else {
//			Debug.diagnoseln ("over = " + over.toString ());
			droppable = true;
		}
		if (droppable && isPortOrRole) {
			droppable = false;
			ElementVisualization vis = ((ElementEditPart) over).getVis();
			droppable = isAllowedType(type, vis);
		}
		return droppable;
	}

	private boolean isAllowedType(
		AcmeElementType acmeType,
		ElementVisualization vis) {
		boolean allowedType = true;
		if (acmeType instanceof AcmePortType
			&& vis instanceof ComponentVisualization) {
			ComponentVisualization cv = (ComponentVisualization) vis;
			if (cv.restrictPortTypes()) {
				allowedType = false;
				LinkedList ll = cv.getAllowedPortTypes();
				String portType = acmeType.getTypeName();
				if (ll.contains(portType)) {
					allowedType = true;
				}
			}
		} else if (
			acmeType instanceof AcmeRoleType
				&& vis instanceof ConnectorVisualization) {
			ConnectorVisualization cv = (ConnectorVisualization) vis;
			if (cv.restrictRoleTypes()) {
				allowedType = false;
				LinkedList ll = cv.getAllowedRoleTypes();
				String roleType = acmeType.getTypeName();
				if (ll.contains(roleType)) {
					allowedType = true;
				}
			}
		}

		return allowedType;
	}

	private boolean isAllowedType(ElementVisualization vis) {
		boolean allowedType = true;
		if (vis instanceof ComponentVisualization) {
			ComponentVisualization cv = (ComponentVisualization) vis;
			if (cv.restrictPortTypes()) {
				allowedType = false;
				LinkedList ll = cv.getAllowedPortTypes();
				String portType = "port";
				if (ll.contains(portType)) {
					allowedType = true;
				}
			}
		} else if (vis instanceof ConnectorVisualization) {
			ConnectorVisualization cv = (ConnectorVisualization) vis;
			if (cv.restrictRoleTypes()) {
				allowedType = false;
				LinkedList ll = cv.getAllowedRoleTypes();
				String roleType = "role";
				if (ll.contains(roleType)) {
					allowedType = true;
				}
			}
		}

		return allowedType;
	}

	/**
	 * @see org.eclipse.gef.dnd.TemplateTransferDropTargetListener#getFactory(Object)
	 */
	protected org.eclipse.gef.requests.CreationFactory getFactory(
		Object template) {
		Point p = getDropLocation();
		if (template instanceof AcmeElementType && shouldLink ((AcmeElementType )template, p)) {
			return new AssignTypeTemplateFactory ((AcmeElementType )template, getElementInDropLocation (p));
		}
		else if (template instanceof Class && canDrop((Class) template, p)) {

			return new SystemTemplateFactory((Class) template);
		} else if (
			template instanceof AcmeElementType
				&& canDrop((AcmeElementType) template, p)) {
			return new SystemTemplateFactory((AcmeElementType) template);
		}
		else {
//			 Try the extensions
			CreationFactory factory = null;
			IExtensionRegistry registry = Platform.getExtensionRegistry ();
			IExtensionPoint point = registry.getExtensionPoint ("AcmeStudio.paletteTabs");
			IExtension [] extensions = point.getExtensions ();
			for (int i = 0; i < extensions.length && factory == null; i++) {
				 IConfigurationElement [] elements = extensions [i].getConfigurationElements();
				 for (int j = 0; j < elements.length; j++) {
					if ("tabEntry".equals (elements [j].getName ())) {
						try {
							IPaletteExtender extender = (IPaletteExtender )elements [j].createExecutableExtension ("extenderClass");
							factory = extender.getCreationFactory(template);
						}
						catch (CoreException e) {
							Debug.errorln (e);
						}
					}
				}
			}
			return factory;
		}
	}

	/**
	 * @param type
	 * @param p
	 * @return
	 */
	private boolean shouldLink(AcmeElementType type, Point p) {
		AcmeElement element = getElementInDropLocation(p);
		boolean link = false;
		if (element != null) {
			link = ((element instanceof AcmeComponent && type instanceof AcmeComponentType) ||
					(element instanceof AcmeConnector && type instanceof AcmeConnectorType) ||
					(element instanceof AcmePort && type instanceof AcmePortType) ||
					(element instanceof AcmeRole && type instanceof AcmeRoleType));
			
		}
		return link;
	}

	/**
	 * @param p
	 * @return
	 */
	private AcmeElement getElementInDropLocation(Point p) {
		EditPart over = getViewer().findObjectAt(p);
		if (over instanceof ElementEditPart) {
			if (over.getModel () instanceof AcmeElement)
				return (AcmeElement )over.getModel ();
		}
		return null;
	}

	/**
	 * @see org.eclipse.gef.dnd.TransferDropTargetListener#isEnabled(DropTargetEvent)
	 */
	public boolean isEnabled(DropTargetEvent event) {
		org.eclipse.swt.graphics.Point p1 =
			getViewer().getControl().toControl(
				new org.eclipse.swt.graphics.Point(event.x, event.y));
		Point p = new Point(p1.x, p1.y);
		Object template =
			((TemplateTransfer) ((DropTarget) event.getSource())
				.getTransfer()[0])
				.getObject();
		boolean ok =
		template instanceof PluginTransferData
				|| (template instanceof Class && canDrop((Class) template, p))
				|| (template instanceof AcmeElementType && canDrop((AcmeElementType) template, p))
				|| (template instanceof AcmeElementType && shouldLink ((AcmeElementType )template, p));

		if (!ok) {
			// Try the extensions
			IExtensionRegistry registry = Platform.getExtensionRegistry ();
			IExtensionPoint point = registry.getExtensionPoint ("AcmeStudio.paletteTabs");
			IExtension [] extensions = point.getExtensions ();
			for (int i = 0; i < extensions.length && !ok; i++) {
				 IConfigurationElement [] elements = extensions [i].getConfigurationElements();
				 for (int j = 0; j < elements.length; j++) {
					if ("tabEntry".equals (elements [j].getName ())) {
						try {
							IPaletteExtender extender = (IPaletteExtender )elements [j].createExecutableExtension ("extenderClass");
							ok = extender.canDrop(template,  getViewer().findObjectAt(p));
						}
						catch (CoreException e) {
							Debug.errorln (e);
						}
					}
				}
			}
		}
		return ok && super.isEnabled(event);
	}

	public void drop(DropTargetEvent event) {
		try {
			if (event.data instanceof PluginTransferData) {
				PluginTransferData pluginData = (PluginTransferData) event.data;
				IDropActionDelegate delegate = getPluginAdapter(pluginData);
				EditPart over = getViewer().findObjectAt(getDropLocation());
				List list = new LinkedList();
				list.add(over);
				list.add(getDropLocation());
				if (!delegate.run(pluginData.getData(), list)) {
					event.detail = DND.DROP_NONE;
				}
				if (over instanceof ScalableFreeformRootEditPart) {
					ScalableFreeformRootEditPart sef = (ScalableFreeformRootEditPart )over;
					sef.getContents().refresh();
				}
				else 
					over.refresh();
			} else {
				super.drop(event);
			}
		} catch (CoreException e) {
			WorkbenchPlugin.log("Drop Failed", e.getStatus()); //$NON-NLS-1$
		}
	}

	/**
	 * Loads the class that will perform the action associated with the given drop
	 * data.
	 *
	 * @param data the drop data
	 * @return the viewer drop adapter
	 */
	protected static IDropActionDelegate getPluginAdapter(PluginTransferData data)
		throws CoreException {
	    IExtensionRegistry registry = Platform.getExtensionRegistry();
		String adapterName = data.getExtensionId();
		IExtensionPoint xpt =
			registry.getExtensionPoint(
				PlatformUI.PLUGIN_ID,
				IWorkbenchConstants.PL_DROP_ACTIONS);
		IExtension[] extensions = xpt.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] configs =
				extensions[i].getConfigurationElements();
			if (configs != null && configs.length > 0) {
				String id = configs[0].getAttribute("id"); //$NON-NLS-1$
				if (id != null && id.equals(adapterName)) {
					return (
						IDropActionDelegate) WorkbenchPlugin.createExtension(
						configs[0],
						PluginDropAdapter.ATT_CLASS);
				}
			}
		}
		return null;
	}

	protected Request createTargetRequest() {
		//Look at the data on templatetransfer.
		//Create factory
		CreateRequest request = new CreateRequest();
		Object template = TemplateTransfer.getInstance().getTemplate();
		request.setFactory(getFactory(template));
		return request;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.dnd.AbstractTransferDropTargetListener#handleDragOver()
	 */
	protected void handleDragOver() {
//		DropTargetEvent event = getCurrentEvent();
//		org.eclipse.swt.graphics.Point p1 =
//			getViewer().getControl().toControl(
//				new org.eclipse.swt.graphics.Point(event.x, event.y));
//		Point p = new Point(p1.x, p1.y);
//		Object template =
//			((TemplateTransfer) ((DropTarget) event.getSource())
//				.getTransfer()[0])
//				.getObject();
//		EditPart over = getViewer ().findObjectAt(p);
//		event.detail = DND.DROP_MOVE;
//		if (over instanceof ComponentEditPart && (template instanceof AcmeComponentType || (template instanceof Class && AcmeComponent.class.equals (template)))) {
//			event.detail = DND.DROP_LINK;
//		}
//		
		
		super.handleDragOver();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.dnd.AbstractTransferDropTargetListener#getTargetRequest()
	 */
	protected Request getTargetRequest() {
		return super.getTargetRequest();
	}

}
