/*
 * Created on Mar 31, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.visualInfo;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;
import acmeLib.properties.AcmeProperty;

/**
 * @author Bradley Schmerl
 * Created Mar 31, 2004
 */
public class RoleVisualInformation extends VisualInformation {
	
	PointList _bendPoints;
	/**
	 * 
	 */
	public RoleVisualInformation(VisualInformation vis) {
		super();
		if (vis != null) {
			this.setHeight(vis.getHeight ());
			this.setWidth (vis.getWidth ());
			this.setX (vis.getX ());
			this.setY (vis.getY ());
		}
	}
	/**
	 * @return Returns the bendPoints.
	 */
	public PointList getBendPoints() {
		if (_bendPoints == null) return null;
		return _bendPoints.getCopy ();
	}
	/**
	 * @param bendPoints The bendPoints to set.
	 */
	public void setBendPoints(PointList bendPoints) {
		if (bendPoints == null || bendPoints.size() == 0) 
			_bendPoints = null;
		else
			_bendPoints = bendPoints.getCopy ();
	}
	/* (non-Javadoc)
	 * @see acmeLib.core.extensions.IElementExtension#extractFromElement(acmeLib.elements.AcmeElement)
	 */
	public void extractFromElement(AcmeElement element) {
		super.extractFromElement(element);
		AcmeProperty visB = element.getProperty ("vis-bendpoints");
		if (visB != null) {
			setBendPoints(Helper.convertBendpointPropertyToPointList (visB));
			element.removeProperty(visB);
		}
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.util.VisualInformation#exportAsXMLElement(org.w3c.dom.Document)
	 */
	public Element exportAsXMLElement(Document doc) {
		Element elem = super.exportAsXMLElement(doc);
		if (_bendPoints != null) {
			elem.setAttribute ("type","ROLE");
			Element bendpointElem = doc.createElement("bendpoints");
			for (int i = 0; i < _bendPoints.size(); i++) {
				Point bp = _bendPoints.getPoint(i);
				Element pointElem = doc.createElement ("point");
				pointElem.setAttribute ("x", Integer.toString(bp.x));
				pointElem.setAttribute ("y", Integer.toString(bp.y));
				bendpointElem.appendChild(pointElem);
			}
			elem.appendChild (bendpointElem);
		}
		return elem;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.util.VisualInformation#importFromXML(org.w3c.dom.Element)
	 */
	public void importFromXML(Element element) {
		super.importFromXML(element);
		NodeList bps = element.getElementsByTagName("bendpoints");
		if (bps.getLength() > 0) {
			_bendPoints = new PointList ();
			Element bpE = (Element )bps.item(0);
			NodeList points = bpE.getElementsByTagName("point");
			for (int i = 0; i < points.getLength(); i++) {
				Element point = (Element )points.item (i);
				String x = point.getAttribute ("x");
				String y = point.getAttribute ("y");
				Point p = new Point ();
				if (x != "") p.x = Integer.parseInt (x);
				if (y != "") p.y = Integer.parseInt (y);
				_bendPoints.addPoint(p);
			}
		}
	}
}
