/*
 * Created on Mar 31, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.visualInfo;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.core.resource.datapersistence.IDataPersistence;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;
import acmeLib.tools.GraphWalker;
import acmeLib.tools.WalkerElementOperation;

/**
 * @author Bradley Schmerl
 * Created Mar 31, 2004
 */
public class VisualInfoPersistor implements IDataPersistence {
    
    /**
     * Default constructor
     */
    public VisualInfoPersistor() {
        super();
    }
    
    /* (non-Javadoc)
     * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#load(AcmeStudio.core.resource.ASDesign, java.io.File)
     */
    public void load(ASDesign design, File resource) {
        String acmeFilename = resource.getAbsolutePath ();
        String visFilename = null;
        if (acmeFilename.endsWith (".acme")) {
            visFilename = generateVisFilename(design, acmeFilename);
            File visFile = new File (visFilename);
            if (!visFile.exists ()) {
                // need to convert from the old format
                if (AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
                    importVisInfoFromProperties (design.getDesign());
                }
            }
            else {
                try {
                    DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
                    Document doc = docBuilder.parse(visFile);
                    importVisualInfo (design.getDesign(), doc.getDocumentElement());
                } catch (ParserConfigurationException e) {
                    e.printStackTrace();
                } catch (FactoryConfigurationError e) {
                    e.printStackTrace();
                } catch (SAXException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
    /**
     * @param design
     */
    private void importVisInfoFromProperties(AcmeDesign design) {
        WalkerElementOperation conversion = new WalkerElementOperation () {
            
            public void doElementOp(AcmeElement elt) {
                VisualInformation vis = new VisualInformation ();
                vis.extractFromElement(elt);
                if (elt instanceof AcmeRole) {
                    vis = new RoleVisualInformation (vis);
                    vis.extractFromElement(elt);
                }
                else if (elt instanceof AcmePort) {
                    vis = new PortVisualInformation (vis);
                    vis.extractFromElement(elt);
                }
                
            }
            
        };
        GraphWalker.WalkOverAllElements(design, conversion, GraphWalker.WALK_TOP_DOWN);
        
        
    }
    
    /* (non-Javadoc)
     * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#save(AcmeStudio.core.resource.ASDesign, java.io.File)
     */
    public void save(ASDesign design, File resource) {
    	String acmeFilename = design.getCanonicalPath();
        String visFilename = null;
        if (acmeFilename.endsWith (".acme")) {
            visFilename = generateVisFilename(design, acmeFilename);
            
            try {
                
                DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
                Document doc = docBuilder.newDocument ();
                Element elem = doc.createElement ("visualInformation");
                elem.setAttribute ("associatedAcmeFile", resource.getAbsolutePath());
                VisualizationSaveWalkerOp walker = new VisualizationSaveWalkerOp (elem, doc);
                GraphWalker.WalkOverAllElements(design.getDesign(), walker, GraphWalker.WALK_TOP_DOWN);
                
                StringWriter sw = new StringWriter ();
                
                TransformerFactory tFactory = TransformerFactory.newInstance();
                Transformer transformer = tFactory.newTransformer();
                DOMSource source = new DOMSource (elem);
                StreamResult result = new StreamResult (sw);
                transformer.transform (source, result);
                sw.close();
                File file = new File (visFilename);
                FileOutputStream fos = new FileOutputStream (file);
                fos.write (sw.toString ().getBytes ());
                fos.close ();
                
            } catch (DOMException e) {
                e.printStackTrace();
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (ParserConfigurationException e) {
                e.printStackTrace();
            } catch (FactoryConfigurationError e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (TransformerConfigurationException e) {
                e.printStackTrace();
            } catch (TransformerException e) {
                e.printStackTrace();
            }
        }
    }
    
    /* (non-Javadoc)
     * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#extractData(AcmeStudio.core.resource.ASDesign)
     */
    public Object extractData(ASDesign design) {
        final Map visMap = new HashMap ();
        WalkerElementOperation extractOp = new WalkerElementOperation () {
            
            public void doElementOp(AcmeElement elt) {
                VisualInformation visInfo = (VisualInformation )elt.getUserData ("vis-information");
                visMap.put (elt.getQualifiedName(), visInfo);
            }
            
        };
        
        GraphWalker.WalkOverAllElements (design.getDesign(), extractOp, GraphWalker.WALK_TOP_DOWN);
        return visMap;
    }
    
    /* (non-Javadoc)
     * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#injectData(AcmeStudio.core.resource.ASDesign, java.lang.Object)
     */
    public void injectData(ASDesign design, Object data) {
        Map visMap = (Map )data;
        for (Iterator iter = visMap.entrySet ().iterator (); iter.hasNext ();) {
            Map.Entry entry = (Map.Entry )iter.next ();
            AcmeElement element = (AcmeElement )design.getDesign().lookupQualifiedName((String )entry.getKey());
            if (element !=null) 
                element.setUserData ("vis-information", (VisualInformation )entry.getValue());
        }
    }
    
    /**
     * @param design
     * @param element
     */
    public static void importVisualInfo(AcmeDesign design, Element element) {
        NodeList nodes = element.getElementsByTagName("visualization");
        for (int i = 0; i < nodes.getLength(); i++) {
            Element visElement = (Element )nodes.item(i);
            VisualInformation visInfo = null;
            if (visElement.getAttribute("type").equals ("PORT")) {
                visInfo = new PortVisualInformation (null);
            }
            else if (visElement.getAttribute ("type").equals ("ROLE")) {
                visInfo = new RoleVisualInformation (null);
            }
            else {
                visInfo = new VisualInformation ();
            }
            String associatedElement = visElement.getAttribute("element");
            if (associatedElement.equals ("")) continue;
            visInfo.importFromXML (visElement);
            AcmeElement acmeElement = (AcmeElement) design.lookupQualifiedName(associatedElement);
            if (acmeElement != null) {
                acmeElement.setUserData("vis-information", visInfo);
                if (visInfo.getVisualization() != null) {
                    VisualStyleRegistry.setVisualizationForElement(acmeElement, visInfo.getVisualization ());
                }
            }
        }
        nodes = element.getElementsByTagName("portConnectionRules");
        if (nodes.getLength () == 1) {
            Element elem = (Element )nodes.item (0);
            ConnectionPatternInformation info = ConnectionPatternInformation.importFromXMLElement (elem);
            String associatedSystem = elem.getAttribute("element");
            AcmeSystem system = (AcmeSystem )design.lookupQualifiedName(associatedSystem);
            system.setUserData (ConnectionPatternInformation.CONNECTION_PATTERNS_KEY, info);
        }
    }
    
    private String generateVisFilename(ASDesign design, String acmeFilename) {
        IPath dataPath = null;
        try {
            dataPath = design.getDataDirectory();
        }
        catch (IOException e) {
            e.printStackTrace();
            return acmeFilename.substring(0, acmeFilename.length () - 5) + ".visinfo";
        }
        Path acmeFilePath = new Path (acmeFilename);
        String filename = acmeFilePath.lastSegment();
        dataPath = dataPath.append(filename.substring(0, filename.length () - 5) + ".visinfo");
        return dataPath.toString ();
    }
    
}
