/*
 * Created on Mar 16, 2004
 *
 */
package AcmeStudio.ui.editors.diagram.visualInfo;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import acmeLib.core.extensions.IElementExtension;
import acmeLib.elements.AcmeElement;
import acmeLib.properties.AcmeFloat;
import acmeLib.properties.AcmeInt;
import acmeLib.properties.AcmeProperty;


/**
 * @author Bradley Schmerl
 * Created Mar 16, 2004
 */
public class VisualInformation implements IElementExtension, PropertyChangeListener {
	
	public static final int NO_POS_VALUE = Integer.MAX_VALUE; 
	
	protected static int getVisualValue(AcmeProperty visProp) {
		if (visProp.getValue () instanceof AcmeFloat)
			return (int )((AcmeFloat )visProp.getValue ()).getValue ();
		else if (visProp.getValue () instanceof AcmeInt) {
			return ((AcmeInt )visProp.getValue ()).getValue ();
		}
		return Integer.MAX_VALUE;
	}

	protected int _height = -1;
	protected int _width= -1;
	protected int _x = NO_POS_VALUE;
	protected int _y = NO_POS_VALUE;
	
	/** The element specific visualization for an element **/
	protected ElementVisualization _vis = null;
	private ElementVisualization	_origVis;
	private int	_zOrder = NO_POS_VALUE;
	
	/**
	 * 
	 */
	public VisualInformation() {
		super ();
	}
	/**
	 * @param _doc
	 * @return
	 */
	public Element exportAsXMLElement(Document doc) {
		Element element = doc.createElement("visualization");
		if (_width > 0) element.setAttribute("width", Integer.toString(_width));
		if (_height > 0) element.setAttribute ("height", Integer.toString(_height));
		if (_x != NO_POS_VALUE) element.setAttribute ("x", Integer.toString (_x));
		if (_y != NO_POS_VALUE) element.setAttribute ("y", Integer.toString (_y));
		if (_zOrder != NO_POS_VALUE) 
			element.setAttribute ("zOrder", Integer.toString(_zOrder));
		if (_vis != null) {
			Element visElement = _vis.export(doc);
			element.appendChild(visElement);
		}
		return element;
		
	}
	/* (non-Javadoc)
	 * @see acmeLib.core.extensions.IElementExtension#extractFromElement(acmeLib.elements.AcmeElement)
	 */
	public void extractFromElement(AcmeElement element) {
		AcmeProperty visX = element.getProperty ("vis-x");
		AcmeProperty visY = element.getProperty ("vis-y");
		AcmeProperty visH = element.getProperty ("vis-height");
		AcmeProperty visW = element.getProperty ("vis-width");
		element.setUserData ("vis-information", this);
		if (visX != null) {
			setX (getVisualValue(visX));
			element.removeProperty(visX);
		}
		if (visY != null) {
			setY (getVisualValue(visY));
			element.removeProperty (visY);
		}
		if (visH != null) {
			setHeight (getVisualValue(visH));
			element.removeProperty (visH);
		}
		if (visW != null)  {
			setWidth (getVisualValue(visW));
			element.removeProperty (visW);
		}
	}
	
	
	
	/**
	 * @return Returns the height.
	 */
	public int getHeight() {
		return _height;
	}
	
	public Point getLocation () {
		if (_x != NO_POS_VALUE && _y != NO_POS_VALUE)
			return new Point (_x, _y);
		else
			return new Point ();
	}
	/**
	 * 
	 */
	public Dimension getSize() {
		return new Dimension (_width, _height);
	}
	/**
	 * @return Returns the width.
	 */
	public int getWidth() {
		return _width;
	}
	/**
	 * @return Returns the x.
	 */
	public int getX() {
		return _x;
	}
	/**
	 * @return Returns the y.
	 */
	public int getY() {
		return _y;
	}
	
	public ElementVisualization getVisualization () {
		return _vis;
	}
	
	/**
	 * @param element
	 */
	public void importFromXML(Element element) {
		String width = element.getAttribute("width");
		String height = element.getAttribute("height");
		String x = element.getAttribute ("x");
		String y = element.getAttribute ("y");
		String zOrder = element.getAttribute ("zOrder");
	
		if (width != "") setWidth (Integer.parseInt (width));
		if (height != "") setHeight (Integer.parseInt (height));
		if (x != "") setX (Integer.parseInt (x));
		if (y != "") setY (Integer.parseInt (y));
		if (zOrder != "") setZOrder (Integer.parseInt(zOrder));
		
		NodeList visNodes = element.getElementsByTagName("visualization");
		if (visNodes.getLength() > 0) {
			Element vis = (Element )visNodes.item(0);
			_vis = ElementVisualization.importVisualization(vis);
		}
	}
	/* (non-Javadoc)
	 * @see acmeLib.core.extensions.IElementExtension#injectIntoElement(acmeLib.elements.AcmeElement)
	 */
	public void injectIntoElement(AcmeElement element) {
	}
	/* (non-Javadoc)
	 * @see acmeLib.core.extensions.IElementExtension#isInjected(acmeLib.elements.AcmeElement)
	 */
	public boolean isInjected(AcmeElement element) {
		return false;
	}
	/* (non-Javadoc)
	 * @see acmeLib.core.extensions.IElementExtension#isPersistent()
	 */
	public boolean isPersistent() {
		return false;
	}
	/**
	 * @param height The height to set.
	 */
	public void setHeight(int height) {
		_height = height;
	}
	/**
	 * @param width The width to set.
	 */
	public void setWidth(int width) {
		_width = width;
	}
	/**
	 * @param x The x to set.
	 */
	public void setX(int x) {
		_x = x;
	}
	/**
	 * @param y The y to set.
	 */
	public void setY(int y) {
		_y = y;
	}
	
	public void setVisualization (ElementVisualization vis) {
		setVisualization (vis, true, true);
	}
	
	public void setVisualization (ElementVisualization vis, boolean copy, boolean listenToOriginal) {
		if (copy) {
			if (_origVis != null && vis != null) {
				_origVis.removePropertyChangeListener("copy", this);
				_vis.dispose ();
			}
			if (vis != null) _vis = vis.copy ();
			else _vis = vis;
		}
		else 
			_vis = vis;
		
		if (copy && listenToOriginal) {
			_origVis = vis;
			if (vis != null) vis.addPropertyChangeListener("copy", this);
		}
		else {
			_origVis = null;
		}
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals (ElementVisualization.PROP_VIS_CHANGED)) {
//			_vis.blendVisualization(_origVis, true);
//			_vis.firePropertyChange(evt.getPropertyName(), evt.getNewValue());
		}
	}
	/**
	 * @return
	 */
	public int getZOrder() {
		return _zOrder;
	}
	/**
	 * @param i
	 */
	public void setZOrder(int i) {
		_zOrder = i;
	}
}
