package AcmeStudio.ui.editors.style;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.part.EditorActionBarContributor;

import AcmeStudio.core.actions.AcmeStudioRetargetAction;
import AcmeStudio.ui.editors.AcmeStyleEditor;
import AcmeStudio.ui.editors.actions.NewDesignAnalysisAction;
import AcmeStudio.ui.editors.actions.NewElementTypeAction;
import AcmeStudio.ui.editors.actions.NewPropertyTypeAction;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class StyleEditorActionBarContributor
    extends EditorActionBarContributor {

    private AcmeStudioRetargetAction _newComponentTypeAction;
    private AcmeStudioRetargetAction _newConnectorTypeAction;
    private AcmeStudioRetargetAction _newPortTypeAction;
    private AcmeStudioRetargetAction _newRoleTypeAction;
	private AcmeStudioRetargetAction _newPropertyTypeAction;
	private AcmeStudioRetargetAction _newDesignAnalysisAction;


    public StyleEditorActionBarContributor () {
        super ();
        
        _newComponentTypeAction =
            new AcmeStudioRetargetAction(
                "AcmeStudio.design.newComponentType",
                "New Component Type");
        _newConnectorTypeAction =
            new AcmeStudioRetargetAction(
                "AcmeStudio.design.newConnectorType",
                "New Connector Type");
        _newPortTypeAction =
            new AcmeStudioRetargetAction(NewElementTypeAction.NEW_PORT_TYPE, "New Port Type");
        _newRoleTypeAction =
            new AcmeStudioRetargetAction(NewElementTypeAction.NEW_ROLE_TYPE, "New Role Type");
        _newPropertyTypeAction = 
            new AcmeStudioRetargetAction (NewPropertyTypeAction.ID, "New Property Type");
        _newDesignAnalysisAction = new AcmeStudioRetargetAction (NewDesignAnalysisAction.ID, "New Function");

    }
    /**
     * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToMenu(IMenuManager)
     */
    public void contributeToMenu(IMenuManager menuManager) {
        super.contributeToMenu(menuManager);
        IMenuManager familyMenu = menuManager.findMenuUsingPath("AcmeStudio.family");
        // Change to family menu, and factor out types into another contributor
        familyMenu.appendToGroup("types", _newComponentTypeAction);
        familyMenu.appendToGroup("types", _newConnectorTypeAction);
        familyMenu.appendToGroup("types", _newPortTypeAction);
        familyMenu.appendToGroup("types", _newRoleTypeAction);
        familyMenu.appendToGroup("types", _newPropertyTypeAction);
        familyMenu.appendToGroup("types", _newDesignAnalysisAction);
    }

    /**
     * @see org.eclipse.ui.IEditorActionBarContributor#setActiveEditor(IEditorPart)
     */
    public void setActiveEditor(IEditorPart targetEditor) {
        if (targetEditor instanceof AcmeStyleEditor) {           
            super.setActiveEditor(targetEditor);
            AcmeStyleEditor editor = (AcmeStyleEditor)targetEditor;
//            IEditorPart styleEditor = editor.getStyleEditor();
            _newComponentTypeAction.setAction(editor.getAction (NewElementTypeAction.NEW_COMPONENT_TYPE));
            _newConnectorTypeAction.setAction(editor.getAction (NewElementTypeAction.NEW_CONNECTOR_TYPE));
            _newPortTypeAction.setAction(editor.getAction (NewElementTypeAction.NEW_PORT_TYPE));
            _newRoleTypeAction.setAction(editor.getAction (NewElementTypeAction.NEW_ROLE_TYPE));
            _newPropertyTypeAction.setAction (editor.getAction (NewPropertyTypeAction.ID));
            _newDesignAnalysisAction.setAction (editor.getAction (NewDesignAnalysisAction.ID));
        }
        else {
            _newComponentTypeAction.setAction(null);
            _newConnectorTypeAction.setAction(null);
            _newPortTypeAction.setAction(null);
            _newRoleTypeAction.setAction(null);
            _newPropertyTypeAction.setAction (null);
            _newDesignAnalysisAction.setAction (null);
        }
            
    }

}
