package AcmeStudio.ui.editors.style.parts;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.Collator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.dialogs.ListSelectionDialog;
import org.eclipse.ui.internal.WorkbenchImages;
import org.eclipse.ui.texteditor.StatusLineContributionItem;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.actions.OpenTypeHierarchyAction;
import AcmeStudio.model.ModelHelper;
import AcmeStudio.model.TypeAssignmentCommand;
import AcmeStudio.model.TypeCommand;
import AcmeStudio.ui.editors.AcmeStyleEditor;
import AcmeStudio.ui.editors.actions.NewElementTypeAction;
import AcmeStudio.ui.editors.actions.SuperTypeAssignmentMenuAction;
import AcmeStudio.ui.editors.diagram.styles.EditVisualizationDialog;
import AcmeStudio.ui.editors.diagram.styles.ElementVisualization;
import AcmeStudio.ui.editors.diagram.styles.VisualStyle;
import AcmeStudio.ui.editors.diagram.styles.VisualStyleRegistry;
import AcmeStudio.ui.views.providers.AcmeElementContentProvider;
import AcmeStudio.ui.views.providers.AcmeViewLabelProvider;
import AcmeStudio.util.*;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import AcmeStudio.util.HoverHelp;
import AcmeStudio.util.ITipInformer;
import acmeLib.AcmeObject;
import acmeLib.elements.*;
/**
 * @author Yan Hong (yh@cs.cmu.edu)
 *
 * Lists all the element styles a table displaying the
 * type, visualization and description. Allows creating
 * and modifying visual styles.
 */
public class ElementTypeList extends StyleList implements PropertyChangeListener,IManagedImageListener {

	private Set _visListeningTo = new HashSet ();
	
	/**
	 * 
	 * @see org.eclipse.swt.widgets.Widget#dispose()
	 **/
	public void dispose() {
		super.dispose ();
		for (Iterator iter = _visListeningTo.iterator (); iter.hasNext ();) {
			ElementVisualization vis = (ElementVisualization )iter.next ();
			vis.removePropertyChangeListener(null, this);
		}
	}
	/**
	 * @author Bradley Schmerl
	 * Created Mar 8, 2004
	 */
	public class ElementTypeSorter extends ViewerSorter {

		/**
		 * 
		 */
		public ElementTypeSorter() {
			super ();
		}

		/**
		 * @param collator
		 */
		public ElementTypeSorter(Collator collator) {
			super (collator);
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ViewerSorter#compare(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public int compare(Viewer viewer, Object e1, Object e2) {
			if (e1 instanceof AcmeStyleHolder && e2 instanceof AcmeStyleHolder) {
				AcmeStyleHolder s1 = (AcmeStyleHolder )e1;
				AcmeStyleHolder s2 = (AcmeStyleHolder )e2;
				return String.CASE_INSENSITIVE_ORDER.compare (s1.getType().getTypeName(), s2.getType ().getTypeName ());
			}
			return super.compare(viewer, e1, e2);
		}
	}
	/** 
	 * Holder of the element type.
	 */
	private class AcmeStyleHolder implements ITipInformer {
		/**The element type being held by the holder**/
		private AcmeElementType _type = null;
		/**
		 * Returns the element type held by this holder
		 * @return the element type
		 */
		public AcmeElementType getType() {
			return _type;
		}
		/**
		 * Returns the element type held by this holder
		 * @return the element type
		 */
		public void setType(AcmeElementType type) {
			_type = type;
		}
		/* (non-Javadoc)
		 * @see AcmeStudio.util.ITipInformer#getTipInformation()
		 */
		public String getTipInformation() {
			return null;
		}
		/* (non-Javadoc)
		 * @see AcmeStudio.util.ITipInformer#getTipImage()
		 */
		public Image getTipImage() {
			return getPreviewImage(_type, false);
		}
		/* (non-Javadoc)
		 * @see AcmeStudio.util.ITipInformer#hasTipInformation()
		 */
		public boolean hasTipInformation() {
			return true;
		}
	}

	// Constants for organizing the table
	private static final int TYPECOL = 1;
	private static final int TYPEWIDTH = 120;
	private static final int VISCOL = 2;
	private static final int VISWIDTH = 60;
	private static final int DESCCOL = 3;
	private static final int DESCWIDTH = 340;
	private static final int PALETTECOL = 0;
	private static final int PALETTEWIDTH = 80;
	private static final String[] PROPERTIES =
		{ "Visualization", "Type", "Description", "Palette" };
	private IContributionItem[] _myContributions;
	// Constants for style editing preference
	private static final int TO_CREATE = 1;
	/** The currently selected property **/
	private AcmeStyleHolder _selectedStyle = null;
	/** The category of this list, possible values are:
	 * Component,
	 * Connector,
	 * Port,
	 * Role
	 */
	private Class _category = null;
	/** The acme types represented by this list **/
	private Vector _acmeTypes = null;
	/** The action performed on double click **/
	private Action _doubleClickAction;
	/** Action to create a new type in this list **/
	private IAction _newTypeAction;
	/** The delete type action **/
	private Action _deleteTypeAction;
	/** The action to browse the hierarchy of a selected type **/
	private Action _browseTypeAction;
	private Action	_changePaletteVisibilityAction;

	/*
	 * The content provider class is responsible for
	 * providing objects to the view. It returns all the 
	 * properties to be displayed in the viewer.
	 */
	private class ViewContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		/**
		 * Returns all the types of the family.
		 * @param parent The AcmeFamily
		 * @return The types
		 */
		public Object[] getElements(Object parent) {
			int size = 0;
			AcmeElementType[] types = null;
			_acmeTypes = Helper.getTypes(_acmeFamily);
			if (_category == AcmeComponent.class) {
				size = _acmeTypes.size();
				types = new AcmeElementType[size];
				int count = 0;
				for (int i = 0; i < size; i++) {
					Object acmeType = _acmeTypes.get(i);
					if (acmeType instanceof AcmeComponentType) {
						types[count++] = (AcmeComponentType) acmeType;
					}
				}
				size = count;
			} else if (_category == AcmeConnector.class) {
				size = _acmeTypes.size();
				types = new AcmeElementType[size];
				int count = 0;
				for (int i = 0; i < size; i++) {
					Object acmeType = _acmeTypes.get(i);
					if (acmeType instanceof AcmeConnectorType) {
						types[count++] = (AcmeConnectorType) acmeType;
					}
				}
				size = count;
			} else if (_category == AcmePort.class) {
				size = _acmeTypes.size();
				types = new AcmeElementType[size];
				int count = 0;
				for (int i = 0; i < size; i++) {
					Object acmeType = _acmeTypes.get(i);
					if (acmeType instanceof AcmePortType) {
						types[count++] = (AcmePortType) acmeType;
					}
				}
				size = count;
			} else if (_category == AcmeRole.class) {
				size = _acmeTypes.size();
				types = new AcmeElementType[size];
				int count = 0;
				for (int i = 0; i < size; i++) {
					Object acmeType = _acmeTypes.get(i);
					if (acmeType instanceof AcmeRoleType) {
						types[count++] = (AcmeRoleType) acmeType;
					}
				}
				size = count;
			}
			Object[] elements = null;
			if (types != null) {
				// Create the elements array out of the enumeration
				elements = new Object[size];
				for (int i = 0; i < size; i++) {
					AcmeStyleHolder holder = new AcmeStyleHolder();
					holder.setType(types[i]);
					elements[i] = holder;
				}
			}
			return elements;
		}
	}
	/** 
	 * Provides routines for getting the text in each of the columns of the table
	 */
	private class ViewLabelProvider
		extends LabelProvider
		implements ITableLabelProvider {
		/**
		 * Returns the text associated with a particular column in the table
		 * @param obj the type to get the information from
		 * @param index the column to return the string for
		 * @return the text to be placed in the column
		 */
		public String getColumnText(Object obj, int index) {
			AcmeStyleHolder acmeStyle = (AcmeStyleHolder) obj;
			String retStr = " ";
			switch (index) {
				case TYPECOL :
					retStr = acmeStyle.getType().getTypeName();
					if (retStr == null) {
						retStr = "<<No Name>>";
					}
					break;
				case VISCOL :
					retStr = "";
					break;
				case DESCCOL :
					retStr =
						Helper.getDescription(
							(AcmeElement) acmeStyle.getType());
					//                    if (retStr.equals("")) {
					//                        Helper.setDescription(
					//                            (AcmeElement) acmeStyle.getType(),
					//                            "test");
					//                        retStr =
					//                            Helper.getDescription(
					//                                (AcmeElement) acmeStyle.getType());
					//                    }
					break;
				case PALETTECOL :
					ElementVisualization vis =
						VisualStyleRegistry.getVisualization(
							acmeStyle._type);
					if (vis.isHiddenFromPalette()) {
						retStr = "Hidden";
					} else
						retStr = "Shown";

					break;
			}
			return retStr;
		}
		/** 
		 * As there are no images associated with the table, just
		 * return null in these routines.
		 */
		public Image getColumnImage(Object obj, int index) {
			AcmeStyleHolder holder = (AcmeStyleHolder) obj;
			Image icon = null;
			switch (index) {
				case VISCOL :
					AcmeElementType type = holder.getType();
					icon = getPreviewImage(type, true);
					if (icon != null && icon.isDisposed()) return null;
					break;
					//                case PALETTECOL:
			}
			return icon;
		}
		public Image getImage(Object obj) {
			return null;
		}
	}
	/**
	 * Constructor for ElementTypeList. Constructs the Style table and adds
	 * it to the appropriate editor part
	 * @param parent the Composite in which to place the table
	 * @param style the style of the table
	 * @param category the category of this list
	 * @param parentEditror the parent style editor
	 */
	public ElementTypeList(
		Composite parent,
		int style,
		Class category,
		final AcmeStyleEditor parentEditor) {
		super(parent, style, parentEditor);

		_category = category;
		FillLayout tableLayout = new FillLayout();
		this.setLayout(tableLayout);
		this.setLayoutData(
			new GridData(
				GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_FILL));
		// Create the table and set it's contents to the active design by default
		_viewer =
			new TableViewer(
				this,
				SWT.BORDER
					| SWT.MULTI
					| SWT.H_SCROLL
					| SWT.V_SCROLL
					| SWT.FULL_SELECTION);

		_viewer.setContentProvider(new ViewContentProvider());
		_viewer.setLabelProvider(new ViewLabelProvider());
		_viewer.setSorter(new ElementTypeSorter ());
		//        _viewer.addSelectionChangedListener(this);
		// Set the column header and widthe information
		final Table table = _viewer.getTable();
		table.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent evt) {
				if (evt.character == SWT.DEL) {
					_deleteTypeAction.run();
				}
			}
			public void keyReleased(KeyEvent evt) {
			}
		});
		
		table.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected (SelectionEvent evt) {
				if (table.getSelectionCount() > 0) {
					TableItem item = table.getSelection() [0];
					AcmeStyleHolder holder = (AcmeStyleHolder )item.getData ();
					parentEditor.dispatchNewSelection(holder.getType ());
				}
			}
		});
		
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		TableColumn palCol = new TableColumn(table, SWT.CENTER, PALETTECOL);
		TableColumn typeCol = new TableColumn(table, SWT.LEFT, TYPECOL);
		TableColumn visCol = new TableColumn(table, SWT.LEFT, VISCOL);
		TableColumn descCol = new TableColumn(table, SWT.LEFT, DESCCOL);
		palCol.setText("Visibility");
		typeCol.setText("Type");
		visCol.setText("Visualization");
		descCol.setText("Description");
		typeCol.addControlListener(this);
		visCol.addControlListener(this);
		descCol.addControlListener(this);
		palCol.addControlListener(this);
		ColumnLayoutData typeLayout = new ColumnPixelData(TYPEWIDTH, true);
		ColumnLayoutData visLayout = new ColumnPixelData(VISWIDTH, true);
		ColumnLayoutData descLayout = new ColumnPixelData(DESCWIDTH, true);
		ColumnLayoutData palLayout = new ColumnPixelData(PALETTEWIDTH, false);
		// set columns in Table layout
		TableLayout layout = new TableLayout();
		layout.addColumnData(palLayout);
		layout.addColumnData(typeLayout);
		layout.addColumnData(visLayout);
		layout.addColumnData(descLayout);

		table.setLayout(layout);
		// Add actions
		_doubleClickAction = new Action() {
			public void run() {
				handleDoubleClick();
			}
		};
		_doubleClickAction.setText ("Change visualization...");
		_doubleClickAction.setDescription ("Change visualization for type");
		hookDoubleClickAction();
		if (_category == AcmeComponent.class) {
			_newTypeAction =
				_parentEditor.getAction(
					NewElementTypeAction.NEW_COMPONENT_TYPE);
		} else if (_category == AcmeConnector.class) {
			_newTypeAction =
				_parentEditor.getAction(
					NewElementTypeAction.NEW_CONNECTOR_TYPE);
		} else if (_category == AcmeRole.class) {
			_newTypeAction =
				_parentEditor.getAction(NewElementTypeAction.NEW_ROLE_TYPE);
		} else if (_category == AcmePort.class) {
			_newTypeAction =
				_parentEditor.getAction(NewElementTypeAction.NEW_PORT_TYPE);
		}
		makeActions();
		_myContributions =
			new ActionContributionItem[] {
				new ActionContributionItem(_newTypeAction),
				new ActionContributionItem(_deleteTypeAction)};
		hookContextMenu();
		// Register with listeners
		AcmeStudioChangeManager.registerListener(this);
		// Start a helper to report errors in the properties
		HoverHelp help = new HoverHelp();
		help.activateHoverHelp(_viewer.getTable());
		table.addSelectionListener(this);
		table.addFocusListener(this);
	}
	/**
	 * Returns the TableView that contains the styles
	 * @return TableViewer
	 */
	public TableViewer getViewer() {
		return _viewer;
	}
	/** Modifies the selected style on a double click
	 */
	public void handleDoubleClick() {
		ISelection selection = _viewer.getSelection();
		try {
			_selectedStyle =
				(AcmeStyleHolder) ((IStructuredSelection) selection)
					.getFirstElement();
			if (_selectedStyle != null) {
				modifyStyle(_selectedStyle._type);
			}
		} catch (ClassCastException e) {
			// Only want to deal with AcmeProperties
			e.printStackTrace();
		} catch (NullPointerException ne) {
			// Ignore if nothing is selected
			ne.printStackTrace();
		}
	}
	/**
	 * Set the selected object to be the new object passed in
	 * @param o the object to view
	 */
	public void setInput(AcmeFamily acmeFamily) {
		if (acmeFamily == null)
			return;
		//_selectedObject = o;
		_acmeFamily = acmeFamily;
		_acmeTypes = Helper.getTypes(_acmeFamily);
		_viewer.setInput(acmeFamily);
		// Assign the cell modifiers -- this is heavy weight
		_viewer.setCellEditors(getCellEditors(_viewer.getTable()));
		_viewer.setColumnProperties(PROPERTIES);
		_viewer.setCellModifier(new StyleCellModifier());
	}
	/**
	 * @see org.eclipse.swt.widgets.Control#setFocus()
	 */
	public boolean setFocus() {
		return _viewer.getControl().setFocus();
	}
	/**
	 * Adds the double click action to the table viewer
	 */
	private void hookDoubleClickAction() {
		_viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				_doubleClickAction.run();
			}
		});
	}
	/**
	 * Makes all the actions for the property list
	 */
	private void makeActions() {
		// The delete action, deletes the currently selected properties
		_deleteTypeAction = new Action() {
			public void run() {
					// Get the selections
	ISelection isel = _viewer.getSelection();
				if (isel instanceof IStructuredSelection) {
					IStructuredSelection iss = (IStructuredSelection) isel;
					Iterator it = iss.iterator();
					// Iterate through the selections
					while (it.hasNext()) {
						Object obj = it.next();
						if (obj instanceof AcmeStyleHolder) {
							AcmeStyleHolder style = (AcmeStyleHolder) obj;
							AcmeElementType acmeType = style.getType();
							TypeCommand command =
								new TypeCommand(
									TypeCommand.REMOVE_TYPE,
									acmeType,
									_acmeFamily,
									_parentEditor);
							command.execute();
						}
					}
					setInput(_acmeFamily);
				}
			}

			public boolean isEnabled() {
				return !_viewer.getSelection().isEmpty();
			}
		};
		_deleteTypeAction.setText("Delete type");
		_deleteTypeAction.setToolTipText("Deletes selected types");
        // As of Eclipse 3.0, IMG_TOOL_DELETE_HOVER == IMG_TOOL_DELETE
		_deleteTypeAction.setImageDescriptor(
			WorkbenchImages.getImageDescriptor(
				ISharedImages.IMG_TOOL_DELETE));

		_browseTypeAction = new Action() {
			public void run() {
				ISelection isel = _viewer.getSelection();

				if (isel instanceof IStructuredSelection) {
					OpenTypeHierarchyAction a = new OpenTypeHierarchyAction();

					AcmeStyleHolder style =
						(AcmeStyleHolder) ((IStructuredSelection) isel)
							.getFirstElement();
					AcmeElementType acmeType = style.getType();
					a.setElement((AcmeElement) acmeType);
					a.run();
				}
			}

			public boolean isEnabled() {
				return !_viewer.getSelection().isEmpty();
			}
		};
		_browseTypeAction.setText("Open in Type Browser");

		_changePaletteVisibilityAction = new Action () {
			public void run() {
				ISelection isel = _viewer.getSelection ();
				if (isel instanceof IStructuredSelection) {
					IStructuredSelection sel = (IStructuredSelection )isel;
					AcmeStyleHolder holder = (AcmeStyleHolder )sel.getFirstElement ();
					ElementVisualization vis = VisualStyleRegistry.getVisualization(holder._type);
					Boolean value = null;
					if (vis != null && vis.isHiddenFromPalette()) {
						value = new Boolean(false);
					} else {
						value = new Boolean(true);
					}
					changePaletteVisibility(holder, value);
				}
			}
		};
		_changePaletteVisibilityAction.setText ("Toggle palette visibility");
		_changePaletteVisibilityAction.setDescription ("Toggles whether the type will be visibile in palettes");
	}
	/**
	 * Creates the context menu for this view, and hooks it into the framework
	 */
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#ElementTypePopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(_viewer.getControl());
		_viewer.getControl().setMenu(menu);
		if (_parentEditor != null) {
			_parentEditor.getSite().registerContextMenu(menuMgr, _viewer);
		}
	}
	/**
	 * Adds the contents of the menu
	 */
	private void fillContextMenu(IMenuManager manager) {
		if (_editable) {
			// Show creation commands first (BugFix #66)
			manager.add(_newTypeAction);
			manager.add(_deleteTypeAction);
			manager.add(new Separator());
			SuperTypeAssignmentMenuAction assFamAction =
				(SuperTypeAssignmentMenuAction) _parentEditor.getAction(
					TypeAssignmentCommand.ASSIGN_TYPE);
			SuperTypeAssignmentMenuAction unassFamAction =
				(SuperTypeAssignmentMenuAction) _parentEditor.getAction(
					TypeAssignmentCommand.UNASSIGN_TYPE);
			manager.add(assFamAction.buildMenu());
			manager.add(unassFamAction.buildMenu());
			manager.add(_browseTypeAction);
			manager.add (new Separator ());
			manager.add (_changePaletteVisibilityAction);
			manager.add (_doubleClickAction);
			// Other plug-ins can contribute there actions here
			manager.add(new Separator("Additions"));
		} else {
			StatusLineContributionItem item =
				new StatusLineContributionItem("$XXX$");
			item.setText(
				"You must open the family file directly to edit this family");
			manager.add(item);
			manager.add(new Separator("Additions"));
		}
	}
	/**
	 * Should be fired when this style bar is selected
	 * @see org.eclipse.ui.ISelectionListener#selectionChanged(IWorkbenchPart, ISelection)
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			IStructuredSelection iSel = (IStructuredSelection) selection;
			if (iSel.getFirstElement() instanceof AcmeElementType) {
				_parentEditor.dispatchNewSelection(
					(AcmeElementType) iSel.getFirstElement());
			}
		}
	}
	public IContributionItem[] getContributions() {
		return _myContributions;
	}

	/**
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#isValidFor(AcmeObject)
	 */
	public boolean isValidFor(AcmeObject o) {
		return true;
	}
	/** 
	 * Modifies the selected style by bring up a dialog
	 */
	private void modifyStyle(AcmeElementType acmeElementType) {
		// Get the shell of to display the dialog box
		Shell shell =
			AcmeStudioPlugin
				.getDefault()
				.getWorkbench()
				.getActiveWorkbenchWindow()
				.getShell();
		AcmeFamily acmeFamily = _parentEditor.getFamily();
		ElementVisualization oldVis =
			VisualStyleRegistry.getVisualization(acmeElementType);
		boolean hasStyle = VisualStyleRegistry.hasVisualStyle(acmeFamily);
		boolean editVis = true;
		int selection = -1;
		VisualStyle style = VisualStyleRegistry.getVisualStyle(acmeFamily);
		if (!hasStyle) {
			if (!VisualStyleRegistry.isDefaultVisualStyle(style)) {
				StyleEditPrefDialog sepd = new StyleEditPrefDialog(shell);
				sepd.open();
				if (IDialogConstants.CANCEL_ID == sepd.getReturnCode()) {
					editVis = false;
				} else {
					selection = sepd.getSelection();
				}
			}
			else {
				MessageDialog dialog = new MessageDialog (shell, "Create new style?", null, "This family does not have a visual style. Create one?" , MessageDialog.QUESTION, new String [] {"Yes", "No"}, 0);
				if (dialog.open() == MessageDialog.OK) {
					selection = TO_CREATE;
				}
				else {
					editVis = false;
				}
			}
		}
		if (editVis) {
			// Creeat the dialog to change the selected property
			EditVisualizationDialog dialog =
				new EditVisualizationDialog(
					shell,
					acmeElementType,
					selection == TO_CREATE);
			// Open the dialog
			int retCode = dialog.open();
			if (retCode == IDialogConstants.OK_ID) {
				// if the user didn't cancel
				ElementVisualization vis = dialog.getVisualization();
				if (hasStyle && style != null) {
					style.add(acmeElementType, vis);
				} else {
					// if user chooses to create a new style
					if (selection == TO_CREATE) {
						style = new VisualStyle();
					}
					style.add(acmeElementType, vis);
					VisualStyleRegistry.addStyle(acmeFamily, style);
					// Give everyone depending on the old visualization
					// the opportunity to find out if they need to use
					// the new visualization
					oldVis.resetVisualization();
				}
				_viewer.refresh();
//				setInput(acmeFamily);
				_parentEditor.getResourceProvider().setDirty(true);
			}
		}
	}
	private class StyleCellModifier implements ICellModifier {
		public boolean canModify(Object element, String property) {
			return _editable;
		}
		public Object getValue(Object element, String property) {
			Object value = null;
			if ((element instanceof AcmeStyleHolder)
				&& canModify(element, property)) {
				AcmeStyleHolder holder = (AcmeStyleHolder) element;
				AcmeElementType type = holder._type;
				value = Helper.getDescription((AcmeElement) type);
				if (property.equals(PROPERTIES[PALETTECOL])) {
					ElementVisualization vis = VisualStyleRegistry.getVisualization(holder._type);
					if (vis != null && vis.isHiddenFromPalette()) {
						value = new Boolean(true);
					} else {
						value = new Boolean(false);
					}
				}

			}
			return value;
		}
		public void modify(Object element, String property, Object value) {
			if (element instanceof TableItem) {
				TableItem item = (TableItem) element;
				AcmeStyleHolder holder = (AcmeStyleHolder) item.getData();
				if (property.equals(PROPERTIES[DESCCOL])) {
					AcmeElementType type = holder._type;
					Helper.setDescription((AcmeElement) type, (String) value);
					_viewer.refresh ();
				} else if (property.equals(PROPERTIES[PALETTECOL])) {
					Boolean val = (Boolean) value;
					changePaletteVisibility (holder, val);
				}
			}
		}
	}
	protected CellEditor[] getCellEditors(Table table) {
		final CellEditor[] editors = new CellEditor[4];
		// Changes to this text editor are automatically triggered
		class AutoApplyTextCellEditor extends TextCellEditor {
			public AutoApplyTextCellEditor(Composite parent) {
				super(parent);
			}
			public void fireApplyEditorValue() {
				super.fireApplyEditorValue();
			}
		}
		class VisCellEditor extends DialogCellEditor {
			public VisCellEditor(Composite parent) {
				super(parent);
			}
			protected Object openDialogBox(Control cellEditorWindow) {
				handleDoubleClick();
				return null;
			}
		}
		class AutoApplyCheckboxCellEditor extends CheckboxCellEditor {
			public AutoApplyCheckboxCellEditor(Composite parent) {
				super(parent);
			}
			public void fireApplyEditorValue() {
				super.fireApplyEditorValue();
			}

		}
		//editors[TYPECOL] = new AutoApplyTextCellEditor(table);
		editors[PALETTECOL] = null; //new AutoApplyCheckboxCellEditor(table);
		editors[DESCCOL] = new AutoApplyTextCellEditor(table);
		editors[DESCCOL].getControl().addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				((AutoApplyTextCellEditor) editors[DESCCOL])
					.fireApplyEditorValue();
			}
		});
		return editors;
	}
	private Image getPreviewImage(AcmeElementType type, boolean zoom) {
		ElementVisualization vis = VisualStyleRegistry.getVisualization(type);
		ManagedImage preview = vis.getLabellessPreviewImage(type);
		preview.addListener(this);
		vis.addPropertyChangeListener(null, this);
		_visListeningTo.add (vis);
		if (!zoom)
			return preview.getImage ();
		ManagedImage small = vis.getSmallImage(type);
		small.addListener(this);
		return small.getImage ();
//		ImageData previewImageData = preview.getImageData();
//		int imageHeight = preview.getBounds().height;
//		int imageWidth = preview.getBounds().width;
//		float aspectRatio = (float) imageHeight / (float) imageWidth;
//		Image smallImage =
//			ImageManager.createUnreferencedImage(previewImageData);
//		if (imageHeight > 12) {
//			int scaledWidth = Utils.zeroGuard((int) (12 * 1 / aspectRatio));
//			if (scaledWidth < 3)
//				scaledWidth = 20;
//			ImageData sid = previewImageData.scaledTo(scaledWidth, 12);
//			smallImage.dispose();
//			smallImage = ImageManager.createUnreferencedImage(sid);
//		} else if (imageWidth > 20) {
//			int scaledHeight = Utils.zeroGuard((int) (20 * aspectRatio));
//			if (scaledHeight < 5)
//				scaledHeight = 12;
//			ImageData smallDataImage =
//				previewImageData.scaledTo(20, scaledHeight);
//			smallImage.dispose();
//			smallImage = ImageManager.createUnreferencedImage(smallDataImage);
//		}
//		return smallImage;
	}
	/**
	 * Sent when selection occurs in the control.
	 * <p>
	 * For example, on some platforms selection occurs in
	 * a List when the user selects an item or items.
	 * </p>
	 *
	 * @param e an event containing information about the selection
	 */
	public void widgetSelected(SelectionEvent e) {
		Object obj = e.getSource();
		Table table = (Table) obj;
		TableItem[] items = table.getSelection();
		if (items.length != 0) {
			TableItem item = items[0];
			AcmeStyleHolder holder = (AcmeStyleHolder) item.getData();
			AcmeElementType type = holder._type;
			_parentEditor.dispatchNewSelection(type);
			_parentEditor.refocusStyleLists(this);
		} else {

		}
	}
	//    /**
	//     * Sent when default selection occurs in the control.
	//     * <p>
	//     * For example, on some platforms default selection occurs
	//     * in a List when the user double-clicks an item or types
	//     * return in a Text.
	//     * </p>
	//     *
	//     * @param e an event containing information about the default selection
	//     */
	//    public void widgetDefaultSelected(SelectionEvent e) {
	//    }
	//    /**
	//     * Notifies that the selection has changed.
	//     *
	//     * @param event event object describing the change
	//     */
	//    public void selectionChanged(SelectionChangedEvent event) {
	//        Debug.diagnoseln("ElementTypeList.selectionChanged(): " + event.toString());
	//    }
	//    /**
	//     * Sent when a control gets focus.
	//     *
	//     * @param e an event containing information about the focus change
	//     */
	//    public void focusGained(FocusEvent e) {
	//    }
	//    /**
	//     * Sent when a control loses focus.
	//     *
	//     * @param e an event containing information about the focus change
	//     */
	//    public void focusLost(FocusEvent e) {
	//    }
	//    public void refresh() {
	//        if (!isDisposed() && !_viewer.getControl().isDisposed())
	//            this._viewer.refresh();
	//    }
	/**
	 * @param evt
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 **/
	public void propertyChange(PropertyChangeEvent evt) {
		_viewer.refresh(true);
	}
	/**
	 * @param holder
	 * @param val
	 */
	private void changePaletteVisibility(AcmeStyleHolder holder, Boolean val) {
		AcmeFamily acmeFamily = _parentEditor.getFamily();
		ElementVisualization oldVis =
			VisualStyleRegistry.getVisualization(holder._type);
		
		boolean hasStyle =
			VisualStyleRegistry.hasVisualStyle(acmeFamily);
		if (!hasStyle) {
			// Must create a new style if the value is changing
			if (MessageDialog
				.openQuestion(
					_viewer.getControl().getShell(),
					"Create new style",
					"This family does not currently have a visual style. Create one?")) {
				VisualStyle style = new VisualStyle();
				oldVis = oldVis.copy();
				style.add(holder._type, oldVis);
			}
		}
		if (VisualStyleRegistry.hasVisualStyle(acmeFamily)) {
			// Find the subtypes to potentially propagate the 
			// property change to
			VisualStyle style = VisualStyleRegistry.getVisualStyle(acmeFamily);
			java.util.List subTypes =
				ModelHelper.getSubtypesInSameScope(holder._type);
			if (!subTypes.isEmpty()) {
				ListSelectionDialog dialog =
					new ListSelectionDialog(
						_viewer.getControl().getShell(),
						subTypes,
						new AcmeElementContentProvider(),
						new AcmeViewLabelProvider(),
						"Select subtypes to "
							+ (val.booleanValue() ? "hide from" : "show in")
							+ " palette.");
				if (dialog.open() == IDialogConstants.OK_ID) {

					// Go through selected subtypes and propagae
					Object[] selectedType = dialog.getResult();
					
					for (int i = 0; i < selectedType.length; i++) {

						AcmeElementType type =
							(AcmeElementType) selectedType[i];
						ElementVisualization vis =
							VisualStyleRegistry.getVisualization(
								type);
						if (VisualStyleRegistry
							.isDefaultVisualization(vis) || vis == oldVis) {
							vis = vis.copy();
							style.add(type, vis);
							}

						vis.setHiddenFromPalette(
							val.booleanValue());
					}
				}
			}
		}
		oldVis.setHiddenFromPalette(val.booleanValue());
		oldVis.resetVisualization();
		_parentEditor.getResourceProvider().setDirty(true);
		_viewer.refresh ();
	}
	/**
	 * @param image
	 * @see AcmeStudio.util.IManagedImageListener#imageDisposed(AcmeStudio.util.ManagedImage)
	 **/
	public void imageDisposed(ManagedImage image) {
		_viewer.refresh (true);
	}

}