/*
 * Created on Jun 4, 2004
 *
 */
package AcmeStudio.ui.editors.style.parts;

import java.util.Hashtable;
import java.util.Vector;

import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.model.TypeAssignmentCommand;
import AcmeStudio.ui.editors.AcmeStyleEditor;
import AcmeStudio.ui.editors.actions.NewElementTypeAction;
import AcmeStudio.ui.editors.actions.SuperTypeAssignmentMenuAction;
import AcmeStudio.ui.editors.diagram.actions.NavigateToImportedFamilyAction;
import AcmeStudio.ui.editors.diagram.styles.*;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeFamily;


/**
 * @author Bradley Schmerl
 * Created Jun 4, 2004
 */
public class ElementTypeListSection extends SectionPart implements ModifyListener, FocusListener, MouseListener {

	private Class _category;
	private Hashtable _descTable = new Hashtable ();
	private AcmeStyleEditor _styleEditor = null;
	private StyleList _styleList = null;
	private TypesPage	_page;
	private static final int	CLIENT_VSPACING	= 4;
    private static final int TO_CREATE = 1;
	

	/**
	 * @param page
	 * @param body
	 * @param category
	 */
	public ElementTypeListSection(TypesPage page, Composite parent, Class category) {
		super (parent, page.getManagedForm().getToolkit(), Section.TITLE_BAR | Section.TWISTIE | Section.EXPANDED);
		_page = page;
		_styleEditor = (AcmeStyleEditor )page.getStyleEditor();
		initialize (page.getManagedForm ());
		getSection ().clientVerticalSpacing = CLIENT_VSPACING;
		getSection ().setData ("part", this);
		_category = category;
		createClient (getSection (), page.getEditor ().getToolkit ());
	}

	/**
	 * @param section
	 * @param toolkit
	 */
	private void createClient(Section section, FormToolkit toolkit) {
		AcmeFamily family = _styleEditor.getFamily ();
		Vector acmeTypes = Helper.getTypes (family);
		
		Composite client = toolkit.createComposite (section);
		  GridLayout layout = new GridLayout();
	        layout.numColumns = 1;
	        layout.marginWidth = 8;
	        layout.verticalSpacing = 7;
	        layout.horizontalSpacing = 6;
	        client.setLayout(layout);
	        section.setClient(client);
	        
	        MenuManager popupMenuManager = new MenuManager();
	        IMenuListener listener = new IMenuListener() {
	            public void menuAboutToShow(IMenuManager mng) {
	                fillContextMenu(mng);
	            }
	        };
	        popupMenuManager.addMenuListener(listener);
	        popupMenuManager.setRemoveAllWhenShown(true);
	        Menu menu = popupMenuManager.createContextMenu(client);
	        client.setMenu(menu);
	        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
	        _styleList = new ElementTypeList (client, SWT.NONE, _category, _styleEditor);
	        _styleList.setLayoutData (gd);
	        _styleList.setInput (family);
	        _styleList.setMenu (menu);
	        toolkit.paintBordersFor(client);
	}

	/**
	 * @param mng
	 */
	protected void fillContextMenu(IMenuManager manager) {
	       SuperTypeAssignmentMenuAction assFamAction =
            (SuperTypeAssignmentMenuAction)_styleEditor.getAction(TypeAssignmentCommand.ASSIGN_FAMILY);
        SuperTypeAssignmentMenuAction unassFamAction =
            (SuperTypeAssignmentMenuAction)_styleEditor.getAction(TypeAssignmentCommand.UNASSIGN_FAMILY);
        NavigateToImportedFamilyAction navToImports = (NavigateToImportedFamilyAction )_styleEditor.getAction (NavigateToImportedFamilyAction.ID);
        if (assFamAction != null) {
            manager.add(new Action("Edit System Visualization") {
                public void run() {
                    modifyFamilyStyle();
                }
            });
            manager.add(new Separator());
            manager.add(assFamAction.buildMenu());
            manager.add(unassFamAction.buildMenu());
            manager.add(navToImports.buildMenu());
            manager.add(new Separator());
            manager.add(_styleEditor.getAction(NewElementTypeAction.NEW_COMPONENT_TYPE));
            manager.add(_styleEditor.getAction(NewElementTypeAction.NEW_CONNECTOR_TYPE));
            manager.add(_styleEditor.getAction(NewElementTypeAction.NEW_PORT_TYPE));
            manager.add(_styleEditor.getAction(NewElementTypeAction.NEW_ROLE_TYPE));
//            manager.add(_styleEditor.getAction(NewPropertyTypeAction.ID));
            manager.add(new Separator("Additions"));
        }
	}

	/**
	 * 
	 */
	protected void modifyFamilyStyle() {
	       // Get the shell of to display the dialog box
        Shell shell = AcmeStudioPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();
        AcmeFamily acmeFamily = _styleEditor.getFamily();
//        ElementVisualization oldVis = VisualStyleRegistry.getVisualization((AcmeElementType)acmeFamily);
        Visualization oldVis =
            VisualStyleRegistry.getSysVis(acmeFamily);
        boolean hasStyle = VisualStyleRegistry.hasVisualStyle(acmeFamily);
        boolean editVis = true;
        int selection = -1;
        VisualStyle style = VisualStyleRegistry.getVisualStyle(acmeFamily);
        if (!hasStyle) {
            StyleEditPrefDialog sepd = new StyleEditPrefDialog(shell);
            sepd.open();
            if (IDialogConstants.CANCEL_ID == sepd.getReturnCode()) {
                editVis = false;
            }
            else {
                selection = sepd.getSelection();
            }
        }
        if (editVis) {
            // Creeat the dialog to change the selected property
            EditSysVisDialog dialog = new EditSysVisDialog(shell, acmeFamily, selection == TO_CREATE);
            // Open the dialog
            int retCode = dialog.open();
            if (retCode == IDialogConstants.OK_ID) {
                // if the user didn't cancel
                SystemVisualization vis = dialog.getSysVis();
                if (hasStyle && style != null) {
                    style.addSysVis(acmeFamily, vis);
                }
                else {
                    // if user chooses to create a new style
                    if (selection == TO_CREATE) {
                        style = new VisualStyle();
                    }
                    style.addSysVis(acmeFamily, vis);
                    VisualStyleRegistry.addStyle(acmeFamily, style);
                    // Give everyone depending on the old visualization
                    // the opportunity to find out if they need to use
                    // the new visualization
                    oldVis.resetVisualization();
                }
                _styleEditor.getResourceProvider().setDirty(true);
            }
        }
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		Object obj = e.getSource ();
		if (obj instanceof Text) {
			Text descText = (Text )obj;
			AcmeElementType acmeType = (AcmeElementType )_descTable.get (descText);
			Helper.setDescription ((AcmeElement )acmeType, descText.getText ());
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.FocusListener#focusGained(org.eclipse.swt.events.FocusEvent)
	 */
	public void focusGained(FocusEvent e) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.FocusListener#focusLost(org.eclipse.swt.events.FocusEvent)
	 */
	public void focusLost(FocusEvent e) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseDoubleClick(MouseEvent e) {
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseDown(MouseEvent e) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseUp(MouseEvent e) {
	}

	public void refresh () {
		_styleList.refresh ();
	}
	/**
	 * @return Returns the styleList.
	 */
	public StyleList getStyleList() {
		return _styleList;
	}
}
