/*
 * Created on Jun 3, 2004
 *
 */
package AcmeStudio.ui.editors.style.parts;

import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.ui.editors.AcmeStyleEditor;
import AcmeStudio.ui.editors.diagram.styles.*;
import AcmeStudio.util.ImageManager;
import acmeLib.elements.AcmeFamily;


/**
 * @author Bradley Schmerl
 * Created Jun 3, 2004
 */
public class FamilySection extends SectionPart implements MouseListener {

	
private static final int	CLIENT_VSPACING	= 4;
	private TypesPage	_page;
	private AcmeStyleEditor	_styleEditor;
	private static final int	TO_CREATE	= 1;

	/**
	 * @param parent
	 * @param toolkit
	 * @param style
	 */
	public FamilySection(TypesPage page, Composite parent) {
		super (parent, page.getManagedForm().getToolkit(), Section.TITLE_BAR | Section.TWISTIE | Section.EXPANDED);
		_page = page;
		_styleEditor = (AcmeStyleEditor )page.getStyleEditor ();
		initialize (page.getManagedForm ());
		getSection ().clientVerticalSpacing = CLIENT_VSPACING;
		getSection ().setData ("part", this);
		createClient (getSection (), page.getEditor ().getToolkit());
	}

	/**
	 * @param section
	 * @param toolkit
	 */
	private void createClient(Section section, FormToolkit toolkit) {
		section.setText ("Family");
		Composite client = toolkit.createComposite(section);
		GridLayout layout = new GridLayout ();
		layout.numColumns = 3;
		layout.marginWidth = 8;
		layout.verticalSpacing = 7;
		layout.horizontalSpacing = 6;
		client.setLayout (layout);
		section.setClient(client);
		MenuManager popupMenuManager = new MenuManager ();
		IMenuListener listener = new IMenuListener () {

			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu (manager);
			}
			
		};
		popupMenuManager.addMenuListener(listener);
		popupMenuManager.setRemoveAllWhenShown(true);
		Menu menu = popupMenuManager.createContextMenu(section);
		section.setMenu(menu);
		
		AcmeFamily family = _styleEditor.getFamily ();
		String familyName = family.getName ();
		Label labelImg = toolkit.createLabel(client, familyName);
		Label labelTxt = toolkit.createLabel (client, familyName);
		
		labelImg.setImage (ImageManager.getSmallImageByType(family));
		labelImg.addMouseListener (this);
		labelTxt.addMouseListener (this);
		labelImg.setMenu(menu);
		labelTxt.setMenu(menu);
		Button familyButton = toolkit.createButton(client, "Connection Patterns", SWT.PUSH);
		familyButton.addSelectionListener (new SelectionAdapter () {
			public void widgetSelected (SelectionEvent e) {
				try {
					modifyFamilyStyle ();
				}
				catch (ClassCastException cce) {
					// Only want to deal with AcmeProperties
					cce.printStackTrace ();
				}
				catch (NullPointerException npe) {
					// Ignore if nothing is selected
				}
			}
		});
	}

	/**
	 * @param manager
	 */
	protected void fillContextMenu(IMenuManager manager) {
	}

	/**
	 * 
	 */
	protected void modifyFamilyStyle() {
        // Get the shell of to display the dialog box
        Shell shell = AcmeStudioPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();
        AcmeFamily acmeFamily = _styleEditor.getFamily();
//        ElementVisualization oldVis = VisualStyleRegistry.getVisualization((AcmeElementType)acmeFamily);
        Visualization oldVis =
            VisualStyleRegistry.getSysVis(acmeFamily);
        boolean hasStyle = VisualStyleRegistry.hasVisualStyle(acmeFamily);
        boolean editVis = true;
        int selection = -1;
        VisualStyle style = VisualStyleRegistry.getVisualStyle(acmeFamily);
        if (!hasStyle) {
            StyleEditPrefDialog sepd = new StyleEditPrefDialog(shell);
            sepd.open();
            if (IDialogConstants.CANCEL_ID == sepd.getReturnCode()) {
                editVis = false;
            }
            else {
                selection = sepd.getSelection();
            }
        }
        if (editVis) {
            // Creeat the dialog to change the selected property
            EditSysVisDialog dialog = new EditSysVisDialog(shell, acmeFamily, selection == TO_CREATE);
            // Open the dialog
            int retCode = dialog.open();
            if (retCode == IDialogConstants.OK_ID) {
                // if the user didn't cancel
                SystemVisualization vis = dialog.getSysVis();
                if (hasStyle && style != null) {
                    style.addSysVis(acmeFamily, vis);
                }
                else {
                    // if user chooses to create a new style
                    if (selection == TO_CREATE) {
                        style = new VisualStyle();
                    }
                    style.addSysVis(acmeFamily, vis);
                    VisualStyleRegistry.addStyle(acmeFamily, style);
                    // Give everyone depending on the old visualization
                    // the opportunity to find out if they need to use
                    // the new visualization
                    oldVis.resetVisualization();
                }
                _styleEditor.getResourceProvider().setDirty(true);
            }
        }
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseDoubleClick(MouseEvent e) {
		try {
			modifyFamilyStyle ();
		}
		catch (ClassCastException cce) {
			
		}
		catch (NullPointerException npe) {
			
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseDown(MouseEvent e) {
		_styleEditor.refocusStyleLists (null);
		_styleEditor.dispatchNewSelection (_styleEditor.getFamily ());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseUp(MouseEvent e) {
	}

}
