/*
 * Created on Jun 10, 2003
 *
 */
package AcmeStudio.ui.editors.style.parts;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;

import AcmeStudio.ui.editors.AcmeStyleEditor;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.elements.AcmeFamily;

/**
 * @author schmerl
 */
public abstract class StyleList
    extends Composite
    implements
        ISelectionListener,
        AcmeModelChangeListener,
        ControlListener,
        SelectionListener,
        ISelectionChangedListener,
        FocusListener {

    protected TableViewer _viewer;
    protected AcmeFamily _acmeFamily;
    protected AcmeStyleEditor _parentEditor;
    protected boolean _editable = true;
    private boolean _beingRefreshed = false;
    
    protected static final Object [] emptyElements = new Object [0];

    /**
     * @param parent
     * @param style
     */
    public StyleList(Composite parent, int style, AcmeStyleEditor ed) {
        super(parent, style);
        _parentEditor = ed;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IWorkbenchPart part, ISelection selection) {

    }

    /* (non-Javadoc)
     * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
     */
    public void changeModel(AcmeStudioChangeEvent evt) {

    }
    
    public boolean interestedInEvent (int et) {
    	return false;
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.events.ControlListener#controlMoved(org.eclipse.swt.events.ControlEvent)
     */
    public void controlMoved(ControlEvent e) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.events.ControlListener#controlResized(org.eclipse.swt.events.ControlEvent)
     */
    public void controlResized(ControlEvent e) {
        Object obj = e.getSource();
        if (obj instanceof TableColumn) {
            if (!_beingRefreshed) {
                _parentEditor.refreshStyleLists(this);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetSelected(SelectionEvent e) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetDefaultSelected(SelectionEvent e) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
     */
    public void selectionChanged(SelectionChangedEvent event) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.events.FocusListener#focusGained(org.eclipse.swt.events.FocusEvent)
     */
    public void focusGained(FocusEvent e) {

    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.events.FocusListener#focusLost(org.eclipse.swt.events.FocusEvent)
     */
    public void focusLost(FocusEvent e) {

    }

    public  TableViewer getViewer () {return _viewer;}
    
    public Table getTable () {return _viewer.getTable ();}
    
    public void setInput (AcmeFamily acmeFamily) {
        if (acmeFamily == null) {
            return;
        }
        _acmeFamily = acmeFamily;
        _viewer.setInput (_acmeFamily);
    }
    
    public void refresh () {
        if (!isDisposed () && !_viewer.getControl ().isDisposed ())
            this._viewer.refresh ();
    }
       
    public void refresh (StyleList source) {
        _beingRefreshed = true;
        Table table = getTable();
        Table sourceTable = source.getTable();
        // BugFix to get around GTK's not so robust table
        // Only refresh if the tables have the same number of columns
        int thisSize = table.getColumnCount();
        int sourceSize = sourceTable.getColumnCount();
        if (thisSize == sourceSize) {
	        for (int i = 0; i < thisSize; i++) {
	            TableColumn tc = table.getColumn(i);
	            TableColumn sourceTc = sourceTable.getColumn(i);
	            tc.setWidth(sourceTc.getWidth());
	        }
	        this.redraw();
        }
        _beingRefreshed = false;        
    }
    
    public void deselectAll () {
        getTable ().deselectAll();
    }
    
    public void setEditable (boolean canEdit) {
        _editable = canEdit;
    }

}
