package AcmeStudio.ui.editors.text;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.*;
import org.eclipse.swt.SWT;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AcmeCodeScanner extends RuleBasedScanner {
    private static String [] _keywords = 
        {"system", "component", "group", "members", "components", "connectors", 
            "connector", "port", "ports", "role", "roles", "element", "attachment",
            "attachments", "to", "property", "properties", "representation",
            "representations", "binding", "bindings", "type", "extended", "extends",
            "with", "new", "family", "style", "import", "exists", "forall",
            "external", "design", "analysis", "invariant", "heuristic", 
            "and", "or", "in", "select", "collect", "set", "sequence", "seq",
            "record", "enum", "unique"};
            
    private static String [] _types = 
        {"float", "integer", "int", "string", "boolean", "bool", "any"};
        
    private static String [] _constants = {"true", "false"};

    public AcmeCodeScanner (AcmeColorProvider provider) {
        TextAttribute kwAtt = new TextAttribute (provider.getColor (AcmeColorProvider.KEYWORD), null,SWT.BOLD);
        Token keyword = new Token (kwAtt);

        IToken type = new Token (new TextAttribute (provider.getColor (AcmeColorProvider.TYPE)));
        IToken string = new Token (new TextAttribute (provider.getColor (AcmeColorProvider.STRING)));
        IToken comment = new Token (new TextAttribute (provider.getColor (AcmeColorProvider.SINGLE_LINE_COMMENT)));
        IToken other = new Token (new TextAttribute (provider.getColor (AcmeColorProvider.DEFAULT)));
        
        List rules = new ArrayList ();
        
        rules.add (new EndOfLineRule ("//", comment));
        rules.add (new SingleLineRule ("\"", "\"", string, '\\'));
        
        rules.add (new WhitespaceRule (new IWhitespaceDetector () {
            public boolean isWhitespace (char ch) {
                return Character.isWhitespace(ch);
            }   
        }));
        
        WordRule wordRule = new WordRule (new AcmeWordDetector (), other) {
            private StringBuffer _buffer = new StringBuffer ();
             
            public IToken evaluate (ICharacterScanner scanner) {
                int c = scanner.read ();
                if (fDetector.isWordStart ((char )c)) {
                    if (fColumn == UNDEFINED || (fColumn == scanner.getColumn () - 1))    {
                        _buffer.setLength (0);
                        do {
                            _buffer.append ((char )c);
                            c = scanner.read ();
                        }
                        while (c != ICharacterScanner.EOF && fDetector.isWordPart ((char )c));
                        scanner.unread ();
                        
                        IToken token = (IToken )fWords.get (_buffer.toString ().toLowerCase ());
                        if (token != null) {
                            return token;
                        }
                        
                        if (fDefaultToken.isUndefined ()) 
                            unreadBuffer (scanner);
                        return fDefaultToken;
                    }
                }
                
                scanner.unread ();
                return Token.UNDEFINED;
                            
            }  
            
            protected void unreadBuffer (ICharacterScanner scanner) {
                for (int i = _buffer.length () - 1; i >= 0; i--) {
                    scanner.unread ();
                }
            }
        };
        
        for (int i = 0; i < _keywords.length; i++) {
     
            wordRule.addWord (_keywords [i],keyword);
        }
        for (int i = 0; i < _types.length; i++) {
            wordRule.addWord (_types [i], type);
        }
        for (int i = 0; i < _constants.length; i++) {
            wordRule.addWord (_constants [i], keyword);
        }
        rules.add (wordRule);
        IRule [] result = new IRule [rules.size ()];
        rules.toArray (result);
        setRules (result);
        
    }

    private class AcmeWordDetector implements IWordDetector {
        public boolean isWordPart (char ch) {
            return Character.isLetter (ch) ||  
                    ch == '_' || ch == '-' ||
                    Character.isDigit (ch);
        }   
        
        public boolean isWordStart (char ch) {
            return Character.isLetter (ch) ||
                ch == '_';   
        }
    }

}
