package AcmeStudio.ui.editors.text;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.rules.*;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AcmePartitionScanner extends RuleBasedPartitionScanner {
    public final static String ACME_MULTILINE_COMMENT = "__java_multiline_comment";
    
    static class EmptyCommentDetector implements IWordDetector {
        public boolean isWordStart (char c) {
            return (c == '/');
        }
        
        public boolean isWordPart (char c) {
            return (c == '*' || c == '/');
        }
    }
    
    static class WordPredicateRule extends WordRule implements IPredicateRule {
        private IToken _successToken;
        
        public WordPredicateRule (IToken successToken) {
            super (new EmptyCommentDetector ());
            _successToken = successToken;
            addWord ("/**/", _successToken);
        }
        
        public IToken evaluate (ICharacterScanner scanner, boolean resume) {
            return super.evaluate (scanner);
        }
        
        public IToken getSuccessToken () {
            return _successToken;
        }
        
    }   
    
    public AcmePartitionScanner () {
        super ();
        
        IToken comment = new Token (ACME_MULTILINE_COMMENT);
        List rules = new ArrayList ();
        
        rules.add (new EndOfLineRule ("//", Token.UNDEFINED));
        rules.add (new SingleLineRule ("\"", "\"", Token.UNDEFINED, '\\'));
        
        rules.add (new WordPredicateRule (comment));
        rules.add (new MultiLineRule ("/*", "*/", comment));
        
        IPredicateRule [] result = new IPredicateRule [rules.size ()];
        rules.toArray (result);
        setPredicateRules (result);
    }        
    
}
