package AcmeStudio.ui.editors.text;

import org.eclipse.jface.text.*;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.BufferedRuleBasedScanner;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.IAnnotationHover;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AcmeSourceViewerConfiguration extends SourceViewerConfiguration {

    static class SingleTokenScanner extends BufferedRuleBasedScanner {
        public SingleTokenScanner (TextAttribute attribute) {
            setDefaultReturnToken(new Token (attribute));
        }   
    }

    public AcmeSourceViewerConfiguration () {
           
    }
    
    public IAnnotationHover getAnnotationHover (ISourceViewer viewer) {
        return new AcmeAnnotationHover ();
    }
    
    public IAutoIndentStrategy getAutoIndentStrategy (ISourceViewer viewer, String contentType) {
        return new DefaultAutoIndentStrategy ();
        //(IDocument.DEFAULT_CONTENT_TYPE.equals (contentType) ?                
    }
    
    

    /**
     * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getConfiguredContentTypes(ISourceViewer)
     */
    public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
        return new String [] {IDocument.DEFAULT_CONTENT_TYPE, AcmePartitionScanner.ACME_MULTILINE_COMMENT};
    }

    /**
     * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getContentAssistant(ISourceViewer)
     */
    public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {
        return super.getContentAssistant(sourceViewer);
    }

    /**
     * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getDefaultPrefixes(ISourceViewer, String)
     */
    public String[] getDefaultPrefixes(
        ISourceViewer sourceViewer,
        String contentType) {
        return new String [] {"\t", "    "};
    }

    /**
     * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getPresentationReconciler(ISourceViewer)
     */
    public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {
        AcmeColorProvider provider = AcmeEditorEnvironment.getAcmeColorProvider ();
        PresentationReconciler reconciler = new PresentationReconciler ();
        DefaultDamagerRepairer dr = new DefaultDamagerRepairer (AcmeEditorEnvironment.getAcmeCodeScanner ());
        reconciler.setDamager (dr, IDocument.DEFAULT_CONTENT_TYPE);
        reconciler.setRepairer (dr, IDocument.DEFAULT_CONTENT_TYPE);
        
        dr = new DefaultDamagerRepairer (new SingleTokenScanner (new TextAttribute (provider.getColor(AcmeColorProvider.MULTI_LINE_COMMENT))));
        reconciler.setDamager (dr, AcmePartitionScanner.ACME_MULTILINE_COMMENT);
        reconciler.setRepairer (dr, AcmePartitionScanner.ACME_MULTILINE_COMMENT);
        return reconciler;
        
        
    }

    /**
     * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getTextHover(ISourceViewer, String)
     */
    public ITextHover getTextHover(
        ISourceViewer sourceViewer,
        String contentType) {
        return super.getTextHover(sourceViewer, contentType);
    }

}
