/*
 * Created on Mar 1, 2003
 *
 */
package AcmeStudio.ui.editors.text;

import java.util.*;

import acmeLib.AcmeObject;
import acmeLib.core.io.Region;
import acmeLib.core.io.RegionManager;

/**
 * @author schmerl
 */
public class TextRegionManager extends RegionManager {
    
    public class TextRegion extends Region {
        
        TextRegion _parentRegion;
        List _childrenRegions;
        boolean calculatedChildren = false;
        
        public TextRegion (Region r) {
            super ();
            this.beginIndex = r.getBeginIndex();
            this.endIndex = r.getEndIndex();
            this.beginLine = r.getBeginLine();
            _childrenRegions = new ArrayList ();
        }
        
        public void setParent (TextRegion parent) {
            _parentRegion = parent;
        }
        
        public void addChild (TextRegion child) {
        	if (!this.enclosesRegion (child)) return;
        	if (_childrenRegions.isEmpty()) {
        		_childrenRegions.add (child);
        	}
        	else {
        		boolean childAdded = false;
        		for (Iterator iter = _childrenRegions.iterator (); iter.hasNext () && !childAdded;) {
					TextRegion region = (TextRegion )iter.next ();
					if (region.enclosesRegion(child)) {
						region.addChild (child);
						childAdded = true;
					}
				}
        		if (!childAdded && this.enclosesRegion (child)) {
        			_childrenRegions.add(child);
        		}
        		
        	}
        }
        
        public TextRegion getEnclosingRegion (int position) {
            if (position <= this.getEndIndex() && position >= this.getBeginIndex()) {
                if (_childrenRegions.isEmpty()) {
                    return this;
                }
                else {
                    TextRegion reg = null;
                    Iterator i = _childrenRegions.iterator();
                    while (i.hasNext () && reg == null) {
                        TextRegion r = (TextRegion )i.next ();
                        reg = r.getEnclosingRegion (position);
                    }
                    return reg==null?this:reg;
                }
            }
            else {
                return null;
            }
        }
        
        public boolean enclosesRegion (TextRegion r) {
            return this.getEndIndex () >= r.getEndIndex () && this.getBeginIndex () <= r.getBeginIndex();
        }
    }
    
    TextRegion _rootRegion;
    Map _region2ObjectMap = new HashMap ();
    
    public TextRegionManager (RegionManager rm) {
        super ();
        
        Map map = rm.getRegionMap ();
        
        // Need to take regions and from the TextRegion tree.
        SortedSet sortedRegions = new TreeSet (new Comparator () {
            public int compare (Object e1, Object e2) {
                TextRegion tr1 = (TextRegion )e1;
                TextRegion tr2 = (TextRegion )e2;
                
                int size1 = tr1.getEndIndex () - tr1.getBeginIndex ();
                int size2 = tr2.getEndIndex () - tr2.getBeginIndex ();
                
                if (size1 <= size2) return -1;
                else if (size1 > size2) return 1;
                else return 0;
                
            }
        });
        
        Iterator i = map.entrySet().iterator ();
        while (i.hasNext()) {
            Map.Entry entry = (Map.Entry )i.next ();
            TextRegion r = new TextRegion ((Region )entry.getValue());
            this.addRegion ((AcmeObject )entry.getKey (), r);
            _region2ObjectMap.put (r, entry.getKey ());
            sortedRegions.add (r);
        }
        
        Object [] regions = sortedRegions.toArray ();
        if (regions.length <= 0) return;
        
        _rootRegion = (TextRegion )regions [regions.length - 1];
        for (int idx = regions.length-1; idx > 0; idx--) {
        	TextRegion region = (TextRegion )regions [idx-1];
        	_rootRegion.addChild (region);
        }
        
//        for (int idx = 0; idx < regions.length; idx ++) {
//            TextRegion reg = (TextRegion )regions [idx];
//            TextRegion enclReg = null;
//            for (int jdx = idx + 1; jdx < regions.length; jdx++) {
//                TextRegion cReg = (TextRegion )regions [jdx];
//                if (cReg.enclosesRegion (reg)) {
//                    if (enclReg == null) enclReg = cReg;
//                    if (enclReg.enclosesRegion (cReg)) {
//                        enclReg = cReg;
//                    }
//                }
//            }
//            if (enclReg != null) {
//                enclReg.addChild (reg);
//                regions [idx] = null;
//            }
//        }

        if (regions.length > 0) {
            _rootRegion = (TextRegion )regions [regions.length-1]; 
        }
    }

    public AcmeObject getAcmeObjectAtPos (int pos) {
        AcmeObject obj = null;
        if (_rootRegion != null) {
            TextRegion inRegion = _rootRegion.getEnclosingRegion(pos);
            obj = (AcmeObject )_region2ObjectMap.get (inRegion);
        }
        
        return obj;
    }
}
