/*
 * Created on Oct 17, 2003
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package AcmeStudio.ui.views;

import java.util.ArrayList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.part.ViewPart;

import AcmeStudio.core.ASContext;
import AcmeStudio.core.actions.GroupAction;
import AcmeStudio.model.GroupCommand;
import AcmeStudio.ui.editors.AcmeStyleEditor;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.ui.editors.diagram.actions.HighlightObjectsInDiagramAction;
import AcmeStudio.ui.views.groups.GroupManager;
import AcmeStudio.ui.views.groups.wizards.GroupRuleWizard;
import AcmeStudio.ui.views.parts.ArmaniRulesFilter;
import AcmeStudio.ui.views.parts.PropertyFilter;
import AcmeStudio.ui.views.parts.PropertyList;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeGroup;
import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl
 *
 * Defines a view that allows groups to be managed, and highlighted in the diagram
 */
public class AcmeGroupView extends ViewPart implements ISelectionListener, AcmeModelChangeListener {

    /** A wrapper so that the property list can update the toolbar **/
    private IElementViewPage thisEVP = new IElementViewPage () {

        public IPageSite getSite() {
            return (IPageSite )AcmeGroupView.this.getSite ();
        }

        public void createControl(Composite parent) {
        }

        public void dispose() {
        }

        public Control getControl() {
            return null;
        }

        public void setActionBars(IActionBars actionBars) {
        }

        public void setFocus() {
        }

        public void addSelectionChangedListener(ISelectionChangedListener listener) {
        }

        public ISelection getSelection() {
            return null;
        }

        public void removeSelectionChangedListener(ISelectionChangedListener listener) {
        }

        public void setSelection(ISelection selection) {
        }
        
    };
    
    /** The groups being managed **/
    private List _lstGroups;
    /** The property list used to display properties of the group **/
    private PropertyList _propertyList;
    /** The list widget used to view members of the selected group **/
    private List _lstMembers;

    /** The system that defines the groups to be viewed **/
    private AcmeSystem _currentSystem;
    /** The diagram editor that is viewing the system **/
    private DiagramEditor _currentDiagramEditor;

    /** The elements selected in the diagram editor **/
    private AcmeElement[] _selection;

    /** The action used to create new groups **/
    private GroupAction _newGroupAction;
    /** The action used to remove groups **/
    private GroupAction _removeGroupAction;

    // The names of fields used to store elements of membership rules
    public static final String RULES_FIELD = "rulePieces";
    public static final String TYPES_FIELD = "discriminatingTypes";
    public static final String INCLUDEATTACHMENTS = "includeAttachments";
    public static final String INCLUDEPARENT = "includeParent";
    public static final String INCLUDECHILDREN = "includeChildren";
	public static final String INCLUDE_TRIGGER = "includeTrigger";

    protected AcmeGroup _selectedGroup;


    public AcmeGroupView () {
        super ();
        GroupManager manager = new GroupManager ();
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
     */
    public void createPartControl(Composite parent) {
        
        Composite top = new Composite (parent, SWT.NONE);
        top.setLayout (new GridLayout (2, false));
        _lstGroups = new List (top, SWT.V_SCROLL);
        
        // The groups part
        _lstGroups.setLayoutData (new GridData (GridData.FILL_BOTH));
        _lstGroups.addSelectionListener(new SelectionAdapter () {
            public void widgetSelected (SelectionEvent evt) {
                String [] selection = _lstGroups.getSelection();
                if (selection.length > 0) {
                    _selectedGroup = (AcmeGroup )_lstGroups.getData(selection [0]);
                    _propertyList.setInput(_selectedGroup);
                    fillMembers (_selectedGroup);
                    _removeGroupAction.setSelectedGroups(new AcmeElement [] {_selectedGroup});
                }
                else {
                    _propertyList.setInput (null);
                    fillMembers (null);
                }
            }
        });
        
        // The members part
        TabFolder folder = new TabFolder (top, SWT.NONE);
        folder.setLayoutData (new GridData (GridData.FILL_BOTH));
        
        TabItem membersTab = new TabItem (folder, SWT.NONE, 0);
        membersTab.setText ("Members");
        _lstMembers = new List (folder, SWT.V_SCROLL);
        membersTab.setControl (_lstMembers);
        
        TabItem propertyTab = new TabItem (folder, SWT.NONE, 1);
        propertyTab.setText("Properties");
        _propertyList = new PropertyList ();
        _propertyList.createControl (folder, SWT.NONE, thisEVP);
        _propertyList.addFilter("NoArmani", new ArmaniRulesFilter (true));
        _propertyList.addFilter("NoVis", new PropertyFilter (new String [] {"vis-"}, true, false));
        propertyTab.setControl (_propertyList.getComposite ());

        getSite ().getPage ().addSelectionListener (this);
        
        createContributions ();
        // prime the selection
        selectionChanged (ASContext.current().getActiveEditor().getDiagramEditor(), getSite ().getPage ().getSelection());
        AcmeStudioChangeManager.registerListener(this);
    }

    /**
     * Displays the members of the selected group to the list of members widget
     * @param group The group to display members from
     */
    protected void fillMembers(AcmeGroup group) {
        _lstMembers.removeAll();
        if (group == null) return;
        AcmeEnumeration members = group.getMembers();
        while (members.moreItems()) {
            AcmeElement element = (AcmeElement) members.getNextItem();
            _lstMembers.add (element.getQualifiedName());
            _lstMembers.setData (element.getQualifiedName (), element);
        }
    }

    /**
     * Creates the contributions out of actions to add to the toolbar
     */
    private void createContributions() {
        IToolBarManager toolbar = getViewSite().getActionBars ().getToolBarManager ();
        
        // Copies all selected elements as members into the current group
        Action copySelectionToGroupAction = new Action () {
            public void run () {
                if (_selection == null || _lstGroups.getSelection () == null ||
                        _selection.length == 0 || _lstGroups.getSelection().length == 0)
                    return;
                AcmeGroup group = (AcmeGroup )_lstGroups.getData (_lstGroups.getSelection () [0]);
                for (int i = 0; i < _selection.length; i++) {
                    try {
                        group.addMember (_selection [i]);
                        AcmeStudioChangeManager.globalChangeManager.onAddMember(group, _selection [i], this);
                    } catch (AcmeIllegalNameException e) {
                        e.printStackTrace();
                    }
                }
            }
           
        };
        copySelectionToGroupAction.setText("Copy selection to group");
        copySelectionToGroupAction.setToolTipText("Copies the selected elements in the diagram to the selected group");
        copySelectionToGroupAction.setImageDescriptor (ImageManager.getImageDescriptor(ImageManager.ID_COPY_TO_GROUP));
        
        // Removes all selected members from the group
        Action copyFromGroupAction = new Action () {
          public void run () {
              if (_lstGroups.getSelection () == null || _lstGroups.getSelection ().length == 0) return;
              AcmeGroup group = (AcmeGroup )_lstGroups.getData (_lstGroups.getSelection() [0]);
              String [] selection = _lstMembers.getSelection ();
              for (int i = 0; i < selection.length; i++) {
                AcmeElement element = (AcmeElement )_lstMembers.getData(selection [i]);
                group.removeMember(element);
                AcmeStudioChangeManager.globalChangeManager.onRemoveMember(group, element, this);
            }
          }
        };
        copyFromGroupAction.setText ("Remove members");
        copyFromGroupAction.setToolTipText("Removes elements from a group");
        copyFromGroupAction.setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_COPY_FROM_GROUP));
        
        // Highlights members of the selected group in the diagram
        Action highlightGroupAction = new Action () {
            public void run () {
                if (_lstGroups.getSelection () == null || _lstGroups.getSelection().length == 0) return;
                if (_currentDiagramEditor != null) {
                    // This action is part of the editor, so get the action corresponding to the editor
                    HighlightObjectsInDiagramAction highlightAction = (HighlightObjectsInDiagramAction) _currentDiagramEditor.getActionRegistry().getAction(HighlightObjectsInDiagramAction.ID);
                    if (highlightAction != null) {
                        java.util.List members = new ArrayList ();
                        for (int i = 0; i < _lstGroups.getSelection ().length; i++) {
                            AcmeGroup group = (AcmeGroup )_lstGroups.getData (_lstGroups.getSelection ()[i]);
                            members.addAll(Helper.convertAcmeEnumerationToList(group.getMembers ()));
                        }

                        highlightAction.setSelection (convertToSelection (members));
                        highlightAction.run ();
                    }
                }
            }
            
        };
        highlightGroupAction.setText ("Highlight in Diagram");
        highlightGroupAction.setToolTipText("Highlights the members of the selected group in the diagram");
        highlightGroupAction.setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_HIGHLIGHT));
        
        // Edits rules associated with the group
        Action editRuleAction = new Action () {
            public void run () {
                if (_lstGroups.getSelection() == null || _lstGroups.getSelection().length == 0) return;
                AcmeGroup group = (AcmeGroup )_lstGroups.getData (_lstGroups.getSelection () [0]);
                GroupRuleWizard wizard = new GroupRuleWizard (group);
                WizardDialog dialog = new WizardDialog (Display.getCurrent ().getActiveShell(), wizard);
                dialog.create();
                if (dialog.open () == WizardDialog.OK) {
                    
                    
                }
            }
        };
        editRuleAction.setText("Edit membership rule");
        editRuleAction.setToolTipText("Edits the membership rule for this group");
        editRuleAction.setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_GROUP_RULES));
        
        // adds the actions to the toolbar
       toolbar.add(copySelectionToGroupAction);
       toolbar.add(copyFromGroupAction);
        toolbar.add(highlightGroupAction);
        toolbar.add (editRuleAction);
        toolbar.add (new Separator ());
        
        // add group actions to the toolbar
        _newGroupAction = new GroupAction (GroupCommand.ADD_GROUP);
        toolbar.add (_newGroupAction);
        _removeGroupAction = new GroupAction (GroupCommand.REMOVE_GROUP);
        toolbar.add (_removeGroupAction);
    }

    /**
     * Converts a list of elements to an ISelection
     * @param members
     * @return
     */
    protected ISelection convertToSelection(java.util.List members) {
        StructuredSelection selection = new StructuredSelection (members);
        return selection;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchPart#setFocus()
     */
    public void setFocus() {

    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IWorkbenchPart part, ISelection selection) {
        if (!(part instanceof MultiPageAcmeEditor)) {
            _currentDiagramEditor = null;
            return;
        }
        MultiPageAcmeEditor mpe = (MultiPageAcmeEditor )part;
        if (mpe.getActiveEditor () instanceof AcmeStyleEditor) {
        	AcmeStyleEditor mpase = (AcmeStyleEditor )mpe.getActiveEditor ();
        	if (mpase.getActiveEditor () instanceof DiagramEditor) {
        		_currentDiagramEditor = (DiagramEditor )mpase.getActiveEditor ();
        	}
        	else {
        		_currentDiagramEditor = null;
        		return;
        	}
        }
        else {
        	if (!(mpe.getActiveEditor() instanceof DiagramEditor)) {
            _currentDiagramEditor = null;
            return;
        }
        _currentDiagramEditor = (DiagramEditor )mpe.getActiveEditor();
        }
        AcmeElement [] elements = Helper.getAcmeElementsFromSelection(selection);
        if (elements == null || elements.length == 0) {
            
        }
        else {
            _selection = elements;
            AcmeSystem system = Helper.getAcmeSystem(elements [0]);
            _newGroupAction.setSystem(system);
            _removeGroupAction.setSystem(system);
            if (system != _currentSystem) {
                _currentSystem = system;
                populateGroups (system);
            }
        }
        
    }

    /**
     * Adds all the groups defined in the system
     * @param system
     */
    private void populateGroups(AcmeSystem system) {
        AcmeEnumeration groups = system.getGroups();
        _lstGroups.removeAll();
        while (groups.moreItems()) {
            AcmeGroup group = (AcmeGroup) groups.getNextItem();
            _lstGroups.add (group.getName ());
            _lstGroups.setData (group.getName (), group);
        }
    }

    /* (non-Javadoc)
     * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
     */
    public void changeModel(AcmeStudioChangeEvent evt) {
        if (evt.getData() != null && _currentSystem != null && Helper.eventRelatedToDesign(evt, _currentSystem.getParentDesign()))
        switch (evt.getEventType()) {
            case AcmeStudioChangeManager.ADD_GROUP:
            case AcmeStudioChangeManager.REMOVE_GROUP:
                populateGroups (_currentSystem);
                break;
            case AcmeStudioChangeManager.ADD_MEMBER:
            case AcmeStudioChangeManager.REMOVE_MEMBER:
                if (evt.getTarget() == _selectedGroup) {
                    fillMembers (_selectedGroup);
                }
                break;
        }
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.util.AcmeModelChangeListener#interestedInEvent(int)
	 */
	public boolean interestedInEvent(int et) {
		return et == AcmeStudioChangeManager.ADD_GROUP ||
			et == AcmeStudioChangeManager.REMOVE_GROUP ||
			et == AcmeStudioChangeManager.ADD_MEMBER ||
			et == AcmeStudioChangeManager.REMOVE_MEMBER;
	}
    
    


}

