/*
 * Created on Mar 31, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package AcmeStudio.ui.views;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.ListenerList;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.part.Page;
import org.eclipse.ui.part.PageBook;

import AcmeStudio.core.ASContext;
import AcmeStudio.core.ASModel;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.model.SetElementNameCommand;
import AcmeStudio.ui.views.parts.ArmaniRulesFilter;
import AcmeStudio.ui.views.parts.ElementViewPart;
import AcmeStudio.ui.views.parts.MaintainedTypesList;
import AcmeStudio.ui.views.parts.PropertyFilter;
import AcmeStudio.ui.views.parts.PropertyList;
import AcmeStudio.ui.views.parts.RepresentationList;
import AcmeStudio.ui.views.parts.RuleList;
import AcmeStudio.ui.views.parts.StructureList;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.Debug;
import AcmeStudio.util.DesignEvaluationCache;
import AcmeStudio.util.Listeners;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.AcmeObject;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeFamily;

/**
 * @author Bradley Schmerl (schmerl@cs.cmu.edu)
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 *
 * ElementViewPage implements the page that shows details of particular elements
 * in an Acme design, such as properties, rules, and sub-structure. The
 * view should track the current selection in the editor, and display the 
 * details. The ElementView manages the following lists:
 *      1. Property List: displays a table showing the properties of an element
 *      2. Rule List: the list of Armani rules associated with an element
 *      3. Structure list: the list of the substructure of the element
 */

public class ElementViewPage
extends Page
implements IElementViewPage, ISelectionChangedListener, AcmeModelChangeListener {

    private class ElementItem {
        ElementViewPart part;
        TabItem tabItem;
        
        public ElementItem (ElementViewPart part, TabItem item) {
        	tabItem = item;
        	this.part = part;
        }
    }

    /** The element currently being viewed **/
    protected AcmeElement _selectedObject = null;

    /** The parent containing this view **/
    private Composite _parent = null;
    /** The name of the element **/
    private Text _name = null;

    private Text _comment = null;

    /** The tabbed folder containing all sub views **/
    private TabFolder _folder = null;

    private ListenerList selectionChangedListeners = new ListenerList();

    private java.util.List _views = new ArrayList ();
    private Map _idToView = new HashMap ();

    /** The property list view **/
    private PropertyList _propertyList = null;

    /** The rule list view **/
    private RuleList _rulesList = null;

    /** The structure list view **/
    private StructureList _structureList = null;

    /** The types list view **/
    private MaintainedTypesList _maintainedTypeList = null;
    
    /** The represenation list view **/
    private RepresentationList _representationList = null;

    /** The toolbar manager of this **/
    private IToolBarManager _myToolBar = null;

    /**
     * Create a new Element View page.
     * @deprecated use #ElementViewPage(ASDesign) instead
     */
    public ElementViewPage (AcmeDesign design) {
        super();
        setAcmeDesign(design);
    }

    /**
     * Create a new Element View page.
     */
    public ElementViewPage (ASDesign design) {
        super();
        setDesign(design);
    }

    /**
     * The <code>ElementViewPage</code> implementation of this 
     * <code>IPageBookViewPage</code> method registers itself as a selection 
     * provider with the site.
     */
    public void init(IPageSite pageSite) {
        super.init(pageSite);
        pageSite.setSelectionProvider(this);
    }

    /**
     * The <code>ElementViewPage</code> implementation of this 
     * <code>IElementViewPage</code> method does nothing, the subclass needs
     * to implement it.
     * 
     * @param parent The place to put the views.
     */
	public void createControl(Composite parent) {
        if (parent instanceof PageBook) {
        }
		if (parent.isDisposed()) return;
        _parent = new Composite(parent, SWT.NONE);

        // Create layout
        GridLayout layout = new GridLayout(4, false);
        _parent.setLayout(layout);

        // Create widgets
        Label nL = new Label (_parent, SWT.NONE);
        nL.setText ("Name: ");
        // two text widgets & property tab widget
        _name = new Text(_parent, SWT.SINGLE | SWT.BORDER);
        GridData gdName = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
        gdName.widthHint = 200;
        _name.setLayoutData(gdName);
        // Define behavior for the name text
        addNameSelectionListeners();

        // Contains comments for the element
        Label cL = new Label (_parent, SWT.NONE);
        cL.setText ("Description");
        _comment = new Text(_parent, SWT.SINGLE | SWT.BORDER);
        GridData gdComment = new GridData(GridData.FILL_HORIZONTAL);
        gdComment.widthHint = 50;
        _comment.setLayoutData(gdComment);
        addCommentSelectionListeners();

        _folder = new TabFolder(_parent, SWT.NONE);
        GridData gdFolder = new GridData(GridData.FILL_BOTH);
        gdFolder.horizontalSpan = GridData.FILL_HORIZONTAL;
        _folder.setLayoutData(gdFolder);

        createTabs();

        // Set the toolbar
        _myToolBar = getSite().getActionBars().getToolBarManager();

        // Add this as a listener to changes in the Acme model
        AcmeStudioChangeManager.registerListener(this);

        // Set up default contributions and the contribution
        // manager which changes the contributions when tabs are
        // changed
        setupContributionManager();
        setEnablement (_selectedObject != null);
    }

    /**
     * Creates all the tabs for the element view, including extensions
     *
     */
    protected void createTabs () {
        // Create the property tab
        TabItem propertyTab = null;
        propertyTab = new TabItem(getFolder(), SWT.NONE, 0);
        propertyTab.setText("Properties");


        _propertyList = new PropertyList();
        _propertyList.createControl (getFolder(), SWT.NONE, this);
        propertyTab.setData (_propertyList); 
        // Filter out Armani constraint properties (should be none anyway, as
        // the design is translated from Acme to Armani by default)
        _propertyList.addFilter("NoArmani", new ArmaniRulesFilter(true));
        // Filter out AcmeStudio visual properties
        _propertyList.addFilter(
            "NoVis",
            new PropertyFilter(new String[] { "vis-" }, true, false));
        propertyTab.setControl(_propertyList.getComposite ());
        _views.add (new ElementItem (_propertyList, propertyTab));
        _idToView.put ("AcmeStudio.PropertyList", new ElementItem (_propertyList, propertyTab));

        TabItem rulesTab = new TabItem(getFolder(), SWT.NONE, 1);
        rulesTab.setText("Rules");
        
        _rulesList = new RuleList();
        _rulesList.createControl (getFolder(), SWT.WRAP, this);
        rulesTab.setControl(_rulesList.getComposite ());
        rulesTab.setData (_rulesList);
        _views.add (new ElementItem (_rulesList, rulesTab));
        _idToView.put ("AcmeStudio.RuleList", new ElementItem (_rulesList, rulesTab));

        _structureList = new StructureList();
        _structureList.createControl (getFolder(), SWT.NONE, this);
        TabItem item = createTabItem (_structureList, "Structure", 2);
        _idToView.put ("AcmeStudio.StructureList", new ElementItem (_structureList, item));
        
        _maintainedTypeList = new MaintainedTypesList ();
        _maintainedTypeList.createControl (getFolder(), SWT.NONE, this);
        item = createTabItem (_maintainedTypeList, "Types", 3);
        _idToView.put ("AcmeStudio.TypeList", new ElementItem (_maintainedTypeList, item));
        
        _representationList = new RepresentationList();
        _representationList.createControl(getFolder(), SWT.WRAP, this);
        item  = createTabItem (_representationList, "Representations", 4);
        _idToView.put ("AcmeStudio.RepresentationList", new ElementItem (_representationList, item));

        createExtensions ();
    }


    /**
     * Creates the tab item for a particular part
     * @param part The element view part to control
     * @param title The title to give the part
     * @param index The index to add the item
     */
    protected TabItem createTabItem (ElementViewPart part, String title,  int index) {
        TabItem ti = new TabItem (getFolder (), SWT.NONE, index);
        ti.setText (title);
        ti.setControl(part.getComposite ());
        ti.setData (part);
        _views.add (new ElementItem (part, ti));
        return ti;
    }

    /**
     * Processes extensions of the type AcmeStudio.elementView
     * This extension allows the addition of element views parts to the element view,
     * and adding filters to filter out properties from the property view
     *
     */
    protected void createExtensions() {
        try {
            IExtensionRegistry registry = Platform.getExtensionRegistry();
            IExtensionPoint point = registry.getExtensionPoint("AcmeStudio.elementViews");
            IExtension[] extensions = point.getExtensions();
            for (int i = 0; i < extensions.length; i++) {
                IConfigurationElement[] elements = extensions[i].getConfigurationElements();
                for (int j = 0; j < elements.length; j++) {
                    if ("elementView".equals(elements[j].getName())) {
                        ElementViewPart object = (ElementViewPart)elements[j].createExecutableExtension("class");
                        object.createControl(getFolder(), SWT.NONE, this);
                        String title = elements[j].getAttribute("name");
                        TabItem item = createTabItem(object, title, 5);
                        _idToView.put(elements [j].getAttribute ("id"), new ElementItem (object, item));
                        processElementView (elements [j]);
                        
                    }
                    else if ("propertyFilter".equals (elements [j].getName ())) {
                        String name = elements [j].getAttribute("name");
                        IConfigurationElement [] filters = elements [j].getChildren();
                        if (filters.length == 1) {
                            IConfigurationElement filter = filters [0];
                            String pattern = filter.getAttribute ("pattern");
                            _propertyList.addFilter (name, new PropertyFilter (new String [] {pattern}, true, false));   
                        }     
                    }
                }
                
            }
        }
        catch (CoreException e) {
            Debug.errorln(e);
        }

    }

    /**
     * Processes the substructure of the element view, which adds property filters
     * to the property view
     * @param element The elementView element
     */
    private void processElementView(IConfigurationElement element) {
        IConfigurationElement [] elements = element.getChildren();
        for (int i = 0; i < elements.length; i++) {
            if ("propertyFilter".equals (elements [i].getName())) {
                IConfigurationElement [] filters = element.getChildren("filter");
                String [] patterns = new String [filters.length];
                for (int j = 0; j < filters.length; j++) {
                    patterns [j] = filters [j].getAttribute("filter");
                }
                _propertyList.addFilter(element.getAttribute ("name"), new PropertyFilter (patterns, true, false));
            }
        }   
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.IPage#dispose()
     */
    public void dispose() {
        AcmeStudioChangeManager.removeListener(this);
        if (_name != null) {
            _name.dispose();
            _name = null;
        }
        if (_comment != null) {
            _comment.dispose();
            _comment = null;
        } 
        if (_idToView != null) {
            _idToView.clear();
            _idToView = null;
        }

        if (_views != null) {
            Iterator i = _views.iterator();
            while (i.hasNext ()) {
            	ElementItem ei = (ElementItem )i.next ();
                ElementViewPart part = ei.part;
                part.dispose();
            }
            _views.clear();
            _views = null;
        }
        if (_propertyList != null) {
            _propertyList.dispose();
            _propertyList = null;
        } 
        if (_rulesList != null) {
            _rulesList.dispose();
            _rulesList = null;
        } 
        if (_structureList != null) {
            _structureList.dispose();
            _structureList = null;
        } 
        if (_maintainedTypeList != null) {
            _maintainedTypeList.dispose();
            _maintainedTypeList = null;
        } 
        if (_representationList != null) {
            _representationList.dispose();
            _representationList = null;
        } 

        if (_folder != null) {
            _folder.dispose();
            _folder = null;
        }
        _selectedObject = null;
        _myToolBar = null;
        _parent = null;

        if (_folder != null) _folder.dispose();

        super.dispose();
    }

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.IPage#getControl()
	 */
    public Control getControl() {
        return _parent;
    }

    /**
     * Returns the Folder containing the various element folder panes.
     * @return TabFolder the tab folder.
     */
    public TabFolder getFolder() {
        return _folder;
    }

    /**
     * Passes the focus request to the viewer's control.
     * @see org.eclipse.ui.part.IPage#setFocus()
	 */
	public void setFocus() {
        if (getActiveTabControl() != null) {
            getActiveTabControl().setFocus();
        }
	}

    /**
     * Responds to a SelectionChangedEvent by setting the focus on the
     * first selection in the selection list.
     * @param event the SelectionChangedEvent to respond to
     * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(SelectionChangedEvent)
     */
    public void selectionChanged(SelectionChangedEvent event) {
        if (_parent == null) {  // page not initialized
            return;
        }
        if (event.getSelection() instanceof IStructuredSelection) {
            IStructuredSelection iss =
                (IStructuredSelection) event.getSelection();
            if (iss.size() == 1 && iss.getFirstElement () instanceof AcmeElement) {
                AcmeElement element = (AcmeElement) iss.getFirstElement();
                this.setFocus(element);
                
                // Force typecheck of selected elements
                DesignEvaluationCache cache = ASModel.current().getRuleManager();
                cache.typecheck (element);
            }
        }

        fireSelectionChanged(event.getSelection());
    }

    /**
     * Responds to changes in the model by updating the appropriate
     * views (e.g., if adding a property, tell the property view about it)
     * @param evt The model change event
     * @see AcmeStudio.model.AcmeModelChangeListener#changeModel(AcmeStudioChangeEvent)
     * 
     */
	public void changeModel(AcmeStudioChangeEvent evt) {
        if (_parent == null) {  // page not initialized
            return;
        }
        // Only if this was not the originator of the change and the element view is
        // focused on the changed object
        if (evt.getAnnouncer() != this && evt.getTarget() == _selectedObject) {
            switch (evt.getEventType()) {
                case AcmeStudioChangeManager.SET_NAME:
                    // Change the name;
                    getName ().setText ((String) evt.getData());
                    break;
                case AcmeStudioChangeManager.SET_COMMENT:
                    getComment ().setText ((String )evt.getData ());
                    break;
                default:
                    Iterator i = _views.iterator ();
                    while (i.hasNext ()) {
                    	ElementItem ei = (ElementItem )i.next ();
                        ElementViewPart part = ei.part;
                        part.setInput(_selectedObject);
                        calculateVisibility(ei);
                        this.getControl().redraw();
                    }
            }

        }
	}
	
	public boolean interestedInEvent (int et) {
		return et != AcmeStudioChangeManager.UPDATE_VIS_INFO;
	}

	private void calculateVisibility(ElementItem ei) {
		if (ei.part.isValidFor(_selectedObject)) {
			if (ei.tabItem == null || ei.tabItem.isDisposed()) {
				ei.tabItem = new TabItem (_folder, SWT.NONE);
				// BugFix 181: Make sure we can get at the contributions
				ei.tabItem.setData(ei.part);
				ei.tabItem.setText(ei.part.getTitle ());       
				ei.tabItem.setControl(ei.part.getComposite());               		
			}
		}
		else {
			if (ei.tabItem != null && !ei.tabItem.isDisposed ()) {
				ei.tabItem.dispose ();
				ei.tabItem = null;
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#addSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void addSelectionChangedListener(ISelectionChangedListener listener) {
        selectionChangedListeners.add(listener);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#getSelection()
	 */
	public ISelection getSelection() {
        return null;
	}

    /**
     * @return Text
     */
    public Text getComment() {
        return _comment;
    }

    /**
     * @return Text
     */
    public Text getName() {
        return _name;
    }

    /**
     * Returns the Propert List.
     * @return PropertyList
     */
    public PropertyList getPropertyList() {
        return _propertyList;
    }

    /**
     * Returns the tool bar manager for this page.
     * @return IToolBarManager
     */
    public IToolBarManager getToolBar() {
        return _myToolBar;
    }

    /**
     * Sets current AcmeDesign
     * @param design  new AcmeDesign to set to
     * @deprecated use instead #setDesign(ASDesign) to ensure proper behavior!
     */
    public void setAcmeDesign (AcmeDesign design) {
        ASContext.current().setActiveDesign(design);
    }

    /**
     * Sets current ASDesign
     * @param design  new ASDesign to set to
     */
    public void setDesign (ASDesign design) {
        ASContext.current().setActiveDesign(design);
    }

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#removeSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void removeSelectionChangedListener(ISelectionChangedListener listener) {
        selectionChangedListeners.remove(listener);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#setSelection(org.eclipse.jface.viewers.ISelection)
	 */
	public void setSelection(ISelection selection) {
       
	}

    /**
     * Changes the focus to a new AcmeObject
     * @param o the object to which to change focus
     */
    public void setFocus(AcmeObject o) {

        if (_selectedObject != o) { // only if selection has changed
            if (o instanceof AcmeElement) {
                setEnablement (true);
                _selectedObject = (AcmeElement) o;
                String name = ((AcmeElement) o).getName();
                getName().setText(name == null ? "" : name);
                
                if (_selectedObject instanceof AcmeFamily) {
                    getName().setEditable(false);
                }
                else {
                    getName().setEditable (true);
                }
                
                String comment = ((AcmeElement )o).getComment ();
                if (comment != null && comment.length () > 2 && comment.substring (0, 2).equals ("//")) {
                    comment = comment.substring (2);   
                }
                getComment().setText (comment == null? "": comment);
                
                Iterator i = _views.iterator ();
                while (i.hasNext ()) {   
                	
                	ElementItem ei = (ElementItem )i.next ();
                    ElementViewPart part = ei.part;
                    part.setInput (o); 
                    calculateVisibility(ei);                  
                }                
               
                getToolBar().update(true);
            }
            else {
                setEnablement (false);
            }
        }
        else if (o == null) {
            setEnablement (false);
        }

    }

    /**
     * Fires a selection changed event.
     *
     * @param selction the new selection
     */
    protected void fireSelectionChanged(ISelection selection) {
        // create an event
        final SelectionChangedEvent event = new SelectionChangedEvent(this, selection);

        // fire the event
        Object[] listeners = selectionChangedListeners.getListeners();
        for (int i = 0; i < listeners.length; ++i) {
            final ISelectionChangedListener l = (ISelectionChangedListener)listeners[i];
            Platform.run(new SafeRunnable() {
                public void run() {
                    l.selectionChanged(event);
                }
                public void handleException(Throwable e) {
                    super.handleException(e);
                    //If and unexpected exception happens, remove it
                    //to make sure the workbench keeps running.
                    removeSelectionChangedListener(l);
                }
            });     
        }
    }

    /**
     * @return  the composite corresponding to the current tabbed item
     */
    protected Composite getActiveTabControl() {
        Composite activeTab = null;
        if (_folder == null) return activeTab;
        if (!_folder.isDisposed()) {
            activeTab =
                (Composite) _folder
                    .getItem(_folder.getSelectionIndex())
                    .getControl();
        }
        return activeTab;
    }
    
    /**
     * Adds the contributions to a toolbar
     * @param tbm the tool bar manager to add contributions to
     * @param conts the contribution items to add
     */
    protected void addContributions(
        IToolBarManager tbm,
        IContributionItem[] conts) {
        for (int i = 0; i < conts.length; i++) {
            tbm.add(conts[i]);
        }
    }

    /**
     * Implements changing of contributions when the managed pages
     * change (like between the property list view and the rule list
     * view.
     */
    protected void setupContributionManager() {
        // Set the toolbar to add and remove contributions when different
        // children become visible
        getFolder().addSelectionListener(new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent evt) {
            }

            public void widgetSelected(SelectionEvent evt) {
                if (evt.item instanceof TabItem) {
                    TabItem item = (TabItem) evt.item;
                    getToolBar().removeAll();
                    getToolBar().update(true);
                    if (item.getData() instanceof ElementViewPart) {
                        ElementViewPart part =
                            (ElementViewPart) item.getData ();
                        addContributions(getToolBar(), part.getContributions());
                        getToolBar().update(true);
                    }
                }

            }
        });

        // The property list is the default view, so add its contributions
        addContributions(getToolBar(), getPropertyList().getContributions());

    }

    protected void setEnablement (boolean enable) {
        
        if (getName () != null) getName().setEnabled(enable)   ;
        if (getComment () != null) getComment().setEnabled (enable);
        if (getFolder () != null) getFolder().setEnabled(enable);
        
        Iterator i = _views.iterator ();
        while (i.hasNext ()) {
        	ElementItem ei = (ElementItem )i.next ();
        
            ElementViewPart part = ei.part;
            part.getComposite().setEnabled(enable);   
    }        

    }        

    /**
     * Adds listeners to the getName() field of the view, so that 
     * invalid identifiers cannot be defined, and the name is
     * changed when enter is pressed.
     */
    protected void addNameSelectionListeners() {
        // Allows the name of the element to be changed when the
        // return key is pressed
        getName().addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
            }

            public void widgetDefaultSelected(SelectionEvent evt) {
                String newName = getName().getText();
                SetElementNameCommand cmd = new SetElementNameCommand ("Change name", newName, _selectedObject.getName (), _selectedObject);
                cmd.execute (_selectedObject.getParentDesign());
                try {
                	cmd.checkError();
                }
                catch (Exception e) {
                    // Just leave the name as it is
                    getName().setText(_selectedObject.getName());
                    // BugFix #95
                    // Need to tell the user if the name is illegal
                    MessageDialog.openError(Display.getDefault().getActiveShell(), "Naming error", e.getMessage());
                }
                
            }
        });

        // Checks each change to make sure that it is a legal Acme
        // name
        getName().addVerifyListener(Listeners.VERIFY_ACMEID);
    }
    
    /**
     * Adds listeners to the getComment() field of the view.
     */
    protected void addCommentSelectionListeners() {
        getComment().addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
            }

            public void widgetDefaultSelected(SelectionEvent evt) {
                String newComment = getComment().getText();
                _selectedObject.setComment(newComment);
                AcmeStudioChangeManager.globalChangeManager.onSetComment(_selectedObject, newComment, ElementViewPage.this);
                
            }
        });
    }

}
