/*
 * Created on Oct 28, 2003
 * 
 */
package AcmeStudio.ui.views.groups;

import java.util.*;

import AcmeStudio.ui.editors.diagram.styles.VisVariant;
import AcmeStudio.ui.views.AcmeGroupView;
import AcmeStudio.ui.views.parts.PropertyValueInspector;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.extensions.IElementExtension;
import acmeLib.elements.*;
import acmeLib.properties.*;
import acmeLib.tools.GraphWalker;
import acmeLib.tools.WalkerElementOperation;

/**
 * @author Bradley Schmerl
 * 
 * Manages group membership, and responds to changes by updating group membership
 */
public class GroupManager implements AcmeModelChangeListener {
    
    /**
     * @author schmerl
     *
     * To change the template for this generated type comment go to
     * Window - Preferences - Java - Code Generation - Code and Comments
     */
    private static class MembershipDependency implements IElementExtension {

        public Map _dependentToSources = new HashMap ();
        public Map _sourceToDependents = new HashMap ();
        
        /* (non-Javadoc)
         * @see acmeLib.core.extensions.IElementExtension#isPersistent()
         */
        public boolean isPersistent() {
            return false;
        }

        /* (non-Javadoc)
         * @see acmeLib.core.extensions.IElementExtension#injectIntoElement(acmeLib.elements.AcmeElement)
         */
        public void injectIntoElement(AcmeElement element) {
        }

        /* (non-Javadoc)
         * @see acmeLib.core.extensions.IElementExtension#extractFromElement(acmeLib.elements.AcmeElement)
         */
        public void extractFromElement(AcmeElement element) {
        }

        /* (non-Javadoc)
         * @see acmeLib.core.extensions.IElementExtension#isInjected(acmeLib.elements.AcmeElement)
         */
        public boolean isInjected(AcmeElement element) {
            return false;
        }

        public void addDependency (AcmeElement source, AcmeElement dependent) {
            List dependentsList = (List )_dependentToSources.get (dependent);
            List sourceList = (List )_sourceToDependents.get (source);
            if (dependentsList == null) {
                dependentsList = new ArrayList (1);
                _dependentToSources.put (dependent, dependentsList);
            }
            if (sourceList == null) {
                sourceList = new ArrayList (1);
                _sourceToDependents.put (source, sourceList);
            }
            dependentsList.add (source);
            sourceList.add (dependent);
        }
        
        public List getDependents (AcmeElement source) {
            return (List )_sourceToDependents.get(source);
        }
        
        public List getSources (AcmeElement dependent) {
            return (List )_dependentToSources.get (dependent);
        }
    }
    
    
    public GroupManager () {
        AcmeStudioChangeManager.registerListener(this);
    }
    
    
    /* (non-Javadoc)
      * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
      */
     public void changeModel(AcmeStudioChangeEvent evt) {
         AcmeSystem system = AcmeStudioChangeManager.getSystemFromEvent (evt);
         AcmeElement affectedElement = getAffectedMember (evt); 
         
         
         if (system != null && affectedElement != null) {
             AcmeEnumeration groups = system.getGroups ();
             while (groups.moreItems()) {
                AcmeGroup group = (AcmeGroup) groups.getNextItem();
                AcmeProperty compRule = group.getProperty ("vis-component-rule");
                AcmeProperty connRule = group.getProperty ("vis-connector-rule");
                AcmeProperty portRule = group.getProperty ("vis-port-rule");
                AcmeProperty roleRule = group.getProperty ("vis-role-rule");
                switch (evt.getEventType ()) {
                    case AcmeStudioChangeManager.REMOVE_COMPONENT:
                    case AcmeStudioChangeManager.REMOVE_CONNECTOR:
                    case AcmeStudioChangeManager.REMOVE_PORT:
                    case AcmeStudioChangeManager.REMOVE_ROLE:
                        if (group.containsMember(affectedElement)) {
                            removeFromGroup (group, affectedElement);
                        }
                        break;
                    case AcmeStudioChangeManager.ADD_COMPONENT:
                    case AcmeStudioChangeManager.ADD_CONNECTOR:
                    case AcmeStudioChangeManager.ADD_PORT:
                    case AcmeStudioChangeManager.ADD_ROLE:
                        addToGroupIfApplicable(group, affectedElement, system, compRule, connRule, portRule, roleRule);
                        break;
                    case AcmeStudioChangeManager.ADD_PROPERTY:
                    case AcmeStudioChangeManager.REMOVE_PROPERTY:
                    case AcmeStudioChangeManager.SET_PROPERTY:
                    	if (group.containsMember (affectedElement)) {
                    		removeFromGroup (group, affectedElement);
                    	}
                    	addToGroupIfApplicable(group, affectedElement, system, compRule, connRule, portRule, roleRule);
                    	
                }
            }
         }
         
     }

     public boolean interestedInEvent (int et) {
     	switch (et) {
     		case AcmeStudioChangeManager.REMOVE_COMPONENT:
            case AcmeStudioChangeManager.REMOVE_CONNECTOR:
            case AcmeStudioChangeManager.REMOVE_PORT:
            case AcmeStudioChangeManager.REMOVE_ROLE:
            case AcmeStudioChangeManager.ADD_COMPONENT:
            case AcmeStudioChangeManager.ADD_CONNECTOR:
            case AcmeStudioChangeManager.ADD_PORT:
            case AcmeStudioChangeManager.ADD_ROLE:
            case AcmeStudioChangeManager.ADD_PROPERTY:
            case AcmeStudioChangeManager.REMOVE_PROPERTY:
            case AcmeStudioChangeManager.SET_PROPERTY:
            	return true;
     	}
     	return false;
     }
  
     /**
     * @param group
     * @param affectedElement
     */
    private void removeFromGroup(AcmeGroup group, AcmeElement element) {
        group.removeMember(element);
        AcmeStudioChangeManager.globalChangeManager.onRemoveMember(group, element, null);
        MembershipDependency deps = (MembershipDependency )group.getUserData("MembershipDependencies");
        if (deps != null) {
            List dependents = deps.getDependents(element);
            for (Iterator iter = dependents.iterator(); iter.hasNext();) {
                AcmeElement dep = (AcmeElement) iter.next();
                if (deps.getSources(dep) != null) {
                    deps.getSources (dep).remove(element);
                    if (deps.getSources (dep).isEmpty()) {
                        group.removeMember (dep);
                        AcmeStudioChangeManager.globalChangeManager.onRemoveMember (group, dep, null);
                    }
                }
            }
        }
    }


    protected static void addMembershipDependency (AcmeGroup group, AcmeElement member, AcmeElement dependent) {
         if (isSourceMember (group, dependent)) return;
         MembershipDependency deps = null;
         if (group.getUserData ("MembershipDependencies") == null) {
             deps = new MembershipDependency ();
             group.setUserData ("MembershipDependencies", deps);
         }
         else {
             deps = (MembershipDependency )group.getUserData("MembershipDependencies");
         }
         
         deps.addDependency(member, dependent);
     }
     
    /**
     * @param group
     * @param dependent
     * @return
     */
    private static boolean isSourceMember(AcmeGroup group, AcmeElement dependent) {
        if (group.containsMember (dependent)) {
            MembershipDependency deps = (MembershipDependency )group.getUserData("MembershipDependencies");
            if (deps != null) {
                if (deps.getSources(dependent) != null) {
                    return true;
                }
            
            }
        }
        return false;
    }


    /**
     * @param evt
     * @return
     */
    private AcmeElement getAffectedMember(AcmeStudioChangeEvent evt) {
        switch (evt.getEventType()) {
            case AcmeStudioChangeManager.ADD_COMPONENT:
            case AcmeStudioChangeManager.ADD_CONNECTOR:
            case AcmeStudioChangeManager.ADD_PORT:
            case AcmeStudioChangeManager.ADD_ROLE:
            case AcmeStudioChangeManager.REMOVE_COMPONENT:
            case AcmeStudioChangeManager.REMOVE_CONNECTOR:
            case AcmeStudioChangeManager.REMOVE_ROLE:
            case AcmeStudioChangeManager.REMOVE_PORT:
                if (evt.getSource()  instanceof AcmeElement) {
                    AcmeElement element = (AcmeElement) evt.getSource() ;
                    return element;
                }
                break;
            case AcmeStudioChangeManager.ADD_PROPERTY:
            case AcmeStudioChangeManager.REMOVE_PROPERTY:
            case AcmeStudioChangeManager.SET_PROPERTY:
                if (evt.getTarget () instanceof AcmeElement) {
                    return (AcmeElement )evt.getTarget ();
                }
                break;
            case AcmeStudioChangeManager.ASSIGN_TYPE:
            case AcmeStudioChangeManager.UNASSIGN_TYPE:
                if (evt.getTarget () instanceof AcmeElement) { 
                    return (AcmeElement )evt.getTarget ();
                }
                
        }
        return null;
    }


    /**
     * Checks to see if the property rules apply to the element
     * @param elt The element to check
     * @param include The state of inclusion so far
     * @param rule The rule to apply
     * @return whether the rule applies to the element
     */
    protected static boolean applyPropertyRules(AcmeElement elt, boolean include, AcmeRecord rule) {
        AcmeEnumeration rules = ((AcmeSequence )rule.getFieldValue (AcmeGroupView.RULES_FIELD)).getValues ();
        boolean valueInclude = false;
        String conjunction = "or";
        boolean haveRules = false;
        while (rules.moreItems()) {
            haveRules = true;
            AcmeString ar = (AcmeString) rules.getNextItem();
            StringTokenizer tokens = new StringTokenizer (ar.getValue (), ":");
            String property = tokens.nextToken ();
            String operator = tokens.nextToken();
            String value = tokens.nextToken ();
                    
            AcmeProperty prop = elt.getProperty(property);
            if (prop != null) {
                try {
                    if (prop.getType () instanceof StringType) {
                        value = "\"" + value + "\"";
                    }
                    AcmePropertyValue valueAcme = PropertyValueInspector.getPropertyValue(value);
                    boolean applies = VisVariant.appliesTo(prop.getValue (), valueAcme, VisVariant.OPNAMES.indexOf(operator));
                    if (conjunction.equals ("or")) {
                        valueInclude |= applies;
                    }
                    else {
                        valueInclude &= applies;
                    }
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            conjunction = tokens.hasMoreTokens()?tokens.nextToken ():"";
        }
        if (haveRules) include &= valueInclude;
        return include;
    }
    
    /**
     * Applies the type rules to the element
     * @param elt The element being checked for inclusion
     * @param include The state of inclusion so far
     * @param rule The rule to apply
     * @return Whether the element declares types indicated by the rule
     */
    protected static boolean applyTypeRules(AcmeElement elt, boolean include, AcmeRecord rule) {
        AcmeEnumeration types = ((AcmeSet )rule.getFieldValue(AcmeGroupView.TYPES_FIELD)).getValues ();
        if (types.getSize () == 0) include = true;
        else {
            while (types.moreItems()) {
                AcmeString aName = (AcmeString) types.getNextItem();
                String typeName = aName.getValue ();
                try {
                    AcmeElementType type = (AcmeElementType )elt.lookupName(typeName, true);
                    include |= elt.declaresType(type) | elt.instantiatesType(type);
                }
                catch (ClassCastException e) {}
            }
        }
        return include;
    }
            
    protected static void addToGroupIfApplicable(final AcmeGroup group, AcmeElement elt, final AcmeSystem system, final AcmeProperty compRule, final AcmeProperty connRule, final AcmeProperty portRule, final AcmeProperty roleRule) {
        // Find the right rule
        AcmeProperty prule = null;
        if (elt instanceof AcmeComponent && compRule != null) {
            prule = compRule;
        }
        else if (elt instanceof AcmeConnector && connRule != null) {
            prule = connRule;
        }
        else if (elt instanceof AcmePort && portRule != null) {
            prule = portRule;
        }
        else if (elt instanceof AcmeRole && roleRule != null) {
            prule = roleRule;
        }
                
        if (prule != null) {
            boolean include = false;
            // Get the rule out of the record
            AcmeRecord rule = (AcmeRecord )prule.getValue();
                    
            // See if it applies
            include = applyTypeRules(elt, include, rule);
            include = applyPropertyRules(elt, include, rule);
                    
            if (include) {
                try {
                    // Add the member
                	AcmeRecordField includeTriggerField = rule.getField(AcmeGroupView.INCLUDE_TRIGGER);
					if (includeTriggerField == null || ((AcmeBoolean )includeTriggerField.getValue ()).getValue()) {
	                    group.addMember (elt);
	                    AcmeStudioChangeManager.globalChangeManager.onAddMember(group, elt, null);
                	}
                            
                    // Add the parent (if indicated in the rule)
                    AcmeBoolean property = (AcmeBoolean )rule.getFieldValue (AcmeGroupView.INCLUDEPARENT);
                    if (property != null && property.getValue()) {
                        group.addMember(elt.getParent());
                        addMembershipDependency (group, elt, elt.getParent ());
                        AcmeStudioChangeManager.globalChangeManager.onAddMember (group, elt.getParent (), null);
                    }
                            
                    // Add attachments (if indicated in the rule)
                    property = (AcmeBoolean )rule.getFieldValue (AcmeGroupView.INCLUDEATTACHMENTS);
                    if (property != null && property.getValue()) {
                        if (elt instanceof AcmePort) {
                            AcmePort port = (AcmePort) elt;
                            AcmeEnumeration roles = system.getAttachments(port);
                            while (roles.moreItems()) {
                                AcmeAttachment att =
                                    (AcmeAttachment) roles.getNextItem();
                                group.addMember (att.getRole ());
                                group.addMember (att.getRole ().getParent ());
                                addMembershipDependency (group, elt, att.getRole ());
                                addMembershipDependency (group, elt, att.getRole ().getParent ());
                                AcmeStudioChangeManager.globalChangeManager.onAddMember (group, att.getRole (), null);
                                AcmeStudioChangeManager.globalChangeManager.onAddMember(group, att.getRole ().getParent (), null);
                            }
                        }
                        else if (elt instanceof AcmeRole) {
                            AcmeRole role = (AcmeRole) elt;
                            AcmeEnumeration ports = system.getAttachments (role);
                            while (ports.moreItems()) {
                                AcmeAttachment att =
                                    (AcmeAttachment )ports.getNextItem();
                                group.addMember (att.getPort ());
                                group.addMember (att.getPort ().getParent ());
                                addMembershipDependency (group, elt, att.getPort ());
                                addMembershipDependency (group, elt, att.getPort ().getParent ());
                                AcmeStudioChangeManager.globalChangeManager.onAddMember (group, att.getPort (), null);
                                AcmeStudioChangeManager.globalChangeManager.onAddMember(group, att.getPort ().getParent (), null);
                            }
                        }
                    }
                            
                    // Add children (if indicated in the rule)
                    property = (AcmeBoolean )rule.getFieldValue (AcmeGroupView.INCLUDECHILDREN);
                    if (property != null && property.getValue()) {
                        AcmeEnumeration children = null;
                        if (elt instanceof  AcmeComponent) {
                            AcmeComponent comp = ( AcmeComponent) elt;
                            children = comp.getPorts ();
                        }
                        else if (elt instanceof AcmeConnector) {
                            AcmeConnector conn = (AcmeConnector) elt;
                            children = conn.getRoles ();
                        }
                        while (children.moreItems()) {
                            AcmePort element =
                            (AcmePort) children.getNextItem();
                            group.addMember (element);
                            addMembershipDependency (group, elt, element);
                            AcmeStudioChangeManager.globalChangeManager.onAddMember(group, element, null);
                        }
                    }   
                } catch (AcmeIllegalNameException e) {
                    e.printStackTrace();
                }       
            }
        }
    } 
    /**
     * Applies membership rules to groups to update their membership
     * @param _group
     */
    public static void updateGroupMembers(final AcmeGroup group) {
        final AcmeSystem system = (AcmeSystem )group.getParent ();
        
        
        final AcmeProperty compRule = group.getProperty ("vis-component-rule");
        final AcmeProperty connRule = group.getProperty ("vis-connector-rule");
        final AcmeProperty portRule = group.getProperty ("vis-port-rule");
        final AcmeProperty roleRule = group.getProperty ("vis-role-rule");
        
        /** The operation to walk over the system **/
        WalkerElementOperation ruleApplicationOp = new WalkerElementOperation () {
    
            public void doElementOp(AcmeElement elt) {
               
                if (elt instanceof AcmeSystem || elt instanceof AcmeGroup) return;
                addToGroupIfApplicable(group, elt, system, compRule, connRule, portRule, roleRule);
            }


    
        };
        
        // Add the members
        GraphWalker.WalkOverAllElements(system, ruleApplicationOp, GraphWalker.WALK_TOP_DOWN);
    }




 
    
    
}
