/*
 * Created on Oct 21, 2003
 * 
 */
package AcmeStudio.ui.views.groups.wizards;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import AcmeStudio.ui.views.AcmeGroupView;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.elements.AcmeGroup;
import acmeLib.properties.AcmeBoolean;

/**
 * @author Bradley Schmerl
 * 
 * Defines the page for defining group membership rules for ports and roles
 */
public class PortRoleSelectionCriteriaPage extends SelectionCriteriaPage {

    /** The category of the parent **/
    private String _parent;
    /** The categories of attachments **/
    private String _attachment;
    /** Allows user to indicate whether to include parents **/
    private Button _btnParent;
    /** Allows user to indicate whether to include attachments **/
    private Button _btnAttachments;

    /**
     * Constructs the page
     * @param category "Port" or "Role"
     * @param group The group to define the rule for
     */
    protected PortRoleSelectionCriteriaPage(String category, AcmeGroup group) {
        super(category, group);
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        super.createControl(parent);
        
        
        _btnParent = new Button (_top, SWT.CHECK);
        _btnParent.setText ("Include parent " + _parent);
        _btnParent.addSelectionListener(new SelectionAdapter () {
           public void widgetSelected (SelectionEvent e) {
               if (_ruleRep.getField (AcmeGroupView.INCLUDEPARENT) == null) {
                   _ruleRep.addField (AcmeGroupView.INCLUDEPARENT, ConstantTypes.BOOLEAN_TYPE, new AcmeBoolean (_btnParent.getSelection()));
               }
               else {
                   _ruleRep.setField(AcmeGroupView.INCLUDEPARENT, new AcmeBoolean (_btnParent.getSelection ()));
               }
           }
        });
        
        _btnAttachments = new Button (_top, SWT.CHECK);
        _btnAttachments.setText ("Include attached " + _attachment);
        _btnAttachments.addSelectionListener(new SelectionAdapter () {
            public void widgetSelected (SelectionEvent e) {
                if (_ruleRep.getField (AcmeGroupView.INCLUDEATTACHMENTS) == null) {
                    _ruleRep.addField (AcmeGroupView.INCLUDEATTACHMENTS, ConstantTypes.BOOLEAN_TYPE, new AcmeBoolean (_btnAttachments.getSelection()));
                }
                else {
                    _ruleRep.setField(AcmeGroupView.INCLUDEATTACHMENTS, new AcmeBoolean (_btnAttachments.getSelection ()));
                }
            }
         });      
        
        fillMyValues ();
        
    }

    /* (non-Javadoc)
     * @see AcmeStudio.ui.views.groups.wizards.SelectionCriteriaPage#setupCategory()
     */
    protected void setupCategory() {
        super.setupCategory();
        if (_category.equals ("Port")) {
            _parent = "component";
            _attachment = "roles and connectors";
        }
        else if (_category.equals ("Role")) {
            _parent = "connector";
            _attachment = "ports and roles";
        }
    }

    /* (non-Javadoc)
     * @see AcmeStudio.ui.views.groups.wizards.SelectionCriteriaPage#fillValues()
     */
    protected void fillMyValues() {
//        super.fillValues();
        if (_ruleRep.getField(AcmeGroupView.INCLUDEPARENT) != null) {
            _btnParent.setSelection(((AcmeBoolean )_ruleRep.getFieldValue(AcmeGroupView.INCLUDEPARENT)).getValue());
        }
        if (_ruleRep.getField (AcmeGroupView.INCLUDEATTACHMENTS) != null) {
            _btnAttachments.setSelection (((AcmeBoolean )_ruleRep.getFieldValue (AcmeGroupView.INCLUDEATTACHMENTS)).getValue ());
        }
    }

}
