/*
 * Created on Oct 21, 2003
 * 
 */
package AcmeStudio.ui.views.groups.wizards;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import acmeLib.elements.AcmeGroup;
import org.eclipse.swt.layout.GridData;

/**
 * @author Bradley Schmerl
 * 
 * Allows the user to indicate which category of elements to define group membership rules for
 */
public class RuleKeyPage extends WizardPage {

    /** The group to define rules for **/
    protected AcmeGroup _group;
    
    /** Allows user to indicate whether to include components **/
    private Button _btnComponents;
    /** Allows user to indicate whether to include connectors **/
     private Button _btnConnectors;
    /** Allows user to indicate whether to include ports **/
     private Button _btnPorts;
    /** Allows user to indicate whether to include roles **/
     private Button _btnRoles;
    
    /**
     * Constructs the page
     * @param group The Acme group to define the rules for
     */
    protected RuleKeyPage(AcmeGroup group) {
        super("Membership key");
        _group = group;
        setTitle ("Select the types of elements to create rules for");
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        Composite top = new Composite (parent, SWT.NONE);
        setControl (top);
        top.setLayout (new GridLayout  (1, false));
        Label l = new Label (top, SWT.NONE);
        l.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        l.setText ("Select the categories of elements to construct rules for:");
        Group bGroup = new Group (top, SWT.NONE);
        bGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        bGroup.setText ("Categories:");
        bGroup.setLayout (new GridLayout (1, false));
        
        SelectionAdapter adapter = new SelectionAdapter () {
          public void widgetSelected (SelectionEvent e) {
              setPageComplete (_btnComponents.getSelection () || _btnConnectors.getSelection () || _btnPorts.getSelection() || _btnRoles.getSelection());
          }
        };
        _btnComponents = new Button (bGroup, SWT.CHECK);
        _btnComponents.setText ("Components");
        _btnComponents.addSelectionListener(adapter);
        
        _btnConnectors = new Button (bGroup, SWT.CHECK);
        _btnConnectors.setText ("Connectors");
        _btnConnectors.addSelectionListener(adapter);
        
        _btnPorts = new Button (bGroup, SWT.CHECK);
        _btnPorts.setText ("Ports");
        _btnPorts.addSelectionListener(adapter);
        
        _btnRoles = new Button (bGroup, SWT.CHECK);
        _btnRoles.setText ("Roles");
        _btnRoles.addSelectionListener(adapter);
        
        _btnComponents.setSelection (_group.getProperty ("vis-component-rule") != null);
        _btnConnectors.setSelection (_group.getProperty ("vis-connector-rule") != null);
        _btnPorts.setSelection (_group.getProperty ("vis-port-rule") != null);
        _btnRoles.setSelection (_group.getProperty ("vis-role-rule") != null);
   }

    /**
     * @return whether the page defines that rules should be defined for components
     */
    public boolean keyOnComponents() {
        return _btnComponents.getSelection();
    }

    /**
     * @return whether the page defines that rules should be defined for connectors
     */
    public boolean keyOnConnectors() {
        return _btnConnectors.getSelection ();
    }

    /**
     * @return whether the page defines that rules should be defined for ports
     */
    public boolean keyOnPorts() {
        return _btnPorts.getSelection ();
    }

    /**
     * @return whether the page defines that rules should be defined for roles
     */
    public boolean keyOnRoles() {
        return _btnRoles.getSelection ();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
     */
    public IWizardPage getNextPage() {
        return ((GroupRuleWizard )getWizard ()).getNextPage(this);
    }

}
