/*
 * Created on Oct 21, 2003
 * 
 */
package AcmeStudio.ui.views.groups.wizards;

import java.util.Iterator;
import java.util.StringTokenizer;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import AcmeStudio.ui.editors.diagram.styles.VisVariant;
import AcmeStudio.ui.views.AcmeGroupView;
import AcmeStudio.ui.widgets.TypeSelectionWidget;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.filters.*;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeGroup;
import acmeLib.properties.*;

/**
 * @author Bradley Schmerl
 * 
 * Defines common rule elements (types and property rules)
 */
public class SelectionCriteriaPage extends WizardPage {

    private AcmeGroup _group;
    protected String _category;
    private AcmeObjectFilter _typeFilter;
    private String _selectAllTitle;
    protected Composite _top;
    private Button _btnSelectAll;
    protected TypeSelectionWidget _wgtTypeSelection;
    
    protected AcmeRecord _ruleRep;
    protected Combo _cmbOperator;
    protected Text _txtProperty;
    protected Text _txtValue;
    private List _lstRules;
    private Button _btnIncludeTrigger;

    /**
     * @param pageName
     */
    protected SelectionCriteriaPage(String category, AcmeGroup group) {
        super(category + " selection criteria");
        _group = group;
        _category = category;
        
        setupCategory ();
        setMessage ("Select the element types to be placed in groups.");
    }

    /**
     * 
     */
    protected void setupCategory() {
        final String SELECT_ALL_BUTTON_TEXT = "Select all ";
        _selectAllTitle = SELECT_ALL_BUTTON_TEXT + _category.toLowerCase() + "s";
        if (_category.equals ("Component")) {
            _typeFilter = new ComponentTypeEnumFilter ();
  
        }
        else if (_category.equals ("Connector")) {
            _typeFilter = new ConnectorTypeEnumFilter ();
        }
        else if (_category.equals ("Port")) {
            _typeFilter = new PortTypeEnumFilter ();
        }
        else if (_category.equals ("Role")) {
            _typeFilter = new RoleTypeEnumFilter ();
        }
        
        AcmeProperty prop = _group.getProperty("vis-" + _category.toLowerCase() + "-rule");
         if (prop == null) {
             constructEmptyRuleRep();
         }
         else {
             _ruleRep = (AcmeRecord )prop.getValue().copy();
         }
    }

    private void constructEmptyRuleRep() {
        AcmeRecord ruleRep;
        ruleRep = new AcmeRecord ();
        ruleRep.addField(AcmeGroupView.TYPES_FIELD, ConstantTypes.ANY_SET_TYPE, new AcmeSet ());
        ruleRep.addField (AcmeGroupView.RULES_FIELD, ConstantTypes.ANY_SEQUENCE_TYPE, new AcmeSequence ());
        _ruleRep = ruleRep;
    }
    
    private void clearPropertySection() {
    	_txtProperty.setText ("");
    	_txtValue.setText ("");
    	_cmbOperator.setText("");
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        _top = new Composite (parent, SWT.NONE);
        _top.setLayout (new GridLayout (1, false));
        setControl (_top);
        
        {
        Group typesGroup = new Group (_top, SWT.NONE);
        typesGroup.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        typesGroup.setText ("Types:");
        typesGroup.setLayout (new GridLayout (1, false));
        _btnSelectAll = new Button (typesGroup, SWT.CHECK);
        _btnSelectAll.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        _btnSelectAll.setText (_selectAllTitle);
        _btnSelectAll.addSelectionListener(new SelectionAdapter () {
            public void widgetSelected (SelectionEvent evt) {
                _wgtTypeSelection.setEnabled (!_btnSelectAll.getSelection());
            }
        });
        
        
        _wgtTypeSelection = new TypeSelectionWidget (typesGroup, _group, _typeFilter);
        _wgtTypeSelection.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_FILL));
        }
        {
        	Group ruleGroup = new Group (_top, SWT.NONE);
        ruleGroup.setLayout (new GridLayout (3, false));
        ruleGroup.setText ("Property matching rules:");
        GridData gridData = new GridData (GridData.FILL_HORIZONTAL);
        gridData.verticalSpan = 2;
        ruleGroup.setLayoutData(gridData);
        createPropertyRuleTable (ruleGroup);
        
        Composite cmpButtons = new Composite (ruleGroup, SWT.NONE);
        cmpButtons.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        cmpButtons.setLayout (new FillLayout (SWT.VERTICAL));
        Button btnAndConjunction = new Button (cmpButtons, SWT.PUSH);
        btnAndConjunction.setText ("< And");
        btnAndConjunction.addSelectionListener(new SelectionAdapter () {
           public void widgetSelected (SelectionEvent evt) {
               AcmeSequence rulepieces = (AcmeSequence )_ruleRep.getFieldValue(AcmeGroupView.RULES_FIELD);
               if (rulepieces.getSize() > 0) {
                   AcmeString value = (AcmeString )rulepieces.getValue(rulepieces.getSize() - 1);
                   String sValue = value.getValue();
                   if (!sValue.endsWith ("and") && !sValue.endsWith ("or")) {
                       sValue += ("and");
                   }
                   else {
                       sValue = sValue.substring (0, sValue.length() - sValue.lastIndexOf(":"));
                       sValue += ":and";
                   }
                   value.setValue(sValue);
               }
               String newValue = _txtProperty.getText () + ":" + _cmbOperator.getText () + ":" + _txtValue.getText () + ":";
               rulepieces.addValue(new AcmeString (newValue));
               clearPropertySection();
               updateRules ();
           }

		
        });
        
        Button btnOrConjunction = new Button (cmpButtons, SWT.PUSH);
        btnOrConjunction.setText("< Or");
        btnOrConjunction.addSelectionListener(new SelectionAdapter () {
            public void widgetSelected (SelectionEvent evt) {
                AcmeSequence rulepieces = (AcmeSequence )_ruleRep.getFieldValue(AcmeGroupView.RULES_FIELD);
                if (rulepieces.getSize() > 0) {
                    AcmeString value = (AcmeString )rulepieces.getValue(rulepieces.getSize() - 1);
                    String sValue = value.getValue();
                    if (!sValue.endsWith ("and") && !sValue.endsWith ("or")) {
                        sValue += ("or");
                    }
                    else {
                        sValue = sValue.substring (0, sValue.length() - sValue.lastIndexOf(":"));
                        sValue += ":or";
                    }
                    value.setValue(sValue);
                }
                String newValue = _txtProperty.getText () + ":" + _cmbOperator.getText () + ":" + _txtValue.getText () + ":";
                rulepieces.addValue(new AcmeString (newValue));
                clearPropertySection ();
                updateRules ();
            }
        });
        
        Button btnRemove = new Button (cmpButtons, SWT.PUSH);
        btnRemove.setText ("Remove");
        btnRemove.addSelectionListener(new SelectionAdapter () {
        	public void widgetSelected (SelectionEvent e) {
        		String [] s = _lstRules.getSelection();
        		for (int i = 0; i < s.length; i++) {
        			_lstRules.remove(s[i]);
				}
        	}
        });
        
        Composite ruleComp = new Composite (ruleGroup, SWT.NONE);
        gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
        gridData.horizontalSpan = 4;
        ruleComp.setLayoutData(gridData);
        GridLayout gridLayout = new GridLayout (3, false);
        gridLayout.numColumns = 3;
        ruleComp.setLayout(gridLayout);
        Label l = new Label (ruleComp, SWT.NONE);
        l.setText ("Property:");
        l = new Label (ruleComp, SWT.NONE);
        l.setText ("Op:");
        l = new Label (ruleComp, SWT.NONE);
        l.setText ("Value:");
        
        _txtProperty = new Text (ruleComp, SWT.BORDER);
        _txtProperty.setLayoutData (new GridData (GridData.HORIZONTAL_ALIGN_FILL));
        ModifyListener propertyModified = new ModifyListener () {

			public void modifyText(ModifyEvent e) {
				checkComplete ();
			}
        	
        };
        _txtProperty.addModifyListener(propertyModified);
        _cmbOperator = new Combo (ruleComp, SWT.BORDER);
        _cmbOperator.setLayoutData (new GridData (GridData.HORIZONTAL_ALIGN_FILL));
        for (Iterator iter = VisVariant.OPNAMES.iterator(); iter.hasNext(); ) {
			String op = (String) iter.next();
			_cmbOperator.add(op);
		}
        _cmbOperator.addModifyListener(propertyModified);
        _txtValue = new Text (ruleComp, SWT.BORDER);
        _txtValue.setLayoutData (new GridData (GridData.HORIZONTAL_ALIGN_FILL));
        _txtValue.addModifyListener(propertyModified);
        }
        _btnIncludeTrigger = new Button (_top, SWT.CHECK);
        _btnIncludeTrigger.setText ("Include the matching " + _category.toLowerCase() + " as a member");
        _btnIncludeTrigger.addSelectionListener(new SelectionAdapter () {
        	public void widgetSelected (SelectionEvent evt) {
        		boolean include = _btnIncludeTrigger.getSelection();
        		AcmeRecordField includeField = _ruleRep.getField(AcmeGroupView.INCLUDE_TRIGGER);
        		if (includeField == null) {
        			includeField = new AcmeRecordField (AcmeGroupView.INCLUDE_TRIGGER, ConstantTypes.BOOLEAN_TYPE, null);
        			_ruleRep.addField(includeField);
        		}
        		includeField.setValue(new AcmeBoolean (include));
        	}
        });
        fillValues ();
        
    }

    /**
	 * 
	 */
	protected void checkComplete() {
		if (_txtValue.getText ().length () > 0 ||
				_txtValue.getText ().length () > 0 ||
				_cmbOperator.getText().length () > 0) {
			setMessage ("Make sure to copy property expression across", WARNING);
		}	
		else {
			setMessage (null);
		}
	}

	/**
     * 
     */
    protected void updateRules() {
    	_lstRules.removeAll ();
        AcmeSequence rules = (AcmeSequence )_ruleRep.getFieldValue(AcmeGroupView.RULES_FIELD);
        AcmeEnumeration rvs = rules.getValues();
        while (rvs.moreItems()) {
            AcmeString rule = (AcmeString) rvs.getNextItem();
            StringTokenizer tokens = new StringTokenizer (rule.getValue (), ":");
            String rep = "";
            while (tokens.hasMoreTokens()) {
                rep += tokens.nextToken () + " ";
            }
            _lstRules.add (rep);
            _lstRules.setData(rep, rule);
        }
    }

    /**
     * 
     */
    protected void fillValues() {
        _wgtTypeSelection.updateAvailableTypes();
        AcmeSet types = (AcmeSet )_ruleRep.getFieldValue (AcmeGroupView.TYPES_FIELD);
        String [] selectedTypes = new String [types.getSize()];
        AcmeEnumeration tvs = types.getValues();
        int index = 0;
        while (tvs.moreItems()) {
            AcmeString tv = (AcmeString) tvs.getNextItem();
            selectedTypes [index++] = tv.getValue (); 
        }
        _wgtTypeSelection.updateSelectedTypes (selectedTypes);
        
        updateRules ();
        
        AcmeRecordField includeField = _ruleRep.getField (AcmeGroupView.INCLUDE_TRIGGER);
		if (includeField == null) {
        	_btnIncludeTrigger.setSelection(true);
        }
		else {
			_btnIncludeTrigger.setSelection (((AcmeBoolean )includeField.getValue()).getValue());
		}
        
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
     */
    public IWizardPage getNextPage() {
        return ((GroupRuleWizard )getWizard ()).getNextPage(this);
    }
    /**
     * @param ruleGroup
     */
    private void createPropertyRuleTable(Group ruleGroup) {
        _lstRules = new List (ruleGroup, SWT.V_SCROLL | SWT.H_SCROLL | SWT.MULTI);
        GridData gd = new GridData (GridData.HORIZONTAL_ALIGN_FILL);
        gd.grabExcessHorizontalSpace = true;
        gd.grabExcessVerticalSpace = true;
        gd.horizontalAlignment = GridData.FILL;
        gd.verticalAlignment = GridData.FILL;
        _lstRules.setLayoutData(gd);
    }

    /**
     * @return
     */
    public AcmePropertyValue getRuleRep() {
        
        AcmeSet values = new AcmeSet ();
        for (Iterator iter = _wgtTypeSelection.getSelectedTypes().iterator(); iter.hasNext();) {
            AcmeElementType type = (AcmeElementType) iter.next();
            values.addValue(new AcmeString (type.getTypeName()));
        }
        _ruleRep.setField("discriminatingTypes", values);
        return _ruleRep;
    }

}
