package AcmeStudio.ui.views.parts;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import AcmeStudio.util.Listeners;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.filters.AcmeObjectFilter;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeFamily;
import acmeLib.elements.AcmeSystem;
import acmeLib.representations.AcmeRepresentation;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AddRepresentationDialog extends TitleAreaDialog {

    protected AcmeElement _target;
    
    protected Text _name;
    protected List _families;
    protected AcmeRepresentation _rep;
    
    public AddRepresentationDialog (Shell shell, AcmeElement target) {
        super (shell);
        _target = target;   
        setTitle ("Add representation");
    }

   
    /**
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite top = (Composite) super.createDialogArea(parent);
        top.setLayout  (new GridLayout (1, false));

//| GridData.FILL_HORIZONTAL);
//        fld.grabExcessHorizontalSpace = true;
//        fld.grabExcessVerticalSpace = true;
        
        Label nameLabel = new Label (top, SWT.NONE);
        nameLabel.setText ("Name:");
        
        _name = new Text (top, SWT.SINGLE | SWT.BORDER);
        _name.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        _name.addVerifyListener(Listeners.VERIFY_ACMEID);
        _name.addModifyListener (new ModifyListener () {
        	public void modifyText (ModifyEvent evt) {
        		validate ();
        	}
        });
        
        Label familiesLabel = new Label (top, SWT.NONE);
        familiesLabel.setText ("Select families for the system:");
        
        _families = new List (top, SWT.MULTI |SWT.BORDER | SWT.V_SCROLL);
        _families.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_VERTICAL));
       
        setValues ();
        
        return top;
    }
    
    /**
	 * 
	 */
	private void validate() {
		String name = _name.getText ();
		if (name == null || name.equals ("")) {
			setErrorMessage ("The representation name cannot be empty");
			getButton (IDialogConstants.OK_ID).setEnabled (false);
			return;
		}
		
		if (_target.lookupName(name, false) != null) {
			setErrorMessage ("There is a duplicate name already defined");
			getButton (IDialogConstants.OK_ID).setEnabled (false);
			return;
		}
		
		setErrorMessage (null);
		setMessage ("Enter the name and type of the new representation");
		getButton (IDialogConstants.OK_ID).setEnabled (true);
	}


	protected void setValues () {
        AcmeEnumeration types = _target.getAvailableTypes();
        types = types.castToAcmeSetVector().enumerate (new AcmeObjectFilter () {
            public boolean test (AcmeObject obj) {
                return obj instanceof AcmeFamily;   
            }
        });
        while (types.moreItems ()) {
            AcmeFamily fam = (AcmeFamily )types.getNextItem ();
            _families.add (fam.getTypeName());
            _families.setData (fam.getTypeName (), fam);
        }    
        
        AcmeEnumeration parentFams = _target.getParent ().getDeclaredTypes();
        if (parentFams.moreItems ()) {
        	AcmeFamily fam = (AcmeFamily )parentFams.getNextItem ();
        	_families.select(_families.indexOf (fam.getName ()));
        }
    }
    
    

    /**
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed() {
        _rep = new AcmeRepresentation ();
        String name = _name.getText();
        try {
            _rep.setName (name);
            
        }
        catch (AcmeIllegalNameException e) {
            // Should never happen because of the verifier
        }
        
        _rep.setSystem (new AcmeSystem (name));
        /*
         * Makes sure the parent design gets set (this doesn't get done
         * in AcmeLib). Fixes Bug #50
         */ 
        _rep.getSystem().setParentDesign(_target.getParentDesign());
        
        String [] selections = _families.getSelection ();
        for (int i = 0; i < selections.length; i++) {
            AcmeFamily fam = (AcmeFamily )_families.getData (selections [i]);
            _rep.getSystem ().addDeclaredType (fam);
            _rep.getSystem ().addInstantiatedType(fam);
        }
        
        _rep.getSystem().instantiateElements();
        super.okPressed();
    }

    public AcmeRepresentation getRepresentation () {
        return _rep;
    }

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createButtonBar(Composite parent) {
		Control c = super.createButtonBar(parent);
		validate ();
		return c;
	}

}
