package AcmeStudio.ui.views.parts;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.*;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.ui.views.DesignBrowser;

/**
 * @author Bradley Schmerl (schmerl@cs.cmu.edu)
 * 
 * Allows the user to define preferences for the display in the
 * Design Browser outline view.
 *
 */
public class DesignBrowserPreferencesDialog extends Dialog {

	/** Where the dialog area is placed **/
	private Composite _parent = null;
	// The buttons for each of the preferences. Each is a check box
	private Button _showInstancesOnly = null;
	private Button _showComponents = null;
	private Button _showProperties = null;
	private Button _showPorts = null;
	private Button _showHiddenProperties = null;
	private Button _showConnectors = null;
	private Button _showRepresentations = null;
	private Button _showRoles = null;

	/**
	 * Constructor for DesignBrowserPreferencesDialog.
	 * @param parentShell The place to put the dialog
	 */
	public DesignBrowserPreferencesDialog(Shell parentShell) {
		super(parentShell);
	}

	/**
	 * Creates the dialog area, which allows selection of all the options
	 * @param parent the Composite in which the buttons are to be placed
	 */
	protected Control createDialogArea(Composite parent) {
		_parent = (Composite) super.createDialogArea(parent);
		
		// The settings for AcmeStudio
		final IDialogSettings settings = AcmeStudioPlugin.getDefault().getDialogSettings();

		RowLayout rl= new RowLayout (SWT.VERTICAL);
		_parent.setLayout (rl);

		// Create the show instances button - tied to the DesignBrowser.showInstancesOnly property		
		_showInstancesOnly = new Button (_parent, SWT.CHECK);
		_showInstancesOnly.setText ("Show instances only");
		_showInstancesOnly.setSelection (settings.getBoolean (DesignBrowser.SHOW_INSTANCES_ONLY_OPTION));
		
		// The properties group
		Group displayPropertiesGroup = new Group (_parent, SWT.BORDER);
		displayPropertiesGroup.setText ("Display Properties");
		
		GridLayout dpLayout = new GridLayout (2, true);
		displayPropertiesGroup.setLayout(dpLayout);

		// Show components - tied to !DesignBrowser.hideComponents property		
		_showComponents = new Button (displayPropertiesGroup, SWT.CHECK);
		_showComponents.setText ("Show components");
		_showComponents.setSelection (!settings.getBoolean (DesignBrowser.HIDE_COMPONENTS_OPTION));

		// Show properties - tied to DesignBrowser.showProperties property
		_showProperties = new Button (displayPropertiesGroup, SWT.CHECK);
		_showProperties.setText ("Show properties");
		_showProperties.setSelection (settings.getBoolean (DesignBrowser.SHOW_PROPERTIES_OPTION));
	
		// Show ports <--> !DesignBrowser.hidePorts
		_showPorts = new Button (displayPropertiesGroup, SWT.CHECK);
		_showPorts.setText ("Show ports");
		_showPorts.setSelection (!settings.getBoolean (DesignBrowser.HIDE_PORTS_OPTION));
		GridData spgd = new GridData ();
		spgd.horizontalIndent = 10;
		_showPorts.setLayoutData (spgd);
		
		// Show hidden properties <--> DesignBrowser.showHiddenProperties
		_showHiddenProperties = new Button (displayPropertiesGroup, SWT.CHECK);
		_showHiddenProperties.setText ("Show hidden properties");
		_showHiddenProperties.setSelection (settings.getBoolean (DesignBrowser.SHOW_HIDDEN_PROPERTIES_OPTION));
		GridData shpgd = new GridData ();
		shpgd.horizontalIndent = 20;
		_showHiddenProperties.setLayoutData (shpgd);
		
		// Show connectors <--> !DesignBrowser.hideConnectors
		_showConnectors = new Button (displayPropertiesGroup, SWT.CHECK);
		_showConnectors.setText ("Show connectors");
		_showConnectors.setSelection (!settings.getBoolean (DesignBrowser.HIDE_CONNECTORS_OPTION));
		
		// Show representations <--> !DesignBrowser.hideRepresentations
		_showRepresentations = new Button (displayPropertiesGroup, SWT.CHECK);
		_showRepresentations.setText ("Show representations");
		_showRepresentations.setSelection (!settings.getBoolean (DesignBrowser.HIDE_REPRESENTATIONS_OPTION));
		
		// Show roles <--> !DesignBrowser.hideRoles
		_showRoles = new Button (displayPropertiesGroup, SWT.CHECK);
		_showRoles.setText ("Show roles");
		_showRoles.setSelection (!settings.getBoolean (DesignBrowser.HIDE_ROLES_OPTION));
		GridData srpd = new GridData ();
		srpd.horizontalIndent = 20;
		_showRoles.setLayoutData (srpd);
		return _parent;
	}
	/**
	 * Triggered when OK is pressed.
	 * Sets the DialogSettings with the new values in the check boxes
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		final IDialogSettings settings = AcmeStudioPlugin.getDefault().getDialogSettings();
		settings.put (DesignBrowser.SHOW_INSTANCES_ONLY_OPTION, _showInstancesOnly.getSelection());
		settings.put (DesignBrowser.HIDE_COMPONENTS_OPTION, !_showComponents.getSelection ());
		settings.put (DesignBrowser.SHOW_PROPERTIES_OPTION, _showProperties.getSelection ());
		settings.put (DesignBrowser.HIDE_PORTS_OPTION, !_showPorts.getSelection ());
		settings.put (DesignBrowser.SHOW_HIDDEN_PROPERTIES_OPTION, _showHiddenProperties.getSelection ());		
		settings.put (DesignBrowser.HIDE_CONNECTORS_OPTION, !_showConnectors.getSelection ());
		settings.put (DesignBrowser.HIDE_REPRESENTATIONS_OPTION, !_showRepresentations.getSelection ());
		settings.put (DesignBrowser.HIDE_ROLES_OPTION, !_showRoles.getSelection ());
		super.okPressed();
	}

}
