/*
 * Created on Jan 20, 2004
 *
 */
package AcmeStudio.ui.views.parts;

import java.io.IOException;

import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;

import AcmeStudio.ui.editors.text.AcmeSourceViewerConfiguration;
import AcmeStudio.ui.views.IElementViewPage;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.AcmeObject;
import acmeLib.core.exceptions.AcmeException;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.core.io.ArmaniExportable;
import acmeLib.elements.AcmeElementType;



/**
 * @author Bradley Schmerl
 * Created Jan 20, 2004
 */
public class ElementSourceView extends ElementViewPart implements ISelectionListener, AcmeModelChangeListener {

	private SourceViewer _viewer;
	/**
	 * 
	 */
	public ElementSourceView() {
		super ();
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#getContributions()
	 */
	public IContributionItem[] getContributions() {
		return new IContributionItem [0];
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#isValidFor(acmeLib.AcmeObject)
	 */
	public boolean isValidFor(AcmeObject o) {
		return o instanceof AcmeElementType;
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#getTitle()
	 */
	public String getTitle() {
		return "Source";
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent)
	 */
	public void changeModel(AcmeStudioChangeEvent evt) {
	}
	
	public boolean interestedInEvent (int et) {
		return false;
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#createControl(org.eclipse.swt.widgets.Composite, int, AcmeStudio.ui.views.IElementViewPage)
	 */
	public void createControl(Composite parent, int style, IElementViewPage vp) {
		super.createControl (parent, style, vp);
		FillLayout layout = new FillLayout ();
		this.getComposite().setLayout(layout);
		
		_viewer = new SourceViewer (this.getComposite(), null, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		_viewer.configure (new AcmeSourceViewerConfiguration ());
		_viewer.getTextWidget().setLayoutData (new GridData (GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL | GridData.GRAB_VERTICAL));
		
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#setInput(acmeLib.AcmeObject)
	 */
	public void setInput(AcmeObject o) {
		super.setInput (o);
		if (o instanceof ArmaniExportable) {
			try {
				ArmaniExportable ex = (ArmaniExportable) o;
				AcmeOutputStream os = new AcmeOutputStream ();
				ex.exportArmani(os);
				os.close ();
				_viewer.setDocument(new Document (os.toString ()));
				_viewer.setEditable (false);
			}
			catch (IOException e) {
				_viewer.setDocument (new Document (o.toString ()));
			}
			catch (AcmeException e) {
				_viewer.setDocument (new Document (o.toString ()));
			}
		}
	}
}
