/*
 * Created on Jan 6, 2004
 *
 */
package AcmeStudio.ui.views.parts;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;



/**
 * @author Bradley Schmerl
 * Created Jan 6, 2004
 */
public class ElementTypeInspector extends Dialog {

	private AcmeElement _type;
	private PropertyList properties;
	private RuleList rules;
	private StructureList struct;
	private MaintainedTypesList types;
	private ElementSourceView source;

	/**
	 * @param parentShell
	 */
	public ElementTypeInspector(Shell parentShell, AcmeElementType type) {
		super (parentShell);
		_type = (AcmeElement )type;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		Composite top = (Composite )super.createDialogArea (parent);
		top.setLayout(new GridLayout (1, false));
		
		Label titleLayer = new Label (top, SWT.NONE);
		titleLayer.setText ("Inspect definition of type: " + _type.getName ());
		
		TabFolder folder = new TabFolder (top, SWT.NONE);
		GridData flgd = new GridData (GridData.FILL_BOTH);
		flgd.verticalSpan = 3;
		flgd.widthHint = 500;
		folder.setLayoutData (flgd);
		
		properties = new PropertyList ();
		properties.createControl (folder, SWT.BORDER, null);
		properties.getComposite ().setLayoutData(new GridData (GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL));
		TabItem pItem = new TabItem (folder, SWT.NONE);
		pItem.setText(properties.getTitle());
		pItem.setControl(properties.getComposite ());
		
		rules = new RuleList ();
		rules.createControl(folder, SWT.BORDER, null);
		rules.getComposite().setLayoutData(new GridData (GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL));
		TabItem rItem = new TabItem (folder, SWT.NONE);
		rItem.setText (rules.getTitle());
		rItem.setControl (rules.getComposite ());
		
		struct = new StructureList ();
		struct.createControl (folder, SWT.BORDER, null);
		struct.getComposite ().setLayoutData(new GridData (GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL));
		TabItem sItem = new TabItem (folder, SWT.NONE);
		sItem.setText (struct.getTitle());
		sItem.setControl(struct.getComposite ());
		
		types = new MaintainedTypesList ();
		types.createControl (folder, SWT.BORDER, null);
		types.getComposite().setLayoutData(new GridData (GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL));
		TabItem tItem = new TabItem (folder, SWT.NONE);
		tItem.setText (types.getTitle ());
		tItem.setControl (types.getComposite ());
		
		source = new ElementSourceView ();
		source.createControl (folder, SWT.BORDER, null);
		source.getComposite().setLayoutData(new GridData (GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL));
		TabItem oItem = new TabItem (folder, SWT.NONE);
		oItem.setText (source.getTitle ());
		oItem.setControl (source.getComposite ());
		
		properties.setInput (_type);
		rules.setInput (_type);
		struct.setInput (_type);
		types.setInput (_type);
		source.setInput (_type);
		return top;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#close()
	 */
	public boolean close() {
		boolean closed = super.close ();
		if (closed) {
			properties.dispose();
			rules.dispose ();
			struct.dispose ();
			types.dispose ();
			source.dispose ();
		}
		return closed;
	}
}
