package AcmeStudio.ui.views.parts;

import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.widgets.Composite;

import AcmeStudio.ui.views.IElementViewPage;
import acmeLib.AcmeObject;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;

/**
 * @author schmerl
 *
 */
abstract public class ElementViewPart  {

    protected Composite _composite;
    protected IElementViewPage _page;
    protected AcmeDesign _acmeDesign;

	/**
	 * Constructor for ElementViewPart.
	 * @param parent
	 * @param style
	 */
    public ElementViewPart () {}

    /**
     * Emulate SWT Widget dispose method.
     */
    public void dispose () {
        if (_composite != null) {
            _composite.dispose();
            _composite = null;
        }
        _page = null;
        _acmeDesign = null;
    }
    
    public Composite getComposite () {
        return _composite;
    }
	
	/**
	 * Sets the input to be an AcmeObject. This object will be displayed
	 * in the ElementViewPart
	 * @param o the object to be displayed
	 */
	public void setInput (AcmeObject o) {
        if (o instanceof AcmeElement) {
            _acmeDesign = ((AcmeElement )o).getParentDesign ();
        }   
    }

    public IElementViewPage getPage () {
        return _page;
    }
    
    /**
     * Creates the control for that will be added to the element view. Users should override this element.
     * @param parent The parent composite
     * @param style The style of the widget to create
     * @param vp The ElementView that this control will be attached to
     */
    public void createControl (Composite parent, int style, IElementViewPage vp) {
        _composite = new Composite (parent, style);
        _page = vp;
    }
    
	/**
	 * Returns the contributions of the ElementViewPart. These
	 * will be added to the toolbar
	 * @return the contributions
	 */
	abstract public IContributionItem [] getContributions ();
	
    /**
     * Returns whether this view part is valid for a particular object. This is used 
     * by the element view to determine whether to display the tab when an
     * element is selected
     * @param o The object that is selected
     * @return whether the view part should be shown for this object
     */
    public abstract boolean isValidFor (AcmeObject o);

	/**
	 * 
	 * @return The title to be used in the tab for this part
	 */
	public abstract String getTitle();

}
