package AcmeStudio.ui.views.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import AcmeStudio.core.ASModel;
import AcmeStudio.ui.views.IElementViewPage;
import AcmeStudio.ui.widgets.*;
import AcmeStudio.util.DesignEvaluationCache;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeElement;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class ErrorList extends ElementViewPart implements PropertyChangeListener {


    private AcmeErrorTree _viewer = null;
    private AcmeObject _selectedObject = null;
    private AcmeEnumeration _errors = null;
    private IContributionItem [] _myContributions = new IContributionItem [0];
    

    
    public void createControl (Composite parent, int style, IElementViewPage parentView) {
        super.createControl(parent, style, parentView);
        FillLayout tableLayout = new FillLayout ();
        this.getComposite ().setLayout (tableLayout);
        
        _viewer = new AcmeErrorTree (this.getComposite ());

        _viewer.setVisible (true);
       
    }

    /**
     * @see AcmeStudio.ui.views.parts.ElementViewPart#dispose()
     */
    public void dispose() {
        ASModel.current().getRuleManager().removePropertyChangeListener(this);
        super.dispose();
    }

    /**
     * @see AcmeStudio.ui.views.parts.ElementViewPart#getContributions()
     */
    public IContributionItem[] getContributions() {
        return _myContributions;
    }

    /**
     * @see AcmeStudio.ui.views.parts.ElementViewPart#setInput(AcmeObject)
     */
    public void setInput(AcmeObject o) {
        super.setInput (o);
        _selectedObject = o;
        if (o != null) {
            _viewer.setErrorSource (o);
            _viewer.setVisible (true);
            if (o instanceof AcmeElement) {
                _acmeDesign = ((AcmeElement )o).getParentDesign();
                DesignEvaluationCache cache = ASModel.current().getRuleManager ();
                if (cache != null) {
                        cache.addPropertyChangeListener(this);
                }
    
            }
        }
        else {
            _viewer.setVisible (false);
        }
    }

    /**
     * @see AcmeStudio.ui.views.parts.ElementViewPart#isValidFor(AcmeObject)
     */
    public boolean isValidFor(AcmeObject o) {
    	AcmeElement element = (AcmeElement )_selectedObject;
    	_errors =  ASModel.current ().getRuleManager().getErrors (element);
        return _errors != null && _errors.getSize () != 0;
    }

    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals (DesignEvaluationCache.PROP_ELEMENT_TYPECHECK) &&
            _selectedObject.equals (evt.getNewValue ())) {
                _viewer.setErrorSource(_selectedObject);
                _errors = (AcmeEnumeration )evt.getOldValue();   
            }
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#getTitle()
	 */
	public String getTitle() {
		return "Errors";
	}

}
