package AcmeStudio.ui.views.parts;

import java.util.Iterator;
import java.util.SortedMap;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import AcmeStudio.core.ASModel;
import AcmeStudio.core.resource.ASFamily;
import AcmeStudio.util.FamilyDetailContentProvider;
import AcmeStudio.util.FamilyDetailLabelProvider;
import acmeLib.elements.AcmeDesign;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class ImportFamilyDialog extends Dialog {

 
    private TreeViewer _familyDetails;
    AcmeDesign _design;
    ASFamily [] _familiesToImport;
    boolean _automaticallyAssignFamily;
    
    Tree _availableFamilies;
    Button _browseButton;
    Group _importOptionsGroup;
    Button _importInline;
    Button _copyToProject;
    Button _leaveInPlace;
    Button _refreshList;
    Button _autoAssignButton;

    public ImportFamilyDialog (Shell shell) {
        super (shell);   
        _design = null;
    }

    /**
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite p = (Composite ) super.createDialogArea(parent);
        p.setLayout(new FillLayout());
        
        Composite top = new Composite (p, SWT.NONE);
        GridLayout layout = new GridLayout (1, false);
        top.setLayout(layout);
        
        Label label = new Label (top, SWT.NONE);
        label.setText ("Choose families to import into the current design.");
        GridData lgd = new GridData (GridData.HORIZONTAL_ALIGN_FILL);
        lgd.horizontalSpan = GridData.FILL_HORIZONTAL;
        label.setLayoutData(lgd);
        Composite composite = new Composite(top, SWT.NONE);
        composite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 2;
        composite.setLayout(gridLayout);
        label = new Label(composite, SWT.NONE);
        label.setText("Available Families:");
        label = new Label(composite, SWT.NONE);
        label.setText("Family Details:");
        
        _availableFamilies = new Tree (composite, SWT.MULTI | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        _availableFamilies.addSelectionListener(new SelectionAdapter() {
        	public void widgetSelected(SelectionEvent e) {
        		TreeItem [] families = _availableFamilies.getSelection();
        		if (families.length > 0) {
        			if (families [0].getData () instanceof java.util.List) {
        				ASFamily family = (ASFamily )((java.util.List )families [0].getData()).get (0);
        				fillDetails (family);
        			}
        			else if (families [0].getData () instanceof ASFamily) {
        				ASFamily family = (ASFamily )families [0].getData ();
        				fillDetails (family);
        			}
        		}
        	}
        });
        _availableFamilies.setLayoutData(new GridData(GridData.FILL_BOTH));
      
        _familyDetails = new TreeViewer(composite, SWT.BORDER);
        _familyDetails.setContentProvider(new FamilyDetailContentProvider ());
        _familyDetails.setLabelProvider( new FamilyDetailLabelProvider ());
        Tree tree = _familyDetails.getTree();
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.heightHint = 100;
        tree.setLayoutData(gridData);
        _familyDetails.setInput(new Object());
//        ald.grabExcessVerticalSpace = true;
//        ald.grabExcessHorizontalSpace = true;
        
        Composite buttons = new Composite (composite, SWT.NONE);
        buttons.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER));
        buttons.setLayout (new FillLayout (SWT.HORIZONTAL));
        _browseButton = new Button (buttons, SWT.PUSH);
        _browseButton.setEnabled(false);
        _browseButton.setText ("Browse...");
        
        _refreshList = new Button (buttons, SWT.PUSH);
        _refreshList.setText ("Refresh");
        _refreshList.setToolTipText("Refresh the list of available families");
        _refreshList.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected (SelectionEvent e) {
                if (e.getSource().equals(_refreshList)) {
                    ASModel.familyRegistry ().refresh(); 
                    _availableFamilies.removeAll();
                    fillFamilies ();
                }
            }
        });
      
        _importOptionsGroup = new Group (top, SWT.BORDER);
        _importOptionsGroup.setLayout (new FillLayout ());
        _importOptionsGroup.setText("Import options");
        GridData fgd = new GridData (GridData.HORIZONTAL_ALIGN_FILL);
        fgd.horizontalSpan = GridData.FILL_HORIZONTAL;
        _importOptionsGroup.setLayoutData(fgd);
          
        _leaveInPlace = new Button (_importOptionsGroup, SWT.RADIO);
        _leaveInPlace.setText ("Leave in place");
       
        _copyToProject = new Button (_importOptionsGroup, SWT.RADIO);
        _copyToProject.setText ("Copy to project");
        
        _importInline = new Button (_importOptionsGroup, SWT.RADIO);
        _importInline.setText ("Inline");
        _leaveInPlace.setSelection(true);
        
        _copyToProject.setEnabled (false);
        _importInline.setEnabled (false);
        
        _autoAssignButton = new Button (top, SWT.CHECK);
        _autoAssignButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        _autoAssignButton.setText ("Automatically assign families on import");
        _autoAssignButton.setSelection(true);
        // Fill the families now that everything is created
        fillFamilies ();
        return p;
    }

    /**
	 * @param family
	 */
	protected void fillDetails(ASFamily family) {
		_familyDetails.setInput (family);
	}

	private void fillFamilies () {
        SortedMap families = ASModel.familyRegistry().getByName();  
//        ASModel.familyRegistry ().refresh(); 
        // (family -> List<Path>)
        Iterator i = families.keySet ().iterator ();
        while (i.hasNext ()) {
            Object o = i.next ();   
            String name = (String )o;
            java.util.List paths = (java.util.List )families.get (name);
            TreeItem nameItem = new TreeItem (_availableFamilies, SWT.NONE);
            nameItem.setText (name);
            nameItem.setData (paths);
            if (paths.size() > 1) {
                Iterator pI = paths.iterator();
                while (pI.hasNext ()) {
                    ASFamily path = (ASFamily )pI.next ();
                    TreeItem pathItem = new TreeItem (nameItem, SWT.NONE);
                    pathItem.setText (path.getPath());
                    pathItem.setData (path);
                }    
            }
            
        }   
        
    }
        

    /**
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed() {
        
        if (_leaveInPlace.getSelection()) {
            TreeItem [] families = _availableFamilies.getSelection(); 
            _familiesToImport = new ASFamily [families.length]; 
            for (int i = 0; i < families.length; i++) {
                if (families [i].getData () instanceof java.util.List) {
                    _familiesToImport [i] = ((ASFamily )((java.util.List )families [i].getData ()).get (0));
                    
                }   
                else if (families [i].getData () instanceof ASFamily) {
                    _familiesToImport [i] = (ASFamily )families [i].getData ();
                    
                }
            }
        }
        
 		_automaticallyAssignFamily = _autoAssignButton.getSelection();
        
        super.okPressed();
    }

    public ASFamily [] getImportingFamilies () {
        return _familiesToImport;
    }
    
    public boolean automaticallyAssignFamilies () {
    	return _automaticallyAssignFamily;
    }
    
}
