package AcmeStudio.ui.views.parts;

import java.util.Iterator;

import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.internal.WorkbenchImages;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.model.ModelCompoundCommand;
import AcmeStudio.model.SupertypeAssignmentCommand;
import AcmeStudio.model.TypeAssignmentCommand;
import AcmeStudio.ui.views.IElementViewPage;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.ImageManager;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.filters.NoBaseClassesEnumFilter;
import acmeLib.elements.*;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class MaintainedTypesList
    extends ElementViewPart
    implements ISelectionListener, AcmeModelChangeListener {
        
    public static final String DISTINGUISH_DECLARED_INSTANTIATED_TYPES = "DistInstDeclTypes";
	private IContributionItem [] _myContributions;
    
    private Action _deleteTypeAction;
    private Action _addTypeAction;
    private AcmeObject _selectedObject;
    private TableViewer _viewer;
    private IElementViewPage _parentView;
    
    private static final Object [] emptyElements = new Object [0];
           
    private class ViewContentProvider implements IStructuredContentProvider {
        public void inputChanged(Viewer v, Object oldInput, Object newInput) {
        }
        public void dispose() {
        }
        
        public Object [] getElements (Object parent) {
            AcmeEnumeration types = null;
            Object [] elements = emptyElements;
            if (parent instanceof AcmeElementType) {
                AcmeElementType type = (AcmeElementType )parent;
                types = type.getSuperTypes().enumerate(new NoBaseClassesEnumFilter ());
                int index = 0;
                elements = new Object [types.getSize ()];
                while (types.moreItems ()) {
                    elements [index++] = types.getNextItem ();
                }
                   
            }
            else if (parent instanceof AcmeElement) {
                AcmeElement el = (AcmeElement )parent;
                types = el.getDeclaredTypes();
                int index = 0;
                elements = new Object [types.getSize()];
                 
                while (types.moreItems ()) {
                    elements [index++] = types.getNextItem ();   
                }      
                
            }
            return elements;
        }
                
    }
    
    private class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
        public String getColumnText (Object obj, int index) {
            AcmeElementType type = (AcmeElementType )obj;
            String retStr = "";
            switch (index) {
                case 0:
                    retStr = type.getTypeName ();
                    break;
                case 1:
                	if (type instanceof AcmeUnresolvedElementType) retStr = "ERROR: Unresolved type";
                	else if (!(type instanceof AcmeFamily) && ((AcmeElement )type).getParent () != null) {
                		retStr = ((AcmeElement )type).getParent ().getName ();
                	}
                	break;
            }
            
            return retStr;
        }
        
        public Image getColumnImage (Object obj, int index) {
            return null;   
        }
        
        public Image getImage (Object obj) {
            return null;
        }
    }
    
    private class NameSorter extends ViewerSorter {
    }
    
        
    public MaintainedTypesList () {
        super ();
    }
    
    /**
     * @see AcmeStudio.ui.views.parts.ElementViewPart#dispose()
     */
    public void dispose() {
        AcmeStudioChangeManager.removeListener(this);
        super.dispose();
    }

    /**
     * @see AcmeStudio.ui.views.parts.ElementViewPart#getContributions()
     */
    public IContributionItem[] getContributions() {
        return _myContributions;
    }

    /**
     * @see AcmeStudio.ui.views.parts.ElementViewPart#setInput(AcmeObject)
     */
    public void setInput(AcmeObject o) {
        super.setInput (o);
        if (o == null) return;
        _selectedObject = o;
        _viewer.setInput (o);
              
        if (!(_selectedObject instanceof AcmeElementType)) {
            TableItem [] items = _viewer.getTable().getItems ();
            for (int i = 0; i < items.length; i++) {
                AcmeElementType type = (AcmeElementType )items [i].getData();
                if (_selectedObject instanceof AcmeElement && ((AcmeElement )_selectedObject).instantiatesType(type)) {
                    items [i].setChecked(true);
                }
            }
        }
        if (_selectedObject instanceof AcmeComponent) {
            _addTypeAction.setImageDescriptor(ImageManager.getSmallImageDescriptor(ImageManager.ID_COMP_TYPE));
            
        }
        else if (_selectedObject instanceof AcmeConnector) {
            _addTypeAction.setImageDescriptor(ImageManager.getSmallImageDescriptor(ImageManager.ID_CONN_TYPE));
            
        }
        else if (_selectedObject instanceof AcmePort) {
            _addTypeAction.setImageDescriptor(ImageManager.getSmallImageDescriptor(ImageManager.ID_PORT_TYPE));
            
        }
        else        if (_selectedObject instanceof AcmeRole) {
            _addTypeAction.setImageDescriptor(ImageManager.getSmallImageDescriptor(ImageManager.ID_ROLE_TYPE));
            
        }
    }

    /**
     * @see org.eclipse.ui.ISelectionListener#selectionChanged(IWorkbenchPart, ISelection)
     */
    public void selectionChanged(IWorkbenchPart part, ISelection selection) {
    }

    /**
     * @see AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudioChangeEvent)
     */
    public void changeModel(
        AcmeStudioChangeManager.AcmeStudioChangeEvent evt) {
    }

    public boolean interestedInEvent (int et) {
    	return false;
    }
    
    private void makeActions () {
        _deleteTypeAction = new Action () {
            public void run () {
                ISelection isel = _viewer.getSelection ();
                if (isel instanceof IStructuredSelection) {
                    IStructuredSelection iss = (IStructuredSelection )isel;
                    Iterator it = iss.iterator ();
                    
                    while (it.hasNext ()) {
                        AcmeElementType type = (AcmeElementType )it.next ();
                        if (_selectedObject instanceof AcmeElement) {
                           AcmeElement element = (AcmeElement )_selectedObject;
                           CompoundCommand command = new CompoundCommand ();
                           command.add (new TypeAssignmentCommand (TypeAssignmentCommand.UNASSIGN_TYPE, element, type, true, _parentView));
                           command.add (new TypeAssignmentCommand (TypeAssignmentCommand.UNASSIGN_TYPE, element, type, false, _parentView));
                           AcmeStudioChangeManager.globalChangeManager.getCommandStack (_acmeDesign).execute (command);
                        }
                    }
                    setInput (_selectedObject);
                }
            }
            public boolean isEnabled () {
                return !_viewer.getSelection ().isEmpty ();
            }
        };
        _deleteTypeAction.setText ("Delete type");
        _deleteTypeAction.setDescription ("Deletes the type from the list of declared and instantiated types for this element");
        // As of Eclipse 3.0, IMG_TOOL_DELETE_HOVER == IMG_TOOL_DELETE
        _deleteTypeAction.setImageDescriptor(WorkbenchImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
    
        _addTypeAction = new Action () {
            public void run () {
                if (_selectedObject instanceof AcmeFamily) {
                           
                }
                else if (_selectedObject instanceof AcmeSystem) {
                        
                }
                else if (_selectedObject instanceof AcmeElementType) {
                    AcmeElementType type = (AcmeElementType )_selectedObject;
                    AssignTypeDialog dialog = new AssignTypeDialog (Display.getDefault ().getActiveShell (), type);
                    if (dialog.open () == IDialogConstants.OK_ID) {
                        ModelCompoundCommand c = new ModelCompoundCommand ("Change super types");
                        Iterator i = dialog.declAdditions.iterator ();
                        while (i.hasNext ()) {
                            AcmeElementType supertype = (AcmeElementType )i.next ();
                            SupertypeAssignmentCommand command = new SupertypeAssignmentCommand (TypeAssignmentCommand.ASSIGN_TYPE, type, supertype, _parentView);                        
                            c.add (command);
                        }    
                        
                        i = dialog.declDeletions.iterator ();
                        while (i.hasNext ()) {
                            AcmeElementType supertype = (AcmeElementType )i.next ();
                            SupertypeAssignmentCommand command = new SupertypeAssignmentCommand (TypeAssignmentCommand.UNASSIGN_TYPE, type, supertype, _parentView);
                            c.add (command);
                        }
                        c.execute (_acmeDesign);
                    }
                }
                else if (_selectedObject instanceof AcmeElement) {
                    AcmeElement element = (AcmeElement )_selectedObject;
                    AssignTypeDialog dialog = new AssignTypeDialog (Display.getDefault ().getActiveShell (), element);
                    if (dialog.open () == IDialogConstants.OK_ID) {
                        ModelCompoundCommand c = new ModelCompoundCommand ("Change types");
                        Iterator i = dialog.declAdditions.iterator ();
                        while (i.hasNext ()) {
                            AcmeElementType type = (AcmeElementType )i.next ();
                            
                            TypeAssignmentCommand command = new TypeAssignmentCommand (TypeAssignmentCommand.ASSIGN_TYPE, element, type, false, _parentView);
                            c.add (command);
                            //command.execute (_acmeDesign);
                            
                           
                        }
                        i = dialog.instAdditions.iterator ();
                        while (i.hasNext ()) {
                            AcmeElementType type = (AcmeElementType )i.next ();
                            TypeAssignmentCommand command = new TypeAssignmentCommand (TypeAssignmentCommand.ASSIGN_TYPE, element, type, true, _parentView);
//                            command.execute (_acmeDesign);
                            c.add (command);
                           
                        }
                        
                        i = dialog.declDeletions.iterator ();
                        while (i.hasNext ()) {
                            AcmeElementType type = (AcmeElementType )i.next ();
                            TypeAssignmentCommand command = new TypeAssignmentCommand (TypeAssignmentCommand.UNASSIGN_TYPE, element, type, false, _parentView);
//                            command.execute (_acmeDesign)
                            c.add (command);
                        }
                        
                        i = dialog.instDeletions.iterator ();
                        while (i.hasNext ()) {
                            AcmeElementType type = (AcmeElementType )i.next ();
                            TypeAssignmentCommand command = new TypeAssignmentCommand (TypeAssignmentCommand.UNASSIGN_TYPE, element, type, true, _parentView);
                            command.execute (_acmeDesign);
                            c.add (command);
                        }
                        c.execute (_acmeDesign);        
                    }
                    setInput (_selectedObject);
                }
                       
            }
        };
        _addTypeAction.setText ("Assign type");
        _addTypeAction.setDescription ("Assigns types to the selected element");
         
         
        _myContributions = new ActionContributionItem [] {
            new ActionContributionItem (_deleteTypeAction) {
                public boolean isDynamic () {return true;}
            },
            new ActionContributionItem (_addTypeAction) {
                public boolean isDynamic () {return true;}
            }   
        };
    }

    /**
     * @see AcmeStudio.ui.views.parts.ElementViewPart#isValidFor(AcmeObject)
     */
    public boolean isValidFor(AcmeObject o) {
        return true;
    }

    /* (non-Javadoc)
     * @see AcmeStudio.ui.views.parts.ElementViewPart#createControl(org.eclipse.swt.widgets.Composite, int, AcmeStudio.ui.views.IElementViewPage)
     */
    public void createControl(Composite parent, int style, IElementViewPage vp) {
        super.createControl(parent, style, vp);
        
        FillLayout tableLayout = new FillLayout ();
        this.getComposite ().setLayout (tableLayout);
        
        int flags = SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION;

       
        
        _viewer = new TableViewer (this.getComposite(), flags);
        _viewer.setContentProvider (new ViewContentProvider ());
        _viewer.setLabelProvider (new ViewLabelProvider ());
        _viewer.setSorter (new NameSorter ());
        
        Table table = _viewer.getTable ();
        
        table.addKeyListener (new KeyListener () {
            public void keyPressed (KeyEvent evt) {
                if (evt.character == SWT.DEL) {
                    _deleteTypeAction.run ();
                }
            }
            public void keyReleased (KeyEvent evt) {}
        });
        

        table.setHeaderVisible (true);
        table.setLinesVisible (true);
        TableColumn nameCol = new TableColumn (table, SWT.LEFT, 0);
        nameCol.setText ("Type");
        TableColumn fromCol = new TableColumn (table, SWT.LEFT, 1);
        fromCol.setText ("From family");
        ColumnLayoutData nameLayout = new ColumnWeightData (50, true);
        TableLayout layout = new TableLayout ();
        layout.addColumnData (nameLayout);
        layout.addColumnData(new ColumnWeightData (50, true));
        table.setLayout (layout);
        
        table.addMouseListener (new MouseListener () {
            public void mouseUp(MouseEvent evt) {
                if (_selectedObject instanceof AcmeElementType) return;
                
                TableItem item =
                    _viewer.getTable().getItem(new Point(evt.x, evt.y));
                if (item == null)
                    return;
               
                if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean (DISTINGUISH_DECLARED_INSTANTIATED_TYPES)) {
                	item.setChecked (true);
                	return;
                }
                
                AcmeElementType type = (AcmeElementType) item.getData();
                AcmeElement elem = (AcmeElement) _selectedObject;
                
                if (elem instanceof AcmeSystem) {
                    item.setChecked (true);
                }
                
                if (item.getChecked() && !elem.instantiatesType(type)) {
                    TypeAssignmentCommand command = new TypeAssignmentCommand (TypeAssignmentCommand.ASSIGN_TYPE, elem, type, true, _parentView);
                    command.execute (_acmeDesign);
                } else if (!item.getChecked() && elem.instantiatesType(type)) {
                    TypeAssignmentCommand command = new TypeAssignmentCommand (TypeAssignmentCommand.UNASSIGN_TYPE, elem, type, true, _parentView);
                    command.execute (_acmeDesign);
                }

            }
            public void mouseDoubleClick (MouseEvent evt) {
            }
            public void mouseDown (MouseEvent evt) {
                
            }
        });
        
        makeActions ();
        //hookContextMenu ();
        
       
    }

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#getTitle()
	 */
	public String getTitle() {
		return "Types";
	}

}
