/*
 * Created on Jul 27, 2004
 *
 */
package AcmeStudio.ui.views.parts;

import java.util.*;
import java.util.List;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import AcmeStudio.model.ModelHelper;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementProtoType;
import acmeLib.representations.AcmeRepresentation;


/**
 * @author Bradley Schmerl
 * Created Jul 27, 2004
 * 
 * A dialog for allowing the user to choose which outer ports to bind inner ports to
 */
public class MatchUnboundDialog extends Dialog {


	
	public class InternalComboBoxEditor extends ComboBoxCellEditor {

		private final Object []	_data;

	/**
	 * Constructor
	 * @param parent the parent
	 * @param labels The list of labels to use for the combo box
	 * @param data The corresponding data
	 */
		public InternalComboBoxEditor(Composite parent, String [] labels, Object [] data) {
			super (parent, labels);
			_data = data;
		}

		/**
		 * @return The value of the combo box
		 * @see org.eclipse.jface.viewers.CellEditor#doGetValue()
		 **/
		protected Object doGetValue() {
			Object o = super.doGetValue ();
			if (o instanceof Integer) {
				int index = ((Integer )o).intValue ();
				// If it's in the combo box, return the associated data
				if (index != -1) return _data [index];
				// return the string that was typed
				CCombo combo = (CCombo )this.getControl ();
				String text = combo.getText ();
				return text;
			}
			return o;
		}
		/**
		 * Sets the initial value of the combo editor
		 * @param value
		 * @see org.eclipse.jface.viewers.CellEditor#doSetValue(java.lang.Object)
		 **/
		protected void doSetValue(Object value) {
			if (value instanceof AcmeElementProtoType) {
				// It's new or auto-generated
				AcmeElementProtoType pt = (AcmeElementProtoType )value;
				if (pt.getName () == null) {
					super.doSetValue(new Integer (0));
				}
				else {
					super.doSetValue (new Integer (-1));
					((CCombo )this.getControl ()).setText (pt.getName ());
				}
			}
			else if (value instanceof AcmeElement) {
				// Bind to this element
				AcmeElement element = (AcmeElement )value;
				int index = 0;
				for (int i = 1; i < _data.length && index == 0; i++) {
					if (_data [i] instanceof AcmeElement) {
						AcmeElement dEl = (AcmeElement )_data [i];
						if (dEl.getName ().equals (element.getName ())) {
							index = i;
						}
					}
				}
				super.doSetValue(new Integer (index));
			}
		}
	
	}
	private static final String [] PROPERTIES = {"inner", "outer"};
	
	class CellModifier implements ICellModifier {
		public boolean canModify(Object element, String property) {
			return property.equals (PROPERTIES [OUTER_COL]);
		}
		public Object getValue(Object element, String property) {
			if (property.equals (PROPERTIES [OUTER_COL])) {
				if (element instanceof Map.Entry) {
					Map.Entry entry = (Map.Entry )element;
					if (entry.getValue() instanceof AcmeElement) {
						return entry.getValue ();
					}
					else if (entry.getValue () instanceof String) {
						return new AcmeElementProtoType ((String )entry.getValue ());
					}
					else 
						return new AcmeElementProtoType ((String )null);
				}
			}
			return null;
		}
		public void modify(Object element, String property, Object value) {
			TableItem item = (TableItem )element;
			Map.Entry entry = (Map.Entry )item.getData ();
			if (property.equals(PROPERTIES [OUTER_COL])) {
				if (value instanceof AcmeElementProtoType) {
					AcmeElementProtoType p = (AcmeElementProtoType )value;
					if (p.getName () == null) {
						_mappings.put (entry.getKey(), null);
					}
					else {
						_mappings.put (entry.getKey (), p.getName ());
					}
				}
				else {
					_mappings.put (entry.getKey (), value);
				}
				_viewer.refresh();
			}
		}
	}
	class TableLabelProvider extends LabelProvider implements ITableLabelProvider {
		
		
		public String getColumnText(Object element, int columnIndex) {
			if (element instanceof Map.Entry) {
				Map.Entry entry = (Map.Entry )element;
				if (columnIndex == MatchUnboundDialog.INNER_COL) {
					return ((AcmeElement )entry.getKey()).getQualifiedName().substring (_qualifiedFilter.length() + 1);
				}
				else if (columnIndex == MatchUnboundDialog.OUTER_COL) {
					if (entry.getValue () instanceof AcmeElement) {
						AcmeElement mappedTo = ((AcmeElement )entry.getValue());
						return mappedTo.getName ();
					}
					else if (entry.getValue () instanceof String) {
						return (String )entry.getValue ();
					}
					else
						return "Generated";
				}
			}
			return "";
		}
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}
	}
	class ContentProvider implements IStructuredContentProvider {
		public Object[] getElements(Object inputElement) {
			return _mappings.entrySet().toArray();
		}
		public void dispose() {
		}
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
	private TableViewer _viewer;
	private Composite _Container;
	private final AcmeRepresentation	_parentRep;
	private final List	_unboundInner;
	private final List	_unboundOuter;
	private final List	_currentlyBound;
	private final Map _mappings = new HashMap ();
	private String	_qualifiedFilter;
	private static final int	INNER_COL	= 0;
	private static final int	OUTER_COL	= 1;
	private CellEditor []	_cellEditors;
	public MatchUnboundDialog(Shell parentShell, List unboundInner, List unboundOuter, List currentlyBound, AcmeRepresentation rep) {
		super (parentShell);
		_unboundInner = unboundInner;
		_unboundOuter = unboundOuter;
		_currentlyBound = currentlyBound;
		_parentRep = rep;
		createDefaultMappings ();
		_qualifiedFilter = rep.getQualifiedName();
	}

	/**
	 * 
	 */
	private void createDefaultMappings() {
		for (Iterator iter = _unboundInner.iterator (); iter.hasNext ();) {
			AcmeElement  element = (AcmeElement  )iter.next ();
			AcmeElement matchingElement = findMatchingElementInList (element, _unboundOuter);
			if (matchingElement == null) {
				matchingElement = findMatchingElementInList (element, _currentlyBound);
			}
			_mappings.put(element, matchingElement);
		}
	}

	/**
	 * @param element
	 * @param list
	 * @return
	 */
	private AcmeElement findMatchingElementInList(AcmeElement element, List list) {
		Iterator i = list.iterator();
		AcmeElement matching = null;
		while (i.hasNext () && matching == null) {
			AcmeElement el = (AcmeElement )i.next ();
			 if (el.getName().equals (element.getName ())) {
			 	matching = el;
			 }
		}
		return matching;
	}

	protected Control createDialogArea(Composite parent) {
		_Container = (Composite )super.createDialogArea (parent);
		_Container.setLayout(new GridLayout());
		
			String elementStr = "ports";
			if (ModelHelper.isConnectorRep(_parentRep.getSystem ())) {
				elementStr = "roles";
			}
		
		{
			Label label = new Label(_Container, SWT.WRAP);
			label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			label.setText("AcmeStudio cannot automatically disambiguate mappings between inner and outer " + elementStr +". Please provide mappings");
		}
		{
			_viewer = new TableViewer(_Container, SWT.BORDER | SWT.FULL_SELECTION);
			_viewer.setCellModifier(new CellModifier());
			_viewer.setLabelProvider(new TableLabelProvider());
			_viewer.setContentProvider(new ContentProvider());
			Table table = _viewer.getTable();
			table.setHeaderVisible(true);
			table.setLinesVisible(true);
			table.setLayoutData(new GridData(GridData.FILL_BOTH));
			TableLayout layout = new TableLayout ();
			{
				TableColumn innerCol = new TableColumn(table, SWT.NONE);
				ColumnWeightData innerColWidth = new ColumnWeightData (50);
				layout.addColumnData(innerColWidth);
				innerCol.setText("Inner");
			}
			{
				TableColumn outerCol = new TableColumn(table, SWT.NONE);
				ColumnWeightData outerColWidth = new ColumnWeightData (50);
				layout.addColumnData (outerColWidth);
				outerCol.setText("Outer");
			}
			table.setLayout (layout);
			_viewer.setCellEditors(getCellEditors(table));
			_viewer.setColumnProperties(PROPERTIES);
			_viewer.setInput(new Object());
		}
		//
		return _Container;
	}

	protected void createButtonsForButtonBar(Composite parent) {
		createButton (parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		createButton (parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Match inner to outer elements");
	}
	
	protected CellEditor [] getCellEditors (Table table) {
		_cellEditors = new CellEditor [OUTER_COL + 1];
		_cellEditors [INNER_COL] = null;
		
		List elements = new ArrayList ();
		elements.add (null);
		elements.addAll(_unboundOuter);
		elements.addAll (_currentlyBound);
		
		String labels [] = new String [elements.size()];
		labels [0] = "Generated";
		for (int i = 1; i < labels.length; i++) {
			labels [i] = ((AcmeElement )elements.get (i)).getName();
		}
		
		_cellEditors [OUTER_COL] = new InternalComboBoxEditor (table, labels, elements.toArray ());
		
		return _cellEditors;
	}

	/**
	 * @return
	 */
	public Map getInnerToOuterMap() {
		return _mappings;
	}
}
