// $Header$
package AcmeStudio.ui.views.parts;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import acmeLib.elements.AcmeElement;

/**
 * @author schmerl
 *
 */
public class ModifyDescriptionDialog extends Dialog {

	/** The parent where this dialog is to be displayed **/
	private Composite _parent;
	
	/** The text editor for editing the rule **/
	private Text _descText;
	
	/** The element that will host the new rule **/
	private AcmeElement _element;
	
	/**
	 * Constructor for ModifyRuleDialog.
	 * @param parentShell
	 */
	public ModifyDescriptionDialog(Shell parentShell, AcmeElement element) {
		super(parentShell);
		_element = element;
	}

	/**
	 * Returns the _designRule.
	 * @return DesignRule
	 */
	public String getDescription() {
		return _descText.getText();
	}
	
	
	/**
	 * Sets the _designRule.
	 * @param _designRule The _designRule to set
	 */
	public void setDescription(String desc) {
		this._descText.setText(desc);
	}



	/**
	 * Creates the rule dialog, which has two text editors and a 
	 * parse button (in addition to OK and Cancel buttons)
	 * @param parent the dialog's parent
	 * @return the Control for the dialog
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		_parent = (Composite) super.createDialogArea(parent);

		// Make sure that the dialog is big enough to display everything
		_parent.setSize(500, 500);
		GridLayout gl = new GridLayout(2, false);
		_parent.setLayout(gl);

		Label ruleLabel = new Label(_parent, SWT.LEFT);
		ruleLabel.setText("Description:");
		ruleLabel.setLayoutData(new GridData());

		// Create the text editor for the rule		
		_descText = new Text (_parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		// Initialize the text
        _descText.setText(_element.getName());		
		GridData ruleData = new GridData (GridData.FILL_HORIZONTAL);
		ruleData.horizontalSpan = 2;
		ruleData.heightHint = 100;
		ruleData.widthHint = 300;
		_descText.setLayoutData (ruleData);
		
		// If the rule is modified, it must first be parsed before
		// it can be added, so disable the OK button (it will be 
		// re-enabled if the rule parses correctly
		_descText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e){
				ModifyDescriptionDialog.this.getButton(IDialogConstants.OK_ID).setEnabled (true);
			}
		});
		
		
		return _parent;

	}


	/**
	 * Creates the OK and Cancel button and button area of the dialog
	 * If the rule that is being modified by the dialog has pre-existing
	 * errors, then disable the OK button.
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonBar(Composite)
	 */
	protected Control createButtonBar(Composite parent) {
		Control buttonBar = super.createButtonBar(parent);
        ModifyDescriptionDialog.this.getButton(IDialogConstants.OK_ID).setEnabled (false);
		return buttonBar;
	}

}
