// $Header$
package AcmeStudio.ui.views.parts;

import java.util.Hashtable;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.util.Listeners;
import acmeLib.AcmeType;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.filters.PropertyTypeEnumFilter;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.designRules.Scope;
import acmeLib.elements.AcmeElement;
import acmeLib.properties.AcmeProperty;
import acmeLib.properties.AcmePropertyType;
import acmeLib.properties.AcmePropertyValue;
import acmeLib.typeManagement.AcmeTypeManager;

/**
 * @author Bradley Schmerl (schmerl@cs.cmu.edu)
 * 
 * Implements a dialog box to modify a property. Everything about
 * the property can be modified, i.e., name, value, type, and meta-properties
 *
 */
public class ModifyPropertyDialog extends Dialog {


    public static int MODIFY_DETAILS = 1;
    public static int MODIFY_VALUE = 2;
	/** The area to add information to this dialog **/
	private Composite _parent;
	
	/** A text editor containing the name of the property **/
	private Text _nameText;
	
	/** A combo box containing all the visible types for the property **/
	private Combo _typeCombo;
	
	/** A check box indicating whether to set a value for the property **/
	private Button _assignValueButton;
	
	/** The property being modifie **/
	private AcmeProperty _propertyToModify = null;
	
	/** The property name **/
	public String _propertyName;
	
	/** the type of the property **/
	public AcmePropertyType _propertyType;
	
	/** The value of the property **/
	public AcmePropertyValue _propertyValue;
	
	private java.util.List _metaProperties;
	
	/** A map containing all the primitive property types **/
	private static Map _primitiveTypes;
	
	/** Initialize the primitive types **/
	static {
		_primitiveTypes = new Hashtable ();
		_primitiveTypes.put ("string", ConstantTypes.STRING_TYPE);
		_primitiveTypes.put ("int", ConstantTypes.INT_TYPE);
		_primitiveTypes.put ("float", ConstantTypes.FLOAT_TYPE);
		_primitiveTypes.put ("boolean", ConstantTypes.BOOLEAN_TYPE);
		_primitiveTypes.put ("sequence", ConstantTypes.ANY_SEQUENCE_TYPE);
		_primitiveTypes.put ("set", ConstantTypes.ANY_SET_TYPE);
		_primitiveTypes.put ("record", ConstantTypes.ANY_RECORD_TYPE);
		_primitiveTypes.put ("enum", ConstantTypes.EMPTY_ENUM_TYPE);
	}
		
	
    private int _modifyFlags = 0;
	private PropertyValueInspector _valueInspector;
	/**
	 * Constructor for ModifyPropertyDialog.
	 * @param parentShell the place to display the dialog
	 * @param prop the property to modify
	 */
	public ModifyPropertyDialog(Shell parentShell, AcmeProperty prop, int flags) {
		super(parentShell);
		_propertyToModify = prop;
	   _modifyFlags = flags;
	}

	/** 
	 * Returns a list of the meta properties for this property
	 * Actually returns those stored in the editor for the meta properties
	 * as that contains all the new meta properties.
	 * @return the list of modified meta properties
	 */
	public java.util.List getMetaProperties () {

		return _metaProperties;
	}


	/**
	 * Creates the dialog area
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		
		_parent = (Composite )super.createDialogArea(parent);
		_parent.setSize (500, 500);
		GridLayout gl = new GridLayout (2, false);
		_parent.setLayout (gl);
		
		// Name : Text
		Label nameLabel = new Label (_parent, SWT.RIGHT);
		nameLabel.setText ("Name:");
		nameLabel.setLayoutData( new GridData ());
		
		_nameText = new Text (_parent, SWT.SINGLE | SWT.BORDER);
		_nameText.setLayoutData (new GridData (GridData.FILL_HORIZONTAL));
		_nameText.addVerifyListener(Listeners.VERIFY_ACMEID);
		if (_propertyToModify.getName () == null) {
			// property is new
			_nameText.setText ("aProp");
		}
		else {
		
			_nameText.setText (_propertyToModify.getName());
		}
	
		// Type : EditCombo
		Label typeLabel = new Label (_parent, SWT.RIGHT);
		typeLabel.setText ("Type:");
		typeLabel.setLayoutData( new GridData ());
		
		_typeCombo = new Combo (_parent, SWT.DROP_DOWN | SWT.READ_ONLY);
		fillTypes ();
		
		if (_propertyToModify.getType () == null) {
			_typeCombo.setText ("int");
		}
		else {
			_typeCombo.setText (_propertyToModify.getType().getTypeName ());
            if (_propertyToModify.getType ().getTypeName ().equals ("")) {
                if (_propertyToModify.getType () == ConstantTypes.UNSPECIFIED_TYPE)  {
                    _typeCombo.setText ("any");
                }
                else {
                _typeCombo.setText (_propertyToModify.getType ().toString ());   
                }
            }
		}
		_typeCombo.setLayoutData (new GridData (GridData.FILL_HORIZONTAL));
		_typeCombo.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected (SelectionEvent evt) {
				int selection = _typeCombo.getSelectionIndex();
				String typeName = _typeCombo.getItem(selection);
				AcmePropertyType type = (AcmePropertyType )_typeCombo.getData (typeName);
				_valueInspector.setType (type);
			}
		});
			
		// [] Assign Value to Property
	    _assignValueButton = new Button (_parent, SWT.CHECK);
		_assignValueButton.setText ("Assign Value to Property");
		_assignValueButton.setSelection (true);
		GridData bd = new GridData ();
		bd.horizontalSpan = 2;
		_assignValueButton.setLayoutData (bd);
		 
		// Only enable to value text area if the assign button is checked
		_assignValueButton.addSelectionListener (
			new SelectionAdapter () {
				public void widgetSelected (SelectionEvent event) {
					if (event.widget == _assignValueButton &&
						_assignValueButton.getSelection()) {
						_valueInspector.setEnabled(true);
					}	
					else if (!_assignValueButton.getSelection ()) {
						_valueInspector.setEnabled (false);
					}
				}
			}
		);
		_assignValueButton.setSelection (_propertyToModify.getValue () != null);
 
		_valueInspector = new PropertyValueInspector (parent, SWT.NONE, _propertyToModify);

        _nameText.setEnabled (false);
        _typeCombo.setEnabled (false);
        _valueInspector.setEnabled (false);
       

        if ((_modifyFlags & MODIFY_DETAILS) == MODIFY_DETAILS) {
            _nameText.setEnabled(true);
            _typeCombo.setEnabled(true);
        }
        if ((_modifyFlags & MODIFY_VALUE) == MODIFY_VALUE) {
            _valueInspector.setEnabled(_propertyToModify.getValue() != null);
        }
        _valueInspector.inspectValue (_propertyToModify);

		return _parent;
	}

	/** 
	 * Fills the type combo with all the property types available in scope
	 */
	private void fillTypes () {
		fillComboWithPropertyTypesInScope (_typeCombo, _propertyToModify.getScope ());
	}
	
	
	public static String [] getAvailablePropertyTypeNames (Scope scope) {
		String [] baseTypes = {"int", "float", "string", "boolean", "sequence", "set", "enum", "record", "any"};
		AcmeElement scopingElement = null;
		if (scope instanceof AcmeProperty) {
			scopingElement = (AcmeElement )((AcmeProperty )scope).getScope ();
		}
		else {
			scopingElement = (AcmeElement )scope;
		}
	
		AcmeEnumeration types = scopingElement.getAvailableTypes ().castToAcmeSetVector ().enumerate (new PropertyTypeEnumFilter ());
		
		String [] availableTypes = new String [baseTypes.length + types.getSize()];
		int index = 0;
		for (int i = 0; i < baseTypes.length; i++) {
			availableTypes [index++] = baseTypes [i];
		}
		
		while (types.moreItems()) {
			availableTypes [index++] = ((AcmeType )types.getNextItem ()).getTypeName();	
		}
		
		return availableTypes;
	}
	
	public static AcmePropertyType [] getAvailablePropertyTypes (Scope scope) {
		AcmePropertyType [] baseTypes = {ConstantTypes.INT_TYPE, ConstantTypes.FLOAT_TYPE, ConstantTypes.STRING_TYPE, ConstantTypes.BOOLEAN_TYPE,
				ConstantTypes.ANY_SEQUENCE_TYPE, ConstantTypes.ANY_SET_TYPE, ConstantTypes.EMPTY_ENUM_TYPE, ConstantTypes.ANY_RECORD_TYPE, ConstantTypes.EXPLICIT_ANY_TYPE};

		AcmeElement scopingElement = null;
		if (scope instanceof AcmeProperty) {
			scopingElement = (AcmeElement )((AcmeProperty )scope).getScope ();
		}
		else {
			scopingElement = (AcmeElement )scope;
		}
		AcmeEnumeration types = scopingElement.getAvailableTypes().castToAcmeSetVector().enumerate (new PropertyTypeEnumFilter ());
		
		AcmePropertyType [] availableTypes = new AcmePropertyType [baseTypes.length + types.getSize ()];
		int index = 0;
		for (int i = 0; i < baseTypes.length; i++) {
			availableTypes [index++] = baseTypes [i];
		}
		while (types.moreItems()) {
			AcmePropertyType type = (AcmePropertyType) types.getNextItem();
			availableTypes [index++] = type;
		}
		
		return availableTypes;
	}
	
	/**
	 * finds all the property types available in scope of the property
	 * @param combo the Combo to fill in the types
	 * @param scope the Scope in which to look up types
	 */
	public static void fillComboWithPropertyTypesInScope (Combo combo, Scope scope) {
		AcmePropertyType [] types = getAvailablePropertyTypes(scope);
		for (int i = 0; i < types.length; i++) {
			combo.add(types [i].getTypeName());
			combo.setData(types [i].getTypeName(), types [i]);
		}
//		combo.setItems (getAvailablePropertyTypeNames(scope));
		
		
	}

	public static AcmePropertyType getPropertyType (String name, Scope scope) {
		if (_primitiveTypes.containsKey (name)) {
			return (AcmePropertyType )_primitiveTypes.get (name);
		}
		else {
            AcmePropertyType t = (AcmePropertyType )scope.lookupName(name, true);
            if (t == null) {
                t = ConstantTypes.EXPLICIT_ANY_TYPE;
            }
			return t;
		}
	}




	/**
	 * When OK is pressed, parse the value and check that it matches the type
	 * Only process the OK if they match.
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		boolean okToCloseDialog = true;
		
		// Set the new name
		_propertyName = _nameText.getText();
		
		// Set the new type
		_propertyType = getPropertyType (_typeCombo.getText(), _propertyToModify.getScope ());
		
		// Only parse value if the assign value button is selected
		if (_assignValueButton.getSelection()) {
			try {
				
				_propertyValue = _valueInspector.getPropertyValue ();
							// Typecheck the property
				boolean checksOK =
					AcmeTypeManager.typecheckProperty(
						_propertyType,
						_propertyValue);
						
				if (!checksOK) { // Bring up error
					Shell shell =
						AcmeStudioPlugin
							.getDefault()
							.getWorkbench()
							.getActiveWorkbenchWindow()
							.getShell();
					MessageDialog.openError(
						shell,
						"Type error",
						"Value does not satisfy selected type");

					okToCloseDialog = false;
				}
			}
			catch (ParseException e) {
				Shell shell =
					AcmeStudioPlugin
						.getDefault()
						.getWorkbench()
						.getActiveWorkbenchWindow()
						.getShell();
				MessageDialog.openError(
					shell,
					"Parse error",
					"Unable to parse the value.");
				okToCloseDialog = false;
			}
		}
		
		_metaProperties = _valueInspector.getMetaProperties ();
		

		
		if (okToCloseDialog) super.okPressed();
	}

}
