package AcmeStudio.ui.views.parts;

import java.util.Iterator;
import java.util.TreeSet;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import AcmeStudio.util.Helper;
import AcmeStudio.util.Listeners;
import AcmeStudio.util.Utils;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.designRules.Scope;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeGroup;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeRole;
import acmeLib.elements.AcmeSystem;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class NewElementDialog extends TitleAreaDialog {

    AcmeElement _element;
    Table _addTypeList;
    Text _name;
    Composite _parent;
    java.util.SortedSet _types;
    Class _clazz;
	private Scope _scope;


    

    public NewElementDialog (Shell shell, AcmeEnumeration types, Class clazz, Scope scope) {
        super (shell);   
        // Make sure there are no duplicates (BugFix #63)
        if (types != null) {     
            _types = new TreeSet (new Utils.AcmeElementNameComparator ());
            _types.addAll (Helper.convertAcmeEnumerationToList(types.castToAcmeSetVector().enumerate()));
           
        }
        _clazz = clazz;
        _scope = scope;
    }

    /**
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite top = (Composite ) super.createDialogArea(parent);
        top.setLayout (new GridLayout (1, false));
        _parent = new Composite (top, SWT.NONE);
        _parent.setLayoutData (new GridData (GridData.FILL_HORIZONTAL | GridData.GRAB_VERTICAL));
        GridLayout gl = new GridLayout (2, false);
        _parent.setLayout(gl);
        
        Label l = new Label (_parent, SWT.NONE);
        l.setText ("Name: ");

        _name = new Text (_parent, SWT.SINGLE | SWT.BORDER);
        GridData gd1 = new GridData (GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL);
//        gd1.horizontalAlignment = GridData.FILL;
        _name.setLayoutData(gd1);
        _name.addVerifyListener(Listeners.VERIFY_ACMEID);
        _name.addModifyListener (new ModifyListener () {
            public void modifyText (ModifyEvent evt) {
                validate ();
            } 
        });
        
        Label l2 = new Label (_parent, SWT.NONE);
        l2.setText ("Select types:");
        GridData gd3 = new GridData ();
        gd3.horizontalSpan = GridData.FILL;
        l2.setLayoutData(gd3);
        
        int style = SWT.MULTI | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL;
        style |= getCheck ();
        
        _addTypeList = new Table (_parent, style);
        _addTypeList.setSize(300, 100);
 
        // Make sure that if an item has been checked, that it is also selected
        _addTypeList.addMouseListener(new MouseListener () {
            public void mouseUp (MouseEvent evt) {
                if (evt.widget == _addTypeList) {
                    TableItem item = _addTypeList.getItem (new Point (evt.x, evt.y));
                    if (item != null) {
                        if (item.getChecked ()) {
                            _addTypeList.select (_addTypeList.indexOf (item));
                        }
                    }
                }
            }
            
            public void mouseDown (MouseEvent evt) {
           }
            
            public void mouseDoubleClick (MouseEvent evt) {
                
            }
        });
        
        TableColumn col1 = new TableColumn (_addTypeList, SWT.LEFT, 0);
        TableLayout tl = new TableLayout ();
        tl.addColumnData (new ColumnWeightData (100));
        _addTypeList.setLayout(tl);
        GridData gd2 = new GridData (GridData.FILL_BOTH);
        gd2.horizontalAlignment = GridData.FILL;   
        gd2.horizontalSpan = GridData.FILL;
        gd2.verticalAlignment = GridData.FILL; 
//        gd2.grabExcessHorizontalSpace = true;
//        gd2.grabExcessVerticalSpace = true;    
        //gd2.verticalSpan = GridData.FILL_VERTICAL;
        _addTypeList.setLayoutData(gd2);
        
        fillTypeList ();
        
        return _parent;
    }

    private int getCheck () {
        if (_clazz != AcmeSystem.class) {
            return SWT.CHECK;
        }   
        return 0;
    }

    private void fillTypeList () {
        Iterator i = _types.iterator();
               
        while (i.hasNext ()) {
            AcmeElementType type = (AcmeElementType )i.next ();
            TableItem item = new TableItem (_addTypeList, SWT.NONE);
            item.setText (type.getTypeName ());
            item.setData (type);
            
        }
    }

    /**
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed() {
               
        String name = _name.getText();
        
        Object [] selections = _addTypeList.getSelection();
        
        _element = constructNewElement (name, selections);
        
        super.okPressed();
    }

    public AcmeElement getNewElement () {
        return _element;
    }

    private AcmeElement constructNewElement (String name, Object [] selections) {
        AcmeElement element = null;
        if (_clazz == AcmeSystem.class) {
            element = new AcmeSystem (name);
        }
        else if (_clazz == AcmeComponent.class) {
            element = new AcmeComponent (name);
        }
        else if (_clazz == AcmeConnector.class) {
            element = new AcmeConnector (name);
        }
        else if (_clazz == AcmeRole.class) {
            element = new AcmeRole (name);
        }
        else if (_clazz == AcmePort.class) {
            element = new AcmePort (name);
        }
        else if (_clazz == AcmeGroup.class) {
            element = new AcmeGroup (name);
        }
        
        for (int i = 0; i < selections.length; i++) {
            TableItem item = (TableItem )selections [i];
            AcmeElementType type = (AcmeElementType )item.getData ();
            element.addDeclaredType(type);
            if (item.getChecked() || _clazz == AcmeSystem.class) {
                element.addInstantiatedType (type);
            }
            element.instantiateElements();
        }
        return element;        
    }              
    /**
     * @see org.eclipse.jface.dialogs.Dialog#createButtonBar(Composite)
     */
    protected Control createButtonBar(Composite parent) {
        Control c =  super.createButtonBar(parent);
        validate ();
        return c;
    }

    private void validate() {
    	String name = _name.getText ();
    	if (name == null || name.equals ("")) {
    		getButton (IDialogConstants.OK_ID).setEnabled (false);
    		setErrorMessage ("The name cannot be empty");
    		return;
    	}
    	
    	if (_scope.lookupName(name, false) != null) {
    		setErrorMessage ("There is a duplicate name already defined");
    		getButton (IDialogConstants.OK_ID).setEnabled (false);
    		return;
    	}
    	
    	setErrorMessage (null);
    	getButton (IDialogConstants.OK_ID).setEnabled (_name.getText ().length () > 0);
    }
}
