/*
 * Created on May 19, 2003
 *
 */
package AcmeStudio.ui.views.parts;

import java.util.StringTokenizer;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.part.PageBook;

import AcmeStudio.util.AutoApplyEditableComboBoxCellEditor;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.elements.AcmeElement;
import acmeLib.properties.*;

/**
 * @author schmerl
 */
public class NewPropertyTypeDialog extends Dialog {


    private PageBook _fldDefinitions;

    protected int _currentSelection;

    /** The place that the new type will be declared in **/
    protected AcmeElement _scope;
    
    /** The property type to base this on (if any) **/
    protected AcmePropertyType _origPropertyType = null;
    
    /** The constructed property type **/
    protected AcmePropertyType _newPropertyType = null;
    
    protected RecordType _recordPlaceholder = new RecordType ("__dummy__");

    protected Text _txtName;
    protected Button _btnNewIntType;
    protected Button _btnNewFloatType;
    protected Button _btnNewStringType;
    protected Button _btnNewBooleanType;
    protected Button _btnNewEnumType;
    protected Button _btnNewRecordType;
    protected Button _btnNewSequenceType;
    protected Button _btnNewSetType;
    protected Button _btnNewOtherType;
    
    protected Combo _cmbContainedType;
    protected TableViewer _recordFields;
    protected Text _txtEnumValues;

    
    protected static final int INT = 0, FLOAT = 1, STRING = 2, BOOL = 3, ENUM = 4, RECORD = 5, SEQ = 6, SET = 7, OTHER = 8;
    protected static final String [] TYPEIMAGE = {"int", "float", "string", "boolean", "enum", "record", "sequence", "set", "other"};
    private static final String [] REC_PROPERTIES = {"name", "type"};
    
    private String [] _availablePropertyTypes;
    private boolean _editable;
    
    protected Composite _pages [];
    
    
    protected void showPage (int page) {
        _fldDefinitions.showPage (_pages [page]);
//        _pages [_currentSelection].setVisible (false);
//        _pages [page].setVisible (true);
//        this.getDialogArea().update();

    }

    
    protected Button createRadioButton(Group radioGroup, final int kind) {
        Button button = new Button (radioGroup, SWT.RADIO);
        button.setText(TYPEIMAGE[kind]);
        button.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                showPage(kind);
                _currentSelection = kind;
            }
        });
        
        return button;
    }

    /**
     * 
     * @param parentShell The shell to raise the dialog in
     * @param scope The that the new property will be defined in
     */
    public NewPropertyTypeDialog(Shell parentShell, AcmeElement scope) {
        super(parentShell);
        _scope = scope;
    }
    
    public NewPropertyTypeDialog (Shell parentShell, AcmeElement scope, AcmePropertyType type) {
        super (parentShell);
        _scope = scope;
        _origPropertyType = type;
    }
    
    public AcmePropertyType getNewType () {
        return _newPropertyType;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite top = (Composite ) super.createDialogArea(parent);
        
        GridLayout tgl = new GridLayout (2, false);
        top.setLayout (tgl);
        
        Label nameLabel = new Label (top, SWT.NONE);
        nameLabel.setText ("Type Name:");
        _txtName = new Text (top, SWT.BORDER);
        GridData ngd = new GridData (GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL);
        _txtName.setLayoutData (ngd);
        
        Group radioGroup = new Group (top, SWT.NONE);
        radioGroup.setText("Type:");
        GridData rgtd = new GridData (GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
        rgtd.horizontalSpan = GridData.FILL;
        radioGroup.setLayoutData(rgtd);
        radioGroup.setLayout (new RowLayout (SWT.HORIZONTAL));
        
        _btnNewIntType = createRadioButton (radioGroup, INT);
        _btnNewFloatType = createRadioButton (radioGroup, FLOAT);
        _btnNewStringType = createRadioButton (radioGroup, STRING);
        _btnNewBooleanType= createRadioButton (radioGroup, BOOL);
        _btnNewEnumType = createRadioButton(radioGroup, ENUM);
        _btnNewRecordType = createRadioButton (radioGroup, RECORD);
        _btnNewSetType = createRadioButton (radioGroup, SET);
        _btnNewSequenceType = createRadioButton (radioGroup, SEQ);
        _btnNewOtherType = createRadioButton (radioGroup, OTHER);
        
        Group pageGroup = new Group (top, SWT.NONE);
        pageGroup.setText("Definition:"); 
        pageGroup.setLayout (new FillLayout ());
        GridData pggd = new GridData (GridData.GRAB_VERTICAL | GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
        pggd.horizontalSpan = GridData.FILL;
        pageGroup.setLayoutData(pggd);
        
        _pages = new Composite [OTHER+1];
        
        _fldDefinitions = new PageBook (pageGroup, SWT.NONE);
        pageGroup.setSize (100, 300);
        Composite empty = new Composite (_fldDefinitions, SWT.NONE);
        empty.setLayout (new FillLayout ());
        Label mtL = new Label (empty, SWT.NONE);
        
        mtL.setText("No further details.");
        empty.setSize (100, 300);
        
        Composite typeChooser = createTypeChooser (_fldDefinitions);
        empty.setVisible(false);
        _pages [INT] = empty;
        _pages [FLOAT] = empty;
        _pages [STRING] = empty;
        _pages [BOOL] = empty;
        _pages [ENUM] = createEnumPage (_fldDefinitions);
        _pages [RECORD] = createRecordPage (_fldDefinitions);
        _pages [SET] = typeChooser;
        _pages [SEQ] = typeChooser;
        _pages [OTHER] = typeChooser;
        
        _btnNewBooleanType.setEnabled(_editable);
        _btnNewEnumType.setEnabled (_editable);
        _btnNewFloatType.setEnabled (_editable);
        _btnNewIntType.setEnabled (_editable);
        _btnNewOtherType.setEnabled (_editable);
        _btnNewRecordType.setEnabled (_editable);
        _btnNewSequenceType.setEnabled(_editable);
        _btnNewSetType.setEnabled(_editable);
        _btnNewStringType.setEnabled (_editable);
        _txtEnumValues.setEditable(_editable);
        _cmbContainedType.setEnabled(_editable);
        
//        createTabItem (_fldDefinitions, INT);
//        createTabItem (_fldDefinitions, FLOAT);
//        createTabItem (_fldDefinitions, STRING);
//        createTabItem (_fldDefinitions, BOOL);
//        createTabItem (_fldDefinitions, ENUM);
//        createTabItem (_fldDefinitions, RECORD);
//        createTabItem (_fldDefinitions, SET);
//        createTabItem (_fldDefinitions, SEQ);
//        createTabItem (_fldDefinitions, OTHER);
        
        fillFromOrigType ();
        
        return top;
    }

    /**
     * @param pageGroup
     * @return
     */
    private Composite createEnumPage(Composite parent) {
        Composite evPage = new Composite (parent, SWT.NONE);
        evPage.setLayout (new GridLayout (2, false));
        Label l = new Label (evPage, SWT.NONE);
        l.setText ("Values:");
        _txtEnumValues = new Text (evPage, SWT.BORDER);
        _txtEnumValues.setToolTipText("Enter a comma separated list of values for the enum");
        _txtEnumValues.setLayoutData(new GridData (GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));
        return evPage;
    }


    /**
     * 
     */
    private void fillFromOrigType() {
        if (_origPropertyType != null) {
            _txtName.setText (_origPropertyType.getTypeName ());
            _txtName.setEnabled(false);
            
            if (_origPropertyType instanceof IntType) {
                _btnNewIntType.setSelection(true);
                showPage (INT);
            }
            else if (_origPropertyType instanceof FloatType) {
                _btnNewFloatType.setSelection (true);
                showPage (FLOAT);
            }
            else if (_origPropertyType instanceof StringType) {
                _btnNewStringType.setSelection (true);
                showPage (STRING);
            }
            else if (_origPropertyType instanceof BooleanType) {
                _btnNewBooleanType.setSelection (true);
                showPage (BOOL);
            }
            else if (_origPropertyType instanceof RecordType) {
                _btnNewRecordType.setSelection (true);
                _recordPlaceholder = (RecordType )((RecordType )_origPropertyType).copy();
                _recordFields.setInput(_recordPlaceholder);
                showPage (RECORD);
            }
            else if (_origPropertyType instanceof SetType) {
                _btnNewSetType.setSelection (true);
                showPage (SET);
            }
            else if (_origPropertyType instanceof SequenceType) {
                _btnNewSequenceType.setSelection (true);
                showPage (SEQ);
            }
            else if (_origPropertyType instanceof EnumType)  {
            	_btnNewEnumType.setSelection (true);
            	showPage (ENUM);
            }
            else if (_origPropertyType instanceof AcmePropertyType) {
                _btnNewOtherType.setSelection (true);
                showPage (OTHER) ;  
            }
            else {
                showPage (0);
            }
               
        }
        else {
            _txtName.setText ("PropT");
            
        }
    }


    /**
     * @param pageGroup
     * @return
     */
    private Composite createRecordPage(Composite parent) {
        Composite rgPage = new Composite (parent, SWT.NONE);
        rgPage.setLayout (new FillLayout ());
        
        _recordFields = new TableViewer (rgPage, SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
        
        Table table = _recordFields.getTable ();
        table.setHeaderVisible(true);
        table.setLinesVisible(true);
        
        TableColumn nameCol = new TableColumn (table, SWT.LEFT, 0);
        TableColumn typeCol = new TableColumn (table, SWT.LEFT, 1);
        nameCol.setText("Field name");
        typeCol.setText ("Field type");
        
        TableLayout layout = new TableLayout ();
        layout.addColumnData (new ColumnWeightData (50, false));
        layout.addColumnData(new ColumnWeightData (50, false));
        table.setLayout(layout);
        _recordFields.setContentProvider(new RecordFieldContentProvider ());
        _recordFields.setLabelProvider(new RecordFieldLabelProvider ());
        _recordFields.setCellModifier (new RecordCellModifier ());
        _recordFields.setCellEditors(getCellEditors (_recordFields.getTable()));
        _recordFields.setInput(_recordPlaceholder);
        _recordFields.setColumnProperties(REC_PROPERTIES);
//        rgPage.setVisible (false);
        return rgPage;
    }


    /**
     * @param table
     * @return
     */
    private CellEditor[] getCellEditors(Table table) {
        final CellEditor [] editors = new CellEditor [2];
        class AutoApplyTextCellEditor extends TextCellEditor {
            public AutoApplyTextCellEditor (Composite parent) {
                super (parent);
            }
            public void fireApplyEditorValue () {
                super.fireApplyEditorValue();   
            }
        }
        
        
        editors [0] = new AutoApplyTextCellEditor (table);
        editors [0].getControl ().addFocusListener(new FocusAdapter () {
            public void focusLost (FocusEvent e) {
                ((AutoApplyTextCellEditor )editors [0]).fireApplyEditorValue ();
            }
        });
        _availablePropertyTypes = ModifyPropertyDialog.getAvailablePropertyTypeNames(_scope);
        editors [1] = new AutoApplyEditableComboBoxCellEditor (table, _availablePropertyTypes);
        editors [1].getControl ().addFocusListener(new FocusAdapter () {
            public void focusLost (FocusEvent e) {
                ((AutoApplyEditableComboBoxCellEditor )editors [1]).fireApplyEditorValue();
            }
        });
        
        return editors;
    }


    /**
     * @param pageGroup
     * @return
     */
    private Composite createTypeChooser(Composite parent) {
        _availablePropertyTypes = ModifyPropertyDialog.getAvailablePropertyTypeNames(_scope);
        
        Composite tcPage = new Composite (parent, SWT.NONE);
        tcPage.setLayout (new GridLayout (2, false));
        Label l = new Label (tcPage, SWT.NONE);
        l.setText ("Contained type");
        _cmbContainedType = new Combo (tcPage, SWT.DROP_DOWN | SWT.READ_ONLY);
        _cmbContainedType. setItems (_availablePropertyTypes);
        GridData ctgd = new GridData (GridData.GRAB_HORIZONTAL);
        ctgd.grabExcessHorizontalSpace = true;
        ctgd.horizontalAlignment = GridData.HORIZONTAL_ALIGN_FILL;
        _cmbContainedType.setLayoutData(ctgd);
        return tcPage;
    }


    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed() {
        createNewType ();
        
        
        super.okPressed();
    }


    /**
     * 
     */
    private void createNewType() {
        switch (_currentSelection) {
            case INT :
                _newPropertyType = new IntType (_txtName.getText ());
                break;
            case FLOAT:
                _newPropertyType = new FloatType (_txtName.getText ());
                break;
            case STRING:
                _newPropertyType = new StringType (_txtName.getText ());
                break;
            case BOOL:
                _newPropertyType = new BooleanType (_txtName.getText ());
                break;
            case ENUM:
                _newPropertyType = new EnumType (_txtName.getText ());
                StringTokenizer tokens = new StringTokenizer (_txtEnumValues.getText (), ",");
                while (tokens.hasMoreTokens()) {
                    ((EnumType )_newPropertyType).addValue(tokens.nextToken());
                }
                break;
            case RECORD:
                _newPropertyType = new RecordType (_txtName.getText ());
                int index = 0;
                while (_recordFields.getElementAt(index) != null) {
                    AcmeRecordField f = (AcmeRecordField )_recordFields.getElementAt(index++);
                    if (f.getName () != null && f.getType() != null) {
                        ((RecordType )_newPropertyType).addField(f);       
                    }   
                } 
                break;
            case SET:
                _newPropertyType = new SetType (_txtName.getText ());
                ((SetType )_newPropertyType).setSetType(ModifyPropertyDialog.getPropertyType(_cmbContainedType.getText (), _scope));
                break;
            case SEQ:
                _newPropertyType = new SequenceType (_txtName.getText ());
                ((SequenceType )_newPropertyType).setSequenceType(ModifyPropertyDialog.getPropertyType(_cmbContainedType.getText (), _scope));
                break;
            
        }
    }


    private class RecordCellModifier implements ICellModifier {

        /**
         * 
         */
        public RecordCellModifier() {
            super();
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object, java.lang.String)
         */
        public boolean canModify(Object element, String property) {
            return _editable;
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object, java.lang.String)
         */
        public Object getValue(Object element, String property) {
            Object value = "";
            if (element instanceof AcmeRecordField) {
                AcmeRecordField f = (AcmeRecordField) element;
                if (property.equals(REC_PROPERTIES[0])) {
                    return f.getName();
                }
                else if (property.equals(REC_PROPERTIES[1])) {
                    boolean foundType = false;
                    int index = 0;
                    if (f.getType() == null)
                        index = -1;
                    else {
                        while (index < _availablePropertyTypes.length && !foundType) {
                            if (_availablePropertyTypes[index].equals(f.getType().getTypeName())) {
                                foundType = true;
                            }
                            else {
                                index++;
                            }
                        }
                    }
                    value = new Integer(index);
                }
            }
            else {
                if (property.equals(REC_PROPERTIES[0])) {
                    value = "";
                }
                else if (property.equals(REC_PROPERTIES[1])) {
                    value = new Integer(-1);
                }
            }
            return value;
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object, java.lang.String, java.lang.Object)
         */
        public void modify(Object element, String property, Object value) {
            if (element instanceof TableItem) {
                TableItem item = (TableItem) element;
                AcmeRecordField f = (AcmeRecordField) item.getData();

                if (property.equals(REC_PROPERTIES[0])) {
                    if (f.getName().equals("") && f.getType() == null) {
                        // this is using the dummy, so add the new field
                        AcmeRecordField newF = new AcmeRecordField((String) value, null, null);
                        _recordPlaceholder.addField(newF);
                    }
                    else {
                        try {
                            _recordPlaceholder.removeField(f.getName ());
                            f.setName((String) value);
                            _recordPlaceholder.addField (f);
                        }
                        catch (AcmeIllegalNameException e) {
                            e.printStackTrace();
                        }
                    }
 
                        
                }
                else if (property.equals(REC_PROPERTIES[1])) {
                    int index = ((Integer )value).intValue ();
                    AcmePropertyType type = null;
                    if (index != -1) {
                        String val = _availablePropertyTypes[((Integer) value).intValue()];
                        type = ModifyPropertyDialog.getPropertyType(val, _scope);
                    }
                    if (f.getName().equals("") && f.getType() == null) {
                        // this is using the dummy, so add the new field
                        AcmeRecordField newF = new AcmeRecordField("", type, null);
                        _recordPlaceholder.addField(newF);
                        _recordFields.refresh (true);
                    }
                    else {
                        f.setType(type);
                    }
                }
                if (f.getName ().equals ("") && f.getType () == null) {
                     _recordPlaceholder.removeField (f);
                 }

            }
            _recordFields.refresh(true);
        }
        

    }

    private static final Object [] emptyElements = new Object [0];

    private class RecordFieldContentProvider implements IStructuredContentProvider {
           



		/* (non-Javadoc)
         * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
         */
        public Object[] getElements(Object inputElement) {
            Object elements [] = emptyElements;
            RecordType rt = (RecordType )inputElement;
            AcmeEnumeration fields = rt.getFields();
            int additional = _editable?1:0;
            elements = new Object [fields.getSize () + additional];
            int index = 0;
            while (fields.moreItems ()) {
                elements [index++] = fields.getNextItem ();       
            }
            if (_editable) {
            	elements [index] = new AcmeRecordField ("", null, null);
            }
            return elements;
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.IContentProvider#dispose()
         */
        public void dispose() {

        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
         */
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

        }

}


    private class RecordFieldLabelProvider extends LabelProvider implements ITableLabelProvider {
        
            /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
         */
        public Image getColumnImage(Object element, int columnIndex) {
            return null;
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
         */
        public String getColumnText(Object element, int columnIndex) {
            if (columnIndex == 0) {
                String name = ((AcmeRecordField )element).getName ();
                if (name.equals ("")) {
                    return "<Enter field name>";
                }
                else 
                    return ((AcmeRecordField )element).getName();   
            }
            else if (columnIndex == 1) {
                AcmePropertyType t = ((AcmeRecordField )element).getType();
                if (t != null) {
                    return t.getTypeName();
                }
                else return "<Enter field type>";   
            }
            return "";
        }

}


	/**
	 * @param b
	 */
	public void setEditable(boolean editable) {
		_editable = editable;

	}
}
