package AcmeStudio.ui.views.parts;

import java.util.Iterator;
import java.util.TreeSet;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import AcmeStudio.util.Listeners;
import AcmeStudio.util.Utils;
import acmeLib.designRules.Scope;
import acmeLib.elements.AcmeComponentType;
import acmeLib.elements.AcmeConnectorType;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeFamily;
import acmeLib.elements.AcmeGroupType;
import acmeLib.elements.AcmePortType;
import acmeLib.elements.AcmeRoleType;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class NewTypeDialog extends TitleAreaDialog {

    protected List _types;
    protected Text _name;
    
    protected AcmeElementType _newType;
    protected java.util.SortedSet _availableTypes;
    protected Class _category;
	private Scope _scope;

    public NewTypeDialog (Shell shell, java.util.List availableTypes, Class clazz, Scope inScope) {
        super (shell);
        if (availableTypes != null) {
            _availableTypes = new TreeSet (new Utils.AcmeElementNameComparator ());
            _availableTypes.addAll (availableTypes);
        }
        _category = clazz;
        _scope = inScope;
    }

    /**
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite t = (Composite ) super.createDialogArea(parent);
        t.setLayout (new GridLayout (1, false));
        Composite top = new Composite (t, SWT.NONE);
        top.setLayoutData (new GridData (GridData.FILL_HORIZONTAL));
        GridLayout tgl = new GridLayout (2, false);
        top.setLayout (tgl);
        
        Label nL = new Label (top, SWT.NONE);
        nL.setText ("Name: ");
        
        _name = new Text (top, SWT.BORDER);
        _name.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        _name.addModifyListener (new ModifyListener () {
            public void modifyText (ModifyEvent evt) {
                validate();
            }

			
        });
        _name.addVerifyListener (Listeners.VERIFY_ACMEID);
        
        
        Label tL = new Label (top, SWT.NONE);
        tL.setText ("Select supertypes:");
        GridData tlgd = new GridData ();
        tlgd.horizontalSpan = GridData.FILL_HORIZONTAL;
        tL.setLayoutData (tlgd);
        
        _types = new List (top, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
        GridData tgd = new GridData (GridData.FILL_BOTH);
        tgd.horizontalSpan = GridData.FILL;
        tgd.horizontalAlignment = GridData.FILL;
        tgd.verticalAlignment = GridData.FILL;
//        tgd.grabExcessHorizontalSpace = true;
//        tgd.grabExcessVerticalSpace = true;
        _types.setLayoutData (tgd);
        fillTypes ();
        return t;
        
    }

    
    private void validate() {
    	String name = _name.getText ();
    	if (name == null || name.equals ("")) {
    		getButton (IDialogConstants.OK_ID).setEnabled (false);
    		setErrorMessage ("The type name cannot be empty");
    		return;
    	}
    	
    	if (_scope.lookupName(name, false) != null) {
    		setErrorMessage ("There is a duplicate name already defined");
    		getButton (IDialogConstants.OK_ID).setEnabled (false);
    		return;
    	}
    	
    	setErrorMessage (null);
    	getButton (IDialogConstants.OK_ID).setEnabled (_name.getText ().length () > 0);
    }
    
    protected void fillTypes () {
        Iterator i = _availableTypes.iterator ();
        while (i.hasNext ()) {
            AcmeElementType et = (AcmeElementType )i.next ();           
            _types.add (et.getTypeName ());
            _types.setData (et.getTypeName (), et);
        }
    }

    /**
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed() {
       
        
        if (_category == AcmeComponentType.class) {
            _newType = new AcmeComponentType (_name.getText ());        
        }
        else if (_category == AcmeConnectorType.class) {
            _newType = new AcmeConnectorType (_name.getText ());
        }
        else if (_category == AcmePortType.class) {
            _newType = new AcmePortType (_name.getText ());
        }
        else if (_category == AcmeRoleType.class) {
            _newType= new AcmeRoleType (_name.getText ());
        }
        else if (_category == AcmeGroupType.class) {
            _newType = new AcmeGroupType (_name.getText ());
        }
        else if (_category == AcmeFamily.class) {
            _newType = new AcmeFamily (_name.getText ());
        }
        
        String [] selections = _types.getSelection();
        for (int i = 0; i < selections.length; i++) {
            _newType.addSuperType ((AcmeElementType )_types.getData(selections [i]));
        }
        
        super.okPressed();
    }

    public AcmeElementType getNewType () {
        return _newType;
    }
    /**
     * @see org.eclipse.jface.dialogs.Dialog#createButtonBar(Composite)
     */
    protected Control createButtonBar(Composite parent) {
        Control c = super.createButtonBar(parent);
        validate ();
        return c;
    }

}
