package AcmeStudio.ui.views.parts;

import acmeLib.AcmeObject;
import acmeLib.core.filters.AcmeObjectFilter;
import acmeLib.properties.AcmeProperty;

/**
 * @author Bradley Schmerl (schmerl@cs.cmu.edu)
 * Implements a filter that filters an enumeration of properties
 *
 */
public class PropertyFilter extends AcmeObjectFilter {

	/** The prefixes of the names of the properties to filter **/
	private String [] _namePrefixes = null;
	/** Filter out or filter in **/
	private boolean _filterOut = false;
	/** if true, filters by the type name rather than the property name **/
	private boolean _filterByType = false;
	
	/**
	 * Constructor for PropertyByNameFilter.
	 * @param namePrefixes the names to apply the filter to
	 * @param filterOut if true, names are filtered out
	 * @param byType if true, the filter is applied to type names rather than property names
	 */
	public PropertyFilter(String [] namePrefixes, boolean filterOut, boolean byType) {
		super();
		_filterOut = filterOut;
		_namePrefixes = namePrefixes;
		_filterByType = byType;
	}

	/**
	 * Tests the AcmeObject to see if it should be filtered.
	 * This is only applied to properties, and not other AcmeObject
	 * @param obj the object to test to see if it should be filtered
	 * @see acmeLib.core.filters.AcmeObjectFilter#test(AcmeObject)
	 */
	public boolean test(AcmeObject obj) {
		boolean include = false;
		if (obj instanceof AcmeProperty) { // Only apply to properties
			AcmeProperty prop = (AcmeProperty )obj;
			String name = prop.getName ();
			if (_filterByType) { // if filtering by type name instead
                if (prop.getType () != null)
                    name = prop.getType ().getTypeName();
			}
			
			// check the prefixes to see if the name qualifies
			for (int i = 0; i < _namePrefixes.length; i++) {
				if (name.indexOf (_namePrefixes [i]) == 0) {
					include = true;	
				}
			}
			
			// if being filtered out, then don't include
			if (_filterOut) {
				include = !include;
			}
		}
		return include;
		
	}

}
