// $Header$
package AcmeStudio.ui.views.parts;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnLayoutData;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchImages;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.model.ChangePropertyCommand;
import AcmeStudio.model.ModelHelper;
import AcmeStudio.model.PropertyCommand;
import AcmeStudio.ui.views.IElementViewPage;
import AcmeStudio.ui.views.parts.propertyEditors.PropertyValueCellEditor;
import AcmeStudio.util.AcmeErroneousPropertyValue;
import AcmeStudio.util.AcmeModelChangeListener;
import AcmeStudio.util.AcmeStudioChangeManager;
import AcmeStudio.util.AutoApplyEditableComboBoxCellEditor;
import AcmeStudio.util.Constants;
import AcmeStudio.util.Helper;
import AcmeStudio.util.HoverHelp;
import AcmeStudio.util.ITipInformer;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.Utils;
import AcmeStudio.util.AcmeStudioChangeManager.AcmeStudioChangeEvent;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.filters.AcmeObjectFilter;
import acmeLib.designRules.Scope;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.properties.AcmeProperty;
import acmeLib.properties.AcmePropertyBearer;
import acmeLib.properties.AcmePropertyValue;
import acmeLib.typeManagement.AcmeTypeManager;

/**
 * @author Bradley Schmerl (schmerl@cs.cmu.edu) Lists all the properties of a
 *         given element in a table displaying the name, type, and value.
 *         Allows editing, deleting, and adding properties. Has the following
 *         actions: - Deleting a property - Adding a property - Filtering KNOWN
 *         BUGS: - Changes to properties do not cause reevaluation of rules
 */
public class PropertyList extends ElementViewPart implements ISelectionListener, AcmeModelChangeListener {

	public static final int FILTEROUT = 0;
	public static final int NEWPROPERTY = 1;
	public static final int DELPROPERTY = 2;

	/**
	 * Will be an indirection if values fail to parse or names already exist in
	 * the the current scope.
	 */
	private class AcmePropertyHolder implements ITipInformer {
		/** The name of the property if it is in error */
		private String erroneousName = null;
		/** The string value of the property if it doesn't parse or */
		private String erroneousValue = null;
		/**The property being held by the holder */
		private AcmeProperty property = null;

		/** The exception that caused the name to erroneous */
		public Exception nameException = null;
		/** The exception that caused the value to be erroneous */
		public Exception valueException = null;

		/**
		 * Returns the name of the property. If the name is erroneous, return
		 * that, otherwise return the property's name
		 * 
		 * @return the name of the property
		 */
		public String getName() {
			return erroneousName == null ? property.getName() : erroneousName;
		}
		/**
		 * Gets the tip information for the current property holder. The tip
		 * information shows the causes of errors in names, etc
		 * 
		 * @see AcmeStudio.util.ITipInformer#getTipInformation()
		 */
		public String getTipInformation() {
			String tip = "";
			if (ModelHelper.isInheritedProperty(property, null)) {
				tip = "Inherited from " + ((AcmeElement )ModelHelper.getInheritedProperty(property, null).getScope()).getQualifiedName ();
			}
			if (erroneousName != null) {
				tip += nameException.getMessage();
			}
			if (erroneousValue != null) {
				if (!tip.equals("")) {
					tip += "\n---------------------\n";
				}
				tip += valueException.getMessage();
			}
			return tip;
		}

		/**
		 * If the value or name are erroneous, then there is an error, so there
		 * is tip information
		 * 
		 * @return whether this AcmeProperyHolder has an error and therefore a
		 *         tip
		 * @see AcmeStudio.util.ITipInformer#hasTipInformation()
		 */
		public boolean hasTipInformation() {
			return (erroneousValue != null) || (erroneousName != null) || (ModelHelper.isInheritedProperty(property, null));
		}
		/* (non-Javadoc) @see AcmeStudio.util.ITipInformer#getTipImage() */
		public Image getTipImage() {
			return null;
		}

	}

	// Constants for organizing the table
	private static final int INHERITCOL = 0;
	private static final int INHERITCOLWIDTH = 5;
	private static final int NAMEATTCOL = 1;
	private static final int NAMEATTWIDTH = 5;
	private static final int NAMECOL = 2;
	private static final int NAMEWIDTH = 20;
	private static final int TYPECOL = 3;
	private static final int TYPEWIDTH = 20;
	private static final int VALUEATTCOL = 4;
	private static final int VALUEATTWIDTH = 5;
	private static final int VALUECOL = 5;
	private static final int VALUEWIDTH = 45;
	private static final String EMPTYVALUE = "<<No Value>>";
	private static final String[] PROPERTIES = { "inherit", "nameatt", "name", "type", "valueatt", "value" };
	private IContributionItem[] _myContributions;
	private String[] availablePropertyTypes = null;
	/** The delete property action */
	private Action _deletePropertyAction;

	/** The add property action */
	private Action _addPropertyAction;

	/** An action to sort by type (not implemented yet) */
	//private Action _sortByTypeAction;

	/** An action to filter out visual properties */
	private Action _filterOutAcmeStudioPropsAction;

	/** The table viewer used to view the properties */
	private TableViewer _viewer = null;

	/** The currently selected property */
	private AcmePropertyHolder _selectedProperty = null;

	/** The object whose properties are being viewed */
	private AcmeObject _selectedObject = null;

	/** The filters applied to the list of properties */
	private Map _filters = new Hashtable();

	/** The parent view of this list */
	private IElementViewPage _parentView = null;

	/** The action performed on double click */
	private Action _doubleClickAction;

	private static final Object[] emptyElements = new Object[0];
	
	private CellEditor [] _cellEditors;
	 PropertySorter	_propertySorter;
	/*
	 * The content provider class is responsible for providing objects to the
	 * view. It returns all the properties to be displayed in the viewer.
	 */
	private class ViewContentProvider implements IStructuredContentProvider {

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}

		Set gatherAllProperties(AcmeElementType et) {
			Set s = new TreeSet(new Utils.AcmePropertyNameComparator());
			s.addAll(Helper.convertAcmeEnumerationToList(((AcmeElement) et).getProperties()));
			AcmeEnumeration types = et.getSuperTypes().enumerate();
			while (types.moreItems()) {
				AcmeElementType type = (AcmeElementType) types.getNextItem();
				s.addAll(gatherAllProperties(type));
			}
			return s;
		}

		/**
		 * Returns all the properties of the object.
		 * 
		 * @param parent
		 *           The AcmeElement or AcmeProperty to view the properties of
		 *           <br>(If it is an AcmeProperty, the meta properties will
		 *           be returned.)
		 * @return The properties of the object
		 */
		public Object[] getElements(Object parent) {
			AcmeEnumeration props = null;
			Object[] elements = emptyElements;

			// Get the right properties
			if (parent instanceof AcmeElementType) {
				AcmeElementType et = (AcmeElementType) parent;
				//				props = ((AcmeElement )et).getProperties ();
				Set s = gatherAllProperties(et);
				props = Helper.convertCollectionToAcmeEnumeration(s);
			}
			else if (parent instanceof AcmeElement) {
				AcmeElement el = (AcmeElement) parent;
				props = el.getProperties();

			}
			else if (parent instanceof AcmeProperty) {
				// The properties are just the meta properties
				AcmeProperty prop = (AcmeProperty) parent;
				props = prop.getProperties();
			}

			// Apply any filters (if there are indeed properties)
			if (props != null) {
				props = props.castToAcmeSetVector().enumerate();
				Iterator keys = _filters.keySet().iterator();
				while (keys.hasNext()) {
					AcmeObjectFilter filter = (AcmeObjectFilter) _filters.get(keys.next());
					props = props.castToAcmeVector().enumerate(filter);
				}

				// Create the elements array out of the enumeration
				elements = new Object[props.getSize()];
				int index = 0;
				while (props.moreItems()) {
					AcmeProperty prop = (AcmeProperty )props.getNextItem ();
					AcmePropertyHolder holder =  (AcmePropertyHolder )_property2Holder.get (prop); //new AcmePropertyHolder();
					if (holder == null) {
						holder = new AcmePropertyHolder ();
						_property2Holder.put (prop, holder);
					}
					holder.property = prop;
					if (holder.property.getValue () != null && !AcmeTypeManager.typecheckProperty(holder.property.getType(), holder.property.getValue ())) {
						holder.erroneousValue = holder.property.getValue ().toString ();
					}
					elements[index++] = holder;
				}
			}
			return elements;

		}
	}

	/**
	 * Provides routines for getting the text in each of the columns of the
	 * table Column 1 = the name; Column 2 = the type; Column 3 = the value
	 */
	private class ViewLabelProvider extends LabelProvider implements ITableLabelProvider, IColorProvider {
		private Font	_boldFont;
		private Font	_italicFont;
		private Color	_errorColor;

		/**
		 * Returns the text associated with a particular column in the table
		 * 
		 * @param obj
		 *           the property to get the information from
		 * @param index
		 *           the column to return the string for
		 * @return the text to be placed in the column
		 */
		public String getColumnText(Object obj, int index) {
			AcmePropertyHolder acmeProp = (AcmePropertyHolder) obj;
			String retStr = "";
			switch (index) {
				case NAMECOL :
					retStr = acmeProp.erroneousName == null ? acmeProp.property.getName() : acmeProp.erroneousName;
					if (retStr == null) {
						retStr = "<<No Name>>";
					}
					break;
				case TYPECOL :

					retStr = acmeProp.property.getType().getTypeName();
					if (retStr == null) {
						retStr = "<<No Type>>";
					}
					else if ("".equals(retStr)) { // this is anonymous
						retStr = acmeProp.property.getType().toString();
					}
					break;
				case VALUECOL :
					// Only return a value if the property has a value
					if (acmeProp.property.getValue() != null || acmeProp.erroneousValue != null) {
						retStr =
							acmeProp.erroneousValue != null
								? acmeProp.erroneousValue
								: acmeProp.property.getValue().toString();
					}
					else {
						retStr = EMPTYVALUE;
					}
					break;
			}
			return retStr;
		}

		/**
		 * As there are no images associated with the table, just return null
		 * in these routines.
		 */
		public Image getColumnImage(Object obj, int index) {
			AcmePropertyHolder holder = (AcmePropertyHolder) obj;
			Image icon = null;
			switch (index) {
				case NAMEATTCOL :
					if (holder.erroneousName != null) {
						icon = ImageManager.getIcon(ImageManager.ID_ERROR);
					}
					break;
				case VALUEATTCOL :
					if (holder.erroneousValue != null) {
						icon = ImageManager.getIcon(ImageManager.ID_ERROR);
					}
					else if (holder.property != null && ModelHelper.isInheritedValue (holder.property, (AcmePropertyBearer )_selectedObject)) {
						icon = ImageManager.getIcon (ImageManager.ID_LOCKED);
					}
					break;
				case INHERITCOL :
					if (ModelHelper.isInheritedProperty(holder.property, (AcmePropertyBearer )_selectedObject)) {
						icon = ImageManager.getIcon(ImageManager.ID_INHERITS);
					}
			}
			return icon;
		}
		public Image getImage(Object obj) {
			return PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_ELEMENT);
		}

		/**
		 * @param element
		 * @return
		 * @see org.eclipse.jface.viewers.IColorProvider#getForeground(java.lang.Object)
		 **/
		public Color getForeground(Object element) {

			return null;
		}

		/**
		 * @param element
		 * @return
		 * @see org.eclipse.jface.viewers.IColorProvider#getBackground(java.lang.Object)
		 **/
		public Color getBackground(Object element) {
			AcmePropertyHolder holder = (AcmePropertyHolder )element;
			if (holder.erroneousName != null || holder.erroneousValue != null) {
				
				return Constants.DEFAULT_ERROR_BACKGROUND;
			}
			return null;
		}
	}

	private class PropertySorter extends ViewerSorter {
		
		public static final int SORT_NONE = 0;
		public static final int SORT_A_Z = 1;
		public static final int SORT_Z_A = 2;
		
		private int state = SORT_A_Z;
		/**
		 * @see org.eclipse.jface.viewers.ViewerSorter#compare(Viewer, Object,
		 *      Object)
		 */
		public int compare(Viewer viewer, Object e1, Object e2) {
			if (state == SORT_NONE) return 0;
				AcmePropertyHolder h1 = (AcmePropertyHolder) e1;
				AcmePropertyHolder h2 = (AcmePropertyHolder) e2;
				String name1 = "";
				String name2 = "";
			if (state == SORT_A_Z) {
				name1 = (h1.erroneousName == null) ? h1.property.getName() : h1.erroneousName;
				name2 = (h2.erroneousName == null) ? h2.property.getName() : h2.erroneousName;
			}
			else {
				name2 = (h1.erroneousName == null) ? h1.property.getName() : h1.erroneousName;
				name1 = (h2.erroneousName == null) ? h2.property.getName() : h2.erroneousName;
			}
			return name1.compareTo(name2);
		}
		
		public void cycleState () {
			state = (state + 1)  % 3;
		}

	}

	/**
	 * Constructor for PropertyList. Constructs the Property table and adds it
	 * to the appropriate view part
	 * 
	 * @param parent
	 *           the Composite in which to place the table
	 * @param style
	 *           the style of the table
	 * @param parentView
	 *           the ViewPart of the parent (IElementViewPage or
	 *           ModigyPropertyDialog)
	 */
	public PropertyList() {
		super();

	}

	/**
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#dispose() */
	public void dispose() {
		AcmeStudioChangeManager.removeListener(this);
		AcmeStudioPlugin.getActiveWindow().getSelectionService().removeSelectionListener(this);
		super.dispose();
	}

	/**
	 * Returns the TableView that contains the properties
	 * 
	 * @return TableViewer
	 */
	public TableViewer getViewer() {
		return _viewer;
	}

	/** Modifies the selected property on a double click */
	public void handleDoubleClick() {
		ISelection selection = _viewer.getSelection();
		try {
			_selectedProperty = (AcmePropertyHolder) ((IStructuredSelection) selection).getFirstElement();
			modifyProperty();
		}
		catch (ClassCastException e) {
			// Only want to deal with AcmeProperties
			e.printStackTrace();
		}
		catch (NullPointerException ne) {
			// Ignore if nothing is selected
			ne.printStackTrace ();
		}

	}

	/**
	 * Does the modification of an Acme property name in a scope It first
	 * checks to see if the name hasn't changed (in which case the property
	 * isn't changed), and then handles errors in setting the name and adding
	 * the name to the scope. Also reports the name change so that others can
	 * react. If the name cannot be added, the holder is given an
	 * erroneousName.
	 * 
	 * @param inScope
	 *           where the property is to be changed
	 * @param holder
	 *           the details of the modified property
	 * @param newName
	 *           the potential new name of the property
	 */
	private void doModifyPropertyName(Scope inScope, AcmePropertyHolder holder, String newName) {
		if (!newName.equals(holder.getName())) { // The new name has to be
												 // different
			if (holder.property.getName().equals(newName)) { // The new name is
															 // the same as the
															 // old name,
				holder.erroneousName = null; // so no need to test change
			}
			else if (inScope.lookupName(newName, false) != null) {
				// The name is already in scope, so this is an error
				holder.erroneousName = newName;
				holder.nameException = new Exception(newName + " already exists.");
			}
			else {
				AcmeStudioChangeManager globalChangeManager = AcmeStudioChangeManager.globalChangeManager;
				AcmeProperty newProperty = new AcmeProperty(holder.property.getType());
				newProperty.setValue(holder.property.getValue());

				// Try changing the name
				if (inScope instanceof AcmeElement) {
					AcmeElement element = (AcmeElement) inScope;

					try {
						newProperty.setName(newName);
						element.removeProperty(holder.property.getName());
						element.addProperty(newProperty);
						// Now undo so commands can do it
						element.removeProperty(newProperty);

						element.addProperty(holder.property);
						CompoundCommand changeNameCommand = new CompoundCommand("ChangePropName");
						changeNameCommand.setLabel("Change Property");
						changeNameCommand.add(
							new PropertyCommand(PropertyCommand.REMOVE_PROP, element, holder.property, _parentView));
						changeNameCommand.add(
							new PropertyCommand(PropertyCommand.ADD_PROP, element, newProperty, _parentView));
						globalChangeManager.getCommandStack(_acmeDesign).execute(changeNameCommand);
						holder.property = newProperty;
						holder.erroneousName = null;
					}
					catch (AcmeIllegalNameException e) {
						// revert to old name
						holder.erroneousName = newName;
						holder.nameException = e;

						try {
							element.addProperty(holder.property);
						}
						catch (AcmeIllegalNameException e1) {
							// Should never happen, as it's just changing back
						}
					}
				}
				else if (inScope instanceof AcmeProperty) {
					// Do the above, but with property as target
					AcmeProperty target = (AcmeProperty) inScope;
					target.removeProperty(holder.property);

					try {
						newProperty.setName(newName);
						target.addProperty(newProperty);
						target.removeProperty(newProperty);
						target.addProperty(holder.property);
						holder.erroneousName = null;
						CompoundCommand changeNameCommand = new CompoundCommand("ChangePropName");
						changeNameCommand.setLabel("Change Property");
						changeNameCommand.add(
							new PropertyCommand(PropertyCommand.REMOVE_PROP, target, holder.property, _parentView));
						changeNameCommand.add(
							new PropertyCommand(PropertyCommand.ADD_PROP, target, newProperty, _parentView));
						globalChangeManager.getCommandStack(_acmeDesign).execute(changeNameCommand);
					}
					catch (AcmeIllegalNameException e) {
						// revert to old name
						holder.erroneousName = newName;
						holder.nameException = e;
						try {
							target.addProperty(holder.property);
						}
						catch (AcmeIllegalNameException e1) {
							// Should never happen, as it's just changing back
						}
					}
				}
			}

		}
		else {
			// property is the same name, chances are something else changed
			ChangePropertyCommand command = new ChangePropertyCommand();
			command.setNewValue(holder.property.getValue());
			command.setProperty(holder.property);
			command.setTarget((AcmeObject) inScope);
			command.execute(_acmeDesign);
		}
	}

	/**
	 * Modifies the selected property by bring up a dialog, and changing only
	 * if the resultant value is valide
	 */
	private void modifyProperty() {
		// Get the shell of to display the dialog box
		Shell shell = AcmeStudioPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();

		int flag = 0;
		if (!ModelHelper.isInheritedProperty(_selectedProperty.property, (AcmePropertyBearer )_selectedObject)) {
			flag |= ModifyPropertyDialog.MODIFY_DETAILS;
		}
		if (!ModelHelper.isInheritedValue(_selectedProperty.property, (AcmePropertyBearer )_selectedObject)) {
			flag |= ModifyPropertyDialog.MODIFY_VALUE;
		}

		// Creeat the dialog to change the selected property
		ModifyPropertyDialog propDialog = new ModifyPropertyDialog(shell, _selectedProperty.property, flag);

		// Open the dialog
		int retCode = propDialog.open();
		if (retCode == IDialogConstants.OK_ID) { // if the user didn't cancel

			if (ModelHelper.isInheritedProperty(_selectedProperty.property, null)) {

				_selectedProperty.property =
					((AcmePropertyBearer) _selectedObject).getProperty(_selectedProperty.property.getName());

			}
			// assign new value and type
			_selectedProperty.property.setType(propDialog._propertyType);
			_selectedProperty.property.setValue(propDialog._propertyValue);

			// also set the meta properties
			// first remove the existing properties
			AcmeEnumeration existingMPs = _selectedProperty.property.getProperties();
			while (existingMPs.moreItems()) {
				AcmeProperty mp = (AcmeProperty) existingMPs.getNextItem();
				_selectedProperty.property.removeProperty(mp);
			}

			boolean haveError = false;
			// Now add the meta properties
			Iterator mpIt = propDialog.getMetaProperties().iterator();
			while (mpIt.hasNext()) {
				try {
					_selectedProperty.property.addProperty((AcmeProperty) mpIt.next());
				}
				catch (AcmeIllegalNameException e) {
					haveError = true;
				}
			}

			if (haveError) {
				MessageDialog.openError(
					shell,
					"Meta Property Error",
					"Property contains invalid meta properties - they were ignored");
			}

			// Change name last because it causes an announcement
			doModifyPropertyName((Scope) _selectedObject, _selectedProperty, propDialog._propertyName);

			_viewer.setInput(_selectedObject);
		}
	}

	/**
	 * Set the selected object to be the new object passed in
	 * 
	 * @param o
	 *           the object to view
	 */
	public void setInput(AcmeObject o) {
		if (o == null)
			return;
		_property2Holder.clear();
		_selectedObject = o;
		_viewer.setInput(o);

		if (o instanceof AcmeElement) {
			_acmeDesign = ((AcmeElement) o).getParentDesign();
		}
		else if (o instanceof AcmeProperty) {
			_acmeDesign = ((AcmeElement) ((AcmeProperty) o).getScope()).getParentDesign();
		}

		updateTypeCellEditor(_viewer.getTable ());

	}
	/**
	 * @see org.eclipse.swt.widgets.Control#setFocus() */
	public boolean setFocus() {
		return _viewer.getControl().setFocus();
	}

	/**
	 * Adds a filter to the property list to determine which properties don't
	 * get shown.
	 * 
	 * @param key
	 *           the key of the filter
	 * @param filter
	 *           the filter to be applied
	 */
	public void addFilter(String key, PropertyFilter filter) {
		_filters.put(key, filter);
		setInput(_selectedObject);
	}

	/**
	 * Removes the filter on the property list
	 * 
	 * @param key
	 *           the key of the filter to remove
	 */
	public void removeFilter(String key) {
		_filters.remove(key);
	}

	/**
	 * Adds the double click action to the table viewer */
	private void hookDoubleClickAction() {

		_viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				_doubleClickAction.run();
			}
		});
	}

	/**
	 * Makes all the actions for the property list */
	private void makeActions() {

		// The delete action, deletes the currently selected properties
		_deletePropertyAction = new Action() {
			public void run() {
					// Get the selections
	ISelection isel = _viewer.getSelection();
				if (isel instanceof IStructuredSelection) {
					IStructuredSelection iss = (IStructuredSelection) isel;

					Iterator it = iss.iterator();
					// Iterate through the selections
					while (it.hasNext()) {
						AcmePropertyHolder prop = (AcmePropertyHolder) it.next();
						if (!ModelHelper.isInheritedProperty(prop.property, (AcmePropertyBearer )_selectedObject)) {
							PropertyCommand command =
								new PropertyCommand(
									PropertyCommand.REMOVE_PROP,
									_selectedObject,
									prop.property,
									_parentView);
							command.execute(_acmeDesign);
						}
					}

					setInput(_selectedObject);
				}
			}
		};
		_deletePropertyAction.setText("Delete property");
		_deletePropertyAction.setToolTipText("Deletes selected properties");
        // As of Eclipse 3.0, IMG_TOOL_DELETE_HOVER == IMG_TOOL_DELETE
		_deletePropertyAction.setImageDescriptor(
			WorkbenchImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));

		// Adds a new property
		_addPropertyAction = new Action() {
			public void run() {
				AcmePropertyHolder newProperty = new AcmePropertyHolder();
				newProperty.property = new AcmeProperty();
           
				try {
					newProperty.property.setName("aProp");
					if (_selectedObject instanceof AcmeElement) {
						newProperty.property.setScope((AcmeElement) _selectedObject);
					}
					else if (_selectedObject instanceof AcmeProperty) {
						newProperty.property.setScope((AcmeProperty) _selectedObject);
					}
				}
				catch (AcmeIllegalNameException e) {
					// Should never be an illegal name
				}
				int flag = 0;
				if (!ModelHelper.isInheritedProperty(newProperty.property, (AcmePropertyBearer )_selectedObject)) {
					flag |= ModifyPropertyDialog.MODIFY_DETAILS;
				}
				if (!ModelHelper.isInheritedValue(newProperty.property, (AcmePropertyBearer )_selectedObject)) {
					flag |= ModifyPropertyDialog.MODIFY_VALUE;
				}
                
				ModifyPropertyDialog dialog =
					new ModifyPropertyDialog(Display.getDefault().getActiveShell(), newProperty.property, flag);
				if (dialog.open() == IDialogConstants.OK_ID) {
					try {
						newProperty.property.setScope(null);
						newProperty.property.setName(dialog._propertyName);

					}
					catch (AcmeIllegalNameException e) {
					}
					newProperty.property.setValue(dialog._propertyValue);
					newProperty.property.setType(dialog._propertyType);
					if (_selectedObject instanceof AcmeElement) {
						AcmeElement element = (AcmeElement) _selectedObject;
						try {
							PropertyCommand command =
								new PropertyCommand(
									PropertyCommand.ADD_PROP,
									element,
									newProperty.property,
									_parentView);
							command.execute(_acmeDesign);
							command.checkError();
						}
						catch (AcmeIllegalNameException e) {
							MessageDialog.openError(
								Display.getDefault().getActiveShell(),
								"New Property Error",
								e.getMessage());

						}
						catch (Exception e1) {
							// Should never happen
							e1.printStackTrace();
						}
					}
					else if (_selectedObject instanceof AcmeProperty) {
						AcmeProperty property = (AcmeProperty) _selectedObject;
						try {

							PropertyCommand command =
								new PropertyCommand(
									PropertyCommand.ADD_PROP,
									property,
									newProperty.property,
									_parentView);
							command.execute(_acmeDesign);
							command.checkError();
						}
						catch (AcmeIllegalNameException e) {
							MessageDialog.openError(
								Display.getDefault().getActiveShell(),
								"New Property Error",
								e.getMessage());
						}
						catch (Exception e1) {
							// Should never happern
							e1.printStackTrace();
						}
					}
				}
				setInput(_selectedObject);
			}
		};
		_addPropertyAction.setText("New Property...");
		_addPropertyAction.setToolTipText("Creates a new property");
		_addPropertyAction.setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_NEW_PROPERTY));

		// Add a filter to filter out visual properties
		_filterOutAcmeStudioPropsAction = new Action() {
			public void run() {
				AcmeObjectFilter existing = (AcmeObjectFilter) _filters.get("NoVis");
				if (existing != null) {
					_filters.remove("NoVis");
					this.setChecked(true);
				}
				else {
					existing = new PropertyFilter(new String[] { "vis-" }, true, false);
					_filters.put("NoVis", existing);
					this.setChecked(false);
				}
				setInput(_selectedObject);
			}

			public int getStyle() {
				return AS_CHECK_BOX;
			}
		};
		_filterOutAcmeStudioPropsAction.setText("Show AcmeStudio Properties");
		_filterOutAcmeStudioPropsAction.setToolTipText("Toggles whether to show AcmeStudio specific properties");
		_filterOutAcmeStudioPropsAction.setImageDescriptor(
			ImageManager.getImageDescriptor(ImageManager.ID_SHOW_VIS_PROPERTIES));
		if (_filters.get("NoVis") == null) {
			_filterOutAcmeStudioPropsAction.setChecked(false);
		}
		else {
			_filterOutAcmeStudioPropsAction.setChecked(true);
		}

	}

	/**
	 * Creates the context menu for this view, and hooks it into the framework */
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu(manager);
			}
		});

		Menu menu = menuMgr.createContextMenu(_viewer.getControl());
		_viewer.getControl().setMenu(menu);
		if (_parentView != null) {
			_parentView.getSite().registerContextMenu("PropertyList", menuMgr, _viewer);
		}
	}

	/**
	 * Adds the contents of the menu */
	private void fillContextMenu(IMenuManager manager) {
		if (_selectedObject != null) {
			manager.add(_addPropertyAction);
			manager.add(_deletePropertyAction);
			manager.add(new Separator("Filters"));
			manager.add(_filterOutAcmeStudioPropsAction);
		}
		// Other plug-ins can contribute there actions here
		manager.add(new Separator("Additions"));
	}

	/**
	 * Should be fired when this property bar is selected
	 * 
	 * @see org.eclipse.ui.ISelectionListener#selectionChanged(IWorkbenchPart,
	 *      ISelection)
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {

	}

	public IContributionItem[] getContributions() {
		return _myContributions;
	}

	private class PropertyCellModifier implements ICellModifier {

		public boolean canModify(Object element, String property) {
			if (element instanceof AcmePropertyHolder) {
				AcmePropertyHolder holder = (AcmePropertyHolder) element;
				boolean isInherited = ModelHelper.isInheritedProperty(holder.property, (AcmePropertyBearer )_selectedObject);
				if ((property.equals(PROPERTIES[NAMECOL]) || property.equals(PROPERTIES[TYPECOL])) && isInherited) {
					return false;
				}
				else if (property.equals(PROPERTIES[VALUECOL]) && ModelHelper.isInheritedValue(holder.property, (AcmePropertyBearer )_selectedObject)) {
					return false;
				}
			}
			return true;
		}

		public Object getValue(Object element, String property) {
			Object value = null;
			if ((element instanceof AcmePropertyHolder) && canModify(element, property)) {
				AcmePropertyHolder holder = (AcmePropertyHolder) element;
				if (property.equals(PROPERTIES[NAMECOL])) {
					if (holder.erroneousName != null) {
						value = holder.erroneousName;
					}
					else {
						value = holder.property.getName();
					}
				}
				else if (property.equals(PROPERTIES[TYPECOL])) {
					// get the index corresponding to this type name
					boolean foundType = false;
					int index = 0;
					while (index < availablePropertyTypes.length && !foundType) {
						if (availablePropertyTypes[index].equals(holder.property.getType().getTypeName())) {
							foundType = true;
						}
						else {
							index++;
						}
					}

					value = new Integer(index);
				}
				else if (property.equals(PROPERTIES[VALUECOL])) {
					if (holder.erroneousValue != null) {
						value = new AcmeErroneousPropertyValue (holder.erroneousValue, holder.property.getType ());
					}
					else if (holder.property.getValue() == null) {
						value = new AcmeErroneousPropertyValue ("", holder.property.getType ());
					}
					else {
						value = holder.property.getValue();
					}
				}
			}
			return value;
		}

		public void modify(Object element, String property, Object value) {
			if (element instanceof TableItem) {
				TableItem item = (TableItem) element;
				AcmePropertyHolder holder = (AcmePropertyHolder) item.getData();
				if (property.equals(PROPERTIES[NAMECOL])) {
					doModifyPropertyName((Scope) _selectedObject, holder, (String) value);
				}
				else if (property.equals(PROPERTIES[TYPECOL])) {
					if (((Integer) value).intValue() != -1) {
						String val = availablePropertyTypes[((Integer) value).intValue()];
						holder.property.setType(ModifyPropertyDialog.getPropertyType(val, (Scope) _selectedObject));
					}
					else {
						// Something was typed in, so we'd better try parsing
						// it
						// it might have been an anonymous type
						//                        String typeToParse = item.getText();
						//                        AcmePropertyType type =
						// ModifyPropertyDialog.parsePropertyType
						// (typeToParse);
						//                           
					}

				}
				else if (property.equals(PROPERTIES[VALUECOL])) {
					if (value instanceof AcmePropertyValue) {
						AcmePropertyValue newValue = (AcmePropertyValue )value;
						
					
						
						if (AcmeTypeManager.typecheckProperty(holder.property.getType(), newValue)) {
							if (ModelHelper.isInheritedProperty(holder.property, (AcmeElement )_selectedObject)) {
								if (_selectedObject instanceof AcmeElementType) {
									holder.property = holder.property.copy();
									try {
										((AcmeElement) _selectedObject).addProperty(holder.property);
									}
									catch (AcmeIllegalNameException e1) {
										e1.printStackTrace();
									}
								}
								else {
									holder.property =
										((AcmePropertyBearer) _selectedObject).getProperty(holder.property.getName());
									ChangePropertyCommand command = new ChangePropertyCommand ();
									command.setProperty (holder.property);
									command.setTarget (_selectedObject);
									command.setNewValue (newValue);
									command.execute (_acmeDesign);
									holder.erroneousValue = null;
								}
							}
							else {

								ChangePropertyCommand command = new ChangePropertyCommand();
								command.setProperty(holder.property);
								command.setTarget(_selectedObject);
								command.setNewValue(newValue);
								command.execute(_acmeDesign);
								holder.erroneousValue = null;
						   }
						}
						else {
							holder.erroneousValue = value.toString ();
							holder.valueException =
								new Exception(
									value + " does not conform to type " + holder.property.getType().getTypeName());
						}
					}
					
				}
				_viewer.update(holder, new String[] { property });
				
			}

		}
	}

	protected CellEditor[] getCellEditors(Table table) {
		_cellEditors = new CellEditor[VALUECOL + 1];
		// Changes to this text editor are automatically triggered
		class AutoApplyTextCellEditor extends TextCellEditor {
			public AutoApplyTextCellEditor(Composite parent) {
				super(parent);
			}
			public void fireApplyEditorValue() {
				super.fireApplyEditorValue();
			}
		}

		

		_cellEditors[INHERITCOL] = null;
		_cellEditors[NAMEATTCOL] = null;
		_cellEditors [VALUEATTCOL] = null;
		_cellEditors[NAMECOL] = new AutoApplyTextCellEditor(table);
		_cellEditors[NAMECOL].getControl().addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				((AutoApplyTextCellEditor) _cellEditors[NAMECOL]).fireApplyEditorValue();
			}
		});

		_cellEditors[VALUECOL] = new PropertyValueCellEditor (table);
//		editors[VALUECOL].getControl().addFocusListener(new FocusAdapter() {
//			public void focusLost(FocusEvent e) {
//				((PropertyValueCellEditor) editors[VALUECOL]).fireApplyEditorValue();
//			}
//		});
		updateTypeCellEditor(table);
		return _cellEditors;
	}
	
	private void updateTypeCellEditor (Table table) {
		if (_cellEditors [TYPECOL] != null) _cellEditors [TYPECOL].dispose();
		if (_selectedObject != null) {
			availablePropertyTypes = ModifyPropertyDialog.getAvailablePropertyTypeNames((Scope) _selectedObject);
			_cellEditors[TYPECOL] = new AutoApplyEditableComboBoxCellEditor(table, availablePropertyTypes);
			_cellEditors[TYPECOL].getControl().addFocusListener(new FocusAdapter() {
				public void focusLost(FocusEvent e) {
					((AutoApplyEditableComboBoxCellEditor) _cellEditors[TYPECOL]).fireApplyEditorValue();
				}
			});
		}
	}

	/**
	 * Responds to changes in properties by other components of the system
	 * 
	 * @see
	 * AcmeStudio.util.AcmeModelChangeListener#changeModel(AcmeStudioChangeEvent)
	 */
	public void changeModel(AcmeStudioChangeManager.AcmeStudioChangeEvent evt) {
		if (evt.getAnnouncer() != _viewer && evt.getTarget() == _selectedObject) {
			switch (evt.getEventType()) {
				case AcmeStudioChangeManager.ADD_PROPERTY :
				case AcmeStudioChangeManager.REMOVE_PROPERTY :
				case AcmeStudioChangeManager.SET_PROPERTY:
					setInput(_selectedObject);
					break;
			}
		}
	}

	public boolean interestedInEvent (int et) {
		return et == AcmeStudioChangeManager.ADD_PROPERTY ||
			et == AcmeStudioChangeManager.REMOVE_PROPERTY ||
			et == AcmeStudioChangeManager.SET_PROPERTY;
	}
	
	/**
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#isValidFor(AcmeObject) */
	public boolean isValidFor(AcmeObject o) {
		return true;
	}


	/*
	 * (non-Javadoc) @see
	 * AcmeStudio.ui.views.parts.ElementViewPart#createControl(org.eclipse.swt.widgets.Composite,
	 * int, AcmeStudio.ui.views.IElementViewPage)
	 */
	public void createControl(Composite parent, int style, IElementViewPage vp) {
		super.createControl(parent, style, vp);

		FillLayout tableLayout = new FillLayout();
		this.getComposite().setLayout(tableLayout);

		// Create the table and set it's contents to the active design by
		// default
		_viewer = new TableViewer(this.getComposite(), SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
		_viewer.setContentProvider(new ViewContentProvider());
		_viewer.setLabelProvider(new ViewLabelProvider());
		_propertySorter = new PropertySorter ();
		_viewer.setSorter(_propertySorter);

		// Set the column header and widthe information
		Table table = _viewer.getTable();
		table.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent evt) {
				if (evt.character == SWT.DEL) {
					_deletePropertyAction.run();
				}
			}

			public void keyReleased(KeyEvent evt) {
			}
		});

		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		TableColumn inheritCol = new TableColumn(table, SWT.CENTER, INHERITCOL);
		TableColumn nameAttCol = new TableColumn (table, SWT.CENTER, NAMEATTCOL);
		TableColumn nameCol = new TableColumn(table, SWT.LEFT, NAMECOL);
		TableColumn typeCol = new TableColumn(table, SWT.LEFT, TYPECOL);
		TableColumn valueAttCol = new TableColumn (table, SWT.CENTER, VALUEATTCOL);
		TableColumn valueCol = new TableColumn(table, SWT.LEFT, VALUECOL);

		nameCol.setText("Name");
		typeCol.setText("Type");
		valueCol.setText("Value");

		nameCol.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected (SelectionEvent evt) {
				_propertySorter.cycleState();
				_viewer.refresh();
			}
		});
		
		ColumnLayoutData inhLayout = new ColumnWeightData(INHERITCOLWIDTH, true);
		ColumnLayoutData nameLayout = new ColumnWeightData(NAMEWIDTH, true);
		ColumnLayoutData typeLayout = new ColumnWeightData(TYPEWIDTH, true);
		ColumnLayoutData valueLayout = new ColumnWeightData(VALUEWIDTH, true);
		ColumnLayoutData nameAttLayout = new ColumnWeightData (NAMEATTWIDTH, true);
		ColumnLayoutData valueAttLayout = new ColumnWeightData (VALUEATTWIDTH, true);
		// set columns in Table layout
		TableLayout layout = new TableLayout();
		layout.addColumnData(inhLayout);
		layout.addColumnData (nameAttLayout);
		layout.addColumnData(nameLayout);
		layout.addColumnData(typeLayout);
		layout.addColumnData (valueAttLayout);
		layout.addColumnData(valueLayout);
		table.setLayout(layout);

		// Add actions
		_doubleClickAction = new Action() {
			public void run() {
				handleDoubleClick();
			}
		};

		hookDoubleClickAction();
		makeActions();
		hookContextMenu();
		_myContributions =
			new ActionContributionItem[] {
				new ActionContributionItem(_filterOutAcmeStudioPropsAction),
				new ActionContributionItem(_addPropertyAction),
				new ActionContributionItem(_deletePropertyAction)};

		// Register with listeners
		AcmeStudioChangeManager.registerListener(this);

		// Start a helper to report errors in the properties
		HoverHelp help = new HoverHelp();
		help.activateHoverHelp(_viewer.getTable());
		_viewer.setCellEditors(getCellEditors(_viewer.getTable()));
		_viewer.setColumnProperties(PROPERTIES);
		_viewer.setCellModifier(new PropertyCellModifier());
	}

	/* (non-Javadoc) @see AcmeStudio.ui.views.parts.ElementViewPart#getTitle() */
	public String getTitle() {
		return "Properties";
	}

	private Map	_property2Holder = new HashMap ();

}