/*
 * Created on Jul 31, 2003
 *  
 */
package AcmeStudio.ui.views.parts;

import java.io.ByteArrayInputStream;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.PageBook;

import AcmeStudio.ui.views.parts.propertyEditors.RecordValueEditor;
import AcmeStudio.ui.views.parts.propertyEditors.SequenceValueEditor;
import AcmeStudio.ui.views.parts.propertyEditors.SetValueEditor;
import AcmeStudio.util.Listeners;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.properties.AcmeEnum;
import acmeLib.properties.AcmeProperty;
import acmeLib.properties.AcmePropertyType;
import acmeLib.properties.AcmePropertyValue;
import acmeLib.properties.AcmeRecord;
import acmeLib.properties.AcmeRecordField;
import acmeLib.properties.AcmeSequence;
import acmeLib.properties.AcmeSet;
import acmeLib.properties.BooleanType;
import acmeLib.properties.EnumType;
import acmeLib.properties.EnumValue;
import acmeLib.properties.FloatType;
import acmeLib.properties.IntType;
import acmeLib.properties.RecordType;
import acmeLib.properties.SequenceType;
import acmeLib.properties.SetType;
import acmeLib.properties.StringType;
/**
 * @author Bradley Schmerl Created Jul 31, 2003
 */
public class PropertyValueInspector extends Composite {

	protected static final int	INT	= 0, FLOAT = 1, STRING = 2, BOOL = 3, ENUM = 4, RECORD = 5, SEQ = 6, SET = 7, OTHER = 8;
	private TabFolder			_folder;
	//	private Text _valueText;
	private PropertyList		_metaPropertyList;
	private AcmeProperty		_property;
	private AcmePropertyType	_propertyType;
	private PageBook			_pbValues;
	private Control				_pages[];
	private int					_activePage;
	// This is no longer activated
//	private SourceViewer		_sourceViewer;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Control#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled) {
		super.setEnabled (enabled);
		//		_folder.setEnabled(enabled);
		_metaPropertyList.getComposite ().setEnabled (enabled);
		if (enabled) setType (_propertyType);
		_pages [_activePage].setEnabled (enabled);
		switch (_activePage) {
			case INT :
			case FLOAT :
			case STRING :
				((Text )_pages [_activePage]).setEditable (enabled);
				break;
			case BOOL :
				((Combo )_pages [_activePage]).setEnabled (enabled);
				break;
			case ENUM :
				((Combo )_pages [_activePage]).setEnabled (enabled);
				break;
			case RECORD :
				((RecordValueEditor )_pages [_activePage]).setEnabled(enabled);
				
				break;
			case SET :
				((SetValueEditor )_pages [_activePage]).setEnabled (enabled);
				break;
			case SEQ :
				((SequenceValueEditor )_pages [_activePage]).setEnabled (enabled);
				break;
			default :
				((Text )_pages [OTHER]).setEditable (enabled);
		}
		// BugFix#238: make sure the book is enabled, so that
		// editing can take place.
		_pbValues.setEnabled (enabled);
		
	}

	/**
	 * @param parent
	 * @param style
	 */
	public PropertyValueInspector(Composite parent, int style, AcmeProperty propertyToModify) {
		super (parent, style);
		_property = propertyToModify;
		_propertyType = _property.getType ();
		// TabControl
		_folder = new TabFolder (parent, SWT.NONE);
		int i = 0;
		GridData gdFolder = new GridData (GridData.FILL_HORIZONTAL);
		gdFolder.horizontalSpan = 2;
		gdFolder.heightHint = 200;
		gdFolder.widthHint = 300;
		_folder.setLayoutData (gdFolder);
		_folder.setLayout (new FillLayout (SWT.VERTICAL));
		TabItem propertyTab = new TabItem (_folder, SWT.NONE, i++);
		propertyTab.setText ("Value");
		_pbValues = new PageBook (_folder, SWT.NONE);
		_pages = new Control [OTHER + 1];
		// Initialize the value text
		Text valueText = new Text (_pbValues, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		_pages [INT] = createIntPage (_pbValues);
		_pages [FLOAT] = createFloatPage (_pbValues);
		_pages [STRING] = createStringPage (_pbValues);
		_pages [BOOL] = createBooleanPage (_pbValues);
		_pages [ENUM] = createEnumPage (_pbValues);
		_pages [RECORD] = createRecordPage (_pbValues);
		_pages [SET] = createSetPage (_pbValues);
		_pages [SEQ] = createSeqPage (_pbValues);
		_pages [OTHER] = valueText;
		inspectValue (propertyToModify);
		propertyTab.setControl (_pbValues);
		// Decide whether the check box is selected and the value text is
		// enabled
		_pbValues.setEnabled (propertyToModify.getValue () != null);
		// Create the meta properties
		TabItem metaPropertyTab = new TabItem (_folder, SWT.NONE, i++);
		metaPropertyTab.setText ("Meta Properties");
		_metaPropertyList = new PropertyList ();
		_metaPropertyList.createControl (_folder, SWT.NONE, null);
		_metaPropertyList.setInput (propertyToModify);
		metaPropertyTab.setControl (_metaPropertyList.getComposite ());
//		TabItem textTab = new TabItem (_folder, SWT.NONE, i++);
//		textTab.setText ("Source");
//		_sourceViewer = new SourceViewer (_folder, null, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
//		_sourceViewer.configure (new AcmeSourceViewerConfiguration ());
//		_sourceViewer.getTextWidget ().setLayoutData (
//				new GridData (GridData.FILL_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_VERTICAL));
//		if (propertyToModify.getValue () != null) {
//			_sourceViewer.setDocument (new Document (propertyToModify.getValue ().toString ()));
//		} else {
//			_sourceViewer.setDocument (new Document (""));
//		}
//		textTab.setControl (_sourceViewer.getTextWidget ());
	}

	/**
	 * @param values
	 * @return
	 */
	private Control createBooleanPage(PageBook values) {
		Combo booleanCombo = new Combo (values, SWT.DROP_DOWN);
		booleanCombo.add ("true");
		booleanCombo.add ("false");
		booleanCombo.setText ("-- Set the value --");
		return booleanCombo;
	}

	/**
	 * @param values
	 * @return
	 */
	private Control createStringPage(PageBook values) {
		Text stringText = new Text (values, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.WRAP);
		Listeners.StringValidator v = new Listeners.StringValidator ();
		v.setTarget (stringText);
		stringText.addVerifyListener (v.getValidator ());
		return stringText;
	}

	/**
	 * @param values
	 * @return
	 */
	private Control createFloatPage(PageBook values) {
		Text floatText = new Text (values, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.WRAP);
		Listeners.DoubleValidator v = new Listeners.DoubleValidator ();
		v.setTarget (floatText);
		floatText.addVerifyListener (v.getValidator ());
		return floatText;
	}

	/**
	 * @param values
	 * @return
	 */
	private Control createIntPage(PageBook values) {
		Text intText = new Text (values, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.WRAP);
		Listeners.IntValidator v = new Listeners.IntValidator ();
		v.setTarget (intText);
		intText.addVerifyListener (v.getValidator ());
		return intText;
	}

	/**
	 * @param values
	 * @return
	 */
	private Control createSeqPage(PageBook values) {
		return new SequenceValueEditor (values, SWT.NONE);
	}

	/**
	 * @param values
	 * @return
	 */
	private Control createSetPage(PageBook values) {
		return new SetValueEditor (values, SWT.NONE);
	}

	/**
	 * @param values
	 * @return
	 */
	private Control createRecordPage(PageBook values) {
		RecordValueEditor editor = new RecordValueEditor (values, SWT.NONE, _property);
		return editor;
	}

	/**
	 * @param values
	 * @return
	 */
	private Control createEnumPage(PageBook values) {
		Combo enumCombo = new Combo (values, SWT.DROP_DOWN);
		enumCombo.setText ("-- Set the value --");
		return enumCombo;
	}

	void inspectValue(AcmeProperty propertyToModify) {
		if (propertyToModify.getValue () != null) {
			setType (_propertyType);
			String value = propertyToModify.getValue ().toString ();
			switch (_activePage) {
				case INT :
				case FLOAT :
					((Text )_pages [_activePage]).setText (value);
					break;
				case STRING : {
					String val = value;
					int end = val.length () - 2;
					if (end < 0) {
						val = "";
					}
					else {
						val = val.substring (1, end);
					}
					((Text )_pages [STRING]).setText (val);
					break;
				}
				case BOOL :
					((Combo )_pages [BOOL]).setText (value);
					break;
				case ENUM :
					{
						Combo enumCombo = (Combo )_pages [ENUM];
						enumCombo.select (enumCombo.indexOf (value));
						//					(enumCombo).setText (value);
						break;
					}
				case RECORD :
					{
						RecordValueEditor editor = (RecordValueEditor )_pages [RECORD];
						editor.setInput ((AcmeRecord )propertyToModify.getValue (), propertyToModify);
						break;
					}
				case SEQ :
				{
					SequenceValueEditor editor = (SequenceValueEditor )_pages [SEQ];
					editor.setInput ((AcmeSequence )propertyToModify.getValue (), _propertyType);
					break;
				}	
				case SET :
				{
					SetValueEditor editor = (SetValueEditor )_pages [SET];
					editor.setInput ((AcmeSet )propertyToModify.getValue (), _propertyType);
					break;
				}
				default :
					((Text )_pages [OTHER]).setText (value);
			}

		}
	}

	public static AcmePropertyValue getPropertyValue(String valueToParse) throws ParseException {
		AcmePropertyValue propertyValue = null;
		ByteArrayInputStream bis = new ByteArrayInputStream (valueToParse.getBytes ());
		if (!ArmaniParser.parserConstructed ()) {
			new ArmaniParser (bis);
		}
		// Parse the property value
		ArmaniParser.ReInit (bis);
		propertyValue = ArmaniParser.PropertyValueDeclaration ();
		return propertyValue;
	}

	/**
	 * @return
	 */
	public AcmePropertyValue getPropertyValue() throws ParseException {
		if (_folder.getSelectionIndex () == 2) {
//			String valueToParse = _sourceViewer.getTextWidget().getText();
//			return getPropertyValue (valueToParse);
			return null;
		} else {
			String valueToParse = "";
			switch (_activePage) {
				case INT :
				case FLOAT :
					valueToParse = ((Text )_pages [_activePage]).getText ();
					break;
				case STRING :
					valueToParse = "\"" + ((Text )_pages [STRING]).getText () + "\"";
					break;
				case BOOL :
					{
						int selection = ((Combo )_pages [BOOL]).getSelectionIndex ();
						if (selection == -1) {
							selection = getSelectionFromComboText ((Combo )_pages [BOOL]);
						}
						if (selection == 0)
							valueToParse = "true";
						else if (selection == 1)
							valueToParse = "false";
						else
							return null;
						break;
					}
				case ENUM :
					{
						Combo combo = (Combo )_pages [ENUM];
						int selection = combo.getSelectionIndex ();
						if (selection == -1) {
							selection = getSelectionFromComboText (combo);
						}
						if (selection == -1)
							return null;
						EnumValue val = (EnumValue )combo.getData (combo.getItem (selection));
						return new AcmeEnum ((EnumType )_propertyType, val.toString ());
					}
				case RECORD :
					{
						RecordValueEditor e = (RecordValueEditor )_pages [RECORD];
						return e.getRecord ();
					}
				case SEQ :
				{
					SequenceValueEditor e = (SequenceValueEditor )_pages [SEQ];
					return e.getSeq();
				}
				case SET :
				{
					SetValueEditor e = (SetValueEditor )_pages [SET];
					return e.getSet();
				}
				default :
					valueToParse = ((Text )_pages [OTHER]).getText ();
			}
			return getPropertyValue (valueToParse);
		}
	}

	/**
	 * @param combo
	 * @return
	 */
	private int getSelectionFromComboText(Combo combo) {
		String [] items = combo.getItems ();
		String text = combo.getText ().toLowerCase ();
		int found = -1;
		for (int i = 0; i < items.length && found == -1; i++) {
			if (items [i].toLowerCase ().equals (text)) {
				found = i;
			}
		}
		return found;
	}

	/**
	 * @return
	 */
	public Vector getMetaProperties() {
		int index = 0;
		Vector _metaProperties = new Vector ();
		while (_metaPropertyList.getViewer ().getElementAt (index) != null) {
			_metaProperties.add (_metaPropertyList.getViewer ().getElementAt (index++));
		}
		return _metaProperties;
	}

	public void setType(AcmePropertyType type) {
		_propertyType = type;
		if (type instanceof IntType) {
			_pbValues.showPage (_pages [INT]);
			_activePage = INT;
		} else if (type instanceof FloatType) {
			_pbValues.showPage (_pages [FLOAT]);
			_activePage = FLOAT;
		} else if (type instanceof StringType) {
			_pbValues.showPage (_pages [STRING]);
			_activePage = STRING;
		} else if (type instanceof BooleanType) {
			_pbValues.showPage (_pages [BOOL]);
			_activePage = BOOL;
		} else if (type instanceof EnumType) {
			EnumType enumType = (EnumType )type;
			_pbValues.showPage (_pages [ENUM]);
			_activePage = ENUM;
			Combo enums = (Combo )_pages [ENUM];
			enums.removeAll ();
			AcmeEnumeration values = enumType.getValues ();
			while (values.moreItems ()) {
				EnumValue enum = (EnumValue )values.getNextItem ();
				enums.add (enum.toString ());
				enums.setData (enum.toString (), enum);
			}
		} else if (type instanceof RecordType) {
			_pbValues.showPage (_pages [RECORD]);
			_activePage = RECORD;
			RecordValueEditor recs = (RecordValueEditor )_pages [RECORD];
			RecordType recType = (RecordType )type;
			AcmeRecord record = new AcmeRecord (recType);
			AcmeEnumeration fields = recType.getFields ();
			while (fields.moreItems ()) {
				AcmeRecordField field = (AcmeRecordField )fields.getNextItem ();
				record.addField (field);
			}
			recs.setInput (record, _property);
		}
		else if (type instanceof SetType) {
			_pbValues.showPage (_pages [SET]);
			_activePage = SET;
			SetValueEditor ed = (SetValueEditor )_pages [SET];
			SetType setType = (SetType )type;
			AcmeSet set = new AcmeSet (setType);
			ed.setInput(set, setType);
		}
		else if (type instanceof SequenceType) {
			_pbValues.showPage (_pages [SEQ]);
			_activePage = SEQ;
			SequenceValueEditor ed = (SequenceValueEditor )_pages [SEQ];
			SequenceType seqType = (SequenceType )type;
			AcmeSequence seq = new AcmeSequence (seqType);
			ed.setInput (seq, seqType);
		}
		else {
			_pbValues.showPage (_pages [OTHER]);
			_activePage = OTHER;
		}
	}
}
