/*
 * Created on Dec 29, 2003
 *
 */
package AcmeStudio.ui.views.parts;

import java.util.Iterator;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LightweightSystem;
import org.eclipse.draw2d.parts.Thumbnail;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.internal.WorkbenchImages;
import AcmeStudio.core.ASContext;
import AcmeStudio.model.ModelCompoundCommand;
import AcmeStudio.model.RepresentationCommand;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.ui.views.IElementViewPage;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeSystem;
import acmeLib.representations.AcmeRepresentation;



/**
 * @author Bradley Schmerl
 * Created Dec 29, 2003
 */
public class RepresentationList extends ElementViewPart {

	/** A pointer to the IElementViewPage containing this view **/
	private IElementViewPage _parentView = null;

	/**
	 * @author Bradley Schmerl
	 * Created Dec 29, 2003
	 */
	public class RepresentationLabelProvider extends LabelProvider {

		/**
		 * 
		 */
		public RepresentationLabelProvider() {
			super ();
		}
		
		
		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
		 */
		public String getText(Object element) {
			String label = "";
			if (element instanceof AcmeRepresentation) {
				label = ((AcmeRepresentation )element).getName ();
			}
			return label;
		}
	}
	private static final Object[] emptySystems = new Object[0];
	

	/**
	 * @author Bradley Schmerl
	 * Created Dec 29, 2003
	 */
	public class RepresentationContentProvider implements IStructuredContentProvider {

		/**
		 * 
		 */
		public RepresentationContentProvider() {
			super ();
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			Object [] representations = RepresentationList.emptySystems;
			
			if (inputElement instanceof AcmeElement) {
				AcmeElement element = (AcmeElement )inputElement;
				AcmeEnumeration reps = element.getRepresentations ();
				representations = Helper.convertAcmeEnumerationToList(reps).toArray();
			}
			return representations;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
	private IContributionItem[] _myContributions = new IContributionItem [0];
	private ListViewer _viewer;
	private FigureCanvas _overview;
	private LightweightSystem _lws;
	private Action _doubleClickAction;
	private Label _defaultFigure;
	
	private ViewSelectionAction _removeRepresentationAction;
	private Action _addRepresentationAction;
	private AcmeObject	_selectedObject;

	/**
	 * 
	 */
	public RepresentationList() {
		super ();
		_defaultFigure = new Label ();
		_defaultFigure.setText("No representation selected");
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#getContributions()
	 */
	public IContributionItem[] getContributions() {
		return _myContributions ;
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#isValidFor(acmeLib.AcmeObject)
	 */
	public boolean isValidFor(AcmeObject o) {
		return o instanceof AcmeComponent ||
                o instanceof AcmeConnector ||
                o instanceof AcmeSystem;
	}

	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#getTitle()
	 */
	public String getTitle() {
		return "Representations";
	}
	
	public void handleDoubleClick () {
		ISelection selection = _viewer.getSelection ();
		try {
			AcmeSystem selectedSystem = ((AcmeRepresentation )((IStructuredSelection )selection).getFirstElement ()).getSystem ();
			MultiPageAcmeEditor mp = ASContext.current ().getActiveEditor ();
			if (mp != null) {
				mp.swapDiagramEditor (selectedSystem, (AcmeSystem )selectedSystem.getParentRepresentation ().getParent ().getParent ());
			}
		}
		catch (ClassCastException e) {
			// only want to deal with Design Rules
		}
		catch (NullPointerException ne) {
			// Ignore if nothing is selected
		}
	}
	
	public void setInput (AcmeObject o) {
		if (o != null) {
			_viewer.setInput (o);
			_selectedObject = o;
			if (_lws != null && _lws.getRootFigure().getChildren ().size () == 1) {
				IFigure fig = (IFigure) _lws.getRootFigure ().getChildren ().get (0);
				if (fig instanceof Thumbnail) {
					Thumbnail t = (Thumbnail) fig;
					t.deactivate();					
				}
			}
			if (_viewer.getSelection() != null &&
				((IStructuredSelection )_viewer.getSelection()).size () > 0) {
				updateOverview ((AcmeRepresentation )((IStructuredSelection )_viewer.getSelection ()).getFirstElement ());
			}
			else if (o instanceof AcmeElement && ((AcmeElement )o).getRepresentations().getSize () > 0){
				AcmeElement elem = (AcmeElement )o;
				AcmeRepresentation rep = (AcmeRepresentation )elem.getRepresentations().getNextItem ();
				_viewer.setSelection(new StructuredSelection (rep));
			}
			else {
				_lws.setContents (_defaultFigure);
			}
		}
	}
	/* (non-Javadoc)
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#createControl(org.eclipse.swt.widgets.Composite, int, AcmeStudio.ui.views.IElementViewPage)
	 */
	public void createControl(Composite parent, int style, IElementViewPage vp) {
		super.createControl (parent, style, vp);
		_parentView = vp;
		FillLayout layout = new FillLayout (SWT.HORIZONTAL);
		this.getComposite ().setLayout (layout);
		
		_viewer = new ListViewer (this.getComposite (), SWT.H_SCROLL | SWT.V_SCROLL);
		_viewer.setContentProvider (new RepresentationContentProvider ());
		_viewer.setLabelProvider (new RepresentationLabelProvider ());
		
		_viewer.setInput (ASContext.current ().getActiveDesign ().getDesign ());
		_viewer.addSelectionChangedListener(new ISelectionChangedListener () {

			public void selectionChanged(SelectionChangedEvent event) {
				AcmeRepresentation rep = (AcmeRepresentation )((IStructuredSelection )event.getSelection()).getFirstElement();
				updateOverview (rep);
				_addRepresentationAction.setEnabled (_addRepresentationAction.isEnabled ());
			}
		
		});
		
		Group overviewGroup = new Group (this.getComposite (), SWT.NONE);
		overviewGroup.setText("Overview:");
		overviewGroup.setLayout (new FillLayout ());
		_overview = new FigureCanvas (overviewGroup);
		_lws = new LightweightSystem (_overview);
		
		_doubleClickAction = new Action () {
			public void run () {
				handleDoubleClick ();
			}
		};
		
		hookDoubleClikcAction ();
		makeActions ();
		hookContextMenu ();
		
	}

	/**
	 * 
	 */
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager ("#PopupMenu");
		menuMgr.setRemoveAllWhenShown (true);
		menuMgr.addMenuListener(new IMenuListener () {

			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu (manager);
			}
		
		});
		
		Menu menu = menuMgr.createContextMenu (_viewer.getControl ());
		_viewer.getControl ().setMenu (menu);
		if (_parentView != null) {
			_parentView.getSite ().registerContextMenu ("RepresentationList", menuMgr, _viewer);
		}
	}

	/**
	 * @param manager
	 */
	protected void fillContextMenu(IMenuManager manager) {
		manager.add (_addRepresentationAction);
		manager.add (_removeRepresentationAction);
		manager.add (new Separator ("Additions"));
	}

	/**
	 * 
	 */
	private void makeActions() {
		_addRepresentationAction = new Action () {
			public void run () {
				if (_selectedObject instanceof AcmeComponent || _selectedObject instanceof AcmeConnector || _selectedObject instanceof AcmeSystem) {
					AcmeElement element = (AcmeElement )_selectedObject;
					AddRepresentationDialog dialog = new AddRepresentationDialog (null, element);
					if (dialog.open () == IDialogConstants.OK_ID) {
						AcmeRepresentation newRep = dialog.getRepresentation();
						RepresentationCommand command = new RepresentationCommand (RepresentationCommand.ADD_REP, newRep, element, null);
						command.execute(element.getParentDesign ());
					}
				}
			}
		};
		_addRepresentationAction.setText("Add Representation...");
		_addRepresentationAction.setDescription("Creates a new representation for the selected element");
		_addRepresentationAction.setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_NEW_REP));
		
		_removeRepresentationAction = new ViewSelectionAction () {
			public void run () {
				IStructuredSelection selection = (IStructuredSelection )_viewer.getSelection ();
				Iterator iter = selection.iterator ();
				ModelCompoundCommand command = new ModelCompoundCommand ("Remove representations");
				while (iter.hasNext ()) {
					AcmeRepresentation rep = (AcmeRepresentation )iter.next ();
					RepresentationCommand repComm = new RepresentationCommand (RepresentationCommand.REMOVE_REP, rep, (AcmeElement )_selectedObject,null);
					command.add(repComm);
				}
				command.execute(((AcmeElement )_selectedObject).getParentDesign());
			}
			
			public boolean isEnabled () {
				ISelection selection = _viewer.getSelection();
				return (selection != null && !selection.isEmpty ()); 
			}
		};
		_removeRepresentationAction.setText("Remove Representation");
		_removeRepresentationAction.setDescription("Removes selected representations");
        // As of Eclipse 3.0, IMG_TOOL_DELETE_HOVER == IMG_TOOL_DELETE
		_removeRepresentationAction.setImageDescriptor(WorkbenchImages.getImageDescriptor (ISharedImages.IMG_TOOL_DELETE));
		_viewer.addSelectionChangedListener(_removeRepresentationAction);
		
		_myContributions = new ActionContributionItem [] {
				new ActionContributionItem (_addRepresentationAction),
				new ActionContributionItem (_removeRepresentationAction)
		};
	}

	/**
	 * 
	 */
	private void hookDoubleClikcAction() {
		_viewer.addDoubleClickListener(new IDoubleClickListener () {

			public void doubleClick(DoubleClickEvent event) {
				_doubleClickAction.run ();
			}
		
		});
	}

	private void updateOverview(AcmeRepresentation rep) {
		DiagramEditor ded = Helper.getDiagramEditorForSystem(rep.getSystem ()); 
		RootEditPart rootEP = ded.getGraphicalViewer ().getRootEditPart();
		if (rootEP != null) {
			ScalableFreeformRootEditPart root = (ScalableFreeformRootEditPart )rootEP;
			IFigure systemView = root.getLayer(LayerConstants.PRINTABLE_LAYERS);
			Thumbnail thumbnail = new Thumbnail (systemView);
			thumbnail.setSource (root.getLayer (LayerConstants.PRINTABLE_LAYERS));
			_lws.setContents(thumbnail);
			
		}
	}


}
