/*
 * Created on Mar 30, 2004
 *
 */
package AcmeStudio.ui.views.parts;

import java.util.List;
import java.util.Set;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import AcmeStudio.core.ASContext;
import AcmeStudio.ui.editors.diagram.actions.HighlightObjectsInDiagramAction;
import AcmeStudio.ui.editors.diagram.actions.UnhighlightAllAction;
import AcmeStudio.ui.views.ruleErrors.AcmeElementTree;
import AcmeStudio.util.Constants;
import AcmeStudio.util.DesignRuleHolder;
import AcmeStudio.util.ImageManager;
import acmeLib.designRules.ExpressionNode;
import acmeLib.designRules.feedback.EvaluationFeedback;
import acmeLib.elements.AcmeElement;

/**
 * @author Bradley Schmerl
 * Created Mar 30, 2004
 */
public class RuleEvaluationBrowserDialog extends Dialog {
	private TableViewer _tableViewer;
	class TableLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object element, int columnIndex) {
			if (element instanceof EvaluationFeedback) {
				EvaluationFeedback eval = (EvaluationFeedback )element;
				if (columnIndex == 0) {
					return eval.getVisitedString();
				}
				else if (columnIndex == 1) {
					return eval.getResult().toString ();
				}
				
			}
			return "";
		}
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}
	}
	class ContentProvider implements IStructuredContentProvider {
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof List) {
				return ((List )inputElement).toArray ();
			}
			return Constants.EMPTY_ELEMENTS;
		}
		public void dispose() {
		}
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
	private Button _highlightButton;
	private DesignRuleBrowser _ruleText;
	private DesignRuleHolder _rule;
	private AcmeElement _scope;
	private AcmeElementTree _elementTree;
	public RuleEvaluationBrowserDialog(Shell parentShell, DesignRuleHolder rule, AcmeElement scope) {
		super(parentShell);
		_rule = rule;
		_scope = scope;
		
	}
	protected Control createDialogArea(Composite parent) {
		Composite area = (Composite) super.createDialogArea(parent);
		area.setLayout(new GridLayout());
		_ruleText = new DesignRuleBrowser (area);
		_ruleText.setSelectionHighlight(true, new ISelectionChangedListener () {

			public void selectionChanged(SelectionChangedEvent event) {
				Object selection = ((IStructuredSelection )event.getSelection()).getFirstElement();
				if (selection instanceof ExpressionNode) {
					ExpressionNode node = (ExpressionNode )selection;
					_tableViewer.setInput(node.getUserData(_scope));
				}
				
			}
			
		});
		_ruleText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_VERTICAL));
		_ruleText.navigateTo(_rule.getDesignRule());
		_ruleText.setEditable(false);
		{
			Composite composite = new Composite(area, SWT.NONE);
			composite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
			GridLayout gridLayout = new GridLayout();
			gridLayout.numColumns = 2;
			composite.setLayout(gridLayout);
			{
				Label label = new Label(composite, SWT.NONE);
				label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				label.setText("Elements causing rule to fail:");
			}
			{
				_highlightButton = new Button(composite, SWT.TOGGLE);
				_highlightButton.addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						if (_highlightButton.getSelection()) {
							Set elements = _elementTree.getAllElements ();
							HighlightObjectsInDiagramAction highlightAction = (HighlightObjectsInDiagramAction )ASContext.current().getActiveEditor().getDiagramEditor().getActionRegistry().getAction(HighlightObjectsInDiagramAction.ID);
							if (highlightAction != null) {
								highlightAction.setSelection(new StructuredSelection (elements.toArray()));
								highlightAction.run ();
							}
						}
						else {
							IAction action = ASContext.current().getActiveEditor().getDiagramEditor().getActionRegistry().getAction(UnhighlightAllAction.ID);
							if (action != null) {
								action.run();
							}
						}
					}
				});
				_highlightButton.setImage(ImageManager.getIcon(ImageManager.ID_HIGHLIGHT_ERRORS));
				_highlightButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
			}
			{
				_tableViewer = new TableViewer(composite, SWT.BORDER | SWT.FULL_SELECTION | SWT.V_SCROLL | SWT.H_SCROLL);
				_tableViewer.setLabelProvider(new TableLabelProvider());
				_tableViewer.setContentProvider(new ContentProvider());
				Table table = _tableViewer.getTable();
				table.setLinesVisible(true);
				table.setHeaderVisible(true);
				GridData gridData = new GridData(GridData.FILL_BOTH);
				gridData.heightHint = 199;
				gridData.horizontalSpan = 2;
				table.setLayoutData(gridData);
				TableLayout layout = new TableLayout ();
				{
					TableColumn visitedColumn = new TableColumn(table, SWT.NONE);
					layout.addColumnData(new ColumnWeightData (75, true));
					visitedColumn.setText("Visited");
				}
				{
					TableColumn resultColumn = new TableColumn(table, SWT.NONE);
					layout.addColumnData(new ColumnWeightData (25, true));
					resultColumn.setText("Result");
				}
				table.setLayout(layout);
				
			}
		}
		{
			

		}
		return area;
	}
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setSize(400, 600);
		newShell.setText(_rule.getLabel());
	}
}
