package AcmeStudio.ui.views.parts;

import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.internal.WorkbenchImages;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.model.*;
import AcmeStudio.model.CreateElementCommand;
import AcmeStudio.model.DeleteElementCommand;
import AcmeStudio.model.ModelCompoundCommand;
import AcmeStudio.model.TypeAssignmentCommand;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.views.IElementViewPage;
import AcmeStudio.util.Helper;
import AcmeStudio.util.ImageManager;
import AcmeStudio.util.Utils;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.filters.ComponentTypeEnumFilter;
import acmeLib.core.filters.ConnectorTypeEnumFilter;
import acmeLib.core.filters.PortTypeEnumFilter;
import acmeLib.core.filters.RoleTypeEnumFilter;
import acmeLib.elements.*;

/**
 * @author schmerl */
public class StructureList extends ElementViewPart implements ISelectionListener {

	private Action _addPortAction;
	private Action _addRoleAction;
	private Action _addComponentAction;
	private Action _addConnectorAction;
	private ViewSelectionAction _deleteElementAction;
	private ViewSelectionAction _changeTypesAction;
	private ViewSelectionAction _selectSubElementAction;

	private IContributionItem[] _myContributions;

	private static final int INHERITCOL = 0;
	private static final int NAMECOL = 1;
	private static final int TYPECOL = 2;
	private static final int NAMEWIDTH = 30;
	private static final int TYPEWIDTH = 65;

	private TableViewer _viewer = null;
	private AcmeObject _selectedObject = null;
	private IElementViewPage _parentView = null;

	private Action _doubleClickAction;
	public static final int NEWPORT = 5;
	public static final int NEWROLE = 6;
	public static final int DELELEMENT = 2;
	private StructureSorter	_sorter;
	protected Object	_selectedSubstructure;

	class ViewContentProvider implements IStructuredContentProvider {

		private Set gatherInheritedStructure(AcmeComponentType ct) {
			Set s = new TreeSet(new Utils.AcmeElementNameComparator());
			s.addAll(Helper.convertAcmeEnumerationToList(((AcmeComponent) ct).getPorts()));
			AcmeEnumeration types = ct.getSuperTypes().enumerate();
			while (types.moreItems()) {
				AcmeElementType type = (AcmeElementType) types.getNextItem();
				if (type instanceof AcmeComponentType) {
					AcmeComponentType cst = (AcmeComponentType) type;
					s.addAll(gatherInheritedStructure(cst));
				}
				else if (type instanceof AcmeElementTypeRef) {
					AcmeElementTypeRef rt = (AcmeElementTypeRef) type;
					if (rt.getReferentType() instanceof AcmeComponentType) {
						AcmeComponentType compT = (AcmeComponentType) rt.getReferentType();
						s.addAll(gatherInheritedStructure(compT));
					}

				}
			}
			return s;
		}

		private Set gatherInheritedStructure(AcmeConnectorType ct) {
			Set s = new TreeSet(new Utils.AcmeElementNameComparator());
			s.addAll(Helper.convertAcmeEnumerationToList(((AcmeConnector) ct).getRoles()));
			AcmeEnumeration types = ct.getSuperTypes().enumerate();
			while (types.moreItems()) {
				AcmeElementType type = (AcmeElementType) types.getNextItem();
				if (type instanceof AcmeConnectorType) {
					AcmeConnectorType cst = (AcmeConnectorType) type;
					s.addAll(gatherInheritedStructure(cst));
				}
				else if (type instanceof AcmeElementTypeRef) {
					AcmeElementTypeRef rt = (AcmeElementTypeRef) type;
					if (rt.getReferentType() instanceof AcmeConnectorType) {
						AcmeConnectorType connT = (AcmeConnectorType) rt.getReferentType();
						s.addAll(gatherInheritedStructure(connT));
					}

				}
			}
			return s;
		}
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		public Object[] getElements(Object parent) {
			if (parent instanceof AcmeSystem) {
				AcmeSystem sys = (AcmeSystem) parent;
				AcmeEnumeration comps = sys.getComponents();
				AcmeEnumeration conns = sys.getConnectors();
				Object[] elements = new Object[comps.getSize() + conns.getSize()];
				int index = 0;
				while (comps.moreItems()) {
					elements[index++] = comps.getNextItem();
				}
				while (conns.moreItems()) {
					elements[index++] = conns.getNextItem();
				}
				return elements;
			}
			else if (parent instanceof AcmeComponentType) {
				AcmeComponentType ct = (AcmeComponentType) parent;
				Set s = gatherInheritedStructure(ct);
				return s.toArray();
			}
			else if (parent instanceof AcmeConnectorType) {
				AcmeConnectorType ct = (AcmeConnectorType) parent;
				Set s = gatherInheritedStructure(ct);
				return s.toArray();

			}
			else if (parent instanceof AcmeComponent) {
				AcmeComponent comp = (AcmeComponent) parent;
				AcmeEnumeration ports = comp.getPorts();
				Object elements[] = new Object[ports.getSize()];
				int index = 0;
				while (ports.moreItems()) {
					elements[index++] = ports.getNextItem();
				}
				return elements;
			}
			else if (parent instanceof AcmeConnector) {
				AcmeConnector comp = (AcmeConnector) parent;
				AcmeEnumeration roles = comp.getRoles();
				Object elements[] = new Object[roles.getSize()];
				int index = 0;
				while (roles.moreItems()) {
					elements[index++] = roles.getNextItem();
				}
				return elements;
			}

			return new Object[0];

		}
	}

	
	private String getTypeLabel(AcmeElement el) {
		String retStr;
		AcmeEnumeration types = el.getDeclaredTypes();
		retStr = "";
		while (types.moreItems()) {
			AcmeElementType et = (AcmeElementType) types.getNextItem();
			retStr = retStr + et.getTypeName() + ",";
		}
		if (!retStr.equals(""))
			retStr = retStr.substring(0, retStr.length() - 1);
		return retStr;
	}
	
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			AcmeElement el = (AcmeElement) obj;
			String retStr = "";
			switch (index) {
				case NAMECOL :
					retStr = el.getName();
					break;
				case TYPECOL :
					{
						retStr = getTypeLabel (el);
					}
			}
			return retStr;
		}
		
		public Image getColumnImage(Object obj, int index) {
			switch (index) {
				case NAMECOL :
					return ImageManager.getSmallImageByType(obj);
				case INHERITCOL:
					if (ModelHelper.isInheritedSubstructure((AcmeElement )obj, (AcmeElement )_selectedObject)) {
						return ImageManager.getIcon(ImageManager.ID_INHERITS);
						
					}
					break;
			}
			return null;
		}
		public Image getImage(Object obj) {
			return null;
		}
	}

	class StructureSorter extends ViewerSorter {
		private static final int SORT_NONE = 0;
		private static final int SORT_A_Z = 1;
		private static final int SORT_Z_A = 2;
		
		private int state = SORT_A_Z;
		protected static final int	CAT_NAME	= 0;
		protected static final int CAT_TYPE = 1;
		private int	_category = CAT_NAME;
		
		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ViewerSorter#compare(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public int compare(Viewer viewer, Object e1, Object e2) {
			if (state == SORT_NONE) return 0;
			AcmeElement elem1 = (AcmeElement )e1;
			AcmeElement elem2 = (AcmeElement )e2;
			String name1 = "";
			String name2 = "";
			if (state == SORT_A_Z) {
				if (_category == CAT_NAME) {
					name1 = elem1.getName ();
					name2 = elem2.getName ();
				}
				else {
					name1 = getTypeLabel(elem1);
					name2 = getTypeLabel(elem2);
				}
			}
			else {
				if (_category == CAT_TYPE) {
					name2 = elem1.getName ();
					name1 = elem2.getName ();
				}
				else {
					name2 = getTypeLabel (elem1);
					name1 = getTypeLabel (elem2);
				}
			}
			return name1.compareTo (name2);
		}
		
		public void cycleState () {
			state = (state + 1) % 3;
		}

		/**
		 * @param string
		 */
		public void setCategory(int cat) {
			if (_category != cat) {
				state = SORT_NONE;
				_category = cat;
			}
			
		}
		
	}

	/**
	 * Constructor for StructureList.
	 * 
	 * @param parent
	 * @param style
	 */
	public StructureList() {
		super();
	}

	/**
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#dispose() */
	public void dispose() {
		AcmeStudioPlugin.getActiveWindow().getSelectionService().removeSelectionListener(this);
		super.dispose();
	}

	public void handleDoubleClick() {
	}

	private void hookDoubleClickAction() {
		_doubleClickAction = _selectSubElementAction;

		_viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				_doubleClickAction.run();
			}
		});
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				if (_selectedObject != null) {
					manager.add(_deleteElementAction);
					if (_addComponentAction.isEnabled()) {
						manager.add(_addComponentAction);
					}
					if (_addConnectorAction.isEnabled()) {
						manager.add(_addConnectorAction);
					}
					if (_addPortAction.isEnabled()) {
						manager.add(_addPortAction);
					}
					if (_addRoleAction.isEnabled()) {
						manager.add(_addRoleAction);
					}
					manager.add(_changeTypesAction);
				}
				manager.add(new Separator("Additions"));
			}
		});

		Menu menu = menuMgr.createContextMenu(_viewer.getControl());
		_viewer.getControl().setMenu(menu);
		if (_parentView != null) {
			_parentView.getSite().registerContextMenu("StructureList", menuMgr, _viewer);
		}
	}

	public void setInput(AcmeObject o) {
		_selectedObject = o;
		_viewer.setInput(o);

		super.setInput(o);
	}
	/**
	 * @see org.eclipse.swt.widgets.Control#setFocus() */
	public boolean setFocus() {
		return _viewer.getControl().setFocus();
	}

	/**
	 * @see org.eclipse.ui.ISelectionListener#selectionChanged(IWorkbenchPart,
	 *      ISelection)
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
	}

	/**
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#getContributions() */
	public IContributionItem[] getContributions() {
		return _myContributions;
	}

	private void enableActionsIfNotInherited () {
		boolean enabled = true;
		if (_viewer.getSelection () != null && !_viewer.getSelection ().isEmpty()) {
			Iterator it = ((IStructuredSelection )_viewer.getSelection ()).iterator ();
			while (it.hasNext ()) {
				AcmeElement elem = (AcmeElement )it.next ();
				enabled = enabled && !ModelHelper.isInheritedSubstructure (elem, (AcmeElement )_selectedObject);
			}
		}
		else 
			enabled = false;
		
		_deleteElementAction.setEnabled(enabled);
		_changeTypesAction.setEnabled (enabled);
	}
	
	private void makeActions() {
		_deleteElementAction = new ViewSelectionAction() {
			public void run() {
				ISelection isel = _viewer.getSelection();
				if (isel instanceof IStructuredSelection) {
					IStructuredSelection iss = (IStructuredSelection) isel;
					Iterator it = iss.iterator();

					ModelCompoundCommand deleteCommand = new ModelCompoundCommand("Delete selection");

					while (it.hasNext()) {
						AcmeElement elem = (AcmeElement) it.next();
						if (!ModelHelper.isInheritedSubstructure(elem, (AcmeElement )_selectedObject)) {
							DeleteElementCommand command =
								new DeleteElementCommand((AcmeElement) _selectedObject, elem, _parentView);
							deleteCommand.add(command);
						}
					}
					deleteCommand.execute(_acmeDesign);
					setInput(_selectedObject);
				}
				
			
			}
			
		};
		_deleteElementAction.setText("Delete");
		_deleteElementAction.setDescription("Deletes the currently selected element");
        // As of Eclipse 3.0, IMG_TOOL_DELETE_HOVER == IMG_TOOL_DELETE
		_deleteElementAction.setImageDescriptor(
			WorkbenchImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		_viewer.addSelectionChangedListener(_deleteElementAction);
		
		_changeTypesAction = new ViewSelectionAction() {
			public void run() {
				ISelection selection = _viewer.getSelection();
				if (selection instanceof StructuredSelection) {
					StructuredSelection s = (StructuredSelection) selection;
					if (s.size() == 1) {
						AcmeElement element = (AcmeElement) s.getFirstElement();
						AssignTypeDialog dialog = new AssignTypeDialog(Display.getDefault().getActiveShell(), element);
						if (dialog.open() == IDialogConstants.OK_ID) {
							ModelCompoundCommand c = new ModelCompoundCommand("Change types");
							Iterator i = dialog.declAdditions.iterator();
							while (i.hasNext()) {
								AcmeElementType type = (AcmeElementType) i.next();

								TypeAssignmentCommand command =
									new TypeAssignmentCommand(
										TypeAssignmentCommand.ASSIGN_TYPE,
										element,
										type,
										false,
										_parentView);
								c.add(command);
								//command.execute (_acmeDesign);

							}
							i = dialog.instAdditions.iterator();
							while (i.hasNext()) {
								AcmeElementType type = (AcmeElementType) i.next();
								TypeAssignmentCommand command =
									new TypeAssignmentCommand(
										TypeAssignmentCommand.ASSIGN_TYPE,
										element,
										type,
										true,
										_parentView);
								//                            command.execute (_acmeDesign);
								c.add(command);

							}

							i = dialog.declDeletions.iterator();
							while (i.hasNext()) {
								AcmeElementType type = (AcmeElementType) i.next();
								TypeAssignmentCommand command =
									new TypeAssignmentCommand(
										TypeAssignmentCommand.UNASSIGN_TYPE,
										element,
										type,
										false,
										_parentView);
								//                            command.execute (_acmeDesign)
								c.add(command);
							}

							i = dialog.instDeletions.iterator();
							while (i.hasNext()) {
								AcmeElementType type = (AcmeElementType) i.next();
								TypeAssignmentCommand command =
									new TypeAssignmentCommand(
										TypeAssignmentCommand.UNASSIGN_TYPE,
										element,
										type,
										true,
										_parentView);
								command.execute(_acmeDesign);
								c.add(command);
							}
							c.execute(_acmeDesign);
						}
						setInput(_selectedObject);

					}
				}
			}

			// 			public boolean isEnabled () {
			// 				return !_viewer.getSelection ().isEmpty();
			// 			}
		};
		_changeTypesAction.setText("Change types");
		_changeTypesAction.setToolTipText("Changes the types of the substructure");
		_changeTypesAction.setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_CHANGE_TYPE));

		_selectSubElementAction = new ViewSelectionAction() {
			public void run() {
				ISelection sel = _viewer.getSelection();
				if (sel instanceof IStructuredSelection) {
					IStructuredSelection s = (IStructuredSelection) sel;
					if (s.size() == 1) {
						AcmeElement element = (AcmeElement) s.getFirstElement();
						SelectionChangedEvent evt =
							new SelectionChangedEvent(
								StructureList.this._parentView.getSite().getSelectionProvider(),
								new StructuredSelection(element));
						ASContext.current().getActiveEditor().selectionChanged(evt);
					}
				}
			}

			public boolean isEnabled () {
				return !_viewer.getSelection ().isEmpty();
			}
		};
		_selectSubElementAction.setText("Browse to element");
		_selectSubElementAction.setToolTipText("Browses to the element in all views");
		_selectSubElementAction.setImageDescriptor(ImageManager.getImageDescriptor(ImageManager.ID_GOTO_ELEMENT));
		_selectSubElementAction.setEnabled (!_viewer.getSelection ().isEmpty());
		_viewer.addSelectionChangedListener(_selectSubElementAction);
		
		_addPortAction = new Action() {
			public void run() {
				if (_selectedObject instanceof AcmeComponent) {
					AcmeComponent comp = (AcmeComponent) _selectedObject;
					AcmeEnumeration availableTypes =
						comp.getAvailableTypes().castToAcmeSetVector().enumerate(new PortTypeEnumFilter());

					NewElementDialog dialog =
						new NewElementDialog(Display.getDefault().getActiveShell(), availableTypes, AcmePort.class, comp);
					if (dialog.open() == IDialogConstants.OK_ID) {
						CreateElementCommand createCommand =
							new CreateElementCommand(comp, dialog.getNewElement(), new Point(0, 0), _parentView);
						createCommand.execute(_acmeDesign);
						setInput(_selectedObject);
					}
				}
			}
			public boolean isEnabled() {
				return _selectedObject instanceof AcmeComponent;
			}
		};
		_addPortAction.setText("New Port");
		_addPortAction.setDescription("Creates a new port in the selected component");
		_addPortAction.setImageDescriptor(ImageManager.getSmallImageDescriptor(ImageManager.ID_PORT));

		_addRoleAction = new Action() {
			public void run() {
				if (_selectedObject instanceof AcmeConnector) {
					AcmeConnector conn = (AcmeConnector) _selectedObject;
					Point connLoc = SystemView.getLocation(conn, true);
					AcmeEnumeration availableTypes =
						conn.getAvailableTypes().castToAcmeSetVector().enumerate(new RoleTypeEnumFilter());

					NewElementDialog dialog =
						new NewElementDialog(Display.getDefault().getActiveShell(), availableTypes, AcmeRole.class, conn);
					if (dialog.open() == IDialogConstants.OK_ID) {
						// BugFix#300: If the parent is a connector type, it has no location, so pass in null
						CreateElementCommand createCommand =
							new CreateElementCommand(
								conn,
								dialog.getNewElement(),
								connLoc!=null?new Point(connLoc.x, connLoc.y + 75):null,
								_parentView);
						createCommand.execute(_acmeDesign);
						setInput(_selectedObject);
					}

				}
			}
			public boolean isEnabled() {
				return _selectedObject instanceof AcmeConnector;
			}

		};
		_addRoleAction.setText("New Role");
		_addRoleAction.setDescription("Creates a new role in the selected connector");
		_addRoleAction.setImageDescriptor(ImageManager.getSmallImageDescriptor(ImageManager.ID_ROLE));

		_addComponentAction = new Action() {
			public void run() {
				if (_selectedObject instanceof AcmeSystem) {
					AcmeSystem sys = (AcmeSystem) _selectedObject;
					AcmeEnumeration availableTypes =
						sys.getAvailableTypes().castToAcmeSetVector().enumerate(new ComponentTypeEnumFilter());

					NewElementDialog dialog =
						new NewElementDialog(
							Display.getDefault().getActiveShell(),
							availableTypes,
							AcmeComponent.class, sys);
					Point location = new Point(Math.random() * 200, Math.random() * 200);
					if (dialog.open() == IDialogConstants.OK_ID) {
						CreateElementCommand createCommand =
							new CreateElementCommand(sys, dialog.getNewElement(), location, _parentView);
						createCommand.execute(_acmeDesign);
						setInput(_selectedObject);
					}
				}
			}

			public boolean isEnabled() {
				return _selectedObject instanceof AcmeSystem;
			}
		};
		_addComponentAction.setText("New Component");
		_addComponentAction.setDescription("Creates a new component in the selected system");
		_addComponentAction.setImageDescriptor(ImageManager.getSmallImageDescriptor(ImageManager.ID_COMP));

		_addConnectorAction = new Action() {
			public void run() {
				if (_selectedObject instanceof AcmeSystem) {
					AcmeSystem sys = (AcmeSystem) _selectedObject;
					AcmeEnumeration availableTypes =
						sys.getAvailableTypes().castToAcmeSetVector().enumerate(new ConnectorTypeEnumFilter());

					NewElementDialog dialog =
						new NewElementDialog(
							Display.getDefault().getActiveShell(),
							availableTypes,
							AcmeComponent.class, sys);
					if (dialog.open() == IDialogConstants.OK_ID) {
						Point location = new Point(Math.random() * 200, Math.random() * 200);
						CreateElementCommand createCommand =
							new CreateElementCommand(sys, dialog.getNewElement(), location, _parentView);
						createCommand.execute(_acmeDesign);
						setInput(_selectedObject);
					}
				}
			}
			public boolean isEnabled() {
				return _selectedObject instanceof AcmeSystem;
			}
		};
		_addConnectorAction.setText("New Connector");
		_addConnectorAction.setDescription("Creates a new connector in the selected system");
		_addConnectorAction.setImageDescriptor(ImageManager.getSmallImageDescriptor(ImageManager.ID_CONN));

		_myContributions =
			new ActionContributionItem[] {
				new ActionContributionItem(_selectSubElementAction) { public boolean isDynamic() { return true;
				}
			}, new ActionContributionItem(_changeTypesAction) {
				public boolean isDynamic() {
					return true;
				}
			}, new ActionContributionItem(_deleteElementAction), new ActionContributionItem(_addComponentAction) {
				public boolean isDynamic() {
					return true;
				}
				public boolean isVisible() {
					return getAction().isEnabled();
				}
			}, new ActionContributionItem(_addConnectorAction) {
				public boolean isDynamic() {
					return true;
				}
				public boolean isVisible() {
					return getAction().isEnabled();
				}
			}, new ActionContributionItem(_addPortAction) {
				public boolean isDynamic() {
					return true;
				}
				public boolean isVisible() {
					return getAction().isEnabled();
				}
			}, new ActionContributionItem(_addRoleAction) {
				public boolean isDynamic() {
					return true;
				}
				public boolean isVisible() {
					return getAction().isEnabled();
				}
			}
		};

	}

	/**
	 * @see AcmeStudio.ui.views.parts.ElementViewPart#isValidFor(AcmeObject) */
	public boolean isValidFor(AcmeObject o) {
		return o instanceof AcmeSystem || o instanceof AcmeComponent || o instanceof AcmeConnector;
	}

	/*
	 * (non-Javadoc) @see
	 * AcmeStudio.ui.views.parts.ElementViewPart#createControl(org.eclipse.swt.widgets.Composite,
	 * int, AcmeStudio.ui.views.IElementViewPage)
	 */
	public void createControl(Composite parent, int style, IElementViewPage vp) {
		super.createControl(parent, style, vp);
		_parentView = vp;
		FillLayout tableLayout = new FillLayout();
		this.getComposite().setLayout(tableLayout);

		_viewer = new TableViewer(this.getComposite(), SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
		_viewer.setContentProvider(new ViewContentProvider());
		_viewer.setLabelProvider(new ViewLabelProvider());
		
		_sorter	= new StructureSorter();
		_viewer.setSorter(_sorter);
		_viewer.setInput(ASContext.current().getActiveDesign().getDesign());
		_viewer.addSelectionChangedListener(new ISelectionChangedListener () {

			public void selectionChanged(SelectionChangedEvent event) {
				enableActionsIfNotInherited();
			}
			
		});

		Table table = _viewer.getTable();
		table.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent evt) {
				if (evt.character == SWT.DEL && _deleteElementAction.isEnabled()) {
					_deleteElementAction.run();
				}
			}
		});
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		TableColumn inheritCol = new TableColumn(table, SWT.CENTER, INHERITCOL);
		TableColumn nameCol = new TableColumn(table, SWT.LEFT, NAMECOL);
		TableColumn typeCol = new TableColumn(table, SWT.LEFT, TYPECOL);

		nameCol.setText("Name");
		typeCol.setText("Type");

		nameCol.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected (SelectionEvent evt) {
				_sorter.setCategory (StructureSorter.CAT_NAME);
				_sorter.cycleState ();
				_viewer.refresh();
			}
		});
		
		typeCol.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected (SelectionEvent evt) {
				_sorter.setCategory (StructureSorter.CAT_TYPE);
				_sorter.cycleState ();
				_viewer.refresh();
			}
		});
		ColumnLayoutData inheritLayout = new ColumnWeightData(INHERITCOL, true);
		ColumnLayoutData nameLayout = new ColumnWeightData(NAMEWIDTH, true);
		ColumnLayoutData typeLayout = new ColumnWeightData(TYPEWIDTH, true);

		// set columns in Table layout
		TableLayout layout = new TableLayout();
		layout.addColumnData(inheritLayout);
		layout.addColumnData(nameLayout);
		layout.addColumnData(typeLayout);
		table.setLayout(layout);

		// Add actions
		_doubleClickAction = new Action() {
			public void run() {
				handleDoubleClick();
			}
		};
		makeActions();
		hookDoubleClickAction();
		hookContextMenu();

		table.setVisible(true);
		enableActionsIfNotInherited();

	}

	/* (non-Javadoc) @see AcmeStudio.ui.views.parts.ElementViewPart#getTitle() */
	public String getTitle() {
		return "Structure";
	}

}
