/*
 * Created on Feb 19, 2004
 *  
 */
package AcmeStudio.ui.views.parts.propertyEditors;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.PageBook;

import AcmeStudio.ui.views.parts.PropertyValueInspector;
import AcmeStudio.util.AcmeErroneousPropertyValue;
import AcmeStudio.util.AutoApplyEditableComboBoxCellEditor;
import AcmeStudio.util.Listeners;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.properties.*;
/**
 * @author Bradley Schmerl Created Feb 19, 2004
 */
public class PropertyValueCellEditor extends CellEditor {

	private class PropertyCellComboCellEditor extends AutoApplyEditableComboBoxCellEditor {

		/**
		 * @param parent
		 * @param list
		 */
		public PropertyCellComboCellEditor(Composite parent, String [] list) {
			super (parent, list);
		}

		/**
		 * 
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.CellEditor#focusLost()
		 */
		protected void focusLost() {
			if (!this.getControl ().isFocusControl ()) {
				super.focusLost ();
				PropertyValueCellEditor.this.focusLost ();
			}
		}
		
		protected Object doGetValue () {
			Object o = super.doGetValue ();
			if (o instanceof Integer && ((Integer )o).intValue() == -1) {
				CCombo combo = (CCombo )this.getControl ();
				return combo.getText ();
			}
			return o;
		}
	}

	private class PropertyValueRecordCellEditor extends PropertyValueDialogEditor {

		/**
		 * @param parent
		 */
		public PropertyValueRecordCellEditor(Composite parent) {
			super (parent);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.CellEditor#fireApplyEditorValue()
		 */
		protected void fireApplyEditorValue() {
			super.fireApplyEditorValue ();
			PropertyValueCellEditor.this.fireApplyEditorValue ();
		}

		protected void focusLost() {
			//			if (!this.getControl ().isFocusControl ()) {
			super.focusLost ();
			PropertyValueCellEditor.this.focusLost ();
			//			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.DialogCellEditor#openDialogBox(org.eclipse.swt.widgets.Control)
		 */
		protected Object openDialogBox(Control cellEditorWindow) {
			Object value = getValue ();
			AcmePropertyType type = null;
			if (value instanceof AcmeSequence) {
				type = ((AcmeSequence )getValue ()).getType ();
			}
			else if (value instanceof AcmeErroneousPropertyValue) {
				type = ((AcmeErroneousPropertyValue )value).getType();
			}
			RecordValueEditorDialog dialog = new RecordValueEditorDialog (cellEditorWindow.getShell (),
					(AcmeRecord )PropertyValueCellEditor.this.getValue ());
			dialog.setValue ((AcmeRecord )PropertyValueCellEditor.this.getValue ());
			if (dialog.open () == IDialogConstants.OK_ID) {
				return dialog.getValue ();
			}
			else {
				this.deactivate ();
				this.getControl ().setVisible (false);
			}
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see AcmeStudio.ui.views.parts.propertyEditors.PropertyValueCellEditor.PropertyValueDialogEditor#setDesiredType(acmeLib.properties.AcmePropertyValue)
		 */
		protected void setDesiredType(AcmePropertyValue val) {
			if (val instanceof AcmeRecord) {
				_desiredType = ((AcmeRecord )val).getType ();
			}
		}
	}

	private class PropertyValueSeqCellEditor extends PropertyValueDialogEditor {

		public PropertyValueSeqCellEditor(Composite parent) {
			super (parent);
		}

		protected void fireApplyEditorValue() {
			super.fireApplyEditorValue ();
			PropertyValueCellEditor.this.fireApplyEditorValue ();
		}

		protected Object openDialogBox(Control control) {
			Object value = getValue ();
			AcmePropertyType type = null;
			if (value instanceof AcmeSequence) {
				type = ((AcmeSequence )getValue ()).getType ();
			}
			else if (value instanceof AcmeErroneousPropertyValue) {
				type = ((AcmeErroneousPropertyValue )value).getType();
			}
			SequenceValueEditorDialog dialog = new SequenceValueEditorDialog (control.getShell (),
					type);
			if (value instanceof AcmeSequence) dialog.setValue ((AcmeSequence )getValue ());
			if (dialog.open () == IDialogConstants.OK_ID) {
				return dialog.getValue ();
			}
			else {
				this.deactivate ();
				this.getControl ().setVisible (false);
			}
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see AcmeStudio.ui.views.parts.propertyEditors.PropertyValueCellEditor.PropertyValueDialogEditor#setDesiredType(acmeLib.properties.AcmePropertyValue)
		 */
		protected void setDesiredType(AcmePropertyValue val) {
			if (val instanceof AcmeSequence) {
				_desiredType = ((AcmeSequence )val).getType ();
			}
		}
	}

	private class PropertyValueSetCellEditor extends PropertyValueDialogEditor {

		public PropertyValueSetCellEditor(Composite parent) {
			super (parent);
		}

		protected void fireApplyEditorValue() {
			super.fireApplyEditorValue ();
			PropertyValueCellEditor.this.fireApplyEditorValue ();
		}

		protected Object openDialogBox(Control control) {
			Object value = getValue ();
			AcmePropertyType type = null;
			if (value instanceof AcmeSet) {
				type = ((AcmeSet )getValue ()).getType ();
			}
			else if (value instanceof AcmeErroneousPropertyValue) {
				type = ((AcmeErroneousPropertyValue )value).getType();
			}
			SetValueEditorDialog dialog = new SetValueEditorDialog (control.getShell (), type);
			if (value instanceof AcmeSet) dialog.setValue ((AcmeSet )getValue ());
			if (dialog.open () == IDialogConstants.OK_ID) {
				return dialog.getValue ();
			}
			else {
				this.deactivate ();
				this.getControl ().setVisible (false);
			}
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see AcmeStudio.ui.views.parts.propertyEditors.PropertyValueCellEditor.PropertyValueDialogEditor#setDesiredType(acmeLib.properties.AcmePropertyValue)
		 */
		protected void setDesiredType(AcmePropertyValue val) {
			if (val instanceof AcmeSet) {
				_desiredType = ((AcmeSet )val).getType ();
			}
		}
	}

	private class PropertyValueTextCellEditor extends TextCellEditor {

		public PropertyValueTextCellEditor(Composite parent) {
			super (parent);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.CellEditor#focusLost()
		 */
		protected void focusLost() {
			if (!this.getControl ().isFocusControl ()) {
				super.focusLost ();
				PropertyValueCellEditor.this.focusLost ();
			}
		}
	}
	protected static final int	INT				= 0, FLOAT = 1, STRING = 2, BOOL = 3, ENUM = 4, RECORD = 5, SEQ = 6, SET = 7,
			OTHER = 8;
	private int					_activePage;
	private EnumType			_currentEnumType;
	private String []			_currentEnumValues;
	CellEditor []				_editors;
	
	PageBook					_pbEditors;

	/**
	 * @param parent
	 */
	public PropertyValueCellEditor(Composite parent) {
		super (parent);
	}

	/**
	 * @param parent
	 * @param style
	 */
	public PropertyValueCellEditor(Composite parent, int style) {
		super (parent, style);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.CellEditor#focusLost()
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.CellEditor#activate()
	 */
	public void activate() {
		super.activate ();
		_editors [_activePage].activate ();
	}

	/**
	 * @param editors
	 * @return
	 */
	private CellEditor createBooleanEditor(PageBook editors) {
		ComboBoxCellEditor editor = new PropertyCellComboCellEditor/* AutoApplyEditableComboBoxCellEditor */(editors,
				new String []{"true", "false"});
		return editor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.CellEditor#createControl(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createControl(Composite parent) {
		_pbEditors = new PageBook (parent, SWT.NONE) {

			public void showPage(Control c) {
//				String page = "";
//				if (c == _editors [INT].getControl ())
//					page = "INT";
//				else if (c == _editors [FLOAT].getControl ())
//					page = "FLOAT";
//				else if (c == _editors [STRING].getControl ())
//					page = "STRING";
//				else if (c == _editors [BOOL].getControl ())
//					page = "BOOL";
//				else if (_editors [ENUM] != null && c == _editors [ENUM].getControl ())
//					page = "ENUM";
//				else if (c == _editors [RECORD].getControl ())
//					page = "RECORD";
//				else if (c == _editors [SEQ].getControl ())
//					page = "SEQ";
//				else if (c == _editors [SET].getControl ())
//					page = "SET";
//				else if (c == _editors [OTHER].getControl ()) page = "OTHER";
				super.showPage (c);
			}
		};
		_editors = new CellEditor [OTHER + 1];
		_editors [INT] = createIntEditor (_pbEditors);
		_editors [FLOAT] = createFloatEditor (_pbEditors);
		_editors [STRING] = createStringEditor (_pbEditors);
		_editors [BOOL] = createBooleanEditor (_pbEditors);
		_editors [ENUM] = null;
		_editors [RECORD] = createRecordEditor (_pbEditors);
		_editors [SET] = createSetEditor (_pbEditors);
		_editors [SEQ] = new PropertyValueSeqCellEditor (_pbEditors);
		_editors [OTHER] = new PropertyValueTextCellEditor (_pbEditors);
		return _pbEditors;
	}

	/**
	 * @param editors
	 * @return
	 */
	private CellEditor createFloatEditor(PageBook editors) {
		TextCellEditor editor = new PropertyValueTextCellEditor (editors);
		Text text = (Text )editor.getControl ();
		Listeners.DoubleValidator v = new Listeners.DoubleValidator ();
		v.setTarget (text);
		text.addVerifyListener (v.getValidator ());
		return editor;
	}

	/**
	 * @param editors
	 * @return
	 */
	private CellEditor createIntEditor(PageBook editors) {
		TextCellEditor editor = new PropertyValueTextCellEditor (editors);
		Text text = (Text )editor.getControl ();
		Listeners.IntValidator v = new Listeners.IntValidator ();
		v.setTarget (text);
		text.addVerifyListener (v.getValidator ());
		return editor;
	}

	/**
	 * @param editors
	 * @return
	 */
	private CellEditor createRecordEditor(PageBook editors) {
		return new PropertyValueRecordCellEditor (editors);
	}

	/**
	 * @param editors
	 * @return
	 */
	private CellEditor createSetEditor(PageBook editors) {
		return new PropertyValueSetCellEditor (editors);
	}

	/**
	 * @param editors
	 * @return
	 */
	private CellEditor createStringEditor(PageBook editors) {
		TextCellEditor editor = new PropertyValueTextCellEditor (editors);
		Text text = (Text )editor.getControl ();
		Listeners.StringValidator v = new Listeners.StringValidator ();
		v.setTarget (text);
		text.addVerifyListener (v.getValidator ());
		return editor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.CellEditor#doGetValue()
	 */
	protected Object doGetValue() {
		switch (_activePage) {
			case INT :
				{
					String val = (String )_editors [INT].getValue ();
					try {
						int intV = Integer.parseInt (val);
						return new AcmeInt (intV);
					}
					catch (NumberFormatException e) {
					}
					break;
				}
			case FLOAT :
				{
					String val = (String )_editors [FLOAT].getValue ();
					try {
						double dubV = Double.parseDouble (val);
						return new AcmeFloat ((float )dubV);
					}
					catch (NumberFormatException e) {
					}
					break;
				}
			case STRING :
				{
					String val = (String )_editors [STRING].getValue ();
					return new AcmeString (val);
				}
			case BOOL :
				{
					Object o = _editors [BOOL].getValue ();
					if (o instanceof Integer) {
						Integer val = (Integer )_editors [BOOL].getValue ();
						if (val.intValue () == 0)
							return new AcmeBoolean (true);
						else if (val.intValue () == 1) return new AcmeBoolean (false);
					}
					else if (o instanceof String) {
						return new AcmeErroneousPropertyValue ((String )o, ConstantTypes.BOOLEAN_TYPE);
					}
				}
			case ENUM :
				{
					Object val = _editors [ENUM].getValue ();
					if (val instanceof String) {
						String anonymousTypeVal = (String )val;
						try {
							return PropertyValueInspector.getPropertyValue (anonymousTypeVal);
						}
						catch (ParseException e) {
						}
					}
					else if (val instanceof Integer) {
						int index = ((Integer )val).intValue ();
						if (_currentEnumValues != null) {
							AcmeEnum enum = new AcmeEnum (_currentEnumValues [index]);
							enum.setType (_currentEnumType);
							return enum;
						}
					}
					break;
				}
			case RECORD :
				{
					Object val = _editors [RECORD].getValue ();
					if (val instanceof AcmeRecord) {
						return val;
					}
				}
			case SEQ :
				{
					Object val = _editors [SEQ].getValue ();
					if (val instanceof AcmeSequence) return val;
					break;
				}
			case SET :
				{
					Object val = _editors [SET].getValue ();
					if (val instanceof AcmeSet) {
						return val;
					}
					break;
				}
			default :
				{
					String val = (String )_editors [OTHER].getValue ();
					try {
						return PropertyValueInspector.getPropertyValue (val);
					}
					catch (ParseException e) {
					}
				}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.CellEditor#doSetFocus()
	 */
	protected void doSetFocus() {
		//		_editors [_activePage].setFocus();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.CellEditor#doSetValue(java.lang.Object)
	 */
	protected void doSetValue(Object value) {
		if (value instanceof AcmePropertyValue) {
			AcmePropertyValue pVal = (AcmePropertyValue )value;
			AcmePropertyType type = pVal.getType ();
			
		
			if (type instanceof IntType) {
//				AcmeInt val = (AcmeInt )value;
				_activePage = INT;
				_editors [INT].dispose ();
				_editors [INT] = createIntEditor (_pbEditors);
				_pbEditors.showPage (_editors [INT].getControl ());
				_editors [INT].setValue (pVal.toString ());
			}
			else if (type instanceof FloatType) {
//				AcmeFloat val = (AcmeFloat )value;
				_activePage = FLOAT;
				_editors [FLOAT].dispose ();
				_editors [FLOAT] = createFloatEditor (_pbEditors);
				_pbEditors.showPage (_editors [FLOAT].getControl ());
				_editors [FLOAT].setValue (pVal.toString ());
			}
			else if (type instanceof StringType) {
//				AcmeString val = (AcmeString )value;
				_activePage = STRING;
				_editors [STRING].dispose ();
				_editors [STRING] = null;
				_editors [STRING] = createStringEditor (_pbEditors);
				_pbEditors.showPage (_editors [STRING].getControl ());
				AcmeString val = null;
				if (pVal instanceof AcmeErroneousPropertyValue) {
					val = new AcmeString (pVal.toString ());
//					val = new AcmeString ("\"" + pVal.toString () + "\"");
				}
				else {
					val = (AcmeString )pVal;
				}
				_editors [STRING].setValue (val.toString ().substring (1, val.toString ().length () - 1));
			}
			else if (type instanceof BooleanType) {
//				AcmeBoolean val = (AcmeBoolean )value;
				_activePage = BOOL;
				_editors [BOOL].dispose ();
				_editors [BOOL] = createBooleanEditor (_pbEditors);
				_pbEditors.showPage (_editors [BOOL].getControl ());
				if (pVal instanceof AcmeBoolean) {
					AcmeBoolean val = (AcmeBoolean )pVal;
					if (val.getValue () == true) {
						_editors [BOOL].setValue (new Integer (0));
					}
					else {
						_editors [BOOL].setValue (new Integer (1));
					}
				}
				else {
					CCombo c = (CCombo )_editors [BOOL].getControl();
					_editors [BOOL].setValue (new Integer (-1));
					c.setText (pVal.toString ());
				}
			}
			else if (type instanceof EnumType) {
				_currentEnumType = (EnumType )type;
				_currentEnumValues = null;
				AcmeEnumeration vals = _currentEnumType.getValues ();
				int valIndex = -1;
				int index = 0;
				_currentEnumValues = new String [vals.getSize ()];
				while (vals.moreItems ()) {
					EnumValue eval = (EnumValue )vals.getNextItem ();
					_currentEnumValues [index++] = eval.toString ();
					if ((value instanceof AcmeEnum ) && (eval.toString ().equals (value.toString ()))) {
						valIndex = index - 1;
					}
				}
					
						
				if (_editors [ENUM] != null) _editors [ENUM].dispose ();
				_editors [ENUM] = new PropertyCellComboCellEditor (_pbEditors, _currentEnumValues);
				if (value instanceof AcmeEnum) {
				
						_editors [ENUM].setValue (new Integer (valIndex));
					
				}
				else if (value instanceof AcmeErroneousPropertyValue){
					CCombo c = (CCombo) _editors [ENUM].getControl ();
					c.setText (value.toString ());
					_editors [ENUM].setValue(new Integer (-1));
				}
				else {
					_editors [ENUM] = new PropertyValueTextCellEditor (_pbEditors);
					((Text )_editors [ENUM].getControl ()).addVerifyListener (Listeners.VERIFY_ACMEID);
					_editors [ENUM].setValue (value.toString ());
				}
				_pbEditors.showPage (_editors [ENUM].getControl ());
				_activePage = ENUM;
			}
			else if (type instanceof RecordType) {
				_editors [RECORD].dispose ();
				_editors [RECORD] = null;
				_editors [RECORD] = createRecordEditor (_pbEditors);
//				AcmeRecord val = (AcmeRecord )value;
				_activePage = RECORD;
				_pbEditors.showPage (_editors [RECORD].getControl ());
				_editors [RECORD].setValue (value);
			}
			else if (type instanceof SequenceType) {
//				AcmeSequence val = (AcmeSequence )value;
				_activePage = SEQ;
				_editors [SEQ].dispose ();
				_editors [SEQ] = new PropertyValueSeqCellEditor (_pbEditors);
				_pbEditors.showPage (_editors [SEQ].getControl ());
				_editors [SEQ].setValue (value);
			}
			else if (type instanceof SetType) {
//				AcmeSet val = (AcmeSet )value;
				_activePage = SET;
				_editors [SET].dispose ();
				_editors [SET] = createSetEditor (_pbEditors);
				_pbEditors.showPage (_editors [SET].getControl ());
				_editors [SET].setValue (value);
			}
			else if (value instanceof AcmePropertyValue) {
				AcmePropertyValue val = (AcmePropertyValue )value;
				_activePage = OTHER;
				_editors [OTHER].dispose ();
				_editors [OTHER] = new PropertyValueTextCellEditor (_pbEditors);
				_pbEditors.showPage (_editors [OTHER].getControl ());
				_editors [OTHER].setValue (val.toString ());
			}
		}
	}




	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.CellEditor#fireApplyEditorValue()
	 */
	public void fireApplyEditorValue() {
		super.fireApplyEditorValue ();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.CellEditor#focusLost()
	 */
	protected void focusLost() {
		super.focusLost ();
	}
}
