/*
 * Created on Feb 22, 2004
 *  
 */
package AcmeStudio.ui.views.parts.propertyEditors;

import java.io.IOException;
import java.text.MessageFormat;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.*;
import AcmeStudio.ui.views.parts.PropertyValueInspector;
import AcmeStudio.util.AcmeErroneousPropertyValue;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.properties.AcmePropertyType;
import acmeLib.properties.AcmePropertyValue;
import acmeLib.typeManagement.AcmeTypeManager;
abstract class PropertyValueDialogEditor extends DialogCellEditor {
	protected AcmePropertyType	_desiredType	= null;
	protected boolean			_needToParse	= false;

	private Text				_propertyText;

	/**
	 * @param parent
	 */
	public PropertyValueDialogEditor(Composite parent) {
		super (parent);
		this.setValidator (new ICellEditorValidator () {

			public String isValid(Object value) {
				if (value instanceof AcmePropertyValue) {
					AcmePropertyValue pVal = (AcmePropertyValue )value;
					if (!AcmeTypeManager.typecheckProperty (_desiredType, pVal)) {
						AcmeOutputStream aos = new AcmeOutputStream ();
						String error = "Value:\n" + value.toString () + "\ndoes not satisfy its type";
						try {
							_desiredType.exportArmani (aos);
							aos.close ();
							error += ":\n" + aos.toString ();
						}
						catch (IOException e) {
						}
						setErrorMessage (error);
						return error;
					}
				}
				return "";
			}
		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.DialogCellEditor#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite cell) {
		_propertyText = new Text (cell, SWT.LEFT);
		_propertyText.setFont (cell.getFont ());
		_propertyText.setBackground (cell.getBackground ());
		return _propertyText;
	}

	protected Control createControl(Composite parent) {
		Control c = super.createControl (parent);
		_propertyText.addKeyListener (new KeyAdapter () {

			public void keyReleased(KeyEvent evt) {
				_needToParse = true;
			}
		});
		//			_propertyText.addSelectionListener(new SelectionAdapter () {
		//				public void widgetSelected (SelectionEvent evt) {
		//					String textToParse = _propertyText.getText();
		//					try {
		//						AcmePropertyValue value =
		// PropertyValueInspector.getPropertyValue(textToParse);
		//						boolean newValidState = isCorrect (value);
		//						if (newValidState) {
		//							markDirty ();
		//							doSetValue (value);
		//						}
		//						else {
		//							setErrorMessage (MessageFormat.format (getErrorMessage (), new
		// Object [] {value.toString ()}));
		//						}
		//						fireApplyEditorValue ();
		//					} catch (ParseException e) {
		//						MessageDialog.openError (Display.getCurrent().getActiveShell(),
		// "Parse error", "The value: " + textToParse + "\n failed to parse for
		// the following reasons:\n" + e.getMessage() + "\nTry editing via the
		// dialog.");
		//						updateContents(getValue ());
		//					}
		//				}
		//			});
		_propertyText.addFocusListener (new FocusAdapter () {

			public void focusLost(FocusEvent evt) {
				Composite c = (Composite )PropertyValueDialogEditor.this.getControl ();
				if (!PropertyValueDialogEditor.this.getControl ().isFocusControl () && !c.getChildren () [1].isFocusControl ()) {
					String textToParse = _propertyText.getText ();
					try {
						AcmePropertyValue value = PropertyValueInspector.getPropertyValue (textToParse);
						boolean newValidState = isCorrect (value);
						if (newValidState) {
							markDirty ();
							doSetValue (value);
						}
						else {
							setErrorMessage (MessageFormat.format (getErrorMessage (), new Object []{value.toString ()}));
							MessageDialog.openError (Display.getCurrent ().getActiveShell (), "Parse error", getErrorMessage ());
							value = new AcmeErroneousPropertyValue (value.toString (), _desiredType);
							markDirty ();
							doSetValue (value);
						}
						fireApplyEditorValue ();
					}
					catch (ParseException e) {
						MessageDialog.openError (Display.getCurrent ().getActiveShell (), "Parse error", "The value: "
								+ textToParse + "\n failed to parse for the following reasons:\n" + e.getMessage ()
								+ "\nTry editing via the dialog.");
						AcmeErroneousPropertyValue value = new AcmeErroneousPropertyValue (textToParse, _desiredType);
						markDirty ();
						doSetValue (value);
//						updateContents (getValue ());
					}
				}
			}
		});
		return c;
	}

	//		/* (non-Javadoc)
	//		 * Method declared on CellEditor.
	//		 */
	//		public Object doGetValue () {
	//			if (!_needToParse) return super.doGetValue ();
	//			String textToParse = _propertyText.getText();
	//			try {
	//				AcmePropertyValue value =
	// PropertyValueInspector.getPropertyValue(textToParse);
	//				return value;
	//			} catch (ParseException e) {
	//				MessageDialog.openError (Display.getCurrent().getActiveShell(), "Parse
	// error", "The value: " + textToParse + "\n failed to parse for the
	// following reasons:\n" + e.getMessage() + "\nTry editing via the
	// dialog.");
	//				return super.doGetValue ();
	//			}
	//		}
	public void doSetValue(Object value) {
		if (value instanceof AcmePropertyValue) {
			AcmePropertyValue pVal = (AcmePropertyValue )value;
			setDesiredType (pVal);
		}
		super.doSetValue (value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.DialogCellEditor#getDefaultLabel()
	 */
	protected Label getDefaultLabel() {
		return null;
	}

	/**
	 * @param val
	 */
	protected abstract void setDesiredType(AcmePropertyValue val);

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.DialogCellEditor#updateContents(java.lang.Object)
	 */
	protected void updateContents(Object value) {
		if (_propertyText == null) return;
		String text = "";
		if (value != null) {
			text = value.toString ();
		}
		_propertyText.setText (text);
		_needToParse = false;
	}
}