/*
 * Created on Feb 19, 2004
 *  
 */
package AcmeStudio.ui.views.parts.propertyEditors;

import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import AcmeStudio.util.Constants;
import AcmeStudio.util.Utils;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.designRules.Scope;
import acmeLib.properties.*;
/**
 * @author Bradley Schmerl Created Feb 19, 2004
 */
public class RecordValueEditor extends Composite {

	private class RecordFieldContentProvider implements IStructuredContentProvider {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object [] getElements(Object inputElement) {
			Object [] elements = Constants.EMPTY_ELEMENTS;
			AcmeRecord record = (AcmeRecord )inputElement;
			AcmeEnumeration fields = record.getFields ();
			int additional = _hasNoType ? 1 : 0;
			elements = new Object [fields.getSize () + additional];
			int index = 0;
			while (fields.moreItems ()) {
				AcmeRecordField element = (AcmeRecordField )fields.getNextItem ();
				elements [index++] = element;
			}
			if (_hasNoType) {
				elements [index] = new AcmeRecordField ("", null, null);
			}
			return elements;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	private class RecordFieldLabelProvider extends LabelProvider implements ITableLabelProvider {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
		 *      int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
		 *      int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			if (columnIndex == 0) {
				String name = ((AcmeRecordField )element).getName ();
				if (name.equals ("")) {
					return "<Enter field name>";
				}
				return name;
			}
			else if (columnIndex == 1) {
				AcmePropertyType t = ((AcmeRecordField )element).getType ();
				if (t != null) {
					return t.getTypeName ();
				}
				else {
					return "<Enter field type>";
				}
			}
			else if (columnIndex == 2) {
				AcmePropertyValue v = ((AcmeRecordField )element).getValue ();
				if (v != null)
					return v.toString ();
				else
					return "<Enter field value>";
			}
			return "";
		}
	}

	/**
	 * @author Bradley Schmerl Created Feb 20, 2004
	 */
	public class RecordFieldModifier implements ICellModifier {

		/**
		 *  
		 */
		public RecordFieldModifier() {
			super ();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
		 *      java.lang.String)
		 */
		public boolean canModify(Object element, String property) {
			if (property.equals ("value")) return true;
			return false;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
		 *      java.lang.String)
		 */
		public Object getValue(Object element, String property) {
			if (element instanceof AcmeRecordField && property.equals ("value")) {
				AcmeRecordField field = (AcmeRecordField )element;
				AcmePropertyValue value = (field).getValue ();
				if (value == null) {
					AcmePropertyType type = field.getType ();
					if (type instanceof IntType) {
						value = DUMMY_INT;
					}
					else if (type instanceof FloatType) {
						value = DUMMY_FLOAT;
					}
					else if (type instanceof StringType) {
						value = DUMMY_STRING;
					}
					else if (type instanceof BooleanType) {
						value = DUMMY_BOOLEAN;
					}
					else if (type instanceof EnumType) {
						DUMMY_ENUM = new AcmeEnum ("???");
						DUMMY_ENUM.setType ((EnumType )type);
						value = DUMMY_ENUM;
					}
					else if (type instanceof RecordType) {
						DUMMY_RECORD = new AcmeRecord ();
						DUMMY_RECORD.setType ((RecordType )type);
						value = DUMMY_RECORD;
					}
					else if (type instanceof SetType) {
						DUMMY_SET = new AcmeSet ();
						DUMMY_SET.setType ((SetType )type);
						value = DUMMY_SET;
					}
					else if (type instanceof SequenceType) {
						DUMMY_SEQUENCE = new AcmeSequence ();
						DUMMY_SEQUENCE.setType ((SequenceType )type);
						value = DUMMY_SEQUENCE;
					}
				}
				return value;
			}
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
		 *      java.lang.String, java.lang.Object)
		 */
		public void modify(Object element, String property, Object value) {
			if (element instanceof TableItem) {
				TableItem item = (TableItem )element;
				AcmeRecordField field = (AcmeRecordField )item.getData ();
				if (property.equals ("value")) {
					field.setValue ((AcmePropertyValue )value);
				}
				_fieldViewer.refresh ();
			}
		}
	}
	private static final AcmePropertyValue	DUMMY_BOOLEAN	= new AcmeBoolean (false);
	private static final AcmePropertyValue	DUMMY_FLOAT		= new AcmeFloat (0);

	private static final AcmePropertyValue	DUMMY_INT		= new AcmeInt (0);
	private static final AcmePropertyValue	DUMMY_STRING	= new AcmeString ("");
	protected static final String []	PROPERTIES		= new String []{"name", "type", "value"};
	private static final String []		REC_PROPERTIES	= {"name", "type", "value"};
	private TableViewer	_fieldViewer;
	private boolean						_hasNoType		= false;
	private AcmeRecord	_record;
	private AcmeEnum						DUMMY_ENUM;
	private AcmeRecord						DUMMY_RECORD;
	private AcmeSequence					DUMMY_SEQUENCE;
	private AcmeSet							DUMMY_SET;

	public RecordValueEditor(Composite parent, int style, Scope scope) {
		super (parent, style);
		setLayout (new GridLayout ());
		{
			_fieldViewer = new TableViewer (this, SWT.BORDER | SWT.FULL_SELECTION | SWT.V_SCROLL);
			final Table table = _fieldViewer.getTable ();
			table.setLayoutData (new GridData (GridData.FILL_BOTH));
			table.setLinesVisible (true);
			table.setHeaderVisible (true);
			TableColumn nameCol = new TableColumn (table, SWT.LEFT, 0);
			TableColumn typeCol = new TableColumn (table, SWT.LEFT, 1);
			nameCol.setText ("Field");
			typeCol.setText ("Type");
			typeCol.setText ("Type");
			TableLayout layout = new TableLayout ();
			layout.addColumnData (new ColumnWeightData (33, false));
			layout.addColumnData (new ColumnWeightData (33, false));
			layout.addColumnData (new ColumnWeightData (34, false));
			table.setLayout (layout);
			_fieldViewer.setContentProvider (new RecordFieldContentProvider ());
			_fieldViewer.setLabelProvider (new RecordFieldLabelProvider ());
			_fieldViewer.setCellEditors (getCellEditors (table));
			_fieldViewer.setCellModifier (new RecordFieldModifier ());
			_fieldViewer.setColumnProperties (PROPERTIES);
			TableColumn valCol = new TableColumn (table, SWT.LEFT, 2);
			valCol.setWidth (271);
			valCol.setText ("Value");
			_fieldViewer.setColumnProperties (REC_PROPERTIES);
		}
	}

	protected void checkSubclass() {
	}

	public void dispose() {
		super.dispose ();
	}

	/**
	 * @param table
	 * @return
	 */
	private CellEditor [] getCellEditors(Table table) {
		CellEditor [] editors = new CellEditor [3];
		editors [0] = new Utils.AutoApplyTextCellEditor (table);
		editors [1] = null; //new AutoApplyEditableComboBoxCellEditor (table,
							// ModifyPropertyDialog.getAvailablePropertyTypeNames
							// (_scope));
		editors [2] = new PropertyValueCellEditor (table);
		return editors;
	}

	public AcmeRecord getRecord() {
		return _record;
	}

	public void setInput(AcmeRecord input, Scope scope) {
		if (input.getFields ().getSize () == 0) {
			RecordType type = (RecordType )input.getType ();
			_record = new AcmeRecord ();
			_record.setType (type);
			AcmeEnumeration fields = type.getFields ();
			while (fields.moreItems ()) {
				AcmeRecordField field = (AcmeRecordField )fields.getNextItem ();
				_record.addField (field.copy ());
			}
		}
		else
			_record = (AcmeRecord )input.copy ();
		_fieldViewer.setInput (_record);
		if (_record.getType () == null) _hasNoType = true;
	}
}
