/*
 * Created on Feb 22, 2004
 *  
 */
package AcmeStudio.ui.views.parts.propertyEditors;

import java.util.Iterator;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import acmeLib.AcmeType;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.properties.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
/**
 * @author Bradley Schmerl Created Feb 22, 2004
 */
public class SequenceValueEditor extends Composite {

	class ContentProvider implements IStructuredContentProvider {

		public void dispose() {
		}

		public Object [] getElements(Object inputElement) {
			AcmeSequence seq = (AcmeSequence )inputElement;
			AcmeEnumeration vals = seq.getValues ();
			int extra = _editable ? 1 : 0;
			Object [] elements = new Object [vals.getSize () + extra];
			int index = 0;
			while (vals.moreItems ()) {
				AcmePropertyValue val = (AcmePropertyValue )vals.getNextItem ();
				SequenceElement element = new SequenceElement (val, index);
				elements [index++] = element;
			}
			if (_editable) {
				elements [index] = new SequenceElement (generateDummyEntry (_constituentType), index);
			}
			return elements;
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	private class SequenceElement {
		public int					index	= -1;

		public AcmePropertyValue	value;

		public SequenceElement(AcmePropertyValue val, int i) {
			value = val;
			index = i;
		}
	}

	/**
	 * @author Bradley Schmerl Created Feb 20, 2004
	 */
	public class SequenceElementModifier implements ICellModifier {

		/**
		 *  
		 */
		public SequenceElementModifier() {
			super ();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
		 *      java.lang.String)
		 */
		public boolean canModify(Object element, String property) {
			if (property.equals ("value"))
				return true;
			else
				return false;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
		 *      java.lang.String)
		 */
		public Object getValue(Object element, String property) {
			if (element instanceof SequenceElement && property.equals ("value")) {
				return ((SequenceElement )element).value;
			}
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
		 *      java.lang.String, java.lang.Object)
		 */
		public void modify(Object element, String property, Object value) {
			if (element instanceof TableItem) {
				TableItem item = (TableItem )element;
				SequenceElement val = (SequenceElement )item.getData ();
				AcmePropertyValue pValue = (AcmePropertyValue )value;
				int index = val.index;
				if (property.equals ("value")) {
					if (isPlaceholder (val.value)) {
						_seq.addValue (pValue.copy ());
					}
					else {
						_seq.removeValueAt (index);
						_seq.insertValueAt (pValue, index);
					}
				}
				_valueViewer.refresh ();
			}
		}
		/**
		 * @param val
		 * @return
		 */
	}

	class TableLabelProvider extends LabelProvider implements ITableLabelProvider {

		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		public String getColumnText(Object element, int columnIndex) {
			SequenceElement el = (SequenceElement )element;
			if (columnIndex == 0) {
				return new Integer (el.index).toString ();
			}
			if (columnIndex == 1) {
				if (!isPlaceholder (el.value)) {
					return el.value.toString ();
				}
				else
					return "<Enter a value>";
			}
			return "";
		}
	}
	private static final AcmePropertyValue	DUMMY_BOOLEAN	= new AcmeBoolean (false);
	private static final AcmePropertyValue	DUMMY_FLOAT		= new AcmeFloat (0);
	private static final AcmePropertyValue	DUMMY_INT		= new AcmeInt (0);
	private static final AcmePropertyValue	DUMMY_STRING	= new AcmeString ("");
	private AcmeType						_constituentType;
	private boolean							_editable		= true;
	private AcmeSequence					_seq;
	private TableViewer						_valueViewer;
	private AcmeEnum						DUMMY_ENUM;
	private AcmeRecord						DUMMY_RECORD;
	private AcmeSequence					DUMMY_SEQUENCE;
	private AcmeSet							DUMMY_SET;

	public SequenceValueEditor(Composite parent, int style) {
		super (parent, style);
		final GridLayout gridLayout = new GridLayout (2, false);
		gridLayout.numColumns = 2;
		setLayout (gridLayout);
		{
			_valueViewer = new TableViewer (this, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);
			_valueViewer.setLabelProvider (new TableLabelProvider ());
			final Table table = _valueViewer.getTable ();
			table.setHeaderVisible (true);
			table.setLayoutData (new GridData (GridData.FILL_BOTH));
			table.addKeyListener (new KeyAdapter () {

				public void keyPressed(KeyEvent e) {
					if (e.character == SWT.DEL) {
						deleteSelectedSetElements ();
					}
				}
			});
			_valueViewer.setContentProvider (new ContentProvider ());
			_valueViewer.setCellEditors (getCellEditors (table));
			_valueViewer.setCellModifier (new SequenceElementModifier ());
			_valueViewer.setColumnProperties (new String []{"index", "value"});
			table.setLinesVisible (true);
			TableColumn indexCol = new TableColumn (table, SWT.LEFT, 0);
			indexCol.setText ("Idx");
			{
				final TableColumn tableColumn = new TableColumn (table, SWT.NONE);
				tableColumn.setText ("Value");
			}
			TableLayout layout = new TableLayout ();
			layout.addColumnData (new ColumnWeightData (10, false));
			layout.addColumnData (new ColumnWeightData (90, false));
			table.setLayout (layout);
		}
		{
			final Composite composite = new Composite (this, SWT.NONE);
			composite.setLayout (new GridLayout ());
			{
				final Button _btnUp = new Button (composite, SWT.ARROW);
				_btnUp.addSelectionListener (new SelectionAdapter () {

					public void widgetSelected(SelectionEvent e) {
						IStructuredSelection sel = (IStructuredSelection )_valueViewer.getSelection ();
						for (Iterator iter = sel.iterator (); iter.hasNext ();) {
							SequenceElement element = (SequenceElement )iter.next ();
							if (!isPlaceholder (element.value)) {
								if (element.index > 0) {
									_seq.removeValueAt (element.index);
									_seq.insertValueAt (element.value, element.index - 1);
									_valueViewer.refresh ();
								}
							}
						}
					}
				});
				_btnUp.setToolTipText ("Move selected elements forward in sequence");
			}
			{
				final Button _btnDown = new Button (composite, SWT.ARROW | SWT.DOWN);
				_btnDown.addSelectionListener (new SelectionAdapter () {

					public void widgetSelected(SelectionEvent e) {
						IStructuredSelection sel = (IStructuredSelection )_valueViewer.getSelection ();
						for (Iterator iter = sel.iterator (); iter.hasNext ();) {
							SequenceElement element = (SequenceElement )iter.next ();
							if (!isPlaceholder (element.value)) {
								if (element.index < _seq.getSize () - 1) {
									_seq.removeValueAt (element.index);
									_seq.insertValueAt (element.value, element.index + 1);
									_valueViewer.refresh ();
								}
							}
						}
					}
				});
				_btnDown.setToolTipText ("Move selected elements back in sequence");
			}
		}
	}

	protected void checkSubclass() {
	}

	/**
	 *  
	 */
	protected void deleteSelectedSetElements() {
		IStructuredSelection sel = (IStructuredSelection )_valueViewer.getSelection ();
		for (Iterator iter = sel.iterator (); iter.hasNext ();) {
			SequenceElement val = (SequenceElement )iter.next ();
			_seq.removeValueAt (val.index);
		}
		_valueViewer.refresh ();
	}

	public void dispose() {
		super.dispose ();
	}

	public AcmePropertyValue generateDummyEntry(AcmeType basedOn) {
		AcmePropertyValue value = null;
		if (basedOn instanceof IntType) {
			value = DUMMY_INT;
		}
		else if (basedOn instanceof FloatType) {
			value = DUMMY_FLOAT;
		}
		else if (basedOn instanceof StringType) {
			value = DUMMY_STRING;
		}
		else if (basedOn instanceof BooleanType) {
			value = DUMMY_BOOLEAN;
		}
		else if (basedOn instanceof EnumType) {
			DUMMY_ENUM = new AcmeEnum ("???");
			DUMMY_ENUM.setType ((EnumType )basedOn);
			value = DUMMY_ENUM;
		}
		else if (basedOn instanceof RecordType) {
			DUMMY_RECORD = new AcmeRecord ();
			DUMMY_RECORD.setType ((RecordType )basedOn);
			value = DUMMY_RECORD;
		}
		else if (basedOn instanceof SetType) {
			DUMMY_SET = new AcmeSet ();
			DUMMY_SET.setType ((SetType )basedOn);
			value = DUMMY_SET;
		}
		else if (basedOn instanceof SequenceType) {
			DUMMY_SEQUENCE = new AcmeSequence ();
			DUMMY_SEQUENCE.setType ((SequenceType )basedOn);
			value = DUMMY_SEQUENCE;
		}
		return value;
	}

	/**
	 * @return
	 */
	private CellEditor [] getCellEditors(Table table) {
		return new CellEditor []{null, new PropertyValueCellEditor (table)};
	}

	public AcmeSequence getSeq() {
		return _seq;
	}

	private boolean isPlaceholder(AcmePropertyValue val) {
		return val == DUMMY_SEQUENCE || val == DUMMY_INT || val == DUMMY_FLOAT || val == DUMMY_STRING || val == DUMMY_BOOLEAN
				|| val == DUMMY_ENUM || val == DUMMY_RECORD || val == DUMMY_SET;
	}

	/**
	 * @param sequence
	 */
	public void setInput(AcmeSequence seq, AcmePropertyType type) {
		if (seq != null) 
			_seq = (AcmeSequence )seq.copy ();
		else
			_seq = new AcmeSequence ();
		if (type instanceof SequenceType) {
			_constituentType = ((SequenceType )type).getSequenceType ();
		}
		_valueViewer.setInput (_seq);
	}
}
