/*
 * Created on Feb 20, 2004
 *  
 */
package AcmeStudio.ui.views.parts.propertyEditors;

import java.util.Iterator;

import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import acmeLib.AcmeType;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.properties.*;
/**
 * @author Bradley Schmerl Created Feb 20, 2004
 */
public class SetValueEditor extends Composite {

	class ContentProvider implements IStructuredContentProvider {

		public void dispose() {
		}

		public Object [] getElements(Object inputElement) {
			AcmeSet set = (AcmeSet )inputElement;
			AcmeEnumeration vals = set.getValues ();
			int extra = _editable ? 1 : 0;
			Object [] elements = new Object [vals.getSize () + extra];
			int index = 0;
			while (vals.moreItems ()) {
				AcmePropertyValue val = (AcmePropertyValue )vals.getNextItem ();
				elements [index++] = val;
			}
			if (_editable) {
				elements [index] = generateDummyEntry(_constituentType);
			}
			return elements;
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	/**
	 * @author Bradley Schmerl Created Feb 20, 2004
	 */
	public class SetElementModifier implements ICellModifier {

		/**
		 *  
		 */
		public SetElementModifier() {
			super ();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
		 *      java.lang.String)
		 */
		public boolean canModify(Object element, String property) {
			if (property.equals ("value"))
				return true;
			else
				return false;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
		 *      java.lang.String)
		 */
		public Object getValue(Object element, String property) {
			if (element instanceof AcmePropertyValue && property.equals ("value")) {
				return (AcmePropertyValue )element;
			}
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
		 *      java.lang.String, java.lang.Object)
		 */
		public void modify(Object element, String property, Object value) {
			if (element instanceof TableItem) {
				TableItem item = (TableItem )element;
				AcmePropertyValue val = (AcmePropertyValue )item.getData ();
				if (property.equals ("value")) {
					if (isPlaceholder (val)) {
						_set.addValue(((AcmePropertyValue )value).copy());
					}
					else {
						_set.removeValue (val);
						_set.addValue ((AcmePropertyValue )value);
					}
				}
				_valueViewer.refresh ();
			}
		}

		/**
		 * @param val
		 * @return
		 */
	}

	class TableLabelProvider extends LabelProvider implements ITableLabelProvider {

		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		public String getColumnText(Object element, int columnIndex) {
			if (columnIndex == 0 && !isPlaceholder ((AcmePropertyValue )element)) {
				return element.toString ();
			}
			return "<Enter a value>";
		}
	}
	private static final AcmePropertyValue	DUMMY_BOOLEAN	= new AcmeBoolean (false);
	private static final AcmePropertyValue	DUMMY_FLOAT		= new AcmeFloat (0);
	private static final AcmePropertyValue	DUMMY_INT		= new AcmeInt (0);
	private static final AcmePropertyValue	DUMMY_STRING	= new AcmeString ("");
	private AcmeType	_constituentType;
	private boolean				_editable = true;
	private AcmeSet	_set;
	private TableViewer			_valueViewer;
	private AcmeEnum						DUMMY_ENUM;
	private AcmeRecord						DUMMY_RECORD;
	private AcmeSequence					DUMMY_SEQUENCE;
	private AcmeSet							DUMMY_SET;

	public SetValueEditor(Composite parent, int style) {
		super (parent, style);
		setLayout (new FillLayout ());
		{
			_valueViewer = new TableViewer (this, SWT.BORDER);
			_valueViewer.setLabelProvider (new TableLabelProvider ());
			final Table table = _valueViewer.getTable ();
			table.addKeyListener (new KeyAdapter () {

				public void keyPressed(KeyEvent e) {
					if (e.character == SWT.DEL) {
						deleteSelectedSetElements ();
					}
				}
			});
			_valueViewer.setContentProvider (new ContentProvider ());
			_valueViewer.setCellEditors (getCellEditors (table));
			_valueViewer.setCellModifier (new SetElementModifier ());
			_valueViewer.setColumnProperties (new String []{"value"});
			
			table.setLinesVisible(true);
			TableColumn col = new TableColumn (table, SWT.LEFT, 0);
			TableLayout layout = new TableLayout ();
			layout.addColumnData (new ColumnWeightData (100, false));
			table.setLayout (layout);
		}
	}

	protected void checkSubclass() {
	}

	/**
	 *  
	 */
	protected void deleteSelectedSetElements() {
		IStructuredSelection sel = (IStructuredSelection )_valueViewer.getSelection();
		for (Iterator iter = sel.iterator (); iter.hasNext ();) {
			AcmePropertyValue val = (AcmePropertyValue )iter.next ();
			_set.removeValue(val);
		}
		_valueViewer.refresh ();
	}

	public void dispose() {
		super.dispose ();
	}
	
	public AcmePropertyValue generateDummyEntry(AcmeType basedOn) {
		AcmePropertyValue value = null;
		if (basedOn instanceof IntType) {
			value = DUMMY_INT;
		} else if (basedOn instanceof FloatType) {
			value = DUMMY_FLOAT;
		} else if (basedOn instanceof StringType) {
			value = DUMMY_STRING;
		} else if (basedOn instanceof BooleanType) {
			value = DUMMY_BOOLEAN;
		} else if (basedOn instanceof EnumType) {
			DUMMY_ENUM = new AcmeEnum ("???");
			DUMMY_ENUM.setType ((EnumType )basedOn);
			value = DUMMY_ENUM;
		} else if (basedOn instanceof RecordType) {
			DUMMY_RECORD = new AcmeRecord ();
			DUMMY_RECORD.setType ((RecordType )basedOn);
			value = DUMMY_RECORD;
		} else if (basedOn instanceof SetType) {
			DUMMY_SET = new AcmeSet ();
			DUMMY_SET.setType ((SetType )basedOn);
			value = DUMMY_SET;
		}
		else if (basedOn instanceof SequenceType) {
			DUMMY_SEQUENCE = new AcmeSequence ();
			DUMMY_SEQUENCE.setType ((SequenceType )basedOn);
			value = DUMMY_SEQUENCE;
		}
		return value;
	}

	/**
	 * @return
	 */
	private CellEditor [] getCellEditors(Table table) {
		return new CellEditor []{new PropertyValueCellEditor (table)};
	}
	
	public AcmeSet getSet () {
		return _set;
	}

		private boolean isPlaceholder(AcmePropertyValue val) {
			return val == DUMMY_SEQUENCE || val == DUMMY_INT || val == DUMMY_FLOAT || val == DUMMY_STRING || val == DUMMY_BOOLEAN || val == DUMMY_ENUM || val == DUMMY_RECORD || val == DUMMY_SET;
		}

	/**
	 * @param sequence
	 */
	public void setInput(AcmeSet set, AcmePropertyType type) {
		if (set != null) 
			_set = (AcmeSet )set.copy ();
		else 
			_set = new AcmeSet ();
		if (type instanceof SetType) {
			_constituentType = ((SetType )type).getSetType();
		}
		_valueViewer.setInput(_set);
	}
}
