/*
 * Created on Oct 1, 2003
 *
 */
package AcmeStudio.ui.views.parts.wizards.functions;

import java.io.ByteArrayInputStream;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TreeItem;

import AcmeStudio.ui.views.parts.DesignRuleBrowser;
import AcmeStudio.ui.views.parts.Messages;
import AcmeStudio.ui.widgets.AcmeErrorTree;
import AcmeStudio.util.DesignRuleHolder;
import AcmeStudio.util.Utils;
import acmeLib.AcmeObject;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.designRules.AcmeResolutionException;
import acmeLib.designRules.DesignRule;
import acmeLib.designRules.ExpressionNode;
import acmeLib.elements.AcmeElement;
import acmeLib.errorManagement.AcmeError;
import acmeLib.errorManagement.AcmeErrorManager;
import acmeLib.errorManagement.AcmeParseError;
import acmeLib.typeManagement.AcmeTypeManager;

/**
 * Page to allow user to define the rule
 * @author Bradley Schmerl Created Oct 1, 2003 */
public class NewFunctionRulePage extends WizardPage {

	/** The parent wizard **/
	private NewFunctionWizard _parent;
	/** The expression for the rule of the function **/
	private ExpressionNode _designExpression;
	/** Any errors in the rule **/
	private AcmeErrorTree _errorText;
	/** The widget to allow rule editing **/
	private DesignRuleBrowser _designRuleEditor;
	/**
	 * Constructs the class
	 * @param parentWizard The parent
	 * @param ns The namespace
	 */
	public NewFunctionRulePage(NewFunctionWizard parentWizard, AcmeElement ns) {
		super("Define function rule");
		_parent = parentWizard;
		_designExpression = null;
	}

	/*
	 * (non-Javadoc) @see
	 * org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite top = new Composite(parent, SWT.NONE);
		top.setSize(500, 500);
		top.setLayout(new GridLayout(1, true));

		//		Label prototype = new Label (top, SWT.NONE);
		//		String pt = generateSignature();
		//		prototype.setText(pt);
		_designRuleEditor = new DesignRuleBrowser(top);
		GridData drgd = new GridData();
		drgd.horizontalAlignment = GridData.HORIZONTAL_ALIGN_FILL;
		drgd.heightHint = 150;
		drgd.widthHint = 500;
		_designRuleEditor.setLayoutData(drgd);
		_designRuleEditor.navigateTo(Utils.EMPTY_RULE);

		Button parseButton = new Button(top, SWT.PUSH);
		parseButton.setText(Messages.getString("ModifyRuleDialog.strParse"));
		parseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				parseRule();
			}
		});

		Label errorLabel = new Label(top, SWT.LEFT);
		errorLabel.setText(Messages.getString("ModifyRuleDialog.strParseErrors"));
		_errorText = new AcmeErrorTree(top);
		_errorText.getTree().addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent evt) {
				if (evt.item instanceof TreeItem) {
					TreeItem item = (TreeItem) evt.item;
					AcmeError e = (AcmeError) item.getData();
					Object source = e.getSource();
					_designRuleEditor.setErrorHighlight((AcmeObject) source);
				}
			}
		});
		GridData etgd = new GridData(GridData.FILL_HORIZONTAL);
		etgd.heightHint = 100;
		etgd.widthHint = 500;
		_errorText.getTree().setLayoutData(etgd);
		setPageComplete(false);

		setTitle("Enter the rule for this analysis");
		setControl(top);
	}

	/**
	 * Parses the rule in the rule editor
	 */
	protected void parseRule() {
		String ruleToParse = _designRuleEditor.getText();
		if (!_parent.getFunctionHeader().getIsExternalAnalysis()) {
			parseDesignRule(ruleToParse);
		}
		else {
			parseJavaMethodCall(ruleToParse);
		}
	}

	/**
	 * Parses the rule as a Java method call (because the analysis is external)
	 * @param ruleToParse 
	 * */
	private void parseJavaMethodCall(String ruleToParse) {
		ByteArrayInputStream bis = new ByteArrayInputStream(ruleToParse.getBytes());
		ArmaniParser.ReInit(bis);
		AcmeErrorManager.clearAllErrors();
		_designExpression = null;
		try {
			ArmaniParser.JavaMethodCallExpr(_parent.getFunctionHeader());
			_designExpression = (ExpressionNode) ArmaniParser.getJJTree().peekNode();
			_designExpression.resolveTypeReferences();
			_designExpression.resolveInstanceReferences();
		}
		catch (AcmeResolutionException e) {
			_designRuleEditor.navigateTo(_designExpression);
			AcmeVector errEnum = new AcmeVector();
			errEnum.addItem(e.getError());
			_errorText.setErrorSource(errEnum.enumerate());
			setPageComplete(false);
			AcmeTypeManager.typecheckingErrorStack.disconnect();

		}
		catch (ParseException e) {
			_designRuleEditor.navigateTo(_designExpression);
			AcmeVector errEnum = new AcmeVector();
			errEnum.addItem(new AcmeParseError(e));
			_errorText.setErrorSource(errEnum.enumerate());
			setPageComplete(false);
			AcmeTypeManager.typecheckingErrorStack.disconnect();
		}
		if (_designExpression.typecheck()) {
			_errorText.setErrorSource (null);
			setPageComplete (true);
			_designRuleEditor.navigateTo (_designExpression);
		}
		else {
			_designRuleEditor.navigateTo (_designExpression);
			AcmeVector errors = AcmeTypeManager.typecheckingErrorStack.getCausalErrors();
			AcmeEnumeration errEnum = errors.enumerate ();
			_errorText.setErrorSource (errEnum);
			AcmeTypeManager.typecheckingErrorStack.disconnect();
		}

	}

	/**
	 * Parses the rule as an armani expression
	 * @param ruleToParse The rule to parse
	 */
	private void parseDesignRule(String ruleToParse) {
		DesignRuleHolder holder = new DesignRuleHolder();
		holder.parseRuleExpression(_parent.getFunctionHeader(), ruleToParse);
		if (holder.hasErrors()) {
			_errorText.setErrorSource(holder.getError());
			setPageComplete(false);
		}
		else {
			DesignRule dr = holder.getDesignRule();
			dr.setDesignRuleScope(_parent.getFunctionHeader());
			_designExpression = dr.getDesignRuleExpression();
			try {
				dr.resolveTypeReferences();
				dr.resolveInstanceReferences();
			}
			catch (AcmeResolutionException e) {
				_designRuleEditor.navigateTo(_designExpression);
				AcmeVector errEnum = new AcmeVector();
				errEnum.addItem(e.getError());
				_errorText.setErrorSource(errEnum.enumerate());
				setPageComplete(false);
				AcmeTypeManager.typecheckingErrorStack.disconnect();
			}
			if (_designExpression.typecheck()) {
				_errorText.setErrorSource(null);
				setPageComplete(true);
				_designRuleEditor.navigateTo(_designExpression);
			}
			else {
				_designRuleEditor.navigateTo(_designExpression);
				AcmeVector errors = AcmeTypeManager.typecheckingErrorStack.getCausalErrors();
				AcmeEnumeration errEnum = errors.enumerate();
				_errorText.setErrorSource(errEnum);
				AcmeTypeManager.typecheckingErrorStack.disconnect();
			}
		}
	}

	/**
	 * @return Returns the designRule. */
	public ExpressionNode getDesignExpression() {
		return _designExpression;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.DialogPage#setMessage(java.lang.String)
	 */
	public void setMessage(String newMessage) {
		newMessage += ". Press \"Parse\" to parse the rule.";
		super.setMessage (newMessage);
	}
}
