/*
 * Created on Sep 30, 2003
 *
 */
package AcmeStudio.ui.views.parts.wizards.functions;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Shell;

import acmeLib.designRules.DesignAnalysis;
import acmeLib.designRules.DesignRuleParameter;
import acmeLib.elements.AcmeElement;

/**
 * Wizard to allow the definition of new design analyese
 * 
 * @author Bradley Schmerl Created Sep 30, 2003
 */
public class NewFunctionWizard extends Wizard {

	/** Where the design analysis will be added */
	private AcmeElement _namespace;
	/** The page to define the signature */
	private NewFunctionSignaturePage _sigPage;
	/** The page to define the rule */
	private NewFunctionRulePage _rulePage;
	/** The new analysis */
	private DesignAnalysis _analysis;

	/**
	 * Constructs the wizard
	 * 
	 * @param parentShell
	 *           Not used
	 * @param ns
	 *           The namespace where the new analysis will be added
	 */
	public NewFunctionWizard(Shell parentShell, AcmeElement ns) {
		_namespace = ns;
	}

	/* (non-Javadoc) @see org.eclipse.jface.wizard.IWizard#addPages() */
	public void addPages() {
		_sigPage = new NewFunctionSignaturePage(this, _namespace);
		_rulePage = new NewFunctionRulePage(this, _namespace);
		addPage(_sigPage);
		addPage(_rulePage);
	}

	/* (non-Javadoc) @see org.eclipse.jface.wizard.IWizard#performFinish() */
	public boolean performFinish() {
		// define the new analysis
		_analysis = _sigPage.getNewFunction();
		_analysis.setScope(_namespace);
		_analysis.setDesignRuleExpression(_rulePage.getDesignExpression());
		return true;
	}

	/**
	 * @return The new design analysis */
	public DesignAnalysis getFunctionHeader() {
		return _sigPage.getNewFunction();
	}

	/**
	 * Generates the Acme signature for the analysis
	 * 
	 * @return The signature
	 */
	public String generateSignature() {
		String pt = "Function signature: ";
		pt += getFunctionHeader().getName();
		pt += " (";
		Vector params = getFunctionHeader().getFormalParams();
		for (Iterator iter = params.iterator(); iter.hasNext();) {
			DesignRuleParameter param = (DesignRuleParameter) iter.next();
			pt += param.getName() + " : " + param.getType().getTypeName();
			if (iter.hasNext())
				pt += ", ";
		}
		pt += ") : " + getFunctionHeader().getReturnType().getTypeName();
		return pt;
	}

	/**
	 * Sets the title of all the pages
	 *
	 */
	public void setPageTitles() {
		String sig = generateSignature();
		_sigPage.setMessage (sig);
		_rulePage.setMessage(sig);
	}
	/**
	 * @return Returns the analysis. */
	public DesignAnalysis getAnalysis() {
		return _analysis;
	}

}
