/*
 * Created on Oct 2, 2003
 *
 */
package AcmeStudio.ui.views.parts.wizards.newTypes;

import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import AcmeStudio.ui.views.parts.StructureList;
import acmeLib.elements.*;

/**
 * Allows the user to define new default structure
 * @author Bradley Schmerl
 * Created Oct 2, 2003
 */
public class StructurePage extends WizardPage {

	/** The list containing all the new structure **/
	private StructureList _structure;
	/** The button to remove substructure **/
	private Button _btnRemove;
	/** The button to add substructure **/
	private Button _btnNew;
	/** The new type **/
	protected AcmeElementType _protoType;

	/**
	 * Constructs the page
	 * @param family The family that the new type will be added to
	 */
	public StructurePage(AcmeFamily family) {
		super("Default structure");
	}


	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite top = new Composite (parent, SWT.NONE);
		setControl (top);
		
		// The structure browser
		top.setLayout (new GridLayout (2, false));
		_structure = new StructureList ();
		_structure.createControl (top, SWT.BORDER, null);
		
		GridData pgd = new GridData();
		pgd.verticalSpan = GridData.FILL;
		pgd.verticalAlignment = GridData.FILL;
		pgd.horizontalAlignment = GridData.FILL;
		pgd.grabExcessHorizontalSpace = true;
		_structure.getComposite().setLayoutData(pgd);

		// Actions for creating and removing structure
		_btnNew = new Button(top, SWT.PUSH);
		_btnNew.setText("New Port...");
		_btnNew.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (_protoType instanceof AcmeComponentType) {
					((ActionContributionItem) _structure.getContributions()[StructureList.NEWPORT]).getAction().run();
				}
				else if (_protoType instanceof AcmeConnectorType) {
					((ActionContributionItem) _structure.getContributions()[StructureList.NEWROLE]).getAction().run();
				}
			}
		});

		_btnRemove = new Button(top, SWT.PUSH);
		_btnRemove.setText("Remove");
		_btnRemove.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				((ActionContributionItem) _structure.getContributions()[StructureList.DELELEMENT]).getAction().run();
			}
		});
		setTitle ("Enter default ports");
	}

	/**
	 * @param protoType The protoType to set.
	 */
	public void setProtoType(AcmeElementType protoType) {
		_protoType = protoType;
		_structure.setInput((AcmeElement )protoType);
		if (_protoType instanceof AcmeComponentType) {
			setTitle ("Enter ports for this type");
			_btnNew.setText ("New Port...");
		}
		else if (_protoType instanceof AcmeConnectorType) {
			setTitle ("Enter roles for this type");
			_btnNew.setText ("New Role...");
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage() {
		IWizardPage next =  super.getNextPage();
		if (next instanceof PropertyPage) {
			PropertyPage pp = (PropertyPage) next;
			pp.setPrototype(_protoType);
		}
		return next;
	}

}
