/*
 * Created on May 16, 2005
 *
 */
package AcmeStudio.ui.views.parts.wizards.weaveFamily;

import java.util.Iterator;
import java.util.SortedMap;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import acmeLib.elements.AcmeElement;
import AcmeStudio.core.ASModel;
import AcmeStudio.core.resource.ASFamily;
import AcmeStudio.core.resource.ASPath;


/**
 * @author Bradley Schmerl
 * Created May 16, 2005
 */
public class ChooseFamilyWizardPage extends WizardPage {

	private Button _refreshButton;
	private Tree _availableFamilies;
	protected ASFamily	_asFamily;
	private final WeaveFamilyWizard	_parentWizard;
	private final AcmeElement	_systemToApplyTo;
	public ChooseFamilyWizardPage(WeaveFamilyWizard wizard, AcmeElement system) {
		super ("wizardPage");
		_parentWizard = wizard;
		_systemToApplyTo = system;
		setTitle ("Select Family");
		setDescription ("Choose a family to mix in to this system.");
		
	}

	public void createControl(Composite parent) {
		Composite container = new Composite (parent, SWT.NULL);
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 1;
		container.setLayout(gridLayout);
		//
		setControl (container);
		{
			Label label = new Label(container, SWT.NONE);
			label.setText("Available families:");
		}
		{
			_availableFamilies = new Tree(container, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
			_availableFamilies.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					if (e.item instanceof TreeItem) {
						TreeItem selectedFamily = (TreeItem )e.item;
						if (e.item.getData () instanceof java.util.List) {
							_asFamily = (ASFamily )((java.util.List )selectedFamily.getData ()).get (0);
						}
						else if (e.item.getData () instanceof ASFamily) {
							_asFamily = (ASFamily )e.item.getData ();
						}
					}
					else {
						_asFamily = null;
					}
					setPageComplete(_asFamily != null);
					((DefinePointCutPage )_parentWizard.getNextPage(ChooseFamilyWizardPage.this)).setSelectedFamily(_asFamily.getFamily());
				}
			});
			_availableFamilies.setLayoutData(new GridData(GridData.FILL_BOTH));
		}
		{
			_refreshButton = new Button(container, SWT.NONE);
			_refreshButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					ASModel.familyRegistry().refresh ();
					fillFamilies ();
				}
			});
			_refreshButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER));
			_refreshButton.setText("Refresh Families");
		}
		setPageComplete (_asFamily != null);
	}
	
    private void fillFamilies() {
        _availableFamilies.removeAll();
        SortedMap families = ASModel.familyRegistry().getByName();
        // (family -> List<Path>)
        Iterator i = families.keySet().iterator();
        while (i.hasNext()) {
            Object o = i.next();
            String name = (String)o;
            java.util.List paths = (java.util.List)families.get(name);
            TreeItem nameItem = new TreeItem(_availableFamilies, SWT.NONE);
            nameItem.setText(name);
            nameItem.setData(paths);
            if (paths.size() > 1) {
                Iterator pI = paths.iterator();
                while (pI.hasNext()) {
                    Object fam = pI.next();
                    if (fam instanceof ASFamily) {
                        ASFamily path = (ASFamily)fam;
                        TreeItem pathItem = new TreeItem(nameItem, SWT.NONE);
                        if (path.getASPath().getPathLevel() == ASPath.GLOBAL_LEVEL) {
                            pathItem.setText("Built In Version (" + path.getASPath().getPathStr() + ")");
                        }
                        else {
                            pathItem.setText("Local Version (" + path.getASPath().getPathStr() + ")");
                        }
                        pathItem.setData(path);
                    }
                }
            }

        }
    }
	public ASFamily getFamily() {
		return _asFamily;
	}
}
