/*
 * Created on May 19, 2005
 *
 */
package AcmeStudio.ui.views.parts.wizards.weaveFamily;

import java.util.ArrayList;
import java.util.Collection;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.jface.viewers.*;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.internal.ide.StringMatcher;
import AcmeStudio.util.BufferedCanvas;
import AcmeStudio.util.Constants;
import AcmeStudio.util.Helper;
import acmeLib.AcmeObject;
import acmeLib.AcmeType;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.filters.*;
import acmeLib.elements.*;
import acmeLib.properties.AcmePropertyType;


/**
 * @author Bradley Schmerl
 * Created May 19, 2005
 */
public class DefinePointCutPage extends WizardPage {
	
	
	private Button _btnApplyToNone;
	private TableColumn _firstColumn;
	private Composite _instructionComposite;
	private Canvas _centerCanvas;
	class AcmeTypeLabelProvider extends LabelProvider {
		public String getText(Object element) {
			if (element instanceof AcmeElementType) {
				return ((AcmeElementType )element).getTypeName();
			}
			return "";
		}
		public Image getImage(Object element) {
			return null;
		}
	}
	class AcmeTypeContentProvider implements IStructuredContentProvider {
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof java.util.List) {
				java.util.List list = (java.util.List )inputElement;
				return list.toArray ();
			}
			return Constants.EMPTY_ELEMENTS;
		}
		public void dispose() {
		}
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
	private Button _btnApplyToSelected;
	class InstructionLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object element, int columnIndex) {
			if (element instanceof WeavingInstruction) {
				WeavingInstruction wi = (WeavingInstruction )element;
				String category = "";
				if (wi.getType () instanceof AcmeComponentType) category = "(Component Type)";
				else if (wi.getType () instanceof AcmeConnectorType) category = "(Connector Type)";
				else if (wi.getType () instanceof AcmeRoleType) category = "(Role Type)";
				else if (wi.getType () instanceof AcmePortType) category="(Port Type)";
				else if (wi.getType () instanceof AcmeElementType) category = "(Element Type)";
				return wi.getType().getTypeName() + " " + category;
			}
			return "";
		}
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}
	}
	class InstructionContentProvider implements IStructuredContentProvider {
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof java.util.List) {
				return ((java.util.List ) inputElement).toArray();
			}
			return Constants.EMPTY_ELEMENTS;
		}
		public void dispose() {
		}
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
	class PatternEnumFilter extends AcmeObjectFilter {

		private final AcmeElementType	_type;
		private final StringMatcher	_pattern;

		/**
		 * @param type
		 * @param pattern
		 */
		public PatternEnumFilter(AcmeElementType type, String pattern) {
			_type = type;
			String filterText = pattern.trim ();
			if (filterText.indexOf ('*') == -1 && filterText.indexOf ('?') == -1) filterText += "*";
			_pattern = new StringMatcher (filterText, true, false);
		}

		public boolean test(AcmeObject obj) {
			if (obj instanceof AcmeElementType) {
				AcmeElementType type = (AcmeElementType )obj;
				if (_pattern.match(type.getTypeName ())) {
					if (type instanceof AcmeComponentType && _type instanceof AcmeComponentType) 
						return true;
					else if (type instanceof AcmeConnectorType && _type instanceof AcmeConnectorType) 
						return true;
					else if (type instanceof AcmeRoleType && _type instanceof AcmeRoleType)
						return true;
					else if (type instanceof AcmePortType && _type instanceof AcmePortType) 
						return true;
					else if (_type instanceof AcmeElementType && !(_type instanceof AcmeComponentType) && !(_type instanceof AcmeConnectorType) && !(_type instanceof AcmeRoleType) && !(_type instanceof AcmePortType))
						return true;
				}
			}
			return false;
		}
		
	}
	private ListViewer _lstApplyToTypesV;
	private Label _lblApply;
	private Button _btnRemoveFromApply;
	private ListViewer _lstSelectTypesV;
	private TableViewer _tblTypesV;
	private Table _tblTypes;
	private Text _txtPattern;
	private Button _btnApplyToPattern;
	private Button _btnCopyToApply;
	private List _lstApplyToTypes;
	private Button _btnApplyToAll;
	private final WeaveFamilyWizard	_parentWizard;
	private final AcmeElement	_systemToApplyTo;
	private List	_lstSelectTypes;
	private AcmeFamily	_familyToWeave;
	private java.util.List	_typeWeavingInstructions;
	private double []	_basicCurveCenter;
	private WeavingInstruction	_currentInstruction;

	
	private final ModifyListener	_patternListener	= new ModifyListener () {

		public void modifyText(ModifyEvent e) {
			String pattern = _txtPattern.getText();
			PatternEnumFilter filter = new PatternEnumFilter (_currentInstruction.getType(), pattern);
			AcmeEnumeration availableTypes = _systemToApplyTo.getAvailableTypes().castToAcmeSetVector().enumerate(filter);
			 _currentInstruction.setSelectedTypes(Helper.convertAcmeEnumerationToList(availableTypes));
			 _currentInstruction.getApplyToTypes().clear ();
			 _currentInstruction.getApplyToTypes().addAll(_currentInstruction.getSelectedTypes());
			_lstApplyToTypesV.refresh ();
		}
		
	};
	
	public DefinePointCutPage(WeaveFamilyWizard wizard, AcmeElement system) {
		super ("wizardPage");
		_parentWizard = wizard;
		_systemToApplyTo = system;
		setTitle ("Mix-in Definition");
		setDescription ("Select types and describe how to mix them into the system");
	}

	public void createControl(Composite parent) {
		Composite container = new Composite (parent, SWT.NULL);
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		gridLayout.numColumns = 3;
		container.setLayout(gridLayout);
		setControl (container);

		_tblTypesV = new TableViewer(container, SWT.FULL_SELECTION);
		_tblTypesV.setLabelProvider(new InstructionLabelProvider());
		_tblTypesV.setContentProvider(new InstructionContentProvider());
		_tblTypes = _tblTypesV.getTable();
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_VERTICAL);
		gridData.widthHint = 250;
		_tblTypes.setLayoutData(gridData);
		_tblTypes.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected(SelectionEvent e) {
				if (e.item.getData() instanceof WeavingInstruction) {
					fillInstructionComposite ((WeavingInstruction )e.item.getData());
					_centerCanvas.redraw ();
				}
			}
		});

		_firstColumn = new TableColumn(_tblTypes, SWT.NONE);
		_firstColumn.setWidth(100);
		_firstColumn.setText("Types");
		_tblTypesV.setInput(new Object());

		_centerCanvas = new BufferedCanvas(container, SWT.NONE) {

			public void doPaint(GC gc) {
				paintCenter (this, gc);
			}
			
		};
		gridData = new GridData(GridData.FILL_VERTICAL);
		gridData.widthHint = 25;
		_centerCanvas.setLayoutData(gridData);

		_instructionComposite = new Composite(container, SWT.NONE);
		_instructionComposite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_LIST_SELECTION));
		gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		_instructionComposite.setLayout(gridLayout);

		Composite buttonComposite = new Composite(_instructionComposite, SWT.NONE);
		gridData = new GridData(GridData.GRAB_VERTICAL);
		gridData.verticalSpan = 2;
		buttonComposite.setLayoutData(gridData);
		buttonComposite.setLayout(new GridLayout());

		SelectionAdapter applicationAdapter = new SelectionAdapter () {
			

			public void widgetSelected(SelectionEvent e) {
				if (e.widget == _btnApplyToAll) {
					_txtPattern.setEnabled (false);
					_txtPattern.removeModifyListener(_patternListener);
					_txtPattern.setText ("");
					_lstSelectTypesV.setInput(Constants.EMPTY_ELEMENTS);
					_lstSelectTypes.setEnabled(false);
					_currentInstruction.setInstruction(WeavingInstruction.APPLY_TO_ALL);
					_currentInstruction.getApplyToTypes().clear ();
					_currentInstruction.getApplyToTypes ().addAll(getPossibleAvailableTypes());
				}
				else if (e.widget == _btnApplyToSelected) {
					_txtPattern.setEnabled (false);
					_txtPattern.removeModifyListener(_patternListener);
					_txtPattern.setText ("");
					_currentInstruction.getApplyToTypes().clear ();
					java.util.List avlList = getPossibleAvailableTypes ();
					_lstSelectTypesV.setInput(avlList);
					_lstSelectTypes.setEnabled(true);
					_currentInstruction.setInstruction(WeavingInstruction.APPLY_TO_SELECTED);
				}
				else if (e.widget == _btnApplyToPattern) {
					_txtPattern.setEnabled (true);
					_lstSelectTypesV.setInput(Constants.EMPTY_ELEMENTS);
					_lstSelectTypes.setEnabled(false);
					_txtPattern.addModifyListener(_patternListener);
					_currentInstruction.setInstruction (WeavingInstruction.APPLY_TO_PATTERN);
				}
				else if (e.widget == _btnApplyToNone) {
					_lstSelectTypesV.setInput(Constants.EMPTY_ELEMENTS);
					_lstSelectTypes.setEnabled(false);
					_txtPattern.setEnabled (false);
					_txtPattern.removeModifyListener(_patternListener);
					_txtPattern.setText ("");
				}
				_lstApplyToTypesV.refresh ();
			}
		};

		_btnApplyToNone = new Button(buttonComposite, SWT.RADIO);
		_btnApplyToNone.setText("Do not apply");
		_btnApplyToNone.addSelectionListener (applicationAdapter);
		
		_btnApplyToAll = new Button(buttonComposite, SWT.RADIO);
		_btnApplyToAll.setText("Apply to all components");
		_btnApplyToAll.addSelectionListener(applicationAdapter);
		
		_btnApplyToSelected = new Button(buttonComposite, SWT.RADIO);
		_btnApplyToSelected.setText("Apply to selected types...");
		_btnApplyToSelected.addSelectionListener(applicationAdapter);
		
		Composite typesComposite = new Composite(buttonComposite, SWT.NONE);
		typesComposite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL));
		gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		typesComposite.setLayout(gridLayout);

		_lstSelectTypesV = new ListViewer(typesComposite, SWT.BORDER);
		_lstSelectTypesV.setLabelProvider(new AcmeTypeLabelProvider());
		_lstSelectTypesV.setContentProvider(new AcmeTypeContentProvider());
		_lstSelectTypes = _lstSelectTypesV.getList();
		gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_FILL);
		gridData.horizontalIndent = 25;
		_lstSelectTypes.setLayoutData(gridData);
		_lstSelectTypesV.setInput(new Object());

		Composite composite_1 = new Composite(typesComposite, SWT.NONE);
		composite_1.setLayout(new GridLayout());

		_btnCopyToApply = new Button(composite_1, SWT.NONE);
		_btnCopyToApply.setText(">>");

		_btnRemoveFromApply = new Button(composite_1, SWT.NONE);
		_btnRemoveFromApply.setText("<<");

		_btnApplyToPattern = new Button(buttonComposite, SWT.RADIO);
		_btnApplyToPattern.setLayoutData(new GridData());
		_btnApplyToPattern.setText("Apply to types with the following pattern:");
		_btnApplyToPattern.addSelectionListener(applicationAdapter);
		
		_txtPattern = new Text(buttonComposite, SWT.BORDER);
		gridData = new GridData();
		gridData.horizontalIndent = 25;
		_txtPattern.setLayoutData(gridData);

		_lblApply = new Label(_instructionComposite, SWT.NONE);
		_lblApply.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_LIST_SELECTION));
		_lblApply.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));
		_lblApply.setText("Apply to components of type:");

		_lstApplyToTypesV = new ListViewer(_instructionComposite, SWT.BORDER);
		this._lstApplyToTypes = _lstApplyToTypesV.getList();
		this._lstApplyToTypes.setLayoutData(new GridData(GridData.FILL_BOTH));
		_lstApplyToTypesV.setContentProvider(new AcmeTypeContentProvider ());
		_lstApplyToTypesV.setLabelProvider(new AcmeTypeLabelProvider ());
		_lstApplyToTypesV.setInput(new Object());
//		PointCutComposite comp = new PointCutComposite (container, SWT.NONE);
//		comp.addPaintListener(new PaintListener () {
//
//			public void paintControl(PaintEvent e) {
//				comp.fillDetails (_systemToApply);
//			}
//			
//		});
	}
	
	protected void paintCenter (Canvas canvas, GC g) {
		Table table = _tblTypesV.getTable();
		_firstColumn.setWidth(table.getClientArea ().width);
		Display display = canvas.getDisplay ();
		Point size = canvas.getSize ();
		g.setBackground (canvas.getBackground()); 
		g.fillRectangle (0, 0, size.x, size.y);
		
		TableItem[] items = table.getSelection();
		if (items.length > 0) {
			Color fillColor = display.getSystemColor(SWT.COLOR_LIST_SELECTION);
			for (int i = 0; i < items.length; i++) {
				TableItem item = items [i];
				Rectangle tableBounds = table.getClientArea();
				Rectangle itemBounds = item.getBounds(0);
				Rectangle widgetBounds = _instructionComposite.getBounds();
				int [] pts = new int [8];
				pts [0] = 0;
				pts [1] = tableBounds.y + itemBounds.y;
				if (pts [1] < 0) pts [1] = 0;
				if (pts [1] > tableBounds.y + tableBounds.height) pts [1] = tableBounds.y + tableBounds.height;
				pts [2] = size.x;
				pts [3] = widgetBounds.y;
				pts [4] = size.x;
				pts [5] = widgetBounds.y + widgetBounds.height;
				pts [6] = 0;
				pts [7] = tableBounds.y + itemBounds.y + itemBounds.height;
				if (pts [7] < 0) pts [7] = 0;
				if (pts [7] > tableBounds.y + tableBounds.height) pts [7] = tableBounds.y + tableBounds.height;
				int [] topPoints = getCenterCurvePoints(pts [0], pts [1], pts [2], pts [3]);
				int [] bottomPoints = getCenterCurvePoints(pts [6], pts [7], pts [4], pts [5]);
				g.setForeground(ColorConstants.black);
				g.drawLine(0, bottomPoints [0], 0, topPoints [0]);
				for (int j = 1; j < bottomPoints.length; j++) {
					g.setForeground (fillColor);
					g.drawLine (j, bottomPoints [j], j, topPoints [j]);
//					g.setForeground(ColorConstants.black);
//					g.drawLine (j-1, topPoints [j-1], j, topPoints [j]);
//					g.drawLine (j-1, bottomPoints[j-1], j, bottomPoints [j]);
				}
				g.setBackground(fillColor);
				g.fillPolygon(pts);
			}
		}
	}
	
	private int[] getCenterCurvePoints(int startx, int starty, int endx, int endy) {
		if (_basicCurveCenter == null)
			buildBaseCenterCurve(endx-startx);
		double height= endy - starty;
		height= height/2;
		int width= endx-startx;
		int[] points= new int[width];
		for (int i= 0; i < width; i++) {
			points[i]= (int) (-height * _basicCurveCenter[i] + height + starty);
		}
		return points;
	}

	private void buildBaseCenterCurve(int w) {
		double width= w;
		_basicCurveCenter= new double[_centerCanvas.getBounds().width];
		for (int i= 0; i < _centerCanvas.getBounds().width; i++) {
			double r= i / width;
			_basicCurveCenter[i]= Math.cos(Math.PI * r);
		}
	}
	/**
	 * @param instruction
	 */
	protected void fillInstructionComposite(WeavingInstruction instruction) {
		_currentInstruction = instruction;
		switch (instruction.getInstruction ()) {
			case WeavingInstruction.APPLY_TO_ALL:
				_btnApplyToAll.setSelection (true);
				break;
			case WeavingInstruction.APPLY_TO_SELECTED:
				_btnApplyToPattern.setSelection (true);
				_lstSelectTypesV.setInput(instruction.getSelectedTypes());
				_lstApplyToTypesV.setInput (instruction.getSelectedTypes ());
//				_currentInstruction.getApplyToTypes().clear();
				break;
			case WeavingInstruction.APPLY_TO_PATTERN:
				_btnApplyToSelected.setSelection (true);
				_txtPattern.addModifyListener(_patternListener);
				_txtPattern.setText(instruction.getPattern ());
				break;
			default:
				_btnApplyToNone.setSelection (true);
		}
		_lstApplyToTypesV.setInput(instruction.getApplyToTypes());
		 
	}

	public void setSelectedFamily (AcmeFamily family) {
		_familyToWeave = family;
		AcmeEnumeration types = family.getTypes();
		_typeWeavingInstructions = new ArrayList ();
		while (types.moreItems ()) {
			AcmeType type = (AcmeType )types.getNextItem ();
			if (!(type instanceof AcmePropertyType)) {
				WeavingInstruction wi = new WeavingInstruction ((AcmeElementType )type);
				_typeWeavingInstructions.add(wi);
			}
		}
		_tblTypesV.setInput(_typeWeavingInstructions);
		
	}

	/**
	 * @return
	 */
	private java.util.List getPossibleAvailableTypes() {
		AcmeEnumeration availableTypes = _systemToApplyTo.getAvailableTypes();
		AcmeElementType type = _currentInstruction.getType ();
		if (type instanceof AcmeComponentType) {
			availableTypes = availableTypes.castToAcmeSetVector().enumerate(new ComponentTypeEnumFilter ());
		}
		else if (type instanceof AcmeConnectorType) {
			availableTypes = availableTypes.castToAcmeSetVector().enumerate (new ConnectorTypeEnumFilter ());
		}
		else if (type instanceof AcmePortType) {
			availableTypes = availableTypes.castToAcmeSetVector().enumerate (new PortTypeEnumFilter ());
		}
		else if (type instanceof AcmeRoleType) {
			availableTypes = availableTypes.castToAcmeSetVector().enumerate (new RoleTypeEnumFilter ());
		}
		java.util.List avlList = Helper.convertAcmeEnumerationToList(availableTypes);
		return avlList;
	}
	
	public Collection getWeavingInstructions () {
		return _typeWeavingInstructions;
	}
}
