/*
 * Created on May 25, 2005
 *
 */
package AcmeStudio.ui.views.parts.wizards.weaveFamily;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import AcmeStudio.model.ModelCommand;
import AcmeStudio.model.TypeAssignmentCommand;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmeSystem;
import acmeLib.tools.GraphWalker;
import acmeLib.tools.WalkerElementOperation;


/**
 * @author Bradley Schmerl
 * Created May 25, 2005
 */
public class WeaveInFamilyCommand extends ModelCommand {

	private final AcmeSystem	_system;
	private final Collection	_weavingInstructions;
	private final List _commands = new ArrayList ();

	/**
	 * @param system
	 * @param weavingInstructions
	 */
	public WeaveInFamilyCommand(AcmeSystem system, Collection weavingInstructions) {
		super ("Weave a family into a system");
		_system = system;
		_weavingInstructions = weavingInstructions;
		
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 **/
	public void execute() {
		for (Iterator wIt = _weavingInstructions.iterator (); wIt.hasNext ();) {
			final WeavingInstruction wi = (WeavingInstruction )wIt.next ();
			
			WalkerElementOperation op = new WalkerElementOperation () {

				public void doElementOp(AcmeElement elt) {
					if (Helper.getAcmeSystem(elt) != _system) return;
					switch (wi.getInstruction()) {
						case WeavingInstruction.APPLY_TO_ALL:
						case WeavingInstruction.APPLY_TO_PATTERN:
						case WeavingInstruction.APPLY_TO_SELECTED: {
							boolean applied = false;
							for (Iterator iter = wi.getApplyToTypes().iterator (); iter.hasNext () && !applied;) {
								AcmeElementType type = (AcmeElementType )iter.next ();
								if (elt.declaresType(type)) {
									applied = true;
									TypeAssignmentCommand cmd = new TypeAssignmentCommand (
											TypeAssignmentCommand.ASSIGN_TYPE,
											elt, wi.getType (), true, this);
									_commands.add (cmd);
								}
							}
						}
					}	
					
				}
			};
			
			if (wi.getInstruction() != WeavingInstruction.APPLY_TO_NONE && wi.getInstruction() != -1) {
				GraphWalker.WalkOverAllElements(_system, op, GraphWalker.WALK_TOP_DOWN);
			}
		}
		redo ();
	}
	
	public void redo() {
		for (Iterator iter = _commands.iterator (); iter.hasNext ();) {
			TypeAssignmentCommand cmd = (TypeAssignmentCommand )iter.next ();
			cmd.execute(_system.getParentDesign());
		}
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 **/
	public void undo() {
		for (Iterator iter = _commands.iterator (); iter.hasNext ();) {
			TypeAssignmentCommand cmd = (TypeAssignmentCommand )iter.next ();
			cmd.undo();
		}
	}

	
	
}
