/*
 * Created on Mar 29, 2004
 *
 */
package AcmeStudio.ui.views.ruleErrors;

import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import AcmeStudio.util.Helper;
import acmeLib.AcmeType;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.designRules.DesignAnalysis;
import acmeLib.designRules.DesignRule;
import acmeLib.designRules.DesignRuleParameter;
import acmeLib.elements.*;
import acmeLib.typeManagement.AcmeTypeManager;


/**
 * @author Bradley Schmerl
 * Created Mar 29, 2004
 */
public class RuleErrorDetailer {

	/**
	 * 
	 */
	public RuleErrorDetailer() {
		super ();
	}
	
	public static AcmeElementTree getErrorDetails (DesignRule rule, AcmeElement context) {
		RuleTransformer rt = new RuleTransformer ();
		Vector formalParams = new Vector (1);
		AcmeType type = null;
		if (context.getDeclaredTypes ().getSize () == 0) {
			if (context instanceof AcmeComponent) {
				type = ConstantTypes.COMPONENT_BASE_TYPE;
			}
			else if (context instanceof AcmeConnector) {
				type = ConstantTypes.CONNECTOR_BASE_TYPE;
			}
			else if (context instanceof AcmePort) {
				type = ConstantTypes.PORT_BASE_TYPE;
			}
			else if (context instanceof AcmeRole) {
				type = ConstantTypes.ROLE_BASE_TYPE;
			}
			else if (context instanceof AcmeGroup) {
				type = ConstantTypes.GROUP_BASE_TYPE;
			}
			else if (context instanceof AcmeSystem) {
				type = ConstantTypes.SYSTEM_BASE_TYPE;
			}
			else {
				type = ConstantTypes.ELEMENT_BASE_TYPE;
			}
		}
		else {
			type = AcmeTypeManager.inferLeastCommonSupertype(context.getDeclaredTypes());
		}
		formalParams.add (new DesignRuleParameter ("self", type));
		
		List transformed = rt.transformRule(rule.getDesignRuleExpression(), formalParams, Helper.getAcmeSystem(context));
		
		AcmeElementTree tree = new AcmeElementTree (context);
		
		Vector params = new Vector (1);
		params.add(context);
		
		getErrorTree (rt, transformed, tree, null, 0);
		return tree;
	}

	/**
	 * @param transformed
	 * @param tree
	 */
	private static void getErrorTree(RuleTransformer rt, List transformed, AcmeElementTree tree, Vector p, int level) {
		if (p == null) {
			p = new Vector (1);
		}
		DesignAnalysis da = (DesignAnalysis )transformed.get (level);
		
		Vector params = new Vector ();
		params.addAll(p);
		params.add (tree.getRootElement());
		da.setScope(tree.getRootElement());
		Set erroneous = rt.evaluateSelection(da, params);
		for (Iterator iter = erroneous.iterator (); iter.hasNext ();) {
			AcmeElement element = (AcmeElement )iter.next ();
			tree.insertChild(element);
			if (level + 1 < transformed.size()) {
				getErrorTree (rt, transformed, tree.getChildTree(element), params, level+1);
			}
		}
	}

	/**
	 * @param designRule
	 * @return
	 */
	public static boolean canDetailRule(DesignRule designRule) {
		return RuleTransformer.canTransformRule (designRule.getDesignRuleExpression());
	}
}
