/*
 * Created on Mar 17, 2004
 *
 */
package AcmeStudio.ui.views.ruleErrors;

import java.io.ByteArrayInputStream;
import java.util.*;

import AcmeStudio.util.Debug;
import acmeLib.AcmeObject;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.core.globals.ExpressionOperatorT;
import acmeLib.designRules.*;
import acmeLib.elements.AcmeSystem;
import acmeLib.errorManagement.AcmeErrorManager;


/**
 * @author Bradley Schmerl
 * Created Mar 17, 2004
 */
public class RuleTransformer {

	/**
	 * 
	 */
	public RuleTransformer() {
		super ();
	}
	
	
	public List transformRule (ExpressionNode origRule, Vector params, AcmeSystem context) {
		List functions = new ArrayList ();
		ExpressionNode rule = null;
	
		if (origRule instanceof DesignAnalysisCallNode) {
			DesignAnalysisCallNode dac = (DesignAnalysisCallNode )origRule;		
			rule = dac.getReferentDesignAnalysis().getDesignRuleExpression();
			rule = rule.cloneExpressionTree();
			
			for (int i = 0; i < dac.getActualParams().size(); i++) {
				DesignRuleParameter param = (DesignRuleParameter) dac.getReferentDesignAnalysis().getFormalParams ().get (i);
				replaceReference (rule, param, (ExpressionNode )dac.getActualParams ().get (i));
			}
			
		}
		else {
			rule = origRule.cloneExpressionTree();
		}
		SetConstructorNode select = null;
		if (rule instanceof QuantifiedExpressionNode) {
			DesignAnalysis da = new DesignAnalysis ();
			try {
				da.setName ("ErrorLevel1");
				da.setScope(context);
			}
			catch (AcmeIllegalNameException e1) {
				e1.printStackTrace();
			}
			da.setFormalParams(params);
			da.setReturnType(ConstantTypes.ANY_SET_TYPE);
			QuantifiedExpressionNode quantExp = (QuantifiedExpressionNode )rule;
			if (quantExp.getQuantOp() == QuantifiedExpressionNode.UNIVERSAL_QUANTIFICATION) {
				select = new SetConstructorNode (SetConstructorNode.SELECT);
				select.setVarName(quantExp.getVarName());
				select.setVarType (quantExp.getVarType ());
				select.setVarTypeInferred(quantExp.getVarTypeInferred());
				select.setSetExpression((ExpressionNode )quantExp.getSetExpression());
				
				BooleanExpressionNode notNode = new BooleanExpressionNode (ArmaniParserTreeConstants.JJTBOOLEANEXPRESSIONNODE);//ExpressionOperatorT.NOT_OP);
				notNode.setOperator (ExpressionOperatorT.NOT_OP);
				notNode.jjtAddChild(quantExp.getEvalExpression(), 0);
				quantExp.getEvalExpression().setInParentheses(true);
				select.setEvalExpression(notNode);
				
				da.setDesignRuleExpression(select);
				// Now, parse this rule to make sure that the expression tree is right
				ByteArrayInputStream bis = new ByteArrayInputStream (da.toString().getBytes ());
				ArmaniParser.ReInit (bis);
				AcmeErrorManager.clearAllErrors ();
				try {
					da = ArmaniParser.DesignAnalysisDeclaration(null);
					functions.add(da);
				}
				catch (ParseException e) {
					Debug.errorln (e);
				}
				
				Vector params2 = new Vector ();
				params2.addAll(params);
				DesignRuleParameter level2 = new DesignRuleParameter (quantExp.getVarName(), quantExp.getVarType ());
				params2.add(level2);
				
				functions.addAll(transformRule (quantExp.getEvalExpression(), params2, context));
			}
		}
/*		if (select != null) {
			select.resolveTypeReferences();
			select.resolveInstanceReferences();
		}*/
		return functions;
	}
	
	/**
	 * @param rule
	 * @param param
	 * @param node
	 */
	private void replaceReference(ExpressionNode rule, DesignRuleParameter param, ExpressionNode node) {
		if (rule instanceof QuantifiedExpressionNode) {
			QuantifiedExpressionNode qExp = (QuantifiedExpressionNode) rule;
			SetExpressionNode setExpr = qExp.getSetExpression();
			replaceReference ((ExpressionNode )setExpr, param, node);
			ExpressionNode evalExpr = qExp.getEvalExpression();
			replaceReference (evalExpr, param, node);
		}
		else if (rule instanceof SetReferenceNode) {
			SetReferenceNode setRef = (SetReferenceNode) rule;
			String identifierName = setRef.getIdentifier();
			String newIdentifier = replaceInQualifiedName(param, node, identifierName);
		     setRef.setIdentifier(newIdentifier);

		}
		else if (rule instanceof BooleanExpressionNode) {
			BooleanExpressionNode boolExpr = (BooleanExpressionNode) rule;
			for (int i = 0; i < boolExpr.jjtGetNumChildren(); i++) {
				replaceReference((ExpressionNode )boolExpr.jjtGetChild(i), param, node);
			}
		}
		else if (rule instanceof DesignAnalysisCallNode) {
			DesignAnalysisCallNode dac = (DesignAnalysisCallNode) rule;
			for (int i = 0; i <dac.jjtGetNumChildren(); i++) {
				replaceReference ((ExpressionNode )dac.jjtGetChild (i),param, node);
			}
		}
		else if (rule instanceof IdentifierNode) {
			IdentifierNode idNode = (IdentifierNode) rule;
			idNode.setIdentifier(replaceInQualifiedName(param, node, idNode.getIdentifier ()));
		}
	}


	private String replaceInQualifiedName(DesignRuleParameter param, ExpressionNode node, String identifierName) {
		Vector qualifiedNames = new Vector (1);
		// parse identifierName to fill in the qualifiedNames vector
		String curToken;
		int curStart = 0;
		int curEnd = 0;
		int vectorIndex = 0;
		boolean endReached = false;
		qualifiedNames.removeAllElements();

		while (!endReached) {
		    curEnd = identifierName.indexOf('.', curStart);
		    if (curEnd == -1) {
		        // then this is the last segment of the name
		        endReached = true;
		        curEnd = identifierName.length();
		    }

		    qualifiedNames.insertElementAt(
		            identifierName.substring(curStart, curEnd),
		            vectorIndex);
		    curEnd++;
		    curStart = curEnd;
		    vectorIndex++;
		};
		for (int i = 0; i < qualifiedNames.size (); i++) {
			if (qualifiedNames.get(i).equals (param.getName())) {
				qualifiedNames.remove(i);
				qualifiedNames.add(i, node.toString ());
			}
		}
		String newIdentifier = "";
		   java.util.Enumeration e = qualifiedNames.elements();
		    if (e.hasMoreElements())
		       newIdentifier += (String) e.nextElement();

		    while (e.hasMoreElements()) {
		        newIdentifier += "." + (String )e.nextElement();
		    };
		return newIdentifier;
	}


	public List transformRule (ExpressionNode origRule) {
		
		List transformedRules = new ArrayList (3);		
		
		ExpressionNode rule = null;
		
		if (origRule instanceof DesignAnalysisCallNode) {
			// Break open design analysis calls
			DesignAnalysisCallNode dac = (DesignAnalysisCallNode )origRule;
			rule = dac.getReferentDesignAnalysis().getDesignRuleExpression();
			rule = rule.cloneExpressionTree();
		}
		else {
			rule = origRule.cloneExpressionTree();
		}
		SetConstructorNode select = null;
		if (rule instanceof QuantifiedExpressionNode) {
			QuantifiedExpressionNode quantExp = (QuantifiedExpressionNode )rule;
			if (quantExp.getQuantOp() == QuantifiedExpressionNode.UNIVERSAL_QUANTIFICATION) {
				select = new SetConstructorNode (SetConstructorNode.SELECT);
				select.setVarName(quantExp.getVarName());
				select.setVarType (quantExp.getVarType ());
				select.setVarTypeInferred(quantExp.getVarTypeInferred());
				select.setSetExpression((ExpressionNode )quantExp.getSetExpression());
				
				BooleanExpressionNode notNode = new BooleanExpressionNode (ArmaniParserTreeConstants.JJTBOOLEANEXPRESSIONNODE);//ExpressionOperatorT.NOT_OP);
				notNode.setOperator (ExpressionOperatorT.NOT_OP);
				notNode.jjtAddChild(quantExp.getEvalExpression(), 0);
				quantExp.getEvalExpression().setInParentheses(true);
				select.setEvalExpression(notNode);
				
				// Now, parse this rule to make sure that the expression tree is right
				ByteArrayInputStream bis = new ByteArrayInputStream (select.toString().getBytes ());
				ArmaniParser.ReInit (bis);
				AcmeErrorManager.clearAllErrors ();
				try {
					ArmaniParser.SetConstructor(null);
					select = (SetConstructorNode )ArmaniParser.getJJTree().peekNode();
					transformedRules.add(select);
				}
				catch (ParseException e) {
					Debug.errorln (e);
				}
				transformedRules.addAll(transformRule (quantExp.getEvalExpression()));
			}
		}

		return transformedRules;
	}
	
	
	public Set evaluateSelection (DesignAnalysis da, Vector params) {
		Set retSet = new HashSet ();
		try {
			da.resolveTypeReferences();
			da.resolveInstanceReferences();
			da.typecheck ();
			SetExpressionNode set = da.evaluateAsSet(params);
			AcmeEnumeration values = set.enumerate();
			while (values.moreItems ()) {
				AcmeObject object = (AcmeObject )values.getNextItem ();
				retSet.add(object);
			}
		}
		catch (DesignRuleEvalException e) {
			e.printStackTrace();
		}
		return retSet;
	}


	/**
	 * @param designRuleExpression
	 * @return
	 */
	public static boolean canTransformRule(ExpressionNode designRuleExpression) {
		return designRuleExpression instanceof QuantifiedExpressionNode || (designRuleExpression instanceof DesignAnalysisCallNode &&
				((DesignAnalysisCallNode )designRuleExpression).getReferentDesignAnalysis().getDesignRuleExpression() instanceof QuantifiedExpressionNode);
	}
}
