/*
 * Created on Mar 5, 2003
 *
 */
package AcmeStudio.ui.widgets;

import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;

import AcmeStudio.core.ASModel;
import AcmeStudio.util.ImageManager;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeElement;
import acmeLib.errorManagement.AcmeError;

/**
 * @author schmerl
 * 
 * Implements tree viewing of an AcmeError, and all its causes
 */
public class AcmeErrorTree {
    private static final Object [] emptyErrors = new Object [0];

    protected TreeViewer _viewer;
    protected AcmeError _rootError;

    class ErrorContentProvider implements IStructuredContentProvider, ITreeContentProvider {
        
            /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
         */
        public Object[] getElements(Object inputElement) {
            if (inputElement instanceof AcmeError) return new Object [] {inputElement};
            return getChildren (inputElement);           
            
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.IContentProvider#dispose()
         */
        public void dispose() {
 
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
         */
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
 
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
         */
        public Object[] getChildren(Object parentElement) {
            Object [] children = AcmeErrorTree.emptyErrors;
            if (parentElement instanceof AcmeElement) {
              
                AcmeElement element = (AcmeElement )parentElement;
                AcmeEnumeration errNum = ASModel.current ().getRuleManager().getErrors (element);
                
                if (errNum != null) {
                    children = new Object [errNum.getSize ()];
                    int index = 0;
                    while (errNum.moreItems ()) {
                        AcmeError error = (AcmeError )errNum.getNextItem ();
                        children [index++] = error;
                    }
                }
            }
            else if (parentElement instanceof AcmeError) {
                AcmeError parentError = (AcmeError )parentElement;
                AcmeEnumeration errors = parentError.getCauses();
                children = new Object [errors.getSize()];
                int index = 0;
                while (errors.moreItems ()) {
                    children [index++] = errors.getNextItem ();
                }
            }
            else if (parentElement instanceof AcmeEnumeration) {
                AcmeEnumeration errors = (AcmeEnumeration )parentElement;
                int index = 0;
                children = new Object [errors.getSize()];
                while (errors.moreItems ()) {
                    children [index++] = errors.getNextItem ();    
                }
            }
            
            return children;
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
         */
        public Object getParent(Object element) {
            return null;
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
         */
        public boolean hasChildren(Object element) {
            if (element instanceof AcmeError) {
                return ((AcmeError )element).getCauses().getSize () != 0;
            }
            else if (element instanceof AcmeElement) {
                AcmeEnumeration errEnum = ASModel.current ().getRuleManager().getErrors ((AcmeElement )element);
                return errEnum != null && errEnum.getSize() != 0;
            }
            return false;
        }

}
    
    class ErrorLabelProvider extends LabelProvider {
            
            /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
         */
        public Image getImage(Object element) {
            if (element instanceof AcmeError) {
                AcmeError error = (AcmeError )element;
                int type = error.getMsgType();
                if (type == AcmeError.ERROR) {
                    return ImageManager.getIcon (ImageManager.ID_ERROR);
                }
                else if (type == AcmeError.WARNING) {
                    return ImageManager.getIcon (ImageManager.ID_WARNING);
                }
            }
            return null;
        }

        /* (non-Javadoc)
         * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
         */
        public String getText(Object element) {
            if (element instanceof AcmeElement) {
                return ((AcmeElement )element).getName ();
            }
            else if (element instanceof AcmeError) {
                return ((AcmeError )element).getErrorMessage();
            }
            return "";
        }

}
        
    public AcmeErrorTree (Composite parent) {
        _viewer = new TreeViewer (parent, SWT.H_SCROLL | SWT.V_SCROLL);
        _viewer.setContentProvider (new ErrorContentProvider ());
        _viewer.setLabelProvider (new ErrorLabelProvider ());       
    }

    public void setErrorSource (AcmeObject element) {
        _viewer.setInput (element);
//        _viewer.refresh ();
    }
    
    public void setVisible (boolean visible) {
        _viewer.getTree().setVisible (visible);
    }
    
    public Tree getTree () {return _viewer.getTree();}
    
}
