/*
 * Created on Oct 21, 2003
 *  
 */
package AcmeStudio.ui.widgets;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.FilteredList;

import AcmeStudio.util.Helper;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.filters.AcmeObjectFilter;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;

/**
 * @author schmerl
 * 
 * To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Generation - Code and Comments
 */
public class TypeSelectionWidget extends Composite {

	/**
	 * @author Bradley Schmerl
	 * Created Mar 31, 2004
	 */
	public class TypeNameProvider extends LabelProvider {
		/**
		 * 
		 */
		public TypeNameProvider() {
			super();
		}
		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
		 */
		public String getText(Object element) {
			if (element instanceof AcmeElementType) {
				return ((AcmeElementType )element).getTypeName ();
			}
			return "";
		}
	}
    private Text _filterText;
    private AcmeElement _scope;
    private AcmeObjectFilter _filter;
    private List _lstSelectedTypes;
    private Set _selectedTypes = new HashSet();
    private Button _btnMoveToSelection;
    private FilteredList _lstAvailableTypes;
    private Button _btnMoveAllToSelection;
    private Button _btnMoveFromSelection;
    private Button _btnMoveAllFromSelection;
    private String _selectedTypeLabel;

    /**
	 * @param parent
	 * @param style
	 */
    public TypeSelectionWidget(
        Composite parent,
        AcmeElement scope,
        AcmeObjectFilter filter) {
        super(parent, SWT.NONE);
        _scope = scope;
        _filter = filter;
        _selectedTypeLabel = "Selected types:";
        createPart(parent);
        updateAvailableTypes ();
    }
    
    public TypeSelectionWidget (Composite parent, AcmeElement scope, AcmeObjectFilter filter, String selectedTypeLabel) {
        super (parent, SWT.NONE);
        _selectedTypeLabel = selectedTypeLabel;
        _scope = scope;
        _filter = filter;
        createPart (parent);
        updateAvailableTypes ();
    }

    /**
     * 
     */
    public void updateAvailableTypes() {
    	AcmeEnumeration availableTypes = _scope.getAvailableTypes ().castToAcmeSetVector().enumerate (_filter);
    	availableTypes = availableTypes.castToAcmeSetVector().enumerate (new AcmeObjectFilter () {

			public boolean test(AcmeObject obj) {
				if (obj instanceof AcmeElementType) {
					if (_selectedTypes.contains(obj)) {
						return false;
					}
				}
				return true;
			}
    		
    	});
    	_lstAvailableTypes.setElements (Helper.convertAcmeEnumerationToList(availableTypes).toArray ());
    	

        AcmeEnumeration types = _scope.getAvailableTypes().castToAcmeSetVector().enumerate (_filter);
        while (types.moreItems()) {
            AcmeElementType type = (AcmeElementType) types.getNextItem();
            _lstAvailableTypes.setData(type.getTypeName (), type);
        }
    }

    /**
	 * @param parent
	 */
    private void createPart(Composite parent) {
        final GridLayout gridLayout = new GridLayout(3, false);
        gridLayout.numColumns = 3;
        this.setLayout(gridLayout);

        Label typesL = new Label(this, SWT.NONE);
        typesL.setText(_selectedTypeLabel);

        Label spacer = new Label(this, SWT.NONE);

        Label avaiLabel = new Label(this, SWT.NONE);
        avaiLabel.setText("Available types:");
    	Label label = new Label(this, SWT.NONE);
    	label = new Label(this, SWT.NONE);
    	label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
    	label.setText("Type Name:");
    	_filterText = new Text(this, SWT.BORDER);
    	_filterText.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
    	Listener listener = new Listener () {
    		public void handleEvent (Event e) {
    			_lstAvailableTypes.setFilter (_filterText.getText ());
    		}
    	};
    	_filterText.addListener (SWT.Modify, listener);
    	_filterText.addKeyListener (new KeyListener () {
    		public void keyPressed (KeyEvent e) {
    			if (e.keyCode == SWT.ARROW_DOWN) {
    				_lstAvailableTypes.setFocus ();
    			}
    		}

			public void keyReleased(KeyEvent e) {
			}
    	});
        _lstSelectedTypes = new List(this, SWT.MULTI | SWT.V_SCROLL);
        GridData stgd = new GridData(GridData.FILL_BOTH);
        stgd.grabExcessHorizontalSpace = true;
        stgd.grabExcessVerticalSpace = true;
        stgd.horizontalAlignment = GridData.FILL;
        stgd.verticalAlignment = GridData.FILL;
        _lstSelectedTypes.setLayoutData(stgd);

        Composite cmpButtons = new Composite(this, SWT.NONE);
        cmpButtons.setLayout(new FillLayout(SWT.VERTICAL));

        _btnMoveToSelection = new Button(cmpButtons, SWT.PUSH);
        _btnMoveToSelection.setText("< Add");
        _btnMoveToSelection.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                moveSelectionFromAvailable();
            	_filterText.setText ("");
                updateTypes();
                updateAvailableTypes ();
            }


        });

        _btnMoveAllToSelection = new Button(cmpButtons, SWT.PUSH);
        _btnMoveAllToSelection.setText("<< Add all");
        _btnMoveAllToSelection.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
            	Control [] cntrl = _lstAvailableTypes.getChildren ();
            	Table lst = (Table )_lstAvailableTypes.getChildren() [0];
            	int num = lst.getItemCount();
            	int [] selection = new int [num];
            	for (int i = 0; i < selection.length; i++) {
					selection [i] = i;
				}
            	_lstAvailableTypes.setSelection(selection);
            	moveSelectionFromAvailable();
            	_filterText.setText ("");
                updateTypes();
                updateAvailableTypes ();
            }
        });

        _btnMoveFromSelection = new Button(cmpButtons, SWT.PUSH);
        _btnMoveFromSelection.setText("Remove >");
        _btnMoveFromSelection.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                String[] selection = _lstSelectedTypes.getSelection();
                for (int i = 0; i < selection.length; i++) {
                    _lstAvailableTypes.setData(
                        selection[i],
                        _lstSelectedTypes.getData(selection[i]));
                    _lstSelectedTypes.remove(selection[i]);
                    updateTypes();
                    updateAvailableTypes ();
                }
            }
        });

        _btnMoveAllFromSelection = new Button(cmpButtons, SWT.PUSH);
        _btnMoveAllFromSelection.setText("Remove All >>");
        _btnMoveAllFromSelection.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
  
            	_lstSelectedTypes.removeAll();
            	updateTypes();
            	updateAvailableTypes ();
            }
        });

         _lstAvailableTypes = new FilteredList (this, SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL, new TypeNameProvider (), true, true, true);
        GridData atgd = new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_FILL);
        atgd.grabExcessHorizontalSpace = true;
//        atgd.grabExcessVerticalSpace = true;
        atgd.horizontalAlignment = GridData.FILL;
        atgd.verticalAlignment = GridData.FILL;
        _lstAvailableTypes.setLayoutData(atgd);
    }

	private void moveSelectionFromAvailable() {
		Object [] selection = _lstAvailableTypes.getSelection();
        for (int i = 0; i < selection.length; i++) {
            String typeName = ((AcmeElementType )selection[i]).getTypeName();
			_lstSelectedTypes.add(typeName);
            _lstSelectedTypes.setData(
                typeName,
				selection [i]);
 
        }
	}
	

	public void clearSelectedTypes () {
		String [] items = _lstSelectedTypes.getItems();
		for (int i = 0; i < items.length; i++) {
			_lstAvailableTypes.setData (items [i], _lstSelectedTypes.getData(items [i]));
			_lstSelectedTypes.remove(items [i]);
		}
		updateAvailableTypes ();
	}
    /**
	 *  
	 */
    protected void updateTypes() {
        _selectedTypes.clear();
        String[] selection = _lstSelectedTypes.getItems();
        for (int i = 0; i < selection.length; i++) {
            _selectedTypes.add(_lstSelectedTypes.getData(selection[i]));
        }
    }

    public Set getSelectedTypes() {
        return _selectedTypes;
    }

    /**
     * @param selectedTypes
     */
    public void updateSelectedTypes(String[] selectedTypes) {
        for (int i = 0; i < selectedTypes.length; i++) {
            if (_lstAvailableTypes.getData (selectedTypes [i]) != null) {
                _lstSelectedTypes.add (selectedTypes [i]);
                _lstSelectedTypes.setData(selectedTypes [i], _lstAvailableTypes.getData (selectedTypes [i]));
//                _lstAvailableTypes.remove(selectedTypes [i]);
            }
        }
        updateTypes ();
        updateAvailableTypes ();
    }
    /* (non-Javadoc)
     * @see org.eclipse.swt.widgets.Control#setEnabled(boolean)
     */
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        _btnMoveAllFromSelection.setEnabled (enabled);
        _btnMoveAllToSelection.setEnabled (enabled);
        _btnMoveFromSelection.setEnabled (enabled);
        _btnMoveToSelection.setEnabled (enabled);
        _lstAvailableTypes.setEnabled (enabled);
        _lstSelectedTypes.setEnabled (enabled);
    }

    public void addAddSelectionListener (SelectionListener l) {
        _btnMoveAllToSelection.addSelectionListener(l);
        _btnMoveToSelection.addSelectionListener (l);
    }
    
    public void addRemoveSelectionListener (SelectionListener l) {
        _btnMoveAllFromSelection.addSelectionListener(l);
        _btnMoveFromSelection.addSelectionListener(l);
    }
    
}
