package AcmeStudio.util;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import AcmeStudio.core.resource.provider.ASResourceProvider;
import acmeLib.elements.AcmeElement;

/**
 * @author Bradley Schmerl (schmerl@cs.cmu.edu)
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 *
 * Implements the Editor input for AcmeStudio's editors.
 */
public class ASEditorInput implements IEditorInput {

    /** The ASResourceProvider providing this editor input */
    private ASResourceProvider _prov = null;
    /** The AcmeElement being edited */
    private AcmeElement _element = null;

    /**
     * Constructor for ASEditorInput.
     */
    public ASEditorInput(ASResourceProvider prov, AcmeElement element) {
        super();

        setResourceProvider(prov);
        setElement(element);
    }

    /**
     * @see org.eclipse.ui.IEditorInput#exists()
     */
    public boolean exists() {
        return true;
    }

    /**
     * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
     */
    public ImageDescriptor getImageDescriptor() {
        return null;
    }

    /**
     * @see org.eclipse.ui.IEditorInput#getName()
     */
    public String getName() {
        if (_element != null) {
            return _element.getName();
        } else {
            return null;
        }
    }

    /**
     * @see org.eclipse.ui.IEditorInput#getPersistable()
     */
    public IPersistableElement getPersistable() {
        return null;
    }

    /**
     * @see org.eclipse.ui.IEditorInput#getToolTipText()
     */
    public String getToolTipText() {
        return getName();
    }

    /**
     * @see org.eclipse.core.runtime.IAdaptable#getAdapter(Class)
     */
    public Object getAdapter(Class adapter) {
        return null;
    }

    /**
     * Returns the element.
     * @return AcmeElement  the element
     */
    public AcmeElement getElement() {
        return _element;
    }

    /**
     * Sets the element.
     * @param element The element to set
     */
    public void setElement(AcmeElement element) {
        _element = element;
    }

    /**
     * Returns the resource provider providing this editor input.
     * @return ASResourceProvider  the resource provider 
     */
    public ASResourceProvider getResourceProvider () {
        return _prov;
    }

    /**
     * Sets the new resource provider providing this editor input.
     * @param prov  the new provider to set
     */
    public void setResourceProvider (ASResourceProvider prov) {
        _prov = prov;
    }

}
