package AcmeStudio.util;

/**
 * @author Bradley Schmerl
 * The interface used on the publish/subscribe bus of AcmeStudio that allows 
 * interested parties to be notified of changes in an Acme model
 *
 */
public interface AcmeModelChangeListener {
	
	/**
	 * This method is called by the change bus when a model is changed
	 * @param evt The event causing the change to occur
	 */
	public void changeModel (AcmeStudioChangeManager.AcmeStudioChangeEvent evt);
	
	/**
	 * Lets parties register their interest in particular types of events
	 * @param eventType The type of event being queried, which will be an int defined in AcmeStudioChangeManager
	 * @return whether the party is interested in this type of event. If it is, changeModel will be called. If not, changeModel
	 * 	will not be called.
	 */
	public boolean interestedInEvent (int eventType);

}
