package AcmeStudio.util;

import java.io.ByteArrayInputStream;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASModel;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.designRules.*;
import acmeLib.elements.AcmeElementType;
import acmeLib.errorManagement.AcmeError;
import acmeLib.errorManagement.AcmeErrorManager;
import acmeLib.errorManagement.AcmeParseError;
import acmeLib.properties.*;

/**
 * @author schmerl
 *
 */
public class DesignRuleHolder implements ITipInformer {

    public static final String NOEVALUATION = "$NoEvaluation";
    public static final String BACKGROUNDEVALUATION = "$EvaluateBackground$";
    public static final String FOREGROUNDEVALUATION = "$EvaluateForeground$";
    public static final boolean FORCEREEVALUATION = false;

	private DesignRule _rule;
	private String _unparsedText = null;
	private ParseException _parseException = null;
    private AcmeError _parseError = null;
	private List _errors = new Vector ();
	private Scope _scope;
	private Scope _evalScope;
	/**
	 * Constructor for DesignRuleHolder.
	 */
	public DesignRuleHolder() {
		super();
	}
	
    public String getRuleText () {
        if (_unparsedText != null) return _unparsedText;
        else if (_rule != null) return _rule.toString ();
        else return "";   
    }
    
	public void setDesignRule (DesignRule rule) {
		_rule = rule;
		if (rule != null) {
			_scope = rule.getDesignRuleScope ();
		}
	}
	
	public void setDesignRule (String rule) {
		_unparsedText = rule;
		parseRule (_unparsedText);
	}
	
	public DesignRule getDesignRule () {
		if (_rule == null && _parseException == null && _unparsedText != null) {
			// unparsed text hasn't been parsed
			parseRule (_unparsedText);
		}
		return _rule;	
	}
	
	public String getUnparsedText () {
		return _unparsedText;
	}
	
	public void setScope (Scope scope) {
		_scope = scope;
	}
	
	public void setEvalScope (Scope scope) {
		_evalScope = scope;
	}
	/**
	 * @param analysis
	 * @param ruleToParse
	 */
	public void parseRuleExpression(DesignAnalysis analysis, String text) {
		ByteArrayInputStream bis = new ByteArrayInputStream (text.getBytes());
		ArmaniParser.ReInit (bis);
		AcmeErrorManager.clearAllErrors ();
		try {
			ArmaniParser.DesignRuleExpression(analysis);
			ExpressionNode expression = (ExpressionNode )ArmaniParser.getJJTree ().peekNode ();
			_rule = new DesignRule ();
			_rule.setDesignRuleExpression(expression);
			if (AcmeErrorManager.getNumErrorsReported() == 0) {
				_unparsedText = null;
				_parseError = null;
				_parseException = null;
			}
			else {
				_parseError = (AcmeError )AcmeErrorManager.getErrors ().getNextItem ();
				_unparsedText = text;
				_rule = null;
			}
		}
		catch (ParseException e) {
			_parseException = e;
			_unparsedText = text;
			_rule = null;
		}
	}
	
	public void parseRule (String text) {
//		StringBufferInputStream sin = new StringBufferInputStream (text);
//        StringReader sw = new StringReader(text);
        ByteArrayInputStream bis = new ByteArrayInputStream(text.getBytes());
		ArmaniParser.ReInit (bis);
        AcmeErrorManager.clearAllErrors();
		try {
			_rule = ArmaniParser.parse_DesignRule (_scope);
            if (AcmeErrorManager.getNumErrorsReported() == 0) {
			 _unparsedText = null;
             _parseError = null;
			 _parseException = null;
            }
            else {
                _parseError = (AcmeError )AcmeErrorManager.getErrors().getNextItem();
                _unparsedText = text;
                _rule = null;
            }
		}
		catch (ParseException e) {
			_parseException = e;
			_unparsedText = text;	
			_rule = null;
            Debug.diagnoseln(e);
		}	
	}
	
	public boolean hasErrors () {
		return _parseException != null || _parseError != null;
	}
	
	public String getErrorText () {
		String text = "";
		if (_parseException != null) {
			text = _parseException.toString ();
		}
		return text;
	}

    public AcmeError getError () {
        if (_parseException != null) {
            return new AcmeParseError (_parseException);
        }
        else if (_parseError != null) {
            return _parseError;
        
        }
        return null;
    }
	/**
	 * @see AcmeStudio.util.ITipInformer#getTipInformation()
	 */
	public String getTipInformation() {
		String tip = ""; // The tip to be displayed
		if (_rule.getDesignRuleScope() != _evalScope) {
			// find out where inherited from
            if (_rule.getDesignRuleScope() instanceof AcmeElementType) {
			     String typeName =
				    ((AcmeElementType) _rule.getDesignRuleScope()).getTypeName();
			     tip = "Inherited from " + typeName + "\n";
            }
            else {
                tip = "Inherited from ???\n";
            }

		}

		// replace | with | and a newline, to enhance formatting
		// DesignRule.toString() does not have any returns, 
		// | indicate a new expression, so this is a convenient place to
		// put a return.
		String ruleText = _rule.toString();
//		ruleText = ruleText.replaceAll("[|]", "|\n");
//		// Process to add indenting
//		String parseText = "";
//		int index = 0;
//		int lastIndex = 0;
//		int indentLevel = 1;
//		while ((index = ruleText.indexOf("|\n", lastIndex)) != -1) {
//			parseText += ruleText.substring(lastIndex, index + 2);
//			lastIndex = index + 2;
//			for (int i = 0; i < indentLevel; i++) {
//				parseText += "  ";
//			}
//			indentLevel++;
//		}
//		parseText += ruleText.substring(lastIndex);
//
//		tip += parseText;
//		
        tip += ruleText;
		if (_errors.size() > 0) {
			Iterator i = _errors.iterator ();
			while (i.hasNext()) {
				tip += "\n";
				tip += ((AcmeError )i.next()).getErrorMessage();
			}
		}
		
		return tip;
	}
	
	public boolean hasTipInformation () {
		return true;
	}

	public void clearErrors () {
		_errors.clear();
	}
	
	public void addError (AcmeError e) {
		_errors.add(e);
	}
	
	private int lastEvaluation = DesignEvaluationCache.FALSE;
	
    public void unevaluate (Scope scope) {
        lastEvaluation = DesignEvaluationCache.UNKNOWN;
        DesignEvaluationCache cache = ASModel.current().getRuleManager();
        cache.clearCachedResult(_rule, scope);
    }
    
    public int typecheck (Scope scope) {
    	if (!hasErrors ()) {
    		AcmeErrorManager.clearAllErrors();
//    		if (!_rule.getWasTypeChecked()) {
    			if (!_rule.typecheck ()) {
    				processErrors ();
    				return DesignEvaluationCache.ERROR;
    			}
//    		}
    		return DesignEvaluationCache.TRUE;
    	} 
    	else {
    		return DesignEvaluationCache.ERROR;
    	}
    	
    }
    
    public int evaluate(Scope scope) {
        int result = DesignEvaluationCache.FALSE;
        if (!hasErrors()) {
            AcmeErrorManager.clearAllErrors();
            if (!_rule.getWasTypeChecked()) {
                if (!_rule.typecheck()) {
                    processErrors();
                    result = DesignEvaluationCache.ERROR;
                    return result;
                }
            }

            try {
                IPreferenceStore store = AcmeStudioPlugin.getDefault().getPreferenceStore ();
                if (store.getString("ConstraintEvaluation").equals (BACKGROUNDEVALUATION)) {
                    
                    DesignEvaluationCache cache = ASModel.current().getRuleManager();
                    lastEvaluation = cache.evaluate(_rule, scope);
                    result = lastEvaluation;
                }
                else if (store.getString ("ConstraintEvaluation").equals (FOREGROUNDEVALUATION)) {
                    _rule.setDesignRuleScope(scope);
                    if (_rule.evaluate ()) {
                        result = DesignEvaluationCache.TRUE;
                    }
                    else {
                        result = DesignEvaluationCache.FALSE;
                    }
                }
                else {
                    result = DesignEvaluationCache.UNKNOWN;
                }

            } catch (DesignRuleEvalException e) {

            }
            
        }

//        AcmeErrorManager.clearAllErrors();
    
        return result;
    }

	public void processErrors () {
		AcmeEnumeration errors = AcmeErrorManager.getErrors ();
		while (errors.moreItems ()) {
			AcmeError error = (AcmeError )errors.getNextItem ();
			addError (error);
		}
	}

    public boolean isInherited () {
        if (hasErrors ()) return false;

        return _evalScope != null && _scope != _evalScope;
        
    }
    
    public boolean shouldEvaluate () {
        if (hasErrors ()) return false;
        if (_rule == null) return true;
        if (AcmeStudioPlugin.getDefault().getPreferenceStore ().getString ("ConstraintEvaluation").equals (DesignRuleHolder.NOEVALUATION)) {
            return false;
        }
        AcmeProperty shouldEval = _rule.getProperty ("evaluate");
        if (shouldEval != null) {
            if (shouldEval.getType() instanceof BooleanType) {
                return ((AcmeBoolean )shouldEval.getValue()).getValue();   
            }
        }
        return true;
    }
    
    public void setEvaluation (boolean evaluate) {
        if (_rule != null) {
            AcmeProperty shouldEval = _rule.getProperty ("evaluate");
            if (evaluate) {
                if (shouldEval != null) {
                    _rule.removeProperty(shouldEval);    
                }
            }
            else {
                if (shouldEval == null) {
                    shouldEval = new AcmeProperty ("evaluate", new BooleanType ("boolean"), new AcmeBoolean (false));
                    try {
                        _rule.addProperty (shouldEval);
                    }
                    catch (AcmeIllegalNameException e) {
                        e.printStackTrace();
                    }
                }
                else {
                    shouldEval.setValue (new AcmeBoolean (false));
                }
            }
        }
    }
    
    public void setErrorMessage (String message) {
        if (_rule != null) {
            AcmeProperty errMsg = _rule.getProperty ("errMsg");
            if (errMsg == null && !message.equals ("")) {
                try {
                    _rule.addProperty ("errMsg", new StringType ("string"), new AcmeString (message));
                    AcmeStudioChangeManager.globalChangeManager.onAddProperty(_rule, _rule.getProperty ("errMsg"), this);
                }
                catch (AcmeIllegalNameException e) {
                    e.printStackTrace();
                }
                
            }
            else {
                if (!message.equals ("")) {
                    
                    errMsg.setValue (new AcmeString (message));
                    AcmeStudioChangeManager.globalChangeManager.onSetProperty(_rule, errMsg, errMsg.getValue(), this);
                }
                else {
                    _rule.removeProperty (errMsg);
                    AcmeStudioChangeManager.globalChangeManager.onRemoveProperty(_rule, errMsg, this);
                }
            }
        }
    }
    
    public void setLabel (String label) {
        if (_rule != null) {
            AcmeProperty labelProp = _rule.getProperty ("label");
            if (labelProp == null && !label.equals ("")) {
                try {
                    _rule.addProperty ("label", new StringType ("string"), new AcmeString (label));
                }
                catch (AcmeIllegalNameException e) {
                    e.printStackTrace ();
                }
            }
            else {
                if (!label.equals ("")) {
                    labelProp.setValue (new AcmeString (label));
                }
                else {
                    _rule.removeProperty (label);
                }
            }
                         
        }   
    }
    
    public boolean hasLabel () {
        return (_rule != null && _rule.getProperty ("label") != null);   
    }
    
    public String getErrorMessage () {
        if (_rule == null) return "";
        AcmeProperty errMsg = _rule.getProperty ("errMsg");
        if (errMsg == null) return "";
        String msg = ((AcmeString )errMsg.getValue ()).getValue();
        // Strip surrounding quotes
//        msg = msg.substring (1, msg.length () - 2);
        return msg;
    }
    
    public String getLabel () {
        if (_rule == null) return "";
        AcmeProperty label = _rule.getProperty ("label");
        if (label == null) return _rule.toString ();
        return ((AcmeString )label.getValue ()).getValue ();   
    }
    
    public Image getEvaluationImage () {
        if (hasErrors ()) {
            return PlatformUI.getWorkbench ().getSharedImages ().getImage (ISharedImages.IMG_OBJS_ERROR_TSK);           
        }
        if (!shouldEvaluate ()) {
            return ImageManager.getIcon (ImageManager.ID_CONSTRAINT_NOT_RUNNING);                
        }
        return null;
    }

    /* (non-Javadoc)
     * @see AcmeStudio.util.ITipInformer#getTipImage()
     */
    public Image getTipImage() {
        return null;
    }



}
