/*
 * Created on Nov 3, 2003
 *
 */
package AcmeStudio.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import AcmeStudio.core.resource.ASFamily;
import acmeLib.AcmeType;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.filters.*;
import acmeLib.elements.AcmeFamily;

/**
 * @author Bradley Schmerl
 * Created Nov 3, 2003
 */
public class FamilyDetailContentProvider implements ITreeContentProvider {

	/**
	 * 
	 */
	public FamilyDetailContentProvider() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) {
		return getChildren (inputElement);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object inputElement) {
		if (inputElement instanceof ASFamily) {
			AcmeFamily family = ((ASFamily )inputElement).getFamily();
			if (family != null) {
				
				AcmeEnumeration allTypes = family.getTypes ();
				AcmeEnumeration compTypes = allTypes.castToAcmeSetVector().enumerate (new ComponentTypeEnumFilter ());
				AcmeEnumeration connTypes = allTypes.castToAcmeSetVector().enumerate (new ConnectorTypeEnumFilter ());
				AcmeEnumeration portTypes = allTypes.castToAcmeSetVector().enumerate (new PortTypeEnumFilter ());
				AcmeEnumeration roleTypes = allTypes.castToAcmeSetVector().enumerate (new RoleTypeEnumFilter ());
				AcmeEnumeration propTypes = allTypes.castToAcmeSetVector().enumerate (new PropertyTypeEnumFilter ());
				
				List l = new ArrayList ();
				if (compTypes.getSize () > 0) l.add (compTypes);
				if (connTypes.getSize () > 0) l.add (connTypes);
				if (portTypes.getSize () > 0) l.add (portTypes);
				if (roleTypes.getSize () > 0) l.add (roleTypes);
				if (propTypes.getSize () > 0) l.add (propTypes);
				return l.toArray();
			}
		}
		else if (inputElement instanceof AcmeEnumeration) {
			AcmeEnumeration enum = (AcmeEnumeration) inputElement;
			// BugFix#239: Reset the enum to make sure that we start at the beginning
			enum.reset();
			Object [] children = new Object [enum.getSize ()];
			int index = 0;
			while (enum.moreItems()) {
				AcmeType type = (AcmeType) enum.getNextItem();
				children [index++] = type;
			}
			return children;
			
		}
		return Constants.EMPTY_ELEMENTS;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element) {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element) {
		if (element instanceof AcmeEnumeration) return true;
		if (element instanceof ASFamily) return true;
		return false;
		
	}

}
