/*
 * Created on Dec 23, 2003
 *
 */
package AcmeStudio.util;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;



/**
 * @author Bradley Schmerl
 * Created Dec 23, 2003
 */
public class Geometry {

	/**
	 * 
	 * @param p0
	 * @param p1
	 * @return the distance between p0 and p1
	 */
	public static double dist (Point p0, Point p1) {
		return p0.getDistance(p1);
	}
	
	/**
	 * returns the interpolation between two points
	 * @param p0
	 * @param p1
	 * @param t
	 * @return
	 */
	public static Point interpolate (Point p0, Point p1, double t) {
		double x = t * p1.x + (1 - t) * p0.x;
		double y = t * p1.y + (1 - t) * p0.y;
		return new Point ((int )(x + 0.5), (int )(y + 0.5));
	}
	
	/**
	 * Evaluates a bezier defined by the control polygon in points
	 * @param points The control polygon
	 * @param t
	 * @return
	 */
	
		public static Point evalBezier(Point arr[],double t) {
			for (int iter = arr.length ; iter > 0 ; iter--) {
				for (int i = 1 ; i < iter ; i++) {
					arr[i-1] = interpolate(arr[i-1],arr[i],t);
				}
			}
			return arr[0];
		}
	
	
	public static Point evalBezierRec (Point points [], double t, int iter) {
		if (iter == 1) return points [0];
		for (int i = 1; i < iter; i++) {
			points [i - 1] = interpolate (points [i-1], points [i], t);
		}
		return evalBezierRec (points, t, iter - 1);
		
	}
	
	/** 
	 * Get's the side (NORTH, EAST, SOUTH, WEST) that a point is on a rectangle
	 * @param point The point
	 * @param centerOfRect The center of the rectangle
	 * @return
	 */
	public static int getSideOfPoint (Point point, Point centerOfRect) {
		double angle = -Math.toDegrees (Math.atan (Geometry.getSlope (centerOfRect, point)));
		int side = -1;
		if (angle == 90) side = PositionConstants.NORTH;
		else if (angle == -90) side = PositionConstants.SOUTH;
		else if (angle >= -45 && angle < 45) side = PositionConstants.EAST;
		else if (angle >= 45 && angle < 135) side = PositionConstants.NORTH;
		else if (angle >= 135 || angle <= -135) side = PositionConstants.WEST;
		else if (angle > -135 && angle < -45) side = PositionConstants.SOUTH;
		return side;
	}

	public static int getOppositeSide(int side)
	{
	    switch (side)
	    {
	        case PositionConstants.NORTH:
	            return PositionConstants.SOUTH;
	        case PositionConstants.EAST:
	            return PositionConstants.WEST;
	        case PositionConstants.SOUTH:
	        	return PositionConstants.NORTH;
	        case PositionConstants.WEST:
	            return PositionConstants.EAST;
	    }
	    return -1;
	}
	
	/**
	 * @param intersectionPoint
	 * @param startPoint
	 * @return
	 */
	public static double getSlope(Point p0, Point p1) {
		double slope = ((double) (p1.y - p0.y)) / ((double) (p1.x - p0.x));
		return slope;
	}
}
