package AcmeStudio.util;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.*;
import javax.swing.UIManager;
import javax.swing.tree.DefaultMutableTreeNode;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.SWTGraphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Display;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.prefs.GlobalVisPreferences;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.ui.editors.diagram.figures.IntersectingPoint;
import AcmeStudio.ui.editors.diagram.figures.SystemView;
import AcmeStudio.ui.editors.diagram.parts.ElementEditPart;
import AcmeStudio.ui.editors.diagram.parts.RoleEditPart;
import AcmeStudio.ui.editors.diagram.styles.*;
import AcmeStudio.ui.editors.diagram.styles.constants.Orientation;
import AcmeStudio.ui.editors.diagram.styles.constants.Shape;
import AcmeStudio.ui.editors.diagram.visualInfo.PortVisualInformation;
import AcmeStudio.ui.editors.diagram.visualInfo.RoleVisualInformation;
import AcmeStudio.ui.editors.diagram.visualInfo.VisualInformation;
import AcmeStudio.wizards.connection.PortConnectionRule;
import AcmeStudio.wizards.connection.PortRoleAssociation;
import acmeLib.AcmeObject;
import acmeLib.AcmeType;
import acmeLib.acmeParser.AcmeParser;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.armaniParser.ParseException;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.designRules.DesignRule;
import acmeLib.designRules.Scope;
import acmeLib.elements.*;
import acmeLib.errorManagement.AcmeError;
import acmeLib.errorManagement.AcmeErrorManager;
import acmeLib.errorManagement.AcmeParseError;
import acmeLib.properties.*;
import acmeLib.representations.AcmeBinding;
import acmeLib.representations.AcmeBindingsMap;
import acmeLib.representations.AcmeRepresentation;
import acmeLib.tools.converters.Acme2Armani;
public class Helper {
	
	private AcmeSystem currentAcmeSystem = null;
	public static void main(String[] args) {
		// Construct Frame
		if (args.length == 0) {
			Debug.warnln("Have nothing to do");
			return;
		}
		try {
			UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
		}
		catch (Exception e) {
			Debug.getLog().error("Failed to set look and feel: ", e);
		}
	}


   /**
    *  Writes the AcmeDesign to the given output stream
    * @param acmeDesign the design
    * @param os the stream to write to
    */
	public static void getOutputStream(AcmeDesign acmeDesign, OutputStream os) {
		try {
			AcmeOutputStream aos = new AcmeOutputStream();
			acmeDesign.exportArmani(aos);
			aos.writeTo(os);
			aos.close();
		}
		catch (Exception e) {
			Debug.getLog().error("Error occurs when parsing acme script file: ", e);
		}
	}

	/**
	 * @deprecated use getAcmeDesignFromInputStream instead
	 * @param acmeDesignStr
	 * @return
	 */
	public DefaultMutableTreeNode parseAcmeScript(String acmeDesignStr) {
		try {
			AcmeDesign acmeDesignObj = null;
			// clear out all old errors so that we'll know which ones were
			// caused by this typechecking run.
			AcmeErrorManager.clearAllErrors();
			// Parse the Acme design that is encoded in acmeDesignStr
			java.io.ByteArrayInputStream compInputStream = new java.io.ByteArrayInputStream(acmeDesignStr.getBytes());
			try {
				new AcmeParser(compInputStream);
			}
			catch (java.lang.Error e) {
				// The parser exists, but was created by another program
				acmeLib.acmeParser.AcmeParser.ReInit(compInputStream);
			}
			acmeLib.acmeParser.AcmeParser.disable_tracing();
			acmeLib.acmeParser.AcmeParser.initDesign();
			try {
				acmeDesignObj = acmeLib.acmeParser.AcmeParser.parse_AcmeDesign();
			}
			catch (acmeLib.acmeParser.ParseError apErr) {
				AcmeError newParseError =
					new AcmeError(acmeDesignObj, "Error encountered parsing AcmeDesign:  " + apErr.getMessage());
				AcmeErrorManager.errorDetected(newParseError);
				return null;
			}
			// Translate the design from Acme to Armani
			Acme2Armani.translate(acmeDesignObj);
			DefaultMutableTreeNode rootNode = convertToTreeNode(acmeDesignObj);
			return rootNode;
		}
		catch (Exception e) {
			Debug.getLog().error("Error occurs parsing the acme script: ", e);
			return null;
		}
	}

	/**
	 * Parses and resolves the acme text that is in the fis
	 * @param monitor The eclipse monitor to use to report progress (can be null)
	 * @param fis The input stream to get the acme text
	 * @return The Acme design object
	 * @throws ParseException If there is a parse error in the file
	 */
	public static AcmeDesign getAcmeDesignFromInputStream (IProgressMonitor monitor, InputStream fis) throws ParseException {
		return Helper.getAcmeDesignFromInputStream(monitor, fis, true);
	}
	
	/**
	 * Parses the acme text that is in the input stream, and optionally resolves the references
	 * @param monitor The Eclipse monitor to use to report progress
	 * @param fis THe input stream containing the Acme text
	 * @param resolve Whether to resolve the names
	 * @return The Acme design object
	 * @throws ParseException If there is a parse error
	 */
	public static AcmeDesign getAcmeDesignFromInputStream(IProgressMonitor monitor, InputStream fis, boolean resolve)
		throws ParseException {
		try {
			if (monitor != null)
				monitor.subTask("Updating paths...");
			AcmeStudioPlugin.updatePathVars(); // get env't paths setup
			if (monitor != null) {
				monitor.worked(1);
				monitor.subTask("Parsing...");
			}
			AcmeDesign acmeDesignObj = null;
			// clear out all old errors so that we'll know which ones were
			// caused by this typechecking run.
			AcmeErrorManager.clearAllErrors();
			if (!ArmaniParser.parserConstructed()) {
				new ArmaniParser(fis);
			}
			ArmaniParser.disable_tracing();
			ArmaniParser.initDesign();
			ArmaniParser.ReInit(fis);
			try {
				acmeDesignObj = ArmaniParser.parse_AcmeDesign();
				
				AcmeEnumeration errors = AcmeErrorManager.getErrors ();
				while (errors.moreItems ()) {
					AcmeError error = (AcmeError) errors.getNextItem ();
					if (error instanceof AcmeParseError) {
						AcmeParseError ape = (AcmeParseError) error;
						throw ape.getException();
					}
				}
			}
			/*catch (ParseException apErr) {
				AcmeError newParseError =
					new AcmeError(acmeDesignObj, "Error encountered parsing AcmeDesign:  " + apErr.getMessage());
				AcmeErrorManager.errorDetected(newParseError);
				throw apErr;
			}*/
			catch (Error e) {
				ParseException pE = new ParseException(e.getMessage());
				pE.currentToken = ArmaniParser.jj_nt;
				throw pE;
			}
			if (resolve) {
				resolveAllInDesign(monitor, acmeDesignObj);
			}
			if (monitor != null)
				monitor.worked(1);
			return acmeDesignObj;
		}
		catch (ParseException pErr) {
			if (monitor != null)
				monitor.setCanceled(true);
			throw pErr;
		}
		catch (Exception e) {
			if (monitor != null)
				monitor.setCanceled(true);
			Debug.getLog().error("Error occurs parsing the acme script: ", e);
			return null;
		}
		catch (Error e) {
			if (monitor != null) {
				monitor.setCanceled(true);
			}
			Debug.getLog().error("Error occurs parsing the acme script: ", e);
			return null;
		}
	}

	/**
	 * Resolves all the references in the design object (to types, elements, etc)
	 * @param monitor The monitor to use to report progress
	 * @param acmeDesignObj The design object to resolve references in
	 * @throws acmeLib.acmeParser.ParseException
	 * @throws ParseException
	 */
	public static void resolveAllInDesign(IProgressMonitor monitor, AcmeDesign acmeDesignObj) throws acmeLib.acmeParser.ParseException, ParseException {
		if (monitor != null) {
			monitor.worked(1);
			monitor.subTask("Resolving imports...");
		}
		acmeDesignObj.resolveImportedDesigns();
		if (monitor != null) {
			monitor.worked(1);
			monitor.subTask("Resolving types...");
		}
		acmeDesignObj.resolveTypeReferences();
		if (monitor != null) {
			monitor.worked(1);
			monitor.subTask("Instantiating elements...");
		}
		acmeDesignObj.instantiateElements();
		if (monitor != null) {
			monitor.worked(1);
			monitor.subTask("Resolving instances...");
		}
		acmeDesignObj.resolveInstanceReferences();
		if (monitor != null) {
			monitor.worked(1);
			monitor.subTask("Translating to Armani...");
		}
		if (AcmeErrorManager.getNumErrorsReported() == 0) {
			// Translate the design from Acme to Armani
			Acme2Armani.translate(acmeDesignObj);
		}
	}
	/**
	 * @deprecated use getAcmeDesignFromInputStream
	 * @param acmeDesignStr
	 * @return
	 */
	public static AcmeDesign parseAcme(String acmeDesignStr) {
		try {
			AcmeStudioPlugin.updatePathVars(); // get env't paths setup
			AcmeDesign acmeDesignObj = null;
			// clear out all old errors so that we'll know which ones were
			// caused by this typechecking run.
			AcmeErrorManager.clearAllErrors();
			// Parse the Acme design that is encoded in acmeDesignStr
			java.io.ByteArrayInputStream compInputStream = new java.io.ByteArrayInputStream(acmeDesignStr.getBytes());
			try {
				new AcmeParser(compInputStream);
			}
			catch (java.lang.Error e) {
				// The parser exists, but was created by another program
				acmeLib.acmeParser.AcmeParser.ReInit(compInputStream);
			}
			acmeLib.acmeParser.AcmeParser.disable_tracing();
			acmeLib.acmeParser.AcmeParser.initDesign();
			try {
				acmeDesignObj = acmeLib.acmeParser.AcmeParser.parse_AcmeDesign();
			}
			catch (acmeLib.acmeParser.ParseError apErr) {
				AcmeError newParseError =
					new AcmeError(acmeDesignObj, "Error encountered parsing AcmeDesign:  " + apErr.getMessage());
				AcmeErrorManager.errorDetected(newParseError);
				return null;
			}
			// Translate the design from Acme to Armani
			Acme2Armani.translate(acmeDesignObj);
			return acmeDesignObj;
		}
		catch (Exception e) {
			Debug.getLog().error("Error occurs parsing the acme script: ", e);
			return null;
		}
	}
	private void convertToTreeNode(DefaultMutableTreeNode rootNode, AcmeSystem acmeSystem) {
		DefaultMutableTreeNode elementNode = null;
		DefaultMutableTreeNode categoryNode = null;
		AcmeEnumeration componentEnum = acmeSystem.getComponents();
		categoryNode = new DefaultMutableTreeNode("Components");
		rootNode.add(categoryNode);
		while (componentEnum.moreItems()) {
			AcmeComponent acmeComponent = (AcmeComponent) componentEnum.getNextItem();
			elementNode = new DefaultMutableTreeNode(acmeComponent);
			categoryNode.add(elementNode);
		}
		AcmeEnumeration connectorEnum = acmeSystem.getConnectors();
		categoryNode = new DefaultMutableTreeNode("Connectors");
		rootNode.add(categoryNode);
		while (connectorEnum.moreItems()) {
			AcmeConnector acmeConnector = (AcmeConnector) connectorEnum.getNextItem();
			elementNode = new DefaultMutableTreeNode(acmeConnector);
			categoryNode.add(elementNode);
		}
		AcmeEnumeration attachmentEnum = acmeSystem.getAttachments();
		categoryNode = new DefaultMutableTreeNode("Attachments");
		rootNode.add(categoryNode);
		while (attachmentEnum.moreItems()) {
			AcmeAttachment acmeAttachment = (AcmeAttachment) attachmentEnum.getNextItem();
			elementNode = new DefaultMutableTreeNode(acmeAttachment);
			categoryNode.add(elementNode);
		}
	}
	private DefaultMutableTreeNode convertToTreeNode(AcmeDesign acmeDesign) {
		AcmeEnumeration systemEnum = new AcmeEnumeration();
		systemEnum.concatenate(acmeDesign.getSystems());
		if (systemEnum.moreItems()) {
			AcmeSystem acmeSystem = (AcmeSystem) systemEnum.getNextItem();
			currentAcmeSystem = acmeSystem;
			DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(acmeSystem);
			convertToTreeNode(rootNode, acmeSystem);
			return rootNode;
		}
		return null;
	}
	
	/**
	 * Returns the first Acme system in the design
	 * @param acmeDesign The design
	 * @return The first acme system
	 */
	public static AcmeSystem getAcmeSystem(AcmeDesign acmeDesign) {
		AcmeEnumeration systemEnum = new AcmeEnumeration();
		systemEnum.concatenate(acmeDesign.getSystems());
		if (systemEnum.moreItems()) {
			AcmeSystem acmeSystem = (AcmeSystem) systemEnum.getNextItem();
			return acmeSystem;
		}
		return null;
	}
	
	/**
	 * Gets the AcmeSystem that is the parent of this child (or the system itself if the child is a system)
	 * @param child The element contained in a system
	 * @return The system containing the child
	 */
	public static AcmeSystem getAcmeSystem(AcmeElement child) {
		if (child == null || child instanceof AcmeSystem)
			return (AcmeSystem) child;
		AcmeElement parent = child.getParent();
		while (parent != null && !(parent instanceof AcmeSystem)) {
			parent = parent.getParent();
		}
		return (AcmeSystem) parent;
	}
	
	/**
	 * Gets all the systems in the design
	 * @param acmeDesign
	 * @return The list of systems that are contained in the design
	 */
	public static List getAcmeSystems(AcmeDesign acmeDesign) {
		AcmeEnumeration systemEnum = new AcmeEnumeration();
		systemEnum.concatenate(acmeDesign.getSystems());
		List sysList = new ArrayList();
		if (systemEnum.moreItems()) {
			sysList.add(systemEnum.getNextItem());
		}
		return sysList;
	}
	
	/**
	 *  Gets all the bindings into a system from a parent representation (if the system is contained in a representation) or an empty vector otherwise
	 * @param acmeSystem The system to get the bindings for
	 * @return The list of AcmeBindings
	 */
	public static List getBindings(AcmeSystem acmeSystem) {
		Vector bindingVec = new Vector();
		AcmeRepresentation containingRep = acmeSystem.getParentRepresentation();
		if (containingRep != null) {
			AcmeBindingsMap bindings = (AcmeBindingsMap) containingRep.getAbstractionMap();
			if (bindings != null) {
				AcmeEnumeration b = bindings.getBindings();
				while (b.moreItems()) {
					bindingVec.add(b.getNextItem());
				}
			}
		}
		return bindingVec;
	}
	
	/**
	 * For a given system, get the list of potential bindings into that system from the parent. For example, if the system is a representation
	 * of a component with ports p1 and p2, but there is only a binding into the system from p1, then return an empty binding for that port
	 * @param acmeSystem The system that contains the bindings
	 * @param existingBindings Already known bindings
	 * @return A list of AcmeBindings that are dangling
	 */
	public static List getAdditionalBindingPoints(AcmeSystem acmeSystem, List existingBindings) {
		List bindingVec = new Vector();
		Map additional = new Hashtable();
		AcmeRepresentation rep = acmeSystem.getParentRepresentation();
		if (rep != null) {
			if (rep.getParent() instanceof AcmeComponent) {
				// Fill additional map
				AcmeComponent parentComp = (AcmeComponent) rep.getParent();
				AcmeEnumeration ports = parentComp.getPorts();
				while (ports.moreItems()) {
					AcmePort port = (AcmePort) ports.getNextItem();
					additional.put(port.getName(), new AcmeBinding(port, null));
				}
				// delete if there is an existing binding
				Iterator binding = existingBindings.iterator();
				while (binding.hasNext()) {
					AcmeBinding b = (AcmeBinding) binding.next();
					if (b == null || b.getOuter () == null) continue;
					additional.remove(b.getOuter().getName());
				}
				binding = additional.keySet().iterator();
				while (binding.hasNext()) {
					bindingVec.add(additional.get(binding.next()));
				}
			}
			else if (rep.getParent() instanceof AcmeConnector) {
				AcmeConnector parentConn = (AcmeConnector) rep.getParent();
				AcmeEnumeration roles = parentConn.getRoles();
				while (roles.moreItems()) {
					AcmeRole role = (AcmeRole) roles.getNextItem();
					additional.put(role.getName(), new AcmeBinding(role, null));
				}

				Iterator binding = existingBindings.iterator();
				while (binding.hasNext()) {
					AcmeBinding b = (AcmeBinding) binding.next();
					additional.remove(b.getOuter().getName());
				}
				binding = additional.keySet().iterator();
				while (binding.hasNext()) {
					bindingVec.add(additional.get(binding.next()));
				}
			}
		}
		return bindingVec;
	}
	
	/**
	 * Converts an AcmeEnumeration to a java.util.List
	 * @param enum The enumration to convert
	 * @return The list containing all the elements in the enumeration
	 */
	public static List convertAcmeEnumerationToList(AcmeEnumeration enum) {
		Vector list = new Vector();
		while (enum.moreItems()) {
			list.add(enum.getNextItem());
		}
		return list;
	}

	/**
	 * Converts a java.util.Collection to an AcmeEnumeration
	 * @param c The collection to convert (must contain AcmeObjects)
	 * @return The AcmeEnumeration containing all the objects that were in c
	 */
	public static AcmeEnumeration convertCollectionToAcmeEnumeration(Collection c) {
		AcmeVector enum = new AcmeVector();
		for (Iterator iter = c.iterator(); iter.hasNext();) {
			AcmeObject obj = (AcmeObject) iter.next();
			enum.addItem(obj);
		}
		return enum.enumerate();
	}
	
	/**
	 * Gets all the children (components, connectors, ports, roles) contained in a system
	 * @param acmeSystem The acme system
	 * @return The vector of children
	 */
	public static Vector getChildren(AcmeSystem acmeSystem) {
		Vector v = new Vector();
		AcmeEnumeration enum = new AcmeEnumeration();
		int numComponents = 0;
		int numConnectors = 0;
		float avgRoles = 0;
		float avgPorts = 0;
		int numAttachments = 0;
		enum.concatenate(acmeSystem.getConnectors());
		while (enum.moreItems()) {
			AcmeConnector e = (AcmeConnector) enum.getNextItem();
			v.add(e);
			v.addAll(getChildren(e));
			numConnectors++;
			avgRoles+=e.getRoles().getSize ();
		}
		
		enum = new AcmeEnumeration();
		enum.concatenate(acmeSystem.getComponents());
		while (enum.moreItems()) {
			AcmeComponent e = (AcmeComponent) enum.getNextItem();
			v.add(e);
			numComponents++;
			avgPorts += e.getPorts().getSize ();
			v.addAll(getChildren(e));
		}

//		Debug.diagnoseln ("Helper.getChildren: " + "#components=" + numComponents + ", avgPorts =" + avgPorts/numComponents);
//		Debug.diagnoseln ("Helper.getChildren: " + "#connectors=" + numConnectors + ", avgRoles = " + avgRoles/numConnectors);
//		Debug.diagnoseln ("Helper.getChildren: " + "#attachments=" + numAttachments);
		return v;
	}
	
	static final Comparator ORDER_COMPARATOR = new Comparator () {

		public int compare(Object o1, Object o2) {
//			if ((o1 instanceof AcmePort && o2 instanceof AcmeComponent) || (o1 instanceof AcmeRole && o2 instanceof AcmeConnector)) return -1;
			if (o1 instanceof AcmeElement && o2 instanceof AcmeElement) {
				AcmeElement e1 = (AcmeElement )o1;
				AcmeElement e2 = (AcmeElement )o2;
				int zOrder1 = Helper.getZOrder (e1);
				int zOrder2 = Helper.getZOrder (e2);
				if (zOrder2 - zOrder1 < 0) return 1; 
				else if (zOrder2 - zOrder1 > 0) return -1;
				else return e2.hashCode() - e1.hashCode();
			}
			else {
				return -1;
			}
		}
	};

	private static final int	ROLE_LINE_SIZE	= 13;

	private static final ElementVisualization	GHOST_COMPONENT_VIS	= new ComponentVisualization ();
	static {
		GHOST_COMPONENT_VIS.setShape(Shape.RECTANGLE);
		GHOST_COMPONENT_VIS.setSize(new Dimension (100, 50));
		GHOST_COMPONENT_VIS.setFillColor(Constants.TRANSPARENT_COLOR);
		GHOST_COMPONENT_VIS.setOutlineStyle(SWT.LINE_DOT);
	}
	
	public static LinkedList getOrderedChildren (AcmeSystem system) {
		Map childMap = new TreeMap (ORDER_COMPARATOR);
		
		AcmeEnumeration enum = system.getConnectors ();
		while (enum.moreItems ()) {
			AcmeConnector connector = (AcmeConnector )enum.getNextItem ();
			List roles = convertAcmeEnumerationToList(connector.getRoles ());
			Collections.sort(roles, ORDER_COMPARATOR);
			childMap.put (connector, roles);
		}
		
		enum = system.getComponents ();
		while (enum.moreItems ()) {
			AcmeComponent component = (AcmeComponent )enum.getNextItem ();
			List ports= convertAcmeEnumerationToList(component.getPorts ());
			Collections.sort (ports, ORDER_COMPARATOR);
			childMap.put (component, ports);
		}
		
		LinkedList children = new LinkedList ();
		for (Iterator iter = childMap.keySet().iterator (); iter.hasNext ();) {
			AcmeElement element = (AcmeElement )iter.next ();
			List childElements = (List )childMap.get (element);
			childElements.add(0, element);
			children.add (childElements);
		}
		
		return children;
	}
	/**
	 * Gets all the ports cotnained in components in the AcmeSystem
	 * @param acmeSystem
	 * @return The vector of all ports
	 */
	public static Vector getPorts(AcmeSystem acmeSystem) {
		Vector v = new Vector();
		AcmeEnumeration enum = new AcmeEnumeration();
		enum.concatenate(acmeSystem.getComponents());
		while (enum.moreItems()) {
			AcmeComponent e = (AcmeComponent) enum.getNextItem();
			v.addAll(getChildren(e));
		}
		return v;
	}
	
	/**
	 * Gets all the attachments in the system
	 * @param acmeSystem
	 * @return
	 */
	public static Vector getAttachments(AcmeSystem acmeSystem) {
		Vector v = new Vector();
		AcmeEnumeration enum = new AcmeEnumeration();
		enum.concatenate(acmeSystem.getAttachments());
		while (enum.moreItems()) {
			AcmeAttachment e = (AcmeAttachment) enum.getNextItem();
			v.add(e);
		}
		return v;
	}

	/**
	 * Gets all the ports of the AcmeComponent
	 * @param acmeComponent
	 * @return
	 */
	public static Vector getChildren(AcmeComponent acmeComponent) {
		AcmeEnumeration enum = new AcmeEnumeration();
		enum.concatenate(acmeComponent.getPorts());
		Vector v = new Vector();
		while (enum.moreItems()) {
			v.add(enum.getNextItem());
		}
		return v;
	}
	
	/**
	 * Gets all the ports of a component in order 
	 * @param acmeComponent
	 * @return
	 */
	public static Vector getChildrenByOrder(AcmeComponent acmeComponent) {
		if (!arePortsOrdered(acmeComponent)) {
			sortPorts(acmeComponent);
		}
		AcmeEnumeration enum = acmeComponent.getPorts();
		Vector v = new Vector();
		AcmeElement[] ap = new AcmeElement[enum.getSize()];
		int order;
		while (enum.moreItems()) {
			AcmePort e = (AcmePort) enum.getNextItem();
			order = getOrder(e);
			if (ap[order] == null) {
				ap[order] = e;
			}
			else {
				while (++order < ap.length && ap[order] != null);
				if (order < ap.length) ap[order] = e;
			}
		}
		for (int i = 0; i < ap.length; i++) {
			v.add(ap[i]);
		}
		return v;
	}
	
	private static boolean arePortsOrdered(AcmeComponent acmeComponent) {
		AcmeEnumeration enum = acmeComponent.getPorts();
		int order = -1;
		boolean ordered = true;
		while (enum.moreItems()) {
			AcmePort e = (AcmePort) enum.getNextItem();
			order = getOrder(e);
			if (order == -1 || order >= enum.getSize()) {
				ordered = false;
				break;
			}
		}
		return ordered;
	}
	
	private static void sortPorts(AcmeComponent acmeComponent) {
		AcmeEnumeration enum = acmeComponent.getPorts();
		int i = 0;
		while (enum.moreItems()) {
			AcmePort e = (AcmePort) enum.getNextItem();
			setOrder(e, i++);
		}
	}
	/**
	 * Exchanges the order of ports in the component
	 * @param acmeComponent The component containing the ports
	 * @param port1 The first port
	 * @param port2 The second port
	 */
	public static void exchangePortOrder(AcmeComponent acmeComponent, String port1, String port2) {
		if (!arePortsOrdered(acmeComponent)) {
			sortPorts(acmeComponent);
		}
		AcmeEnumeration enum = acmeComponent.getPorts();
		int order1 = -1;
		int order2 = -1;
		AcmePort e1 = null;
		while (enum.moreItems()) {
			AcmePort e = (AcmePort) enum.getNextItem();
			if (e.getName().equals(port1) || e.getName().equals(port2)) {
				if (order1 == -1) {
					e1 = e;
					order1 = getOrder(e);
				}
				else {
					order2 = getOrder(e);
					setOrder(e, order1);
					break;
				}
			}
		}
		setOrder(e1, order2);
	}
	
	/**
	 * Gets all the roles in the connector
	 * @param acmeConnector
	 * @return
	 */
	public static Vector getChildren(AcmeConnector acmeConnector) {
		AcmeEnumeration enum = new AcmeEnumeration();
		enum.concatenate(acmeConnector.getRoles());
		Vector v = new Vector();
		while (enum.moreItems()) {
			v.add(enum.getNextItem());
		}
		return v;
	}
	
	
	private  HashMap getPropertyMap(AcmeEnumeration enum) {
		HashMap propertyMap = new HashMap();
		while (enum.moreItems()) {
			AcmeProperty acmeProperty = (AcmeProperty) enum.getNextItem();
			String name = acmeProperty.getName();
			String type = acmeProperty.getType().getTypeName();
			Object value = null;
			AcmePropertyValue propertyValue = acmeProperty.getValue();
			if (propertyValue == null)
				continue;
			if (type.equals("string")) {
				value = ((AcmeString) propertyValue).getValue();
			}
			else if (type.equals("int")) {
				value = Integer.toString(((AcmeInt) propertyValue).getValue());
			}
			else if (type.equals("float")) {
				value = Float.toString(((AcmeFloat) propertyValue).getValue());
			}
			propertyMap.put(name, value);
		}
		return propertyMap;
	}
	private HashMap getPropertyMap(String elementName) {
		if (elementName == null)
			return null;
		AcmeSystem acmeSystem = currentAcmeSystem;
		if (elementName.equals(acmeSystem.getName())) {
			return getPropertyMap(acmeSystem.getProperties());
		}
		AcmeEnumeration enum = acmeSystem.getComponents();
		while (enum.moreItems()) {
			AcmeComponent acmeElement = (AcmeComponent) enum.getNextItem();
			if (elementName.equals(acmeElement.getName())) {
				return getPropertyMap(acmeElement.getProperties());
			}
		}
		enum = acmeSystem.getConnectors();
		while (enum.moreItems()) {
			AcmeConnector acmeElement = (AcmeConnector) enum.getNextItem();
			if (elementName.equals(acmeElement.getName())) {
				return getPropertyMap(acmeElement.getProperties());
			}
		}
		return null;
	}

	private static void setOrder(AcmePort element, int order) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
		AcmeProperty visOrder = element.getProperty("vis-order");
		if (visOrder == null) {
			visOrder = new AcmeProperty(new IntType("int"));
			try {
				visOrder.setName("vis-order");
				element.addProperty(visOrder);
			}
			catch (AcmeIllegalNameException e) {
				return;
			}
		}
		visOrder.setValue(new AcmeInt(order));
		}
		else {
			VisualInformation vis = (VisualInformation) element.getUserData ("vis-information");
			PortVisualInformation pVis = null;
			if (vis == null || !(vis instanceof PortVisualInformation)) {
				pVis = new PortVisualInformation (vis);
				element.setUserData ("vis-information", pVis);
			}
			else {
				pVis = (PortVisualInformation )vis;
			}
			pVis.setOrder (order);
		}
	}
	
	/** 
	 * Sets the x location of the element
	 * @param element
	 * @param x
	 */
	public static void setX(AcmeElement element, int x) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visX = element.getProperty("vis-x");
			if (visX == null) {
				visX = new AcmeProperty(new FloatType("float"));
				try {
					visX.setName("vis-x");
					element.addProperty(visX);
				}
				catch (AcmeIllegalNameException e) {
					return;
				}
			}
			visX.setValue(new AcmeFloat(x));
		}
		else {
			VisualInformation vis = getVisualInformation(element);
			vis.setX(x);
		}
	}
	
	/**
	 * Gets the visual information associated with the element, and creates and associates it if it doesn't exist
	 * @param element
	 * @return
	 */
	public static VisualInformation getVisualInformation(AcmeElement element) {
		VisualInformation vis = null;
		if (element.getProperty ("vis-x") != null) {
			vis = new VisualInformation ();
			vis.extractFromElement(element);
		}
		vis = (VisualInformation )element.getUserData ("vis-information");
		if (vis == null) {
			vis = new VisualInformation ();
			element.setUserData ("vis-information", vis);
		}
		return vis;
	}
	
	/**
	 * Sets the y location of the element
	 * @param element
	 * @param y
	 */
	public static void setY(AcmeElement element, int y) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visY = element.getProperty("vis-y");
		
			if (visY == null) {
				visY = new AcmeProperty(new FloatType("float"));
				try {
					visY.setName("vis-y");
					element.addProperty(visY);
				}
				catch (AcmeIllegalNameException e) {
					return;
				}
			}
			visY.setValue(new AcmeFloat(y));
		}
		VisualInformation vis = getVisualInformation (element);
		vis.setY (y);
	}
	
	/**
	 * Clears the x location of the element
	 * @param element
	 */
	public static void clearX(AcmeElement element) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			
			AcmeProperty visX = element.getProperty("vis-x");
	
			if (visX != null) {
				element.removeProperty("vis-x");
			}
		}
		else {
			VisualInformation vis = getVisualInformation(element);
			vis.setX (VisualInformation.NO_POS_VALUE);
		}
	}
	
	/**
	 * Clears the y location of the element
	 * @param element
	 */
	public static void clearY(AcmeElement element) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visY = element.getProperty("vis-y");

			if (visY != null) {
				element.removeProperty("vis-y");
			}
		}
		else {
			VisualInformation vis = getVisualInformation (element);
			vis.setY (VisualInformation.NO_POS_VALUE);
		}
	}
	
	/**
	 * Clears all the bendpoints associated with the rol
	 * @param element
	 */
	public static void clearBendpoints(AcmeRole element) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visX = element.getProperty("vis-bendpoints");
	
			if (visX != null) {
				element.removeProperty("vis-bendpoints");
			}
		}
		else {
			VisualInformation vis = (VisualInformation )element.getUserData ("vis-information");
			if (vis != null) {
				if (!(vis instanceof RoleVisualInformation)) {
					vis = new RoleVisualInformation (vis);
					element.setUserData("vis-information", vis);
				}
				RoleVisualInformation rVis = (RoleVisualInformation )vis;
				if (rVis != null) {
					rVis.setBendPoints(null);
				}
			}
		}
	}
	public static void setDescription(AcmeElement element, String description) {
		//        AcmeProperty visDesc = element.getProperty("vis-description");
		//        if (visDesc == null) {
		//            visDesc = new AcmeProperty (new StringType ("string"));
		//            try {
		//                visDesc.setName ("vis-description");
		//                element.addProperty (visDesc);
		//            }
		//            catch (AcmeIllegalNameException e) {
		//                // Done all the prechecks to make sure this never happens
		//            }
		//        }
		//        visDesc.setValue (new AcmeString (description));
		element.setComment("// " + description);
		AcmeStudioChangeManager.globalChangeManager.onSetComment(element, description, null);
	}
	
	/**
	 * Gets the x location of the element
	 * @param acmeElement
	 * @return
	 */
	public static Point getX(AcmeElement acmeElement) {
		Point point = new Point(0,0);
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visX = acmeElement.getProperty("vis-x");
			if (visX == null) {
				return null;
			}
			point = new Point();
			Object vX = visX.getValue();
			if (vX != null) {
				int x = (int) ((AcmeFloat) visX.getValue()).getValue();
				point.setLocation(x, 0);
			}
		}
		else {
			
			VisualInformation vis = getVisualInformation(acmeElement);
			if (vis.getX () == VisualInformation.NO_POS_VALUE) return null;
			point.x = vis.getX ();
		}
		return point;
	}
	
	/**
	 * Gets the bendpoints for the role
	 * @param acmeRole
	 * @return
	 */
	public static PointList getBendpoints(AcmeRole acmeRole) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visBendpoints = acmeRole.getProperty("vis-bendpoints");
			if (visBendpoints == null) {
				return null;
			}
	
			return convertBendpointPropertyToPointList(visBendpoints);
		}
		else {
			RoleVisualInformation rVis = null;
			if (acmeRole.getProperty("vis-bendpoints") != null) {
				rVis = new RoleVisualInformation (null);
				rVis.extractFromElement(acmeRole);
			}
			VisualInformation vis = getVisualInformation(acmeRole);
			if ( !(vis instanceof RoleVisualInformation)) {
				return null;
			}
			else {
				rVis = (RoleVisualInformation )vis;
			}
			return rVis.getBendPoints();
		}
	}
	
	/**
	 * Converts the bendpoints specified as a string in the AcmeProperty to a PointList
	 * @param visBendpoints
	 * @return
	 */
	public static PointList convertBendpointPropertyToPointList(AcmeProperty visBendpoints) {
		PointList pl = new PointList();
		Object v = visBendpoints.getValue();
		String s = null;
		if (v != null) {
			s = ((AcmeString) visBendpoints.getValue()).getValue();
		}
		if (s == null) {
			return null;
		}
		String[] ps = s.split(" ");
		for (int i = 0; i < ps.length; i++) {
			String p = ps[i];
			String[] xy = p.split(",");
			if (xy.length < 2)
				continue;
			try {
				int x = Integer.parseInt(xy[0]);
				int y = Integer.parseInt(xy[1]);
				Point bendpoint = new Point(x, y);
				pl.addPoint(bendpoint);
			}
			catch (Exception e) {
				continue;
			}
		}
		if (pl.size() == 0)
			return null;
		return pl;
	}
	
	/**
	 * Sets teh bendpoints for the role
	 * @param acmeRole
	 * @param bendpoints
	 */
	public static void setBendpoints(AcmeRole acmeRole, PointList bendpoints) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			if (bendpoints == null)
		
					return;
				AcmeProperty visBendpoints = acmeRole.getProperty("vis-bendpoints");
				if (visBendpoints == null) {
					visBendpoints = new AcmeProperty(new StringType("string"));
					try {
						visBendpoints.setName("vis-bendpoints");
						acmeRole.addProperty(visBendpoints);
					}
					catch (AcmeIllegalNameException e) {
						return;
					}
				}
				String s = "";
				for (int i = 0; i < bendpoints.size(); i++) {
					Point p = bendpoints.getPoint(i);
					String l = "" + p.x + "," + p.y;
					s = s.equals("") ? l : s + " " + l;
			}
			visBendpoints.setValue(new AcmeString(s));
		}
		else {
			VisualInformation vis = getVisualInformation(acmeRole);
			RoleVisualInformation rVis = null;
			if ( !(vis instanceof RoleVisualInformation)) {
				rVis = new RoleVisualInformation (vis);
				acmeRole.setUserData("vis-information", rVis);
			}
			else {
				rVis = (RoleVisualInformation )vis;
			}
			rVis.setBendPoints(bendpoints);
		}
	}
	
	/**
	 * Adds a bendpoint for the role
	 * @param acmeRole The role
	 * @param index Where to add the bendpoint
	 * @param bendpoint The bendpoint to add
	 * @param bendpoints The bendpoints list
	 * @return
	 */
	public static PointList addBendpoint(
		AcmeRole acmeRole,
		int index,
		Point bendpoint,
		PointList bendpoints) {
		if (bendpoints == null)
			bendpoints = new PointList();
		if (bendpoints.size() >= index)
			bendpoints.insertPoint(bendpoint, index);
		return bendpoints;
	}
	
	// TODO: Check the difference between setBendpoint and addBendpoint
	/**
	 * Sets the bendpoint in the role
	 * @param acmeRole
	 * @param index
	 * @param bendpoint
	 * @param bendpoints
	 * @return
	 */
	public static PointList setBendpoint(AcmeRole acmeRole, int index, Point bendpoint, PointList bendpoints) {
		if (bendpoints != null && bendpoints.size() > index) {
			bendpoints.setPoint(bendpoint, index);
			setBendpoints(acmeRole, bendpoints);
		}
		return bendpoints;
	}
	/**
	 * Gets the routing rule for the role
	 * @param acmeRole
	 * @return
	 */
	public static ConnectorVisualization.RoleRouteRule getRoleRouteRule(AcmeRole acmeRole) {
		ConnectorVisualization.RoleRouteRule rule = null;
		if (acmeRole != null) {
			AcmeElement obj = acmeRole.getParent();
			if (obj instanceof AcmeConnector) {
				AcmeConnector acmeConnector = (AcmeConnector) obj;
				Vector roles = Helper.getChildren(acmeConnector);
				List rules = Helper.getRoleRouteRules(roles, acmeConnector);
				for (int i = 0; i < roles.size(); i++) {
					AcmeRole role = (AcmeRole) roles.get(i);
					if (acmeRole.equals(role) && rules.size() > i) {
						rule = (ConnectorVisualization.RoleRouteRule) rules.get(i);
						break;
						//return rule;
					}
				}
			}
		}
		if (rule != null && rule._roleRoute != ConnectorVisualization.RoleRouteOptions.STRAIGHT)
			return rule;
		else
			return null;
	}
	

	
	/**
	 * Removes a bendpoint
	 * @param acmeRole
	 * @param index
	 * @param roleStart
	 * @param roleEnd
	 */
	public static synchronized void removeBendpoint(AcmeRole acmeRole, int index, Point roleStart, Point roleEnd) {
		PointList bendpoints = getBendpoints(acmeRole);
		if (bendpoints != null && bendpoints.size() > index && bendpoints.getPoint(index) != null) {
			bendpoints.removePoint(index);
			setBendpoints(acmeRole, bendpoints);
		}
	}
	
	/**
	 * Gets a port's order
	 * @param acmeElement
	 * @return
	 */
	private static int getOrder(AcmePort acmeElement) {
		int x = -1;
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visX = acmeElement.getProperty("vis-order");
			if (visX == null) {
				return -1;
			}
			Object vX = visX.getValue();
			if (vX != null) {
				x = ((AcmeInt) visX.getValue()).getValue();
			}
			
		}
		else {
			VisualInformation vis = null;
			PortVisualInformation pVis = null;
			vis = (VisualInformation )acmeElement.getUserData ("vis-information");
			
			if (acmeElement.getProperty("vis-order") != null) {
				pVis = new PortVisualInformation (null);
				pVis.extractFromElement(acmeElement);
			}
			 vis = getVisualInformation(acmeElement);
			if ( !(vis instanceof PortVisualInformation)) {
				return -1;
			}
			else {
				pVis = (PortVisualInformation )vis;
			}
			if (vis != null && (vis instanceof PortVisualInformation)) {
				pVis = (PortVisualInformation )vis;
				x = pVis.getOrder();
			}
		}
		return x;
	}
	
	/**
	 * Gets the y location of an element
	 * @param acmeElement
	 * @return
	 */
	public static Point getY(AcmeElement acmeElement) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			Point point = new Point();
			AcmeProperty visY = acmeElement.getProperty("vis-y");
			if (visY == null) {
				return null;
			}
			point = new Point();
			Object vY = visY.getValue();
			if (vY != null) {
				int y = (int) ((AcmeFloat) visY.getValue()).getValue();
				point.setLocation(0, y);
			}
			return point;
		}
		else {
			Point point = new Point (0,0);
			VisualInformation vis = (VisualInformation )acmeElement.getUserData ("vis-information");
			if (vis.getY () == VisualInformation.NO_POS_VALUE) return null;
			point.y = vis.getY ();
			return point;
		}
	}
	public static String getDescription(AcmeElement acmeElement) {
		String description = acmeElement.getComment();
		if (description == null)
			description = "";
		if (description.length() >= 2 && description.substring(0, 2).equals("//")) {
			description = description.substring(2);
		}
		// Bugfix #91
		return description.trim();
	}
	
	/**
	 * Gets the size of an element
	 * @param element
	 * @return
	 */
	public static Dimension getSize(AcmeElement element) {
		AcmeProperty visWidth = element.getProperty("vis-width");
		AcmeProperty visHeight = element.getProperty("vis-height");
		
		if (visWidth == null || visHeight == null) {
			VisualInformation vis = (VisualInformation )element.getUserData("vis-information");
			if (vis != null) {
				if (vis.getHeight() > 0 && vis.getWidth () > 0) {
					return vis.getSize ();
				}
			}
		}
		
		if (visWidth == null || visHeight == null) {
			ElementVisualization vis = VisualStyleRegistry.getVisualization(element);
			if (vis != null) {
				
				return vis.getSizeWithLabel(element);
			}
			else {
				return null;
			}
		}
		if (visWidth.getValue() != null && visHeight.getValue() != null) {
			float fVisWidth = ((AcmeFloat) visWidth.getValue()).getValue();
			float fVisHeight = ((AcmeFloat) visHeight.getValue()).getValue();
			return new Dimension((int) fVisWidth, (int) fVisHeight);
		}
		else {
			return null;
		}
	}
	
	/**
	 * Sets the size of an element
	 * @param element
	 * @param size
	 */
	public static void setSize(AcmeElement element, Dimension size) {
		if (!AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			AcmeProperty visWidth = element.getProperty("vis-width");
			AcmeProperty visHeight = element.getProperty("vis-height");
			AcmePropertyType type = new acmeLib.properties.FloatType("float");
			AcmePropertyValue visWidthVal = new AcmeFloat(size.width);
			AcmePropertyValue visHeightVal = new AcmeFloat(size.height);
			if (visWidth == null) {
				visWidth = new AcmeProperty(type);
				try {
					visWidth.setName("vis-width");
					element.addProperty(visWidth);
				}
				catch (AcmeIllegalNameException e) {
				}
			}
			if (visHeight == null) {
				visHeight = new AcmeProperty(type);
				try {
					visHeight.setName("vis-height");
					element.addProperty(visHeight);
				}
				catch (AcmeIllegalNameException e) {
				}
			}
			visHeight.setValue(visHeightVal);
			visWidth.setValue(visWidthVal);
		}
		else {
			VisualInformation vis = getVisualInformation(element);
			ElementVisualization ev = VisualStyleRegistry.getVisualization(element);
			if (size.height == ev.getSize().height && size.width == ev.getSize ().width) {
				vis.setHeight(-1); vis.setWidth(-1);
			}
			else {
				vis.setHeight (size.height);
				vis.setWidth (size.width);
			}
		}
	}
	
    /**
     * Scales a location by "scale" factor
     * @param location
     * @param scale
     */
	public static void scaleLocation(Point location, double scale) {
		location.x = (int) (location.x * scale);
		location.y = (int) (location.y * scale);
	}
	
	/**
	 * Scales a size by "scale" factor
	 * @param size
	 * @param scale
	 */
	public static void scaleSize(Dimension size, double scale) {
		if (size != null) {
			size.width = (int) (size.width * scale);
			size.height = (int) (size.height * scale);
		}
	}


	/**
	 * Makes a name unique in the namespace of the target AcmeElement to which
	 * the <code>name</code> element is to be added. This is done by
	 * appending a digit or incrementing that digit to the end of the name.
	 * 
	 * @param parent
	 *           the AcmeElement to add to, in whose namespace <code>name</code>
	 *           must be unique
	 * @param name
	 *           the name of the AcmeElement being added, whose name is to be
	 *           made unique
	 * @return String the unique name
	 */
	public static String getUniqueName(AcmeElement parent, String name) {
		if (parent == null)
			return null;
		String childNameBase = getBaseName (name);
		String childName = name;
		int i = 0;
		while (parent.getNamespace().containsName(childName)) {
			childName = childNameBase + i++;
		}
		return childName;
	}
	
	public static String getBaseName(String name) {
		String childNameBase = name;
		int index = childNameBase.length() - 1;
		while (index >= 0 && Character.isDigit(childNameBase.charAt(index))) {
			index--;
		}
		childNameBase = childNameBase.substring(0, index + 1);
		return childNameBase;
	}
	
	/**
	 * Same as getUniqueName, but avoids names that are also in additionalRestrictions
	 * @param parent
	 * @param additionalRestrictions
	 * @param name
	 * @return
	 */
	public static String getUniqueName (AcmeElement parent, Map additionalRestrictions, String name) {
		if (parent == null) return null;
		String childNameBase = getBaseName (name);
		String childName = name;
		int i = 0;
		while (parent.getNamespace ().containsName (childName) || additionalRestrictions.containsValue (childName)) {
			childName = childNameBase + i++;
		}
		return childName;

	}


	public static boolean isUniqueConnectorName(String childName, AcmeSystem parent) {
		if (parent == null)
			return false;
		return parent.getConnector(childName) == null;
	}
	public static boolean isUniqueComponentName(String childName, AcmeSystem parent) {
		if (parent == null)
			return false;
		return parent.getComponent(childName) == null;
	}
	public static boolean isUniquePortName(String childName, AcmeComponent parent) {
		if (parent == null)
			return false;
		return !parent.getNamespace().containsName(childName);
	}
	public static boolean isUniqueRoleName(String childName, AcmeConnector parent) {
		if (parent == null)
			return false;
		return !parent.getNamespace().containsName(childName);
	}
	
	/**
	 * Finds the component in a diagram that is closes the the location
	 * @param acmeSystem
	 * @param location
	 * @return
	 */
	public static AcmeComponent findClosestComponent(AcmeSystem acmeSystem, Point location) {
		if (acmeSystem == null)
			return null;
		AcmeComponent acmeComponent = null;
		int minDistance = Integer.MAX_VALUE;
		AcmeEnumeration enum = new AcmeEnumeration();
		enum.concatenate(acmeSystem.getComponents());
		while (enum.moreItems()) {
			AcmeComponent e = (AcmeComponent) enum.getNextItem();
			Point pos = SystemView.getLocation(e, true);
			pos = new Point(pos.x + 30, pos.y + 20);
			int distance = location.getDistance2(pos);
			if (distance < minDistance) {
				minDistance = distance;
				acmeComponent = e;
			}
		}
		return acmeComponent;
	}
	
	/**
	 * Finds the connector in a diagram that is closest to the location
	 * @param acmeSystem
	 * @param location
	 * @return
	 */
	public static AcmeConnector findClosestConnector(AcmeSystem acmeSystem, Point location) {
		if (acmeSystem == null)
			return null;
		AcmeConnector acmeConnector = null;
		int minDistance = Integer.MAX_VALUE;
		AcmeEnumeration enum = new AcmeEnumeration();
		enum.concatenate(acmeSystem.getConnectors());
		while (enum.moreItems()) {
			AcmeConnector e = (AcmeConnector) enum.getNextItem();
			Point pos = SystemView.getLocation(e, true);
			pos = new Point(pos.x + 15, pos.y + 15);
			int distance = location.getDistance2(pos);
			if (distance < minDistance) {
				minDistance = distance;
				acmeConnector = e;
			}
		}
		return acmeConnector;
	}
	
	/**
	 * Gets the children of an element
	 * @param parent
	 * @return
	 */
	public static Vector getChildren(AcmeElement parent) {
		if (parent instanceof AcmeComponent)
			return getChildren((AcmeComponent) parent);
		if (parent instanceof AcmeConnector)
			return getChildren((AcmeConnector) parent);
		if (parent instanceof AcmeSystem)
			return getChildren((AcmeSystem) parent);
		return new Vector();
	}

	/**
	 * Gets a random point location for a port
	 * @param size
	 * @param obj
	 * @return
	 */
	public static Point calculateRandomPortPoint(Dimension size, Object obj) {
		Point pLocation = new Point();
		int random1 = ((int )(Math.random() * obj.hashCode ())) % 100;
		int random2 = ((int )(Math.random () * obj.hashCode())) % 10;
		//if (Math.random() < 0.5) {
		if (random1 < 50) {
			//pLocation.x = (int) Math.random() * size.width - (size.width /
			// 2);
			pLocation.x = random1 * size.width / 100 - (size.width / 2);
			//if (Math.random() < 0.5) {
			if (random2 < 5) {
				pLocation.y = size.height / 2;
			}
			else {
				pLocation.y = -size.height / 2;
			}
		}
		else {
			//pLocation.y = (int) Math.random() * size.height - (size.height /
			// 2);
			pLocation.y = random1 * size.height / 100 - (size.height / 2);
			//if (Math.random() < 0.5) {
			if (random2 < 5) {
				pLocation.x = size.width / 2;
			}
			else {
				pLocation.x = -size.width / 2;
			}
		}
		return pLocation;
	}
	
	/**
	 * Gets the list of routing rules for the roles in a connector
	 * @param roles
	 * @param connector
	 * @return
	 */
	public static List getRoleRouteRules(Vector roles, AcmeConnector connector) {
		Object vis = VisualStyleRegistry.getVisualization(connector);
		ConnectorVisualization cv = null;
		if (vis instanceof ConnectorVisualization)
			cv = (ConnectorVisualization) vis;
		LinkedList ll = null;
		List ruleList = new ArrayList();
		Iterator iterRule = null;
		if (cv != null) {
			ll = cv.getRoleRouteRules();
			if (ll != null)
				iterRule = ll.iterator();
		}
		ConnectorVisualization.RoleRouteRule allMatchRule = null;
		while (iterRule != null && iterRule.hasNext()) {
			ConnectorVisualization.RoleRouteRule obj = (ConnectorVisualization.RoleRouteRule) iterRule.next();
			if (obj._roleType.equals("role")) {
				allMatchRule = obj;
				break;
			}
		}
		for (int i = 0; ll != null && i < roles.size(); i++) {
			if (allMatchRule != null) {
				ruleList.add(allMatchRule);
				continue;
			}
			AcmeRole role = (AcmeRole) roles.get(i);
			AcmeEnumeration ae = role.getDeclaredTypes();
			ConnectorVisualization.RoleRouteRule rule = null;
			boolean foundRule = false;
			while (!foundRule && ae.moreItems()) {
				String roleType = ((AcmeElementType) ae.getNextItem()).getTypeName();
				Iterator iter = ll.iterator();
				while (iter.hasNext()) {
					ConnectorVisualization.RoleRouteRule obj = (ConnectorVisualization.RoleRouteRule) iter.next();
					if (obj._roleType.equals("role") || obj._roleType.equals(roleType)) {
						if (obj._roleRoute != ConnectorVisualization.RoleRouteOptions.STRAIGHT) {
							foundRule = true;
							rule = obj;
							break;
						}
					}
				}
			}
			ruleList.add(rule);
		}
		return ruleList;
	}
	
	/**
	 * Gets the root Acme system for a child
	 * @param child
	 * @return
	 */
	public static AcmeSystem getRootAcmeSystem(AcmeElement child) {
		if (child instanceof AcmeDesign) {
			return getAcmeSystem((AcmeDesign) child);
		}
		if (child instanceof AcmeSystem) {
			return (AcmeSystem) child;
		}
		if (child == null || child.getParent() instanceof AcmeDesign)
			return (AcmeSystem) child;
		AcmeElement parent = child.getParent();
		while (parent != null && !(parent instanceof AcmeSystem)) {
			parent = parent.getParent();
		} //        parent != null && !(parent.getParent() instanceof AcmeDesign)) {
		//            parent = parent.getParent();
		//        }
		return (AcmeSystem) parent;
	}

	/**
	 * Gets the systems in representations of an acmeElement
	 * @param acmeElement
	 * @return
	 */
	public static Vector getSubAcmeSystems(AcmeElement acmeElement) {
		Vector v = new Vector();
		v.addAll(getRepresentationSystems(acmeElement));
		Vector children = getChildren(acmeElement);
		for (int i = 0; i < children.size(); i++) {
			Object obj = children.get(i);
			if (obj instanceof AcmeSystem) {
				v.add(obj);
			}
			if (obj instanceof AcmeElement) {
				Vector sub = getSubAcmeSystems((AcmeElement) obj);
				v.addAll(sub);
				/*
				 * for (int j = 0; j < sub.size(); j++) { Object o =
				 * sub.get(j); if (!v.contains(o)) { v.add(o); }
				 */
			}
		}
		return v;
	}
	public static Vector getRepresentationSystems(AcmeComponent acmeComponent) {
		AcmeEnumeration enum = new AcmeEnumeration();
		enum.concatenate(acmeComponent.getRepresentations());
		Vector v = new Vector();
		while (enum.moreItems()) {
			AcmeSystem sys = ((AcmeRepresentation) enum.getNextItem()).getSystem();
			v.add(sys);
		}
		return v;
	}
	public static Vector getRepresentationSystems(AcmeConnector acmeConnector) {
		AcmeEnumeration enum = new AcmeEnumeration();
		enum.concatenate(acmeConnector.getRepresentations());
		Vector v = new Vector();
		while (enum.moreItems()) {
			AcmeSystem sys = ((AcmeRepresentation) enum.getNextItem()).getSystem();
			v.add(sys);
		}
		return v;
	}
	public static Vector getRepresentationSystems(AcmeSystem acmeSystem) {
		Vector v = new Vector();
		AcmeEnumeration enum = new AcmeEnumeration();
		if (acmeSystem != null) {
			enum.concatenate(acmeSystem.getRepresentations());
			while (enum.moreItems()) {
				AcmeSystem sys = ((AcmeRepresentation) enum.getNextItem()).getSystem();
				v.add(sys);
			}
		}
		return v;
	}
	public static Vector getRepresentationSystems(AcmeElement parent) {
		if (parent instanceof AcmeComponent)
			return getRepresentationSystems((AcmeComponent) parent);
		if (parent instanceof AcmeConnector)
			return getRepresentationSystems((AcmeConnector) parent);
		if (parent instanceof AcmeSystem)
			return getRepresentationSystems((AcmeSystem) parent);
		return new Vector();
	}
	public static List getAcmeFamilies(AcmeDesign acmeDesign) {
		return convertAcmeEnumerationToList(acmeDesign.getFamilies());
	}

	/**
	 * Retrieves the first immediate family of the supplied AcmeDesign, without
	 * traversing into the imported design of the supplied AcmeDesign.
	 * 
	 * @param acmeDesign
	 * @return
	 */
	public static AcmeFamily getAcmeFamily(AcmeDesign acmeDesign) {
		AcmeEnumeration styleEnum = new AcmeEnumeration();
		styleEnum.concatenate(acmeDesign.getFamilies());
		AcmeFamily family = null;
		if (styleEnum.moreItems()) {
			family = (AcmeFamily) styleEnum.getNextItem();
		}
		return family;
	}

	public static List getAllFamilies(AcmeDesign acmeDesign) {
		List l = convertAcmeEnumerationToList(acmeDesign.getFamilies());
		AcmeEnumeration imports = acmeDesign.getImportedDesigns().enumerate();
		while (imports.moreItems()) {
			AcmeImportedDesignRecord desRec = (AcmeImportedDesignRecord) imports.getNextItem();
			if (desRec.getResolvedDesign() != null) {
				l.addAll(convertAcmeEnumerationToList(desRec.getResolvedDesign().getFamilies()));
			}
		}
		return l;
	}
	public static List getAcmeImports(AcmeDesign acmeDesign) {
		AcmeEnumeration designEnum = new AcmeEnumeration();
		designEnum.concatenate(acmeDesign.getImportedDesigns().enumerate());
		List imports = new Vector();
		while (designEnum.moreItems()) {
			imports.add(((AcmeImportedDesignRecord) designEnum.getNextItem()).getResolvedDesign());
		}
		return imports;
	}
	public static AcmeDesign getAcmeImport(AcmeDesign acmeDesign) {
		AcmeEnumeration designEnum = new AcmeEnumeration();
		designEnum.concatenate(acmeDesign.getImportedDesigns().enumerate());
		AcmeDesign aImport = null;
		while (designEnum.moreItems()) {
			aImport = ((AcmeImportedDesignRecord) designEnum.getNextItem()).getResolvedDesign();
		}
		return aImport;
	}
	public static Vector getTypes(AcmeFamily acmeFamily) {
		Vector v = new Vector();
		AcmeEnumeration enum = new AcmeEnumeration();
		enum.concatenate(acmeFamily.getTypes());
		while (enum.moreItems()) {
			Object o = enum.getNextItem();
			if (o instanceof AcmeElementType) {
				AcmeElementType e = (AcmeElementType) o;
				v.add(e);
			}
		}
		return v;
	}
	public static List retrieveAllReferencesToType(AcmeType type, AcmeFamily inFamily) {
		return retrieveAllReferencesToType(type, inFamily.getParentDesign());
	}
	public static List retrieveAllReferencesToType(AcmeType type, AcmeSystem system) {
		List objects = getAllObjects(system);
		return filterTypeReferences(type, objects);
	}
	public static List getAllObjects(AcmeFamily fam) {
		List objects = new ArrayList();
		AcmeEnumeration types = fam.getTypes();
		while (types.moreItems()) {
			objects.add(types.getNextItem());
		}
		objects.addAll(getAllObjects((AcmeSystem) fam));
		return objects;
	}
	public static List getAllObjects(AcmeDesign design) {
		List objects = new ArrayList();
		AcmeEnumeration types = design.getTypes();
		while (types.moreItems()) {
			AcmeType type = (AcmeType) types.getNextItem();
			if (type instanceof AcmeFamily) {
				objects.addAll(getAllObjects((AcmeFamily) type));
			}
			objects.add(type);
		}
		AcmeEnumeration systems = design.getSystems();
		while (systems.moreItems()) {
			AcmeSystem sys = (AcmeSystem) systems.getNextItem();
			objects.addAll(getAllObjects(sys));
			objects.add(sys);
		}
		return objects;
	}
	public static List getAllObjects(AcmeSystem sys) {
		List objects = new ArrayList();
		AcmeEnumeration components = sys.getComponents();
		while (components.moreItems()) {
			AcmeComponent comp = (AcmeComponent) components.getNextItem();
			objects.addAll(getAllObjects(comp));
			objects.add(comp);
		}
		AcmeEnumeration connectors = sys.getConnectors();
		while (connectors.moreItems()) {
			AcmeConnector conn = (AcmeConnector) connectors.getNextItem();
			objects.addAll(getAllObjects(conn));
			objects.add(conn);
		}
		objects.addAll(getAllObjects((AcmeElement) sys));
		return objects;
	}
	public static List getAllObjects(AcmeComponent comp) {
		List objects = new ArrayList();
		AcmeEnumeration ports = comp.getPorts();
		while (ports.moreItems()) {
			AcmePort port = (AcmePort) ports.getNextItem();
			objects.addAll(getAllObjects(port));
			objects.add(port);
		}
		return objects;
	}
	public static List getAllObjects(AcmeConnector comp) {
		List objects = new ArrayList();
		AcmeEnumeration ports = comp.getRoles();
		while (ports.moreItems()) {
			AcmeRole port = (AcmeRole) ports.getNextItem();
			objects.addAll(getAllObjects(port));
			objects.add(port);
		}
		return objects;
	}
	public static List getAllObjects(AcmeElement elem) {
		List objects = new ArrayList();
		AcmeEnumeration properties = elem.getProperties();
		while (properties.moreItems()) {
			objects.add(properties.getNextItem());
		}
		AcmeEnumeration reps = elem.getRepresentations();
		while (reps.moreItems()) {
			AcmeRepresentation rep = (AcmeRepresentation) reps.getNextItem();
			objects.addAll(getAllObjects(rep.getSystem()));
			objects.add(rep.getSystem());
		}
		return objects;
	}
	public static List retrieveAllReferencesToType(AcmeType type, AcmeDesign design) {
		List everyObject = getAllObjects(design);
		return filterTypeReferences(type, everyObject);
	}
	protected static List filterTypeReferences(AcmeType type, List objects) {
		List filteredObjects = new ArrayList();
		Iterator i = objects.iterator();
		while (i.hasNext()) {
			AcmeObject obj = (AcmeObject) i.next();
			if (type instanceof AcmeFamily) {
				AcmeEnumeration types = ((AcmeFamily) type).getTypes();
				while (types.moreItems()) {
					filteredObjects.addAll(filterTypeReferences((AcmeType) types.getNextItem(), objects));
				}
			}
			else if (type instanceof AcmeElementType && obj instanceof AcmeElement) {
				AcmeElement elem = (AcmeElement) obj;
				AcmeElementType et = (AcmeElementType) type;
				if (elem.declaresType(et) || elem.instantiatesType(et)) {
					filteredObjects.add(elem);
				}
			}
			else if (type instanceof AcmePropertyType && obj instanceof AcmeProperty) {
				AcmeProperty property = (AcmeProperty) obj;
				AcmePropertyType pt = (AcmePropertyType) type;
				if (property.getType() == pt) {
					filteredObjects.add(property);
				}
			}
		}
		return filteredObjects;
	}
	public static boolean eventRelatedToDesign(AcmeStudioChangeManager.AcmeStudioChangeEvent evt, AcmeDesign design) {
		if (evt.getData() == null || design == null) {
			return false;
		}
		switch (evt.getEventType()) {
			case AcmeStudioChangeManager.SET_NAME :
				{
					AcmeElement el = (AcmeElement) evt.getTarget();
					if (el.getParentDesign() == null)
						return false;
					return el.getParentDesign().equals(design);
				}
			case AcmeStudioChangeManager.SET_PROPERTY :
			case AcmeStudioChangeManager.UPDATE_VIS_INFO:
				{
					//                AcmeProperty p = (AcmeProperty )evt.getTarget ();
					if (evt.getTarget () instanceof AcmeElement) {
							AcmeElement el = (AcmeElement) evt.getTarget();
							if (el.getParentDesign() == null)
								return false;
							return el.getParentDesign().equals(design);
					}
					else if (evt.getTarget () instanceof DesignRule) {
						DesignRule rule = (DesignRule )evt.getTarget ();
						AcmeElement scope = (AcmeElement )rule.getDesignRuleScope();
						if (scope.getParentDesign() == null) return false;
						else return scope.getParentDesign ().equals (design);
					}
					else if (evt.getTarget () instanceof AcmeProperty) {
						AcmeProperty prop = (AcmeProperty )evt.getTarget ();
						AcmeElement scope = (AcmeElement )prop.getScope();
						if (scope.getParentDesign () == null) return false;
						else return scope.getParentDesign ().equals(design);
					}
				}
			case AcmeStudioChangeManager.ADD_TYPE:
			case AcmeStudioChangeManager.REMOVE_TYPE:
			{
				AcmeFamily fam = (AcmeFamily )evt.getTarget ();
                boolean isFam = design.getFamilies().castToAcmeVector().contains(fam);
                boolean hasFam = design.getImportedDesigns().contains(AcmeDesign.findDesignRecordByDesign(fam.getParentDesign ())) && 
					Helper.getAcmeSystem (design) != null && 
					Helper.getAcmeSystem(design).declaresType(fam);
                return isFam || hasFam;
			}
			case AcmeStudioChangeManager.REMOVE_ROLE:
			{
				AcmeRole role = (AcmeRole )evt.getSource ();
				AcmeConnector connector = (AcmeConnector )evt.getTarget ();
				return role.getParentDesign ().equals (design) || connector.getParentDesign ().equals (design);
			}
			case AcmeStudioChangeManager.REMOVE_PORT:
			{
				AcmeComponent component = (AcmeComponent) evt.getTarget ();
				AcmePort port = (AcmePort )evt.getSource ();
				return (component != null && component.getParentDesign().equals (design)) || port.getParentDesign ().equals (design);
			}
			case AcmeStudioChangeManager.SET_COMMENT:
			{
				if (evt.getTarget () instanceof AcmeElement) {
					AcmeElement object = (AcmeElement )evt.getTarget ();
					return object.getParentDesign ().equals (design);
				}
			}
			default :
				return evt.getData().equals(design);
		}
	}

	/**
	 * Gets the AcmeElements pointed to from the selection ASSUMPTION: Every
	 * element of the selection refers to an AcmeElement
	 * 
	 * @param selection
	 *           The selection
	 * @return
	 */
	public static AcmeElement[] getAcmeElementsFromSelection(ISelection selection) {
		List elements = null;
		if (selection instanceof IStructuredSelection) {
			IStructuredSelection iss = (IStructuredSelection) selection;
			elements = new ArrayList();
			Iterator i = iss.iterator();
			while (i.hasNext()) {
				Object theSel = i.next();
				if (theSel instanceof AcmeElement) {
					elements.add(theSel);
				}
				else if (theSel instanceof ElementEditPart) {
					Object m = ((ElementEditPart) theSel).getModel();
					if (m instanceof AcmeElement)
						elements.add(((ElementEditPart) theSel).getModel());
				}
				else if (theSel instanceof RoleEditPart) {
					elements.add (((RoleEditPart )theSel).getModel ());
				}
				else {
					// Can't get the model from the selection

				}
			}
		}
		if (elements != null) {
			AcmeElement[] ret = new AcmeElement[elements.size()];
			Iterator i = elements.iterator();
			int index = 0;
			while (i.hasNext()) {
				ret[index++] = (AcmeElement) i.next();
			}
			return ret;
		}
		else {
			return null;

		}

	}
	/**
	 * @param string
	 * @return
	 */
	public static String convertToAcmeFilename(String name) {
		String result = name;
		boolean endsWithAcme = false;
		int dotLoc = result.indexOf('.');
		if (dotLoc != -1) {
			String ext = result.substring(dotLoc + 1);
			if (ext.equalsIgnoreCase(ASDesign.FILE_EXT)) {
				endsWithAcme = true;
			}
		}
		if (!endsWithAcme) {
			result += "." + ASDesign.FILE_EXT;
		}
		return result;
	}
	
	public static DiagramEditor getDiagramEditorForSystem(AcmeSystem sys) {
		DiagramEditor retDE = null;
		MultiPageAcmeEditor mp = ASContext.current ().getActiveEditor ();
		if (mp != null) {
			Vector editors = mp.getSystemEditors ();
			for (int i = 0; i < editors.size () && retDE == null; i++) {
				DiagramEditor editor = (DiagramEditor )editors.get (i);	
				if (editor.getAcmeSystem().equals (sys)) {
					retDE = editor;
//					rootEP = editor.getGraphicalViewer().getRootEditPart();
					break;
				}
				
			}
		}
		return retDE;
	}
	/**
	 * @param role
	 * @return
	 */
	public static boolean isRoleAttached(AcmeRole role) {
		return getAcmeSystem(role).getAttachments(role).getSize () > 0;
	}


	/**
	 * @param p0
	 * @param p1
	 * @param cp0
	 * @param cp1
	 * @return
	 */
	public static IntersectingPoint getIntersectingPoint(Point p0, Point p1, Point cp0, Point cp1) {
		IntersectingPoint localPoint = new IntersectingPoint (0, 0);
		double num = (p1.y - p0.y) * (p0.x - cp0.x) - (p1.x - p0.x) * (p0.y - cp0.y);
		double denom = (p1.y - p0.y) * (cp1.x - cp0.x) - (p1.x - p0.x) * (cp1.y - cp0.y);
		if (denom != 0) {
			localPoint.x = (int) (cp0.x + (cp1.x - cp0.x) * num / denom);
			localPoint.y = (int) (cp0.y + (cp1.y - cp0.y) * num / denom);
		}
		return localPoint;
	}


	/**
	 * @param e1
	 * @return
	 */
	public static int getZOrder(AcmeElement e1) {
		if (AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			VisualInformation vis =getVisualInformation(e1);
		
			if (vis == null) {
				return 0;
			}
			else {
				if (vis.getZOrder () == VisualInformation.NO_POS_VALUE) return 0;
				else return vis.getZOrder ();
			}
		}
		return 0;
	}


	/**
	 * @param element
	 * @param i
	 */
	public static void setZOrder(AcmeElement element, int i) {
		if (AcmeStudioPlugin.getDefault().getPreferenceStore().getBoolean(GlobalVisPreferences.KEEP_VIS_SEPARATE)) {
			VisualInformation vis =getVisualInformation(element);
			if (vis != null) {
				vis.setZOrder (i);
			}
		}
	}
	
	public static Image generateConnectionRuleImage (PortConnectionRule rule, Scope context) {
		
		final int SHRINKAGE = 2;
		
		ElementVisualization connVis = VisualStyleRegistry.DEFAULT_CONNECTOR_VISUALIZATION;
		ElementVisualization srcPortVis = VisualStyleRegistry.DEFAULT_PORT_VISUALIZATION;
		ElementVisualization tgtPortVis = VisualStyleRegistry.DEFAULT_PORT_VISUALIZATION;
		ElementVisualization srcRoleVis = VisualStyleRegistry.DEFAULT_ROLE_VISUALIZATION;
		ElementVisualization tgtRoleVis = VisualStyleRegistry.DEFAULT_ROLE_VISUALIZATION;
		ElementVisualization srcCompVis = VisualStyleRegistry.DEFAULT_COMPONENT_VISUALIZATION;
		ElementVisualization tgtCompVis = VisualStyleRegistry.DEFAULT_COMPONENT_VISUALIZATION;
		PortRoleAssociation[] associations = rule.getAssociations();
		
		String connectorType = rule.getConnectorType();
		if (!(connectorType == null || "".equals (connectorType) || "connector".equals (connectorType))) {
			AcmeElementType type = (AcmeElementType )context.lookupName(connectorType, true);
			if (type != null && type != ConstantTypes.CONNECTOR_BASE_TYPE) {
				connVis = VisualStyleRegistry.getVisualization(type);
			}
		}
		connVis = connVis.copy ();
		
		String srcPortType = associations [0].getPortType();
		srcPortVis = getPortVisualization (context, srcPortType);
		
		String tgtPortType = associations [1].getPortType();
		tgtPortVis = getPortVisualization (context, tgtPortType);
		
		srcRoleVis = getRoleVisualization(context, associations [0].getRoleType());
		tgtRoleVis = getRoleVisualization(context, associations [1].getRoleType ());
		
		srcCompVis = getComponentVisualization (context, associations [0].getComponentType()).copy ();
		tgtCompVis = getComponentVisualization (context, associations [1].getComponentType()).copy ();
		
		
		adjustSize (connVis);
		adjustSize (srcCompVis);
		adjustSize (tgtCompVis);
		
		
		int imageHeight = 0;
		imageHeight = Math.max (srcCompVis.getSize().height, imageHeight);
		imageHeight = Math.max (tgtCompVis.getSize ().height, imageHeight);
		imageHeight = Math.max (connVis.getSize ().height, imageHeight);
		imageHeight = Math.max (srcRoleVis.getSize ().height, imageHeight);
		imageHeight = Math.max (tgtRoleVis.getSize ().height, imageHeight);
		imageHeight = Math.max (tgtPortVis.getSize ().height, imageHeight);
		imageHeight = Math.max (srcPortVis.getSize ().height, imageHeight);
		imageHeight += 4;
		imageHeight /= SHRINKAGE;
		
		int imageWidth = 2*ROLE_LINE_SIZE;
		imageWidth += srcCompVis.getSize ().width/SHRINKAGE;
		imageWidth += Math.max (srcPortVis.getSize ().width, srcRoleVis.getSize ().width) / (2* SHRINKAGE);
		imageWidth += connVis.getSize ().width/SHRINKAGE;
		imageWidth += Math.max (tgtPortVis.getSize().width, tgtRoleVis.getSize ().width)/ (2 * SHRINKAGE);
		imageWidth += tgtCompVis.getSize().width/SHRINKAGE + 2;
		
		Image srcImage = new Image (Display.getDefault(), imageWidth, imageHeight); //ImageManager.createImage(rule.toString (),  imageWidth, imageHeight);
		GC gc = new GC (srcImage);
		SWTGraphics graphics = new SWTGraphics (gc);
		graphics.pushState ();
		graphics.setForegroundColor(Constants.TRANSPARENT_COLOR);
		graphics.setBackgroundColor(Constants.TRANSPARENT_COLOR);
		graphics.fillRectangle (0, 0, imageWidth, imageHeight);
		graphics.popState ();
		Point srcRoleLoc = new Point (1+ srcCompVis.getSize ().width/SHRINKAGE - srcRoleVis.getSize ().width/(2/* *SHRINKAGE*/),
				imageHeight/2 - srcRoleVis.getSize().height/(2/* *SHRINKAGE*/));
		Dimension srcRoleSize = new Dimension (srcRoleVis.getSize ().width/*/SHRINKAGE*/,
					srcRoleVis.getSize ().height/*/SHRINKAGE*/);
		Rectangle srcRoleRect =
			new Rectangle (srcRoleLoc, srcRoleSize);
		
		Point connLoc = new Point (srcRoleRect.getCenter().x + srcRoleRect.width /2/*/ SHRINKAGE*/ + ROLE_LINE_SIZE , imageHeight/2 - connVis.getSize().height/(2*SHRINKAGE));
		Dimension connSize = new Dimension (connVis.getSize ().width/SHRINKAGE, connVis.getSize ().height/SHRINKAGE);
		Rectangle connRect = new Rectangle (connLoc, connSize);
		
		double srcRotation =0.0; 
		if (srcRoleVis instanceof RoleVisualization && ((RoleVisualization )srcRoleVis).rotateRole()) {
			Dimension diff = srcRoleRect.getCenter ().getDifference (connRect.getCenter ());
			srcRotation = Math.atan2 (diff.height, diff.width);
			if (((RoleVisualization )srcRoleVis).getOrientation() == Orientation.IN) {
				srcRotation += Math.PI;
			}
		}
		graphics.pushState();
		graphics.setForegroundColor(srcRoleVis.getOutlineColor());
		graphics.setLineStyle(srcRoleVis.getOutlineStyle());
		graphics.setLineWidth(srcRoleVis.getOutlineWidth());
		graphics.drawLine (srcRoleRect.getCenter (), connRect.getCenter ());
		graphics.popState();
		srcRoleVis.fillShape(graphics, srcRoleRect, null, srcRotation);
		srcRoleVis.outlineShape (graphics, srcRoleRect);
		Point tgtRoleLoc = new Point (imageWidth - 1 - tgtCompVis.getSize().width/SHRINKAGE - tgtRoleVis.getSize ().width/(2/* *SHRINKAGE*/), imageHeight/2 - tgtRoleVis.getSize().height/(2/* *SHRINKAGE*/));
		Dimension tgtRoleSize = new Dimension ( tgtRoleVis.getSize ().width/*/SHRINKAGE*/, tgtRoleVis.getSize ().height/*/SHRINKAGE*/);
		Rectangle tgtRoleRect = new Rectangle (tgtRoleLoc, tgtRoleSize);
		
		double tgtRotation = 0.0;
		if (tgtRoleVis instanceof RoleVisualization && ((RoleVisualization )tgtRoleVis).rotateRole()) {
			Dimension diff = tgtRoleRect.getCenter ().getDifference (connRect.getCenter ());
			tgtRotation = Math.atan2 (diff.height, diff.width);
			if (((RoleVisualization )tgtRoleVis).getOrientation() == Orientation.IN) {
				tgtRotation += Math.PI;
			}
		}
		graphics.pushState();
		graphics.setForegroundColor(tgtRoleVis.getOutlineColor());
		graphics.setLineStyle(tgtRoleVis.getOutlineStyle());
		graphics.setLineWidth(tgtRoleVis.getOutlineWidth());
		graphics.drawLine (tgtRoleRect.getCenter (), connRect.getCenter ());
		graphics.popState();
		tgtRoleVis.fillShape (graphics, tgtRoleRect, null, tgtRotation);
		tgtRoleVis.outlineShape (graphics, tgtRoleRect);
		
		connVis.fillShape (graphics, connRect, null, 0);
		connVis.outlineShape (graphics, connRect);
		
		Rectangle srcCompRect = new Rectangle (1, imageHeight/SHRINKAGE - srcCompVis.getSize ().height/(2*SHRINKAGE), srcCompVis.getSize().width/SHRINKAGE, srcCompVis.getSize ().height/SHRINKAGE);
		srcCompVis.fillShape (graphics, srcCompRect, null, 0);
		srcCompVis.outlineShape(graphics, srcCompRect);
		graphics.drawText("src", srcCompVis.getSize().width/(2*SHRINKAGE) - 10, imageHeight/2 - 6);
		
		Rectangle tgtCompRect = new Rectangle (imageWidth -1 - tgtCompVis.getSize ().width / SHRINKAGE, imageHeight/2 - tgtCompVis.getSize ().height/(2 *SHRINKAGE), tgtCompVis.getSize().width/SHRINKAGE, tgtCompVis.getSize ().height/SHRINKAGE );
		tgtCompVis.fillShape(graphics, tgtCompRect, null, 0);
		tgtCompVis.outlineShape(graphics, tgtCompRect);
		graphics.drawText ("tgt", imageWidth - tgtCompVis.getSize ().width/(2*SHRINKAGE) - 10, imageHeight/2 - 6);
		
		Rectangle srcPortRect = new Rectangle (srcRoleRect.getCenter ().x - srcPortVis.getSize ().width / (2/**SHRINKAGE*/),
				srcRoleRect.getCenter ().y - srcPortVis.getSize ().height/(2/**SHRINKAGE*/), 
				srcPortVis.getSize ().width/* / SHRINKAGE*/, srcPortVis.getSize ().height /*/SHRINKAGE*/ );
		
		srcRotation = 0;
		if (srcPortVis instanceof PortVisualization && ((PortVisualization )srcPortVis).alignPort()) {
			Dimension diff = srcRoleRect.getCenter ().getDifference (connRect.getCenter ());
			srcRotation = Math.atan2 (diff.height, diff.width);
			if (((PortVisualization )srcPortVis).getOrientation() == Orientation.OUT) {
				srcRotation += Math.PI;
			}
		}
		else if (srcPortVis instanceof PortVisualization && ((PortVisualization )srcPortVis).rotatePort()) {
			Dimension diff = new Dimension (srcCompRect.getCenter ().getDifference (srcPortRect.getCenter ()));
			srcRotation = Math.atan2 (diff.height, diff.width);
			if (((PortVisualization )srcPortVis).getOrientation() == Orientation.OUT) srcRotation += Math.PI;
		}
		
		srcPortVis.fillShape(graphics, srcPortRect, null, srcRotation);
		srcPortVis.outlineShape (graphics, srcPortRect);
		
		Rectangle tgtPortRect = new Rectangle (tgtRoleRect.getCenter ().x - tgtPortVis.getSize ().width / (2/**SHRINKAGE*/),
				tgtRoleRect.getCenter ().y - tgtPortVis.getSize ().height/(2/**SHRINKAGE*/), 
				tgtPortVis.getSize ().width /*/ SHRINKAGE*/, tgtPortVis.getSize ().height /*/SHRINKAGE*/ );
		
		tgtRotation = 0;
		if (tgtPortVis instanceof PortVisualization && ((PortVisualization )tgtPortVis).alignPort()) {
			Dimension diff = tgtRoleRect.getCenter ().getDifference (connRect.getCenter ());
			tgtRotation = Math.atan2 (diff.height, diff.width);
			if (((PortVisualization )tgtPortVis).getOrientation() == Orientation.OUT) {
				tgtRotation += Math.PI;
			}
		}
		else if (tgtPortVis instanceof PortVisualization && ((PortVisualization )tgtPortVis).rotatePort()) {
			Dimension diff = new Dimension (tgtCompRect.getCenter ().getDifference (tgtPortRect.getCenter ()));
			tgtRotation = Math.atan2 (diff.height, diff.width);
			if (((PortVisualization )tgtPortVis).getOrientation() == Orientation.OUT) tgtRotation += Math.PI;
		}
		
		tgtPortVis.fillShape (graphics, tgtPortRect, null, tgtRotation);
		tgtPortVis.outlineShape (graphics, tgtPortRect);
		
		graphics.dispose ();
		gc.dispose ();
	
		Image finalImage = new Image (null, imageWidth, imageHeight); // ImageManager.createImage(rule.toString (), imageWidth, imageHeight);
		gc = new GC (finalImage);
		graphics = new SWTGraphics (gc);
		graphics.drawImage (srcImage, finalImage.getBounds().width/2 - srcImage.getBounds().width/2, finalImage.getBounds().height/2 - srcImage.getBounds ().height/2);
		gc.dispose ();
		graphics.dispose ();
		srcImage.dispose ();
		tgtCompVis.dispose ();
		srcCompVis.dispose ();
		connVis.dispose ();
		ImageData data = finalImage.getImageData();
		data.transparentPixel = data.palette.getPixel (Constants.TRANSPARENT_COLOR.getRGB());
		
		finalImage.dispose ();
		finalImage = ImageManager.createImage(rule.toString (), data, false);
		
		return finalImage;
		
		
	}


	/**
	 * @param connVis
	 */
	private static void adjustSize(ElementVisualization connVis) {
		if (connVis.getSize ().width > 100) {
			connVis.setSize (new Dimension (100, connVis.getSize().height));
		}
		if (connVis.getSize ().height > 60) {
			connVis.setSize (new Dimension (connVis.getSize ().width, 60));
		}
	}


	/**
	 * @param context
	 * @param srcPortType
	 */
	private static ElementVisualization  getPortVisualization(Scope context, String srcPortType) {
		
		if (!(srcPortType == null || "".equals (srcPortType) || "port".equals (srcPortType))) {
			AcmeElementType type = (AcmeElementType )context.lookupName(srcPortType, true);
			if (type != null && type != ConstantTypes.PORT_BASE_TYPE) {
				return  VisualStyleRegistry.getVisualization (type);
			}
		}
		return VisualStyleRegistry.DEFAULT_PORT_VISUALIZATION;
	}
	
	/**
	 * @param context
	 * @param roleType
	 */
	private static ElementVisualization  getRoleVisualization(Scope context, String roleType) {
		
		if (!(roleType == null || "".equals (roleType) || "role".equals (roleType))) {
			AcmeElementType type = (AcmeElementType )context.lookupName(roleType, true);
			if (type != null && type != ConstantTypes.ROLE_BASE_TYPE) {
				return  VisualStyleRegistry.getVisualization (type);
			}
		}
		return VisualStyleRegistry.DEFAULT_ROLE_VISUALIZATION;
	}
	
	/**
	 * @param context
	 * @param compType
	 */
	private static ElementVisualization  getComponentVisualization (Scope context, String compType) {
		
		if (!(compType == null || "".equals (compType) || "component".equals (compType))) {
			AcmeElementType type = (AcmeElementType )context.lookupName(compType, true);
			if (type != null && type != ConstantTypes.ROLE_BASE_TYPE) {
				return  VisualStyleRegistry.getVisualization (type);
			}
		}
		else if (compType != null && "".equals (compType)) {
			return GHOST_COMPONENT_VIS;
		}
		return VisualStyleRegistry.DEFAULT_COMPONENT_VISUALIZATION;
	}
}
