package AcmeStudio.util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
/**
 * @author schmerl
 *
 */
public class HoverHelp {
    /**
     * Constructor for HoverHelp.
     */
    public HoverHelp() {
        super();
    }
    private Shell _tipShell;
    private Text _tipLabelText;
    private Control _tipControl;
    private Point _tipPosition;
    private ITipInformer _currentInformer = null;
	private Label	_tipLabel;
    public void activateHoverHelp(Control tipControl) {
        final Display display = tipControl.getDisplay();
        _tipControl = tipControl;
        _tipShell = new Shell(display, SWT.ON_TOP);
        FillLayout fl = new FillLayout (SWT.VERTICAL);
        fl.marginHeight = 2;
        fl.marginWidth = 2;
        GridLayout gl = new GridLayout();
        gl.numColumns = 2;
        gl.marginWidth = 2;
        gl.marginHeight = 2;
        _tipShell.setLayout(gl);
        _tipShell.setBackground(
            display.getSystemColor(SWT.COLOR_INFO_BACKGROUND));
        // BugFix#248: Tip is no longer cut off.
        _tipLabel = new Label (_tipShell, SWT.NONE);
        _tipLabelText = new Text(_tipShell, SWT.MULTI | SWT.WRAP);
        _tipLabelText.setForeground(
            display.getSystemColor(SWT.COLOR_INFO_FOREGROUND));
        _tipLabelText.setBackground(
            display.getSystemColor(SWT.COLOR_INFO_BACKGROUND));
//        _tipLabelText.setLayoutData(
//            new GridData(
//                GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_CENTER));
        //_tipLabelText.setEditable (false);		
        _tipControl.addMouseListener(new MouseAdapter() {
            public void mouseDown(MouseEvent e) {
                if (_tipShell.isVisible()) {
                    _tipShell.setVisible(false);
                    
                }
            }
        });
        _tipControl.addMouseTrackListener(new MouseTrackAdapter() {
            // If th mouse exits the control, then stop displaying
            // the tip
            public void mouseExit(MouseEvent e) {
                if (_tipShell.isVisible()) {
                    _tipShell.setVisible(false);
                    
                }
                _currentInformer = null;
            }
            public void mouseHover(MouseEvent event) {
                Point pt = new Point(event.x, event.y);
                Widget widget = event.widget;
                if (widget instanceof Table) {
                    Table table = (Table) widget;
                    TableItem item = table.getItem(pt);
                    if (item != null && item.getData() != null) {
                        if (item.getData() instanceof ITipInformer) {
                            ITipInformer informer =
                                (ITipInformer) item.getData();
                            if (_currentInformer == null
                                || _currentInformer != informer) {
                                _currentInformer = informer;
                                if (informer.hasTipInformation()) {
                                    _tipPosition = _tipControl.toDisplay(pt);
                                    String tip = informer.getTipInformation();
                                    if (tip != null)
                                        _tipLabelText.setText(tip);
                                    Image tipImage = informer.getTipImage();
                                    if (tipImage != null)
                                        _tipLabel.setImage(tipImage);
                                    if (tipImage != null || tip != null) {
                                        _tipShell.pack();
                                        setHoverLocation(
                                            _tipShell,
                                            _tipPosition);
                                        _tipShell.setVisible(true);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        });
    }
    /**
    * Sets the location for a hovering shell
    * @param shell the object that is to hover
    * @param position the position of a widget to hover over
    * @return the top-left location for a hovering box
    */
    private void setHoverLocation(Shell shell, Point position) {
        Rectangle displayBounds = shell.getDisplay().getBounds();
        Rectangle shellBounds = shell.getBounds();
        shellBounds.x =
            Math.max(
                Math.min(position.x, displayBounds.width - shellBounds.width),
                0);
        shellBounds.y =
            Math.max(
                Math.min(
                    position.y + 16,
                    displayBounds.height - shellBounds.height),
                0);
        shell.setBounds(shellBounds);
    }
}
