package AcmeStudio.util;

import java.io.InputStream;
import java.util.*;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Display;

import AcmeStudio.AcmeStudioPlugin;
import acmeLib.elements.*;
import acmeLib.properties.AcmeProperty;
import acmeLib.properties.AcmePropertyType;

/**
 * @author schmerl
 *
 */
public class ImageManager {

	private static boolean alreadyLoaded = false;

	private static Map _descriptors = new HashMap ();
	private static ImageRegistry _images = AcmeStudioPlugin.getDefault().getImageRegistry();
    
    private static Map _createdImages = new HashMap ();
    private static Map _referenceMap = new HashMap ();
	
	public final static String ID_COMP = "ID_COMP";
	public final static String ID_CONN = "ID_CONN";
	public final static String ID_PORT = "ID_PORT";
	public final static String ID_ROLE = "ID_ROLE";
	public final static String ID_COMP_TYPE = "ID_COMP_TYPE";
	public final static String ID_CONN_TYPE = "ID_CONN_TYPE";
	public final static String ID_PORT_TYPE = "ID_PORT_TYPE";
	public final static String ID_ROLE_TYPE = "ID_ROLE_TYPE";
    public final static String ID_ELEMENT_TYPE = "ID_ELEMENT_TYPE";
	public final static String ID_SYSTEM = "ID_SYSTEM";
	public final static String ID_FAMILY = "ID_FAMILY";
	public final static String ID_PROPERTY_TYPE = "ID_PROPERTY_TYPE";
	public final static String ID_PROPERTY = "ID_PROPERTY";
	public final static String ID_ERROR = "ID_ERROR";
	public final static String ID_BIG_ERROR = "ID_BIG_ERROR";
	public final static String ID_WARNING = "ID_WARNING";

    public final static String ID_DEL_GROUP = "ID_DEL_GROUP";
    public final static String ID_NEW_GROUP = "ID_NEW_GROUP";
    public final static String ID_NEW_REP = "ID_NEW_REP";
    public final static String ID_OK = "ID_OK";
	public final static String ID_INHERITS = "ID_INHERITS";
	public final static String ID_EDIT_TYPE_VISUALIZATION = "ID_EDIT_TYPE_VIS";
	
	public final static String ID_DELETE_PROPERTY = "ID_DELETE_PROPERTY";
	public final static String ID_NEW_PROPERTY = "ID_NEW_PROPERTY";
	public final static String ID_SHOW_VIS_PROPERTIES = "ID_SHOW_VIS_PROPERTIES";
	public final static String ID_DELETE_ARMANI = "ID_DELETE_ARMANI_RULE";
	public final static String ID_NEW_ARMANI = "ID_NEW_ARMANI_RULE";
    public final static String ID_TYPE_ERROR = "ID_TYPE_ERROR";
	public final static String SMALL = "_small";
	
    public static final String ID_FORM_BANNER = "ID_FORM_BANNER";
    public static final String ID_NO_VIS = "ID_NO_VIS";
    
    public static final String ID_ZOOM_IN = "ID_ZOOM_IN";
    public static final String ID_ZOOM_OUT = "ID_ZOOM_OUT";
    public static final String ID_ALIGN_BOTTOM  = "ID_ALIGN_BOTTOM";
    public static final String ID_ALIGN_CENTER = "ID_ALIGN_CENTER";
    public static final String ID_ALIGN_LEFT = "ID_ALIGN_LEFT";
    public static final String ID_ALIGN_MIDDLE = "ID_ALIGN_MIDDLE";
    public static final String ID_ALIGN_RIGHT = "ID_ALIGN_RIGHT";
    public static final String ID_ALIGN_TOP = "ID_ALIGN_TOP";
    public static final String ID_CONSTRAINT_RUNNING = "ID_CONSTRAINT_RUNNING";

	public static final String ID_HASREP = "ID_HASREP";
    public static final String ID_CONSTRAINT_NOT_RUNNING = "ID_STOPPED_CONSTRAINT";
    
    public static final String ID_OUTLINE = "ID_OUTLINE";
    public static final String ID_OVERVIEW = "ID_OVERVIEW";
    public static final String ID_PARSE = "ID_PARSE";
    public static final String ID_GRID = "ID_GRID";
    public static final String ID_ZOOM_TO_FIT = "ID_ZOOM_TO_FIT";
    public static final String ID_NAV_UP = "ID_NAV_UP";
    public static final String ID_NAV_UP_DISABLED = "ID_NAV_UP_DISABLED";
    
    public static final String ID_STRAIGHT_ROLE = "ID_STRAIGHT_ROLE";
    public static final String ID_HBUS_ROLE = "ID_HBUS_ROLE";
    public static final String ID_VBUS_ROLE = "ID_VBUS_ROLE";
    public static final String ID_ELBOW_ROLE = "ID_ELBOW_ROLE";
    
    public static final String ID_GOTO_ELEMENT = "ID_GOTO_ELEMENT";
    public static final String ID_CHANGE_TYPE = "ID_CHANGE_TYPE";
    
    public static final String ID_CLEAR = "ID_CLEAR";
    public static final String ID_COPY_TO_GROUP = "ID_CP2GROUP";

    public static final String ID_HIGHLIGHT = "ID_HIGHLIGHT";
    public static final String ID_HIGHLIGHT_ERRORS = "ID_HI_ERRORS";
    
    public static final String ID_COPY_FROM_GROUP = "ID_CPFROMGROUP";
    public static final String ID_GROUP_RULES = "ID_GROUP_RULES";
    
    public static final String ID_SHAPE_RECTANGLE = "SP_RECT";
    public static final String ID_SHAPE_ROUNDRECTANGLE = "SP_RRECT";
    public static final String ID_SHAPE_OVAL = "SP_OVAL";
    public static final String ID_SHAPE_DIAMOND = "SP_DIAMOND";
    public static final String ID_SHAPE_PAGE = "SP_PAGE";
    public static final String ID_SHAPE_FOLDER = "SP_FOLDER";
    public static final String ID_SHAPE_FILTER = "SP_FILTER";
    public static final String ID_SHAPE_REPOSITORY = "SP_REPOSITORY";
    public static final String ID_SHAPE_TRIANGLE="SP_TRIANGLE";
    public static final String ID_SHAPE_OPEN_OVAL = "SP_OPEN_OVAL";
    
    public static final String ID_SYNCH_UP = "ID_SYNCH_UP";
    public static final String ID_SYNCHED = "ID_SYNCH";

	public static final String	ID_LOCKED	= "ID_LOCKED";
	public static final String ID_MOVE_FORWARD = "ID_MOVE_FORWARD";
	public static final String ID_MOVE_BACK = "ID_MOVE_BACK";
	public static final String ID_MOVE_TO_FRONT = "ID_MOVE2FRONT";
	public static final String ID_MOVE_TO_BACK = "ID_MOVE2BACK";
    
	private ImageManager () {}
	
	public static void loadIcons () {
		
		if (alreadyLoaded) return;
		alreadyLoaded = true;
		
		addImage (ID_COMP, "icons/component.gif");
		addImage (ID_CONN, "icons/connector.gif");
		addImage (ID_PORT, "icons/port.gif");
		addImage (ID_ROLE, "icons/role.gif");
		
		addImage (ID_COMP+SMALL, "icons/component16.gif");
		addImage (ID_CONN+SMALL, "icons/connector16.gif");
		addImage (ID_PORT+SMALL, "icons/port16.gif");
		addImage (ID_ROLE+SMALL, "icons/role16.gif");
        addImage (ID_ELEMENT_TYPE+SMALL, "icons/element_type.gif");
		
		addImage (ID_SYSTEM+SMALL, "icons/sys_small.gif");
		addImage (ID_FAMILY+SMALL, "icons/family_small.gif");
		addImage (ID_PROPERTY+SMALL, "icons/property_small.gif");
		addImage (ID_PROPERTY_TYPE+SMALL, "icons/propertytype_small.gif");
		addImage (ID_COMP_TYPE+SMALL, "icons/comptype_small.gif");
		addImage (ID_CONN_TYPE+SMALL, "icons/conntype_small.gif");
		addImage (ID_PORT_TYPE+SMALL, "icons/porttype_small.gif");	
		addImage (ID_ROLE_TYPE+SMALL, "icons/roletype_small.gif");	
		addImage (ID_ERROR, "icons/error.gif");
		addImage (ID_BIG_ERROR, "icons/big_error.gif");
		addImage (ID_WARNING, "icons/warning.gif");
		addImage (ID_INHERITS, "icons/inherits.gif");
        addImage (ID_OK, "icons/OK.gif");
		
		addImage (ID_DELETE_PROPERTY, "icons/delete_property.gif");
		addImage (ID_NEW_PROPERTY, "icons/new_property.gif");
		addImage (ID_SHOW_VIS_PROPERTIES, "icons/show_vis_property.gif");
		addImage (ID_DELETE_ARMANI, "icons/delete_armani_rule.gif");
		addImage (ID_NEW_ARMANI, "icons/new_armani_rule.gif");
		addImage (ID_TYPE_ERROR, "icons/typeerror.gif");
		addImage (ID_EDIT_TYPE_VISUALIZATION, "icons/edit_type_vis.gif");

        addImage (ID_FORM_BANNER, "icons/form_banner.gif");
        addImage (ID_NO_VIS, "icons/no_vis.gif");
      	addImage (ID_ZOOM_TO_FIT, "icons/zoomtofit.gif");
        addImage (ID_ALIGN_BOTTOM , "icons/gef/abottom.gif");
        addImage (ID_ALIGN_CENTER , "icons/gef/acenter.gif");
        addImage (ID_ALIGN_LEFT , "icons/gef/aleft.gif");
        addImage (ID_ALIGN_MIDDLE , "icons/gef/amiddle.gif");
        addImage (ID_ALIGN_RIGHT , "icons/gef/aright.gif");
        addImage (ID_ALIGN_TOP , "icons/gef/atop.gif");
        
        addImage (ID_CONSTRAINT_NOT_RUNNING, "icons/constraint_stopped.gif");
        addImage (ID_CONSTRAINT_RUNNING, "icons/constraint_running.gif");
        
        addImage (ID_OVERVIEW, "icons/overview.gif");
        addImage (ID_OUTLINE, "icons/outline.gif");
        addImage (ID_PARSE, "icons/parse.gif");
        addImage (ID_GRID, "icons/grid.gif");
        addImage (ID_NAV_UP, "icons/uprep.gif");
        addImage (ID_NAV_UP_DISABLED, "icons/uprep-disabled.gif");
        
        addImage (ID_STRAIGHT_ROLE, "icons/straight_role.gif");
        addImage (ID_HBUS_ROLE, "icons/hbus_role.gif");
        addImage (ID_VBUS_ROLE, "icons/vbus_role.gif");
        addImage (ID_ELBOW_ROLE, "icons/elbow_role.gif");
        
        addImage (ID_GOTO_ELEMENT, "icons/goto.gif");
        addImage (ID_CHANGE_TYPE, "icons/changetype.gif");
        
        addImage (ID_CLEAR, "icons/clear.gif");
        
        addImage (ID_COPY_TO_GROUP, "icons/copy_to_group.gif");
        addImage (ID_HIGHLIGHT, "icons/highlight.gif");
        addImage (ID_HIGHLIGHT_ERRORS, "icons/highlight_errors.gif");
        addImage (ID_NEW_GROUP, "icons/new_group.gif");
        addImage (ID_DEL_GROUP, "icons/del_group.gif");
        addImage (ID_NEW_REP, "icons/new_rep.gif");
        addImage (ID_COPY_FROM_GROUP, "icons/copy_from_group.gif");
        addImage (ID_GROUP_RULES, "icons/group_rules.gif");
        addImage (ID_HASREP, "icons/hasrep.gif");
        
        addImage (ID_SHAPE_DIAMOND, "icons/shapes/diamond.gif");
        addImage (ID_SHAPE_FILTER, "icons/shapes/filter.gif");
        addImage (ID_SHAPE_FOLDER, "icons/shapes/folder.gif");
        addImage (ID_SHAPE_OPEN_OVAL, "icons/shapes/open-circle.gif");
        addImage (ID_SHAPE_OVAL, "icons/shapes/circle.gif");
        addImage (ID_SHAPE_PAGE, "icons/shapes/page.gif");
        addImage (ID_SHAPE_RECTANGLE, "icons/shapes/rectangle.gif");
        addImage (ID_SHAPE_REPOSITORY, "icons/shapes/repository.gif");
        addImage (ID_SHAPE_ROUNDRECTANGLE, "icons/shapes/rounded-rectangle.gif");
        addImage (ID_SHAPE_TRIANGLE, "icons/shapes/triangle.gif");
        
        addImage (ID_SYNCH_UP, "icons/synch.gif");
        addImage (ID_SYNCHED, "icons/synced.gif");
        addImage (ID_LOCKED, "icons/locked.gif");
        addImage (ID_MOVE_BACK, "icons/moveback.gif");
        addImage (ID_MOVE_FORWARD, "icons/moveforward.gif");
        addImage (ID_MOVE_TO_BACK, "icons/move2back.gif");
        addImage (ID_MOVE_TO_FRONT, "icons/move2front.gif");
        
	}
	
	private static void addImage (String id, String filename) {
		ImageDescriptor desc = ImageDescriptor.createFromFile (AcmeStudioPlugin.class, filename);
		if (desc == null) {
			Debug.errorln (filename + " does not exist");
		}
        //        if (url == null) {
//            Helper.debug("Adding image " + filename + " against resource " + AcmeStudioPlugin.class.toString() + " == " + null);
//        } else {
//            Helper.debug("Adding image " + filename + " against resource " + AcmeStudioPlugin.class.toString() + " == " + url.toExternalForm());
//        }
		_images.put (id, desc);
		_descriptors.put (id, desc);
	}
	
	public static Image getIcon (String id) {
        loadIcons ();
		return _images.get (id);
	}
	
	public static ImageDescriptor getImageDescriptor (String id) {
        loadIcons ();
		return (ImageDescriptor )_descriptors.get (id);
	}
	
	
	public static Image getSmallIcon (String id) {
        loadIcons ();
		return getIcon (id+SMALL);
	}
	
	public static ImageDescriptor getSmallImageDescriptor (String id) {
        loadIcons ();
		return getImageDescriptor (id+SMALL);
	}
	
	
	public static Image getSmallImageByType (Object o) {
        loadIcons ();
		if (o instanceof AcmeFamily) {
			return getIcon (ID_FAMILY+SMALL);
		}
		else if (o instanceof AcmePropertyType) {
			return getIcon (ID_PROPERTY_TYPE+SMALL);
		}
		else if (o instanceof AcmeComponentType) {
			return getIcon (ID_COMP_TYPE+SMALL);
		}
		else if (o instanceof AcmeConnectorType) {
			return getIcon (ID_CONN_TYPE+SMALL);
		}
		else if (o instanceof AcmePortType) {
			return getIcon (ID_PORT_TYPE+SMALL);
		}
		else if (o instanceof AcmeRoleType) {
			return getIcon (ID_ROLE_TYPE+SMALL);
		}
        else if (o instanceof AcmeElementType) {
            return getIcon (ID_ELEMENT_TYPE+SMALL);   
        }
		else if (o instanceof AcmeProperty) {
			return getIcon (ID_PROPERTY+SMALL);
		}
		else if (o instanceof AcmeComponent) {
			return getIcon (ID_COMP+SMALL);
		}
		else if (o instanceof AcmeConnector) {
			return getIcon (ID_CONN+SMALL);
		}
		else if (o instanceof AcmePort) {
			return getIcon (ID_PORT+SMALL);
		}
		else if (o instanceof AcmeRole) {
			return getIcon (ID_ROLE+SMALL);
		}
		else if (o instanceof AcmeSystem) {
			return getIcon (ID_SYSTEM+SMALL);
		}
		return null;
	}		
	
	
	public static Image getImageByType (Object o) {
        loadIcons ();
		if (o instanceof AcmeFamily) {
			return getIcon (ID_FAMILY);
		}
		else if (o instanceof AcmePropertyType) {
			return getIcon (ID_PROPERTY_TYPE);
		}
		else if (o instanceof AcmeComponentType) {
			return getIcon (ID_COMP_TYPE);
		}
		else if (o instanceof AcmeConnectorType) {
			return getIcon (ID_CONN_TYPE);
		}
		else if (o instanceof AcmePortType) {
			return getIcon (ID_PORT_TYPE);
		}
		else if (o instanceof AcmeRoleType) {
			return getIcon (ID_ROLE_TYPE);
		}
		else if (o instanceof AcmeProperty) {
			return getIcon (ID_PROPERTY);
		}
		else if (o instanceof AcmeComponent) {
			return getIcon (ID_COMP);
		}
		else if (o instanceof AcmeConnector) {
			return getIcon (ID_CONN);
		}
		else if (o instanceof AcmePort) {
			return getIcon (ID_PORT);
		}
		else if (o instanceof AcmeRole) {
			return getIcon (ID_ROLE);
		}
		else if (o instanceof AcmeSystem) {
			return getIcon (ID_SYSTEM);
		}
		return null;
	}
		
               
    public static Image getExistingImage (String key) {
        if (_createdImages.containsKey (key)) {
//            Helper.debug ("Image " + key + " pre");
            return (Image )_createdImages.get (key);
        }
//        Helper.debug ("New image " + key);
        return null;
    }
               
    public static Image createImage (String key, int width, int height, boolean replace) {
        Image newImage = getExistingImage (key);
        if (newImage != null && replace) {
        	newImage.dispose ();
        	newImage = null;
        }
        if (newImage == null) {
            newImage = new Image (Display.getDefault (), width, height);
            ImageManager.imageCreated ("ImageManager.createImage (1)", newImage);
            _createdImages.put(key, newImage);
        }
        
        return newImage;
        
    }
    
    public static Image createImage (String key, InputStream stream, boolean replace) {
        Image newImage = getExistingImage (key);
        if (newImage != null && replace) {
        	newImage.dispose ();
        	newImage = null;
        }
        if (newImage == null) {
            newImage = new Image (Display.getDefault (), stream);
            imageCreated ("ImageManager.createImage(2)", newImage);
        
            _createdImages.put (key, newImage);
        }
        return newImage;   
    }
    
    public static Image createImage (String key, ImageData data, boolean replace) {
        Image newImage = getExistingImage (key);
        if (newImage != null && replace) {
        	newImage.dispose ();
        	newImage = null;
        }
        if (newImage == null) {
            newImage = new Image (Display.getDefault (), data);
            imageCreated ("ImageManager.createImage(3)", newImage);
            _createdImages.put (key, newImage);
        }
        return newImage;           
    }
    
    public static Image createImage (String key, String filename, boolean replace) {
        Image newImage = getExistingImage (key);

        if (newImage == null) {
            newImage = new Image (Display.getDefault (), filename);
            imageCreated ("ImageManager.createImage(4)",newImage);
            _createdImages.put (key, newImage);
        }
        return newImage;        
    }
    
    protected static void addReference (String key, Image img) {
        _createdImages.put (key, img);
        if (_referenceMap.containsKey (img)) {
            Set referencedBy = (Set )_referenceMap.get (img);
            referencedBy.add (key);
        }   
        else {
            Set referencedBy = new HashSet ();
            referencedBy.add (key);
            _referenceMap.put (img, referencedBy);
        }
    }
    
    public static void disposeImage (String key) {
        if (_createdImages.containsKey (key)) {
            Image i = (Image )_createdImages.get (key);
            _createdImages.remove (key);
            ImageManager.imageDisposed ("ImageManager.disposeImage", i);
            i.dispose (); // this removes the image from the table   
        }        
    }
    
    private static class MyException extends Exception {
        public MyException () {
            super ();
        }
    }
    
    public static List unreferencedImages = new ArrayList ();
    
    public static Image createUnreferencedImage (ImageData data) {
        Image img = null;
        try {
        
            img = new Image (null, data);
            imageCreated ("ImageManager.createUnreferencedImage", img);
            unreferencedImages.add (img);
            throw new MyException ();
        }
        catch (MyException e) {
//            e.printStackTrace ();
        }

        return img;
    }
    
    
    public static Set images = new HashSet ();
    
    public static Map traces = new HashMap ();
    
    public static void beginTrace () {
//        traces.clear();
//        images.clear ();
    }
    
    public static void endTrace () {
//        Iterator i = traces.keySet().iterator ();
//        Debug.diagnoseln ("---------------------------\nStill have images from");
//        while (i.hasNext ()) {
//            Exception e = (Exception )traces.get (i.next ());
//            if (e != null)
//            e.printStackTrace ();       
//        }   
//        Debug.diagnoseln ("---------------------------");
    }
    
    public static void imageCreated (String place, Image img) {
////        Debug.diagnoseln (generateTrace () + ":" + place + ": Creating image " + img + " #= " + (images.size () + 1));
//        images.add (img);  
//        traces.put (img, generateTrace ()); 
    }
    
    public static void imageDisposed (String place, Image i) {
//        Debug.diagnoseln("Disposing " + i + " (" + generateTraceString ((Exception )traces.get (i)) + ") # = " + (images.size() - 1));
//        images.remove(i);   
//        traces.remove (i);
    }
    
    
    public static String generateTraceString (Exception e) {
        if (e == null) return "No stack info";
        String trace = "";
      StackTraceElement [] st = e.getStackTrace();
      for (int i = 2; i < st.length && i < 5; i++) {
          trace += st [i].getClassName () + "." + st[i].getMethodName() + ":";   
      }    
      return trace;       
    }
    
    public static Exception generateTrace () {
        try {
             throw new Exception ();
         }
         catch (Exception e) {
             return e;
//             StackTraceElement [] st = e.getStackTrace();
//             for (int i = 2; i < st.length && i < 5; i++) {
//                 trace += st [i].getClassName () + "." + st[i].getMethodName() + ":";;   
//             }
         } 
//         return null;  
    }
    
}
	

