package AcmeStudio.util;

import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.widgets.Text;

import AcmeStudio.ui.views.parts.PropertyValueInspector;
import acmeLib.armaniParser.ParseException;
import acmeLib.properties.AcmePropertyValue;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public interface Listeners {



    public class NumberVerifier implements VerifyListener {
        // BugFix #139: Change lower bound to 1
        protected int _lb = 1;
        protected int _ub = 500;
               
        public NumberVerifier (int lowerBound, int upperBound) {
            _lb = lowerBound;
            _ub = upperBound;
        }
        
        public NumberVerifier () {
            
        }
        
        protected boolean isANumber(String text) {
            boolean doit = true;
            for (int i = 0; i < text.length(); i++) {
                if (!Character.isDigit(text.charAt(i))) {
                    doit = false;
                }
            }
            return doit;
        }
        
        protected boolean liesInBounds (String text) {
            int number = Integer.parseInt(text);
            if (number <= _ub && number >= _lb) {
                return true;
            }
            return false;
        }
        
        public void verifyText (VerifyEvent evt) {
            if (!evt.text.equals ("")) {
                evt.doit = isANumber (evt.text);
                if (evt.doit) {
                    if (evt.widget instanceof Text) {
                        Text t = (Text )evt.widget;
                        String num = t.getText ();
                        // BugFix#246&247: Verifier wasn't checking bounds properly
						if (num.length() >= 1) 
                            evt.doit = liesInBounds (num + evt.text);
                        else 
                            evt.doit = liesInBounds (evt.text);
                    }
                }
 
            }
        }
    }

    /**
     * A verifier that can be attached to text boxes that checks to make
     * sure that the text entered is a valid number.
     */
    public static VerifyListener VERIFY_NUMBER = new NumberVerifier ();


    public static VerifyListener VERIFY_ACMEID = new VerifyListener () {
        public void verifyText(VerifyEvent evt) {
            if (!evt.text.equals("")) {
                // Replace illegal name characters with an underscore
                // The legal identifier regexp is: [A-Za-z][A-Za-z0-9_-]*

                // Check to see if the start is legal - disallow illegal starts
                if (evt.start == 0) {
                    char startChar = evt.text.charAt(0);
                    if (!Character.isLetter(startChar)) {
                        evt.doit = false;
                    }
                }
                else {
                    char candidate = evt.text.charAt(0);
                    if (!Character.isDigit(candidate)
                        && !Character.isLetter(candidate)
                        && candidate != '_' 
                        && candidate != '-') {
                        evt.text = "_";
                    }
                }
            }
        }

    
    };

    public interface Validator {
    	public VerifyListener getValidator ();
    	public void setTarget (Text text);
    };
    
	public class IntValidator implements Validator {
		Text _txt;

		/* (non-Javadoc)
		 * @see AcmeStudio.util.Listeners.Validator#getValidator()
		 */
		public VerifyListener getValidator() {
			return new VerifyListener () {
				public void verifyText (VerifyEvent evt) {
					try {
						String newText = _txt.getText ();
						newText = newText.substring (0, evt.start) + evt.text + newText.substring (evt.start, evt.end);
						Integer.parseInt(newText);
						evt.doit = true;
					}
					catch (NumberFormatException e) {
						evt.doit = false;
					}
				}
			};
		}

		/* (non-Javadoc)
		 * @see AcmeStudio.util.Listeners.Validator#setTarget(org.eclipse.swt.widgets.Text)
		 */
		public void setTarget(Text text) {
			_txt = text;
		}
	}
    
    public class DoubleValidator implements Validator {
        Text _txt; 
        
        
        public VerifyListener getValidator () {
            return new VerifyListener () {
                public void verifyText (VerifyEvent evt) {
                    try {
                        String newText = _txt.getText ();
                        newText = newText.substring (0, evt.start) + evt.text + newText.substring (evt.end);
                        Double.parseDouble(newText); 
                        evt.doit = true;
                    }
                    catch (NumberFormatException e) {
                        evt.doit = false;
                    }      
                }
            };
        }


		/* (non-Javadoc)
		 * @see AcmeStudio.util.Listeners.Validator#setTarget(org.eclipse.swt.widgets.Text)
		 */
		public void setTarget(Text text) {
			_txt = text;
		}
    };
    
	/**
	 * @author Bradley Schmerl
	 * Created Feb 19, 2004
	 */
	public class StringValidator implements Validator {

		Text _txt;
		/**
		 * 
		 */
		public StringValidator() {
			super ();
		}

		/* (non-Javadoc)
		 * @see AcmeStudio.util.Listeners.Validator#getValidator()
		 */
		public VerifyListener getValidator() {
			return new VerifyListener () {

				public void verifyText(VerifyEvent evt) {
					String newText = _txt.getText ();
					 newText = newText.substring (0, evt.start) + evt.text + newText.substring (evt.end);
					 newText = "\"" + newText + "\"";
					 try {
						AcmePropertyValue v = PropertyValueInspector.getPropertyValue(newText);
					} catch (ParseException e) {
						evt.doit = false;
					}
				}
				
			};
		}

		/* (non-Javadoc)
		 * @see AcmeStudio.util.Listeners.Validator#setTarget(org.eclipse.swt.widgets.Text)
		 */
		public void setTarget(Text text) {
			_txt = text;
		}
	}
}
