package AcmeStudio.wizards;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

import AcmeStudio.core.resource.ASPath;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.util.Helper;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeFamily;

/**
 * This is a sample new wizard. Its role is to create a new file 
 * resource in the provided container. If the container resource
 * (a folder or a project) is selected in the workspace 
 * when the wizard is opened, it will accept it as the target
 * container. The wizard creates one file with the extension
 * "acme". If a sample multi-page editor (also available
 * as a template) is registered for the same extension, it will
 * be able to open it.
 */

public class AcmeStudioNewFamilyWizard extends AcmeStudioNewWizard {

    /**
     * Constructor for AcmeStudioNewWizard.
     */
    public AcmeStudioNewFamilyWizard() {
        super();
    }

    /**
     * Adding the page to the wizard.
     */
    public void addPages() {
        pages = new AcmeStudioNewWizardPage [2];
        pages [0] = new AcmeStudioNewFamilyWizardPage(selection);
        pages [1] = new AssignFamilyWizardPage (selection);
        addPage(pages [0]);
        addPage (pages [1]);
    }

    /**
     * The subclass-specific worker method.
     */
    protected void doFinish(
        String containerName,
        String name,
        IProgressMonitor monitor)
    throws CoreException {
    }

    protected InputStream generateNewFileStream(String systemName) {
        return generateNewFamilyStream (systemName);
    }

    private InputStream generateNewFamilyStream(String familyName) {
        AcmeOutputStream aos = new AcmeOutputStream();
        try {
            AcmeDesign newAcmeDesign = new AcmeDesign();
            
            ASPath [] families = ((AssignFamilyWizardPage )pages [1])._familiesToImport;
            List familyNames = ((AssignFamilyWizardPage )pages [1])._familyNames;
            if (families != null && families.length > 0) {
                for (int i = 0; i < families.length; i++) {
                    newAcmeDesign.addDesignToImport (families [i].getPathStr ());
                }
                newAcmeDesign.resolveImportedDesigns ();   
            }
            
            
            AcmeFamily newFamily = new AcmeFamily(familyName);
            
            //Assign superfamilies
            List familiesEnum = Helper.getAllFamilies (newAcmeDesign);
            Iterator i = familiesEnum.iterator ();
            
            while (i.hasNext ()) {
                AcmeFamily family = (AcmeFamily )i.next ();
                if (familyNames.contains (family.getName ())) {
                    newFamily.addSuperType(family);   
                }   
            }
            
            newAcmeDesign.addFamily(newFamily);
            newAcmeDesign.exportArmani(aos);
        } catch (Exception e) {
        }
        InputStream result =
            new ByteArrayInputStream(aos.toString().getBytes());
        return result;
    }

    /* (non-Javadoc)
     * @see AcmeStudio.wizards.AcmeStudioNewWizard#setActivePage(AcmeStudio.ui.editors.MultiPageAcmeEditor)
     */
    protected void setActivePage(MultiPageAcmeEditor editor) {
        editor.setActiveAcmePage(MultiPageAcmeEditor.FAMILY_PAGE);
    }

}