package AcmeStudio.wizards;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;

/**
 * The "New" wizard page allows setting the container for
 * the new file as well as the file name. The page
 * will only accept file name without the extension OR
 * with the extension that matches the expected one (acme).
 */

public class AcmeStudioNewFamilyWizardPage extends AcmeStudioNewWizardPage {

	/**
	 * Constructor
	 */
	public AcmeStudioNewFamilyWizardPage(ISelection selection) {
		super(selection);
        _objToCreate = "Family";
        _defaultName = "Family1";
        setTitle("Style Editor File");
	}

    /**
     * Enforces the families subdir in the container name.
     * @see AcmeStudio.wizards.AcmeStudioNewWizardPage#updateContainer()
     */
    protected void updateContainer() {
        if (_containerText != null) {
            String container = getContainerName();
            IPath containerPath = new Path(container);
            // check to make sure the second segment of the path is always FAMDIR
            if (containerPath.segmentCount() > 1) {
                if (! containerPath.segment(1).equals(FAMDIR)) {
                    // if not, enforce it by inserting the FAMDIR into the 2nd segment
                    IPath newCont = containerPath.uptoSegment(1);
                    containerPath = containerPath.removeFirstSegments(1);
                    containerPath = newCont.append(FAMDIR).append(containerPath).addTrailingSeparator();
                    container = containerPath.toString();
                    _containerText.setText(container);
                    // BRS: Avoid eclipse bug that changes grid layout when something
                    //      is selected in the text before the text is displayed
//                    _containerText.setSelection(location, getContainerName().length());
                }
            } else if (containerPath.segmentCount() == 1) {  // append FAMDIR
                containerPath = containerPath.append(FAMDIR).addTrailingSeparator();
                container = containerPath.toString();
                _containerText.setText(container);
                // BRS: Avoid eclipse bug that changes grid layout when something
                //      is selected in the text before the text is displayed
//                    _containerText.setSelection(getContainerName().length());
            }
                
            // check to make sure that no more than one segment is FAMDIR
            if (containerPath.segmentCount() > 2
                && containerPath.segment(2).equals(FAMDIR)) {
                // if not, enforce it by removing the FAMDIR from the 3rd segment
                IPath newCont = containerPath.uptoSegment(2);
                containerPath = containerPath.removeFirstSegments(3);
                containerPath = newCont.append(containerPath).addTrailingSeparator();
                container = containerPath.toString();
                _containerText.setText(container);
                // BRS: Avoid eclipse bug that changes grid layout when something
                //      is selected in the text before the text is displayed
//                _containerText.setSelection(getContainerName().length());
            }
        }
    }

    protected void updateDescription() {
        String name = _defaultName;
        if (_nameText != null) {
            name = getFileName();
        }
        _description = "This wizard creates a new acme family file under " +
            getContainerName() +"\nwith the name of " + name
            + ".acme that can be opened by a style editor.";
    }

//    /* (non-Javadoc)
//     * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
//     */
//    public IWizardPage getNextPage() {
//        IWizardPage page = null;
//        boolean doAppend = MessageDialog.openQuestion(getShell(), "Insert Family Subdirectory?",
//            "Would you like AcmeStudio to append the\nfamilies subdirectory for you?");
//
//        if (doAppend) {
//            page = super.getNextPage();
//        }
//        return page;
//    }
}
