package AcmeStudio.wizards;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.util.Debug;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 *
 * Provides the basic new project wizard, with AcmeStudio-specific behavior
 * on finish of project creation.
 */
public class AcmeStudioNewProjectWizard extends BasicNewProjectResourceWizard {

    /**
     * Creates a AcmeStudioNewProjectWizard instance.
     */
    public AcmeStudioNewProjectWizard() {
        super();
    }

    /**
     * Extends the default behavior with creation of the families subdir.
     * @see org.eclipse.jface.wizard.IWizard#performFinish()
     */
    public boolean performFinish() {
        boolean retStat = super.performFinish();

        // create a families subdirectory.
        IProject newProj = getNewProject();
        if (newProj == null) {  // can't continue
            return false;
        }
        String subdirName = AcmeStudioPlugin.getResourceString("AcmeStudio.family.subdir");
        IFolder subdir = newProj.getFolder(subdirName);
        if (!subdir.exists()) {
            try {
                subdir.create(true, true, null);
                retStat = true;
            } catch (CoreException e) {
                Debug.errorln(e);
                MessageDialog.openError(Display.getDefault().getActiveShell(),
                    AcmeStudioPlugin.getMessage("AcmeStudioNewProjectWizard.errorTitle"),
                    AcmeStudioPlugin.getMessage("AcmeStudioNewProjectWizard.createFailed"));
                retStat = false;
            }
        }

        return retStat;
    }

}
