package AcmeStudio.wizards;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Iterator;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

import AcmeStudio.core.resource.ASPath;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import AcmeStudio.util.Debug;
import AcmeStudio.util.Helper;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeFamily;
import acmeLib.elements.AcmeSystem;

/**
 * This is a sample new wizard. Its role is to create a new file 
 * resource in the provided container. If the container resource
 * (a folder or a project) is selected in the workspace 
 * when the wizard is opened, it will accept it as the target
 * container. The wizard creates one file with the extension
 * "acme". If a sample multi-page editor (also available
 * as a template) is registered for the same extension, it will
 * be able to open it.
 */

public class AcmeStudioNewSystemWizard extends AcmeStudioNewWizard {
    /**
     * Constructor for AcmeStudioNewWizard.
     */
    public AcmeStudioNewSystemWizard() {
        super();
    }

    /**
     * Adding the page to the wizard.
     */

    public void addPages() {
        pages = new AcmeStudioNewWizardPage [2];
        pages [0] = new AcmeStudioNewSystemWizardPage(selection);
        pages [1] = new AssignFamilyWizardPage (selection);
        addPage (pages [0]);
        addPage (pages [1]);
    }
    /**
     * The subclass-specific worker method.
     */
    protected void doFinish(
        String containerName,
        String name,
        IProgressMonitor monitor)
    throws CoreException {
    }

    protected InputStream generateNewFileStream(String systemName) {
        return generateNewSystemStream(systemName);
    }

    private InputStream generateNewSystemStream(String systemName) {
        AcmeOutputStream aos = new AcmeOutputStream();
        try {
            AcmeDesign newAcmeDesign = new AcmeDesign();
           
            ASPath [] families = ((AssignFamilyWizardPage )pages [1])._familiesToImport;
            java.util.List familyNames = ((AssignFamilyWizardPage )pages [1])._familyNames;
            if (families != null && families.length > 0) {
                for (int i = 0; i < families.length; i++) {
                    newAcmeDesign.addDesignToImport(families [i].getPathStr());
                    
                }   
                newAcmeDesign.resolveImportedDesigns();
            }
            
            AcmeSystem newAcmeSystem = new AcmeSystem(systemName);
            
            
            java.util.List familiesEnum = Helper.getAllFamilies (newAcmeDesign);
            Iterator i = familiesEnum.iterator ();
            
            while (i.hasNext ()) {
                AcmeFamily family = (AcmeFamily )i.next (); 
                if (familyNames.contains (family.getName ())) {
                    newAcmeSystem.addDeclaredType (family);
//                    newAcmeSystem.addInstantiatedType (family);
                }                
            }
            
         
                        
            newAcmeDesign.addSystem(newAcmeSystem);
            newAcmeDesign.exportArmani(aos);
        } catch (Exception e) {
        }
       
       	Debug.traceln("AcmeStudioNewSystemWizard.generateNewSystemStream: " + aos.toString ());
        InputStream result =
            new ByteArrayInputStream(aos.toString().getBytes());
        return result;
    }

    /* (non-Javadoc)
     * @see AcmeStudio.wizards.AcmeStudioNewWizard#setActivePage(AcmeStudio.ui.editors.MultiPageAcmeEditor)
     */
    protected void setActivePage(MultiPageAcmeEditor editor) {
        editor.setActiveAcmePage(MultiPageAcmeEditor.SYSTEM_PAGE);

    }




}