package AcmeStudio.wizards;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;

/**
 * The "New" wizard page allows setting the container for
 * the new file as well as the file name. The page
 * will only accept file name without the extension OR
 * with the extension that matches the expected one (acme).
 */

public class AcmeStudioNewSystemWizardPage extends AcmeStudioNewWizardPage {

	/**
	 * Constructor
	 */
	public AcmeStudioNewSystemWizardPage(ISelection selection) {
		super(selection);
        _objToCreate = "System";
        _defaultName = "System1";
        setTitle("System Editor File");
	}

    /**
     * Enforces not allowing the families subdir in the container name.
     * @see AcmeStudio.wizards.AcmeStudioNewWizardPage#updateContainer()
     */
    protected void updateContainer() {
        super.updateContainer();

        if (_containerText != null) {
            String container = getContainerName();
            IPath containerPath = new Path(container);
            // check to make sure the second segment of the path is never FAMDIR
            if (containerPath.segmentCount() > 1
                && containerPath.segment(1).equals(FAMDIR)) {

                // if yes, strip it off
                IPath newCont = containerPath.uptoSegment(1);
                containerPath = containerPath.removeFirstSegments(2);
                containerPath = newCont.append(containerPath).addTrailingSeparator();
                container = containerPath.toString();
                _containerText.setText(container);
                // BRS: Avoid eclipse bug that changes grid layout when something
                //      is selected in the text before the text is displayed
//                _containerText.setSelection(getContainerName().length());
            }
        }
    }

    protected void updateDescription() {
        String name = _defaultName;
        if (this._nameText != null) {
            name = _nameText.getText();
        }
        _description = "This wizard creates a new acme system file with name of\n"
            + name + ".acme that can be opened by a system editor.";
    }        
}
