package AcmeStudio.wizards;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.SortedMap;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import AcmeStudio.core.ASModel;
import AcmeStudio.core.resource.ASFamily;
import AcmeStudio.core.resource.ASPath;
import AcmeStudio.util.FamilyDetailContentProvider;
import AcmeStudio.util.FamilyDetailLabelProvider;

/**
 * @author schmerl
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AssignFamilyWizardPage extends AcmeStudioNewWizardPage {

    public ISelection _selection;
    //    protected String [] _familiesToImport;
    protected String _description = "";
    protected Tree _availableFamilies;
    protected Button _browseButton;
    protected Button _refreshButton;
    public ASPath[] _familiesToImport;
    public java.util.List _familyNames;
	private TreeViewer _familyDetails;

    AssignFamilyWizardPage(ISelection selection) {
        super(selection);
        updateDescription();
        _selection = selection;
        this.setDescription(_description);
    }
    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(Composite)
     */
    public void createControl(Composite parent) {
        Composite container = new Composite(parent, SWT.NULL);

        GridLayout layout = new GridLayout(2, false);
        layout.numColumns = 2;
        container.setLayout(layout);

        Label avLabel = new Label (container, SWT.NONE);
        avLabel.setText ("Available families:");
        Label detLabel = new Label (container, SWT.NONE);
        detLabel.setText ("Details:");
        
        _availableFamilies = new Tree(container, SWT.MULTI | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.CHECK);
        _availableFamilies.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent evt) {
                if (evt.item instanceof TreeItem) {
                	TreeItem [] allItems = _availableFamilies.getItems();
                	int numChecked = 0;
                	for (int i = 0; i < allItems.length; i++) {
                		if (allItems [i].getItemCount() > 0) {
                			for (int j = 0; j < allItems [i].getItemCount (); j++) {
								if (allItems [i].getItems() [j].getChecked ()) numChecked++;
							}
                			allItems [i].setChecked (false);
                			allItems [i].setGrayed(true);
                		}
						else if (allItems [i].getChecked()) numChecked++;
					}
                    _familiesToImport = new ASPath[numChecked];
                    _familyNames = new ArrayList();
                    TreeItem [] families = new TreeItem [numChecked];
                    int index = 0;
                    for (int i = 0; i < allItems.length; i++) {
                    	if (allItems [i].getItemCount() > 0) {
                			for (int j = 0; j < allItems [i].getItemCount (); j++) {
								if (allItems [i].getItems() [j].getChecked ()) families [index++] = allItems [i].getItems () [j];
							}
//                			allItems [i].setChecked (false);
                		}
                    	else if (allItems [i].getChecked ()) families [index++] = allItems [i];
                    }
                    for (int i = 0; i < families.length; i++) {
                        if (families[i].getData() instanceof java.util.List) {
                            ASFamily family = (ASFamily )((java.util.List)families[i].getData()).get(0);
							_familiesToImport[i] =
                                family.getASPath();
                            _familyNames.add(families[i].getText());
                            fillDetails (family);
                        }
                        else if (families[i].getData() instanceof ASFamily) {
                            ASFamily family = (ASFamily)families[i].getData();
							_familiesToImport[i] = (family).getASPath();

                            _familyNames.add(families[i].getParentItem().getText());
                            fillDetails (family);
                        }

                    }
                }
            }
            public void widgetDefaultSelected(SelectionEvent evt) {
            }

        });
        GridData ald = new GridData(GridData.FILL_BOTH);
//        ald.horizontalSpan = GridData.FILL_HORIZONTAL;
        ald.grabExcessHorizontalSpace = true;
        _availableFamilies.setLayoutData(ald);

        _familyDetails = new TreeViewer (container, SWT.V_SCROLL);
        GridData fdgd = new GridData ();
        fdgd.grabExcessHorizontalSpace = true;
        fdgd.horizontalAlignment = GridData.FILL;
        fdgd.verticalAlignment = GridData.FILL;
        fdgd.widthHint = 250;
        _familyDetails.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
        _familyDetails.setContentProvider(new FamilyDetailContentProvider ());
        _familyDetails.setLabelProvider(new FamilyDetailLabelProvider ());
        
        
        Composite buttons = new Composite(container, SWT.NONE);
        buttons.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER));
        buttons.setLayout(new FillLayout(SWT.HORIZONTAL));
        _browseButton = new Button(buttons, SWT.PUSH);
        _browseButton.setText("Browse...");
        _browseButton.setEnabled(false);

        _refreshButton = new Button(buttons, SWT.PUSH);
        _refreshButton.setText("Refresh");
        _refreshButton.setToolTipText("Refresh the list of available families");
        _refreshButton.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if (e.getSource().equals(_refreshButton)) {
                    refreshFamilies();
                }
            }
        });

        fillFamilies();
        setControl(container);
    }

    /**
	 * @param family
	 */
	protected void fillDetails(ASFamily family) {
		_familyDetails.setInput(family);
	}
	protected void refreshFamilies() {
        //BugFix# 363:  removed check to see if refresh is needed; so force refresh if clicked
        ASModel.familyRegistry().refresh();
        fillFamilies();
    }

    protected void updateDescription() {
        _description = "Import and assign families to system";
    }

    private void fillFamilies() {
        _availableFamilies.removeAll();
        SortedMap families = ASModel.familyRegistry().getByName();
        // (family -> List<Path>)
        Iterator i = families.keySet().iterator();
        while (i.hasNext()) {
            Object o = i.next();
            String name = (String)o;
            java.util.List paths = (java.util.List)families.get(name);
            TreeItem nameItem = new TreeItem(_availableFamilies, SWT.NONE);
            nameItem.setText(name);
            nameItem.setData(paths);
            if (paths.size() > 1) {
                Iterator pI = paths.iterator();
                while (pI.hasNext()) {
                    Object fam = pI.next();
                    if (fam instanceof ASFamily) {
                        ASFamily path = (ASFamily)fam;
                        TreeItem pathItem = new TreeItem(nameItem, SWT.NONE);
                        if (path.getASPath().getPathLevel() == ASPath.GLOBAL_LEVEL) {
                            pathItem.setText("Built In Version (" + path.getASPath().getPathStr() + ")");
                        }
                        else {
                            pathItem.setText("Local Version (" + path.getASPath().getPathStr() + ")");
                        }
                        pathItem.setData(path);
                    }
                }
            }

        }
    }

}
