/*
 * Created on Oct 13, 2003
 *  
 */
package AcmeStudio.wizards;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.wizards.connection.PortConnectionRule;
import acmeLib.elements.AcmeSystem;
import com.swtdesigner.SWTResourceManager;

/**
 * @author Bradley Schmerl Created Oct 13, 2003
 */
public class ConnectionPatternNamePage extends WizardPage
{
    //Hold constants to determine radio button status
    public final static int USE_RULE_ONE_TIME=0;
    public final static int USE_RULE_STICKY=1;
    
    //5 main combo boxes holding user's selections
    private Text[] txtPorts;
    private Text[] txtRoles;

    private Button radBi;
    private Button radNotBi;
    
    //These two arrays store the items that the user selected in a previous dialog
    private String[] chosenComps;

    public String[] namePorts;
    public String[] nameRoles;
    public boolean ruleBi;

    private PortConnectionRule modRule;
    
    /*
     * C'Tor
     * 
     * @param AcmeSystem system
     * @param Shell parentShell
     * @param AcmeComponentType[] ct - This array holds the comps the user clicked on
     */
    public ConnectionPatternNamePage(AcmeSystem system, Shell parentShell, String[] ct, PortConnectionRule mod)
    {
        super("Set Rule Names");

        txtPorts = new Text[2];
        txtRoles = new Text[2];

        chosenComps = ct;

        if (chosenComps[0] == null)
        {
            chosenComps = new String[2];
            chosenComps[0] = new String("");
            chosenComps[1] = new String("");
        }
            
        
        namePorts = new String[2];
        nameRoles = new String[2];
        
        modRule = mod;
    }
    
	private static int nullToInt(Object p)
	{
	    if (p == null)
	        return 0;
	    return 1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent)
	{
	    Composite container = new Composite(parent, SWT.NONE);;
	    setControl(container);
        final GridLayout gridLayout = new GridLayout();
        gridLayout.makeColumnsEqualWidth = true;
        gridLayout.numColumns = 4;
        container.setLayout(gridLayout);
        {
            final Label label = new Label(container, SWT.NONE);
            label.setFont(SWTResourceManager.getFont (AcmeStudioPlugin.getDefaultFontName() + " Black", 12, SWT.BOLD));
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            gridData.horizontalSpan = 5;
            label.setLayoutData(gridData);
            label.setText("Set desired names for objects (leave blank for default)");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER);
            gridData.horizontalSpan = 2;
            label.setLayoutData(gridData);
            //Display source component info in the label
            if (!chosenComps[0].equals(""))
                label.setText("Source Component - " + chosenComps[0]);
            else
                label.setText("Source Component - N/A");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER);
            gridData.horizontalSpan = 2;
            label.setLayoutData(gridData);
            //Display target component info in the label
            if (!chosenComps[1].equals(""))
                label.setText("Target Component - " + chosenComps[1]);
            else
                label.setText("Target Component - N/A");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("Source Port Name");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("First Role Name");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("Second Role Name");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("Target Port Name");
        }
        {
            txtPorts[0] = new Text(container, SWT.BORDER);
            txtPorts[0].setLayoutData(new GridData( GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
            if (modRule != null)
                txtPorts[0].setText(modRule.getAssociations()[0].getPortName());
        }
        {
            txtRoles[0] = new Text(container, SWT.BORDER);
            txtRoles[0].setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
            if (modRule != null)
                txtRoles[0].setText(modRule.getAssociations()[0].getRoleName());
        }
        {
            txtRoles[1] = new Text(container, SWT.BORDER);
            txtRoles[1].setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
            if (modRule != null)
                txtRoles[1].setText(modRule.getAssociations()[1].getRoleName());
        }
        {
            txtPorts[1] = new Text(container, SWT.BORDER);
            txtPorts[1].setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
            if (modRule != null)
                txtPorts[1].setText(modRule.getAssociations()[1].getPortName());
        }
        if (modRule == null) //we are not modifying an existing rule
        {
	        final Group group = new Group(container, SWT.NONE);
	        final GridData gridData = new GridData(
	                GridData.HORIZONTAL_ALIGN_FILL);
	        gridData.verticalSpan = 2;
	        gridData.horizontalSpan = 4;
	        group.setLayoutData(gridData);
	        group.setLayout(new GridLayout());
	        {
	            radBi = new Button(group, SWT.RADIO);
	            radBi.setSelection(true);
	            radBi.setLayoutData(new GridData(
	                    GridData.HORIZONTAL_ALIGN_FILL));
	            radBi.setText("Rule should be bidirectional");
	        }
	        {
	            radNotBi = new Button(group, SWT.RADIO);
	            radNotBi.setLayoutData(new GridData(
	                    GridData.HORIZONTAL_ALIGN_FILL));
	            radNotBi
	                    .setText("Rule should only work one way");
	        }
        }
	}
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage()
	{    
        setVars();
	    return super.getNextPage();
	}
    /**
     * 
     */
    public void setVars()
    {
        //Set the variables which will need to be accessed by the caller
        namePorts[0] = txtPorts[0].getText().equals("") ? "p" : txtPorts[0].getText();
        namePorts[1] = txtPorts[1].getText().equals("") ? "p" : txtPorts[1].getText();
        nameRoles[0] = txtRoles[0].getText().equals("") ? "r" : txtRoles[0].getText();
        nameRoles[1] = txtRoles[1].getText().equals("") ? "r" : txtRoles[1].getText();
        if (modRule == null)
            ruleBi = radBi.getSelection();
    }

    /**
     * @return Returns the chosenComps.
     */
    public String[] getChosenComps()
    {
        return chosenComps;
    }
    /**
     * @param chosenComps The chosenComps to set.
     */
    public void setChosenComps(String[] chosenComps)
    {
        this.chosenComps = chosenComps;
    }
}
