/*
 * Created on Oct 13, 2003
 *  
 */
package AcmeStudio.wizards;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import com.swtdesigner.SWTResourceManager;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.util.Helper;
import AcmeStudio.wizards.connection.PortConnectionRule;
import AcmeStudio.wizards.connection.PortRoleAssociation;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.core.filters.ComponentTypeEnumFilter;
import acmeLib.core.filters.PortTypeEnumFilter;
import acmeLib.elements.AcmeComponentType;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmePortType;
import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl Created Oct 13, 2003
 */
public class ConnectionPatternTypePage extends WizardPage
{
    public final static String CHOICE_SELECTION = "?????";
    //  Hold constants to determine radio button status
    public final static int USE_RULE_ONE_TIME = 0;
    public final static int USE_RULE_STICKY = 1;

    //5 main combo boxes holding user's selections
    private Combo[] cboComps;
    private Combo[] cboPorts;
    private Combo[] cboRoles;
    private Combo connector;

    //Vector containing all possible rule matches for this instance of the
    // dialog
    private List rules;

    //These vectors contain the valid rules, broken down into the pieces
    //to add in to the drop down boxes
    private Vector[] ports;
    private Vector[] roles;
    private Vector connectors;

    //These two arrays store the items that the user clicked on in their
    // diagram
    private AcmePortType[] chosenPorts;
    private AcmeComponentType[] chosenComps;

    //flags for onChange - determine when the user has 'activated' a filter
    //on a drop down by making a selection other than CHOICE_SELECTION'
    private boolean[] portChanged = { false, false };
    private boolean[] roleChanged = { false, false };
    private boolean connChanged = false;

    //Flag to make sure onChange events don't circularly fire
    private boolean populateFreeze = true;

    //For each port drop down, if a possible matching rule specifies component
    //info instead of port, the possibility exists that all ports might need to
    // be
    //matched -- if, in processing, it is realized that this is the case, set
    // the
    //appropriate allPortsFlag
    private boolean[] allPortsFlag = { false, false };

    //Remember what the user clicked in the drop down so it can be reselected
    //during processing of other lists after click
    private String[] savedPorts = { ConnectionPatternTypePage.CHOICE_SELECTION, ConnectionPatternTypePage.CHOICE_SELECTION };
    private String[] savedRoles = { ConnectionPatternTypePage.CHOICE_SELECTION, ConnectionPatternTypePage.CHOICE_SELECTION };
    private String savedConn = ConnectionPatternTypePage.CHOICE_SELECTION;

    private AcmeSystem _system;

    public String[] selectionComps;
    public String[] selectionPorts;
    public String[] selectionRoles;
    public String selectionConn;
    public int selectionRad;

    private Button radOneTime;
    private Button radStay;

    //This flag represents whether the user was using the general connect tool
    // or not
    private boolean fGeneralConnect;

    //These flags represent whether it is a connector based connection and if
    // so,
    //if the connector is the source of the connection
    private boolean fConnectorConn;
    private boolean fConnSource;

    //This flag represents whether this dialog is being used to create a new
    // rule
    private boolean fCreateNewRule;

    //This rule represents the rule being modified.  If we are not modifying 
    //a rule, it will be null.  This is used to determine what initial combo
    //values should be
    private PortConnectionRule modRule;
    
    /*
     * C'Tor
     * 
     * @param AcmeSystem system @param Shell parentShell @param Vector rules -
     * This vector holds all the possible matching rules @param AcmePortType[]
     * pt - This array holds the ports the user clicked on @param
     * AcmeComponentType[] ct - This array holds the comps the user clicked on
     */
    public ConnectionPatternTypePage(AcmeSystem system, Shell parentShell, List rule,
            AcmePortType[] pt, AcmeComponentType[] ct, boolean newRule, PortConnectionRule mod)
    {
        super("Create New Rule");

        cboPorts = new Combo[2];
        cboRoles = new Combo[2];

        if (newRule)
            cboComps = new Combo[2];

        chosenPorts = pt;
        chosenComps = ct;

        this.rules = rule;

        _system = system;

        selectionPorts = new String[2];
        selectionRoles = new String[2];

        if (newRule)
            selectionComps = new String[2];

        if (rules != null && rules.size() > 1)
            fGeneralConnect = true;
        else
            fGeneralConnect = false;

        if (!newRule)
            setConnectorFlags();
        else
        {
            fConnectorConn = false;
            fConnSource = false;
        }

        fCreateNewRule = newRule;

        modRule = mod;
        
        //Initially parse the ruleset into the appropriate vectors for DD
        // population
        parseRules(rules, true);
    }

    private void setConnectorFlags()
    {
        fConnectorConn = ((nullToInt(chosenPorts[0])
                + nullToInt(chosenComps[0]) == 0) || (nullToInt(chosenPorts[1])
                + nullToInt(chosenComps[1]) == 0));

        if (fConnectorConn)
        {
            fConnSource = (nullToInt(chosenPorts[0])
                    + nullToInt(chosenComps[0]) == 0) ? true : false;
        }
        else
            fConnSource = false;
    }

    private static int nullToInt(Object p)
    {
        if (p == null)
            return 0;
        return 1;
    }

    /*
     * Universal method for clearing out the vectors which hold the data to be
     * shown in each drop down
     */
    private void clearStorage()
    {
        ports = new Vector[2];
        roles = new Vector[2];
        for (int i = 0; i < 2; i++)
        {
            ports[i] = new Vector();
            roles[i] = new Vector();
        }
        connectors = new Vector();
    }

    /*
     * @author LFW
     * 
     * @param Vector rules - This vector holds the rules to parse @param boolean
     * f - This flag determines whether or not the all ports flags should be
     * reset before computation It is useful because when parseRules is called
     * and the user has not made a new selection (the end of calcValidRules),
     * these flags should not be reset
     */
    private void parseRules(List rules, boolean f)
    {
        //Clear the items to be re-populated
        clearStorage();

        if (f)
        {
            //Reset flags
            allPortsFlag[0] = false;
            allPortsFlag[1] = false;
        }

        for (Iterator iter = rules.iterator(); iter.hasNext();)
        {
            //For each rule
            PortConnectionRule thisRule = (PortConnectionRule) iter.next();

            //For each rule add the items to the appropriate vectors
            PortRoleAssociation[] assocs = thisRule.getAssociations();

            for (int i = 0; i < 2; i++)
            {
                PortRoleAssociation assoc = assocs[i];
                //Grab the port data out of this association

                //If the port is not "" and is not already in the vector
                if (!assoc.getPortType().equals("")
                        && !ports[i].contains(assoc.getPortType()))
                    ports[i].add(assoc.getPortType());
                else if (assoc.getPortType().equals("") //do we
                                                                       // need
                                                                       // to set
                                                                       // the
                                                                       // allportsflag?
                        && (cboPorts[i] == null || cboPorts[i].getText()
                                .equals(ConnectionPatternTypePage.CHOICE_SELECTION)))
                    allPortsFlag[i] = true;

                //Get the role data
                if (!roles[i].contains(assoc.getRoleType()))
                    roles[i].add(assoc.getRoleType());
            }

            //get the connectors
            if (!connectors.contains(thisRule.getConnectorType()))
                connectors.add(thisRule.getConnectorType());
        }

        //Sort the vectors
        //ports[0].

        Collections.sort(ports[0]);
        Collections.sort(ports[1]);
        Collections.sort(roles[0]);
        Collections.sort(roles[1]);
        Collections.sort(connectors);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
        Composite container = new Composite(parent, SWT.NONE);;
        setControl(container);
        final GridLayout gridLayout = new GridLayout();
        gridLayout.makeColumnsEqualWidth = true;
        if (!fCreateNewRule)
            gridLayout.numColumns = 5;
        else
            gridLayout.numColumns = 7;

        container.setLayout(gridLayout);
        {
            final Label label = new Label(container, SWT.NONE);
            label.setFont(SWTResourceManager.getFont (AcmeStudioPlugin.getDefaultFontName() + " Black", 12, SWT.BOLD));
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            gridData.horizontalSpan = (fCreateNewRule) ? 7 : 5;
            label.setLayoutData(gridData);
            if (!fCreateNewRule)
                //label.setText("Ambiguous Rule -- Choose Desired Pattern");
                setTitle("Ambiguous Rule -- Choose Desired Pattern");
            else
                //label.setText("Create New Rule -- Choosen Desired Types For Objects");
                setTitle("Create New Rule -- Chosen Desired Types For Objects");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER);
            gridData.horizontalSpan = (fCreateNewRule) ? 3 : 2;
            label.setLayoutData(gridData);
            //Display source component info in the label
            if (!fCreateNewRule && (!fConnectorConn || !fConnSource))
                label.setText("Source Component - "
                        + ((chosenComps[0] != null) ? chosenComps[0].getName()
                                : ((AcmeElementType) chosenPorts[0].getParent()
                                        .getDeclaredTypes().getNextItem())
                                        .getTypeName()));
            else
                label.setText("Source Component - N/A");
        }
        {
            final Label label = new Label(container, SWT.NONE);
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER);
            gridData.horizontalSpan = (fCreateNewRule) ? 3 : 2;
            label.setLayoutData(gridData);
            //Display target component info in the label
            if (!fCreateNewRule && (!fConnectorConn || fConnSource))
                label.setText("Target Component - "
                        + ((chosenComps[1] != null) ? chosenComps[1].getName()
                                : ((AcmeElementType) chosenPorts[1].getParent()
                                        .getDeclaredTypes().getNextItem())
                                        .getTypeName()));
            else
                label.setText("Target Component - N/A");
        }
        if (fCreateNewRule)
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("Source Component Type");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("Source Port Type");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("First Role Type");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("Connector Type");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("Second Role Type");
        }
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("Target Port Type");
        }
        if (fCreateNewRule)
        {
            final Label label = new Label(container, SWT.NONE);
            final GridData gridData = new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER
                            | GridData.VERTICAL_ALIGN_END);
            gridData.verticalSpan = 2;
            label.setLayoutData(gridData);
            label.setText("Target Component Type");
        }
        if (fCreateNewRule)
        {
            cboComps[0] = new Combo(container, SWT.READ_ONLY);
            cboComps[0].setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER));
            cboComps[0].addSelectionListener(new SelectionAdapter()
                    {
                public void widgetSelected(SelectionEvent e)
                {
                    if (fCreateNewRule)
                        validate();
                }
            });
        }
        {
            cboPorts[0] = new Combo(container, SWT.READ_ONLY);
            cboPorts[0].setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER));
            cboPorts[0].addSelectionListener(new SelectionAdapter()
            {
                public void widgetSelected(SelectionEvent e)
                {
//                  Make sure not to get in a circular loop
                    if (!populateFreeze && !fCreateNewRule) 
                    {
                        populateFreeze = true;

                        if (!cboPorts[0].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION))
                            portChanged[0] = true;
                        else
                            portChanged[0] = false;

                        savedPorts[0] = cboPorts[0].getText();

                        rePopulateAll(); //set the other dropdowns
                                               // according to this selection
                        validate(); //make sure the OK button is enabled
                                        // correctly

                        populateFreeze = false;
                    }
                    else if (fCreateNewRule)
                        validate();
                }
            });
        }
        {
            cboRoles[0] = new Combo(container, SWT.READ_ONLY);
            cboRoles[0].setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER));
            cboRoles[0].addSelectionListener(new SelectionAdapter()
                    {
                public void widgetSelected(SelectionEvent e)
                {
//                  Make sure not to get in a circular loop
                    if (!populateFreeze && !fCreateNewRule) 
                    {
                        populateFreeze = true;
                        //resetAllFlags();
                        if (!cboRoles[0].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION))
                            roleChanged[0] = true;
                        else
                            roleChanged[0] = false;

                        savedRoles[0] = cboRoles[0].getText();

                        rePopulateAll(); //set the other dropdowns
                                               // according to this selection
                        validate(); //make sure the OK button is enabled
                                        // correctly
                        populateFreeze = false;
                    }
                    else if (fCreateNewRule)
                        validate();
                }
            });

        }
        {
            connector = new Combo(container, SWT.READ_ONLY);
            connector.setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER));
            connector.addSelectionListener(new SelectionAdapter()
                    {
                public void widgetSelected(SelectionEvent e)
                {
//                  Make sure not to get in a circular loop
                    if (!populateFreeze && !fCreateNewRule)
                    {
                        populateFreeze = true;
                        //resetAllFlags();
                        if (!connector.getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION))
                            connChanged = true;
                        else
                            connChanged = false;

                        savedConn = connector.getText();

                        rePopulateAll(); //set the other dropdowns
                                               // according to this selection
                        validate(); //make sure the OK button is enabled
                                        // correctly
                        populateFreeze = false;
                    }
                    else if (fCreateNewRule)
                        validate();
                }
            });
        }
        {
            cboRoles[1] = new Combo(container, SWT.READ_ONLY);
            cboRoles[1].setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER));
            cboRoles[1].addSelectionListener(new SelectionAdapter()
                    {
                public void widgetSelected(SelectionEvent e)
                {
//                  Make sure not to get in a circular loop
                    if (!populateFreeze && !fCreateNewRule)
                    {
                        populateFreeze = true;
                        //resetAllFlags();
                        if (!cboRoles[1].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION))
                            roleChanged[1] = true;
                        else
                            roleChanged[1] = false;
                        savedRoles[1] = cboRoles[1].getText();
                        rePopulateAll(); //set the other dropdowns
                                               // according to this selection
                        validate(); //make sure the OK button is enabled
                                        // correctly
                        populateFreeze = false;
                    }
                    else if (fCreateNewRule)
                        validate();
                }
            });
        }
        {
            cboPorts[1] = new Combo(container, SWT.READ_ONLY);
            cboPorts[1].setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER));
            cboPorts[1].addSelectionListener(new SelectionAdapter()
                    {
                public void widgetSelected(SelectionEvent e)
                {
//                  Make sure not to get in a circular loop
                    if (!populateFreeze && !fCreateNewRule)
                    {
                        populateFreeze = true;
                        //resetAllFlags();
                        if (!cboPorts[1].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION))
                            portChanged[1] = true;
                        else
                            portChanged[1] = false;
                        savedPorts[1] = cboPorts[1].getText();
                        rePopulateAll(); //set the other dropdowns
                                               // according to this selection
                        validate(); //make sure the OK button is enabled
                                        // correctly
                        populateFreeze = false;
                    }
                    else if (fCreateNewRule)
                        validate();
                }
            });
        }
        if (fCreateNewRule)
        {
            cboComps[1] = new Combo(container, SWT.READ_ONLY);
            cboComps[1].setLayoutData(new GridData(
                    GridData.HORIZONTAL_ALIGN_CENTER));
            cboComps[1].addSelectionListener(new SelectionAdapter()
                    {
                public void widgetSelected(SelectionEvent e)
                {
                    if (fCreateNewRule)
                        validate();
                }
            });
        }
        {
            if (!fCreateNewRule && fGeneralConnect)
            {
                final Group group = new Group(container, SWT.NONE);
                final GridData gridData = new GridData(
                        GridData.HORIZONTAL_ALIGN_FILL);
                gridData.verticalSpan = 2;
                gridData.horizontalSpan = 5;
                group.setLayoutData(gridData);
                group.setLayout(new GridLayout());
                {
                    radOneTime = new Button(group, SWT.RADIO);
                    radOneTime.setSelection(true);
                    radOneTime.setLayoutData(new GridData(
                            GridData.HORIZONTAL_ALIGN_FILL));
                    radOneTime.setText("Use this rule this one time");
                }
                {
                    radStay = new Button(group, SWT.RADIO);
                    radStay.setLayoutData(new GridData(
                            GridData.HORIZONTAL_ALIGN_FILL));
                    radStay
                            .setText("Continue using rule until smart connection is deselected");
                }
            }
        }

        /*
         * initially populate each item
         */
        populatePortDD(0);
        populatePortDD(1);
        populateRoleDD(0);
        populateRoleDD(1);
        populateConnectors();
        populateFreeze = false;

        if (fCreateNewRule)
            populateCompDDs();
        validate();
    }

    /**
     * This method simply populates both component drop downs with all possible
     * component types since this will only be used with creating a new rule
     */
    private void populateCompDDs()
    {
        int[] selectIndex = new int[2];
        selectIndex[0] = 0;
        selectIndex[1] = 0;
        
        //Store all components in this vector
        AcmeEnumeration allCompsEnum = _system.getAvailableTypes()
                .castToAcmeSetVector().enumerate(new ComponentTypeEnumFilter());

        List sortedComps = Helper.convertAcmeEnumerationToList(allCompsEnum);
        Collections.sort(sortedComps, new Comparator()
        {

            public int compare(Object arg0, Object arg1)
            {
                AcmeComponentType ct0 = (AcmeComponentType) arg0;
                AcmeComponentType ct1 = (AcmeComponentType) arg1;
                return String.CASE_INSENSITIVE_ORDER.compare(ct0.getTypeName(),
                        ct1.getTypeName());
            }

        });

        Vector allComps = new Vector(sortedComps);

        cboComps[0].add(ConnectionPatternTypePage.CHOICE_SELECTION);
        cboComps[1].add(ConnectionPatternTypePage.CHOICE_SELECTION);

        //LFW: fixed functionality 8/16/04
        //Also add the choice 'any' in to the choices
        cboComps[0].add("Any");
        cboComps[1].add("Any");
        
        for (int i = 0; i < allComps.size(); i++)
        {
            cboComps[0]
                    .add(((AcmeComponentType) allComps.get(i)).getTypeName());
            cboComps[1]
                    .add(((AcmeComponentType) allComps.get(i)).getTypeName());
            
            if (modRule != null)
            {
                //store the selection values
                for (int j = 0; j < 2; j++)
                {
                    String cboVal = ((AcmeComponentType)allComps.get(i)).getTypeName();
                    String savedRule = modRule.getAssociations()[j].getComponentType(); 
                    if (cboVal.equals(savedRule))
                        selectIndex[j] = i+1; //i+1 because select is the first item in the combo
                }
            }
        }

        cboComps[0].select(selectIndex[0]);
        cboComps[1].select(selectIndex[1]);
    }

    /*
     * This method populates the appropraite port DD with data
     */
    private void populatePortDD(int i)
    {
        int selectIndex = 0;
        selectIndex = 0;
        int counter = 0;
        
        //clear the DD
        cboPorts[i].removeAll();

        if (!fConnectorConn || (fConnSource && i == 1)
                || (!fConnSource && i == 0))
        {
            //Store all ports in this vector in case all ports need to be
            // displayed
            AcmeVector allPorts = _system.getAvailableTypes()
                    .castToAcmeSetVector().enumerate(new PortTypeEnumFilter())
                    .castToAcmeVector();

            Vector portsHolder = ExtractTypeStrings(allPorts);

            //if (chosenPorts[i] == null) //The user didn't pick a specific
            // port
            {
                if (portChanged[i] == true || ports[i].size() > 1
                        || allPortsFlag[i] == true)
                    cboPorts[i].add(ConnectionPatternTypePage.CHOICE_SELECTION);

                if (ports[i].size() == 0 && !savedPorts[i].equals(ConnectionPatternTypePage.CHOICE_SELECTION))
                {
                    //This must be the situation where a port was selected and
                    // the
                    // only match was a rule with no port info supplied
                    cboPorts[i].add(savedPorts[i]);
                }
                else
                {
                    counter = 0;
                    for (Iterator iter = ports[i].iterator(); iter.hasNext();)
                    {
                        String temp = iter.next().toString();

                        //remove the item from the list of all ports (which
                        // keeps track of
                        //which port types we DIDN'T add yet)
                        portsHolder.remove(temp);

                        if (modRule != null)
                        {
                            //store the selection values
                            String savedRule = modRule.getAssociations()[i].getPortType(); 
                            if (temp.equals(savedRule))
                                selectIndex = counter+1; //i+1 because select is the first item in the combo
                        }
                        
                        cboPorts[i].add(temp);
                        counter++;
                    }
                }
            }
            //else
            // cboPorts[i].add(chosenPorts[i].getTypeName());

            //add the remaining port types not added if AllPorts is set
            if (allPortsFlag[i] == true)
                for (Iterator iter = portsHolder.iterator(); iter.hasNext();)
                {    
                    String temp = iter.next().toString();
                    cboPorts[i].add(temp);
                    if (modRule != null)
                    {
                        //store the selection values
                        String savedRule = modRule.getAssociations()[i].getPortType(); 
                        if (temp.equals(savedRule))
                            selectIndex = counter+1; //i+1 because select is the first item in the combo
                    }
                }

            //Select the appropriate item after selection - either ????? or
            // the selection, or, if we are modifying, the stored var
            if (modRule != null)
                cboPorts[i].select(selectIndex);
            else
                cboPorts[i].select((portChanged[i]) ? 1 : 0);
        }
        else
        {
            cboPorts[i].add("N/A");
            cboPorts[i].select(0);
        }
    }

    /**
     * Grab the type strings out of an AcmeVector of AcmePortType objects
     */
    private Vector ExtractTypeStrings(AcmeVector allPorts)
    {
        Vector ret = new Vector();
        for (int i = 0; i < allPorts.getSize(); i++)
        {
            ret.add(((AcmePortType) allPorts.getItemAt(i)).getTypeName());
        }
        return ret;
    }

    /*
     * This method populates the appropriate Role drop down with data
     */
    private void populateRoleDD(int i)
    {
        int selectIndex = 0;
        selectIndex = 0;
        int counter = 0;
        
        cboRoles[i].removeAll();

        if (!fConnectorConn || (fConnSource && i == 1)
                || (!fConnSource && i == 0))
        {
            //If the user made a selection, or if there is more than one obj to
            // choose from
            if (roleChanged[i] == true || roles[i].size() > 1)
                cboRoles[i].add(ConnectionPatternTypePage.CHOICE_SELECTION);

            //add all the roles
            counter = 0;
            for (Iterator iter = roles[i].iterator(); iter.hasNext();)
            {
                
                String temp = iter.next().toString();
                cboRoles[i].add(temp);
                if (modRule != null)
                {
                    //store the selection values
                    String savedRule = modRule.getAssociations()[i].getRoleType(); 
                    if (temp.equals(savedRule))
                        selectIndex = counter+1; //i+1 because select is the first item in the combo
                }
                counter++;
            }

            //Select the appropriate item after selection - either ????? or
            // the selection, of if we are modifying, the stored var
            if (modRule != null)
                cboRoles[i].select(selectIndex);
            else
                cboRoles[i].select((roleChanged[i]) ? 1 : 0);
        }
        else
        {
            cboRoles[i].add("N/A");
            cboRoles[i].select(0);
        }
    }

    /*
     * This method populates the connector drop down with data
     */
    private void populateConnectors()
    {
        int selectIndex = 0;
        selectIndex = 0;
        int counter = 0;
        
        connector.removeAll();

        //If the user made a selection, or if there is more than on eobj to
        // choose from
        if (connChanged == true || connectors.size() > 1)
            connector.add(ConnectionPatternTypePage.CHOICE_SELECTION);

        counter = 0;
        for (Iterator iter = connectors.iterator(); iter.hasNext();)
        {
            String temp = iter.next().toString();
            connector.add(temp);
            if (modRule != null)
            {
                //store the selection values
                String savedRule = modRule.getConnectorType(); 
                if (temp.equals(savedRule))
                    selectIndex = counter+1; //i+1 because select is the first item in the combo
            }
            counter++;
        }

        if (modRule != null)
            connector.select(selectIndex);
        else
            connector.select((connChanged) ? 1 : 0);
    }

    private void resetAllFlags()
    {
        for (int i = 0; i < 2; i++)
        {
            portChanged[i] = false;
            roleChanged[i] = false;
        }
        connChanged = false;
    }

    /*
     * This method calculates the useable rules based on user selections, and
     * makes sure all drop downs have the appropriate data and correct item
     * selected
     */
    private void rePopulateAll()
    {
        //re-parse the original all rules vector and fill the drop downs with
        // this data,
        //so all possible matches can be determined -- needed especially for
        // when the
        //user deselects a selection by re-clicking on ?????
        parseRules(rules, false);

        populatePortDD(0);
        populatePortDD(1);
        populateRoleDD(0);
        populateRoleDD(1);
        populateConnectors();

        //After re-filling the DDs, reset the selections to the items they were
        // prior
        //to the refilling
        resetSelections();

        //Calculate which rules are useable based on the different selections
        calcValidRules(rules);

        //refill the DDs with these newly calculated useable rules
        populatePortDD(0);
        populatePortDD(1);
        populateRoleDD(0);
        populateRoleDD(1);
        populateConnectors();
    }

    /*
     * This method takes in a set of rules and determines which of them are
     * still useable despite the user's selections
     */
    private void calcValidRules(List rules)
    {
        Vector ret = new Vector();
        for (Iterator iter = rules.iterator(); iter.hasNext();)
        {
            //For each rule
            PortConnectionRule thisRule = (PortConnectionRule) iter.next();

            //For each rule add the items to the appropriate vectors
            PortRoleAssociation[] assocs = thisRule.getAssociations();

            for (int i = 0; i < 2; i++)
            {
                PortRoleAssociation assoc = assocs[i];
                //Grab the port data out of this association
                if (!(cboPorts[i].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION)
                        || assoc.getPortType().equals("")
                        || cboPorts[i].getText().equals(assoc.getPortType()) || cboPorts[i]
                        .getText().equals("N/A")))
                {
                    i = 2; //go to next rule
                    continue;
                }

                //Get the role data
                if (!(cboRoles[i].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION)
                        || cboRoles[i].getText().equals(assoc.getRoleType()) || cboRoles[i]
                        .getText().equals("N/A")))
                {
                    i = 2; //go to next rule
                    continue;
                }

                //get the connectors
                if (i == 1)
                {
                    //only do this once
                    if (!(connector.getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION) || connector
                            .getText().equals(thisRule.getConnectorType())))
                        continue;

                    //****************************************************
                    //If we made it here without continuing (and i==2), the
                    // rule is ok
                    //****************************************************
                    ret.add(thisRule);
                }
            }
        }
        parseRules(ret, true);
    }

    /*
     * This method, using hte variables with the saved state of the DDs,
     * iterates through and reselects the saved items
     */
    private void resetSelections()
    {
        for (int j = 0; j < 2; j++)
        {
            for (int i = 0; i < cboPorts[j].getItems().length; i++)
            {
                if (cboPorts[j].getItem(i).equals(savedPorts[j]))
                    cboPorts[j].select(i);
            }

            for (int i = 0; i < cboRoles[j].getItems().length; i++)
            {
                if (cboRoles[j].getItem(i).equals(savedRoles[j]))
                    cboRoles[j].select(i);
            }
        }

        for (int i = 0; i < connector.getItems().length; i++)
        {
            if (connector.getItem(i).equals(savedConn))
                connector.select(i);
        }
    }

    /*
     * This method checks to see if the user still has selections to make, and
     * if so, disables the OK button
     */
    private void validate()
    {
        if (cboPorts[0].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION)
                || cboPorts[1].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION)
                || cboRoles[0].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION)
                || cboRoles[1].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION)
                || connector.getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION)
                || (fCreateNewRule && cboComps[0].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION))
                || (fCreateNewRule && cboComps[1].getText().equals(ConnectionPatternTypePage.CHOICE_SELECTION)))
            setPageComplete(false);
        else
            setPageComplete(true);
    }

    public int determineSticky()
    {
        if (radOneTime.getSelection())
            return ConnectionPatternTypePage.USE_RULE_ONE_TIME;

        return ConnectionPatternTypePage.USE_RULE_STICKY;
    }

    public void setValues()
    {
        //	  Set the variables which will need to be accessed by the caller
        if (!fCreateNewRule && fGeneralConnect)
            selectionRad = determineSticky();
        selectionPorts[0] = cboPorts[0].getText();
        selectionPorts[1] = cboPorts[1].getText();
        selectionRoles[0] = cboRoles[0].getText();
        selectionRoles[1] = cboRoles[1].getText();
        selectionConn = connector.getText();
        if (fCreateNewRule)
        {
            selectionComps[0] = cboComps[0].getText().equals("Any") ? ""
                    : cboComps[0].getText();
            selectionComps[1] = cboComps[1].getText().equals("Any") ? ""
                    : cboComps[1].getText();
            
            //set the chosen comps 
            ConnectionPatternWizard wizard = (ConnectionPatternWizard)getWizard();
            wizard.getNamePage().setChosenComps(selectionComps);
        }        
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
     */
    public IWizardPage getNextPage()
    {
        setValues();
        return super.getNextPage();
    }
}