/*
 * Created on Oct 9, 2003
 *  
 */
package AcmeStudio.wizards;

import java.util.List;
import java.util.Vector;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;

import AcmeStudio.util.Helper;
import AcmeStudio.wizards.connection.PortConnectionRule;
import AcmeStudio.wizards.connection.PortRoleAssociation;
import acmeLib.core.container.AcmeVector;
import acmeLib.core.filters.ConnectorTypeEnumFilter;
import acmeLib.core.filters.PortTypeEnumFilter;
import acmeLib.core.filters.RoleTypeEnumFilter;
import acmeLib.elements.AcmeComponentType;
import acmeLib.elements.AcmeConnectorType;
import acmeLib.elements.AcmeFamily;
import acmeLib.elements.AcmePortType;
import acmeLib.elements.AcmeRoleType;
import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl Created Oct 9, 2003
 * 
 * @author Larry Waldman Created 5/24/04
 * 
 * Modified the class name and content to add the RuleChoicePage into the
 * wizard flow
 */
public class ConnectionPatternWizard extends Wizard
{
    AcmeSystem _sys;
    ConnectionPatternTypePage typePage;
    ConnectionPatternNamePage namePage;
    boolean newRule;
    List rule;
    AcmePortType[] pt;
    AcmeComponentType[] ct;
    PortConnectionRule modRule;
    
	public ConnectionPatternWizard(AcmeFamily family, boolean newRule)
	{
		super();
		_sys = Helper.getAcmeSystem(family);
		this.newRule = newRule;
		modRule = null;
	}

	public ConnectionPatternWizard(AcmeFamily family, boolean newRule, PortConnectionRule mod)
	{
		super();
		_sys = Helper.getAcmeSystem(family);
		this.newRule = newRule;
		modRule = mod;
	}
	
	public ConnectionPatternWizard(AcmeSystem sys, List matchRule, AcmePortType[] pt, AcmeComponentType[] ct, boolean newRule)
	{
	    super();
	    this.newRule = newRule;
	    _sys = sys;
	    rule = matchRule;
	    this.pt = pt;
	    this.ct = ct;
	    modRule = null;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish()
	{
	    if (!newRule || modRule != null)
	    {
	        typePage.setValues();
	        
	        if (modRule != null)
	            namePage.setVars();
	    }
	    else if (newRule)
	    {
	        namePage.setVars();
	    }
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages()
	{
	    if (newRule)
	    {
		    Vector rules = getRulesWithAllTypes(_sys);
		    
		    typePage = new ConnectionPatternTypePage(
		            _sys, Display.getDefault().getActiveShell(), rules,
	                null, null,newRule, modRule);
	    }
	    else
	    {
		    typePage = new ConnectionPatternTypePage(
		            _sys, Display.getDefault().getActiveShell(), rule,
	                pt, ct , newRule, modRule); 
	    }
	    
	    addPage(typePage);
	    
	    if (newRule)
	    {
	        namePage = new ConnectionPatternNamePage(_sys, Display.getDefault().getActiveShell(), typePage.selectionComps, modRule);
		    addPage(namePage);
	    }
	}
	
	/**
     * This method takes a system object and returns a vector full of 
     * PortConnectionRules.  These rules ensure that every type (port, role, and conn)
     * is represented in both source and targets of rules
     * 
     * @param AcmeSystem The system object from which to get types
     * 
     * @return Vector a vector containing a set of portConnectionRules which
     * 							ensure that every type is represented in source and targets
     */
    protected Vector getRulesWithAllTypes(AcmeSystem _sys)
    {
        Vector ret = new Vector();
        
        AcmeVector allPorts = _sys.getAvailableTypes().castToAcmeSetVector()
        .enumerate(new PortTypeEnumFilter()).castToAcmeVector();
        
        AcmeVector allRoles = _sys.getAvailableTypes().castToAcmeSetVector()
        .enumerate(new RoleTypeEnumFilter()).castToAcmeVector();
        
        AcmeVector allConns = _sys.getAvailableTypes().castToAcmeSetVector()
        .enumerate(new ConnectorTypeEnumFilter()).castToAcmeVector();
        
        int portSize = allPorts.getSize();
        int roleSize = allRoles.getSize();
        int connSize = allConns.getSize();
        
        int i = 0;
        while (i < portSize || i < roleSize || i < connSize)
        {
            //There are still more objects (ports, roles, or connectors) which need
            //to be included in a rule
            PortConnectionRule rule = new PortConnectionRule();
            rule.setConnectorType(((AcmeConnectorType)allConns.getItemAt(i < connSize ? i : connSize - 1)).getTypeName());
            
            PortRoleAssociation assoc = new PortRoleAssociation(
                    ((AcmePortType)allPorts.getItemAt(i < portSize ? i : portSize - 1)).getTypeName(),
                    ((AcmeRoleType)allRoles.getItemAt(i < roleSize ? i : roleSize - 1)).getTypeName(),"");
            
            rule.addAssociation(assoc);
            //add it again so all of the types get added as source and target
            rule.addAssociation(assoc);
            
            ret.add(rule);
            i++;
        }
        
        return ret;
    }
	
    /**
     * @return Returns the namePage.
     */
    public ConnectionPatternNamePage getNamePage()
    {
        return namePage;
    }
    /**
     * @param namePage The namePage to set.
     */
    public void setNamePage(ConnectionPatternNamePage namePage)
    {
        this.namePage = namePage;
    }
    /**
     * @return Returns the typePage.
     */
    public ConnectionPatternTypePage getTypePage()
    {
        return typePage;
    }
    /**
     * @param typePage The typePage to set.
     */
    public void setTypePage(ConnectionPatternTypePage typePage)
    {
        this.typePage = typePage;
    }
}
