/*
 * Created on Oct 14, 2003
 *  
 */
package AcmeStudio.wizards.connection;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import AcmeStudio.util.Utils;
import acmeLib.elements.AcmePortType;

/**
 * @author Bradley Schmerl Created Oct 14, 2003
 */
public class PortConnectionRule
{
	List /* <PortRoleAssociation> */_associations = new ArrayList();
	String _connectorType;
	private String name;
	private int _source;
	private boolean priorityRule;

	/**
	 *  
	 */
	public PortConnectionRule()
	{
		super();
		name = "";
		priorityRule = false; 
	}

	/**
	 * @return Returns the connectorType.
	 */
	public String getConnectorType()
	{
		return _connectorType;
	}

	/*
	 * @author LFW
	 * Created on 6/8/04
	 * 
	 * This method implements a way to clone a PortConnectionRule
	 * object.  It is necessary, because once the PCRs are read into
	 * AcmeStudio from the XML file, they need to be modified
	 * during the behavior of the connection wizard.  Doing so in place
	 * would cause >1 instances of the wizard not to function correctly.
	 * 
	 * As such, the method simply creates a new PCR, and populates
	 * it with the data from the rule passed in, creating a 'deep copy'
	 */
	static public PortConnectionRule getClone(PortConnectionRule p)
	{
		PortConnectionRule ret = new PortConnectionRule();
		
		ret._connectorType = p._connectorType;
		ret._source = p.getSourceIndex();
		
		for(Iterator iter = p._associations.iterator(); iter.hasNext(); )
		{
//			Make a new PortRoleAssociation so we don't overwrite
			//the ones loaded in from the file
			PortRoleAssociation temp = (PortRoleAssociation)iter.next();
			PortRoleAssociation newPRA = new PortRoleAssociation(temp._portType,
																 temp._roleType,
																 temp._roleName,
																 temp._portName,
																 temp._componentType);
			ret.addAssociation(newPRA);
		}
		
		return ret;
	}
	
	/**
	 * @param connectorType
	 *                 The connectorType to set.
	 */
	public void setConnectorType(String connectorType)
	{
		_connectorType = connectorType;
	}

	public void addAssociation(PortRoleAssociation assoc)
	{
		_associations.add(assoc);
	}

	public void removeAllAssociations()
	{
		_associations.clear();
	}

	public PortRoleAssociation[] getAssociations()
	{
		PortRoleAssociation[] assocs = new PortRoleAssociation[_associations
		.size()];
		int index = 0;
		for (Iterator iter = _associations.iterator(); iter.hasNext();)
		{
			PortRoleAssociation assoc = (PortRoleAssociation) iter.next();
			assocs[index++] = assoc;
		}
		return assocs;
	}

	public void setDirectionless()
	{
		_source = -1;
	}

	public void setSource(String type)
	{
		PortRoleAssociation[] assocs = getAssociations();
		if (assocs.length != 2)
			_source = -1;
		if (assocs[0].getPortType() == type)
			_source = 0;
		else if (assocs[1].getPortType() == type)
			_source = 1;
		else
			_source = -1;
	}

	public int getSourceIndex()
	{
		return _source;
	}

	public int getTargetIndex()
	{
		if (_source == 0)
			return 1;
		if (_source == 1)
			return 0;
		return -1;
	}

	public PortConnectionRuleKey generateKey()
	{
		return new PortConnectionRuleKey(this);
	}

	/**
	 * @return
	 */
	public String getSource()
	{
		if (_source != -1)
		{
			return getAssociations()[_source].getPortType();
		}
		return null;
	}

	/**
	 * the toString method simply consolidates the data stored in this and
	 * outputs it
	 * 
	 * @author Larry Waldman created on 5/24/04
	 *  
	 */
	public String toString()
	{
		PortRoleAssociation[] assocs = this.getAssociations();
		int i = getSourceIndex();
		int j = getTargetIndex();

		//Construct the string in the following mannor:
		//[component1-port1] | role 1 -> connector -> [port2-component2] |
		// role2

		String comp0 = assocs[0].getComponentType();
		String comp1 = assocs[1].getComponentType();
		String port0 = assocs[0].getPortType();
		String port1 = assocs[1].getPortType();
		String role0 = assocs[0].getRoleType();
		String role1 = assocs[1].getRoleType();
		String conn = _connectorType;
		
		if ("".equals (comp0) && "".equals (comp1) && port0.equals("")
		    && port1.equals("") && role0.equals("") && role1.equals("") 
		    && conn.equals("")) // this is the generic rule, so reset the names
		{
			comp0 = "component";
			comp1 = "component";
			port0 = "port";
			port1 = "port";
			role0 = "role";
			role1 = "role";
			conn = "connector"; 
		}
		
		StringBuffer ret = new StringBuffer(
		"[" + comp0 + "-" + port0 + "] | " + role0);
		ret.append(" -> ");
		ret.append(conn);
		ret.append(" -> ");
		ret.append(role1 + " | [" + port1 + "-" + comp1 + "]");
		return ret.toString().replaceAll("null","");
	}

	public String getCompLevelString()
	{
	    StringBuffer ret = new StringBuffer();
	    PortRoleAssociation[] assocs = this.getAssociations();
	    int i = getSourceIndex();
	    int j = getTargetIndex();
	    boolean bi = false;
	    
	    if (i == -1)
	    {
	        i = 0;
	        j = 1;
	        bi = true;
	    }
	    
	    ret.append(assocs[i]._componentType.equals("") ? "*" : assocs[i]._componentType);
	    //ret.append(" " + (bi ? "<-> " : "-> "));
	    ret.append(" -> ");
	    ret.append(assocs[j]._componentType.equals("") ? "*" : assocs[j]._componentType);
	       
	    return ret.toString();
	}
	
	public String getPortLevelString()
	{
	    StringBuffer ret = new StringBuffer();
	    PortRoleAssociation[] assocs = this.getAssociations();
	    int i = getSourceIndex();
	    int j = getTargetIndex();
	    boolean bi = false;
	    
	    if (i == -1)
	    {
	        i = 0;
	        j = 1;
	        bi = true;
	    }
	    
	    ret.append(assocs[i]._portType.equals("") ? "*" : assocs[i]._portType);
	    //ret.append(" " + (bi ? "<-> " : "-> "));
	    ret.append(" -> ");
	    ret.append(assocs[j]._portType.equals("") ? "*" : assocs[j]._portType);
	       
	    return ret.toString();
	}
	
	/**
	 * Get the ports involved in the connection rule public AcmePort
	 * 
	 * @author Larry Waldman
	 *  
	 */
	public AcmePortType[] getPortTypes()
	{
		AcmePortType[] types = new AcmePortType[_associations.size()];
		int index = 0;
		for (Iterator iter = _associations.iterator(); iter.hasNext();)
		{
			PortRoleAssociation assoc = (PortRoleAssociation) iter.next();
			types[index++] = new AcmePortType(assoc.getPortType());
		}
		return types;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * @param name The name to set.
	 */
	public void setName(String name)
	{
		this.name = name;
	}

    /**
     * @return Returns the priorityRule.
     */
    public boolean isPriorityRule()
    {
        return priorityRule;
    }
    /**
     * @param priorityRule The priorityRule to set.
     */
    public void setPriorityRule(boolean priorityRule)
    {
        this.priorityRule = priorityRule;
    }
    
    /*
     * @author LFW
     * 
     * This method is used when an orthogonal bidirectional rule must
     * be created.  It returns the analgous rule object representing the reverse
     * of the current rule
     */
    public PortConnectionRule getReverse()
    {
        PortConnectionRule rev;
        
        rev = PortConnectionRule.getClone(this);
        rev.setName(this.name + "_reverse");
        rev._associations = reverseList(rev._associations);
        
        return (rev);
    }
    
    private List reverseList(List a)
    {
        List ret = new ArrayList();
        
        for (int i=a.size()-1; i >= 0; i--)
        {
            ret.add(a.get(i));
        }
        
        return ret;
    }

	/**
	 * @param doc
	 * @param this
	 * @return
	 */
	public Element exportAsXMLElement(Document doc) {
		Element e = doc.createElement ("portConnectionRule");
		Utils.safeSetXMLAttribute(e, "connector", getConnectorType ());
		Utils.safeSetXMLAttribute(e, "name", getName ());
		PortRoleAssociation [] assocs = getAssociations();
		for (int i = 0; i < assocs.length; i++) {
			Element assoc = doc.createElement ("association");
			Utils.safeSetXMLAttribute(assoc, "portType", assocs[i].getPortType());
			Utils.safeSetXMLAttribute(assoc, "roleType", assocs[i].getRoleType());
			Utils.safeSetXMLAttribute(assoc, "componentType", assocs[i].getComponentType());
			if (assocs[i].getRoleName() != null)
			{
				assoc.setAttribute("roleName", assocs[i].getRoleName());
			}
			if (assocs[i].getPortName() != null)
			{
			    assoc.setAttribute("portName", assocs[i].getPortName());
			}
			e.appendChild(assoc);
		}
		return e;
	}

	public static PortConnectionRule importFromXMLElement (Element e)
	{
	    // Need to be backward compatible
	    if (!e.getAttribute("port1").equals(""))
	    {
	    	// Old style
	    	PortConnectionRule rule = new PortConnectionRule();
	    	PortRoleAssociation assoc1 = new PortRoleAssociation(e
	    			.getAttribute("port1"), e.getAttribute("role1"),
	    			null);
	    	PortRoleAssociation assoc2 = new PortRoleAssociation(e
	    			.getAttribute("port2"), e.getAttribute("role2"),
	    			null);
	    	rule.addAssociation(assoc1);
	    	rule.addAssociation(assoc2);
	    	rule.setConnectorType(e.getAttribute("connector"));
	    	return rule;
	    }
	    else
	    {
	    	String source = e.getAttribute("source");
	    	String connector = e.getAttribute("connector");
	    	String name = e.getAttribute("name");
	    	PortConnectionRule rule = new PortConnectionRule();
	    	rule.setConnectorType(connector);
	    	rule.setName(name);
	    	NodeList associations = e
	    			.getElementsByTagName("association");
	    	for (int j = 0; j < associations.getLength(); j++)
	    	{
	    		Element association = (Element) associations.item(j);
	    		String portType = association.getAttribute("portType");
	    		String roleType = association.getAttribute("roleType");
	    		String roleName = association.getAttribute("roleName");
	    		String portName = association.getAttribute("portName");
	    		if (portName == null || portName.equals(""))
	    		{
	    		    portName = "p";
	    		}
	    		String componentType = association
	    				.getAttribute("componentType");
	    		PortRoleAssociation assoc = new PortRoleAssociation(
	    				portType, roleType, roleName, portName, componentType);
	    		rule.addAssociation(assoc);
	    	}
	    	rule.setSource(source);
	    	return rule;
	    }
	}
	
	
	/**
	 * @param obj
	 * @return
	 * @see java.lang.Object#equals(java.lang.Object)
	 **/
	public boolean equals(Object obj) {
		if (obj instanceof PortConnectionRule) {
			PortConnectionRule pcr = (PortConnectionRule )obj;
			if (!pcr.name.equals (name)) return false;
			if (!pcr._connectorType.equals (_connectorType)) return false;
			if (pcr._source != _source) return false;
			if (pcr._associations.size() != _associations.size ()) return false;
			boolean eq = true;
			for (Iterator iter = pcr._associations.iterator (); iter.hasNext () && eq;) {
				PortRoleAssociation assoc = (PortRoleAssociation )iter.next ();
				eq = _associations.contains(assoc);
			}
			return eq;
		}
		return false;
	}
}


