/*
 * Created on Oct 14, 2003
 *  
 */
package AcmeStudio.wizards.connection;

import java.util.ArrayList;
import java.util.List;

import AcmeStudio.core.ASContext;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeComponentType;
import acmeLib.elements.AcmeConnectorType;
import acmeLib.elements.AcmeElementType;
import acmeLib.elements.AcmePortType;
import acmeLib.elements.AcmeSystem;
import acmeLib.typeManagement.AcmeTypeManager;

/**
 * @author Bradley Schmerl Created Oct 14, 2003
 */
public class PortConnectionRuleKey
{
	List _ports = new ArrayList();
	//LFW added: comps -- holds components for rule
	List _comps = new ArrayList();
	int _source = -1;
	String connector;
	
	/**
	 *  
	 */

	public PortConnectionRuleKey(
			PortConnectionRule rule)
	{
		super();
		PortRoleAssociation[] assocs = rule.getAssociations();
		for (int i = 0; i < assocs.length; i++)
		{
			//if (assocs[i].getPortType() != null && !assocs[i].getPortType().equals(""))
			_ports.add((assocs[i].getPortType() == null || assocs[i].getPortType().equals("")) ? null : assocs[i].getPortType());
			
			//if (assocs[i].getComponentType() != null && !assocs[i].getComponentType().equals("") )
			_comps.add((assocs[i].getComponentType() == null || assocs[i].getComponentType().equals("")) ? null : assocs[i].getComponentType());
		}
		_source = rule.getSourceIndex();
		connector = rule.getConnectorType();
	}

	public PortConnectionRuleKey(
			AcmePortType[] ports, AcmeComponentType[] comps, AcmeConnectorType conn)
	{
		for (int i = 0; i < ports.length; i++)
		{
			//if (ports[i] != null)
				_ports.add((ports[i] == null || ports[i].equals("")) ? null : ports[i].getTypeName());
		}
		for (int i = 0; i < comps.length; i++)
		{
			//if (comps[i] != null)
				_comps.add((comps[i] == null || comps[i].equals("")) ? null : comps[i].getTypeName());
		}
		connector = conn == null ? null : conn.getTypeName(); 
	}

	public List getPortTypes()
	{
		return _ports;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	/*
	 * Modified 6/3/04 by LFW Change: the equals method has been upgraded to
	 * support components in the check
	 */
	public boolean matches(Object arg0)
	{	
		if (arg0 instanceof PortConnectionRuleKey)
		{
			PortConnectionRuleKey key = (PortConnectionRuleKey) arg0;

			boolean retval = true;
			boolean genericRule = false;
			
			//always match the generic rule, if this is what we are checking against
			if (key._comps.get(0) == null && key._comps.get(1) == null)
			    if (key._ports.get(0) == null && key._ports.get(1) == null)
			        if (key.connector.equals(""))
			            genericRule = true;
			
			for (int i = 0; i < 2 && retval == true && genericRule == false; i++)
			{
				String thisPort = (String) this._ports.get(i);
				String arg0Port = (String) key._ports.get(i);
				retval = PortConnectionRuleKey.isSubtype(thisPort, arg0Port);
			}
			for (int i = 0; i < 2 && retval == true && genericRule == false; i++)
			{
			    String thisComp = (String) this._comps.get(i);
				String arg0Comp = (String) key._comps.get(i);
				
				retval = PortConnectionRuleKey.isSubtype(thisComp, arg0Comp);				
			}				 
			
			if (retval == true && genericRule == false) //check the connector, but only if this rule is still
			{ 							//a viable match
			    retval = PortConnectionRuleKey.isSubtype(connector, key.connector);
			}
			
			//LFW: modified 7/6 to ensure that we only try to match in one direction
			//This is allowable as per the decision made by Bradley Schmerl and I
			//to remove the notion of bidirectionality and simply create two reverse rules
			//for the rules that are created with bidirectionality desired
			
			return (genericRule) ? genericRule : retval;
		}
		return false;
	}
	
	/*
	 * Modified 6/3/04 by LFW Change: the equals method has been upgraded to
	 * support components in the check
	 */
	public boolean equals(Object arg0)
	{	
		if (arg0 instanceof PortConnectionRuleKey)
		{
			PortConnectionRuleKey key = (PortConnectionRuleKey) arg0;

			boolean retval = true;
						
			for (int i = 0; i < 2 && retval == true; i++)
			{
				if (this._ports.get(i) != null && key._ports.get(i) != null)
				{
					if (!this._ports.get(i).equals(key._ports.get(i)))
						retval = false;
				}					
			}
			
			for (int i = 0; i < 2 && retval == true; i++)
			{
				if (this._comps.get(i) != null && key._comps.get(i) != null)
				{
					if (!this._comps.get(i).equals(key._comps.get(i)))
						retval = false;
				}				
			}				 
			
			if (retval == true) //test the connector, but only if equality is still viable
			{
			    if (this.connector != null && !this.connector.equals("") && key.connector != null && !key.connector.equals(""))
			        if (!this.connector.equals(key.connector))
			            retval = false;
			}
			
			//LFW: modified 7/6 to ensure that we only try to match in one direction
			//This is allowable as per the decision made by Bradley Schmerl and I
			//to remove the notion of bidirectionality and simply create two reverse rules
			//for the rules that are created with bidirectionality desired
			
			return retval;
		}
		return false;
	}
	
	/**
     * @param retval
     * @param context
     * @param thisComp
     * @param arg0Comp
     * @return
     */
    public static boolean isSubtype(String thisComp, String arg0Comp)
    {
        boolean retval = true;
		AcmeSystem context = Helper.getAcmeSystem(ASContext.current().getActiveDesign().getDesign());
		
        if (thisComp != null && arg0Comp != null)
        {
            //Convert type names to Acme types in the current context
        	Object thisCompObj = context.lookupName(thisComp, true);
        	Object arg0CompObj = context.lookupName (arg0Comp, true);
        	if (!(thisCompObj instanceof AcmeElementType) || !(arg0CompObj instanceof AcmeElementType)) {
        		retval = false;
        	}
        	else {
        		AcmeElementType thisCompType = (AcmeElementType) thisCompObj;
        		AcmeElementType arg0CompType = (AcmeElementType) arg0CompObj;
        		
        		if (!AcmeTypeManager.isDeclaredSubtype(thisCompType, arg0CompType)) {
        			retval = false;
        		}
        		
        	}
        }
        return retval;
    }

    public String toString()
	{
	    StringBuffer ret = new StringBuffer();
	    
	    ret.append("Ports: " + _ports.toString() + "\n");
	    ret.append("Comps: " + _comps.toString() + "\n");
	    ret.append("Source: " + _source + "\n");
	    ret.append("Connector: " + connector + "\n");
	    return (ret.toString());
	    
	}
	
	
}