/*
 * Created on Oct 6, 2003
 *
 */
package AcmeStudio.wizards.convertFromVersion1;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.util.Helper;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeException;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.globals.ConstantTypes;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeFamily;
import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl Created Oct 6, 2003 */
public class SplitAcmeDesignWizard extends Wizard {

	private AcmeDesign _design;
	private SplitPage[] _pages;
	private OpenClarificationPage _openClarificationPage;

	/**
	 * 
	 */
	public SplitAcmeDesignWizard(AcmeDesign design) {
		super();
		_design = design;
	}

	/* (non-Javadoc) @see org.eclipse.jface.wizard.IWizard#performFinish() */
	public boolean performFinish() {
		List importIntoOldDesign = new ArrayList ();
		List newDesigns = new ArrayList ();
		for (int i = 0; i < _pages.length; i++) {
			SplitPage page = _pages[i];
			// Create the design
			final AcmeDesign newDesign = new AcmeDesign();
			List typesToImport = null;
			if (page.getFamily() instanceof AcmeFamily) {
				AcmeFamily family = (AcmeFamily) page.getFamily();
				try {
					newDesign.addFamily(family);
				}
				catch (AcmeIllegalNameException e) {
					e.printStackTrace();
				}
				typesToImport = Helper.convertAcmeEnumerationToList(family.getSuperTypes().enumerate());
				_design.removeFamily (family.getName ());
				importIntoOldDesign.add (page);
//				_design.addDesignToImport(family.getName());
			}
			else {
				try {
					newDesign.addSystem(page.getFamily());
				}
				catch (AcmeIllegalNameException e) {
					e.printStackTrace();
				}
				typesToImport = getTypesToImportFromSystem(page.getFamily ());
				_design.removeSystem (page.getFamily());
			}
			for (Iterator iter = typesToImport.iterator(); iter.hasNext();) {
				AcmeFamily family = (AcmeFamily) iter.next();
				if (family != ConstantTypes.SYSTEM_BASE_TYPE) {
					newDesign.addDesignToImport(family.getName ());
				}
			}


			// Create the files
			
			final String containerName = page.getContainerName();
			final String filename = page.getFilename();
			WorkspaceModifyOperation op = new WorkspaceModifyOperation () {
				public void execute (final IProgressMonitor monitor) throws CoreException {
					try {
						createNewDesignFile(newDesign, containerName, filename);	
					}
					finally {
						monitor.done ();
					}
				}
			};
			try {
				getContainer ().run(false, false, op);
				
			}
			catch (InterruptedException e) {
				return false;
			}
			catch (InvocationTargetException e) {
				Throwable realException = e.getTargetException();
				MessageDialog.openError (getShell (), "Error", realException.getMessage ());
				return false;
			}
			newDesigns.add(newDesign);
		}
		
//		for (Iterator iter = newDesigns.iterator(); iter.hasNext();) {
//			AcmeDesign design = (AcmeDesign) iter.next();
//			try {
//				Helper.resolveAllInDesign(null, design);
//			}
//			catch (ParseException e) {
//				e.printStackTrace();
//			}
//			catch (acmeLib.armaniParser.ParseException e) {
//				e.printStackTrace();
//			}
//		}

		importIntoOldDesign = getTypesToImportFromSystem(Helper.getAcmeSystem(_design));
		for (Iterator iter = importIntoOldDesign.iterator(); iter.hasNext();) {
//			SplitPage page = (SplitPage) iter.next();
			AcmeFamily fam = (AcmeFamily )iter.next ();
			_design.addDesignToImport(fam.getName ());
			
		}
		return true;
	}

	private List getTypesToImportFromSystem(AcmeSystem system) {
		List typesToImport;
		AcmeEnumeration types = system.getDeclaredTypes();
		types.concatenate(system.getInstantiatedTypes());
		typesToImport = Helper.convertAcmeEnumerationToList(types.castToAcmeSetVector().enumerate());
		return typesToImport;
	}

	private void createNewDesignFile(AcmeDesign newDesign, final String containerName, final String filename) throws CoreException {
		IPath contPath = new Path(containerName);
		IContainer cont = AcmeStudioPlugin.getRoot().getProject(contPath.segment(0));
		if (cont == null || !cont.exists()) {
			IStatus status = new Status (IStatus.ERROR, "AcmeStudio", IStatus.OK, "The project \"" + contPath.segment(0) + "\" does not exist!", null);
			throw new CoreException (status);
		}
		if (contPath.segmentCount() > 1) {
			IFolder folder = cont.getFolder(contPath.removeFirstSegments(1));
			if (!folder.exists()) {
				folder.create(true, true, null);
			}
			cont = folder;
		}
		final IFile f = cont.getFile(new Path(Helper.convertToAcmeFilename(filename)));
		try {
			AcmeOutputStream aos = new AcmeOutputStream ();
			newDesign.exportArmani(aos);
			aos.close ();
			InputStream stream = new ByteArrayInputStream (aos.toString ().getBytes ());
			if (f.exists ()) {
				f.setContents (stream, true, true, null);
			}
			else {
				f.create(stream, true, null);
			}
			stream.close ();
		}
		catch (IOException e) {}
		catch (AcmeException e) {}
	}

	/* (non-Javadoc) @see org.eclipse.jface.wizard.IWizard#addPages() */
	public void addPages() {
		AcmeEnumeration families = _design.getFamilies();
		AcmeEnumeration systems = _design.getSystems();
		_pages = new SplitPage[families.getSize() + systems.getSize() - 1];
		int index = 0;
		while (families.moreItems()) {
			AcmeFamily fam = (AcmeFamily) families.getNextItem();
			_pages[index++] = new SplitPage(fam);
			addPage(_pages[index - 1]);
		}

		if (systems.getSize () > 1) {
			while (systems.moreItems()) {
				AcmeSystem sys = (AcmeSystem) systems.getNextItem();
				if (systems.moreItems()) {
					_pages[index] = new SplitPage(sys);
					addPage(_pages[index++]);
				}
			}
		}
		_openClarificationPage = new OpenClarificationPage(_pages);
		addPage(_openClarificationPage);
	}

}
