/*
 * Created on Oct 6, 2003
 *
 */
package AcmeStudio.wizards.convertFromVersion1;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;

import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.ASContext;
import AcmeStudio.util.Helper;
import acmeLib.elements.AcmeFamily;
import acmeLib.elements.AcmeSystem;

/**
 * @author Bradley Schmerl Created Oct 6, 2003 */
public class SplitPage extends WizardPage {
	protected final String FAMDIR = AcmeStudioPlugin.getResourceString("AcmeStudio.family.subdir");
	private AcmeSystem _family;
	private Text _txtContainer;
	private Text _txtFilename;
	private Button _btnOverwriteExisting;
	private boolean _isFamily;
	private boolean _open = true;

	/**
	 * @param pageName */
	protected SplitPage(AcmeFamily fam) {
		super("Create file for " + fam.getName());
		_family = fam;
		_isFamily = true;
	}
	
	protected SplitPage (AcmeSystem sys) {
		super ("Create file for " + sys.getName ());
		_family = sys;
		_isFamily = false;
	}

	/*
	 * (non-Javadoc) @see
	 * org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite top = new Composite(parent, SWT.NONE);
		setControl(top);
		GridLayout tgl = new GridLayout(3, false);
		top.setLayout(tgl);

		Label famLabel = new Label(top, SWT.NONE);
		String l = _isFamily?"Family":"System";
		famLabel.setText(l + " : " + _family.getName());
		GridData flgd = new GridData();
		flgd.horizontalSpan = GridData.FILL;
		famLabel.setLayoutData(flgd);

		Label projLabel = new Label(top, SWT.NONE);
		projLabel.setText("&Container:");
		_txtContainer = new Text(top, SWT.BORDER | SWT.SINGLE);
		_txtContainer.setText("/");
		_txtContainer.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});
		_txtContainer.setLayoutData(new GridData (GridData.HORIZONTAL_ALIGN_FILL));
		Button browse = new Button(top, SWT.PUSH);
		browse.setText("&Browse...");
		browse.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleBrowse();
			}
		});

		Label fileLabel = new Label(top, SWT.NONE);
		fileLabel.setText("Filename:");
		_txtFilename = new Text(top, SWT.BORDER | SWT.SINGLE);
		_txtFilename.setText(_family.getName());
		GridData fgd = new GridData(GridData.FILL_HORIZONTAL);
		_txtFilename.setLayoutData(fgd);
		_txtFilename.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});


		
		_btnOverwriteExisting = new Button (top, SWT.CHECK);
		_btnOverwriteExisting.setText ("Overwrite existing file");
		_btnOverwriteExisting.setEnabled(false);
		_btnOverwriteExisting.addSelectionListener(new SelectionAdapter () {
			public void widgetSelected (SelectionEvent e) {
				validate ();
			}
		});
		GridData oegd = new GridData ();
		oegd.horizontalSpan = GridData.FILL;
		_btnOverwriteExisting.setLayoutData(oegd);
		IProject proj = ASContext.current().getActiveProject();
		if (proj != null) {
			_txtContainer.setText(proj.getFullPath().toString());
		}
		validate ();
		
	}

	/**
	 * 
	 */
	protected void validate() {
				
		String container = _txtContainer.getText();
		String filename = _txtFilename.getText();

		if (container.length() == 0) {
			updateStatus("Container must be specified");
			return;
		}

		if (filename.length() == 0) {
			updateStatus("Filename must be specified");
			return;
		}
		updateContainer();  // enforce any container content rules
		container = getContainerName();
		IPath contPath = new Path(container);
		if (contPath.segmentCount() == 0) { // can't check
			return;
		}
		IContainer cont = AcmeStudioPlugin.getRoot().getProject(contPath.segment(0));
		if (cont == null || !cont.exists()) {
			updateStatus("The Project \"" + contPath.segment(0) + "\" does not exist!");
			return;
		}
		if (contPath.segmentCount() > 1) {
			cont = cont.getFolder(contPath.removeFirstSegments(1));
		}
		if (!cont.exists()) {
			updateStatus("The Project folder path " + container + " does not exist");
			return;
		}
		else {
			ASContext.current().setActiveProject(cont.getProject());
			IFile f = cont.getFile(new Path (Helper.convertToAcmeFilename(getFilename())));
			if (f.exists() && !_btnOverwriteExisting.getSelection()) {
				updateStatus("File " + f.getFullPath() + " already exists");
				_btnOverwriteExisting.setEnabled (true);
				return;
			}
		}
		
		if (!filename.equals (_family.getName ())) {
			String l = _isFamily?"family":"system";
			setErrorMessage (null);
			setMessage("Filename and " + l + " name should be the same", WARNING);
			setPageComplete (true);
			return;
		}
		
		_btnOverwriteExisting.setEnabled (false);
		updateStatus (null);
		
	}

	private void updateContainer() {
		if (_txtContainer != null && _isFamily) {
			String container = getContainerName();
			IPath containerPath = new Path(container);
			// check to make sure the second segment of the path is always FAMDIR
			if (containerPath.segmentCount() > 1) {
				if (! containerPath.segment(1).equals(FAMDIR)) {
					// if not, enforce it by inserting the FAMDIR into the 2nd segment
					IPath newCont = containerPath.uptoSegment(1);
					containerPath = containerPath.removeFirstSegments(1);
					containerPath = newCont.append(FAMDIR).append(containerPath).addTrailingSeparator();
					container = containerPath.toString();
					_txtContainer.setText(container);
					// BRS: Avoid eclipse bug that changes grid layout when something
					//      is selected in the text before the text is displayed
//					  _containerText.setSelection(location, getContainerName().length());
				}
			} else if (containerPath.segmentCount() == 1) {  // append FAMDIR
				containerPath = containerPath.append(FAMDIR).addTrailingSeparator();
				container = containerPath.toString();
				_txtContainer.setText(container);
				// BRS: Avoid eclipse bug that changes grid layout when something
				//      is selected in the text before the text is displayed
//					  _containerText.setSelection(getContainerName().length());
			}
                
			// check to make sure that no more than one segment is FAMDIR
			if (containerPath.segmentCount() > 2
				&& containerPath.segment(2).equals(FAMDIR)) {
				// if not, enforce it by removing the FAMDIR from the 3rd segment
				IPath newCont = containerPath.uptoSegment(2);
				containerPath = containerPath.removeFirstSegments(3);
				containerPath = newCont.append(containerPath).addTrailingSeparator();
				container = containerPath.toString();
				_txtContainer.setText(container);
				// BRS: Avoid eclipse bug that changes grid layout when something
				//      is selected in the text before the text is displayed
//				  _containerText.setSelection(getContainerName().length());
			}
		}
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}
	
	private void handleBrowse() {
		ContainerSelectionDialog dialog =
			new ContainerSelectionDialog(
				getShell(),
				ResourcesPlugin.getWorkspace().getRoot(),
				false,
				"Select new container");
		if (dialog.open() == ContainerSelectionDialog.OK) {
			Object[] result = dialog.getResult();
			if (result.length == 1) {
				Object obj = result[0];
				if (obj instanceof Path) {
					Path path = (Path) obj;
					_txtContainer.setText(path.toString());
				}
				else if (obj instanceof IContainer) {
					IContainer container = (IContainer) result[0];
					_txtContainer.setText(container.getFullPath().toString());
				}
			}
		}
	}

	/**
	 * @return Returns the family.
	 */
	public AcmeSystem getFamily() {
		return _family;
	}

	public String getContainerName () {
		return _txtContainer.getText ();
	}
	
	public String getFilename () {
		return _txtFilename.getText ();
	}

	/**
	 * @return
	 */
	public boolean willOpen() {
		return _open;
	}

	/**
	 * @param b
	 */
	public void setOpen(boolean b) {
		_open = b;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage() {
		IWizardPage next = super.getNextPage();
		if (next instanceof OpenClarificationPage) {
			OpenClarificationPage page = (OpenClarificationPage) next;
			page.refreshTable();
		}
		return next;
	}

}
