/*
 * Created on Jan 3, 2005
 *
 */
package AcmeStudioShapeExtension;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.palette.CombinedTemplateCreationEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.jface.resource.ImageDescriptor;
import AcmeStudio.ui.editors.diagram.palette.extension.IPaletteExtender;
import AcmeStudioShapeExtension.model.*;
import AcmeStudioShapeExtension.palette.LineCreationToolEntry;
import AcmeStudioShapeExtension.palette.ShapeCreationToolEntry;
import acmeLib.elements.AcmeSystem;


/**
 * @author Bradley Schmerl
 * Created Jan 3, 2005
 */
public class AnnotationPaletteExtender implements IPaletteExtender {

	/**
	 * Adds all the annotation drawing tools to the palette. The tools are added to parentGroup
	 * @param parentGroup
	 * @see AcmeStudio.ui.editors.diagram.palette.extension.IPaletteExtender#createPaletteChildren(org.eclipse.gef.palette.PaletteGroup)
	 **/
	public void createPaletteChildren(PaletteGroup parentGroup, AcmeSystem targetSystem) {
	
		CombinedTemplateCreationEntry component = new ShapeCreationToolEntry (
				"Rectangle",
				"Create a rectangular shape",
				RectangleAnnotation.class,
				new SimpleFactory (RectangleAnnotation.class),
				ImageDescriptor.createFromFile (AcmeStudioShapeExtensionPlugin.class, "icons/rectangle16.gif"),
				null);
		parentGroup.add (component);
			
	    component = new ShapeCreationToolEntry (
				"Ellipse",
				"Create an elliptical shape",
				EllipseAnnotation.class,
				new SimpleFactory (EllipseAnnotation.class),
				ImageDescriptor.createFromFile(AcmeStudioShapeExtensionPlugin.class, "icons/ellipse16.gif"),
				null);
		
		parentGroup.add (component);
		
		component = new ShapeCreationToolEntry (
				"RoundedRect",
				"Create a rounded rectangle",
				RoundedRectangleAnnotation.class,
				new SimpleFactory (RoundedRectangleAnnotation.class),
				ImageDescriptor.createFromFile (AcmeStudioShapeExtensionPlugin.class, "icons/rounded_rectangle16.gif"),
				null);
		parentGroup.add (component);
		
		component = new LineCreationToolEntry (
				"Line",
				"Create a line",
				LineAnnotation.class,
				new SimpleFactory (LineAnnotation.class),
				ImageDescriptor.createFromFile (AcmeStudioShapeExtensionPlugin.class, "icons/line16.gif"),
				null);
		parentGroup.add (component);
		
		component = new CombinedTemplateCreationEntry (
				"Label",
				"Create a label",
				LabelAnnotation.class,
				new SimpleFactory (LabelAnnotation.class),
				ImageDescriptor.createFromFile (AcmeStudioShapeExtensionPlugin.class, "icons/label16.gif"),
				null);
		parentGroup.add (component);
		
		component = new CombinedTemplateCreationEntry (
				"Note",
				"Create a note",
				NoteAnnotation.class,
				new SimpleFactory (NoteAnnotation.class),
				ImageDescriptor.createFromFile (AcmeStudioShapeExtensionPlugin.class, "icons/note16.gif"),
				null);
		parentGroup.add (component);
		
	}

	/**
	 * Returns true - annotations are always applicable
	 * @param system
	 * @return
	 * @see AcmeStudio.ui.editors.diagram.palette.extension.IPaletteExtender#isApplicable(acmeLib.elements.AcmeSystem)
	 **/
	public boolean isApplicable(AcmeSystem system) {
		return AcmeStudioShapeExtensionPlugin.getDefault().getPreferenceStore().getBoolean(AnnotationPrefs.ENABLE_ANNOTATIONS_PREF);
	}

	/**
	 * @param parentEditPart
	 * @param model
	 * @return
	 * @see AcmeStudio.ui.editors.diagram.palette.extension.IPaletteExtender#createEditPart(org.eclipse.gef.EditPart, java.lang.Object)
	 **/
	public EditPart createEditPart(EditPart parentEditPart, Object model) {
		return null;
	}

	/**
	 * @param toDrop
	 * @param over
	 * @return
	 * @see AcmeStudio.ui.editors.diagram.palette.extension.IPaletteExtender#canDrop(java.lang.Object, org.eclipse.gef.EditPart)
	 **/
	public boolean canDrop(Object toDrop, EditPart over) {
		return toDrop instanceof AnnotationElement;
	}

	/**
	 * @param toDrop
	 * @return
	 * @see AcmeStudio.ui.editors.diagram.palette.extension.IPaletteExtender#getCreationFactory(java.lang.Object)
	 **/
	public CreationFactory getCreationFactory(Object toDrop) {
		return null;
	}
}
