/*
 * Created on Oct 13, 2004
 *
 */
package AcmeStudioShapeExtension;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.core.resource.datapersistence.IDataPersistence;
import AcmeStudio.util.Helper;
import AcmeStudioShapeExtension.model.AcmeAnnotations;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.extensions.IElementExtension;
import acmeLib.elements.AcmeDesign;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeSystem;


/**
 * @author Bradley Schmerl
 * Created Oct 13, 2004
 */
public class AnnotationPersistor implements IDataPersistence {


	/**
	 * @author Bradley Schmerl
	 * Created Oct 13, 2004
	 */
	public class AnnotationTracker implements IElementExtension {

		/**
		 * @return
		 * @see acmeLib.core.extensions.IElementExtension#isPersistent()
		 **/
		public boolean isPersistent() {
			return false;
		}

		/**
		 * @param element
		 * @see acmeLib.core.extensions.IElementExtension#injectIntoElement(acmeLib.elements.AcmeElement)
		 **/
		public void injectIntoElement(AcmeElement element) {
		}

		/**
		 * @param element
		 * @see acmeLib.core.extensions.IElementExtension#extractFromElement(acmeLib.elements.AcmeElement)
		 **/
		public void extractFromElement(AcmeElement element) {
		}

		/**
		 * @param element
		 * @return
		 * @see acmeLib.core.extensions.IElementExtension#isInjected(acmeLib.elements.AcmeElement)
		 **/
		public boolean isInjected(AcmeElement element) {
			return false;
		}
	}
	/**
	 * Loads all annotations from a .annotatations file
	 * @param design
	 * @param resource
	 * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#load(AcmeStudio.core.resource.ASDesign, File)
	 **/
	public void load(ASDesign asDesign, File resource) {
		String acmeFilename = resource.getAbsolutePath ();
        AcmeDesign design = asDesign.getDesign();
		String annotationFilename = null;
		if (acmeFilename.endsWith (".acme")) {
			annotationFilename = createAnnotationFilename (asDesign, acmeFilename);
			File annotationFile = new File (annotationFilename);
			// Only try to load if there are annoations
			if (!annotationFile.exists ()) {
				return;
			}
			try {
				InputStream inFile = new FileInputStream (annotationFile); 
				ObjectInputStream in = new ObjectInputStream (inFile);
				AcmeAnnotations annotations = (AcmeAnnotations )in.readObject();
				in.close ();
				
				AcmeSystem system = Helper.getAcmeSystem (design);
				system.setUserData (AcmeStudioShapeExtensionPlugin.class, annotations);
				design.setUserData (AcmeStudioShapeExtensionPlugin.class, new AnnotationTracker ());
			}
			catch (FileNotFoundException e) {
				e.printStackTrace();
			}
			catch (IOException e) {
				e.printStackTrace();
			}
			catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
			
		}
	}

	/**
	 * @param design
	 * @param resource
	 * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#save(AcmeStudio.core.resource.ASDesign, File)
	 **/
	public void save(ASDesign asDesign, File resource) {
        AcmeDesign design = asDesign.getDesign();
		AcmeEnumeration systems = design.getSystems ();
		while (systems.moreItems ()) {
			AcmeSystem system = (AcmeSystem )systems.getNextItem ();
			AcmeAnnotations annotations = (AcmeAnnotations )system.getUserData (AcmeStudioShapeExtensionPlugin.class);
			String acmeFilename = resource.getAbsolutePath();
			String annotationFilename = null;
			if (annotations == null) {
				AnnotationTracker tracker = (AnnotationTracker )design.getUserData (AcmeStudioShapeExtensionPlugin.class);
				if (tracker == null) {
					return;
				}
				annotationFilename = createAnnotationFilename (asDesign, acmeFilename);
				File annotationFile = new File (annotationFilename);
				if (annotationFile.exists ()) {
					annotationFile.delete ();
					design.setUserData (AcmeStudioShapeExtensionPlugin.class, null);
				}
			}
			else if (acmeFilename.endsWith (".acme")){
				try {
					annotationFilename = createAnnotationFilename (asDesign, acmeFilename);
					ByteArrayOutputStream bos = new ByteArrayOutputStream ();
					ObjectOutputStream oos= new ObjectOutputStream (bos);
					oos.writeObject (annotations);
					oos.close ();
					bos.close ();
					File file = new File (annotationFilename);
					FileOutputStream fos = new FileOutputStream (file);
					fos.write(bos.toByteArray());
					fos.close ();
				}
				catch (FileNotFoundException e) {
					e.printStackTrace();
				}
				catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * @param design
	 * @param acmeFilename
	 * @return
	 */
	private String createAnnotationFilename(ASDesign design, String acmeFilename) {
		IPath dataPath = null;
		try {
			dataPath = design.getDataDirectory ();
		}
		catch (IOException e) {
			String annotationFilename = acmeFilename.substring (0, acmeFilename.length () - 5) + ".annotations";
			return annotationFilename;
		}
		
		Path acmeFilepath = new Path (acmeFilename);
		String filename = acmeFilepath.lastSegment ();
		dataPath = dataPath.append (filename.substring (0, filename.length () - 5)  + ".annotations");
		return dataPath.toString ();
	}

	/**
	 * @param design
	 * @return
	 * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#extractData(AcmeStudio.core.resource.ASDesign, java.io.File)
	 **/
	public Object extractData(ASDesign asDesign) {
		Map annotationMap = new HashMap ();
		AcmeEnumeration systems = asDesign.getDesign().getSystems ();
		while (systems.moreItems ()) {
			AcmeSystem system = (AcmeSystem )systems.getNextItem ();
			AcmeAnnotations annotations = (AcmeAnnotations )system.getUserData (AcmeStudioShapeExtensionPlugin.class);
			if (annotations != null) annotationMap.put (system.getName (), annotations);
		}
		return annotationMap;
	}

	/**
	 * @param design
	 * @param data
	 * @see AcmeStudio.core.resource.datapersistence.IDataPersistence#injectData(AcmeStudio.core.resource.ASDesign, java.io.File)
	 **/
	public void injectData(ASDesign asDesign, Object data) {
		Map annotationMap = (Map )data;
		for (Iterator iter = annotationMap.entrySet().iterator (); iter.hasNext ();) {
			Map.Entry entry = (Map.Entry )iter.next ();
			AcmeSystem system = asDesign.getDesign().getSystem((String )entry.getKey ());
			system.setUserData (AcmeStudioShapeExtensionPlugin.class, (AcmeAnnotations )entry.getValue ());
		}
	}
}
