/*
 * Created on Oct 15, 2004
 *
 */
package AcmeStudioShapeExtension.model;

import java.io.Serializable;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import AcmeStudioShapeExtension.AnnotationConstants;
import AcmeStudioShapeExtension.editors.FontCellEditor;


/**
 * @author Bradley Schmerl
 * Created Oct 15, 2004
 */
public class LabelAnnotation extends Annotation {

	public static final String FONT_PROP = "Label.Font";
	public static final String SIZE_PROP = "Label.Size";
	public static final String TEXT_PROP = "Label.Text";
	
	class FontInfo implements Serializable {
		String name;
		int height;
		int style;
	}
	
	protected String label;
	protected FontInfo fontInfo;
	transient protected FontData fontData;
	
	public LabelAnnotation (String label) {
		this ();
		setLabel (label);
	}
	/**
	 * 
	 */
	public LabelAnnotation() {
		super ();
		label = "label";
		descriptors.remove (3); descriptors.remove (2);
		descriptors.add (new TextPropertyDescriptor (TEXT_PROP, "Text"));
		PropertyDescriptor fontPropertyDescriptor = new PropertyDescriptor (FONT_PROP, "Font") {
			
			/**
			 * @param parent
			 * @return
			 * @see org.eclipse.ui.views.properties.PropertyDescriptor#createPropertyEditor(org.eclipse.swt.widgets.Composite)
			 **/
			public CellEditor createPropertyEditor(Composite parent) {
				CellEditor editor = new FontCellEditor (parent);
				if (getValidator () != null) editor.setValidator(getValidator ());
				return editor;
			}
			

		};
		fontPropertyDescriptor.setLabelProvider(new LabelProvider () {
			
			/**
			 * @param element
			 * @return
			 * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
			 **/
			public String getText(Object element) {
				if (element instanceof FontData) {
					FontData fd = (FontData )element;
					return AnnotationConstants.getFontDataString (fd);
				}
				else if (element instanceof FontData []) {
					FontData [] fd = (FontData [] )element;
					return AnnotationConstants.getFontDataString(fd [0]);
				}
				return "";
			}
		
		});
		descriptors.add(fontPropertyDescriptor);
	}
	
	
	/**
	 * 
	 * @see AcmeStudioShapeExtension.model.Annotation#initializeDescriptors()
	 **/
	protected void initializeDescriptors() {
		super.initializeDescriptors ();
	}
	/**
	 * @return
	 * @see AcmeStudioShapeExtension.model.Annotation#getIcon()
	 **/
	public Image getIcon() {
		return null;
	}
	
	
	
	/**
	 * @return Returns the label.
	 */
	public String getLabel() {
		return label;
	}
	/**
	 * @param label The label to set.
	 */
	public void setLabel(String label) {
		this.label = label;
		firePropertyChange(TEXT_PROP, "", label);
	}
	
	
	/**
	 * @param propertyId
	 * @return
	 * @see AcmeStudioShapeExtension.model.Annotation#getPropertyValue(java.lang.Object)
	 **/
	public Object getPropertyValue(Object propertyId) {
		if (FONT_PROP.equals (propertyId)) {
			return fontData;
		}
		else if (TEXT_PROP.equals (propertyId)) {
			return label;
		}
		else 
			return super.getPropertyValue (propertyId);
	}
	
	
	/**
	 * @param propertyId
	 * @param value
	 * @see AcmeStudioShapeExtension.model.Annotation#setPropertyValue(java.lang.Object, java.lang.Object)
	 **/
	public void setPropertyValue(Object propertyId, Object value) {
		if (FONT_PROP.equals (propertyId)) {
			setFontData (((FontData [])value) [0]);
		}
		else if (TEXT_PROP.equals(propertyId)) {
			setLabel ((String )value);
		}
		super.setPropertyValue (propertyId, value);
	}
	
	/**
	 * @return Returns the fontData.
	 */
	public FontData getFontData() {
		if (fontData == null && fontInfo != null) {
			fontData = new FontData (fontInfo.name, fontInfo.height, fontInfo.style);
		}
		
		return fontData;
	}
	/**
	 * @param fontData The fontData to set.
	 */
	public void setFontData(FontData fontData) {
		this.fontData = fontData;
		if (fontInfo == null) {
			fontInfo = new FontInfo ();
		}
		fontInfo.name = fontData.getName();
		fontInfo.height = fontData.getHeight();
		fontInfo.style = fontData.getStyle();
		
		firePropertyChange(FONT_PROP, null, fontData);
	}
	
	
	/**
	 * @return
	 * @see AcmeStudioShapeExtension.model.Annotation#getSize()
	 **/
	public Dimension getSize() {
		return new Dimension (size.width, -1);
	}
	
	
	/**
	 * @param d
	 * @see AcmeStudioShapeExtension.model.Annotation#setSize(org.eclipse.draw2d.geometry.Dimension)
	 **/
	public void setSize(Dimension d) {
		if (d == null) d = new Dimension (-1,-1);
		d.height = -1;
		super.setSize (d);
	}
}
