/*
 * Created on Oct 12, 2004
 *
 */
package AcmeStudioShapeExtension.model.command;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateRequest;
import AcmeStudio.core.ASContext;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudioShapeExtension.AcmeStudioShapeExtensionPlugin;
import AcmeStudioShapeExtension.model.AcmeAnnotations;
import AcmeStudioShapeExtension.model.Annotation;
import acmeLib.elements.AcmeSystem;


/**
 * @author Bradley Schmerl
 * Created Oct 12, 2004
 */
public class AnnotationCreateCommand extends Command {
	
	
	private final AcmeSystem	_system;
	private final CreateRequest	_request;
	private Annotation	_newAnnotation;
	private SystemEditPart	systemEditPart;

	/**
	 * 
	 */
	public AnnotationCreateCommand(AcmeSystem system, CreateRequest req) {
		if (system == null || req == null || !(req.getNewObject () instanceof Annotation)) {
			throw new IllegalArgumentException ();
		}
		_system = system;
		_request = req;
		systemEditPart =  (SystemEditPart )((DiagramEditor )ASContext.current().getActiveEditor().getActiveEditor()).getEditPart(_system);
		setLabel ("Create Annotation");	
	}
		
		
		/**
		 * @return
		 * @see org.eclipse.gef.commands.Command#canUndo()
		 **/
		public boolean canUndo() {
			return true;
		}
		
		
		/**
		 * 
		 * @see org.eclipse.gef.commands.Command#execute()
		 **/
		public void execute() {
			_newAnnotation = (Annotation )_request.getNewObject();
			Point location = _request.getLocation ().getCopy ();
			systemEditPart.getFigure ().translateToRelative(location);
			Dimension size = new Dimension (50,50);
			if (_request.getSize () != null) 
				size = _request.getSize().getCopy ();
			systemEditPart.getFigure ().translateToRelative (size);
			_newAnnotation.setLocation (location);
			_newAnnotation.setSize(size);
			_newAnnotation.setParentSystem(_system);
			redo ();
		}
		
		
		/**
		 * 
		 * @see org.eclipse.gef.commands.Command#redo()
		 **/
		public void redo() {
			AcmeAnnotations annotations = AcmeStudioShapeExtensionPlugin.getAnnotations (_system);
			
			annotations.add (_newAnnotation);
	
			systemEditPart.refresh ();
		}
		
		
		/**
		 * 
		 * @see org.eclipse.gef.commands.Command#undo()
		 **/
		public void undo() {
			AcmeStudioShapeExtensionPlugin.getAnnotations(_system).remove (_newAnnotation);
			
			systemEditPart.refresh ();
		}
}
