/*
 * Created on Oct 21, 2004
 *
 */
package AcmeStudioShapeExtension.model.command;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import AcmeStudioShapeExtension.model.LineAnnotation;


/**
 * @author Bradley Schmerl
 * Created Oct 21, 2004
 */
public class ChangeLineCommand extends Command {

	public static final int START = 0;
	public static final int END = 1;
	
	private final LineAnnotation	_model;
	private final int	_index;
	private final Point	_location;
	private Point	_oldPoint;

	
	
	/**
	 * @param annotation
	 * @param index
	 * @param location
	 */
	public ChangeLineCommand(LineAnnotation annotation, int index, Point location) {
		_model = annotation;
		_index = index;
		_location = location;
	}
	
	
	/**
	 * @return
	 * @see org.eclipse.gef.commands.Command#canUndo()
	 **/
	public boolean canUndo() {
		return true;
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 **/
	public void execute() {
		switch (_index) {
			case START:
				_oldPoint = _model.getStart();
				break;
			case END:
				_oldPoint = _model.getEnd ();
		}
		
		redo ();
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 **/
	public void redo() {
		if (_index == START) _model.setStart (_location);
		else if (_index == END) _model.setEnd( _location);
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 **/
	public void undo() {
		if (_index == START) _model.setStart (_oldPoint);
		else if (_index == END) _model.setEnd( _oldPoint);
	}
}
