/*
 * Created on Oct 20, 2004
 *
 */
package AcmeStudioShapeExtension.model.command;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateRequest;
import AcmeStudio.core.ASContext;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudio.ui.editors.diagram.parts.SystemEditPart;
import AcmeStudioShapeExtension.AcmeStudioShapeExtensionPlugin;
import AcmeStudioShapeExtension.model.AcmeAnnotations;
import AcmeStudioShapeExtension.model.LineAnnotation;
import acmeLib.elements.AcmeSystem;


/**
 * @author Bradley Schmerl
 * Created Oct 20, 2004
 */
public class LineAnnotationCreateCommand extends Command {

	private final AcmeSystem	_system;
	private final CreateRequest	_request;
	private LineAnnotation	_newLine;
	private SystemEditPart	systemEditPart;
	public static final Object	END	= "LineEnd";
	public static final Object	START	= "LineStart";
	
	

	/**
	 * @param system
	 * @param request
	 */
	public LineAnnotationCreateCommand(AcmeSystem system, CreateRequest request) {
		if (system == null || request == null || !(request.getNewObject() instanceof LineAnnotation)) {
			throw new IllegalArgumentException ();
		}
		_system = system;
		_request = request;
		systemEditPart =  (SystemEditPart )((DiagramEditor )ASContext.current().getActiveEditor().getActiveEditor()).getEditPart(_system);

		setLabel ("Create Line");
	}
	
	
	/**
	 * @return
	 * @see org.eclipse.gef.commands.Command#canUndo()
	 **/
	public boolean canUndo() {
		return true;
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 **/
	public void execute() {
		_newLine = (LineAnnotation )_request.getNewObject ();
		Point startLocation = (Point )_request.getExtendedData ().get(START);
//		systemEditPart.getFigure ().translateToRelative(startLocation);
		if (startLocation == null) startLocation = _request.getLocation ();
		
		Point endLocation = (Point )_request.getExtendedData().get(END);
//		systemEditPart.getFigure ().translateToRelative(endLocation);
		
		if (endLocation == null) endLocation = startLocation.getCopy ();
		
		if (endLocation.equals (startLocation)) {
			endLocation = endLocation.translate(50,0);
		}
		_newLine.setStart(startLocation);
		_newLine.setEnd (endLocation);
		
//		_newLine.setEnd (location.getCopy ().translate(50, 0));
		_newLine.setParentSystem (_system);
		redo ();
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 **/
	public void redo() {
		AcmeAnnotations annotations = AcmeStudioShapeExtensionPlugin.getAnnotations(_system);
		annotations.add (_newLine);
		
		systemEditPart.refresh ();
	}
	
	
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 **/
	public void undo() {
		AcmeStudioShapeExtensionPlugin.getAnnotations(_system).remove (_newLine);
		systemEditPart.refresh ();
	}
	
}
