/*
 * Created on Jan 6, 2005
 *
 */
package AcmeStudioShapeExtension.palette;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionRectangle;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.SnapToHelper;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.tools.CreationTool;
import AcmeStudioShapeExtension.model.command.LineAnnotationCreateCommand;


/**
 * @author Bradley Schmerl
 * Created Jan 6, 2005
 */
public class LineCreationTool extends CreationTool {
	
	private LineFeedbackFigure	lineFeedbackFigure;
	private SnapToHelper	helper;

	/**
	 * @param factory
	 */
	public LineCreationTool(CreationFactory factory) {
		super (factory);
	}

	
	/**
	 * @param button
	 * @return
	 * @see org.eclipse.gef.tools.CreationTool#handleButtonDown(int)
	 **/
	protected boolean handleButtonDown(int button) {
		if (button != 1) {
			setState(STATE_INVALID);
			handleInvalidInput();
			return true;
		}
		if (stateTransition(STATE_INITIAL, STATE_DRAG)) {
			getCreateRequest().setLocation(getLocation());
			lockTargetEditPart(getTargetEditPart());
			// Snap only when size on drop is employed
			helper = (SnapToHelper)getTargetEditPart().getAdapter(SnapToHelper.class);
			getTargetRequest ().getExtendedData().put (LineAnnotationCreateCommand.START, getStartLocation());
		}
		return true;
	}
	/**
	 * @return
	 * @see org.eclipse.gef.tools.CreationTool#handleDragInProgress()
	 **/
	protected boolean handleDragInProgress() {
		boolean ret = super.handleDragInProgress ();
		if (isInState(STATE_DRAG_IN_PROGRESS)) {
			showCreationFeedback ();
		}
		return ret;
	}
	
	private void showCreationFeedback() {
		Point start = getStartLocation ().getCopy ();
		Point end = getLocation ().getCopy ();
		getFeedbackFigure ().translateToRelative (start);
		getFeedbackFigure ().translateToRelative (end);
		getFeedbackFigure ().setStart (start);
		getFeedbackFigure ().setEnd (end);
	}
	
	private LineFeedbackFigure getFeedbackFigure () {
		if (lineFeedbackFigure == null) {
			lineFeedbackFigure = new LineFeedbackFigure ();
			addFeedback(lineFeedbackFigure);
		}
		return lineFeedbackFigure;
	}
	/**
	 * 
	 * @see org.eclipse.gef.tools.AbstractTool#handleFinished()
	 **/
	protected void handleFinished() {
		super.handleFinished ();
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.tools.CreationTool#deactivate()
	 **/
	public void deactivate() {
		if (isInState (STATE_DRAG_IN_PROGRESS)) {
			eraseCreationFeedback ();
			eraseTargetFeedback();
		}
		super.deactivate ();
		setState (STATE_TERMINAL);
		helper = null;
	}
	
	private void eraseCreationFeedback () {
		if (lineFeedbackFigure != null) {
			removeFeedback (lineFeedbackFigure);
			lineFeedbackFigure = null;
		}
	}
	
	
	/**
	 * @param button
	 * @return
	 * @see org.eclipse.gef.tools.CreationTool#handleButtonUp(int)
	 **/
	protected boolean handleButtonUp(int button) {
		if (stateTransition(STATE_DRAG | STATE_DRAG_IN_PROGRESS, STATE_TERMINAL)) {
			getTargetRequest().getExtendedData().put (LineAnnotationCreateCommand.END, getLocation ());
			eraseTargetFeedback ();
			eraseCreationFeedback ();
			performCreation (button);
		}
		handleFinished ();
		return true;
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.tools.CreationTool#updateTargetRequest()
	 **/
	protected void updateTargetRequest() {
		CreateRequest req = getCreateRequest();
		if (isInState (STATE_DRAG_IN_PROGRESS)) {
			if (!getCurrentInput ().isAltKeyDown() && helper != null) {
				Point start = (Point )req.getExtendedData ().get (LineAnnotationCreateCommand.START);
				Rectangle bounds = new Rectangle (start, start);
				PrecisionRectangle baseRect = new PrecisionRectangle (bounds);
				PrecisionRectangle result = baseRect.getPreciseCopy();
				helper.snapRectangle (req, PositionConstants.NSEW, baseRect, result);
				req.getExtendedData().put (LineAnnotationCreateCommand.START, result.getLocation());
				
				Point end = (Point )req.getLocation();
				 bounds = new Rectangle (end, end);
				 baseRect = new PrecisionRectangle (bounds);
				 result = baseRect.getPreciseCopy();
				helper.snapRectangle (req, PositionConstants.NSEW, baseRect, result);
				req.setLocation (result.getLocation());
				
			}
		}
	}
	
	class LineFeedbackFigure extends Figure {
		
		Point start, end;
		
		/**
		 * @param graphics
		 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
		 **/
		protected void paintFigure(Graphics graphics) {
			Rectangle bounds = getBounds ().getCopy ();
			bounds.width -=1;
			bounds.height -=1;

			graphics.setXORMode(true);
			graphics.setForegroundColor(ColorConstants.white);
			graphics.setBackgroundColor(ColorConstants.black);
			graphics.setLineStyle(Graphics.LINE_DOT);
			
			CreateRequest req = (CreateRequest )getTargetRequest ();

			graphics.drawLine (start, end);
			
		}
		public void setEnd(Point end) {
			this.end = end;
			if (start != null)
				setBounds (new Rectangle (start, end));
		}
		public void setStart(Point start) {
			this.start = start;
			if (end != null) 
				setBounds (new Rectangle (start, end));
		}
	}
	
}
