/*
 * Created on Jan 6, 2005
 *
 */
package AcmeStudioShapeExtension.palette;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.tools.CreationTool;
import AcmeStudioShapeExtension.figures.ShapeFeedbackFigure;
import AcmeStudioShapeExtension.model.Annotation;


/**
 * @author Bradley Schmerl
 * Created Jan 6, 2005
 */
public class ShapeCreationTool extends CreationTool {
	
	private ShapeFeedbackFigure	shapeFeedbackFigure;

	/**
	 * @param factory
	 */
	public ShapeCreationTool(CreationFactory factory) {
		super (factory);
	}

	
	/**
	 * @return
	 * @see org.eclipse.gef.tools.CreationTool#handleDragInProgress()
	 **/
	protected boolean handleDragInProgress() {
		boolean ret = super.handleDragInProgress ();
		if (isInState(STATE_DRAG_IN_PROGRESS)) {
			showCreationFeedback ();
		}
		return ret;
	}
	
	private void showCreationFeedback() {
		CreateRequest req = (CreateRequest )getTargetRequest ();
		Rectangle rect = new Rectangle (req.getLocation (), req.getSize ());
		getFeedbackFigure ().translateToRelative(rect);
		getFeedbackFigure().setBounds (rect);
	}
	
	private IFigure getFeedbackFigure () {
		if (shapeFeedbackFigure == null) {
			shapeFeedbackFigure = new ShapeFeedbackFigure ((Annotation )((CreateRequest )getTargetRequest()).getNewObject ());
			FigureUtilities.makeGhostShape(shapeFeedbackFigure);
			shapeFeedbackFigure.setLineStyle (Graphics.LINE_DOT);
			shapeFeedbackFigure.setForegroundColor (ColorConstants.black);
			addFeedback(shapeFeedbackFigure);
		}
		return shapeFeedbackFigure;
	}
	/**
	 * 
	 * @see org.eclipse.gef.tools.AbstractTool#handleFinished()
	 **/
	protected void handleFinished() {
		super.handleFinished ();
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.tools.CreationTool#deactivate()
	 **/
	public void deactivate() {
		if (isInState (STATE_DRAG_IN_PROGRESS)) {
			eraseCreationFeedback ();
			eraseTargetFeedback();
		}
		super.deactivate ();
		setState (STATE_TERMINAL);
	}
	
	private void eraseCreationFeedback () {
		if (shapeFeedbackFigure != null) {
			removeFeedback (shapeFeedbackFigure);
			shapeFeedbackFigure = null;
		}
	}
	
	
	/**
	 * @param button
	 * @return
	 * @see org.eclipse.gef.tools.CreationTool#handleButtonUp(int)
	 **/
	protected boolean handleButtonUp(int button) {
		if (stateTransition(STATE_DRAG | STATE_DRAG_IN_PROGRESS, STATE_TERMINAL)) {
			eraseTargetFeedback ();
			eraseCreationFeedback ();
			performCreation (button);
		}
		handleFinished ();
		return true;
	}
	
}
