/*
 * Created on Oct 15, 2004
 *
 */
package AcmeStudioShapeExtension.parts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;

import AcmeStudioShapeExtension.AnnotationConstants;
import AcmeStudioShapeExtension.figures.IAnnotationTextProvider;
import AcmeStudioShapeExtension.figures.LabelFigure;
import AcmeStudioShapeExtension.figures.StickyNoteFigure;
import AcmeStudioShapeExtension.model.AnnotationElement;
import AcmeStudioShapeExtension.model.LabelAnnotation;
import AcmeStudioShapeExtension.model.NoteAnnotation;
import AcmeStudioShapeExtension.policies.LabelDirectEditPolicy;
import AcmeStudioShapeExtension.policies.ShapeComponentEditPolicy;


/**
 * @author Bradley Schmerl
 * Created Oct 15, 2004
 */
public class LabelEditPart extends AnnotationEditPart {

	private LabelEditManager	manager;

	
	/**
	 * 
	 */
	public LabelEditPart() {
		super ();
	}
	/**
	 * @return
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 **/
	protected IFigure createFigure() {
		Figure text = null;
		if (getModel () instanceof NoteAnnotation) {
			StickyNoteFigure note = new StickyNoteFigure ();
			note.setText ("");
			text = note;
		}
		else  text = new LabelFigure ("label");
		return text;
	}

	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 **/
	protected void createEditPolicies() {
		installEditPolicy (EditPolicy.COMPONENT_ROLE, new ShapeComponentEditPolicy ());
		installEditPolicy (EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy ());
	}

	/**
	 * @param evt
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 **/
	public void propertyChange(PropertyChangeEvent evt) {
		refreshVisuals ();
	}

	/**
	 * @param connection
	 * @return
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 **/
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return new ChopboxAnchor (getFigure ());
	}

	/**
	 * @param connection
	 * @return
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 **/
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return new ChopboxAnchor (getFigure ());
	}

	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 **/
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return new ChopboxAnchor (getFigure ());
	}

	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 **/
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return new ChopboxAnchor (getFigure ());
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.EditPart#deactivate()
	 **/
	public void deactivate() {
		if (isActive ()) {
			super.deactivate ();
			((AnnotationElement )getModel ()).removePropertyChangeListener (this);
		}
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.EditPart#activate()
	 **/
	public void activate() {
		if (!isActive ()) {
			super.activate ();
			((AnnotationElement )getModel ()).addPropertyChangeListener(this);
		}
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 **/
	protected void refreshVisuals() {
		LabelAnnotation anno = (LabelAnnotation )getModel ();
		FontData fd = anno.getFontData ();
		Font labelFont = null;
		if (fd != null)
			labelFont = AnnotationConstants.getFont (anno.getFontData());
		else
			labelFont = figure.getFont();
//		Dimension size = FigureUtilities.getTextExtents(anno.getLabel (), labelFont);
//		if (!anno.getSize ().equals (size)) anno.setSize(size);
		Rectangle bounds = new Rectangle (anno.getLocation (), anno.getSize ());
		figure.setBounds (bounds);
		((GraphicalEditPart )getParent ()).setLayoutConstraint (this, figure, bounds);
		figure.setForegroundColor(AnnotationConstants.getColor (anno.getForegroundColor())); 
		figure.setBackgroundColor(AnnotationConstants.getColor(anno.getBackgroundColor()));
		
		figure.setFont(labelFont);
		((IAnnotationTextProvider )figure).setText(anno.getLabel ());
	}
	
	private void performDirectEdit () {
		if (manager == null) 
			manager = new LabelEditManager (this, TextCellEditor.class, new LabelCellEditorLocator (getFigure ()));
		manager.show ();
	}
	
	
	/**
	 * @param request
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 **/
	public void performRequest(Request request) {
		if (request.getType () == RequestConstants.REQ_DIRECT_EDIT) {
			performDirectEdit ();
		}
	}
}
