/*
 * Created on Oct 20, 2004
 *
 */
package AcmeStudioShapeExtension.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.ConcurrentModificationException;
import java.util.List;

import org.eclipse.draw2d.AbsoluteBendpoint;
import org.eclipse.draw2d.Bendpoint;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.XYAnchor;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ZoomListener;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;

import AcmeStudioShapeExtension.AnnotationConstants;
import AcmeStudioShapeExtension.model.AnnotationElement;
import AcmeStudioShapeExtension.model.LineAnnotation;
import AcmeStudioShapeExtension.model.LineBendpoint;
import AcmeStudioShapeExtension.model.command.ChangeLineCommand;
import AcmeStudioShapeExtension.model.command.LinePointRequest;
import AcmeStudioShapeExtension.policies.LineBendpointEditPolicy;
import AcmeStudioShapeExtension.policies.ShapeComponentEditPolicy;


/**
 * @author Bradley Schmerl
 * Created Oct 20, 2004
 */
public class LineEditPart extends AbstractConnectionEditPart implements IAnnotationEditPart,PropertyChangeListener {

	/**
	 * @return
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 **/
	protected IFigure createFigure() {
		PolylineConnection line = new PolylineConnection ();
		line.setConnectionRouter(new BendpointConnectionRouter () {
			public Object getConstraint(Connection connection) {
				Object cons =  super.getConstraint (connection);
				if (cons == null) {
					cons = new ArrayList ();
					setConstraint(connection, cons);
				}
				return cons;
			}
			
			public void route(Connection conn) {
				super.route (conn);
			}
		});
		
		// This hack is to make the line position properly because the endpoints are screwed up!
		if (getParent ().getParent () instanceof ScalableFreeformRootEditPart) {
			final ScalableFreeformRootEditPart sfrep = (ScalableFreeformRootEditPart )getParent ().getParent ();
			sfrep.getZoomManager().addZoomListener(new ZoomListener () {

				public void zoomChanged(double zoom) {
					LineEditPart.this.refreshVisuals();
					
					// Can't remove the listener here, because we get a concurrent modification exception
					// so start a thread and and try removing until successful.
					final ZoomListener listener = this;
					Runnable unschedule = new Runnable () {
						public void run () {
							boolean doRun = true;
							while (doRun) {
								try {
									sfrep.getZoomManager().removeZoomListener(listener);
									doRun = false;
								}
								catch (ConcurrentModificationException e ) {
									try {
										Thread.sleep (1000);
									}
									catch (InterruptedException e1) {
									}
								}
							}
						}
					};
					Thread thread = new Thread (unschedule);
					thread.start();
				}
				
			});
		}
		return line;
	}

	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 **/
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE, new ConnectionEndpointEditPolicy ());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new LineBendpointEditPolicy ());
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new ShapeComponentEditPolicy ());
//		installEditPolicy(EditPolicy.CONNECTION_ROLE, new LineEditPolicy ());
//		installEditPolicy (EditPolicy.CONNECTION_BENDPOINTS_ROLE, new LineBendpointPolicy ());

	}

	/**
	 * @param evt
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 **/
	public void propertyChange(PropertyChangeEvent evt) {
		refreshVisuals ();
	}

	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 **/
	public void deactivate() {
		if (isActive ()) {
			super.deactivate ();
			((AnnotationElement )getModel ()).removePropertyChangeListener (this);
		}
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 **/
	public void activate() {
		if (!isActive ()) {
			super.activate ();
			((AnnotationElement )getModel ()).addPropertyChangeListener(this);
	
		}
	}
	
	
    
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#refresh()
	 **/
	public void refresh() {
		super.refresh ();
		}
	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 **/
	protected void refreshVisuals() {
		LineAnnotation line = (LineAnnotation )getModel ();
		
	
		
		
		refreshBendpoints (line);
		PolylineConnection figure = (PolylineConnection )getConnectionFigure();
//		figure.setStart(line.getStart ());
//		figure.setEnd (line.getEnd ());
//		if (getFigure ().getParent () != null)
//			((GraphicalEditPart )getParent ()).setLayoutConstraint(this, figure, figure.getBounds ());
		figure.setForegroundColor(AnnotationConstants.getColor (line.getLineColorRGB ()));
		figure.setLineStyle(line.getLineStyle().intValue());
		figure.setLineWidth(line.getLineWidth().intValue());
	}
	
	
	/**
	 * @param line
	 * @return
	 */
	private void refreshBendpoints(LineAnnotation line) {
		List modelConstraint = line.getBendpoints();
		List figureConstraint = new ArrayList ();
		
		for (int i = 0; i < modelConstraint.size (); i++) {
			LineBendpoint lbp = (LineBendpoint )modelConstraint.get (i);
			AbsoluteBendpoint abp = new AbsoluteBendpoint (lbp.getLocation());
			figureConstraint.add(abp);
		}
		getConnectionFigure ().setRoutingConstraint(figureConstraint);
		((PolylineConnection )getConnectionFigure ()).layout (); //getConnectionRouter().route(getConnectionFigure());
//		getConnectionFigure ().setSourceAnchor(new XYAnchor (line.getStart()));
//		getConnectionFigure ().setTargetAnchor (new XYAnchor (line.getEnd ()));
	}

	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.EditPart#understandsRequest(org.eclipse.gef.Request)
	 **/
	public boolean understandsRequest(Request request) {
		return (request instanceof LinePointRequest || super.understandsRequest(request) );
	}
	
	
	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.EditPart#getCommand(org.eclipse.gef.Request)
	 **/
	public Command getCommand(Request request) {
		if (request instanceof LinePointRequest) {
			LinePointRequest lpr = (LinePointRequest )request;
			Point location = lpr.getLocation ().getCopy ();
			getFigure().translateToRelative(location);
			ChangeLineCommand command = new ChangeLineCommand ((LineAnnotation )getModel (), lpr.getIndex(), location);
			return command;
		}
		return super.getCommand(request);
	}
	
	
	/**
	 * @return
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#getSource()
	 **/
	public EditPart getSource() {
		return this;
	}
	
	
	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.EditPart#getTargetEditPart(org.eclipse.gef.Request)
	 **/
	public EditPart getTargetEditPart(Request request) {
		return this;
	}
	
	
	/**
	 * @return
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#getSourceConnectionAnchor()
	 **/
	protected ConnectionAnchor getSourceConnectionAnchor() {
//		return null;
		
		LineAnnotation la = (LineAnnotation )getModel ();
		
		Point start = la.getStart().getCopy ();
//		getFigure ().translateToParent(start);
//		return new XYAnchor (start);
//		return new RelativeXYAnchor (this.getFigure (), start);
		return new XYAnchor (start);
	}

	/**
	 * @return
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#getTargetConnectionAnchor()
	 **/
	protected ConnectionAnchor getTargetConnectionAnchor() {
//		return null;
		LineAnnotation la = (LineAnnotation )getModel ();
		Point end = la.getEnd ().getCopy ();
//		if (getFigure ().getParent () != null) 
//		getFigure ().getParent ().translateToRelative (end);
//		getFigure ().translateToRelative(end);
//		return new RelativeXYAnchor (this.getFigure (), end);
		return new XYAnchor (end);
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#activateFigure()
	 **/
	protected void activateFigure() {
//		((PolylineConnection )getConnectionFigure ()).layout ();
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#deactivateFigure()
	 **/
	protected void deactivateFigure() {
//		getConnectionFigure().setSourceAnchor(null);
//		getConnectionFigure ().setTargetAnchor(null);
	}
	
	
	class RelativeBendpoint implements Bendpoint {
		private Connection connection;
		private float weight = 0.5f;
		private Dimension d1, d2;

		/**
		 * Constructs a new RelativeBendpoint.
		 * 
		 * @since 2.0
		 */
		public RelativeBendpoint() { }

		/**
		 * Constructs a new RelativeBendpoint and associates it with the given Connection.
		 * @param conn The Connection this Bendpoint is associated with
		 * @since 2.0
		 */
		public RelativeBendpoint(Connection conn) {
			setConnection(conn);
		}

		/**
		 * Returns the Connection this Bendpoint is associated with.
		 * @return The Connection this Bendpoint is associated with
		 * @since 2.0
		 */
		protected Connection getConnection() {
			return connection;
		}

		/**
		 * Calculates and returns this Bendpoint's new location.
		 * @return This Bendpoint's new location
		 * @since 2.0
		 */
		public Point getLocation() {
			LineAnnotation lineAnnotation = (LineAnnotation )getSource ().getModel ();
			Point a1 = lineAnnotation.getStart ().getCopy ();
			Point a2 = lineAnnotation.getEnd ().getCopy ();
			
			getConnection ().getParent ().translateToAbsolute(a1);
			getConnection ().getParent ().translateToAbsolute(a2);
			
			Point p = new Point();
			Dimension dim1 = d1.getCopy(), dim2 = d2.getCopy();
			
			getConnection().getParent ().translateToAbsolute(dim1);
			getConnection().getParent ().translateToAbsolute(dim2);
			
			p.x = (int)((a1.x + dim1.width) * (1f - weight) + weight * (a2.x + dim2.width));
			p.y = (int)((a1.y + dim1.height) * (1f - weight) + weight * (a2.y + dim2.height));
			getConnection().getParent ().translateToRelative(p);
			return p;
		}

		/**
		 * Sets the Connection this bendpoint should be associated with.
		 * @param conn The Connection this bendpoint should be associated with
		 * @since 2.0
		 */
		public void setConnection(Connection conn) {
			connection = conn;
		}

		/**
		 * Sets the Dimensions representing the X and Y distances this Bendpoint is from the start
		 * and end points of the Connection. These Dimensions are generally set once and are used
		 * in  calculating the Bendpoint's location.
		 * @param dim1 The X and Y distances this Bendpoint is from the start of the Connection
		 * @param dim2 The X and Y distances this Bendpoint is from the end of the Connection
		 * @since 2.0
		 */
		public void setRelativeDimensions(Dimension dim1, Dimension dim2) {
			d1 = dim1;
			d2 = dim2;
		}

		/**
		 * Sets the weight this Bendpoint should use to calculate its location. The weight should
		 * be between 0.0 and 1.0. A weight of 0.0 will cause the Bendpoint to follow the start
		 * point, while a weight of 1.0 will cause the Bendpoint to follow the end point. A weight
		 * of 0.5 (the default) will cause the Bendpoint to maintain its original aspect ratio
		 * between the start and end points.
		 * @param w The weight
		 * @since 2.0
		 */
		public void setWeight(float w) {
			weight = w;
		}

	}
}
