/*
 * Created on Oct 12, 2004
 *
 */
package AcmeStudioShapeExtension.parts;

import java.beans.PropertyChangeEvent;
import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import AcmeStudioShapeExtension.AnnotationConstants;
import AcmeStudioShapeExtension.model.*;
import AcmeStudioShapeExtension.policies.ShapeComponentEditPolicy;
import AcmeStudioShapeExtension.policies.ShapeSelectionEditPolicy;


/**
 * @author Bradley Schmerl
 * Created Oct 12, 2004
 */
public class ShapeEditPart extends AnnotationEditPart {

	/**
	 * @return
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 **/
	protected IFigure createFigure() {
		Shape f = createFigureForModel ();
		f.setOpaque (true);
		return f;
	}

	/**
	 * @return
	 */
	private Shape  createFigureForModel() {
		if (getModel () instanceof EllipseAnnotation) {
			return new Ellipse ();
		}
		else if (getModel () instanceof RectangleAnnotation) {
			return new RectangleFigure ();
		}
		else if (getModel () instanceof RoundedRectangleAnnotation) {
			return new RoundedRectangle ();
		}
		else return null;
	}

	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 **/
	protected void createEditPolicies() {
		installEditPolicy (EditPolicy.COMPONENT_ROLE, new ShapeComponentEditPolicy ());
//		removeEditPolicy (EditPolicy.PRIMARY_DRAG_ROLE);
		installEditPolicy (EditPolicy.PRIMARY_DRAG_ROLE, new ShapeSelectionEditPolicy ());
//		installEditPolicy (EditPolicy.SELECTION_FEEDBACK_ROLE, new ShapeSelectionEditPolicy ());
//		installEditPolicy (EditPolicy.GRAPHICAL_NODE_ROLE, new GraphicalNodeEditPolicy () {
//			
//		});
	}
	
	
	/**
	 * @param role
	 * @param editPolicy
	 * @see org.eclipse.gef.EditPart#installEditPolicy(java.lang.Object, org.eclipse.gef.EditPolicy)
	 **/
	public void installEditPolicy(Object role, EditPolicy editPolicy) {
		if (role.equals (EditPolicy.PRIMARY_DRAG_ROLE) && !(editPolicy instanceof ShapeSelectionEditPolicy)) return;
		super.installEditPolicy (role, editPolicy);
	}

	/**
	 * @param evt
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 **/
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
//		if (Annotation.SIZE_PROP.equals (prop) || Annotation.LOCATION_PROP.equals (prop)) {
			refreshVisuals ();
//		}
	}

	/**
	 * @param connection
	 * @return
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 **/
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return new ChopboxAnchor (getFigure ());
	}

	/**
	 * @param connection
	 * @return
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 **/
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return new ChopboxAnchor (getFigure ());
	}

	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 **/
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return new ChopboxAnchor (getFigure ());
	}

	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 **/
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return new ChopboxAnchor (getFigure ());
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.EditPart#deactivate()
	 **/
	public void deactivate() {
		if (isActive()) {
			super.deactivate ();
			((AnnotationElement )getModel ()).removePropertyChangeListener(this);
		}
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.EditPart#activate()
	 **/
	public void activate() {
		if (!isActive ()) {
			super.activate ();
			((AnnotationElement )getModel ()).addPropertyChangeListener(this);
		}
	}
	
	
	/**
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 **/
	protected void refreshVisuals() {
		Annotation anno = (Annotation )getModel ();
		Rectangle bounds = new Rectangle (anno.getLocation (), anno.getSize());
		figure.setBounds(bounds);
		((GraphicalEditPart )getParent ()).setLayoutConstraint (this, figure, bounds);
		figure.setForegroundColor(AnnotationConstants.getColor (anno.getForegroundColor()));
		figure.setBackgroundColor(AnnotationConstants.getColor(anno.getBackgroundColor ()));
		((Shape )figure).setFill (anno.getUseFill().booleanValue());
		((Shape )figure).setLineStyle(anno.getLineStyle().intValue());
		((Shape )figure).setLineWidth(anno.getLineWidth().intValue ());
	}
}
