/*
 * Created on Oct 12, 2004
 *
 */
package AcmeStudioShapeExtension.policies;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.PrecisionRectangle;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import AcmeStudio.core.ASContext;
import AcmeStudio.ui.editors.DiagramEditor;
import AcmeStudioShapeExtension.model.Annotation;
import AcmeStudioShapeExtension.model.AnnotationElement;
import AcmeStudioShapeExtension.model.LineAnnotation;
import AcmeStudioShapeExtension.model.command.AnnotationCreateCommand;
import AcmeStudioShapeExtension.model.command.ChangeLineCommand;
import AcmeStudioShapeExtension.model.command.LineAnnotationCreateCommand;
import AcmeStudioShapeExtension.model.command.SetAnnotationConstraintCommand;
import acmeLib.elements.AcmeSystem;


/**
 * @author Bradley Schmerl
 * Created Oct 12, 2004
 */
public class LayoutPolicyExtension extends XYLayoutEditPolicy {

	/**
	 * @param child
	 * @param constraint
	 * @return
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 **/
	protected Command createAddCommand(EditPart child, Object constraint) {
		return null;
	}

	/**
	 * @param child
	 * @param constraint
	 * @return
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 **/
	protected Command createChangeConstraintCommand(EditPart child, Object constraint) {
		if (child.getModel () instanceof Annotation) {
			SetAnnotationConstraintCommand locationCommand = new SetAnnotationConstraintCommand ((GraphicalEditPart )child);
			locationCommand.setElement ((Annotation )child.getModel ());
			locationCommand.setLocation ((Rectangle )constraint);
			return locationCommand;
		}
		return null;
		
	}
	
	
	/**
	 * @param request
	 * @param child
	 * @param constraint
	 * @return
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.requests.ChangeBoundsRequest, org.eclipse.gef.EditPart, java.lang.Object)
	 **/
	protected Command createChangeConstraintCommand(ChangeBoundsRequest request, EditPart child, Object constraint) {
		GraphicalEditPart part = (GraphicalEditPart )child;
		if (part.getModel() instanceof Annotation) {
			SetAnnotationConstraintCommand cmd = (SetAnnotationConstraintCommand )createChangeConstraintCommand (child, constraint);
			return cmd;
		}
		return null;
	}

	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 **/
	protected Command getCreateCommand(CreateRequest request) {
		AcmeSystem acmeSystem = ((DiagramEditor )ASContext.current().getActiveEditor().getActiveEditor()).getAcmeSystem();
		if (request.getNewObject () instanceof LineAnnotation) {
			return new LineAnnotationCreateCommand (acmeSystem,request);
		}
		if (request.getNewObject() instanceof AnnotationElement) {
			return new AnnotationCreateCommand (acmeSystem, request);
		}
		return null;
	}

	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 **/
	protected Command getDeleteDependantCommand(Request request) {
		return null;
	}
	
	
	/**
	 * @param request
	 * @return
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getCommand(org.eclipse.gef.Request)
	 **/
	public Command getCommand(Request request) {
		
		if (REQ_RECONNECT_SOURCE.equals (request.getType ())) {
			ReconnectRequest rreq = (ReconnectRequest )request;
			if (!(rreq.getConnectionEditPart ().getModel()  instanceof LineAnnotation)) return null;
			ChangeLineCommand command = new ChangeLineCommand ((LineAnnotation )rreq.getConnectionEditPart().getModel (), ChangeLineCommand.START, rreq.getLocation());
			return command;
//			return null;
//			ChangeLineCommand command = new ChangeLineCommand ()
		}
		else if (REQ_RECONNECT_TARGET.equals (request.getType ())) {
			ReconnectRequest rreq = (ReconnectRequest )request;
			if (!(rreq.getConnectionEditPart().getModel () instanceof LineAnnotation)) return null;
			ChangeLineCommand command = new ChangeLineCommand ((LineAnnotation )rreq.getConnectionEditPart().getModel (), ChangeLineCommand.END, rreq.getLocation());
			return command;
		}
		else return super.getCommand (request);
	}
	
	
	/**
	 * @param request
	 * @param child
	 * @return
	 * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy#getConstraintFor(org.eclipse.gef.requests.ChangeBoundsRequest, org.eclipse.gef.GraphicalEditPart)
	 **/
	protected Object getConstraintFor(ChangeBoundsRequest request, GraphicalEditPart child) {
		Rectangle rect = new PrecisionRectangle (child.getFigure ().getBounds ());
		child.getFigure ().translateToAbsolute (rect);
		Rectangle original = rect.getCopy ();
		rect = request.getTransformedRectangle(rect);
		child.getFigure ().translateToRelative (rect);
		rect.translate (getLayoutOrigin ().getNegated ());
		if (request.getSizeDelta().width == 0 && request.getSizeDelta ().height == 0) {
			Rectangle cons = getCurrentConstraintFor(child);
			if (cons != null) rect.setSize (cons.width, cons.height);
		}
		else {
			Dimension minSize = getMinimumSizeFor(child);
			if (rect.width < minSize.width) {
				rect.width = minSize.width;
				if (rect.x > (original.right() - minSize.width))
					rect.x = original.right() - minSize.width;
			}
			if (rect.height < minSize.height) {
				rect.height = minSize.height;
				if (rect.y > (original.bottom() - minSize.height))
					rect.y = original.bottom() - minSize.height;
			}
		}
		return getConstraintFor(rect);
	}
}
