/*
 * Created on Jul 8, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package AcmeTranslator.wizards;


import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.dialogs.WizardExportResourcesPage;

import acme.translators.AcmeTranslator;

/**
 * @author Bradley Schmerl
 * 
 * This page implements a page for the export of a file to xAcme or ADML. Currently, you must only 
 * select an Acme resource (*.acme file).
 */
public class XMLExportFileSystemResourcePage1
	extends WizardExportResourcesPage
	implements Listener {

	// Resource strings that are used to remember options chosen
	private static final String STORE_EXPORT_OPTIONS = Messages.getString("XMLExportFileSystemResourcePage1.XMLExportResource.exportFamilyOptions_id"); //$NON-NLS-1$
	private static final String STORE_FORMAT_XACME_ID = Messages.getString("XMLExportFileSystemResourcePage1.XMLExportResource.formatXAcme_id"); //$NON-NLS-1$
	private static final String STORE_CREATE_STRUCTURES_ID  = Messages.getString("XMLExportFileSystemResourcePage1.XMLExportResource.createStructures_id"); //$NON-NLS-1$
	private static final String STORE_INLINE_FAMILIES_ID = Messages.getString("XMLExportFileSystemResourcePage1.XMLExportResource.inlineFamilies_id"); //$NON-NLS-1$
	private static final String STORE_OVERWRITE_EXISTING_FILES_ID = Messages.getString("XMLExportFileSystemResourcePage1.XMLExportResource.overrideExisitingFiles_id"); //$NON-NLS-1$
	private static final String STORE_DESTINATION_NAMES_ID = Messages.getString("XMLExportFileSystemResourcePage1.XMLExportResource.destinationNames_id"); //$NON-NLS-1$


	// Widgets to implement the dialog
	private Button _btnExportFamilies;
	private Button _btnLeaveReference;
	private Button _btnFormatADML;
	private Button _btnFormatXAcme;
	private Button _btnOverwriteExistingFiles;
	private Button _btnInlineFamilies;
	private Button _btnCreateSelectionOnly;
	private Button _btnCreateDirectoryStructure;
	private Button _btnDestinationBrowse;
	private Combo _cmbDestinationName;

	/**
	 * Constructs and instance of this page
	 * @param selection The initially selected resource
	 */
	public XMLExportFileSystemResourcePage1 (IStructuredSelection selection) {
		super (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.widgetTitle"), selection); //$NON-NLS-1$
		setTitle (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.pageTitle")); //$NON-NLS-1$
		setDescription (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.pageDescription")); //$NON-NLS-1$
	}

	/**
	 * Creates the widgets for setting the destination
	 */
	protected void createDestinationGroup(Composite parent) {
		Font font = parent.getFont ();
		Composite destinationSelectionGroup = new Composite (parent, SWT.NONE);
		GridLayout layout = new GridLayout (3, false);
		destinationSelectionGroup.setLayout(layout);
		destinationSelectionGroup.setLayoutData (new GridData (GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL));
		destinationSelectionGroup.setFont (font);
		
		Label destinationLabel = new Label (destinationSelectionGroup, SWT.NONE);
		destinationLabel.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.directoryOptionsTitle")); //$NON-NLS-1$
		destinationLabel.setFont (font);
		
		_cmbDestinationName = new Combo (destinationSelectionGroup, SWT.SINGLE | SWT.BORDER);
		_cmbDestinationName.addListener (SWT.Modify, this);
		_cmbDestinationName.addListener (SWT.Selection, this);
		GridData data = new GridData (GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL);
		data.widthHint = 250;
		_cmbDestinationName.setLayoutData (data);
		_cmbDestinationName.setFont (font);
		
		_btnDestinationBrowse = new Button (destinationSelectionGroup, SWT.PUSH);
		_btnDestinationBrowse.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.browseDestinationButton")); //$NON-NLS-1$
		_btnDestinationBrowse.addListener (SWT.Selection, this);
		_btnDestinationBrowse.setFont (font);
		setButtonLayoutData(_btnDestinationBrowse);
		
		getTypesToExport ().add("*.acme"); //$NON-NLS-1$
		new Label (parent, SWT.NONE);

	}

	/**
	 * Handles an event on one of the widgets, to either handle finding a destination directory
	 * or updating the status of the dialog.
	 * @param e the event to be handled
	 */
	public void handleEvent(Event e) {
		Widget source = e.widget;
		
		if (source == _btnDestinationBrowse) {
			handleDestinationBrowseButtonPressed ();
		}
		updatePageCompletion ();
	}

	/**
	 * Implements browsing to a directoy to set the destination to export
	 */
	private void handleDestinationBrowseButtonPressed() {
		DirectoryDialog dialog = new DirectoryDialog (getContainer ().getShell (), SWT.SAVE);
		dialog.setMessage (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.selectDestinationMessage")); //$NON-NLS-1$
		dialog.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.exportDestinationDirectory")); //$NON-NLS-1$
		dialog.setFilterPath (getDestinationValue ());
		String selectedDirectoryName = dialog.open ();
		if (selectedDirectoryName != null) {
			setErrorMessage (null);
			setDestinationValue (selectedDirectoryName);
		}
		
	}

	/**
	 * 
	 * @return The destination directory
	 */
	private String getDestinationValue() {
		return _cmbDestinationName.getText().trim ();
	}

	/**
	 * Sets the destination directoy
	 * @param selectedDirectoryName The name to set the directory to
	 */
	private void setDestinationValue(String selectedDirectoryName) {
		_cmbDestinationName.setText(selectedDirectoryName);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		super.createControl(parent);
		giveFocusToDestination ();
	}

	/**
	 * Makes sure that the focus is on the destination widget
	 */
	private void giveFocusToDestination() {
		_cmbDestinationName.setFocus();
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.WizardDataTransferPage#createOptionsGroupButtons(org.eclipse.swt.widgets.Group)
	 */
	protected void createOptionsGroupButtons(Group optionsGroup) {
		Font font = optionsGroup.getFont ();
		createOverwriteExisting (optionsGroup, font);
		createDirectoryStructureOptions (optionsGroup, font);
		createFamilyInclusionOptions (optionsGroup, font);
		createXMLFormatOptions (optionsGroup, font);
	}

	/**
	 * Sets up widgets for allowing the user to choose the XML format to export to (either xAcme or ADML)
	 * TODO: Enable ADML exporting
	 * @param optionsGroup Where to put the options
	 * @param font What to set the font to
	 */
	private void createXMLFormatOptions(Group optionsGroup, Font font) {
		Group formatGroup = new Group (optionsGroup, SWT.NONE);
		formatGroup.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.formatOptionTitle")); //$NON-NLS-1$
		formatGroup.setLayout(new FillLayout (SWT.HORIZONTAL));
		_btnFormatXAcme = new Button (formatGroup, SWT.RADIO);
		_btnFormatXAcme.setText ("xAcme"); //$NON-NLS-1$
		_btnFormatXAcme.setFont (font);
		_btnFormatXAcme.setSelection(true);
		
		_btnFormatADML = new Button (formatGroup, SWT.RADIO);
		_btnFormatADML.setText ("ADML"); //$NON-NLS-1$
		_btnFormatADML.setFont (font);
		_btnFormatADML.setSelection (false);

		// Temporarily disable the ADML exporting
//		_btnFormatADML.setEnabled(false);
		
		
	}

	/**
	 * Allows the user to choose what to do with families that are imported
	 * TODO: Enable the export to same directory option - exporting works ok, but importing doesn't
	 * @param optionsGroup Where to put the options
	 * @param font The font to use
	 */
	private void createFamilyInclusionOptions(Group optionsGroup, Font font) {
		Group familyGroup = new Group (optionsGroup, SWT.NONE);
		familyGroup.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.importFamilyOptionTitle")); //$NON-NLS-1$
		familyGroup.setLayout (new FillLayout (SWT.VERTICAL));
		_btnLeaveReference = new Button (familyGroup, SWT.RADIO | SWT.LEFT);
		_btnLeaveReference.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.referenceImportedFamily")); //$NON-NLS-1$
		_btnLeaveReference.setSelection (true);
		_btnLeaveReference.setFont(font);
		_btnInlineFamilies = new Button (familyGroup, SWT.RADIO | SWT.LEFT);
		_btnInlineFamilies.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.inlineImportedFamily")); //$NON-NLS-1$
		_btnInlineFamilies.setSelection(false);
		_btnInlineFamilies.setFont (font);
		_btnExportFamilies = new Button (familyGroup, SWT.RADIO | SWT.LEFT);
		_btnExportFamilies.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.exportImportedFamily")); //$NON-NLS-1$
		_btnExportFamilies.setSelection (false);
		_btnExportFamilies.setFont(font);
		
		// Temporarily disable this option
//		_btnExportFamilies.setEnabled(false);
		
	}

	private void createDirectoryStructureOptions(Group optionsGroup, Font font) {
		_btnCreateDirectoryStructure = new Button (optionsGroup, SWT.RADIO | SWT.LEFT);
		_btnCreateDirectoryStructure.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.directoryStructure")); //$NON-NLS-1$
		_btnCreateDirectoryStructure.setSelection (false);
		_btnCreateDirectoryStructure.setFont (font);
		
		_btnCreateSelectionOnly = new Button (optionsGroup, SWT.RADIO | SWT.LEFT);
		_btnCreateSelectionOnly.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.createDirectory")); //$NON-NLS-1$
		_btnCreateSelectionOnly.setSelection (true);
		_btnCreateSelectionOnly.setFont (font);
	}


	private void createOverwriteExisting(Group optionsGroup, Font font) {
		_btnOverwriteExistingFiles = new Button (optionsGroup, SWT.CHECK | SWT.LEFT);
		_btnOverwriteExistingFiles.setText (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.overwriteAll")); //$NON-NLS-1$
		_btnOverwriteExistingFiles.setFont (font);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.WizardExportResourcesPage#internalSaveWidgetValues()
	 */
	protected void internalSaveWidgetValues() {
		IDialogSettings settings = getDialogSettings();
		if (settings != null) {
			String [] directoryNames = settings.getArray (STORE_DESTINATION_NAMES_ID);
			if (directoryNames == null) {
				directoryNames = new String [0];
			}
			
			directoryNames = addToHistory(directoryNames, getDestinationValue());
			settings.put (STORE_DESTINATION_NAMES_ID, directoryNames);
			
			settings.put (STORE_OVERWRITE_EXISTING_FILES_ID, _btnOverwriteExistingFiles.getSelection ());
			settings.put (STORE_CREATE_STRUCTURES_ID, _btnOverwriteExistingFiles.getSelection ());
			settings.put (STORE_INLINE_FAMILIES_ID, _btnInlineFamilies.getSelection ());
			settings.put (STORE_FORMAT_XACME_ID, _btnFormatXAcme.getSelection ());
			
			settings.put (STORE_EXPORT_OPTIONS, getImporFamilyAction());
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.WizardDataTransferPage#restoreWidgetValues()
	 */
	protected void restoreWidgetValues() {
		IDialogSettings settings = getDialogSettings ();
		if (settings != null) {
			String [] directoryNames = settings.getArray (STORE_DESTINATION_NAMES_ID);
			if (directoryNames == null) return;
			
			setDestinationValue (directoryNames [0]);
			for (int i = 0; i < directoryNames.length; i++) {
				addDestinationItem (directoryNames [i]);
			}
			
			_btnOverwriteExistingFiles.setSelection (settings.getBoolean (STORE_OVERWRITE_EXISTING_FILES_ID));
			boolean createDirectories  = settings.getBoolean (STORE_CREATE_STRUCTURES_ID);
			_btnCreateDirectoryStructure.setSelection (createDirectories);
			_btnCreateSelectionOnly.setSelection (!createDirectories);
			_btnInlineFamilies.setSelection (settings.getBoolean (STORE_INLINE_FAMILIES_ID));
			boolean formatXAcme = settings.getBoolean (STORE_FORMAT_XACME_ID);
			_btnFormatADML.setSelection (!formatXAcme);
			_btnFormatXAcme.setSelection (formatXAcme);
			
			int importAction = settings.getInt(STORE_EXPORT_OPTIONS);
			switch (importAction) {
				case AcmeTranslator.REFERENCE_IMPORTED_FAMILIES: 
					_btnLeaveReference.setSelection (true);
					break;
				case AcmeTranslator.INLINE_IMPORTED_FAMILIES:
					_btnInlineFamilies.setSelection (true);
					break;
				case AcmeTranslator.EXPORT_IMPORTED_FAMILIES:
					_btnExportFamilies.setSelection (true);
					break;
			}
		}
	}

	/**
	 * Adds a destination to the destination combo
	 * @param string
	 */
	private void addDestinationItem(String string) {
		_cmbDestinationName.add (string);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.WizardDataTransferPage#validateDestinationGroup()
	 */
	protected boolean validateDestinationGroup() {
		String destinationValue = getDestinationValue();
		if (destinationValue.length () == 0) {
			setMessage (destinationEmptyMessage ());
			return false;
		}
		
		String conflictingContainer = getConflictingContainerNameFor (destinationValue);
		if (conflictingContainer == null) {
			setErrorMessage (""); //$NON-NLS-1$
		}
		else {
			setErrorMessage (Messages.getString("XMLExportFileSystemResourcePage1.exportPage.destinationConflict") + conflictingContainer); //$NON-NLS-1$
			giveFocusToDestination ();
			return false;
			
		}
		return true;
	}

	/**
	 * @return
	 */
	private String destinationEmptyMessage() {
		return Messages.getString("XMLExportFileSystemResourcePage1.exportPage.emptyDirectory"); //$NON-NLS-1$
	}

	/**
	 * Returns a message if the container chosen as the destination conflicts with internals
	 * @param destinationValue The proposed value
	 * @return A message on conflict
	 */
	private String getConflictingContainerNameFor(String targetDirectory) {
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot ();
		IPath testPath = new Path (targetDirectory);
		
		if (root.getLocation ().isPrefixOf (testPath)) {
			return Messages.getString("XMLExportFileSystemResourcePage1.exportPage.workspaceRoot"); //$NON-NLS-1$
		}
		
		IProject [] projects = root.getProjects ();
		
		for (int i = 0; i < projects.length; i++) {
			if (projects [i].getLocation ().isPrefixOf (testPath)) return projects [i].getName ();
		}
		
		return null;
	}

	/**
	 * Finishes the operation. First, checks to make sure that all is valid, and then
	 * executes an exportOperation
	 * @return whether operation was successful
	 */
	public boolean finish() {
		if (!ensureTargetIsValid (new File (getDestinationValue ()))) return false;
		
		List resourcesToExport = getWhiteCheckedResources();
		
		saveDirtyEditors();
		saveWidgetValues();
		
		if (resourcesToExport.size () > 0) {
			return executeExportOperation (new XMLExportOperation (null, resourcesToExport, getDestinationValue (), this));
		}
		
		MessageDialog.openInformation (getContainer ().getShell (), Messages.getString("XMLExportFileSystemResourcePage1.exportPage.information"), Messages.getString("XMLExportFileSystemResourcePage1.exportPage.noSelectedResources")); //$NON-NLS-1$ //$NON-NLS-2$
		return false;
		
	}

	/**
	 * Executes the export operation
	 * @param operation
	 * @return
	 */
	private boolean executeExportOperation(XMLExportOperation op) {
		op.setCreateLeadupStructure(_btnCreateDirectoryStructure.getSelection ());
		op.setOverwriteFiles(_btnOverwriteExistingFiles.getSelection ());
		op.setExportXAcme (_btnFormatXAcme.getSelection ());
		op.setImportFamilyAction (getImporFamilyAction ());
		
		try {
			getContainer ().run (true, true, op);
		}
		catch (InterruptedException e) {
			return false;
		}
		catch (InvocationTargetException e) {
			displayErrorDialog(e.getTargetException());
			return false;
		}
		
		IStatus status = op.getStatus ();
		if (!status.isOK ()) {
			ErrorDialog.openError(getContainer ().getShell (), Messages.getString("XMLExportFileSystemResourcePage1.exportPage.problem"), null, status); //$NON-NLS-1$
			return false;
		}
		return true;
	}

	/**
	 * @return the action to be done on imported families
	 */
	private int getImporFamilyAction() {
		if (_btnLeaveReference.getSelection()) {
			return AcmeTranslator.REFERENCE_IMPORTED_FAMILIES;
				
		}
		else if (_btnInlineFamilies.getSelection ()) {
			return AcmeTranslator.INLINE_IMPORTED_FAMILIES;
		}
		else if (_btnExportFamilies.getSelection ()) {
			return AcmeTranslator.EXPORT_IMPORTED_FAMILIES;
		}		
		return AcmeTranslator.REFERENCE_IMPORTED_FAMILIES;
	}

	/**
	 * @param file
	 * @return
	 */
	private boolean ensureTargetIsValid(File file) {
		if (file.exists () && !file.isDirectory()) {
			displayErrorDialog(Messages.getString("XMLExportFileSystemResourcePage1.exportPage.existentTargetDirectory")); //$NON-NLS-1$
			giveFocusToDestination();
			return false;
		}
		
		return ensureDirectoryExists (file);
	}

	/**
	 * @param file
	 * @return
	 */
	private boolean ensureDirectoryExists(File file) {
		if (!file.exists ()) {
			if (!queryYesNoQuestion(Messages.getString("XMLExportFileSystemResourcePage1.exportPage.createDirectoryQuery"))) { //$NON-NLS-1$
				return false;
			}
			
			if (!file.mkdirs()) {
				displayErrorDialog(Messages.getString("XMLExportFileSystemResourcePage1.exportPage.createDirectoryProblem")); //$NON-NLS-1$
				return false;
			}
		}
		return true;
	}

}
