/*
 * Created on Jul 7, 2003
 */
package AcmeTranslator.wizards;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.OpenStrategy;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.*;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.part.FileEditorInput;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import AcmeStudio.AcmeStudioPlugin;
import AcmeStudio.core.resource.ASDesign;
import AcmeStudio.ui.editors.MultiPageAcmeEditor;
import acme.translators.ADMLImporter;
import acme.translators.XAcmeImporter;
import acme.translators.XMLImporter;
import acmeLib.core.exceptions.AcmeException;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.elements.AcmeDesign;

/**
 * @author Bradley Schmerl Implements the wizard for importing from ADML or
 *         xAcme
 */
public class XMLImportWizard extends Wizard implements IImportWizard {

	private XMLImportWizardPage _mainPage;

	IStructuredSelection selection;

	private DocumentBuilder	_documentBuilder;

	/* (non-Javadoc) @see org.eclipse.jface.wizard.Wizard#performFinish() */
	public boolean performFinish() {
		final String containerName = ((XMLImportWizardPage) getPages()[0]).getContainerName();
		final String filename = ((XMLImportWizardPage) getPages()[0]).getFileName();
		final String acmeFile = ((XMLImportWizardPage) getPages()[0]).getAcmeFilename();
		WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
			public void execute(final IProgressMonitor monitor) throws CoreException {
				try {
					performFinish(containerName, filename, acmeFile, monitor);
				}
				finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		}
		catch (InterruptedException e) {
			return false;
		}
		catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
		return true;
	}

	/**
	 * Does the importing
	 * 
	 * @param containerName
	 * @param filename
	 * @param monitor
	 */
	protected void performFinish(String containerName, String filename, String acmeFile, IProgressMonitor monitor)
		throws CoreException {
		monitor.beginTask("Importing " + filename, 3);
		IPath contPath = new Path(containerName);
		IContainer cont = AcmeStudioPlugin.getRoot().getProject(contPath.segment(0));
		if (cont == null || !cont.exists()) {
			throwCoreException("The project \"" + contPath.segment(0) + "\" does not exist.");

		}

		try {
			AcmeDesign design = null;
			if (_mainPage.isXAcme(_mainPage.getXmlDocument().getDocumentElement())) {

				Document xAcmeDoc = _mainPage.getXmlDocument();
				Element typedDesign = xAcmeDoc.getDocumentElement();

				XAcmeImporter importer = new XAcmeImporter(xAcmeDoc);

				if (_mainPage.mustImportFamilies()) {
					String path = getImportPath(filename);
					processImportedFamilies(importer, typedDesign, path, cont, monitor);
				}
				//				importer.setFamiliesImported
				// (_mainPage.mustImportFamilies());
				design = importer.importDesign();
			}
			else if (_mainPage.isADML(_mainPage.getXmlDocument().getDocumentElement())) {
				Document admlDoc = _mainPage.getXmlDocument();
				Element designXML = admlDoc.getDocumentElement();
				ADMLImporter importer = new ADMLImporter(admlDoc);
				if (_mainPage.mustImportFamilies()) {
					String path = getImportPath(filename);
					processImportedFamilies(importer, designXML, path, cont, monitor);
				}
				design = importer.importDesign();
			}

			else {
				throwCoreException("XML Document is not valid xAcme");
			}
			design.doPostParsingProcessing();
			monitor.worked(1);

			// Create the acme file
			monitor.setTaskName("Creating " + acmeFile + " Acme file...");
			String acmeFilename = convertToAcmeFileName(acmeFile);

			AcmeOutputStream aos = new AcmeOutputStream();
			design.exportArmani(aos);

			aos.close();
			InputStream stream = new ByteArrayInputStream(aos.toString().getBytes());
			final IFile af = cont.getFile(new Path(acmeFilename));
			if (af.exists()) {
				af.setContents(stream, true, true, monitor);

			}
			else {
				af.create(stream, true, monitor);
			}
			stream.close();
			monitor.worked(2);
			monitor.setTaskName("Opening file for editing...");
			getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
					try {
						page.openEditor(new FileEditorInput (af), "AcmeStudio.ui.editors.MultiPageAcmeEditor", OpenStrategy.activateOnOpen());
						IEditorPart editor = page.getActiveEditor();
						if (editor instanceof MultiPageAcmeEditor) {
							MultiPageAcmeEditor acmeEditor = (MultiPageAcmeEditor) editor;
							acmeEditor.setActiveAcmePage(MultiPageAcmeEditor.SYSTEM_PAGE);
							//							  acmeEditor.setActiveAcmePage(acmeEditor.FAMILY_PAGE);
						}
					}
					catch (PartInitException e) {
					}

				}
			});
			monitor.worked(3);

		}
		catch (FileNotFoundException e) {
			throwCoreException("The file \"" + filename + "\" does not exist.");
		}
		catch (IOException e) {
			throwCoreException("Error opening \"" + filename + "\".");
		}
		catch (AcmeIllegalNameException e) {
			throwCoreException("Error translating \"" + filename + "\" to Acme: " + e.getMessage());
		}
		catch (AcmeException e) {
			throwCoreException("Error creating Acme file: " + e.getMessage());
		}
		catch (SAXException e) {
			throwCoreException("Error creating Acme file: " + e.getMessage());
		}

	}

	/**
	 * @param filename
	 * @return
	 */
	private String getImportPath(String filename) {
		String FSEP = System.getProperty("file.separator");
		int index = filename.lastIndexOf(FSEP);
		return filename.substring(0, index);
	}

	/**
	 * @param importer
	 * @param typedDesign
	 */
	private void processImportedFamilies(
		XAcmeImporter importer,
		Element typedDesign,
		String path,
		IContainer cont,
		IProgressMonitor monitor)
		throws SAXException, IOException, AcmeException, CoreException {
		NodeList imports = XMLImporter.getElementsAtThisLevel(typedDesign, "importDesign");
		for (int i = 0; i < imports.getLength(); i++) {
			Element importEl = (Element) imports.item(i);
			String tA = importEl.getAttribute("type");
			if (tA != null && tA.equals("xml")) {
				String name = importEl.getAttribute("name");
				File file = new File(path + "/" + name);

				Document document = _documentBuilder.parse (file);
				XAcmeImporter imp = new XAcmeImporter(document);
				processImportedFamilies(imp, document.getDocumentElement(), path, cont, monitor);
				AcmeDesign des = imp.importDesign();
				importer.addImportedDesign(name, des);

				IFile familesDir = cont.getFile(new Path("families"));
				IFile familyFile = cont.getFile(new Path("families/" + convertToAcmeFileName(name)));

				AcmeOutputStream aos = new AcmeOutputStream();
				des.exportArmani(aos);
				aos.close();
				InputStream stream = new ByteArrayInputStream(aos.toString().getBytes());
				if (familyFile.exists()) {
					familyFile.setContents(stream, true, true, monitor);
				}
				else {
					familyFile.create(stream, true, monitor);
				}

			}
		}

	}

	/**
	 * @param importer
	 * @param typedDesign
	 */
	private void processImportedFamilies(
		ADMLImporter importer,
		Element typedDesign,
		String path,
		IContainer cont,
		IProgressMonitor monitor)
		throws SAXException, IOException, AcmeException, CoreException {
		NodeList imports = XMLImporter.getElementsAtThisLevel(typedDesign, "ImportPathname");
		for (int i = 0; i < imports.getLength(); i++) {
			Element importEl = (Element) imports.item(i);
			String name = importEl.getAttribute("pathname");
			if (name.endsWith("xml")) {
				File file = new File(path + "/" + name);
	
				Document document = _documentBuilder.parse (file);
				XAcmeImporter imp = new XAcmeImporter(document);
				processImportedFamilies(imp,document.getDocumentElement(), path, cont, monitor);
				AcmeDesign des = imp.importDesign();
				importer.addImportedDesign(name, des);

				IFile familesDir = cont.getFile(new Path("families"));
				IFile familyFile = cont.getFile(new Path("families/" + convertToAcmeFileName(name)));

				AcmeOutputStream aos = new AcmeOutputStream();
				des.exportArmani(aos);
				aos.close();
				InputStream stream = new ByteArrayInputStream(aos.toString().getBytes());
				if (familyFile.exists()) {
					familyFile.setContents(stream, true, true, monitor);
				}
				else {
					familyFile.create(stream, true, monitor);
				}

			}
		}

	}
	protected String convertToAcmeFileName(String name) {
		String result = name;
		boolean endsWithAcme = false;
		int dotLoc = result.indexOf('.');
		if (dotLoc != -1) {
			String ext = result.substring(dotLoc + 1);
			if (ext.equalsIgnoreCase(ASDesign.FILE_EXT)) {
				endsWithAcme = true;
			}
			else if (ext.equals("xml")) {
				result = name.substring(0, dotLoc);
			}
		}
		if (!endsWithAcme) {
			result += "." + ASDesign.FILE_EXT;
		}
		return result;
	}

	protected void throwCoreException(String message) throws CoreException {
		IStatus status = new Status(IStatus.ERROR, "AcmeStudio", IStatus.OK, message, null);
		throw new CoreException(status);
	}

	/*
	 * (non-Javadoc) @see
	 * org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 * org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;

	}

	public XMLImportWizard() {
		super();
		setNeedsProgressMonitor(true);
		try {
			_documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		}
		catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		catch (FactoryConfigurationError e) {
			e.printStackTrace();
		}
	}

	/* (non-Javadoc) @see org.eclipse.jface.wizard.IWizard#addPages() */
	public void addPages() {
		_mainPage = new XMLImportWizardPage(selection);
		addPage(_mainPage);
	}

}
