import java.io.*;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.Vector;

import acmeLib.core.exceptions.AcmeException;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.elements.*;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmePort;
import acmeLib.elements.AcmeSystem;

/*
 * Created on Mar 21, 2003
 *
 */

/**
 * @author schmerl
 */
public class GenerateTestCases {

    public static final int NUM_COMPONENTS = 100;
    public static final int NUM_CONNECTORS = 100;
    public static final int PORTS_PER_COMP = 2;
    public static final int ROLES_PER_CONN = 2;

    public static Vector _comps = new Vector (NUM_COMPONENTS);
    public static Vector _conns = new Vector (NUM_CONNECTORS);
    public static Vector _roles = new Vector (NUM_CONNECTORS*PORTS_PER_COMP);
    public static Vector _ports = new Vector (NUM_CONNECTORS*ROLES_PER_CONN);


    public static AcmeComponent createComponent (int num, AcmeSystem sys) {
        AcmeComponent comp = null;
        String name = "comp" + num;
        comp = new AcmeComponent (name);
        
        AcmePort port1 = new AcmePort ("p0");
        AcmePort port2 = new AcmePort ("p1");
        
        try {
            comp.addPort (port1);
            comp.addPort (port2);
            _ports.add (port1);
            _ports.add (port2);
            sys.addComponent (comp);
        }
        catch (AcmeIllegalNameException e) {
        }
        
        return comp;
    }
    
    public static AcmeConnector createConnector (int num, AcmeSystem sys) {
        AcmeConnector conn = null;
        String name = "conn" + num;
        conn = new AcmeConnector (name);
        
        AcmeRole role1 = new AcmeRole ("r0");
        AcmeRole role2 = new AcmeRole ("r1");
        
        try {
            conn.addRole (role1);
            conn.addRole (role2);
            _roles.add (role1);
            _roles.add (role2);
            sys.addConnector (conn);
        }
        catch (AcmeIllegalNameException e) {
        }
        
        return conn;
    }

    public static void main (String [] args) {
        
        AcmeDesign des = new AcmeDesign ();
        AcmeSystem sys = new AcmeSystem ("testSys");
        try {
            des.addSystem(sys);
        }
        catch (AcmeIllegalNameException e) {
        }
        
        for (int i = 0; i < NUM_COMPONENTS; i++) {
            _comps.add (createComponent (i, sys));
            _conns.add (createConnector (i, sys));
        }
        
        for (int i = 0; i < NUM_CONNECTORS*ROLES_PER_CONN; i++) {
            int numRolesLeft = _roles.size();
            int numPortsLeft = _ports.size ();
            int chosenRole = (int )(Math.random() * numRolesLeft);
            int chosenPort = (int )(Math.random () * numPortsLeft);
            
            sys.addAttachment((AcmePort )_ports.get (chosenPort), (AcmeRole )_roles.get (chosenRole));
            _ports.remove (chosenPort);
            _roles.remove (chosenRole);
        }
        
        AcmeOutputStream aos = new AcmeOutputStream ();
        try {
            des.exportArmani(aos);
            aos.close ();
            OutputStream os = new FileOutputStream ("testSys" + NUM_COMPONENTS + ".acme");
            os.write (aos.toString().getBytes());
            os.close ();
        }
        catch (FileNotFoundException e1) {
        }
        catch (IOException e1) {
        }
        catch (AcmeException e1) {
        }
        
        
        
    }

}
