package acme.translators;
import java.io.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import acmeLib.AcmeObject;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeDesign;
import acmeLib.errorManagement.AcmeError;
import acmeLib.errorManagement.AcmeErrorManager;

/** Class: ADMLTranslator
 *  Purpose: Provides a means to translate between Acme and ADML (XML)
 *
 * CLSID for this class is {758FDB34-2827-4566-8B13-22129ADBE442}
 *
 * Revision History:
 *   Date    Person    Change
 *  6/12/2001 BRS      Created initial version
**/

/* Magic registration chant to register this class with the Windows repository 
   and create a typelib that stores the appropriate type information.

   javareg /register /class:acme.translators.ADMLTranslator
           /clsid:{758FDB34-2827-4566-8B13-22129ADBE442}
           /progid:acme.translators.ADMLTranslator.java.1
           /typelib:ADMLTranslator.tlb
*/
public class ADMLTranslator implements AcmeTranslator {

	protected AcmeEnumeration errorMessages = null;
	protected DocumentBuilder parserXML ;

	public ADMLTranslator() {
		errorMessages = new AcmeEnumeration();
		try {
			parserXML = DocumentBuilderFactory.newInstance ().newDocumentBuilder ();
		}
		catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		catch (FactoryConfigurationError e) {
			e.printStackTrace();
		}
	}

	public String getFileType() {
		return "ADML|*.xml";
	}

	public String importAcme(String path) {
		String acme = "";
		try {
			File f = new File(path);
			Document ADMLDoc = parserXML.parse (f);
			ADMLDoc.normalize();
			acmeLib.elements.AcmeDesign design = null;
			Element designXML = ADMLDoc.getDocumentElement();
			if (!designXML.getNodeName().equals("Design")) {
				acme =
					"Document is not an ADML design -- "
						+ ADMLDoc.getNodeName();
			} else {
				ADMLImporter importer = new ADMLImporter(ADMLDoc);
				try {
					design = importer.importDesign();
					acmeLib.core.io.AcmeOutputStream st =
						new acmeLib.core.io.AcmeOutputStream();
					design.exportAcme(st);
					File tempFile = new File("C:\\temp\\export");
					FileOutputStream fo = new FileOutputStream(tempFile);
					st.writeTo(fo);
					fo.close();

					BufferedReader in =
						new BufferedReader(new FileReader("C:\\temp\\export"));
					String line = " ";
					while (line != null) {
						line = in.readLine();
						if (line != null) {
							acme += line + "\n";
						}
					}
					in.close();
				} catch (Exception e) {
					reportException(e);
				}
			}
		} catch (Exception e) {
			reportException(e);
		}
		return acme;
	}

	public boolean exportAcme(String path, String acmeDesignStr) {
		try {
			// Convert design to acmeLib.elements.AcmeDesign object
			AcmeDesign design = null;

			ByteArrayInputStream compInputStream =
				new ByteArrayInputStream(acmeDesignStr.getBytes());
			if (!ArmaniParser.parserConstructed()) {

				new ArmaniParser(compInputStream);
			} else {
				ArmaniParser.ReInit(compInputStream);
			}

			ArmaniParser.disable_tracing();
			ArmaniParser.initDesign();

			design = ArmaniParser.parse_AcmeDesign();

			//	    acmeLib.tools.converters.Acme2Armani.translate (design);
			design.doPostParsingProcessing();

			ADMLExporter exporter =
				new ADMLExporter(new acmeLib.core.io.AcmeOutputStream());
			exporter.exportDesign(design);
			exporter.writeToFile(path);
			return true;
		} catch (Throwable e) {
			reportException(e);
			return false;
		}
	}

	public String exportAcme(String acmeDesignStr) {
		try {
			// Convert design to acmeLib.elements.AcmeDesign object
			AcmeDesign design = null;

			ByteArrayInputStream compInputStream =
				new ByteArrayInputStream(acmeDesignStr.getBytes());
			if (!ArmaniParser.parserConstructed()) {

				new ArmaniParser(compInputStream);
			} else {
				ArmaniParser.ReInit(compInputStream);
			}

			ArmaniParser.disable_tracing();
			ArmaniParser.initDesign();

			design = ArmaniParser.parse_AcmeDesign();

			//	    acmeLib.tools.converters.Acme2Armani.translate (design);
			design.doPostParsingProcessing();

			ADMLExporter exporter =
				new ADMLExporter(new acmeLib.core.io.AcmeOutputStream());
			exporter.exportDesign(design);
			return exporter.writeToString();
		} catch (Throwable e) {
			reportException(e);
			return null;
		}
	}

	private void reportException(Throwable t) {
		StringWriter trace = new StringWriter();
		PrintWriter tr = new PrintWriter(trace);
		t.printStackTrace(tr);
		tr.close();
		AcmeError err =
			new AcmeError(
				(acmeLib.elements.AcmeDesign) null,
				"Error encountered translating Acme design: "
					+ t.toString()
					+ trace.toString());
		AcmeErrorManager.errorDetected(err);
		errorMessages = AcmeErrorManager.getErrors();
		System.err.println("Error encountered:" + trace.toString());
	}

	public boolean moreResults() {
		return this.errorMessages.moreItems();
	}

	public AcmeError getNextResult() {
		AcmeObject nextMsg = this.errorMessages.getNextItem();
		if (nextMsg instanceof AcmeError) {
			return (AcmeError) nextMsg;
		} else {
			return null;
		}
	}

	public static void main(String[] args) {
		if (args.length >= 1) {
			String ext =
				args[0].substring(args[0].indexOf('.') + 1, args[0].length());
			if (ext.equals("xml")) {
				ADMLTranslator translator = new ADMLTranslator();
				String acme = translator.importAcme(args[0]);
				acme =
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?><AcmeScript>\n"
						+ acme
						+ "\n</AcmeScript>";
				System.out.println(acme);
			} else if (ext.equals("acme")) {
				ADMLTranslator translator = new ADMLTranslator();
				try {
					File f = new File(args[0]);
					BufferedReader buf =
						new BufferedReader(
							new InputStreamReader(new FileInputStream(f)));

					String line = buf.readLine();
					String acmeDesc = "";
					while (line != null) {
						acmeDesc += line + "\n";
						line = buf.readLine();
					}

					boolean result = translator.exportAcme(args[1], acmeDesc);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
}
