package acme.translators;
import java.io.*;

import acmeLib.AcmeObject;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.errorManagement.AcmeError;
import acmeLib.errorManagement.AcmeErrorManager;

/** Class: RoseRTTranslator<br>
 *  Purpose: Provides a means to translate between Acme and Rational Rose Real-Time<br>
 *
 *  CLSID for this class is {3B404DCB-5EF1-4542-934F-EEDBCB82C3A3}.<br>
 *
 *  Revision History:<br>
 *     Date    Person   Change<br>
 *  5/10/2001   SWC     Created initial version<br>
*/

/* Magic registration chant to register this class with the Windows repository 
   and create a typelib that stores the appropriate type information.

   javareg /register /class:acme.translators.RoseRTTranslator
           /clsid:{3B404DCB-5EF1-4542-934F-EEDBCB82C3A3}
           /progid:acme.translators.RoseRTTranslator.java.1
           /typelib:RoseRTTranslator.tlb
*/
public class RoseRTTranslator implements AcmeTranslator {

	protected AcmeEnumeration errorMessages = null;
	protected String theDesign;

	public RoseRTTranslator() {
		errorMessages = new AcmeEnumeration();
	}

	public String getFileType() {
		return "*.rtmdl";
	}

	public String importAcme(String path) {
		String acme = "Cannot import RoseRT!";
		return acme;
	}

	private void reportException(Throwable t) {
		StringWriter trace = new StringWriter();
		PrintWriter tr = new PrintWriter(trace);
		t.printStackTrace(tr);
		tr.close();
		AcmeError err =
			new AcmeError(
				(acmeLib.elements.AcmeDesign) null,
				"Error encountered translating Acme design: " + t.toString() + trace.toString() + theDesign);
		AcmeErrorManager.errorDetected(err);
		errorMessages = AcmeErrorManager.getErrors();
		System.err.println("Error encountered:" + trace.toString());
	}

	public boolean exportAcme(String path, String acmeDesignStr) {
		theDesign = acmeDesignStr;
		try {
			// Convert design to acmeLib.elements.AcmeDesign object
			acmeLib.elements.AcmeDesign design = null;

			java.io.ByteArrayInputStream compInputStream = new java.io.ByteArrayInputStream(acmeDesignStr.getBytes());
			if (!ArmaniParser.parserConstructed()) {

				acmeLib.armaniParser.ArmaniParser acmeParser = new acmeLib.armaniParser.ArmaniParser(compInputStream);
			}

			acmeLib.armaniParser.ArmaniParser.ReInit(compInputStream);

			acmeLib.armaniParser.ArmaniParser.disable_tracing();
			acmeLib.armaniParser.ArmaniParser.initDesign();

			try {
				design = acmeLib.armaniParser.ArmaniParser.parse_AcmeDesign();
			}
			catch (acmeLib.armaniParser.ParseError ex) {
				StringWriter trace = new StringWriter();
				PrintWriter tr = new PrintWriter(trace);
				ex.printStackTrace(tr);
				tr.close();
				AcmeError parseError =
					new AcmeError(
						design,
						"Error encountered parsing Acme design: " + ex.getMessage() + trace.toString() + acmeDesignStr);
				acmeLib.errorManagement.AcmeErrorManager.errorDetected(parseError);
				//debug
				System.err.println("Error encountered parsing Acme design!");
				System.err.println(ex.getMessage() + trace.toString());
				return false;
			}

			acmeLib.tools.converters.Acme2Armani.translate(design);

			// Call the RoseRTExporter
			acme.translators.RoseRTExporter exporter = null;
			try {
				exporter = new acme.translators.RoseRTExporter(new acmeLib.core.io.AcmeOutputStream());
				exporter.exportDesign(design);
				exporter.writeToFile(path);
			}
			catch (Exception e) {
				exporter.writeToFile(path);
				reportException(e);
			}
			return true;
		}
		catch (Exception e) {
			reportException(e);
			return false;
		}
		catch (Throwable t) {
			reportException(t);
			return false;

		}
	}

	public boolean moreResults() {
		return this.errorMessages.moreItems();
	}

	public AcmeError getNextResult() {
		AcmeObject nextMsg = this.errorMessages.getNextItem();
		if (nextMsg instanceof AcmeError) {
			return (AcmeError) nextMsg;
		}
		else {
			return null;
		}
	}

	public static void main(String[] args) {
		if (args.length >= 1) {
			int dotIdx = args[0].indexOf('.');
			String file = args[0].substring(0, dotIdx);
			String ext = args[0].substring(dotIdx + 1, args[0].length());
			if (ext.equals("rtmdl")) {
				RoseRTTranslator translator = new RoseRTTranslator();
				String acme = translator.importAcme(args[0]);
				acme = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?><AcmeScript>\n" + acme + "\n</AcmeScript>";
				System.out.println(acme);
			}
			else if (ext.equals("acme")) {
				RoseRTTranslator translator = new RoseRTTranslator();
				try {
					File f = new File(args[0]);
					BufferedReader buf = new BufferedReader(new InputStreamReader(new FileInputStream(f)));

					String line = buf.readLine();
					String acmeDesc = "";
					while (line != null) {
						acmeDesc += line + "\n";
						line = buf.readLine();
					}

					//debug
					System.out.println("Target File: " + file + ".rtmdl");
					//System.out.println("AcmeDesc:\n" + acmeDesc);

					boolean result = translator.exportAcme(args[1], acmeDesc);
				}
				catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		else {
			System.out.println("");
			System.out.println("Nothing to process!");
			System.out.println("");
		}
	}
}
