package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTCapsuleCollab <br>
 * Purpose: Provides a Rose RT capsule collaboration object, that is, objects
 *   that compose the internal structure of the capsule. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTCapsuleCollab extends RoseRTObject {

    static private final String OBJ = "CapsuleCollaboration";
    static private final String NAM = "Structure";
    static private final String LBL = "structure";

    protected RoseRTList classifierRoles = null;
    protected RoseRTList ports = null;
    protected RoseRTList connectors = null;

    /**
     * Default class constructor.
     */
    public RoseRTCapsuleCollab() {
	super();

	this.setType(OBJ);
	this.setName(NAM);
	this.setLabel(LBL);
	this.classifierRoles = new RoseRTList();
	this.classifierRoles.setLabel("classifierRoles");
	this.classifierRoles.setType("classifierRolesList");
	this.ports = new RoseRTList();
	this.ports.setLabel("ports");
	this.ports.setType("portsList");
	this.connectors = new RoseRTList();
	this.connectors.setLabel("connectors");
	this.connectors.setType("connectorsList");
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTCapsuleCollab(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the list of classifier roles of this collaboration.
     *
     * @return RoseRTList
     */
    public RoseRTList getClassifierRoles() {
	return this.classifierRoles;
    }

    /**
     * Replaces the list of classifier roles of this collaboration.
     *
     * @param list new RoseRTList to replace with
     */
    public void setClassifierRoles(RoseRTList list) {
	this.classifierRoles = list;
    }

    /**
     * Returns the list of ports of this collaboration.
     *
     * @return RoseRTList
     */
    public RoseRTList getPorts() {
	return this.ports;
    }

    /**
     * Replaces the list of ports of this collaboration.
     *
     * @param list new RoseRTList to replace with
     */
    public void setPorts(RoseRTList list) {
	this.ports = list;
    }

    /**
     * Returns the list of connectors of this collaboration.
     *
     * @return RoseRTList
     */
    public RoseRTList getConnectors() {
	return this.connectors;
    }

    /**
     * Replaces the list of connectors of this collaboration.
     *
     * @param list new RoseRTList to replace with
     */
    public void setConnectors(RoseRTList list) {
	this.connectors = list;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	String clsStr = classifierRoles.toString(ind, indIncr);
	String portStr = ports.toString(ind, indIncr);
	String connStr = connectors.toString(ind, indIncr);
	str += Util.indent(leftMarg) + this.getLabel() + " (object "
	    + this.getType() + " \"" + this.getName() + "\"" +
	    (clsStr.equals("") ? "" : ("\n" + clsStr)) +
	    (portStr.equals("") ? "" : ("\n" + portStr)) +
	    (connStr.equals("") ? "" : ("\n" + connStr)) +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
