package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTCapsulePerimeterVw <br>
 * Purpose: Provides a Rose RT capsule perimeter view object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTCapsulePerimeterVw extends RoseRTView {

    static private final String OBJ = "CapsulePerimeterVw";

    private RoseRTList icons = null;

    /**
     * Default class constructor.
     */
    public RoseRTCapsulePerimeterVw() {
	super();

	this.setType(OBJ);
	this.icons = new RoseRTList();
	this.icons.setLabel("boundIcons");
	this.icons.setType("boundIconsList");
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTCapsulePerimeterVw(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the list of bound icons of this view object.
     *
     * @return RoseRTList
     */
    public RoseRTList getIcons() {
	return this.icons;
    }

    /**
     * Replaces the list of icons of this view object.
     *
     * @param list new RoseRTList to replace with
     */
    public void setIcons(RoseRTList list) {
	this.icons = list;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	String iconsStr = icons.toString(ind, indIncr);
	str += Util.indent(leftMarg) + "(object " + this.getType() + " \""
	    + this.getName() + "\" @" + this.getID() +
	    ((isSet()) ?
	     ("\n" + Util.indent(ind) + "pos " + getPos().toString()) : "") +
	    ((getWidth() > 0) ?
	     ("\n" + Util.indent(ind) + "width " + getWidth()) : "") +
	    ((getHeight() > 0) ?
	     ("\n" + Util.indent(ind) + "height " + getHeight()) : "") +
	    "\n" + Util.indent(ind) + "positionBySuperclass "
	    + Util.printBool(positionBySuperclass) +
	    (iconsStr.equals("") ? "" : ("\n" + iconsStr)) +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
