package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTCapsuleRole <br>
 * Purpose: Provides a Rose RT capsuleRole object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTCapsuleRole extends RoseRTObject {

    static private final String OBJ = "CapsuleRole";

    private String classifier = null;
    private String genericity = "fixed";

    /**
     * Default class constructor.
     */
    public RoseRTCapsuleRole() {
	super();

	this.setType(OBJ);
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTCapsuleRole(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the classifier name of this object.
     *
     * @return String
     */
    public String getClassifier() {
	return this.classifier;
    }

    /**
     * Sets the classifier name of this object.
     *
     * @param classifier classifier string to set to
     */
    public void setClassifier(String classifier) {
	this.classifier = classifier;
    }

    /**
     * Returns the genericity string of this object.
     *
     * @return String
     */
    public String getGenericity() {
	return this.genericity;
    }

    /**
     * Sets the genericity string of this object.
     *
     * @param genericity genericity string to set to
     */
    public void setGenericity(String genericity) {
	this.genericity = genericity;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";

	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + "(object " + this.getType()
	    + " \"" + this.getName() + "\"\n" +
	    Util.indent(ind) + "classifier \"" + classifier + "\"\n" +
	    Util.indent(ind) + "genericity \"" + genericity + "\"" +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
