package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTCapsuleVw <br>
 * Purpose: Provides a Rose RT capsule view object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/25/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTCapsuleVw extends RoseRTView {

    static private final String OBJ = "CapsuleVw";

    private boolean showCompartmentStereotypes = true;
    private boolean includeAttrib = true;
    private boolean includeOp = true;
    private RoseRTItemLabel stereotype = null;
    private String iconStyle = "Decoration";
    private RoseRTCompartment compartment = null;
    private int annotation = 8;
    private boolean autoResize = true;
    private boolean showPorts = true;

    /**
     * Default class constructor.
     */
    public RoseRTCapsuleVw() {
	super();

	this.setType(OBJ);
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTCapsuleVw(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the stereotype label of this view object.
     *
     * @return RoseRTItemLabel
     */
    public RoseRTItemLabel getStereotype() {
	return this.stereotype;
    }

    /**
     * Replaces the stereotype label of this view object.
     *
     * @param stereotype new RoseRTItemLabel to replace with
     */
    public void setStereotype(RoseRTItemLabel stereotype) {
	this.stereotype = stereotype;
    }

    /**
     * Returns the compartment object of this view object.
     *
     * @return RoseRTCompartment
     */
    public RoseRTCompartment getCompartment() {
	return this.compartment;
    }

    /**
     * Replaces the compartment object of this view object.
     *
     * @param compartment new RoseRTCompartment to replace with
     */
    public void setCompartment(RoseRTCompartment compartment) {
	this.compartment = compartment;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + "(object " + this.getType() + " \""
	    + this.getName() + "\" @" + this.getID() +
	    "\n" + Util.indent(ind) + "ShowCompartmentStereotypes "
	    + Util.printBool(showCompartmentStereotypes) +
	    "\n" + Util.indent(ind) + "IncludeAttribute "
	    + Util.printBool(includeAttrib) +
	    "\n" + Util.indent(ind) + "IncludeOperation "
	    + Util.printBool(includeOp) +
	    ((isSet()) ?
	     ("\n" + Util.indent(ind) + "pos " + getPos().toString()) : "") +
	    ((objLabel != null) ?
	     ("\n" + objLabel.toString(ind, indIncr)) : "") +
	    ((stereotype != null) ?
	     ("\n" + stereotype.toString(ind, indIncr)) : "") +
	    "\n" + Util.indent(ind) + "icon_style \"" + iconStyle + "\"" +
	    ((compartment != null) ?
	     ("\n" + compartment.toString(ind, indIncr)) : "") +
	    ((isSet() && getWidth() > 0) ?
	     ("\n" + Util.indent(ind) + "width " + getWidth()) : "") +
	    ((isSet() && getHeight() > 0) ?
	     ("\n" + Util.indent(ind) + "height " + getHeight()) : "") +
	    "\n" + Util.indent(ind) + "annotation " + annotation +
	    "\n" + Util.indent(ind) + "autoResize "
	    + Util.printBool(autoResize) +
	    "\n" + Util.indent(ind) + "showPorts "
	    + Util.printBool(showPorts) +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
