package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTClassAttrib <br>
 * Purpose: Provides a Rose RT class attribute object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTClassAttrib extends RoseRTObject {

    static private final String OBJ = "ClassAttribute";

    private String valueType = null;
    private String visibility = null;

    /**
     * Default class constructor.
     */
    public RoseRTClassAttrib() {
	super();

	this.setType(OBJ);
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTClassAttrib(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the value type name of this attribute.
     *
     * @return String
     */
    public String getValueType() {
	return this.valueType;
    }

    /**
     * Sets the value type name of this attribute.
     *
     * @param type type name to set to
     */
    public void setValueType(String type) {
	this.valueType = type;
    }

    /**
     * Returns the visibility of this attribute.
     *
     * @return String
     */
    public String getVisibility() {
	return this.visibility;
    }

    /**
     * Sets the visibility of this attribute.
     * May be public, protected, or private.
     *
     * @param visibility visibility string to set to
     */
    public void setVisibility(String visib) {
	this.visibility = visib;
    }

    /**
     * Sets the visibility of this attribute as public.
     */
    public void setPublic() {
	this.visibility = "Public";
    }

    /**
     * Sets the visibility of this attribute as protected.
     */
    public void setProtected() {
	this.visibility = "Protected";
    }

    /**
     * Sets the visibility of this attribute as private.
     */
    public void setPrivate() {
	this.visibility = "Private";
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";

	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + "(object " + this.getType()
	    + " \"" + this.getName() + "\"\n" +
	    Util.indent(ind) + "type \"" + valueType + "\"\n" +
	    Util.indent(ind) + "visibility \"" + visibility + "\"" +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
