package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTClassCat <br>
 * Purpose: Provides an object containing all of the elements of a
 *   particular view. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTClassCat extends RoseRTObject {

    static private final String LBL = "root_category";
    static private final String OBJ = "Class_Category";
    static private final String NAM = "Logical View";

    private boolean global = true;
    private String exportControl = "Public";
    public RoseRTList logicalModels = null;
    public RoseRTList logicalPresentations = null;

    /**
     * Default class constructor.
     */
    public RoseRTClassCat() {
	super();

	this.setLabel(LBL);
	this.setType(OBJ);
	this.setName(NAM);
	this.logicalModels = new RoseRTList();
	this.logicalModels.setLabel("logical_models");
	this.logicalModels.setType("unit_reference_list");
	this.logicalPresentations = new RoseRTList();
	this.logicalPresentations.setLabel("logical_presentations");
	this.logicalPresentations.setType("unit_reference_list");
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this Defaults object
     */
    public RoseRTClassCat(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	String lmodStr = logicalModels.toString(ind, indIncr);
	String lpresStr = logicalPresentations.toString(ind, indIncr);
	str += Util.indent(leftMarg) + this.getLabel() + " (object "
	    + this.getType() + " \"" + this.getName() + "\"" +
	    "\n" + Util.indent(ind) + "global " + Util.printBool(global) +
	    "\n" + Util.indent(ind) + "exportControl \"" + exportControl
	    + "\"" +
	    (lmodStr.equals("") ? "" : ("\n" + lmodStr)) +
	    (lpresStr.equals("") ? "" : ("\n" + lpresStr)) +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
