package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTCollabDiagram <br>
 * Purpose: Provides a Rose RT collaboration diagram object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTCollabDiagram extends RoseRTObject {

    static private final String OBJ = "CollaborationDiagram";
    static private final String LBL = "structureDiagram";

    private RoseRTList items = null;
    private int filterFlags = 521;  // not sure what this is!

    /**
     * Default class constructor.
     */
    public RoseRTCollabDiagram() {
	super();

	this.setType(OBJ);
	this.setLabel(LBL);
	this.items = new RoseRTList();
	this.items.setLabel("items");
	this.items.setType("diagram_item_list");
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTCollabDiagram(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the list of items of this collaboration diagram.
     *
     * @return RoseRTList
     */
    public RoseRTList getItems() {
	return this.items;
    }

    /**
     * Replaces the list of items of this collaboration diagram.
     *
     * @param list new RoseRTList to replace with
     */
    public void setItems(RoseRTList list) {
	this.items = list;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	String itemsStr = items.toString(ind, indIncr);
	str += Util.indent(leftMarg) + this.getLabel() + " (object "
	    + this.getType() + " \"" + this.getName() + "\"" +
	    (itemsStr.equals("") ? "" : ("\n" + itemsStr)) +
	    "\n" + Util.indent(ind) + "filterFlags " + filterFlags +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
