package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTCompartment <br>
 * Purpose: Provides a Rose RT class compartment object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/25/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTCompartment extends RoseRTLabel {

    static private final String OBJ = "Compartment";
    static private final String LBL = "compartment";

    protected int anchor = 2;
    protected int nlines = 5;
    protected int justify = 1;

    /**
     * Default class constructor.
     */
    public RoseRTCompartment() {
	super();

	this.setLabel(LBL);
	this.setType(OBJ);
    }

    /**
     * Returns the number of lines occupied by this label.
     *
     * @return int > 0; 0 means not set
     */
    public int getNLines() {
	return this.nlines;
    }

    /**
     * Sets the number of lines occupied by this label.
     *
     * @param nlines new number of lines to set to
     */
    public void setNLines(int nlines) {
	this.nlines = nlines;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + this.getLabel() + "(object "
	    + this.getType() + "\n" +
	    ((pos != null && pos.getX() > 0 && pos.getY() > 0) ?
	     (Util.indent(ind) + pos.toString() + ")\n") : "") +
	    Util.indent(ind) + "anchor " + anchor + "\n" +
	    Util.indent(ind) + "nlines " + nlines + "\n" +
	    ((maxWidth > 0) ?
	     (Util.indent(ind) + "max_width " + maxWidth + "\n") : "") +
	    Util.indent(ind) + "justify " + justify +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
