package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTConnector <br>
 * Purpose: Provides a Rose RT connector object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTConnector extends RoseRTObject {

    static private final String OBJ = "Connector";
    static private final String NAM = "c";

    private String cardinality = null;
    private RoseRTList ends = null;

    /**
     * Default class constructor.
     */
    public RoseRTConnector() {
	super();

	this.setType(OBJ);
	this.setName(NAM);
	this.ends = new RoseRTList();
	this.ends.setLabel("ends");
	this.ends.setType("endsList");
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTConnector(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the cardinality string of this connector.
     * Currently represented as a string, but may need to find better way
     * later if more syntax if found.
     *
     * @return String
     */
    public String getCardinality() {
	return this.cardinality;
    }

    /**
     * Sets the cardinality string of this connector.
     *
     * @param cardinality cardinality string to set to
     */
    public void setCardinality(String cardinality) {
	this.cardinality = cardinality;
    }

    /**
     * Returns the list of ends of this connector.
     *
     * @return RoseRTList
     */
    public RoseRTList getEnds() {
	return this.ends;
    }

    /**
     * Replaces the list of ends of this connector.
     *
     * @param list new RoseRTList to replace with
     */
    public void setEnds(RoseRTList list) {
	this.ends = list;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	String endsStr = ends.toString(ind, indIncr);
	str += Util.indent(leftMarg) + "(object " + this.getType()
	    + " \"" + this.getName() + "\"" +
	    "\n" + Util.indent(ind) + "connectorCardinality ";
	// try and see if cardinality is a single integer
	try {
	    Integer.parseInt(cardinality);
	    // at this point, no exception occurred, so we know it's single int
	    str += "constValue";
	} catch (NumberFormatException nfe) {
	    if (cardinality.equals("*")) {
		str += "constValue";
	    } else {
		str += "constClass";
	    }
	}
	// compose rest of string
	str += " \"" + cardinality + "\"" +
	    (endsStr.equals("") ? "" : ("\n" + endsStr)) +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
