package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTConnectorVw <br>
 * Purpose: Provides a Rose RT connector view object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTConnectorVw extends RoseRTView {

    static private final String OBJ = "ConnectorVw";
    static private final int OBLIQUE = 6144;
    static private final int RECTILINEAR = 39939;  //6147;

    protected boolean usesFillColor = false;
    protected int client = 0;
    protected int supplier = 0;
    protected RoseRTList vertices = null;
    protected int lineStyle = OBLIQUE;  // defaults to oblique

    /**
     * Default class constructor.
     */
    public RoseRTConnectorVw() {
	super();

	this.setType(OBJ);
	this.vertices = new RoseRTList();
	this.vertices.setLabel("vertices");
	this.vertices.setType("Points");
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTConnectorVw(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the ID of the client of this view object.
     *
     * @return int > 0; 0 means not set
     */
    public int getClient() {
	return this.client;
    }

    /**
     * Sets the ID of the client of this view object.
     *
     * @param id new id to set to
     */
    public void setClient(int id) {
	this.client = id;
    }

    /**
     * Returns the ID of the supplier of this view object.
     *
     * @return int > 0; 0 means not set
     */
    public int getSupplier() {
	return this.supplier;
    }

    /**
     * Sets the ID of the supplier of this view object.
     *
     * @param id new id to set to
     */
    public void setSupplier(int id) {
	this.supplier = id;
    }

    /**
     * Returns the list of vertices of this view object.
     *
     * @return RoseRTList
     */
    public RoseRTList getVertices() {
	return this.vertices;
    }

    /**
     * Replaces the list of vertices of this view object.
     *
     * @param vertices new RoseRTList to replace with
     */
    public void setVertices(RoseRTList vertices) {
	this.vertices = vertices;
    }

    /**
     * Sets line style to oblique
     */
    public void setOblique() {
	this.lineStyle = OBLIQUE;
    }

    /**
     * Sets line style to rectilinear
     */
    public void setRectilinear() {
	this.lineStyle = RECTILINEAR;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	String verStr = vertices.toString(ind, indIncr);
	str += Util.indent(leftMarg) + "(object " + this.getType() + " \""
	    + this.getName() + "\" @" + this.getID() +
	    "\n" + Util.indent(ind) + "uses_fill_color "
	    + Util.printBool(usesFillColor) +
	    ((objLabel != null) ?
	     ("\n" + objLabel.toString(ind, indIncr)) : "") +
	    "\n" + Util.indent(ind) + "positionBySuperclass "
	    + Util.printBool(positionBySuperclass) +
	    "\n" + Util.indent(ind) + "client @" + client +
	    "\n" + Util.indent(ind) + "supplier @" + supplier +
	    (verStr.equals("") ? "" : ("\n" + verStr)) +
	    "\n" + Util.indent(ind) + "line_style " + lineStyle +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
