package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTDefaults <br>
 * Purpose: Provides an object containing all default settings of Petal
 *   content. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTDefaults extends RoseRTObject {

    static private final String DEFAULT_OBJ = "defaults";

    private double     rightMargin = 0.25;
    private double     leftMargin = 0.25;
    private double     topMargin = 0.25;
    private double     bottomMargin = 0.50;
    private double     pageOverlap = 0.25;
    private boolean    clipIconLabels = true;
    private boolean    autoResize = true;
    private boolean    snapToGrid = true;
    private int        gridX = 16;
    private int        gridY = 16;
    private RoseRTFont defaultFont = null;
    private int        showMessageNum = 3;
    private boolean    showClassOfObject = true;
    private String     notation = "Unified";

    /**
     * Default class constructor.
     */
    public RoseRTDefaults() {
	super();

	this.setType(DEFAULT_OBJ);
	this.setLabel(DEFAULT_OBJ);
	this.defaultFont = new RoseRTFont();
    }

    /**
     * Class constructor for initializing label.
     *
     * @param label the given label of this Defaults object
     */
    public RoseRTDefaults(String label) {
	this();

	this.setLabel(label);
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";

	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + this.getLabel()
	    + " (object " + getType() + "\n" +
	    Util.indent(ind) + "rightMargin " + rightMargin + "\n" +
	    Util.indent(ind) + "leftMargin " + leftMargin + "\n" +
	    Util.indent(ind) + "topMargin " + topMargin + "\n" +
	    Util.indent(ind) + "bottomMargin " + bottomMargin + "\n" +
	    Util.indent(ind) + "pageOverlap " + pageOverlap + "\n" +
	    Util.indent(ind) + "clipIconLabels "
	    + Util.printBool(clipIconLabels) + "\n" +
	    Util.indent(ind) + "autoResize " + Util.printBool(autoResize)
	    + "\n" +
	    Util.indent(ind) + "snapToGrid " + Util.printBool(snapToGrid)
	    + "\n" +
	    Util.indent(ind) + "gridX " + gridX + "\n" +
	    Util.indent(ind) + "gridY " + gridY + "\n" +
	    defaultFont.toString(ind, indIncr) + "\n" +
	    Util.indent(ind) + "showMessageNum " + showMessageNum + "\n" +
	    Util.indent(ind) + "showClassOfObject "
	    + Util.printBool(showClassOfObject) + "\n" +
	    Util.indent(ind) + "notation \"" + notation + "\"" +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
