package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTDesign <br>
 * Purpose: Provides a design object containing all of the design contents. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTDesign extends RoseRTObject {

    static private final String DESIGN_OBJ = "Design";
    static private final String DESIGN_NAM = "Model";

    private boolean isUnit = true;
    private boolean isLoaded = true;
    private RoseRTDefaults defaults = null;
    public  RoseRTClassCat classCat = null;

    /**
     * Default class constructor.
     */
    public RoseRTDesign() {
	super();

	this.setType(DESIGN_OBJ);
	this.setName(DESIGN_NAM);
	this.defaults = new RoseRTDefaults();
	this.classCat = new RoseRTClassCat("Logical View");
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this Defaults object
     */
    public RoseRTDesign(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns this design's class category object.
     *
     * @return RoseRTClassCat
     */
    /*
    public RoseRTClassCat getClassCat() {
	return this.classCat;
    }
    */

    /**
     * Replaces this design's class category object.
     *
     * @param cat new RoseRTClassCat to replace with
     */
    /*
    public void setClassCat(RoseRTClassCat cat) {
	this.classCat = classCat;
    }
    */

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";

	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + "(object " + this.getType()
	    + " \"" + this.getName() + "\"\n" +
	    Util.indent(ind) + "is_unit " + Util.printBool(isUnit) + "\n" +
	    Util.indent(ind) + "is_loaded " + Util.printBool(isLoaded) + "\n" +
	    defaults.toString(ind, indIncr) + "\n" +
	    classCat.toString(ind, indIncr) +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
