package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTFont <br>
 * Purpose: Provides an object containing font settings of Petal content. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTFont extends RoseRTObject {

    static private final String FONT_OBJ = "Font";
    static private final String FONT_LBL = "defaultFont";

    private int size = 8;
    private String face = "MS Sans Serif";
    private boolean bold = false;
    private boolean italics = false;
    private boolean underline = false;
    private boolean strike = false;
    private long color = 16711680;
    private boolean defaultColor = true;

    /**
     * Default class constructor.
     */
    public RoseRTFont() {
	super();

	this.type = FONT_OBJ;
	this.label = FONT_LBL;
    }

    /**
     * Class constructor for initializing label.
     *
     * @param label the given label of this Font object
     */
    public RoseRTFont(String label) {
	this();

	this.setLabel(label);
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";

	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + this.getLabel()
	    + " (object " + getType() + "\n" +
	    Util.indent(ind) + "size " + size + "\n" +
	    Util.indent(ind) + "face \"" + face + "\"\n" +
	    Util.indent(ind) + "bold " + Util.printBool(bold) + "\n" +
	    Util.indent(ind) + "italics " + Util.printBool(italics) + "\n" +
    	    Util.indent(ind) + "underline " + Util.printBool(underline)
	    + "\n" +
    	    Util.indent(ind) + "strike " + Util.printBool(strike) + "\n" +
	    Util.indent(ind) + "color " + color + "\n" +
	    Util.indent(ind) + "default_color " + Util.printBool(defaultColor)
	    + ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
