package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTInheritVw <br>
 * Purpose: Provides a Rose RT class inherit relationship view object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/25/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTInheritVw extends RoseRTConnectorVw {

    static private final String OBJ = "InheritView";

    protected boolean showStereotype = true;
    protected RoseRTRectangle orig = null;
    protected RoseRTRectangle term = null;

    /**
     * Default class constructor.
     */
    public RoseRTInheritVw() {
	super();

	this.setType(OBJ);
	this.orig = new RoseRTRectangle();
	this.term = new RoseRTRectangle();
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTInheritVw(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the originating position of this View object.
     *
     * @return RoseRTPoint
     */
    public RoseRTPoint getOrigPos() {
	return this.orig.getCenter();
    }

    /**
     * Sets the originating position of this View object in terms of a point
     *   object.
     *
     * @param pos position value to set to
     */
    public void setOrigPos(RoseRTPoint pos) {
	this.orig.setCenter(pos);
    }

    /**
     * Sets the originating position of this View object in terms of
     *   coordinates.
     *
     * @param x x coordinate to set to
     * @param y y coordinate to set to
     */
    public void setOrigPos(int x, int y) {
	this.orig.setCenter(x, y);
    }

    /**
     * Returns the terminating position of this View object.
     *
     * @return RoseRTPoint
     */
    public RoseRTPoint getTermPos() {
	return this.term.getCenter();
    }

    /**
     * Sets the terminating position of this View object in terms of a point
     *   object.
     *
     * @param pos position value to set to
     */
    public void setTermPos(RoseRTPoint pos) {
	this.term.setCenter(pos);
    }

    /**
     * Sets the terminating position of this View object in terms of
     *   coordinates.
     *
     * @param x x coordinate to set to
     * @param y y coordinate to set to
     */
    public void setTermPos(int x, int y) {
	this.term.setCenter(x, y);
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	String verStr = vertices.toString(ind, indIncr);
	str += Util.indent(leftMarg) + "(object " + this.getType() + " \""
	    + this.getName() + "\" @" + this.getID() +
	    "\n" + Util.indent(ind) + "stereotype "
	    + Util.printBool(showStereotype) +
	    "\n" + Util.indent(ind) + "uses_fill_color "
	    + Util.printBool(usesFillColor) +
	    "\n" + Util.indent(ind) + "positionBySuperclass "
	    + Util.printBool(positionBySuperclass) +
	    "\n" + Util.indent(ind) + "client @" + client +
	    "\n" + Util.indent(ind) + "supplier @" + supplier +
	    (verStr.equals("") ? "" : ("\n" + verStr)) +
	    "\n" + Util.indent(ind) + "line_style " + lineStyle +
	    ((orig.isSet()) ? ("\n" + Util.indent(ind) + "origPos "
			       + getOrigPos().toString()) : "") +
	    ((term.isSet()) ? ("\n" + Util.indent(ind) + "termPos "
			       + getTermPos().toString()) : "") +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
