package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTLabel <br>
 * Purpose: Provides a Rose RT generic label object from which specific types
 *   of labels can extend, e.g. ItemLabel, SegLabel. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTLabel extends RoseRTObject {

    static private final String LBL = "label";

    protected RoseRTPoint pos = null;
    protected boolean hidden = false;
    protected boolean positionBySuperclass = false;
    protected int maxWidth = 0;
    protected String itemLabel = null;

    /**
     * Default class constructor.
     */
    public RoseRTLabel() {
	super();

	this.setLabel(LBL);
    }

    /**
     * Returns the position of this label.
     *
     * @return RoseRTPoint
     */
    public RoseRTPoint getPos() {
	return this.pos;
    }

    /**
     * Sets the position of this label in terms of a point object.
     *
     * @param pos position value to set to
     */
    public void setPos(RoseRTPoint pos) {
	this.pos = pos;
    }

    /**
     * Sets the position of this label in terms of coordinates.
     *
     * @param x x coordinate to set to
     * @param y y coordinate to set to
     */
    public void setPos(int x, int y) {
	if (this.pos == null) {
	    this.pos = new RoseRTPoint();
	}
	this.pos.setX(x);
	this.pos.setY(y);
    }

    /**
     * Returns whether this label is hidden.
     *
     * @return boolean
     */
    public boolean getHidden() {
	return this.hidden;
    }

    /**
     * Sets the flag whether to hide this label.
     *
     * @param value boolean value to set to
     */
    public void setHidden(boolean value) {
	this.hidden = value;
    }

    /**
     * Returns whether this label is positioned by the super class.
     *
     * @return boolean
     */
    public boolean getPositionBySuperclass() {
	return this.positionBySuperclass;
    }

    /**
     * Sets the flag whether to position this label by the super class.
     *
     * @param value boolean value to set to
     */
    public void setPositionBySuperclass(boolean value) {
	this.positionBySuperclass = value;
    }

    /**
     * Returns the maximum width of this label as painted on the GUI.
     *
     * @return int > 0; 0 means not set
     */
    public int getMaxWidth() {
	return this.maxWidth;
    }

    /**
     * Sets the maximum width of this label as painted on the GUI.
     *
     * @param width new width to set to
     */
    public void setMaxWidth(int width) {
	this.maxWidth = width;
    }

    /**
     * Returns the item label string of this label.
     *
     * @return String
     */
    public String getItemLabel() {
	return this.itemLabel;
    }

    /**
     * Sets the item label string of this label.
     *
     * @param label label string to set to
     */
    public void setItemLabel(String label) {
	this.itemLabel = label;
    }
}
