package acme.translators.RoseRTTypes;

//import java.util.Collection;
//import java.util.List;
//import java.util.ListIterator;
//import java.util.LinkedList;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Class: RoseRTList <br>
 * Purpose: Provides a list containing RoseRTObjects <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTList extends RoseRTObject {

    //protected LinkedList list = null;
    protected Vector list = null;

    /**
     * Default class constructor.
     */
    public RoseRTList() {
	super();

	this.list = new Vector();
    }

    /**
     * Protected class constructor used for cloning, sets up a RoseRTList
     * with a default list.
     */
    protected RoseRTList(Vector list) {
	this();

	this.list = list;
    }

    /**
     * Finds a Rose RT object with the given class name, and returns its list
     *   index.
     *
     * @param cls class name of the object
     * @return int >= 0; -1 means not found
     */
    public int findClass(Class cls) {
	RoseRTObject obj = null;
	for (Enumeration li = this.elements(); li.hasMoreElements();) {
	    obj = (RoseRTObject)li.nextElement();
	    if (obj.getClass().getName().equals(cls.getName())) {
		// found it
		return this.indexOf(obj);
	    }
	}
	// not found!
	return -1;
    }

    /**
     * Finds a Rose RT object with the given id, and returns its list index.
     *
     * @param id id of the object
     * @return int >= 0; -1 means not found
     */
    public int findID(int id) {
	RoseRTObject obj = null;
	for (Enumeration li = this.elements(); li.hasMoreElements();) {
	    obj = (RoseRTObject)li.nextElement();
	    if (obj.getID() == id) {  // found it
		return this.indexOf(obj);
	    }
	}
	// not found!
	return -1;
    }

    /**
     * Finds a Rose RT object with the given name, and returns its list index.
     *
     * @param name name of the object
     * @return int >= 0; -1 means not found
     */
    public int findName(String name) {
	RoseRTObject obj = null;
	for (Enumeration li = this.elements(); li.hasMoreElements();) {
	    obj = (RoseRTObject)li.nextElement();
	    if (obj.getName().equalsIgnoreCase(name)) {  // found it
		return this.indexOf(obj);
	    }
	}
	// not found!
	return -1;
    }

    /**
     * Returns the raw linked list, useful for casting to Collection.
     *
     * @return LinkedList
     */
    //public LinkedList getList() {
    public Vector getList() {
	return this.list;
    }

    /**
     * Returns the string representation of this list, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	if (this.size() == 0) {  // nothing to print
	    return "";
	}

	String str = "";
	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + this.getLabel() + " (list "
	    + this.getType();
	RoseRTObject obj = null;
	for (Enumeration li = this.elements();
	     li.hasMoreElements();) {
	    obj = (RoseRTObject)li.nextElement();
	    str += "\n";
	    if (obj instanceof RoseRTCapsule) {
		str += ((RoseRTCapsule)obj).toString(ind, indIncr);
	    } else if (obj instanceof RoseRTCapsuleRole) {
		str += ((RoseRTCapsuleRole)obj).toString(ind, indIncr);
	    } else if (obj instanceof RoseRTPort) {
		str += ((RoseRTPort)obj).toString(ind, indIncr);
	    } else if (obj instanceof RoseRTConnector) {
		str += ((RoseRTConnector)obj).toString(ind, indIncr);
	    } else if (obj instanceof RoseRTPoint) {
		str += Util.indent(ind) + ((RoseRTPoint)obj).toString();
	    } else {
		str += obj.toString(ind, indIncr);
	    }
	}
	str += ")";

	return str;
    }

    /**
     * Returns the string representation of this list, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }


    // LinkedList override Vector methods

    public void add(int index, RoseRTObject element) {
	//this.list.add(index, element);
	this.list.insertElementAt(element, index);
    }

    //public boolean add(RoseRTObject o) {
	//return this.list.add(o);
    public void add(RoseRTObject o) {
	this.list.addElement(o);
    }

    //public boolean addAll(Collection c) {
    //    return this.list.addAll(c);
    public void addAll(Vector list) {
	RoseRTObject obj = null;
	for (Enumeration li = list.elements(); li.hasMoreElements();) {
	    obj = (RoseRTObject)li.nextElement();
	    this.list.addElement(obj);
	}
    }

    //public boolean addAll(int index, Collection c) {
    //    return this.list.addAll(index, c);
    //}

    public void addFirst(RoseRTObject o) {
	//this.list.addFirst(o);
	this.list.insertElementAt(o, 0);
    }

    public void addLast(RoseRTObject o) {
	//this.list.addLast(o);
	this.list.addElement(o);
    }

    public void clear() {
	//this.list.clear();
	this.list.removeAllElements();
    }

    public Object clone() {
	//LinkedList llist = (LinkedList)this.list.clone();
	Vector llist = (Vector)this.list.clone();
	RoseRTList rrtlist = new RoseRTList(llist);
	rrtlist.setType(getType());
	return rrtlist;
    }

    public boolean contains(RoseRTObject o) {
	return this.list.contains(o);
    }

    public Object get(int index) {
	//return this.list.get(index);
	return this.list.elementAt(index);
    }

    public Object getFirst() {
	//return this.list.getFirst();
	return this.list.elementAt(0);
    }

    public Object getLast() {
	//return this.list.getLast();
	return this.list.elementAt(this.list.size() - 1);
    }

    public int indexOf(RoseRTObject o) {
	return this.list.indexOf(o);
    }

    public int lastIndexOf(RoseRTObject o) {
	return this.list.lastIndexOf(o);
    }

    //public ListIterator listIterator(int index) {
    //    return this.list.listIterator(index);
    //}
    public Enumeration elements() {
	return this.list.elements();
    }

    public Object remove(int index) {
	//return this.list.remove(index);
	Object obj = this.list.elementAt(index);
	this.list.removeElementAt(index);
	return obj;
    }

    public boolean remove(RoseRTObject o) {
	//return this.list.remove(o);
	return this.list.removeElement(o);
    }

    public Object removeFirst() {
	//return this.list.removeFirst();
	Object obj = this.list.elementAt(0);
	this.list.removeElementAt(0);
	return obj;
    }

    public Object removeLast() {
	//return this.list.removeLast();
	Object obj = this.list.elementAt(this.list.size() - 1);
	this.list.removeElementAt(this.list.size() - 1);
	return obj;
    }

    public Object set(int index, RoseRTObject element) {
	//return this.list.set(index, element);
	Object obj = this.list.elementAt(index);
	this.list.setElementAt(element, index);
	return obj;
    }

    public int size() {
	return this.list.size();
    }

    //public Object[] toArray() {
    //    return this.list.toArray();
    //}

    //public Object[] toArray(RoseRTObject[] a) {
    //    return this.list.toArray(a);
    //}
}
