package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTObject <br>
 * Purpose: Provides the base type for Rose Real-Time Petal file <br>
 * Type tree: <br>
 *   * Object <br>
 *     + Design <br>
 *     + Defaults <br>
 *     + Font <br>
 *     + ClassCategory <br>
 *     + Capsule <br>
 *     + CapsuleCollaboration <br>
 *       += Refinement <br>
 *     + CapsuleRole <br>
 *     + Port <br>
 *     + Connector <br>
 *     + Protocol <br>
 *     + ClassAttribute <br>
 *     + InheritanceRel <br>
 *     + ClassDiagram <br>
 *     + CollaborationDiagram <br>
 *     + View <br>
 *       += CapsuleVw <br>
 *       += CapsulePerimeterVw <br>
 *       += CapsuleRoleVw <br>
 *       += ConnectionPtVw <br>
 *       += PortVw <br>
 *       += ConnectorVw <br>
 *          +# InheritVw <br>
 *             +x CapsuleRoleRelVw <br>
 *     + Label <br>
 *       += ItemLabel <br>
 *       += SegLabel <br>
 *       += Compartment <br>
 *     + List <br>
 *     + Point <br>
 *     + Rectangle <br>
 *     + String <br>
 *     + Layout <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions: <br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTObject extends Object {

    // the optional String that comes before "("
    protected String label;
    // the String that comes after "(..."
    protected String type;
    // the optional String that comes after "(... type " in quotes
    protected String name;
    // the optional numeric id
    protected int id;


    /**
     * Default class constructor.
     */
    public RoseRTObject() {
	super();

	this.label = null;
	this.type = null;
	this.name = null;
	this.id = 0;
    }

    /**
     * Class constructor for initializing with label, type, name, and id.
     */
    public RoseRTObject(String label, String type, String name, int id) {
	this();

	this.label = label;
	this.type = type;
	this.name = name;
	this.id = id;
    }

    /**
     * Returns the label string of this object.
     *
     * @return String
     */
    public String getLabel() {
	return this.label;
    }

    /**
     * Sets the label string of this object.
     *
     * @param label label string to set to
     */
    public void setLabel(String label) {
	this.label = label;
    }

    /**
     * Returns the name string of this object.
     *
     * @return String
     */
    public String getName() {
	return this.name;
    }

    /**
     * Sets the name string of this object.
     *
     * @param name name string to set to
     */
    public void setName(String name) {
	this.name = name;
    }

    /**
     * Returns the type name used to identify what type of elements this
     * object contains.
     *
     * @return String
     */
    public String getType() {
	return this.type;
    }

    /**
     * Sets the type name of this object.
     *
     * @param type type string to set to
     */
    public void setType(String type) {
	this.type = type;
    }

    /**
     * Returns the assigned ID of this object.
     *
     * @return int > 0
     */
    public int getID() {
	return this.id;
    }

    /**
     * Sets the assigned ID of this object.
     *
     * @param id id value to set to
     */
    public void setID(int id) {
	this.id = id;
    }

    /**
     * Returns the string representation of this object.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";

	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + "(object " + this.getType()
	    + " \"" + this.getName() + "\"" +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
