package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTPoint <br>
 * Purpose: Provides a Rose RT point object with X and Y coordinates. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTPoint extends RoseRTObject {

    protected int posX = 0;
    protected int posY = 0;

    /**
     * Default class constructor.
     */
    public RoseRTPoint() {
	super();
    }

    /**
     * Class constructor for initializing x and y coordinates.
     *
     * @param x X coordinate
     * @param y Y coordinate
     */
    public RoseRTPoint(int x, int y) {
	super();

	this.posX = x;
	this.posY = y;
    }

    /**
     * Returns the X position of this point.
     *
     * @return int > 0; 0 means not set
     */
    public int getX() {
	return this.posX;
    }

    /**
     * Sets the X position of this point.
     *
     * @param pos position value to set X to
     */
    public void setX(int pos) {
	this.posX = pos;
    }

    /**
     * Returns the Y position of this point.
     *
     * @return int > 0; 0 means not set
     */
    public int getY() {
	return this.posY;
    }

    /**
     * Sets the Y position of this point.
     *
     * @param pos position value to set Y to
     */
    public void setY(int pos) {
	this.posY = pos;
    }

    /**
     * Returns the string representation of this object.
     *
     * @return String
     */
    public String toString() {
	return ("(" + this.getX() + "," + this.getY() + ")");
    }
}
