package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTPort <br>
 * Purpose: Provides a Rose RT port object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTPort extends RoseRTObject {

    static private final String OBJ = "Port";
    static private final String NAM = "port";

    private String classifier = null;
    private boolean isConjugated = false;

    /**
     * Default class constructor.
     */
    public RoseRTPort() {
	super();

	this.setType(OBJ);
	this.setName(NAM);
    }

    /**
     * Class constructor for initializing name.
     *
     * @param name the given name of this object
     */
    public RoseRTPort(String name) {
	this();

	this.setName(name);
    }

    /**
     * Returns the classifier name of this object.
     *
     * @return String
     */
    public String getClassifier() {
	return this.classifier;
    }

    /**
     * Sets the classifier name of this object.
     *
     * @param classifier classifier string to set to
     */
    public void setClassifier(String classifier) {
	this.classifier = classifier;
    }

    /**
     * Returns whether this port is a conjugate.
     *
     * @return boolean
     */
    public boolean isConjugated() {
	return this.isConjugated;
    }

    /**
     * Sets the conjugate flag of this port.
     *
     * @param value boolean value to set to
     */
    public void setConjugated(boolean value) {
	this.isConjugated = value;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";

	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + "(object " + this.getType()
	    + " \"" + this.getName() + "\"\n" +
	    Util.indent(ind) + "classifier \"" + classifier + "\"\n" +
	    Util.indent(ind) + "isConjugated " + Util.printBool(isConjugated) +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
