package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTRectangle <br>
 * Purpose: Provides a rectangle graph object center coordinate and half-length
 *   and half-width to the corners. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTRectangle extends RoseRTObject {

    protected RoseRTPoint center = null;
    protected RoseRTPoint dim = null;
    protected RoseRTList  links = null;
    // for graph tree traversal
    public boolean traversed = false;

    /**
     * Default class constructor.
     */
    public RoseRTRectangle() {
	super();

	this.center = new RoseRTPoint(0,0);
	this.dim = new RoseRTPoint(0,0);
	this.links = new RoseRTList();
    }

    /**
     * Class constructor for initializing center x and y coordinates.
     *
     * @param x X coordinate
     * @param y Y coordinate
     */
    public RoseRTRectangle(int x, int y, int width, int height) {
	this();

	this.center.setX(x);
	this.center.setY(y);
	this.dim.setX(width);
	this.dim.setY(height);
    }

    /**
     * Returns whether rectangle has been set with a center.
     *
     * @return boolean
     */
    public boolean isSet() {
	if (getCenter().getX() > 0 && getCenter().getY() > 0) {
	    return true;
	} else {
	    return false;
	}
    }

    /**
     * Returns the center coordinate of this rectangle.
     *
     * @return RoseRTPoint any 0 means not set
     */
    public RoseRTPoint getCenter() {
	return this.center;
    }

    /**
     * Sets the center coordinate of this rectangle.
     *
     * @param pos position coordinate to set to
     */
    public void setCenter(RoseRTPoint pos) {
	this.center = pos;
    }

    /**
     * Sets the center coordinate of this rectangle.
     *
     * @param x x to set to
     * @param y y to set to
     */
    public void setCenter(int x, int y) {
	if (this.center == null) {
	    this.center = new RoseRTPoint(x, y);
	} else {
	    this.center.setX(x);
	    this.center.setY(y);
	}
    }

    /**
     * Returns the dimension of this rectangle.
     *
     * @return RoseRTPoint any 0 means not set
     */
    public RoseRTPoint getDim() {
	return this.dim;
    }

    /**
     * Sets the dimension of this rectangle.
     *
     * @param dim dimension values to set to
     */
    public void setDim(RoseRTPoint dim) {
	this.dim = dim;
    }

    /**
     * Sets the dimension of this rectangle.
     *
     * @param w width to set to
     * @param h height to set to
     */
    public void setDim(int w, int h) {
	if (this.dim == null) {
	    this.dim = new RoseRTPoint(w, h);
	} else {
	    this.dim.setX(w);
	    this.dim.setY(h);
	}
    }

    /**
     * Returns the width of this rectangle.
     *
     * @return int > 0, = 0 means not set
     */
    public int getWidth() {
	return this.dim.getX();
    }

    /**
     * Sets the width of this rectangle.
     *
     * @param width new width to set to
     */
    public void setWidth(int width) {
	this.dim.setX(width);
    }

    /**
     * Returns the height of this rectangle.
     *
     * @return int > 0, = 0 means not set
     */
    public int getHeight() {
	return this.dim.getY();
    }

    /**
     * Sets the height of this rectangle.
     *
     * @param height new height to set to
     */
    public void setHeight(int height) {
	this.dim.setY(height);
    }

    /**
     * Returns list of linked objects
     *
     * @return RoseRTList list of objects
     */
    public RoseRTList getLinks() {
	return this.links;
    }

    /**
     * Sets list of linked objects
     *
     * @param list new list to set to
     */
    public void setLinks(RoseRTList list) {
	this.links = list;
    }

    /**
     * Returns the string representation of this object.
     *
     * @return String
     */
    public String toString() {
	return ("(" + getCenter().getX() + "," + getCenter().getY() + ") " +
		getWidth() + " x " + getHeight());
    }
}
