package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTRefinement <br>
 * Purpose: Provides a Rose RT capsule refinement object, that is, objects
 *   that compose the internal structure of the capsule. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTRefinement extends RoseRTCapsuleCollab {

    static private final String OBJ = "Refinement";

    protected String supplierStructure = null;

    public RoseRTRefinement() {
	super();

	this.setType(OBJ);
    }

    /**
     * Returns the supplier structure label of this refinement.
     *
     * @return String
     */
    public String getSupplierStructure() {
	return this.supplierStructure;
    }

    /**
     * Sets the supplier structure label of this refinement.
     *
     * @param label label string to set to
     */
    public void setSupplierStructure(String label) {
	this.supplierStructure = label;
    }
    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	String clsStr = classifierRoles.toString(ind, indIncr);
	String portStr = ports.toString(ind, indIncr);
	String connStr = connectors.toString(ind, indIncr);
	str += Util.indent(leftMarg) + "(object " + this.getType() +
	    (clsStr.equals("") ? "" : ("\n" + clsStr)) +
	    (portStr.equals("") ? "" : ("\n" + portStr)) +
	    (connStr.equals("") ? "" : ("\n" + connStr)) +
	    ((supplierStructure != null) ?
	     ("\n" + Util.indent(ind) + "supplierStructure \""
	      + supplierStructure) + "\"" : "") +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
