package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTSegLabel <br>
 * Purpose: Provides a Rose RT segment label object. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/24/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTSegLabel extends RoseRTLabel {

    static private final String OBJ = "SegLabel";
    static private final String LBL = "label";

    protected int anchor = 1;
    protected double pctDist = 0.0;
    protected int height = 0;
    protected int orientation = 1;

    /**
     * Default class constructor.
     */
    public RoseRTSegLabel() {
	super();

	this.setLabel(LBL);
	this.setType(OBJ);
    }

    /**
     * Returns the percent distance of this label (from what??).
     *
     * @return double > 0.0; 0.0 means not set
     */
    public double getPctDist() {
	return this.pctDist;
    }

    /**
     * Sets the percent distance of this label (from what??).
     *
     * @param pct new percent distance to set to
     */
    public void setPctDist(double pct) {
	this.pctDist = pct;
    }

    /**
     * Returns the height of this label.
     *
     * @return int > 0; 0 means not set
     */
    public int getHeight() {
	return this.height;
    }

    /**
     * Sets the height of this label.
     *
     * @param height height value to set to
     */
    public void setHeight(int height) {
	this.height = height;
    }

    /**
     * Returns the orientation of this label.
     *
     * @return int > 0; 0 means not set
     */
    public int getOrientation() {
	return this.orientation;
    }

    /**
     * Sets the orientation of this label.
     *
     * @param orientation orientation value to set to
     */
    public void setOrientation(int orientation) {
	this.orientation = orientation;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @param leftMarg amount of spaces to indent for all lines
     * @param indIncr amount of increment spaces to indent for subsequent lines
     * @return String
     */
    public String toString(int leftMarg, int indIncr) {
	String str = "";
	int ind = leftMarg + indIncr;
	str += Util.indent(leftMarg) + this.getLabel() + " (object "
	    + this.getType() + " @" + this.getID() + "\n" +
	    ((pos != null && pos.getX() > 0 && pos.getY() > 0) ?
	     (Util.indent(ind) + "pos " + pos.toString() + "\n") : "") +
	    (hidden ? (Util.indent(ind) + "hidden "
		       + Util.printBool(hidden) + "\n") : "") +
	    Util.indent(ind) + "positionBySuperclass "
	    + Util.printBool(positionBySuperclass) + "\n" +
	    (hidden ? (Util.indent(ind) + "anchor " + anchor + "\n") : "") +
	    ((maxWidth > 0) ?
	     (Util.indent(ind) + "max_width " + maxWidth + "\n") : "") +
	    ((itemLabel != null) ?
	     (Util.indent(ind) + "label \"" + itemLabel + "\"\n") : "") +
	    ((pctDist > 0.0) ?
	     (Util.indent(ind) + "pctDist " + pctDist + "\n") : "") +
	    ((height > 0) ?
	     (Util.indent(ind) + "height " + height + "\n") : "") +
	    Util.indent(ind) + "orientation " + orientation +
	    ")";

	return str;
    }

    /**
     * Returns the string representation of this object, basically generating
     * all the content.
     *
     * @return String
     */
    public String toString() {
	return this.toString(0, 8);
    }
}
