package acme.translators.RoseRTTypes;

/**
 * Class: RoseRTView <br>
 * Purpose: Provides a Rose RT generic view object from which various views
 *   extend.  Most view object contain the common elements here.  Label is
 *   generic here and specified within each view object subclass. <br>
 * <br>
 * Revision History: <br>
 *   Date    Person    Change: <br>
 * 5/23/01    SWC      Created initial version <br>
 * <br>
 * Known bugs/omissions:<br>
 *   - <br>
 * @author Owen Cheng (SWC)
 */

public class RoseRTView extends RoseRTObject {

    protected RoseRTRectangle rect = null;
    protected RoseRTLabel objLabel = null;
    protected boolean positionBySuperclass = false;

    // content not printed
    protected RoseRTPoint origin = null;
    protected boolean isUpLow = false;  // whether sits on upper/lower border

    /**
     * Default class constructor.
     */
    public RoseRTView() {
	super();

	this.rect = new RoseRTRectangle();
    }


    /**
     * Returns whether rectangle has been set with a center.
     *
     * @return boolean
     */
    public boolean isSet() {
	return rect.isSet();
    }

    /**
     * Returns the rectangle graph obj of this View object.
     *
     * @return RoseRTRectangle
     */
    public RoseRTRectangle getRect() {
	return this.rect;
    }

    /**
     * Sets the rectangle graph obj of this View object.
     *
     * @param rect new rectangle object to set to
     */
    public void setRect(RoseRTRectangle rect) {
	this.rect = rect;
    }

    /**
     * Returns the position of this View object.
     *
     * @return RoseRTPoint
     */
    public RoseRTPoint getPos() {
	return this.rect.getCenter();
    }

    /**
     * Sets the position of this View object in terms of a point object.
     *
     * @param pos position value to set to
     */
    public void setPos(RoseRTPoint pos) {
	this.rect.setCenter(pos);
    }

    /**
     * Sets the position of this View object in terms of coordinates.
     *
     * @param x x coordinate to set to
     * @param y y coordinate to set to
     */
    public void setPos(int x, int y) {
	this.rect.setCenter(x, y);
    }

    /**
     * Returns the width of this view object.
     *
     * @return int > 0; 0 means not set
     */
    public int getWidth() {
	return this.rect.getWidth();
    }

    /**
     * Sets the width of this view object.
     *
     * @param width new width to set to
     */
    public void setWidth(int width) {
	this.rect.setWidth(width);
    }

    /**
     * Returns the height of this view object.
     *
     * @return int > 0; 0 means not set
     */
    public int getHeight() {
	return this.rect.getHeight();
    }

    /**
     * Sets the height of this view object.
     *
     * @param height new height to set to
     */
    public void setHeight(int height) {
	this.rect.setHeight(height);
    }

    /**
     * Returns the label of this View object.
     *
     * @return RoseRTLabel
     */
    public RoseRTLabel getObjLabel() {
	return this.objLabel;
    }

    /**
     * Sets the label of this View object.
     *
     * @param objLabel new label to set to
     */
    public void setObjLabel(RoseRTLabel objLabel) {
	this.objLabel = objLabel;
    }

    /**
     * Returns whether this View object is positioned by the super class.
     *
     * @return boolean
     */
    public boolean getPositionBySuperclass() {
	return this.positionBySuperclass;
    }

    /**
     * Sets the flag whether to position this View object by the super class.
     *
     * @param value boolean value to set to
     */
    public void setPositionBySuperclass(boolean value) {
	this.positionBySuperclass = value;
    }

    /**
     * Returns the origin coord of the object this view object is attached to.
     *
     * @return RoseRTPoint
     */
    public RoseRTPoint getOrigin() {
	return this.origin;
    }

    /**
     * Sets the origin coord of the object this view object is attached to.
     *
     * @param pos position value to set to
     */
    public void setOrigin(RoseRTPoint origin) {
	this.origin = origin;
    }

    /**
     * Returns whether this View object is positioned on the upper or lower
     *   border of another object.
     *
     * @return boolean
     */
    public boolean isUpLow() {
	return this.isUpLow;
    }

    /**
     * Sets the flag whether this View object is positioned on the upper or
     *   lower border of another object.
     *
     * @param value boolean value to set to
     */
    public void setUpLow(boolean value) {
	this.isUpLow = value;
    }
}
