package acme.translators;
import java.io.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.errorManagement.AcmeError;
import acmeLib.errorManagement.AcmeErrorManager;

/** Class: XAcmeTranslator 
 *  Purpose: Provides a means to translate between Acme and xAcme (XML)
 *
 *  CLSID for this class is {E8F7274E-9276-4168-9D89-8E850D9A2205}.
 *
 *  Revision History:
 *    Date    Person   Change
 *  5/4/2001   BRS      Created initial version
*/

/* Magic registration chant to register this class with the Windows repository 
   and create a typelib that stores the appropriate type information.

   javareg /register /class:acme.translators.XAcmeTranslator
           /clsid:{E8F7274E-9276-4168-9D89-8E850D9A2205}
           /progid:acme.translators.XAcmeTranslator.java.1
           /typelib:XAcmeTranslator.tlb
*/
public class XAcmeTranslator implements AcmeTranslator {

    protected DocumentBuilder parser;
    protected AcmeEnumeration errorMessages = null;

    public XAcmeTranslator() {
        errorMessages = new AcmeEnumeration();
        try {
			parser = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		}
		catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		catch (FactoryConfigurationError e) {
			e.printStackTrace();
		}
    }

    public String getFileType() {
        return "xAcme|*.xml";
    }

    public String importAcme(String path) {
        String acme = "";
        try {
            File f = new File(path);
            Document xAcmeDoc = parser.parse (f);
            acmeLib.elements.AcmeDesign design = null;
            Element typedDesign = xAcmeDoc.getDocumentElement();
            if (!typedDesign.getNodeName().equals("typedDesign")) {
                // Needs to be an AcmeError object
                acme = "Document is not a typed design";
            }
            else {

                // Call the xAcmeImporter
                XAcmeImporter importer = new XAcmeImporter(xAcmeDoc);
                try {
                    design = importer.importDesign();
                    acmeLib.core.io.AcmeOutputStream st = new acmeLib.core.io.AcmeOutputStream();
                    design.exportAcme(st);
                    File tempFile = new File("C:\\temp\\export");
                    FileOutputStream fo = new FileOutputStream(tempFile);
                    st.writeTo(fo);
                    fo.close();

                    BufferedReader in = new BufferedReader(new FileReader("C:\\temp\\export"));
                    String line = " ";
                    while (line != null) {
                        line = in.readLine();
                        if (line != null) {
                            acme += line + "\n";
                        }
                    }
                    in.close();
                }
                catch (acmeLib.core.exceptions.AcmeIllegalNameException e) {
                    AcmeError pError =
                        new AcmeError(design, "Error encountered building AcmeDesign: " + e.getMessage());
                    AcmeErrorManager.errorDetected(pError);
                    errorMessages = AcmeErrorManager.getErrors();
                    return "Error:" + e;
                }
                catch (Exception e) {
                    AcmeError pError = new AcmeError(null, "Error encountered building AcmeDesign: " + e);
                    AcmeErrorManager.errorDetected(pError);
                    errorMessages = AcmeErrorManager.getErrors();
                    return "Error: " + e;

                }

            }
        }
        catch (org.xml.sax.SAXException e) {
            AcmeError pError = new AcmeError(null, "SAX Error encountered building AcmeDesign: " + e);
            AcmeErrorManager.errorDetected(pError);
            errorMessages = AcmeErrorManager.getErrors();
            return "Error: " + e;
        }
        catch (Exception e) {
            AcmeError pError = new AcmeError(null, "Error encountered building AcmeDesign: " + e);
            AcmeErrorManager.errorDetected(pError);
            errorMessages = AcmeErrorManager.getErrors();
            return "Error: " + e;
        }

        return acme;
    }

    public boolean exportAcme(String path, String acmeDesignStr) {
        try {
            // Convert design to acmeLib.elements.AcmeDesign object
            acmeLib.elements.AcmeDesign design = null;

            java.io.ByteArrayInputStream compInputStream = new java.io.ByteArrayInputStream(acmeDesignStr.getBytes());

            // This assumes that the parser is the only one active. In
            // fact, when this is used as a COM object, there may be
            // multiple instances of the parser. This needs to be
            // changed (I need to find out if the parser has already
            // been initialized, and then I need to make sure that the
            // top level design is set. 
            // There is still an error if I use ReInit - a null
            // pointer results in Acme2Armani.translate (the design is
            // null). I need to work this out.

            try {
                acmeLib.armaniParser.ArmaniParser armaniParser = new acmeLib.armaniParser.ArmaniParser(compInputStream);
            }
            catch (java.lang.Error e) {
                acmeLib.armaniParser.ArmaniParser.ReInit(compInputStream);
            }

            acmeLib.armaniParser.ArmaniParser.disable_tracing();
            acmeLib.armaniParser.ArmaniParser.initDesign();

            try {
                design = acmeLib.armaniParser.ArmaniParser.parse_AcmeDesign();
            }
            catch (acmeLib.armaniParser.ParseError ex) {
                AcmeError parseError =
                    new AcmeError(
                        design,
                        "Error encountered parsing Acme design: "
                            + ex.getMessage()
                            + "params: "
                            + path
                            + ", "
                            + acmeDesignStr);
                acmeLib.errorManagement.AcmeErrorManager.errorDetected(parseError);
                errorMessages = AcmeErrorManager.getErrors();
                return false;
            }

            //	    acmeLib.tools.converters.Acme2Armani.translate (design);

            // Call the xAcmeExporter

            try {
                XAcmeExporter exporter = new XAcmeExporter(new acmeLib.core.io.AcmeOutputStream());
                exporter.exportDesign(design);
                exporter.writeToFile(path);
            }
            catch (Exception e) {
                StringWriter trace = new StringWriter();
                PrintWriter tr = new PrintWriter(trace);
                e.printStackTrace(tr);
                tr.close();

                try {
                    trace.close();
                }
                catch (Exception ex) {
                }
                AcmeError err =
                    new AcmeError(
                        (acmeLib.elements.AcmeDesign) null,
                        "Error encountered translating Acme design: " + trace.toString());
                AcmeErrorManager.errorDetected(err);
                errorMessages = AcmeErrorManager.getErrors();

            }
            return true;
        }
        catch (Exception e) {
            StringWriter trace = new StringWriter();
            PrintWriter tr = new PrintWriter(trace);
            e.printStackTrace(tr);
            tr.close();

            try {
                trace.close();
            }
            catch (Exception ex) {
            }

            AcmeError err =
                new AcmeError(
                    (acmeLib.elements.AcmeDesign) null,
                    "Error encountered translating Acme design: " + e.toString() + trace.toString() + acmeDesignStr);
            AcmeErrorManager.errorDetected(err);
            errorMessages = AcmeErrorManager.getErrors();
            return false;
        }
    }

    public boolean moreResults() {
        return this.errorMessages.moreItems();
    }

    public AcmeError getNextResult() {
        AcmeObject nextMsg = this.errorMessages.getNextItem();
        if (nextMsg instanceof AcmeError) {
            return (AcmeError) nextMsg;
        }
        else {
            return null;
        }
    }

    public static void main(String[] args) {
        if (args.length >= 1) {
            String ext = args[0].substring(args[0].indexOf('.') + 1, args[0].length());
            if (ext.equals("xml")) {
                XAcmeTranslator translator = new XAcmeTranslator();
                String acme = translator.importAcme(args[0]);
                acme = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?><AcmeScript>\n" + acme + "\n</AcmeScript>";
                System.out.println(acme);
            }
            else if (ext.equals("acme")) {
                XAcmeTranslator translator = new XAcmeTranslator();
                try {
                    File f = new File(args[0]);
                    BufferedReader buf = new BufferedReader(new InputStreamReader(new FileInputStream(f)));

                    String line = buf.readLine();
                    String acmeDesc = "";
                    while (line != null) {
                        acmeDesc += line + "\n";
                        line = buf.readLine();
                    }

                    boolean result = translator.exportAcme(args[1], acmeDesc);
                }
                catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
