/*
 * Created on Sep 26, 2003
 *
 */
package acme.translators;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import acmeLib.AcmeObject;
import acmeLib.AcmeType;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.*;

/**
 * @author Bradley Schmerl
 * Created Sep 26, 2003
 */
public class XMLExporter {

	/**
	 * 
	 */
	public XMLExporter() {
		super();
	}

	private Map _familyToXML = new HashMap ();
	protected int familyAction = AcmeTranslator.REFERENCE_IMPORTED_FAMILIES;
	protected Map tidMap = new HashMap ();
	/** Contains mappings from AcmeDesign -> XAcmeExporter **/
	private Map _designExporters = new HashMap ();

	public XAcmeExporter getExporterForDesign(AcmeDesign des) {
		return (XAcmeExporter )_designExporters.get (des);
	}

	public void addExporterForDesign(AcmeDesign des, XMLExporter xporter) {
		_designExporters.put(des, xporter);
	}

	public String getTypeIdInImport(AcmeObject o) {
		return (String )tidMap.get (o);	
	}

	/**
	 * @param object
	 * @return
	 */
	protected String getTypeReference(AcmeElement element, AcmeObject object) {
		if (object instanceof AcmeElementType) {
			if ((fromSameDesign(element, object)) || familyAction == AcmeTranslator.INLINE_IMPORTED_FAMILIES) {
				return "#" + (String )tidMap.get(object);
			}
			else if (!fromSameDesign (element, object) && familyAction == AcmeTranslator.REFERENCE_IMPORTED_FAMILIES) {
				return ((AcmeElement )object).getParent ().getName () + "#" + ((AcmeElementType )object).getTypeName();	
			}
			else if (!fromSameDesign (element, object) && familyAction == AcmeTranslator.EXPORT_IMPORTED_FAMILIES) {
				AcmeElement fam = ((AcmeElement )object).getParent ();
				if (object instanceof AcmeFamily) fam = (AcmeFamily )object;
				if (fam instanceof AcmeFamily) {
					String xmlFile = (String )_familyToXML.get (fam);
					if (xmlFile == null) {
						// place the xmlized name of the imported design in a map so that I know how to reference it
						AcmeDesign importedDesign = fam.getParentDesign();
						AcmeDesign currentDesign = element.getParentDesign();
						AcmeEnumeration ids = currentDesign.getImportedDesigns().enumerate ();
						boolean found = false;
						AcmeImportedDesignRecord rec = null;
						while (ids.moreItems () && !found) {
							rec = (AcmeImportedDesignRecord )ids.getNextItem ();
							found = rec.getResolvedDesign() == importedDesign;
							if (!found) {
								rec = recurseFindDesign (importedDesign, rec.getResolvedDesign ());
								found = rec != null;
							}
						}
						if (found) {
							xmlFile = convertFilenameToXML (rec.getCanonicalFilename());
							_familyToXML.put (fam, xmlFile);		
						}
						else {
							System.out.println ("Should never get here");
						}
					}
					return xmlFile + "#" + getExporterForDesign(fam.getParentDesign ()).getTypeIdInImport(object);
				}	
			}
		}
		return null;
	}

	/**
	 * @param currentDesign
	 * @param importedDesign
	 * @return
	 */
	private AcmeImportedDesignRecord recurseFindDesign(AcmeDesign currentDesign, AcmeDesign importedDesign) {
		AcmeEnumeration ids = importedDesign.getImportedDesigns().enumerate ();
		boolean found = false;
		AcmeImportedDesignRecord rec = null;
		while (ids.moreItems () && !found) {
			rec = (AcmeImportedDesignRecord )ids.getNextItem ();
			found = currentDesign == rec.getResolvedDesign ();
			if (!found) {
				rec = recurseFindDesign(currentDesign, rec.getResolvedDesign());
				found = rec != null;
			}
		}
		return rec;
	}

	/**
	 * @param string
	 * @return
	 */
	public static String convertFilenameToXML(String string) {
		int indexOfSlash = string.lastIndexOf("/");
		if (indexOfSlash == -1) {
			indexOfSlash = string.lastIndexOf("\\");
		}
		
		String filename = string;
		if (indexOfSlash != -1) {
			filename = filename.substring (indexOfSlash + 1);	
		}
		
		int indexOfDot = filename.lastIndexOf(".");
		if (indexOfDot == -1) {
			filename = filename + ".xml";
		}
		else {
			filename = filename.substring (0, indexOfDot) + ".xml";	
		}
		return filename;
				
	}

	private boolean fromSameDesign(AcmeElement element, AcmeObject object) {
		return ((AcmeElement )object).getParentDesign() == element.getParentDesign ();
	}

	/**
	 * @param b
	 */
	public void setFamilyAction(int b) {
		familyAction = b;
	}

	/**
	 * @param x
	 */
	public void transferExporters(XMLExporter x) {
		Iterator keySet = x._designExporters.keySet().iterator();
		while (keySet.hasNext()) {
			Object key = keySet.next ();
			this._designExporters.put(key, x._designExporters.get(key));
		}
	}

	protected String generateTypeID(AcmeType typeACME) {
		String id = typeACME.getTypeName() + "::" + nextId++;
		tidMap.put(typeACME, id);
		return id;
	}

	protected int nextId = 0;

}
