package acme.translators;
import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import acmeLib.core.container.AcmeEnumeration;
import acmeLib.elements.AcmeAttachment;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeConnector;
import acmeLib.elements.AcmeSystem;
import acmeLib.properties.AcmeProperty;


/**
 * Class: XMLImporter <br>
 * Purpose: Defines common routines that may be used in specific XML Importers <br>
 * Revision History:<br>
 *   Date   Person      Change<br>
 * 6/15/01  BRS         Created initial version
 * 
 * @author Bradley Schmerl
 **/
public class XMLImporter {

    protected String PROPERTY_TAG;


    // For use in utility functions to return a list of DOM nodes
    protected static class DOMNodeList implements NodeList {
	private Vector elements;
	
	public DOMNodeList () {
	    elements = new Vector ();
	}

	public int getLength () {
	    return elements.size ();
	}

	public org.w3c.dom.Node item (int index) {
	    return (org.w3c.dom.Node )elements.elementAt (index);
	}

	public void addElement (org.w3c.dom.Node n) {
	    elements.addElement (n);
	}
    }


    // 
    // private utility functions
    //

    /*
     * Finds the first child of the parent that is an element
     * @param parent Element 
    */
    protected Element findFirstElement (Element parent) {
	Element el = null;
	NodeList children = parent.getChildNodes ();
	boolean found = false;
	int i = 0;
	while (i < children.getLength () && !found) {
	    org.w3c.dom.Node candidate = children.item (i);
	    if (candidate.getNodeType () == org.w3c.dom.Node.ELEMENT_NODE) {
		found = true;
		el = (Element )candidate;
	    }
	    i++;
	}
	return el;
    }

    /**
     * Returns a list of all the children node of the parent that
     * are Elements with the tag tagName
     * @param parent org.w3c.dom.Node
     * @param tagName String
     **/
    public static NodeList getElementsAtThisLevel (org.w3c.dom.Node parent, String tagName) {
	DOMNodeList elements = new DOMNodeList ();
	NodeList children = parent.getChildNodes ();
	
	for (int i=0; i < children.getLength (); i++) {
	    org.w3c.dom.Node candidate = children.item (i);
	    if (candidate.getNodeType () == org.w3c.dom.Node.ELEMENT_NODE &&
		candidate.getNodeName ().equals (tagName)) {
		elements.addElement (candidate);
	    }
	}

	return elements;
    }


    /**
     * Translates the contents of one system to another
     * @param from AcmeSystem
     * @param to AcmeSystem
     **/
    protected void transferSystem (AcmeSystem from, AcmeSystem to) 
	throws acmeLib.core.exceptions.AcmeIllegalNameException 
    {
	AcmeEnumeration properties = from.getProperties ();
	while (properties.moreItems ()) {
	    AcmeProperty property = (AcmeProperty )properties.getNextItem ();
	    to.addProperty (property);
	}

	AcmeEnumeration components = from.getComponents ();
	while (components.moreItems ()) {
	    AcmeComponent comp = (AcmeComponent )components.getNextItem ();
	    to.addComponent (comp);
	}

	AcmeEnumeration connectors = from.getConnectors ();
	while (connectors.moreItems ()) {
	    AcmeConnector conn = (AcmeConnector )connectors.getNextItem ();
	    to.addConnector (conn);
	}

	AcmeEnumeration attachments = from.getAttachments ();
	while (attachments.moreItems ()) {
	    AcmeAttachment att = (AcmeAttachment )attachments.getNextItem ();
	    to.addAttachment (att);
	}

    }


}
