package acmeLib;

import java.io.*;
import java.util.StringTokenizer;

import acmeLib.armaniParser.ArmaniParser;
import acmeLib.elements.AcmeElement;

public class Debug {
	public static PrintStream debugFile = null;

	public static void initDebug(String filename) {
		try {
			debugFile = new PrintStream(new FileOutputStream(filename));
		} catch (Exception e) {
		}
	}

	public static void out(String msg) {
		if (debugFile != null)
			debugFile.println(msg);
	}

	private static int trace_indent = 0;

	public static final void trace_call (Object o) {
		if (! ArmaniParser.isTracingEnabled()) {
			return;
		}
		String indent = computeIndent();
		String str = ">:  " + o.toString();
		StringTokenizer tokens = new StringTokenizer(str, "\n");
		while (tokens.hasMoreTokens()) {
			System.out.print(indent);
			System.out.println(tokens.nextToken());
		}
		trace_indent += 2;
	}

	public static final void trace (Object o) {
		if (! ArmaniParser.isTracingEnabled()) {
			return;
		}
		String indent = computeIndent();
		String str = "**  ";
		if (o instanceof AcmeElement) {
			str += ((AcmeElement) o).getName();
		} else {
			str += o.toString();
		}
		StringTokenizer tokens = new StringTokenizer(str, "\n");
		while (tokens.hasMoreTokens()) {
			System.out.print(indent);
			System.out.println(tokens.nextToken());
		}
	}

	public static final void trace_return(Object o, boolean val) {
		if (! ArmaniParser.isTracingEnabled()) {
			return;
		}
		trace_indent -= 2;
		String indent = computeIndent();
		System.out.println(indent + "<:  " + val);
	}

	/**
	 * @return Space indentation of the trace_indent size.
	 */
	private static String computeIndent() {
		StringBuffer indentBuf = new StringBuffer();
		for (int i = 0; i < trace_indent; i++) { 
			indentBuf.append(" ");
		}
		String indent = indentBuf.toString();
		return indent;
	}

	public static void close() {
		if (debugFile != null)
			debugFile.close();
	}
}