
package acmeLib.core.container;

// import java.awt.*;
import java.io.IOException;
import acmeLib.*;


/** 
 *  Class:   ContainerTest.java
<BR>
 *  Purpose: Test mechanism for container classes.
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *     4/14/97  RTM       Created file.
 <BR><BR>
 *
 *  @author Bob Monroe<BR>
 */


public class ContainerTest {

    public static void main(String args[]) {
        System.out.println("testing containers");
        System.out.println("");

        // test the vector type.
        System.out.println("*************************  AcmeVector test *****************");
        AcmeVector v = new AcmeVector();
        testVector(v);
        System.out.println("Final configuration of AcmeVector v = " +
                            "\n" + v.toString());
        System.out.println("");

        // test the Map type
        System.out.println("*************************  AcmeMap test *****************");
        AcmeMap map = new AcmeMap();
        testMap(map);

        // test the Enumeration type
        System.out.println("******************** AcmeEnumeration test *****************");
        testEnumerations();

        System.out.println("");
        System.out.println("Show's over folks, please return to your previous activities...");
        System.out.println("(press Enter to exit)");
        try {
            System.in.read();
        } catch (IOException e) {
            return;
        }
    };

  /**
   * This method tests AcmeVectors
   * @param v the AcmeVector to be tested
   */
    public static void testVector(AcmeVector v) {
        // create five AcmeObjects and add them to the vector
        for (int i = 1; i <= 5; i++) {
            v.addItem(new AcmeObject());
        };

        // replace item 3 with a new AcmeObject
        v.setItemAt(new AcmeObject(), 3);
        System.out.println("V = " + v.toString());

        // remove some of the elements one by one.
        for (int i = 1; i <= 3; i++) {
            v.removeItemAt(0);
            System.out.println("");
            System.out.println("Size = " + v.getSize() +
                               "\n" + "Removing element at index 0");
            System.out.println("V = " + v.toString());
            System.out.println("");
        };

        // add another one and check whether "contains" finds it.
        AcmeObject d = new AcmeObject();
        v.addItem(d);
        System.out.println("");
        System.out.println("Added a design object " + d.toString() +
                           " Vector v says v.contains(d) = " + v.contains(d));
        System.out.println("");


        // create a new vector and copy the items into it.
        AcmeVector newVector = new AcmeVector();
        v.copyItems(newVector);
        System.out.println("");
        System.out.println("The new vector = " + newVector.toString());
        System.out.println("");

        // now remove elment d
        v.removeItem(d);
        System.out.println("");
        System.out.println("Removed design object d:" + d.toString() +
                           " Vector v says v.contains(d) = " + v.contains(d));
        System.out.println("Vector v = " + v.toString());
        System.out.println("");

        // test enumerations
        AcmeEnumeration e = new AcmeEnumeration();
        v.enumerate(e,null);
        System.out.println("Enumeration e of Vector v = ");
        while(e.moreItems()) {
            System.out.println("\t" + e.getNextItem());
        }
        System.out.println("");

    };

  /**
   * This method tests AcmeMaps
   * @param map the AcmeMap to be tested
   */
    public static void testMap(AcmeMap map) {

        // fill the map up with some instances.
        String[] strs = {"one", "two", "three", "four", "five"};
        for (int i = 0; i < 5; i++)
            map.addItem(strs[i], new AcmeObject());

        System.out.println("The AcmeMap's structure is: \n" + map.toString());

        // look an instance up
        System.out.println("Looking up \"three\" ...");
        if (map.lookupItem("three") != null)
            System.out.println("\tFound it, \"three\" = " + map.lookupItem("three").toString());
        else
            System.out.println("\tsorry, couldn't find it.");

        // test an enumeration
        AcmeEnumeration e = new AcmeEnumeration();
        map.enumerate(e,null);
        System.out.println("Enumeration e of AcmeMap map = ");
        while(e.moreItems()) {
            System.out.println("\t" + e.getNextItem().toString());
        }
        System.out.println("");

    };

  /**
   * This method tests AcmeEnumerations
   */
    public static void testEnumerations() {
        // test the expansion capabilities.
        AcmeEnumeration e = new AcmeEnumeration(2);

        for (int i = 0; i < 10; i++) {
            if (i % 2 == 0)
                e.insertItem(new AcmeObject());
            else
                e.insertItem(new AcmeObject());
        };

        System.out.println("Enumeration e now = \n" + e.toString());
    };
};


