/*
 * Created on Jul 15, 2003
 *
 */
package acmeLib.core.extensions;

import acmeLib.elements.AcmeElement;

/**
 * @author Bradley Schmerl
 * Created Jul 15, 2003
 * Interface to add extensions to elements. Extensions may be persistent, 
 * in which case there may be a mapping from the standard Acme to the particular
 * extension and back into the Acme
 */
public interface IElementExtension {
	
	/**
	 * 
	 * @return whether it is intended that this extension be saved with the Acme (use injectIntoElement to make it part of standard Acme)
	 */
	public boolean isPersistent ();
	
	/**
	 * Injects this extension into an AcmeElement. This should change the AcmeElement
	 * so that the extension can be exported 
	 * @param element
	 */
	public void injectIntoElement (AcmeElement element);
	
	/**
	 * Extracts an extension from an AcmeElement. This may cause changes to the
	 * element.
	 * @param element The element to extract extension information from
	 * @return The extension object
	 */
	public void extractFromElement (AcmeElement element);

	public boolean isInjected (AcmeElement element);
}
