
package acmeLib.core.globals;


/** 
 *  Class:   CategoryT
<BR>
 *  Purpose: Class that holds constant values indicating the set of
<BR>
 *           valid ACME categories.
 <BR>
 *  Description: The CategoryT class replaces the enumerated type CategoryT
<BR>
 *           in the original C++ implementation of ACME-Lib.  It's purpose
<BR>
 *           is to declare a set of values that indicate which category
<BR>
 *           an AcmeObject is -- component, connector, port, role, system,
<BR>
 *           or unknown.
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    ?/?/97    DK      Created C++ version
<BR>
 *    4/7/97    RTM     Translated to Java
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */

final public class CategoryT {

    // instance categories
    public static final CategoryT ACME_UNKNOWN_CATEGORY = new CategoryT(0);
    public static final CategoryT ACME_ELEMENT = new CategoryT(1);
    public static final CategoryT ACME_SYSTEM = new CategoryT(2);
    public static final CategoryT ACME_COMPONENT = new CategoryT(3);
    public static final CategoryT ACME_CONNECTOR = new CategoryT(4);
    public static final CategoryT ACME_PORT = new CategoryT(5);
    public static final CategoryT ACME_ROLE = new CategoryT(6);

    public static final CategoryT ACME_PROPERTY = new CategoryT(7);
    public static final CategoryT ACME_REPRESENTATION = new CategoryT(8);

    public static final CategoryT ACME_FAMILY = new CategoryT(9);
    public static final CategoryT ACME_DESIGN = new CategoryT(10);

    // type categories
    public static final CategoryT ACME_COMPONENT_TYPE = new CategoryT(21);
    public static final CategoryT ACME_CONNECTOR_TYPE = new CategoryT(22);
    public static final CategoryT ACME_PORT_TYPE = new CategoryT(23);
    public static final CategoryT ACME_ROLE_TYPE = new CategoryT(24);

    public static final CategoryT ACME_PROPERTY_TYPE = new CategoryT(25);


  /**
   * This is the constructor for this class
   * @param state used to set the state of "this"
   */
    private CategoryT(int state) { this.state = (byte) state; }
    private byte state;

  /**
   * This method converts a CategoryT object to String
   * @return a String representation of the object's state
   */
    public String toString() {
        switch(state) {
            case 0:  return new String("ACME_UNKNOWN_CATEGORY");
            case 1:  return new String("ACME_ELEMENT");
            case 2:  return new String("ACME_SYSTEM");
            case 3:  return new String("ACME_COMPONENT");
            case 4:  return new String("ACME_CONNECTOR");
            case 5:  return new String("ACME_PORT");
            case 6:  return new String("ACME_ROLE");
            case 7:  return new String("ACME_PROPERTY");
            case 8:  return new String("ACME_REPRESENTATION");
            case 9:  return new String("ACME_FAMILY");
            case 10: return new String("ACME_DESIGN");
            case 21: return new String("ACME_COMPONENT_TYPE");
            case 22: return new String("ACME_CONNECTOR_TYPE");
            case 23: return new String("ACME_PORT_TYPE");
            case 24: return new String("ACME_ROLE_TYPE");
            case 25: return new String("ACME_PROPERTY_TYPE");
            default:
              return "ACME_UNKNOWN_CATEGORY";
        }
    };
}

