
package acmeLib.core.globals;

import acmeLib.*;
import acmeLib.properties.*;
import acmeLib.elements.*;


/**
 *  Class:   ConstantTypes
<BR>
 *  Purpose: Class that holds a set of constant type objects representing
<BR>
 *           the generic base type classes (int, float, string, component, etc.)
<BR><BR>
 *
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *   2/3/98    RTM      created initial class & file
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
final public class ConstantTypes {


    public static final AcmeUnresolvedType UNRESOLVED_TYPE =
                                     new AcmeUnresolvedType("unresolved-type");
     public static final AcmeUnresolvedType UNRESOLVABLE_TYPE =
                                     new AcmeUnresolvedType("unresolvable-type");

     public static final AcmeUnresolvedPropertyType UNRESOLVABLE_PROPERTY_TYPE =
                   new AcmeUnresolvedPropertyType("unresolvable-property-type");
     public static final AcmeUnresolvedElementType UNRESOLVABLE_ELEMENT_TYPE =
                   new AcmeUnresolvedElementType("unresolvable-element-type");

    public static final BooleanType BOOLEAN_TYPE =
                                    new BooleanType("boolean");
    public static final IntType INT_TYPE =
                                    new IntType("int");
    public static final FloatType FLOAT_TYPE =
                                    new FloatType("float");
    public static final StringType STRING_TYPE =
                                    new StringType("string");
    public static final AnyType UNSPECIFIED_TYPE =
                                    new AnyType("");
    public static final AnyType EXPLICIT_ANY_TYPE =
                                    new AnyType("any");
    public static final SetType ANY_SET_TYPE =
                                    new SetType("set{}",UNSPECIFIED_TYPE);
    public static final SequenceType ANY_SEQUENCE_TYPE =
                                    new SequenceType("sequence<>",UNSPECIFIED_TYPE);
    public static final RecordType ANY_RECORD_TYPE =
                                    new RecordType("record[]");
    public static final EnumType EMPTY_ENUM_TYPE =
                                    new EnumType("enum{}");


    public static final BaseElementParameterType ELEMENT_BASE_TYPE =
                                    new BaseElementParameterType("element");
    public static final AcmeMapType MAP_BASE_TYPE = new AcmeMapType("map");
    public static final AcmeFamily SYSTEM_BASE_TYPE =
                                    new AcmeFamily("system");
    public static final AcmeComponentType COMPONENT_BASE_TYPE =
                                    new AcmeComponentType("component");
    public static final AcmeGroupType GROUP_BASE_TYPE =
                                    new AcmeGroupType("group");
    public static final AcmeConnectorType CONNECTOR_BASE_TYPE =
                                    new AcmeConnectorType("connector");
    public static final AcmePortType PORT_BASE_TYPE =
                                    new AcmePortType("port");
    public static final AcmeRoleType ROLE_BASE_TYPE =
                                    new AcmeRoleType("role");

  /**
   * constant parameter types to support the passing of Type, Property,
   * and Representation objects as parameters and return values.
   */
    public static final AcmeTypeParameter TYPE_REFERENCE_TYPE =
                                    new AcmeTypeParameter("type");
//    public static final AcmePropertyType PROPERTY_REFERENCE_TYPE =
//                                    new AcmePropertyType("property");
    public static final PropertyReferenceType PROPERTY_REFERENCE_TYPE =
                                    new PropertyReferenceType("property");
    public static final AcmeFamily REPRESENTATION_REFERENCE_TYPE =
                                    SYSTEM_BASE_TYPE;
 
    /**
      * @param type the type to check
      * @return whether the type is a base type
    */
    public static boolean isBaseType (AcmeType type) {
    	return type == ELEMENT_BASE_TYPE || type == SYSTEM_BASE_TYPE || type == COMPONENT_BASE_TYPE || type == GROUP_BASE_TYPE || type == CONNECTOR_BASE_TYPE || type == PORT_BASE_TYPE || type == ROLE_BASE_TYPE;
    }
    
/*** RTM 12/1/98 -- changed these to export the "property" keyword as a type
                    rather than the "default-*" names.  Still not clear whether
                    this breaks anything else...  Also changed the PROPERTY_PARAMETER_TYPE
                    to the PROPERTY_REFERENCE_TYPE.  This can be changed back if
                    it presents a bunch of problems to calling clients...

    public static final AcmeTypeParameter TYPE_PARAMETER_TYPE =
                                    new AcmeTypeParameter("type");
    public static final AcmePropertyType PROPERTY_PARAMETER_TYPE =
                                    new AcmePropertyType("default-property-type");
    public static final AcmeUnresolvedType REPRESENTATION_PARAMETER_TYPE =
                                    new AcmeUnresolvedType("default-rep-type");
***/
}
