
package acmeLib.core.globals;

/* class ExpressionOperatorT
 <BR>

     The ExpressionOperatorT class is an enumeration of the operators that
     a design rule expression can use.

*/

/** 
 *  Class:   ExpressionOperatorT
 <BR>
 *  Purpose: Class that holds constant values indicating the operator of
 <BR>
 *           a design rule expression
 <BR>
 *  Description: The ExpressionOperatorT class is an enumeration of the operators that
 <BR>
 *           a design rule expression can use.
  <BR> <BR>
 *
 *  Revision History:
 <BR>
 *    Date:     Person    Change
 <BR>
 *     10/7/97    RTM     Created
 <BR> <BR>
 *
 *  @author Bob Monroe <BR>
 */
final public class ExpressionOperatorT {

 
    public static final ExpressionOperatorT NOT_OP =
                                        new ExpressionOperatorT(0);
    public static final ExpressionOperatorT EQUALS_OP =
                                        new ExpressionOperatorT(1);
    public static final ExpressionOperatorT NOT_EQUALS_OP =
                                        new ExpressionOperatorT(2);
    public static final ExpressionOperatorT LESS_OR_EQUAL_OP =
                                        new ExpressionOperatorT(3);
    public static final ExpressionOperatorT GREATER_OR_EQUAL_OP =
                                        new ExpressionOperatorT(4);
    public static final ExpressionOperatorT LESS_THAN_OP =
                                        new ExpressionOperatorT(5);
    public static final ExpressionOperatorT GREATER_THAN_OP =
                                        new ExpressionOperatorT(6);
    public static final ExpressionOperatorT OR_OP =
                                        new ExpressionOperatorT(7);
    public static final ExpressionOperatorT AND_OP =
                                        new ExpressionOperatorT(8);
    public static final ExpressionOperatorT IMPLIES_OP =
                                        new ExpressionOperatorT(9);
    public static final ExpressionOperatorT IFF_OP =
                                        new ExpressionOperatorT(10);

  
  // arithmetic operators
   
    public static final ExpressionOperatorT PLUS_OP =
                                        new ExpressionOperatorT(20);
    public static final ExpressionOperatorT MINUS_OP =
                                        new ExpressionOperatorT(21);
    public static final ExpressionOperatorT MULT_OP =
                                        new ExpressionOperatorT(22);
    public static final ExpressionOperatorT DIVIDE_OP =
                                        new ExpressionOperatorT(23);
    public static final ExpressionOperatorT MODULO_OP =
                                        new ExpressionOperatorT(24);
    public static final ExpressionOperatorT UNARY_MINUS_OP =
                                        new ExpressionOperatorT(25);

  
  // set operators
  // RTM note: not clear if these should be implemented as operators
  //          or as functions.  Because Armani doesn't currently
  //          support user-specified functions (only design rules)
  //          I'm including them as operators for the time being.
  //
    public static final ExpressionOperatorT SET_UNION_OP =
                                        new ExpressionOperatorT(30);
    public static final ExpressionOperatorT SET_INTERSECTION_OP =
                                        new ExpressionOperatorT(31);
    public static final ExpressionOperatorT SET_MEMBER_OP =
                                        new ExpressionOperatorT(32);
    public static final ExpressionOperatorT SET_DIFFERENCE_OP =
                                        new ExpressionOperatorT(33);

  //
  // no-op is used for primitive expressions (e.g. literals and references)
  //
    public static final ExpressionOperatorT NO_OP =
                                        new ExpressionOperatorT(40);

  /** 
   * This is the constructor for this class
   * @param state the value state will take
   */
    private ExpressionOperatorT(int state) { this.state = (byte) state; }
    private byte state;

  /** 
   * This method returns the String representation of the opereator
   * @return the String representation of the opereator
   */
    public String toString() {
        switch(state) {
            case 0:   return new String("!");
            case 1:   return new String("==");
            case 2:   return new String("!=");
            case 3:   return new String("<=");
            case 4:   return new String(">=");
            case 5:   return new String("<");
            case 6:   return new String(">");
            case 7:   return new String("OR");
            case 8:   return new String("AND");
            case 9:   return new String("->");
            case 10:  return new String("<->");
            case 20:  return new String("+");
            case 21:  return new String("-");
            case 22:  return new String("*");
            case 23:  return new String("/");
            case 24:  return new String("%");
            case 25:  return new String("-");
            case 30:  return new String("union");
            case 31:  return new String("intersection");
            case 32:  return new String("member");
            case 33:  return new String("set_difference");
            case 40:  return new String("NO_OP");
            default:
              return "NO_OP";
        }
    };

 /* old versions were text-based.  converted to work for exporting.
    public String toString() {
        switch(state) {
            case 0:   return new String("NOT_OP");
            case 1:   return new String("EQUALS_OP");
            case 2:   return new String("NOT_EQUALS_OP");
            case 3:   return new String("LESS_OR_EQUAL_OP");
            case 4:   return new String("GREATER_OR_EQUAL_OP");
            case 5:   return new String("LESS_THAN_OP");
            case 6:   return new String("GREATER_THAN_OP");
            case 7:   return new String("OR_OP");
            case 8:   return new String("AND_OP");
            case 9:   return new String("IMPLIES_OP");
            case 10:  return new String("IFF_OP");
            case 20:  return new String("PLUS_OP");
            case 21:  return new String("MINUS_OP");
            case 22:  return new String("MULT_OP");
            case 23:  return new String("DIVIDE_OP");
            case 24:  return new String("MODULO_OP");
            case 25:  return new String("UNARY_MINUS_OP");
            case 30:  return new String("SET_UNION_OP");
            case 31:  return new String("SET_INTERSECTION_OP");
            case 32:  return new String("SET_MEMBER_OP");
            case 33:  return new String("SET_DIFFERENCE_OP");
            case 40:  return new String("NO_OP");
            default:
              return "NO_OP";
        }
    };
  */
}
