
package acmeLib.core.globals;

/* class InstantiationMethodT

     The InstantiationMethodT class replaces the enumerated type
     InstantiationMethodT in the original C++ implementation of
     ACME-Lib.  It's purpose is to declare a set of values that
     indicate how a specific AcmeElement was instantiated.
*/

/** 
 *  Class:   InstantiationMethodT
  <BR>
 *  Purpose: Class that holds constant values indicating how an Acme
<BR>
 *           element was instantiated
<BR>
 *  Description:  The InstantiationMethodT class replaces the enumerated type
<BR>
 *           InstantiationMethodT in the original C++ implementation of
<BR>
 *           ACME-Lib.  It's purpose is to declare a set of values that
<BR>
 *           indicate how a specific AcmeElement was instantiated.
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *     4/24/97  RTM      created in Java
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
final public class InstantiationMethodT {

    public static final InstantiationMethodT CONCRETE =
                                        new InstantiationMethodT(0);
    public static final InstantiationMethodT NEW_TYPE =
                                        new InstantiationMethodT(1);
    public static final InstantiationMethodT TEMPLATE =
                                        new InstantiationMethodT(2);
    public static final InstantiationMethodT UNKNOWN =
                                        new InstantiationMethodT(3);


  /**
   * This is the constructor for this class
   * @param state the value state will take
   */
    private InstantiationMethodT(int state) {
        this.state = (byte) state;
    };

    private byte state;

  /** 
   * This method returns a String representation of the object's state
   * @return a String representation of the object's state
   */
    public String toString() {
        switch(state) {
            case 0:  return new String("CONCRETE");
            case 1:  return new String("NEW_TYPE");
            case 2:  return new String("TEMPLATE");
            case 3:  return new String("UNKNOWN");
            default:
              return "UNKNOWN_INSTANTIATION_METHOD";
        }
    };

}

