// import java.awt.*;
package acmeLib.core.io;

import java.io.IOException;
import java.util.Vector;

public class IOTest {

  /**
   * This method tests the AcmeOutputStream class
   * @exception java.io.IOException when error occurs when writing
   */
    public static void main(String args[]) throws IOException {
        System.out.println("IO Classes test");
        System.out.println("");

        // test the AcmeOutputStream class
        AcmeOutputStream s = new AcmeOutputStream();
        s.write("System simple = ");
        exportSystem(s);
        s.writeTo(System.out);

        System.out.println("");
        System.out.println("\n\nTesting Vectors:");

        Vector v = new Vector();
        v.addElement("first");
        v.addElement("second");
        v.addElement("third");
        System.out.println("Vector v = " + v.toString());
        System.out.println("Vector v's size is: " + v.size());
        System.out.println("v[0] = " + v.elementAt(0));
        System.out.println("v[1] = " + v.elementAt(1));
        System.out.println("v[2] = " + v.elementAt(2));

        System.out.println("(press Enter to exit)");
        try {
            System.in.read();
        } catch (IOException e) {
            return;
        }
    };

  /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs when writing
   */
    public static void exportSystem(AcmeOutputStream s) throws IOException {
       	s.writeln ("{");

        s.indent();
    	exportComponent (s, "server");
//    	exportComponent (s, "client");
//    	exportConnector (s, "rpc");

    	// export attachments test
    	s.writeln ("Attachments {");
    	s.indent();
        s.writeln("client.call to rpc.call;");
        s.writeln("server.receive to rpc.receive;");
    	s.unIndent();
    	s.writeln ("};");
    	s.unIndent();

        // add a property test

        s.unIndent();
    	s.writeln ("}; /* end system */");
    }

  /**
   * @param s AcmeOutputStream
   * @param compName the name of the component
   * @exception java.io.IOException when error occurs when writing
   */
    public static void exportComponent(AcmeOutputStream s, String compName)
      throws IOException {
        s.writeln("Component " + compName + " : AcmeTypeT = {");
        s.indent();
        exportFullPort(s, "call");
        exportSmallPort(s, "smallPort");
        s.write("Property p : someType = ");
        s.writeln("someValue;");
        s.writeln("Property p' : someOtherType = someOtherValue;");
        s.unIndent();
        s.writeln("};");
    };

  /**
   * @param s AcmeOutputStream
   * @param portName the name of the port
   * @exception java.io.IOException when error occurs when writing
   */
    public static void exportFullPort(AcmeOutputStream s, String portName)
      throws IOException {
        s.writeln("Port " + portName + " : AcmePortTypeT = {");
        s.indent();
        s.write("Property port-p : someType = ");
        s.write("someValue;");
        s.writeln("");
        s.writeln("Property port-p' : someOtherType = someOtherValue;");
        s.unIndent();
        s.writeln("};");
    };

  /**
   * @param s AcmeOutputStream
   * @param  portName the name of the port
   * @exception java.io.IOException when error occurs when writing
   */
    public static void exportSmallPort(AcmeOutputStream s, String portName)
      throws IOException {
        s.writeln("Port tiny;");
    };
};

