package acmeLib.designRules;

import acmeLib.AcmeType;

/**
 * Class: ExpressionCastException<br>
 * Purpose: Thrown when an expression is inappropriately cast to a<br>
 *          "non-optimal" type.  For example, attempting to evaluate<br>
 *          the string "blue" as a float.<br>
 *<br>
 * Revision History:<br>
 *    Date:     Person    Change<br>
 *    10/27/97    RTM     Created<br>
 *    2/2/98      RTM     Types now stored as AcmePropertyTypes<br>
 *<br>
 * @author Bob Monroe
 */
public class ExpressionCastException extends ExpressionTypeException {

  /** 
   * This is the constructor for this class
   */
    public ExpressionCastException() {
        super("Illegal typecast in design rule expression");
    };

  /** 
   * This is the constructor for this class
   * @param errMsg The error message
   */ 
    public ExpressionCastException(String errMsg) {
        super(errMsg);
    };

  /** 
   * This is the constructor for this class
   * @param problemScope The problemed scope
   * @param problemNode The problemed node
   * @param targetType The target AcmeType
   * @param nativeType The native AcmeType
   */
    public ExpressionCastException(Scope problemScope,
                                   ExpressionNode problemNode,
                                   AcmeType targetType,
                                   AcmeType nativeType)
    {
        super(problemScope, problemNode, targetType, nativeType,
              "Illegal typecast in design rule expression");
    };

  /** 
   * This is the constructor for this class
   * @param problemScope The problemed scope
   * @param problemNode The problemed node
   * @param targetType The target AcmeType
   * @param nativeType The native AcmeType
   * @param errMsg The error message
   */
    public ExpressionCastException(Scope problemScope,
                                   ExpressionNode problemNode,
                                   AcmeType targetType,
                                   AcmeType nativeType,
                                   String errMsg)
    {
        super(problemScope, problemNode, targetType, nativeType, errMsg);
    };

};
