package acmeLib.designRules;

import acmeLib.AcmeType;

/**
 * Class: ExpressionTypeException<br>
 * Purpose: Base class for all exceptions thrown during the evaluation<br>
 *          of an Armani design rule.<br>
 *<br>
 * Revision History:<br>
 *    Date:     Person    Change<br>
 *    10/27/97    RTM     Created<br>
 *    2/2/98      RTM     Types now stored as AcmePropertyTypes<br>
 *<br>
 * @author Bob Monroe
 */
public class ExpressionTypeException extends DesignRuleEvalException {

    protected Scope nodeScope = null;
    protected ExpressionNode node = null;
    protected AcmeType targetType = null;
    protected AcmeType nativeType = null;

  /**
   * This is the constructor of this class
   */
    public ExpressionTypeException() {
        super("Type mismatch in design rule expression");
    };

  /** 
   * This is the constructor of this class
   * @param errMsg The error message
   */
    public ExpressionTypeException(String errMsg) {
        super(errMsg);
    };

  /**
   * This is the constructor of this class
   * @param problemScope The problemed scope
   * @param problemNode The problemed node
   * @param targetType The target type
   * @param nativeType The native type
   */

    public ExpressionTypeException(Scope problemScope,
                                   ExpressionNode problemNode,
                                   AcmeType targetType,
                                   AcmeType nativeType)
    {
        super("Illegal typecast in design rule expression");
        this.nodeScope = problemScope;
        this.node = problemNode;
        this.targetType = targetType;
        this.nativeType = nativeType;
    };

  /**
   * This is the constructor of this class
   * @param problemScope The problemed scope
   * @param problemNode The problemed node
   * @param targetType The target type
   * @param nativeType The native type
   * @param errMsg The error message
   */
    public ExpressionTypeException(Scope problemScope,
                                   ExpressionNode problemNode,
                                   AcmeType targetType,
                                   AcmeType nativeType,
                                   String errMsg)
    {
        super(errMsg);
        this.nodeScope = problemScope;
        this.node = problemNode;
        this.targetType = targetType;
        this.nativeType = nativeType;
    };


    // accessor and settor methods
  /**
   * Get the scope
   * @return The scope
   */
    public Scope getScope() { return this.nodeScope; }
  /**
   * Set the scope
   * @param newScope The new scope
   */
    public void setScope(Scope newScope) { this.nodeScope = newScope; }

  /**
   * Get the expression node
   * @return The expression node
   */
    public ExpressionNode getExpressionNode() { return this.node; }
  /**
   * Set the expression node
   * @param problemNode The problemed node
   */
    public void setExpressionNode(ExpressionNode problemNode) {
        this.node = problemNode;
    };

  /**
   * Get the target type
   * @return The target type
   */
    public AcmeType getTargetType() { return this.targetType; }
  /**
   * Set the target type
   * @param newTargetType The new target type
   */
    public void setTargetType(AcmeType newTargetType) {
        this.targetType = newTargetType;
    };

  /**
   * Get the native type
   * @return The native type
   */
    public AcmeType getNativeType() { return this.nativeType; }
  /**
   * Set the native type
   * @param newNativeType The new native type
   */
    public void setNativeType(AcmeType newNativeType) {
        this.nativeType = newNativeType;
    };

};
