package acmeLib.designRules;


/**
 * Class: ParameterTypeException<br>
 * Purpose: exception thrown when an actual parameter passed to a function<br>
 *          or design rule for evaluation doesn't satisfy the type<br>
 *          requirements of the formal parameter it's replacing.<br>
 *<br>
 * Revision History:<br>
 *    Date:     Person    Change<br>
 *    10/28/97    RTM     Created<br>
 *<br>
 * @author Bob Monroe
 */
public class ParameterTypeException extends DesignRuleEvalException {

    protected ExpressionNode expressionNode = null;
    protected DesignRuleParameter actualParam = null;
    protected DesignRuleParameter formalParam = null;

  /**
   * This is the constructor for this class
   */
    public ParameterTypeException() {
        super("Type mismatch in design rule parameters");
    };

  /**
   * This is the constructor for this class
   * @param errMsg The error message
   */
    public ParameterTypeException(String errMsg) {
        super(errMsg);
    };

  /**
   * This is the constructor for this class
   * @param expression The expression node
   * @param actualParam The actual parameter
   * @param formalParam The formal parameter
   */  
    public ParameterTypeException(ExpressionNode expression,
                                  DesignRuleParameter actualParam,
                                  DesignRuleParameter formalParam)
    {
        super("Type mismatch in design rule parameters");
        this.expressionNode = expression;
        this.actualParam = actualParam;
        this.formalParam = formalParam;
    };


  /**
   * This is the constructor for this class
   * @param expression The expression node
   * @param actualParam The actual parameter
   * @param formalParam The formal parameter
   * @param errMsg The error message
   */  
    public ParameterTypeException(ExpressionNode expression,
                                  DesignRuleParameter actualParam,
                                  DesignRuleParameter formalParam,
                                  String errMsg)
    {
        super(errMsg);
        this.expressionNode = expression;
        this.actualParam = actualParam;
        this.formalParam = formalParam;
    };


    // accessor and settor methods
  /**
   * Get the expression node
   * @return The expression node
   */
    public ExpressionNode getExpressionNode() { return this.expressionNode; }
  /**
   * Set the expression node
   * @param problemNode The problem node
   */
    public void setExpressionNode(ExpressionNode problemNode) {
        this.expressionNode = problemNode;
    };

  /**
   * Get the actual parameters
   * @return The actual parameters
   */
    public DesignRuleParameter getActualParam() { return this.actualParam; };
  /**
   * Set the actual parameters
   * @param param The actual parameters
   */
    public void setActualParam(DesignRuleParameter param) {
        this.actualParam = param;
    };

  /**
   * Get the formal parameters
   * @return The formal parameters
   */
    public DesignRuleParameter getFormalParam() { return this.actualParam; };
  /**
   * Set the formal parameters
   * @param param The formal parameters
   */
    public void setFormalParam(DesignRuleParameter param) {
        this.actualParam = param;
    };

};
