package acmeLib.designRules;

/**
 * Class: ReferenceNotFoundException<br>
 * Purpose: Exception thrown when a reference is unable to find the object<br>
 *          it is supposed to refer to.<br>
 *<br>
 *<br>
 * Revision History:<br>
 *    Date:     Person    Change<br>
 *    10/27/97    RTM     Created<br>
 *<br>
 * @author Bob Monroe
 */ 
public class ReferenceNotFoundException extends DesignRuleEvalException {

    protected Scope referenceScope = null;
    protected ReferenceNode referenceNode = null;

  /** 
   * This is the constructor for this class
   */
    public ReferenceNotFoundException() {
        super("Unable to dereference requested reference");
    };

  /** 
   * This is the constructor for this class
   * @param refScope The reference scope
   * @param problemNode The problemed node
   */
    public ReferenceNotFoundException(Scope refScope,
                                      ReferenceNode problemNode)
    {
        super("Unable to dereference requested reference");
        this.referenceScope = refScope;
        this.referenceNode = problemNode;
    };

  /** 
   * This is the constructor for this class
   * @param errMsg The error message
   */
    public ReferenceNotFoundException(String errMsg) {
        super(errMsg);
    };

  /** 
   * This is the constructor for this class
   * @param refScope The reference scope
   * @param problemNode The problemed node
   * @param errMsg The error message
   */
    public ReferenceNotFoundException(Scope refScope,
                                      ReferenceNode problemNode,
                                      String errMsg)
    {
        super(errMsg);
        this.referenceScope = refScope;
        this.referenceNode = problemNode;
    };

    // accessor and settor methods
  /**
   * Get the reference scope
   * @return The scope
   */
    public Scope getReferenceScope() { return this.referenceScope; }
  /**
   * Set the reference scope
   * @param newScope The new scope
   */ 
    public void setReferenceScope(Scope newScope) {
        this.referenceScope = newScope;
    };

  /**
   * Get the reference node
   * @return The reference node
   */
    public ReferenceNode getReferenceNode() { return this.referenceNode; }
  /**
   * Set the reference node
   * @param problemNode The problemed node
   */
    public void setReferenceNode(ReferenceNode problemNode) {
        this.referenceNode = problemNode;
    };

};

