package acmeLib.designRules;

/**
 * Interface: Scope<br>
 * Purpose: A class that implements the Scope interface provides<br>
 *          methods to look up the values of identifiers, abstract<br>
 *          design rules, and eventually functions by name within<br>
 *          a well defined scope.<br>
 *<br>
 * Revision History:<br>
 *    Date:     Person    Change<br>
 *    10/20/97    RTM     Created initial interface definition<br>
 *<br>
 * @author Bob Monroe
 */
public interface Scope {

    //
    // public members
    //

  /**
   * looks up "name" in this scope.  If there is no object referred
   * to by "name" within this scope, it returns null.
   * The boolean parameter "bubbleUp" specifies whether the method
   * should continue looking recursively in its parent's scope
   * if it does not find "name" in its own scope.  For example,
   * a search for a typename should bubble all the way up to families
   * and then the global design space, but the search for a property
   * name from within a type invariant should only look for properties
   * within the type's scope.
   * @param name The name
   * @bubbleUp pass the request up
   * @return The object
   */
    public Object lookupName(String name, boolean bubbleUp);
}