package acmeLib.designRules;

import acmeLib.AcmeObject;
import acmeLib.AcmeType;
import acmeLib.core.container.AcmeEnumeration;

/**
 * Class: SetExpressionNode<br>
 * Purpose: Interface that set expression nodes parsed in an Armani constraint<br>
 *          specification must implement.  The storage of an<br>
 *          actual set and the operations on it are added and implemented<br>
 *          by the classes that implement SetExpressionNode.<br>
 *<br>
 * Revision History:<br>
 *    Date:     Person    Change<br>
 *    1/9/98     RTM       Created<br>
 *    1/12/98    RTM       Changed from an abstract base class to an interface.<br>
 *<br>
 * @author Bob Monroe
 */
public interface SetExpressionNode
{

    /** accessors/settors for set expression scope. */
  /**
   * Get the set scope
   * @return The set scope
   */
    public Scope getSetScope();
  /**
   * Set the set scope 
   * @param newScope The new scope
   */
    public void setSetScope(Scope newScope);

    /** accessors/settors for set type. */
  /**
   * Get the set type 
   * @return The AcmeType
   */
    public AcmeType getSetType();
  /**
   * Set the set type 
   * @param newType The new AcmeType
   */
    public void setSetType(AcmeType newType);

  /** 
   * returns the size of the current set 
   * @return The size
   */
    public abstract int size();


  /** 
   * refreshSet() recomputes the value of the set.  This is implemented
   * by SetExpressionNode subclasses as appropriate for the types of
   * set expressions that they represent. 
   */
    public abstract void refreshSet();

  /** 
   * contains(Object elt) returns true iff elt is a member of this set 
   * @param elt The element
   * @return true if element is in the set
   */
    public abstract boolean contains(AcmeObject elt);

  /** 
   * enumerate() returns an enumeration of all of the elements of the set 
   * @return The enumeration of all elements in set
   */
    public abstract AcmeEnumeration enumerate();

  /** 
   * return the string representing this set (for exporting/unparsing) 
   * @return The output string
   */
    public String toString();
  /**
   * return the string representing this set
   * @param prefix The prefix
   * @return The output string
   */
    public String toString(String prefix);
}
