// Class: TempSet
// Purpose: Works as a helper class that stores a temporary set of
//          elements that has been either constructed or created by
//          application of some set-typed functions.  A TempSet can
//          also be an object generated to be the referentObject of
//          a set reference node.  In general, this class will be
//          created by a SetExpressionNode to store and manipulate
//          sets that are derived, generated, or otherwise created.
//
// Revision History:
//    Date:      Person    Change
//    2/25/98     RTM       Created

package acmeLib.designRules;

import acmeLib.AcmeObject;
import acmeLib.AcmeType;
import acmeLib.AcmeUnresolvedType;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.properties.SetType;


public class TempSet
{

    //
    // protected members
    //

    // stores the elements of this set
    protected AcmeVector setVector;

    // stores the type of entities stored within this set.
    protected AcmeType setType;


    //
    // public members
    //

    /** constructors */
    public TempSet() {
        this.setVector = new AcmeVector(2,1);
        this.setSetType(new SetType("tempSet",
                     new AcmeUnresolvedType("unresolved")));
    }

    public TempSet(AcmeType newSetType) {
        this.setVector = new AcmeVector(2,1);
        this.setSetType(newSetType);
    }


    /** accessors/settors for set type. */
    public AcmeType getSetType() { return this.setType; };
    public void setSetType(AcmeType newType) { this.setType = newType; };


    /** accessors/settors for set vector. */
    public AcmeVector getSetVector() { return this.setVector; };
    public void setSetType(AcmeVector newVector) { this.setVector = newVector; };


    /** returns the size of the current set */
    public int size() { return setVector.getSize(); }


    /** remove all of the elements from this set */
    public void emptySet() { this.setVector.removeAllItems(); };


    /** contains(Object elt) returns true iff elt is a member of this set */
    public boolean contains(AcmeObject elt) {
        return this.setVector.contains(elt);
    };


    /** addElement(Object elt) adds elt to this set, using set addition
        semantics.  That is, if elt is already a member of this set then
        elt will not be re-added. */
    public void addElement(AcmeObject elt) {
        // RTM 1/12/98 -- this doesn't do an "equals" comparison, only
        //                a reference comparison (that is, is this exact
        //                object already stored in the vector, not does this
        //                vector contain an equivalent object.)
        if (!(setVector.contains(elt)))
            setVector.addItem(elt);
        // else quietly do nothing, as elt is already in the set.
    };


    /** removeElement(Object elt) removes elt from this set. If elt is
        not a member of this set then the set will not be changed. */
    public void removeElement(AcmeObject elt) {
        // RTM 1/12/98 -- this doesn't do an "equals" comparison, only
        //                a reference comparison (that is, is this exact
        //                object already stored in the vector, not does this
        //                vector contain an equivalent object.)
        setVector.removeItem(elt);
    };


    /** enumerate() returns an enumeration of all of the elements of the set */
    public AcmeEnumeration enumerate() {
        return this.setVector.enumerate();
    }

}
