/*
 * Created on Jul 14, 2004
 *
 */
package acmeLib.designRules.feedback;

import java.util.Vector;
import acmeLib.designRules.DesignAnalysisCallNode;
import acmeLib.designRules.DesignRuleParameter;
import acmeLib.designRules.IdentifierNode;
import acmeLib.designRules.VolatileVariable;
import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;
import acmeLib.properties.AcmeProperty;


/**
 * @author Bradley Schmerl
 * Created Jul 14, 2004
 */
public class DesignAnalysisCallEvaluationFeedback extends EvaluationFeedback {

	private final DesignAnalysisCallNode	_node;
	private final Vector	_actualParams;

	/**
	 * @param result
	 */
	public DesignAnalysisCallEvaluationFeedback(DesignAnalysisCallNode node, Vector actualParams, Object result) {
		super (result);
		_node = node;
		_actualParams = new Vector (actualParams.size());
		for (int i = 0; i < actualParams.size(); i++) {
			Object param = actualParams.get(i);
			if (param instanceof IdentifierNode) {
				IdentifierNode n = (IdentifierNode )param;
				param = node.getReferenceScope().lookupName (n.getIdentifier(), true);
				if (param instanceof VolatileVariable) param = ((VolatileVariable )param).referentObject;
			}
			_actualParams.add(param);
		}
	}

	/**
	 * @return
	 * @see acmeLib.designRules.feedback.EvaluationFeedback#getVisitedString()
	 **/
	public String getVisitedString() {
		String call = _node.toString ();
		StringBuffer param = new StringBuffer ("");
		Vector formalParams = _node.getReferentDesignAnalysis().getFormalParams();
		if (formalParams.size () > 0) param.append("[");
		for (int i = 0; i < formalParams.size(); i++) {
			param.append (((DesignRuleParameter )formalParams.get(i)).getName());
			param.append (" = ");
			Object o = _actualParams.get (i);
			if (o instanceof AcmeElementType) {
				AcmeElementType type = (AcmeElementType )o;
				param.append (type.getTypeName ());
			} 
			else if (o instanceof AcmeElement) {
				AcmeElement element = (AcmeElement )o;
				param.append (element.getQualifiedName());
			}
			else if (o instanceof AcmeProperty) {
				AcmeProperty prop = (AcmeProperty )o;
				param.append (prop.getQualifiedName());
			}
			else {
				param.append (o.toString ());
			}
			if (i < formalParams.size ()) param.append (",");
		}
		if (formalParams.size () > 0) param.append ("]");
		return call + param.toString();
	}
}
