package acmeLib.designRules;

import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmePort;

/*
import acmeLib.core.io.*;
import acmeLib.core.globals.*;
import acmeLib.typeManagement.*;
import acmeLib.core.container.*;
import acmeLib.elements.*;
import acmeLib.errorManagement.*;
import acmeLib.properties.*;

import java.lang.reflect.*;
import java.util.Vector;
*/

/**
 * Class: testExternal<br>
 * Purpose: defines a number of external analyses that will be used to<br>
 *          test the design and implementation of the external analysis<br>
 *          capabilities.<br>
 *<br>
 * Revision History:<br>
 *    Date:     Person    Change<br>
 *    3/23/98    RTM      * Created original class<br>
 *<br>
 */
public class testExternal {

  /**
   * all external analyses need to be hooked in as public static methods.
   * @param x Expression node x
   * @param y Expression node y
   * @return The integer sum of x and y
   * @exception DesignRuleEvalException If there is an evaluation error
   */

    public static int addInts(ExpressionNode x, ExpressionNode y)
        throws DesignRuleEvalException
    {
        return x.evaluateAsInt() + y.evaluateAsInt();
    };



  /**
   * @param x Expression node x
   * @return true if x > 10
   * @exception DesignRuleEvalException If there is an evaluation error
   */
    public static boolean xGreaterThanTen(ExpressionNode x)
        throws DesignRuleEvalException
    {
        return x.evaluateAsInt() > 10;
    };



  /**
   * @param strNode String expression node
   * @return The doubled string
   * @exception DesignRuleEvalException If there is an evaluation error
   */
    public static String doubleString(ExpressionNode strNode)
        throws DesignRuleEvalException
    {
        return strNode.evaluateAsString() + strNode.evaluateAsString();
    };



  /** 
   * @param node The expression node
   * @return The AcmeComponent
   * @exception DesignRuleEvalException If there is an evaluation error
   */
    public static AcmeComponent parentComp(ExpressionNode node)
        throws DesignRuleEvalException
    {
        AcmePort port = null;
        if (node == null) {
            throw new DesignRuleEvalException("ExpressionNode passed as a Port "
                        + "to parentComp(p) is null");
        } else {
            if (node instanceof ReferenceNode) {
                port = (AcmePort) ((ReferenceNode) node).getReferentObject();
                if (port == null) {
                    ((ReferenceNode) node).refreshReferentObject();
                    if (port == null) {
                        // if it's still null then something's messed up...
                        throw new DesignRuleEvalException("Unable to resolve Expression "
                                    + "node passed to parentComp(p) as a Port object");
                    };
                };
            } else {
                throw new DesignRuleEvalException("ExpressionNode passed "
                            + "to parentComp(p) is not a valid ReferenceNode");
            };
        };

        return (AcmeComponent) port.getParent();
    };

};
