
package acmeLib.elements;

import acmeLib.*;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.designRules.Scope;


/** 
 *  Class:  AcmeElementType
<BR>
 *  Purpose: Standard interface for accessing type information
<BR>
 *           for element types.
<BR><BR>
 *
 *           Currently adds nothing to AcmeType except for the ability
<BR>
 *           to do run-time type identification so that we can determine
<BR>
 *           whether it is an element type or a property type
<BR><BR>
 * 
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    6/13/97   RTM     Created new, simpler, type interface.
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public interface AcmeElementType extends AcmeType, Scope {

    // methods for adding and removing supertypes to or from a type description,
    // and retrieving a type's set of supertypes,
  /**
   * This method adds a supertype to a type description
   * @param superType the AcmeElementType to be added to a type description
   */
    public void addSuperType(AcmeElementType superType);
  /**
   * This method removes a supertype from a type description
   * @param superType the AcmeElementType to be removed from the type description
   */
    public void removeSuperType(AcmeElementType superType);
  /**
   * This method returns the supertype of the type description
   * @return supertype of the type description
   */
    public AcmeVector getSuperTypes();

  /**
   * Thid method makes a copy of this element type object.
   * @return a copy of this element type object
   */
    public AcmeElementType copyType();

  /** 
   * This method checks whether this element type is internally type-correct.
   * @return whether this element type typeckecks
   */
    public boolean typecheck();
    
    public AcmeEnumeration getAvailableTypes ();
};
