/*
 * Created on Aug 20, 2003
 *
 */
package acmeLib.elements;

import java.io.IOException;

import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.core.exceptions.AcmeException;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.extensions.IElementExtension;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.designRules.DesignRule;
import acmeLib.designRules.Scope;
import acmeLib.properties.AcmeProperty;
import acmeLib.properties.AcmePropertyType;
import acmeLib.properties.AcmePropertyValue;
import acmeLib.representations.AcmeRepresentation;
import acmeLib.typeManagement.AcmeNamespace;
import acmeLib.typeManagement.IInheritable;

/**
 * @author Bradley Schmerl
 * Created Aug 20, 2003
 */
public class AcmeElementTypeRef extends AcmeElement implements AcmeElementType {

	protected AcmeElementType referentType;
	protected AcmeFamily referentFamily;

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#addDeclaredType(acmeLib.elements.AcmeElementType)
	 */
	public void addDeclaredType(AcmeElementType newType) {
		((AcmeElement) referentType).addDeclaredType(newType);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#addHeuristic(acmeLib.designRules.DesignRule)
	 */
	public void addHeuristic(DesignRule newHeuristic) {
		((AcmeElement) referentType).addHeuristic(newHeuristic);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#addInheritedSource(acmeLib.typeManagement.IInheritable)
	 */
	public void addInheritedSource(IInheritable source) {
		((AcmeElement) referentType).addInheritedSource(source);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#addInstantiatedType(acmeLib.elements.AcmeElementType)
	 */
	public void addInstantiatedType(AcmeElementType newType) {
		((AcmeElement) referentType).addInstantiatedType(newType);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#addInvariant(acmeLib.designRules.DesignRule)
	 */
	public void addInvariant(DesignRule newInvariant) {
		((AcmeElement) referentType).addInvariant(newInvariant);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#addMap(acmeLib.elements.AcmeMapInstance)
	 */
	public void addMap(AcmeMapInstance map) throws AcmeIllegalNameException {
		((AcmeElement) referentType).addMap(map);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#addProperty(acmeLib.properties.AcmeProperty)
	 */
	public void addProperty(AcmeProperty p) throws AcmeIllegalNameException {
		((AcmeElement) referentType).addProperty(p);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#addProperty(java.lang.String, acmeLib.properties.AcmePropertyType, acmeLib.properties.AcmePropertyValue)
	 */
	public void addProperty(String name, AcmePropertyType t, AcmePropertyValue val) throws AcmeIllegalNameException {
		((AcmeElement) referentType).addProperty(name, t, val);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#addRepresentation(acmeLib.representations.AcmeRepresentation)
	 */
	public void addRepresentation(AcmeRepresentation rep) throws AcmeIllegalNameException {
		((AcmeElement) referentType).addRepresentation(rep);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#childNameChanged(java.lang.String, acmeLib.AcmeObject)
	 */
	public void childNameChanged(String oldName, AcmeObject obj) throws AcmeIllegalNameException {
		((AcmeElement) referentType).childNameChanged(oldName, obj);
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#copy(acmeLib.AcmeObject)
	 */
	public void copy(AcmeObject newObject) {
		((AcmeElement) referentType).copy(newObject);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#copyAsPrototype(java.lang.String)
	 */
	public AcmeElementType copyAsPrototype(String newTypeName) {
		return ((AcmeElement) referentType).copyAsPrototype(newTypeName);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#copyMaps(acmeLib.elements.AcmeElement)
	 */
	public void copyMaps(AcmeElement destObj) {
		((AcmeElement) referentType).copyMaps(destObj);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#copyProperties(acmeLib.elements.AcmeElement)
	 */
	public void copyProperties(AcmeElement destObj) {
		((AcmeElement) referentType).copyProperties(destObj);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#declaresType(acmeLib.elements.AcmeElementType)
	 */
	public boolean declaresType(AcmeElementType type) {
		return ((AcmeElement) referentType).declaresType(type);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#doPostParsingProcessing()
	 */
	public void doPostParsingProcessing() {
		((AcmeElement) referentType).doPostParsingProcessing();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#edit()
	 */
	public void edit() {
		((AcmeElement) referentType).edit();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#exportAcme(acmeLib.core.io.AcmeOutputStream)
	 */
	public void exportAcme(AcmeOutputStream s) throws IOException, AcmeException {
		((AcmeElement) referentType).exportAcme(s);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#exportArmani(acmeLib.core.io.AcmeOutputStream)
	 */
	public void exportArmani(AcmeOutputStream s) throws IOException, AcmeException {
		((AcmeElement) referentType).exportArmani(s);
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#exportComment(acmeLib.core.io.AcmeOutputStream)
	 */
	public void exportComment(AcmeOutputStream s) throws IOException {
		((AcmeElement) referentType).exportComment(s);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getDeclaredTypes()
	 */
	public AcmeEnumeration getDeclaredTypes() {
		return ((AcmeElement) referentType).getDeclaredTypes();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getHeuristics(boolean)
	 */
	public AcmeEnumeration getHeuristics(boolean includeInherited) {
		return ((AcmeElement) referentType).getHeuristics(includeInherited);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getInheritedSources()
	 */
	public AcmeEnumeration getInheritedSources() {
		return ((AcmeElement) referentType).getInheritedSources();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getInstantiatedTypes()
	 */
	public AcmeEnumeration getInstantiatedTypes() {
		return ((AcmeElement) referentType).getInstantiatedTypes();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getInvariants(boolean)
	 */
	public AcmeEnumeration getInvariants(boolean includeInherited) {
		return ((AcmeElement) referentType).getInvariants(includeInherited);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getMap(java.lang.String)
	 */
	public AcmeMapInstance getMap(String name) {
		return ((AcmeElement) referentType).getMap(name);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getMaps()
	 */
	public AcmeEnumeration getMaps() {
		return ((AcmeElement) referentType).getMaps();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getName()
	 */
	public String getName() {
		return ((AcmeElement) referentType).getName();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getNamespace()
	 */
	public AcmeNamespace getNamespace() {
		return ((AcmeElement) referentType).getNamespace();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getNumberOfProperties()
	 */
	public int getNumberOfProperties() {
		return ((AcmeElement) referentType).getNumberOfProperties();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getNumberOfReps()
	 */
	public int getNumberOfReps() {
		return ((AcmeElement) referentType).getNumberOfReps();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getParent()
	 */
	public AcmeElement getParent() {
		return ((AcmeElement) referentType).getParent();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getParentDesign()
	 */
	public AcmeDesign getParentDesign() {
		return ((AcmeElement) referentType).getParentDesign();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getParentRepresentation()
	 */
	public AcmeRepresentation getParentRepresentation() {
		return ((AcmeElement) referentType).getParentRepresentation();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getProperties()
	 */
	public AcmeEnumeration getProperties() {
		return ((AcmeElement) referentType).getProperties();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getProperty(java.lang.String)
	 */
	public AcmeProperty getProperty(String name) {
		return ((AcmeElement) referentType).getProperty(name);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getQualifiedName()
	 */
	public String getQualifiedName() {
		return ((AcmeElement) referentType).getQualifiedName();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getRepresentation(java.lang.String)
	 */
	public AcmeRepresentation getRepresentation(String name) {
		return ((AcmeElement) referentType).getRepresentation(name);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getRepresentations()
	 */
	public AcmeEnumeration getRepresentations() {
		return ((AcmeElement) referentType).getRepresentations();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getResolvedInstanceReference()
	 */
	public AcmeElement getResolvedInstanceReference() {
		return ((AcmeElement) referentType).getResolvedInstanceReference();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getScope()
	 */
	public Scope getScope() {
		return ((AcmeElement) referentType).getScope();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getTypeSignature()
	 */
	public String getTypeSignature() {
		return ((AcmeElement) referentType).getTypeSignature();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#getUserData(java.lang.Object)
	 */
	public IElementExtension getUserData(Object key) {
		return ((AcmeElement) referentType).getUserData(key);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#instantiateElements()
	 */
	public void instantiateElements() {
		((AcmeElement) referentType).instantiateElements();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#instantiatesType(acmeLib.elements.AcmeElementType)
	 */
	public boolean instantiatesType(AcmeElementType type) {
		return ((AcmeElement) referentType).instantiatesType(type);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#isInherited()
	 */
	public boolean isInherited() {
		return ((AcmeElement) referentType).isInherited();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#isSameAsInherited()
	 */
	public boolean isSameAsInherited() {
		return ((AcmeElement) referentType).isSameAsInherited();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#numberOfDeclaredTypes()
	 */
	public int numberOfDeclaredTypes() {
		return ((AcmeElement) referentType).numberOfDeclaredTypes();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#numberOfInstantiatedTypes()
	 */
	public int numberOfInstantiatedTypes() {
		return ((AcmeElement) referentType).numberOfInstantiatedTypes();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeAllDeclaredTypes()
	 */
	public void removeAllDeclaredTypes() {
		((AcmeElement) referentType).removeAllDeclaredTypes();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeAllInstantiatedTypes()
	 */
	public void removeAllInstantiatedTypes() {
		((AcmeElement) referentType).removeAllInstantiatedTypes();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeDeclaredType(acmeLib.elements.AcmeElementType)
	 */
	public void removeDeclaredType(AcmeElementType typeToRemove) {
		((AcmeElement) referentType).removeDeclaredType(typeToRemove);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeHeuristic(acmeLib.designRules.DesignRule)
	 */
	public void removeHeuristic(DesignRule heuristicToRemove) {
		((AcmeElement) referentType).removeHeuristic(heuristicToRemove);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeInstantiatedType(acmeLib.elements.AcmeElementType)
	 */
	public void removeInstantiatedType(AcmeElementType typeToRemove) {
		((AcmeElement) referentType).removeInstantiatedType(typeToRemove);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeInvariant(acmeLib.designRules.DesignRule)
	 */
	public void removeInvariant(DesignRule invariantToRemove) {
		((AcmeElement) referentType).removeInvariant(invariantToRemove);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeMap(acmeLib.elements.AcmeMapInstance)
	 */
	public void removeMap(AcmeMapInstance map) {
		((AcmeElement) referentType).removeMap(map);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeMap(java.lang.String)
	 */
	public void removeMap(String name) {
		((AcmeElement) referentType).removeMap(name);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeProperty(acmeLib.properties.AcmeProperty)
	 */
	public void removeProperty(AcmeProperty prop) {
		((AcmeElement) referentType).removeProperty(prop);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeProperty(java.lang.String)
	 */
	public void removeProperty(String name) {
		((AcmeElement) referentType).removeProperty(name);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeRepresentation(acmeLib.representations.AcmeRepresentation)
	 */
	public void removeRepresentation(AcmeRepresentation rep) {
		((AcmeElement) referentType).removeRepresentation(rep);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#removeRepresentation(java.lang.String)
	 */
	public void removeRepresentation(String name) {
		((AcmeElement) referentType).removeRepresentation(name);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#resolveInstanceReferences()
	 */
	public void resolveInstanceReferences() {
		((AcmeElement) referentType).resolveInstanceReferences();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#resolveTypeReferences()
	 */
	public void resolveTypeReferences() {
		((AcmeElement) referentType).resolveTypeReferences();
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#setComment(java.lang.String)
	 */
	public void setComment(String comment) {
		((AcmeElement) referentType).setComment(comment);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#setName(java.lang.String)
	 */
	public void setName(String newName) throws AcmeIllegalNameException {
		((AcmeElement) referentType).setName(newName);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#setParent(acmeLib.elements.AcmeElement)
	 */
	public void setParent(AcmeElement parent) {
		((AcmeElement) referentType).setParent(parent);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#setParentDesign(acmeLib.elements.AcmeDesign)
	 */
	public void setParentDesign(AcmeDesign newParent) {
		((AcmeElement) referentType).setParentDesign(newParent);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#setUserData(java.lang.Object, acmeLib.core.extensions.IElementExtension)
	 */
	public void setUserData(Object key, IElementExtension data) {
		((AcmeElement) referentType).setUserData(key, data);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#view()
	 */
	public void view() {
		((AcmeElement) referentType).view();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof AcmeElementType) {
			AcmeElementType objType = (AcmeElementType) obj;
			if (objType instanceof AcmeElementTypeRef) {
				objType = ((AcmeElementTypeRef) objType).getReferentType();
			}

			return referentType.equals(obj);
		}
		return false;
	}

	/**
	 * @param fam
	 * @param refType
	 */
	public AcmeElementTypeRef(AcmeFamily fam, AcmeElementType refType) {
		referentType = refType;
		referentFamily = fam;
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElementType#addSuperType(acmeLib.elements.AcmeElementType)
	 */
	public void addSuperType(AcmeElementType superType) {
		referentType.addSuperType(superType);
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElementType#removeSuperType(acmeLib.elements.AcmeElementType)
	 */
	public void removeSuperType(AcmeElementType superType) {
		referentType.removeSuperType(superType);
		if (superType instanceof AcmeElementTypeRef) {
			referentType.removeSuperType(((AcmeElementTypeRef) superType).getReferentType());
		}
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElementType#getSuperTypes()
	 */
	public AcmeVector getSuperTypes() {
		return referentType.getSuperTypes();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElementType#copyType()
	 */
	public AcmeElementType copyType() {
		return referentType.copyType();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElementType#typecheck()
	 */
	public boolean typecheck() {
		return referentType.typecheck();
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElementType#getAvailableTypes()
	 */
	public AcmeEnumeration getAvailableTypes() {
		return referentType.getAvailableTypes();
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeType#getTypeName()
	 */
	public String getTypeName() {
		return referentFamily.getName() + "." + referentType.getTypeName();
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeType#setTypeName(java.lang.String)
	 */
	public void setTypeName(String typeName) throws AcmeIllegalNameException {
		referentType.setTypeName(typeName);
	}

	/* (non-Javadoc)
	 * @see acmeLib.designRules.Scope#lookupName(java.lang.String, boolean)
	 */
	public Object lookupName(String name, boolean bubbleUp) {
		return referentType.lookupName(name, bubbleUp);
	}

	/**
	 * @return
	 */
	public AcmeElementType getReferentType() {
		return referentType;
	}

	public AcmeFamily getReferentFamily() {
		return referentFamily;
	}

	/* (non-Javadoc)
	 * @see acmeLib.elements.AcmeElement#copy()
	 */
	public AcmeElement copy() {
		return (AcmeElement) this.copyType();
	}

}
