/**
 * Class: AcmeImportedDesignRecord <BR>
 * Purpose: Stores the record of a design that is imported by another AcmeDesign.
 * <BR>
 *
 * Revision History: <BR>
 *    5/2/98    RTM      * Created record as a public class
 *
 * @author Bob Monroe <BR>
 */

package acmeLib.elements;

import acmeLib.AcmeObject;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class AcmeImportedDesignRecord extends AcmeOpenedDesignRecord {

    /**
     * filename for design file provided in design's Armani "import <designFilename>" statement
     */
    private String designFilename = null;

    /**
     * Stores the list of designs importing this design.
     */
    private List importers = new ArrayList();

    public void copy(AcmeObject newObject)
    {
        super.copy(newObject);
        AcmeImportedDesignRecord newRec = (AcmeImportedDesignRecord) newObject;
        newRec.designFilename = new String(this.designFilename);
    }

	public void setDesignFilename(String designFilename) {
		this.designFilename = designFilename;
	}

	public String getDesignFilename() {
		return designFilename;
	}

    public void addImporter (AcmeDesign design) {
        importers.add(design);
    }

    public Iterator getImporterIterator () {
        return importers.iterator();
    }
    
    public void removeImporter (AcmeDesign design) {
        importers.remove(design);
    }

    public void clearImporters () {
        importers.clear();
    }

};

