
package acmeLib.elements;

import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeMap;
import acmeLib.core.container.AcmeVector;


/**
 *  Class:  AcmeRecordType
 *
 *  Purpose:the type object for a record
 *
 *  Revision History:
 *    Date:     Person    Change
 *	11/5/99		JH		Created
 */
public class AcmeRecordType extends AcmeRecordValue 
	implements AcmeCompoundElementType
{
	protected AcmeMap fields;

	//
	// public members
	//
	/**
	 * This is the constructor for this class
	 * @param name the name of the record type being created
	 */
	public AcmeRecordType (String name) {
		setTypeName(name);
		fields = new AcmeMap();
	}
	
	/**
	 * @return the name of the type
	 */
	public String getTypeName() {return this.getName();}

	/**
	 * @param typeName used to set the name of the type
	 * @exception AcmeIllegalNameException type name is illegal
	 */
	public void setTypeName(String typeName){
		try{
			this.setName(typeName);
		}catch (Exception e){}
	}
	
	public void addSuperType(AcmeElementType superType){}
	
	public void removeSuperType(AcmeElementType superType){}
	
	public AcmeVector getSuperTypes(){return null;}
	
	/**
	 * This method makes a deep copy of AcmeRecordType
	 */
	public AcmeElementType copyType(){
		AcmeRecordType new_rec_type = new AcmeRecordType(this.type.getTypeName());
		
		copyFields(new_rec_type);

		return new_rec_type;
	}
	
	/**
	 * This method returns the fields of this record
	 * @return the fields of this record
	 */
	public AcmeEnumeration getFields() { return fields.enumerate(); };

	/**
	 * This method returns a specific field of this record
	 * @param fieldName the name of the field to be returned
	 * @return the field corresponding to fieldName
	 */
	public AcmeRecordFieldValue getField(String fieldName) {
		return (AcmeRecordFieldValue) fields.lookupItem(fieldName);
	}

	/**
	 * This method adds a field to a record type
	 * @param name the name of the field to be added
	 * @param t the type of the field to be added
	 */
	public void addField (String name, AcmeElementType t) {
		fields.addItem(name, new AcmeRecordFieldValue (name, t, null));
	}

	/**
	 * This method returns a deep-copy of "this" AcmeRecordType object
	 * @return a copy of this AcmeRecordType object
	 */
	public AcmeElement copy() {
		return (AcmeElement) copyType();
	}


	/**
	 * @param s AcmeOutputStream
	 * @exception java.io.IOException when an error occurs while writing
	 */
/*	public void exportAcme (AcmeOutputStream s) throws java.io.IOException {
		this.export(s);
	}*/

	/**
	 * @param s AcmeOutputStream
	 * @exception java.io.IOException when an error occurs while writing
	 */
/*	public void exportArmani(AcmeOutputStream s) throws java.io.IOException {
		this.export(s);
	}*/


	/**
	 * This method resolves the property type object for all of this record's fields.
	 * @param scope acmeLib.designRules.Scope
	 */
/*	public void resolveTypeReferences(acmeLib.designRules.Scope scope) {
		// iterate through all of the fields, resolving their typenames.
		AcmeEnumeration e = this.getFields();
		AcmeRecordFieldValue f = null;
		while (e.moreItems()) {
			f = (AcmeRecordFieldValue) e.getNextItem();
			f.resolveTypeReferences(scope);
		}
	}*/

	/**
	 * Implements the acmeLib.designRules.Scope interface to map names
	 * to record fields.
	 * @param name the name of the property
	 * @param bubbleUp boolean
	 * @return the Object corresponding to "name"
	 */
	public Object lookupName(String name, boolean bubbleUp)
	{
		return getField(name);
	}

}
