
package acmeLib.elements;

import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;


/** 
 *  Class:  AcmeUnresolvedElementType
<BR>
 *  Purpose: Stores a placeholder for a real ElementType.  In the first
<BR>
 *          pass of the latest & greatest AcmeParser (as of 1/14/98)
<BR>
 *          typenames are unresolved after the first pass through parsing,
<BR>
 *          so one of these objects are stored in its place, and the
<BR>
 *          typename can be resolved on the name resolution pass through
<BR>
 *          the design.
<BR><BR>
 *<BR>
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    1/14/98   RTM     Created class
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public class AcmeUnresolvedElementType extends AcmeObject
                                       implements AcmeElementType {

    //
    // protected members
    //
    protected String typeName;
    protected String qualifyingName;

    //
    // public members
    //

    /**
     * This is the contructor for this class
     * @param typeName the name of the element type to be created
     */
    public AcmeUnresolvedElementType(String typeName) {
        this.typeName = typeName;
    };
    
    public AcmeUnresolvedElementType(String qualifier, String typeName) {
    	this.qualifyingName = qualifier;
    	this.typeName = typeName;
    }

    /**
     * This method returns the name of the element type
     * @return the name of the element type
     */
    public String getTypeName() { return this.typeName; };
    /**
     * This method sets the name of this element type
     * @param typeName the name to be given to this element type
     */
    public void setTypeName(String typeName) { this.typeName = typeName; };

	public String getQualifier () { return qualifyingName;}
	
	public void setQualifier (String q) {qualifyingName = q;}

    // The following methods are here only for "real" element
    // type objects, and don't make much sense for Unresolved typenames.
    /**
     * This method adds supertype to the element type object
     * @param superType supertype to be added to the element type object
     */
    public void addSuperType(AcmeElementType superType) {}
    /**
     * This method removes a supertype from the element type object
     * @param superType the supertype to be removed from the element type object
     */ 
    public void removeSuperType(AcmeElementType superType) {}
    /**
     * This method returns a vector of supertypes
     * @return a vector of supertypes of this element type
     */
    public AcmeVector getSuperTypes() { return new AcmeVector(); }
    /**
     *  @param name the name to be mapped to a type or design analyses
     *  @param bubbleUp boolean, if falso don't check superclass 
     *  @return Object corresponding to "name"
     */
    public Object lookupName(String name, boolean bubbleUp) { return null; };

    /**
     * @return false
     */
    public boolean typecheck() {
        return false;  // an unresolved type is inherently type-incorrect.
    };


    /**
     * @return a copy of this element type
     */
    public AcmeElementType copyType() {
        return new AcmeUnresolvedElementType(this.getTypeName(),qualifyingName);
    };
    
    public AcmeEnumeration getAvailableTypes () {return null;}

	/**
	 * @return
	 */
	public boolean isQualified() {
		return qualifyingName != null;
	}
    
}
