
package acmeLib.elements;

import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;


/** 
 *  Class:   BaseElementParameterType
<BR>
 *  Purpose:Simple class that acts as a parameter type to support the passing
<BR>
 *          of arbitrary element objects as parameters to and return values
<BR>
 *          from, design analyses.
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    3/29/98  RTM       created class
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public class BaseElementParameterType extends AcmeObject
                                      implements AcmeElementType
{
    protected String typeName;
    protected static AcmeVector superTypes = new AcmeVector(1,1);

/**
     * This is the contructor for this class
     * @param newTypeName the name of the element type to be created
     */
    public BaseElementParameterType(String newTypeName) {
        this.typeName = newTypeName;
    };

 /**
     * This method returns the name of the element type
     * @return the name of the element type
     */
    public String getTypeName() { return this.typeName; }
 /**
     * This method sets the name of this element type
     * @param newTypeName the name to be given to this element type
     */
    public void setTypeName(String newTypeName) { this.typeName = newTypeName; }


    // stub methods to satisfy AcmeElementType, but they can do nothing, as
    // this is basically intended only to be a constant that acts as a placeholder
    // for _any_ ElementType where an object is needed instead of just an
    // interface.
   /**
     * This method adds supertype to the parameter type object
     * @param superType supertype to be added to the parameter type object
     */
    public void addSuperType(AcmeElementType superType) {};
 /**
     * This method removes a supertype from the parameter type object
     * @param superType the supertype to be removed from the parameter type object
     */ 
    public void removeSuperType(AcmeElementType superType) {};
 /**
     * This method returns a vector of supertypes 
     * @return a vector of supertypes of this parameter type
     */
    public AcmeVector getSuperTypes() { return superTypes; };

    
  /** 
   * This method does not make a copy, just return this.
   * @return same element type
   */
    public AcmeElementType copyType() { return this; };

  /**
   *  this class holds nothing that could make it type-incorrect.
   * @return true
   */
    public boolean typecheck() { return true; };

  /**
   *  this class holds nothing as its scope.
   *  @return null
   */
    public Object lookupName(String name, boolean bubbleUp) { return null; }
    /**
     * @see acmeLib.elements.AcmeElementType#getAvailableTypes()
     */
    public AcmeEnumeration getAvailableTypes() {
        return null;
    }

}
