package acmeLib.errorManagement;

import acmeLib.designRules.DesignRuleParameter;

/**
 * Class: DesignAnalysisParamError<br>
 * Purpose: Describes an error that occured because a parameter for a<br>
 *          design analysis is somehow messed up.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    3/10/98  RTM       created initial class<br>
 * @author Bob Monroe
 */

public class DesignAnalysisParamError extends AcmeError {

    //
    // protected members
    //

  /**
   * stores the expression object that doesn't typecheck
   */
    protected DesignRuleParameter badParam;

    //
    // public members
    //

  /**
   * default constructors that primarily make use of super-class constructor
   */
    public DesignAnalysisParamError() {
        super();
        this.errorMessage = "Unspecified DesignAnalysisParamError";
        this.setBadParam(null);
    };

  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public DesignAnalysisParamError(Object errorSource, String msg) {
        super(errorSource,msg);
    };

  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param problemParam The problem parameter
   */
    public DesignAnalysisParamError(Object errorSource, String msg,
                                    DesignRuleParameter problemParam)
    {
        super(errorSource, msg);
        this.setBadParam(problemParam);
    };



    // accessors and settors
  /**
   * Get the bad parameter
   * @return the bad parameter
   */
    public DesignRuleParameter getBadParam() { return this.badParam; };
  /**
   * Set the bad parameter
   * @param newBadParam The new bad parameter
   */
    public void setBadParam(DesignRuleParameter newBadParam) {
        this.badParam = newBadParam;
    };
};