package acmeLib.errorManagement;

import acmeLib.designRules.DesignRuleEvalException;
import acmeLib.designRules.ExpressionNode;
import acmeLib.designRules.Scope;

/**
 * Class: DesignRuleEvalError<br>
 * Purpose: Describes an error that occured because a design rule<br>
 *          could not be properly evaluated.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    4/1/98   RTM       created initial class<br>
 * @author Bob Monroe
 */

public class DesignRuleEvalError extends AcmeError {

    //
    // protected members
    //

  /**
   * stores the expression object that doesn't typecheck
   */
    protected ExpressionNode problemExpression;

  /** 
   * stores the scope in which the design rule was evaluated and
   * found not to hold.
   */
    protected Scope exprScope;

    //
    // public members
    //

  /**
   * default constructors that primarily make use of super-class constructor
   */
    public DesignRuleEvalError() {
        super();
        this.errorMessage = "Unspecified DesignRuleEvalError";
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public DesignRuleEvalError(Object errorSource, String msg) {
        super(errorSource, msg);
        if (errorSource instanceof ExpressionNode)
            this.setProblemExpression((ExpressionNode) errorSource);
    };


  /**
   * This is the constructor for this class
   * @param drExc The Design Rule Error Exception
   */
    public DesignRuleEvalError(DesignRuleEvalException drExc) {
        super();
        if (drExc.getMessage() != null) {
            this.setErrorMessage(drExc.getMessage());
        };
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param problemExprNode The problem expression node
   * @param exprScope The expression scope
   */
    public DesignRuleEvalError(Object errorSource, String msg,
                                    ExpressionNode problemExprNode,
                                    Scope exprScope)
    {
        super(errorSource, msg);
        this.setProblemExpression(problemExprNode);
        this.setExpressionScope(exprScope);
    };



    // accessors and settors
  /**
   * Get the problem expression
   * @return the problem expression
   */
    public ExpressionNode getProblemExpression() {
        return this.problemExpression;
    };
  /**
   * Set the problem expression
   * @param newProblemExpr The new problem expression
   */
    public void setProblemExpression(ExpressionNode newProblemExpr) {
        this.problemExpression = newProblemExpr;
    };

  /**
   * Get the expression scope
   * @return the expression scope
   */
    public Scope getExprScope() { return this.exprScope; };
  /**
   * Set the expression scope
   * @param newExprScope The new expression scope
   */
    public void setExpressionScope(Scope newExprScope) {
        this.exprScope = newExprScope;
    };
};