package acmeLib.errorManagement;

import acmeLib.designRules.DesignRule;
import acmeLib.designRules.Scope;

/**
 * Class: DesignRuleViolationError<br>
 * Purpose: Describes an error that occured because a design rule<br>
 *          was violated.  Superclass to InvariantViolationError<br>
 *          and HeuristicViolationError classes.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    2/13/98  RTM       created initial class<br>
 * @author Bob Monroe
 */

public class DesignRuleViolationError extends AcmeError {

    //
    // protected members
    //

  /**
   * stores the expression object that doesn't typecheck
   */
    protected DesignRule violatedDesignRule;

  /** 
   * stores the scope in which the design rule was evaluated and
   * found not to hold.
   */
    protected Scope evalScope;

    //
    // public members
    //

  /**
   * default constructors that primarily make use of super-class constructor
   */
    public DesignRuleViolationError() {
        super();
        this.errorMessage = "Unspecified DesignRuleViolationError";
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public DesignRuleViolationError(Object errorSource, String msg) {
        super(errorSource, msg);
        if (errorSource instanceof DesignRule)
            this.setDesignRule((DesignRule) errorSource);
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param problemDesignRule The problem expression node
   * @param evaluationScope The expression scope
   */
    public DesignRuleViolationError(Object errorSource, String msg,
                                    DesignRule problemDesignRule,
                                    Scope evaluationScope)
    {
        super(errorSource, msg);
        this.setDesignRule(problemDesignRule);
        this.setEvaluationScope(evaluationScope);
    };



    // accessors and settors
  /**
   * Get the design rule
   * @return the design rule
   */
    public DesignRule getDesignRule() { return this.violatedDesignRule; };
  /**
   * Set the design rule
   * @param newViolatedDesignRule The new violated design rule
   */
    public void setDesignRule(DesignRule newViolatedDesignRule) {
        this.violatedDesignRule = newViolatedDesignRule;
    };

  /**
   * Get the evaluation scope
   * @return the evaluation scope
   */
    public Scope getEvaluationScope() { return this.evalScope; };
  /**
   * Set the evaluation scope
   * @param evaluationScope The new evaluation scope
   */
    public void setEvaluationScope(Scope evaluationScope) {
        this.evalScope = evaluationScope;
    };
};