package acmeLib.errorManagement;

import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;

/**
 * Class: ElementTypeSatisfactionError<br>
 * Purpose: Describes an error that occured because an element (ccrps)<br>
 *          does not satisfy one of its declared types.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    2/15/98  RTM       created initial class<br>
 * @author Bob Monroe
 */

public class ElementTypeSatisfactionError extends AcmeError {

    //
    // protected members
    //

  /**
   * stores the element that doesn't typecheck
   */
    protected AcmeElement errorElement;

  /**
   * stores the element type that the element doesn't satisfy
   */
    protected AcmeElementType errorElementType;

    //
    // public members
    //

  /**
   * default constructors that primarily make use of super-class constructor
   */
    public ElementTypeSatisfactionError() {
        super();
        this.errorMessage = "Unspecified ElementTypeSatisfactionError";
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public ElementTypeSatisfactionError(Object errorSource, String msg) {
        super(errorSource, msg);
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param elt The AcmeElement
   * @param eltType The AcmeElementType
   */
    public ElementTypeSatisfactionError(Object errorSource, String msg,
                                 AcmeElement elt, AcmeElementType eltType)
    {
        super(errorSource, msg);
        this.setElement(elt);
        this.setElementType(eltType);
    };



    // accessors and settors
  /**
   * Get the element
   * @return The element
   */
    public AcmeElement getElement() { return this.errorElement; };
  /**
   * Set the element
   * @param newElement The new element
   */
    public void setElement(AcmeElement newElement) {
        this.errorElement = newElement;
    };

  /**
   * Get the element type
   * @return The element type
   */
    public AcmeElementType getElementType() { return this.errorElementType; };
  /**
   * Set the element type
   * @param newElementType The new element type
   */
    public void setElementType(AcmeElementType newElementType) {
        this.errorElementType = newElementType;
        if (newElementType instanceof AcmeElement) {
            setSeeAlso( ((AcmeElement) newElementType).getQualifiedName() );
        };
    };

};