package acmeLib.errorManagement;

import acmeLib.elements.AcmeElementType;

/**
 * Class: ElementTypeTypecheckError<br>
 * Purpose: Describes an error that occured because an element type<br>
 *          does not typecheck (it is not internally-consistent).<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    2/13/98  RTM       created initial class<br>
 * @author Bob Monroe
 */

public class ElementTypeTypecheckError extends AcmeError {

    //
    // protected members
    //

  /**
   * stores the element type that the element doesn't satisfy
   */
    protected AcmeElementType errorElementType;

    //
    // public members
    //

  /**
   * default constructors that primarily make use of super-class constructor
   */
    public ElementTypeTypecheckError() {
        super();
        this.errorMessage = "Unspecified ElementTypeTypecheckError";
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public ElementTypeTypecheckError(Object errorSource, String msg) {
        super(errorSource, msg);
        if (errorSource instanceof AcmeElementType)
            this.setElementType((AcmeElementType) errorSource);
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param eltType The AcmeElementType
   */
    public ElementTypeTypecheckError(Object errorSource, String msg,
                                     AcmeElementType eltType)
    {
        super(errorSource, msg);
        this.setElementType(eltType);
    };



    // accessors and settors
  /**
   * Get the element type
   * @return The element type
   */
    public AcmeElementType getElementType() { return this.errorElementType; };
  /**
   * Set the element type
   * @param newElementType The new element type
   */
    public void setElementType(AcmeElementType newElementType) {
        this.errorElementType = newElementType;
    };

};