package acmeLib.errorManagement;

import acmeLib.designRules.ExpressionNode;

/**
 * Class: ExpressionTypecheckError<br>
 * Purpose: Describes an error that occured because a design rule (or<br>
 *          design analysis) expression does not typecheck.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    2/13/98  RTM       created initial class<br>
 * @author Bob Monroe
 */

public class ExpressionTypecheckError extends AcmeError {

    //
    // protected members
    //

  /**
   * stores the expression object that doesn't typecheck
   */
    protected ExpressionNode expression;

    //
    // public members
    //

  /**
   * default constructors that primarily make use of super-class constructors
   */
    public ExpressionTypecheckError() {
        super();
        this.errorMessage = "Unspecified ExpressionTypecheckError";
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public ExpressionTypecheckError(Object errorSource, String msg) {
        super(errorSource, msg);
        if (errorSource instanceof ExpressionNode)
            this.setExpression((ExpressionNode) errorSource);
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param problemExpression The problem Expression
   */
    public ExpressionTypecheckError(Object errorSource, String msg,
                                    ExpressionNode problemExpression)
    {
        super(errorSource, msg);
        this.setExpression(problemExpression);
    };



    // accessors and settors
  /**
   * Get the expression
   * @return The expression
   */
    public ExpressionNode getExpression() { return this.expression; };
  /**
   * Set the expression
   * @param newExpression The new expression
   */
    public void setExpression(ExpressionNode newExpression) {
        this.expression = newExpression;
    };
};