package acmeLib.errorManagement;

import acmeLib.designRules.DesignRule;
import acmeLib.designRules.Scope;

/**
 * Class: HeuristicViolationError<br>
 * Purpose: Describes an error/warning that occured because a Design Heuristic<br>
 *          was violated.  Tools receiving notice of this violation can decide<br>
 *          what to do about it, as violations of heuristics are generally<br>
 *          warnings rather than errors.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    2/13/98  RTM       created initial class<br>
 * @author Bob Monroe
 */

public class HeuristicViolationError extends DesignRuleViolationError {

  /**
   * default constructors that primarily make use of super-class constructor
   */

    public HeuristicViolationError() {
        super();
        this.errorMessage = "Unspecified HeuristicViolationError";
        this.setMsgType(AcmeError.WARNING);
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public HeuristicViolationError(Object errorSource, String msg) {
        super(errorSource, msg);
        this.setMsgType(AcmeError.WARNING);
    };



  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param problemDesignRule The problemed design rule
   * @param evaluation Scope The evaluation scope
   */
    public HeuristicViolationError(Object errorSource, String msg,
                                    DesignRule problemDesignRule,
                                    Scope evaluationScope)
    {
        super(errorSource, msg, problemDesignRule, evaluationScope);
        this.setMsgType(AcmeError.WARNING);
    };

};