package acmeLib.errorManagement;

import acmeLib.elements.AcmeElement;
import acmeLib.elements.AcmeElementType;

/**
 * Class: IllegalSubtypeError<br>
 * Purpose: Describes an error that occured because an element type claims<br>
 *          it is a subclass of a class that is a different category (ccrps)<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    2/16/98  RTM       created initial class<br>
 * @author Bob Monroe
 */

public class IllegalSubtypeError extends AcmeError {

    //
    // protected members
    //

  /**
   * stores the element that doesn't typecheck
   */
    protected AcmeElementType subType = null;

  /**
   * stores the element type that the element doesn't satisfy
   */
    protected AcmeElementType superType = null;

    //
    // public members
    //

  /**
   * default constructors that primarily make use of super-class constructors
   */
    public IllegalSubtypeError() {
        super();
        this.errorMessage = "Unspecified IllegalSubtypeError";
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public IllegalSubtypeError(Object errorSource, String msg) {
        super(errorSource, msg);
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param badSubType bad subtype
   * @param badSuperType bad supertype
   */
    public IllegalSubtypeError(Object errorSource, String msg,
                               AcmeElementType badSubType,
                               AcmeElementType badSuperType)
    {
        super(errorSource, msg);
        this.setSubType(badSubType);
        this.setSuperType(badSuperType);
    };


    // accessors and settors
  /**
   * Get the subtype
   * @return the subtype
   */
    public AcmeElementType getSubType() { return this.subType; };
  /**
   * Set the subtype
   * @param newSubType The new subtype
   */
    public void setSubType(AcmeElementType newSubType) {
        this.subType = newSubType;
    };

  /**
   * Get the supertype
   * @return the supertype
   */
    public AcmeElementType getSuperType() { return this.superType; };
  /**
   * Set the supertype
   * @param newSuperType The new supertype
   */
    public void setSuperType(AcmeElementType newSuperType) {
        this.superType = newSuperType;
        if (newSuperType instanceof AcmeElement) {
            this.setSeeAlso( ((AcmeElement) newSuperType).getQualifiedName() );
        };
    };

};