package acmeLib.errorManagement;


/**
 * Class: InstantiationConflictError<br>
 * Purpose: Describes an error that occured because a type was not<br>
 *          able to be properly resolved.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    2/11/98  RTM       created initial class<br>
 *<br>
 * @author Bob Monroe
 */
public class InstantiationConflictError extends AcmeError {

    //
    // protected members
    //

  /**
   * the existing entity, type, or value.
   */
    protected Object existingEntity = null;

  /**
   * the new entity, type, or value that attempted to overwrite it.
   */
    protected Object overwritingEntity = null;

    //
    // public members
    //

  /**
   * This is the constructor for this class
   */
    public InstantiationConflictError() {
        super();
        this.errorMessage = "Unspecified InstantiationConflictError";
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public InstantiationConflictError(Object errorSource, String msg) {
        super(errorSource, msg);
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param newExistingEntity The new existing entity
   * @param newOverwritingEntity The new overwriting entity
   */
    public InstantiationConflictError(Object errorSource, String msg,
	                                  Object newExistingEntity,
                                      Object newOverwritingEntity)
    {
        super(errorSource, msg);
        this.setExistingEntity(newExistingEntity);
        this.setOverwritingEntity(newOverwritingEntity);
    };



    // accessors and settors
  /**
   * Get the existing entity
   * @return the existing entity
   */
    public Object getExistingEntity() { return this.existingEntity; };
  /**
   * Set the exsiting entity
   * @param newExistingEntity The new existing entity
   */
    public void setExistingEntity(Object newExistingEntity) {
        this.existingEntity = newExistingEntity;
    };

  /**
   * Get the overwriting entity
   * @return the overwriting entity
   */
    public Object getOverwritingEntity() { return this.overwritingEntity; };
  /**
   * Set the overwriting entity
   * @param newOverwritingEntity The new overwriting entity
   */
    public void setOverwritingEntity(Object newOverwritingEntity) {
        this.overwritingEntity = newOverwritingEntity;
    };


   /**
    * the "seeAlso" field for this error is of the form:
    *  <"existingEntityName", "overwritingEntityName">
    **/
    public String seeAlso() {
        StringBuffer buf = new StringBuffer();
        buf.append(findQualifiedName(this.existingEntity));
        buf.append(", " + this.findQualifiedName(this.overwritingEntity));
        return buf.toString();
    };


};