package acmeLib.errorManagement;

import acmeLib.properties.AcmeProperty;

/**
 * Class: PropertyTypecheckError<br>
 * Purpose: Describes an error that occured because a property<br>
 *          is not type-correct.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    2/13/98  RTM       created initial class<br>
 *<br>
 * @author Bob Monroe
 */
public class PropertyTypecheckError extends AcmeError {

    //
    // protected members
    //

  /**
   * stores the property that is not type-correct.
   */
    protected AcmeProperty errorProperty;

    //
    // public members
    //

  /**
   * default constructors that primarily make use of super-class constructors
   */
    public PropertyTypecheckError() {
        super();
        this.errorMessage = "Unspecified PropertyTypecheckError";
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public PropertyTypecheckError(Object errorSource, String msg) {
        super(errorSource, msg);
        if (errorSource instanceof AcmeProperty)
            this.setProperty((AcmeProperty) errorSource);
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param prop The AcmeProperty
   */
    public PropertyTypecheckError(Object errorSource, String msg,
                                  AcmeProperty prop)
    {
        super(errorSource, msg);
        this.setProperty(prop);
    };



    // accessors and settors
  /**
   * Get the property
   * @return the property
   */
    public AcmeProperty getProperty() { return this.errorProperty; };
  /**
   * Set the property
   * @param newProperty The new property
   */
    public void setProperty(AcmeProperty newProperty) {
        this.errorProperty = newProperty;
        if (newProperty != null) {
            this.setSeeAlso(newProperty.getQualifiedName());
        };
    };

};