/**
 * Class: TypeInstantiationError<br>
 * Purpose: Describes an error that occured because during an attempt to<br>
 *          instantiate a type.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    3/5/98  RTM       created initial class<br>
 *<br>
 * @author Bob Monroe
 */
package acmeLib.errorManagement;

import acmeLib.*;
import acmeLib.properties.*;
import acmeLib.elements.*;

public class TypeInstantiationError extends AcmeError {

    //
    // protected members
    //

  /**
   * the new entity, type, or value that attempted to overwrite it.
   */
    protected AcmeType entityType = null;

    //
    // public members
    //

  /**
   * This is the constructor for this class
   */
    public TypeInstantiationError() {
        super();
        this.errorMessage = "Unspecified TypeInstantiationError";
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public TypeInstantiationError(Object errorSource, String msg) {
        super(errorSource, msg);
    };


  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param problemType The problem type
   */
    public TypeInstantiationError(Object errorSource, String msg,
                                  AcmeType problemType)
    {
        super(errorSource, msg);
        this.setEntityType(problemType);
    };



    // accessors and settors
  /**
   * Get the entity type
   * @return the entity type
   */
    public AcmeType getEntityType() { return this.entityType; };
  /**
   * Set the entity type
   * @param newEntityType The new entity type
   */
    public void setEntityType(AcmeType newEntityType) {
        this.entityType = newEntityType;
        if (newEntityType instanceof AcmePropertyType) {
            this.setSeeAlso( ((AcmePropertyType) newEntityType).getQualifiedName() );
        } else if (newEntityType instanceof AcmeElement) {
            this.setSeeAlso( ((AcmeElement) newEntityType).getQualifiedName() );
        };
    };

};