package acmeLib.errorManagement;


/**
 * Class: TypeRefResolutionError<br>
 * Purpose: Describes an error that occured because a type was not<br>
 *          able to be properly resolved.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    2/11/98  RTM       created initial class<br>
 *<br>
 * @author Bob Monroe
 */
public class TypeRefResolutionError extends AcmeError {

    //
    // protected members
    //

  /**
   * stores either the unresolved{element,property,type,attachment,binding,var}
   * object doing the referencing.
   */
    protected Object unresolvedRef = null;

  /**
   * the name of the entity that was not resolvable
   */
    protected String name = null;

    //
    // public members
    //

  /**
   * default constructors that primarily make use of super-class constructors
   */
    public TypeRefResolutionError() {
        super();
        this.errorMessage = "Unspecified TypeRefResolutionError";
    };

  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   */
    public TypeRefResolutionError(Object errorSource, String msg) {
        super(errorSource, msg);
    };

  /**
   * This is the constructor for this class
   * @param errorSource The source of error
   * @param msg The error message
   * @param refName The ref name
   */
    public TypeRefResolutionError(Object errorSource, String msg, String refName)
    {
        super(errorSource, msg);
        this.setName(refName);
    };


    // accessors and settors
  /**
   * Get the name
   * @return the name
   */
    public String getName() { return this.name; };
  /**
   * Set the name
   * @param newName The new name
   */
    public void setName(String newName) {
        this.name = newName;
        this.setSeeAlso(newName);
    };

  /**
   * Get the unresolved ref
   * @return the unresolved ref
   */
    public Object getUnresolvedRef() { return this.unresolvedRef; };
  /**
   * Set the unresolved ref
   * @param ref The new unresolved ref
   */
    public void setUnresolvedRef(Object ref) { this.unresolvedRef = ref; };
};