// Class: ArmaniAcmeStudioIntf
// Purpose: Provides an interface to Armani that is of use to the Visio
//          graph editor that has been designed for editing Armani systems.
//
// CLSID for this class is {684C4980-8E12-11D2-BC3D-00609737B51A}.
//
//
// Revision History:
//    Date:    Person    Change
//    12/7/98   RTM       Created initial class


/** Magic registration chant to register this class with the Windows repository
 * and create a typelib that stores the appropriate type information.
 *
 * %javareg /register /class:acmeLib.gui.ArmaniAcmeStudioIntf
 * /clsid:{684C4980-8E12-11D2-BC3D-00609737B51A}
 * /progid:acmeLib.gui.ArmaniAcmeStudioIntf.java.1
 * /typelib:ArmaniAcmeStudioIntf.tlb
 **/

/**
 * Sample VB code to test the COM interface.  Note that the FileSelectDialog
 * is the name of a common dialog control that is used for opening files.
 *
 * Private Sub TestIntfBtn_Click()
 * Dim testStr, filename As String
 * Dim typechecks As Boolean
 * Dim fileNumber As Integer
 * Dim fileLength As Long
 * ' Dim ArmaniAcmeIntf As acmeLib.gui.ArmaniAcmeStudioIntf
 *
 * ' open up an Acme file, and run it through the Armani typechecker
 * ' via COM.
 * FileSelectDialog.ShowOpen
 * filename = FileSelectDialog.filename
 *
 * If Not (filename = "") Then
 * fileNumber = FreeFile
 * fileLength = FileLen(filename)
 *
 * MsgBox "Selected the file named: " & filename & " of length " + Str(fileLength)
 *
 * Open filename For Input As fileNumber
 * testStr = Input(fileLength, fileNumber)
 *
 * 'testStr = "System s = { component c; component d; };"
 *
 * If (ArmaniIntfObj Is Nothing) Then
 * Set ArmaniIntfObj = CreateObject("acmeLib.gui.ArmaniAcmeStudioIntf.java.1")
 * ' ArmaniIntfObj.setVisioApplication VisioApp
 * End If
 * ' ArmaniIntfObj.editElement (shpObj.UniqueID(visGetOrMakeGUID))
 * typechecks = ArmaniIntfObj.typecheckAcmeStudioDesign(testStr)
 * If (typechecks) Then
 * MsgBox "typecheckAcmeStudioDesign returned with typechecks = true"
 * Else
 * MsgBox "typecheckAcmeStudioDesign returned with typechecks = false"
 * End If
 *
 * ' close the input file and return.
 * Close fileNumber
 * Else
 * ' else no file was selected.  quietly return.
 * End If
 *
 * End Sub
 *
 **  end sample VB code **/

package acmeLib.gui;


import java.io.*;
import java.util.Vector;

import acmeLib.AcmeObject;
import acmeLib.Debug;
import acmeLib.acmeParser.AcmeParser;
import acmeLib.armaniParser.ArmaniParser;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.container.AcmeVector;
import acmeLib.elements.AcmeDesign;
import acmeLib.errorManagement.AcmeError;
import acmeLib.errorManagement.AcmeErrorManager;
import acmeLib.tools.GraphWalker;
import acmeLib.tools.converters.Acme2Armani;
import acmeLib.tools.converters.AcmeStudioPropertyStripper;

public class ArmaniAcmeStudioIntf  {
    
    /**
     * Have a static ArmaniParser and AcmeParser that is shared by and
     * available to all instances of this interface.
     **/
    protected static ArmaniParser armaniParserObj = null;
    protected static AcmeParser acmeParserObj = null;
    
    
    /**
     * stores the error and warning messages received after typechecking.
     **/
    protected AcmeEnumeration errorMessages = null;
    
    
    /** stores a static reference to the current AcmeStudioObject being
     *  used by the running Visio application.
     *  RTM 6/22/98 -- This is basically a hack to deal with the fact that
     *                 the Java/ActiveX integration doesn't allow static
     *                 Java references to ActiveX objects, and the fact that
     *                 Visio/VBA has a tendency to "lose" it's reference
     *                 to this interface, and thus has to keep making
     *                 a new one.
     */
    // public static ArmaniAcmeStudioIntf curVisioInterface = null;
    
    // // keep a (non-static) reference to the visio application that we're using
    // protected visio.vislib32.IVApplication globalApp;
    
    /** Toggles debugging on and off **/
    protected static boolean debugArmaniAcmeStudioIntf = true;
    protected static boolean obtrusiveDebugging = false;
    
    
    /** send an error message to the screen in a message box.  This should be
     *  used differently from the debugMsg only in its use.  The debugMsg box
     *  should be used for temporary bug hunting, and then deleted.  The
     *  errorMsg box should be used when it is expected to be a permanent
     *  fixture for flagging errors in a method.
     *  @param msg  The error message to display
     **/
    protected void errorMsg (String msg) {
//        MessageFrame msgBox = new MessageFrame ("Error Message");
//        msgBox.appendText (msg);
//        msgBox.setVisible (true);
//        msgBox.toFront ();
    };
    
    
//    /** Single console for displaying debugging messages */
//    static MessageFrame debugMsgBox;
    
    
    
    /** send a debugging message to the screen, if the debugArmaniAcmeStudioIntf
     *  flag is set to true.  Useful for dealing with Visio/VBA/Java
     *  integration bugs and debugging.
     **/
    protected void debugMsg (String msg) {
//        if (ArmaniAcmeStudioIntf.debugArmaniAcmeStudioIntf) {
//            if (debugMsgBox == null) {
//                debugMsgBox = new MessageFrame ("Debugging Messages");
//                debugMsgBox.setVisible (true);
//            };
//            
//            debugMsgBox.appendText ("\n\n------ Debug: " /* + time? */
//            + " ------------------\n" + msg);
//            
//            if (!(debugMsgBox.isShowing ())) {
//                debugMsgBox.show ();
//            };
//            
//            if (ArmaniAcmeStudioIntf.obtrusiveDebugging) {
//                debugMsgBox.setVisible (true);
//                debugMsgBox.toFront ();
//            };
//        };
    };
    
    
    /** toggle debugging messages */
    public void turnDebugginOn ()  {
        ArmaniAcmeStudioIntf.debugArmaniAcmeStudioIntf = true;
    };
    
    public void turnDebugginOff ()  {
        ArmaniAcmeStudioIntf.debugArmaniAcmeStudioIntf = false;
    };
    
    public void obtrusiveDebugginOn ()  {
        ArmaniAcmeStudioIntf.obtrusiveDebugging = true;
    };
    
    public void obtrusiveDebugginOff ()  {
        ArmaniAcmeStudioIntf.obtrusiveDebugging = false;
    };
    
    
    
    // Do not have the public constructor do anything, that way anything
    // can instantiate it and I don't need to worry about Visio or VB
    // garbage collecting (or losing) the reference to this object.  I can
    // just create a new COM object that is really just an interface to the
    // (effectively) static methods.
    public ArmaniAcmeStudioIntf () {
        // initialize all of the static property type variables, (the called
        // methods make sure that they are only initialized once...)
        // this is necessary in the Visio interface.  it may or may not be
        // necessary in the AcmeStudio interface...
        
        // create an enumeration to store typechecking messages in.
        errorMessages = new AcmeEnumeration ();
    };
    
    
    
    
    /************************************************************************
     ** Begin actual interface specification to AcmeStudio.                **
     ************************************************************************/
    
    
    /**
     * Reads in and typechecks a design that has been exported from the
     * AcmeStudio.  This method performs the following steps: <BR>
     * 1) Parse the Acme design that is encoded in acmeDesignStr <BR>
     * 2) Translate the design from Acme to Armani <BR>
     * 3) Typecheck the design.  All errors and warnings found will be
     *     placed in the AcmeErrorManager where they can be retrieved later. <BR>
     * 4) Return true to indicate that the design typechecks, or false to
     *     indicate that it does not typecheck. <BR>
     *
     * @param acmeDesignStr A string containing the entire description of
     *        the AcmeDesign to be typechecked.  Armani design rules should
     *        be embedded as properties in the Acme description.
     * @return true if the design typechecks, false otherwise.
     **/
    public boolean typecheckAcmeStudioDesign (String acmeDesignStr) {
        try {
            AcmeDesign acmeDesignObj = null;
            // clear out all old errors so that we'll know which ones were
            // caused by this typechecking run.
            AcmeErrorManager.clearAllErrors ();
            
            // Parse the Acme design that is encoded in acmeDesignStr
            java.io.ByteArrayInputStream compInputStream =
            new java.io.ByteArrayInputStream (acmeDesignStr.getBytes ());
            
            if (acmeParserObj == null) {
                try {
                    acmeParserObj = new AcmeParser (compInputStream);
                }
                catch (java.lang.Error e) {
                    // The parser exists, but was created by another program
                    acmeLib.acmeParser.AcmeParser.ReInit (compInputStream);
                }
            } else {
                AcmeParser.ReInit (compInputStream);
            };
            
            acmeLib.acmeParser.AcmeParser.disable_tracing ();
            acmeLib.acmeParser.AcmeParser.initDesign ();
            
            try {
                acmeDesignObj = acmeLib.acmeParser.AcmeParser.parse_AcmeDesign ();
            }
            catch (acmeLib.acmeParser.ParseError apErr) {
                AcmeError newParseError = new AcmeError (acmeDesignObj,
                "Error encountered parsing AcmeDesign:  "
                + apErr.getMessage ());
                AcmeErrorManager.errorDetected (newParseError);
                return false;
            };
            
            // Strip out any AcmeStudio-specific properties
            AcmeStudioPropertyStripper stripper = new AcmeStudioPropertyStripper();
            GraphWalker.WalkOverAllElements(acmeDesignObj, stripper, GraphWalker.WALK_TOP_DOWN);
            // Translate the design from Acme to Armani
            Acme2Armani.translate (acmeDesignObj);
 
            
            // Typecheck the design.
            boolean typechecks = acmeDesignObj.typecheck ();
            
            
            // retrieve all of the error and warning messages from the
            // AcmeErrorManager and put them into the errorMessage enumeration
            // for subsequent retrieval.
            errorMessages = AcmeErrorManager.getErrors ();
            
            
            // Return true to indicate that the design typechecks, or false to
            // indicate that it does not typecheck.
            return typechecks;
        } catch (Exception e) {
            Debug.out (e.toString ());
            Debug.close ();
            return true;
        }
    };
    
    
    
    
    /**
     * @return the number of unread messages waiting in the list of
     *         error and warning messages.
     **/
    public int getNumUnreadErrorMessages () {
        return this.errorMessages.getSize ();
    };
    
    
    /**
     * @return true if there are unread messages waiting in the list of
     *         error and warning messages, else false.
     **/
    public boolean moreResults () {
        return this.errorMessages.moreItems ();
    };
    
    
    
    /**
     * reset the list of messages for retrieval.  This moves the cursor
     * back to the beginning of the errors list without removing any
     * of the messages from the list.
     **/
    public void resetMessageList () {
        // clear out all old errors so that we'll know which ones were
        // caused by this typechecking run.
        this.errorMessages.reset ();
    };
    
    
    
    /**
     * clear the list of messages for retrieval.  This moves the cursor
     * back to the beginning of the errors list without removing any
     * of the messages from the list.
     **/
    public void clearMessageList () {
        // clear out all old errors so that we'll know which ones were
        // caused by this typechecking run.
        errorMessages.reset ();
    };
    
    
    
    /**
     * @return the next error message from the list of error and warning
     *         messages.
     **/
    public AcmeError getNextResult () {
        AcmeObject nextMsg = this.errorMessages.getNextItem ();
        if (nextMsg instanceof AcmeError) {
            return (AcmeError) nextMsg;
        }
        return null;
    };
    
    
    /**
     * test method to get basic cross-app-communication working properly...
     **/
    public String testCOMmethod (String multiplyMe, int numTimes) {
        StringBuffer buf = new StringBuffer (multiplyMe);
        for (int i = 0; i < numTimes; i++) {
            buf.append ("..." + multiplyMe);
        };
        return buf.toString ();
    };
    
	public static void main(String[] args) {
		ArmaniAcmeStudioIntf aasi = new ArmaniAcmeStudioIntf();
		String acmeScript = "";

		try {
			if (args.length == 1) {
				FileReader fr = new FileReader(args[0]);
				BufferedReader br = new BufferedReader(fr);
				String aLine = null;
				while ((aLine = br.readLine()) != null) {
					acmeScript += aLine + "\n";
				}
			}
			else if (args.length == 0) {
				InputStreamReader r = new InputStreamReader(System.in);
				BufferedReader br = new BufferedReader(r);
				String aLine = null;
				while ((aLine = br.readLine()) != null) {
					acmeScript += aLine + "\n";
				}
			}
			else {
				System.out.println("Wrong number of params");
				return;
			}
		}
		catch (FileNotFoundException e) {
			System.out.println ("Could not read file " + args [0]);
		}
		catch (IOException e) {
			System.err.println ("Could not read file");
		}

		try {

			//System.out.println(acmeScript);
			aasi.typecheckAcmeStudioDesign(acmeScript);
			//System.out.println(aasi.getNumUnreadErrorMessages());
			Vector errors = new Vector();
			while (aasi.moreResults()) {
				AcmeError ae = aasi.getNextResult();
				Vector error = new Vector();
				generateErrorTree(ae, 0, error);
				String err = "";
				for (int j = 0; j < error.size(); j++) {
					err += (String) error.get(j);
				}
				errors.add(err);
			}
			for (int i = 0; i < errors.size(); i++) {
				System.out.print("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
				String err = (String) errors.get(i);
				err = err.replace('\n', ' ');
				System.out.println(err);
			}
			if (errors.size() == 0) {
				System.out.print("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
				String err = "<AcmeNote text=\"Typecheck succeeded.\"></AcmeNote>";
				System.out.println(err);
			}
		}
		catch (Exception e) {
			System.out.println("Error reading file ");
		}


		return;
	}
    
    private static void generateErrorTree(AcmeError ae, int depth, Vector tree) {
        String msg = "<AcmeError ";
        msg += "text=\""+ae.getErrorMessage().replace('\"', '\'')+"\" source=\""+ae.getMsgSource()+"\">";
        tree.add(msg);
        AcmeVector children = ae.getCausedByVector();
        if (ae != null && children.getSize()>0) {
            for (int j = 0; j < children.getSize(); j++) {  
                AcmeError aeChild = (AcmeError)children.getItemAt(j);
                generateErrorTree(aeChild, depth+1, tree);
            }
        }
       tree.add("</AcmeError>");
    }
};  // end class ArmaniAcmeStudioIntf
