
package acmeLib.properties;

import acmeLib.designRules.DesignRuleEvalException;
import acmeLib.designRules.ExpressionNode;


/** 
 *  Class:  AcmeBoolean
<BR>
 *  Purpose: Stores an ACME boolean value.
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    ?/?/97    DK      Created C++ version
<BR>
 *    4/30/97   RTM     Translated to Java
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public class AcmeBoolean extends AcmePropertyValue {

  //
  // protected members
  //
  protected boolean val;
  protected BooleanType type;

  //
  // public members
  //
  /**
   * This is a constructor for this class 
   * @param value the value to be assigned to val
   */
  public AcmeBoolean (boolean value) {
    this.val = value;
    this.type = acmeLib.core.globals.ConstantTypes.BOOLEAN_TYPE;
  };

  /**
   * This is a constructor for this class
   * @param t the value to be assigned to type
   * @param value the value to be assigned to val
   */
  public AcmeBoolean (BooleanType t, boolean value) {
    this.val = value;
    this.type = t;
  };


  /**
   * This method sets the type of this boolean
   * @param t the value to be assigned to type
   */
  public void setType (BooleanType t) { this.type = t; };
  /**
   * This method returns the type of this boolean
   * @return the type of this boolean
   */
  public AcmePropertyType getType () { return type; };

  /**
   * This method returns the value of this boolean
   * @return the value of this boolean
   */
  public boolean getValue() { return this.val; };
 /**
   * This method sets the value of this boolean
   * @param value the value of this boolean
   */
  public void setValue (boolean value) { this.val = value; };

  /** 
   * This method checks whether two booleans are equal
   * @param test_value the boolean to be compared
   * @return whether "this" and test_value are equal
   */
  public boolean equals(Object test_value) {
    if (test_value instanceof AcmeBoolean) {
        return (this.val == ((AcmeBoolean)test_value).getValue());
    }
    if (test_value instanceof ExpressionNode) {
        try {
            return this.val == ((ExpressionNode) test_value).evaluateAsBoolean();
        } catch (DesignRuleEvalException drExc) {
        };
    }
        return false;
  };


  /**
   * This method returns a copy of this boolean
   * @return a copy of this boolean
   */
  public AcmePropertyValue copy() {
    AcmeBoolean copyVal = new AcmeBoolean (type, val);
    copyVal.setIsDefaultValue(this.getIsDefaultValue());
    return copyVal;
  };


  /**
   * This method converts AcmeBoolean to String
   * @return the String value of this boolean
   */
  public String toString() {
	if (this.getValue())
	  return "true";
	else
	  return "false";
  }

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyValue#assignType(acmeLib.properties.AcmePropertyType)
 */
public void assignType(AcmePropertyType type) {
	if (type instanceof BooleanType) setType ((BooleanType )type);
};
  


};
