
package acmeLib.properties;

import acmeLib.core.io.AcmeOutputStream;


/**
 *  Class:   AcmeEnum
<BR>
 *  Purpose: Stores an ACME enumerated value.  This is simply a string
<BR>
 *          used to represent the particular value.  An ACMEEnum value
<BR>
 *          conforms to its type if this value is in the valid set of
<BR>
 *          enumerated values defined by the type.
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    ?/?/97    DK      Created C++ version
<BR>
 *    4/30/97   RTM     Translated to Java
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public class AcmeEnum extends AcmePropertyValue {

  //
  // protected members
  //
  protected EnumType type;
  protected String val;

  //
  // public members
  //
 /**
   * This is a constructor for this class
   * @param value the value to be assigned to val
   * @param t the value to be assigned to type
   */
  public AcmeEnum (EnumType t, String value) {
    this.setType(t);
    this.setValue(value);
  };

 /**
   * This is a constructor for this class
   * @param value the value to be assigned to val
   */
  public AcmeEnum (String value) {
    this.setType(null);
    this.setValue(value);
  };

 /**
   * This method returns a copy of this enumerated value
   * @return a copy of this enumerated value
   */
  public AcmePropertyValue copy() {
    AcmeEnum copyVal = new AcmeEnum ((EnumType )this.getType(), this.getValue());
    copyVal.setIsDefaultValue(this.getIsDefaultValue());
    return copyVal;
  };

 /**
   * This method checks whether two enumerted values are equal
   * @param test_value the enumerated value to be compared
   * @return whether "this" and test_value are equal
   */
  public boolean equals(Object test_value) {
    if ((test_value instanceof AcmeEnum) && (this.val != null)) {
        return (this.val.equals(((AcmeEnum)test_value).getValue()));
    }
        return false;
  };


 /**
   * This method sets the type of this enumerated value
   * @param t the value to be assigned to type
   */
  public void setType (EnumType t) { this.type = t; };
/**
   * This method returns the type of this enumerated value
   * @return the type of this enumerated value
   */
  public AcmePropertyType getType () { return this.type; };

 /**
   * This method returns the value of this enumerated value
   * @return the value of this enumerated value
   */
  public String getValue () { return this.val; };
 /**
   * This method sets the value of this enumerated value
   * @param value the value of this enumerated value
   */
  public void setValue (String value) { this.val = value; };

 /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
  public void exportAcme (AcmeOutputStream s) throws java.io.IOException {
      s.beginRegion (this);
        s.write(this.getValue());
      s.endRegion (this);
  };

  /**
   * This method converts AcmeEnum to String
   * @return the String value of this enumerated value
   */
  public String toString() {
    return this.getValue();
  }

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyValue#assignType(acmeLib.properties.AcmePropertyType)
 */
public void assignType(AcmePropertyType type) {
	if (type instanceof EnumType) setType ((EnumType )type);
};

};

