
package acmeLib.properties;

import acmeLib.designRules.DesignRuleEvalException;
import acmeLib.designRules.ExpressionNode;


/** 
 *  Class:  AcmeFloat
<BR> 
 *  Purpose:  Stores an ACME float value as a java float
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    ?/?/97    DK      Created C++ version
<BR>
 *    4/30/97   RTM     Translated to Java
<BR>
 *   10/08/97   RTM     Converted from "double" storage to native float
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public class AcmeFloat extends AcmePropertyValue {

  //
  // protected members
  //
  protected float val;
  protected FloatType type;

  //
  // public members
  //
 /**
   * This is a constructor for this class 
   * @param value the value to be assigned to val
   */
  public AcmeFloat (float value) {
    this.setType(null);
    this.setValue(value);
  };

 /**
   * This is a constructor for this class
   * @param t the value to be assigned to type
   * @param value the value to be assigned to val
   */
  public AcmeFloat (FloatType t, float value) {
    this.setType(t);
    this.setValue(value);
  };

 /**
   * This method returns a copy of this float
   * @return a copy of this float
   */
  public AcmePropertyValue copy() {
    AcmeFloat copyVal = new AcmeFloat((FloatType )this.getType(), this.getValue());
    copyVal.setIsDefaultValue(this.getIsDefaultValue());
    return copyVal;
  };

  /** 
   * This method checks whether two floatss are equal
   * @param test_value the float to be compared
   * @return whether "this" and test_value are equal
   */
  public boolean equals(Object test_value) {
    if (test_value instanceof AcmeFloat) {
        return (this.val == ((AcmeFloat)test_value).getValue());
    } else if (test_value instanceof AcmeInt) {
        return ((float) ((AcmeInt) test_value).getValue()) == this.val;
    } else if (test_value instanceof ExpressionNode) {
        try {
            return this.val == ((ExpressionNode) test_value).evaluateAsFloat();
        } catch (DesignRuleEvalException drExc) {
        };
    }
        return false;
  };


 /**
   * This method sets the type of this float
   * @param t the value to be assigned to type
   */
  public void setType (FloatType t) { this.type = t; };
 /**
   * This method returns the type of this float
   * @return the type of this float
   */
  public AcmePropertyType getType () { return this.type; };

  /**
   * This method returns the value of this float
   * @return the value of this float
   */
  public float getValue () { return this.val; };
 /**
   * This method sets the value of this float
   * @param value the value of this float
   */
  public void setValue (float value) { this.val = value; };

  /**
   * This method converts AcmeFloat to String
   * @return the String value of this float
   */
  public String toString() {
    return String.valueOf(this.getValue());
  }

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyValue#assignType(acmeLib.properties.AcmePropertyType)
 */
public void assignType(AcmePropertyType type) {
	if (type instanceof FloatType) setType ((FloatType )type);
};
  

};
