
package acmeLib.properties;

import acmeLib.designRules.DesignRuleEvalException;
import acmeLib.designRules.ExpressionNode;


/** 
 *  Class:  AcmeInt<BR>
 *  Purpose: Stores an ACME integer value.
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    ?/?/97    DK      Created C++ version
<BR>
 *    4/30/97   RTM     Translated to Java
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public class AcmeInt extends AcmePropertyValue {

  //
  // protected members
  //
  protected int val;
  protected IntType type;

  //
  // public members
  //
 /**
   * This is a constructor for this class 
   * @param value the value to be assigned to val
   */
  public AcmeInt (int value) {
    this.setType(null);
    this.setValue(value);
  };

 /**
   * This is a constructor for this class
   * @param t the value to be assigned to type
   * @param value the value to be assigned to val
   */
  public AcmeInt (IntType t, int value) {
    this.setType(t);
    this.setValue(value);
  };

  /**
   * This method returns a copy of this integer
   * @return a copy of this integer
   */
  public AcmePropertyValue copy () {
    AcmeInt copyVal = new AcmeInt((IntType )this.getType(), this.getValue());
    copyVal.setIsDefaultValue(this.getIsDefaultValue());
    return copyVal;
  };



 /** 
   * This method checks whether two integers are equal
   * @param test_value the integer to be compared
   * @return whether "this" and test_value are equal
   */
  public boolean equals(Object test_value) {
    if (test_value instanceof AcmeInt) {
        return (this.getValue() == ((AcmeInt)test_value).getValue());
    } else if (test_value instanceof AcmeFloat) {
        return (((float) this.getValue()) == ((AcmeFloat) test_value).getValue());
    } if (test_value instanceof ExpressionNode) {
        try {
            if (((ExpressionNode) test_value).getType() instanceof FloatType) {
                return ( ((float) this.getValue()) ==
                            ((ExpressionNode) test_value).evaluateAsFloat());
            }
                return (this.getValue() == ((ExpressionNode) test_value).evaluateAsInt());

        } catch (DesignRuleEvalException drExc) {
        };
    } 
        return false;

  };



 /**
   * This method sets the type of this integer
   * @param t the value to be assigned to type
   */
  public void setType (IntType t) { this.type = t; };
  /**
   * This method returns the type of the integer
   * @return the type of the integer
   */
  public AcmePropertyType getType () { return type; };

/**
   * This method sets the value of this integer
   * @param value the value of this integer
   */
  public void setValue (int value) { this.val = value; };
 /**
   * This method returns the value of this integer
   * @return the value of this integer
   */
  public int getValue () { return val; };

 /**
   * This method converts AcmeInt to String
   * @return the String value of this integer
   */
  public String toString() {
    return String.valueOf(this.val);
  }

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyValue#assignType(acmeLib.properties.AcmePropertyType)
 */
public void assignType(AcmePropertyType type) {
	if (type instanceof IntType) setType ((IntType )type);
}
  
  
};


