/*
 * Created on Aug 21, 2003
 *
 */
package acmeLib.properties;

import java.io.IOException;

import acmeLib.AcmeObject;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.core.exceptions.AcmeIllegalNameException;
import acmeLib.core.exceptions.AcmeUnnamedPropertyException;
import acmeLib.core.io.AcmeOutputStream;
import acmeLib.designRules.Scope;
import acmeLib.elements.AcmeFamily;

/**
 * @author Bradley Schmerl
 * Created Aug 21, 2003
 */
public class AcmePropertyTypeRef extends AcmePropertyType {

	Scope referentScope;
	AcmePropertyType referentType;
	/**
	 * @return
	 */
	public AcmePropertyType getReferentType() {
		return referentType;
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#getNextCreationNumber()
	 */
	public static int getNextCreationNumber() {
		return AcmePropertyType.getNextCreationNumber();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#addMetaProperty(acmeLib.properties.AcmeProperty)
	 */
	public void addMetaProperty(AcmeProperty p) throws AcmeUnnamedPropertyException {
		referentType.addMetaProperty(p);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#copy()
	 */
	public AcmePropertyType copy() {
		return referentType.copy();
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#copy(acmeLib.AcmeObject)
	 */
	public void copy(AcmeObject newObject) {
		referentType.copy(newObject);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		return referentType.equals(obj);
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#exportComment(acmeLib.core.io.AcmeOutputStream)
	 */
	public void exportComment(AcmeOutputStream s) throws IOException {
		referentType.exportComment(s);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getAliasedType()
	 */
	public AcmePropertyType getAliasedType() {
		return referentType.getAliasedType();
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#getComment()
	 */
	public String getComment() {
		return referentType.getComment();
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#getCreationOrder()
	 */
	public int getCreationOrder() {
		return referentType.getCreationOrder();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getDeclarationScope()
	 */
	public Scope getDeclarationScope() {
		return referentType.getDeclarationScope();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getInlineTypeSpec()
	 */
	public boolean getInlineTypeSpec() {
		return referentType.getInlineTypeSpec();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getIsAlias()
	 */
	public boolean getIsAlias() {
		return referentType.getIsAlias();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getMetaProperties()
	 */
	public AcmeEnumeration getMetaProperties() {
		return referentType.getMetaProperties();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getMetaProperty(java.lang.String)
	 */
	public AcmeProperty getMetaProperty(String name) {
		return referentType.getMetaProperty(name);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getQualifiedName()
	 */
	public String getQualifiedName() {
		return referentType.getQualifiedName();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getQualifier()
	 */
	public String getQualifier() {
		return referentType.getQualifier();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getTypeName()
	 */
	public String getTypeName() {
		return ((AcmeFamily )referentScope).getName () + "." + referentType.getTypeName();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return referentType.hashCode();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#resolveTypeReferences(acmeLib.designRules.Scope)
	 */
	public void resolveTypeReferences(Scope scope) {
		referentType.resolveTypeReferences(scope);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#setAliasedType(acmeLib.properties.AcmePropertyType)
	 */
	public void setAliasedType(AcmePropertyType newAliasedType) {
		referentType.setAliasedType(newAliasedType);
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#setComment(java.lang.String)
	 */
	public void setComment(String comment) {
		referentType.setComment(comment);
	}

	/* (non-Javadoc)
	 * @see acmeLib.AcmeObject#setCreationOrder(int)
	 */
	public void setCreationOrder(int newOrder) {
		referentType.setCreationOrder(newOrder);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#setDeclarationScope(acmeLib.designRules.Scope)
	 */
	public void setDeclarationScope(Scope newScope) {
		referentType.setDeclarationScope(newScope);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#setInlineTypeSpec(boolean)
	 */
	public void setInlineTypeSpec(boolean new_val) {
		referentType.setInlineTypeSpec(new_val);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#setIsAlias(boolean)
	 */
	public void setIsAlias(boolean newIsAlias) {
		referentType.setIsAlias(newIsAlias);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#setQualifier(java.lang.String)
	 */
	public void setQualifier(String q) {
		referentType.setQualifier(q);
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return referentType.toString();
	}

	/**
	 * 
	 */
	public AcmePropertyTypeRef(Scope s, AcmePropertyType type) {
		super();
		referentScope = s;
		referentType = type;
	}


	/* (non-Javadoc)
	 * @see acmeLib.core.io.Exportable#exportAcme(acmeLib.core.io.AcmeOutputStream)
	 */
	public void exportAcme(AcmeOutputStream s) throws IOException {
		referentType.exportAcme (s);
	}

	/* (non-Javadoc)
	 * @see acmeLib.core.io.ArmaniExportable#exportArmani(acmeLib.core.io.AcmeOutputStream)
	 */
	public void exportArmani(AcmeOutputStream s) throws IOException {
		referentType.exportArmani (s);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getAcmeIdentifierString()
	 */
	public String getAcmeIdentifierString() {
		return ((AcmeFamily )referentScope).getName () + "." + referentType.getAcmeIdentifierString();
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#typecheck(acmeLib.properties.AcmePropertyValue)
	 */
	public boolean typecheck(AcmePropertyValue val) {
		return referentType.typecheck (val);
	}

	/* (non-Javadoc)
	 * @see acmeLib.properties.AcmePropertyType#getDefinitionAsString()
	 */
	public String getDefinitionAsString() {
		return referentType.getDefinitionAsString();
	}

}
