
package acmeLib.properties;

import java.io.IOException;

import acmeLib.core.io.AcmeOutputStream;



/** 
 *  Class: AcmePropertyValue<BR><BR>
 *
 *  Purpose: abstract base class for the classes used to represent property<BR>
 *           values in ACME.<BR><BR>
 *
 *  Revision History:<BR>
 *    Date:     Person    Change<BR>
 *    ?/?/97    DK      Created C++ version<BR>
 *    4/7/97    RTM     Translated to Java<BR>
 *    8/6/97    RTM     removed deprecated getCategoryName method<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
abstract public class AcmePropertyValue extends acmeLib.AcmeObject
                                        implements acmeLib.designRules.Scope
{
 protected boolean isDefaultValue = false;

 abstract public AcmePropertyValue copy();

  /**
   * This method returns this property value's isDefaultValue
   * @return this property value's isDefaultValue
   */
 public boolean getIsDefaultValue() { return this.isDefaultValue; };
  /**
   * This method sets this property value's isDefaultValue
   * @param isDefault the value assigned to this property value's isDefaultValue
   */
 public void setIsDefaultValue(boolean isDefault) {
    this.isDefaultValue = isDefault;
 };

 /** 
  * This method resolves the property type object that this property refers to.
  * this is an abstract method that any relevant subclasses can override. 
  * @param scope acmeLib.designRules.Scope
  */
 public void resolveTypeReferences(acmeLib.designRules.Scope scope) {};

  /**
   * This method implements the Scope interface.  It's an abstract method that subclasses for
   * specific values (e.g. records) can implement if they so choose.
   * @param name the name of the property value
   * @param bubbleUp boolean
   * @return null
   */
 public Object lookupName(String name, boolean bubbleUp) {
    return null;
 };

 public void exportAcme (AcmeOutputStream s) throws IOException {
     s.beginRegion (this);
     s.write (toString ());
     s.endRegion (this);
 }

/**
 * This method assigns a type to the value, so that type checking can occur
 * @param type the type to assign
 */
public abstract void assignType(AcmePropertyType type);

public abstract AcmePropertyType getType ();

};
