
package acmeLib.properties;

import java.io.IOException;

import acmeLib.*;
import acmeLib.core.io.*;


/**
 *  Class: AcmeUnresolvedPropertyType
<BR><BR>
 *
 *  Purpose:Stores a placeholder for a real AcmePropertyType.  In the first
<BR>
 *          pass of the latest & greatest AcmeParser (as of 1/14/98)
<BR>
 *          typenames are unresolved after the first pass through parsing,
<BR>
 *          so one of these objects are stored in its place, and the
<BR>
 *          typename can be resolved on the name resolution pass through
<BR>
 *          the design.
<BR><BR>
 *
 *  Revision History:
<BR><BR>
 *    Date:     Person    Change
<BR>
 *   1/14/98   RTM     Created class
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public class AcmeUnresolvedPropertyType extends AcmePropertyType
                                        implements AcmeType, Exportable
{

	/* (non-Javadoc)
	 * @see acmeLib.AcmeType#getTypeName()
	 */
	public String getTypeName() {
		return (qualifier!=null?qualifier+ ".":"") + super.getTypeName();
	}

	public String getUnqualifiedTypeName () {
		return super.getTypeName ();
	}

    //
    // public members
    //
  /**
   * This is the constructor for this class
   * @param typeName the value to be assigned to the PropertyType's name
   */
    public AcmeUnresolvedPropertyType(String typeName) {
        this.setTypeName(typeName);
    };

	public AcmeUnresolvedPropertyType (String qualifier, String typeName) {
		this.setQualifier (qualifier);
		this.setTypeName(typeName);
	}

  /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
    public void exportAcmeIdentifier (AcmeOutputStream s)
	  throws java.io.IOException {
        s.write(this.getTypeName());
    };

  /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
	public void exportAcme (AcmeOutputStream s) throws java.io.IOException {
        s.beginRegion (this);
        if (this.getQualifier () != null) {
        	s.write(this.getQualifier () + ".");
        }
        s.write(this.getTypeName());
        s.endRegion (this);
    }

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#exportArmani(acmeLib.core.io.AcmeOutputStream)
 */
public void exportArmani(AcmeOutputStream s) throws IOException {
    exportAcme (s);
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#getAcmeIdentifierString()
 */
public String getAcmeIdentifierString() {
    return this.getTypeName ();
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#typecheck(acmeLib.properties.AcmePropertyValue)
 */
public boolean typecheck(AcmePropertyValue val) {
    return false;
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#getDefinitionAsString()
 */
public String getDefinitionAsString() {
    return "";
};

public boolean isQualified () {return this.getQualifiedName() != null;}

}

