
package acmeLib.properties;

import acmeLib.core.io.AcmeOutputStream;


/**
 *  Class: AnyType
<BR><BR>
 *
 *  Purpose:the type object for a property that of type "any"
<BR>
 *          all non-null property values satisfy the type "any"
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    6/19/97   RTM     Created
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public class AnyType extends AcmePropertyType {

    //
    // public members
    //
  /**
   * This is the constructor for this class
   * @param name the name of the type to be created
   */
    public AnyType (String name) { setTypeName(name); };

  /**
   * This method returns a deep-copy of "this" AcmePropertyType object
   * @return a copy of this type
   */
    public AcmePropertyType copy() {
      AnyType newType = new AnyType(this.getTypeName());
      this.copyMetaProperties(newType);
      return newType;
    };


  /**
   * @return the the IndetifierString for this type
   */
	public String getAcmeIdentifierString() {
        if (getTypeName() != null) {
            return this.getTypeName();
        } else {
            return "any";
        }
    };

  /**
   * This is a helper function that implements "export*" for all *'s
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
    protected void export(AcmeOutputStream s) throws java.io.IOException {
        s.beginRegion (this);
        exportComment (s);
        if (this.getIsAlias() == true) {
            s.write ("Property Type ");
            s.write (getTypeName() + " = ");
            if (this.getAliasedType() != null) {
                s.writeln(" = " + this.getAliasedType().getTypeName() + ";");
            } else {
                s.writeln(" = any;  /* warning: aliased type with null alias value */");
            };
        } else {
    	    s.write ("Property Type ");
    	    s.write (getTypeName());
    	    s.writeln (" = any;");
    	};
        s.endRegion (this);
    };

 /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
	public void exportAcme (AcmeOutputStream s) throws java.io.IOException {
        this.export(s);
	};

 /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
	public void exportArmani(AcmeOutputStream s) throws java.io.IOException {
        this.export(s);
	}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#typecheck(acmeLib.properties.AcmePropertyValue)
 */
public boolean typecheck(AcmePropertyValue val) {
    return true;
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#getDefinitionAsString()
 */
public String getDefinitionAsString() {
    if (getIsAlias () && getAliasedType () != null) {
        return getAliasedType ().getTypeName ();
    }
    else {
        return "any";
    }
};
};
