
package acmeLib.properties;

import acmeLib.core.io.AcmeOutputStream;


/**
 *  Class:  BooleanType
 *
 *  Purpose:the type object for a boolean type declaration
 *
 *  Revision History:
 *    Date:     Person    Change
 *    ?/?/97    DK      Created C++ version
 *    6/1/97    RTM     Translated to Java
 *    6/19/97   RTM     Added copy method
 *
 *  @author Bob Monroe
 */
public class BooleanType extends AcmePropertyType {

  /**
   *  static, default property value for a BooleanType.
   */
  public static AcmeBoolean defaultValue = new AcmeBoolean(false);

  //
  // public methods
  //
 /**
   * This is the constructor for this class
   * @param name the name of the boolean type to be created
   */
  public BooleanType (String name) { setTypeName(name); };

  /**
   * This method returns a deep-copy of "this" AcmePropertyType object
   * @return a copy of this boolean type
   */
  public AcmePropertyType copy() {
    BooleanType newType = new BooleanType(this.getTypeName());
    this.copyMetaProperties(newType);
    return newType;
  }

  /**
   * This method returns the IdentifierString corresponding to this boolean type
   * @return the IdentifierString corresponding to this boolean type
   */
  public String getAcmeIdentifierString() {
    if (getTypeName() != null) {
        return this.getTypeName();
    } else {
        return "boolean";
    }
  };

  /**
   * This is a helper function that implements "export*" for all *'s
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
  protected void export(AcmeOutputStream s) throws java.io.IOException {
    exportComment (s);
    s.beginRegion (this);
    if (this.getIsAlias() == true) {
        s.write ("Property Type ");
        s.write (getTypeName() + " = ");
        if (this.getAliasedType() != null) {
            s.writeln(" = " + this.getAliasedType().getTypeName() + ";");
        } else {
            s.writeln(" = boolean;  /* warning: aliased type with null alias value */");
        };
    } else {
        s.write ("Property Type ");
        s.write (getTypeName());
        s.writeln (" = boolean;");
    };
    s.endRegion (this);
  };

  /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
  public void exportAcme (AcmeOutputStream s) throws java.io.IOException {
    this.export(s);
  };

  /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
  public void exportArmani(AcmeOutputStream s) throws java.io.IOException {
    this.export(s);
  }

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#typecheck(acmeLib.properties.AcmePropertyValue)
 */
public boolean typecheck(AcmePropertyValue val) {
    return true;
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#getDefinitionAsString()
 */
public String getDefinitionAsString() {
    if (getIsAlias() && getAliasedType() != null) return getAliasedType().getTypeName();
    else return "boolean";
};

};
