
package acmeLib.properties;

import acmeLib.core.io.AcmeOutputStream;


/**
 *  Class:FloatType
<BR><BR>
 *
 *  Purpose: the type object for a float type declaration
<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *   ?/?/97    DK      Created C++ version
<BR>
 *    6/1/97    RTM     Translated to Java
<BR>
 *    6/19/97   RTM     Added copy method
<BR><BR>
 *
 *  @author Bob Monroe<BR>
 */
public class FloatType extends AcmePropertyType {

  // static, default property value for a BooleanType.
  public static AcmeFloat defaultValue = new AcmeFloat((float)0.0);

  //
  // public methods
  //
 /**
   * This is the constructor for this class
   * @param name the name of the float type being created
   */
  public FloatType (String name) { setTypeName(name); };

  /**
   * This method returns a deep-copy of "this" AcmePropertyType object
   * @return a copy to this  AcmePropertyType
   */
  public AcmePropertyType copy() {
    FloatType newType = new FloatType(this.getTypeName());
    this.copyMetaProperties(newType);
    return newType;
  }


 /**
   * @return the float type's IdentifierString
   */
  public String getAcmeIdentifierString() {
    if (getTypeName() != null) {
        return this.getTypeName();
    } else {
        return "float";
    }
  };

  /**
   * This is a helper function that implements "export*" for all *'s
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
  protected void export(AcmeOutputStream s) throws java.io.IOException {
    exportComment (s);
    s.beginRegion (this);
    if (this.getIsAlias() == true) {
        s.write ("Property Type ");
        s.write (getTypeName() + " = ");
        if (this.getAliasedType() != null) {
            s.writeln(" = " + this.getAliasedType().getTypeName() + ";");
        } else {
            s.writeln(" = float;  /* warning: aliased type with null alias value */");
        };
    } else {
        s.write ("Property Type ");
        s.write (getTypeName());
        s.writeln (" = float;");
    };
    s.endRegion (this);
  };

  /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
  public void exportAcme (AcmeOutputStream s) throws java.io.IOException {
    this.export(s);
  };

  /**
   * @param s AcmeOutputStream
   * @exception java.io.IOException when error occurs while writing
   */
  public void exportArmani(AcmeOutputStream s) throws java.io.IOException {
    this.export(s);
  }

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#typecheck(acmeLib.properties.AcmePropertyValue)
 */
public boolean typecheck(AcmePropertyValue val) {
    return true;
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#getDefinitionAsString()
 */
public String getDefinitionAsString() {
    if (getIsAlias() && getAliasedType() != null) return getAliasedType().getTypeName ();
    else return "float";
};

};

