
package acmeLib.properties;

import java.io.IOException;

import acmeLib.core.io.AcmeOutputStream;


/**
 *  Class:PropertyReferenceType
<BR><BR>
 *
 *  Purpose: A type object that is assigned to all variables whose
 *           type is a reference to a property, rather than the value
 *           of that property.  This class is intended primarily for internal
 *           Armani housekeeping.  It's only significant extension to the
 *           generic AcmePropertyType class is a change in exporting and
 *           support for the lookup of the "name", "value", "type", and
 *           "ref" fields.<BR><BR>
 *
 *  Revision History:
<BR>
 *    Date:     Person    Change
<BR>
 *    12/2/98   RTM         Created class
 *
 *  @author Bob Monroe<BR>
 */
public class PropertyReferenceType extends AcmePropertyType
                                   implements acmeLib.designRules.Scope
{

  public PropertyReferenceType(String newName) { super(newName); };

  /**
   * Implements the acmeLib.designRules.Scope interface to lookup the
   * specific substructure of the PropertyReferenceType.  Specifically, that
   * is the ability to lookup the type or the name of the
   * to property substructure and meta-properties.  Subclasses of
   * @param name the name of the property
   * @param bubbleUp boolean
   * @return the Object corresponding to "name"
   */
    public Object lookupName(String name, boolean bubbleUp)
    {
        Object retObj = null;
        if (name.equalsIgnoreCase("type"))
            retObj = this;
        else if (name.equalsIgnoreCase("name"))
            retObj = new acmeLib.designRules.StringLiteralNode(
                                    "<DefaultPropertyReferenceTypeName>");

        return retObj;
    }

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#exportAcme(acmeLib.core.io.AcmeOutputStream)
 */
public void exportAcme(AcmeOutputStream s) throws IOException {
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#exportArmani(acmeLib.core.io.AcmeOutputStream)
 */
public void exportArmani(AcmeOutputStream s) throws IOException {
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#getAcmeIdentifierString()
 */
public String getAcmeIdentifierString() {
    return "";
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#typecheck(acmeLib.properties.AcmePropertyValue)
 */
public boolean typecheck(AcmePropertyValue val) {
    return false;
}

/* (non-Javadoc)
 * @see acmeLib.properties.AcmePropertyType#getDefinitionAsString()
 */
public String getDefinitionAsString() {
    return "";
};


};

