package acmeLib.tools;

import acmeLib.AcmeType;
import acmeLib.elements.*;
import acmeLib.properties.AcmeProperty;
import acmeLib.properties.AcmePropertyBearer;
import acmeLib.properties.AcmePropertyValue;
import acmeLib.representations.AcmeBinding;
import acmeLib.representations.AcmeRepresentation;

/**
 * @author schmerl
 * Provides the interface for tools to report changes made to the model.
 * This may be integrated with the model elements in acmeLib directly at
 * a later date.
 */
public interface AcmeChangeManager {

	public void onAddSystem (AcmeDesign design, AcmeSystem s, Object announcer);
	public void onAddComponent (AcmeSystem sys, AcmeComponent comp, Object announcer) ;
	public void onAddConnector (AcmeSystem sys, AcmeConnector conn, Object announcer);
	public void onAddPort (AcmeComponent comp, AcmePort p, Object announcer);
	public void onAddMember (AcmeGroup group, AcmeElement e, Object announcer);
	public void onRemoveMember (AcmeGroup group, AcmeElement e, Object announcer);
	public void onAddGroup (AcmeSystem sys, AcmeGroup group, Object announcer);
	public void onRemoveGroup (AcmeSystem sys, AcmeGroup group, Object announcer);
	public void onAddRole (AcmeConnector conn, AcmeRole r, Object announcer);
	public void onRemoveComponent (AcmeSystem sys, AcmeComponent comp, Object announcer);
	public void onRemoveConnector (AcmeSystem sys, AcmeConnector conn, Object announcer);
	public void onRemovePort (AcmeComponent comp, AcmePort p, Object announcer);
	public void onRemoveRole (AcmeConnector conn, AcmeRole r, Object announcer);
	public void onAddProperty (AcmePropertyBearer pb, AcmeProperty p, Object announcer);
	public void onSetProperty (AcmePropertyBearer pb, AcmeProperty p, AcmePropertyValue val, Object announcer);
	public void onRemoveProperty (AcmePropertyBearer pb, AcmeProperty p, Object announcer);
	public void onAddRepresentation (AcmeElement e, AcmeRepresentation r, Object announcer);
	public void onRemoveRepresentation (AcmeElement e, AcmeRepresentation r, Object announcer);
	public void onAddBinding (AcmeRepresentation r, AcmeBinding b, Object announcer);
	public void onRemoveBinding (AcmeRepresentation r, AcmeBinding b, Object announcer);
	public void onAddAttachment (AcmeSystem s, AcmeAttachment a, Object announcer);
	public void onRemoveAttachment (AcmeSystem s, AcmeAttachment a, Object announcer);
	public void onSetName (AcmeElement e, String new_name, Object announcer);
	public void onAssignFamily (AcmeSystem s, AcmeFamily f, Object announcer);
	public void onUnAssignFamily (AcmeSystem s, AcmeFamily f, Object announcer);
	public void onAssignType (AcmeElement e, AcmeElementType t, Object announcer);
	public void onUnAssignType (AcmeElement e, AcmeElementType t, Object announcer);
	public void onRemoveType (AcmeFamily fam, AcmeType t, Object announcer);
	public void onRemoveType (AcmeDesign des, AcmeType t, Object announcer);
	public void onAddType (AcmeFamily ts, AcmeType t, Object announcer);
	public void onAddType (AcmeDesign des, AcmeType t, Object announcer);


}
