/*
 * Created on Mar 22, 2005
 */
package acmeLib.tools;

import java.util.ArrayList;
import java.util.List;

import acmeLib.elements.AcmeDesign;

/**
 * @author Shang-Wen Cheng (zensoul@cs.cmu.edu)
 *
 * A central dispatcher that opened design records notify of AcmeDesign changes.
 * The dispatcher in turn notify interested listeners of these changes. 
 */
public class AcmeDesignChangeNotifier {

    /** Private singleton instance of the dispatcher */
    private static AcmeDesignChangeNotifier _dispatcher = null;
    /** List of change listeners. */
    private List _listeners = null;

    private AcmeDesignChangeNotifier () {
        _listeners = new ArrayList();
    }

    protected void finalize() {
        if (_listeners != null) {
            _listeners.clear();
            _listeners = null;
        }
    }

    public static AcmeDesignChangeNotifier getInstance() {
        if (_dispatcher == null) {
            _dispatcher = new AcmeDesignChangeNotifier();
        }
        return _dispatcher;
    }
    
    public void onDesignChange (String canonPath, AcmeDesign oldDesign, AcmeDesign newDesign) {
        // tell listeners about the AcmeDesign instance change
        notifyListeners(new AcmeDesignChangeEvent(canonPath, oldDesign, newDesign));
    }
    
    public void addDesignChangeListener (IAcmeDesignChangeListener listener) {
        if (listener != null && !_listeners.contains(listener)) {
            _listeners.add(listener);
        }
    }

    public void removeDesignChangeListener (IAcmeDesignChangeListener listener) {
        if (listener != null && _listeners.contains(listener)) {
            _listeners.remove(listener);
        }
    }

    private void notifyListeners (AcmeDesignChangeEvent evt) {
        for (int i=0; i < _listeners.size(); i++) {
            IAcmeDesignChangeListener listener =
                (IAcmeDesignChangeListener )_listeners.get(i);
            listener.designChanged(evt);
        }
    }

}
