// Class: RateAnalyses
// Purpose: defines a set of functions, predicates, and operations for
//          calculating flow-rates.
//
// Revision History:
//    Date:     Person    Change
//    3/30/98    RTM      * Created original class

package acmeLib.tools;

import acmeLib.designRules.DesignRuleEvalException;
import acmeLib.designRules.ExpressionNode;
import acmeLib.elements.AcmeComponent;
import acmeLib.elements.AcmeElement;

public class RateAnalyses {

    //
    // protected methods can be defined as helpers for the public methods.
    //

    /** <no protected helper methods defined> */



    //
    // public static methods.
    // all external analyses need to be hooked in as public static methods.
    //

    /** calculates the throughput rate for a component
        Armani signature:
            External Analysis throughputRate(comp : Component) : int =
	            external acmeLib.analysisTools.rateAnalyses.throughputRate(comp);
     */
    public static int throughputRate(ExpressionNode elementNode)
                                     throws DesignRuleEvalException
    {
        // dereference the ExpressionNode to retrieve the component
        // to analyze.
        AcmeElement resolvedElement = elementNode.evaluateAsElement();

        // cast the element to a component, if possible.  If it is not
        // a component then throw a DesignRuleEvalException.
        AcmeComponent comp = null;
        if (resolvedElement instanceof AcmeComponent) {
            comp = (AcmeComponent) resolvedElement;
        } else {
            throw new DesignRuleEvalException("Error! Non-Component type parameter "
                + "passed to acmeLib.analysisTools.RateAnalyses.throughputRate"
                + "(Component).");
        };

        // Put the throughput calculating code here.  For now, the throughput
        // rate is simply equal to 100 * the number of characters in the
        // component's name.  This is simply a stub implementation to show
        // how to link external analyses into Armani...
        int calculatedRate;
        if ((comp == null) || (comp.getName() == null)) {
            calculatedRate = 0;
        } else {
            calculatedRate = comp.getName().length() * 100;
        };

        return calculatedRate;
    };
};