
 package acmeLib.tools.junli;

 import acmeLib.AcmeObject;
import acmeLib.AcmeType;
import acmeLib.core.container.AcmeEnumeration;
import acmeLib.designRules.*;
import acmeLib.elements.*;
import acmeLib.properties.*;
import acmeLib.typeManagement.AcmeTypeManager;


 public class SetTest {

 //construct a set of components which have input ports connected from an
 //output P.
 public static SetExpressionNode componentsConnectedFromPort(
							ExpressionNode ElementNode) throws DesignRuleEvalException {

     AcmeElement element = null;

     try {
          element = ElementNode.evaluateAsElement();
		 }
     catch ( ClassCastException ccExc) {
       throw new DesignRuleEvalException("unable to cast the element argument"
           + "passed to componentsConnectedFromPort(.) external analysis");
	 }
     catch (DesignRuleEvalException drExc) {
       throw new DesignRuleEvalException("unable to evalue the element argument"
          + "passed to componentsConnectedFromPort(.) external analysis");
	 }


     if  (!(element instanceof AcmePort)) {
         throw new DesignRuleEvalException("unable to evaluate the element argument as AcmePort in componentsConnectedFromPort(.) external analysis");
     }
	 else {
      AcmePort ThisPort = (AcmePort)element;
      AcmeComponent ComponentOfPort=(AcmeComponent)ThisPort.getParent();
      AcmeSystem OverallSystem= (AcmeSystem)ComponentOfPort.getParent();
      AcmeEnumeration Connectors = OverallSystem.getConnectors();

      AcmeEnumeration candidateComponents= new AcmeEnumeration();


      if (!(isaMSPort(ThisPort) && isaMSOutputPort(ThisPort))) {
          throw new DesignRuleEvalException("port: " + ThisPort.getName()+
                   "either has not a MSPort type or is not an output port in componentsConnectedFromPort(.) external analysis");
	  }

      while (Connectors.moreItems() ) {
        AcmeConnector connector = (AcmeConnector)Connectors.getNextItem();

        if (!isaMSConnector(connector)) {
           throw new DesignRuleEvalException("connector: " +
               connector.getName() +" does not have MSConnector type in componentsConnectedFromPort(.) external analysis");
				}
//a MSConnector typed connector always has MSSource and MSSink two roles!

        AcmeRole SourceRole = connector.getRole("msSource");
        AcmeRole SinkRole= connector.getRole("msSink");

        if (OverallSystem.getAttachment(ThisPort, SourceRole)!=null) {
					//the above condiction indicates that the first half of attachement
					//is finished!
             AcmeEnumeration components= OverallSystem.getComponents();
             while (components.moreItems()) {
                AcmeComponent tComp= (AcmeComponent)components.getNextItem();
                if (!tComp.equals(ComponentOfPort)) {
									//two different components!
                     AcmeEnumeration tports = tComp.getPorts();
                     while(tports.moreItems()){
                        AcmePort tp = (AcmePort)tports.getNextItem();
                        if ( isaMSPort(tp) && isaMSInputPort(tp)) {
													if (OverallSystem.getAttachment(tp,SinkRole)!=null){
														//i.e., we get the second half of attachement!
                             candidateComponents.insertItem((AcmeObject)
                                                               tComp);
                             break;
													}
												}
										 }//all ports
								}
						 }//all components
				}
			}//search all the connectors!

	  System.out.println("\n port: " + ComponentOfPort.getName() +"." +ThisPort.getName() +
	     " has number of destination components: "
	    + candidateComponents.getSize());

  	  TransientSet retValSet = new TransientSet(
	         candidateComponents.castToAcmeVector());
	  AcmeType unionedType = AcmeTypeManager.inferLeastCommonSupertype(
	              candidateComponents.castToAcmeVector());
	  retValSet.setType(new SetType(unionedType.getTypeName(), unionedType));
	  retValSet.setSetType(unionedType);

	  //Do I really need the retValSet type specified for the set?!
	  return retValSet;

	 }



} //end of method componentsConnectedFrom(.);



 protected  static boolean  isaMSConnector(AcmeConnector connector){
  boolean retVal =false;

  AcmeEnumeration types = connector.getDeclaredTypes();
   while (types.moreItems()) {
     AcmeType ttype= (AcmeType)types.getNextItem();
     if (ttype instanceof AcmeConnectorType) {
        //later on, compare the property type object equivalence instead.
			 if (ttype.getTypeName().equalsIgnoreCase("MSConnector")) {
             retVal = true;
             break;
			 }
		 }
	 }

 return retVal;
 }


protected static boolean isaMSPort(AcmePort port) {

  boolean retVal =false;

  AcmeEnumeration types = port.getDeclaredTypes();
   while (types.moreItems()) {
     AcmeType ttype= (AcmeType)types.getNextItem();
     if (ttype instanceof AcmePortType) {
         //later on, compare the property type object equivalence instead.
			 if (ttype.getTypeName().equalsIgnoreCase("MSPort")) {
             retVal = true;
             break;
			 }
		 }
	 }

 return retVal;
 }


protected  static boolean isaMSOutputPort(AcmePort port) {
  boolean retVal = false;
  AcmeEnumeration props=port.getProperties();

  while (props.moreItems()) {
     AcmeProperty tprop = (AcmeProperty)props.getNextItem();
     if (tprop.getName().equalsIgnoreCase("msDirection")) {
         AcmePropertyValue value = tprop.getValue();
         if (value instanceof AcmeEnum) {
					 if (value.toString().equalsIgnoreCase("msOut")) {
                  retVal = true;
                  break;
					 }
				}
		 }
	}

  return retVal;

}

 protected static boolean isaMSInputPort(AcmePort port) {
  boolean retVal = false;
  AcmeEnumeration props=port.getProperties();

  while (props.moreItems()) {
     AcmeProperty tprop = (AcmeProperty)props.getNextItem();
     if (tprop.getName().equalsIgnoreCase("msDirection")) {
         AcmePropertyValue value = tprop.getValue();
         if (value instanceof AcmeEnum) {
					 if (value.toString().equalsIgnoreCase("msIn")) {
                  retVal = true;
                  break;
					 }
				}
		 }
	}

  return retVal;

}


}



