package acmeLib.typeManagement;

import acmeLib.AcmeObject;

import java.util.Hashtable;
import java.util.Enumeration;

/**
 * Class: AcmeNamespace<br>
 * Purpose: An AcmeNamespace is a lookup-table for names.  It<br>
 *          simply stores the names of all elements within a given<br>
 *          scope and provides access and removal methods.<br>
 *<br>
 *          An AcmeNamespace explicitly makes no distinction between<br>
 *          the names of different categories of elements, as Acme<br>
 *          specifies a flat namespace for all categories.  That is,<br>
 *          once a name has been claimed for an element or property<br>
 *          of one category that name is no longer available for use<br>
 *          by elements or properties of a different category.<br>
 *<br>
 * Revision History:<br>
 *    Date:   Person    Change<br>
 *    7/15/97   RTM     Created<br>
 *<br>
 * @author Bob Monroe
 */
public class AcmeNamespace extends AcmeObject {

    //
    // private members
    //

  /**
   * symbol table that holds type descriptions
   */
    private Hashtable nameMap;   // originally private AcmeTypeMap types;

    //
    // public members
    //

  /**
   * This is the constructor for this class
   */
    public AcmeNamespace() { nameMap = new Hashtable(); };

  /**
   * Add a name to the table
   * @param name name to be added
   */
    public void addName(String name) { nameMap.put(name,name); };
  /**
   * Remove a name from the table
   * @param name name to be removed
   */
    public void removeName(String name) { nameMap.remove(name); };

  /**
   * Check to see if the name is in the table
   * @param name name to be tested
   * @return true if the name is in the table
   */
    public boolean containsName(String name) {
        return nameMap.containsKey(name);
    };

  /**
   * Return the size of the table
   * @return the size 
   */
    public int size() { return nameMap.size(); };

  /**
   * Get the contents of the table
   * @return Enumeration of the contents
   */
    public Enumeration getNames() { return nameMap.elements(); };

};