/*
 * Created on Aug 20, 2003
 *
 */
package acmeLib.typeManagement;

import acmeLib.core.container.AcmeEnumeration;

/**
 * @author Bradley Schmerl
 * Created Aug 20, 2003
 */
public interface IInheritable {

	/**
	 * Adds a source that this inheritable is based on. If the inheritable is
	 * unified from an inheritable of a supertype or instantiated type, then the 
	 * inheritable will point to the inherited inheritable.
	 * @param typeProp The super-property that this property is inherited from
	 */
	public void addInheritedSource(IInheritable source);

	/**
	 * Returns true if this inheritable is the same as an inherited inheritable. 
	 * For this to be true, everything needs to be the same.
	 * @return whether the property is the same as one of its inherited
	 */
	public boolean isSameAsInherited();

	/**
	 * @return whether the inheritable is based on a inheritable from a supertype
	 */
	public boolean isInherited();
	
	/**
	 * @return returns the enumeration of inheritables that this is based on
	 */
	public AcmeEnumeration getInheritedSources ();
	
}
